# coding: utf8
from __future__ import unicode_literals

LOOKUP = {
'a':'a',
'aabakken':'aabakk',
'aabakkens':'aabakk',
'aarbakke':'aarbakk',
'aarum':'aarum',
'aase':'aas',
'ab':'ab',
'abitanti':'abitanti',
'abitazione':'abitazion',
'ablegøyer':'ablegøy',
'absolutt':'absolutt',
'absorberes':'absorber',
'ad':'ad',
'adgang':'adgang',
'adgangen':'adgang',
'adkomst':'adkomst',
'adkomstdokument':'adkomstdokument',
'adkomstdokumenter':'adkomstdokument',
'adkomsten':'adkomst',
'adlyde':'adlyd',
'adlyder':'adlyd',
'adm':'adm',
'administrasjon':'administrasjon',
'administrasjonen':'administrasjon',
'administrasjonsdepartementet':'administrasjonsdepartement',
'administrasjonskostnadene':'administrasjonskostnad',
'administrativ':'administrativ',
'administrative':'administrativ',
'administrativt':'administrativ',
'administrere':'administrer',
'administrerende':'administrer',
'administreres':'administrer',
'administrert':'administrer',
'adopsjon':'adopsjon',
'adopsjonspenger':'adopsjonspeng',
'adresse':'adress',
'adresser':'adress',
'adskillelse':'adskill',
'adskilt':'adskilt',
'advare':'advar',
'advarselen':'advarsel',
'advart':'advart',
'advokat':'advokat',
'advokatar':'advokat',
'advokatfirmaet':'advokatfirma',
'advokatfullmektig':'advokatfullmekt',
'agder':'agd',
'agenter':'agent',
'agenturer':'agentur',
'aggregeres':'aggreger',
'aggregert':'aggreger',
'agn':'agn',
'agnar':'agn',
'agreement':'agreement',
'ajourføring':'ajourføring',
'akademikerne':'akademikern',
'akademikernes':'akademikern',
'ake':'ake',
'akershus':'akershus',
'akkord':'akkord',
'akkumulerte':'akkumuler',
'akkurat':'akkurat',
'aksept':'aksept',
'akseptabel':'akseptabel',
'akseptabelt':'akseptabelt',
'akseptable':'akseptabl',
'aksepterast':'aksepter',
'akseptere':'aksepter',
'aksepteres':'aksepter',
'akseptert':'aksepter',
'aksjane':'aksj',
'aksjar':'aksj',
'aksje':'aksj',
'aksjeandel':'aksjeandel',
'aksjeavkastning':'aksjeavkastning',
'aksjebrev':'aksjebrev',
'aksjebustader':'aksjebustad',
'aksjeeigar':'aksje',
'aksjeeigarane':'aksjeeigar',
'aksjeeigarar':'aksjeeigar',
'aksjeeigaravtale':'aksjeeigaravtal',
'aksjeeigaren':'aksjeeigar',
'aksjegevinster':'aksjegevinst',
'aksjeinnskot':'aksjeinnskot',
'aksjekapital':'aksjekapital',
'aksjekapitalen':'aksjekapital',
'aksjelov':'aksj',
'aksjelova':'aksj',
'aksjeloven':'aksj',
'aksjelovens':'aksj',
'aksjelovgjeving':'aksjelovgjeving',
'aksjelovgjevinga':'aksjelovgjeving',
'aksjer':'aksj',
'aksjerettslig':'aksjeretts',
'aksjeselskap':'aksjeselskap',
'aksjeselskaper':'aksjeselskap',
'aksjeselskapet':'aksjeselskap',
'aksjeselskapsformen':'aksjeselskapsform',
'aksjeselskapsrettslege':'aksjeselskapsretts',
'aksjonær':'aksjonær',
'aksjonæroppgaver':'aksjonæroppgav',
'aktersetet':'akterset',
'aktes':'akt',
'aktiv':'aktiv',
'aktiva':'aktiv',
'aktivaklasser':'aktivaklass',
'aktivasammensetning':'aktivasammensetning',
'aktive':'aktiv',
'aktivitet':'aktivit',
'aktivitetane':'aktivitet',
'aktivitetar':'aktivitet',
'aktiviteten':'aktivitet',
'aktivitetene':'aktivitet',
'aktiviteter':'aktivitet',
'aktivitetsøkning':'aktivitetsøkning',
'aktivt':'aktiv',
'aktlaust':'aktlaust',
'aktløyse':'aktløys',
'aktsemd':'aktsemd',
'aktsemdplikta':'aktsemdplikt',
'aktualisere':'aktualiser',
'aktuar':'aktu',
'aktuell':'aktuell',
'aktuelle':'aktuell',
'aktuelt':'aktuelt',
'aktør':'aktør',
'aktørane':'aktør',
'aktørar':'aktør',
'aktørene':'aktør',
'aktørenes':'aktør',
'aktørens':'aktør',
'aktører':'aktør',
'aktørers':'aktør',
'aktørgruppe':'aktørgrupp',
'aku':'aku',
'alarmsignalet':'alarmsignal',
'aldeles':'aldel',
'alder':'ald',
'alderdom':'alderdom',
'alderdommen':'alderdomm',
'alderen':'alder',
'alderkilde':'alderkild',
'alderpensjonen':'alderpensjon',
'alderpensjonskapital':'alderpensjonskapital',
'alders':'ald',
'aldersdifferanse':'aldersdifferans',
'aldersforskjellen':'aldersforskjell',
'aldersgrense':'aldersgrens',
'aldersgrensen':'aldersgrens',
'aldersgrensene':'aldersgrens',
'aldersgrupper':'aldersgrupp',
'alderspensjon':'alderspensjon',
'alderspensjonen':'alderspensjon',
'alderspensjoner':'alderspensjon',
'alderspensjonist':'alderspensjonist',
'alderspensjonistene':'alderspensjonist',
'alderspensjonister':'alderspensjonist',
'alderspensjonsforsikring':'alderspensjonsforsikring',
'alderspensjonsforsikringen':'alderspensjonsforsikring',
'alderspensjonsforsikringsavtale':'alderspensjonsforsikringsavtal',
'alderspensjonsforsikringsavtaler':'alderspensjonsforsikringsavtal',
'alderspensjonskapital':'alderspensjonskapital',
'alderspensjonskapitalbevis':'alderspensjonskapitalbevis',
'alderspensjonskapitalen':'alderspensjonskapital',
'alderspensjonskapitalens':'alderspensjonskapital',
'alderspensjonskapitalkontoen':'alderspensjonskapitalkonto',
'alderspensjonskonti':'alderspensjonskonti',
'alderspensjonskonto':'alderspensjonskonto',
'alderspensjonskontoen':'alderspensjonskonto',
'alderspensjonsnivå':'alderspensjonsnivå',
'alderspensjonsordning':'alderspensjonsordning',
'alderspensjonsordninger':'alderspensjonsordning',
'alderspensjonsrettigheter':'alderspensjonsrett',
'alderspensjonsspareavtale':'alderspensjonsspareavtal',
'alderspensjonsytelse':'alderspensjonsyt',
'alderssamansetjinga':'alderssamansetjing',
'aldre':'aldr',
'aldri':'aldri',
'aldringen':'aldring',
'aleine':'alein',
'alene':'alen',
'alf':'alf',
'alfhild':'alfhild',
'all':'all',
'alldeles':'alldel',
'alle':'all',
'aller':'all',
'allerede':'aller',
'allereie':'allerei',
'alles':'all',
'allfader':'allfad',
'alliance':'allianc',
'alliansen':'allians',
'allianser':'allians',
'allierte':'allier',
'allmannyttiga':'allmannytt',
'allmektig':'allmekt',
'allmen':'allm',
'allmenn':'allmenn',
'allmennaksjelova':'allmennaksj',
'allmennaksjeloven':'allmennaksj',
'allmennaksjeselskap':'allmennaksjeselskap',
'allmennaksjeselskaper':'allmennaksjeselskap',
'allmenne':'allmenn',
'allmennhetens':'allmenn',
'allmennspråket':'allmennspråk',
'allmennyttiga':'allmennytt',
'allmennyttige':'allmennytt',
'allment':'allment',
'allmugen':'allmug',
'allsidige':'allsid',
'alltid':'alltid',
'alltids':'alltid',
'allting':'allting',
'almen':'alm',
'almene':'alm',
'almenmerknadene':'almenmerknad',
'almennaksjelovens':'almennaksj',
'almennyttig':'almennytt',
'alminnelig':'alminn',
'alminnelige':'alminn',
'alminnelighet':'alminn',
'alnabru':'alnabru',
'alt':'alt',
'alta':'alt',
'alternativ':'alternativ',
'alternative':'alternativ',
'alternativene':'alternativ',
'alternativer':'alternativ',
'alternativet':'alternativ',
'alternativt':'alternativ',
'alternerer':'alterner',
'altfor':'altfor',
'altoverveiende':'altovervei',
'altså':'altså',
'aluminium':'aluminium',
'alvalde':'alvald',
'alvene':'alv',
'alvenes':'alv',
'alver':'alv',
'alveslekt':'alveslekt',
'alvheim':'alvheim',
'alvim':'alvim',
'alvor':'alvor',
'alvorleg':'alvor',
'alvorlege':'alvor',
'alvorlig':'alvor',
'ambisjonsnivået':'ambisjonsnivå',
'amma':'amm',
'ammas':'amm',
'amsterdam':'amsterdam',
'amtmannsvingen':'amtmannsving',
'an':'an',
'analogt':'analogt',
'analyse':'analys',
'analysen':'analys',
'analysene':'analys',
'analyser':'analys',
'analysere':'analyser',
'analysert':'analyser',
'anbefaler':'anbefal',
'anbefales':'anbefal',
'anbefaling':'anbefaling',
'anbefalingene':'anbefaling',
'anbefalinger':'anbefaling',
'anbefalt':'anbefalt',
'anbefalte':'anbefalt',
'and':'and',
'andamål':'andamål',
'andamålsenlig':'andamålsen',
'andel':'andel',
'andelane':'andel',
'andelar':'andel',
'andeleigar':'andel',
'andeleigarane':'andeleigar',
'andeleigarbok':'andeleigarbok',
'andelen':'andel',
'andelene':'andel',
'andeler':'andel',
'andelsbevis':'andelsbevis',
'andelsbok':'andelsbok',
'andelsboka':'andelsbok',
'andelsboligforening':'andelsboligforening',
'andelsboligforeninger':'andelsboligforening',
'andelsboligorganisation':'andelsboligorganisation',
'andelsboligorganisationer':'andelsboligorganisation',
'andelsbrev':'andelsbrev',
'andelsbrevet':'andelsbrev',
'andelsdokument':'andelsdokument',
'andelseier':'andelsei',
'andelseiere':'andelseier',
'andelseierne':'andelseiern',
'andelseiernes':'andelseiern',
'andelseigar':'andels',
'andelseigarane':'andelseigar',
'andelseigarar':'andelseigar',
'andelseigarbok':'andelseigarbok',
'andelseigarboka':'andelseigarbok',
'andelseigaren':'andelseigar',
'andelseigarens':'andelseigar',
'andelseigarforholdet':'andelseigarforhold',
'andelseigarposisjonen':'andelseigarposisjon',
'andelseigarregister':'andelseigarregist',
'andelseigartal':'andelseigartal',
'andelshaver':'andelshav',
'andelshavers':'andelshav',
'andelsinnskot':'andelsinnskot',
'andelsinnskotet':'andelsinnskot',
'andelsinnskottet':'andelsinnskott',
'andelskapital':'andelskapital',
'andelskapitalen':'andelskapital',
'andelslag':'andelslag',
'andelslaget':'andelslag',
'andelsobligasjonane':'andelsobligasjon',
'andelsobligasjonar':'andelsobligasjon',
'andelsrett':'andelsrett',
'andelsselskaber':'andelsselskab',
'andelsselskapa':'andelsselskap',
'andelssum':'andelssum',
'andelssummen':'andelssumm',
'andelsteikning':'andelsteikning',
'andelsteikninga':'andelsteikning',
'anders':'and',
'anderssen':'anderss',
'andra':'andr',
'andre':'andr',
'andres':'andr',
'andrimne':'andrimn',
'andsynes':'andsyn',
'andvare':'andvar',
'andvarefoss':'andvarefoss',
'aner':'aner',
'anerkjenne':'anerkjenn',
'anerkjente':'anerkjent',
'anføre':'anfør',
'anfører':'anfør',
'anføres':'anfør',
'anført':'anført',
'angerboda':'angerbod',
'angi':'angi',
'angir':'angir',
'angis':'angis',
'angitt':'angitt',
'angitte':'angitt',
'angivelse':'angiv',
'angjeldende':'angjeld',
'angre':'angr',
'angrefrist':'angrefrist',
'angrep':'angrep',
'angrepet':'angrep',
'angrer':'angr',
'angret':'angr',
'angripe':'angrip',
'angriper':'angrip',
'angst':'angst',
'angt':'angt',
'angår':'angår',
'ankler':'ankl',
'ankommer':'ankomm',
'ankre':'ankr',
'anledning':'anledning',
'anledninger':'anledning',
'anlegg':'anlegg',
'anleggene':'anlegg',
'anlegget':'anlegg',
'anleggsmessig':'anleggsmess',
'anleggsmidler':'anleggsmidl',
'anliggende':'anligg',
'anløp':'anløp',
'anløpe':'anløp',
'anløpene':'anløp',
'anløper':'anløp',
'anløpes':'anløp',
'anløpsavgift':'anløpsavgift',
'anløpsfrekvens':'anløpsfrekv',
'anløpshavn':'anløpshavn',
'anløpshavner':'anløpshavn',
'anløpshyppighet':'anløpshypp',
'anløpsmønster':'anløpsmønst',
'anløpsmønstre':'anløpsmønstr',
'anløpspunkter':'anløpspunkt',
'anløpssteder':'anløpssted',
'anløpsutgiftene':'anløpsutgift',
'anmodet':'anmod',
'anna':'ann',
'annan':'annan',
'annaniassen':'annaniass',
'annat':'annat',
'anne':'ann',
'annen':'ann',
'annenhver':'annenhv',
'annenhvert':'annenhver',
'annens':'ann',
'annerledes':'annerl',
'annet':'ann',
'annleis':'annleis',
'ansatt':'ansatt',
'ansatte':'ansatt',
'ansattes':'ansatt',
'anse':'ans',
'ansees':'anse',
'anser':'ans',
'anses':'ans',
'ansett':'ansett',
'ansettelse':'ansett',
'ansettelsen':'ansett',
'ansettelsesforhold':'ansettelsesforhold',
'ansettelsesforholdet':'ansettelsesforhold',
'ansettelseskrav':'ansettelseskrav',
'ansettelsestid':'ansettelsestid',
'ansettes':'ansett',
'ansiennitet':'ansiennit',
'ansienniteten':'ansiennitet',
'ansiennitetsreglar':'ansiennitetsregl',
'ansikt':'ansikt',
'ansiktet':'ansikt',
'anskaffelse':'anskaff',
'anslag':'anslag',
'anslagene':'anslag',
'anslaget':'anslag',
'anslagsvis':'anslagsvis',
'anslå':'anslå',
'anslår':'anslår',
'anslås':'anslås',
'anslått':'anslått',
'anslåtte':'anslått',
'anstrenger':'anstreng',
'ansvar':'ansv',
'ansvaret':'ansvar',
'ansvarleg':'ansvar',
'ansvarlege':'ansvar',
'ansvarlig':'ansvar',
'ansvarlige':'ansvar',
'ansvarliges':'ansvar',
'ansvarsavgrensing':'ansvarsavgrensing',
'ansvarsavgrensinga':'ansvarsavgrensing',
'ansvarsfri':'ansvarsfri',
'ansvarsfritak':'ansvarsfritak',
'ansvarsfråskrivingar':'ansvarsfråskriving',
'ansvarsgrunnlaget':'ansvarsgrunnlag',
'ansvarsinnskot':'ansvarsinnskot',
'ansvarskapital':'ansvarskapital',
'ansvarsområde':'ansvarsområd',
'ansvarspulverisering':'ansvarspulverisering',
'ansvarsregel':'ansvarsregel',
'ansvarsreglar':'ansvarsregl',
'ansvarsreguleringa':'ansvarsregulering',
'ansvarsspørsmåla':'ansvarsspørsmål',
'ant':'ant',
'anta':'ant',
'antagelig':'antag',
'antakelig':'antak',
'antakelse':'antak',
'antakelsen':'antak',
'antakelsene':'antak',
'antakelser':'antak',
'antall':'antall',
'antallet':'antall',
'antar':'ant',
'antas':'ant',
'antatt':'antatt',
'antatte':'antatt',
'anten':'ant',
'antenne':'antenn',
'antenneanlegg':'antenneanlegg',
'antitetisk':'antitetisk',
'antwerpen':'antwerp',
'antydede':'antyd',
'antyder':'antyd',
'antydes':'antyd',
'antydet':'antyd',
'anvende':'anv',
'anvendelse':'anvend',
'anvendelsen':'anvend',
'anvendelser':'anvend',
'anvendelsesområde':'anvendelsesområd',
'anvendelsesområder':'anvendelsesområd',
'anvender':'anvend',
'anvendes':'anv',
'anvendt':'anvend',
'anviser':'anvis',
'anvisning':'anvisning',
'ap':'ap',
'apparat':'apparat',
'april':'april',
'ar':'ar',
'arbeid':'arbeid',
'arbeidast':'arbeid',
'arbeidde':'arbeidd',
'arbeide':'arbeid',
'arbeider':'arbeid',
'arbeidere':'arbeider',
'arbeiderpartiet':'arbeiderparti',
'arbeides':'arbeid',
'arbeidet':'arbeid',
'arbeidgiver':'arbeidgiv',
'arbeids':'arbeid',
'arbeidsavtale':'arbeidsavtal',
'arbeidsavtalen':'arbeidsavtal',
'arbeidsavtalene':'arbeidsavtal',
'arbeidsavtalenes':'arbeidsavtal',
'arbeidsavtaler':'arbeidsavtal',
'arbeidsdag':'arbeidsdag',
'arbeidsdeling':'arbeidsdeling',
'arbeidsdelinga':'arbeidsdeling',
'arbeidsdelingen':'arbeidsdeling',
'arbeidsdepartementet':'arbeidsdepartement',
'arbeidsforhold':'arbeidsforhold',
'arbeidsforholdet':'arbeidsforhold',
'arbeidsfør':'arbeidsfør',
'arbeidsgiver':'arbeidsgiv',
'arbeidsgiveravgift':'arbeidsgiveravgift',
'arbeidsgiveravgiften':'arbeidsgiveravgift',
'arbeidsgivere':'arbeidsgiver',
'arbeidsgiveren':'arbeidsgiver',
'arbeidsgiverens':'arbeidsgiver',
'arbeidsgiverfinansiert':'arbeidsgiverfinansier',
'arbeidsgiverne':'arbeidsgivern',
'arbeidsgiverorganisasjoner':'arbeidsgiverorganisasjon',
'arbeidsgivers':'arbeidsgiv',
'arbeidsgjevar':'arbeidsgjev',
'arbeidsgrunnlag':'arbeidsgrunnlag',
'arbeidsgruppa':'arbeidsgrupp',
'arbeidsgruppas':'arbeidsgrupp',
'arbeidsgruppe':'arbeidsgrupp',
'arbeidsgruppen':'arbeidsgrupp',
'arbeidsgruppens':'arbeidsgrupp',
'arbeidsinnsats':'arbeidsinnsat',
'arbeidsinntekt':'arbeidsinntekt',
'arbeidskapasitet':'arbeidskapasit',
'arbeidskonflikter':'arbeidskonflikt',
'arbeidskontrakt':'arbeidskontrakt',
'arbeidskraft':'arbeidskraft',
'arbeidskraftens':'arbeidskraft',
'arbeidskraftsundersøkelse':'arbeidskraftsundersøk',
'arbeidsliv':'arbeidsliv',
'arbeidslivet':'arbeidsliv',
'arbeidslivets':'arbeidsliv',
'arbeidsløyse':'arbeidsløys',
'arbeidsmarkedet':'arbeidsmarked',
'arbeidsmarkedsbidrag':'arbeidsmarkedsbidrag',
'arbeidsmarkedspensjoner':'arbeidsmarkedspensjon',
'arbeidsmengda':'arbeidsmengd',
'arbeidsmiljøloven':'arbeidsmiljø',
'arbeidsmåte':'arbeidsmåt',
'arbeidsmøter':'arbeidsmøt',
'arbeidsområde':'arbeidsområd',
'arbeidsoppgaver':'arbeidsoppgav',
'arbeidsoppgåver':'arbeidsoppgåv',
'arbeidsopplegg':'arbeidsopplegg',
'arbeidsopplegget':'arbeidsopplegg',
'arbeidsperioden':'arbeidsperiod',
'arbeidsplassar':'arbeidsplass',
'arbeidsplasser':'arbeidsplass',
'arbeidsrelaterte':'arbeidsrelater',
'arbeidsrettslig':'arbeidsretts',
'arbeidsrettslige':'arbeidsretts',
'arbeidssammenheng':'arbeidssammenheng',
'arbeidsstokken':'arbeidsstokk',
'arbeidssøkere':'arbeidssøker',
'arbeidstagere':'arbeidstager',
'arbeidstakar':'arbeidstak',
'arbeidstakarane':'arbeidstakar',
'arbeidstakaren':'arbeidstakar',
'arbeidstaker':'arbeidstak',
'arbeidstakere':'arbeidstaker',
'arbeidstakeren':'arbeidstaker',
'arbeidstakerens':'arbeidstaker',
'arbeidstakeres':'arbeidstaker',
'arbeidstakergrupper':'arbeidstakergrupp',
'arbeidstakerhold':'arbeidstakerhold',
'arbeidstakerne':'arbeidstakern',
'arbeidstakernes':'arbeidstakern',
'arbeidstakerorganisasjonene':'arbeidstakerorganisasjon',
'arbeidstakers':'arbeidstak',
'arbeidstid':'arbeidstid',
'arbeidstilhøve':'arbeidstilhøv',
'arbeidsuke':'arbeidsuk',
'arbeidsutvalg':'arbeidsutvalg',
'arbeidsvilkår':'arbeidsvilkår',
'arbeidsvilkårene':'arbeidsvilkår',
'arbetsinsats':'arbetsinsat',
'areal':'areal',
'arealanvendelse':'arealanvend',
'arealbehov':'arealbehov',
'arealbehovet':'arealbehov',
'arealbruk':'arealbruk',
'arealbruken':'arealbruk',
'arealbruksendring':'arealbruksendring',
'arealbrukskonfliktene':'arealbrukskonflikt',
'arealdel':'arealdel',
'arealdelen':'arealdel',
'arealdelplan':'arealdelplan',
'arealdelplanen':'arealdelplan',
'arealdisponering':'arealdisponering',
'arealeffektiviserende':'arealeffektiviser',
'arealeffektivisering':'arealeffektivisering',
'arealeffektivitet':'arealeffektivit',
'arealene':'areal',
'arealer':'areal',
'arealervervelser':'arealerverv',
'arealet':'areal',
'arealforhold':'arealforhold',
'arealintensiv':'arealintensiv',
'arealkapasiteten':'arealkapasitet',
'arealknapphet':'arealknapp',
'arealkonflikter':'arealkonflikt',
'arealkostnadene':'arealkostnad',
'arealkostnader':'arealkostnad',
'arealkrevende':'arealkrev',
'arealmessig':'arealmess',
'arealmessige':'arealmess',
'arealplanen':'arealplan',
'arealplaner':'arealplan',
'arealplanlegging':'arealplanlegging',
'arealpolitikk':'arealpolitikk',
'arealpolitikken':'arealpolitikk',
'arealproduktivitet':'arealproduktivit',
'arealreguleringstiltak':'arealreguleringstiltak',
'arealreservene':'arealreserv',
'arealressurser':'arealressurs',
'arealsituasjon':'arealsituasjon',
'arealsituasjonen':'arealsituasjon',
'arealspørsmål':'arealspørsmål',
'arealtilgang':'arealtilgang',
'arealtilgangen':'arealtilgang',
'arealtilganger':'arealtilgang',
'arealtilgjengelighet':'arealtilgjeng',
'arealtilskot':'arealtilskot',
'arealutnyttelse':'arealutnytt',
'arealutnyttelsen':'arealutnytt',
'arealutvidelser':'arealutvid',
'arealutvidelsesmuligheter':'arealutvidelsesmu',
'arealverdien':'arealverdi',
'arealøkning':'arealøkning',
'arena':'aren',
'argeste':'argest',
'argument':'argument',
'argumentasjon':'argumentasjon',
'argumentasjonen':'argumentasjon',
'argumentene':'argument',
'argumenter':'argument',
'argumenterer':'argumenter',
'argumenteres':'argumenter',
'argumentet':'argument',
'arm':'arm',
'armbåndet':'armbånd',
'armen':'arm',
'armene':'arm',
'armer':'arm',
'armhulen':'armhul',
'armring':'armring',
'armringer':'armring',
'arne':'arn',
'arnhild':'arnhild',
'arrangerte':'arranger',
'arrest':'arrest',
'arresten':'arrest',
'art':'art',
'arte':'art',
'arten':'art',
'arter':'art',
'artikkel':'artikkel',
'artikkelen':'artikkel',
'artiklene':'artikl',
'arv':'arv',
'arveavgift':'arveavgift',
'arveavgiftslov':'arveavgift',
'arveavgiftsloven':'arveavgift',
'arveavgiftslovgivningen':'arveavgiftslovgivning',
'arveavgiftsmessige':'arveavgiftsmess',
'arveavgiftsreglene':'arveavgiftsregl',
'arveavgiftsutvalgets':'arveavgiftsutvalg',
'arvebuet':'arvebu',
'arvede':'arv',
'arvefall':'arvefall',
'arvefallet':'arvefall',
'arvegangsreglene':'arvegangsregl',
'arvegangsregler':'arvegangsregl',
'arveloven':'arv',
'arven':'arv',
'arveovergangen':'arveovergang',
'arvet':'arv',
'arvetilfella':'arvetilfell',
'arving':'arving',
'arvingane':'arving',
'arvingar':'arving',
'arvingen':'arving',
'arvinger':'arving',
'arvlataren':'arvlatar',
'as':'as',
'asia':'asi',
'asiatiske':'asiatisk',
'ask':'ask',
'aske':'ask',
'askelut':'askelut',
'asken':'ask',
'asker':'ask',
'asketreet':'asketre',
'asl':'asl',
'aspekt':'aspekt',
'aspekter':'aspekt',
'aspektet':'aspekt',
'assev':'assev',
'associazione':'associazion',
'assosiasjoner':'assosiasjon',
'assosierte':'assosier',
'asylsøkjarar':'asylsøkjar',
'asymmetrien':'asymmetri',
'at':'at',
'atferden':'atferd',
'atkomstdokument':'atkomstdokument',
'atkomstdokumentene':'atkomstdokument',
'atkomstdokumentet':'atkomstdokument',
'atskillelse':'atskill',
'atskilt':'atskilt',
'att':'att',
'atten':'att',
'attende':'att',
'attendebetaling':'attendebetaling',
'attendebetalt':'attendebetalt',
'attendeføring':'attendeføring',
'attendekalling':'attendekalling',
'attendelevering':'attendelevering',
'atterhald':'atterhald',
'atterhaldet':'atterhald',
'attest':'attest',
'attestasjonar':'attestasjon',
'attestert':'attester',
'attgjevast':'attgjev',
'attom':'attom',
'attraktiv':'attraktiv',
'attraktive':'attraktiv',
'attraktivitet':'attraktivit',
'attraktivt':'attraktiv',
'attval':'attval',
'au':'au',
'aud':'aud',
'audhumla':'audhuml',
'august':'august',
'auka':'auk',
'aukande':'auk',
'aukar':'auk',
'aukast':'auk',
'auke':'auk',
'auprema':'auprem',
'aurvandil':'aurvandil',
'aurvandils':'aurvandil',
'australia':'australi',
'automatisk':'automatisk',
'autorisasjon':'autorisasjon',
'autorisasjonen':'autorisasjon',
'autorisasjonsordning':'autorisasjonsordning',
'autoriserte':'autoriser',
'av':'av',
'avbrot':'avbrot',
'avbroten':'avbrot',
'avbrytast':'avbryt',
'avbryte':'avbryt',
'avbryter':'avbryt',
'avbøtende':'avbøt',
'avdekke':'avdekk',
'avdekker':'avdekk',
'avdekket':'avdekk',
'avdeling':'avdeling',
'avdelingane':'avdeling',
'avdelingar':'avdeling',
'avdelingsdirektør':'avdelingsdirektør',
'avdempet':'avdemp',
'avdrag':'avdrag',
'avdraga':'avdrag',
'avdragsplanen':'avdragsplan',
'avdød':'avdød',
'avdødes':'avdød',
'averterer':'averter',
'avfall':'avfall',
'avfallshandtering':'avfallshandtering',
'avga':'avg',
'avgang':'avgang',
'avgangsalder':'avgangsald',
'avgangstider':'avgangstid',
'avgi':'avgi',
'avgifer':'avgif',
'avgift':'avgift',
'avgiften':'avgift',
'avgiftene':'avgift',
'avgifter':'avgift',
'avgiftsbestemmelsene':'avgiftsbestemm',
'avgiftsforskrifter':'avgiftsforskrift',
'avgiftsfrie':'avgiftsfri',
'avgiftsfritt':'avgiftsfritt',
'avgiftsinnkreving':'avgiftsinnkreving',
'avgiftsinntektene':'avgiftsinntekt',
'avgiftsmessig':'avgiftsmess',
'avgiftsmessige':'avgiftsmess',
'avgiftsmidlene':'avgiftsmidl',
'avgiftsmidler':'avgiftsmidl',
'avgiftsnivået':'avgiftsnivå',
'avgiftsområder':'avgiftsområd',
'avgiftsplikt':'avgiftsplikt',
'avgiftspliktig':'avgiftsplikt',
'avgiftsreglene':'avgiftsregl',
'avgiftsregulativene':'avgiftsregulativ',
'avgiftsregulativer':'avgiftsregulativ',
'avgiftsspørsmål':'avgiftsspørsmål',
'avgiftssystem':'avgiftssystem',
'avgiftstyper':'avgiftstyp',
'avgir':'avgir',
'avgis':'avgis',
'avgitt':'avgitt',
'avgivelse':'avgiv',
'avgjer':'avgj',
'avgjerande':'avgjer',
'avgjerast':'avgjer',
'avgjerd':'avgjerd',
'avgjerda':'avgjerd',
'avgjerdene':'avgjerd',
'avgjerder':'avgjerd',
'avgjerdskompetansen':'avgjerdskompetans',
'avgjere':'avgjer',
'avgjord':'avgjord',
'avgjort':'avgjort',
'avgjør':'avgjør',
'avgjøre':'avgjør',
'avgjørelse':'avgjør',
'avgjørelsen':'avgjør',
'avgjørelsene':'avgjør',
'avgjørelser':'avgjør',
'avgjørelsesmyndighet':'avgjørelsesmynd',
'avgjørende':'avgjør',
'avgjøres':'avgjør',
'avgrensa':'avgrens',
'avgrensar':'avgrens',
'avgrensast':'avgrens',
'avgrense':'avgrens',
'avgrenser':'avgrens',
'avgrenses':'avgrens',
'avgrenset':'avgrens',
'avgrensing':'avgrensing',
'avgrensinga':'avgrensing',
'avgrensingane':'avgrensing',
'avgrensingar':'avgrensing',
'avgrensinger':'avgrensing',
'avgrensning':'avgrensning',
'avgrunnen':'avgrunn',
'avhenda':'avhend',
'avhendar':'avhend',
'avhendaren':'avhendar',
'avhendast':'avhend',
'avhende':'avh',
'avhending':'avhending',
'avhendinga':'avhending',
'avhendingslova':'avhending',
'avhenge':'avheng',
'avhenger':'avheng',
'avhengig':'avheng',
'avhengige':'avheng',
'avhengighet':'avheng',
'avhjelpast':'avhjelp',
'avhl':'avhl',
'avholdt':'avhold',
'avhugget':'avhugg',
'avis':'avis',
'avisa':'avis',
'avisrom':'avisrom',
'avkall':'avkall',
'avkasting':'avkasting',
'avkastning':'avkastning',
'avkastningen':'avkastning',
'avkastningprosent':'avkastningprosent',
'avkastningsavhengig':'avkastningsavheng',
'avkastningsdeling':'avkastningsdeling',
'avkastningsgaranti':'avkastningsgaranti',
'avkastningsgarantier':'avkastningsgaranti',
'avkastningsgrense':'avkastningsgrens',
'avkastningskatt':'avkastningskatt',
'avkastningsnivå':'avkastningsnivå',
'avkastningsnivåer':'avkastningsnivå',
'avkastningsnivået':'avkastningsnivå',
'avkastningsprosent':'avkastningsprosent',
'avkastningsprosenten':'avkastningsprosent',
'avkastningsrater':'avkastningsrat',
'avkastningsrisiko':'avkastningsrisiko',
'avkastningsskatt':'avkastningsskatt',
'avkastningsskatten':'avkastningsskatt',
'avkastningsusikkerheten':'avkastningsusikker',
'avkjøring':'avkjøring',
'avklare':'avklar',
'avklaring':'avklaring',
'avklart':'avklart',
'avklårast':'avklår',
'avklåre':'avklår',
'avklåring':'avklåring',
'avklåringa':'avklåring',
'avklårt':'avklårt',
'avkortes':'avkort',
'avkorting':'avkorting',
'avkortinga':'avkorting',
'avkuttede':'avkutt',
'avlaste':'avlast',
'avlaster':'avlast',
'avlastning':'avlastning',
'avlastningsbehov':'avlastningsbehov',
'avlastningsbehovet':'avlastningsbehov',
'avlastningshavn':'avlastningshavn',
'avlastningshavner':'avlastningshavn',
'avlastningsløsninger':'avlastningsløsning',
'avlastningstilbudet':'avlastningstilbud',
'avlastningsveg':'avlastningsveg',
'avledes':'avl',
'avledet':'avled',
'avleggjast':'avleggj',
'avleidde':'avleidd',
'avlen':'avl',
'avleverte':'avlever',
'avling':'avling',
'avlønning':'avlønning',
'avløse':'avløs',
'avløsningsbeløp':'avløsningsbeløp',
'avløyst':'avløyst',
'avregningen':'avregning',
'avregninger':'avregning',
'avsa':'avs',
'avsatt':'avsatt',
'avse':'avs',
'avsender':'avsend',
'avsendere':'avsender',
'avsenderfylke':'avsenderfylk',
'avsendersted':'avsendersted',
'avsendinga':'avsending',
'avsetjast':'avsetj',
'avsetjing':'avsetjing',
'avsetjinga':'avsetjing',
'avsetning':'avsetning',
'avsetningar':'avsetning',
'avsetningen':'avsetning',
'avsetninger':'avsetning',
'avsetningsadgang':'avsetningsadgang',
'avsettes':'avsett',
'avsindig':'avsind',
'avskiper':'avskip',
'avskipere':'avskiper',
'avskipssted':'avskipssted',
'avskjere':'avskjer',
'avskjære':'avskjær',
'avskriving':'avskriving',
'avskrivninger':'avskrivning',
'avsky':'avsky',
'avslag':'avslag',
'avslege':'avs',
'avslutningsfase':'avslutningsfas',
'avslutta':'avslutt',
'avsluttar':'avslutt',
'avsluttast':'avslutt',
'avslutte':'avslutt',
'avslutter':'avslutt',
'avsluttes':'avslutt',
'avsluttet':'avslutt',
'avslå':'avslå',
'avsnitt':'avsnitt',
'avsnittene':'avsnitt',
'avsnittet':'avsnitt',
'avspeile':'avspeil',
'avspeiler':'avspeil',
'avstand':'avstand',
'avstanden':'avstand',
'avstander':'avstand',
'avsted':'avsted',
'avstedkommer':'avstedkomm',
'avstemming':'avstemming',
'avstemt':'avstemt',
'avstå':'avstå',
'avstår':'avstår',
'avta':'avt',
'avtakende':'avtak',
'avtala':'avtal',
'avtalane':'avtal',
'avtalar':'avtal',
'avtalast':'avtal',
'avtale':'avtal',
'avtalebrot':'avtalebrot',
'avtaleelementet':'avtaleelement',
'avtaleerverv':'avtaleerverv',
'avtaleervervarar':'avtaleervervar',
'avtaleervervaren':'avtaleervervar',
'avtalefestede':'avtalefest',
'avtaleforhold':'avtaleforhold',
'avtaleforholdet':'avtaleforhold',
'avtalefriheten':'avtalefri',
'avtaleklausul':'avtaleklausul',
'avtaleliknande':'avtalelikn',
'avtalelova':'avtal',
'avtalen':'avtal',
'avtalene':'avtal',
'avtalens':'avtal',
'avtaleområdet':'avtaleområd',
'avtalepant':'avtalepant',
'avtalepanthavarar':'avtalepanthavar',
'avtalepart':'avtalepart',
'avtalepartar':'avtalepart',
'avtalepartene':'avtalepart',
'avtalepraksis':'avtalepraksis',
'avtaler':'avtal',
'avtalereglane':'avtaleregl',
'avtalerett':'avtalerett',
'avtalerettslege':'avtaleretts',
'avtales':'avtal',
'avtalesituasjonen':'avtalesituasjon',
'avtaletida':'avtaletid',
'avtaletidspunktet':'avtaletidspunkt',
'avtaletypar':'avtaletyp',
'avtalpension':'avtalpension',
'avtalt':'avtalt',
'avtalte':'avtalt',
'avtar':'avt',
'avveging':'avveging',
'avveginga':'avveging',
'avveining':'avveining',
'avveiningen':'avveining',
'avvente':'avvent',
'avvik':'avvik',
'avvikande':'avvik',
'avvike':'avvik',
'avvikene':'avvik',
'avviker':'avvik',
'avviket':'avvik',
'avvikla':'avvikl',
'avviklast':'avvikl',
'avvikle':'avvikl',
'avvikler':'avvikl',
'avvikles':'avvikl',
'avviklet':'avvikl',
'avvikling':'avvikling',
'avviklinga':'avvikling',
'avviklingen':'avvikling',
'avviklingsbalanse':'avviklingsbalans',
'avviklingsbuet':'avviklingsbu',
'avviklingsperioden':'avviklingsperiod',
'avviklingsstyre':'avviklingsstyr',
'avviklingsstyret':'avviklingsstyr',
'avviklingstida':'avviklingstid',
'avviklingstyret':'avviklingstyr',
'avviklingsvedtak':'avviklingsvedtak',
'avvisast':'avvis',
'avvist':'avvist',
'avviste':'avvist',
'b':'b',
'ba':'ba',
'babler':'babl',
'bachmann':'bachmann',
'bad':'bad',
'baderom':'baderom',
'baderomsgolv':'baderomsgolv',
'badet':'bad',
'badstue':'badstu',
'badstuen':'badstu',
'bak':'bak',
'bakbena':'bakben',
'baken':'bak',
'bakenfor':'bakenfor',
'bakenforliggende':'bakenforligg',
'bakfoten':'bakfot',
'bakgrunn':'bakgrunn',
'bakgrunnen':'bakgrunn',
'bakgrunnsarbeid':'bakgrunnsarbeid',
'bakgrunnsmaterialet':'bakgrunnsmaterial',
'bakgrunnsregel':'bakgrunnsregel',
'bakgrunnsreglane':'bakgrunnsregl',
'bakgrunnsreglar':'bakgrunnsregl',
'bakhand':'bakhand',
'bakke':'bakk',
'bakkekontakt':'bakkekontakt',
'bakken':'bakk',
'bakkene':'bakk',
'bakkete':'bakket',
'bakketopp':'bakketopp',
'baklandet':'bakland',
'baklengs':'bakleng',
'bakover':'bakov',
'bakre':'bakr',
'bakrus':'bakrus',
'bakser':'baks',
'bakveien':'bakvei',
'balanse':'balans',
'balansen':'balans',
'balansere':'balanser',
'balanserer':'balanser',
'balansert':'balanser',
'balanserte':'balanser',
'balder':'bald',
'balders':'bald',
'balkong':'balkong',
'balkongar':'balkong',
'ball':'ball',
'ballonger':'ballong',
'bamble':'bambl',
'band':'band',
'banda':'band',
'bandt':'band',
'bane':'ban',
'baneforbindelse':'baneforbind',
'banekorridorene':'banekorridor',
'banekorridorer':'banekorridor',
'banemann':'banemann',
'banenett':'banenett',
'banenettet':'banenett',
'banetilknytning':'banetilknytning',
'banetransport':'banetransport',
'bank':'bank',
'bankar':'bank',
'banken':'bank',
'bankenes':'bank',
'banker':'bank',
'bankforening':'bankforening',
'bankgaranti':'bankgaranti',
'bankinnskot':'bankinnskot',
'bankinnskudd':'bankinnskudd',
'bankinnskuddene':'bankinnskudd',
'bankinnskuddet':'bankinnskudd',
'bankkonto':'bankkonto',
'banklovgivingen':'banklovgiving',
'banklovkommisjonen':'banklovkommisjon',
'banksparing':'banksparing',
'banksystemet':'banksystem',
'banner':'bann',
'barbent':'barbent',
'barber':'barb',
'bare':'bar',
'barfot':'barfot',
'barkebit':'barkebit',
'barkebiten':'barkebit',
'barken':'bark',
'barlaup':'barlaup',
'barn':'barn',
'barna':'barn',
'barnas':'barn',
'barndomstraktene':'barndomstrakt',
'barne':'barn',
'barnebarn':'barnebarn',
'barnebarns':'barnebarn',
'barnefamiliar':'barnefamili',
'barnehagar':'barnehag',
'barnehage':'barnehag',
'barnehager':'barnehag',
'barnepass':'barnepass',
'barnepensjon':'barnepensjon',
'barnepensjonen':'barnepensjon',
'barnepensjonens':'barnepensjon',
'barnet':'barn',
'barnetrygd':'barnetrygd',
'barnets':'barn',
'barnløse':'barnløs',
'barre':'barr',
'barten':'bart',
'basbelopp':'basbelopp',
'basehavn':'basehavn',
'basen':'bas',
'basere':'baser',
'baserer':'baser',
'baseres':'baser',
'basert':'baser',
'baserte':'baser',
'basis':'basis',
'basisalternativet':'basisalternativ',
'basisår':'basisår',
'basisåret':'basisår',
'basketak':'basketak',
'bassenget':'basseng',
'bastet':'bast',
'baten':'bat',
'bauge':'baug',
'baugen':'baug',
'bauges':'baug',
'bbl':'bbl',
'be':'be',
'bearbeidede':'bearbeid',
'beboerdemokrati':'beboerdemokrati',
'beboerkantiner':'beboerkantin',
'bebuarane':'bebuar',
'bebuarar':'bebuar',
'bebuardemokrati':'bebuardemokrati',
'bebuaren':'bebuar',
'bebudede':'bebud',
'bebyggelsen':'bebygg',
'bebygget':'bebygg',
'bede':'bed',
'bedre':'bedr',
'bedres':'bedr',
'bedret':'bedr',
'bedrift':'bedrift',
'bedrifta':'bedrift',
'bedriften':'bedrift',
'bedriftene':'bedrift',
'bedriftenes':'bedrift',
'bedriftens':'bedrift',
'bedrifter':'bedrift',
'bedrifters':'bedrift',
'bedriftsforsamling':'bedriftsforsamling',
'bedriftsinterne':'bedriftsintern',
'bedriftsordningen':'bedriftsordning',
'bedriftsstørrelser':'bedriftsstørr',
'bedring':'bedring',
'bedrivas':'bedriv',
'bedt':'bedt',
'bedømme':'bedømm',
'bedømminga':'bedømming',
'bedømt':'bedømt',
'befaler':'befal',
'befaringene':'befaring',
'befaringer':'befaring',
'befinne':'befinn',
'befinner':'befinn',
'befolkede':'befolk',
'befolkningen':'befolkning',
'befolkningens':'befolkning',
'befolkningskonsentrasjon':'befolkningskonsentrasjon',
'befolkningsvekst':'befolkningsvekst',
'befri':'befri',
'begagna':'begagn',
'begeistret':'begeistr',
'beger':'beg',
'begeret':'beger',
'begge':'begg',
'begivenhet':'begiven',
'begjæring':'begjæring',
'begjært':'begjært',
'begrense':'begrens',
'begrensede':'begrens',
'begrenser':'begrens',
'begrenses':'begrens',
'begrenset':'begrens',
'begrensete':'begrenset',
'begrensing':'begrensing',
'begrensinger':'begrensing',
'begrensning':'begrensning',
'begrensningene':'begrensning',
'begrensninger':'begrensning',
'begrensningsregler':'begrensningsregl',
'begrep':'begrep',
'begrepene':'begrep',
'begreper':'begrep',
'begrepet':'begrep',
'begrepsbruk':'begrepsbruk',
'begriper':'begrip',
'begrunne':'begrunn',
'begrunnede':'begrunn',
'begrunnelse':'begrunn',
'begrunnelsen':'begrunn',
'begrunnelsene':'begrunn',
'begrunnelser':'begrunn',
'begrunner':'begrunn',
'begrunnet':'begrunn',
'begunstigede':'begunst',
'begunstigedes':'begunst',
'begunstiget':'begunst',
'begynne':'begynn',
'begynnelse':'begynn',
'begynnelsen':'begynn',
'begynnende':'begynn',
'begynner':'begynn',
'begynt':'begynt',
'begynte':'begynt',
'behag':'behag',
'behandla':'behandl',
'behandlar':'behandl',
'behandlast':'behandl',
'behandle':'behandl',
'behandler':'behandl',
'behandles':'behandl',
'behandlet':'behandl',
'behandling':'behandling',
'behandlinga':'behandling',
'behandlingen':'behandling',
'behandlingsmåte':'behandlingsmåt',
'behandlingspunkter':'behandlingspunkt',
'beheftet':'beheft',
'beherske':'behersk',
'behersker':'behersk',
'behold':'behold',
'beholde':'behold',
'beholder':'behold',
'beholdes':'behold',
'beholdning':'beholdning',
'behov':'behov',
'behova':'behov',
'behovene':'behov',
'behovet':'behov',
'behovsprøvet':'behovsprøv',
'behøver':'behøv',
'behøves':'behøv',
'beinkrok':'beinkrok',
'beist':'beist',
'beistet':'beist',
'beitemarker':'beitemark',
'beitet':'beit',
'bekjempe':'bekjemp',
'bekk':'bekk',
'bekkelaget':'bekkelag',
'bekkelagskai':'bekkelagskai',
'bekkelagskaia':'bekkelagskai',
'bekkelagskaiene':'bekkelagskai',
'bekkelagsområdet':'bekkelagsområd',
'bekostning':'bekostning',
'bekreftelse':'bekreft',
'bekrefter':'bekreft',
'bekymrer':'bekymr',
'bekymret':'bekymr',
'belaste':'belast',
'belastede':'belast',
'belastende':'belast',
'belaster':'belast',
'belastes':'belast',
'belastet':'belast',
'belastning':'belastning',
'belastningen':'belastning',
'belastningene':'belastning',
'belastninger':'belastning',
'belgen':'belg',
'beliggenhet':'beliggen',
'beltestedet':'beltested',
'beltet':'belt',
'belyse':'belys',
'belyses':'belys',
'belyst':'belyst',
'belønne':'belønn',
'beløp':'beløp',
'beløpe':'beløp',
'beløpene':'beløp',
'beløpet':'beløp',
'beløpets':'beløp',
'beløpsbegrenset':'beløpsbegrens',
'beløpsgrense':'beløpsgrens',
'beløpsmessig':'beløpsmess',
'bemerke':'bemerk',
'bemerker':'bemerk',
'bemerket':'bemerk',
'ben':'ben',
'bena':'ben',
'benefits':'benefit',
'benevnes':'benevn',
'benevningene':'benevning',
'benevnt':'benevnt',
'benkene':'benk',
'benkepryd':'benkepryd',
'benker':'benk',
'benkeradene':'benkerad',
'benket':'benk',
'benrester':'benrest',
'bensin':'bensin',
'bensplinter':'bensplint',
'bent':'bent',
'benytte':'benytt',
'benytter':'benytt',
'benyttes':'benytt',
'benyttet':'benytt',
'ber':'ber',
'berande':'ber',
'berar':'ber',
'berast':'ber',
'berdal':'berdal',
'bere':'ber',
'beredskap':'beredskap',
'beredskapsarbeidet':'beredskapsarbeid',
'beredskapshensyn':'beredskapshensyn',
'beredskapsmessig':'beredskapsmess',
'beredskapsmessige':'beredskapsmess',
'beredskapsplanlegging':'beredskapsplanlegging',
'bereevne':'bereevn',
'beregne':'beregn',
'beregnede':'beregn',
'beregner':'beregn',
'beregnes':'beregn',
'beregnet':'beregn',
'beregning':'beregning',
'beregningen':'beregning',
'beregningene':'beregning',
'beregninger':'beregning',
'beregningsarbeidet':'beregningsarbeid',
'beregningsgrunnlag':'beregningsgrunnlag',
'beregningsgrunnlaget':'beregningsgrunnlag',
'beregningsmessig':'beregningsmess',
'beregningsprinsippet':'beregningsprinsipp',
'beregningsrenten':'beregningsrent',
'beregningsresultatene':'beregningsresultat',
'beregningsteknisk':'beregningsteknisk',
'beregningstidspunktet':'beregningstidspunkt',
'berekraftig':'berekraft',
'beretninger':'beretning',
'berette':'berett',
'beretter':'berett',
'berettige':'berett',
'berettigede':'berett',
'berettiger':'berett',
'berettiget':'berett',
'berg':'berg',
'berge':'berg',
'bergelmer':'bergelm',
'bergen':'berg',
'berget':'berg',
'bergets':'berg',
'bergknauser':'bergknaus',
'bergrise':'bergris',
'bergriser':'bergris',
'bergtatt':'bergtatt',
'bero':'bero',
'beror':'beror',
'berre':'berr',
'berserk':'berserk',
'beruset':'berus',
'berømmelse':'berømm',
'berømt':'berømt',
'berøre':'berør',
'berører':'berør',
'berøres':'berør',
'berørt':'berørt',
'berørte':'berørt',
'bes':'bes',
'beseiret':'beseir',
'besk':'besk',
'beskatning':'beskatning',
'beskatningen':'beskatning',
'beskatningsmyndigheten':'beskatningsmynd',
'beskatningsprinsipp':'beskatningsprinsipp',
'beskatningsprinsippet':'beskatningsprinsipp',
'beskatte':'beskatt',
'beskattede':'beskatt',
'beskattes':'beskatt',
'beskattet':'beskatt',
'beskjed':'beskjed',
'beskjeden':'beskjed',
'beskjedent':'beskjedent',
'beskjedne':'beskjedn',
'beskrankninger':'beskrankning',
'beskrevet':'beskrev',
'beskrivast':'beskriv',
'beskrive':'beskriv',
'beskrivelse':'beskriv',
'beskrivelser':'beskriv',
'beskrivende':'beskriv',
'beskriver':'beskriv',
'beskrives':'beskriv',
'beskrivingar':'beskriving',
'beskytte':'beskytt',
'beskytter':'beskytt',
'beslag':'beslag',
'beslutning':'beslutning',
'beslutningen':'beslutning',
'beslutningene':'beslutning',
'beslutninger':'beslutning',
'beslutningsgrunnlag':'beslutningsgrunnlag',
'beslutningskjeden':'beslutningskjed',
'beslutningsmyndighet':'beslutningsmynd',
'beslutningsorganer':'beslutningsorgan',
'beslutningsproblem':'beslutningsproblem',
'beslutningsprosess':'beslutningsprosess',
'beslutningsprosessen':'beslutningsprosess',
'beslutningsrekkefølgen':'beslutningsrekkefølg',
'beslutningsrekkene':'beslutningsrekk',
'beslutningstaker':'beslutningstak',
'beslutningsvariabel':'beslutningsvariabel',
'beslutningsvariabelen':'beslutningsvariabel',
'beslutte':'beslutt',
'besluttende':'beslutt',
'beslutter':'beslutt',
'best':'best',
'bestandig':'bestand',
'beste':'best',
'besteme':'bestem',
'bestemme':'bestemm',
'bestemmelse':'bestemm',
'bestemmelsen':'bestemm',
'bestemmelsene':'bestemm',
'bestemmelsens':'bestemm',
'bestemmelser':'bestemm',
'bestemmelseshavn':'bestemmelseshavn',
'bestemmer':'bestemm',
'bestemmes':'bestemm',
'bestemor':'bestemor',
'bestemt':'bestemt',
'bestemte':'bestemt',
'bestevilkårsprinsippet':'bestevilkårsprinsipp',
'bestille':'bestill',
'bestilling':'bestilling',
'bestla':'bestl',
'besto':'besto',
'bestod':'bestod',
'bestå':'bestå',
'bestående':'bestå',
'består':'består',
'bestått':'bestått',
'besvare':'besvar',
'besvart':'besvart',
'besvergelser':'besverg',
'besvimer':'besvim',
'besøk':'besøk',
'besøke':'besøk',
'besøker':'besøk',
'besøket':'besøk',
'bet':'bet',
'betalast':'betal',
'betale':'betal',
'betaler':'betal',
'betales':'betal',
'betaling':'betaling',
'betalinga':'betaling',
'betalingen':'betaling',
'betalingsblankettar':'betalingsblankett',
'betalingsevne':'betalingsevn',
'betalingsforpliktelser':'betalingsforplikt',
'betalingsmishaldet':'betalingsmishald',
'betalingsperioder':'betalingsperiod',
'betalingsplikt':'betalingsplikt',
'betalingssystema':'betalingssystem',
'betalingstid':'betalingstid',
'betalt':'betalt',
'betalte':'betalt',
'betegnelse':'betegn',
'betegnelsen':'betegn',
'betegner':'betegn',
'betegnes':'betegn',
'betegnet':'betegn',
'betenkte':'betenkt',
'betingede':'beting',
'betingelse':'beting',
'betingelsene':'beting',
'betingelser':'beting',
'betinger':'beting',
'betinget':'beting',
'betjene':'betj',
'betjener':'betjen',
'betjenes':'betj',
'betjening':'betjening',
'betjeningen':'betjening',
'betrakte':'betrakt',
'betraktelig':'betrakt',
'betraktelige':'betrakt',
'betrakter':'betrakt',
'betraktes':'betrakt',
'betraktning':'betraktning',
'betraktningen':'betraktning',
'betraktninger':'betraktning',
'betraktningsmåte':'betraktningsmåt',
'betraktningsmåten':'betraktningsmåt',
'betre':'betr',
'betreffend':'betreffend',
'betrodde':'betrodd',
'betror':'betror',
'betryggende':'betrygg',
'bety':'bety',
'betydeleg':'betyd',
'betydelig':'betyd',
'betydelige':'betyd',
'betydning':'betydning',
'betydningen':'betydning',
'betydninger':'betydning',
'betyr':'betyr',
'beundrer':'beundr',
'beuthien':'beuthi',
'bevegelse':'beveg',
'bevegelsene':'beveg',
'beveger':'beveg',
'bevilge':'bevilg',
'bevilgede':'bevilg',
'bevis':'bevis',
'beviset':'bevis',
'bevisreglar':'bevisregl',
'bevisstløs':'bevisstløs',
'bevøkollen':'bevøkoll',
'bezwecken':'bezweck',
'bhv':'bhv',
'bhvs':'bhvs',
'bibeholder':'bibehold',
'bibliotek':'bibliotek',
'bidra':'bidr',
'bidrag':'bidrag',
'bidraget':'bidrag',
'bidrar':'bidr',
'bidratt':'bidratt',
'bidro':'bidro',
'biene':'bien',
'bifrost':'bifrost',
'bikkje':'bikkj',
'bikkjer':'bikkj',
'bil':'bil',
'bilde':'bild',
'bildene':'bild',
'bilder':'bild',
'bildet':'bild',
'biled':'biled',
'biler':'bil',
'biletet':'bilet',
'bilferjer':'bilferj',
'bilimport':'bilimport',
'bilindustrien':'bilindustri',
'billedspråk':'billedspråk',
'billeg':'bil',
'billettsalg':'billettsalg',
'billig':'bil',
'billigere':'billiger',
'billing':'billing',
'billings':'billing',
'bilskirne':'bilskirn',
'biltrafikk':'biltrafikk',
'biltransport':'biltransport',
'bindande':'bind',
'binde':'bind',
'bindende':'bind',
'binder':'bind',
'bindes':'bind',
'bindinga':'binding',
'bindingen':'binding',
'bindingsliste':'bindingslist',
'bindingslister':'bindingslist',
'bindingstid':'bindingstid',
'bindingstida':'bindingstid',
'bindingstider':'bindingstid',
'bingen':'bing',
'biologisk':'biologisk',
'bisk':'bisk',
'bispevika':'bispevik',
'bisselet':'bissel',
'bistand':'bistand',
'bistert':'bister',
'bistå':'bistå',
'bit':'bit',
'bite':'bit',
'bitene':'bit',
'biter':'bit',
'bitre':'bitr',
'bitt':'bitt',
'bitte':'bitt',
'bitter':'bitt',
'bittert':'bitter',
'bjeff':'bjeff',
'bjeffer':'bjeff',
'bjelkelag':'bjelkelag',
'bjelker':'bjelk',
'bjelkeveggene':'bjelkevegg',
'bjella':'bjell',
'bjerkeris':'bjerkeris',
'bjørn':'bjørn',
'bjørnens':'bjørn',
'bjørvika':'bjørvik',
'bjørvikatunnelen':'bjørvikatunnel',
'bl':'bl',
'bladene':'blad',
'blader':'blad',
'blanda':'bland',
'blandast':'bland',
'blande':'bland',
'blander':'bland',
'blandet':'bland',
'blandingen':'blanding',
'blandingsforma':'blandingsform',
'blandingsformer':'blandingsform',
'blandt':'bland',
'blanke':'blank',
'blankettar':'blankett',
'blant':'blant',
'ble':'ble',
'blek':'blek',
'bleke':'blek',
'blekksprut':'blekksprut',
'blekner':'blekn',
'bli':'bli',
'blikk':'blikk',
'blikket':'blikk',
'blind':'blind',
'blinde':'blind',
'blink':'blink',
'blinker':'blink',
'blir':'blir',
'blitt':'blitt',
'blod':'blod',
'blodbadet':'blodbad',
'blodet':'blod',
'blodfeide':'blodfeid',
'blodig':'blod',
'blodige':'blod',
'blodighov':'blodighov',
'blodskutte':'blodskutt',
'blodsprengte':'blodsprengt',
'blokka':'blokk',
'blokker':'blokk',
'blokkstein':'blokkstein',
'blokktog':'blokktog',
'blokktogavganger':'blokktogavgang',
'blokktogforbindelse':'blokktogforbind',
'blomst':'blomst',
'blomster':'blomst',
'blomsterkranser':'blomsterkrans',
'blond':'blond',
'blunker':'blunk',
'blå':'blå',
'blåfrosne':'blåfrosn',
'blåklokker':'blåklokk',
'blåmalte':'blåmalt',
'blåner':'blån',
'blåse':'blås',
'blåsebelgen':'blåsebelg',
'blåser':'blås',
'blåsingen':'blåsing',
'blåst':'blåst',
'blåsvart':'blåsvart',
'blør':'blør',
'bløt':'bløt',
'bnp':'bnp',
'bo':'bo',
'board':'board',
'bob':'bob',
'bobler':'bobl',
'bod':'bod',
'bodd':'bodd',
'bodet':'bod',
'bodrunden':'bodrund',
'bodø':'bodø',
'boende':'boend',
'boet':'boet',
'boets':'boet',
'boforhold':'boforhold',
'boforholdsundersøkelsen':'boforholdsundersøk',
'boger':'bog',
'boh':'boh',
'bointeresser':'bointeress',
'boka':'bok',
'bokn':'bokn',
'bokreserver':'bokreserv',
'boks':'boks',
'bokstav':'bokstav',
'bokstaver':'bokstav',
'bokstavlig':'bokstav',
'bokstavtro':'bokstavtro',
'bol':'bol',
'bolig':'bol',
'boligarealer':'boligareal',
'boligbebyggelse':'boligbebygg',
'boligbygg':'boligbygg',
'boligbyggelag':'boligbyggelag',
'boligbyggelaget':'boligbyggelag',
'boligbyggelags':'boligbyggelag',
'boligbyggelagsloven':'boligbyggelag',
'boligdekning':'boligdekning',
'boligen':'bol',
'boligene':'bol',
'boliger':'bol',
'boligetablering':'boligetablering',
'boligfeltet':'boligfelt',
'boligfordeling':'boligfordeling',
'boligforetak':'boligforetak',
'boligfællesskaber':'boligfællesskab',
'boligkonsentrasjoner':'boligkonsentrasjon',
'boliglov':'bolig',
'boliglover':'bolig',
'boligmarkedet':'boligmarked',
'boligmassen':'boligmass',
'boligområdene':'boligområd',
'boligområder':'boligområd',
'boligområdet':'boligområd',
'boligpolitikk':'boligpolitikk',
'boligpolitikken':'boligpolitikk',
'boligpolitiske':'boligpolitisk',
'boligsameier':'boligsamei',
'boligsamvirket':'boligsamvirk',
'boligsektoren':'boligsektor',
'boligselskabernes':'boligselskabern',
'boligselskap':'boligselskap',
'boligselskaper':'boligselskap',
'boligsjef':'boligsjef',
'bollingmo':'bollingmo',
'bolt':'bolt',
'bolten':'bolt',
'boltet':'bolt',
'boltorns':'boltorn',
'bolverk':'bolverk',
'bom':'bom',
'bombinga':'bombing',
'bomiljø':'bomiljø',
'bomiljøer':'bomiljø',
'bomiljøet':'bomiljø',
'bommestad':'bommestad',
'bompenger':'bompeng',
'bondekone':'bondekon',
'bonden':'bond',
'bondens':'bond',
'boområder':'boområd',
'boområdet':'boområd',
'bopel':'bopel',
'bor':'bor',
'bord':'bord',
'bordene':'bord',
'bordet':'bord',
'bordkant':'bordkant',
'bordplaten':'bordplat',
'bords':'bord',
'bore':'bor',
'borealis':'borealis',
'borehullet':'borehull',
'boret':'bor',
'borettshaverne':'borettshavern',
'borettslag':'borettslag',
'borettslagene':'borettslag',
'borettslagenes':'borettslag',
'borettslaget':'borettslag',
'borettslagsandel':'borettslagsandel',
'borettslagsandelen':'borettslagsandel',
'borettslagsloven':'borettslag',
'borettsloven':'borett',
'borettslovene':'borett',
'borettslovkomiteen':'borettslovkomite',
'borettslovutvalget':'borettslovutvalg',
'borettsregister':'borettsregist',
'borettsregisteret':'borettsregister',
'borg':'borg',
'borge':'borg',
'borgen':'borg',
'borgere':'borger',
'borgerne':'borgern',
'borgestad':'borgestad',
'borgmuren':'borgmur',
'borgporten':'borgport',
'borgs':'borg',
'born':'born',
'borna':'born',
'borre':'borr',
'borregaard':'borregaard',
'bort':'bort',
'borte':'bort',
'bortenfor':'bortenfor',
'bortfall':'bortfall',
'bortfaller':'bortfall',
'bortfallet':'bortfall',
'bortforklare':'bortforklar',
'borti':'borti',
'bortkomne':'bortkomn',
'bortleige':'bortl',
'bortlån':'bortlån',
'bortsett':'bortsett',
'borttaking':'borttaking',
'bosetter':'bosett',
'bosetting':'bosetting',
'bosettingen':'bosetting',
'bosettingsmønsteret':'bosettingsmønster',
'bostadrattsforeningar':'bostadrattsforening',
'bostadrattsforeninger':'bostadrattsforening',
'bostadsforeningars':'bostadsforeningar',
'bostadsforetag':'bostadsforetag',
'bostadsforetags':'bostadsforetag',
'bostadsratt':'bostadsratt',
'bostadsratter':'bostadsratt',
'bostadsrattforeningar':'bostadsrattforening',
'bostadsratthavare':'bostadsratthavar',
'bostadsrattsforening':'bostadsrattsforening',
'bostadsrattsforeningar':'bostadsrattsforening',
'bostadsrattsforeningarna':'bostadsrattsforeningarn',
'bostadsrattsforeningen':'bostadsrattsforening',
'bostadsrattsforeninger':'bostadsrattsforening',
'bostadsrattshavarna':'bostadsrattshavarn',
'bostadsrattslagen':'bostadsrattslag',
'bostadsrattsregister':'bostadsrattsregist',
'bostadsrattsutredningen':'bostadsrattsutredning',
'bostadsrettsforening':'bostadsrettsforening',
'bot':'bot',
'boten':'bot',
'botnen':'botn',
'botrygghet':'botrygg',
'bottom':'bottom',
'bra':'bra',
'brage':'brag',
'bragernesløpet':'bragernesløp',
'bragernesutløpet':'bragernesutløp',
'bragt':'bragt',
'brak':'brak',
'brakerøya':'brakerøy',
'brakkvann':'brakkvann',
'brann':'brann',
'brannsår':'brannsår',
'branntomt':'branntomt',
'bransjenemnder':'bransjenemnd',
'bransjeorganisasjoner':'bransjeorganisasjon',
'bransjer':'bransj',
'bratsbergbanen':'bratsbergban',
'bratte':'bratt',
'brattegard':'brattegard',
'brattstrom':'brattstrom',
'bred':'bred',
'bredbent':'bredbent',
'bredde':'bredd',
'bredden':'bredd',
'brede':'bred',
'bredere':'breder',
'bredt':'bredt',
'brei':'brei',
'breidablikk':'breidablikk',
'breidd':'breidd',
'breidda':'breidd',
'breie':'breie',
'breitt':'breitt',
'breker':'brek',
'brekke':'brekk',
'bremerhaven':'bremerhav',
'brenn':'brenn',
'brenne':'brenn',
'brennende':'brenn',
'brenner':'brenn',
'brent':'brent',
'brer':'brer',
'brett':'brett',
'bretter':'brett',
'brev':'brev',
'breva':'brev',
'brevik':'brevik',
'breviksterminalen':'breviksterminal',
'brevikstrømmen':'brevikstrømm',
'brevikterminalen':'brevikterminal',
'brime':'brim',
'bring':'bring',
'bringe':'bring',
'bringer':'bring',
'bringes':'bring',
'bringsværd':'bringsværd',
'brisinga':'brising',
'briste':'brist',
'brister':'brist',
'britiske':'britisk',
'brl':'brl',
'bro':'bro',
'broen':'broen',
'brokk':'brokk',
'brokks':'brokk',
'bror':'bror',
'broren':'bror',
'brorparten':'brorpart',
'brors':'bror',
'brot':'brot',
'broten':'brot',
'bru':'bru',
'brua':'brua',
'brualternativet':'brualternativ',
'brud':'brud',
'brudd':'brudd',
'brudeferd':'brudeferd',
'brudegave':'brudegav',
'brudelin':'brudelin',
'brudelinet':'brudelin',
'bruden':'brud',
'brudens':'brud',
'brudeparet':'brudepar',
'brudepike':'brudepik',
'brudepiken':'brudepik',
'bruer':'bruer',
'brugsforeninger':'brugsforening',
'bruk':'bruk',
'brukar':'bruk',
'brukarane':'brukar',
'brukarar':'brukar',
'brukareige':'brukar',
'brukareigeprinsipp':'brukareigeprinsipp',
'brukareigeprinsippet':'brukareigeprinsipp',
'brukaren':'brukar',
'brukarens':'brukar',
'brukarhusstanden':'brukarhusstand',
'brukast':'bruk',
'brukbar':'brukb',
'brukbart':'brukbart',
'bruke':'bruk',
'bruken':'bruk',
'bruker':'bruk',
'brukeravgiftene':'brukeravgift',
'brukere':'bruker',
'brukeren':'bruker',
'brukergrupper':'brukergrupp',
'brukerinteressene':'brukerinteress',
'brukerne':'brukern',
'brukerorganisasjonene':'brukerorganisasjon',
'brukerrepresentanter':'brukerrepresentant',
'brukes':'bruk',
'brukseining':'brukseining',
'bruksnummer':'bruksnumm',
'bruksoverlating':'bruksoverlating',
'bruksoverlatinga':'bruksoverlating',
'bruksovertaking':'bruksovertaking',
'bruksregulering':'bruksregulering',
'bruksrett':'bruksrett',
'bruksrettane':'bruksrett',
'bruksrettar':'bruksrett',
'bruksretten':'bruksrett',
'bruksrettsforhold':'bruksrettsforhold',
'bruksrettsforholdet':'bruksrettsforhold',
'brukt':'brukt',
'brukte':'brukt',
'brummer':'brumm',
'brundtland':'brundtland',
'brune':'brun',
'brunrøde':'brunrød',
'brunst':'brunst',
'brutto':'brutto',
'bruttobeskattes':'bruttobeskatt',
'bruttoformue':'bruttoformu',
'bry':'bry',
'brydd':'brydd',
'bryet':'bryet',
'brygge':'brygg',
'bryggekar':'bryggek',
'bryggekaret':'bryggekar',
'brygger':'brygg',
'brygget':'brygg',
'bryllups':'bryllup',
'bryllupsgjest':'bryllupsgjest',
'bryllupsgjestene':'bryllupsgjest',
'bryllupsreise':'bryllupsreis',
'bryn':'bryn',
'bryne':'bryn',
'brynene':'bryn',
'brynestein':'brynestein',
'brynesten':'brynest',
'brynestenen':'brynesten',
'brynet':'bryn',
'brynje':'brynj',
'brynjekledde':'brynjekledd',
'brynjer':'brynj',
'brynjulv':'brynjulv',
'bryr':'bryr',
'bryst':'bryst',
'brystene':'bryst',
'bryster':'bryst',
'brystet':'bryst',
'bryte':'bryt',
'brytekamp':'brytekamp',
'brytekampen':'brytekamp',
'bryter':'bryt',
'brytes':'bryt',
'brå':'brå',
'bråk':'bråk',
'bråker':'bråk',
'bråket':'bråk',
'bråsinte':'bråsint',
'bråthen':'bråth',
'brått':'brått',
'bråvender':'bråvend',
'bråvåkner':'bråvåkn',
'brød':'brød',
'brødre':'brødr',
'brødrene':'brødr',
'brøkdel':'brøkdel',
'brøken':'brøk',
'brøle':'brøl',
'brøler':'brøl',
'brønn':'brønn',
'brønnen':'brønn',
'brønner':'brønn',
'brønnøysundregistra':'brønnøysundregistr',
'brøt':'brøt',
'bu':'bu',
'buande':'buand',
'bud':'bud',
'budde':'budd',
'budsjett':'budsjett',
'budsjettere':'budsjetter',
'budsjetteringa':'budsjettering',
'budsjettet':'budsjett',
'budsjettproposisjonane':'budsjettproposisjon',
'budsjettsaker':'budsjettsak',
'budskapet':'budskap',
'budt':'budt',
'budtjeneste':'budtjenest',
'bue':'bue',
'buen':'buen',
'buer':'buer',
'bueskyting':'bueskyting',
'bueskytter':'bueskytt',
'buestrengen':'buestreng',
'buet':'buet',
'buffer':'buff',
'buffere':'buffer',
'buform':'buform',
'buformer':'buform',
'buinteressene':'buinteress',
'buinteresser':'buinteress',
'bukke':'bukk',
'bukkekjerra':'bukkekjerr',
'bukkene':'bukk',
'bukker':'bukk',
'bukkeskinnene':'bukkeskinn',
'bukostnadene':'bukostnad',
'buktende':'bukt',
'bukter':'bukt',
'buktet':'bukt',
'buldrende':'buldr',
'buldrer':'buldr',
'bulk':'bulk',
'bulkanlegg':'bulkanlegg',
'bulkfartøyer':'bulkfartøy',
'bulkgods':'bulkgod',
'bulkhavn':'bulkhavn',
'bulkklassifiserte':'bulkklassifiser',
'bulklast':'bulkl',
'bulklaster':'bulklast',
'bulkprodukter':'bulkprodukt',
'bulktrafikken':'bulktrafikk',
'bulktransporter':'bulktransport',
'bulkvare':'bulkvar',
'bulkvarer':'bulkvar',
'bulmeurt':'bulmeurt',
'bumiljø':'bumiljø',
'bumiljøarbeid':'bumiljøarbeid',
'bumiljøet':'bumiljø',
'bumiljøtiltak':'bumiljøtiltak',
'bunde':'bund',
'bunden':'bund',
'bundesverband':'bundesverband',
'bundet':'bund',
'bundne':'bundn',
'bunnen':'bunn',
'bunner':'bunn',
'bunnfradraget':'bunnfradrag',
'bunnløst':'bunnløst',
'bunns':'bunn',
'buofl':'buofl',
'buplikt':'buplikt',
'bur':'bur',
'burde':'burd',
'bure':'bur',
'bures':'bur',
'burett':'burett',
'buretten':'burett',
'burettsbustader':'burettsbustad',
'burettshavar':'burettshav',
'burettshavarane':'burettshavar',
'burettshavarar':'burettshavar',
'burettshavaren':'burettshavar',
'burettshavarinteressene':'burettshavarinteress',
'burettsinnskot':'burettsinnskot',
'burettsinnskota':'burettsinnskot',
'burettslag':'burettslag',
'burettslaga':'burettslag',
'burettslagandel':'burettslagandel',
'burettslagbustader':'burettslagbustad',
'burettslaget':'burettslag',
'burettslagets':'burettslag',
'burettslagsandel':'burettslagsandel',
'burettslagsandelane':'burettslagsandel',
'burettslagsandelar':'burettslagsandel',
'burettslagsandelen':'burettslagsandel',
'burettslagsbustad':'burettslagsbustad',
'burettslagsbustadene':'burettslagsbustad',
'burettslagsbustader':'burettslagsbustad',
'burettslagsforhold':'burettslagsforhold',
'burettslagsforma':'burettslagsform',
'burettslagslov':'burettslag',
'burettslagslova':'burettslag',
'burettslagsmidlar':'burettslagsmidl',
'burettslagsmodell':'burettslagsmodell',
'burettslagsmodellen':'burettslagsmodell',
'burettslov':'burett',
'burettslovene':'burett',
'burettslovgjeving':'burettslovgjeving',
'burettslovgjevinga':'burettslovgjeving',
'burettslovutvalet':'burettslovutval',
'burettsregister':'burettsregist',
'burettsregisteret':'burettsregister',
'buseti':'buseti',
'busituasjon':'busituasjon',
'busituasjonen':'busituasjon',
'busk':'busk',
'buskapen':'buskap',
'busker':'busk',
'buskerud':'buskerud',
'bustad':'bustad',
'bustadaksjeselskap':'bustadaksjeselskap',
'bustadaksjeselskapet':'bustadaksjeselskap',
'bustadar':'bustad',
'bustadbank':'bustadbank',
'bustadbehov':'bustadbehov',
'bustadbehova':'bustadbehov',
'bustadbygg':'bustadbygg',
'bustadbygga':'bustadbygg',
'bustadbygging':'bustadbygging',
'bustadbygginga':'bustadbygging',
'bustadbyggjelag':'bustadbyggjelag',
'bustadbyggjelaga':'bustadbyggjelag',
'bustadbyggjelaget':'bustadbyggjelag',
'bustadbyggjelagsforma':'bustadbyggjelagsform',
'bustadbyggjelagslov':'bustadbyggjelag',
'bustadbyggjelagslova':'bustadbyggjelag',
'bustadbyte':'bustadbyt',
'bustadeigarar':'bustadeigar',
'bustadeigedom':'bustadeigedom',
'bustaden':'bustad',
'bustadene':'bustad',
'bustadeneburetten':'bustadeneburett',
'bustader':'bustad',
'bustadetablering':'bustadetablering',
'bustadfellesskapar':'bustadfellesskap',
'bustadfellesskapen':'bustadfellesskap',
'bustadfinansieringa':'bustadfinansiering',
'bustadforbruket':'bustadforbruk',
'bustadfordelinga':'bustadfordeling',
'bustadforhold':'bustadforhold',
'bustadform':'bustadform',
'bustadformene':'bustadform',
'bustadformer':'bustadform',
'bustadfornying':'bustadfornying',
'bustadforsyning':'bustadforsyning',
'bustadforsyninga':'bustadforsyning',
'bustadforvaltning':'bustadforvaltning',
'bustadforvaltninga':'bustadforvaltning',
'bustadføremål':'bustadføremål',
'bustadføremålet':'bustadføremål',
'bustadføretak':'bustadføretak',
'bustadføretaka':'bustadføretak',
'bustadinndelinga':'bustadinndeling',
'bustadinteressentane':'bustadinteressent',
'bustadinteressenten':'bustadinteressent',
'bustadinteressentskap':'bustadinteressentskap',
'bustadkjøparar':'bustadkjøpar',
'bustadkonsumet':'bustadkonsum',
'bustadkooperasjonen':'bustadkooperasjon',
'bustadkooperativ':'bustadkooperativ',
'bustadkooperativa':'bustadkooperativ',
'bustadkooperative':'bustadkooperativ',
'bustadkooperativet':'bustadkooperativ',
'bustadkooperativt':'bustadkooperativ',
'bustadlause':'bustadlaus',
'bustadleige':'bustadl',
'bustadlov':'bustad',
'bustadlovene':'bustad',
'bustadlovgjevinga':'bustadlovgjeving',
'bustadmarknad':'bustadmarknad',
'bustadmarknaden':'bustadmarknad',
'bustadmasse':'bustadmass',
'bustadmeldingane':'bustadmelding',
'bustadmiljø':'bustadmiljø',
'bustadområde':'bustadområd',
'bustadområdet':'bustadområd',
'bustadomsetninga':'bustadomsetning',
'bustadoppføringslova':'bustadoppføring',
'bustadorganisasjon':'bustadorganisasjon',
'bustadorganisasjonane':'bustadorganisasjon',
'bustadorganisasjonar':'bustadorganisasjon',
'bustadorganisering':'bustadorganisering',
'bustadpolitikk':'bustadpolitikk',
'bustadpolitikken':'bustadpolitikk',
'bustadpolitiske':'bustadpolitisk',
'bustadprisane':'bustadpris',
'bustadproduksjon':'bustadproduksjon',
'bustadproduksjonen':'bustadproduksjon',
'bustadprosjekt':'bustadprosjekt',
'bustadreglane':'bustadregl',
'bustadrett':'bustadrett',
'bustadsamanslutningane':'bustadsamanslutning',
'bustadsameiger':'bustadsam',
'bustadsamvirke':'bustadsamvirk',
'bustadsamvirket':'bustadsamvirk',
'bustadsamvirkets':'bustadsamvirk',
'bustadseksjon':'bustadseksjon',
'bustadseksjonane':'bustadseksjon',
'bustadseksjonar':'bustadseksjon',
'bustadsituasjon':'bustadsituasjon',
'bustadskaffing':'bustadskaffing',
'bustadsosiale':'bustadsosial',
'bustadspørsmåla':'bustadspørsmål',
'bustadstiftingar':'bustadstifting',
'bustadtilbod':'bustadtilbod',
'bustadverksemd':'bustadverksemd',
'bustehode':'bustehod',
'busten':'bust',
'bustønad':'bustønad',
'butikkar':'butikk',
'butilhøva':'butilhøv',
'butilhøve':'butilhøv',
'butt':'butt',
'buutgifter':'buutgift',
'by':'by',
'byane':'byan',
'byar':'byar',
'bybroen':'bybro',
'bydel':'bydel',
'byen':'byen',
'byene':'byen',
'byenes':'byen',
'byens':'byen',
'byer':'byer',
'byformål':'byformål',
'byfornying':'byfornying',
'byfornyinga':'byfornying',
'byfunksjoner':'byfunksjon',
'bygd':'bygd',
'bygda':'bygd',
'bygde':'bygd',
'bygder':'bygd',
'bygg':'bygg',
'bygga':'bygg',
'bygge':'bygg',
'byggemarked':'byggemarked',
'bygger':'bygg',
'bygges':'bygg',
'byggesoner':'byggeson',
'bygget':'bygg',
'byggetrinn':'byggetrinn',
'byggevirksomhet':'byggevirksom',
'byggherre':'byggherr',
'byggherrecentral':'byggherrecentral',
'byggherren':'byggherr',
'byggherreombod':'byggherreombod',
'byggherresida':'byggherresid',
'bygging':'bygging',
'bygginga':'bygging',
'byggjast':'byggj',
'byggje':'byggj',
'byggjebudsjett':'byggjebudsjett',
'byggjebudsjettet':'byggjebudsjett',
'byggjefeil':'byggjefeil',
'byggjefelt':'byggjefelt',
'byggjeforskriftene':'byggjeforskrift',
'byggjeklare':'byggjeklar',
'byggjekostnader':'byggjekostnad',
'byggjelån':'byggjelån',
'byggjematerialar':'byggjematerial',
'byggjeoppdrag':'byggjeoppdrag',
'byggjeperioden':'byggjeperiod',
'byggjeprosess':'byggjeprosess',
'byggjeprosessen':'byggjeprosess',
'byggjeprosjekt':'byggjeprosjekt',
'byggjeprosjekta':'byggjeprosjekt',
'byggjeprosjektet':'byggjeprosjekt',
'byggjer':'byggj',
'byggjerekneskapen':'byggjerekneskap',
'byggjesak':'byggjesak',
'byggjesaka':'byggjesak',
'byggjeskikk':'byggjeskikk',
'byggjestyret':'byggjestyr',
'byggjetekniske':'byggjeteknisk',
'byggjetida':'byggjetid',
'byggjeverksemda':'byggjeverksemd',
'byggkvalitet':'byggkvalit',
'byggmester':'byggmest',
'byggmesteren':'byggmester',
'byggnadsforening':'byggnadsforening',
'byggøkonomi':'byggøkonomi',
'bygning':'bygning',
'bygningar':'bygning',
'bygningen':'bygning',
'bygninger':'bygning',
'bygningsetaten':'bygningsetat',
'bygningsetatens':'bygningsetat',
'bygningskrav':'bygningskrav',
'bygningsloven':'bygning',
'bygningslovens':'bygning',
'bygningslovgjevinga':'bygningslovgjeving',
'bygningsmateriale':'bygningsmaterial',
'bygt':'bygt',
'bykassen':'bykass',
'bykjernen':'bykjern',
'byleist':'byleist',
'bymessig':'bymess',
'bymiljø':'bymiljø',
'byområdene':'byområd',
'byområder':'byområd',
'byområdet':'byområd',
'bypass':'bypass',
'byr':'byr',
'byrden':'byrd',
'byrelaterte':'byrelater',
'byrett':'byrett',
'byrje':'byrj',
'byrådet':'byråd',
'byrådsbehandling':'byrådsbehandling',
'byrådsmøte':'byrådsmøt',
'byrådssak':'byrådssak',
'byrådsvedtak':'byrådsvedtak',
'bys':'bys',
'bysamfunn':'bysamfunn',
'bysamfunnene':'bysamfunn',
'bysamfunnet':'bysamfunn',
'bysentraene':'bysentra',
'byspredning':'byspredning',
'bystruktur':'bystruktur',
'bystrøk':'bystrøk',
'bystyre':'bystyr',
'bystyret':'bystyr',
'bystyrevedtak':'bystyrevedtak',
'bystyrevedtaket':'bystyrevedtak',
'bytast':'byt',
'byte':'byt',
'byteforhold':'byteforhold',
'bytte':'bytt',
'byttet':'bytt',
'byutvikling':'byutvikling',
'byutviklingen':'byutvikling',
'byutviklings':'byutvikling',
'byutviklingsformål':'byutviklingsformål',
'byutviklingsmønster':'byutviklingsmønst',
'byutviklingsområder':'byutviklingsområd',
'byutviklingsplaner':'byutviklingsplan',
'byutviklingstankegang':'byutviklingstankegang',
'både':'båd',
'bål':'bål',
'bålene':'bål',
'bålet':'bål',
'bålferden':'bålferd',
'båløyg':'båløyg',
'bånd':'bånd',
'båndet':'bånd',
'båret':'bår',
'båt':'båt',
'båten':'båt',
'båter':'båt',
'båtplass':'båtplass',
'båtripa':'båtrip',
'bære':'bær',
'bærebjelker':'bærebjelk',
'bæreevne':'bæreevn',
'bærekraftig':'bærekraft',
'bærende':'bær',
'bærer':'bær',
'bæres':'bær',
'bærum':'bærum',
'bøe':'bøe',
'bøen':'bøen',
'bøkene':'bøk',
'bøker':'bøk',
'bølgene':'bølg',
'bølger':'bølg',
'bølgje':'bølgj',
'bønder':'bønd',
'bønnfaller':'bønnfall',
'bønnfalt':'bønnfalt',
'bør':'bør',
'børge':'børg',
'børs':'bør',
'børstet':'børst',
'bøte':'bøt',
'bøter':'bøt',
'bøtter':'bøtt',
'bøyd':'bøyd',
'bøye':'bøye',
'bøyer':'bøyer',
'bøyes':'bøyes',
'bøyet':'bøyet',
'bøyla':'bøyl',
'c':'c',
'ca':'ca',
'cargo':'cargo',
'cargos':'cargo',
'carl':'carl',
'cecodhas':'cecodh',
'cellulose':'cellulos',
'centralforening':'centralforening',
'chilenske':'chilensk',
'christina':'christin',
'christine':'christin',
'cif':'cif',
'clay':'clay',
'cluster':'clust',
'clusteret':'cluster',
'co':'co',
'coloroll':'coloroll',
'com':'com',
'comite':'comit',
'common':'common',
'compensation':'compensation',
'confederazione':'confederazion',
'container':'contain',
'containeraktivitet':'containeraktivit',
'containeraktivitetene':'containeraktivitet',
'containerareal':'containerareal',
'containerarealene':'containerareal',
'containerarealer':'containerareal',
'containerarealet':'containerareal',
'containerdepotene':'containerdepot',
'containere':'container',
'containeren':'container',
'containerfordeling':'containerfordeling',
'containergods':'containergod',
'containergodset':'containergods',
'containerhavn':'containerhavn',
'containerhavnen':'containerhavn',
'containerhavnene':'containerhavn',
'containerhavner':'containerhavn',
'containerhåndtering':'containerhåndtering',
'containerhåndteringen':'containerhåndtering',
'containerhåndteringskapasitet':'containerhåndteringskapasit',
'containerhåndteringsområdet':'containerhåndteringsområd',
'containerisere':'containeriser',
'containerisering':'containerisering',
'containeriseringsgrad':'containeriseringsgrad',
'containeriseringsgraden':'containeriseringsgrad',
'containerisert':'containeriser',
'containeriserte':'containeriser',
'containerkapasitet':'containerkapasit',
'containerkraner':'containerkran',
'containerlast':'containerl',
'containermarkedet':'containermarked',
'containerne':'containern',
'containernes':'containern',
'containeromlasting':'containeromlasting',
'containeromsetning':'containeromsetning',
'containeromslag':'containeromslag',
'containeromslaget':'containeromslag',
'containerrederiene':'containerrederi',
'containerruter':'containerrut',
'containerskip':'containerskip',
'containerskipene':'containerskip',
'containerskipenes':'containerskip',
'containerterminal':'containerterminal',
'containerterminalen':'containerterminal',
'containerterminalene':'containerterminal',
'containerterminaler':'containerterminal',
'containertrafikk':'containertrafikk',
'containertrafikken':'containertrafikk',
'containertransport':'containertransport',
'containertransporten':'containertransport',
'containertransporter':'containertransport',
'containerutstyret':'containerutstyr',
'containervekst':'containervekst',
'containerveksten':'containervekst',
'containervirksomhet':'containervirksom',
'containervirksomheten':'containervirksom',
'containervolum':'containervolum',
'containervolumene':'containervolum',
'containervolumer':'containervolum',
'cooperative':'cooperativ',
'coordination':'coordination',
'cost':'cost',
'cruise':'cruis',
'ctse':'ctse',
'd':'d',
'da':'da',
'dag':'dag',
'dagar':'dag',
'dagbok':'dagbok',
'dagen':'dag',
'dagene':'dag',
'dagens':'dag',
'dager':'dag',
'dagfinn':'dagfinn',
'daggry':'daggry',
'daghjem':'daghjem',
'dagjeldande':'dagjeld',
'dagjeldende':'dagjeld',
'dagleg':'dag',
'daglege':'dag',
'daglig':'dag',
'daglige':'dag',
'dagligliv':'dagligliv',
'dagliglivet':'dagligliv',
'dagligvarer':'dagligvar',
'dagmulkt':'dagmulkt',
'dagslys':'dagsly',
'dagslysbegrensninger':'dagslysbegrensning',
'dagslyset':'dagslys',
'dagtid':'dagtid',
'dal':'dal',
'dalarnas':'dalarn',
'dale':'dal',
'daler':'dal',
'dalsøkk':'dalsøkk',
'dam':'dam',
'damper':'damp',
'danielsen':'dani',
'danmark':'danmark',
'danmarks':'danmark',
'danne':'dann',
'dannelsen':'dann',
'danner':'dann',
'dannes':'dann',
'dannet':'dann',
'dansen':'dans',
'danser':'dans',
'danset':'dans',
'dansk':'dansk',
'danske':'dansk',
'data':'dat',
'database':'databas',
'dataene':'data',
'datainnsamlingsmetode':'datainnsamlingsmetod',
'datamateriale':'datamaterial',
'datasystem':'datasystem',
'datasystema':'datasystem',
'daterast':'dater',
'datere':'dater',
'datert':'dater',
'dato':'dato',
'datoen':'dato',
'datter':'datt',
'datteren':'datter',
'datterselskap':'datterselskap',
'datterselskaper':'datterselskap',
'daværende':'davær',
'de':'de',
'debatt':'debatt',
'debatten':'debatt',
'debitert':'debiter',
'debitor':'debitor',
'dedikerte':'dediker',
'defekte':'defekt',
'definere':'definer',
'definerer':'definer',
'defineres':'definer',
'definert':'definer',
'definerte':'definer',
'definisjon':'definisjon',
'definisjonane':'definisjon',
'definisjonar':'definisjon',
'definisjonen':'definisjon',
'definisjonene':'definisjon',
'definisjoner':'definisjon',
'definitive':'definitiv',
'definitivt':'definitiv',
'deg':'deg',
'dei':'dei',
'deig':'deig',
'deilig':'deil',
'deilige':'deil',
'deiligste':'deiligst',
'deira':'deir',
'deiser':'deis',
'dekar':'dek',
'dekk':'dekk',
'dekke':'dekk',
'dekkende':'dekk',
'dekker':'dekk',
'dekkes':'dekk',
'dekket':'dekk',
'dekkjast':'dekkj',
'dekkje':'dekkj',
'dekkjer':'dekkj',
'dekkshøyde':'dekkshøyd',
'dekning':'dekning',
'dekninga':'dekning',
'dekningen':'dekning',
'dekningslova':'dekning',
'dekningsområde':'dekningsområd',
'dekningsprosent':'dekningsprosent',
'dekningsrett':'dekningsrett',
'dekt':'dekt',
'dekte':'dekt',
'del':'del',
'delar':'del',
'delast':'del',
'dele':'del',
'delegasjon':'delegasjon',
'delegere':'deleger',
'delegering':'delegering',
'delegert':'deleger',
'delegerte':'deleger',
'delegertforsamling':'delegertforsamling',
'delegertordning':'delegertordning',
'delegertordningar':'delegertordning',
'deleige':'del',
'delen':'del',
'delene':'del',
'delentreprisar':'delentrepris',
'deler':'del',
'deles':'del',
'deling':'deling',
'delinga':'deling',
'delingen':'deling',
'delingsforholdet':'delingsforhold',
'delingsreglene':'delingsregl',
'delingsregler':'delingsregl',
'delingstilfella':'delingstilfell',
'delingsåret':'delingsår',
'delje':'delj',
'delkapittel':'delkapittel',
'della':'dell',
'delle':'dell',
'delling':'delling',
'delmål':'delmål',
'delmålene':'delmål',
'delområder':'delområd',
'delplanarbeidet':'delplanarbeid',
'delrapport':'delrapport',
'delrapporter':'delrapport',
'dels':'del',
'delsystemet':'delsystem',
'delt':'delt',
'delta':'delt',
'deltagelse':'deltag',
'deltagende':'deltag',
'deltager':'deltag',
'deltagerforhold':'deltagerforhold',
'deltagerkommunene':'deltagerkommun',
'deltagerkommunenes':'deltagerkommun',
'deltakar':'deltak',
'deltakarane':'deltakar',
'deltakarar':'deltakar',
'deltakartal':'deltakartal',
'deltakelse':'deltak',
'deltakende':'deltak',
'deltaker':'deltak',
'deltakere':'deltaker',
'deltakerforhold':'deltakerforhold',
'deltakerkommunene':'deltakerkommun',
'deltakerkommunenes':'deltakerkommun',
'deltakerlignende':'deltakerlign',
'deltakerne':'deltakern',
'deltaking':'deltaking',
'deltakinga':'deltaking',
'deltar':'delt',
'deltatt':'deltatt',
'delte':'delt',
'deltid':'deltid',
'deltids':'deltid',
'deltidsandel':'deltidsandel',
'deltidsansatte':'deltidsansatt',
'deltidsansattes':'deltidsansatt',
'deltidsarbeidere':'deltidsarbeider',
'deltidsstilling':'deltidsstilling',
'deltok':'deltok',
'delutredning':'delutredning',
'delutredninger':'delutredning',
'delvis':'delvis',
'dem':'dem',
'demmes':'demm',
'demokrati':'demokrati',
'demokratisk':'demokratisk',
'demokratiske':'demokratisk',
'demper':'demp',
'demre':'demr',
'den':'den',
'denger':'deng',
'denne':'denn',
'dennes':'denn',
'denofa':'denof',
'dens':'den',
'departement':'departement',
'departementene':'departement',
'departementer':'departement',
'departementet':'departement',
'departementets':'departement',
'deponerast':'deponer',
'deponere':'deponer',
'deponerer':'deponer',
'deponering':'deponering',
'deponeringslova':'deponering',
'deponeringsordning':'deponeringsordning',
'depot':'depot',
'depoter':'depot',
'depothold':'depothold',
'depotholdet':'depothold',
'depotsted':'depotsted',
'der':'der',
'dere':'der',
'deres':'der',
'deretter':'derett',
'derfor':'derfor',
'derfra':'derfr',
'derfrå':'derfrå',
'derigjennom':'derigjennom',
'derimot':'derimot',
'dermed':'dermed',
'dernest':'dernest',
'dersom':'dersom',
'dertil':'dertil',
'derunder':'derund',
'derved':'derved',
'des':'des',
'desember':'desemb',
'desentralisert':'desentraliser',
'desse':'dess',
'dessutan':'dessutan',
'dessuten':'dessut',
'destinasjon':'destinasjon',
'destinasjoner':'destinasjon',
'destinasjonsmønstre':'destinasjonsmønstr',
'desto':'desto',
'det':'det',
'detalj':'detalj',
'detaljane':'detalj',
'detaljeringsgrad':'detaljeringsgrad',
'detaljert':'detaljer',
'detaljerte':'detaljer',
'detaljleddet':'detaljledd',
'detaljregulere':'detaljreguler',
'detaljregulert':'detaljreguler',
'dets':'det',
'detta':'dett',
'dette':'dett',
'deutscher':'deutsch',
'dfds':'dfds',
'dh':'dh',
'di':'di',
'dialog':'dialog',
'die':'die',
'differanse':'differans',
'differansen':'differans',
'differanser':'differans',
'differensierte':'differensier',
'diger':'dig',
'digert':'diger',
'digre':'digr',
'dikte':'dikt',
'diktere':'dikter',
'dikteren':'dikter',
'dikteres':'dikter',
'dikterkunstens':'dikterkunst',
'dilemma':'dilemm',
'diltende':'dilt',
'dimensjoner':'dimensjon',
'dimensjonerende':'dimensjoner',
'dimensjonering':'dimensjonering',
'din':'din',
'dine':'din',
'dingler':'dingl',
'dir':'dir',
'direkte':'direkt',
'direktekrav':'direktekrav',
'direktekravsregel':'direktekravsregel',
'direkteutdelingar':'direkteutdeling',
'direktiv':'direktiv',
'direktivbestemmelsen':'direktivbestemm',
'direktivet':'direktiv',
'direktivets':'direktiv',
'direktør':'direktør',
'dirigerer':'diriger',
'dirigeres':'diriger',
'dirre':'dirr',
'dirrer':'dirr',
'disaggregert':'disaggreger',
'diskriminerande':'diskriminer',
'diskriminere':'diskriminer',
'diskriminering':'diskriminering',
'diskrimineringsforbod':'diskrimineringsforbod',
'diskusjon':'diskusjon',
'diskusjonar':'diskusjon',
'diskusjonen':'diskusjon',
'diskutere':'diskuter',
'diskuteres':'diskuter',
'diskutert':'diskuter',
'dispensasjon':'dispensasjon',
'dispensasjoner':'dispensasjon',
'disponere':'disponer',
'disponerer':'disponer',
'disponeres':'disponer',
'disponering':'disponering',
'disponeringen':'disponering',
'disponert':'disponer',
'disponerte':'disponer',
'disposisjon':'disposisjon',
'disposisjonane':'disposisjon',
'disposisjonar':'disposisjon',
'disposisjonen':'disposisjon',
'disposisjoner':'disposisjon',
'disposisjonsrett':'disposisjonsrett',
'disposisjonsretten':'disposisjonsrett',
'disse':'diss',
'dissens':'diss',
'dissensen':'dissens',
'disses':'diss',
'distanse':'distans',
'distanseavhengige':'distanseavheng',
'distansen':'distans',
'distanser':'distans',
'distribuerer':'distribuer',
'distribueres':'distribuer',
'distribusjon':'distribusjon',
'distribusjonen':'distribusjon',
'distribusjonsbiler':'distribusjonsbil',
'distribusjonsdistanser':'distribusjonsdistans',
'distribusjonsenhet':'distribusjonsen',
'distribusjonsfunksjonen':'distribusjonsfunksjon',
'distribusjonsfunksjoner':'distribusjonsfunksjon',
'distribusjonskanalene':'distribusjonskanal',
'distribusjonskanaler':'distribusjonskanal',
'distribusjonslagre':'distribusjonslagr',
'distribusjonsløsningene':'distribusjonsløsning',
'distribusjonsløsninger':'distribusjonsløsning',
'distribusjonsmønsteret':'distribusjonsmønster',
'distribusjonsmønstre':'distribusjonsmønstr',
'distribusjonsroller':'distribusjonsroll',
'distribusjonsselskaper':'distribusjonsselskap',
'distribusjonssentraler':'distribusjonssentral',
'distribusjonssentrum':'distribusjonssentrum',
'distribusjonssystemene':'distribusjonssystem',
'distribusjonsterminaler':'distribusjonsterminal',
'distribusjonstransporter':'distribusjonstransport',
'distribusjonsvirksomheten':'distribusjonsvirksom',
'distributører':'distributør',
'distriktene':'distrikt',
'distrikts':'distrikt',
'dit':'dit',
'ditt':'ditt',
'diverse':'divers',
'diversifisering':'diversifisering',
'diversifisert':'diversifiser',
'djerv':'djerv',
'djervere':'djerver',
'dkk':'dkk',
'dobbel':'dobbel',
'dobbelt':'dobbelt',
'dobbeltbeskatning':'dobbeltbeskatning',
'dobbelte':'dobbelt',
'dobbeltpantsetjing':'dobbeltpantsetjing',
'dobbeltrolle':'dobbeltroll',
'dobbeltrøyst':'dobbeltrøyst',
'dobbeltsal':'dobbeltsal',
'dobbeltspor':'dobbeltspor',
'dobbeltsporet':'dobbeltspor',
'doble':'dobl',
'dobles':'dobl',
'dobling':'dobling',
'dog':'dog',
'dokument':'dokument',
'dokumenta':'dokument',
'dokumentasjon':'dokumentasjon',
'dokumentavgift':'dokumentavgift',
'dokumenteigenskapar':'dokumenteigenskap',
'dokumentene':'dokument',
'dokumenter':'dokument',
'dokumentert':'dokumenter',
'dokumentet':'dokument',
'dokumenthandtering':'dokumenthandtering',
'dom':'dom',
'dominans':'dominan',
'dominerande':'dominer',
'dominere':'dominer',
'dominerende':'dominer',
'dominerer':'dominer',
'domineres':'dominer',
'dominert':'dominer',
'dommarar':'dommar',
'dommen':'domm',
'dommene':'domm',
'dommer':'domm',
'dommere':'dommer',
'domstolane':'domstol',
'domstolar':'domstol',
'domstolen':'domstol',
'domstolens':'domstol',
'domstolslova':'domstol',
'dotter':'dott',
'dotterforetag':'dotterforetag',
'dotterselskap':'dotterselskap',
'dotterselskapet':'dotterselskap',
'dottertiltak':'dottertiltak',
'doven':'dov',
'dovent':'dovent',
'dovne':'dovn',
'down':'down',
'dra':'dra',
'drabelig':'drab',
'drabelige':'drab',
'drag':'drag',
'draga':'drag',
'drage':'drag',
'dragehodet':'dragehod',
'dragen':'drag',
'drager':'drag',
'dragsug':'dragsug',
'drakk':'drakk',
'dramatisk':'dramatisk',
'drammen':'dramm',
'drammenselva':'drammenselv',
'drammensfjorden':'drammensfjord',
'drammensregionens':'drammensregion',
'drapet':'drap',
'drar':'drar',
'drasil':'drasil',
'drastisk':'drastisk',
'dratt':'dratt',
'draug':'draug',
'draupne':'draupn',
'dreide':'dreid',
'dreie':'dreie',
'dreier':'dreier',
'dreining':'dreining',
'drepe':'drep',
'dreper':'drep',
'drept':'drept',
'drepte':'drept',
'dressmann':'dressmann',
'drev':'drev',
'drevet':'drev',
'drift':'drift',
'drifta':'drift',
'driften':'drift',
'driftoversikt':'driftoversikt',
'driftsansvar':'driftsansv',
'driftsansvaret':'driftsansvar',
'driftsbudsjett':'driftsbudsjett',
'driftsformer':'driftsform',
'driftsinnskrenkinger':'driftsinnskrenking',
'driftsinnskrenkninger':'driftsinnskrenkning',
'driftsinntektene':'driftsinntekt',
'driftskostnader':'driftskostnad',
'driftsmidler':'driftsmidl',
'driftsopplegg':'driftsopplegg',
'driftsoversikt':'driftsoversikt',
'driftsoverskotet':'driftsoverskot',
'driftsselskapet':'driftsselskap',
'driftssystemer':'driftssystem',
'drikk':'drikk',
'drikke':'drikk',
'drikkegilde':'drikkegild',
'drikkehorn':'drikkehorn',
'drikkehornet':'drikkehorn',
'drikken':'drikk',
'drikker':'drikk',
'dristig':'drist',
'dristige':'drist',
'driv':'driv',
'drivast':'driv',
'drive':'driv',
'driven':'driv',
'drivende':'driv',
'driver':'driv',
'drives':'driv',
'drivkrefter':'drivkreft',
'drivne':'drivn',
'dro':'dro',
'dronning':'dronning',
'dronningen':'dronning',
'drukket':'drukk',
'drukne':'drukn',
'druknede':'drukn',
'drukner':'drukn',
'druknet':'drukn',
'dryppe':'drypp',
'drypper':'drypp',
'dråpe':'dråp',
'dråpene':'dråp',
'drøbak':'drøbak',
'drøbaksundet':'drøbaksund',
'drøfta':'drøft',
'drøftast':'drøft',
'drøfte':'drøft',
'drøftelsen':'drøft',
'drøfter':'drøft',
'drøftes':'drøft',
'drøftet':'drøft',
'drøfting':'drøfting',
'drøftinga':'drøfting',
'drøftingar':'drøfting',
'drøftingen':'drøfting',
'drøftinger':'drøfting',
'drøm':'drøm',
'drømme':'drømm',
'drømmene':'drømm',
'drømmer':'drømm',
'drømt':'drømt',
'drønnende':'drønn',
'drønner':'drønn',
'drøy':'drøy',
'drøye':'drøye',
'drøyt':'drøyt',
'du':'du',
'duger':'dug',
'duggen':'dugg',
'duk':'duk',
'dukker':'dukk',
'dukket':'dukk',
'dulter':'dult',
'dum':'dum',
'dumme':'dumm',
'dummer':'dumm',
'dummere':'dummer',
'dummeste':'dummest',
'dummet':'dumm',
'dumt':'dumt',
'dun':'dun',
'dunder':'dund',
'dundrende':'dundr',
'dundrer':'dundr',
'dunker':'dunk',
'dur':'dur',
'durer':'dur',
'dverg':'dverg',
'dvergen':'dverg',
'dvergene':'dverg',
'dvergenes':'dverg',
'dvergens':'dverg',
'dverger':'dverg',
'dvergers':'dverg',
'dvergeslekt':'dvergeslekt',
'dvergsten':'dvergst',
'dvs':'dvs',
'dy':'dy',
'dybde':'dybd',
'dybdeproblemer':'dybdeproblem',
'dybedal':'dybedal',
'dykker':'dykk',
'dyktig':'dykt',
'dyktighet':'dykt',
'dynamikk':'dynamikk',
'dynamisk':'dynamisk',
'dynke':'dynk',
'dyp':'dyp',
'dype':'dyp',
'dypere':'dyper',
'dypeskaten':'dypeskat',
'dypest':'dypest',
'dypet':'dyp',
'dypgående':'dypgå',
'dyppes':'dypp',
'dypt':'dypt',
'dyr':'dyr',
'dyrare':'dyrar',
'dyre':'dyr',
'dyrebare':'dyrebar',
'dyrehald':'dyrehald',
'dyrehaldet':'dyrehald',
'dyrere':'dyrer',
'dyret':'dyr',
'dyrt':'dyrt',
'dystert':'dyster',
'dystre':'dystr',
'dåres':'dår',
'dårleg':'dår',
'dårlegare':'dårlegar',
'dårlig':'dår',
'dårlige':'dår',
'dårligere':'dårliger',
'dø':'dø',
'død':'død',
'døde':'død',
'dødelighet':'død',
'dødelighetsarv':'dødelighetsarv',
'dødelighetsarven':'dødelighetsarv',
'døden':'død',
'dødes':'død',
'dødning':'dødning',
'dødningene':'dødning',
'dødninger':'dødning',
'dødningeskaller':'dødningeskall',
'dødsbo':'dødsbo',
'dødsboet':'dødsbo',
'dødsfall':'dødsfall',
'dødsfallet':'dødsfall',
'dødsfallstidspunktet':'dødsfallstidspunkt',
'dødsriket':'dødsrik',
'dødsrikets':'dødsrik',
'dødsrisiko':'dødsrisiko',
'dødt':'dødt',
'døgn':'døgn',
'døgnet':'døgn',
'døgnets':'døgn',
'døma':'døm',
'døme':'døm',
'dømes':'døm',
'dømet':'døm',
'dømme':'dømm',
'dømmer':'dømm',
'dønn':'dønn',
'dør':'dør',
'døren':'dør',
'dørene':'dør',
'dører':'dør',
'dørgende':'dørg',
'dørhellen':'dørhell',
'dørsprekken':'dørsprekk',
'døråpningen':'døråpning',
'døtre':'døtr',
'døtrene':'døtr',
'e':'e',
'econ':'econ',
'econs':'econ',
'ed':'ed',
'edda':'edd',
'eddas':'edd',
'edderkopper':'edderkopp',
'edel':'edel',
'eder':'eder',
'edilizie':'edilizi',
'edw':'edw',
'edward':'edward',
'ef':'ef',
'effekt':'effekt',
'effekten':'effekt',
'effekter':'effekt',
'effektiv':'effektiv',
'effektive':'effektiv',
'effektivisering':'effektivisering',
'effektiviseringprinsippene':'effektiviseringprinsipp',
'effektiviseringsgevinster':'effektiviseringsgevinst',
'effektivitet':'effektivit',
'effektiviteten':'effektivitet',
'effektivitetshensyn':'effektivitetshensyn',
'effektivitetskrav':'effektivitetskrav',
'effektivt':'effektiv',
'efta':'eft',
'egalitær':'egalitær',
'egen':'egen',
'egenandel':'egenandel',
'egene':'egen',
'egenkapital':'egenkapital',
'egenkapitalen':'egenkapital',
'egenskapen':'egenskap',
'egenskaper':'egenskap',
'egentlig':'egent',
'eget':'eget',
'egge':'egg',
'egger':'egg',
'eggeskall':'eggeskall',
'egget':'egg',
'eggum':'eggum',
'egil':'egil',
'egils':'egil',
'egne':'egn',
'egnede':'egn',
'egner':'egn',
'egnet':'egn',
'ei':'ei',
'eid':'eid',
'eidanger':'eidang',
'eidangerfjorden':'eidangerfjord',
'eide':'eid',
'eie':'eie',
'eiendelene':'eiendel',
'eiendeler':'eiendel',
'eiendom':'eiendom',
'eiendommen':'eiendomm',
'eiendommene':'eiendomm',
'eiendommer':'eiendomm',
'eiendomserverv':'eiendomserverv',
'eiendomsmegling':'eiendomsmegling',
'eiendomspriser':'eiendomspris',
'eiendomsrett':'eiendomsrett',
'eiendomsretten':'eiendomsrett',
'eier':'eier',
'eierandel':'eierandel',
'eierandeler':'eierandel',
'eiere':'eier',
'eierforhold':'eierforhold',
'eierforholdet':'eierforhold',
'eierinteresse':'eierinteress',
'eierinteresser':'eierinteress',
'eierkommune':'eierkommun',
'eierkommunene':'eierkommun',
'eierkommunenes':'eierkommun',
'eierne':'eiern',
'eierseksjoner':'eierseksjon',
'eierseksjonsloven':'eierseksjon',
'eierseksjonssameier':'eierseksjonssamei',
'eierskap':'eierskap',
'eiersl':'eiersl',
'eies':'eies',
'eiet':'eiet',
'eig':'eig',
'eiga':'eig',
'eigar':'eig',
'eigarane':'eigar',
'eigarar':'eigar',
'eigarbrøk':'eigarbrøk',
'eigarbustad':'eigarbustad',
'eigarbustadene':'eigarbustad',
'eigarbustader':'eigarbustad',
'eigardelar':'eigardel',
'eigaren':'eigar',
'eigarens':'eigar',
'eigarforhold':'eigarforhold',
'eigarforholdet':'eigarforhold',
'eigarinteressa':'eigarinteress',
'eigarinteresse':'eigarinteress',
'eigarinteressene':'eigarinteress',
'eigarinteresser':'eigarinteress',
'eigarleilegheiter':'eigarleilegheit',
'eigarpart':'eigarpart',
'eigarpartane':'eigarpart',
'eigarposisjonen':'eigarposisjon',
'eigarpreg':'eigarpreg',
'eigarrisikoen':'eigarrisiko',
'eigarsamanslutningar':'eigarsamanslutning',
'eigarseksjon':'eigarseksjon',
'eigarseksjonar':'eigarseksjon',
'eigarseksjonsforma':'eigarseksjonsform',
'eigarseksjonslov':'eigarseksjon',
'eigarseksjonslova':'eigarseksjon',
'eigarseksjonssameiger':'eigarseksjonssam',
'eigarskifte':'eigarskift',
'eigarskifteforsikring':'eigarskifteforsikring',
'eigarskiftet':'eigarskift',
'eigarstatusen':'eigarstatus',
'eigast':'eig',
'eige':'eig',
'eigeansvar':'eigeansv',
'eigedom':'eigedom',
'eigedommane':'eigedomm',
'eigedommar':'eigedomm',
'eigedommen':'eigedomm',
'eigedomseiningar':'eigedomseining',
'eigedomsforhold':'eigedomsforhold',
'eigedomsformidling':'eigedomsformidling',
'eigedomsforvaltning':'eigedomsforvaltning',
'eigedomskjøp':'eigedomskjøp',
'eigedomsmeklar':'eigedomsmekl',
'eigedomsmeklarar':'eigedomsmeklar',
'eigedomsmekling':'eigedomsmekling',
'eigedomsmeklingslova':'eigedomsmekling',
'eigedomsmeklingsløyve':'eigedomsmeklingsløyv',
'eigedomsovergangen':'eigedomsovergang',
'eigedomsrett':'eigedomsrett',
'eigedomsretten':'eigedomsrett',
'eigedomsselskap':'eigedomsselskap',
'eigedomssjefen':'eigedomssjef',
'eigeform':'eigeform',
'eigeformene':'eigeform',
'eigeformer':'eigeform',
'eigen':'eig',
'eigendelar':'eigendel',
'eigenkapital':'eigenkapital',
'eigenkapitalbeitragen':'eigenkapitalbeitrag',
'eigenkapitalen':'eigenkapital',
'eigenkapitalinnskot':'eigenkapitalinnskot',
'eigenskap':'eigenskap',
'eigentleg':'eigent',
'eigentlege':'eigent',
'eigne':'eign',
'eignedelane':'eignedel',
'eignedelar':'eignedel',
'eik':'eik',
'eika':'eik',
'eikenøtt':'eikenøtt',
'eiketre':'eiketr',
'eikrem':'eikrem',
'ein':'ein',
'einaste':'einast',
'eine':'ein',
'einebustad':'einebustad',
'einebustader':'einebustad',
'eineeigar':'eine',
'eineeige':'eine',
'einerett':'einerett',
'eineretten':'einerett',
'einerådande':'eineråd',
'eingong':'eingong',
'eingongsbetalingar':'eingongsbetaling',
'eingongsinnbetalingar':'eingongsinnbetaling',
'einherjene':'einherj',
'einherjer':'einherj',
'einherjers':'einherj',
'eining':'eining',
'einingar':'eining',
'einingsregisteret':'einingsregister',
'eins':'ein',
'einsidig':'einsid',
'einskapen':'einskap',
'einskild':'einskild',
'einskildbustader':'einskildbustad',
'einskilde':'einskild',
'einskildmedlemmer':'einskildmedlemm',
'einskildpersonar':'einskildperson',
'einskildsaker':'einskildsak',
'einslege':'eins',
'eintydig':'eintyd',
'eit':'eit',
'eiter':'eit',
'eiterdryppet':'eiterdrypp',
'eiteren':'eiter',
'eiterorm':'eiterorm',
'eiterormen':'eiterorm',
'eitrende':'eitr',
'eitt':'eitt',
'eittårsfristen':'eittårsfrist',
'ejerlejligheder':'ejerlejlighed',
'ekeberganleggene':'ekeberganlegg',
'ekebergtunnelen':'ekebergtunnel',
'ekebergåsen':'ekebergås',
'ekle':'ekl',
'ekonomisk':'ekonomisk',
'ekonomiska':'ekonomisk',
'ekornet':'ekorn',
'eks':'eks',
'eksakt':'eksakt',
'eksakte':'eksakt',
'eksempel':'eksempel',
'eksempelvis':'eksempelvis',
'eksemplene':'eksempl',
'eksempler':'eksempl',
'eksemplet':'eksempl',
'eksistensen':'eksistens',
'eksisterande':'eksister',
'eksistere':'eksister',
'eksisterende':'eksister',
'eksisterer':'eksister',
'ekskl':'ekskl',
'eksklusiv':'eksklusiv',
'eksklusive':'eksklusiv',
'eksmemplene':'eksmempl',
'eksogent':'eksogent',
'ekspansjon':'ekspansjon',
'ekspederes':'ekspeder',
'eksperthjelp':'eksperthjelp',
'ekspertisen':'ekspertis',
'eksplisitt':'eksplisitt',
'eksponeringa':'eksponering',
'eksponert':'eksponer',
'eksport':'eksport',
'eksportbedrifter':'eksportbedrift',
'eksportbedrifters':'eksportbedrift',
'eksportcontainere':'eksportcontainer',
'eksporten':'eksport',
'eksporterer':'eksporter',
'eksporteres':'eksporter',
'eksportert':'eksporter',
'eksporterte':'eksporter',
'eksportfunksjonene':'eksportfunksjon',
'eksportfylke':'eksportfylk',
'eksportgods':'eksportgod',
'eksportgodset':'eksportgods',
'eksporthavnene':'eksporthavn',
'eksporthavner':'eksporthavn',
'eksportkundenes':'eksportkund',
'eksportlaster':'eksportlast',
'eksportmengden':'eksportmengd',
'eksportnæringer':'eksportnæring',
'eksportorienterte':'eksportorienter',
'eksportoverskudd':'eksportoverskudd',
'eksportprofil':'eksportprofil',
'eksportrettede':'eksportrett',
'eksportrettet':'eksportrett',
'eksportsammenheng':'eksportsammenheng',
'eksportsiden':'eksportsid',
'eksporttransportene':'eksporttransport',
'eksporttransporter':'eksporttransport',
'eksportutvikling':'eksportutvikling',
'eksportvarer':'eksportvar',
'eksportvekst':'eksportvekst',
'eksportveksten':'eksportvekst',
'eksportverdi':'eksportverdi',
'eksportvolum':'eksportvolum',
'eksportvolumene':'eksportvolum',
'eksportvolumer':'eksportvolum',
'eksportvolumet':'eksportvolum',
'ekstern':'ekstern',
'eksterne':'ekstern',
'ekstingverte':'ekstingver',
'ekstinksjon':'ekstinksjon',
'ekstinksjonen':'ekstinksjon',
'ekstinksjonsregelen':'ekstinksjonsregel',
'ekstra':'ekstr',
'ekstrakostnader':'ekstrakostnad',
'ekstraordinær':'ekstraordinær',
'ekstraordinære':'ekstraordinær',
'ekstremt':'ekstremt',
'ekte':'ekt',
'ektefelle':'ektefell',
'ektefellen':'ektefell',
'ektefellepensjon':'ektefellepensjon',
'ektefellepensjonens':'ektefellepensjon',
'ektefeller':'ektefell',
'ektefelles':'ektefell',
'ektemakar':'ektemak',
'ektemake':'ektemak',
'ektemaken':'ektemak',
'ektemakens':'ektemak',
'ektemann':'ektemann',
'ektepar':'ektep',
'ekteparet':'ektepar',
'ekteskap':'ekteskap',
'ekteskapet':'ekteskap',
'ekteskaps':'ekteskap',
'ekteskapslova':'ekteskap',
'ekteskapslovgivningen':'ekteskapslovgivning',
'el':'el',
'eld':'eld',
'elde':'eld',
'eldre':'eldr',
'eldrebustader':'eldrebustad',
'eldst':'eldst',
'eldste':'eldst',
'elektrisk':'elektrisk',
'elektronisk':'elektronisk',
'elektroniske':'elektronisk',
'element':'element',
'elementa':'element',
'elementene':'element',
'elementer':'element',
'elementet':'element',
'elendig':'elend',
'elendige':'elend',
'elendighet':'elend',
'elg':'elg',
'elgar':'elg',
'eliminere':'eliminer',
'elisabeth':'elisabeth',
'eljudne':'eljudn',
'elkem':'elkem',
'elkjøp':'elkjøp',
'elle':'ell',
'eller':'ell',
'ellers':'ell',
'elles':'ell',
'elleve':'ellev',
'ellingsen':'ellings',
'elsk':'elsk',
'elske':'elsk',
'elsker':'elsk',
'elsket':'elsk',
'elskovssyk':'elskovssyk',
'elv':'elv',
'elva':'elv',
'elveløpet':'elveløp',
'elven':'elv',
'elver':'elv',
'elvevegen':'elveveg',
'embla':'embl',
'emden':'emd',
'emne':'emn',
'empirisk':'empirisk',
'en':'en',
'enda':'end',
'endar':'end',
'ende':'end',
'endeleg':'end',
'endelege':'end',
'endelig':'end',
'endelige':'end',
'enden':'end',
'endepunkt':'endepunkt',
'ender':'end',
'endevendt':'endevend',
'endingar':'ending',
'endr':'endr',
'endra':'endr',
'endrar':'endr',
'endrast':'endr',
'endre':'endr',
'endrede':'endr',
'endrer':'endr',
'endres':'endr',
'endret':'endr',
'endrete':'endret',
'endring':'endring',
'endringa':'endring',
'endringane':'endring',
'endringar':'endring',
'endringen':'endring',
'endringene':'endring',
'endringer':'endring',
'endringsarbeid':'endringsarbeid',
'endringsavtale':'endringsavtal',
'endringsdirektivets':'endringsdirektiv',
'endringsforslag':'endringsforslag',
'endringslov':'endring',
'endringstidspunktet':'endringstidspunkt',
'endt':'endt',
'endte':'endt',
'ene':'ene',
'eneansvarlig':'eneansvar',
'enearving':'enearving',
'enerett':'enerett',
'energi':'energi',
'energieffektivisering':'energieffektivisering',
'energiøkonomi':'energiøkonomi',
'enerom':'enerom',
'eneste':'enest',
'enestående':'enestå',
'eng':'eng',
'engangs':'engang',
'engangsbeløp':'engangsbeløp',
'engangsbeløpet':'engangsbeløp',
'engangsinnbetalinger':'engangsinnbetaling',
'engangspremie':'engangspremi',
'engangspremier':'engangspremi',
'engangssum':'engangssum',
'engangsutbetaling':'engangsutbetaling',
'engangsutbetalinger':'engangsutbetaling',
'engangsutbetalt':'engangsutbetalt',
'engangsutgifter':'engangsutgift',
'engangsytelse':'engangsyt',
'engangsytelsen':'engangsyt',
'engangsytelser':'engangsyt',
'engasjement':'engasjement',
'engasjert':'engasjer',
'engelske':'engelsk',
'england':'england',
'engroslagre':'engroslagr',
'engrosvirksomhet':'engrosvirksom',
'engsoleie':'engsolei',
'engstelig':'engst',
'engstet':'engst',
'enhet':'enh',
'enheter':'enhet',
'enheterisering':'enheterisering',
'enheterisert':'enheteriser',
'enhetlig':'enhet',
'enhetlige':'enhet',
'enhetsgods':'enhetsgod',
'enhetslaster':'enhetslast',
'enhetslasteskip':'enhetslasteskip',
'enhetslastet':'enhetslast',
'enhetslasthavner':'enhetslasthavn',
'enhetslasthåndtering':'enhetslasthåndtering',
'enhetslasttrafikk':'enhetslasttrafikk',
'enhetslasttrafikken':'enhetslasttrafikk',
'enhver':'enhv',
'enig':'enig',
'enige':'enig',
'enkel':'enkel',
'enkelhet':'enkel',
'enkelt':'enkelt',
'enkeltaksjer':'enkeltaksj',
'enkeltaktørene':'enkeltaktør',
'enkelte':'enkelt',
'enkeltengasjementer':'enkeltengasjement',
'enkeltes':'enkelt',
'enkeltforetak':'enkeltforetak',
'enkeltforhold':'enkeltforhold',
'enkeltforslag':'enkeltforslag',
'enkelthavner':'enkelthavn',
'enkeltkommuners':'enkeltkommun',
'enkeltlandenes':'enkeltland',
'enkeltmannsforetak':'enkeltmannsforetak',
'enkeltmannsføretak':'enkeltmannsføretak',
'enkeltmannsføretaket':'enkeltmannsføretak',
'enkeltområder':'enkeltområd',
'enkeltpersoner':'enkeltperson',
'enkeltprosjekter':'enkeltprosjekt',
'enkeltsaker':'enkeltsak',
'enkeltsaksbehandlingen':'enkeltsaksbehandling',
'enkeltspørsmål':'enkeltspørsmål',
'enkeltvalg':'enkeltvalg',
'enkeltvedtak':'enkeltvedtak',
'enkeltvis':'enkeltvis',
'enkeltår':'enkeltår',
'enkemann':'enkemann',
'enklare':'enklar',
'enklaste':'enklast',
'enkle':'enkl',
'enklere':'enkler',
'enkleste':'enklest',
'enn':'enn',
'enno':'enno',
'ennå':'ennå',
'enorme':'enorm',
'ensbetydende':'ensbetyd',
'enser':'ens',
'ensidig':'ensid',
'enslig':'ens',
'enslige':'ens',
'ensom':'ensom',
'enten':'ent',
'entreprenør':'entreprenør',
'entreprenørar':'entreprenør',
'entreprenøren':'entreprenør',
'entreprenørføretak':'entreprenørføretak',
'entreprise':'entrepris',
'entydig':'entyd',
'enøyd':'enøyd',
'enøyde':'enøyd',
'epes':'epes',
'eple':'epl',
'eplene':'epl',
'epler':'epl',
'eplet':'epl',
'epletrær':'epletrær',
'equivalent':'equivalent',
'er':'er',
'erfaren':'erfar',
'erfaring':'erfaring',
'erfaringene':'erfaring',
'erfaringer':'erfaring',
'erfaringsmateriale':'erfaringsmaterial',
'erfaringstall':'erfaringstall',
'ergerlig':'erger',
'ergerlige':'erger',
'ergrelse':'ergr',
'erik':'erik',
'erkjennelse':'erkjenn',
'erling':'erling',
'erstatning':'erstatning',
'erstatningsansvaret':'erstatningsansvar',
'erstatte':'erstatt',
'erstattes':'erstatt',
'erstattet':'erstatt',
'erte':'ert',
'erter':'ert',
'ertet':'ert',
'erverv':'erverv',
'erverva':'erverv',
'ervervar':'erverv',
'ervervarane':'ervervar',
'ervervarar':'ervervar',
'ervervaren':'ervervar',
'ervervarens':'ervervar',
'erverve':'erverv',
'ervervede':'erverv',
'ervervet':'erverv',
'ervervsevnen':'ervervsevn',
'ervervsfasen':'ervervsfas',
'ervervsinntekt':'ervervsinntekt',
'ervervskostnadene':'ervervskostnad',
'ervervsmessig':'ervervsmess',
'ervervsprøves':'ervervsprøv',
'ervervsprøving':'ervervsprøving',
'ervervsprøvingen':'ervervsprøving',
'erwerbes':'erwerb',
'erwerbs':'erwerb',
'estetikk':'estetikk',
'estetiske':'estetisk',
'estimert':'estimer',
'et':'et',
'etablerast':'etabler',
'etablere':'etabler',
'etablerer':'etabler',
'etableres':'etabler',
'etablering':'etablering',
'etableringa':'etablering',
'etableringen':'etablering',
'etableringsadgangen':'etableringsadgang',
'etableringshorisont':'etableringshorisont',
'etableringsproblem':'etableringsproblem',
'etableringsstaten':'etableringsstat',
'etablert':'etabler',
'etablerte':'etabler',
'etappe':'etapp',
'etappen':'etapp',
'etapper':'etapp',
'etappevis':'etappevis',
'etappevise':'etappevis',
'etasje':'etasj',
'etat':'etat',
'etatar':'etat',
'etaten':'etat',
'etatene':'etat',
'etatens':'etat',
'etc':'etc',
'ete':'ete',
'eter':'eter',
'ethvert':'ethver',
'etiske':'etisk',
'etla':'etl',
'etnisk':'etnisk',
'ett':'ett',
'etter':'ett',
'etterarbeid':'etterarbeid',
'etterfølgende':'etterfølg',
'etterkome':'etterkom',
'etterkomme':'etterkomm',
'etterkommere':'etterkommer',
'etterkommes':'etterkomm',
'etterkrigstida':'etterkrigstid',
'etterlate':'etterlat',
'etterlatt':'etterlatt',
'etterlatte':'etterlatt',
'etterlattedekning':'etterlattedekning',
'etterlattepensjon':'etterlattepensjon',
'etterlattepensjonenes':'etterlattepensjon',
'etterlattepensjoner':'etterlattepensjon',
'etterlattes':'etterlatt',
'etterlatteytelsene':'etterlatteyt',
'etterlatteytelser':'etterlatteyt',
'etterlattforsikring':'etterlattforsikring',
'etterlattpensjon':'etterlattpensjon',
'etterlattpensjoner':'etterlattpensjon',
'etterleving':'etterleving',
'ettermiddag':'ettermiddag',
'ettermiddagen':'ettermiddag',
'ettermæle':'ettermæl',
'etterprioriterte':'etterprioriter',
'etterprøve':'etterprøv',
'etterpå':'etterpå',
'etterskot':'etterskot',
'etterskotsvis':'etterskotsvis',
'etterslepseffekten':'etterslepseffekt',
'ettersom':'ettersom',
'etterspurde':'etterspurd',
'etterspurnad':'etterspurnad',
'etterspurnaden':'etterspurnad',
'etterspør':'etterspør',
'etterspørelsen':'etterspør',
'etterspørsel':'etterspørsel',
'etterspørselen':'etterspørsel',
'etterspørselens':'etterspørsel',
'etterspørselsregulering':'etterspørselsregulering',
'etterspørselssiden':'etterspørselssid',
'etterspørselsvekst':'etterspørselsvekst',
'etterspørselsveksten':'etterspørselsvekst',
'etterståande':'etterstå',
'ettersyn':'ettersyn',
'ettertid':'ettertid',
'etterutluting':'etterutluting',
'ettårige':'ettår',
'eu':'eu',
'euro':'euro',
'europa':'europ',
'europaparlaments':'europaparlament',
'europas':'europ',
'european':'european',
'europeen':'europe',
'europeisk':'europeisk',
'europeiske':'europeisk',
'eus':'eus',
'eventuell':'eventuell',
'eventuelle':'eventuell',
'eventuelt':'eventuelt',
'eventyr':'eventyr',
'eventyrene':'eventyr',
'eventyret':'eventyr',
'evig':'evig',
'evige':'evig',
'eviggrønne':'eviggrønn',
'evighet':'evig',
'evne':'evn',
'evnen':'evn',
'evt':'evt',
'eøf':'eøf',
'eøs':'eøs',
'f':'f',
'fabrikker':'fabrikk',
'fact':'fact',
'fager':'fag',
'fagforeining':'fagforeining',
'fagforeiningar':'fagforeining',
'faggrupper':'faggrupp',
'fagleg':'fag',
'faglig':'fag',
'faglige':'fag',
'fagmyndigheter':'fagmynd',
'fagorganisasjonar':'fagorganisasjon',
'fagrest':'fagrest',
'fakkeltog':'fakkeltog',
'fakket':'fakk',
'fakler':'fakl',
'faktisk':'faktisk',
'faktiske':'faktisk',
'faktor':'faktor',
'faktorar':'faktor',
'faktorene':'faktor',
'faktorer':'faktor',
'faktoringpant':'faktoringpant',
'faktum':'faktum',
'fakulteta':'fakultet',
'falk':'falk',
'falke':'falk',
'falkehammen':'falkehamm',
'falkemannen':'falkemann',
'falken':'falk',
'fall':'fall',
'falle':'fall',
'falleferdig':'falleferd',
'fallen':'fall',
'faller':'fall',
'fallet':'fall',
'falne':'faln',
'falnes':'faln',
'falsk':'falsk',
'falskest':'falskest',
'falskhet':'falsk',
'falt':'falt',
'familiane':'famili',
'familie':'famili',
'familieforhold':'familieforhold',
'familieliv':'familieliv',
'familiemedlemmer':'familiemedlemm',
'familien':'famili',
'familierettslege':'familieretts',
'familiesituasjon':'familiesituasjon',
'famler':'faml',
'fang':'fang',
'fanga':'fang',
'fangar':'fang',
'fange':'fang',
'fangen':'fang',
'fanger':'fang',
'fanges':'fang',
'fanget':'fang',
'fangst':'fangst',
'fangsten':'fangst',
'fanst':'fanst',
'fant':'fant',
'fanteri':'fanteri',
'fantes':'fant',
'far':'far',
'farbror':'farbror',
'fare':'far',
'faren':'far',
'farende':'far',
'farer':'far',
'fargede':'farg',
'farkosten':'farkost',
'farleden':'farled',
'farledene':'farled',
'farleder':'farled',
'farledsnettet':'farledsnett',
'farledssektoren':'farledssektor',
'farlig':'far',
'farlige':'far',
'farligere':'farliger',
'farligste':'farligst',
'farmasøyt':'farmasøyt',
'farmasøytiske':'farmasøytisk',
'farriseidet':'farriseid',
'fars':'far',
'fart':'fart',
'farten':'fart',
'fartsdempende':'fartsdemp',
'fartøy':'fartøy',
'fartøyene':'fartøy',
'fartøyer':'fartøy',
'fartøystørrelse':'fartøystørr',
'farvann':'farvann',
'farvannet':'farvann',
'farvannets':'farvann',
'farvannslov':'farvann',
'farvannsloven':'farvann',
'farvannslovens':'farvann',
'farvannssaker':'farvannssak',
'farve':'farv',
'farvel':'farvel',
'farvene':'farv',
'fascinerer':'fasciner',
'fase':'fas',
'fasen':'fas',
'fasiliteter':'fasilitet',
'fast':'fast',
'faste':'fast',
'fastere':'faster',
'fastlagt':'fastlagt',
'fastlandet':'fastland',
'fastlands':'fastland',
'fastlege':'fast',
'fastlegge':'fastlegg',
'fastlegger':'fastlegg',
'fastlegges':'fastlegg',
'fastlegging':'fastlegging',
'fastlegginga':'fastlegging',
'fastleggjast':'fastleggj',
'fastleggje':'fastleggj',
'fastlåst':'fastlåst',
'fastprisavtalar':'fastprisavtal',
'fastrenteinnskudd':'fastrenteinnskudd',
'fastrentelån':'fastrentelån',
'fastsatt':'fastsatt',
'fastsatte':'fastsatt',
'fastset':'fasts',
'fastsetjast':'fastsetj',
'fastsetje':'fastsetj',
'fastsetjing':'fastsetjing',
'fastsetjinga':'fastsetjing',
'fastsetjingsdom':'fastsetjingsdom',
'fastsett':'fastsett',
'fastsette':'fastsett',
'fastsettelse':'fastsett',
'fastsettelsen':'fastsett',
'fastsetter':'fastsett',
'fastsettes':'fastsett',
'fastslege':'fasts',
'fastslo':'fastslo',
'fastslå':'fastslå',
'fastslår':'fastslår',
'fastslås':'fastslås',
'fat':'fat',
'fatet':'fat',
'fatt':'fatt',
'fatte':'fatt',
'fatter':'fatt',
'fattes':'fatt',
'fattet':'fatt',
'fattig':'fatt',
'fattigdom':'fattigdom',
'fauna':'faun',
'favne':'favn',
'favnen':'favn',
'favner':'favn',
'favntak':'favntak',
'favoriserer':'favoriser',
'favorisering':'favorisering',
'favoriseringen':'favorisering',
'favør':'favør',
'fdb':'fdb',
'fe':'fe',
'februar':'febru',
'federabitazione':'federabitazion',
'federazione':'federazion',
'feederbåter':'feederbåt',
'feederhavner':'feederhavn',
'feederløsninger':'feederløsning',
'feedernettet':'feedernett',
'feederrederiene':'feederrederi',
'feederskip':'feederskip',
'feedersystem':'feedersystem',
'feedertrafikk':'feedertrafikk',
'feedertrafikken':'feedertrafikk',
'feedertransport':'feedertransport',
'feedertransportene':'feedertransport',
'feedertransporter':'feedertransport',
'feedervolumer':'feedervolum',
'feeedertransport':'feeedertransport',
'feier':'feier',
'feig':'feig',
'feige':'feig',
'feigeste':'feigest',
'feil':'feil',
'feilen':'feil',
'feiler':'feil',
'feilkalkylar':'feilkalkyl',
'feire':'feir',
'fekk':'fekk',
'fekter':'fekt',
'fektet':'fekt',
'felixstowe':'felixstow',
'fell':'fell',
'felle':'fell',
'fellen':'fell',
'feller':'fell',
'felles':'fell',
'fellesareal':'fellesareal',
'fellesareala':'fellesareal',
'fellesbestemmelser':'fellesbestemm',
'felleseiga':'felles',
'fellesfinansiering':'fellesfinansiering',
'fellesfond':'fellesfond',
'fellesformue':'fellesformu',
'fellesforvaltninga':'fellesforvaltning',
'fellesgjeld':'fellesgjeld',
'fellesgjelda':'fellesgjeld',
'felleshavn':'felleshavn',
'fellesinstallasjonar':'fellesinstallasjon',
'felleskapital':'felleskapital',
'felleskapitalen':'felleskapital',
'felleskonto':'felleskonto',
'felleskontor':'felleskontor',
'felleskostnadene':'felleskostnad',
'felleskostnader':'felleskostnad',
'felleskostnder':'felleskostnd',
'felleslån':'felleslån',
'felleslåna':'felleslån',
'felleslångjevaren':'felleslångjevar',
'fellesnevneren':'fellesnevner',
'fellesobligasjon':'fellesobligasjon',
'fellesoppgåve':'fellesoppgåv',
'fellesoppgåver':'fellesoppgåv',
'fellesorganisasjon':'fellesorganisasjon',
'fellesskap':'fellesskap',
'fellesskapar':'fellesskap',
'fellesskapen':'fellesskap',
'fellesskapet':'fellesskap',
'fellesskapsretten':'fellesskapsrett',
'fellesskyldnader':'fellesskyldnad',
'fellestenester':'fellestenest',
'fellestiltak':'fellestiltak',
'fellestiltaka':'fellestiltak',
'fellestrekk':'fellestrekk',
'felt':'felt',
'felter':'felt',
'fem':'fem',
'femdel':'femdel',
'femdels':'femdel',
'femne':'femn',
'femner':'femn',
'femte':'femt',
'femtedel':'femtedel',
'femten':'femt',
'femteparten':'femtepart',
'femti':'femti',
'femårsperioden':'femårsperiod',
'fenja':'fenj',
'fenre':'fenr',
'fenrisulven':'fenrisulv',
'ferd':'ferd',
'ferde':'ferd',
'ferden':'ferd',
'ferdes':'ferd',
'ferdig':'ferd',
'ferdige':'ferd',
'ferdiggjering':'ferdiggjering',
'ferdigheter':'ferd',
'ferdigstillast':'ferdigstill',
'ferdigstilles':'ferdigstill',
'ferdigstilling':'ferdigstilling',
'ferdigstillinga':'ferdigstilling',
'ferdigstilt':'ferdigstilt',
'ferdigstilte':'ferdigstilt',
'ferdigvarer':'ferdigvar',
'ferdsel':'ferdsel',
'fergen':'ferg',
'fergene':'ferg',
'ferger':'ferg',
'fergetilbud':'fergetilbud',
'fergetrafikk':'fergetrafikk',
'fergetrafikken':'fergetrafikk',
'ferje':'ferj',
'ferjeankomstene':'ferjeankomst',
'ferjefartens':'ferjefart',
'ferjeforbindelsen':'ferjeforbind',
'ferjeforbindelsene':'ferjeforbind',
'ferjehavnen':'ferjehavn',
'ferjehavner':'ferjehavn',
'ferjekai':'ferjekai',
'ferjelinjene':'ferjelinj',
'ferjelinjer':'ferjelinj',
'ferjen':'ferj',
'ferjene':'ferj',
'ferjer':'ferj',
'ferjerederiene':'ferjerederi',
'ferjerederier':'ferjerederi',
'ferjeruten':'ferjerut',
'ferjerutene':'ferjerut',
'ferjeruter':'ferjerut',
'ferjeterminalene':'ferjeterminal',
'ferjeterminaler':'ferjeterminal',
'ferjetrafikk':'ferjetrafikk',
'ferjetrafikken':'ferjetrafikk',
'ferrosilisium':'ferrosilisium',
'ferskere':'fersker',
'ferskvann':'ferskvann',
'ferskvannsgehalten':'ferskvannsgehalt',
'ferskvannstilførselen':'ferskvannstilførsel',
'fersum':'fersum',
'fest':'fest',
'festa':'fest',
'festar':'fest',
'feste':'fest',
'festen':'fest',
'fester':'fest',
'festet':'fest',
'festning':'festning',
'festningen':'festning',
'festningstunnelen':'festningstunnel',
'festningsverket':'festningsverk',
'festsal':'festsal',
'festsalen':'festsal',
'festsaler':'festsal',
'fet':'fet',
'fett':'fett',
'fid':'fid',
'fiende':'fiend',
'fiendeføtter':'fiendeføtt',
'fiendehæren':'fiendehær',
'fienden':'fiend',
'fiendene':'fiend',
'fiendens':'fiend',
'fiender':'fiend',
'fiendskap':'fiendskap',
'fiendtlig':'fiendt',
'figur':'figur',
'figurbetraktninger':'figurbetraktning',
'figuren':'figur',
'figurene':'figur',
'fikk':'fikk',
'fiktive':'fiktiv',
'filial':'filial',
'filialer':'filial',
'filialetablering':'filialetablering',
'filialvirksomhet':'filialvirksom',
'fililaler':'fililal',
'filipstad':'filipstad',
'filipstadområdet':'filipstadområd',
'filipstadterminalen':'filipstadterminal',
'filipstadutbygging':'filipstadutbygging',
'fillehaug':'fillehaug',
'fillene':'fill',
'filler':'fill',
'fillete':'fillet',
'fimafeng':'fimafeng',
'fimbul':'fimbul',
'fin':'fin',
'final':'final',
'finans':'finan',
'finansdepartementet':'finansdepartement',
'finansdepartementets':'finansdepartement',
'finanseringsvirksomhetsloven':'finanseringsvirksom',
'finansforetak':'finansforetak',
'finansiell':'finansiell',
'finansielle':'finansiell',
'finansielt':'finansielt',
'finansierast':'finansier',
'finansiere':'finansier',
'finansieres':'finansier',
'finansiering':'finansiering',
'finansieringa':'finansiering',
'finansieringen':'finansiering',
'finansierings':'finansiering',
'finansieringsføretak':'finansieringsføretak',
'finansieringskilde':'finansieringskild',
'finansieringsordning':'finansieringsordning',
'finansieringsordningar':'finansieringsordning',
'finansieringsplan':'finansieringsplan',
'finansieringsplanen':'finansieringsplan',
'finansieringsverksemd':'finansieringsverksemd',
'finansieringsverksemdslova':'finansieringsverksemd',
'finansieringsvirksomhet':'finansieringsvirksom',
'finansieringsvirksomhetsloven':'finansieringsvirksom',
'finansiert':'finansier',
'finansinnstillingen':'finansinnstilling',
'finansinntekter':'finansinntekt',
'finansinstitusjon':'finansinstitusjon',
'finansinstitusjonar':'finansinstitusjon',
'finansinstitusjonens':'finansinstitusjon',
'finansinstitusjoner':'finansinstitusjon',
'finanskomiteen':'finanskomite',
'finanskomiteens':'finanskomite',
'finansmarkedet':'finansmarked',
'finansmatematikk':'finansmatematikk',
'finansnæringen':'finansnæring',
'finanspolitikk':'finanspolitikk',
'finanssparing':'finanssparing',
'fine':'fin',
'finere':'finer',
'fineste':'finest',
'finger':'fing',
'fingre':'fingr',
'fingrene':'fingr',
'finland':'finland',
'finn':'finn',
'finnast':'finn',
'finne':'finn',
'finnenes':'finn',
'finner':'finn',
'finnes':'finn',
'finnmark':'finnmark',
'finrekne':'finrekn',
'finst':'finst',
'fint':'fint',
'fire':'fir',
'firedel':'firedel',
'firedelar':'firedel',
'firefelts':'firefelt',
'firkantede':'firkant',
'firma':'firm',
'firmaattest':'firmaattest',
'firmaet':'firma',
'firmateiknar':'firmateikn',
'fisjon':'fisjon',
'fisjoner':'fisjon',
'fisk':'fisk',
'fiskarlag':'fiskarlag',
'fiske':'fisk',
'fiskeeksport':'fiskeeksport',
'fiskelykke':'fiskelykk',
'fisken':'fisk',
'fiskens':'fisk',
'fiskeolje':'fiskeolj',
'fisker':'fisk',
'fiskere':'fisker',
'fiskeren':'fisker',
'fiskeridepartementet':'fiskeridepartement',
'fiskeridepartementets':'fiskeridepartement',
'fiskerihavner':'fiskerihavn',
'fiskerileder':'fiskeriled',
'fiskeriministeren':'fiskeriminister',
'fiskerinæringen':'fiskerinæring',
'fiskesalslaga':'fiskesalslag',
'fiskeskjell':'fiskeskjell',
'fiskestang':'fiskestang',
'fiskestangen':'fiskestang',
'fiskestenger':'fiskesteng',
'fisketur':'fisketur',
'fisketuren':'fisketur',
'fiskeværene':'fiskevær',
'fisks':'fisk',
'fistelstemme':'fistelstemm',
'fjalar':'fjal',
'fjas':'fjas',
'fjell':'fjell',
'fjellanlegg':'fjellanlegg',
'fjellene':'fjell',
'fjellet':'fjell',
'fjellfestningen':'fjellfestning',
'fjellgård':'fjellgård',
'fjellkløft':'fjellkløft',
'fjellknauser':'fjellknaus',
'fjells':'fjell',
'fjellsiden':'fjellsid',
'fjellsider':'fjellsid',
'fjerde':'fjerd',
'fjerdeparten':'fjerdepart',
'fjern':'fjern',
'fjerne':'fjern',
'fjerning':'fjerning',
'fjeset':'fjes',
'fjollete':'fjollet',
'fjord':'fjord',
'fjordane':'fjord',
'fjordarm':'fjordarm',
'fjordby':'fjordby',
'fjordbyen':'fjordby',
'fjordbyløsningen':'fjordbyløsning',
'fjordbyscenariet':'fjordbyscenari',
'fjorden':'fjord',
'fjorten':'fjort',
'fjær':'fjær',
'fjæresteinene':'fjærestein',
'fjærhammen':'fjærhamm',
'fjøset':'fjøs',
'fjøskone':'fjøskon',
'fl':'fl',
'flak':'flak',
'flakket':'flakk',
'flakse':'flaks',
'flaksende':'flaks',
'flakser':'flaks',
'flakvarp':'flakvarp',
'flakvarpområdet':'flakvarpområd',
'flammende':'flamm',
'flammene':'flamm',
'flammer':'flamm',
'flammesverd':'flammesverd',
'flammesverdet':'flammesverd',
'flaskehalsen':'flaskehals',
'flaskehalser':'flaskehals',
'flat':'flat',
'flate':'flat',
'flater':'flat',
'flatt':'flatt',
'flaum':'flaum',
'fleirbustadbustadhus':'fleirbustadbustadhus',
'fleirbustadhus':'fleirbustadhus',
'fleire':'fleir',
'fleirtal':'fleirtal',
'fleirtalet':'fleirtal',
'fleirtalskompetanse':'fleirtalskompetans',
'fleirtalskrav':'fleirtalskrav',
'fleirtalskrava':'fleirtalskrav',
'fleirtalskravet':'fleirtalskrav',
'fleirtalsmakt':'fleirtalsmakt',
'fleirtalsmisbruk':'fleirtalsmisbruk',
'fleirtalsmynde':'fleirtalsmynd',
'fleirtalsregelen':'fleirtalsregel',
'fleirtalsvedtak':'fleirtalsvedtak',
'flekken':'flekk',
'flekker':'flekk',
'flekkete':'flekket',
'fleksibel':'fleksibel',
'fleksibelt':'fleksibelt',
'fleksibilitet':'fleksibilit',
'fleksibiliteten':'fleksibilitet',
'fleksible':'fleksibl',
'flenger':'fleng',
'flere':'fler',
'flertall':'flertall',
'flertallet':'flertall',
'flertallets':'flertall',
'flertallsregelen':'flertallsregel',
'flesk':'flesk',
'flest':'flest',
'fleste':'flest',
'fletter':'flett',
'flettet':'flett',
'flg':'flg',
'flight':'flight',
'flimrer':'flimr',
'flink':'flink',
'flirer':'flir',
'flis':'flis',
'fliser':'flis',
'flodbølge':'flodbølg',
'flokete':'floket',
'flokk':'flokk',
'flokken':'flokk',
'flokker':'flokk',
'flora':'flor',
'flott':'flott',
'flotteste':'flottest',
'flua':'flua',
'flue':'flue',
'fluer':'fluer',
'flukt':'flukt',
'fly':'fly',
'flydrivstoff':'flydrivstoff',
'flykte':'flykt',
'flykter':'flykt',
'flyktningar':'flyktning',
'flyplasser':'flyplass',
'flyr':'flyr',
'flyt':'flyt',
'flytende':'flyt',
'flyter':'flyt',
'flytrafikk':'flytrafikk',
'flytransport':'flytransport',
'flytta':'flytt',
'flyttar':'flytt',
'flyttbar':'flyttb',
'flyttbare':'flyttbar',
'flyttbart':'flyttbart',
'flytte':'flytt',
'flytteadgang':'flytteadgang',
'flyttebestemmelse':'flyttebestemm',
'flytteforskriften':'flytteforskrift',
'flytter':'flytt',
'flytterett':'flytterett',
'flyttes':'flytt',
'flyttet':'flytt',
'flytting':'flytting',
'flyttingen':'flytting',
'flyvende':'flyv',
'flådd':'flådd',
'flår':'flår',
'flåte':'flåt',
'flåter':'flåt',
'fløy':'fløy',
'fnis':'fnis',
'fnise':'fnis',
'fnisende':'fnis',
'fniser':'fnis',
'fnyser':'fnys',
'fob':'fob',
'fokus':'fokus',
'fokuserast':'fokuser',
'fokusere':'fokuser',
'fokuserer':'fokuser',
'fokuseres':'fokuser',
'fokusert':'fokuser',
'folde':'fold',
'fole':'fol',
'folk':'folk',
'folkene':'folk',
'folkeparti':'folkeparti',
'folket':'folk',
'folketrygd':'folketrygd',
'folketrygda':'folketrygd',
'folketrygden':'folketrygd',
'folketrygdens':'folketrygd',
'folketrygdloven':'folketrygd',
'folketrygdlovens':'folketrygd',
'folketrygdpensjon':'folketrygdpensjon',
'folketrygdpensjonen':'folketrygdpensjon',
'folketrygdytelse':'folketrygdyt',
'folketrygdytelsen':'folketrygdyt',
'folkevalgte':'folkevalgt',
'folksomt':'folksomt',
'folkvang':'folkvang',
'follo':'follo',
'fond':'fond',
'fondene':'fond',
'fondering':'fondering',
'fondert':'fonder',
'fondet':'fond',
'fondets':'fond',
'fonds':'fond',
'fondsandeler':'fondsandel',
'fondsavkastning':'fondsavkastning',
'fondsinnskudd':'fondsinnskudd',
'fondskonstruksjoner':'fondskonstruksjon',
'fondsoppbyggede':'fondsoppbygg',
'fondsoppbygging':'fondsoppbygging',
'fonner':'fonn',
'foot':'foot',
'for':'for',
'forakter':'forakt',
'foran':'foran',
'forandre':'forandr',
'forandres':'forandr',
'forandret':'forandr',
'forankrast':'forankr',
'forankring':'forankring',
'forannevnte':'forannevnt',
'forarbeidene':'forarbeid',
'forarbeidet':'forarbeid',
'forargelse':'forarg',
'forarget':'forarg',
'forbannelse':'forbann',
'forbannelsen':'forbann',
'forbannelser':'forbann',
'forbannet':'forbann',
'forbauselse':'forbaus',
'forbedre':'forbedr',
'forbedres':'forbedr',
'forbedret':'forbedr',
'forbedring':'forbedring',
'forbedringene':'forbedring',
'forbedringer':'forbedring',
'forbehold':'forbehold',
'forberede':'forber',
'forberedelse':'forbered',
'forbereder':'forbered',
'forberedt':'forbered',
'forbi':'forbi',
'forbigått':'forbigått',
'forbindelse':'forbind',
'forbindelsen':'forbind',
'forbindelser':'forbind',
'forbinder':'forbind',
'forbli':'forbli',
'forblir':'forblir',
'forblitt':'forblitt',
'forblåste':'forblåst',
'forbod':'forbod',
'forboden':'forbod',
'forbodet':'forbod',
'forbrente':'forbrent',
'forbruk':'forbruk',
'forbrukar':'forbruk',
'forbrukarane':'forbrukar',
'forbrukarar':'forbrukar',
'forbrukaravtalar':'forbrukaravtal',
'forbrukare':'forbrukar',
'forbrukaren':'forbrukar',
'forbrukarforhold':'forbrukarforhold',
'forbrukarkjøp':'forbrukarkjøp',
'forbrukarkontor':'forbrukarkontor',
'forbrukarkooperative':'forbrukarkooperativ',
'forbrukarlovgjeving':'forbrukarlovgjeving',
'forbrukarlovgjevinga':'forbrukarlovgjeving',
'forbrukarombodet':'forbrukarombod',
'forbrukarorganisasjonar':'forbrukarorganisasjon',
'forbrukarpolitikk':'forbrukarpolitikk',
'forbrukarrådet':'forbrukarråd',
'forbrukarsamvirke':'forbrukarsamvirk',
'forbrukarsamvirkelag':'forbrukarsamvirkelag',
'forbrukarsamvirkelaga':'forbrukarsamvirkelag',
'forbrukarsamvirket':'forbrukarsamvirk',
'forbrukarstyrte':'forbrukarstyrt',
'forbrukartilhøve':'forbrukartilhøv',
'forbrukartvistar':'forbrukartvist',
'forbrukartvistutval':'forbrukartvistutval',
'forbrukartvistutvalet':'forbrukartvistutval',
'forbrukermarkeder':'forbrukermarked',
'forbruket':'forbruk',
'forbruksavhengige':'forbruksavheng',
'forbruksmessige':'forbruksmess',
'forbruksvarer':'forbruksvar',
'forbud':'forbud',
'forbudet':'forbud',
'forbudt':'forbud',
'forbundet':'forbund',
'forby':'forby',
'forbyr':'forbyr',
'fordel':'fordel',
'fordelaktig':'fordelakt',
'fordelane':'fordel',
'fordelar':'fordel',
'fordelast':'fordel',
'fordele':'fordel',
'fordelen':'fordel',
'fordelene':'fordel',
'fordeler':'fordel',
'fordeles':'fordel',
'fordeling':'fordeling',
'fordelinga':'fordeling',
'fordelingen':'fordeling',
'fordelinger':'fordeling',
'fordelingsmekanisme':'fordelingsmekanism',
'fordelingsmessig':'fordelingsmess',
'fordelingsmessige':'fordelingsmess',
'fordelingsnøkkel':'fordelingsnøkkel',
'fordelingsoppgaver':'fordelingsoppgav',
'fordelingspolitikken':'fordelingspolitikk',
'fordelingspolitiske':'fordelingspolitisk',
'fordelingsprofil':'fordelingsprofil',
'fordelingsprofilen':'fordelingsprofil',
'fordelingsreglane':'fordelingsregl',
'fordelingsreglene':'fordelingsregl',
'fordelingsscenarier':'fordelingsscenari',
'fordelingsspørsmåla':'fordelingsspørsmål',
'fordelingsvirkninger':'fordelingsvirkning',
'fordelsbeskattes':'fordelsbeskatt',
'fordelt':'fordelt',
'fordelte':'fordelt',
'forderung':'forderung',
'fordervelse':'forderv',
'fordi':'fordi',
'fordobling':'fordobling',
'fordre':'fordr',
'fordring':'fordring',
'fordringer':'fordring',
'fordringshavernes':'fordringshavern',
'fordyrende':'fordyr',
'fordømte':'fordømt',
'foredle':'foredl',
'foredling':'foredling',
'foredlingsrad':'foredlingsrad',
'foredlingsverksemder':'foredlingsverksemd',
'foregå':'foregå',
'foregående':'foregå',
'foregår':'foregår',
'foregått':'foregått',
'foreining':'foreining',
'foreininga':'foreining',
'foreiningane':'foreining',
'foreiningar':'foreining',
'foreiningskassa':'foreiningskass',
'foreiningsreglane':'foreiningsregl',
'forekomme':'forekomm',
'forekommer':'forekomm',
'forekommet':'forekomm',
'forelagt':'forelagt',
'forelda':'foreld',
'forelding':'forelding',
'foreldinga':'forelding',
'foreldingsfrist':'foreldingsfrist',
'foreldingsfristen':'foreldingsfrist',
'foreldingsregelen':'foreldingsregel',
'foreldre':'foreldr',
'foreldrene':'foreldr',
'forelegg':'forelegg',
'forelegges':'forelegg',
'foreligge':'foreligg',
'foreliggende':'foreligg',
'foreligger':'foreligg',
'forelskelser':'forelsk',
'forelsker':'forelsk',
'forelsket':'forelsk',
'forelå':'forelå',
'foreløbig':'foreløb',
'foreløpig':'foreløp',
'forene':'for',
'forening':'forening',
'foreningar':'forening',
'foreningen':'forening',
'foreningens':'forening',
'foreninger':'forening',
'forenings':'forening',
'foreningsandel':'foreningsandel',
'foreningsliv':'foreningsliv',
'forenkla':'forenkl',
'forenklende':'forenkl',
'forenklet':'forenkl',
'forenkling':'forenkling',
'forenklingar':'forenkling',
'forenlig':'foren',
'foreskrivs':'foreskriv',
'foreslo':'foreslo',
'foreslå':'foreslå',
'foreslår':'foreslår',
'foreslås':'foreslås',
'foreslått':'foreslått',
'foreslåtte':'foreslått',
'forestille':'forestill',
'forestå':'forestå',
'forestående':'forestå',
'forestår':'forestår',
'foresvinner':'foresvinn',
'foreta':'foret',
'foretak':'foretak',
'foretakene':'foretak',
'foretakenes':'foretak',
'foretaket':'foretak',
'foretakets':'foretak',
'foretakpensjonsordninger':'foretakpensjonsordning',
'foretaks':'foretaks',
'foretaksendringer':'foretaksendring',
'foretaksopprettet':'foretaksopprett',
'foretaksordning':'foretaksordning',
'foretaksordningen':'foretaksordning',
'foretaksorganisert':'foretaksorganiser',
'foretakspensjon':'foretakspensjon',
'foretakspensjonen':'foretakspensjon',
'foretakspensjoner':'foretakspensjon',
'foretakspensjonsforsikring':'foretakspensjonsforsikring',
'foretakspensjonsordning':'foretakspensjonsordning',
'foretakspensjonsordningen':'foretakspensjonsordning',
'foretakspensjonsordningene':'foretakspensjonsordning',
'foretakspensjonsordninger':'foretakspensjonsordning',
'foretakspensjonsordningers':'foretakspensjonsordning',
'foretaksstørrelse':'foretaksstørr',
'foretar':'foret',
'foretas':'foret',
'foretatt':'foretatt',
'foretatte':'foretatt',
'foretrekke':'foretrekk',
'forfall':'forfall',
'forfalle':'forfall',
'forfallen':'forfall',
'forfallsdagen':'forfallsdag',
'forfallstid':'forfallstid',
'forfallstidspunktet':'forfallstidspunkt',
'forfalne':'forfaln',
'forfalska':'forfalsk',
'forfalsking':'forfalsking',
'forfalte':'forfalt',
'forfattarverksemd':'forfattarverksemd',
'forfedre':'forfedr',
'forfell':'forfell',
'forferdet':'forferd',
'forfordelte':'forfordelt',
'forfulgt':'forfulgt',
'forfølger':'forfølg',
'forføye':'forføy',
'forføyer':'forføy',
'forgjeves':'forgjev',
'forgreininger':'forgreining',
'forhandla':'forhandl',
'forhandle':'forhandl',
'forhandler':'forhandl',
'forhandlet':'forhandl',
'forhandling':'forhandling',
'forhandlingar':'forhandling',
'forhandlinger':'forhandling',
'forhandlingssituasjon':'forhandlingssituasjon',
'forhandlingsspørsmål':'forhandlingsspørsmål',
'forhekselse':'forheks',
'forhindre':'forhindr',
'forhindrer':'forhindr',
'forhindret':'forhindr',
'forhold':'forhold',
'forholda':'forhold',
'forholde':'forhold',
'forholdene':'forhold',
'forholder':'forhold',
'forholdet':'forhold',
'forholdsmessig':'forholdsmess',
'forholdsmessige':'forholdsmess',
'forholdsmessighet':'forholdsmess',
'forholdsmessighetprinsippet':'forholdsmessighetprinsipp',
'forholdsmessighetskriterier':'forholdsmessighetskriteri',
'forholdsmessighetsprinsipp':'forholdsmessighetsprinsipp',
'forholdsmessighetsprinsippet':'forholdsmessighetsprinsipp',
'forholdsvis':'forholdsvis',
'forholdt':'forhold',
'forhånd':'forhånd',
'forhåndsavgjørelse':'forhåndsavgjør',
'forhåndsavgjørelser':'forhåndsavgjør',
'forhåndsfinansieres':'forhåndsfinansier',
'forhåndsfinansiert':'forhåndsfinansier',
'forhåndssikres':'forhåndssikr',
'forhøye':'forhøy',
'forhøyes':'forhøy',
'forhøyet':'forhøy',
'forkant':'forkant',
'forkasta':'forkast',
'forkaste':'forkast',
'forkastet':'forkast',
'forkasting':'forkasting',
'forkjøpsrett':'forkjøpsrett',
'forkjøpsrettane':'forkjøpsrett',
'forkjøpsrettar':'forkjøpsrett',
'forkjøpsretten':'forkjøpsrett',
'forkjøpsretthavaren':'forkjøpsretthavar',
'forkjøpsrettshavar':'forkjøpsrettshav',
'forkjøpsrettshavarane':'forkjøpsrettshavar',
'forkjøpsrettshavarar':'forkjøpsrettshavar',
'forkjøpsrettshavaren':'forkjøpsrettshavar',
'forkjøpsspørsmålet':'forkjøpsspørsmål',
'forklare':'forklar',
'forklarer':'forklar',
'forklares':'forklar',
'forklaring':'forklaring',
'forklaringer':'forklaring',
'forklart':'forklart',
'forkledd':'forkledd',
'forkledning':'forkledning',
'forkledningen':'forkledning',
'forkledninger':'forkledning',
'forkler':'forkl',
'forklet':'forkl',
'forklåre':'forklår',
'forklåring':'forklåring',
'forklåringa':'forklåring',
'forkortes':'forkort',
'forlag':'forlag',
'forlaget':'forlag',
'forlagsinsatser':'forlagsinsats',
'forlagskapital':'forlagskapital',
'forlagt':'forlagt',
'forlangast':'forlang',
'forlanger':'forlang',
'forlate':'forlat',
'forlatt':'forlatt',
'forleden':'forled',
'forlengede':'forleng',
'forlengelse':'forleng',
'forlengelsen':'forleng',
'forlenget':'forleng',
'forlik':'forlik',
'forliksråda':'forliksråd',
'forliksrådet':'forliksråd',
'forlikt':'forlikt',
'forlikte':'forlikt',
'forlot':'forlot',
'forløp':'forløp',
'form':'form',
'forma':'form',
'formalia':'formali',
'formalisering':'formalisering',
'formalisert':'formaliser',
'formaliserte':'formaliser',
'formannskap':'formannskap',
'formannskapet':'formannskap',
'formannskapets':'formannskap',
'forme':'form',
'formell':'formell',
'formelle':'formell',
'formelt':'formelt',
'formen':'form',
'formene':'form',
'former':'form',
'formet':'form',
'formidabel':'formidabel',
'formidable':'formidabl',
'formidla':'formidl',
'formidlar':'formidl',
'formidlarane':'formidlar',
'formidlarar':'formidlar',
'formidler':'formidl',
'formidling':'formidling',
'formidlinga':'formidling',
'formidlingsoppdrag':'formidlingsoppdrag',
'formidlingsverksemd':'formidlingsverksemd',
'formidlingsverksemda':'formidlingsverksemd',
'formkrav':'formkrav',
'formodentlig':'formodent',
'formodning':'formodning',
'formodningen':'formodning',
'formue':'formu',
'formueforvaltning':'formueforvaltning',
'formueforvaltninga':'formueforvaltning',
'formuen':'formu',
'formuesbeskatning':'formuesbeskatning',
'formuesbeskattes':'formuesbeskatt',
'formuesdata':'formuesdat',
'formueselementer':'formueselement',
'formuesforøkelse':'formuesforøk',
'formuesgoda':'formuesgod',
'formuesgode':'formuesgod',
'formuesgoder':'formuesgod',
'formuesgodet':'formuesgod',
'formuesgrupper':'formuesgrupp',
'formuesmasse':'formuesmass',
'formuesobjekt':'formuesobjekt',
'formuesoppbygginga':'formuesoppbygging',
'formuesskatt':'formuesskatt',
'formuesskattegrunnlaget':'formuesskattegrunnlag',
'formuesskatten':'formuesskatt',
'formuesskatteplikt':'formuesskatteplikt',
'formuesskattepliktig':'formuesskatteplikt',
'formuesverdien':'formuesverdi',
'formuesøkning':'formuesøkning',
'formuesøkningen':'formuesøkning',
'formulerast':'formuler',
'formulering':'formulering',
'formuleringa':'formulering',
'formuleringar':'formulering',
'formuleringen':'formulering',
'formulert':'formuler',
'formulerte':'formuler',
'formynderargument':'formynderargument',
'formål':'formål',
'formålet':'formål',
'formålstjenlige':'formålstjen',
'fornemmer':'fornemm',
'fornuft':'fornuft',
'fornuftens':'fornuft',
'fornuftig':'fornuft',
'fornuftsmessige':'fornuftsmess',
'fornyelse':'forny',
'fornyet':'forny',
'fornying':'fornying',
'fornærmer':'fornærm',
'fornærmet':'fornærm',
'fornøyd':'fornøyd',
'fornøyde':'fornøyd',
'forpinte':'forpint',
'forplikta':'forplikt',
'forpliktar':'forplikt',
'forpliktelse':'forplikt',
'forpliktelsene':'forplikt',
'forpliktelser':'forplikt',
'forpliktende':'forplikt',
'forpliktet':'forplikt',
'forrang':'forrang',
'forrenta':'forrent',
'forrentast':'forrent',
'forrenting':'forrenting',
'forresten':'forrest',
'forretningsbygg':'forretningsbygg',
'forretningsdrift':'forretningsdrift',
'forretningseigedommar':'forretningseigedomm',
'forretningsførar':'forretningsfør',
'forretningsførarane':'forretningsførar',
'forretningsførarar':'forretningsførar',
'forretningsføraravtalane':'forretningsføraravtal',
'forretningsføraravtalar':'forretningsføraravtal',
'forretningsføraravtale':'forretningsføraravtal',
'forretningsføraravtalen':'forretningsføraravtal',
'forretningsføraren':'forretningsførar',
'forretningsførarforhold':'forretningsførarforhold',
'forretningsførarhonorar':'forretningsførarhonor',
'forretningsførarhonoraret':'forretningsførarhonorar',
'forretningsførarkontraktar':'forretningsførarkontrakt',
'forretningsføraroppdraga':'forretningsføraroppdrag',
'forretningsføraroppgåver':'forretningsføraroppgåv',
'forretningsførarstyre':'forretningsførarstyr',
'forretningsførarverksemda':'forretningsførarverksemd',
'forretningsføretak':'forretningsføretak',
'forretningsførsel':'forretningsførsel',
'forretningsførselen':'forretningsførsel',
'forretningskommunar':'forretningskommun',
'forretningskontor':'forretningskontor',
'forretningskontoret':'forretningskontor',
'forretningslokale':'forretningslokal',
'forretningsmessig':'forretningsmess',
'forretningsmessige':'forretningsmess',
'forretningsområda':'forretningsområd',
'forretningspraksis':'forretningspraksis',
'forretningsstaden':'forretningsstad',
'forretningssted':'forretningssted',
'forretningsverksemd':'forretningsverksemd',
'forrige':'forr',
'forringa':'forring',
'forringelse':'forring',
'forringing':'forringing',
'forrykking':'forrykking',
'forrykkje':'forrykkj',
'forrykkjer':'forrykkj',
'forråtnelse':'forråtn',
'forsamlingar':'forsamling',
'forseinka':'forseink',
'forseinking':'forseinking',
'forseinkingsrente':'forseinkingsrent',
'forseinkingstilfelle':'forseinkingstilfell',
'forsendelsene':'forsend',
'forsendelser':'forsend',
'forseres':'forser',
'forseringer':'forsering',
'forsert':'forser',
'forsete':'forset',
'forsettleg':'forsett',
'forsettlege':'forsett',
'forsikre':'forsikr',
'forsikrede':'forsikr',
'forsikrer':'forsikr',
'forsikring':'forsikring',
'forsikringa':'forsikring',
'forsikringane':'forsikring',
'forsikringar':'forsikring',
'forsikringen':'forsikring',
'forsikringene':'forsikring',
'forsikringens':'forsikring',
'forsikringer':'forsikring',
'forsikrings':'forsikring',
'forsikringsavtale':'forsikringsavtal',
'forsikringsavtaleloven':'forsikringsavtal',
'forsikringsavtalen':'forsikringsavtal',
'forsikringsavtalene':'forsikringsavtal',
'forsikringsavtaler':'forsikringsavtal',
'forsikringsbasert':'forsikringsbaser',
'forsikringsbaserte':'forsikringsbaser',
'forsikringsbeløpet':'forsikringsbeløp',
'forsikringsbransjen':'forsikringsbransj',
'forsikringselement':'forsikringselement',
'forsikringselementer':'forsikringselement',
'forsikringsfond':'forsikringsfond',
'forsikringsfondet':'forsikringsfond',
'forsikringsforbund':'forsikringsforbund',
'forsikringsform':'forsikringsform',
'forsikringsformer':'forsikringsform',
'forsikringsforpliktelser':'forsikringsforplikt',
'forsikringsgaranti':'forsikringsgaranti',
'forsikringsinstitusjoner':'forsikringsinstitusjon',
'forsikringskapital':'forsikringskapital',
'forsikringsklasser':'forsikringsklass',
'forsikringskollektiv':'forsikringskollektiv',
'forsikringskollektivet':'forsikringskollektiv',
'forsikringskontrakt':'forsikringskontrakt',
'forsikringskontraktene':'forsikringskontrakt',
'forsikringskontrakter':'forsikringskontrakt',
'forsikringsløsning':'forsikringsløsning',
'forsikringsløsninger':'forsikringsløsning',
'forsikringsmekanismen':'forsikringsmekanism',
'forsikringsmeklarar':'forsikringsmeklar',
'forsikringsmessig':'forsikringsmess',
'forsikringsmessige':'forsikringsmess',
'forsikringsmuligheter':'forsikringsmu',
'forsikringsområdet':'forsikringsområd',
'forsikringsordning':'forsikringsordning',
'forsikringsordninger':'forsikringsordning',
'forsikringsorganisert':'forsikringsorganiser',
'forsikringspoliser':'forsikringspolis',
'forsikringspremiar':'forsikringspremi',
'forsikringsprodukt':'forsikringsprodukt',
'forsikringsprodukter':'forsikringsprodukt',
'forsikringsrisiko':'forsikringsrisiko',
'forsikringsselskap':'forsikringsselskap',
'forsikringsselskapene':'forsikringsselskap',
'forsikringsselskaper':'forsikringsselskap',
'forsikringsselskapers':'forsikringsselskap',
'forsikringsselskapet':'forsikringsselskap',
'forsikringsspørsmål':'forsikringsspørsmål',
'forsikringssum':'forsikringssum',
'forsikringstaker':'forsikringstak',
'forsikringstakeren':'forsikringstaker',
'forsikringstakerne':'forsikringstakern',
'forsikringsteknisk':'forsikringsteknisk',
'forsikringstekniske':'forsikringsteknisk',
'forsikringstid':'forsikringstid',
'forsikringstiden':'forsikringstid',
'forsikringstilfellet':'forsikringstilfell',
'forsikringsvilkårene':'forsikringsvilkår',
'forsikringsvirksomhet':'forsikringsvirksom',
'forsikringsvirksomhetsloven':'forsikringsvirksom',
'forsikringsvirksomhetslovens':'forsikringsvirksom',
'forsikringsytelsen':'forsikringsyt',
'forsikringsytelsene':'forsikringsyt',
'forsiktig':'forsikt',
'forsiktige':'forsikt',
'forsiktighet':'forsikt',
'forsiktighetshensyn':'forsiktighetshensyn',
'forsinkelser':'forsink',
'forsinket':'forsink',
'forsker':'forsk',
'forskjell':'forskjell',
'forskjellen':'forskjell',
'forskjellene':'forskjell',
'forskjeller':'forskjell',
'forskjellig':'forskjel',
'forskjellige':'forskjel',
'forskjellsbehandle':'forskjellsbehandl',
'forskjellsbehandling':'forskjellsbehandling',
'forskning':'forskning',
'forskningsinstitutter':'forskningsinstitutt',
'forskot':'forskot',
'forskota':'forskot',
'forskotet':'forskot',
'forskotsbetaling':'forskotsbetaling',
'forskrekket':'forskrekk',
'forskrift':'forskrift',
'forskrifta':'forskrift',
'forskriften':'forskrift',
'forskriftene':'forskrift',
'forskriftens':'forskrift',
'forskrifter':'forskrift',
'forskrifts':'forskrift',
'forskriftshjemmel':'forskriftshjemmel',
'forskriftsverket':'forskriftsverk',
'forskuddsbetaling':'forskuddsbetaling',
'forskuddsbetalt':'forskuddsbetalt',
'forskuddsfinansierte':'forskuddsfinansier',
'forskuddsinnskudd':'forskuddsinnskudd',
'forskuddspremie':'forskuddspremi',
'forskuddsvis':'forskuddsvis',
'forskyvning':'forskyvning',
'forslag':'forslag',
'forslagene':'forslag',
'forslaget':'forslag',
'forslagets':'forslag',
'forslitte':'forslitt',
'forslår':'forslår',
'forsprang':'forsprang',
'forspranget':'forsprang',
'forstand':'forstand',
'forstandardskapet':'forstandardskap',
'forstandarskapet':'forstandarskap',
'forstanden':'forstand',
'forstarkt':'forstarkt',
'forsterke':'forsterk',
'forsterker':'forsterk',
'forsterkes':'forsterk',
'forsterket':'forsterk',
'forsto':'forsto',
'forstod':'forstod',
'forstoffer':'forstoff',
'forstsetter':'forstsett',
'forstyrre':'forstyrr',
'forstyrrelser':'forstyrr',
'forstå':'forstå',
'forståelig':'forstå',
'forståelse':'forstå',
'forståelsen':'forstå',
'forståelsesfull':'forståelsesfull',
'forståing':'forståing',
'forståinga':'forståing',
'forstår':'forstår',
'forstås':'forstås',
'forstått':'forstått',
'forstørrelsesglass':'forstørrelsesglass',
'forsvant':'forsvant',
'forsvarar':'forsvar',
'forsvarast':'forsvar',
'forsvare':'forsvar',
'forsvarer':'forsvar',
'forsvares':'forsvar',
'forsvaret':'forsvar',
'forsvarleg':'forsvar',
'forsvarlege':'forsvar',
'forsvarlig':'forsvar',
'forsvarsmessig':'forsvarsmess',
'forsvart':'forsvart',
'forsvinne':'forsvinn',
'forsvinner':'forsvinn',
'forsvunnet':'forsvunn',
'forsyningsberedskap':'forsyningsberedskap',
'forsyningsberedskapen':'forsyningsberedskap',
'forsøk':'forsøk',
'forsøke':'forsøk',
'forsøker':'forsøk',
'forsøksordningar':'forsøksordning',
'forsøkt':'forsøkt',
'forsøme':'forsøm',
'forsømer':'forsøm',
'forsøming':'forsøming',
'forsømt':'forsømt',
'forsørge':'forsørg',
'forsørgelsesbyrdene':'forsørgelsesbyrd',
'forsørget':'forsørg',
'fort':'fort',
'fortalt':'fortalt',
'fortalte':'fortalt',
'fortell':'fortell',
'fortelle':'fortell',
'forteller':'fortell',
'forteneste':'fortenest',
'fortere':'forter',
'fortid':'fortid',
'fortjener':'fortjen',
'fortjent':'fortjent',
'fortolkinga':'fortolking',
'fortolkning':'fortolkning',
'fortolles':'fortoll',
'fortollet':'fortoll',
'fortolling':'fortolling',
'fortollingssted':'fortollingssted',
'fortrengde':'fortrengd',
'fortrenger':'fortreng',
'fortrengje':'fortrengj',
'fortrengning':'fortrengning',
'fortrinn':'fortrinn',
'fortrinnsvis':'fortrinnsvis',
'fortrolige':'fortro',
'fortrolighet':'fortro',
'fortsatt':'fortsatt',
'fortsatte':'fortsatt',
'fortsette':'fortsett',
'fortsettelse':'fortsett',
'fortsettelsesforsikring':'fortsettelsesforsikring',
'fortsettelsesforsikringen':'fortsettelsesforsikring',
'fortsettelsesforsikringer':'fortsettelsesforsikring',
'fortsettelsessikring':'fortsettelsessikring',
'fortsetter':'fortsett',
'fortumlet':'fortuml',
'fortvilelse':'fortvil',
'fortvilet':'fortvil',
'fortærende':'fortær',
'fortøyningene':'fortøyning',
'fortøyninger':'fortøyning',
'forum':'forum',
'forunderlig':'forunder',
'forunderlige':'forunder',
'forundret':'forundr',
'forundring':'forundring',
'forureining':'forureining',
'forurensing':'forurensing',
'forurensingsmål':'forurensingsmål',
'forurensning':'forurensning',
'forut':'forut',
'forutberegnelighet':'forutberegn',
'foruten':'forut',
'forutgående':'forutgå',
'forutsatt':'forutsatt',
'forutsatte':'forutsatt',
'forutseielege':'forutsei',
'forutses':'foruts',
'forutsetning':'forutsetning',
'forutsetningen':'forutsetning',
'forutsetningene':'forutsetning',
'forutsetninger':'forutsetning',
'forutsetningsvis':'forutsetningsvis',
'forutsette':'forutsett',
'forutsetter':'forutsett',
'forutsettes':'forutsett',
'forutsies':'forutsi',
'forutsigbar':'forutsigb',
'forutsigbarhet':'forutsigbar',
'forvalta':'forvalt',
'forvaltar':'forvalt',
'forvaltarar':'forvaltar',
'forvaltaren':'forvaltar',
'forvaltarrolle':'forvaltarroll',
'forvaltast':'forvalt',
'forvalte':'forvalt',
'forvalter':'forvalt',
'forvalterens':'forvalter',
'forvaltes':'forvalt',
'forvaltet':'forvalt',
'forvaltinga':'forvalting',
'forvaltning':'forvaltning',
'forvaltninga':'forvaltning',
'forvaltningen':'forvaltning',
'forvaltningsalternativene':'forvaltningsalternativ',
'forvaltningsavtalar':'forvaltningsavtal',
'forvaltningsenhet':'forvaltningsen',
'forvaltningsforetak':'forvaltningsforetak',
'forvaltningsinstitusjon':'forvaltningsinstitusjon',
'forvaltningsinstitusjonen':'forvaltningsinstitusjon',
'forvaltningsinstitusjonens':'forvaltningsinstitusjon',
'forvaltningskapital':'forvaltningskapital',
'forvaltningskontrakt':'forvaltningskontrakt',
'forvaltningskostnader':'forvaltningskostnad',
'forvaltningslova':'forvaltning',
'forvaltningsloven':'forvaltning',
'forvaltningsmessige':'forvaltningsmess',
'forvaltningsoppdraget':'forvaltningsoppdrag',
'forvaltningsoppgåve':'forvaltningsoppgåv',
'forvaltningsoppgåvene':'forvaltningsoppgåv',
'forvaltningsoppgåver':'forvaltningsoppgåv',
'forvaltningsorgan':'forvaltningsorgan',
'forvaltningspraksis':'forvaltningspraksis',
'forvaltningsprodukter':'forvaltningsprodukt',
'forvaltningsrettane':'forvaltningsrett',
'forvaltningsrettar':'forvaltningsrett',
'forvaltningsselskap':'forvaltningsselskap',
'forvaltningsselskaper':'forvaltningsselskap',
'forvaltningsselskapet':'forvaltningsselskap',
'forvaltningsspørsmålet':'forvaltningsspørsmål',
'forvaltningstenestene':'forvaltningstenest',
'forvaltningstenester':'forvaltningstenest',
'forvaltningsverksemd':'forvaltningsverksemd',
'forvaltningsverksemda':'forvaltningsverksemd',
'forvandler':'forvandl',
'forvandles':'forvandl',
'forvandlet':'forvandl',
'forvandling':'forvandling',
'forvente':'forvent',
'forventede':'forvent',
'forventer':'forvent',
'forventes':'forvent',
'forventet':'forvent',
'forventninger':'forventning',
'forvirret':'forvirr',
'forvirring':'forvirring',
'forvirringens':'forvirring',
'forårsaket':'forårsak',
'forøvrig':'forøvr',
'foss':'foss',
'fossefall':'fossefall',
'fossen':'foss',
'fosser':'foss',
'fosset':'foss',
'fosterbarn':'fosterbarn',
'fosterborn':'fosterborn',
'fosterbror':'fosterbror',
'fosterbroren':'fosterbror',
'fosterfar':'fosterf',
'fostermoren':'fostermor',
'fostersønn':'fostersønn',
'fostre':'fostr',
'fostret':'fostr',
'fot':'fot',
'foten':'fot',
'fotnote':'fotnot',
'fotografere':'fotografer',
'fots':'fot',
'fotside':'fotsid',
'fottøyet':'fottøy',
'fra':'fra',
'fradrag':'fradrag',
'fradraget':'fradrag',
'fradragsbegrensning':'fradragsbegrensning',
'fradragsberettigede':'fradragsberett',
'fradragsberettiget':'fradragsberett',
'fradragshjemmel':'fradragshjemmel',
'fradragsramme':'fradragsramm',
'fradragsrammen':'fradragsramm',
'fradragsrammer':'fradragsramm',
'fradragsreglene':'fradragsregl',
'fradragsregler':'fradragsregl',
'fradragsrett':'fradragsrett',
'fradragsretten':'fradragsrett',
'fraflyttede':'fraflytt',
'fragmenter':'fragment',
'fragmentert':'fragmenter',
'frakt':'frakt',
'fraktdistansen':'fraktdistans',
'frakte':'frakt',
'frakter':'frakt',
'fraktes':'frakt',
'fraktet':'frakt',
'fraktkostnadene':'fraktkostnad',
'fraktkostnader':'fraktkostnad',
'fraktområdet':'fraktområd',
'fraktraten':'fraktrat',
'fraktratene':'fraktrat',
'fraktrater':'fraktrat',
'fraktruter':'fraktrut',
'fraktsammenheng':'fraktsammenheng',
'fraktskip':'fraktskip',
'fram':'fram',
'framfor':'framfor',
'framføring':'framføring',
'framføringstid':'framføringstid',
'framgangsmåtar':'framgangsmåt',
'framgangsmåte':'framgangsmåt',
'framgangsmåten':'framgangsmåt',
'framgangsmåter':'framgangsmåt',
'framgå':'framgå',
'framgår':'framgår',
'framheva':'framhev',
'framheve':'framhev',
'framhever':'framhev',
'framhevet':'framhev',
'framheving':'framheving',
'framja':'framj',
'framkomme':'framkomm',
'framkommelighet':'framkomm',
'framkommeligheten':'framkomm',
'framkommer':'framkomm',
'framlegg':'framlegg',
'framlegga':'framlegg',
'framlegge':'framlegg',
'framlegget':'framlegg',
'framleggs':'framlegg',
'framleigar':'framl',
'framleigarens':'framleigar',
'framleige':'framl',
'framleigerettar':'framleigerett',
'framleigt':'framleigt',
'framleis':'framleis',
'framme':'framm',
'frammøte':'frammøt',
'frammøtet':'frammøt',
'frammøtte':'frammøtt',
'framom':'framom',
'framover':'framov',
'framsetjing':'framsetjing',
'framsetter':'framsett',
'framskaffe':'framskaff',
'framskaffelse':'framskaff',
'framskaffing':'framskaffing',
'framskrevet':'framskrev',
'framskrive':'framskriv',
'framskriving':'framskriving',
'framskrivingen':'framskriving',
'framskrivingene':'framskriving',
'framskrivinger':'framskriving',
'framskrivingsalternativer':'framskrivingsalternativ',
'framskrivingsresultater':'framskrivingsresultat',
'framskrivning':'framskrivning',
'framskynde':'framskynd',
'framstavnen':'framstavn',
'framstille':'framstill',
'framstilling':'framstilling',
'framstillinga':'framstilling',
'framstillingen':'framstilling',
'framstilt':'framstilt',
'framstå':'framstå',
'framstår':'framstår',
'framstått':'framstått',
'framtid':'framtid',
'framtida':'framtid',
'framtidas':'framtid',
'framtiden':'framtid',
'framtidens':'framtid',
'framtidig':'framtid',
'framtidige':'framtid',
'framtidsbilde':'framtidsbild',
'framtidsbildene':'framtidsbild',
'framtidsbilder':'framtidsbild',
'framtidsbildet':'framtidsbild',
'framtredende':'framtred',
'framveksten':'framvekst',
'framvising':'framvising',
'frankrike':'frankrik',
'fraser':'fras',
'fraskrive':'fraskriv',
'fratre':'fratr',
'fratredelse':'fratred',
'fratredelsen':'fratred',
'fratredelsestidspunktet':'fratredelsestidspunkt',
'fratreden':'fratred',
'fratrekk':'fratrekk',
'fratrer':'fratr',
'fratrukket':'fratrukk',
'fratrådt':'fratråd',
'fratrådte':'fratråd',
'fratrådtes':'fratråd',
'fravike':'fravik',
'fravikes':'fravik',
'fred':'fred',
'fredelige':'fred',
'freden':'fred',
'frederikshavn':'frederikshavn',
'fredrikstad':'fredrikstad',
'fredstid':'fredstid',
'free':'free',
'freight':'freight',
'freistnad':'freistnad',
'freke':'frek',
'frekvens':'frekv',
'frem':'frem',
'fremadskuende':'fremadsku',
'fremdeles':'fremdel',
'fremfor':'fremfor',
'fremgår':'fremgår',
'fremja':'fremj',
'fremjar':'fremj',
'fremjast':'fremj',
'fremje':'fremj',
'fremjing':'fremjing',
'fremleie':'fremlei',
'fremme':'fremm',
'fremmed':'fremmed',
'fremmede':'fremm',
'fremmer':'fremm',
'fremmes':'fremm',
'fremmet':'fremm',
'fremover':'fremov',
'fremskrittspartiet':'fremskrittsparti',
'fremst':'fremst',
'fremste':'fremst',
'fremt':'fremt',
'fremtid':'fremtid',
'fremtiden':'fremtid',
'fremtidig':'fremtid',
'frendedrap':'frendedrap',
'frender':'frend',
'fresende':'fres',
'freser':'fres',
'fri':'fri',
'friare':'friar',
'fridom':'fridom',
'friere':'frier',
'frierferd':'frierferd',
'frierføtter':'frierføtt',
'frieriet':'frieri',
'frifunne':'frifunn',
'frigg':'frigg',
'friggs':'frigg',
'frigis':'frigis',
'frigitte':'frigitt',
'frigivelse':'frigiv',
'frigjort':'frigjort',
'frigjorte':'frigjort',
'frigjør':'frigjør',
'frigjøre':'frigjør',
'frigjøres':'frigjør',
'frigjøring':'frigjøring',
'frihavn':'frihavn',
'frihavnen':'frihavn',
'frihet':'frih',
'frikortgrensen':'frikortgrens',
'friluftsinteresser':'friluftsinteress',
'friluftsliv':'friluftsliv',
'friluftslivet':'friluftsliv',
'friluftslivs':'friluftsliv',
'friområde':'friområd',
'fripolise':'fripolis',
'fripolisen':'fripolis',
'fripolisene':'fripolis',
'fripolisens':'fripolis',
'fripoliser':'fripolis',
'fripoliseregisteret':'fripoliseregister',
'fripoliserettighetene':'fripoliserett',
'fripoliserettigheter':'fripoliserett',
'fripolises':'fripolis',
'frisk':'frisk',
'friske':'frisk',
'friskt':'friskt',
'frist':'frist',
'fristane':'frist',
'fristar':'frist',
'friste':'frist',
'fristed':'fristed',
'fristen':'frist',
'fristende':'frist',
'frister':'frist',
'fristforsøminga':'fristforsøming',
'fristille':'fristill',
'fristoversitjing':'fristoversitjing',
'fristrekning':'fristrekning',
'fritak':'fritak',
'fritaket':'fritak',
'fritar':'frit',
'fritas':'frit',
'fritatt':'fritatt',
'friteken':'fritek',
'fritidsaktivitetar':'fritidsaktivitet',
'fritidsaktiviteter':'fritidsaktivitet',
'fritidsarealer':'fritidsareal',
'fritidsbustad':'fritidsbustad',
'fritidsbustader':'fritidsbustad',
'fritidssysler':'fritidssysl',
'fritt':'fritt',
'frittliggjande':'frittliggj',
'frittståande':'frittstå',
'frittstående':'frittstå',
'friviljug':'friviljug',
'friviljuge':'friviljug',
'frivillig':'frivil',
'frivillige':'frivil',
'frode':'frod',
'frodes':'frod',
'frodig':'frod',
'frogn':'frogn',
'frognerkilen':'frognerkil',
'frokost':'frokost',
'frokostbord':'frokostbord',
'frosne':'frosn',
'frosset':'fross',
'frost':'frost',
'frukt':'frukt',
'fruktbare':'fruktbar',
'fruktimporten':'fruktimport',
'fryd':'fryd',
'fryder':'fryd',
'frydet':'fryd',
'frykte':'frykt',
'fryktelig':'frykt',
'frykter':'frykt',
'fryktet':'frykt',
'frynset':'fryns',
'fryse':'frys',
'fryseboksanlegg':'fryseboksanlegg',
'fryser':'frys',
'fryses':'frys',
'frå':'frå',
'frådelt':'frådelt',
'frådelte':'frådelt',
'fråden':'fråd',
'frådende':'fråd',
'frådrag':'frådrag',
'fråfalle':'fråfall',
'fråflytting':'fråflytting',
'fråga':'fråg',
'frånangersfossen':'frånangersfoss',
'fråsegn':'fråsegn',
'fråsegna':'fråsegn',
'fråvere':'fråver',
'fråvikast':'fråvik',
'fråvikelege':'fråvik',
'fråviking':'fråviking',
'fråvikinga':'fråviking',
'fråvikingsklausular':'fråvikingsklausul',
'frøs':'frøs',
'frøy':'frøy',
'frøya':'frøya',
'frøyas':'frøyas',
'frøys':'frøys',
'fugl':'fugl',
'fuglen':'fugl',
'fuglene':'fugl',
'fuglenes':'fugl',
'fuglens':'fugl',
'fugler':'fugl',
'fuglers':'fugl',
'fugleskrik':'fugleskrik',
'fuglum':'fuglum',
'fukt':'fukt',
'fuktig':'fukt',
'fuktighet':'fukt',
'fulgt':'fulgt',
'fulgte':'fulgt',
'full':'full',
'fulla':'full',
'fulle':'full',
'fullere':'fuller',
'fullføring':'fullføring',
'fullført':'fullført',
'fullførte':'fullført',
'fullgod':'fullgod',
'fullmakt':'fullmakt',
'fullmakta':'fullmakt',
'fullmaktene':'fullmakt',
'fullmakter':'fullmakt',
'fullmektig':'fullmekt',
'fullmektigen':'fullmekt',
'fullstendig':'fullstend',
'fullstendige':'fullstend',
'fullt':'fullt',
'fulltids':'fulltid',
'fulltidsinnskudd':'fulltidsinnskudd',
'fulltidsstilling':'fulltidsstilling',
'fullvoksen':'fullvoks',
'fungere':'funger',
'fungerende':'funger',
'fungerer':'funger',
'funkler':'funkl',
'funksjon':'funksjon',
'funksjonell':'funksjonell',
'funksjonelt':'funksjonelt',
'funksjonen':'funksjon',
'funksjoner':'funksjon',
'funksjonshemma':'funksjonshemm',
'funksjonshemmede':'funksjonshemm',
'funksjonshemming':'funksjonshemming',
'funksjonstida':'funksjonstid',
'funksjonstiden':'funksjonstid',
'funksjonær':'funksjonær',
'funksjonærer':'funksjonær',
'funne':'funn',
'funnene':'funn',
'funnet':'funn',
'furten':'furt',
'fusjon':'fusjon',
'fusjoner':'fusjon',
'fusjonerast':'fusjoner',
'fusjonsplanen':'fusjonsplan',
'fyker':'fyk',
'fyldigare':'fyldigar',
'fylgjene':'fylgj',
'fylke':'fylk',
'fylkene':'fylk',
'fylker':'fylk',
'fylkes':'fylk',
'fylkesdelplan':'fylkesdelplan',
'fylkesdelplaner':'fylkesdelplan',
'fylkeskommunal':'fylkeskommunal',
'fylkeskommunale':'fylkeskommunal',
'fylkeskommunalt':'fylkeskommunalt',
'fylkeskommunar':'fylkeskommun',
'fylkeskommune':'fylkeskommun',
'fylkeskommunen':'fylkeskommun',
'fylkeskommunene':'fylkeskommun',
'fylkeskommunenes':'fylkeskommun',
'fylkeskommuner':'fylkeskommun',
'fylkesmannen':'fylkesmann',
'fylkesmannens':'fylkesmann',
'fylkesmiljøvernsjef':'fylkesmiljøvernsjef',
'fylkesplanen':'fylkesplan',
'fylkesplanlegging':'fylkesplanlegging',
'fylkesting':'fylkesting',
'fylkesveg':'fylkesveg',
'fylkesvis':'fylkesvis',
'fylket':'fylk',
'fylkinger':'fylking',
'fylle':'fyll',
'fyller':'fyll',
'fylles':'fyll',
'fyllprat':'fyllprat',
'fylt':'fylt',
'fylte':'fylt',
'fyr':'fyr',
'fyrbelysning':'fyrbelysning',
'fyrer':'fyr',
'fyrste':'fyrst',
'fyrtjenestens':'fyrtjenest',
'fysisk':'fysisk',
'fysiske':'fysisk',
'få':'få',
'fåmælt':'fåmælt',
'fåmælte':'fåmælt',
'får':'får',
'fårbaute':'fårbaut',
'fåtall':'fåtall',
'fått':'fått',
'fåvne':'fåvn',
'fåvnes':'fåvn',
'fåvnesbane':'fåvnesb',
'fæl':'fæl',
'fællesforbund':'fællesforbund',
'fællesforeningen':'fællesforening',
'fælt':'fælt',
'færder':'færd',
'færre':'færr',
'færrest':'færrest',
'føde':'fød',
'føder':'fød',
'føderåd':'føderåd',
'fødes':'fød',
'fødsels':'fødsel',
'fødselsdato':'fødselsdato',
'fødselsgave':'fødselsgav',
'fødselsnummer':'fødselsnumm',
'født':'født',
'fødte':'født',
'føle':'føl',
'følelser':'føl',
'føler':'føl',
'følg':'følg',
'følgde':'følgd',
'følge':'følg',
'følgelig':'følg',
'følgende':'følg',
'følgene':'følg',
'følger':'følg',
'følges':'følg',
'følgesvenn':'følgesvenn',
'følgja':'følgj',
'følgjande':'følgj',
'følgjast':'følgj',
'følgje':'følgj',
'følgjene':'følgj',
'følgjer':'følgj',
'følgt':'følgt',
'føll':'føll',
'føllet':'føll',
'følsomhet':'følsom',
'følsomheten':'følsom',
'følsomme':'følsomm',
'før':'før',
'førarhund':'førarhund',
'førast':'før',
'føre':'før',
'førearbeida':'førearbeid',
'førebels':'førebel',
'førebu':'førebu',
'førebuast':'førebu',
'førebudd':'førebudd',
'førebudde':'førebudd',
'førebuing':'førebuing',
'førebuinga':'førebuing',
'førebur':'førebur',
'førehand':'førehand',
'førehandsavtale':'førehandsavtal',
'førehandsgodkjenning':'førehandsgodkjenning',
'førehandssamtykke':'førehandssamtykk',
'førehandsvarsel':'førehandsvarsel',
'førehaving':'førehaving',
'føremonene':'føremon',
'føremoner':'føremon',
'føremål':'føremål',
'føremåla':'føremål',
'føremålet':'føremål',
'føremålstenleg':'føremålsten',
'førende':'før',
'fører':'før',
'førerett':'førerett',
'føreretten':'førerett',
'føres':'før',
'føresegn':'føresegn',
'føresegna':'føresegn',
'føresegnene':'føresegn',
'føresegner':'føresegn',
'føreset':'føres',
'føresetnad':'føresetnad',
'føresetnaden':'føresetnad',
'førespurnaden':'førespurnad',
'føretak':'føretak',
'føretaka':'føretak',
'føretaket':'føretak',
'føretaksform':'føretaksform',
'føretaksformer':'føretaksform',
'føretaksregisteret':'føretaksregister',
'føretaksregistret':'føretaksregistr',
'førevegen':'føreveg',
'føring':'føring',
'føringa':'føring',
'føringer':'føring',
'førre':'førr',
'først':'først',
'første':'først',
'førsteamanuensis':'førsteamanuensis',
'førstedestinasjon':'førstedestinasjon',
'førstegongs':'førstegong',
'førstekonsulent':'førstekonsulent',
'førstemottakere':'førstemottaker',
'førstkomande':'førstkom',
'førstnevnte':'førstnevnt',
'førstnevntes':'førstnevnt',
'ført':'ført',
'førte':'ført',
'førti':'førti',
'førtidspensjon':'førtidspensjon',
'førtidspensjoner':'førtidspensjon',
'førtidspensjoneringen':'førtidspensjonering',
'førtidspensjonert':'førtidspensjoner',
'førtidspensjonerte':'førtidspensjoner',
'førtidspensjonister':'førtidspensjonist',
'førutset':'føruts',
'føttene':'føtt',
'føtter':'føtt',
'føyd':'føyd',
'føye':'føye',
'føyelig':'føyel',
'føyer':'føyer',
'g':'g',
'ga':'ga',
'gagn':'gagn',
'gagnråd':'gagnråd',
'gal':'gal',
'galar':'gal',
'galder':'gald',
'galdrer':'galdr',
'galdt':'gald',
'gale':'gal',
'galer':'gal',
'galge':'galg',
'galopperende':'galopper',
'galskap':'galskap',
'galt':'galt',
'galte':'galt',
'galten':'galt',
'gambara':'gambar',
'gamle':'gaml',
'gamlingene':'gamling',
'gammal':'gammal',
'gammel':'gammel',
'gammelt':'gammelt',
'gang':'gang',
'gangen':'gang',
'gangene':'gang',
'ganger':'gang',
'ganske':'gansk',
'gantry':'gantry',
'gantrykran':'gantrykran',
'gap':'gap',
'gape':'gap',
'gaper':'gap',
'gapet':'gap',
'gaphals':'gaphal',
'gapskratter':'gapskratt',
'gapte':'gapt',
'garaas':'gara',
'garantere':'garanter',
'garanterer':'garanter',
'garanteres':'garanter',
'garantert':'garanter',
'garanterte':'garanter',
'garanti':'garanti',
'garantiane':'garanti',
'garantiar':'garanti',
'garantien':'garanti',
'garantier':'garanti',
'garantikapital':'garantikapital',
'garantikostnaden':'garantikostnad',
'garantirisikoen':'garantirisiko',
'garantistillelse':'garantistill',
'garasjar':'garasj',
'garasje':'garasj',
'gardering':'gardering',
'gardermoen':'gardermo',
'gards':'gard',
'gardsbruk':'gardsbruk',
'garm':'garm',
'garn':'garn',
'garnet':'garn',
'garnmaskene':'garnmask',
'gass':'gass',
'gater':'gat',
'gats':'gat',
'gatt':'gatt',
'gauler':'gaul',
'gaupehetter':'gaupehett',
'gav':'gav',
'gave':'gav',
'gaven':'gav',
'gavene':'gav',
'gaver':'gav',
'gavmild':'gavmild',
'gdw':'gdw',
'gebyr':'gebyr',
'gebyra':'gebyr',
'gebyrer':'gebyr',
'gebyrordning':'gebyrordning',
'gebyrsystemet':'gebyrsystem',
'geipe':'geip',
'geirrød':'geirrød',
'geirrøds':'geirrød',
'geit':'geit',
'geita':'geit',
'geitebukk':'geitebukk',
'geitebukken':'geitebukk',
'geitebukker':'geitebukk',
'geitene':'geit',
'geitepiss':'geitepiss',
'geiteryggen':'geiterygg',
'geiteskinnene':'geiteskinn',
'gemeinschaftlichen':'gemeinschaftlich',
'general':'general',
'generalforsamling':'generalforsamling',
'generalforsamlinga':'generalforsamling',
'generalforsamlingaallmenne':'generalforsamlingaallmenn',
'generalforsamlingane':'generalforsamling',
'generalforsamlingar':'generalforsamling',
'generalforsamlingsvedtak':'generalforsamlingsvedtak',
'generaliserte':'generaliser',
'generalklausulen':'generalklausul',
'generasjon':'generasjon',
'generasjonane':'generasjon',
'generasjonar':'generasjon',
'generasjoner':'generasjon',
'generell':'generell',
'generelle':'generell',
'generelt':'generelt',
'genererasjoner':'genererasjon',
'generert':'generer',
'genererte':'generer',
'genom':'genom',
'genossenschaft':'genossenschaft',
'gent':'gent',
'geografisk':'geografisk',
'geografiske':'geografisk',
'gerd':'gerd',
'gere':'ger',
'geschaftsbetriebes':'geschaftsbetrieb',
'geschlossener':'geschlossen',
'gesellschaften':'gesellschaft',
'gesetz':'gesetz',
'gevinst':'gevinst',
'gevinstbeskatning':'gevinstbeskatning',
'gevinsten':'gevinst',
'gevinster':'gevinst',
'geysirer':'geysir',
'ghv':'ghv',
'gi':'gi',
'gidder':'gidd',
'gift':'gift',
'gifte':'gift',
'giften':'gift',
'gifter':'gift',
'giftet':'gift',
'giftig':'gift',
'giftige':'gift',
'gikk':'gikk',
'gilde':'gild',
'gildet':'gild',
'gilling':'gilling',
'gillings':'gilling',
'gillion':'gillion',
'ginnunagagap':'ginnunagagap',
'ginnungagap':'ginnungagap',
'gir':'gir',
'gis':'gis',
'gisler':'gisl',
'gissel':'gissel',
'gitt':'gitt',
'gitte':'gitt',
'gjallarbru':'gjallarbru',
'gjallarbrua':'gjallarbru',
'gjallarhorn':'gjallarhorn',
'gjallarhornet':'gjallarhorn',
'gjalp':'gjalp',
'gje':'gje',
'gjeddes':'gjedd',
'gjekk':'gjekk',
'gjeld':'gjeld',
'gjelda':'gjeld',
'gjeldande':'gjeld',
'gjelde':'gjeld',
'gjeldende':'gjeld',
'gjelder':'gjeld',
'gjeldsandel':'gjeldsandel',
'gjeldsbrevlova':'gjeldsbrev',
'gjeldsforhandling':'gjeldsforhandling',
'gjeldshøve':'gjeldshøv',
'gjeldsinstrumenter':'gjeldsinstrument',
'gjeldsnemnda':'gjeldsnemnd',
'gjeldssanering':'gjeldssanering',
'gjeldt':'gjeld',
'gjeller':'gjell',
'gjem':'gjem',
'gjemme':'gjemm',
'gjemmer':'gjemm',
'gjemt':'gjemt',
'gjenforsikring':'gjenforsikring',
'gjenfortelles':'gjenfortell',
'gjengitt':'gjengitt',
'gjengjeld':'gjengjeld',
'gjengs':'gjeng',
'gjenkjøpsverdi':'gjenkjøpsverdi',
'gjenlevende':'gjenlev',
'gjenlyd':'gjenlyd',
'gjennom':'gjennom',
'gjennomboret':'gjennombor',
'gjennomførast':'gjennomfør',
'gjennomføre':'gjennomfør',
'gjennomfører':'gjennomfør',
'gjennomføres':'gjennomfør',
'gjennomføring':'gjennomføring',
'gjennomføringa':'gjennomføring',
'gjennomføringen':'gjennomføring',
'gjennomføringsloven':'gjennomføring',
'gjennomføringsorganet':'gjennomføringsorgan',
'gjennomført':'gjennomført',
'gjennomførte':'gjennomført',
'gjennomgang':'gjennomgang',
'gjennomgangen':'gjennomgang',
'gjennomgangsbustader':'gjennomgangsbustad',
'gjennomgangstrafikk':'gjennomgangstrafikk',
'gjennomgangsåre':'gjennomgangsår',
'gjennomgå':'gjennomgå',
'gjennomgåande':'gjennomgå',
'gjennomgående':'gjennomgå',
'gjennomgåinga':'gjennomgåing',
'gjennomgås':'gjennomgås',
'gjennomgått':'gjennomgått',
'gjennomsiktige':'gjennomsikt',
'gjennomslag':'gjennomslag',
'gjennomsnitlig':'gjennomsnit',
'gjennomsnitt':'gjennomsnitt',
'gjennomsnittet':'gjennomsnitt',
'gjennomsnittlig':'gjennomsnitt',
'gjennomsnittlige':'gjennomsnitt',
'gjennomsnittsavkastningen':'gjennomsnittsavkastning',
'gjennomsnittsberegninger':'gjennomsnittsberegning',
'gjennomsnittsbetraktninger':'gjennomsnittsbetraktning',
'gjennomsnittsinnskudd':'gjennomsnittsinnskudd',
'gjennomsnittsinnskuddene':'gjennomsnittsinnskudd',
'gjennomsnittsinnskuddet':'gjennomsnittsinnskudd',
'gjennomsnittslønn':'gjennomsnittslønn',
'gjennomsnittssatsen':'gjennomsnittssats',
'gjennomsnittsskattesatsen':'gjennomsnittsskattesats',
'gjennomsnittstall':'gjennomsnittstall',
'gjennomsnittsverdi':'gjennomsnittsverdi',
'gjennomsnittsverdien':'gjennomsnittsverdi',
'gjennomsyn':'gjennomsyn',
'gjennomtrekken':'gjennomtrekk',
'gjennoppstår':'gjennoppstår',
'gjenom':'gjenom',
'gjenoppta':'gjenoppt',
'gjenopptagelse':'gjenopptag',
'gjensidig':'gjensid',
'gjensidige':'gjensid',
'gjenspeiles':'gjenspeil',
'gjenstand':'gjenstand',
'gjenstående':'gjenstå',
'gjenstår':'gjenstår',
'gjensyn':'gjensyn',
'gjentar':'gjent',
'gjentatt':'gjentatt',
'gjenværende':'gjenvær',
'gjer':'gjer',
'gjerast':'gjer',
'gjerdene':'gjerd',
'gjere':'gjer',
'gjerne':'gjern',
'gjerning':'gjerning',
'gjerrige':'gjerr',
'gjerrigknark':'gjerrigknark',
'gjersem':'gjersem',
'gjerseme':'gjersem',
'gjest':'gjest',
'gjestearbeidende':'gjestearbeid',
'gjestebud':'gjestebud',
'gjestehus':'gjestehus',
'gjesten':'gjest',
'gjestene':'gjest',
'gjester':'gjest',
'gjesterom':'gjesterom',
'gjestfrihet':'gjestfri',
'gjestfrihetens':'gjestfri',
'gjette':'gjett',
'gjev':'gjev',
'gjevast':'gjev',
'gjeve':'gjev',
'gjeven':'gjev',
'gjevjon':'gjevjon',
'gjevne':'gjevn',
'gjoll':'gjoll',
'gjord':'gjord',
'gjorde':'gjord',
'gjort':'gjort',
'gjærer':'gjær',
'gjødning':'gjødning',
'gjødsel':'gjødsel',
'gjør':'gjør',
'gjøre':'gjør',
'gjøres':'gjør',
'glad':'glad',
'glade':'glad',
'gladsheim':'gladsheim',
'glasruter':'glasrut',
'glassperler':'glassperl',
'glassverk':'glassverk',
'glatt':'glatt',
'glatte':'glatt',
'glede':'gled',
'gleden':'gled',
'gleder':'gled',
'glefser':'glefs',
'gleipne':'gleipn',
'glem':'glem',
'glemmer':'glemm',
'glemsel':'glemsel',
'glemselsfuglen':'glemselsfugl',
'glemt':'glemt',
'glemte':'glemt',
'glipp':'glipp',
'glir':'glir',
'glitrende':'glitr',
'glitrer':'glitr',
'glitter':'glitt',
'globale':'global',
'globalisering':'globalisering',
'globalt':'globalt',
'globe':'glob',
'globes':'glob',
'glohete':'glohet',
'glomma':'glomm',
'glommafarvannet':'glommafarvann',
'glugge':'glugg',
'gluggen':'glugg',
'glødende':'glød',
'gløder':'glød',
'glør':'glør',
'gløttet':'gløtt',
'gnage':'gnag',
'gnager':'gnag',
'gni':'gni',
'gnir':'gnir',
'gnist':'gnist',
'gnister':'gnist',
'gnistrer':'gnistr',
'gnitahei':'gnitahei',
'gnitaheia':'gnitahei',
'gnå':'gnå',
'god':'god',
'godbit':'godbit',
'gode':'god',
'godet':'god',
'godkjend':'godkjend',
'godkjende':'godkj',
'godkjennast':'godkjenn',
'godkjenne':'godkjenn',
'godkjenner':'godkjenn',
'godkjennes':'godkjenn',
'godkjenning':'godkjenning',
'godkjenninga':'godkjenning',
'godkjenningsinstans':'godkjenningsinstan',
'godkjenningsinstansen':'godkjenningsinstans',
'godkjenningskompetanse':'godkjenningskompetans',
'godkjenningsnekting':'godkjenningsnekting',
'godkjenningsnektinga':'godkjenningsnekting',
'godkjenningsordning':'godkjenningsordning',
'godkjenningsordninga':'godkjenningsordning',
'godkjenningsregelen':'godkjenningsregel',
'godkjenningsspørsmålet':'godkjenningsspørsmål',
'godkjent':'godkjent',
'godkjente':'godkjent',
'gods':'god',
'godsaktivitetene':'godsaktivitet',
'godsaktiviteter':'godsaktivitet',
'godsbanegården':'godsbanegård',
'godsbiler':'godsbil',
'godset':'gods',
'godsforsendelse':'godsforsend',
'godsgrunnlag':'godsgrunnlag',
'godsgrunnlaget':'godsgrunnlag',
'godshavna':'godshavn',
'godshavnaktivitet':'godshavnaktivit',
'godshavnen':'godshavn',
'godshavner':'godshavn',
'godshåndtering':'godshåndtering',
'godshåndteringen':'godshåndtering',
'godshåndteringsutstyr':'godshåndteringsutstyr',
'godskapasitet':'godskapasit',
'godskrevet':'godskrev',
'godskrivast':'godskriv',
'godskriver':'godskriv',
'godskrives':'godskriv',
'godsmarkedet':'godsmarked',
'godsmengde':'godsmengd',
'godsmengden':'godsmengd',
'godsmengdene':'godsmengd',
'godsmengdenes':'godsmengd',
'godsmengder':'godsmengd',
'godsomlasting':'godsomlasting',
'godsomlastingen':'godsomlasting',
'godsomlastingstallene':'godsomlastingstall',
'godsomslag':'godsomslag',
'godsomslaget':'godsomslag',
'godsprognosene':'godsprognos',
'godsprognoser':'godsprognos',
'godsspor':'godsspor',
'godssporene':'godsspor',
'godsstrømmen':'godsstrømm',
'godsstrømmene':'godsstrømm',
'godsstrømmer':'godsstrømm',
'godsterminal':'godsterminal',
'godsterminalen':'godsterminal',
'godstog':'godstog',
'godstomlasting':'godstomlasting',
'godstrafikk':'godstrafikk',
'godstrafikken':'godstrafikk',
'godstransport':'godstransport',
'godstransporten':'godstransport',
'godstransportene':'godstransport',
'godstransporter':'godstransport',
'godstransportknutepunkt':'godstransportknutepunkt',
'godstransportkorridorer':'godstransportkorridor',
'godstransportmønstrene':'godstransportmønstr',
'godstransportstrømmene':'godstransportstrømm',
'godstransporttjenester':'godstransporttjenest',
'godstrømmene':'godstrømm',
'godstrømmer':'godstrømm',
'godstyper':'godstyp',
'godsvirksomhet':'godsvirksom',
'godsvolum':'godsvolum',
'godsvolumene':'godsvolum',
'godsvolumer':'godsvolum',
'godt':'godt',
'godta':'godt',
'godtakast':'godtak',
'godtatt':'godtatt',
'godtek':'godtek',
'godteke':'godtek',
'godtgjer':'godtgj',
'godtgjere':'godtgjer',
'godtgjersla':'godtgjersl',
'godtgjersle':'godtgjersl',
'godtgjort':'godtgjort',
'godtgjørelse':'godtgjør',
'godtgjørelsene':'godtgjør',
'godtgjøres':'godtgjør',
'godtgjøring':'godtgjøring',
'godtgjøringer':'godtgjøring',
'godtok':'godtok',
'godtruerverv':'godtruerverv',
'golv':'golv',
'golvbelegg':'golvbelegg',
'golvet':'golv',
'golvteppe':'golvtepp',
'gom':'gom',
'gomlet':'goml',
'gommen':'gomm',
'gommene':'gomm',
'gong':'gong',
'gongen':'gong',
'gonger':'gong',
'gotter':'gott',
'gotterier':'gotteri',
'gottet':'gott',
'grad':'grad',
'graden':'grad',
'gradert':'grader',
'gradvis':'gradvis',
'gradvise':'gradvis',
'granbar':'granb',
'grane':'gran',
'granlegg':'granlegg',
'grann':'grann',
'grannar':'grann',
'granneeigedom':'granneeigedom',
'granskar':'gransk',
'granskarane':'granskar',
'granskarar':'granskar',
'gransking':'gransking',
'granskinga':'gransking',
'granskingsrapporten':'granskingsrapport',
'grantoppene':'grantopp',
'grantrær':'grantrær',
'gratis':'gratis',
'grav':'grav',
'gravalvorlige':'gravalvor',
'grave':'grav',
'graven':'grav',
'graver':'grav',
'gravet':'grav',
'gravid':'gravid',
'gravøl':'gravøl',
'gre':'gre',
'green':'green',
'grei':'grei',
'greiast':'greiast',
'greiaste':'greiast',
'greid':'greid',
'greidd':'greidd',
'greidde':'greidd',
'greide':'greid',
'greie':'greie',
'greier':'greier',
'greip':'greip',
'greitt':'greitt',
'gren':'gren',
'grendehus':'grendehus',
'grenene':'gren',
'grener':'gren',
'grenland':'grenland',
'grenlandsfarvannet':'grenlandsfarvann',
'grenlandsområdet':'grenlandsområd',
'grensa':'grens',
'grense':'grens',
'grensedraging':'grensedraging',
'grensedraginga':'grensedraging',
'grensekryssende':'grensekryss',
'grenseland':'grenseland',
'grensen':'grens',
'grensene':'grens',
'grenseoverskridende':'grenseoverskrid',
'grensepassering':'grensepassering',
'grenser':'grens',
'grenseskogen':'grenseskog',
'grenseverdier':'grenseverdi',
'grepstad':'grepstad',
'grer':'grer',
'gress':'gress',
'gressbakke':'gressbakk',
'gresser':'gress',
'gresset':'gress',
'gressgangene':'gressgang',
'gresstorven':'gresstorv',
'gretten':'grett',
'grettent':'grettent',
'gribber':'gribb',
'grid':'grid',
'gridarvol':'gridarvol',
'grimase':'grimas',
'grimne':'grimn',
'grimsbo':'grimsbo',
'grind':'grind',
'grinden':'grind',
'griner':'grin',
'grini':'grini',
'grip':'grip',
'gripe':'grip',
'griper':'grip',
'gris':'gris',
'grisen':'gris',
'grisk':'grisk',
'griske':'grisk',
'griskheten':'grisk',
'grjotunagard':'grjotunagard',
'gro':'gro',
'groa':'groa',
'gror':'gror',
'groruddalen':'groruddal',
'groruddalens':'groruddal',
'grossist':'grossist',
'grossister':'grossist',
'grossisters':'grossist',
'grossistlagre':'grossistlagr',
'grossistledd':'grossistledd',
'grossistleddet':'grossistledd',
'grossistvirksomheter':'grossistvirksom',
'grotte':'grott',
'grotter':'grott',
'grov':'grov',
'grove':'grov',
'gru':'gru',
'grubler':'grubl',
'gruen':'gruen',
'gruer':'gruer',
'grundig':'grund',
'grundigare':'grundigar',
'grunn':'grunn',
'grunna':'grunn',
'grunnane':'grunn',
'grunnar':'grunn',
'grunnareal':'grunnareal',
'grunnareala':'grunnareal',
'grunnast':'grunn',
'grunnbeløp':'grunnbeløp',
'grunnbeløpet':'grunnbeløp',
'grunnbok':'grunnbok',
'grunnboka':'grunnbok',
'grunnbokheimel':'grunnbokheimel',
'grunndokument':'grunndokument',
'grunndraga':'grunndrag',
'grunne':'grunn',
'grunneier':'grunnei',
'grunnen':'grunn',
'grunnene':'grunn',
'grunner':'grunn',
'grunnet':'grunn',
'grunnfond':'grunnfond',
'grunnfondbevis':'grunnfondbevis',
'grunnfondet':'grunnfond',
'grunnfondsbevis':'grunnfondsbevis',
'grunnfondsbevisa':'grunnfondsbevis',
'grunnforholdene':'grunnforhold',
'grunngje':'grunngj',
'grunngjevast':'grunngjev',
'grunngjeve':'grunngjev',
'grunngjeving':'grunngjeving',
'grunngjevinga':'grunngjeving',
'grunngjevne':'grunngjevn',
'grunnivået':'grunnivå',
'grunnkravene':'grunnkrav',
'grunnlag':'grunnlag',
'grunnlaget':'grunnlag',
'grunnlagselementene':'grunnlagselement',
'grunnlagselementer':'grunnlagselement',
'grunnlagsmaterialet':'grunnlagsmaterial',
'grunnlagsrente':'grunnlagsrent',
'grunnlagsrenten':'grunnlagsrent',
'grunnlause':'grunnlaus',
'grunnleggende':'grunnlegg',
'grunnleggjande':'grunnleggj',
'grunnleggjarane':'grunnleggjar',
'grunnlinjen':'grunnlinj',
'grunnlova':'grunn',
'grunnlønn':'grunnlønn',
'grunnmateriale':'grunnmaterial',
'grunnorganisasjonane':'grunnorganisasjon',
'grunnpensjon':'grunnpensjon',
'grunnpensjonen':'grunnpensjon',
'grunnprinsipp':'grunnprinsipp',
'grunnsikring':'grunnsikring',
'grunnsikringens':'grunnsikring',
'grunnstykke':'grunnstykk',
'grunnstøting':'grunnstøting',
'grunnstøtinger':'grunnstøting',
'grunnstøtninger':'grunnstøtning',
'grunnsummen':'grunnsumm',
'grunnvilkår':'grunnvilkår',
'gruntvannsområder':'gruntvannsområd',
'gruppa':'grupp',
'gruppe':'grupp',
'gruppeinndeling':'gruppeinndeling',
'gruppeliv':'gruppeliv',
'gruppelivsforsikring':'gruppelivsforsikring',
'gruppen':'grupp',
'gruppene':'grupp',
'gruppens':'grupp',
'grupper':'grupp',
'grupperinger':'gruppering',
'gruppert':'grupper',
'grus':'grus',
'grusomme':'grusomm',
'grusomt':'grusomt',
'grynt':'grynt',
'grynter':'grynt',
'grytidlig':'grytid',
'grå':'grå',
'gråbein':'gråbein',
'gråbeinskikkelse':'gråbeinskikk',
'grådig':'gråd',
'grådige':'gråd',
'grådighet':'gråd',
'grålysningen':'grålysning',
'gråspurv':'gråspurv',
'gråt':'gråt',
'gråte':'gråt',
'gråtekone':'gråtekon',
'gråtende':'gråt',
'gråter':'gråt',
'gråtes':'gråt',
'grått':'grått',
'grøft':'grøft',
'grønlia':'grønli',
'grønn':'grønn',
'grønnboken':'grønnbok',
'grønne':'grønn',
'grønnsaker':'grønnsak',
'grønt':'grønt',
'grøntområde':'grøntområd',
'grøsser':'grøss',
'grøt':'grøt',
'gud':'gud',
'gudeborgen':'gudeborg',
'gudehøvdingen':'gudehøvding',
'gudekongen':'gudekong',
'gudekongens':'gudekong',
'gudekreftene':'gudekreft',
'guden':'gud',
'gudene':'gud',
'gudenes':'gud',
'gudepakket':'gudepakk',
'guder':'gud',
'gudeslekten':'gudeslekt',
'gudeslekter':'gudeslekt',
'gudespytt':'gudespytt',
'gudinne':'gudinn',
'gudinnen':'gudinn',
'gudinnene':'gudinn',
'gudinner':'gudinn',
'gufse':'gufs',
'gufser':'gufs',
'gulating':'gulating',
'gule':'gul',
'gull':'gull',
'gullbrikkespill':'gullbrikkespill',
'gullbrua':'gullbru',
'gullbust':'gullbust',
'gullbånd':'gullbånd',
'gullet':'gull',
'gullfakse':'gullfaks',
'gullhjelmen':'gullhjelm',
'gullhorn':'gullhorn',
'gullhåret':'gullhår',
'gullklump':'gullklump',
'gullregn':'gullregn',
'gullringen':'gullring',
'gulltennene':'gulltenn',
'gulltopp':'gulltopp',
'gullveig':'gullv',
'gulne':'guln',
'gulv':'gulv',
'gulvet':'gulv',
'gungne':'gungn',
'gunnlod':'gunnlod',
'gunst':'gunst',
'gunstig':'gunst',
'gunstige':'gunst',
'gunstigere':'gunstiger',
'gusten':'gust',
'gustne':'gustn',
'gutt':'gutt',
'guttebarnet':'guttebarn',
'gutten':'gutt',
'guttene':'gutt',
'gutter':'gutt',
'gyger':'gyg',
'gygra':'gygr',
'gyldig':'gyld',
'gyldige':'gyld',
'gyldne':'gyldn',
'gyllenbuste':'gyllenbust',
'gylve':'gylv',
'gymes':'gym',
'gymesdatter':'gymesdatt',
'gyver':'gyv',
'gå':'gå',
'går':'går',
'gård':'gård',
'gården':'gård',
'gårder':'gård',
'gårdsfolket':'gårdsfolk',
'gårdshundene':'gårdshund',
'gåsøya':'gåsøy',
'gåtefulle':'gåtefull',
'gåtene':'gåt',
'gåter':'gåt',
'gått':'gått',
'gåve':'gåv',
'gåver':'gåv',
'gøteborg':'gøteborg',
'h':'h',
'ha':'ha',
'haavind':'haavind',
'habilitet':'habilit',
'habilitetsreglane':'habilitetsregl',
'habitat':'habitat',
'hadde':'hadd',
'hafslund':'hafslund',
'haglskur':'haglskur',
'haken':'hak',
'hakk':'hakk',
'hakon':'hakon',
'haldast':'hald',
'halde':'hald',
'halden':'hald',
'haldenleia':'haldenlei',
'haldne':'haldn',
'haldningar':'haldning',
'hale':'hal',
'halen':'hal',
'haler':'hal',
'hall':'hall',
'hallen':'hall',
'halm':'halm',
'hals':'hal',
'halse':'hals',
'halsen':'hals',
'halset':'hals',
'halshugget':'halshugg',
'halsringen':'halsring',
'halt':'halt',
'haltende':'halt',
'halv':'halv',
'halvdel':'halvdel',
'halvdelen':'halvdel',
'halve':'halv',
'halvfabrikata':'halvfabrikat',
'halvgjort':'halvgjort',
'halvparten':'halvpart',
'halvsøvne':'halvsøvn',
'halvt':'halv',
'halvtroll':'halvtroll',
'halvveis':'halvveis',
'halvår':'halvår',
'halvårsvis':'halvårsvis',
'ham':'ham',
'hamburg':'hamburg',
'hammen':'hamm',
'hammer':'hamm',
'hammeren':'hammer',
'hammerfest':'hammerfest',
'hammerskaftet':'hammerskaft',
'hammerslag':'hammerslag',
'hamrer':'hamr',
'hamsdokka':'hamsdokk',
'han':'han',
'hand':'hand',
'handel':'handel',
'handeland':'handeland',
'handelen':'handel',
'handels':'handel',
'handelsagenter':'handelsagent',
'handelsbedrifter':'handelsbedrift',
'handelsdepartementet':'handelsdepartement',
'handelshindringer':'handelshindring',
'handelsorganisasjon':'handelsorganisasjon',
'handelspartnere':'handelspartner',
'handheva':'handhev',
'handheve':'handhev',
'handhevinga':'handheving',
'handkasser':'handkass',
'handla':'handl',
'handlar':'handl',
'handle':'handl',
'handleevne':'handleevn',
'handlefridom':'handlefridom',
'handlefrihet':'handlefri',
'handleplikt':'handleplikt',
'handles':'handl',
'handlet':'handl',
'handling':'handling',
'handlinga':'handling',
'handlinger':'handling',
'handlingsfrist':'handlingsfrist',
'handlingsprogram':'handlingsprogram',
'handlingsprogrammer':'handlingsprogramm',
'handlingsrom':'handlingsrom',
'handlingsrommet':'handlingsromm',
'handpantsetjing':'handpantsetjing',
'handtering':'handtering',
'handterleg':'handter',
'handverkar':'handverk',
'handverkarar':'handverkar',
'handverkartenestelova':'handverkartenest',
'handverkartenester':'handverkartenest',
'hane':'han',
'hanegal':'hanegal',
'hanen':'han',
'haner':'han',
'hang':'hang',
'hanken':'hank',
'hans':'han',
'hansker':'hansk',
'hanstholm':'hanstholm',
'har':'har',
'harald':'harald',
'hard':'hard',
'hardare':'hardar',
'harde':'hard',
'hardere':'harder',
'hardfør':'hardfør',
'hardt':'hard',
'harer':'har',
'harker':'hark',
'harlem':'harlem',
'harme':'harm',
'harmonisere':'harmoniser',
'harmonisering':'harmonisering',
'harsk':'harsk',
'harstad':'harstad',
'hasteordrer':'hasteordr',
'haster':'hast',
'hastighet':'hast',
'hastverk':'hastverk',
'hat':'hat',
'hate':'hat',
'hater':'hat',
'hatet':'hat',
'hatt':'hatt',
'hattebrem':'hattebrem',
'hattebremmen':'hattebremm',
'hatten':'hatt',
'haug':'haug',
'haugen':'haug',
'hauger':'haug',
'haugesund':'haugesund',
'hauk':'hauk',
'hauker':'hauk',
'hausten':'haust',
'hav':'hav',
'havarer':'havar',
'havarere':'havarer',
'havet':'hav',
'havfolk':'havfolk',
'havfruer':'havfru',
'havgapet':'havgap',
'havheksen':'havheks',
'havkongen':'havkong',
'havkongens':'havkong',
'havmenn':'havmenn',
'havmennene':'havmenn',
'havn':'havn',
'havna':'havn',
'havnas':'havn',
'havne':'havn',
'havneaktivitetene':'havneaktivitet',
'havneaktiviteter':'havneaktivitet',
'havneanlegg':'havneanlegg',
'havneanløp':'havneanløp',
'havneareal':'havneareal',
'havnearealene':'havneareal',
'havnearealer':'havneareal',
'havnearealet':'havneareal',
'havneavgiftene':'havneavgift',
'havneavgifter':'havneavgift',
'havneavgiftssystemet':'havneavgiftssystem',
'havneavsnitt':'havneavsnitt',
'havneavsnittene':'havneavsnitt',
'havneavsnittet':'havneavsnitt',
'havnebassenget':'havnebasseng',
'havnebehov':'havnebehov',
'havnebehovet':'havnebehov',
'havnebruker':'havnebruk',
'havneby':'havneby',
'havnebyen':'havneby',
'havnebyene':'havneby',
'havnebyer':'havneby',
'havnebyscenariet':'havnebyscenari',
'havnedistrikt':'havnedistrikt',
'havnedistriktene':'havnedistrikt',
'havnedistrikter':'havnedistrikt',
'havnedistriktet':'havnedistrikt',
'havnedistriktets':'havnedistrikt',
'havnedrift':'havnedrift',
'havnedriften':'havnedrift',
'havneeffektivitet':'havneeffektivit',
'havneeier':'havneei',
'havneeiere':'havneeier',
'havneenheter':'havneen',
'havneforbund':'havneforbund',
'havneforbundets':'havneforbund',
'havneformål':'havneformål',
'havneforvaltningen':'havneforvaltning',
'havnefunksjonene':'havnefunksjon',
'havnefunksjoner':'havnefunksjon',
'havnefylkene':'havnefylk',
'havnefylker':'havnefylk',
'havnehagen':'havnehag',
'havneinfrastrukturen':'havneinfrastruktur',
'havneinnretningene':'havneinnretning',
'havneinnretninger':'havneinnretning',
'havneinteresser':'havneinteress',
'havnekapasitet':'havnekapasit',
'havnekassa':'havnekass',
'havnekasse':'havnekass',
'havnekassemidler':'havnekassemidl',
'havnekassen':'havnekass',
'havnekassene':'havnekass',
'havnekassens':'havnekass',
'havnelokalisering':'havnelokalisering',
'havneloven':'havn',
'havnelovens':'havn',
'havneløsning':'havneløsning',
'havneløsningene':'havneløsning',
'havneløsninger':'havneløsning',
'havnemessig':'havnemess',
'havnemyndighetene':'havnemynd',
'havnemyndigheter':'havnemynd',
'havnemønster':'havnemønst',
'havnen':'havn',
'havnene':'havn',
'havnenes':'havn',
'havnens':'havn',
'havneområdene':'havneområd',
'havneområder':'havneområd',
'havneområdet':'havneområd',
'havneoperatørene':'havneoperatør',
'havneoperatørenes':'havneoperatør',
'havneoperatører':'havneoperatør',
'havneoppgaver':'havneoppgav',
'havneplan':'havneplan',
'havneplanlegging':'havneplanlegging',
'havnepolitikk':'havnepolitikk',
'havnepolitikken':'havnepolitikk',
'havner':'havn',
'havnerelatert':'havnerelater',
'havnerettet':'havnerett',
'havners':'havn',
'havneråd':'havneråd',
'havnerådet':'havneråd',
'havnerådets':'havneråd',
'havnesaker':'havnesak',
'havnesamarbeid':'havnesamarbeid',
'havnesamarbeide':'havnesamarbeid',
'havnesamarbeider':'havnesamarbeid',
'havnesamarbeidet':'havnesamarbeid',
'havnesammenheng':'havnesammenheng',
'havnescenariene':'havnescenari',
'havnesektoren':'havnesektor',
'havneselskaper':'havneselskap',
'havnesiden':'havnesid',
'havnespørsmål':'havnespørsmål',
'havnestatistikk':'havnestatistikk',
'havnestruktur':'havnestruktur',
'havnestrukturen':'havnestruktur',
'havnestrukturer':'havnestruktur',
'havnestyre':'havnestyr',
'havnestyrer':'havnestyr',
'havnestyret':'havnestyr',
'havnestyrets':'havnestyr',
'havnesystem':'havnesystem',
'havneterminal':'havneterminal',
'havneterminalen':'havneterminal',
'havneterminalene':'havneterminal',
'havnetilbud':'havnetilbud',
'havnetilbudet':'havnetilbud',
'havnetiltakene':'havnetiltak',
'havnetjenesten':'havnetjenest',
'havnetjenestene':'havnetjenest',
'havnetjenester':'havnetjenest',
'havnetrafikk':'havnetrafikk',
'havnetrafikken':'havnetrafikk',
'havnetyper':'havnetyp',
'havneutbygging':'havneutbygging',
'havneutbyggingen':'havneutbygging',
'havneutflytting':'havneutflytting',
'havneutvalget':'havneutvalg',
'havneutvalgets':'havneutvalg',
'havneutvikling':'havneutvikling',
'havneutviklingen':'havneutvikling',
'havnevalg':'havnevalg',
'havnevesen':'havneves',
'havnevesenet':'havnevesen',
'havnevesenets':'havnevesen',
'havnevesens':'havneves',
'havnevirksomhet':'havnevirksom',
'havnevirksomheten':'havnevirksom',
'havnevirksomhetene':'havnevirksom',
'havneøkonomi':'havneøkonomi',
'havns':'havn',
'havre':'havr',
'havs':'hav',
'heder':'hed',
'hedmark':'hedmark',
'hefta':'heft',
'heftar':'heft',
'heftast':'heft',
'hefte':'heft',
'hefteform':'hefteform',
'hefter':'heft',
'heftet':'heft',
'hei':'hei',
'heia':'heia',
'heider':'heid',
'heidrun':'heidrun',
'heiene':'heien',
'heildagsmøte':'heildagsmøt',
'heile':'heil',
'heilskapleg':'heilskap',
'heilt':'heilt',
'heilårsbustader':'heilårsbustad',
'heim':'heim',
'heimdall':'heimdall',
'heimdalls':'heimdall',
'heime':'heim',
'heimekontor':'heimekontor',
'heimel':'heimel',
'heimelen':'heimel',
'heimelsdokument':'heimelsdokument',
'heimelsdokumenta':'heimelsdokument',
'heimelshavar':'heimelshav',
'heimelsoverføring':'heimelsoverføring',
'heimelsovergang':'heimelsovergang',
'heimen':'heim',
'heimesider':'heimesid',
'heimlar':'heiml',
'heise':'heis',
'heisen':'heis',
'heiser':'heis',
'heises':'heis',
'heiter':'heit',
'heks':'heks',
'heksen':'heks',
'hekser':'heks',
'hel':'hel',
'helagt':'helagt',
'helblinde':'helblind',
'held':'held',
'heldagsmøter':'heldagsmøt',
'heldig':'held',
'heldige':'held',
'heldigvis':'heldigvis',
'hele':'hel',
'helgedag':'helgedag',
'helgedagar':'helgedag',
'helhet':'hel',
'helhetlig':'helhet',
'helhetlige':'helhet',
'helhetsvurdering':'helhetsvurdering',
'helhunden':'helhund',
'hell':'hell',
'heller':'hell',
'hellet':'hell',
'hellig':'hel',
'hellige':'hel',
'hels':'hel',
'helse':'hels',
'helsefarlege':'helsefar',
'helsesituasjon':'helsesituasjon',
'helst':'helst',
'helt':'helt',
'heltedåder':'heltedåd',
'heltegjerninger':'heltegjerning',
'heltene':'helt',
'helter':'helt',
'heltidsansatte':'heltidsansatt',
'heltidsstilling':'heltidsstilling',
'helveien':'helvei',
'hemmelege':'hemm',
'hemmeleghald':'hemmeleghald',
'hemmelig':'hemm',
'hemmelige':'hemm',
'hemmelighet':'hemm',
'hemmeligheten':'hemm',
'hemmeligheter':'hemm',
'hemmelighetsfulle':'hemmelighetsfull',
'hen':'hen',
'hende':'hend',
'hendene':'hend',
'hender':'hend',
'hending':'hending',
'hendt':'hend',
'hendte':'hend',
'henføres':'henfør',
'henført':'henført',
'heng':'heng',
'henge':'heng',
'hengekjeft':'hengekjeft',
'hengende':'heng',
'henger':'heng',
'henges':'heng',
'henhold':'henhold',
'henholdsvis':'henholdsvis',
'henne':'henn',
'hennes':'henn',
'henning':'henning',
'henseende':'hense',
'hensett':'hensett',
'hensettingsspor':'hensettingsspor',
'hensikt':'hensikt',
'hensikten':'hensikt',
'hensiktsmessig':'hensiktsmess',
'hensiktsmessige':'hensiktsmess',
'hensiktsmessighetsnorm':'hensiktsmessighetsnorm',
'hensiktsmessighetssider':'hensiktsmessighetssid',
'henstille':'henstill',
'henstilling':'henstilling',
'hensyn':'hensyn',
'hensynene':'hensyn',
'hensynet':'hensyn',
'hensynsløst':'hensynsløst',
'hensynta':'hensynt',
'hensyntas':'hensynt',
'henta':'hent',
'hentar':'hent',
'hentast':'hent',
'hente':'hent',
'henter':'hent',
'hentes':'hent',
'hentet':'hent',
'henting':'henting',
'henvendelser':'henvend',
'henviser':'henvis',
'henvises':'henvis',
'henvisning':'henvisning',
'henvisningen':'henvisning',
'henvist':'henvist',
'her':'her',
'heradsrett':'heradsrett',
'herav':'herav',
'heretter':'herett',
'herfra':'herfr',
'hergarten':'hergart',
'herjan':'herjan',
'herje':'herj',
'herjer':'herj',
'herjet':'herj',
'herligste':'herligst',
'hermed':'hermed',
'hermod':'hermod',
'herre':'herr',
'herrer':'herr',
'herres':'herr',
'hersker':'hersk',
'herunder':'herund',
'herøya':'herøy',
'hese':'hes',
'heslige':'hes',
'hest':'hest',
'hestehover':'hestehov',
'hestekar':'hestek',
'hesten':'hest',
'hestene':'hest',
'hester':'hest',
'hesteryggen':'hesterygg',
'het':'het',
'hete':'het',
'heter':'het',
'heva':'hev',
'hevast':'hev',
'hevde':'hevd',
'hevder':'hevd',
'hevdes':'hevd',
'heve':'hev',
'heving':'heving',
'hevingsregelen':'hevingsregel',
'hevingsrett':'hevingsrett',
'hevn':'hevn',
'hevne':'hevn',
'hevnens':'hevn',
'hevner':'hevn',
'hevnes':'hevn',
'hevnet':'hevn',
'hht':'hht',
'hhv':'hhv',
'hikker':'hikk',
'hikster':'hikst',
'hilde':'hild',
'hils':'hil',
'hilse':'hils',
'hilser':'hils',
'hilset':'hils',
'himler':'himl',
'himling':'himling',
'himlingsplater':'himlingsplat',
'himmel':'himmel',
'himmelberget':'himmelberg',
'himmelen':'himmel',
'himmelhvelvingen':'himmelhvelving',
'himmelretninger':'himmelretning',
'himmelske':'himmelsk',
'himmelvogn':'himmelvogn',
'hindarfjell':'hindarfjell',
'hinder':'hind',
'hindra':'hindr',
'hindrar':'hindr',
'hindre':'hindr',
'hindrer':'hindr',
'hindring':'hindring',
'hindringa':'hindring',
'hindringer':'hindring',
'hingst':'hingst',
'hingsten':'hingst',
'hink':'hink',
'hinkende':'hink',
'hinket':'hink',
'hinne':'hinn',
'hirtshals':'hirtshal',
'hisse':'hiss',
'hissig':'hiss',
'historien':'histori',
'historier':'histori',
'historikk':'historikk',
'historisk':'historisk',
'historiske':'historisk',
'hit':'hit',
'hittil':'hittil',
'hiver':'hiv',
'hjalp':'hjalp',
'hjel':'hjel',
'hjelm':'hjelm',
'hjelp':'hjelp',
'hjelpe':'hjelp',
'hjelpeløs':'hjelpeløs',
'hjelpeløse':'hjelpeløs',
'hjelpemidler':'hjelpemidl',
'hjelper':'hjelp',
'hjelpesmann':'hjelpesmann',
'hjem':'hjem',
'hjemkomsten':'hjemkomst',
'hjemland':'hjemland',
'hjemlandet':'hjemland',
'hjemlands':'hjemland',
'hjemlandskontroll':'hjemlandskontroll',
'hjemler':'hjeml',
'hjemlet':'hjeml',
'hjemløs':'hjemløs',
'hjemme':'hjemm',
'hjemmehørende':'hjemmehør',
'hjemmel':'hjemmel',
'hjemmelen':'hjemmel',
'hjemmelsbruken':'hjemmelsbruk',
'hjemmet':'hjemm',
'hjemover':'hjemov',
'hjemsøkes':'hjemsøk',
'hjemvei':'hjemvei',
'hjernen':'hjern',
'hjerte':'hjert',
'hjerteblod':'hjerteblod',
'hjertelig':'hjert',
'hjertens':'hjert',
'hjertet':'hjert',
'hjort':'hjort',
'hjorter':'hjort',
'hjortland':'hjortland',
'hjortnes':'hjortn',
'hjul':'hjul',
'hjulene':'hjul',
'hjulet':'hjul',
'hjå':'hjå',
'hjørne':'hjørn',
'hjørner':'hjørn',
'hjørnesteinsbedriften':'hjørnesteinsbedrift',
'hlin':'hlin',
'hm':'hm',
'ho':'ho',
'hobbyrom':'hobbyrom',
'hod':'hod',
'hoddmimes':'hoddmim',
'hode':'hod',
'hodeduk':'hodeduk',
'hodene':'hod',
'hodepine':'hodepin',
'hoder':'hod',
'hodeskalle':'hodeskall',
'hodeskallen':'hodeskall',
'hodet':'hod',
'hods':'hod',
'hoffet':'hoff',
'hoftene':'hoft',
'hofter':'hoft',
'hoho':'hoho',
'hoi':'hoi',
'hold':'hold',
'holde':'hold',
'holdent':'holdent',
'holdepunkter':'holdepunkt',
'holder':'hold',
'holdes':'hold',
'holdning':'holdning',
'holdninger':'holdning',
'holdt':'hold',
'holme':'holm',
'holmen':'holm',
'holmer':'holm',
'holmestrand':'holmestrand',
'holmgang':'holmgang',
'holt':'holt',
'homofil':'homofil',
'homogene':'homog',
'honning':'honning',
'honningdugg':'honningdugg',
'hop':'hop',
'hopar':'hop',
'hoppa':'hopp',
'hoppe':'hopp',
'hoppende':'hopp',
'hopper':'hopp',
'hoppet':'hopp',
'hordaland':'hordaland',
'horisont':'horisont',
'horisontalt':'horisontalt',
'horisonten':'horisont',
'horisontår':'horisontår',
'horn':'horn',
'hornene':'horn',
'hornet':'horn',
'horten':'hort',
'hos':'hos',
'hospits':'hospit',
'housing':'housing',
'hovedadresse':'hovedadress',
'hovedaktiviteten':'hovedaktivitet',
'hovedaktør':'hovedaktør',
'hovedaktørene':'hovedaktør',
'hovedalternativ':'hovedalternativ',
'hovedalternativer':'hovedalternativ',
'hovedalternativet':'hovedalternativ',
'hovedandelen':'hovedandel',
'hovedansvaret':'hovedansvar',
'hovedargumentet':'hovedargument',
'hovedavsnitt':'hovedavsnitt',
'hoveddelen':'hoveddel',
'hoveddelene':'hoveddel',
'hoveddeler':'hoveddel',
'hoveddepotet':'hoveddepot',
'hovedelementer':'hovedelement',
'hovedforbindelse':'hovedforbind',
'hovedforbindelsen':'hovedforbind',
'hovedformål':'hovedformål',
'hovedforutsetning':'hovedforutsetning',
'hovedgodsstrømmene':'hovedgodsstrømm',
'hovedgodsterminal':'hovedgodsterminal',
'hovedgruppene':'hovedgrupp',
'hovedgrupper':'hovedgrupp',
'hovedhavn':'hovedhavn',
'hovedhavnen':'hovedhavn',
'hovedhavneområder':'hovedhavneområd',
'hovedhavner':'hovedhavn',
'hovedhensyn':'hovedhensyn',
'hovedhensynene':'hovedhensyn',
'hovedinndeling':'hovedinndeling',
'hovedinndelinger':'hovedinndeling',
'hovedinnfartsvegene':'hovedinnfartsveg',
'hovedinntekts':'hovedinntekt',
'hovedkonklusjonene':'hovedkonklusjon',
'hovedkonklusjoner':'hovedkonklusjon',
'hovedkontoradresse':'hovedkontoradress',
'hovedkontoreffekt':'hovedkontoreffekt',
'hovedkorridorene':'hovedkorridor',
'hovedkravet':'hovedkrav',
'hovedkriterium':'hovedkriterium',
'hovedled':'hovedled',
'hovedlinjene':'hovedlinj',
'hovedmoment':'hovedmoment',
'hovedmål':'hovedmål',
'hovedmålet':'hovedmål',
'hovedmønsteret':'hovedmønster',
'hovedoppgave':'hovedoppgav',
'hovedoppgavene':'hovedoppgav',
'hovedoppgaver':'hovedoppgav',
'hovedordning':'hovedordning',
'hovedorganisasjon':'hovedorganisasjon',
'hovedperspektiver':'hovedperspektiv',
'hovedplan':'hovedplan',
'hovedpoeng':'hovedpoeng',
'hovedpremisser':'hovedpremiss',
'hovedprinsipp':'hovedprinsipp',
'hovedprinsipper':'hovedprinsipp',
'hovedprinsippet':'hovedprinsipp',
'hovedproblemene':'hovedproblem',
'hovedproblemstilling':'hovedproblemstilling',
'hovedproblemstillingen':'hovedproblemstilling',
'hovedproblemstillinger':'hovedproblemstilling',
'hovedprognose':'hovedprognos',
'hovedpunkter':'hovedpunkt',
'hovedrapport':'hovedrapport',
'hovedrapporten':'hovedrapport',
'hovedregel':'hovedregel',
'hovedregelen':'hovedregel',
'hovedretningslinjer':'hovedretningslinj',
'hovedrolle':'hovedroll',
'hovedsak':'hovedsak',
'hovedsakelig':'hovedsak',
'hovedsamarbeidspartner':'hovedsamarbeidspartn',
'hovedsammenslutningene':'hovedsammenslutning',
'hovedsete':'hovedset',
'hovedside':'hovedsid',
'hovedsiden':'hovedsid',
'hovedskipsleia':'hovedskipslei',
'hovedspørsmål':'hovedspørsmål',
'hovedspørsmålene':'hovedspørsmål',
'hovedspørsmålet':'hovedspørsmål',
'hovedstaden':'hovedstad',
'hovedstadsområdet':'hovedstadsområd',
'hovedstadsutvalget':'hovedstadsutvalg',
'hovedstrømmene':'hovedstrømm',
'hovedsynspunkt':'hovedsynspunkt',
'hovedtall':'hovedtall',
'hovedtanken':'hovedtank',
'hovedtema':'hovedtem',
'hovedtemaene':'hovedtema',
'hovedtemaer':'hovedtema',
'hovedterminal':'hovedterminal',
'hovedterminalen':'hovedterminal',
'hovedtransportåren':'hovedtransportår',
'hovedtrekk':'hovedtrekk',
'hovedtrekkene':'hovedtrekk',
'hovedtrekket':'hovedtrekk',
'hovedtrender':'hovedtrend',
'hovedtyngden':'hovedtyngd',
'hovedtypene':'hovedtyp',
'hovedtyper':'hovedtyp',
'hovedutfordringene':'hovedutfordring',
'hovedutfyllingene':'hovedutfylling',
'hovedvalg':'hovedvalg',
'hovedvaregrupper':'hovedvaregrupp',
'hovedvareslag':'hovedvareslag',
'hovedvegnett':'hovedvegnett',
'hovedvegnettet':'hovedvegnett',
'hovedvegsystem':'hovedvegsystem',
'hovedvegsystemet':'hovedvegsystem',
'hovedvekt':'hovedvekt',
'hovedvirksomheten':'hovedvirksom',
'hovedvurderinger':'hovedvurdering',
'hovedytelse':'hovedyt',
'hovi':'hovi',
'hovudaktivitetane':'hovudaktivitet',
'hovudalternativ':'hovudalternativ',
'hovudansvaret':'hovudansvar',
'hovuddrag':'hovuddrag',
'hovuddraga':'hovuddrag',
'hovudformer':'hovudform',
'hovudføremål':'hovudføremål',
'hovudføremålet':'hovudføremål',
'hovudinnhaldet':'hovudinnhald',
'hovudkontor':'hovudkontor',
'hovudleigar':'hovudl',
'hovudliner':'hovudlin',
'hovudmål':'hovudmål',
'hovudorganisasjonen':'hovudorganisasjon',
'hovudpunkt':'hovudpunkt',
'hovudregel':'hovudregel',
'hovudregelen':'hovudregel',
'hovudreglane':'hovudregl',
'hovudreglar':'hovudregl',
'hovudsak':'hovudsak',
'hovudsakleg':'hovudsak',
'hovudsaklege':'hovudsak',
'hovudsamanslutninga':'hovudsamanslutning',
'hovudstol':'hovudstol',
'hovudsynspunktet':'hovudsynspunkt',
'hovudtypar':'hovudtyp',
'hovudvekta':'hovudvekt',
'hovvarpne':'hovvarpn',
'hsb':'hsb',
'hubs':'hub',
'huden':'hud',
'hudfarge':'hudfarg',
'hudfarve':'hudfarv',
'huff':'huff',
'huge':'hug',
'hugen':'hug',
'hugge':'hugg',
'hugger':'hugg',
'hugget':'hugg',
'huggormer':'huggorm',
'hugin':'hugin',
'huk':'huk',
'huke':'huk',
'huldrefolket':'huldrefolk',
'huldrer':'huldr',
'hule':'hul',
'huler':'hul',
'hulker':'hulk',
'hull':'hull',
'hullet':'hull',
'humanitære':'humanitær',
'humør':'humør',
'hun':'hun',
'hunaland':'hunaland',
'hund':'hund',
'hundehus':'hundehus',
'hunden':'hund',
'hundene':'hund',
'hunder':'hund',
'hundre':'hundr',
'hundreår':'hundreår',
'hundreåret':'hundreår',
'hunger':'hung',
'hurtig':'hurt',
'hurtige':'hurt',
'hurtiggående':'hurtiggå',
'hurtigruta':'hurtigrut',
'hurum':'hurum',
'hus':'hus',
'husa':'hus',
'husbank':'husbank',
'husbanken':'husbank',
'husbankterminar':'husbanktermin',
'husbråk':'husbråk',
'huseigarar':'huseigar',
'husene':'hus',
'huset':'hus',
'husflid':'husflid',
'husgeråd':'husgeråd',
'hushalda':'hushald',
'husholdningene':'husholdning',
'husholdningenes':'husholdning',
'husholdninger':'husholdning',
'husk':'husk',
'huske':'husk',
'husker':'husk',
'huskes':'husk',
'husleieloven':'huslei',
'husleiereguleringsloven':'husleieregulering',
'husleiga':'husl',
'husleige':'husl',
'husleigeavtalane':'husleigeavtal',
'husleigeavtale':'husleigeavtal',
'husleigebetaling':'husleigebetaling',
'husleigedomstolar':'husleigedomstol',
'husleigeforhold':'husleigeforhold',
'husleigeinnkrevjing':'husleigeinnkrevjing',
'husleigelova':'husleig',
'husleigelovgjevinga':'husleigelovgjeving',
'husleigeretten':'husleigerett',
'husleigerettsleg':'husleigeretts',
'husleigetap':'husleigetap',
'husleigetvistar':'husleigetvist',
'husleigetvistutval':'husleigetvistutval',
'husleigetvistutvala':'husleigetvistutval',
'husordensreglar':'husordensregl',
'husrom':'husrom',
'husstand':'husstand',
'husstanden':'husstand',
'husstandsfellesskap':'husstandsfellesskap',
'husstandsfellesskapslova':'husstandsfellesskap',
'husstandsmedlem':'husstandsmedlem',
'husstandsmedlemmen':'husstandsmedlemm',
'husstandsmedlemmer':'husstandsmedlemm',
'hussøkjande':'hussøkj',
'hussøkjarane':'hussøkjar',
'hussøkjarar':'hussøkjar',
'hussøkjarinteressene':'hussøkjarinteress',
'hustelefonsentral':'hustelefonsentral',
'hustru':'hustru',
'husvære':'husvær',
'hutrende':'hutr',
'hutrer':'hutr',
'hutte':'hutt',
'hva':'hva',
'hvalene':'hval',
'hvaler':'hval',
'hvalp':'hvalp',
'hvasse':'hvass',
'hvasst':'hvasst',
'hvelving':'hvelving',
'hvem':'hvem',
'hver':'hver',
'hverandre':'hverandr',
'hverdag':'hverdag',
'hvergelme':'hvergelm',
'hverken':'hverk',
'hvert':'hvert',
'hvese':'hves',
'hveser':'hves',
'hvil':'hvil',
'hvile':'hvil',
'hviler':'hvil',
'hvilke':'hvilk',
'hvilken':'hvilk',
'hvilket':'hvilk',
'hvirvlende':'hvirvl',
'hvirvler':'hvirvl',
'hvis':'hvis',
'hviske':'hvisk',
'hvisker':'hvisk',
'hvisket':'hvisk',
'hvisler':'hvisl',
'hvit':'hvit',
'hvite':'hvit',
'hviteste':'hvitest',
'hvitner':'hvitn',
'hvitt':'hvitt',
'hvittensand':'hvittensand',
'hvor':'hvor',
'hvorav':'hvorav',
'hvordan':'hvordan',
'hvoretter':'hvorett',
'hvorfor':'hvorfor',
'hvorledes':'hvorl',
'hvorvidt':'hvorvid',
'hybelbuarar':'hybelbuar',
'hyblar':'hybl',
'hydro':'hydro',
'hydros':'hydro',
'hyggelig':'hygg',
'hyggeligste':'hyggeligst',
'hyl':'hyl',
'hyler':'hyl',
'hylland':'hylland',
'hyllandutvalget':'hyllandutvalg',
'hylte':'hylt',
'hyme':'hym',
'hymes':'hym',
'hypotekforeiningar':'hypotekforeining',
'hypp':'hypp',
'hypper':'hypp',
'hyppige':'hypp',
'hyppigere':'hyppiger',
'hyre':'hyr',
'hyresgasternas':'hyresgastern',
'hyrrokken':'hyrrokk',
'hysj':'hysj',
'hytta':'hytt',
'hytte':'hytt',
'hyttefelt':'hyttefelt',
'hytter':'hytt',
'hå':'hå',
'håkan':'håkan',
'hånd':'hånd',
'hånde':'hånd',
'hånden':'hånd',
'håndflaten':'håndflat',
'håndheve':'håndhev',
'håndheving':'håndheving',
'håndkvern':'håndkvern',
'håndledd':'håndledd',
'håndleddet':'håndledd',
'håndslag':'håndslag',
'håndtak':'håndtak',
'håndtere':'håndter',
'håndterer':'håndter',
'håndteres':'håndter',
'håndtering':'håndtering',
'håndteringen':'håndtering',
'håndteringsevne':'håndteringsevn',
'håndteringsform':'håndteringsform',
'håndteringskapasitet':'håndteringskapasit',
'håndteringsteknologi':'håndteringsteknologi',
'håndteringstjenester':'håndteringstjenest',
'håndteringsutstyr':'håndteringsutstyr',
'håndtert':'håndter',
'håndterte':'håndter',
'håne':'hån',
'håner':'hån',
'hånlig':'hån',
'håp':'håp',
'håpe':'håp',
'håper':'håp',
'håpet':'håp',
'håpløsheten':'håpløs',
'hår':'hår',
'håret':'hår',
'hårete':'håret',
'håvamål':'håvamål',
'hæl':'hæl',
'hælene':'hæl',
'hæler':'hæl',
'hær':'hær',
'hæren':'hær',
'hærene':'hær',
'hærfører':'hærfør',
'hærguden':'hærgud',
'hærklær':'hærklær',
'hærskare':'hærskar',
'hærskarer':'hærskar',
'hæs':'hæs',
'høflig':'høf',
'høg':'høg',
'høgare':'høgar',
'høgast':'høg',
'høgaste':'høgast',
'høge':'høg',
'høgst':'høgst',
'høgste':'høgst',
'høgsteretts':'høgsterett',
'høgt':'høgt',
'høilund':'høilund',
'høne':'høn',
'hør':'hør',
'høre':'hør',
'hører':'hør',
'høres':'hør',
'høring':'høring',
'høringsfrist':'høringsfrist',
'høringsnotatet':'høringsnotat',
'høringsuttalelse':'høringsuttal',
'høringsuttalelsene':'høringsuttal',
'hørsel':'hørsel',
'hørt':'hørt',
'hørte':'hørt',
'høst':'høst',
'høste':'høst',
'høsten':'høst',
'høststormene':'høststorm',
'høvding':'høvding',
'høvdingen':'høvding',
'høvdinger':'høvding',
'høvdings':'høvding',
'høvdingsetet':'høvdingset',
'høve':'høv',
'høveleg':'høv',
'høvet':'høv',
'høvik':'høvik',
'høvisk':'høvisk',
'høy':'høy',
'høyavkastningsperiode':'høyavkastningsperiod',
'høyde':'høyd',
'høydebegrensninger':'høydebegrensning',
'høyden':'høyd',
'høye':'høye',
'høyere':'høyer',
'høyes':'høyes',
'høyest':'høyest',
'høyeste':'høyest',
'høyfrekvent':'høyfrekvent',
'høyfrekvente':'høyfrekvent',
'høyhastighetstog':'høyhastighetstog',
'høyinntektsgruppen':'høyinntektsgrupp',
'høyrde':'høyrd',
'høyre':'høyr',
'høyrer':'høyr',
'høyringsmøta':'høyringsmøt',
'høyringsmøte':'høyringsmøt',
'høyringsnotatet':'høyringsnotat',
'høyringsrunden':'høyringsrund',
'høyrt':'høyrt',
'høysete':'høyset',
'høyseter':'høyset',
'høysetet':'høyset',
'høyst':'høyst',
'høyt':'høyt',
'høytlønte':'høytlønt',
'høyvann':'høyvann',
'i':'i',
'iallfall':'iallfall',
'ica':'ica',
'ida':'ida',
'idag':'idag',
'idavollen':'idavoll',
'iddefjorden':'iddefjord',
'ide':'ide',
'idealistisk':'idealistisk',
'idear':'ide',
'ideell':'ideell',
'ideelle':'ideell',
'ideelt':'ideelt',
'ideen':'ide',
'identifiseres':'identifiser',
'identisk':'identisk',
'identiske':'identisk',
'identiteten':'identitet',
'idet':'idet',
'idretter':'idrett',
'idretts':'idrett',
'idunn':'idunn',
'idunns':'idunn',
'ifra':'ifr',
'ifølge':'ifølg',
'igangsatt':'igangsatt',
'igangsette':'igangsett',
'igangsettes':'igangsett',
'igjen':'igj',
'igjennom':'igjennom',
'igår':'igår',
'ihjel':'ihjel',
'ihrer':'ihr',
'iht':'iht',
'ii':'ii',
'ikke':'ikk',
'ikkeeksisterende':'ikkeeksister',
'ikkje':'ikkj',
'ikraft':'ikraft',
'ikraftsetjing':'ikraftsetjing',
'ikraftsetjinga':'ikraftsetjing',
'ikrafttredelse':'ikrafttred',
'ikrafttreden':'ikrafttred',
'iksl':'iksl',
'ild':'ild',
'ilden':'ild',
'ildfolket':'ildfolk',
'ildkjempen':'ildkjemp',
'ildkulen':'ildkul',
'ildlandet':'ildland',
'ildringen':'ildring',
'ildrytterne':'ildryttern',
'ildrød':'ildrød',
'ildsmykket':'ildsmykk',
'ildstedet':'ildsted',
'ildstormen':'ildstorm',
'ille':'ill',
'illeluktende':'illelukt',
'illusorisk':'illusorisk',
'illustrasjon':'illustrasjon',
'illustrasjonene':'illustrasjon',
'illustrasjonens':'illustrasjon',
'illustrasjoner':'illustrasjon',
'illustrasjonsmessig':'illustrasjonsmess',
'illustrere':'illustrer',
'illustrerende':'illustrer',
'illustrerer':'illustrer',
'illustreres':'illustrer',
'illustrert':'illustrer',
'ilo':'ilo',
'iltre':'iltr',
'imellom':'imellom',
'imens':'imen',
'imidlertid':'imidlertid',
'immingham':'immingham',
'imorgen':'imorg',
'imot':'imot',
'implementere':'implementer',
'implementeres':'implementer',
'implementert':'implementer',
'impliserer':'impliser',
'implisitt':'implisitt',
'import':'import',
'importanlegg':'importanlegg',
'importanløpssteder':'importanløpssted',
'importbedrifter':'importbedrift',
'importen':'import',
'importeres':'importer',
'importert':'importer',
'importerte':'importer',
'importgods':'importgod',
'importgodset':'importgods',
'importhavn':'importhavn',
'importhavnen':'importhavn',
'importlagre':'importlagr',
'importlast':'importl',
'importlasten':'importlast',
'importmarkeder':'importmarked',
'importoverskudd':'importoverskudd',
'importsiden':'importsid',
'importtilbøyelighet':'importtilbøy',
'importtransportene':'importtransport',
'importvarer':'importvar',
'importvekst':'importvekst',
'importveksten':'importvekst',
'importverdi':'importverdi',
'importvirksomheter':'importvirksom',
'importvolum':'importvolum',
'importvolumene':'importvolum',
'importvolumer':'importvolum',
'importvolumet':'importvolum',
'importør':'importør',
'importører':'importør',
'impulsen':'impuls',
'imøtekomme':'imøtekomm',
'imøtekommer':'imøtekomm',
'in':'in',
'inatt':'inatt',
'incitamenter':'incitament',
'indeksen':'indeks',
'indeksobligasjoner':'indeksobligasjon',
'indikasjon':'indikasjon',
'indikasjoner':'indikasjon',
'indikator':'indikator',
'indikatorbasert':'indikatorbaser',
'indikatorene':'indikator',
'indikatorer':'indikator',
'indikere':'indiker',
'indikerer':'indiker',
'indirekte':'indirekt',
'individbasert':'individbaser',
'individualisert':'individualiser',
'individualiserte':'individualiser',
'individuell':'individuell',
'individuelle':'individuell',
'individuelt':'individuelt',
'indre':'indr',
'industri':'industri',
'industrianlegg':'industrianlegg',
'industriareal':'industriareal',
'industribedriftene':'industribedrift',
'industribedrifter':'industribedrift',
'industrielle':'industriell',
'industrien':'industri',
'industriens':'industri',
'industriforetak':'industriforetak',
'industriforetakene':'industriforetak',
'industrigods':'industrigod',
'industrigrupperinger':'industrigruppering',
'industrihavnen':'industrihavn',
'industrihavner':'industrihavn',
'industrikaiene':'industrikai',
'industrikaier':'industrikai',
'industriland':'industriland',
'industrinasjoner':'industrinasjon',
'industriområdene':'industriområd',
'industriproduksjon':'industriproduksjon',
'industris':'industris',
'industrivarene':'industrivar',
'industrivarer':'industrivar',
'industrivirksomheter':'industrivirksom',
'ineffektiv':'ineffektiv',
'inflasjon':'inflasjon',
'inflasjonen':'inflasjon',
'inflasjonsendringer':'inflasjonsendring',
'inflasjonsrate':'inflasjonsrat',
'inflasjonsraten':'inflasjonsrat',
'inflasjonssikring':'inflasjonssikring',
'influensområde':'influensområd',
'influerer':'influer',
'informasjon':'informasjon',
'informasjoner':'informasjon',
'informasjons':'informasjon',
'informasjonsansvar':'informasjonsansv',
'informasjonsgrunnlaget':'informasjonsgrunnlag',
'informasjonskravet':'informasjonskrav',
'informasjonsplikt':'informasjonsplikt',
'informasjonsteknologi':'informasjonsteknologi',
'informasjonsutveksling':'informasjonsutveksling',
'informere':'informer',
'informerer':'informer',
'informert':'informer',
'infrastructure':'infrastructur',
'infrastruktur':'infrastruktur',
'infrastrukturbehov':'infrastrukturbehov',
'infrastrukturelle':'infrastrukturell',
'infrastrukturen':'infrastruktur',
'infrastrukturene':'infrastruktur',
'infrastrukturinvesteringer':'infrastrukturinvestering',
'infrastrukturproblemer':'infrastrukturproblem',
'infrastrukturprosjekter':'infrastrukturprosjekt',
'infrastrukturtiltak':'infrastrukturtiltak',
'inga':'ing',
'ingen':'ing',
'ingenting':'ingenting',
'inger':'ing',
'ingrid':'ingrid',
'inhabil':'inhabil',
'inhabile':'inhabil',
'inhabilitet':'inhabilit',
'inhabilitetsreglane':'inhabilitetsregl',
'initiativ':'initiativ',
'initiativene':'initiativ',
'initiativet':'initiativ',
'inkassolova':'inkasso',
'inkl':'inkl',
'inkludere':'inkluder',
'inkluderer':'inkluder',
'inkluderes':'inkluder',
'inkludert':'inkluder',
'inkluderte':'inkluder',
'inklusive':'inklusiv',
'inkonsekvent':'inkonsekvent',
'inn':'inn',
'innad':'innad',
'innafor':'innafor',
'innan':'innan',
'innarbeidast':'innarbeid',
'innarbeidd':'innarbeidd',
'innarbeidde':'innarbeidd',
'innarbeide':'innarbeid',
'innarbeides':'innarbeid',
'innarbeidet':'innarbeid',
'innbakt':'innbakt',
'innbeding':'innbeding',
'innbefatter':'innbefatt',
'innberetning':'innberetning',
'innberette':'innberett',
'innberettes':'innberett',
'innberettet':'innberett',
'innbetale':'innbetal',
'innbetaler':'innbetal',
'innbetales':'innbetal',
'innbetaling':'innbetaling',
'innbetalinga':'innbetaling',
'innbetalingane':'innbetaling',
'innbetalingar':'innbetaling',
'innbetalingen':'innbetaling',
'innbetalingene':'innbetaling',
'innbetalinger':'innbetaling',
'innbetalingsperiode':'innbetalingsperiod',
'innbetalingsperioden':'innbetalingsperiod',
'innbetalingsprofil':'innbetalingsprofil',
'innbetalingsprofilen':'innbetalingsprofil',
'innbetalingstid':'innbetalingstid',
'innbetalingstiden':'innbetalingstid',
'innbetalingstidens':'innbetalingstid',
'innbetalingstidspunktet':'innbetalingstidspunkt',
'innbetalingsår':'innbetalingsår',
'innbetalt':'innbetalt',
'innbetalte':'innbetalt',
'innbille':'innbill',
'innbringe':'innbring',
'innbrot':'innbrot',
'innbu':'innbu',
'innbyggere':'innbygger',
'innbyrdes':'innbyrd',
'inndeling':'inndeling',
'inndelinga':'inndeling',
'inndelingen':'inndeling',
'inndelinger':'inndeling',
'inndelt':'inndelt',
'inndrive':'inndriv',
'inndriving':'inndriving',
'inne':'inn',
'innebar':'inneb',
'inneber':'inneb',
'innebere':'inneber',
'innebygde':'innebygd',
'innebære':'innebær',
'innebærer':'innebær',
'innefor':'innefor',
'inneha':'inneh',
'innehalde':'innehald',
'innehar':'inneh',
'innehavarane':'innehavar',
'innehavaren':'innehavar',
'innehaver':'innehav',
'innehaveren':'innehaver',
'inneheld':'inneheld',
'inneholde':'innehold',
'inneholder':'innehold',
'inneklemt':'inneklemt',
'innen':'inn',
'innenfor':'innenfor',
'innenlands':'innenland',
'innenlandsk':'innenlandsk',
'innenlandske':'innenlandsk',
'innenlandstrafikk':'innenlandstrafikk',
'innenriks':'innenriks',
'innenrikstrafikken':'innenrikstrafikk',
'innenrikstransporter':'innenrikstransport',
'inneriks':'inneriks',
'innerst':'innerst',
'inneståande':'innestå',
'innestående':'innestå',
'innestår':'innestår',
'inneværende':'innevær',
'innfart':'innfart',
'innfartsvegene':'innfartsveg',
'innfartsveger':'innfartsveg',
'innflytelse':'innflyt',
'innflytting':'innflytting',
'innflyttingsklar':'innflyttingskl',
'innflyttingsklare':'innflyttingsklar',
'innforstått':'innforstått',
'innfortolke':'innfortolk',
'innfri':'innfri',
'innfridd':'innfridd',
'innfriing':'innfriing',
'innfrir':'innfrir',
'innfris':'innfris',
'innførast':'innfør',
'innføre':'innfør',
'innfører':'innfør',
'innføres':'innfør',
'innføring':'innføring',
'innføringa':'innføring',
'innføringen':'innføring',
'innført':'innført',
'innførte':'innført',
'inngang':'inngang',
'inngangen':'inngang',
'inngikk':'inngikk',
'inngis':'inngis',
'inngrep':'inngrep',
'inngå':'inngå',
'inngåande':'inngå',
'inngåelse':'inngå',
'inngående':'inngå',
'inngåing':'inngåing',
'inngår':'inngår',
'inngås':'inngås',
'inngått':'inngått',
'innhald':'innhald',
'innhaldet':'innhald',
'innhaldsrike':'innhaldsrik',
'innhenta':'innhent',
'innhente':'innhent',
'innhentet':'innhent',
'innhenting':'innhenting',
'innhold':'innhold',
'innholdet':'innhold',
'innhule':'innhul',
'innkalling':'innkalling',
'innkallinga':'innkalling',
'innkallingsplikt':'innkallingsplikt',
'innkjøp':'innkjøp',
'innkjøpslag':'innkjøpslag',
'innkjøpsmakt':'innkjøpsmakt',
'innkomne':'innkomn',
'innkrevingssentral':'innkrevingssentral',
'innkrevjing':'innkrevjing',
'innledning':'innledning',
'innledningen':'innledning',
'innledningsvis':'innledningsvis',
'innleiande':'innlei',
'innleiing':'innleiing',
'innleiinga':'innleiing',
'innlemme':'innlemm',
'innlemmes':'innlemm',
'innlevering':'innlevering',
'innlosjering':'innlosjering',
'innlysande':'innlys',
'innlån':'innlån',
'innlånte':'innlånt',
'innlåsing':'innlåsing',
'innløpet':'innløp',
'innløsning':'innløsning',
'innløsningsretten':'innløsningsrett',
'innløst':'innløst',
'innløysing':'innløysing',
'innløysingspapir':'innløysingspapir',
'innløysingsregelen':'innløysingsregel',
'innløysingssummen':'innløysingssumm',
'innmelde':'innmeld',
'innmeldes':'innmeld',
'innmelding':'innmelding',
'innom':'innom',
'innovativt':'innovativ',
'innover':'innov',
'innpasset':'innpass',
'innpå':'innpå',
'innreidde':'innreidd',
'innretning':'innretning',
'innretningen':'innretning',
'innretninger':'innretning',
'innretta':'innrett',
'innrettar':'innrett',
'innrette':'innrett',
'innrettet':'innrett',
'innretting':'innretting',
'innrømme':'innrømm',
'innrømmer':'innrømm',
'innrømmes':'innrømm',
'innrømmet':'innrømm',
'innsamling':'innsamling',
'innsats':'innsat',
'innsatsfaktorer':'innsatsfaktor',
'innseiling':'innseiling',
'innseilingen':'innseiling',
'innseilingskorridorene':'innseilingskorridor',
'innseilingsled':'innseilingsled',
'innseilingsleder':'innseilingsled',
'innsende':'inns',
'innsending':'innsending',
'innser':'inns',
'innsida':'innsid',
'innsiden':'innsid',
'innsigelse':'innsig',
'innsigelsen':'innsig',
'innsigelser':'innsig',
'innsigelses':'innsig',
'innsikt':'innsikt',
'innsjø':'innsjø',
'innsjøer':'innsjø',
'innskot':'innskot',
'innskota':'innskot',
'innskoten':'innskot',
'innskotet':'innskot',
'innskotsbasert':'innskotsbaser',
'innskotsbaserte':'innskotsbaser',
'innskotsforma':'innskotsform',
'innskotskapital':'innskotskapital',
'innskotskapitalen':'innskotskapital',
'innskottsbasert':'innskottsbaser',
'innskrenkende':'innskrenk',
'innskrenker':'innskrenk',
'innskrenkes':'innskrenk',
'innskrenking':'innskrenking',
'innskrenkning':'innskrenkning',
'innskrenkningen':'innskrenkning',
'innskudd':'innskudd',
'innskuddene':'innskudd',
'innskuddenes':'innskudd',
'innskuddet':'innskudd',
'innskuddets':'innskudd',
'innskudds':'innskudd',
'innskuddsalternativer':'innskuddsalternativ',
'innskuddsatser':'innskuddsats',
'innskuddsbasering':'innskuddsbasering',
'innskuddsbasert':'innskuddsbaser',
'innskuddsbaserte':'innskuddsbaser',
'innskuddsbolig':'innskuddsbo',
'innskuddsfond':'innskuddsfond',
'innskuddsfondet':'innskuddsfond',
'innskuddsfondets':'innskuddsfond',
'innskuddsfondsmidlene':'innskuddsfondsmidl',
'innskuddsfondsmidler':'innskuddsfondsmidl',
'innskuddsforskjeller':'innskuddsforskjell',
'innskuddsfradrag':'innskuddsfradrag',
'innskuddsfritak':'innskuddsfritak',
'innskuddsfritaket':'innskuddsfritak',
'innskuddsgivende':'innskuddsgiv',
'innskuddsgrenser':'innskuddsgrens',
'innskuddskvittering':'innskuddskvittering',
'innskuddskvitteringene':'innskuddskvittering',
'innskuddskvitteringer':'innskuddskvittering',
'innskuddsordning':'innskuddsordning',
'innskuddsordningen':'innskuddsordning',
'innskuddsordninger':'innskuddsordning',
'innskuddsordningers':'innskuddsordning',
'innskuddsordnings':'innskuddsordning',
'innskuddspensjon':'innskuddspensjon',
'innskuddspensjoner':'innskuddspensjon',
'innskuddspensjonsordning':'innskuddspensjonsordning',
'innskuddspensjonsordningen':'innskuddspensjonsordning',
'innskuddspensjonsordningene':'innskuddspensjonsordning',
'innskuddspensjonsordningens':'innskuddspensjonsordning',
'innskuddspensjonsordninger':'innskuddspensjonsordning',
'innskuddspensjonsspesifikke':'innskuddspensjonsspesifikk',
'innskuddsplan':'innskuddsplan',
'innskuddsplanen':'innskuddsplan',
'innskuddsplaner':'innskuddsplan',
'innskuddsprinsipper':'innskuddsprinsipp',
'innskuddsprofil':'innskuddsprofil',
'innskuddsprosent':'innskuddsprosent',
'innskuddsprosenten':'innskuddsprosent',
'innskuddsprosentene':'innskuddsprosent',
'innskuddsprosenter':'innskuddsprosent',
'innskuddsrammene':'innskuddsramm',
'innskuddsreserve':'innskuddsreserv',
'innskuddsreserven':'innskuddsreserv',
'innskuddssats':'innskuddssat',
'innskuddssatsen':'innskuddssats',
'innskuddssatsene':'innskuddssats',
'innskuddssatser':'innskuddssats',
'innskuddstid':'innskuddstid',
'innskuddstidspunktet':'innskuddstidspunkt',
'innskuddsvalg':'innskuddsvalg',
'innskuddsår':'innskuddsår',
'innskuddsåret':'innskuddsår',
'innskutt':'innskutt',
'innskutte':'innskutt',
'innskytarane':'innskytar',
'innskytarar':'innskytar',
'innskytere':'innskyter',
'innskytergaranti':'innskytergaranti',
'innslag':'innslag',
'innsnevring':'innsnevring',
'innsnevringer':'innsnevring',
'innsparing':'innsparing',
'innsparingar':'innsparing',
'innspill':'innspill',
'innst':'innst',
'innstille':'innstill',
'innstiller':'innstill',
'innstilling':'innstilling',
'innstillingen':'innstilling',
'innstillingene':'innstilling',
'innstilt':'innstilt',
'innstilte':'innstilt',
'innstramminga':'innstramming',
'innsyn':'innsyn',
'innta':'innt',
'inntatt':'inntatt',
'inntekstbortfall':'inntekstbortfall',
'inntekt':'inntekt',
'inntekta':'inntekt',
'inntekten':'inntekt',
'inntektene':'inntekt',
'inntekter':'inntekt',
'inntekts':'inntekt',
'inntektsbeskatning':'inntektsbeskatning',
'inntektsbeskattes':'inntektsbeskatt',
'inntektsbortfall':'inntektsbortfall',
'inntektseffekt':'inntektseffekt',
'inntektsfordelinga':'inntektsfordeling',
'inntektsfordelingen':'inntektsfordeling',
'inntektsfradrag':'inntektsfradrag',
'inntektsfradraget':'inntektsfradrag',
'inntektsfradragets':'inntektsfradrag',
'inntektsfrådrag':'inntektsfrådrag',
'inntektsføres':'inntektsfør',
'inntektsføring':'inntektsføring',
'inntektsgrensene':'inntektsgrens',
'inntektsgrunnlag':'inntektsgrunnlag',
'inntektsintervallene':'inntektsintervall',
'inntektsintervaller':'inntektsintervall',
'inntektsintervallet':'inntektsintervall',
'inntektskilder':'inntektskild',
'inntektsnivå':'inntektsnivå',
'inntektsnivået':'inntektsnivå',
'inntektsprøvet':'inntektsprøv',
'inntektssikring':'inntektssikring',
'inntektsstruktur':'inntektsstruktur',
'inntektstap':'inntektstap',
'inntektstavhengig':'inntektstavheng',
'inntektsuavhengig':'inntektsuavheng',
'inntektsvilkår':'inntektsvilkår',
'inntektsår':'inntektsår',
'inntektsåret':'inntektsår',
'inntil':'inntil',
'inntjening':'inntjening',
'inntransport':'inntransport',
'inntreffe':'inntreff',
'inntreffer':'inntreff',
'inntrer':'inntr',
'inntrykk':'inntrykk',
'inntrykket':'inntrykk',
'inntrådt':'inntråd',
'innunder':'innund',
'innvendast':'innvend',
'innvende':'innv',
'innvendig':'innvend',
'innvendige':'innvend',
'innvending':'innvending',
'innvendinga':'innvending',
'innvendingane':'innvending',
'innvendingar':'innvending',
'innvendinger':'innvending',
'innverknad':'innverknad',
'innverknaden':'innverknad',
'innvinne':'innvinn',
'innvinnes':'innvinn',
'innvinning':'innvinning',
'innvirking':'innvirking',
'innvirkning':'innvirkning',
'innvollene':'innvoll',
'innvoller':'innvoll',
'innvunnet':'innvunn',
'insentivene':'insentiv',
'insentiver':'insentiv',
'insitament':'insitament',
'insitamenter':'insitament',
'insitamentstrukturen':'insitamentstruktur',
'installasjonar':'installasjon',
'installeres':'installer',
'installering':'installering',
'instans':'instan',
'institusjon':'institusjon',
'institusjonar':'institusjon',
'institusjonen':'institusjon',
'institusjonene':'institusjon',
'institusjonenes':'institusjon',
'institusjonens':'institusjon',
'institusjoner':'institusjon',
'institusjonstilsynet':'institusjonstilsyn',
'institusjonstype':'institusjonstyp',
'institutt':'institutt',
'instituttet':'institutt',
'instituttsjef':'instituttsjef',
'instruere':'instruer',
'instruks':'instruks',
'instruksar':'instruks',
'instruksen':'instruks',
'instruksjoner':'instruksjon',
'instrumenter':'instrument',
'insurance':'insuranc',
'intakt':'intakt',
'integrasjon':'integrasjon',
'integrasjonen':'integrasjon',
'integrere':'integrer',
'integrering':'integrering',
'integrert':'integrer',
'integrerte':'integrer',
'intensjonen':'intensjon',
'intensjonene':'intensjon',
'intenst':'intenst',
'inter':'int',
'interessa':'interess',
'interessant':'interessant',
'interessante':'interessant',
'interesse':'interess',
'interesseavveging':'interesseavveging',
'interessegrupper':'interessegrupp',
'interessekonflikt':'interessekonflikt',
'interessekonfliktar':'interessekonflikt',
'interessemotsetnader':'interessemotsetnad',
'interessemotsetningar':'interessemotsetning',
'interessene':'interess',
'interessentane':'interessent',
'interessentar':'interessent',
'interesseorganisasjon':'interesseorganisasjon',
'interesseorganisasjonar':'interesseorganisasjon',
'interesser':'interess',
'interesserer':'interesser',
'interessert':'interesser',
'interesserte':'interesser',
'intereuropeisk':'intereuropeisk',
'interkommunal':'interkommunal',
'interkommunale':'interkommunal',
'interkommunalt':'interkommunalt',
'intermodale':'intermodal',
'intern':'intern',
'internasjonal':'internasjonal',
'internasjonale':'internasjonal',
'internasjonalisering':'internasjonalisering',
'internasjonaliseringen':'internasjonalisering',
'internasjonalisert':'internasjonaliser',
'internasjonalt':'internasjonalt',
'international':'international',
'interne':'intern',
'internettsider':'internettsid',
'internt':'internt',
'interregional':'interregional',
'intervall':'intervall',
'intervallene':'intervall',
'intervaller':'intervall',
'intervallet':'intervall',
'intressen':'intress',
'introduksjon':'introduksjon',
'inventar':'invent',
'investeres':'invester',
'investering':'investering',
'investeringa':'investering',
'investeringar':'investering',
'investeringene':'investering',
'investeringer':'investering',
'investeringsalternativene':'investeringsalternativ',
'investeringsalternativer':'investeringsalternativ',
'investeringsbehovet':'investeringsbehov',
'investeringsbeslutningene':'investeringsbeslutning',
'investeringsbeslutninger':'investeringsbeslutning',
'investeringskostnader':'investeringskostnad',
'investeringsmidler':'investeringsmidl',
'investeringsobjekter':'investeringsobjekt',
'investeringsorienterte':'investeringsorienter',
'investeringsperiode':'investeringsperiod',
'investeringsportefølje':'investeringsportefølj',
'investeringsporteføljen':'investeringsportefølj',
'investeringsporteføljene':'investeringsportefølj',
'investeringsporteføljer':'investeringsportefølj',
'investeringsporteføljes':'investeringsportefølj',
'investeringsstrategier':'investeringsstrategi',
'investeringstiltak':'investeringstiltak',
'investeringsvalg':'investeringsvalg',
'investeringsvalget':'investeringsvalg',
'investert':'invester',
'investorer':'investor',
'involvere':'involver',
'involverer':'involver',
'iogforseg':'iogforseg',
'ipa':'ipa',
'ir':'ir',
'ira':'ira',
'irland':'irland',
'ironisk':'ironisk',
'irritert':'irriter',
'is':'is',
'isbreene':'isbre',
'isbreer':'isbre',
'isen':'isen',
'isf':'isf',
'isfjell':'isfjell',
'isflak':'isflak',
'iskalde':'iskald',
'iskaldt':'iskald',
'island':'island',
'iso':'iso',
'isolert':'isoler',
'isproblemer':'isproblem',
'istand':'istand',
'istandsetjing':'istandsetjing',
'istappene':'istapp',
'istapper':'istapp',
'istedenfor':'istedenfor',
'istedet':'isted',
'istykker':'istykk',
'isødet':'isød',
'it':'it',
'italia':'itali',
'italiane':'itali',
'italiensk':'italiensk',
'italienske':'italiensk',
'itp':'itp',
'iv':'iv',
'ivalde':'ivald',
'ivareta':'ivaret',
'ivaretagelse':'ivaretag',
'ivaretakast':'ivaretak',
'ivaretakelse':'ivaretak',
'ivaretaking':'ivaretaking',
'ivaretar':'ivaret',
'ivaretas':'ivaret',
'ivaretatt':'ivaretatt',
'ivareteke':'ivaretek',
'ivaretekne':'ivaretekn',
'iverksatt':'iverksatt',
'iverksetjing':'iverksetjing',
'iverksetjinga':'iverksetjing',
'iverksette':'iverksett',
'iverksettelse':'iverksett',
'iverksettelsen':'iverksett',
'iverksettes':'iverksett',
'ivrig':'ivr',
'ivrige':'ivr',
'ivrigste':'ivrigst',
'ja':'ja',
'jage':'jag',
'jager':'jag',
'jakt':'jakt',
'jaktbikkjene':'jaktbikkj',
'jakte':'jakt',
'jakten':'jakt',
'jakter':'jakt',
'jaktet':'jakt',
'jaktfalk':'jaktfalk',
'jaktlykke':'jaktlykk',
'jammen':'jamm',
'jammer':'jamm',
'jamrer':'jamr',
'jamringen':'jamring',
'jamstellast':'jamstell',
'jamt':'jamt',
'jan':'jan',
'januar':'janu',
'jarl':'jarl',
'jarnsaksa':'jarnsaks',
'javel':'javel',
'javisst':'javisst',
'jbv':'jbv',
'jeg':'jeg',
'jegeren':'jeger',
'jeløya':'jeløy',
'jentene':'jent',
'jenteord':'jenteord',
'jenter':'jent',
'jern':'jern',
'jernbane':'jernb',
'jernbanebru':'jernbanebru',
'jernbaneforbindelse':'jernbaneforbind',
'jernbaneforbindelser':'jernbaneforbind',
'jernbanefrakt':'jernbanefrakt',
'jernbaneknutepunktet':'jernbaneknutepunkt',
'jernbaneløsninger':'jernbaneløsning',
'jernbanen':'jernban',
'jernbanenett':'jernbanenett',
'jernbanenettet':'jernbanenett',
'jernbanens':'jernban',
'jernbanespor':'jernbanespor',
'jernbanestasjon':'jernbanestasjon',
'jernbanetilknytning':'jernbanetilknytning',
'jernbanetilknytningen':'jernbanetilknytning',
'jernbanetiltak':'jernbanetiltak',
'jernbanetransport':'jernbanetransport',
'jernbanetransportens':'jernbanetransport',
'jernbanetrase':'jernbanetras',
'jernbaneverket':'jernbaneverk',
'jernbanevogn':'jernbanevogn',
'jernbanevogner':'jernbanevogn',
'jernbitene':'jernbit',
'jernhansker':'jernhansk',
'jernlenker':'jernlenk',
'jernsakser':'jernsaks',
'jernskogen':'jernskog',
'jernstang':'jernstang',
'jernstenger':'jernsteng',
'jernstykke':'jernstykk',
'jernstykket':'jernstykk',
'jernvotter':'jernvott',
'jessica':'jessic',
'jevn':'jevn',
'jevnbyrdige':'jevnbyrd',
'jevnere':'jevner',
'jevnt':'jevnt',
'jf':'jf',
'jo':'jo',
'jobb':'jobb',
'jobbskifte':'jobbskift',
'jobbytte':'jobbytt',
'joda':'jod',
'johansen':'johans',
'johnstad':'johnstad',
'jomfruer':'jomfru',
'jord':'jord',
'jorden':'jord',
'jorder':'jord',
'jordskjelv':'jordskjelv',
'jormungand':'jormungand',
'jotnen':'jotn',
'jotnene':'jotn',
'jotnens':'jotn',
'jotner':'jotn',
'jotun':'jotun',
'jotunborgen':'jotunborg',
'jotundatter':'jotundatt',
'jotundatteren':'jotundatter',
'jotundatters':'jotundatt',
'jotundøtrene':'jotundøtr',
'jotunfoten':'jotunfot',
'jotunheim':'jotunheim',
'jotunhodet':'jotunhod',
'jotunjente':'jotunjent',
'jotunkjemper':'jotunkjemp',
'jotunkjerringa':'jotunkjerring',
'jotunkongen':'jotunkong',
'jotunkvinne':'jotunkvinn',
'jotunkvinnen':'jotunkvinn',
'jotunkvinner':'jotunkvinn',
'jotunmor':'jotunmor',
'jotunsinnet':'jotunsinn',
'jotunskalle':'jotunskall',
'jotunskallen':'jotunskall',
'jotunskaller':'jotunskall',
'jotunslekt':'jotunslekt',
'jotunsønnen':'jotunsønn',
'jotunætt':'jotunætt',
'journal':'journal',
'journalar':'journal',
'jubel':'jubel',
'jubler':'jubl',
'juks':'juks',
'juli':'juli',
'julius':'julius',
'juni':'juni',
'jur':'jur',
'juret':'jur',
'juridisk':'juridisk',
'juridiske':'juridisk',
'just':'just',
'justerast':'juster',
'justerbar':'justerb',
'justere':'juster',
'justerer':'juster',
'justering':'justering',
'justert':'juster',
'justerte':'juster',
'justisdepartementet':'justisdepartement',
'juv':'juv',
'juveler':'juvel',
'juvet':'juv',
'k':'k',
'kabelanlegg':'kabelanlegg',
'kadavere':'kadaver',
'kaholmen':'kaholm',
'kai':'kai',
'kaianlegg':'kaianlegg',
'kaianlegget':'kaianlegg',
'kaibygging':'kaibygging',
'kaiene':'kaien',
'kaier':'kaier',
'kaifronten':'kaifront',
'kaifronter':'kaifront',
'kaikapasiteten':'kaikapasitet',
'kailengde':'kailengd',
'kaimetre':'kaimetr',
'kaiområde':'kaiområd',
'kaiområder':'kaiområd',
'kakker':'kakk',
'kald':'kald',
'kalde':'kald',
'kaldere':'kalder',
'kaldt':'kald',
'kalenderdagane':'kalenderdag',
'kalenderdagar':'kalenderdag',
'kalenderår':'kalenderår',
'kalk':'kalk',
'kalkulere':'kalkuler',
'kalla':'kall',
'kallar':'kall',
'kallast':'kall',
'kalle':'kall',
'kallen':'kall',
'kaller':'kall',
'kalles':'kall',
'kalt':'kalt',
'kalte':'kalt',
'kalvene':'kalv',
'kalver':'kalv',
'kalveskinnsko':'kalveskinnsko',
'kam':'kam',
'kameratslig':'kamerats',
'kammerpike':'kammerpik',
'kamp':'kamp',
'kampen':'kamp',
'kampestener':'kampesten',
'kamptreningen':'kamptrening',
'kan':'kan',
'kanalar':'kanal',
'kanalen':'kanal',
'kanskje':'kanskj',
'kant':'kant',
'kanten':'kant',
'kanter':'kant',
'kantre':'kantr',
'kantrer':'kantr',
'kantsatt':'kantsatt',
'kap':'kap',
'kapasitet':'kapasit',
'kapasiteten':'kapasitet',
'kapasitetsbegrensninger':'kapasitetsbegrensning',
'kapasitetsbehov':'kapasitetsbehov',
'kapasitetsbehovet':'kapasitetsbehov',
'kapasitetsbeskrankninger':'kapasitetsbeskrankning',
'kapasitetsforhold':'kapasitetsforhold',
'kapasitetsforutsetninger':'kapasitetsforutsetning',
'kapasitetsgrense':'kapasitetsgrens',
'kapasitetsgrensen':'kapasitetsgrens',
'kapasitetsgrenser':'kapasitetsgrens',
'kapasitetsmangel':'kapasitetsmangel',
'kapasitetsmessige':'kapasitetsmess',
'kapasitetsmuligheter':'kapasitetsmu',
'kapasitetspotensiale':'kapasitetspotensial',
'kapasitetspotensialet':'kapasitetspotensial',
'kapasitetspress':'kapasitetspress',
'kapasitetsproblem':'kapasitetsproblem',
'kapasitetsproblemer':'kapasitetsproblem',
'kapasitetsreserver':'kapasitetsreserv',
'kapasitetssituasjonen':'kapasitetssituasjon',
'kapasitetsskranker':'kapasitetsskrank',
'kapasitetsspørsmål':'kapasitetsspørsmål',
'kapasitetstall':'kapasitetstall',
'kapasitetstallene':'kapasitetstall',
'kapasitetsutvidelse':'kapasitetsutvid',
'kapasitetsutvidelser':'kapasitetsutvid',
'kapasitetsutvidelsestiltak':'kapasitetsutvidelsestiltak',
'kapasitetsutvikling':'kapasitetsutvikling',
'kapasitetsutviklingen':'kapasitetsutvikling',
'kapasitetsvurderinger':'kapasitetsvurdering',
'kapasitetsøkende':'kapasitetsøk',
'kapasitetsøkning':'kapasitetsøkning',
'kapital':'kapital',
'kapitalavkasting':'kapitalavkasting',
'kapitalavkastning':'kapitalavkastning',
'kapitalavkastningen':'kapitalavkastning',
'kapitalavsetning':'kapitalavsetning',
'kapitalen':'kapital',
'kapitalens':'kapital',
'kapitalforsikring':'kapitalforsikring',
'kapitalforsikringene':'kapitalforsikring',
'kapitalforsikringer':'kapitalforsikring',
'kapitalforvaltning':'kapitalforvaltning',
'kapitalforvaltningen':'kapitalforvaltning',
'kapitalforvaltningsforskriften':'kapitalforvaltningsforskrift',
'kapitalforvaltningsregler':'kapitalforvaltningsregl',
'kapitalgrunnlaget':'kapitalgrunnlag',
'kapitalinnskot':'kapitalinnskot',
'kapitalinnskota':'kapitalinnskot',
'kapitalinnskudd':'kapitalinnskudd',
'kapitalinnskytarar':'kapitalinnskytar',
'kapitalinntekt':'kapitalinntekt',
'kapitaliseringsprodukt':'kapitaliseringsprodukt',
'kapitaliseringsprodukter':'kapitaliseringsprodukt',
'kapitaloppbygging':'kapitaloppbygging',
'kapitaloppbygginga':'kapitaloppbygging',
'kapitaloppbyggingen':'kapitaloppbygging',
'kapitaloppsparing':'kapitaloppsparing',
'kapitalpensjoner':'kapitalpensjon',
'kapitalsamanslutningane':'kapitalsamanslutning',
'kapitalsamanslutningar':'kapitalsamanslutning',
'kapitalsituasjonen':'kapitalsituasjon',
'kapitalsterke':'kapitalsterk',
'kapitaltilskot':'kapitaltilskot',
'kapitalverdien':'kapitalverdi',
'kapitel':'kapitel',
'kapitlene':'kapitl',
'kapitler':'kapitl',
'kapitlet':'kapitl',
'kapitlets':'kapitl',
'kapittel':'kapittel',
'kapittelet':'kapittel',
'kapp':'kapp',
'kappdrikker':'kappdrikk',
'kappe':'kapp',
'kappen':'kapp',
'kapper':'kapp',
'kappri':'kappri',
'kappridning':'kappridning',
'kar':'kar',
'karakter':'karakt',
'karakteren':'karakter',
'karakteriseres':'karakteriser',
'karakteristika':'karakteristik',
'karakteristikken':'karakteristikk',
'karen':'kar',
'karene':'kar',
'karensperiode':'karensperiod',
'karer':'kar',
'karet':'kar',
'kari':'kari',
'karl':'karl',
'karmar':'karm',
'karmsund':'karmsund',
'karmøy':'karmøy',
'karre':'karr',
'karriere':'karrier',
'karrieren':'karrier',
'karriereutvikling':'karriereutvikling',
'kartlagt':'kartlagt',
'kartlegge':'kartlegg',
'kartlegging':'kartlegging',
'kartleggingen':'kartlegging',
'kartlegginger':'kartlegging',
'kartleggingsarbeidet':'kartleggingsarbeid',
'kartleggingsfasen':'kartleggingsfas',
'kaserne':'kasern',
'kasernebygningen':'kasernebygning',
'kasernen':'kasern',
'kasse':'kass',
'kast':'kast',
'kasta':'kast',
'kastar':'kast',
'kastast':'kast',
'kaste':'kast',
'kaster':'kast',
'kastes':'kast',
'kastet':'kast',
'katalog':'katalog',
'katalysator':'katalysator',
'kategorien':'kategori',
'kategoriene':'kategori',
'kategorier':'kategori',
'katt':'katt',
'katte':'katt',
'katten':'katt',
'kattene':'katt',
'kattepoter':'kattepot',
'katter':'katt',
'katteskinn':'katteskinn',
'kattunger':'kattung',
'kave':'kav',
'kaver':'kav',
'kgl':'kgl',
'kh':'kh',
'kiel':'kiel',
'kikke':'kikk',
'kikker':'kikk',
'kilde':'kild',
'kilden':'kild',
'kilder':'kild',
'kildevannet':'kildevann',
'kiler':'kil',
'kilo':'kilo',
'kilometer':'kilomet',
'kilometerdistanse':'kilometerdistans',
'kinn':'kinn',
'kinnene':'kinn',
'kinnet':'kinn',
'kinnskjegget':'kinnskjegg',
'kiosk':'kiosk',
'kirkenes':'kirk',
'kiste':'kist',
'kjede':'kjed',
'kjededannelsene':'kjededann',
'kjededannelser':'kjededann',
'kjedegrupperinger':'kjedegruppering',
'kjedeintegrasjon':'kjedeintegrasjon',
'kjedelige':'kjed',
'kjedens':'kjed',
'kjeder':'kjed',
'kjedestrukturene':'kjedestruktur',
'kjedet':'kjed',
'kjeft':'kjeft',
'kjeften':'kjeft',
'kjeftesmella':'kjeftesmell',
'kjekkaseri':'kjekkaseri',
'kjelde':'kjeld',
'kjelder':'kjeld',
'kjele':'kjel',
'kjelen':'kjel',
'kjeler':'kjel',
'kjelker':'kjelk',
'kjellar':'kjell',
'kjellarar':'kjellar',
'kjellstad':'kjellstad',
'kjem':'kjem',
'kjemikalier':'kjemikali',
'kjemiske':'kjemisk',
'kjemmet':'kjemm',
'kjempe':'kjemp',
'kjempediger':'kjemped',
'kjempedigre':'kjempedigr',
'kjempekrefter':'kjempekreft',
'kjempen':'kjemp',
'kjempene':'kjemp',
'kjempens':'kjemp',
'kjemper':'kjemp',
'kjempes':'kjemp',
'kjempestor':'kjempestor',
'kjempet':'kjemp',
'kjend':'kjend',
'kjende':'kjend',
'kjennast':'kjenn',
'kjenne':'kjenn',
'kjennelse':'kjenn',
'kjennemerka':'kjennemerk',
'kjennemerke':'kjennemerk',
'kjennemerket':'kjennemerk',
'kjenner':'kjenn',
'kjennes':'kjenn',
'kjennetegner':'kjennetegn',
'kjennetegnes':'kjennetegn',
'kjennetegnet':'kjennetegn',
'kjenneteikna':'kjenneteikn',
'kjenneteiknar':'kjenneteikn',
'kjenneteiknet':'kjenneteikn',
'kjennskap':'kjennskap',
'kjent':'kjent',
'kjente':'kjent',
'kjepp':'kjepp',
'kjeppjage':'kjeppjag',
'kjeppjages':'kjeppjag',
'kjeppjaget':'kjeppjag',
'kjerneaktivitetene':'kjerneaktivitet',
'kjerneaktiviteter':'kjerneaktivitet',
'kjerneverksemda':'kjerneverksemd',
'kjerra':'kjerr',
'kjerre':'kjerr',
'kjerring':'kjerring',
'kjerringa':'kjerring',
'kjerter':'kjert',
'kjettingen':'kjetting',
'kjevene':'kjev',
'kjolen':'kjol',
'kjælent':'kjælent',
'kjælne':'kjæln',
'kjære':'kjær',
'kjæremål':'kjæremål',
'kjæremålsutval':'kjæremålsutval',
'kjæreste':'kjærest',
'kjærester':'kjærest',
'kjærlighet':'kjær',
'kjærligheten':'kjær',
'kjærlighets':'kjærligh',
'kjærlighetsgudinnen':'kjærlighetsgudinn',
'kjærtegn':'kjærtegn',
'kjøkenapparat':'kjøkenapparat',
'kjøkenbenken':'kjøkenbenk',
'kjøkeninnreiing':'kjøkeninnreiing',
'kjøkenvifter':'kjøkenvift',
'kjøkkenet':'kjøkken',
'kjøkøsundet':'kjøkøsund',
'kjøl':'kjøl',
'kjølevarer':'kjølevar',
'kjølvannet':'kjølvann',
'kjønn':'kjønn',
'kjønnene':'kjønn',
'kjønnsforskjellen':'kjønnsforskjell',
'kjønnsforskjellene':'kjønnsforskjell',
'kjønnslem':'kjønnslem',
'kjønnsnøytralt':'kjønnsnøytralt',
'kjøp':'kjøp',
'kjøpar':'kjøp',
'kjøparar':'kjøpar',
'kjøparen':'kjøpar',
'kjøpast':'kjøp',
'kjøpe':'kjøp',
'kjøpekraft':'kjøpekraft',
'kjøpekraftsforbedring':'kjøpekraftsforbedring',
'kjøper':'kjøp',
'kjøperett':'kjøperett',
'kjøpes':'kjøp',
'kjøpesenter':'kjøpesent',
'kjøpesummen':'kjøpesumm',
'kjøpeutbyte':'kjøpeutbyt',
'kjøpsavtalar':'kjøpsavtal',
'kjøpsavtale':'kjøpsavtal',
'kjøpslova':'kjøp',
'kjøpssituasjonen':'kjøpssituasjon',
'kjøpt':'kjøpt',
'kjøpte':'kjøpt',
'kjøre':'kjør',
'kjøreavtaler':'kjøreavtal',
'kjørehastighet':'kjørehast',
'kjører':'kjør',
'kjøretøy':'kjøretøy',
'kjøretøyer':'kjøretøy',
'kjørte':'kjørt',
'kjøteren':'kjøter',
'kjøtt':'kjøtt',
'kjøttet':'kjøtt',
'klagast':'klag',
'klage':'klag',
'klageadgang':'klageadgang',
'klagefristen':'klagefrist',
'klageinteresse':'klageinteress',
'klagen':'klag',
'klagenemnd':'klagenemnd',
'klager':'klag',
'klagereglane':'klageregl',
'klagerett':'klagerett',
'klagesaker':'klagesak',
'klaget':'klag',
'klam':'klam',
'klamrer':'klamr',
'klang':'klang',
'klapper':'klapp',
'klar':'klar',
'klarar':'klar',
'klare':'klar',
'klarer':'klar',
'klarere':'klarer',
'klargjere':'klargjer',
'klargjort':'klargjort',
'klargjør':'klargjør',
'klargjøre':'klargjør',
'klargjøring':'klargjøring',
'klarlagt':'klarlagt',
'klarlegge':'klarlegg',
'klarsignal':'klarsignal',
'klart':'klart',
'klasse':'klass',
'klasseforskriften':'klasseforskrift',
'klassen':'klass',
'klassifiseres':'klassifiser',
'klassifisering':'klassifisering',
'klassifiseringen':'klassifisering',
'klassifisert':'klassifiser',
'klassifiserte':'klassifiser',
'klatre':'klatr',
'klatrer':'klatr',
'klatret':'klatr',
'klauslane':'klausl',
'klausul':'klausul',
'klausulane':'klausul',
'klausular':'klausul',
'klausulering':'klausulering',
'klausuleringa':'klausulering',
'klausuleringar':'klausulering',
'klausulert':'klausuler',
'klausulerte':'klausuler',
'kle':'kle',
'kleberget':'kleberg',
'kledd':'kledd',
'kledt':'kledt',
'klegg':'klegg',
'klem':'klem',
'klemmer':'klemm',
'kler':'kler',
'klevstrand':'klevstrand',
'klientkontomodellen':'klientkontomodell',
'klimaavtale':'klimaavtal',
'klimagasser':'klimagass',
'kling':'kling',
'klippeblokk':'klippeblokk',
'klippen':'klipp',
'klipper':'klipp',
'klippes':'klipp',
'klippevegger':'klippevegg',
'klirrende':'klirr',
'klisjeer':'klisje',
'klok':'klok',
'kloke':'klok',
'klokere':'kloker',
'klokeste':'klokest',
'klokkeslett':'klokkeslett',
'klokskap':'klokskap',
'klokt':'klokt',
'klorer':'klor',
'klp':'klp',
'klps':'klps',
'klubblokaler':'klubblokal',
'klump':'klump',
'klumpen':'klump',
'klumper':'klump',
'klynk':'klynk',
'klynkende':'klynk',
'klynker':'klynk',
'klyper':'klyp',
'klår':'klår',
'klårare':'klårar',
'klårast':'klår',
'klåre':'klår',
'klårgjerande':'klårgjer',
'klårgjere':'klårgjer',
'klårleggje':'klårleggj',
'klårt':'klårt',
'klær':'klær',
'klærne':'klærn',
'kløftede':'kløft',
'kløkt':'kløkt',
'klør':'klør',
'klørne':'klørn',
'kløv':'kløv',
'kløver':'kløv',
'km':'km',
'knaker':'knak',
'knapp':'knapp',
'knappe':'knapp',
'knapphet':'knapp',
'knapt':'knapt',
'knasende':'knas',
'knaser':'knas',
'kne':'kne',
'kneet':'kneet',
'knehasene':'knehas',
'knekke':'knekk',
'knekker':'knekk',
'knekkes':'knekk',
'knekkpunkt':'knekkpunkt',
'knekkpunktene':'knekkpunkt',
'knekt':'knekt',
'kneler':'knel',
'knep':'knep',
'knepper':'knepp',
'kneppeskjærsutstikkeren':'kneppeskjærsutstikker',
'knipe':'knip',
'knipen':'knip',
'kniper':'knip',
'knipser':'knips',
'knipset':'knips',
'knitrende':'knitr',
'kniv':'kniv',
'kniven':'kniv',
'kniver':'kniv',
'knokene':'knok',
'knoklene':'knokl',
'knokler':'knokl',
'knopper':'knopp',
'knurrer':'knurr',
'knuse':'knus',
'knuser':'knus',
'knuses':'knus',
'knuste':'knust',
'knut':'knut',
'knute':'knut',
'knuten':'knut',
'knutepunkt':'knutepunkt',
'knutepunkter':'knutepunkt',
'knutepunkthavnene':'knutepunkthavn',
'knutepunktsprinsippet':'knutepunktsprinsipp',
'knuter':'knut',
'knytast':'knyt',
'knyte':'knyt',
'knyter':'knyt',
'knytt':'knytt',
'knytte':'knytt',
'knyttede':'knytt',
'knytter':'knytt',
'knyttes':'knytt',
'knyttet':'knytt',
'knyttneven':'knyttnev',
'knærne':'knærn',
'kobler':'kobl',
'koblet':'kobl',
'kobling':'kobling',
'koblingen':'kobling',
'koften':'koft',
'koke':'kok',
'kokegrop':'kokegrop',
'kokekar':'kokek',
'koker':'kok',
'kokingen':'koking',
'kokken':'kokk',
'kokt':'kokt',
'kolber':'kolb',
'koldbrann':'koldbrann',
'kollegium':'kollegium',
'kollektiv':'kollektiv',
'kollektive':'kollektiv',
'kollektivet':'kollektiv',
'kollektivordning':'kollektivordning',
'kollektivordninger':'kollektivordning',
'kollektivt':'kollektiv',
'kollektivtrafikken':'kollektivtrafikk',
'kolliderer':'kollider',
'kollisjon':'kollisjon',
'kollisjonar':'kollisjon',
'kollisjoner':'kollisjon',
'kolonne':'kolonn',
'kolonnen':'kolonn',
'kolonnene':'kolonn',
'kom':'kom',
'komande':'kom',
'kombinasjon':'kombinasjon',
'kombinasjonar':'kombinasjon',
'kombinasjonen':'kombinasjon',
'kombinasjonsarealene':'kombinasjonsareal',
'kombinere':'kombiner',
'kombineres':'kombiner',
'kombinert':'kombiner',
'kombinerte':'kombiner',
'kombiskip':'kombiskip',
'kombiterminal':'kombiterminal',
'kombitransporter':'kombitransport',
'kome':'kom',
'komen':'kom',
'komite':'komit',
'komiteen':'komite',
'komiteens':'komite',
'komitemedlemmene':'komitemedlemm',
'komiteutgreiingar':'komiteutgreiing',
'kommandittselskaper':'kommandittselskap',
'komme':'komm',
'kommentarer':'kommentar',
'kommentert':'kommenter',
'kommenterte':'kommenter',
'kommer':'komm',
'kommersiell':'kommersiell',
'kommersielle':'kommersiell',
'kommet':'komm',
'kommisjonen':'kommisjon',
'kommisjonens':'kommisjon',
'kommunal':'kommunal',
'kommunaldepartementet':'kommunaldepartement',
'kommunale':'kommunal',
'kommunalrettslige':'kommunalretts',
'kommunalt':'kommunalt',
'kommunane':'kommun',
'kommunar':'kommun',
'kommune':'kommun',
'kommuneadministrasjon':'kommuneadministrasjon',
'kommunedelplan':'kommunedelplan',
'kommunedelplanen':'kommunedelplan',
'kommunegrensen':'kommunegrens',
'kommunegrensene':'kommunegrens',
'kommunekassen':'kommunekass',
'kommunelov':'kommun',
'kommuneloven':'kommun',
'kommunelovens':'kommun',
'kommunen':'kommun',
'kommunene':'kommun',
'kommunenes':'kommun',
'kommunens':'kommun',
'kommuneplan':'kommuneplan',
'kommuneplanen':'kommuneplan',
'kommuneplanens':'kommuneplan',
'kommuner':'kommun',
'kommuners':'kommun',
'kommunes':'kommun',
'kommunestyre':'kommunestyr',
'kommunestyrene':'kommunestyr',
'kommunestyrer':'kommunestyr',
'kommunestyret':'kommunestyr',
'kommunestyrets':'kommunestyr',
'kommunevalgperioden':'kommunevalgperiod',
'kommunikasjon':'kommunikasjon',
'kommunikasjonar':'kommunikasjon',
'kommunikasjonen':'kommunikasjon',
'kommunikasjonsforhold':'kommunikasjonsforhold',
'komne':'komn',
'kompensasjon':'kompensasjon',
'kompensasjonen':'kompensasjon',
'kompensasjonsgrad':'kompensasjonsgrad',
'kompensasjonsgraden':'kompensasjonsgrad',
'kompensasjonsgrader':'kompensasjonsgrad',
'kompensasjonsnivå':'kompensasjonsnivå',
'kompensasjonsnivåene':'kompensasjonsnivå',
'kompensasjonsnivåer':'kompensasjonsnivå',
'kompensasjonsnivået':'kompensasjonsnivå',
'kompensere':'kompenser',
'kompenserer':'kompenser',
'kompensert':'kompenser',
'kompetanse':'kompetans',
'kompetansen':'kompetans',
'kompetansepersoner':'kompetanseperson',
'kompetansestrid':'kompetansestrid',
'kompleksitet':'kompleksit',
'kompletteringsregel':'kompletteringsregel',
'kompletteringsregelen':'kompletteringsregel',
'komplikasjonar':'komplikasjon',
'komplikasjoner':'komplikasjon',
'komplisere':'kompliser',
'komplisert':'kompliser',
'kompliserte':'kompliser',
'komponent':'komponent',
'komponenter':'komponent',
'komprimert':'komprimer',
'kompromiss':'kompromiss',
'kon':'kon',
'kona':'kon',
'kone':'kon',
'koneløs':'koneløs',
'konen':'kon',
'koner':'kon',
'konflikt':'konflikt',
'konfliktar':'konflikt',
'konflikten':'konflikt',
'konfliktene':'konflikt',
'konflikter':'konflikt',
'konfliktforhold':'konfliktforhold',
'konfliktfylt':'konfliktfylt',
'konfliktområdet':'konfliktområd',
'konfliktsituasjonar':'konfliktsituasjon',
'konfliktsituasjoner':'konfliktsituasjon',
'kong':'kong',
'konge':'kong',
'kongelig':'kong',
'kongen':'kong',
'kongens':'kong',
'konger':'kong',
'kongsgården':'kongsgård',
'kongshavn':'kongshavn',
'kongssønnene':'kongssønn',
'konjunkturavhengig':'konjunkturavheng',
'konjunkturer':'konjunktur',
'konjunktursvingningene':'konjunktursvingning',
'konkluderer':'konkluder',
'konkluderte':'konkluder',
'konklusjon':'konklusjon',
'konklusjonen':'konklusjon',
'konklusjonene':'konklusjon',
'konklusjoner':'konklusjon',
'konklusjonskapitlet':'konklusjonskapitl',
'konkret':'konkr',
'konkrete':'konkret',
'konkretiserer':'konkretiser',
'konkretisering':'konkretisering',
'konkretisert':'konkretiser',
'konkurransdyktige':'konkurransdykt',
'konkurranse':'konkurrans',
'konkurransebegrensende':'konkurransebegrens',
'konkurransedyktig':'konkurransedykt',
'konkurransedyktige':'konkurransedykt',
'konkurransedyktighet':'konkurransedykt',
'konkurranseevne':'konkurranseevn',
'konkurransefaktor':'konkurransefaktor',
'konkurranseflatene':'konkurranseflat',
'konkurranseflater':'konkurranseflat',
'konkurranseforhold':'konkurranseforhold',
'konkurransefortrinn':'konkurransefortrinn',
'konkurransehemmende':'konkurransehemm',
'konkurransekraft':'konkurransekraft',
'konkurransekriteriene':'konkurransekriteri',
'konkurranselova':'konkurrans',
'konkurranseloven':'konkurrans',
'konkurranselovens':'konkurrans',
'konkurranselovgivning':'konkurranselovgivning',
'konkurranselovgivningen':'konkurranselovgivning',
'konkurranselovgjeving':'konkurranselovgjeving',
'konkurranselovgjevinga':'konkurranselovgjeving',
'konkurransemarked':'konkurransemarked',
'konkurransemessige':'konkurransemess',
'konkurransen':'konkurrans',
'konkurranseorienterte':'konkurranseorienter',
'konkurranseparameter':'konkurranseparamet',
'konkurranseparametre':'konkurranseparametr',
'konkurranseregler':'konkurranseregl',
'konkurranserettslege':'konkurranseretts',
'konkurransesituasjonen':'konkurransesituasjon',
'konkurranseskadeleg':'konkurranseskad',
'konkurransetilsynet':'konkurransetilsyn',
'konkurranseutsatt':'konkurranseutsatt',
'konkurranseutsatte':'konkurranseutsatt',
'konkurransevilkår':'konkurransevilkår',
'konkurransevilkåra':'konkurransevilkår',
'konkurransevridende':'konkurransevrid',
'konkurransvridende':'konkurransvrid',
'konkurrent':'konkurrent',
'konkurrentar':'konkurrent',
'konkurrerande':'konkurrer',
'konkurrere':'konkurrer',
'konkurrerende':'konkurrer',
'konkurrerer':'konkurrer',
'konkurrert':'konkurrer',
'konkurs':'konkur',
'konkursar':'konkurs',
'konkursbehandling':'konkursbehandling',
'konkursbu':'konkursbu',
'konkursbuet':'konkursbu',
'konkursen':'konkurs',
'konkurskarantene':'konkurskarant',
'konkurslova':'konkur',
'konkursopning':'konkursopning',
'konkursopninga':'konkursopning',
'konkursskyldnar':'konkursskyldn',
'konkurstilfelle':'konkurstilfell',
'konr':'konr',
'konsekvens':'konsekv',
'konsekvensane':'konsekvens',
'konsekvensar':'konsekvens',
'konsekvensberegningene':'konsekvensberegning',
'konsekvensberegninger':'konsekvensberegning',
'konsekvensen':'konsekvens',
'konsekvensene':'konsekvens',
'konsekvenser':'konsekvens',
'konsekvensutredes':'konsekvensutr',
'konsekvensutredning':'konsekvensutredning',
'konsekvensutredningen':'konsekvensutredning',
'konsekvensutredninger':'konsekvensutredning',
'konsekvensutredningsforskriftens':'konsekvensutredningsforskrift',
'konsekvensutredningsmeldingen':'konsekvensutredningsmelding',
'konsekvensutredningsreglene':'konsekvensutredningsregl',
'konsekvensvurderingene':'konsekvensvurdering',
'konsekvensvurderinger':'konsekvensvurdering',
'konsentrasjon':'konsentrasjon',
'konsentrasjonen':'konsentrasjon',
'konsentrerast':'konsentrer',
'konsentrere':'konsentrer',
'konsentrerer':'konsentrer',
'konsentrert':'konsentrer',
'konsentrerte':'konsentrer',
'konsern':'konsern',
'konsernbegrepet':'konsernbegrep',
'konserner':'konsern',
'konsernet':'konsern',
'konsernets':'konsern',
'konsernforetak':'konsernforetak',
'konsernforhold':'konsernforhold',
'konsernforholdet':'konsernforhold',
'konserninterne':'konsernintern',
'konsernkontoordningar':'konsernkontoordning',
'konsernlignende':'konsernlign',
'konsernmodell':'konsernmodell',
'konsernmodellen':'konsernmodell',
'konservativt':'konservativ',
'konsesjon':'konsesjon',
'konsesjonstildeling':'konsesjonstildeling',
'konsistente':'konsistent',
'konsolidering':'konsolidering',
'konstant':'konstant',
'konstatere':'konstater',
'konstaterer':'konstater',
'konstatert':'konstater',
'konstituerande':'konstituer',
'konstruerer':'konstruer',
'konstruert':'konstruer',
'konstruksjon':'konstruksjon',
'konstruksjonar':'konstruksjon',
'konstruksjonsløsninger':'konstruksjonsløsning',
'konsulent':'konsulent',
'konsulentar':'konsulent',
'konsulenter':'konsulent',
'konsulentrolle':'konsulentroll',
'konsulenttenester':'konsulenttenest',
'konsulenttjenester':'konsulenttjenest',
'konsum':'konsum',
'konsumenter':'konsument',
'konsumprisindeksen':'konsumprisindeks',
'konsumvarer':'konsumvar',
'kontakt':'kontakt',
'kontakten':'kontakt',
'kontaktutvalget':'kontaktutvalg',
'kontant':'kontant',
'kontantar':'kontant',
'kontantbetalinga':'kontantbetaling',
'kontanter':'kontant',
'kontantstrømanalyse':'kontantstrømanalys',
'kontantvederlag':'kontantvederlag',
'kontantverdi':'kontantverdi',
'konti':'konti',
'kontinent':'kontinent',
'kontinentale':'kontinental',
'kontinentet':'kontinent',
'kontinuerleg':'kontinuer',
'kontinuerlig':'kontinuer',
'kontinuitet':'kontinuit',
'kontinuiteten':'kontinuitet',
'konto':'konto',
'kontoen':'konto',
'kontoens':'konto',
'kontoer':'konto',
'kontoføring':'kontoføring',
'kontohaver':'kontohav',
'kontohaveren':'kontohaver',
'kontohold':'kontohold',
'kontorbedrifter':'kontorbedrift',
'kontorbygg':'kontorbygg',
'kontorer':'kontor',
'kontorfellesskapet':'kontorfellesskap',
'kontorlokaler':'kontorlokal',
'kontorstifta':'kontorstift',
'kontorstiftast':'kontorstift',
'kontorstifte':'kontorstift',
'kontorstifting':'kontorstifting',
'kontoutskrift':'kontoutskrift',
'kontoutskriften':'kontoutskrift',
'kontoutskrifter':'kontoutskrift',
'kontrakt':'kontrakt',
'kontrakten':'kontrakt',
'kontraktens':'kontrakt',
'kontrakter':'kontrakt',
'kontrakters':'kontrakt',
'kontraktsfestede':'kontraktsfest',
'kontraktsforhold':'kontraktsforhold',
'kontraktsforpliktelser':'kontraktsforplikt',
'kontraktsinngåelse':'kontraktsinngå',
'kontraktslast':'kontraktsl',
'kontraktslovene':'kontrakt',
'kontraktsplikter':'kontraktsplikt',
'kontraktsreglar':'kontraktsregl',
'kontraktsretten':'kontraktsrett',
'kontraktsrettsleg':'kontraktsretts',
'kontroll':'kontroll',
'kontrollansvar':'kontrollansv',
'kontrollansvaret':'kontrollansvar',
'kontrollansvarsregelen':'kontrollansvarsregel',
'kontrollansvarsreglane':'kontrollansvarsregl',
'kontrollen':'kontroll',
'kontrollerast':'kontroller',
'kontrollere':'kontroller',
'kontrollerer':'kontroller',
'kontrolleres':'kontroller',
'kontrollert':'kontroller',
'kontrollerte':'kontroller',
'kontrollhensyn':'kontrollhensyn',
'kontrollinstanser':'kontrollinstans',
'kontrolloppgaver':'kontrolloppgav',
'kontrollsystemer':'kontrollsystem',
'kontrolltiltak':'kontrolltiltak',
'konturene':'kontur',
'konvensjonell':'konvensjonell',
'konvensjonelle':'konvensjonell',
'konvensjonelt':'konvensjonelt',
'konvertere':'konverter',
'konverteres':'konverter',
'konvertering':'konvertering',
'konverteringen':'konvertering',
'konverteringstidspunktet':'konverteringstidspunkt',
'kooperasjon':'kooperasjon',
'kooperasjonen':'kooperasjon',
'kooperativ':'kooperativ',
'kooperativa':'kooperativ',
'kooperative':'kooperativ',
'kooperativet':'kooperativ',
'kooperativt':'kooperativ',
'koordinere':'koordiner',
'koordinerer':'koordiner',
'koordinering':'koordinering',
'koordinerings':'koordinering',
'koordinert':'koordiner',
'kopi':'kopi',
'kopiar':'kopi',
'kopla':'kopl',
'kor':'kor',
'kork':'kork',
'korkje':'korkj',
'korleis':'korleis',
'korn':'korn',
'kornsjø':'kornsjø',
'kornåkrene':'kornåkr',
'korreksjon':'korreksjon',
'korrekt':'korrekt',
'korrekte':'korrekt',
'korrelert':'korreler',
'korrespondanse':'korrespondans',
'korridor':'korridor',
'korridorene':'korridor',
'korridorer':'korridor',
'korridorgruppa':'korridorgrupp',
'korridorgruppen':'korridorgrupp',
'korridortenkning':'korridortenkning',
'korrigering':'korrigering',
'korrigert':'korriger',
'kort':'kort',
'kortare':'kortar',
'kortast':'kort',
'korte':'kort',
'kortere':'korter',
'kortest':'kortest',
'korteste':'kortest',
'kortfattet':'kortfatt',
'kortform':'kortform',
'korthet':'kort',
'kortsiden':'kortsid',
'kortsiktige':'kortsikt',
'kortvarig':'kortvar',
'kosnadseffektive':'kosnadseffektiv',
'kost':'kost',
'kostar':'kost',
'kostbar':'kostb',
'kostbare':'kostbar',
'kostbarhetene':'kostbar',
'kostbarheter':'kostbar',
'koste':'kost',
'kostelig':'kost',
'koster':'kost',
'kostn':'kostn',
'kostnad':'kostnad',
'kostnaden':'kostnad',
'kostnadene':'kostnad',
'kostnader':'kostnad',
'kostnadsauke':'kostnadsauk',
'kostnadsbaserte':'kostnadsbaser',
'kostnadseffekter':'kostnadseffekt',
'kostnadseffektiv':'kostnadseffektiv',
'kostnadseffektive':'kostnadseffektiv',
'kostnadseffektivitet':'kostnadseffektivit',
'kostnadsgevinster':'kostnadsgevinst',
'kostnadshensyn':'kostnadshensyn',
'kostnadskontroll':'kostnadskontroll',
'kostnadskrevende':'kostnadskrev',
'kostnadskrevjande':'kostnadskrevj',
'kostnadsmessig':'kostnadsmess',
'kostnadsmessige':'kostnadsmess',
'kostnadsnivå':'kostnadsnivå',
'kostnadsnivået':'kostnadsnivå',
'kostnadsoptimale':'kostnadsoptimal',
'kostnadsoverslag':'kostnadsoverslag',
'kostnadsoverslaget':'kostnadsoverslag',
'kostnadssiden':'kostnadssid',
'kostnadstallene':'kostnadstall',
'kostnadsøkningen':'kostnadsøkning',
'kr':'kr',
'krabbe':'krabb',
'krabber':'krabb',
'kraft':'kraft',
'kraften':'kraft',
'kraftig':'kraft',
'kraftige':'kraft',
'kraftigst':'kraftigst',
'kraftkjøp':'kraftkjøp',
'kraftlag':'kraftlag',
'krakk':'krakk',
'kran':'kran',
'kraner':'kran',
'krangel':'krangel',
'krangle':'krangl',
'krangler':'krangl',
'kranhåndtering':'kranhåndtering',
'kranser':'krans',
'kransystemer':'kransystem',
'kratt':'kratt',
'krattet':'kratt',
'krattskog':'krattskog',
'krav':'krav',
'krava':'krav',
'kravde':'kravd',
'kravene':'krav',
'kravet':'krav',
'kravsmål':'kravsmål',
'kravsmålet':'kravsmål',
'kravstor':'kravstor',
'kravt':'kravt',
'krediterast':'krediter',
'kreditor':'kreditor',
'kreditorane':'kreditor',
'kreditorar':'kreditor',
'kreditoren':'kreditor',
'kreditorer':'kreditor',
'kreditorinteressene':'kreditorinteress',
'kreditors':'kreditor',
'kreditorsvik':'kreditorsvik',
'kreditorvarsel':'kreditorvarsel',
'kreditorvarselet':'kreditorvarsel',
'kreditt':'kreditt',
'kredittavtaler':'kredittavtal',
'kredittforholdet':'kredittforhold',
'kredittgjevinga':'kredittgjeving',
'kredittilgangen':'kredittilgang',
'kredittilsynet':'kredittilsyn',
'kredittilsynets':'kredittilsyn',
'kredittilsynsloven':'kredittilsyn',
'kredittinstitusjon':'kredittinstitusjon',
'kredittinstitusjoner':'kredittinstitusjon',
'kredittstellet':'kredittstell',
'kredittverdig':'kredittverd',
'kreftene':'kreft',
'krefter':'kreft',
'krek':'krek',
'kremter':'kremt',
'krets':'kret',
'kretser':'krets',
'krev':'krev',
'kreve':'krev',
'krevende':'krev',
'krever':'krev',
'kreves':'krev',
'krevjast':'krevj',
'krevje':'krevj',
'krevst':'krevst',
'krig':'krig',
'krigen':'krig',
'kriger':'krig',
'krigere':'kriger',
'krigerne':'krigern',
'krigersk':'krigersk',
'krigstog':'krigstog',
'krigstrommene':'krigstromm',
'kring':'kring',
'krins':'krin',
'krinsar':'krins',
'krinsen':'krins',
'krisa':'kris',
'kriser':'kris',
'krisescenarier':'krisescenari',
'krisesituasjonar':'krisesituasjon',
'krisesituasjoner':'krisesituasjon',
'kriseåra':'kriseår',
'kristeleg':'krist',
'kristelig':'krist',
'kristensen':'kristens',
'kristiansand':'kristiansand',
'kristiansund':'kristiansund',
'kristin':'kristin',
'kriteriene':'kriteri',
'kriterier':'kriteri',
'kriteriet':'kriteri',
'kriterium':'kriterium',
'kritikk':'kritikk',
'kritiserast':'kritiser',
'kritisert':'kritiser',
'kritisk':'kritisk',
'kritiske':'kritisk',
'kroer':'kroer',
'krok':'krok',
'kroken':'krok',
'kroker':'krok',
'krokrygget':'krokrygg',
'kronebeløp':'kronebeløp',
'kronen':'kron',
'kroner':'kron',
'kroneverdien':'kroneverdi',
'krongle':'krongl',
'kropp':'kropp',
'kroppen':'kropp',
'kroppene':'kropp',
'kroppsløse':'kroppsløs',
'krummer':'krumm',
'krus':'krus',
'kruset':'krus',
'kryp':'kryp',
'krype':'kryp',
'kryper':'kryp',
'kryr':'kryr',
'kryss':'kryss',
'kryssende':'kryss',
'krysser':'kryss',
'krysset':'kryss',
'kryssområder':'kryssområd',
'kråke':'kråk',
'kråkes':'kråk',
'krøker':'krøk',
'krøp':'krøp',
'ku':'ku',
'kua':'kua',
'kuer':'kuer',
'kulde':'kuld',
'kulden':'kuld',
'kull':'kull',
'kulltangenbroen':'kulltangenbro',
'kullterminal':'kullterminal',
'kulturelle':'kulturell',
'kulturlandskap':'kulturlandskap',
'kulturminner':'kulturminn',
'kun':'kun',
'kundane':'kund',
'kundar':'kund',
'kunde':'kund',
'kundeforhold':'kundeforhold',
'kundegrupper':'kundegrupp',
'kundehensyn':'kundehensyn',
'kundemarkedet':'kundemarked',
'kunden':'kund',
'kundene':'kund',
'kundens':'kund',
'kunder':'kund',
'kune':'kun',
'kunna':'kunn',
'kunne':'kunn',
'kunnet':'kunn',
'kunngjerast':'kunngjer',
'kunngjere':'kunngjer',
'kunngjering':'kunngjering',
'kunngjeringa':'kunngjering',
'kunngjeringar':'kunngjering',
'kunngjeringsmåten':'kunngjeringsmåt',
'kunnskap':'kunnskap',
'kunnskapar':'kunnskap',
'kunnskaper':'kunnskap',
'kunnskapsoppbygging':'kunnskapsoppbygging',
'kunnskapstørst':'kunnskapstørst',
'kunst':'kunst',
'kunsten':'kunst',
'kunster':'kunst',
'kuppel':'kuppel',
'kurant':'kurant',
'kurs':'kur',
'kursen':'kurs',
'kursivering':'kursivering',
'kursreguleringsfond':'kursreguleringsfond',
'kurvatur':'kurvatur',
'kurve':'kurv',
'kutter':'kutt',
'kuttes':'kutt',
'kva':'kva',
'kvalifikasjonar':'kvalifikasjon',
'kvalifisert':'kvalifiser',
'kvalifiserte':'kvalifiser',
'kvalitativ':'kvalitativ',
'kvalitativt':'kvalitativ',
'kvalitet':'kvalit',
'kvalitetar':'kvalitet',
'kvaliteten':'kvalitet',
'kvalitetsaspektet':'kvalitetsaspekt',
'kvalitetskontroll':'kvalitetskontroll',
'kvalitetssikring':'kvalitetssikring',
'kvalm':'kvalm',
'kvalt':'kvalt',
'kvanta':'kvant',
'kvar':'kvar',
'kvarande':'kvar',
'kvarandre':'kvarandr',
'kvardagane':'kvardag',
'kvardagar':'kvardag',
'kvart':'kvart',
'kvartalsvis':'kvartalsvis',
'kvaser':'kvas',
'kvasers':'kvas',
'kvasse':'kvass',
'kveder':'kved',
'kveile':'kveil',
'kveiler':'kveil',
'kveld':'kveld',
'kvelden':'kveld',
'kvelds':'kveld',
'kveldsmat':'kveldsmat',
'kveldsmaten':'kveldsmat',
'kveles':'kvel',
'kven':'kven',
'kverke':'kverk',
'kvern':'kvern',
'kverna':'kvern',
'kvernstein':'kvernstein',
'kvernsteinene':'kvernstein',
'kvier':'kvier',
'kvikk':'kvikk',
'kvikt':'kvikt',
'kvile':'kvil',
'kviler':'kvil',
'kvinne':'kvinn',
'kvinneklær':'kvinneklær',
'kvinnelig':'kvinn',
'kvinnelige':'kvinn',
'kvinnen':'kvinn',
'kvinnene':'kvinn',
'kvinnens':'kvinn',
'kvinner':'kvinn',
'kvinners':'kvinn',
'kvinnfolk':'kvinnfolk',
'kvist':'kvist',
'kvisten':'kvist',
'kvister':'kvist',
'kvitrer':'kvitr',
'kvitt':'kvitt',
'kvitta':'kvitt',
'kvittar':'kvitt',
'kvitte':'kvitt',
'kvitteringer':'kvittering',
'kvm':'kvm',
'kvoten':'kvot',
'kyllingene':'kylling',
'kyllinger':'kylling',
'kyndig':'kynd',
'kyndige':'kynd',
'kyr':'kyr',
'kyrkje':'kyrkj',
'kyss':'kyss',
'kysse':'kyss',
'kysser':'kyss',
'kyst':'kyst',
'kystdirektoratet':'kystdirektorat',
'kysten':'kyst',
'kystkommuner':'kystkommun',
'kystsonen':'kystson',
'kystsoneplan':'kystsoneplan',
'kystterminal':'kystterminal',
'kåre':'kår',
'kåt':'kåt',
'kåte':'kåt',
'kåtere':'kåter',
'kåteste':'kåtest',
'kø':'kø',
'københavn':'københavn',
'køen':'køen',
'køer':'køer',
'køsystem':'køsystem',
'l':'l',
'la':'la',
'labbene':'labb',
'laboratorie':'laboratori',
'lag':'lag',
'laga':'lag',
'lagar':'lag',
'lagast':'lag',
'lagd':'lagd',
'lagde':'lagd',
'lage':'lag',
'lagen':'lag',
'lagene':'lag',
'lager':'lag',
'lagerbedrifter':'lagerbedrift',
'lagerbygninger':'lagerbygning',
'lagerfunksjoner':'lagerfunksjon',
'lagerhold':'lagerhold',
'lagerholdsfunksjoner':'lagerholdsfunksjon',
'lagersted':'lagersted',
'lagervirksomheter':'lagervirksom',
'lages':'lag',
'laget':'lag',
'lagetkrav':'lagetkrav',
'lagets':'lag',
'lagmannsrett':'lagmannsrett',
'lagmannsretten':'lagmannsrett',
'lagre':'lagr',
'lagrene':'lagr',
'lagring':'lagring',
'lagsdrifta':'lagsdrift',
'lagsforholdet':'lagsforhold',
'lagsføremålet':'lagsføremål',
'lagskontoret':'lagskontor',
'lagsmidlane':'lagsmidl',
'lagsorgan':'lagsorgan',
'lagsorgana':'lagsorgan',
'lagsskyldnadene':'lagsskyldnad',
'lagt':'lagt',
'lagtinget':'lagting',
'laks':'laks',
'laksens':'laks',
'laksestammen':'laksestamm',
'lam':'lam',
'lammekjøtt':'lammekjøtt',
'lammeskinnslue':'lammeskinnslu',
'lampe':'lamp',
'lan':'lan',
'land':'land',
'landa':'land',
'landarealer':'landareal',
'landarealet':'landareal',
'landbasert':'landbaser',
'landbaserte':'landbaser',
'landbruk':'landbruk',
'landbruket':'landbruk',
'landbruksavtalar':'landbruksavtal',
'landbrukssamvirket':'landbrukssamvirk',
'landbrukssamvirkets':'landbrukssamvirk',
'landdistribusjon':'landdistribusjon',
'lande':'land',
'landegrensene':'landegrens',
'landene':'land',
'lander':'land',
'landet':'land',
'landets':'land',
'landeveg':'landeveg',
'landfast':'landf',
'landfordeling':'landfordeling',
'landområde':'landområd',
'lands':'land',
'landsbygda':'landsbygd',
'landsdekkjande':'landsdekkj',
'landsdelene':'landsdel',
'landsdeler':'landsdel',
'landsforbund':'landsforbund',
'landsforening':'landsforening',
'landsgjennomsnittet':'landsgjennomsnitt',
'landsiden':'landsid',
'landskapet':'landskap',
'landskillet':'landskill',
'landsorganisasjonen':'landsorganisasjon',
'landspensjonskasse':'landspensjonskass',
'landssamanslutning':'landssamanslutning',
'landssamband':'landssamband',
'landsting':'landsting',
'landstotalen':'landstotal',
'landtransport':'landtransport',
'landtransportdistanse':'landtransportdistans',
'landtransporten':'landtransport',
'landvegs':'landveg',
'landverts':'landvert',
'lang':'lang',
'langbent':'langbent',
'langbordene':'langbord',
'langbordet':'langbord',
'lange':'lang',
'langesund':'langesund',
'langesundbukta':'langesundbukt',
'langesundsfjorden':'langesundsfjord',
'langesundterminalen':'langesundterminal',
'langobarder':'langobard',
'langs':'lang',
'langsetter':'langsett',
'langsiktig':'langsikt',
'langsiktige':'langsikt',
'langskjegger':'langskjegg',
'langsomme':'langsomm',
'langsommere':'langsommer',
'langsomste':'langsomst',
'langsomt':'langsomt',
'langt':'langt',
'langtidsplaner':'langtidsplan',
'langtidsprogram':'langtidsprogram',
'langtidsprogramma':'langtidsprogramm',
'langtidsprogrammet':'langtidsprogramm',
'langtidsprograms':'langtidsprogram',
'langtur':'langtur',
'langvarig':'langvar',
'langvarige':'langvar',
'langveggen':'langvegg',
'langveggene':'langvegg',
'lansert':'lanser',
'lanterne':'lantern',
'lapper':'lapp',
'lappeteppe':'lappetepp',
'lar':'lar',
'larm':'larm',
'larmen':'larm',
'larvik':'larvik',
'larviksfjorden':'larviksfjord',
'lass':'lass',
'lassevis':'lassevis',
'last':'last',
'lastbærer':'lastbær',
'laste':'last',
'lastebil':'lastebil',
'lastebilen':'lastebil',
'lastebiler':'lastebil',
'lastebiltransport':'lastebiltransport',
'lastebærer':'lastebær',
'lastebærere':'lastebærer',
'lasteeiere':'lasteeier',
'lasteevne':'lasteevn',
'lasten':'last',
'lastens':'last',
'laster':'last',
'lasterommet':'lasteromm',
'lasteskip':'lasteskip',
'lastesystemer':'lastesystem',
'lastet':'last',
'lasteutstyr':'lasteutstyr',
'lastevekt':'lastevekt',
'lastevolum':'lastevolum',
'lasting':'lasting',
'lat':'lat',
'late':'lat',
'later':'lat',
'latt':'latt',
'latter':'latt',
'latteren':'latter',
'laurdag':'laurdag',
'laurdagar':'laurdag',
'lausøyre':'lausøyr',
'lausøyreregisteret':'lausøyreregister',
'lausøyreting':'lausøyreting',
'lauvøy':'lauvøy',
'lauvøysson':'lauvøysson',
'lav':'lav',
'lavavkastningsperiode':'lavavkastningsperiod',
'lave':'lav',
'lavere':'laver',
'lavest':'lavest',
'laveste':'lavest',
'lavkostnadsland':'lavkostnadsland',
'lavori':'lavori',
'lavoro':'lavoro',
'lavt':'lavt',
'lavtlønte':'lavtlønt',
'le':'le',
'leca':'lec',
'ledd':'ledd',
'leddet':'ledd',
'ledds':'ledd',
'lede':'led',
'ledelse':'led',
'ledelsen':'led',
'leden':'led',
'ledende':'led',
'ledene':'led',
'leder':'led',
'ledervervet':'lederverv',
'ledes':'led',
'ledetid':'ledetid',
'ledetiden':'ledetid',
'ledig':'led',
'ledige':'led',
'leet':'leet',
'lega':'leg',
'legacoop':'legacoop',
'legaldefinisjon':'legaldefinisjon',
'legalpant':'legalpant',
'legalpanterett':'legalpanterett',
'legalpanteretten':'legalpanterett',
'lege':'leg',
'legedrikk':'legedrikk',
'legekunst':'legekunst',
'legende':'leg',
'legesenter':'legesent',
'legg':'legg',
'legge':'legg',
'legger':'legg',
'legges':'legg',
'leggjast':'leggj',
'leggje':'leggj',
'legitimiteten':'legitimitet',
'legning':'legning',
'lei':'lei',
'leia':'leia',
'leiar':'leiar',
'leiarar':'leiar',
'leiaren':'leiar',
'leiast':'leiast',
'leid':'leid',
'leidd':'leidd',
'leidningar':'leidning',
'leie':'leie',
'leieavtaler':'leieavtal',
'leieforhold':'leieforhold',
'leiegårder':'leiegård',
'leieinntekter':'leieinntekt',
'leier':'leier',
'leieren':'leier',
'leierett':'leierett',
'leieretten':'leierett',
'leiga':'leig',
'leigar':'leig',
'leigarane':'leigar',
'leigarar':'leigar',
'leigaren':'leigar',
'leigarens':'leigar',
'leigast':'leig',
'leigde':'leigd',
'leige':'leig',
'leigeavtalane':'leigeavtal',
'leigeavtalar':'leigeavtal',
'leigeavtale':'leigeavtal',
'leigeavtalen':'leigeavtal',
'leigebetaling':'leigebetaling',
'leigebuarorganisasjonane':'leigebuarorganisasjon',
'leigebuarrørsla':'leigebuarrørsl',
'leigebustad':'leigebustad',
'leigebustaden':'leigebustad',
'leigebustader':'leigebustad',
'leigeforhold':'leigeforhold',
'leigeforholdet':'leigeforhold',
'leigegard':'leigegard',
'leigegardslova':'leigegard',
'leigeinntekter':'leigeinntekt',
'leigekonstruksjonen':'leigekonstruksjon',
'leigekontraktar':'leigekontrakt',
'leigemodellen':'leigemodell',
'leigenivået':'leigenivå',
'leigeobjektet':'leigeobjekt',
'leiger':'leig',
'leigereguleringa':'leigeregulering',
'leigeregulert':'leigereguler',
'leigerett':'leigerett',
'leigerettane':'leigerett',
'leigerettar':'leigerett',
'leigeretten':'leigerett',
'leigt':'leigt',
'leiing':'leiing',
'leiinga':'leiing',
'leikeplass':'leikeplass',
'leikeplassar':'leikeplass',
'leir':'leir',
'leire':'leir',
'leiren':'leir',
'leirkjempen':'leirkjemp',
'leirtrollet':'leirtroll',
'leit':'leit',
'lek':'lek',
'leke':'lek',
'leken':'lek',
'lekent':'lekent',
'leker':'lek',
'lekne':'lekn',
'lekte':'lekt',
'lekter':'lekt',
'lemlestede':'lemlest',
'lempast':'lemp',
'lempeligere':'lempeliger',
'lemper':'lemp',
'lempes':'lemp',
'lemping':'lemping',
'lempingsreglar':'lempingsregl',
'lener':'len',
'lengd':'lengd',
'lengda':'lengd',
'lengde':'lengd',
'lengden':'lengd',
'lenge':'leng',
'lenger':'leng',
'lenging':'lenging',
'lengje':'lengj',
'lengre':'lengr',
'lengsel':'lengsel',
'lengst':'lengst',
'lengste':'lengst',
'lengstefrist':'lengstefrist',
'lengstetida':'lengstetid',
'lengte':'lengt',
'lengter':'lengt',
'lengtet':'lengt',
'lenke':'lenk',
'lenken':'lenk',
'lenkene':'lenk',
'lenker':'lenk',
'lenket':'lenk',
'lensmannen':'lensmann',
'lensmannsdistrikt':'lensmannsdistrikt',
'leppene':'lepp',
'ler':'ler',
'lese':'les',
'lesen':'les',
'leser':'les',
'leses':'les',
'leske':'lesk',
'let':'let',
'lete':'let',
'leter':'let',
'lett':'lett',
'lettar':'lett',
'lettare':'lettar',
'lettast':'lett',
'lette':'lett',
'letter':'lett',
'lettere':'letter',
'lettest':'lettest',
'lettet':'lett',
'levd':'levd',
'levde':'levd',
'leve':'lev',
'levealder':'leveald',
'levealderen':'levealder',
'levedyktig':'levedykt',
'leveform':'leveform',
'levekår':'levekår',
'levekårspolitikken':'levekårspolitikk',
'levekårsundersøkelsen':'levekårsundersøk',
'levelalder':'levelald',
'leven':'lev',
'levende':'lev',
'levenet':'leven',
'lever':'lev',
'leverandørar':'leverandør',
'leverandørens':'leverandør',
'leverandører':'leverandør',
'leveranser':'leverans',
'leveransesikkerhet':'leveransesikker',
'leverantorer':'leverantor',
'leverast':'lever',
'levere':'lever',
'levereglene':'leveregl',
'leveregler':'leveregl',
'leverer':'lever',
'leveres':'lever',
'levering':'levering',
'leveringens':'levering',
'leveringer':'levering',
'leveringskjeden':'leveringskjed',
'leveringskrav':'leveringskrav',
'leveringspålitelighet':'leveringspålit',
'leveringsservice':'leveringsservic',
'leveringstidspunkt':'leveringstidspunkt',
'leveringsønsker':'leveringsønsk',
'levert':'lever',
'levesett':'levesett',
'levet':'lev',
'levetid':'levetid',
'levetida':'levetid',
'levetidkilde':'levetidkild',
'lf':'lf',
'li':'li',
'lidd':'lidd',
'lide':'lid',
'lidelsen':'lid',
'lidskjalv':'lidskjalv',
'lidt':'lidt',
'lier':'lier',
'lierstranda':'lierstrand',
'life':'lif',
'lift':'lift',
'ligg':'ligg',
'ligge':'ligg',
'liggende':'ligg',
'ligger':'ligg',
'ligget':'ligg',
'liggetid':'liggetid',
'liggetiden':'liggetid',
'liggjande':'liggj',
'liggje':'liggj',
'light':'light',
'ligne':'lign',
'lignende':'lign',
'ligner':'lign',
'lignet':'lign',
'ligning':'ligning',
'ligningen':'ligning',
'lignings':'ligning',
'ligningsforhold':'ligningsforhold',
'ligningsforvaltning':'ligningsforvaltning',
'ligningsloven':'ligning',
'ligningsmyndighetene':'ligningsmynd',
'ligningsmyndighetenes':'ligningsmynd',
'ligningsmyndigheter':'ligningsmynd',
'ligningspraksis':'ligningspraksis',
'ligningstekniske':'ligningsteknisk',
'lik':'lik',
'like':'lik',
'likebehandles':'likebehandl',
'likebehandling':'likebehandling',
'likeins':'likein',
'likeledes':'likel',
'likemenn':'likemenn',
'likene':'lik',
'liker':'lik',
'likestille':'likestill',
'likestilles':'likestill',
'likestilling':'likestilling',
'likestillingshensyn':'likestillingshensyn',
'likestillingsmessig':'likestillingsmess',
'likestilt':'likestilt',
'likestilte':'likestilt',
'liket':'lik',
'likevel':'likevel',
'likeverdig':'likeverd',
'likeverdige':'likeverd',
'likhet':'lik',
'likheter':'lik',
'likhetskrav':'likhetskrav',
'likhetsprinsipp':'likhetsprinsipp',
'likhetsprinsippet':'likhetsprinsipp',
'likhetsvurderinger':'likhetsvurdering',
'liknande':'likn',
'liknar':'likn',
'likne':'likn',
'likninga':'likning',
'likningsstyresmaktene':'likningsstyresmakt',
'likskap':'likskap',
'likskapen':'likskap',
'likt':'likt',
'likte':'likt',
'likvide':'likvid',
'likvider':'likvid',
'likviditet':'likvidit',
'likviditeten':'likviditet',
'likviditetsbehov':'likviditetsbehov',
'likviditetsmessig':'likviditetsmess',
'likviditetsproblemer':'likviditetsproblem',
'likviditetsregnskap':'likviditetsregnskap',
'lille':'lill',
'lillebroren':'lillebror',
'lilleholt':'lilleholt',
'limited':'limited',
'limt':'limt',
'line':'lin',
'linene':'lin',
'liner':'lin',
'lines':'lin',
'lineær':'lineær',
'lineære':'lineær',
'lineært':'lineært',
'linje':'linj',
'linjefart':'linjefart',
'linjefarten':'linjefart',
'linjeføringen':'linjeføring',
'linjegods':'linjegod',
'linjene':'linj',
'linjenes':'linj',
'linjenettet':'linjenett',
'linjeopplegg':'linjeopplegg',
'linjer':'linj',
'linjerederiene':'linjerederi',
'linjerederienes':'linjerederi',
'linjerederier':'linjerederi',
'linjeskip':'linjeskip',
'linjetilbud':'linjetilbud',
'linjetrafikken':'linjetrafikk',
'lintråd':'lintråd',
'lir':'lir',
'lirker':'lirk',
'lisens':'lis',
'list':'list',
'lista':'list',
'liste':'list',
'lister':'list',
'listet':'list',
'listig':'list',
'listverk':'listverk',
'lit':'lit',
'lite':'lit',
'liten':'lit',
'litt':'litt',
'litteraturen':'litteratur',
'litterære':'litterær',
'liv':'liv',
'live':'liv',
'livet':'liv',
'livets':'liv',
'livgivende':'livgiv',
'livrente':'livrent',
'livrenteforsikring':'livrenteforsikring',
'livrenteforsikringen':'livrenteforsikring',
'livrenteforsikringer':'livrenteforsikring',
'livrenteforskriften':'livrenteforskrift',
'livrenten':'livrent',
'livrenteordninger':'livrenteordning',
'livrenteproduktene':'livrenteprodukt',
'livrenter':'livrent',
'livs':'liv',
'livsarving':'livsarving',
'livsfase':'livsfas',
'livsforsikring':'livsforsikring',
'livsforsikringar':'livsforsikring',
'livsforsikringsavtalen':'livsforsikringsavtal',
'livsforsikringsdirektiv':'livsforsikringsdirektiv',
'livsforsikringsdirektivene':'livsforsikringsdirektiv',
'livsforsikringspremie':'livsforsikringspremi',
'livsforsikringsselskap':'livsforsikringsselskap',
'livsforsikringsselskapa':'livsforsikringsselskap',
'livsforsikringsselskapene':'livsforsikringsselskap',
'livsforsikringsselskapenes':'livsforsikringsselskap',
'livsforsikringsselskaper':'livsforsikringsselskap',
'livsforsikringsselskapet':'livsforsikringsselskap',
'livsforsikringsvirksomhet':'livsforsikringsvirksom',
'livsinntekten':'livsinntekt',
'livskvalitet':'livskvalit',
'livslang':'livslang',
'livslange':'livslang',
'livsledsager':'livsledsag',
'livsløpet':'livsløp',
'livssituasjon':'livssituasjon',
'livstørst':'livstørst',
'livsvarig':'livsvar',
'livsvarige':'livsvar',
'livtrase':'livtras',
'ljåene':'ljåen',
'lo':'lo',
'lock':'lock',
'lodne':'lodn',
'logisk':'logisk',
'logistikk':'logistikk',
'logistikkbedrifter':'logistikkbedrift',
'logistikkfunksjonene':'logistikkfunksjon',
'logistikkfunksjoner':'logistikkfunksjon',
'logistikkkostnader':'logistikkkostnad',
'logistikkløsning':'logistikkløsning',
'logistikkløsninger':'logistikkløsning',
'logistikksammenheng':'logistikksammenheng',
'logistikktankegang':'logistikktankegang',
'logistikkteminalene':'logistikkteminal',
'logistikkterminalene':'logistikkterminal',
'logistiske':'logistisk',
'logrer':'logr',
'lohavn':'lohavn',
'lokal':'lokal',
'lokale':'lokal',
'lokalisere':'lokaliser',
'lokaliseres':'lokaliser',
'lokalisering':'lokalisering',
'lokaliseringen':'lokalisering',
'lokaliseringsmønsteret':'lokaliseringsmønster',
'lokaliseringspunkt':'lokaliseringspunkt',
'lokaliseringsspørsmål':'lokaliseringsspørsmål',
'lokaliseringsvalg':'lokaliseringsvalg',
'lokaliseringsvurderingen':'lokaliseringsvurdering',
'lokalisert':'lokaliser',
'lokalområder':'lokalområd',
'lokalpolitiske':'lokalpolitisk',
'lokalsamfunn':'lokalsamfunn',
'lokalsamfunnet':'lokalsamfunn',
'lokalt':'lokalt',
'lokaltrafikk':'lokaltrafikk',
'lokaltrafikken':'lokaltrafikk',
'loke':'lok',
'loker':'lok',
'lokes':'lok',
'lokk':'lokk',
'lokke':'lokk',
'lokker':'lokk',
'lokket':'lokk',
'lombardia':'lombardi',
'loneskatt':'loneskatt',
'loppe':'lopp',
'loran':'loran',
'los':'los',
'losbordingsfelt':'losbordingsfelt',
'losgebyrene':'losgebyr',
'losing':'losing',
'losplikten':'losplikt',
'loss':'loss',
'losse':'loss',
'lossearbeider':'lossearbeid',
'losses':'loss',
'lossesystemet':'lossesystem',
'losseutstyr':'losseutstyr',
'lossing':'lossing',
'lostjenesten':'lostjenest',
'losvirksomhet':'losvirksom',
'lot':'lot',
'lov':'lov',
'lova':'lov',
'lovarbeid':'lovarbeid',
'lovarbeidet':'lovarbeid',
'lovbestemmelse':'lovbestemm',
'lovbestemmelsene':'lovbestemm',
'lovbestemt':'lovbestemt',
'lovbestemte':'lovbestemt',
'love':'lov',
'loven':'lov',
'lovendring':'lovendring',
'lovendringar':'lovendring',
'lovendringen':'lovendring',
'lovendringene':'lovendring',
'lovendringenes':'lovendring',
'lovene':'lov',
'lovens':'lov',
'lover':'lov',
'lovet':'lov',
'lovfastsett':'lovfastsett',
'lovfastsette':'lovfastsett',
'lovfesta':'lovfest',
'lovfestast':'lovfest',
'lovfeste':'lovfest',
'lovfestede':'lovfest',
'lovfestes':'lovfest',
'lovfestet':'lovfest',
'lovfesting':'lovfesting',
'lovforslaget':'lovforslag',
'lovforslagets':'lovforslag',
'lovføresegn':'lovføresegn',
'lovføresegnene':'lovføresegn',
'lovgivingen':'lovgiving',
'lovgivning':'lovgivning',
'lovgivningen':'lovgivning',
'lovgivningskompetanse':'lovgivningskompetans',
'lovgjevaren':'lovgjevar',
'lovgjevarens':'lovgjevar',
'lovgjeving':'lovgjeving',
'lovgjevinga':'lovgjeving',
'lovgrunnlag':'lovgrunnlag',
'lovheimel':'lovheimel',
'lovleg':'lov',
'lovlege':'lov',
'lovlig':'lov',
'lovmotiv':'lovmotiv',
'lovmotiva':'lovmotiv',
'lovn':'lovn',
'lovnad':'lovnad',
'lovpåbod':'lovpåbod',
'lovpålagt':'lovpålagt',
'lovregelen':'lovregel',
'lovreglar':'lovregl',
'lovregler':'lovregl',
'lovregulering':'lovregulering',
'lovreguleringa':'lovregulering',
'lovreguleringen':'lovregulering',
'lovregulert':'lovreguler',
'lovregulerte':'lovreguler',
'lovstrukturelt':'lovstrukturelt',
'lovstrukturutvalet':'lovstrukturutval',
'lovteknisk':'lovteknisk',
'lovtekst':'lovtekst',
'lovtekstar':'lovtekst',
'lovteksten':'lovtekst',
'lovtekster':'lovtekst',
'lovutkast':'lovutk',
'lovutkastet':'lovutkast',
'lovutkastets':'lovutkast',
'lovutval':'lovutval',
'lovval':'lovval',
'lovverk':'lovverk',
'lovverkene':'lovverk',
'lovverket':'lovverk',
'ltp':'ltp',
'ltps':'ltps',
'lue':'lue',
'luft':'luft',
'luften':'luft',
'luftetur':'luftetur',
'luftfart':'luftfart',
'luftfartslova':'luftfart',
'luftfartsverket':'luftfartsverk',
'luftfarty':'luftfarty',
'luftfartyregister':'luftfartyregist',
'luftfartyregisteret':'luftfartyregister',
'luftfartøyregister':'luftfartøyregist',
'luftforurensing':'luftforurensing',
'luftforurensning':'luftforurensning',
'lufthavn':'lufthavn',
'luker':'luk',
'lukke':'lukk',
'lukkede':'lukk',
'lukker':'lukk',
'lukket':'lukk',
'lukt':'lukt',
'lukter':'lukt',
'lumske':'lumsk',
'lunden':'lund',
'lunder':'lund',
'lur':'lur',
'lure':'lur',
'lurer':'lur',
'lurest':'lurest',
'lurt':'lurt',
'lurte':'lurt',
'lurvelevenet':'lurveleven',
'lurvete':'lurvet',
'lus':'lus',
'luskende':'lusk',
'lusker':'lusk',
'lut':'lut',
'lutast':'lut',
'lutkast':'lutk',
'lutryggete':'lutrygget',
'ly':'ly',
'lyd':'lyd',
'lyde':'lyd',
'lyden':'lyd',
'lyder':'lyd',
'lydløse':'lydløs',
'lykke':'lykk',
'lykkelig':'lykk',
'lykten':'lykt',
'lyn':'lyn',
'lyner':'lyn',
'lyng':'lyng',
'lynget':'lyng',
'lyngve':'lyngv',
'lynsnart':'lynsnart',
'lys':'lys',
'lysaker':'lysak',
'lysast':'lys',
'lyse':'lys',
'lysende':'lys',
'lysene':'lys',
'lyser':'lys',
'lysere':'lyser',
'lysest':'lysest',
'lyset':'lys',
'lysingsblad':'lysingsblad',
'lyskryss':'lyskryss',
'lysner':'lysn',
'lyst':'lyst',
'lyste':'lyst',
'lysten':'lyst',
'lyster':'lyst',
'lystig':'lyst',
'lystrer':'lystr',
'lyt':'lyt',
'lyte':'lyt',
'lytt':'lytt',
'lytte':'lytt',
'lytter':'lytt',
'lyttet':'lytt',
'lyve':'lyv',
'lyver':'lyv',
'lå':'lå',
'låg':'låg',
'lågare':'lågar',
'lågaste':'lågast',
'låge':'låg',
'lågen':'låg',
'lågt':'lågt',
'lån':'lån',
'låna':'lån',
'lånast':'lån',
'låne':'lån',
'lånebevis':'lånebevis',
'lånefinansieringen':'lånefinansiering',
'låneforhold':'låneforhold',
'lånegarantier':'lånegaranti',
'låneinnskot':'låneinnskot',
'låneinnskota':'låneinnskot',
'låneinstitusjonar':'låneinstitusjon',
'lånekapital':'lånekapital',
'lånemarkedet':'lånemarked',
'låneopptak':'låneopptak',
'låneopptaka':'låneopptak',
'låner':'lån',
'lånes':'lån',
'lånet':'lån',
'låneterminar':'lånetermin',
'lånetidspunktet':'lånetidspunkt',
'låneutmåling':'låneutmåling',
'låneutmålinga':'låneutmåling',
'lånevilkårene':'lånevilkår',
'långjevarar':'långjevar',
'lånt':'lånt',
'låntakar':'låntak',
'lånte':'lånt',
'lår':'lår',
'lårbenet':'lårben',
'lårene':'lår',
'låsen':'lås',
'låser':'lås',
'låst':'låst',
'lær':'lær',
'lærdom':'lærdom',
'lære':'lær',
'lærer':'lær',
'læres':'lær',
'lært':'lært',
'lærte':'lært',
'læsø':'læsø',
'lød':'lød',
'løfte':'løft',
'løfter':'løft',
'løftet':'løft',
'løgn':'løgn',
'løgnen':'løgn',
'løgnene':'løgn',
'løgner':'løgn',
'løkkeberg':'løkkeberg',
'løkker':'løkk',
'løne':'løn',
'lønn':'lønn',
'lønnen':'lønn',
'lønninger':'lønning',
'lønns':'lønn',
'lønnsbegrep':'lønnsbegrep',
'lønnsbegrepet':'lønnsbegrep',
'lønnsbeskatning':'lønnsbeskatning',
'lønnsbrøk':'lønnsbrøk',
'lønnsendring':'lønnsendring',
'lønnsendringer':'lønnsendring',
'lønnsevnen':'lønnsevn',
'lønnsfordelingen':'lønnsfordeling',
'lønnsforhandlinger':'lønnsforhandling',
'lønnsforholdene':'lønnsforhold',
'lønnsgrunnlag':'lønnsgrunnlag',
'lønnsgrunnlaget':'lønnsgrunnlag',
'lønnsgrupper':'lønnsgrupp',
'lønnsinntekt':'lønnsinntekt',
'lønnsintervall':'lønnsintervall',
'lønnsintervallene':'lønnsintervall',
'lønnsintervaller':'lønnsintervall',
'lønnskostnad':'lønnskostnad',
'lønnskostnader':'lønnskostnad',
'lønnsmottakeren':'lønnsmottaker',
'lønnsnivå':'lønnsnivå',
'lønnsnivåene':'lønnsnivå',
'lønnsnivåer':'lønnsnivå',
'lønnsnivået':'lønnsnivå',
'lønnsomhet':'lønnsom',
'lønnsomheten':'lønnsom',
'lønnsomt':'lønnsomt',
'lønnsoppgaver':'lønnsoppgav',
'lønnsstigning':'lønnsstigning',
'lønnstaker':'lønnstak',
'lønnstakere':'lønnstaker',
'lønnstakerne':'lønnstakern',
'lønnstillegg':'lønnstillegg',
'lønnsutbetalinger':'lønnsutbetaling',
'lønnsutvikling':'lønnsutvikling',
'lønnsutviklingen':'lønnsutvikling',
'lønnsvekst':'lønnsvekst',
'lønnsveksten':'lønnsvekst',
'lønnsøkning':'lønnsøkning',
'lønnsøkningen':'lønnsøkning',
'lønnsøkninger':'lønnsøkning',
'lønsam':'lønsam',
'lønsame':'lønsam',
'lønsamt':'lønsamt',
'lønsemd':'lønsemd',
'lønte':'lønt',
'løp':'løp',
'løpe':'løp',
'løpefart':'løpefart',
'løpende':'løp',
'løper':'løp',
'løperen':'løper',
'løpet':'løp',
'løpetid':'løpetid',
'løpetiden':'løpetid',
'løpt':'løpt',
'lørenskogområdet':'lørenskogområd',
'løs':'løs',
'løsbart':'løsbart',
'løse':'løs',
'løselig':'løs',
'løsepenger':'løsepeng',
'løser':'løs',
'løses':'løs',
'løsgods':'løsgod',
'løsne':'løsn',
'løsner':'løsn',
'løsning':'løsning',
'løsningen':'løsning',
'løsningene':'løsning',
'løsninger':'løsning',
'løsningers':'løsning',
'løst':'løst',
'løv':'løv',
'løvverket':'løvverk',
'løysast':'løys',
'løyse':'løys',
'løysing':'løysing',
'løysinga':'løysing',
'løysingane':'løysing',
'løysingar':'løysing',
'løysingskravet':'løysingskrav',
'løysingsrett':'løysingsrett',
'løysingsrettar':'løysingsrett',
'løysingsretten':'løysingsrett',
'løysingsrettslova':'løysingsrett',
'løysingssummen':'løysingssumm',
'løysingsummen':'løysingsumm',
'løyst':'løyst',
'løyste':'løyst',
'løyvde':'løyvd',
'løyve':'løyv',
'løyvet':'løyv',
'løyvinga':'løyving',
'm':'m',
'madslien':'madsli',
'maersk':'maersk',
'magemålet':'magemål',
'mager':'mag',
'magi':'magi',
'magisk':'magisk',
'magiske':'magisk',
'magne':'magn',
'magnor':'magnor',
'magnus':'magnus',
'mai':'mai',
'makeløs':'makeløs',
'maken':'mak',
'makeskifte':'makeskift',
'makrobaserte':'makrobaser',
'makroøkonomiske':'makroøkonomisk',
'maksimal':'maksimal',
'maksimale':'maksimal',
'maksimalgrense':'maksimalgrens',
'maksimalgrensen':'maksimalgrens',
'maksimalgrenser':'maksimalgrens',
'maksimalpris':'maksimalpris',
'maksimalprisane':'maksimalpris',
'maksimalprisar':'maksimalpris',
'maksimalprisen':'maksimalpris',
'maksimalprisregulering':'maksimalprisregulering',
'maksimalprisreguleringa':'maksimalprisregulering',
'maksimalsatsene':'maksimalsats',
'maksimalsatser':'maksimalsats',
'maksimalt':'maksimalt',
'maksimalytelser':'maksimalyt',
'maksimere':'maksimer',
'maksimum':'maksimum',
'makt':'makt',
'makta':'makt',
'maktar':'makt',
'maktene':'makt',
'makter':'makt',
'maktmisbruk':'maktmisbruk',
'mal':'mal',
'malaren':'malar',
'male':'mal',
'malende':'mal',
'maler':'mal',
'mallmen':'mallm',
'malm':'malm',
'malstrøm':'malstrøm',
'malt':'malt',
'man':'man',
'mandat':'mandat',
'mandatet':'mandat',
'mandatets':'mandat',
'manen':'man',
'mange':'mang',
'mangehodete':'mangehodet',
'mangel':'mangel',
'mangelen':'mangel',
'mangelfull':'mangelfull',
'mangelfulle':'mangelfull',
'mangelskrav':'mangelskrav',
'mangelskrava':'mangelskrav',
'mangfold':'mangfold',
'manglande':'mangl',
'manglane':'mangl',
'manglar':'mangl',
'manglende':'mangl',
'mangler':'mangl',
'manglet':'mangl',
'mangt':'mangt',
'manken':'mank',
'mann':'mann',
'mannefall':'mannefall',
'mannen':'mann',
'mannfolkgæern':'mannfolkgæern',
'manngard':'manngard',
'manngarden':'manngard',
'mannlig':'mann',
'mannlige':'mann',
'manns':'mann',
'mannskap':'mannskap',
'mannvonde':'mannvond',
'mao':'mao',
'mara':'mar',
'marburg':'marburg',
'margareta':'margaret',
'margen':'marg',
'marginale':'marginal',
'marginalsatsen':'marginalsats',
'marginalskatt':'marginalskatt',
'marginalskatten':'marginalskatt',
'marginalskatteprosentene':'marginalskatteprosent',
'marginalskattesatsen':'marginalskattesats',
'marginalt':'marginalt',
'maritim':'maritim',
'maritime':'maritim',
'mark':'mark',
'marked':'marked',
'markedene':'marked',
'markeder':'marked',
'markedet':'marked',
'markedets':'marked',
'markeds':'marked',
'markedsadgang':'markedsadgang',
'markedsadgangen':'markedsadgang',
'markedsaktørene':'markedsaktør',
'markedsaktører':'markedsaktør',
'markedsandel':'markedsandel',
'markedsandeler':'markedsandel',
'markedsavkastning':'markedsavkastning',
'markedsdeltaker':'markedsdeltak',
'markedsdominert':'markedsdominer',
'markedsføring':'markedsføring',
'markedsinndeks':'markedsinndeks',
'markedsmakt':'markedsmakt',
'markedsmekanismer':'markedsmekanism',
'markedsmessig':'markedsmess',
'markedsmessige':'markedsmess',
'markedsnærhet':'markedsnær',
'markedsnærheten':'markedsnær',
'markedsnært':'markedsnært',
'markedsområde':'markedsområd',
'markedsområder':'markedsområd',
'markedsposisjon':'markedsposisjon',
'markedsregulering':'markedsregulering',
'markedsrente':'markedsrent',
'markedsrisiko':'markedsrisiko',
'markedssituasjonen':'markedssituasjon',
'markedstilgang':'markedstilgang',
'markedstilpasning':'markedstilpasning',
'markedstilpassede':'markedstilpass',
'markedsutviklingen':'markedsutvikling',
'markedsverdien':'markedsverdi',
'markedsverdiene':'markedsverdi',
'markedsverdiprinsippet':'markedsverdiprinsipp',
'markedsvilkår':'markedsvilkår',
'marken':'mark',
'markene':'mark',
'markerast':'marker',
'markere':'marker',
'markering':'markering',
'markert':'marker',
'market':'mark',
'marknad':'marknad',
'marknaden':'marknad',
'marknadsendringar':'marknadsendring',
'marknadspris':'marknadspris',
'marknadsprisane':'marknadspris',
'marknadsprisar':'marknadspris',
'marknadsprisen':'marknadspris',
'marknadstilhøva':'marknadstilhøv',
'marknadstilhøve':'marknadstilhøv',
'marknadsutviklinga':'marknadsutvikling',
'marknadsverdien':'marknadsverdi',
'markspiste':'markspist',
'markstukne':'markstukn',
'mars':'mar',
'martin':'martin',
'mase':'mas',
'masende':'mas',
'maser':'mas',
'maskene':'mask',
'masker':'mask',
'maskeradene':'maskerad',
'maskin':'maskin',
'maskiner':'maskin',
'maskinlesbar':'maskinlesb',
'maskinstasjonar':'maskinstasjon',
'maskinstopp':'maskinstopp',
'massetransporter':'massetransport',
'mastemyr':'mastemyr',
'masten':'mast',
'mat':'mat',
'matbordet':'matbord',
'maten':'mat',
'mater':'mat',
'materialane':'material',
'materialar':'material',
'materiale':'material',
'materialet':'material',
'materialiserer':'materialiser',
'materiell':'materiell',
'matjord':'matjord',
'matlyst':'matlyst',
'matmor':'matmor',
'matrestene':'matrest',
'matsekken':'matsekk',
'matvarer':'matvar',
'maurtue':'maurtu',
'mauseth':'mauseth',
'mave':'mav',
'mavemålet':'mavemål',
'maven':'mav',
'med':'med',
'medan':'medan',
'meddele':'meddel',
'medeier':'medei',
'medeigar':'med',
'medeigarar':'medeigar',
'medføre':'medfør',
'medfører':'medfør',
'medført':'medført',
'medgift':'medgift',
'medgått':'medgått',
'medhald':'medhald',
'medhjelpar':'medhjelp',
'medhjelparen':'medhjelpar',
'medhjelper':'medhjelp',
'medhold':'medhold',
'medkontrahent':'medkontrahent',
'medkontrahentar':'medkontrahent',
'medkontrahenten':'medkontrahent',
'medkontrahentens':'medkontrahent',
'medlem':'medlem',
'medlemene':'medlem',
'medlemer':'medlem',
'medlemmarna':'medlemmarn',
'medlemmarnas':'medlemmarn',
'medlemmen':'medlemm',
'medlemmene':'medlemm',
'medlemmenes':'medlemm',
'medlemmer':'medlemm',
'medlemmers':'medlemm',
'medlemmet':'medlemm',
'medlemmets':'medlemm',
'medlems':'medlem',
'medlemsaktiviteten':'medlemsaktivitet',
'medlemsblad':'medlemsblad',
'medlemsdemokrati':'medlemsdemokrati',
'medlemsfordel':'medlemsfordel',
'medlemsfordelane':'medlemsfordel',
'medlemsfordelar':'medlemsfordel',
'medlemsforeiningar':'medlemsforeining',
'medlemsføretak':'medlemsføretak',
'medlemsgenerasjonar':'medlemsgenerasjon',
'medlemsgrupper':'medlemsgrupp',
'medlemsinteresser':'medlemsinteress',
'medlemskap':'medlemskap',
'medlemskapen':'medlemskap',
'medlemskapet':'medlemskap',
'medlemskapsperioden':'medlemskapsperiod',
'medlemskarrieren':'medlemskarrier',
'medlemskommunene':'medlemskommun',
'medlemskontroll':'medlemskontroll',
'medlemskontrollen':'medlemskontroll',
'medlemskontrollerte':'medlemskontroller',
'medlemskort':'medlemskort',
'medlemslag':'medlemslag',
'medlemslaga':'medlemslag',
'medlemsland':'medlemsland',
'medlemslands':'medlemsland',
'medlemsorganisasjon':'medlemsorganisasjon',
'medlemsorganisasjonar':'medlemsorganisasjon',
'medlemsskap':'medlemsskap',
'medlemsskapsreglene':'medlemsskapsregl',
'medlemsstatene':'medlemsstat',
'medlemsstatenes':'medlemsstat',
'medlemsstater':'medlemsstat',
'medlemsstokken':'medlemsstokk',
'medlemsstyret':'medlemsstyr',
'medlemsstyring':'medlemsstyring',
'medlemsstyrte':'medlemsstyrt',
'medlemstal':'medlemstal',
'medlemstallet':'medlemstall',
'medlemstenester':'medlemstenest',
'medlemstid':'medlemstid',
'medlemstiden':'medlemstid',
'medmindre':'medmindr',
'medregne':'medregn',
'medregnet':'medregn',
'medregning':'medregning',
'medregningen':'medregning',
'medregningsadgangen':'medregningsadgang',
'medregningsreglene':'medregningsregl',
'medrekna':'medrekn',
'medråderett':'medråderett',
'medråderetten':'medråderett',
'medverka':'medverk',
'medverkande':'medverk',
'medverkar':'medverk',
'medverkaransvaret':'medverkaransvar',
'medverke':'medverk',
'medverknad':'medverknad',
'medverknaden':'medverknad',
'medvind':'medvind',
'medvirke':'medvirk',
'medvirkning':'medvirkning',
'meg':'meg',
'meget':'meg',
'megleren':'megler',
'mehr':'mehr',
'meieri':'meieri',
'meine':'mein',
'meiner':'mein',
'meininga':'meining',
'meiningane':'meining',
'meiningar':'meining',
'meiningsinnhald':'meiningsinnhald',
'meint':'meint',
'meinte':'meint',
'meir':'meir',
'meirinntekter':'meirinntekt',
'meirkostnader':'meirkostnad',
'meirverdiavgift':'meirverdiavgift',
'meirverdiavgifta':'meirverdiavgift',
'mekanisk':'mekanisk',
'mekanisme':'mekanism',
'meklartenester':'meklartenest',
'mektig':'mekt',
'mektige':'mekt',
'mektigere':'mektiger',
'mektigste':'mektigst',
'meld':'meld',
'meldast':'meld',
'melde':'meld',
'melder':'meld',
'meldes':'meld',
'melding':'melding',
'meldinga':'melding',
'meldingane':'melding',
'meldingar':'melding',
'meldingen':'melding',
'meldingene':'melding',
'meldinger':'melding',
'meldt':'meld',
'melk':'melk',
'melket':'melk',
'mellan':'mellan',
'mellom':'mellom',
'mellombels':'mellombel',
'mellomkomst':'mellomkomst',
'mellomlagring':'mellomlagring',
'mellomlang':'mellomlang',
'mellomlange':'mellomlang',
'mellomlangsiktige':'mellomlangsikt',
'mellomlangt':'mellomlangt',
'mellomliggjande':'mellomliggj',
'mellomløsninger':'mellomløsning',
'mellomløysing':'mellomløysing',
'mellommann':'mellommann',
'mellommenn':'mellommenn',
'mellomnivå':'mellomnivå',
'mellomperiode':'mellomperiod',
'mellomrekning':'mellomrekning',
'mellomrekninga':'mellomrekning',
'mellomrekningsmidlane':'mellomrekningsmidl',
'mellomrekningsordninga':'mellomrekningsordning',
'mellomrom':'mellomrom',
'mellomstasjon':'mellomstasjon',
'mellomstasjonen':'mellomstasjon',
'mellomstore':'mellomstor',
'mellomtida':'mellomtid',
'mellomtiden':'mellomtid',
'mellomting':'mellomting',
'mellomveg':'mellomveg',
'melz':'melz',
'men':'men',
'mene':'men',
'mener':'men',
'menes':'men',
'mengd':'mengd',
'mengda':'mengd',
'mengde':'mengd',
'mengden':'mengd',
'mengdene':'mengd',
'mengder':'mengd',
'menger':'meng',
'mening':'mening',
'meningen':'mening',
'menja':'menj',
'menn':'menn',
'mennene':'menn',
'menneske':'mennesk',
'menneskebarn':'menneskebarn',
'menneskebarna':'menneskebarn',
'menneskehår':'menneskehår',
'menneskekryp':'menneskekryp',
'menneskelignende':'menneskelign',
'menneskeliv':'menneskeliv',
'menneskene':'mennesk',
'menneskenes':'mennesk',
'menneskepar':'menneskep',
'mennesker':'mennesk',
'menneskeslekt':'menneskeslekt',
'menneskeslekten':'menneskeslekt',
'menns':'menn',
'mens':'men',
'ment':'ment',
'mente':'ment',
'mer':'mer',
'merarbeid':'merarbeid',
'meravkastningen':'meravkastning',
'merbelastning':'merbelastning',
'merbelastningene':'merbelastning',
'merchanisms':'merchanism',
'merforpliktelsen':'merforplikt',
'merinnskudd':'merinnskudd',
'merk':'merk',
'merkbar':'merkb',
'merke':'merk',
'merkelig':'merk',
'merkelige':'merk',
'merkeligste':'merkeligst',
'merkene':'merk',
'merker':'merk',
'merkes':'merk',
'merkesystemet':'merkesystem',
'merket':'merk',
'merketjenesten':'merketjenest',
'merkevarer':'merkevar',
'merking':'merking',
'merknad':'merknad',
'merknadene':'merknad',
'merknader':'merknad',
'merkostnader':'merkostnad',
'merksam':'merksam',
'merksame':'merksam',
'merkverdig':'merkverd',
'merutgiftene':'merutgift',
'messing':'messing',
'mest':'mest',
'mesta':'mest',
'meste':'mest',
'mesteparten':'mestepart',
'mester':'mest',
'mestersmed':'mestersmed',
'mestersmeder':'mestersmed',
'mestertyv':'mestertyv',
'mestrer':'mestr',
'mestret':'mestr',
'metall':'metall',
'metaller':'metall',
'metallklumpen':'metallklump',
'metallproduksjon':'metallproduksjon',
'metallprodukter':'metallprodukt',
'meter':'met',
'metergrunnen':'metergrunn',
'meters':'met',
'metersbeltet':'metersbelt',
'metode':'metod',
'metoden':'metod',
'metoder':'metod',
'metodisk':'metodisk',
'mette':'mett',
'mfl':'mfl',
'mht':'mht',
'michael':'michael',
'michaels':'michael',
'middagstid':'middagstid',
'middel':'middel',
'middelklassen':'middelklass',
'middels':'middel',
'midgard':'midgard',
'midgards':'midgard',
'midgardsormen':'midgardsorm',
'midgardsormon':'midgardsormon',
'midlane':'midl',
'midlar':'midl',
'midlene':'midl',
'midlenes':'midl',
'midler':'midl',
'midlere':'midler',
'midlertidig':'midlertid',
'midlertidige':'midlertid',
'midnattstid':'midnattstid',
'midt':'midt',
'midten':'midt',
'mil':'mil',
'milano':'milano',
'mild':'mild',
'mildare':'mildar',
'militær':'militær',
'militære':'militær',
'militærteneste':'militærtenest',
'militærtjeneste':'militærtjenest',
'miljø':'miljø',
'miljøavdelingen':'miljøavdeling',
'miljøbelastende':'miljøbelast',
'miljøbelastning':'miljøbelastning',
'miljøbelastningen':'miljøbelastning',
'miljøbelastninger':'miljøbelastning',
'miljødepartementet':'miljødepartement',
'miljødifferensierte':'miljødifferensier',
'miljøeffekter':'miljøeffekt',
'miljøet':'miljø',
'miljøforbedring':'miljøforbedring',
'miljøforbedringer':'miljøforbedring',
'miljøforhold':'miljøforhold',
'miljøforstyrrende':'miljøforstyrr',
'miljøhensyn':'miljøhensyn',
'miljøkonflikter':'miljøkonflikt',
'miljøkostnadene':'miljøkostnad',
'miljøkostnader':'miljøkostnad',
'miljøkrav':'miljøkrav',
'miljøkriteriene':'miljøkriteri',
'miljømessig':'miljømess',
'miljømessige':'miljømess',
'miljøpolitikk':'miljøpolitikk',
'miljøpolitikkens':'miljøpolitikk',
'miljøpolitisk':'miljøpolitisk',
'miljøpolitiske':'miljøpolitisk',
'miljøprioriteringstiltak':'miljøprioriteringstiltak',
'miljøproblemene':'miljøproblem',
'miljøproblemer':'miljøproblem',
'miljørevisjon':'miljørevisjon',
'miljøsikkerhet':'miljøsikker',
'miljøspørsmål':'miljøspørsmål',
'miljøspørsmåla':'miljøspørsmål',
'miljøulemper':'miljøulemp',
'miljøvennlig':'miljøvenn',
'miljøvennlige':'miljøvenn',
'miljøvernavdeling':'miljøvernavdeling',
'miljøverndepartementet':'miljøverndepartement',
'miljøverndepartementets':'miljøverndepartement',
'mill':'mill',
'milliarder':'milliard',
'million':'million',
'millionar':'million',
'millioner':'million',
'mime':'mim',
'mimes':'mim',
'min':'min',
'mindre':'mindr',
'mindretal':'mindretal',
'mindretalet':'mindretal',
'mindretall':'mindretall',
'mindretallet':'mindretall',
'mindretallets':'mindretall',
'mindretalsvern':'mindretalsvern',
'mindretalsvernet':'mindretalsvern',
'mindreårig':'mindreår',
'mine':'min',
'mineralprodukter':'mineralprodukt',
'minimal':'minimal',
'minimale':'minimal',
'minimalt':'minimalt',
'minimerer':'minimer',
'minimert':'minimer',
'minimum':'minimum',
'minimumskrav':'minimumskrav',
'ministeriet':'ministeri',
'ministerio':'ministerio',
'minka':'mink',
'minkar':'mink',
'minke':'mink',
'minker':'mink',
'minket':'mink',
'minnast':'minn',
'minne':'minn',
'minner':'minn',
'minnet':'minn',
'minske':'minsk',
'minsker':'minsk',
'minskes':'minsk',
'minst':'minst',
'minste':'minst',
'minstealder':'minsteald',
'minsteavkastning':'minsteavkastning',
'minstefradraget':'minstefradrag',
'minstefrist':'minstefrist',
'minstegrad':'minstegrad',
'minstegrense':'minstegrens',
'minstegrensen':'minstegrens',
'minstegrenser':'minstegrens',
'minstekapital':'minstekapital',
'minstekapitalen':'minstekapital',
'minstekrav':'minstekrav',
'minstekravene':'minstekrav',
'minstekravet':'minstekrav',
'minstelønn':'minstelønn',
'minstenivå':'minstenivå',
'minstenivåer':'minstenivå',
'minstenivået':'minstenivå',
'minstepensjon':'minstepensjon',
'minstepensjonen':'minstepensjon',
'minstesikring':'minstesikring',
'minstetal':'minstetal',
'minstetid':'minstetid',
'minstetida':'minstetid',
'minusgrader':'minusgrad',
'minutter':'minutt',
'misbruk':'misbruk',
'misbrukes':'misbruk',
'misbruksregel':'misbruksregel',
'misbruksregelen':'misbruksregel',
'misbruksreglane':'misbruksregl',
'misbruksreglar':'misbruksregl',
'misbrukt':'misbrukt',
'misbrukte':'misbrukt',
'misforståelse':'misforstå',
'misforståelser':'misforstå',
'mishald':'mishald',
'mishaldande':'mishald',
'mishalde':'mishald',
'mishaldet':'mishald',
'mishaldne':'mishaldn',
'mishaldskrav':'mishaldskrav',
'mishaldssynspunkt':'mishaldssynspunkt',
'mishaldstilfella':'mishaldstilfell',
'mishaldstilfelle':'mishaldstilfell',
'mishaldsverknader':'mishaldsverknad',
'mishaldsvurderinga':'mishaldsvurdering',
'misheld':'misheld',
'mishøve':'mishøv',
'mislykkes':'mislykk',
'misnøgd':'misnøgd',
'misnøya':'misnøy',
'misnøye':'misnøy',
'misoppfattast':'misoppfatt',
'mista':'mist',
'mistanke':'mistank',
'mistar':'mist',
'miste':'mist',
'misteltein':'misteltein',
'mistelteinen':'misteltein',
'mister':'mist',
'mistet':'mist',
'mistro':'mistro',
'mistydingar':'mistyding',
'misunnelig':'misunn',
'misunner':'misunn',
'misvisande':'misvis',
'mitgleiderzahl':'mitgleiderzahl',
'mitglieder':'mitglied',
'mitt':'mitt',
'mittels':'mittel',
'mjaue':'mjaue',
'mjauende':'mjauend',
'mjauer':'mjauer',
'mjød':'mjød',
'mjøden':'mjød',
'mjølner':'mjøln',
'mjøndalen':'mjøndal',
'mobile':'mobil',
'mobilisering':'mobilisering',
'mobilitet':'mobilit',
'mobiliteten':'mobilitet',
'mode':'mod',
'modell':'modell',
'modellane':'modell',
'modellar':'modell',
'modellberegningene':'modellberegning',
'modellberegninger':'modellberegning',
'modellen':'modell',
'modellene':'modell',
'modeller':'modell',
'moden':'mod',
'moderat':'moderat',
'modereres':'moderer',
'moderne':'modern',
'modernisering':'modernisering',
'moderniseringsprosjekter':'moderniseringsprosjekt',
'modernisert':'moderniser',
'modgunn':'modgunn',
'modifikasjonar':'modifikasjon',
'modig':'mod',
'modige':'mod',
'modigste':'modigst',
'mogleg':'mog',
'moglege':'mog',
'mokkurkalve':'mokkurkalv',
'moland':'moland',
'molandutvalgets':'molandutvalg',
'moment':'moment',
'momenta':'moment',
'momenter':'moment',
'momsnummer':'momsnumm',
'momsnummerbaserte':'momsnummerbaser',
'mon':'mon',
'monner':'monn',
'monopol':'monopol',
'monopollignende':'monopollign',
'monster':'monst',
'monsteret':'monster',
'monstre':'monstr',
'mor':'mor',
'moralsk':'moralsk',
'mord':'mord',
'moren':'mor',
'morer':'mor',
'morgen':'morg',
'morgendugg':'morgendugg',
'morgenen':'morgen',
'morkne':'morkn',
'moro':'moro',
'morro':'morro',
'mors':'mor',
'morselskapet':'morselskap',
'morsom':'morsom',
'mortifikasjon':'mortifikasjon',
'mortifisere':'mortifiser',
'mose':'mos',
'mosegrodd':'mosegrodd',
'mosegrodde':'mosegrodd',
'mosjons':'mosjon',
'moss':'moss',
'mosseporten':'mosseport',
'mosseregionen':'mosseregion',
'mossevegen':'mosseveg',
'mot':'mot',
'motargument':'motargument',
'motargumenter':'motargument',
'motbydelige':'motbyd',
'motet':'mot',
'motiv':'motiv',
'motivasjonen':'motivasjon',
'motiver':'motiv',
'motivere':'motiver',
'motiveres':'motiver',
'motløs':'motløs',
'motmelding':'motmelding',
'motorveg':'motorveg',
'motorvegar':'motorveg',
'motorvegbru':'motorvegbru',
'motorveger':'motorveg',
'motsatt':'motsatt',
'motsatte':'motsatt',
'motsegn':'motsegn',
'motsegner':'motsegn',
'motset':'mots',
'motsetje':'motsetj',
'motsetnad':'motsetnad',
'motsetning':'motsetning',
'motsetningar':'motsetning',
'motsetninger':'motsetning',
'motsetningsforhold':'motsetningsforhold',
'motsetningsforholdet':'motsetningsforhold',
'motsett':'motsett',
'motsette':'motsett',
'motsi':'motsi',
'motstand':'motstand',
'motstanderen':'motstander',
'motstrid':'motstrid',
'motstridande':'motstrid',
'motstrøms':'motstrøm',
'motstykke':'motstykk',
'motstykket':'motstykk',
'motståande':'motstå',
'motsvarer':'motsvar',
'motsvares':'motsvar',
'motta':'mott',
'mottak':'mottak',
'mottakar':'mottak',
'mottakaren':'mottakar',
'mottaker':'mottak',
'mottakere':'mottaker',
'mottakerer':'mottaker',
'mottakerfylke':'mottakerfylk',
'mottakersted':'mottakersted',
'mottaksanlegg':'mottaksanlegg',
'mottaksfylke':'mottaksfylk',
'mottaksordninger':'mottaksordning',
'mottakssted':'mottakssted',
'mottar':'mott',
'mottatt':'mottatt',
'motteke':'mottek',
'motteken':'mottek',
'mottekne':'mottekn',
'mottiltak':'mottiltak',
'mottok':'mottok',
'motvekt':'motvekt',
'motverka':'motverk',
'motverke':'motverk',
'motvilje':'motvilj',
'motvirke':'motvirk',
'motvirkes':'motvirk',
'moum':'moum',
'mounted':'mounted',
'mrd':'mrd',
'mudret':'mudr',
'muldvarp':'muldvarp',
'mulig':'mul',
'mulige':'mul',
'muligens':'mul',
'muliggjør':'muliggjør',
'mulighet':'mul',
'muligheten':'mul',
'mulighetene':'mul',
'muligheter':'mul',
'mulighetsområde':'mulighetsområd',
'mulighetsområdet':'mulighetsområd',
'multipliser':'multiplis',
'multipliseres':'multipliser',
'mumlende':'muml',
'mumler':'muml',
'mumlig':'mum',
'munck':'munck',
'mundilfare':'mundilfar',
'munin':'munin',
'munn':'munn',
'munnen':'munn',
'munnet':'munn',
'munnfull':'munnfull',
'munnhell':'munnhell',
'munnvikene':'munnvik',
'muntre':'muntr',
'mur':'mur',
'muren':'mur',
'murene':'mur',
'murer':'mur',
'musejakt':'musejakt',
'musikk':'musikk',
'musklene':'muskl',
'muskler':'muskl',
'muspellsheim':'muspellsheim',
'muspellshæren':'muspellshær',
'muspellsønnene':'muspellsønn',
'mutt':'mutt',
'mutue':'mutu',
'mv':'mv',
'mye':'mye',
'mygg':'mygg',
'myke':'myk',
'mykere':'myker',
'mykje':'mykj',
'mykt':'mykt',
'mynde':'mynd',
'myndet':'mynd',
'myndige':'mynd',
'myndighet':'mynd',
'myndigheten':'mynd',
'myndighetene':'mynd',
'myndighetenes':'mynd',
'myndigheter':'mynd',
'myndigheters':'myndighet',
'myndighetsområde':'myndighetsområd',
'myndighetsutøvelse':'myndighetsutøv',
'myndighetsutøvelsen':'myndighetsutøv',
'myr':'myr',
'myrhull':'myrhull',
'myser':'mys',
'mysing':'mysing',
'må':'må',
'måkeskrik':'måkeskrik',
'mål':'mål',
'måla':'mål',
'målast':'mål',
'målbevisst':'målbevisst',
'måle':'mål',
'målene':'mål',
'måler':'mål',
'målet':'mål',
'måling':'måling',
'målkonflikter':'målkonflikt',
'målrettet':'målrett',
'målsatt':'målsatt',
'målsatte':'målsatt',
'målsetning':'målsetning',
'målsetningen':'målsetning',
'målsetninger':'målsetning',
'målsetting':'målsetting',
'målsettingen':'målsetting',
'målsettinger':'målsetting',
'målsikkerhet':'målsikker',
'målstørrelse':'målstørr',
'målt':'målt',
'måltid':'måltid',
'måltidet':'måltid',
'månad':'månad',
'månaden':'månad',
'månadene':'månad',
'månader':'månad',
'månaders':'månad',
'månadleg':'månad',
'månadlege':'månad',
'månadsvis':'månadsvis',
'måne':'mån',
'måned':'måned',
'månedene':'måned',
'måneder':'måned',
'månedlige':'måned',
'måneløs':'måneløs',
'månen':'mån',
'måper':'måp',
'måtar':'måt',
'måte':'måt',
'måtelig':'måt',
'måten':'måt',
'måtene':'måt',
'måter':'måt',
'måtta':'mått',
'måtte':'mått',
'måttet':'mått',
'mæland':'mæland',
'mæle':'mæl',
'møblar':'møbl',
'mødre':'mødr',
'mølen':'møl',
'mønet':'møn',
'mønster':'mønst',
'mønsteret':'mønster',
'mønsterkontrakten':'mønsterkontrakt',
'mønstervedtektene':'mønstervedtekt',
'mørbanket':'mørbank',
'møre':'mør',
'mørk':'mørk',
'mørke':'mørk',
'mørkemaktene':'mørkemakt',
'mørket':'mørk',
'mørketiden':'mørketid',
'mørkets':'mørk',
'mørkt':'mørkt',
'mørkved':'mørkved',
'møta':'møt',
'møtast':'møt',
'møte':'møt',
'møtedagen':'møtedag',
'møteleiar':'møtelei',
'møteleiaren':'møteleiar',
'møteleiarens':'møteleiar',
'møteleiing':'møteleiing',
'møtene':'møt',
'møteplikt':'møteplikt',
'møter':'møt',
'møtereglar':'møteregl',
'møterett':'møterett',
'møteretten':'møterett',
'møtes':'møt',
'møtesituasjoner':'møtesituasjon',
'møtest':'møtest',
'møtet':'møt',
'møtevirksomhet':'møtevirksom',
'møtt':'møtt',
'møtte':'møtt',
'møy':'møy',
'n':'n',
'nabokommune':'nabokommun',
'naboland':'naboland',
'nag':'nag',
'naglfar':'naglf',
'naken':'nak',
'nakent':'nakent',
'nakken':'nakk',
'nakne':'nakn',
'namn':'namn',
'namna':'namn',
'namneendring':'namneendring',
'namneendringa':'namneendring',
'namnet':'namn',
'namngjeven':'namngjev',
'namngjevne':'namngjevn',
'namsretten':'namsrett',
'namsstyresmakta':'namsstyresmakt',
'namsstyresmaktene':'namsstyresmakt',
'nanna':'nann',
'nappe':'napp',
'napper':'napp',
'nara':'nar',
'narr':'narr',
'narra':'narr',
'narre':'narr',
'narret':'narr',
'narve':'narv',
'narvesen':'narves',
'narvik':'narvik',
'nasjonal':'nasjonal',
'nasjonalbudsjett':'nasjonalbudsjett',
'nasjonalbudsjettet':'nasjonalbudsjett',
'nasjonale':'nasjonal',
'nasjonalitet':'nasjonalit',
'nasjonalt':'nasjonalt',
'nasjonene':'nasjon',
'natt':'natt',
'natta':'natt',
'nattefrosten':'nattefrost',
'nattely':'nattely',
'nattemørket':'nattemørk',
'natten':'natt',
'nattens':'natt',
'nattestengning':'nattestengning',
'nattsvart':'nattsvart',
'natur':'natur',
'naturaldeling':'naturaldeling',
'naturalia':'naturali',
'naturalytelser':'naturalyt',
'naturgitte':'naturgitt',
'naturinngrep':'naturinngrep',
'naturlig':'natur',
'naturlige':'natur',
'naturligvis':'naturligvis',
'naturmessige':'naturmess',
'naturreservat':'naturreservat',
'naturreservatet':'naturreservat',
'naturressurser':'naturressurs',
'naudssituasjonar':'naudssituasjon',
'nautiske':'nautisk',
'nav':'nav',
'navet':'nav',
'navigasjon':'navigasjon',
'navigasjonsassistanse':'navigasjonsassistans',
'navigasjonsdekning':'navigasjonsdekning',
'navigasjonshjelpemidler':'navigasjonshjelpemidl',
'navigasjonspolitikk':'navigasjonspolitikk',
'navigasjonsspørsmål':'navigasjonsspørsmål',
'navigatørene':'navigatør',
'navigere':'naviger',
'navn':'navn',
'navnene':'navn',
'navnet':'navn',
'navprinsippet':'navprinsipp',
'nazionale':'nazional',
'nbbl':'nbbl',
'nbbls':'nbbls',
'nbo':'nbo',
'nebb':'nebb',
'nebbene':'nebb',
'nebbet':'nebb',
'ned':'ned',
'nedafor':'nedafor',
'nedbetaling':'nedbetaling',
'nedbetalt':'nedbetalt',
'nedbetalte':'nedbetalt',
'nedbrente':'nedbrent',
'nedbrytning':'nedbrytning',
'nedbygge':'nedbygg',
'nedbygging':'nedbygging',
'nede':'ned',
'nedenfor':'nedenfor',
'nedenfra':'nedenfr',
'nedenstående':'nedenstå',
'nedereland':'nedereland',
'nederlaget':'nederlag',
'nederland':'nederland',
'nederste':'nederst',
'nedfelle':'nedfell',
'nedfelles':'nedfell',
'nedfelt':'nedfelt',
'nedjustert':'nedjuster',
'nedlagd':'nedlagd',
'nedlagt':'nedlagt',
'nedlegge':'nedlegg',
'nedleggelse':'nedlegg',
'nedlegges':'nedlegg',
'nedlegging':'nedlegging',
'nedover':'nedov',
'nedprioritere':'nedprioriter',
'nedpå':'nedpå',
'nedre':'nedr',
'nedsatt':'nedsatt',
'nedsatte':'nedsatt',
'nedsetjing':'nedsetjing',
'nedsette':'nedsett',
'nedsettelse':'nedsett',
'nedskrivninger':'nedskrivning',
'nedsnedd':'nedsnedd',
'nedstigande':'nedst',
'negativ':'negativ',
'negative':'negativ',
'negativt':'negativ',
'neglene':'negl',
'negler':'negl',
'nei':'nei',
'neida':'neid',
'neie':'neie',
'neier':'neier',
'nekta':'nekt',
'nektar':'nekt',
'nektast':'nekt',
'nekte':'nekt',
'nekter':'nekt',
'nektet':'nekt',
'nekting':'nekting',
'nektinga':'nekting',
'nels':'nel',
'nemleg':'nem',
'nemlig':'nem',
'nemnande':'nemn',
'nemnast':'nemn',
'nemnd':'nemnd',
'nemnda':'nemnd',
'nemnde':'nemnd',
'nemndene':'nemnd',
'nemnder':'nemnd',
'nemne':'nemn',
'nemner':'nemn',
'nemning':'nemning',
'nemninga':'nemning',
'nemningane':'nemning',
'nemningar':'nemning',
'nemnt':'nemnt',
'nemo':'nemo',
'neppe':'nepp',
'nepsdatter':'nepsdatt',
'neseborene':'nesebor',
'nesen':'nes',
'nest':'nest',
'neste':'nest',
'nesten':'nest',
'nestenulykker':'nestenulykk',
'nestleder':'nestled',
'nestledervervet':'nestlederverv',
'nestleiar':'nestlei',
'nett':'nett',
'netter':'nett',
'nettet':'nett',
'netto':'netto',
'nettoeiga':'netto',
'nettoen':'netto',
'nettoformue':'nettoformu',
'nettopp':'nettopp',
'nettopremier':'nettopremi',
'nettoverdiane':'nettoverdi',
'nettoverdien':'nettoverdi',
'nettoverdier':'nettoverdi',
'nettsider':'nettsid',
'nettverk':'nettverk',
'network':'network',
'neve':'nev',
'nevekamp':'nevekamp',
'nevekjemper':'nevekjemp',
'neven':'nev',
'nevene':'nev',
'never':'nev',
'nevn':'nevn',
'nevne':'nevn',
'nevner':'nevn',
'nevnes':'nevn',
'nevneverdige':'nevneverd',
'nevnt':'nevnt',
'nevnte':'nevnt',
'new':'new',
'nf':'nf',
'nff':'nff',
'nho':'nho',
'ni':'ni',
'niande':'niand',
'nibr':'nibr',
'nibrs':'nibr',
'nicht':'nicht',
'nidhogg':'nidhogg',
'niende':'niend',
'niflheim':'niflheim',
'nikker':'nikk',
'nils':'nil',
'nina':'nin',
'nisjer':'nisj',
'nistemat':'nistemat',
'nisten':'nist',
'nistepose':'nistepos',
'nisteposen':'nistepos',
'nisteskreppa':'nisteskrepp',
'nitberg':'nitberg',
'nitbergs':'nitberg',
'nivå':'nivå',
'nivåene':'nivå',
'nivåer':'nivå',
'nivået':'nivå',
'njord':'njord',
'njords':'njord',
'nkl':'nkl',
'no':'no',
'noahs':'noah',
'noatun':'noatun',
'noe':'noe',
'noen':'noen',
'noenlunde':'noenlund',
'nogjeldande':'nogjeld',
'nok':'nok',
'noka':'nok',
'noko':'noko',
'nokolunde':'nokolund',
'nokon':'nokon',
'nokre':'nokr',
'nokså':'nokså',
'nomenklaturen':'nomenklatur',
'nominell':'nominell',
'nominelle':'nominell',
'nominelt':'nominelt',
'nor':'nor',
'norcem':'norcem',
'norconsult':'norconsult',
'nord':'nord',
'nordeuropeisk':'nordeuropeisk',
'nordisk':'nordisk',
'nordiska':'nordisk',
'nordiske':'nordisk',
'nordlige':'nord',
'nordover':'nordov',
'nordre':'nordr',
'nordsjøen':'nordsjø',
'nordsjøtrafikken':'nordsjøtrafikk',
'nordøst':'nordøst',
'noreg':'noreg',
'noregs':'noreg',
'norge':'norg',
'norges':'norg',
'norgeslast':'norgesl',
'norm':'norm',
'normal':'normal',
'normale':'normal',
'normalordninga':'normalordning',
'normalt':'normalt',
'normalvedtektene':'normalvedtekt',
'normann':'normann',
'normer':'norm',
'normert':'normer',
'nornen':'norn',
'nornene':'norn',
'norner':'norn',
'norsk':'norsk',
'norske':'norsk',
'noss':'noss',
'notat':'notat',
'notatet':'notat',
'note':'not',
'noter':'not',
'notering':'notering',
'noteringa':'notering',
'notert':'noter',
'noterte':'noter',
'notifikasjonskravet':'notifikasjonskrav',
'nou':'nou',
'november':'novemb',
'noverande':'nover',
'nr':'nr',
'nsb':'nsb',
'nsbs':'nsbs',
'ntp':'ntp',
'null':'null',
'numedalslågen':'numedalslåg',
'nummer':'numm',
'nummeret':'nummer',
'nummerordningar':'nummerordning',
'nustad':'nustad',
'nvvp':'nvvp',
'ny':'ny',
'nyanlegget':'nyanlegg',
'nyansatte':'nyansatt',
'nyansert':'nyanser',
'nyanserte':'nyanser',
'nyare':'nyar',
'nyaste':'nyast',
'nybrottsmenn':'nybrottsmenn',
'nybyen':'nyby',
'nybygg':'nybygg',
'nybygget':'nybygg',
'nybygging':'nybygging',
'nybygginga':'nybygging',
'nydelig':'nyd',
'nye':'nye',
'nyere':'nyer',
'nyetablering':'nyetablering',
'nyetablerte':'nyetabler',
'nyfødte':'nyfød',
'nyinnmelde':'nyinnmeld',
'nykommerne':'nykommern',
'nyleg':'nyleg',
'nylig':'nyl',
'nymalt':'nymalt',
'nynnende':'nynn',
'nynner':'nynn',
'nyoppførte':'nyoppført',
'nysgjerrig':'nysgjerr',
'nysgjerrige':'nysgjerr',
'nysgjerrighet':'nysgjerr',
'nyskapning':'nyskapning',
'nyskapninger':'nyskapning',
'nyskapt':'nyskapt',
'nystifta':'nystift',
'nyte':'nyt',
'nyter':'nyt',
'nytt':'nytt',
'nytta':'nytt',
'nyttar':'nytt',
'nyttast':'nytt',
'nytte':'nytt',
'nytteeffekter':'nytteeffekt',
'nyttekostnadsanalyse':'nyttekostnadsanalys',
'nyttekostnadsanalyser':'nyttekostnadsanalys',
'nytten':'nytt',
'nytter':'nytt',
'nyttes':'nytt',
'nyttet':'nytt',
'nyttetrafikken':'nyttetrafikk',
'nyttig':'nytt',
'nyttiggjøre':'nyttiggjør',
'nyval':'nyval',
'nå':'nå',
'nådd':'nådd',
'nåde':'nåd',
'nådeløs':'nådeløs',
'någjeldende':'någjeld',
'nål':'nål',
'når':'når',
'nås':'nås',
'nåstrand':'nåstrand',
'nåtid':'nåtid',
'nått':'nått',
'nåverdi':'nåverdi',
'nåverdien':'nåverdi',
'nåværende':'nåvær',
'nær':'nær',
'nærare':'nærar',
'nærast':'nær',
'næraste':'nærast',
'nære':'nær',
'nærhet':'nær',
'nærheten':'nær',
'næring':'næring',
'næringer':'næring',
'nærings':'næring',
'næringsaktivitet':'næringsaktivit',
'næringsaktiviteter':'næringsaktivitet',
'næringsdrivande':'næringsdriv',
'næringsforhold':'næringsforhold',
'næringsformål':'næringsformål',
'næringsinteresser':'næringsinteress',
'næringsliv':'næringsliv',
'næringslivet':'næringsliv',
'næringslivets':'næringsliv',
'næringslivsaktivitet':'næringslivsaktivit',
'næringslivspolitikk':'næringslivspolitikk',
'næringslivsutvikling':'næringslivsutvikling',
'næringsplan':'næringsplan',
'næringspolitikk':'næringspolitikk',
'næringspolitikken':'næringspolitikk',
'næringspolitikkens':'næringspolitikk',
'næringspolitisk':'næringspolitisk',
'næringspolitiske':'næringspolitisk',
'næringsrettede':'næringsrett',
'næringsråd':'næringsråd',
'næringssameiger':'næringssam',
'næringsseksjonar':'næringsseksjon',
'næringsutvikling':'næringsutvikling',
'næringsutviklingen':'næringsutvikling',
'næringsutøvere':'næringsutøver',
'næringsverksemd':'næringsverksemd',
'næringsvirksomhet':'næringsvirksom',
'næringsvirksomheten':'næringsvirksom',
'næringsvirksomheter':'næringsvirksom',
'næringsøkonomi':'næringsøkonomi',
'næringsøkonomiske':'næringsøkonomisk',
'nærliggende':'nærligg',
'nærliggjande':'nærliggj',
'nærmar':'nærm',
'nærme':'nærm',
'nærmer':'nærm',
'nærmere':'nærmer',
'nærmest':'nærmest',
'nærmeste':'nærmest',
'nærmiljøet':'nærmiljø',
'nærområde':'nærområd',
'nærområdene':'nærområd',
'nærområder':'nærområd',
'nærskipsfarten':'nærskipsfart',
'nærståande':'nærstå',
'nærstående':'nærstå',
'nært':'nært',
'nærtliggende':'nærtligg',
'nærtrafikk':'nærtrafikk',
'nærvere':'nærver',
'nød':'nød',
'nødig':'nød',
'nødt':'nødt',
'nødvendig':'nødvend',
'nødvendige':'nødvend',
'nødvendigheten':'nødvend',
'nødvendigvis':'nødvendigvis',
'nøgde':'nøgd',
'nøkk':'nøkk',
'nøkkel':'nøkkel',
'nøkkelen':'nøkkel',
'nøkkelknippe':'nøkkelknipp',
'nøkkelroller':'nøkkelroll',
'nøktern':'nøktern',
'nøkterne':'nøktern',
'nøle':'nøl',
'nøler':'nøl',
'nøtt':'nøtt',
'nøtteskall':'nøtteskall',
'nøyaktig':'nøyakt',
'nøyd':'nøyd',
'nøye':'nøye',
'nøyer':'nøyer',
'nøyere':'nøyer',
'nøytral':'nøytral',
'nøytrale':'nøytral',
'nøytralitet':'nøytralit',
'nøytralt':'nøytralt',
'o':'o',
'objekta':'objekt',
'objektet':'objekt',
'obligasjonar':'obligasjon',
'obligasjonen':'obligasjon',
'obligasjonens':'obligasjon',
'obligasjoner':'obligasjon',
'obligasjonsinnskot':'obligasjonsinnskot',
'obligasjonsleilegheiter':'obligasjonsleilegheit',
'obligasjonsleilighet':'obligasjonslei',
'obligatorisk':'obligatorisk',
'obligatoriske':'obligatorisk',
'obos':'obo',
'observasjon':'observasjon',
'observatør':'observatør',
'observatørar':'observatør',
'observatøren':'observatør',
'observert':'observer',
'och':'och',
'od':'od',
'odder':'odd',
'odelstinget':'odelsting',
'oder':'oder',
'odin':'odin',
'odins':'odin',
'oecd':'oecd',
'of':'of',
'off':'off',
'offentleg':'offent',
'offentlege':'offent',
'offentlegrettslege':'offentlegretts',
'offentlig':'offent',
'offentlige':'offent',
'offentliges':'offent',
'offentligheten':'offent',
'offerfesten':'offerfest',
'offisiell':'offisiell',
'ofres':'ofr',
'oftare':'oftar',
'oftast':'oft',
'ofte':'oft',
'oftere':'ofter',
'oftest':'oftest',
'og':'og',
'også':'også',
'ogvalfader':'ogvalfad',
'ohnstad':'ohnstad',
'ohv':'ohv',
'ohvs':'ohv',
'oioioi':'oioioi',
'okolne':'okoln',
'okse':'oks',
'oksehodet':'oksehod',
'okselår':'okselår',
'oksene':'oks',
'okser':'oks',
'oktober':'oktob',
'ol':'ol',
'ola':'ola',
'olav':'olav',
'oldebarn':'oldebarn',
'oldemor':'oldemor',
'oldinger':'olding',
'olje':'olj',
'oljehavn':'oljehavn',
'oljehavnen':'oljehavn',
'oljeprodukter':'oljeprodukt',
'oljeraffineriet':'oljeraffineri',
'olmt':'olmt',
'olsen':'ols',
'om':'om',
'ombestemmer':'ombestemm',
'ombodsordning':'ombodsordning',
'ombord':'ombord',
'ombygging':'ombygging',
'ombygginga':'ombygging',
'ombyting':'ombyting',
'ombyttingsbetraktning':'ombyttingsbetraktning',
'omdannast':'omdann',
'omdanne':'omdann',
'omdannelse':'omdann',
'omdannes':'omdann',
'omdanning':'omdanning',
'omdanninga':'omdanning',
'omdanningen':'omdanning',
'omdanninger':'omdanning',
'omdanningsreglane':'omdanningsregl',
'omdanningstidspunktet':'omdanningstidspunkt',
'omdanningsvedtaket':'omdanningsvedtak',
'omdeling':'omdeling',
'omdiskutert':'omdiskuter',
'omdisponeres':'omdisponer',
'omdisponering':'omdisponering',
'omegn':'omegn',
'omfang':'omfang',
'omfanget':'omfang',
'omfatta':'omfatt',
'omfattande':'omfatt',
'omfattar':'omfatt',
'omfattast':'omfatt',
'omfatte':'omfatt',
'omfattende':'omfatt',
'omfatter':'omfatt',
'omfattes':'omfatt',
'omfattet':'omfatt',
'omfavne':'omfavn',
'omfavner':'omfavn',
'omfavnet':'omfavn',
'omfordelende':'omfordel',
'omfram':'omfram',
'omgang':'omgang',
'omgitt':'omgitt',
'omgivelsene':'omgiv',
'omgjering':'omgjering',
'omgjeringa':'omgjering',
'omgjort':'omgjort',
'omgjøre':'omgjør',
'omgrepa':'omgrep',
'omgrepet':'omgrep',
'omgåelse':'omgå',
'omgåing':'omgåing',
'omgåingar':'omgåing',
'omhandler':'omhandl',
'omhandles':'omhandl',
'omhandlet':'omhandl',
'omkjøring':'omkjøring',
'omkostningene':'omkostning',
'omkostninger':'omkostning',
'omkranser':'omkrans',
'omkring':'omkring',
'omkringliggende':'omkringligg',
'omlag':'omlag',
'omland':'omland',
'omlastet':'omlast',
'omlasting':'omlasting',
'omlastingen':'omlasting',
'omlastingsarealer':'omlastingsareal',
'omlastingsområder':'omlastingsområd',
'omlastingspotensialet':'omlastingspotensial',
'omlastingsterminal':'omlastingsterminal',
'omlastingsterminaler':'omlastingsterminal',
'omlastingsvekst':'omlastingsvekst',
'omlastingsvolumer':'omlastingsvolum',
'omlastningen':'omlastning',
'omlastninger':'omlastning',
'omlastningsforhold':'omlastningsforhold',
'omlastningshavn':'omlastningshavn',
'omlastningsterminaler':'omlastningsterminal',
'omlastningsvolum':'omlastningsvolum',
'omlegging':'omlegging',
'omleggingar':'omlegging',
'omlegginger':'omlegging',
'omliggende':'omligg',
'omlokalisering':'omlokalisering',
'omløpshastighet':'omløpshast',
'omme':'omm',
'omorganisere':'omorganiser',
'omorganiseres':'omorganiser',
'omorganisert':'omorganiser',
'omprioritering':'omprioritering',
'omprioriteringer':'omprioritering',
'omregnet':'omregn',
'omregning':'omregning',
'områ':'områ',
'område':'områd',
'områdene':'områd',
'områder':'områd',
'områders':'områd',
'området':'områd',
'områdets':'områd',
'områdingstid':'områdingstid',
'omsetjast':'omsetj',
'omsetje':'omsetj',
'omsetjeleg':'omsetj',
'omsetjelege':'omsetj',
'omsetjing':'omsetjing',
'omsetjinga':'omsetjing',
'omsetning':'omsetning',
'omsetninga':'omsetning',
'omsetningservervarar':'omsetningservervar',
'omsetningsregulering':'omsetningsregulering',
'omsetningsverdien':'omsetningsverdi',
'omsette':'omsett',
'omsettes':'omsett',
'omskiftninger':'omskiftning',
'omskipa':'omskip',
'omskiping':'omskiping',
'omslaget':'omslag',
'omsorg':'omsorg',
'omsorgsbehov':'omsorgsbehov',
'omsorgsbustader':'omsorgsbustad',
'omsorgstenester':'omsorgstenest',
'omstenda':'omstend',
'omstende':'omst',
'omstendighet':'omstend',
'omstillinger':'omstilling',
'omstillings':'omstilling',
'omstillingsdyktig':'omstillingsdykt',
'omstillingstiltak':'omstillingstiltak',
'omstillingstiltaka':'omstillingstiltak',
'omstreifer':'omstreif',
'omstridd':'omstridd',
'omstridde':'omstridd',
'omstridt':'omstrid',
'omstrukturering':'omstrukturering',
'omstruktureringer':'omstrukturering',
'omstøytast':'omstøyt',
'omstøyting':'omstøyting',
'omstøytingskravet':'omstøytingskrav',
'omstøytingsregelen':'omstøytingsregel',
'omsyn':'omsyn',
'omsyna':'omsyn',
'omsynet':'omsyn',
'omsynstaking':'omsynstaking',
'omtala':'omtal',
'omtalast':'omtal',
'omtale':'omtal',
'omtalen':'omtal',
'omtaler':'omtal',
'omtales':'omtal',
'omtalt':'omtalt',
'omtrent':'omtrent',
'omtvista':'omtvist',
'omvandrende':'omvandr',
'omveg':'omveg',
'omvei':'omvei',
'omvende':'omv',
'omvendt':'omvend',
'omverda':'omverd',
'on':'on',
'ond':'ond',
'onde':'ond',
'ondskap':'ondskap',
'ondskapsfull':'ondskapsfull',
'ondskapsfulle':'ondskapsfull',
'ondskapsfullt':'ondskapsfullt',
'ondt':'ondt',
'ondurdis':'ondurdis',
'onkelen':'onkel',
'ope':'ope',
'open':'open',
'openberre':'openberr',
'openbert':'openber',
'openheit':'openheit',
'operasjon':'operasjon',
'operativ':'operativ',
'operative':'operativ',
'operativt':'operativ',
'operatør':'operatør',
'operatøren':'operatør',
'operatørene':'operatør',
'operatører':'operatør',
'operatørhold':'operatørhold',
'operatørselskaper':'operatørselskap',
'operatørvirksomhet':'operatørvirksom',
'operer':'oper',
'operere':'operer',
'opererer':'operer',
'opereres':'operer',
'opinionsleiarar':'opinionsleiar',
'opna':'opn',
'opnar':'opn',
'opnare':'opnar',
'opnast':'opn',
'opne':'opn',
'opning':'opning',
'opninga':'opning',
'opningsbalanse':'opningsbalans',
'opningsbalansen':'opningsbalans',
'opp':'opp',
'oppad':'oppad',
'opparbeide':'opparbeid',
'opparbeidede':'opparbeid',
'opparbeidelse':'opparbeid',
'opparbeider':'opparbeid',
'opparbeides':'opparbeid',
'opparbeidet':'opparbeid',
'opparbeiding':'opparbeiding',
'oppattbygging':'oppattbygging',
'oppbevarer':'oppbevar',
'oppbevaring':'oppbevaring',
'oppblåst':'oppblåst',
'oppblåste':'oppblåst',
'oppbrente':'oppbrent',
'oppbygd':'oppbygd',
'oppbygde':'oppbygd',
'oppbygget':'oppbygg',
'oppbygging':'oppbygging',
'oppbygginga':'oppbygging',
'oppbyggingen':'oppbygging',
'oppdage':'oppdag',
'oppdager':'oppdag',
'oppdaterte':'oppdater',
'oppdeling':'oppdeling',
'oppdelingen':'oppdeling',
'oppdelt':'oppdelt',
'oppdrag':'oppdrag',
'oppdraget':'oppdrag',
'oppdragsavtale':'oppdragsavtal',
'oppdragsgivere':'oppdragsgiver',
'oppdragstakaren':'oppdragstakar',
'oppe':'opp',
'oppebærer':'oppebær',
'oppfarende':'oppfar',
'oppfatning':'oppfatning',
'oppfatninga':'oppfatning',
'oppfatningane':'oppfatning',
'oppfatningar':'oppfatning',
'oppfatningene':'oppfatning',
'oppfatninger':'oppfatning',
'oppfatta':'oppfatt',
'oppfattar':'oppfatt',
'oppfattast':'oppfatt',
'oppfatte':'oppfatt',
'oppfatter':'oppfatt',
'oppfattes':'oppfatt',
'oppfordre':'oppfordr',
'oppfylle':'oppfyll',
'oppfyller':'oppfyll',
'oppfylles':'oppfyll',
'oppfylling':'oppfylling',
'oppfyllinga':'oppfylling',
'oppfylt':'oppfylt',
'oppfylte':'oppfylt',
'oppfølging':'oppfølging',
'oppfølginga':'oppfølging',
'oppfølgning':'oppfølgning',
'oppfører':'oppfør',
'oppføring':'oppføring',
'oppføringa':'oppføring',
'oppført':'oppført',
'oppførte':'oppført',
'oppgangar':'oppgang',
'oppgave':'oppgav',
'oppgaven':'oppgav',
'oppgavene':'oppgav',
'oppgaveplikten':'oppgaveplikt',
'oppgaver':'oppgav',
'oppgir':'oppgir',
'oppgis':'oppgis',
'oppgitt':'oppgitt',
'oppgjer':'oppgj',
'oppgjeret':'oppgjer',
'oppgjevast':'oppgjev',
'oppgjeve':'oppgjev',
'oppgjeven':'oppgjev',
'oppgjevne':'oppgjevn',
'oppgjør':'oppgjør',
'oppgjøret':'oppgjør',
'oppgjørstidspunktet':'oppgjørstidspunkt',
'oppgradering':'oppgradering',
'oppgåva':'oppgåv',
'oppgåve':'oppgåv',
'oppgåvefordelinga':'oppgåvefordeling',
'oppgåvene':'oppgåv',
'oppgåver':'oppgåv',
'opphald':'opphald',
'opphav':'opphav',
'opphavet':'opphav',
'opphavleg':'opphav',
'opphavlege':'opphav',
'opphavssted':'opphavssted',
'oppheva':'opphev',
'oppheve':'opphev',
'oppheves':'opphev',
'opphevet':'opphev',
'opphisset':'opphiss',
'oppholder':'opphold',
'opphør':'opphør',
'opphøre':'opphør',
'opphørende':'opphør',
'opphører':'opphør',
'opphøret':'opphør',
'opphørsgrunner':'opphørsgrunn',
'opphørstidspunktet':'opphørstidspunkt',
'opphørsåret':'opphørsår',
'opphørt':'opphørt',
'oppjusteres':'oppjuster',
'oppkalt':'oppkalt',
'oppkavet':'oppkav',
'oppkjøp':'oppkjøp',
'oppkrevd':'oppkrevd',
'oppkreve':'oppkrev',
'oppkreves':'oppkrev',
'opplagring':'opplagring',
'opplagt':'opplagt',
'oppland':'oppland',
'opplandet':'oppland',
'opplegg':'opplegg',
'opplegget':'opplegg',
'opplevd':'opplevd',
'oppleve':'opplev',
'opplever':'opplev',
'oppleves':'opplev',
'opplistingen':'opplisting',
'opplysande':'opplys',
'opplysast':'opplys',
'opplyse':'opplys',
'opplyser':'opplys',
'opplyses':'opplys',
'opplysning':'opplysning',
'opplysninga':'opplysning',
'opplysningane':'opplysning',
'opplysningar':'opplysning',
'opplysningene':'opplysning',
'opplysninger':'opplysning',
'opplysningsplikt':'opplysningsplikt',
'opplysningsplikta':'opplysningsplikt',
'opplysningsverksemd':'opplysningsverksemd',
'opplyst':'opplyst',
'opplåning':'opplåning',
'opplæring':'opplæring',
'oppløses':'oppløs',
'oppløsning':'oppløsning',
'oppløsningsvedtaket':'oppløsningsvedtak',
'oppløysast':'oppløys',
'oppløyse':'oppløys',
'oppløysing':'oppløysing',
'oppløysinga':'oppløysing',
'oppløysingar':'oppløysing',
'oppløysingsgrunnen':'oppløysingsgrunn',
'oppløysingsreglane':'oppløysingsregl',
'oppløysingssituasjon':'oppløysingssituasjon',
'oppløysingstilfella':'oppløysingstilfell',
'oppløysingsvedtaket':'oppløysingsvedtak',
'oppløyst':'oppløyst',
'oppløyste':'oppløyst',
'oppmerking':'oppmerking',
'oppmerkingen':'oppmerking',
'oppmerksom':'oppmerksom',
'oppmerksomhet':'oppmerksom',
'oppmode':'oppmod',
'oppmoding':'oppmoding',
'oppmodinga':'oppmoding',
'oppmuntres':'oppmuntr',
'oppnemnast':'oppnemn',
'oppnemnde':'oppnemnd',
'oppnemne':'oppnemn',
'oppnemner':'oppnemn',
'oppnemning':'oppnemning',
'oppnemningsrett':'oppnemningsrett',
'oppnemnt':'oppnemnt',
'oppnevne':'oppnevn',
'oppnevner':'oppnevn',
'oppnevnes':'oppnevn',
'oppnevning':'oppnevning',
'oppnevnt':'oppnevnt',
'oppnå':'oppnå',
'oppnådd':'oppnådd',
'oppnådde':'oppnådd',
'oppnår':'oppnår',
'oppnås':'oppnås',
'oppover':'oppov',
'oppregnet':'oppregn',
'oppregning':'oppregning',
'oppregningen':'oppregning',
'oppregulering':'oppregulering',
'oppreising':'oppreising',
'oppreist':'oppreist',
'opprekning':'opprekning',
'opprekninga':'opprekning',
'opprett':'opprett',
'oppretta':'opprett',
'opprettast':'opprett',
'opprette':'opprett',
'opprettelse':'opprett',
'opprettelsen':'opprett',
'oppretter':'opprett',
'opprettes':'opprett',
'opprettet':'opprett',
'opprettholde':'oppretthold',
'opprettholdelse':'oppretthold',
'opprettholder':'oppretthold',
'opprettholdes':'oppretthold',
'opprettholdt':'oppretthold',
'oppretting':'oppretting',
'opprinnelig':'opprinn',
'opprinnelige':'opprinn',
'opprinnelse':'opprinn',
'opprinnelsen':'opprinn',
'opprinnelses':'opprinn',
'opprinnelsessted':'opprinnelsessted',
'opprinnelsessteder':'opprinnelsessted',
'opprusting':'opprusting',
'opprustning':'opprustning',
'opprydding':'opprydding',
'opprykksrett':'opprykksrett',
'opprør':'opprør',
'oppsamla':'oppsaml',
'oppsamlede':'oppsaml',
'oppsamlet':'oppsaml',
'oppsamlingsfond':'oppsamlingsfond',
'oppsatt':'oppsatt',
'oppseielege':'oppsei',
'oppseiing':'oppseiing',
'oppseiinga':'oppseiing',
'oppseiingar':'oppseiing',
'oppseiingsfrist':'oppseiingsfrist',
'oppseiingsfristane':'oppseiingsfrist',
'oppseiingsfristen':'oppseiingsfrist',
'oppseiingsreglane':'oppseiingsregl',
'oppseiingsretten':'oppseiingsrett',
'oppseiingstid':'oppseiingstid',
'oppseiingsvern':'oppseiingsvern',
'oppsettelsestiden':'oppsettelsestid',
'oppsettet':'oppsett',
'oppsikt':'oppsikt',
'oppskrives':'oppskriv',
'oppskrivninger':'oppskrivning',
'oppskåret':'oppskår',
'oppslag':'oppslag',
'oppslutning':'oppslutning',
'oppslutninga':'oppslutning',
'oppsparing':'oppsparing',
'oppsparingen':'oppsparing',
'oppsparingsbehovet':'oppsparingsbehov',
'oppsparingselement':'oppsparingselement',
'oppsparingsfasen':'oppsparingsfas',
'oppsparingsperioden':'oppsparingsperiod',
'oppsparingstiden':'oppsparingstid',
'oppspart':'oppspart',
'oppsparte':'oppspart',
'oppspinn':'oppspinn',
'oppsplitting':'oppsplitting',
'oppstiging':'oppstiging',
'oppstigingsrett':'oppstigingsrett',
'oppstilles':'oppstill',
'oppstilling':'oppstilling',
'oppstillingsplaner':'oppstillingsplan',
'oppstilt':'oppstilt',
'oppsto':'oppsto',
'oppstod':'oppstod',
'oppstyret':'oppstyr',
'oppstå':'oppstå',
'oppstår':'oppstår',
'oppstått':'oppstått',
'oppsummerer':'oppsummer',
'oppsummering':'oppsummering',
'oppsummert':'oppsummer',
'oppsøke':'oppsøk',
'oppsøker':'oppsøk',
'oppta':'oppt',
'opptak':'opptak',
'opptakelse':'opptak',
'opptaksvilkårene':'opptaksvilkår',
'opptas':'oppt',
'opptatt':'opptatt',
'oppteikning':'oppteikning',
'oppteikninga':'oppteikning',
'opptil':'opptil',
'opptjene':'opptj',
'opptjener':'opptjen',
'opptjenes':'opptj',
'opptjening':'opptjening',
'opptjeningen':'opptjening',
'opptjeningsgrunnlag':'opptjeningsgrunnlag',
'opptjeningsmodell':'opptjeningsmodell',
'opptjeningsperioden':'opptjeningsperiod',
'opptjeningsprofil':'opptjeningsprofil',
'opptjeningsprofilen':'opptjeningsprofil',
'opptjeningssystem':'opptjeningssystem',
'opptjeningstid':'opptjeningstid',
'opptjeningstiden':'opptjeningstid',
'opptjeningstidspunktet':'opptjeningstidspunkt',
'opptjeningsår':'opptjeningsår',
'opptjent':'opptjent',
'opptjente':'opptjent',
'opptreden':'opptred',
'opptrer':'opptr',
'opptrinnsrett':'opptrinnsrett',
'oppussing':'oppussing',
'oppveier':'oppvei',
'oppå':'oppå',
'optimal':'optimal',
'optimale':'optimal',
'optimaliseres':'optimaliser',
'optimalisering':'optimalisering',
'optimalisert':'optimaliser',
'optimalt':'optimalt',
'ord':'ord',
'orda':'ord',
'ordbruken':'ordbruk',
'orde':'ord',
'orden':'ord',
'ordene':'ord',
'ordens':'ord',
'ordensreglar':'ordensregl',
'ordentlig':'ordent',
'ordet':'ord',
'ordføraren':'ordførar',
'ordinær':'ordinær',
'ordinære':'ordinær',
'ordinært':'ordinært',
'ordlegginga':'ordlegging',
'ordlyd':'ordlyd',
'ordlyden':'ordlyd',
'ordna':'ordn',
'ordnast':'ordn',
'ordne':'ordn',
'ordnet':'ordn',
'ordning':'ordning',
'ordninga':'ordning',
'ordningane':'ordning',
'ordningar':'ordning',
'ordningen':'ordning',
'ordningene':'ordning',
'ordningenes':'ordning',
'ordningens':'ordning',
'ordninger':'ordning',
'ordre':'ordr',
'ordren':'ordr',
'ordreplassering':'ordreplassering',
'ordreproduksjon':'ordreproduksjon',
'ordrer':'ordr',
'ordrestyres':'ordrestyr',
'ordrestyrt':'ordrestyrt',
'ordskiftet':'ordskift',
'ordspill':'ordspill',
'organ':'organ',
'organa':'organ',
'organer':'organ',
'organet':'organ',
'organisasjon':'organisasjon',
'organisasjonane':'organisasjon',
'organisasjonar':'organisasjon',
'organisasjonen':'organisasjon',
'organisasjoner':'organisasjon',
'organisasjonsarbeid':'organisasjonsarbeid',
'organisasjonsform':'organisasjonsform',
'organisasjonsforma':'organisasjonsform',
'organisasjonsformen':'organisasjonsform',
'organisasjonsmodell':'organisasjonsmodell',
'organisasjonsmodellen':'organisasjonsmodell',
'organisasjonsnummer':'organisasjonsnumm',
'organisation':'organisation',
'organisatorisk':'organisatorisk',
'organisatoriske':'organisatorisk',
'organiserast':'organiser',
'organisere':'organiser',
'organiserer':'organiser',
'organiseres':'organiser',
'organisering':'organisering',
'organiseringa':'organisering',
'organiseringen':'organisering',
'organiseringsmessige':'organiseringsmess',
'organiseringsmodell':'organiseringsmodell',
'organiseringsspørsmålet':'organiseringsspørsmål',
'organisert':'organiser',
'organiserte':'organiser',
'organismer':'organism',
'orientere':'orienter',
'orienterer':'orienter',
'orientering':'orientering',
'orientert':'orienter',
'orienterte':'orienter',
'originaler':'original',
'orke':'ork',
'orker':'ork',
'orket':'ork',
'orm':'orm',
'ormebol':'ormebol',
'ormen':'orm',
'ormens':'orm',
'ormer':'orm',
'ormsund':'ormsund',
'ormsundkaia':'ormsundkai',
'ormsundterminalen':'ormsundterminal',
'orsakar':'orsak',
'orskurd':'orskurd',
'orskurdar':'orskurd',
'orskurden':'orskurd',
'osb':'osb',
'oslo':'oslo',
'oslobehov':'oslobehov',
'oslofjord':'oslofjord',
'oslofjorden':'oslofjord',
'oslofjordens':'oslofjord',
'oslofjordforbindelsen':'oslofjordforbind',
'oslofjordhavnene':'oslofjordhavn',
'oslofjordhavnenes':'oslofjordhavn',
'oslofjordhavner':'oslofjordhavn',
'oslofjordhavneutvalget':'oslofjordhavneutvalg',
'oslofjordområdet':'oslofjordområd',
'oslofjordregionen':'oslofjordregion',
'oslogodset':'oslogods',
'oslomarkedet':'oslomarked',
'osloområdet':'osloområd',
'oslos':'oslo',
'oss':'oss',
'osv':'osv',
'ot':'ot',
'oter':'oter',
'oteren':'oter',
'oterskinnet':'oterskinn',
'out':'out',
'ovafor':'ovafor',
'ovenfor':'ovenfor',
'ovennevnte':'ovennevnt',
'oventil':'oventil',
'over':'over',
'overalt':'overalt',
'overblikk':'overblikk',
'overbygning':'overbygning',
'overbygningen':'overbygning',
'overdra':'overdr',
'overdragande':'overdrag',
'overdragast':'overdrag',
'overdragelse':'overdrag',
'overdragelsen':'overdrag',
'overdragende':'overdrag',
'overdraging':'overdraging',
'overdraginga':'overdraging',
'overdreg':'overdreg',
'overdrive':'overdriv',
'overende':'over',
'overens':'over',
'overensstemmelse':'overensstemm',
'overenstemmelse':'overenstemm',
'overfall':'overfall',
'overfaller':'overfall',
'overfalt':'overfalt',
'overflod':'overflod',
'overflyttes':'overflytt',
'overflødig':'overflød',
'overfor':'overfor',
'overførast':'overfør',
'overførbar':'overførb',
'overføre':'overfør',
'overfører':'overfør',
'overføres':'overfør',
'overføring':'overføring',
'overføringa':'overføring',
'overføringar':'overføring',
'overføringen':'overføring',
'overføringer':'overføring',
'overføringsadgang':'overføringsadgang',
'overføringsordningar':'overføringsordning',
'overføringsrett':'overføringsrett',
'overføringsretten':'overføringsrett',
'overført':'overført',
'overførte':'overført',
'overgang':'overgang',
'overgangen':'overgang',
'overganger':'overgang',
'overgangsbestemmelse':'overgangsbestemm',
'overgangsfasar':'overgangsfas',
'overgangsfase':'overgangsfas',
'overgangsordning':'overgangsordning',
'overgangsordninger':'overgangsordning',
'overgangsperiode':'overgangsperiod',
'overgangsperioden':'overgangsperiod',
'overgangsproblem':'overgangsproblem',
'overgangsregel':'overgangsregel',
'overgangsregelen':'overgangsregel',
'overgangsreglane':'overgangsregl',
'overgangsreglar':'overgangsregl',
'overgangsreglarikraftsetjing':'overgangsreglarikraftsetjing',
'overgangsregler':'overgangsregl',
'overgangstid':'overgangstid',
'overgangstida':'overgangstid',
'overgangstilfella':'overgangstilfell',
'overgir':'overgir',
'overgå':'overgå',
'overgår':'overgår',
'overhalden':'overhald',
'overhodet':'overhod',
'overholde':'overhold',
'overholdelse':'overhold',
'overholdes':'overhold',
'overholdt':'overhold',
'overkant':'overkant',
'overkapasitet':'overkapasit',
'overkjeven':'overkjev',
'overkomeleg':'overkom',
'overkroppen':'overkropp',
'overkurs':'overkur',
'overlastast':'overlast',
'overlatast':'overlat',
'overlate':'overlat',
'overlaten':'overlat',
'overlating':'overlating',
'overlatinga':'overlating',
'overlegninger':'overlegning',
'overlever':'overlev',
'overleverast':'overlever',
'overleveres':'overlever',
'overlevering':'overlevering',
'overleveringa':'overlevering',
'overmakten':'overmakt',
'overmodig':'overmod',
'overmotet':'overmot',
'overnatte':'overnatt',
'overnattet':'overnatt',
'overoppsyn':'overoppsyn',
'overorda':'overord',
'overordna':'overordn',
'overordnede':'overordn',
'overordnet':'overordn',
'overordnete':'overordnet',
'overprøving':'overprøving',
'overprøvinga':'overprøving',
'overraskelse':'overrask',
'overraskende':'overrask',
'overrasket':'overrask',
'oversendelsesforslag':'oversendelsesforslag',
'oversendes':'overs',
'oversendt':'oversend',
'overseten':'overset',
'oversetne':'oversetn',
'oversikt':'oversikt',
'oversikten':'oversikt',
'oversikter':'oversikt',
'oversiktleg':'oversikt',
'oversiktlege':'oversikt',
'oversiktlig':'oversikt',
'oversiktlige':'oversikt',
'oversiktlighet':'oversikt',
'oversiktsbildet':'oversiktsbild',
'oversiktsmessig':'oversiktsmess',
'oversiktstallene':'oversiktstall',
'oversitje':'oversitj',
'oversitjing':'oversitjing',
'oversjøisk':'oversjøisk',
'oversjøiske':'oversjøisk',
'overskot':'overskot',
'overskotet':'overskot',
'overskotslikviditet':'overskotslikvidit',
'overskotslikviditeten':'overskotslikviditet',
'overskotsutdeling':'overskotsutdeling',
'overskott':'overskott',
'overskottsutdelning':'overskottsutdelning',
'overskrevs':'overskrev',
'overskride':'overskrid',
'overskrides':'overskrid',
'overskriding':'overskriding',
'overskridingar':'overskriding',
'overskriften':'overskrift',
'overskudd':'overskudd',
'overskuddene':'overskudd',
'overskuddet':'overskudd',
'overskuddsdeling':'overskuddsdeling',
'overskuddsfond':'overskuddsfond',
'overskuddsgods':'overskuddsgod',
'overskuddstildeling':'overskuddstildeling',
'overskuelig':'oversku',
'overskytande':'overskyt',
'overskytende':'overskyt',
'overskyter':'overskyt',
'overskøyting':'overskøyting',
'overslaget':'overslag',
'overstige':'overst',
'overstiger':'overst',
'oversvømmelsen':'oversvømm',
'oversyn':'oversyn',
'overta':'overt',
'overtagelse':'overtag',
'overtakande':'overtak',
'overtakast':'overtak',
'overtakende':'overtak',
'overtaket':'overtak',
'overtaking':'overtaking',
'overtakinga':'overtaking',
'overtale':'overtal',
'overtaler':'overtal',
'overtar':'overt',
'overtatt':'overtatt',
'overtek':'overtek',
'overteke':'overtek',
'overteken':'overtek',
'overtekne':'overtekn',
'overtid':'overtid',
'overtidsbetaling':'overtidsbetaling',
'overtok':'overtok',
'overtredelse':'overtred',
'overtyding':'overtyding',
'overveielser':'overvei',
'overvekt':'overvekt',
'overvinne':'overvinn',
'overvunne':'overvunn',
'overvurdering':'overvurdering',
'overvurdert':'overvurder',
'overvurderte':'overvurder',
'overvåking':'overvåking',
'overvåkingen':'overvåking',
'ovh':'ovh',
'ovn':'ovn',
'ownership':'ownership',
'p':'p',
'pa':'pa',
'pakke':'pakk',
'pakker':'pakk',
'pakkes':'pakk',
'pakket':'pakk',
'palletert':'palleter',
'pannen':'pann',
'pant':'pant',
'pantebrev':'pantebrev',
'pantebreva':'pantebrev',
'pantekravet':'pantekrav',
'pantel':'pantel',
'pantelova':'pant',
'panteobjekt':'panteobjekt',
'panterett':'panterett',
'panterettane':'panterett',
'panterettar':'panterett',
'panteretten':'panterett',
'pantesikra':'pantesikr',
'pantesikring':'pantesikring',
'pantet':'pant',
'pantetrygd':'pantetrygd',
'panthavar':'panthav',
'panthavarane':'panthavar',
'panthavarar':'panthavar',
'panthavaren':'panthavar',
'panthavarforhold':'panthavarforhold',
'pantobjekt':'pantobjekt',
'pantobjektet':'pantobjekt',
'pantobligasjonane':'pantobligasjon',
'pantobligasjonar':'pantobligasjon',
'pantobligasjonen':'pantobligasjon',
'pantsetjaren':'pantsetjar',
'pantsetjast':'pantsetj',
'pantsetje':'pantsetj',
'pantsetjing':'pantsetjing',
'pantsett':'pantsett',
'pantsettelse':'pantsett',
'pantsettes':'pantsett',
'paper':'pap',
'papir':'papir',
'papireksport':'papireksport',
'papirindustri':'papirindustri',
'papirindustrien':'papirindustri',
'papirvarer':'papirvar',
'papp':'papp',
'par':'par',
'parabolantenna':'parabolantenn',
'parabolantenner':'parabolantenn',
'paradoks':'paradoks',
'paradoksale':'paradoksal',
'paragraf':'paragraf',
'paragrafane':'paragraf',
'paragrafar':'paragraf',
'paragrafen':'paragraf',
'paragrafens':'paragraf',
'paragrafer':'paragraf',
'parallell':'parallell',
'parallellar':'parallell',
'parallelle':'parallell',
'parallellen':'parallell',
'paralleller':'parallell',
'parallelt':'parallelt',
'parametere':'parameter',
'parametre':'parametr',
'parentes':'parent',
'paret':'par',
'parkeringsplass':'parkeringsplass',
'part':'part',
'partane':'part',
'partar':'part',
'parten':'part',
'parter':'part',
'parti':'parti',
'partier':'parti',
'partilast':'partil',
'partnarar':'partnar',
'partner':'partn',
'partnerskap':'partnerskap',
'partnerskapsliknende':'partnerskapslikn',
'partsforhold':'partsforhold',
'pass':'pass',
'passande':'pass',
'passar':'pass',
'passasjerer':'passasjer',
'passasjerfergene':'passasjerferg',
'passasjerferjen':'passasjerferj',
'passasjerferjene':'passasjerferj',
'passasjerferjer':'passasjerferj',
'passasjergrunnlag':'passasjergrunnlag',
'passasjertrafikken':'passasjertrafikk',
'passe':'pass',
'passende':'pass',
'passer':'pass',
'passere':'passer',
'passerer':'passer',
'passering':'passering',
'passerte':'passer',
'passiv':'passiv',
'passiva':'passiv',
'passivitet':'passivit',
'patterød':'patterød',
'pause':'paus',
'pausen':'paus',
'pbe':'pbe',
'pbes':'pbes',
'pbl':'pbl',
'peikast':'peik',
'peike':'peik',
'peiker':'peik',
'peikt':'peikt',
'peikte':'peikt',
'peisen':'peis',
'pek':'pek',
'peke':'pek',
'pekefinger':'pekefing',
'pekepinn':'pekepinn',
'peker':'pek',
'pekes':'pek',
'pekt':'pekt',
'peler':'pel',
'pels':'pel',
'pendler':'pendl',
'pene':'pen',
'peneste':'penest',
'pengane':'peng',
'pengar':'peng',
'penge':'peng',
'pengefordringer':'pengefordring',
'pengekrav':'pengekrav',
'pengekreditorar':'pengekreditor',
'pengemangel':'pengemangel',
'penger':'peng',
'pensionkostnader':'pensionkostnad',
'pensions':'pension',
'pensjon':'pensjon',
'pensjonalderen':'pensjonalder',
'pensjonen':'pensjon',
'pensjonene':'pensjon',
'pensjonenes':'pensjon',
'pensjonens':'pensjon',
'pensjoner':'pensjon',
'pensjonering':'pensjonering',
'pensjoneringstidspunktet':'pensjoneringstidspunkt',
'pensjonerte':'pensjoner',
'pensjonist':'pensjonist',
'pensjonistbeskatningen':'pensjonistbeskatning',
'pensjonistektepar':'pensjonistektep',
'pensjonisten':'pensjonist',
'pensjonistene':'pensjonist',
'pensjonistenes':'pensjonist',
'pensjonistens':'pensjonist',
'pensjonister':'pensjonist',
'pensjonists':'pensjonist',
'pensjonisttilværelse':'pensjonisttilvær',
'pensjonisttilværelsen':'pensjonisttilvær',
'pensjonkapitalen':'pensjonkapital',
'pensjonnivå':'pensjonnivå',
'pensjons':'pensjon',
'pensjonsalder':'pensjonsald',
'pensjonsalderen':'pensjonsalder',
'pensjonsaldersnormen':'pensjonsaldersnorm',
'pensjonsaldre':'pensjonsaldr',
'pensjonsavtale':'pensjonsavtal',
'pensjonsavtalene':'pensjonsavtal',
'pensjonsavtaler':'pensjonsavtal',
'pensjonsbegrepet':'pensjonsbegrep',
'pensjonsbehov':'pensjonsbehov',
'pensjonsbehovet':'pensjonsbehov',
'pensjonsbeløp':'pensjonsbeløp',
'pensjonsberegningen':'pensjonsberegning',
'pensjonsberettigede':'pensjonsberett',
'pensjonsberettigedes':'pensjonsberett',
'pensjonsbeskatning':'pensjonsbeskatning',
'pensjonsbeskatningen':'pensjonsbeskatning',
'pensjonsbevis':'pensjonsbevis',
'pensjonsfond':'pensjonsfond',
'pensjonsfondene':'pensjonsfond',
'pensjonsforhold':'pensjonsforhold',
'pensjonsforholdene':'pensjonsforhold',
'pensjonsformer':'pensjonsform',
'pensjonsformue':'pensjonsformu',
'pensjonsformuen':'pensjonsformu',
'pensjonsformål':'pensjonsformål',
'pensjonsforpliktelse':'pensjonsforplikt',
'pensjonsforpliktelser':'pensjonsforplikt',
'pensjonsforsikring':'pensjonsforsikring',
'pensjonsforsikringen':'pensjonsforsikring',
'pensjonsforsikringer':'pensjonsforsikring',
'pensjonsforsikringsavtale':'pensjonsforsikringsavtal',
'pensjonsforsikringsavtaler':'pensjonsforsikringsavtal',
'pensjonsforsikringskontrakt':'pensjonsforsikringskontrakt',
'pensjonsforsikringskontrakter':'pensjonsforsikringskontrakt',
'pensjonsforsikringsordning':'pensjonsforsikringsordning',
'pensjonsforsikringsordninger':'pensjonsforsikringsordning',
'pensjonsforsikringspremie':'pensjonsforsikringspremi',
'pensjonsforsikringsutvalget':'pensjonsforsikringsutvalg',
'pensjonsforum':'pensjonsforum',
'pensjonsgivende':'pensjonsgiv',
'pensjonsgrunnlaget':'pensjonsgrunnlag',
'pensjonsinnbetaling':'pensjonsinnbetaling',
'pensjonsinnbetalingene':'pensjonsinnbetaling',
'pensjonsinnbetalinger':'pensjonsinnbetaling',
'pensjonsinnretningen':'pensjonsinnretning',
'pensjonsinnretningene':'pensjonsinnretning',
'pensjonsinnskudd':'pensjonsinnskudd',
'pensjonsinnskuddene':'pensjonsinnskudd',
'pensjonsinnskuddenes':'pensjonsinnskudd',
'pensjonsinnskuddet':'pensjonsinnskudd',
'pensjonsinntekt':'pensjonsinntekt',
'pensjonsintervallet':'pensjonsintervall',
'pensjonskapital':'pensjonskapital',
'pensjonskapitalbevis':'pensjonskapitalbevis',
'pensjonskapitalbeviset':'pensjonskapitalbevis',
'pensjonskapitalen':'pensjonskapital',
'pensjonskapitalens':'pensjonskapital',
'pensjonskapitalkonti':'pensjonskapitalkonti',
'pensjonskapitalkontoen':'pensjonskapitalkonto',
'pensjonskapitel':'pensjonskapitel',
'pensjonskasse':'pensjonskass',
'pensjonskasseforskriften':'pensjonskasseforskrift',
'pensjonskassen':'pensjonskass',
'pensjonskassene':'pensjonskass',
'pensjonskassenes':'pensjonskass',
'pensjonskassens':'pensjonskass',
'pensjonskasser':'pensjonskass',
'pensjonskassers':'pensjonskass',
'pensjonskollektivet':'pensjonskollektiv',
'pensjonskonti':'pensjonskonti',
'pensjonskonto':'pensjonskonto',
'pensjonskontoen':'pensjonskonto',
'pensjonskostnaden':'pensjonskostnad',
'pensjonskostnadene':'pensjonskostnad',
'pensjonskostnader':'pensjonskostnad',
'pensjonsløsninger':'pensjonsløsning',
'pensjonsmessig':'pensjonsmess',
'pensjonsmessige':'pensjonsmess',
'pensjonsmidlene':'pensjonsmidl',
'pensjonsmidler':'pensjonsmidl',
'pensjonsnivå':'pensjonsnivå',
'pensjonsnivåene':'pensjonsnivå',
'pensjonsnivået':'pensjonsnivå',
'pensjonsoppbygging':'pensjonsoppbygging',
'pensjonsopptjening':'pensjonsopptjening',
'pensjonsordning':'pensjonsordning',
'pensjonsordningen':'pensjonsordning',
'pensjonsordningene':'pensjonsordning',
'pensjonsordningens':'pensjonsordning',
'pensjonsordninger':'pensjonsordning',
'pensjonsordningers':'pensjonsordning',
'pensjonsordnings':'pensjonsordning',
'pensjonspareavtaler':'pensjonspareavtal',
'pensjonsparing':'pensjonsparing',
'pensjonsplan':'pensjonsplan',
'pensjonsplanen':'pensjonsplan',
'pensjonsplanene':'pensjonsplan',
'pensjonsplaner':'pensjonsplan',
'pensjonspolitikk':'pensjonspolitikk',
'pensjonspolitisk':'pensjonspolitisk',
'pensjonspolitiske':'pensjonspolitisk',
'pensjonsprosent':'pensjonsprosent',
'pensjonsprosenten':'pensjonsprosent',
'pensjonsreguleringsfond':'pensjonsreguleringsfond',
'pensjonsreserve':'pensjonsreserv',
'pensjonsrett':'pensjonsrett',
'pensjonsrettigheten':'pensjonsrett',
'pensjonsrettighetene':'pensjonsrett',
'pensjonsrettigheter':'pensjonsrett',
'pensjonssammenheng':'pensjonssammenheng',
'pensjonsskyddscentralen':'pensjonsskyddscentral',
'pensjonsspareavtale':'pensjonsspareavtal',
'pensjonsspareavtalen':'pensjonsspareavtal',
'pensjonsspareavtalene':'pensjonsspareavtal',
'pensjonsspareavtaler':'pensjonsspareavtal',
'pensjonsspareren':'pensjonssparer',
'pensjonssparerens':'pensjonssparer',
'pensjonssparing':'pensjonssparing',
'pensjonssparingen':'pensjonssparing',
'pensjonsspørsmål':'pensjonsspørsmål',
'pensjonssystem':'pensjonssystem',
'pensjonssystemene':'pensjonssystem',
'pensjonssystemer':'pensjonssystem',
'pensjonssystemet':'pensjonssystem',
'pensjonstilbud':'pensjonstilbud',
'pensjonstilfellet':'pensjonstilfell',
'pensjonstilskudd':'pensjonstilskudd',
'pensjonstilskuddet':'pensjonstilskudd',
'pensjonsutbetaling':'pensjonsutbetaling',
'pensjonsutbetalingen':'pensjonsutbetaling',
'pensjonsutbetalingene':'pensjonsutbetaling',
'pensjonsutbetalinger':'pensjonsutbetaling',
'pensjonsutgifter':'pensjonsutgift',
'pensjonsvilkår':'pensjonsvilkår',
'pensjonsytelse':'pensjonsyt',
'pensjonsytelsen':'pensjonsyt',
'pensjonsytelsene':'pensjonsyt',
'pensjonsytelser':'pensjonsyt',
'pensjonsøyemed':'pensjonsøyemed',
'pent':'pent',
'per':'per',
'periodar':'period',
'periode':'period',
'perioden':'period',
'periodene':'period',
'periodens':'period',
'perioder':'period',
'periodiseres':'periodiser',
'periodiseringen':'periodisering',
'periodiseringsprinsipp':'periodiseringsprinsipp',
'periodiseringsprinsipper':'periodiseringsprinsipp',
'periodisert':'periodiser',
'periodisk':'periodisk',
'periodiske':'periodisk',
'permanens':'perman',
'permanent':'permanent',
'permanente':'permanent',
'permisjon':'permisjon',
'permisjonen':'permisjon',
'permisjoner':'permisjon',
'permisjonstiden':'permisjonstid',
'permisjonstidspunktet':'permisjonstidspunkt',
'permittering':'permittering',
'permitteringer':'permittering',
'permittert':'permitter',
'permitterte':'permitter',
'person':'person',
'personane':'person',
'personar':'person',
'personbiler':'personbil',
'personen':'person',
'personene':'person',
'personens':'person',
'personer':'person',
'personfellesskap':'personfellesskap',
'persongrupper':'persongrupp',
'personinntekt':'personinntekt',
'personinntekten':'personinntekt',
'personleg':'person',
'personlege':'person',
'personopplysningar':'personopplysning',
'personskifte':'personskift',
'persontrafikken':'persontrafikk',
'perspektiv':'perspektiv',
'perspektivene':'perspektiv',
'perspektiver':'perspektiv',
'perspektivet':'perspektiv',
'persson':'persson',
'peser':'pes',
'pest':'pest',
'peter':'pet',
'peterson':'peterson',
'petter':'pett',
'pga':'pga',
'philipps':'philipp',
'piggeple':'piggepl',
'pike':'pik',
'piken':'pik',
'pil':'pil',
'pilen':'pil',
'piler':'pil',
'pilskudd':'pilskudd',
'pimpesten':'pimpest',
'pinefulle':'pinefull',
'pinne':'pinn',
'pinsler':'pinsl',
'pipende':'pip',
'piper':'pip',
'pipete':'pipet',
'pirret':'pirr',
'pisker':'pisk',
'pisser':'piss',
'pjokk':'pjokk',
'pjusket':'pjusk',
'pjuskete':'pjusket',
'pkt':'pkt',
'plage':'plag',
'plager':'plag',
'plaget':'plag',
'plagsam':'plagsam',
'plan':'plan',
'planane':'plan',
'planar':'plan',
'planarbeid':'planarbeid',
'planarbeider':'planarbeid',
'planarbeidet':'planarbeid',
'plandokument':'plandokument',
'planen':'plan',
'planene':'plan',
'planens':'plan',
'planer':'plan',
'planforutsetningene':'planforutsetning',
'planforutsetninger':'planforutsetning',
'plangrunnlag':'plangrunnlag',
'plangrunnlaget':'plangrunnlag',
'planhorisont':'planhorisont',
'planhorisonter':'planhorisont',
'planlagde':'planlagd',
'planlagt':'planlagt',
'planlagte':'planlagt',
'planlegg':'planlegg',
'planlegge':'planlegg',
'planlegger':'planlegg',
'planlegges':'planlegg',
'planlegging':'planlegging',
'planlegginga':'planlegging',
'planleggingen':'planlegging',
'planleggings':'planlegging',
'planleggingsstadiet':'planleggingsstadi',
'planleggingsvirksomhet':'planleggingsvirksom',
'planleggje':'planleggj',
'planlovgivningen':'planlovgivning',
'planløsninger':'planløsning',
'planmessig':'planmess',
'planoppgave':'planoppgav',
'planperioden':'planperiod',
'planprosessen':'planprosess',
'planrammen':'planramm',
'planrammer':'planramm',
'plansituasjonen':'plansituasjon',
'planstatus':'planstatus',
'planstyresmakt':'planstyresmakt',
'plante':'plant',
'planter':'plant',
'plantet':'plant',
'plantype':'plantyp',
'planvedtak':'planvedtak',
'plar':'plar',
'plaske':'plask',
'plasker':'plask',
'plass':'plass',
'plassen':'plass',
'plasserast':'plasser',
'plassere':'plasser',
'plasserer':'plasser',
'plasseres':'plasser',
'plassering':'plassering',
'plasseringa':'plassering',
'plasseringar':'plassering',
'plasseringen':'plassering',
'plasseringer':'plassering',
'plasseringsalternativer':'plasseringsalternativ',
'plasseringsformene':'plasseringsform',
'plasseringsformer':'plasseringsform',
'plassert':'plasser',
'plasserte':'plasser',
'plattform':'plattform',
'pleie':'pleie',
'pleier':'pleier',
'plenumsmøter':'plenumsmøt',
'plikt':'plikt',
'plikta':'plikt',
'pliktbrot':'pliktbrot',
'plikten':'plikt',
'pliktene':'plikt',
'plikter':'plikt',
'pliktet':'plikt',
'pliktig':'plikt',
'pliktige':'plikt',
'plm':'plm',
'plogen':'plog',
'pludder':'pludd',
'plukke':'plukk',
'plukket':'plukk',
'pluss':'pluss',
'plutselig':'pluts',
'plystrer':'plystr',
'pløyer':'pløyer',
'poeng':'poeng',
'poenget':'poeng',
'poenghistorie':'poenghistori',
'poetisk':'poetisk',
'poetiske':'poetisk',
'polisen':'polis',
'politikk':'politikk',
'politikken':'politikk',
'politikkområde':'politikkområd',
'politikkområder':'politikkområd',
'politisk':'politisk',
'politiske':'politisk',
'porsgrunn':'porsgrunn',
'port':'port',
'portefølje':'portefølj',
'porteføljen':'portefølj',
'porteføljer':'portefølj',
'porteføljesammensetningen':'porteføljesammensetning',
'porten':'port',
'portnerservice':'portnerservic',
'ports':'port',
'portstolper':'portstolp',
'portvakt':'portvakt',
'posisjon':'posisjon',
'posisjonar':'posisjon',
'posisjonen':'posisjon',
'posisjonere':'posisjoner',
'posisjoneres':'posisjoner',
'posisjonering':'posisjonering',
'posisjonert':'posisjoner',
'positiv':'positiv',
'positive':'positiv',
'positivt':'positiv',
'post':'post',
'postadresse':'postadress',
'postkassene':'postkass',
'postselskaper':'postselskap',
'poten':'pot',
'potensial':'potensial',
'potensiale':'potensial',
'potensialene':'potensial',
'potensialet':'potensial',
'potensiell':'potensiell',
'potensielle':'potensiell',
'potensielt':'potensielt',
'pr':'pr',
'praksis':'praksis',
'praksisen':'praksis',
'praksisens':'praksis',
'praktiserbart':'praktiserbart',
'praktiserer':'praktiser',
'praktisering':'praktisering',
'praktiseringa':'praktisering',
'praktiseringen':'praktisering',
'praktisert':'praktiser',
'praktisk':'praktisk',
'praktiske':'praktisk',
'prammen':'pramm',
'prat':'prat',
'preferanser':'preferans',
'prefereres':'preferer',
'preg':'preg',
'prega':'preg',
'preges':'preg',
'preget':'preg',
'preklusiv':'preklusiv',
'preklusivt':'preklusiv',
'prekær':'prekær',
'prekære':'prekær',
'preller':'prell',
'premiar':'premi',
'premie':'premi',
'premiebeløp':'premiebeløp',
'premiebeløpene':'premiebeløp',
'premieberegning':'premieberegning',
'premieberegningen':'premieberegning',
'premiebetaling':'premiebetaling',
'premiebetalingen':'premiebetaling',
'premiebetalinger':'premiebetaling',
'premiebetalingsperiodens':'premiebetalingsperiod',
'premiebetalingstiden':'premiebetalingstid',
'premiefastsettelsen':'premiefastsett',
'premiefond':'premiefond',
'premiefondet':'premiefond',
'premiefondets':'premiefond',
'premiefondsmidlene':'premiefondsmidl',
'premiefondsmidler':'premiefondsmidl',
'premiefritak':'premiefritak',
'premiefritaket':'premiefritak',
'premieinnbetaling':'premieinnbetaling',
'premieinnbetalingar':'premieinnbetaling',
'premieinnbetalingen':'premieinnbetaling',
'premieinnbetalingene':'premieinnbetaling',
'premieinnbetalinger':'premieinnbetaling',
'premien':'premi',
'premiene':'premi',
'premier':'premi',
'premiereserve':'premiereserv',
'premiereserven':'premiereserv',
'premieskatteloven':'premieskatt',
'premieskatten':'premieskatt',
'premietariffene':'premietariff',
'premieøkningene':'premieøkning',
'premiss':'premiss',
'premissane':'premiss',
'premissar':'premiss',
'premissene':'premiss',
'premisser':'premiss',
'presentasjon':'presentasjon',
'presentasjonen':'presentasjon',
'presentere':'presenter',
'presenterer':'presenter',
'presenteres':'presenter',
'presentert':'presenter',
'presenterte':'presenter',
'presisere':'presiser',
'presiserer':'presiser',
'presiseres':'presiser',
'presisering':'presisering',
'presiseringa':'presisering',
'presiseringen':'presisering',
'presisert':'presiser',
'presisjon':'presisjon',
'presist':'presist',
'press':'press',
'pressede':'press',
'presser':'press',
'presserende':'presser',
'presses':'press',
'presset':'press',
'presstendenser':'presstendens',
'presumptivt':'presumptiv',
'preventivt':'preventiv',
'previdenza':'previdenz',
'prikk':'prikk',
'primære':'primær',
'primært':'primært',
'prinsessen':'prinsess',
'prinsipalt':'prinsipalt',
'prinsipiell':'prinsipiell',
'prinsipielle':'prinsipiell',
'prinsipielt':'prinsipielt',
'prinsipp':'prinsipp',
'prinsippa':'prinsipp',
'prinsippene':'prinsipp',
'prinsipper':'prinsipp',
'prinsippet':'prinsipp',
'prinsippforskjeller':'prinsippforskjell',
'prinsippføresegn':'prinsippføresegn',
'priori':'priori',
'prioritere':'prioriter',
'prioriteres':'prioriter',
'prioritering':'prioritering',
'prioriteringer':'prioritering',
'prioritert':'prioriter',
'prioriterte':'prioriter',
'prioritet':'priorit',
'prioritetar':'prioritet',
'prioriteten':'prioritet',
'prioritetsendringar':'prioritetsendring',
'prioritetsreglane':'prioritetsregl',
'prioritetsreglar':'prioritetsregl',
'prioritetsviking':'prioritetsviking',
'pris':'pris',
'prisane':'pris',
'prisar':'pris',
'prisavslag':'prisavslag',
'prisdanninga':'prisdanning',
'prisen':'pris',
'prisene':'pris',
'priser':'pris',
'prises':'pris',
'priset':'pris',
'prisfastsetjing':'prisfastsetjing',
'prising':'prising',
'prisingen':'prising',
'prisklasse':'prisklass',
'prisklausular':'prisklausul',
'priskonkurranse':'priskonkurrans',
'prisnivå':'prisnivå',
'prisnivået':'prisnivå',
'prisreglane':'prisregl',
'prisreglar':'prisregl',
'prisregulering':'prisregulering',
'prisreguleringa':'prisregulering',
'prisregulert':'prisreguler',
'prissetjinga':'prissetjing',
'prissetting':'prissetting',
'prisskilnader':'prisskilnad',
'prisstiging':'prisstiging',
'prisstigning':'prisstigning',
'prisstigningen':'prisstigning',
'prisvilkår':'prisvilkår',
'privat':'privat',
'privatdisponibel':'privatdisponibel',
'private':'privat',
'privatrett':'privatrett',
'privatrettslege':'privatretts',
'privatrettslig':'privatretts',
'problem':'problem',
'problematikken':'problematikk',
'problematisk':'problematisk',
'problemene':'problem',
'problemer':'problem',
'problemet':'problem',
'problemstilling':'problemstilling',
'problemstillinga':'problemstilling',
'problemstillingane':'problemstilling',
'problemstillingar':'problemstilling',
'problemstillingen':'problemstilling',
'problemstillingene':'problemstilling',
'problemstillinger':'problemstilling',
'prod':'prod',
'produksjon':'produksjon',
'produksjonen':'produksjon',
'produksjons':'produksjon',
'produksjonsanlegg':'produksjonsanlegg',
'produksjonsbedrifter':'produksjonsbedrift',
'produksjonsgrunnlag':'produksjonsgrunnlag',
'produksjonskapasitet':'produksjonskapasit',
'produksjonskostnader':'produksjonskostnad',
'produksjonsleddet':'produksjonsledd',
'produksjonsmønster':'produksjonsmønst',
'produksjonsmønstre':'produksjonsmønstr',
'produksjonsområde':'produksjonsområd',
'produksjonssted':'produksjonssted',
'produksjonsvirksomhet':'produksjonsvirksom',
'produksjonsvirksomheter':'produksjonsvirksom',
'produkt':'produkt',
'produkta':'produkt',
'produkter':'produkt',
'produktive':'produktiv',
'produktkrav':'produktkrav',
'produktpris':'produktpris',
'produktspesialisering':'produktspesialisering',
'produsent':'produsent',
'produsenter':'produsent',
'produsentleddet':'produsentledd',
'produsentorganisasjonar':'produsentorganisasjon',
'produsere':'produser',
'profesjonalitet':'profesjonalit',
'profesjonell':'profesjonell',
'profesjonelle':'profesjonell',
'profesjonelt':'profesjonelt',
'profesjonsvis':'profesjonsvis',
'professor':'professor',
'profil':'profil',
'profilen':'profil',
'profilene':'profil',
'profiler':'profil',
'proforma':'proform',
'proformaselskaper':'proformaselskap',
'prognose':'prognos',
'prognosealternativet':'prognosealternativ',
'prognosearbeid':'prognosearbeid',
'prognosearbeider':'prognosearbeid',
'prognoseformål':'prognoseformål',
'prognosemakeren':'prognosemaker',
'prognosen':'prognos',
'prognosene':'prognos',
'prognoser':'prognos',
'prognoseresultater':'prognoseresultat',
'programkategorien':'programkategori',
'programmene':'programm',
'programmer':'programm',
'progressiv':'progressiv',
'proklama':'proklam',
'prokura':'prokur',
'prokuralova':'prokura',
'prop':'prop',
'property':'property',
'proposisjon':'proposisjon',
'proposisjonen':'proposisjon',
'proratarisk':'proratarisk',
'prosedyre':'prosedyr',
'prosent':'prosent',
'prosentandel':'prosentandel',
'prosentandelen':'prosentandel',
'prosentdel':'prosentdel',
'prosenten':'prosent',
'prosentene':'prosent',
'prosentfordelingene':'prosentfordeling',
'prosentpoeng':'prosentpoeng',
'prosentsats':'prosentsat',
'prosentsatsen':'prosentsats',
'prosentsatser':'prosentsats',
'prosentvis':'prosentvis',
'prosentvise':'prosentvis',
'prosess':'prosess',
'prosessen':'prosess',
'prosjekt':'prosjekt',
'prosjekta':'prosjekt',
'prosjektene':'prosjekt',
'prosjekter':'prosjekt',
'prosjektet':'prosjekt',
'prosjektrapport':'prosjektrapport',
'prospekt':'prospekt',
'protesterer':'protester',
'protokoll':'protokoll',
'protokollar':'protokoll',
'protokollen':'protokoll',
'protokollføring':'protokollføring',
'proveny':'proveny',
'provenymessige':'provenymess',
'provenytap':'provenytap',
'provenytapet':'provenytap',
'provenyvirkningen':'provenyvirkning',
'provenyvirkningene':'provenyvirkning',
'provisjon':'provisjon',
'prp':'prp',
'prufungsverbande':'prufungsverb',
'pruster':'prust',
'prøv':'prøv',
'prøvast':'prøv',
'prøvd':'prøvd',
'prøvde':'prøvd',
'prøve':'prøv',
'prøver':'prøv',
'prøves':'prøv',
'prøving':'prøving',
'prøvt':'prøvt',
'pst':'pst',
'psykisk':'psykisk',
'psykiske':'psykisk',
'pubblici':'pubblici',
'publikum':'publikum',
'publisert':'publiser',
'puff':'puff',
'pung':'pung',
'pungen':'pung',
'punkt':'punkt',
'punkta':'punkt',
'punktanslag':'punktanslag',
'punktene':'punkt',
'punkter':'punkt',
'punktet':'punkt',
'punktlighet':'punkt',
'punktum':'punktum',
'punktumet':'punktum',
'puslespillbrikker':'puslespillbrikk',
'puslete':'puslet',
'pusling':'pusling',
'pust':'pust',
'puste':'pust',
'pusten':'pust',
'puster':'pust',
'pyntet':'pynt',
'pyse':'pys',
'på':'på',
'påbegynner':'påbegynn',
'påbegynnes':'påbegynn',
'påbod':'påbod',
'påboda':'påbod',
'påbode':'påbod',
'påboden':'påbod',
'påbodet':'påbod',
'påbodne':'påbodn',
'påby':'påby',
'påbygging':'påbygging',
'påbyggingssaker':'påbyggingssak',
'pådra':'pådr',
'pådrage':'pådrag',
'pådregne':'pådregn',
'pådriv':'pådriv',
'påfølgende':'påfølg',
'påførast':'påfør',
'påføre':'påfør',
'påfører':'påfør',
'påføres':'påfør',
'påført':'påført',
'pågående':'pågå',
'pågår':'pågår',
'pågått':'pågått',
'påhefta':'påheft',
'påhefte':'påheft',
'påhviler':'påhvil',
'påkjærast':'påkjær',
'påklages':'påklag',
'påkommende':'påkomm',
'påkostingar':'påkosting',
'påkostning':'påkostning',
'påkostninger':'påkostning',
'påkrav':'påkrav',
'påkrevet':'påkrev',
'pål':'pål',
'påla':'pål',
'pålagde':'pålagd',
'pålagt':'pålagt',
'pålagte':'pålagt',
'pålegg':'pålegg',
'pålegga':'pålegg',
'pålegge':'pålegg',
'pålegger':'pålegg',
'pålegges':'pålegg',
'pålegget':'pålegg',
'påleggjast':'påleggj',
'påleggje':'påleggj',
'påleggskompetanse':'påleggskompetans',
'pålitelege':'pålit',
'pålitelig':'pålit',
'pålitelige':'pålit',
'pålitelighet':'pålit',
'pålydande':'pålyd',
'pålydende':'pålyd',
'påløpe':'påløp',
'påløper':'påløp',
'påløpt':'påløpt',
'påløpte':'påløpt',
'påmelde':'påmeld',
'påminning':'påminning',
'påpeikt':'påpeikt',
'påpeke':'påpek',
'påpeker':'påpek',
'påpekes':'påpek',
'påpeking':'påpeking',
'påpekningen':'påpekning',
'påpekt':'påpekt',
'pårekneleg':'pårekn',
'påse':'pås',
'påslag':'påslag',
'påstand':'påstand',
'påstått':'påstått',
'påta':'påt',
'påtale':'påtal',
'påtar':'påt',
'påtatt':'påtatt',
'påtegning':'påtegning',
'påtrengende':'påtreng',
'påvente':'påvent',
'påverka':'påverk',
'påverkar':'påverk',
'påverke':'påverk',
'påverknaden':'påverknad',
'påvirke':'påvirk',
'påvirker':'påvirk',
'påvirkes':'påvirk',
'påvirket':'påvirk',
'påvirkning':'påvirkning',
'påvisast':'påvis',
'påvise':'påvis',
'påvist':'påvist',
'pøse':'pøs',
'quorum':'quorum',
'r':'r',
'rabalder':'rabald',
'rabattar':'rabatt',
'rabattavtalar':'rabattavtal',
'rabatten':'rabatt',
'rabattordningar':'rabattordning',
'rad':'rad',
'radar':'rad',
'radardekning':'radardekning',
'radarovervåkningen':'radarovervåkning',
'radarseilas':'radarseil',
'radarutrustning':'radarutrustning',
'raden':'rad',
'rader':'rad',
'radikalt':'radikalt',
'radio':'radio',
'radiokommunikasjon':'radiokommunikasjon',
'ragnarok':'ragnarok',
'rail':'rail',
'rake':'rak',
'raker':'rak',
'rakk':'rakk',
'rakkestadsvingen':'rakkestadsving',
'ramlende':'raml',
'ramler':'raml',
'ramma':'ramm',
'ramme':'ramm',
'rammeavtale':'rammeavtal',
'rammebetingelsene':'rammebeting',
'rammebetingelser':'rammebeting',
'rammelovgiving':'rammelovgiving',
'rammen':'ramm',
'rammene':'ramm',
'rammeprogram':'rammeprogram',
'rammeprogrammet':'rammeprogramm',
'rammer':'ramm',
'rammes':'ramm',
'rammet':'ramm',
'rammeverk':'rammeverk',
'rammeverket':'rammeverk',
'rammevilkår':'rammevilkår',
'rammevilkåra':'rammevilkår',
'ramper':'ramp',
'ramse':'rams',
'ramser':'rams',
'randbebyggelsen':'randbebygg',
'randi':'randi',
'randsonen':'randson',
'range':'rang',
'rangeres':'ranger',
'rangering':'rangering',
'rangt':'rangt',
'rank':'rank',
'rant':'rant',
'raper':'rap',
'rappkjeftet':'rappkjeft',
'rapport':'rapport',
'rapporten':'rapport',
'rapportering':'rapportering',
'rar':'rar',
'rart':'rart',
'ras':'ras',
'rasa':'ras',
'rase':'ras',
'rasende':'ras',
'raser':'ras',
'raseri':'raseri',
'raset':'ras',
'rasjonaliseringsgevinster':'rasjonaliseringsgevinst',
'rasjonaliseringsvinstar':'rasjonaliseringsvinst',
'rasjonell':'rasjonell',
'rasjonelle':'rasjonell',
'rasjonelt':'rasjonelt',
'rask':'rask',
'raske':'rask',
'raskere':'rasker',
'raskt':'raskt',
'rastløs':'rastløs',
'rastløse':'rastløs',
'rastløsheten':'rastløs',
'ratatosk':'ratatosk',
'rate':'rat',
'rateforsikring':'rateforsikring',
'rateforsikringer':'rateforsikring',
'rateoppsparing':'rateoppsparing',
'rateordningene':'rateordning',
'ratten':'ratt',
'raudung':'raudung',
'raus':'raus',
'ravneguden':'ravnegud',
'ravnene':'ravn',
'ravner':'ravn',
'rdir':'rdir',
'reaksjoner':'reaksjon',
'realavkastning':'realavkastning',
'realavkastnings':'realavkastning',
'realindeksert':'realindekser',
'realisasjon':'realisasjon',
'realisasjonen':'realisasjon',
'realisere':'realiser',
'realiserer':'realiser',
'realiseres':'realiser',
'realisering':'realisering',
'realisert':'realiser',
'realiserte':'realiser',
'realismen':'realism',
'realistisk':'realistisk',
'realistiske':'realistisk',
'realitet':'realit',
'realitetane':'realitet',
'realiteten':'realitet',
'realitetene':'realitet',
'realitetsendring':'realitetsendring',
'realitetsforskjell':'realitetsforskjell',
'realitetsspørsmåla':'realitetsspørsmål',
'realitetsvotering':'realitetsvotering',
'reallønnsvekst':'reallønnsvekst',
'realregister':'realregist',
'realregistera':'realregister',
'realregistrerte':'realregistrer',
'realrenteavgift':'realrenteavgift',
'realrenteavgiften':'realrenteavgift',
'recent':'recent',
'red':'red',
'redd':'redd',
'redde':'redd',
'redder':'redd',
'reddest':'reddest',
'reddhare':'reddhar',
'rede':'red',
'redegjort':'redegjort',
'redegjøre':'redegjør',
'redegjørelse':'redegjør',
'redelig':'red',
'rederi':'rederi',
'rederiene':'rederi',
'rederienes':'rederi',
'rederier':'rederi',
'rederiforbund':'rederiforbund',
'rederinæringen':'rederinæring',
'redistribueres':'redistribuer',
'redsel':'redsel',
'redselshjelm':'redselshjelm',
'redskap':'redskap',
'redskapene':'redskap',
'redskapet':'redskap',
'reduksjon':'reduksjon',
'reduksjonen':'reduksjon',
'reduserast':'reduser',
'redusere':'reduser',
'reduserer':'reduser',
'reduseres':'reduser',
'redusering':'redusering',
'redusert':'reduser',
'reduserte':'reduser',
'reell':'reell',
'reelle':'reell',
'reelt':'reelt',
'referansegruppe':'referansegrupp',
'referansegruppen':'referansegrupp',
'referat':'referat',
'refererer':'referer',
'refereres':'referer',
'refererte':'referer',
'reflektere':'reflekter',
'reflekterer':'reflekter',
'reflektert':'reflekter',
'refse':'refs',
'refundert':'refunder',
'refusjon':'refusjon',
'regel':'regel',
'regelen':'regel',
'regelendringar':'regelendring',
'regelene':'regel',
'regelmessige':'regelmess',
'regelsetta':'regelsett',
'regelutforming':'regelutforming',
'regelverk':'regelverk',
'regelverkene':'regelverk',
'regelverket':'regelverk',
'regelverkets':'regelverk',
'regelverksutforming':'regelverksutforming',
'regi':'regi',
'regin':'regin',
'region':'region',
'regional':'regional',
'regionaldepartementet':'regionaldepartement',
'regionale':'regional',
'regionalplanlegging':'regionalplanlegging',
'regionalpolitikken':'regionalpolitikk',
'regionalpolitisk':'regionalpolitisk',
'regionalpolitiske':'regionalpolitisk',
'regionalsjef':'regionalsjef',
'regionalt':'regionalt',
'regionaløkonomiske':'regionaløkonomisk',
'regionane':'region',
'regionen':'region',
'regionene':'region',
'regionens':'region',
'regioner':'region',
'regionforskning':'regionforskning',
'regionstyresmaktene':'regionstyresmakt',
'register':'regist',
'registera':'register',
'registereigaren':'registereigar',
'registeret':'register',
'registerføraren':'registerførar',
'registerordning':'registerordning',
'registerspørsmålet':'registerspørsmål',
'registert':'register',
'registerutskrift':'registerutskrift',
'registrerast':'registrer',
'registrere':'registrer',
'registrerer':'registrer',
'registreres':'registrer',
'registrering':'registrering',
'registreringa':'registrering',
'registreringane':'registrering',
'registreringer':'registrering',
'registreringsdag':'registreringsdag',
'registreringsdagen':'registreringsdag',
'registreringsfeil':'registreringsfeil',
'registreringsordning':'registreringsordning',
'registreringsordninga':'registreringsordning',
'registreringsordningane':'registreringsordning',
'registreringsordningar':'registreringsordning',
'registreringsreglane':'registreringsregl',
'registreringstidspunktet':'registreringstidspunkt',
'registrert':'registrer',
'registrerte':'registrer',
'regjering':'regjering',
'regjeringa':'regjering',
'regjeringen':'regjering',
'regjeringens':'regjering',
'regjeringspartiene':'regjeringsparti',
'regjerningen':'regjerning',
'reglane':'regl',
'reglar':'regl',
'reglementet':'reglement',
'reglene':'regl',
'regler':'regl',
'regn':'regn',
'regnbue':'regnbu',
'regnbuebroen':'regnbuebro',
'regnbuen':'regnbu',
'regne':'regn',
'regneeksempler':'regneeksempl',
'regner':'regn',
'regnes':'regn',
'regnet':'regn',
'regnskap':'regnskap',
'regnskapene':'regnskap',
'regnskaper':'regnskap',
'regnskapet':'regnskap',
'regnskapsavleggelse':'regnskapsavlegg',
'regnskapsføring':'regnskapsføring',
'regnskapskontrollør':'regnskapskontrollør',
'regnskapslov':'regnskap',
'regnskapsloven':'regnskap',
'regnskapslovens':'regnskap',
'regnskapslovutvalget':'regnskapslovutvalg',
'regnskapsplikt':'regnskapsplikt',
'regnskapsplikten':'regnskapsplikt',
'regnskapspliktiges':'regnskapsplikt',
'regnskapsprinsipper':'regnskapsprinsipp',
'regnskapsprinsippet':'regnskapsprinsipp',
'regnskapsreglene':'regnskapsregl',
'regnskapsregler':'regnskapsregl',
'regnskapsskikk':'regnskapsskikk',
'regnskapsstandarden':'regnskapsstandard',
'regnskapsår':'regnskapsår',
'regnskapsåret':'regnskapsår',
'regnvåte':'regnvåt',
'regnvær':'regnvær',
'regress':'regress',
'regresskrav':'regresskrav',
'regressomgangen':'regressomgang',
'regularitet':'regularit',
'regulariteten':'regularitet',
'regularitetsproblemer':'regularitetsproblem',
'regulerast':'reguler',
'regulere':'reguler',
'regulerer':'reguler',
'reguleres':'reguler',
'regulering':'regulering',
'reguleringa':'regulering',
'reguleringen':'regulering',
'reguleringsforslag':'reguleringsforslag',
'reguleringskonflikter':'reguleringskonflikt',
'reguleringsmekanismene':'reguleringsmekanism',
'reguleringsmekanismer':'reguleringsmekanism',
'reguleringsmessig':'reguleringsmess',
'reguleringsmessige':'reguleringsmess',
'reguleringsmyndighet':'reguleringsmynd',
'reguleringsplan':'reguleringsplan',
'reguleringsplanarbeidet':'reguleringsplanarbeid',
'reguleringsplanen':'reguleringsplan',
'reguleringsplaner':'reguleringsplan',
'reguleringsplanforslagets':'reguleringsplanforslag',
'reguleringsplannivå':'reguleringsplannivå',
'reguleringssaker':'reguleringssak',
'reguleringstiltak':'reguleringstiltak',
'reguleringsvedtak':'reguleringsvedtak',
'regulert':'reguler',
'regulerte':'reguler',
'regulær':'regulær',
'regulære':'regulær',
'rehabilitering':'rehabilitering',
'reidmar':'reidm',
'reidmars':'reidmar',
'rein':'rein',
'reine':'rein',
'reinertsen':'reinerts',
'reinsing':'reinsing',
'reint':'reint',
'reinvestere':'reinvester',
'reinvesteres':'reinvester',
'reinvestering':'reinvestering',
'reinvesteringsadgang':'reinvesteringsadgang',
'reinvesteringsadgangen':'reinvesteringsadgang',
'reinvesterte':'reinvester',
'reis':'reis',
'reisast':'reis',
'reise':'reis',
'reiseaktivitet':'reiseaktivit',
'reisebyrå':'reisebyrå',
'reisefeller':'reisefell',
'reisefølget':'reisefølg',
'reiseklare':'reiseklar',
'reisen':'reis',
'reisene':'reis',
'reiser':'reis',
'reises':'reis',
'reising':'reising',
'reist':'reist',
'rekalkuleres':'rekalkuler',
'rekende':'rek',
'rekk':'rekk',
'rekke':'rekk',
'rekker':'rekk',
'rekkevidden':'rekkevidd',
'rekkje':'rekkj',
'rekkjefølgja':'rekkjefølgj',
'rekkjefølgje':'rekkjefølgj',
'rekkjehus':'rekkjehus',
'rekkjevidda':'rekkjevidd',
'reklamasjonar':'reklamasjon',
'reklamasjonsfristar':'reklamasjonsfrist',
'reklamasjonsnemnder':'reklamasjonsnemnd',
'rekna':'rekn',
'reknar':'rekn',
'reknast':'rekn',
'rekne':'rekn',
'rekneskap':'rekneskap',
'rekneskapane':'rekneskap',
'rekneskapen':'rekneskap',
'rekneskapsførsel':'rekneskapsførsel',
'rekneskapskunnige':'rekneskapskunn',
'rekneskapslova':'rekneskap',
'rekneskapsmateriale':'rekneskapsmaterial',
'rekneskapsopplysningar':'rekneskapsopplysning',
'rekneskapsplikt':'rekneskapsplikt',
'rekneskapsrapportering':'rekneskapsrapportering',
'rekneskapsregisteret':'rekneskapsregister',
'rekneskapsreglane':'rekneskapsregl',
'rekneskapsreglar':'rekneskapsregl',
'rekneskapsskikk':'rekneskapsskikk',
'rekneskapsår':'rekneskapsår',
'rekneskapsåret':'rekneskapsår',
'reknestykket':'reknestykk',
'rekning':'rekning',
'rekningsbasis':'rekningsbasis',
'rekningsprinsipp':'rekningsprinsipp',
'rekningsprinsippet':'rekningsprinsipp',
'rekningssvarande':'rekningssvar',
'rekreasjon':'rekreasjon',
'rekreasjonsformål':'rekreasjonsformål',
'rekreasjonsverdier':'rekreasjonsverdi',
'relasjoner':'relasjon',
'relatert':'relater',
'relativ':'relativ',
'relative':'relativ',
'relativt':'relativ',
'relevans':'relevan',
'relevant':'relevant',
'relevante':'relevant',
'religiøs':'religiøs',
'rem':'rem',
'remmen':'remm',
'ren':'ren',
'rene':'ren',
'rengjøring':'rengjøring',
'renkespill':'renkespill',
'renn':'renn',
'renne':'renn',
'renner':'renn',
'renovering':'renovering',
'rent':'rent',
'renta':'rent',
'rente':'rent',
'renteavkastning':'renteavkastning',
'rentebetingelser':'rentebeting',
'rentefastsettelsen':'rentefastsett',
'renteforsikring':'renteforsikring',
'renteforsikringer':'renteforsikring',
'rentefot':'rentefot',
'renteinntektene':'renteinntekt',
'renteinntekter':'renteinntekt',
'rentelova':'rent',
'rentemargin':'rentemargin',
'rentemarkedet':'rentemarked',
'renten':'rent',
'rentenivå':'rentenivå',
'renteoppgaver':'renteoppgav',
'renter':'rent',
'rentesatser':'rentesats',
'rentesrente':'rentesrent',
'rentevilkår':'rentevilkår',
'reparasjon':'reparasjon',
'reparasjonane':'reparasjon',
'reparasjonar':'reparasjon',
'reparasjonsverksted':'reparasjonsverksted',
'reparere':'reparer',
'reparerer':'reparer',
'representant':'representant',
'representantane':'representant',
'representantar':'representant',
'representanten':'representant',
'representantene':'representant',
'representanter':'representant',
'representantskap':'representantskap',
'representantskapet':'representantskap',
'representantskapets':'representantskap',
'representasjon':'representasjon',
'representasjonen':'representasjon',
'representasjonsrett':'representasjonsrett',
'representativ':'representativ',
'representative':'representativ',
'representativitet':'representativit',
'representativt':'representativ',
'representer':'represent',
'representere':'representer',
'representerer':'representer',
'representert':'representer',
'representerte':'representer',
'rere':'rer',
'res':'res',
'reservar':'reserv',
'reservasjonen':'reservasjon',
'reservasjoner':'reservasjon',
'reservatets':'reservat',
'reserve':'reserv',
'reservefond':'reservefond',
'reserver':'reserv',
'reservere':'reserver',
'reserveregel':'reserveregel',
'reservert':'reserver',
'reserverte':'reserver',
'resolusjon':'resolusjon',
'respekt':'respekt',
'respekterast':'respekter',
'respektere':'respekter',
'respektert':'respekter',
'respektive':'respektiv',
'ressurs':'ressur',
'ressursane':'ressurs',
'ressursbruk':'ressursbruk',
'ressursbruken':'ressursbruk',
'ressurseffektiv':'ressurseffektiv',
'ressursene':'ressurs',
'ressurser':'ressurs',
'ressursinnsats':'ressursinnsat',
'ressurskrevende':'ressurskrev',
'ressursomsyna':'ressursomsyn',
'ressursutnyttelse':'ressursutnytt',
'ressursutnytting':'ressursutnytting',
'restansane':'restans',
'restansar':'restans',
'resten':'rest',
'restene':'rest',
'resterende':'rester',
'restgjelda':'restgjeld',
'restriksjonar':'restriksjon',
'restriksjoner':'restriksjon',
'restruktureringene':'restrukturering',
'resultat':'resultat',
'resultatavhengig':'resultatavheng',
'resultatene':'resultat',
'resultater':'resultat',
'resultatet':'resultat',
'resultatlikhet':'resultatlik',
'resultatmessig':'resultatmess',
'resultatmessige':'resultatmess',
'resultatregnskap':'resultatregnskap',
'resultatutviklinga':'resultatutvikling',
'resultere':'resulter',
'resulterer':'resulter',
'resursbruk':'resursbruk',
'retning':'retning',
'retninger':'retning',
'retningsbalanse':'retningsbalans',
'retningsbalansen':'retningsbalans',
'retningsgivende':'retningsgiv',
'retningsgiver':'retningsgiv',
'retningslina':'retningslin',
'retningslinene':'retningslin',
'retningsliner':'retningslin',
'retningslinje':'retningslinj',
'retningslinjene':'retningslinj',
'retningslinjer':'retningslinj',
'retningsubalanse':'retningsubalans',
'rett':'rett',
'retta':'rett',
'rettane':'rett',
'rettar':'rett',
'rettargangslovgjevinga':'rettargangslovgjeving',
'rettast':'rett',
'rettaste':'rettast',
'rette':'rett',
'rettede':'rett',
'retteleg':'rett',
'retten':'rett',
'retter':'rett',
'rettes':'rett',
'rettet':'rett',
'rettferd':'rettferd',
'rettferdig':'rettferd',
'rettferdighet':'rettferd',
'rettferdighetsideal':'rettferdighetsideal',
'rettigheten':'rett',
'rettighetene':'rett',
'rettigheter':'rett',
'rettighetsforholdene':'rettighetsforhold',
'rettighetshaver':'rettighetshav',
'rettighetshaveren':'rettighetshaver',
'rettighetsregister':'rettighetsregist',
'retting':'retting',
'rettinga':'retting',
'rettkjende':'rettkj',
'rettkomen':'rettkom',
'rettleiande':'rettlei',
'rettleiing':'rettleiing',
'rettsavgjerd':'rettsavgjerd',
'rettsavgjerder':'rettsavgjerd',
'rettserverv':'rettserverv',
'rettservervet':'rettserverv',
'rettsforhold':'rettsforhold',
'rettsforholda':'rettsforhold',
'rettsforholdet':'rettsforhold',
'rettsgebyr':'rettsgebyr',
'rettsgebyret':'rettsgebyr',
'rettsgebyrlova':'rettsgebyr',
'rettshavar':'rettshav',
'rettshavarane':'rettshavar',
'rettshavaren':'rettshavar',
'rettskraft':'rettskraft',
'rettskraftig':'rettskraft',
'rettskraftverknad':'rettskraftverknad',
'rettsleg':'retts',
'rettslege':'retts',
'rettslig':'retts',
'rettsmekling':'rettsmekling',
'rettsovergang':'rettsovergang',
'rettspraksis':'rettspraksis',
'rettsregister':'rettsregist',
'rettsregistera':'rettsregister',
'rettsregisteret':'rettsregister',
'rettsregistrering':'rettsregistrering',
'rettsregler':'rettsregl',
'rettssak':'rettssak',
'rettsspørsmål':'rettsspørsmål',
'rettsstiftelser':'rettsstift',
'rettsstifting':'rettsstifting',
'rettsstiftingane':'rettsstifting',
'rettsstiftingar':'rettsstifting',
'rettsstilling':'rettsstilling',
'rettsstillinga':'rettsstilling',
'rettsstoffet':'rettsstoff',
'rettssubjekt':'rettssubjekt',
'rettssubjekter':'rettssubjekt',
'rettstilstand':'rettstilstand',
'rettstilstanden':'rettstilstand',
'rettsuvisse':'rettsuviss',
'rettsverknadene':'rettsverknad',
'rettsverknader':'rettsverknad',
'rettsvern':'rettsvern',
'rettsvernakt':'rettsvernakt',
'rettsvernet':'rettsvern',
'rettsvernreglane':'rettsvernregl',
'rettsvirkninger':'rettsvirkning',
'returkommisjonar':'returkommisjon',
'returlast':'returl',
'returprovisjonar':'returprovisjon',
'returreisen':'returreis',
'rev':'rev',
'reversering':'reversering',
'reversibel':'reversibel',
'revestreker':'revestrek',
'revet':'rev',
'reviderast':'revider',
'revidert':'revider',
'reviderte':'revider',
'revierhavna':'revierhavn',
'revisjon':'revisjon',
'revisjonen':'revisjon',
'revisjonsmelding':'revisjonsmelding',
'revisjonsmeldinga':'revisjonsmelding',
'revisjonsplikt':'revisjonsplikt',
'revisor':'revisor',
'revisorar':'revisor',
'revisorers':'revisor',
'revisorfråsegn':'revisorfråsegn',
'revisorlova':'revisor',
'revisormelding':'revisormelding',
'revisoroppdraget':'revisoroppdrag',
'revisors':'revisor',
'revner':'revn',
'revurdering':'revurdering',
'ri':'ri',
'ridd':'ridd',
'ridekunst':'ridekunst',
'ridende':'rid',
'rideredskap':'rideredskap',
'ridetur':'ridetur',
'rig':'rig',
'rigger':'rigg',
'rigide':'rigid',
'rik':'rik',
'rikdom':'rikdom',
'rike':'rik',
'rikelig':'rik',
'rikeste':'rikest',
'riket':'rik',
'rikke':'rikk',
'riksbyggen':'riksbygg',
'riksdagens':'riksdag',
'riksdekkende':'riksdekk',
'riksforbund':'riksforbund',
'riksgrensen':'riksgrens',
'riksnivå':'riksnivå',
'rikspolitiske':'rikspolitisk',
'riksveg':'riksveg',
'riksveger':'riksveg',
'riksvegforbindelse':'riksvegforbind',
'riksvegnettet':'riksvegnett',
'riksvegplaner':'riksvegplan',
'riktig':'rikt',
'riktige':'rikt',
'riktignok':'riktignok',
'rim':'rim',
'rimeleg':'rim',
'rimelegare':'rimelegar',
'rimelege':'rim',
'rimelig':'rim',
'rimelige':'rim',
'rimelighet':'rim',
'rimet':'rim',
'rimfakse':'rimfaks',
'rimfrost':'rimfrost',
'rimgrimne':'rimgrimn',
'rimkledde':'rimkledd',
'rimlaget':'rimlag',
'rimslåtte':'rimslått',
'rimtussenes':'rimtuss',
'rimtusser':'rimtuss',
'rimtusseslekter':'rimtusseslekt',
'rind':'rind',
'rinds':'rind',
'ring':'ring',
'ringdal':'ringdal',
'ringen':'ring',
'ringer':'ring',
'ringhorne':'ringhorn',
'ringveien':'ringvei',
'ringvirkninger':'ringvirkning',
'rir':'rir',
'rise':'ris',
'riser':'ris',
'risikabel':'risikabel',
'risikabelt':'risikabelt',
'risikere':'risiker',
'risikerer':'risiker',
'risiko':'risiko',
'risikoaksept':'risikoaksept',
'risikoberegning':'risikoberegning',
'risikobetont':'risikobetont',
'risikoelement':'risikoelement',
'risikoen':'risiko',
'risikofond':'risikofond',
'risikofondet':'risikofond',
'risikoforhold':'risikoforhold',
'risikoforsikringer':'risikoforsikring',
'risikofri':'risikofri',
'risikofrie':'risikofri',
'risikofylt':'risikofylt',
'risikofylte':'risikofylt',
'risikokapital':'risikokapital',
'risikokapitalen':'risikokapital',
'risikomessig':'risikomess',
'risikomomentet':'risikomoment',
'risikomåling':'risikomåling',
'risikonivået':'risikonivå',
'risikoplassering':'risikoplassering',
'risikopremie':'risikopremi',
'risikopremier':'risikopremi',
'risikoprofil':'risikoprofil',
'risikospredning':'risikospredning',
'risikotaking':'risikotaking',
'risikoviljug':'risikoviljug',
'risikovillig':'risikovil',
'risikovurderinger':'risikovurdering',
'risikoytelser':'risikoyt',
'risk':'risk',
'risp':'risp',
'risser':'riss',
'risset':'riss',
'riste':'rist',
'rister':'rist',
'ristet':'rist',
'risting':'risting',
'rive':'riv',
'river':'riv',
'rives':'riv',
'rivning':'rivning',
'ro':'ro',
'roar':'roar',
'robuste':'robust',
'robusthet':'robust',
'rockwool':'rockwool',
'rodd':'rodd',
'roende':'roend',
'rogaland':'rogaland',
'rognebusk':'rognebusk',
'rokvam':'rokvam',
'rolig':'rol',
'roligheten':'rol',
'roll':'roll',
'rolla':'roll',
'rolle':'roll',
'rolledelingen':'rolledeling',
'rollefordeling':'rollefordeling',
'rollen':'roll',
'roller':'roll',
'rom':'rom',
'romerike':'romerik',
'romerrike':'romerrik',
'rommer':'romm',
'rommet':'romm',
'romsdal':'romsdal',
'romslig':'roms',
'romslige':'roms',
'rop':'rop',
'rope':'rop',
'roper':'rop',
'ropt':'ropt',
'ropte':'ropt',
'ror':'ror',
'roret':'ror',
'ros':'ros',
'roser':'ros',
'roses':'ros',
'roskva':'roskv',
'roskvas':'roskv',
'rostiophus':'rostiophus',
'rostratt':'rostratt',
'rot':'rot',
'rote':'rot',
'roten':'rot',
'roter':'rot',
'roterminalene':'roterminal',
'roteskadar':'roteskad',
'rotet':'rot',
'rotterdam':'rotterdam',
'rovdyrtenner':'rovdyrtenn',
'rpr':'rpr',
'rprs':'rprs',
'rt':'rt',
'rufsete':'rufset',
'ruger':'rug',
'rugger':'rugg',
'rugtvedt':'rugtved',
'ruiner':'ruin',
'rukket':'rukk',
'rullende':'rull',
'ruller':'rull',
'rullering':'rullering',
'rullestenene':'rullesten',
'rullestokkene':'rullestokk',
'rumpe':'rump',
'rund':'rund',
'rundar':'rund',
'runde':'rund',
'rundt':'rund',
'rundtur':'rundtur',
'rundturer':'rundtur',
'runekunst':'runekunst',
'runer':'run',
'runetegnene':'runetegn',
'runger':'rung',
'rungne':'rungn',
'rungnes':'rungn',
'rushtrafikken':'rushtrafikk',
'rusk':'rusk',
'rusproblem':'rusproblem',
'russiske':'russisk',
'russland':'russland',
'ruste':'rust',
'rustet':'rust',
'rustfarget':'rustfarg',
'rustne':'rustn',
'rute':'rut',
'rutemønster':'rutemønst',
'rutemønstre':'rutemønstr',
'ruten':'rut',
'rutene':'rut',
'rutenes':'rut',
'rutenettet':'rutenett',
'ruteopplegg':'ruteopplegg',
'rutepapir':'rutepapir',
'ruter':'rut',
'rutetilbud':'rutetilbud',
'rutetilbudet':'rutetilbud',
'rutinane':'rutin',
'rutinar':'rutin',
'rutine':'rutin',
'ruver':'ruv',
'rydda':'rydd',
'rydder':'rydd',
'ryddet':'rydd',
'ryen':'ryen',
'rygge':'rygg',
'ryggen':'rygg',
'rygger':'rygg',
'ryke':'ryk',
'ryker':'ryk',
'rykk':'rykk',
'rykke':'rykk',
'rykker':'rykk',
'rykket':'rykk',
'rykter':'rykt',
'ryktet':'rykt',
'rynke':'rynk',
'rynkede':'rynk',
'rynkene':'rynk',
'rynker':'rynk',
'rynket':'rynk',
'rynkete':'rynket',
'ryster':'ryst',
'rytter':'rytt',
'ryttere':'rytter',
'rytteren':'rytter',
'rytterne':'ryttern',
'rå':'rå',
'råd':'råd',
'rådde':'rådd',
'råde':'råd',
'rådene':'råd',
'råder':'råd',
'råderett':'råderett',
'råderetten':'råderett',
'råderettsforbod':'råderettsforbod',
'rådet':'råd',
'rådgivende':'rådgiv',
'rådgiver':'rådgiv',
'rådgivere':'rådgiver',
'rådgivning':'rådgivning',
'rådgjevar':'rådgjev',
'rådgjevaren':'rådgjevar',
'rådgjevinga':'rådgjeving',
'rådighet':'råd',
'rådsdirektiv':'rådsdirektiv',
'rådslagning':'rådslagning',
'rådsmøte':'rådsmøt',
'rådsmøter':'rådsmøt',
'rådsvedtak':'rådsvedtak',
'rådvill':'rådvill',
'råk':'råk',
'råka':'råk',
'råkar':'råk',
'råkast':'råk',
'råmaterialet':'råmaterial',
'rån':'rån',
'rår':'rår',
'råsterke':'råsterk',
'råstoff':'råstoff',
'råstoffer':'råstoff',
'råtne':'råtn',
'råtner':'råtn',
'rått':'rått',
'råtten':'rått',
'råvareinntak':'råvareinntak',
'råvarer':'råvar',
'ræsvelg':'ræsvelg',
'rød':'rød',
'røde':'rød',
'rødeste':'rødest',
'rødhåret':'rødhår',
'rødmer':'rødm',
'rødskjeggete':'rødskjegget',
'rødsprengt':'rødsprengt',
'rødt':'rødt',
'røk':'røk',
'rømme':'rømm',
'rømmer':'rømm',
'røpe':'røp',
'røper':'røp',
'røre':'rør',
'rører':'rør',
'rørt':'rørt',
'rørte':'rørt',
'røsker':'røsk',
'røttene':'røtt',
'røtter':'røtt',
'røve':'røv',
'røver':'røv',
'røvet':'røv',
'røyken':'røyk',
'røynsla':'røynsl',
'røynsle':'røynsl',
'røynslene':'røynsl',
'røynsler':'røynsl',
'røyr':'røyr',
'røys':'røys',
'røyst':'røyst',
'røysta':'røyst',
'røystar':'røyst',
'røyste':'røyst',
'røysteforklåringar':'røysteforklåring',
'røystene':'røyst',
'røyster':'røyst',
'røystereglane':'røysteregl',
'røystereglar':'røysteregl',
'røysterett':'røysterett',
'røysteretten':'røysterett',
'røystetal':'røystetal',
'røystetalet':'røystetal',
'røysting':'røysting',
'røystinga':'røysting',
'røystingane':'røysting',
's':'s',
'sa':'sa',
'sabo':'sabo',
'sabotere':'saboter',
'sadeltøy':'sadeltøy',
'saf':'saf',
'safir':'safir',
'sagd':'sagd',
'sagt':'sagt',
'sak':'sak',
'saka':'sak',
'saken':'sak',
'sakene':'sak',
'saker':'sak',
'sakleg':'sak',
'saklege':'sak',
'saklig':'sak',
'saklige':'sak',
'saklista':'saklist',
'sakliste':'saklist',
'saksbehandler':'saksbehandl',
'saksbehandling':'saksbehandling',
'saksbehandlinga':'saksbehandling',
'saksbehandlingsfeil':'saksbehandlingsfeil',
'saksbehandlingsregelen':'saksbehandlingsregel',
'saksbehandlingsregler':'saksbehandlingsregl',
'saksdokument':'saksdokument',
'saksforberedelse':'saksforbered',
'saksforutsetninger':'saksforutsetning',
'saksområde':'saksområd',
'sakspapir':'sakspapir',
'saksøkerens':'saksøker',
'saksøkjarane':'saksøkjar',
'saksøkjaren':'saksøkjar',
'saksøkte':'saksøkt',
'sakte':'sakt',
'saktens':'sakt',
'sal':'sal',
'saldo':'saldo',
'salen':'sal',
'saler':'sal',
'sales':'sal',
'salet':'sal',
'salg':'salg',
'salget':'salg',
'salgsavtaler':'salgsavtal',
'salgsinntektene':'salgsinntekt',
'salgsinntekter':'salgsinntekt',
'salslag':'salslag',
'salsoppdrag':'salsoppdrag',
'salspålegg':'salspålegg',
'salspålegget':'salspålegg',
'salstilfella':'salstilfell',
'salsverdi':'salsverdi',
'salsverdien':'salsverdi',
'salt':'salt',
'saltaskene':'saltask',
'salte':'salt',
'salve':'salv',
'salver':'salv',
'saman':'saman',
'samanblanding':'samanblanding',
'samandrag':'samandrag',
'samanhalde':'samanhald',
'samanhalden':'samanhald',
'samanheng':'samanheng',
'samanhengar':'samanheng',
'samanhengen':'samanheng',
'samanhengens':'samanheng',
'samanknytinga':'samanknyting',
'samanlikna':'samanlikn',
'samanliknar':'samanlikn',
'samanliknast':'samanlikn',
'samanlikne':'samanlikn',
'samanlikning':'samanlikning',
'samanlikninga':'samanlikning',
'samansett':'samansett',
'samansette':'samansett',
'samanskriving':'samanskriving',
'samanslegne':'samanslegn',
'samanslutning':'samanslutning',
'samanslutninga':'samanslutning',
'samanslutningane':'samanslutning',
'samanslutningar':'samanslutning',
'samanslutningsforma':'samanslutningsform',
'samanslutningsformene':'samanslutningsform',
'samanslutningsrettsleg':'samanslutningsretts',
'samanslutningsrettslege':'samanslutningsretts',
'samanslåing':'samanslåing',
'samanslåinga':'samanslåing',
'samanslåingar':'samanslåing',
'samarbeid':'samarbeid',
'samarbeidd':'samarbeidd',
'samarbeide':'samarbeid',
'samarbeidende':'samarbeid',
'samarbeider':'samarbeid',
'samarbeidet':'samarbeid',
'samarbeidets':'samarbeid',
'samarbeidsforhold':'samarbeidsforhold',
'samarbeidsformen':'samarbeidsform',
'samarbeidsformer':'samarbeidsform',
'samarbeidskommune':'samarbeidskommun',
'samarbeidskommunene':'samarbeidskommun',
'samarbeidskommunenes':'samarbeidskommun',
'samarbeidsløsninger':'samarbeidsløsning',
'samarbeidsmodeller':'samarbeidsmodell',
'samarbeidsområdet':'samarbeidsområd',
'samarbeidsorganer':'samarbeidsorgan',
'samarbeidsorganet':'samarbeidsorgan',
'samarbeidsorganisasjon':'samarbeidsorganisasjon',
'samarbeidspartnar':'samarbeidspartn',
'samarbeidspartnarar':'samarbeidspartnar',
'samarbeidstiltak':'samarbeidstiltak',
'samarbeidsutval':'samarbeidsutval',
'samarbeidsutvala':'samarbeidsutval',
'samarbeidsutvalet':'samarbeidsutval',
'samarbeidt':'samarbeid',
'samband':'samband',
'sambandskomite':'sambandskomit',
'samboer':'sambo',
'sambuar':'sambu',
'sambuarar':'sambuar',
'sambuarforholdet':'sambuarforhold',
'sambuarskap':'sambuarskap',
'samd':'samd',
'samde':'samd',
'same':'sam',
'sameie':'samei',
'sameieforholdet':'sameieforhold',
'sameiermøtet':'sameiermøt',
'sameiets':'samei',
'sameiga':'sam',
'sameigar':'sam',
'sameigarane':'sameigar',
'sameigarar':'sameigar',
'sameigaren':'sameigar',
'sameigarmøte':'sameigarmøt',
'sameigarmøtet':'sameigarmøt',
'sameige':'sam',
'sameigeforhold':'sameigeforhold',
'sameigelova':'sameig',
'sameigepart':'sameigepart',
'sameigepartane':'sameigepart',
'sameigepartar':'sameigepart',
'sameigeparten':'sameigepart',
'sameiger':'sam',
'sameigevedtektene':'sameigevedtekt',
'sameigevedtekter':'sameigevedtekt',
'sameinast':'samein',
'sameine':'samein',
'sameleis':'sameleis',
'samferdsel':'samferdsel',
'samferdsels':'samferdsel',
'samferdselsdepartement':'samferdselsdepartement',
'samferdselsdepartementet':'samferdselsdepartement',
'samferdselskomiteen':'samferdselskomite',
'samferdselsløsninger':'samferdselsløsning',
'samferdselsmessig':'samferdselsmess',
'samferdselsmessige':'samferdselsmess',
'samferdselsmønsteret':'samferdselsmønster',
'samferdselsnett':'samferdselsnett',
'samferdselsnettet':'samferdselsnett',
'samferdselsplanlegging':'samferdselsplanlegging',
'samferdselspolitikken':'samferdselspolitikk',
'samferdselspolitisk':'samferdselspolitisk',
'samferdselspolitiske':'samferdselspolitisk',
'samferdselssektoren':'samferdselssektor',
'samferdselstilbudet':'samferdselstilbud',
'samfunn':'samfunn',
'samfunnet':'samfunn',
'samfunnets':'samfunn',
'samfunns':'samfunn',
'samfunnsansvar':'samfunnsansv',
'samfunnsforskning':'samfunnsforskning',
'samfunnshensyn':'samfunnshensyn',
'samfunnsinteresser':'samfunnsinteress',
'samfunnsmessige':'samfunnsmess',
'samfunnsnyttig':'samfunnsnytt',
'samfunnsøkonomisk':'samfunnsøkonomisk',
'samfunnsøkonomiske':'samfunnsøkonomisk',
'samhandel':'samhandel',
'samhandelen':'samhandel',
'samhandlar':'samhandl',
'samhandle':'samhandl',
'samkjøringsgevinster':'samkjøringsgevinst',
'samkvem':'samkvem',
'samla':'saml',
'samlande':'saml',
'samlar':'saml',
'samlast':'saml',
'samlastere':'samlaster',
'samlasterne':'samlastern',
'samlasternes':'samlastern',
'samlastgods':'samlastgod',
'samlasting':'samlasting',
'samlastsentralene':'samlastsentral',
'samlastterminal':'samlastterminal',
'samlastterminalene':'samlastterminal',
'samlastterminaler':'samlastterminal',
'samle':'saml',
'samlede':'saml',
'samler':'saml',
'samles':'saml',
'samleseksjon':'samleseksjon',
'samlet':'saml',
'samling':'samling',
'samlivsbrot':'samlivsbrot',
'samlokalisering':'samlokalisering',
'samlokaliseringen':'samlokalisering',
'samme':'samm',
'sammelignbare':'sammelignbar',
'sammen':'samm',
'sammenblanding':'sammenblanding',
'sammendrag':'sammendrag',
'sammendraget':'sammendrag',
'sammenfall':'sammenfall',
'sammenfallende':'sammenfall',
'sammenheng':'sammenheng',
'sammenhengen':'sammenheng',
'sammenhengende':'sammenheng',
'sammenhenger':'sammenheng',
'sammenholder':'sammenhold',
'sammenholdt':'sammenhold',
'sammenknytning':'sammenknytning',
'sammenlagt':'sammenlagt',
'sammenlignbar':'sammenlignb',
'sammenlignbare':'sammenlignbar',
'sammenlignbarhet':'sammenlignbar',
'sammenligne':'sammenlign',
'sammenligner':'sammenlign',
'sammenlignet':'sammenlign',
'sammenligning':'sammenligning',
'sammenligninger':'sammenligning',
'sammenliknbare':'sammenliknbar',
'sammenlikne':'sammenlikn',
'sammenlikner':'sammenlikn',
'sammenliknet':'sammenlikn',
'sammenlikning':'sammenlikning',
'sammenlikningen':'sammenlikning',
'sammenlikninger':'sammenlikning',
'sammensatt':'sammensatt',
'sammensatte':'sammensatt',
'sammensetning':'sammensetning',
'sammensetningen':'sammensetning',
'sammensettes':'sammensett',
'sammenslutning':'sammenslutning',
'sammenslutningen':'sammenslutning',
'sammenslutninger':'sammenslutning',
'sammenslåing':'sammenslåing',
'sammenslåingen':'sammenslåing',
'sammenslåinger':'sammenslåing',
'sammenslåingsprosess':'sammenslåingsprosess',
'sammenslåingsvariant':'sammenslåingsvariant',
'sammenslåtte':'sammenslått',
'sammenstilling':'sammenstilling',
'sammenstøtet':'sammenstøt',
'samordna':'samordn',
'samordnast':'samordn',
'samordne':'samordn',
'samordnende':'samordn',
'samordner':'samordn',
'samordnet':'samordn',
'samordning':'samordning',
'samordninga':'samordning',
'samordningen':'samordning',
'samråd':'samråd',
'samrøystes':'samrøyst',
'sams':'sam',
'samspel':'samspel',
'samspelet':'samspel',
'samspill':'samspill',
'samstundes':'samstund',
'samsvar':'samsv',
'samsvarar':'samsvar',
'samsvarer':'samsvar',
'samsvaret':'samsvar',
'samsøy':'samsøy',
'samt':'samt',
'samtale':'samtal',
'samtaler':'samtal',
'samtidig':'samtid',
'samtidige':'samtid',
'samtlige':'samt',
'samtykka':'samtykk',
'samtykke':'samtykk',
'samtykkeregelen':'samtykkeregel',
'samtykket':'samtykk',
'samtykkje':'samtykkj',
'samtykkjer':'samtykkj',
'samtykt':'samtykt',
'samvariasjon':'samvariasjon',
'samverkan':'samverkan',
'samvirka':'samvirk',
'samvirke':'samvirk',
'samvirkeboka':'samvirkebok',
'samvirkeforma':'samvirkeform',
'samvirkeføretak':'samvirkeføretak',
'samvirkeføretaka':'samvirkeføretak',
'samvirkeføretaket':'samvirkeføretak',
'samvirkelag':'samvirkelag',
'samvirkelaga':'samvirkelag',
'samvirkelaget':'samvirkelag',
'samvirkelov':'samvirk',
'samvirkelovutvalet':'samvirkelovutval',
'samvirkeorganisasjon':'samvirkeorganisasjon',
'samvirkeorganisasjonane':'samvirkeorganisasjon',
'samvirkeorganisasjonar':'samvirkeorganisasjon',
'samvirkeorganisering':'samvirkeorganisering',
'samvirkepreget':'samvirkepreg',
'samvirkeprinsipp':'samvirkeprinsipp',
'samvirkeprinsippa':'samvirkeprinsipp',
'samvirkeprinsipper':'samvirkeprinsipp',
'samvirkerørsla':'samvirkerørsl',
'samvirkesamanheng':'samvirkesamanheng',
'samvirket':'samvirk',
'samvirketiltak':'samvirketiltak',
'samvirketiltaka':'samvirketiltak',
'samvirkeutvalget':'samvirkeutvalg',
'samvittighet':'samvitt',
'sand':'sand',
'sandefjord':'sandefjord',
'sandessjøen':'sandessjø',
'sandnessjøen':'sandnessjø',
'sandstrand':'sandstrand',
'sanerast':'saner',
'sang':'sang',
'sanger':'sang',
'sanksjoner':'sanksjon',
'sann':'sann',
'sannelig':'sann',
'sannhet':'sann',
'sannheten':'sann',
'sannsynlig':'sannsyn',
'sannsynlige':'sannsyn',
'sannsynliggjøre':'sannsynliggjør',
'sannsynlighet':'sannsyn',
'sannsynligheten':'sannsyn',
'sannsynligvis':'sannsynligvis',
'sans':'san',
'sanser':'sans',
'sant':'sant',
'sarpsborg':'sarpsborg',
'sarskild':'sarskild',
'sarskilda':'sarskild',
'sats':'sat',
'satsa':'sats',
'satse':'sats',
'satsen':'sats',
'satsene':'sats',
'satser':'sats',
'satses':'sats',
'satset':'sats',
'satsing':'satsing',
'satsingsområde':'satsingsområd',
'satsstruktur':'satsstruktur',
'satsstrukturen':'satsstruktur',
'satsstrukturene':'satsstruktur',
'satsstrukturer':'satsstruktur',
'satt':'satt',
'satte':'satt',
'sauene':'sauen',
'sauer':'sauer',
'saugbrugsforeningen':'saugbrugsforening',
'saugbruksforeningen':'saugbruksforening',
'sauholmene':'sauholm',
'sauøya':'sauøya',
'savner':'savn',
'sbc':'sbc',
'scan':'scan',
'scanproof':'scanproof',
'scc':'scc',
'scenariebeskrivelsene':'scenariebeskriv',
'scenarier':'scenari',
'scenario':'scenario',
'schwencke':'schwenck',
'schøne':'schøn',
'score':'scor',
'se':'se',
'sea':'sea',
'sedd':'sedd',
'sedde':'sedd',
'sees':'sees',
'seg':'seg',
'segment':'segment',
'segmenter':'segment',
'segner':'segn',
'seiast':'seiast',
'seid':'seid',
'seie':'seie',
'seier':'seier',
'seil':'seil',
'seilas':'seil',
'seilbare':'seilbar',
'seile':'seil',
'seilene':'seil',
'seiler':'seil',
'seilet':'seil',
'seilingsforhold':'seilingsforhold',
'seilingsopplegg':'seilingsopplegg',
'seilingsreglene':'seilingsregl',
'seilingsregler':'seilingsregl',
'seilingstider':'seilingstid',
'seilt':'seilt',
'sein':'sein',
'seinare':'seinar',
'seinast':'sein',
'seinaste':'seinast',
'seinere':'seiner',
'seinke':'seink',
'seint':'seint',
'seire':'seir',
'seiren':'seir',
'seirens':'seir',
'seirer':'seir',
'seiret':'seir',
'sek':'sek',
'sekk':'sekk',
'sekken':'sekk',
'sekretariat':'sekretariat',
'sekretariatet':'sekretariat',
'sekretær':'sekretær',
'seks':'seks',
'seksjon':'seksjon',
'seksjonane':'seksjon',
'seksjonar':'seksjon',
'seksjonen':'seksjon',
'seksjoner':'seksjon',
'seksjonering':'seksjonering',
'seksjoneringa':'seksjonering',
'seksjonerte':'seksjoner',
'seksjonsameiga':'seksjonsam',
'seksjonseigarane':'seksjonseigar',
'seksjonseigarar':'seksjonseigar',
'seksjonseigaren':'seksjonseigar',
'seksjonsforma':'seksjonsform',
'seksjonssameier':'seksjonssamei',
'seksjonssameiga':'seksjonssam',
'seksjonssameigarar':'seksjonssameigar',
'seksjonssameige':'seksjonssam',
'seksjonssameigene':'seksjonssam',
'seksjonssameiger':'seksjonssam',
'seksjonssjef':'seksjonssjef',
'sektor':'sektor',
'sektorar':'sektor',
'sektoren':'sektor',
'sektorer':'sektor',
'sektors':'sektor',
'sektorvise':'sektorvis',
'sekundærpantsetjing':'sekundærpantsetjing',
'sekundært':'sekundært',
'sel':'sel',
'seld':'seld',
'selde':'seld',
'seleksjon':'seleksjon',
'seleksjonsproblemene':'seleksjonsproblem',
'seleksjonsproblemer':'seleksjonsproblem',
'selene':'sel',
'seler':'sel',
'selge':'selg',
'selges':'selg',
'seljar':'selj',
'seljaransvar':'seljaransv',
'seljarar':'seljar',
'seljaren':'seljar',
'seljast':'selj',
'selje':'selj',
'seljefløyte':'seljefløyt',
'selskabsformerne':'selskabsformern',
'selskap':'selskap',
'selskapa':'selskap',
'selskapene':'selskap',
'selskapenes':'selskap',
'selskaper':'selskap',
'selskapers':'selskap',
'selskapet':'selskap',
'selskapets':'selskap',
'selskaps':'selskap',
'selskapsavtalen':'selskapsavtal',
'selskapsdirektiv':'selskapsdirektiv',
'selskapsendringer':'selskapsendring',
'selskapsform':'selskapsform',
'selskapsformer':'selskapsform',
'selskapskreditorar':'selskapskreditor',
'selskapslova':'selskap',
'selskapsloven':'selskap',
'selskapspartane':'selskapspart',
'selskapspartar':'selskapspart',
'selskapsrettsleg':'selskapsretts',
'selskapsrettslege':'selskapsretts',
'selskapsrettslige':'selskapsretts',
'selskapsskattelov':'selskapsskatt',
'selskapsskatteloven':'selskapsskatt',
'selt':'selt',
'selv':'selv',
'selve':'selv',
'selveste':'selvest',
'selvfinansierende':'selvfinansier',
'selvforsterkende':'selvforsterk',
'selvfølgelig':'selvfølg',
'selvig':'selv',
'selvsagt':'selvsagt',
'selvsikkert':'selvsikker',
'selvstendig':'selvstend',
'selvstendige':'selvstend',
'selvstyre':'selvstyr',
'sement':'sement',
'semitrailer':'semitrail',
'semitrailere':'semitrailer',
'sen':'sen',
'send':'send',
'sendast':'send',
'sende':'send',
'sendebud':'sendebud',
'sendebudene':'sendebud',
'sender':'send',
'sendes':'send',
'sendingsstørrelser':'sendingsstørr',
'sendt':'send',
'sendte':'send',
'sener':'sen',
'senere':'sener',
'senest':'senest',
'seneste':'senest',
'seng':'seng',
'sengehalmen':'sengehalm',
'sengekammer':'sengekamm',
'sengen':'seng',
'senger':'seng',
'sengestolpen':'sengestolp',
'sengestolpene':'sengestolp',
'sengs':'seng',
'seniorrådgiver':'seniorrådgiv',
'seniorrådgjevar':'seniorrådgjev',
'sent':'sent',
'senter':'sent',
'senterpartiet':'senterparti',
'sentral':'sentral',
'sentralbyrå':'sentralbyrå',
'sentralbyrås':'sentralbyrås',
'sentrale':'sentral',
'sentralene':'sentral',
'sentraleuropeiske':'sentraleuropeisk',
'sentralforbund':'sentralforbund',
'sentralhavn':'sentralhavn',
'sentralhavna':'sentralhavn',
'sentralisering':'sentralisering',
'sentraliseringseffekt':'sentraliseringseffekt',
'sentraliseringskrefter':'sentraliseringskreft',
'sentraliseringstendens':'sentraliseringstend',
'sentraliseringstendenser':'sentraliseringstendens',
'sentralisert':'sentraliser',
'sentrallager':'sentrallag',
'sentrallagre':'sentrallagr',
'sentralt':'sentralt',
'sentrum':'sentrum',
'sentrumsområdene':'sentrumsområd',
'sentrumsplanen':'sentrumsplan',
'separasjon':'separasjon',
'separasjonen':'separasjon',
'separat':'separat',
'separate':'separat',
'separerte':'separer',
'september':'septemb',
'ser':'ser',
'sertifikat':'sertifikat',
'sertifikater':'sertifikat',
'service':'servic',
'servicehavn':'servicehavn',
'servicenæringens':'servicenæring',
'services':'servic',
'servituttar':'servitutt',
'ses':'ses',
'sesong':'sesong',
'sesongarbeider':'sesongarbeid',
'sesongarbeidere':'sesongarbeider',
'sesongarbeideres':'sesongarbeider',
'sessrymne':'sessrymn',
'set':'set',
'sete':'set',
'setjast':'setj',
'setje':'setj',
'setning':'setning',
'setninger':'setning',
'setningsskadar':'setningsskad',
'sett':'sett',
'sette':'sett',
'setter':'sett',
'settes':'sett',
'sevjen':'sevj',
'shapiro':'shapiro',
'si':'si',
'sia':'sia',
'sida':'sid',
'side':'sid',
'sidelastere':'sidelaster',
'siden':'sid',
'sidene':'sid',
'sideportsbåter':'sideportsbåt',
'sider':'sid',
'siderom':'siderom',
'siderommet':'sideromm',
'sidespor':'sidespor',
'sidesprang':'sidesprang',
'sidestilles':'sidestill',
'sier':'sier',
'sies':'sies',
'siffernivå':'siffernivå',
'sige':'sig',
'signal':'signal',
'signaler':'signal',
'signalisere':'signaliser',
'signifikant':'signifikant',
'sigurd':'sigurd',
'sigyn':'sigyn',
'sikatomta':'sikatomt',
'sikker':'sikk',
'sikkerhet':'sikker',
'sikkerheten':'sikker',
'sikkerhets':'sikkerh',
'sikkerhetsgebyr':'sikkerhetsgebyr',
'sikkerhetsmessige':'sikkerhetsmess',
'sikkerhetsnivå':'sikkerhetsnivå',
'sikkerhetssystemet':'sikkerhetssystem',
'sikkert':'sikker',
'siklende':'sikl',
'sikler':'sikl',
'sikra':'sikr',
'sikrar':'sikr',
'sikrare':'sikrar',
'sikrast':'sikr',
'sikre':'sikr',
'sikrer':'sikr',
'sikrere':'sikrer',
'sikres':'sikr',
'sikreste':'sikrest',
'sikret':'sikr',
'sikring':'sikring',
'sikringsordning':'sikringsordning',
'sikringsordningane':'sikringsordning',
'sikringsordningar':'sikringsordning',
'sikt':'sikt',
'sikta':'sikt',
'siktbegrensninger':'siktbegrensning',
'sikte':'sikt',
'siktemål':'siktemål',
'siktemålet':'siktemål',
'sikter':'sikt',
'siktes':'sikt',
'siljan':'siljan',
'silkebånd':'silkebånd',
'simpelt':'simpelt',
'simultanstifting':'simultanstifting',
'sin':'sin',
'sindige':'sind',
'sindre':'sindr',
'sine':'sin',
'singasteinen':'singastein',
'singel':'singel',
'single':'singl',
'sinne':'sinn',
'sinnet':'sinn',
'sinns':'sinn',
'sint':'sint',
'sinte':'sint',
'sintere':'sinter',
'sirkelen':'sirkel',
'sirkulasjon':'sirkulasjon',
'sist':'sist',
'siste':'sist',
'sistnemnde':'sistnemnd',
'sistnevnte':'sistnevnt',
'sit':'sit',
'sitatet':'sitat',
'sitc':'sitc',
'sitert':'siter',
'siterte':'siter',
'sitjande':'sitj',
'sitje':'sitj',
'sitt':'sitt',
'sitte':'sitt',
'sittende':'sitt',
'sitter':'sitt',
'sittet':'sitt',
'situasjon':'situasjon',
'situasjonar':'situasjon',
'situasjonen':'situasjon',
'situasjoner':'situasjon',
'situasjonsbeskrivelse':'situasjonsbeskriv',
'siv':'siv',
'sivil':'sivil',
'sivile':'sivil',
'sivillovboka':'sivillovbok',
'sjablonmessig':'sjablonmess',
'sjalu':'sjalu',
'sjansen':'sjans',
'sjanser':'sjans',
'sjavn':'sjavn',
'sjekta':'sjekt',
'sjel':'sjel',
'sjeldan':'sjeldan',
'sjelden':'sjeld',
'sjeldnare':'sjeldnar',
'sjeldne':'sjeldn',
'sjeldnere':'sjeldner',
'sjelen':'sjel',
'sjenanse':'sjenans',
'sjenert':'sjener',
'sjette':'sjett',
'sjettedel':'sjettedel',
'sjikanering':'sjikanering',
'sju':'sju',
'sjuande':'sjuand',
'sjukdom':'sjukdom',
'sjuke':'sjuk',
'sjukeheimar':'sjukeheim',
'sjursøya':'sjursøy',
'sjursøyas':'sjursøy',
'sjå':'sjå',
'sjåast':'sjåast',
'sjælland':'sjælland',
'sjø':'sjø',
'sjøarealer':'sjøareal',
'sjødyrene':'sjødyr',
'sjøen':'sjøen',
'sjøfarende':'sjøfar',
'sjøfart':'sjøfart',
'sjøfartsdirektoratet':'sjøfartsdirektorat',
'sjøfolk':'sjøfolk',
'sjøfraktene':'sjøfrakt',
'sjøjotner':'sjøjotn',
'sjøkantarealer':'sjøkantareal',
'sjøkantbaserte':'sjøkantbaser',
'sjøkantlokalisert':'sjøkantlokaliser',
'sjøkantlokaliserte':'sjøkantlokaliser',
'sjøkart':'sjøkart',
'sjøkonge':'sjøkong',
'sjøkongen':'sjøkong',
'sjølv':'sjølv',
'sjølvdømme':'sjølvdømm',
'sjølve':'sjølv',
'sjølveigande':'sjølv',
'sjølveigarar':'sjølveigar',
'sjølvhjelp':'sjølvhjelp',
'sjølvhjelpsorganisasjonar':'sjølvhjelpsorganisasjon',
'sjølvhjelpstanken':'sjølvhjelpstank',
'sjølvkost':'sjølvkost',
'sjølvkostprinsipp':'sjølvkostprinsipp',
'sjølvkostprinsippet':'sjølvkostprinsipp',
'sjølvmelding':'sjølvmelding',
'sjølvmotseiande':'sjølvmotsei',
'sjølvsagt':'sjølvsagt',
'sjølvskiftande':'sjølvskift',
'sjølvskuldgaranti':'sjølvskuldgaranti',
'sjølvstendig':'sjølvstend',
'sjølvstendige':'sjølvstend',
'sjølvstyre':'sjølvstyr',
'sjølvstyret':'sjølvstyr',
'sjølvstyrt':'sjølvstyrt',
'sjølvstyrte':'sjølvstyrt',
'sjøområdene':'sjøområd',
'sjøområder':'sjøområd',
'sjøreise':'sjøreis',
'sjørholt':'sjørholt',
'sjøs':'sjøs',
'sjøside':'sjøsid',
'sjøsiden':'sjøsid',
'sjøsidens':'sjøsid',
'sjøsideområder':'sjøsideområd',
'sjøtrafikk':'sjøtrafikk',
'sjøtrafikken':'sjøtrafikk',
'sjøtransport':'sjøtransport',
'sjøtransportarbeidet':'sjøtransportarbeid',
'sjøtransporten':'sjøtransport',
'sjøtransportens':'sjøtransport',
'sjøtransporter':'sjøtransport',
'sjøtransportkorridorer':'sjøtransportkorridor',
'sjøtransportløsninger':'sjøtransportløsning',
'sjøtransportpolitikk':'sjøtransportpolitikk',
'sjøtransporttilbud':'sjøtransporttilbud',
'sjøtransporttilbudet':'sjøtransporttilbud',
'sjøvegen':'sjøveg',
'sjøvegs':'sjøveg',
'sjøveien':'sjøvei',
'sjøverts':'sjøvert',
'ska':'ska',
'skabbete':'skabbet',
'skadane':'skad',
'skadar':'skad',
'skadd':'skadd',
'skadde':'skadd',
'skade':'skad',
'skadebot':'skadebot',
'skadebotansvar':'skadebotansv',
'skadebotansvaret':'skadebotansvar',
'skadebotansvarleg':'skadebotansvar',
'skadebotansvarlege':'skadebotansvar',
'skadebotkrav':'skadebotkrav',
'skadebotkravet':'skadebotkrav',
'skadebotlova':'skadebot',
'skadebotregel':'skadebotregel',
'skadebotregelen':'skadebotregel',
'skadebotreglane':'skadebotregl',
'skadebotreglar':'skadebotregl',
'skadefryd':'skadefryd',
'skadelidande':'skadelid',
'skadelidne':'skadelidn',
'skadelidte':'skadelid',
'skaden':'skad',
'skadens':'skad',
'skadeoppgjer':'skadeoppgj',
'skader':'skad',
'skadet':'skad',
'skadeutvikling':'skadeutvikling',
'skaff':'skaff',
'skaffa':'skaff',
'skaffar':'skaff',
'skaffast':'skaff',
'skaffe':'skaff',
'skaffer':'skaff',
'skaffet':'skaff',
'skaftet':'skaft',
'skal':'skal',
'skala':'skal',
'skalaen':'skala',
'skaldekunst':'skaldekunst',
'skaldemjøden':'skaldemjød',
'skalder':'skald',
'skalle':'skall',
'skallen':'skall',
'skallene':'skall',
'skalpeller':'skalpell',
'skam':'skam',
'skamferte':'skamfer',
'skamfull':'skamfull',
'skammelig':'skamm',
'skandia':'skandi',
'skap':'skap',
'skapdører':'skapdør',
'skape':'skap',
'skapelse':'skap',
'skaper':'skap',
'skaperverk':'skaperverk',
'skapes':'skap',
'skapet':'skap',
'skapning':'skapning',
'skapningene':'skapning',
'skapninger':'skapning',
'skapt':'skapt',
'skapte':'skapt',
'skar':'skar',
'skare':'skar',
'skarp':'skarp',
'skarpe':'skarp',
'skarve':'skarv',
'skatt':'skatt',
'skatte':'skatt',
'skattebegrensningsregelen':'skattebegrensningsregel',
'skattebegunstiget':'skattebegunst',
'skattebesparelsen':'skattebespar',
'skattebestemmelsene':'skattebestemm',
'skattebetalerne':'skattebetalern',
'skattedirektoratet':'skattedirektorat',
'skattefavorisere':'skattefavoriser',
'skattefavorisering':'skattefavorisering',
'skattefavoriseringen':'skattefavorisering',
'skattefavorisert':'skattefavoriser',
'skattefavoriserte':'skattefavoriser',
'skattefordel':'skattefordel',
'skattefordelar':'skattefordel',
'skattefordelen':'skattefordel',
'skattefordelene':'skattefordel',
'skattefordeler':'skattefordel',
'skattefradrag':'skattefradrag',
'skattefri':'skattefri',
'skattefrihet':'skattefri',
'skatteinntekter':'skatteinntekt',
'skatteinsentiver':'skatteinsentiv',
'skatteklasse':'skatteklass',
'skattekontroll':'skattekontroll',
'skattelettelser':'skattelett',
'skatteligningen':'skatteligning',
'skattelov':'skatt',
'skattelova':'skatt',
'skatteloven':'skatt',
'skattelovgivingen':'skattelovgiving',
'skattelovgivningen':'skattelovgivning',
'skattelovgivningens':'skattelovgivning',
'skattemessig':'skattemess',
'skattemessige':'skattemess',
'skattemyndigheten':'skattemynd',
'skattemyndighetene':'skattemynd',
'skatten':'skatt',
'skattenøytralitet':'skattenøytralit',
'skatteplikt':'skatteplikt',
'skatteplikten':'skatteplikt',
'skattepliktens':'skatteplikt',
'skattepliktig':'skatteplikt',
'skattepliktige':'skatteplikt',
'skattepliktiges':'skatteplikt',
'skattepolitikk':'skattepolitikk',
'skatter':'skatt',
'skattereformen':'skattereform',
'skattereglane':'skatteregl',
'skattereglar':'skatteregl',
'skattereglene':'skatteregl',
'skatteregler':'skatteregl',
'skatteretten':'skatterett',
'skatterettslig':'skatteretts',
'skatterettslige':'skatteretts',
'skattesammenheng':'skattesammenheng',
'skattesats':'skattesat',
'skattespørsmål':'skattespørsmål',
'skattestimulert':'skattestimuler',
'skattestimulerte':'skattestimuler',
'skattestyresmakter':'skattestyresmakt',
'skattesubjekt':'skattesubjekt',
'skattesystem':'skattesystem',
'skattesystemet':'skattesystem',
'skattetrekket':'skattetrekk',
'skatteutsettelse':'skatteutsett',
'skatteutsettelsen':'skatteutsett',
'skattevedtak':'skattevedtak',
'skattevedtaket':'skattevedtak',
'skattlagt':'skattlagt',
'skattlagte':'skattlagt',
'skattlegges':'skattlegg',
'skattlegging':'skattlegging',
'skattyter':'skattyt',
'skattytere':'skattyter',
'skattyteren':'skattyter',
'skattyters':'skattyt',
'skavanker':'skavank',
'ske':'ske',
'skeivdeling':'skeivdeling',
'skeive':'skeiv',
'skeivt':'skeivt',
'skepsisen':'skepsis',
'skeptisk':'skeptisk',
'ski':'ski',
'skidbladner':'skidbladn',
'skien':'skien',
'skift':'skift',
'skifta':'skift',
'skiftar':'skift',
'skifte':'skift',
'skiftelova':'skift',
'skiftenede':'skiften',
'skifter':'skift',
'skifteretten':'skifterett',
'skiftes':'skift',
'skiftestasjonen':'skiftestasjon',
'skiftet':'skift',
'skifting':'skifting',
'skigarden':'skigard',
'skigudinne':'skigudinn',
'skikka':'skikk',
'skikke':'skikk',
'skikkelig':'skikk',
'skikkelse':'skikk',
'skikkelsen':'skikk',
'skikkelser':'skikk',
'skikker':'skikk',
'skikket':'skikk',
'skil':'skil',
'skilde':'skild',
'skilja':'skilj',
'skiljast':'skilj',
'skilje':'skilj',
'skiljet':'skilj',
'skiljevegger':'skiljevegg',
'skille':'skill',
'skiller':'skill',
'skilles':'skill',
'skillet':'skill',
'skilnad':'skilnad',
'skilnaden':'skilnad',
'skilnadene':'skilnad',
'skilnader':'skilnad',
'skilsdom':'skilsdom',
'skilsdomsavtalar':'skilsdomsavtal',
'skilsmål':'skilsmål',
'skilt':'skilt',
'skiltet':'skilt',
'skimrer':'skimr',
'skimter':'skimt',
'skinfakse':'skinfaks',
'skingrende':'skingr',
'skinnegående':'skinnegå',
'skinnende':'skinn',
'skinner':'skinn',
'skinnet':'skinn',
'skinnfellen':'skinnfell',
'skinnpung':'skinnpung',
'skint':'skint',
'skinte':'skint',
'skip':'skip',
'skipa':'skip',
'skipar':'skip',
'skipe':'skip',
'skipene':'skip',
'skipenes':'skip',
'skipet':'skip',
'skiping':'skiping',
'skipling':'skipling',
'skipningene':'skipning',
'skipningsmønster':'skipningsmønst',
'skipningsvolum':'skipningsvolum',
'skipsaksjeselskap':'skipsaksjeselskap',
'skipsandel':'skipsandel',
'skipsanløpene':'skipsanløp',
'skipsdekket':'skipsdekk',
'skipsfart':'skipsfart',
'skipsfarten':'skipsfart',
'skipsfraktede':'skipsfrakt',
'skipshastighet':'skipshast',
'skipsløsninger':'skipsløsning',
'skipsregister':'skipsregist',
'skipsregistera':'skipsregister',
'skipsstørrelse':'skipsstørr',
'skipsstørrelser':'skipsstørr',
'skipstilbud':'skipstilbud',
'skipstrafikk':'skipstrafikk',
'skipstrafikken':'skipstrafikk',
'skipstransport':'skipstransport',
'skipstransporten':'skipstransport',
'skipstyper':'skipstyp',
'skirne':'skirn',
'skisse':'skiss',
'skissemessige':'skissemess',
'skissere':'skisser',
'skisserer':'skisser',
'skisseres':'skisser',
'skissert':'skisser',
'skisserte':'skisser',
'skitten':'skitt',
'skittent':'skittent',
'skive':'skiv',
'skje':'skje',
'skjebne':'skjebn',
'skjebnen':'skjebn',
'skjebnetråd':'skjebnetråd',
'skjedd':'skjedd',
'skjedde':'skjedd',
'skjeftet':'skjeft',
'skjegg':'skjegg',
'skjegget':'skjegg',
'skjeggete':'skjegget',
'skjeggløse':'skjeggløs',
'skjell':'skjell',
'skjellete':'skjellet',
'skjellsord':'skjellsord',
'skjelne':'skjeln',
'skjelve':'skjelv',
'skjelvende':'skjelv',
'skjelver':'skjelv',
'skjematisk':'skjematisk',
'skjemt':'skjemt',
'skjenk':'skjenk',
'skjenke':'skjenk',
'skjenker':'skjenk',
'skjer':'skjer',
'skjere':'skjer',
'skjerma':'skjerm',
'skjerme':'skjerm',
'skjermes':'skjerm',
'skjermet':'skjerm',
'skjerming':'skjerming',
'skjerpe':'skjerp',
'skjerpede':'skjerp',
'skjett':'skjett',
'skjev':'skjev',
'skjevheter':'skjev',
'skjevseleksjon':'skjevseleksjon',
'skjold':'skjold',
'skjoldene':'skjold',
'skjoldet':'skjold',
'skjoldungene':'skjoldung',
'skjule':'skjul',
'skjuler':'skjul',
'skjult':'skjult',
'skjær':'skjær',
'skjære':'skjær',
'skjærende':'skjær',
'skjærer':'skjær',
'skjøn':'skjøn',
'skjønet':'skjøn',
'skjønne':'skjønn',
'skjønner':'skjønn',
'skjønnhet':'skjønn',
'skjønnsmessig':'skjønnsmess',
'skjørt':'skjørt',
'skjørtene':'skjørt',
'skjøter':'skjøt',
'skjøvet':'skjøv',
'skli':'skli',
'sko':'sko',
'skodden':'skodd',
'skoene':'skoen',
'skog':'skog',
'skogen':'skog',
'skogene':'skog',
'skoger':'skog',
'skogholt':'skogholt',
'skoglandet':'skogland',
'skogs':'skog',
'skoll':'skoll',
'skolten':'skolt',
'skote':'skot',
'skotter':'skott',
'skou':'skou',
'skrangler':'skrangl',
'skrankene':'skrank',
'skranker':'skrank',
'skrapeklev':'skrapeklev',
'skraper':'skrap',
'skratter':'skratt',
'skrekkelig':'skrekk',
'skrekkelige':'skrekk',
'skremme':'skremm',
'skremmende':'skremm',
'skremmer':'skremm',
'skremt':'skremt',
'skremte':'skremt',
'skrever':'skrev',
'skrevet':'skrev',
'skrevs':'skrev',
'skriftelig':'skrift',
'skriftleg':'skrift',
'skriftlig':'skrift',
'skrik':'skrik',
'skrike':'skrik',
'skrikene':'skrik',
'skriker':'skrik',
'skrin':'skrin',
'skrinet':'skrin',
'skrinlegges':'skrinlegg',
'skritt':'skritt',
'skriv':'skriv',
'skrivast':'skriv',
'skrive':'skriv',
'skriver':'skriv',
'skrives':'skriv',
'skrivne':'skrivn',
'skrotet':'skrot',
'skrotstein':'skrotstein',
'skryme':'skrym',
'skrymes':'skrym',
'skryt':'skryt',
'skryte':'skryt',
'skryter':'skryt',
'skrå':'skrå',
'skudd':'skudd',
'skuddet':'skudd',
'skuer':'skuer',
'skuffet':'skuff',
'skular':'skul',
'skuld':'skuld',
'skulder':'skuld',
'skulderen':'skulder',
'skuldre':'skuldr',
'skuldrene':'skuldr',
'skuler':'skul',
'skulle':'skull',
'skumdråper':'skumdråp',
'skumle':'skuml',
'skummel':'skummel',
'skummende':'skumm',
'skumpet':'skump',
'skur':'skur',
'skure':'skur',
'skurer':'skur',
'skurt':'skurt',
'skutt':'skutt',
'skvetter':'skvett',
'skvettes':'skvett',
'skybanke':'skybank',
'skyene':'skyen',
'skyer':'skyer',
'skygge':'skygg',
'skyggen':'skygg',
'skygger':'skygg',
'skyid':'skyid',
'skyld':'skyld',
'skyldansvar':'skyldansv',
'skyldansvaret':'skyldansvar',
'skylden':'skyld',
'skylder':'skyld',
'skyldes':'skyld',
'skyldforhold':'skyldforhold',
'skyldgrunnlag':'skyldgrunnlag',
'skyldig':'skyld',
'skyldige':'skyld',
'skyldnadene':'skyldnad',
'skyldnader':'skyldnad',
'skyldneren':'skyldner',
'skyldnerens':'skyldner',
'skylle':'skyll',
'skyller':'skyll',
'skynd':'skynd',
'skynde':'skynd',
'skynder':'skynd',
'skyndet':'skynd',
'skyr':'skyr',
'skyt':'skyt',
'skytast':'skyt',
'skyte':'skyt',
'skyter':'skyt',
'skytes':'skyt',
'skyve':'skyv',
'skyver':'skyv',
'skyves':'skyv',
'skål':'skål',
'skåler':'skål',
'skåne':'skån',
'skåner':'skån',
'skånsomt':'skånsomt',
'skøyte':'skøyt',
'sladder':'sladd',
'sladre':'sladr',
'sladrehanker':'sladrehank',
'sladrer':'sladr',
'sladret':'sladr',
'slag':'slag',
'slaga':'slag',
'slagentangen':'slagentang',
'slaget':'slag',
'slagmarken':'slagmark',
'slags':'slag',
'slagsmål':'slagsmål',
'slakte':'slakt',
'slaktedyr':'slaktedyr',
'slakter':'slakt',
'slaktet':'slakt',
'slange':'slang',
'slangehodet':'slangehod',
'slangen':'slang',
'slapp':'slapp',
'slarvekjerringer':'slarvekjerring',
'slede':'sled',
'sleden':'sled',
'slege':'sleg',
'slegne':'slegn',
'sleipner':'sleipn',
'sleivkjeftet':'sleivkjeft',
'slekt':'slekt',
'slekten':'slekt',
'slekter':'slekt',
'slektning':'slektning',
'slektningar':'slektning',
'slenger':'sleng',
'slengkyss':'slengkyss',
'slengt':'slengt',
'slepende':'slep',
'sleper':'slep',
'slepp':'slepp',
'sleppe':'slepp',
'slept':'slept',
'slet':'slet',
'slett':'slett',
'sletta':'slett',
'slettast':'slett',
'slette':'slett',
'sletten':'slett',
'sletter':'slett',
'sletting':'sletting',
'slik':'slik',
'slike':'slik',
'slikke':'slikk',
'slikker':'slikk',
'slikket':'slikk',
'slikt':'slikt',
'slim':'slim',
'slimet':'slim',
'slipp':'slipp',
'slippe':'slipp',
'slipper':'slipp',
'slippes':'slipp',
'slit':'slit',
'slitasje':'slitasj',
'slite':'slit',
'sliten':'slit',
'sliter':'slit',
'slites':'slit',
'slitt':'slitt',
'slo':'slo',
'sloss':'sloss',
'slottet':'slott',
'slottsparktunnelen':'slottsparktunnel',
'slu':'slu',
'sludre':'sludr',
'sludrer':'sludr',
'sluke':'sluk',
'sluker':'sluk',
'slukke':'slukk',
'slukkes':'slukk',
'slukket':'slukk',
'slukne':'slukn',
'slukner':'slukn',
'slukt':'slukt',
'slukøret':'slukør',
'sluppet':'slupp',
'slurk':'slurk',
'slurken':'slurk',
'slutt':'slutt',
'sluttar':'slutt',
'sluttbehandlet':'sluttbehandl',
'sluttbruker':'sluttbruk',
'slutte':'slutt',
'slutten':'slutt',
'slutter':'slutt',
'sluttes':'slutt',
'sluttet':'slutt',
'sluttfasen':'sluttfas',
'sluttført':'sluttført',
'sluttlønn':'sluttlønn',
'sluttlønnen':'sluttlønn',
'sluttlønnsbasert':'sluttlønnsbaser',
'sluttlønnsbaserte':'sluttlønnsbaser',
'sluttlønnsprinsippet':'sluttlønnsprinsipp',
'sluttregnskapet':'sluttregnskap',
'slynger':'slyng',
'slå':'slå',
'slåande':'slåand',
'slåast':'slåast',
'slåen':'slåen',
'slår':'slår',
'slås':'slås',
'slåss':'slåss',
'slåsskamp':'slåsskamp',
'slåsskjempene':'slåsskjemp',
'slåsskjemper':'slåsskjemp',
'slått':'slått',
'slåttefolk':'slåttefolk',
'slør':'slør',
'sløret':'slør',
'sløve':'sløv',
'sløvet':'sløv',
'smak':'smak',
'smake':'smak',
'smaker':'smak',
'smakt':'smakt',
'smal':'smal',
'smale':'smal',
'smalere':'smaler',
'smatter':'smatt',
'smed':'smed',
'smedarbeid':'smedarbeid',
'smeden':'smed',
'smedene':'smed',
'smeder':'smed',
'smedkunster':'smedkunst',
'smelle':'smell',
'smeller':'smell',
'smelte':'smelt',
'smelter':'smelt',
'smeltet':'smelt',
'smertene':'smert',
'smi':'smi',
'smia':'smia',
'smidd':'smidd',
'smidde':'smidd',
'smidig':'smid',
'smiene':'smien',
'smiger':'smig',
'smigre':'smigr',
'smil':'smil',
'smile':'smil',
'smiler':'smil',
'smilet':'smil',
'smilte':'smilt',
'smir':'smir',
'smiske':'smisk',
'smisker':'smisk',
'smittende':'smitt',
'smitter':'smitt',
'smoltens':'smolt',
'smurte':'smurt',
'smyge':'smyg',
'smyger':'smyg',
'smykkelåsen':'smykkelås',
'smykker':'smykk',
'smykket':'smykk',
'smykketyven':'smykketyv',
'små':'små',
'småbiter':'småbit',
'småbonde':'småbond',
'småfe':'småf',
'småguttene':'smågutt',
'smågutter':'smågutt',
'småhus':'småhus',
'småpakkeforsendelser':'småpakkeforsend',
'småpakker':'småpakk',
'småvilt':'småvilt',
'smører':'smør',
'smøring':'smøring',
'snakk':'snakk',
'snakke':'snakk',
'snakker':'snakk',
'snakkes':'snakk',
'snakket':'snakk',
'snar':'snar',
'snarare':'snarar',
'snarast':'snar',
'snare':'snar',
'snarere':'snarer',
'snarlig':'snar',
'snart':'snart',
'snartur':'snartur',
'snarvegen':'snarveg',
'snarvei':'snarvei',
'snarveien':'snarvei',
'snarveier':'snarvei',
'snauklippet':'snauklipp',
'snaut':'snaut',
'snautt':'snautt',
'sne':'sne',
'sneen':'sneen',
'snefonnene':'snefonn',
'snehule':'snehul',
'snek':'snek',
'sneket':'snek',
'snerrende':'snerr',
'snerrer':'snerr',
'snevre':'snevr',
'snevrere':'snevrer',
'sniker':'snik',
'snill':'snill',
'snille':'snill',
'snitt':'snitt',
'sno':'sno',
'snor':'snor',
'snoren':'snor',
'snorke':'snork',
'snorkende':'snork',
'snorker':'snork',
'snu':'snu',
'snubler':'snubl',
'snudd':'snudd',
'snufser':'snufs',
'snur':'snur',
'snus':'snus',
'snø':'snø',
'snøen':'snøen',
'snøfter':'snøft',
'snøre':'snør',
'snøret':'snør',
'snøvler':'snøvl',
'social':'social',
'sog':'sog',
'sogn':'sogn',
'sokkeletasjen':'sokkeletasj',
'sol':'sol',
'solberg':'solberg',
'solbergkrysset':'solbergkryss',
'soldatene':'soldat',
'soldater':'soldat',
'solen':'sol',
'solgt':'solgt',
'solgte':'solgt',
'solid':'solid',
'solidarisk':'solidarisk',
'solidaritet':'solidarit',
'solidariteten':'solidaritet',
'solidaritetsalternativet':'solidaritetsalternativ',
'soliditet':'solidit',
'soliditetskrav':'soliditetskrav',
'soloppgang':'soloppgang',
'soloppgangen':'soloppgang',
'solskinn':'solskinn',
'soltegnet':'soltegn',
'solvognen':'solvogn',
'som':'som',
'somme':'somm',
'sommer':'somm',
'sommerdag':'sommerdag',
'sommeren':'sommer',
'sommerfuglvinger':'sommerfuglving',
'sommersesongen':'sommersesong',
'somtid':'somtid',
'son':'son',
'sorg':'sorg',
'sorgløst':'sorgløst',
'sort':'sort',
'sorte':'sort',
'sortering':'sortering',
'sorteringsanlegg':'sorteringsanlegg',
'sortkledd':'sortkledd',
'sortkledde':'sortkledd',
'sortkledte':'sortkled',
'sosial':'sosial',
'sosiale':'sosial',
'sosialforsikringssystem':'sosialforsikringssystem',
'sosialistisk':'sosialistisk',
'sosialkomiteen':'sosialkomite',
'sosialkomiteens':'sosialkomite',
'sosialkontor':'sosialkontor',
'sosialpolitisk':'sosialpolitisk',
'sosialt':'sosialt',
'sosialtenestelova':'sosialtenest',
'sosialtjenesten':'sosialtjenest',
'sot':'sot',
'sotbrun':'sotbrun',
'sou':'sou',
'sov':'sov',
'sove':'sov',
'sovekammeret':'sovekammer',
'sovende':'sov',
'sover':'sov',
'soverommet':'soveromm',
'sovet':'sov',
'sovner':'sovn',
'sovnet':'sovn',
'spakt':'spakt',
'spankulerer':'spankuler',
'spann':'spann',
'spare':'spar',
'spareavtale':'spareavtal',
'spareavtalen':'spareavtal',
'spareavtalene':'spareavtal',
'spareavtaler':'spareavtal',
'sparebank':'sparebank',
'sparebankane':'sparebank',
'sparebankforeningen':'sparebankforening',
'sparebanklova':'sparebank',
'sparebeløp':'sparebeløp',
'sparedelen':'sparedel',
'spareelement':'spareelement',
'spareform':'spareform',
'spareformen':'spareform',
'spareformer':'spareform',
'spareinnskot':'spareinnskot',
'spareinnskota':'spareinnskot',
'sparekapital':'sparekapital',
'sparekasser':'sparekass',
'sparekontrakter':'sparekontrakt',
'sparelag':'sparelag',
'sparemarkedet':'sparemarked',
'sparemidlane':'sparemidl',
'sparemidlar':'sparemidl',
'sparemidlene':'sparemidl',
'sparemidler':'sparemidl',
'spareordning':'spareordning',
'spareordningene':'spareordning',
'spareordninger':'spareordning',
'spareprodukt':'spareprodukt',
'spareprodukter':'spareprodukt',
'sparer':'spar',
'spareren':'sparer',
'sparerens':'sparer',
'sparesaldoen':'sparesaldo',
'spareverksemda':'spareverksemd',
'sparing':'sparing',
'sparinga':'sparing',
'sparingen':'sparing',
'spark':'spark',
'sparkassan':'sparkassan',
'sparkasse':'sparkass',
'sparke':'spark',
'sparker':'spark',
'sparket':'spark',
'spart':'spart',
'spedisjons':'spedisjon',
'spedisjonstjenester':'spedisjonstjenest',
'speditørene':'speditør',
'speditører':'speditør',
'spegla':'spegl',
'speide':'speid',
'speider':'speid',
'speilbilde':'speilbild',
'spekter':'spekt',
'spekulant':'spekulant',
'spekulasjonsobjekt':'spekulasjonsobjekt',
'spela':'spel',
'spelar':'spel',
'spele':'spel',
'spelerom':'spelerom',
'spenene':'spen',
'spenn':'spenn',
'spennende':'spenn',
'spenner':'spenn',
'spennet':'spenn',
'spenninga':'spenning',
'spenningsfelt':'spenningsfelt',
'spenntak':'spenntak',
'spennvidden':'spennvidd',
'spent':'spent',
'spente':'spent',
'sperre':'sperr',
'sperrer':'sperr',
'spesialbustader':'spesialbustad',
'spesialhavn':'spesialhavn',
'spesialisering':'spesialisering',
'spesialisert':'spesialiser',
'spesialiserte':'spesialiser',
'spesialitetsprinsippet':'spesialitetsprinsipp',
'spesialmerknadene':'spesialmerknad',
'spesialmerknader':'spesialmerknad',
'spesialområder':'spesialområd',
'spesialregel':'spesialregel',
'spesialtilfelle':'spesialtilfell',
'spesiell':'spesiell',
'spesielle':'spesiell',
'spesielt':'spesielt',
'spesifikasjon':'spesifikasjon',
'spesifikasjonskrav':'spesifikasjonskrav',
'spesifikke':'spesifikk',
'spesifikt':'spesifikt',
'spesifiserer':'spesifiser',
'spesifiseres':'spesifiser',
'spesifisert':'spesifiser',
'spesifiserte':'spesifiser',
'spikker':'spikk',
'spilene':'spil',
'spill':'spill',
'spille':'spill',
'spiller':'spill',
'spilloppmakeren':'spilloppmaker',
'spilt':'spilt',
'spire':'spir',
'spise':'spis',
'spiser':'spis',
'spises':'spis',
'spissen':'spiss',
'spist':'spist',
'spiste':'spist',
'spk':'spk',
'splinten':'splint',
'splintene':'splint',
'splintres':'splintr',
'sponene':'spon',
'sponhaug':'sponhaug',
'sponhaugen':'sponhaug',
'sponvikskansen':'sponvikskans',
'spor':'spor',
'sporadisk':'sporadisk',
'sporene':'spor',
'sporet':'spor',
'sporforbindelse':'sporforbind',
'sportilknytning':'sportilknytning',
'spott':'spott',
'spotter':'spott',
'sprakende':'sprak',
'sprakk':'sprakk',
'sprang':'sprang',
'spranget':'sprang',
'spratt':'spratt',
'spre':'spre',
'spreading':'spreading',
'spredning':'spredning',
'spredningen':'spredning',
'spredningsalternativer':'spredningsalternativ',
'spredningsformer':'spredningsform',
'spredningsmønster':'spredningsmønst',
'spredningsnett':'spredningsnett',
'spredt':'spredt',
'spreiast':'spreiast',
'spreidde':'spreidd',
'sprekere':'spreker',
'sprekk':'sprekk',
'sprekke':'sprekk',
'sprekker':'sprekk',
'sprell':'sprell',
'sprellende':'sprell',
'spreller':'sprell',
'sprenge':'spreng',
'sprenger':'spreng',
'sprenges':'spreng',
'sprengkulda':'sprengkuld',
'sprengninger':'sprengning',
'sprengrir':'sprengrir',
'sprengt':'sprengt',
'spres':'spres',
'spretter':'sprett',
'sprik':'sprik',
'spriker':'sprik',
'spriket':'sprik',
'springer':'spring',
'springskaller':'springskall',
'spruten':'sprut',
'spruter':'sprut',
'sprutet':'sprut',
'språk':'språk',
'språkbruk':'språkbruk',
'språket':'språk',
'språkkunnskapar':'språkkunnskap',
'språklig':'språk',
'spurt':'spurt',
'spurte':'spurt',
'spyd':'spyd',
'spydet':'spyd',
'spydkast':'spydk',
'spyende':'spyend',
'spyr':'spyr',
'spytt':'spytt',
'spytte':'spytt',
'spytter':'spytt',
'spyttet':'spytt',
'spå':'spå',
'spådd':'spådd',
'spådommen':'spådomm',
'spåkjerringer':'spåkjerring',
'spåkoner':'spåkon',
'spåkunster':'spåkunst',
'spåkvinne':'spåkvinn',
'spåkvinnen':'spåkvinn',
'spåkvinner':'spåkvinn',
'spøk':'spøk',
'spøker':'spøk',
'spør':'spør',
'spørje':'spørj',
'spørre':'spørr',
'spørrende':'spørr',
'spørres':'spørr',
'spørreundersøkelse':'spørreundersøk',
'spørsmål':'spørsmål',
'spørsmåla':'spørsmål',
'spørsmålene':'spørsmål',
'spørsmålet':'spørsmål',
'ssbs':'ssbs',
'st':'st',
'stabber':'stabb',
'stabburet':'stabbur',
'stabil':'stabil',
'stabile':'stabil',
'stabilisert':'stabiliser',
'stabilitet':'stabilit',
'stabilt':'stabilt',
'stablekraner':'stablekran',
'stablekransystemer':'stablekransystem',
'stablekranteknologi':'stablekranteknologi',
'stables':'stabl',
'stableteknologi':'stableteknologi',
'stabling':'stabling',
'stad':'stad',
'staden':'stad',
'stader':'stad',
'stades':'stad',
'stadfesta':'stadfest',
'stadfestast':'stadfest',
'stadfeste':'stadfest',
'stadfestes':'stadfest',
'stadfestet':'stadfest',
'stadfesting':'stadfesting',
'stadfestingsorskurd':'stadfestingsorskurd',
'stadig':'stad',
'stadige':'stad',
'stagge':'stagg',
'stagger':'stagg',
'stagnasjon':'stagnasjon',
'stagnere':'stagner',
'staking':'staking',
'stakk':'stakk',
'stakkar':'stakk',
'stakkars':'stakkar',
'stamfar':'stamf',
'stammen':'stamm',
'stammer':'stamm',
'stamnett':'stamnett',
'stamnettet':'stamnett',
'stamper':'stamp',
'stamveg':'stamveg',
'stamvegforbindelsen':'stamvegforbind',
'stamvegnettet':'stamvegnett',
'stamvegtilknytning':'stamvegtilknytning',
'stand':'stand',
'standard':'standard',
'standardberegnet':'standardberegn',
'standardberegning':'standardberegning',
'standarden':'standard',
'standardfradrag':'standardfradrag',
'standardfradragene':'standardfradrag',
'standardisering':'standardisering',
'standardiserte':'standardiser',
'standardsikring':'standardsikring',
'standpunkt':'standpunkt',
'standpunkta':'standpunkt',
'stange':'stang',
'stanger':'stang',
'stanken':'stank',
'stans':'stan',
'stansa':'stans',
'stanse':'stans',
'stanser':'stans',
'stanset':'stans',
'stappe':'stapp',
'star':'star',
'starkad':'starkad',
'start':'start',
'starte':'start',
'starter':'start',
'startes':'start',
'startet':'start',
'startlønn':'startlønn',
'stas':'stas',
'staselig':'stas',
'staselige':'stas',
'stasjon':'stasjon',
'stasjons':'stasjon',
'stat':'stat',
'staten':'stat',
'statens':'stat',
'stater':'stat',
'states':'stat',
'statistikk':'statistikk',
'statistisk':'statistisk',
'statistiske':'statistisk',
'statlege':'stat',
'statlig':'stat',
'statlige':'stat',
'statlånsrente':'statlånsrent',
'stats':'stat',
'statsautorisert':'statsautoriser',
'statsautoriserte':'statsautoriser',
'statsborgarskap':'statsborgarskap',
'statsborgere':'statsborger',
'statsborgerskap':'statsborgerskap',
'statsbudsjettet':'statsbudsjett',
'statsen':'stats',
'statsmidler':'statsmidl',
'statsråd':'statsråd',
'statsstøtte':'statsstøtt',
'status':'status',
'statusen':'status',
'staur':'staur',
'staute':'staut',
'stav':'stav',
'stavanger':'stavang',
'stavbærerske':'stavbærersk',
'staven':'stav',
'stavnen':'stavn',
'stavrende':'stavr',
'stebarn':'stebarn',
'sted':'sted',
'stedbundet':'stedbund',
'stede':'sted',
'steden':'sted',
'stedene':'sted',
'steder':'sted',
'stedet':'sted',
'stedsforming':'stedsforming',
'stedvis':'stedvis',
'steg':'steg',
'steget':'steg',
'steil':'steil',
'steilene':'steil',
'stein':'stein',
'steineksport':'steineksport',
'steinene':'stein',
'steinnes':'steinn',
'steinrøys':'steinrøy',
'steinrøyser':'steinrøys',
'steke':'stek',
'stekt':'stekt',
'stell':'stell',
'stelle':'stell',
'stelt':'stelt',
'stemme':'stemm',
'stemmen':'stemm',
'stemmene':'stemm',
'stemmer':'stemm',
'stemmerett':'stemmerett',
'stemninga':'stemning',
'stemte':'stemt',
'sten':'sten',
'stendig':'stend',
'stenen':'sten',
'stenene':'sten',
'stener':'sten',
'stengd':'stengd',
'stengde':'stengd',
'stenge':'steng',
'stengjast':'stengj',
'stengje':'stengj',
'stengjer':'stengj',
'stengt':'stengt',
'stenk':'stenk',
'stenkisten':'stenkist',
'stenmurene':'stenmur',
'stenstolpe':'stenstolp',
'stephen':'steph',
'steppene':'stepp',
'sterk':'sterk',
'sterkare':'sterkar',
'sterkaste':'sterkast',
'sterke':'sterk',
'sterkere':'sterker',
'sterkest':'sterkest',
'sterkeste':'sterkest',
'sterkt':'sterkt',
'stesønn':'stesønn',
'stesønnen':'stesønn',
'stettar':'stett',
'stevnemøte':'stevnemøt',
'stien':'stien',
'stier':'stier',
'stifta':'stift',
'stiftar':'stift',
'stiftarane':'stiftar',
'stiftarar':'stiftar',
'stiftaren':'stiftar',
'stiftast':'stift',
'stifte':'stift',
'stiftelse':'stift',
'stiftelser':'stift',
'stiftelsesgrunnlaget':'stiftelsesgrunnlag',
'stiftelsesloven':'stiftelse',
'stiftelseslovens':'stiftelse',
'stifting':'stifting',
'stiftinga':'stifting',
'stiftingar':'stifting',
'stiftingsdokument':'stiftingsdokument',
'stiftingsdokumentet':'stiftingsdokument',
'stiftingsgrunnlaget':'stiftingsgrunnlag',
'stiftingslova':'stifting',
'stiftingsmøtet':'stiftingsmøt',
'stiftingsutgiftene':'stiftingsutgift',
'stiftingsutgifter':'stiftingsutgift',
'stiftingsvilkåra':'stiftingsvilkår',
'stig':'stig',
'stige':'stig',
'stigende':'stig',
'stiger':'stig',
'stigninger':'stigning',
'stikk':'stikk',
'stikke':'stikk',
'stikker':'stikk',
'stikkes':'stikk',
'stikkord':'stikkord',
'stilisert':'stiliser',
'stiliserte':'stiliser',
'stillast':'still',
'stille':'still',
'stiller':'still',
'stilles':'still',
'stilling':'stilling',
'stillinga':'stilling',
'stillingar':'stilling',
'stillingen':'stilling',
'stillinger':'stilling',
'stillingsandel':'stillingsandel',
'stillingsfullmakt':'stillingsfullmakt',
'stillingsgruppe':'stillingsgrupp',
'stillingsinstruksar':'stillingsinstruks',
'stillingsskifter':'stillingsskift',
'stilt':'stilt',
'stilte':'stilt',
'stime':'stim',
'stimler':'stiml',
'stimulansen':'stimulans',
'stimulere':'stimuler',
'stimulerer':'stimuler',
'stimulering':'stimulering',
'stinker':'stink',
'stipulert':'stipuler',
'stirre':'stirr',
'stirrer':'stirr',
'stiv':'stiv',
'stive':'stiv',
'stivnet':'stivn',
'stjal':'stjal',
'stjele':'stjel',
'stjerne':'stjern',
'stjernen':'stjern',
'stjernene':'stjern',
'stjerner':'stjern',
'stk':'stk',
'sto':'sto',
'stockholm':'stockholm',
'stod':'stod',
'stoda':'stod',
'stokk':'stokk',
'stokke':'stokk',
'stokken':'stokk',
'stokkene':'stokk',
'stol':'stol',
'stole':'stol',
'stolen':'stol',
'stolene':'stol',
'stoler':'stol',
'stolpe':'stolp',
'stolpen':'stolp',
'stolt':'stolt',
'stolte':'stolt',
'stoltere':'stolter',
'stolthet':'stolt',
'stoppe':'stopp',
'stopper':'stopp',
'stoppet':'stopp',
'stor':'stor',
'storbritannia':'storbritanni',
'storbyene':'storby',
'stordriftsfordelene':'stordriftsfordel',
'stordriftsfordeler':'stordriftsfordel',
'store':'stor',
'storebroren':'storebror',
'storesøster':'storesøst',
'storleik':'storleik',
'storleiken':'storleik',
'storm':'storm',
'stormannsgal':'stormannsgal',
'stormen':'storm',
'stormende':'storm',
'stormer':'storm',
'stormet':'storm',
'stormkastene':'stormkast',
'stormvind':'stormvind',
'stormvær':'stormvær',
'storstilet':'storstil',
'stort':'stort',
'stortinget':'storting',
'stortingets':'storting',
'stortingsbehandling':'stortingsbehandling',
'stortingsbehandlingen':'stortingsbehandling',
'stortingskomiteen':'stortingskomite',
'stortingsmelding':'stortingsmelding',
'stortingsmeldingen':'stortingsmelding',
'stortingsmeldingene':'stortingsmelding',
'stortingsmeldinger':'stortingsmelding',
'stortingsvedtak':'stortingsvedtak',
'stortåen':'stortå',
'storverk':'storverk',
'storøyde':'storøyd',
'stp':'stp',
'straff':'straff',
'straffast':'straff',
'straffe':'straff',
'straffeføresegn':'straffeføresegn',
'straffeføresegner':'straffeføresegn',
'straffelova':'straff',
'strafferegelen':'strafferegel',
'straffereglane':'strafferegl',
'straffes':'straff',
'straffesak':'straffesak',
'straks':'straks',
'stramme':'stramm',
'strammer':'stramm',
'strammere':'strammer',
'strand':'strand',
'stranden':'strand',
'strandet':'strand',
'strandlinjen':'strandlinj',
'strandområdene':'strandområd',
'strandområder':'strandområd',
'strandsonen':'strandson',
'strategi':'strategi',
'strategier':'strategi',
'strategiplan':'strategiplan',
'strategiplanarbeidet':'strategiplanarbeid',
'strategiplanen':'strategiplan',
'strategiplanens':'strategiplan',
'strategiplaner':'strategiplan',
'strategiplans':'strategiplan',
'strategisk':'strategisk',
'strategiske':'strategisk',
'straum':'straum',
'streife':'streif',
'streifer':'streif',
'streik':'streik',
'strekke':'strekk',
'strekker':'strekk',
'strekkje':'strekkj',
'strekkjer':'strekkj',
'strekning':'strekning',
'strekningen':'strekning',
'strekninger':'strekning',
'strekpunkt':'strekpunkt',
'strekpunktene':'strekpunkt',
'streng':'streng',
'strengare':'strengar',
'strenge':'streng',
'strengen':'streng',
'strengere':'strenger',
'strengt':'strengt',
'strev':'strev',
'strever':'strev',
'stri':'stri',
'strid':'strid',
'stride':'strid',
'striden':'strid',
'strider':'strid',
'strides':'strid',
'stridigheter':'strid',
'strie':'strie',
'striglet':'strigl',
'strimer':'strim',
'strippes':'stripp',
'stripping':'stripping',
'strir':'strir',
'stritter':'stritt',
'struktur':'struktur',
'strukturelle':'strukturell',
'strukturen':'struktur',
'strukturendringer':'strukturendring',
'strukturer':'struktur',
'strupen':'strup',
'stryk':'stryk',
'stryke':'stryk',
'strykene':'stryk',
'stryker':'stryk',
'strå':'strå',
'stråler':'strål',
'stråpersonar':'stråperson',
'strøk':'strøk',
'strøm':'strøm',
'strømforholdene':'strømforhold',
'strømføringen':'strømføring',
'strømme':'strømm',
'strømmen':'strømm',
'strømmende':'strømm',
'strømmer':'strømm',
'strømmet':'strømm',
'strømstad':'strømstad',
'stubbe':'stubb',
'studere':'studer',
'studerer':'studer',
'studie':'studi',
'studiene':'studi',
'studier':'studi',
'studietur':'studietur',
'stuedøra':'stuedør',
'stuffes':'stuff',
'stuffing':'stuffing',
'stukket':'stukk',
'stum':'stum',
'stumme':'stumm',
'stummende':'stumm',
'stump':'stump',
'stund':'stund',
'stundom':'stundom',
'stup':'stup',
'stuper':'stup',
'stupet':'stup',
'sture':'stur',
'sturer':'stur',
'sturte':'sturt',
'stusser':'stuss',
'stuten':'stut',
'stygg':'stygg',
'stygge':'stygg',
'styggelig':'stygg',
'styggeste':'styggest',
'stygt':'stygt',
'stykke':'stykk',
'stykker':'stykk',
'stykket':'stykk',
'stykkgods':'stykkgod',
'stykkgodsbåter':'stykkgodsbåt',
'stykkgodsdelen':'stykkgodsdel',
'stykkgodsdistribusjon':'stykkgodsdistribusjon',
'stykkgodsdistribusjonen':'stykkgodsdistribusjon',
'stykkgodseksporten':'stykkgodseksport',
'stykkgodset':'stykkgods',
'stykkgodsfarten':'stykkgodsfart',
'stykkgodshandel':'stykkgodshandel',
'stykkgodshaven':'stykkgodshav',
'stykkgodshavn':'stykkgodshavn',
'stykkgodshavnen':'stykkgodshavn',
'stykkgodshavnene':'stykkgodshavn',
'stykkgodshåndtering':'stykkgodshåndtering',
'stykkgodshåndteringen':'stykkgodshåndtering',
'stykkgodsimport':'stykkgodsimport',
'stykkgodsimporten':'stykkgodsimport',
'stykkgodskaier':'stykkgodskai',
'stykkgodsklassifiserte':'stykkgodsklassifiser',
'stykkgodsmarkedet':'stykkgodsmarked',
'stykkgodsmengdene':'stykkgodsmengd',
'stykkgodsmengder':'stykkgodsmengd',
'stykkgodsomlasting':'stykkgodsomlasting',
'stykkgodsomlastingen':'stykkgodsomlasting',
'stykkgodsomsetning':'stykkgodsomsetning',
'stykkgodsomsetningen':'stykkgodsomsetning',
'stykkgodsomslaget':'stykkgodsomslag',
'stykkgodspotensialet':'stykkgodspotensial',
'stykkgodstrafikk':'stykkgodstrafikk',
'stykkgodstrafikken':'stykkgodstrafikk',
'stykkgodstransport':'stykkgodstransport',
'stykkgodsutviklingen':'stykkgodsutvikling',
'stykkgodsvarer':'stykkgodsvar',
'stykkgodsveksten':'stykkgodsvekst',
'stykkgodsvolumer':'stykkgodsvolum',
'styr':'styr',
'styra':'styr',
'styrande':'styr',
'styrast':'styr',
'styre':'styr',
'styrearbeidet':'styrearbeid',
'styrebehandling':'styrebehandling',
'styrebehandlinga':'styrebehandling',
'styreinstruks':'styreinstruks',
'styrekompetansen':'styrekompetans',
'styreleiar':'styrelei',
'styreleiaren':'styreleiar',
'styremedlem':'styremedlem',
'styremedlemmen':'styremedlemm',
'styremedlemmene':'styremedlemm',
'styremedlemmer':'styremedlemm',
'styremøta':'styremøt',
'styremøte':'styremøt',
'styremøtet':'styremøt',
'styrende':'styr',
'styreprotokoll':'styreprotokoll',
'styrer':'styr',
'styrerepresentasjon':'styrerepresentasjon',
'styres':'styr',
'styresakene':'styresak',
'styresamansetjing':'styresamansetjing',
'styresamansetjinga':'styresamansetjing',
'styresmaktene':'styresmakt',
'styresmakter':'styresmakt',
'styret':'styr',
'styrets':'styr',
'styrevalet':'styreval',
'styrevedtak':'styrevedtak',
'styrevedtaka':'styrevedtak',
'styrevedtaket':'styrevedtak',
'styring':'styring',
'styringa':'styring',
'styringen':'styring',
'styringsgruppe':'styringsgrupp',
'styringsgruppen':'styringsgrupp',
'styringsgruppens':'styringsgrupp',
'styringsinstrument':'styringsinstrument',
'styringsorgan':'styringsorgan',
'styringsorganer':'styringsorgan',
'styringsredskap':'styringsredskap',
'styrke':'styrk',
'styrkebelte':'styrkebelt',
'styrkebeltet':'styrkebelt',
'styrken':'styrk',
'styrkeprøve':'styrkeprøv',
'styrkeprøven':'styrkeprøv',
'styrker':'styrk',
'styrket':'styrk',
'styrking':'styrking',
'styrkje':'styrkj',
'styrkjer':'styrkj',
'styrkt':'styrkt',
'styrt':'styrt',
'styrte':'styrt',
'styrter':'styrt',
'stå':'stå',
'ståande':'ståand',
'stående':'ståend',
'stål':'stål',
'stålimport':'stålimport',
'står':'står',
'stått':'stått',
'stø':'stø',
'støere':'støer',
'støkk':'støkk',
'støl':'støl',
'støle':'støl',
'stønad':'stønad',
'stønn':'stønn',
'stønner':'stønn',
'større':'størr',
'størrelse':'størr',
'størrelsen':'størr',
'størrelsene':'størr',
'størrelser':'størr',
'størrelsesorden':'størrelsesord',
'størst':'størst',
'største':'størst',
'størstedelen':'størstedel',
'støtte':'støtt',
'støtteordningene':'støtteordning',
'støtter':'støtt',
'støttet':'støtt',
'støy':'støy',
'støymessig':'støymess',
'subber':'subb',
'subjektene':'subjekt',
'subjekter':'subjekt',
'subsidiar':'subsidi',
'subsidieordningar':'subsidieordning',
'subsidier':'subsidi',
'subsidiering':'subsidiering',
'subsidiert':'subsidier',
'subsidierte':'subsidier',
'subsidiær':'subsidiær',
'subsidiært':'subsidiært',
'substansiell':'substansiell',
'substituert':'substituer',
'sug':'sug',
'sugende':'sug',
'suger':'sug',
'suget':'sug',
'sukk':'sukk',
'sukker':'sukk',
'sult':'sult',
'sulte':'sult',
'sulten':'sult',
'sultet':'sult',
'sultne':'sultn',
'sum':'sum',
'summar':'summ',
'summarisk':'summarisk',
'summen':'summ',
'summende':'summ',
'sundagar':'sundag',
'sunde':'sund',
'sunget':'sung',
'sunket':'sunk',
'sunn':'sunn',
'super':'sup',
'suppen':'supp',
'supplement':'supplement',
'supplementary':'supplementary',
'supplere':'suppler',
'supplerende':'suppler',
'supplerer':'suppler',
'supplering':'supplering',
'suppleringsval':'suppleringsval',
'supplert':'suppler',
'sur':'sur',
'surere':'surer',
'surklende':'surkl',
'surre':'surr',
'surret':'surr',
'surrogat':'surrogat',
'surt':'surt',
'sus':'sus',
'suse':'sus',
'susende':'sus',
'suser':'sus',
'susete':'suset',
'sutrer':'sutr',
'suttung':'suttung',
'suttungs':'suttung',
'svaberg':'svaberg',
'svadilfare':'svadilfar',
'svaier':'svaier',
'svak':'svak',
'svakare':'svakar',
'svake':'svak',
'svakere':'svaker',
'svakhet':'svak',
'svakheter':'svak',
'svakt':'svakt',
'svalbard':'svalbard',
'svanene':'svan',
'svaner':'svan',
'svangerskap':'svangerskap',
'svangerskapspermisjon':'svangerskapspermisjon',
'svanser':'svans',
'svanvik':'svanvik',
'svar':'svar',
'svara':'svar',
'svarar':'svar',
'svare':'svar',
'svarende':'svar',
'svarer':'svar',
'svares':'svar',
'svaret':'svar',
'svart':'svart',
'svartalver':'svartalv',
'svartalvheim':'svartalvheim',
'svarte':'svart',
'svartmale':'svartmal',
'svartner':'svartn',
'svartsyk':'svartsyk',
'sveio':'sveio',
'sveits':'sveit',
'sveket':'svek',
'svekke':'svekk',
'svekkelse':'svekk',
'svekkes':'svekk',
'svekket':'svekk',
'svelg':'svelg',
'svelge':'svelg',
'svelger':'svelg',
'svelvik':'svelvik',
'svelvikstrømmen':'svelvikstrømm',
'svelvikstrømmens':'svelvikstrømm',
'svenner':'svenn',
'svensk':'svensk',
'svenske':'svensk',
'svenskegrensen':'svenskegrens',
'svenskekongen':'svenskekong',
'sverd':'sverd',
'sverdet':'sverd',
'sverdspissen':'sverdspiss',
'sverdsvingere':'sverdsvinger',
'sverg':'sverg',
'sverger':'sverg',
'sverget':'sverg',
'sverige':'sver',
'sveriges':'sver',
'svett':'svett',
'svette':'svett',
'svetter':'svett',
'svevende':'svev',
'svever':'svev',
'svi':'svi',
'svidd':'svidd',
'svik':'svik',
'svike':'svik',
'svikta':'svikt',
'svikter':'svikt',
'sviktet':'svikt',
'svimle':'sviml',
'svimmel':'svimmel',
'svinekjøtt':'svinekjøtt',
'svineskinn':'svineskinn',
'svinesund':'svinesund',
'svinet':'svin',
'sving':'sving',
'svinga':'sving',
'svinge':'sving',
'svingende':'sving',
'svinger':'sving',
'svingingane':'svinging',
'svingninger':'svingning',
'svingte':'svingt',
'svinser':'svins',
'svir':'svir',
'svirelaget':'svirelag',
'svovelkis':'svovelkis',
'svært':'svært',
'svøm':'svøm',
'svømme':'svømm',
'svømmebasseng':'svømmebasseng',
'svømmehud':'svømmehud',
'svømmer':'svømm',
'syd':'syd',
'syder':'syd',
'sydfra':'sydfr',
'sydgående':'sydgå',
'sydhavna':'sydhavn',
'sydhimmelen':'sydhimmel',
'sydover':'sydov',
'sydsiden':'sydsid',
'sydspissen':'sydspiss',
'sydvestover':'sydvestov',
'sydøst':'sydøst',
'syk':'syk',
'sykdom':'sykdom',
'sykdommen':'sykdomm',
'syke':'syk',
'sykeleie':'sykelei',
'syl':'syl',
'symbolsk':'symbolsk',
'symmetri':'symmetri',
'syn':'syn',
'synd':'synd',
'syne':'syn',
'syner':'syn',
'synes':'syn',
'synest':'synest',
'synet':'syn',
'synfaring':'synfaring',
'synge':'syng',
'synger':'syng',
'synke':'synk',
'synkende':'synk',
'synker':'synk',
'synlege':'syn',
'synlig':'syn',
'synlige':'syn',
'synsmåtane':'synsmåt',
'synspunkt':'synspunkt',
'synspunkta':'synspunkt',
'synspunkter':'synspunkt',
'synspunktet':'synspunkt',
'synsstad':'synsstad',
'synsstaden':'synsstad',
'syntes':'synt',
'syr':'syr',
'sysken':'sysk',
'sysselsatte':'sysselsatt',
'sysselsetting':'sysselsetting',
'sysselsettingsvirkninger':'sysselsettingsvirkning',
'system':'system',
'systematisk':'systematisk',
'systematiske':'systematisk',
'systemer':'system',
'systemet':'system',
'systemkonkurranse':'systemkonkurrans',
'systemløsninger':'systemløsning',
'systemtogløsninger':'systemtogløsning',
'syte':'syt',
'syvende':'syv',
'så':'så',
'sådan':'sådan',
'sådd':'sådd',
'såfall':'såfall',
'såfremt':'såfremt',
'såg':'såg',
'såkalla':'såkall',
'såkalt':'såkalt',
'såkalte':'såkalt',
'således':'sål',
'såleis':'såleis',
'sånn':'sånn',
'såpass':'såpass',
'sår':'sår',
'såre':'sår',
'sårene':'sår',
'sårer':'sår',
'såret':'sår',
'sårt':'sårt',
'såvel':'såvel',
'såvidt':'såvid',
'sæming':'sæming',
'særaldersgrenser':'særaldersgrens',
'særbehandling':'særbehandling',
'særbehandlingen':'særbehandling',
'særbestemmelser':'særbestemm',
'særdeles':'særdel',
'særdrag':'særdrag',
'særdraga':'særdrag',
'særfordeler':'særfordel',
'særfradrag':'særfradrag',
'særimne':'særimn',
'særinteresse':'særinteress',
'særinteresser':'særinteress',
'særleg':'sær',
'særlege':'sær',
'særlig':'sær',
'særlige':'sær',
'særlovgjeving':'særlovgjeving',
'særlovgjevinga':'særlovgjeving',
'særløysingar':'særløysing',
'særmerknad':'særmerknad',
'særmerknadetil':'særmerknadetil',
'særordningar':'særordning',
'særpreg':'særpreg',
'særprega':'særpreg',
'særpregar':'særpreg',
'særpreget':'særpreg',
'særregel':'særregel',
'særregelen':'særregel',
'særreglane':'særregl',
'særreglar':'særregl',
'særreglene':'særregl',
'særregler':'særregl',
'særregulering':'særregulering',
'særrettar':'særrett',
'særskild':'særskild',
'særskilde':'særskild',
'særskilt':'særskilt',
'særskilte':'særskilt',
'særstilling':'særstilling',
'særtillegg':'særtillegg',
'særtillegget':'særtillegg',
'særvilkår':'særvilkår',
'sæther':'sæth',
'søk':'søk',
'søke':'søk',
'søker':'søk',
'søkes':'søk',
'søkjaransiennitet':'søkjaransiennit',
'søkjaransienniteten':'søkjaransiennitet',
'søkje':'søkj',
'søkjegod':'søkjegod',
'søkjer':'søkj',
'søknad':'søknad',
'søknaden':'søknad',
'søksmål':'søksmål',
'søksmålet':'søksmål',
'søksmåls':'søksmål',
'søksmålsfrist':'søksmålsfrist',
'søksmålsfristen':'søksmålsfrist',
'søksmålsrett':'søksmålsrett',
'søkt':'søkt',
'søkte':'søkt',
'sølv':'sølv',
'søndre':'søndr',
'sønn':'sønn',
'sønnavind':'sønnavind',
'sønnen':'sønn',
'sønnene':'sønn',
'sønner':'sønn',
'sør':'sør',
'sørenga':'søreng',
'sørge':'sørg',
'sørgelig':'sørg',
'sørger':'sørg',
'sørget':'sørg',
'sørgjer':'sørgj',
'sørkorridoren':'sørkorridor',
'sørlandsbanen':'sørlandsban',
'sørover':'sørov',
'sørpehull':'sørpehull',
'sørside':'sørsid',
'sørsiden':'sørsid',
'sørvendte':'sørvend',
'sørvisinnstilling':'sørvisinnstilling',
'søsken':'søsk',
'søster':'søst',
'søsteren':'søster',
'søstre':'søstr',
'søstrene':'søstr',
'søt':'søt',
'søtere':'søter',
'søvn':'søvn',
'søvne':'søvn',
'søvntorn':'søvntorn',
'søyle':'søyl',
'søyler':'søyl',
't':'t',
'ta':'ta',
'tabell':'tabell',
'tabellen':'tabell',
'tabellene':'tabell',
'taggete':'tagget',
'tak':'tak',
'takast':'tak',
'takbjelkene':'takbjelk',
'takernes':'takern',
'taket':'tak',
'takk':'takk',
'takke':'takk',
'takker':'takk',
'takket':'takk',
'takknemlig':'takknem',
'takknemlighet':'takknem',
'takkonstruksjon':'takkonstruksjon',
'takle':'takl',
'takskjegget':'takskjegg',
'takstnemnder':'takstnemnd',
'takt':'takt',
'takten':'takt',
'taktfast':'taktf',
'tal':'tal',
'tala':'tal',
'talar':'tal',
'tale':'tal',
'taler':'tal',
'talerett':'talerett',
'talet':'tal',
'tall':'tall',
'talleksempler':'talleksempl',
'tallene':'tall',
'tallet':'tall',
'tallfeste':'tallfest',
'tallgrunnlaget':'tallgrunnlag',
'tallrik':'tallrik',
'tallrike':'tallrik',
'talt':'talt',
'tamme':'tamm',
'tang':'tang',
'tangen':'tang',
'tangent':'tangent',
'tangstrødde':'tangstrødd',
'tankane':'tank',
'tanke':'tank',
'tankegang':'tankegang',
'tankegangen':'tankegang',
'tankeløs':'tankeløs',
'tanken':'tank',
'tankene':'tank',
'tanker':'tank',
'tankespråk':'tankespråk',
'tann':'tann',
'tanngnjost':'tanngnjost',
'tanngrisne':'tanngrisn',
'tannløs':'tannløs',
'tannløse':'tannløs',
'tap':'tap',
'tapa':'tap',
'tape':'tap',
'taper':'tap',
'tapet':'tap',
'tapetsering':'tapetsering',
'tappa':'tapp',
'tappar':'tapp',
'tappe':'tapp',
'tapper':'tapp',
'tapping':'tapping',
'tappinga':'tapping',
'tapre':'tapr',
'tapsfordeling':'tapsfordeling',
'tapspostane':'tapspost',
'tapsrisiko':'tapsrisiko',
'tapsrisikoen':'tapsrisiko',
'tapt':'tapt',
'tapte':'tapt',
'tar':'tar',
'tariffavtale':'tariffavtal',
'tariffavtaler':'tariffavtal',
'tarifforhandlingene':'tarifforhandling',
'tarmene':'tarm',
'tas':'tas',
'tassen':'tass',
'tassler':'tassl',
'tatt':'tatt',
'tau':'tau',
'taubåt':'taubåt',
'taubåthjelp':'taubåthjelp',
'tauender':'tauend',
'taus':'taus',
'tause':'taus',
'taushet':'taus',
'taustump':'taustump',
'td':'td',
'team':'team',
'teften':'teft',
'tegn':'tegn',
'tegne':'tegn',
'tegner':'tegn',
'tegnes':'tegn',
'tegnet':'tegn',
'tegning':'tegning',
'tegningstidspunktet':'tegningstidspunkt',
'teieplikt':'teieplikt',
'teieplikta':'teieplikt',
'teiepliktreglane':'teiepliktregl',
'teikna':'teikn',
'teiknar':'teikn',
'teiknast':'teikn',
'teikne':'teikn',
'teikning':'teikning',
'teikninga':'teikning',
'teikningar':'teikning',
'teikningsavgift':'teikningsavgift',
'teikningsrett':'teikningsrett',
'tek':'tek',
'teke':'tek',
'teken':'tek',
'tekne':'tekn',
'teknisk':'teknisk',
'tekniske':'teknisk',
'teknologi':'teknologi',
'teksten':'tekst',
'tekstene':'tekst',
'tekster':'tekst',
'tel':'tel',
'telefon':'telefon',
'telekomunikasjonsinfrastruktur':'telekomunikasjonsinfrastruktur',
'telemark':'telemark',
'teletenester':'teletenest',
'telje':'telj',
'telle':'tell',
'teller':'tell',
'temaet':'tema',
'tematisk':'tematisk',
'temme':'temm',
'temmet':'temm',
'temperaturer':'temperatur',
'ten':'ten',
'tendens':'tend',
'tene':'ten',
'tener':'ten',
'teneste':'tenest',
'tenestebustader':'tenestebustad',
'tenestene':'tenest',
'tenester':'tenest',
'tenestetid':'tenestetid',
'tenestetida':'tenestetid',
'tenestettida':'tenestettid',
'tenk':'tenk',
'tenke':'tenk',
'tenkelige':'tenk',
'tenker':'tenk',
'tenkes':'tenk',
'tenkjast':'tenkj',
'tenkje':'tenkj',
'tenkjer':'tenkj',
'tenksom':'tenksom',
'tenksomt':'tenksomt',
'tenkt':'tenkt',
'tenkte':'tenkt',
'tenleg':'ten',
'tenlege':'ten',
'tennene':'tenn',
'tenner':'tenn',
'tennes':'tenn',
'tent':'tent',
'tente':'tent',
'teoretisk':'teoretisk',
'teoretiske':'teoretisk',
'teori':'teori',
'teorien':'teori',
'teppe':'tepp',
'teppet':'tepp',
'ter':'ter',
'termer':'term',
'terminal':'terminal',
'terminalbehandles':'terminalbehandl',
'terminalbehandling':'terminalbehandling',
'terminalbehandlingen':'terminalbehandling',
'terminaldrift':'terminaldrift',
'terminaldriften':'terminaldrift',
'terminalen':'terminal',
'terminalene':'terminal',
'terminaler':'terminal',
'terminalfunksjon':'terminalfunksjon',
'terminalhavn':'terminalhavn',
'terminalhåndteringen':'terminalhåndtering',
'terminallokalisering':'terminallokalisering',
'terminalområder':'terminalområd',
'terminalområdet':'terminalområd',
'terminaloperasjonen':'terminaloperasjon',
'terminaloperatørene':'terminaloperatør',
'terminaloperatørenes':'terminaloperatør',
'terminaloperatører':'terminaloperatør',
'terminaltypene':'terminaltyp',
'terminalvirksomhet':'terminalvirksom',
'terminalvirksomheter':'terminalvirksom',
'terminbeløp':'terminbeløp',
'terminbeløpet':'terminbeløp',
'terminer':'termin',
'terminvise':'terminvis',
'terne':'tern',
'terner':'tern',
'terskelverdier':'terskelverdi',
'tett':'tett',
'tettbebyggelsen':'tettbebygg',
'tettbygd':'tettbygd',
'tettbygde':'tettbygd',
'tette':'tett',
'tettere':'tetter',
'tettpakket':'tettpakk',
'tettstader':'tettstad',
'tettsteder':'tettsted',
'tettsteds':'tettsted',
'tettstedsbebyggelse':'tettstedsbebygg',
'tettstedsområder':'tettstedsområd',
'teu':'teu',
'teus':'teus',
'the':'the',
'ti':'ti',
'tiandepart':'tiandepart',
'tid':'tid',
'tida':'tid',
'tide':'tid',
'tidel':'tidel',
'tiden':'tid',
'tidene':'tid',
'tidenes':'tid',
'tider':'tid',
'tidfestet':'tidfest',
'tidkrevende':'tidkrev',
'tidkrevjande':'tidkrevj',
'tidleg':'tid',
'tidlegare':'tidlegar',
'tidlegast':'tid',
'tidlege':'tid',
'tidlig':'tid',
'tidlige':'tid',
'tidligere':'tidliger',
'tidligst':'tidligst',
'tidligsådd':'tidligsådd',
'tidobbelte':'tidobbelt',
'tids':'tid',
'tidsangivelsene':'tidsangiv',
'tidsavgrensa':'tidsavgrens',
'tidsavgrensing':'tidsavgrensing',
'tidsavgrensingar':'tidsavgrensing',
'tidsavhengig':'tidsavheng',
'tidsbruk':'tidsbruk',
'tidseffektive':'tidseffektiv',
'tidsforskyvningen':'tidsforskyvning',
'tidsfrist':'tidsfrist',
'tidshorisont':'tidshorisont',
'tidshorisonten':'tidshorisont',
'tidshorisontene':'tidshorisont',
'tidskontoordning':'tidskontoordning',
'tidskostnader':'tidskostnad',
'tidspartar':'tidspart',
'tidsperspektiv':'tidsperspektiv',
'tidsperspektivene':'tidsperspektiv',
'tidsperspektiver':'tidsperspektiv',
'tidsperspektivet':'tidsperspektiv',
'tidspunkt':'tidspunkt',
'tidspunkter':'tidspunkt',
'tidspunktet':'tidspunkt',
'tidsramme':'tidsramm',
'tidsrammen':'tidsramm',
'tidsrom':'tidsrom',
'tidsrommet':'tidsromm',
'tidsskriftrom':'tidsskriftrom',
'tidstap':'tidstap',
'tidsubestemt':'tidsubestemt',
'tidvis':'tidvis',
'tie':'tie',
'tiendedel':'tiendedel',
'tier':'tier',
'tiet':'tiet',
'tigger':'tigg',
'til':'til',
'tilbake':'tilbak',
'tilbakebetaling':'tilbakebetaling',
'tilbakeføre':'tilbakefør',
'tilbakeføres':'tilbakefør',
'tilbakeføring':'tilbakeføring',
'tilbakeføringa':'tilbakeføring',
'tilbakeføringsåret':'tilbakeføringsår',
'tilbakeført':'tilbakeført',
'tilbakeførte':'tilbakeført',
'tilbakeholde':'tilbakehold',
'tilbakeholdenhet':'tilbakeholden',
'tilbakeholder':'tilbakehold',
'tilbakeholdes':'tilbakehold',
'tilbakeverkande':'tilbakeverk',
'tilbakeverknad':'tilbakeverknad',
'tilbod':'tilbod',
'tilbodet':'tilbod',
'tilbringertjeneste':'tilbringertjenest',
'tilbud':'tilbud',
'tilbudene':'tilbud',
'tilbudet':'tilbud',
'tilbudssiden':'tilbudssid',
'tilbudt':'tilbud',
'tilbudte':'tilbud',
'tilby':'tilby',
'tilbyder':'tilbyd',
'tilbydere':'tilbyder',
'tilbyderne':'tilbydern',
'tilbydernes':'tilbydern',
'tilbydt':'tilbyd',
'tilbyr':'tilbyr',
'tilbys':'tilby',
'tilbørleg':'tilbør',
'tildeles':'tildel',
'tildeling':'tildeling',
'tildelt':'tildelt',
'tildelte':'tildelt',
'tildligpensjonering':'tildligpensjonering',
'tildriv':'tildriv',
'tileigne':'tileign',
'tilfalle':'tilfall',
'tilfaller':'tilfall',
'tilfalt':'tilfalt',
'tilfanget':'tilfang',
'tilfeldig':'tilfeld',
'tilfeldige':'tilfeld',
'tilfeldighet':'tilfeld',
'tilfell':'tilfell',
'tilfella':'tilfell',
'tilfelle':'tilfell',
'tilfellene':'tilfell',
'tilfeller':'tilfell',
'tilfellet':'tilfell',
'tilflyter':'tilflyt',
'tilflytting':'tilflytting',
'tilfredsstillande':'tilfredsstill',
'tilfredsstille':'tilfredsstill',
'tilfredsstillende':'tilfredsstill',
'tilfredsstiller':'tilfredsstill',
'tilføre':'tilfør',
'tilfører':'tilfør',
'tilføres':'tilfør',
'tilføring':'tilføring',
'tilførsel':'tilførsel',
'tilført':'tilført',
'tilføyde':'tilføyd',
'tilføyelse':'tilføy',
'tilføyelsen':'tilføy',
'tilgang':'tilgang',
'tilgangen':'tilgang',
'tilgir':'tilgir',
'tilgjenge':'tilgjeng',
'tilgjengeleg':'tilgjeng',
'tilgjengelege':'tilgjeng',
'tilgjengelig':'tilgjeng',
'tilgjengelige':'tilgjeng',
'tilgjengelighet':'tilgjeng',
'tilgodehavande':'tilgodehav',
'tilgodesedd':'tilgodesedd',
'tilgodeser':'tilgodes',
'tilgodesjå':'tilgodesjå',
'tilgrising':'tilgrising',
'tilhenger':'tilheng',
'tilhengere':'tilhenger',
'tilhøre':'tilhør',
'tilhørende':'tilhør',
'tilhører':'tilhør',
'tilhøva':'tilhøv',
'tilhøve':'tilhøv',
'tilhøvet':'tilhøv',
'tilhøyrande':'tilhøyr',
'tilhøyrde':'tilhøyrd',
'tilhøyre':'tilhøyr',
'tilhøyrer':'tilhøyr',
'tilhøyrsel':'tilhøyrsel',
'tilhøyrsle':'tilhøyrsl',
'tilhøyrslereglane':'tilhøyrsleregl',
'tilknyting':'tilknyting',
'tilknytinga':'tilknyting',
'tilknytingsband':'tilknytingsband',
'tilknytingsbanda':'tilknytingsband',
'tilknytingsforhold':'tilknytingsforhold',
'tilknytingsforholdet':'tilknytingsforhold',
'tilknytingsforma':'tilknytingsform',
'tilknytingsformer':'tilknytingsform',
'tilknytning':'tilknytning',
'tilknytningen':'tilknytning',
'tilknytninger':'tilknytning',
'tilknytningsforhold':'tilknytningsforhold',
'tilknytt':'tilknytt',
'tilknytte':'tilknytt',
'tilknyttede':'tilknytt',
'tilknyttes':'tilknytt',
'tilknyttet':'tilknytt',
'tilkomme':'tilkomm',
'tilkommer':'tilkomm',
'till':'till',
'tillagd':'tillagd',
'tillagde':'tillagd',
'tillagt':'tillagt',
'tillatast':'tillat',
'tillate':'tillat',
'tillatelse':'tillat',
'tillaten':'tillat',
'tillates':'tillat',
'tillatne':'tillatn',
'tillatt':'tillatt',
'tillegg':'tillegg',
'tillegge':'tillegg',
'tillegger':'tillegg',
'tillegges':'tillegg',
'tilleggsavsetninger':'tilleggsavsetning',
'tilleggsavtalar':'tilleggsavtal',
'tilleggsavtalen':'tilleggsavtal',
'tilleggsbevilgning':'tilleggsbevilgning',
'tilleggsføremål':'tilleggsføremål',
'tilleggsføremåla':'tilleggsføremål',
'tilleggsføremålet':'tilleggsføremål',
'tilleggsinnkalling':'tilleggsinnkalling',
'tilleggsinnskot':'tilleggsinnskot',
'tilleggsinnskotet':'tilleggsinnskot',
'tilleggskapasitet':'tilleggskapasit',
'tilleggskapital':'tilleggskapital',
'tilleggskontraktane':'tilleggskontrakt',
'tilleggskontraktar':'tilleggskontrakt',
'tilleggsmandat':'tilleggsmandat',
'tilleggsopplysningar':'tilleggsopplysning',
'tilleggspensjon':'tilleggspensjon',
'tilleggspensjonen':'tilleggspensjon',
'tilleggspensjonsordning':'tilleggspensjonsordning',
'tilleggspensjonsordningen':'tilleggspensjonsordning',
'tilleggspensjonsordninger':'tilleggspensjonsordning',
'tilleggspensjonsprosent':'tilleggspensjonsprosent',
'tilleggsreglar':'tilleggsregl',
'tilleggsregler':'tilleggsregl',
'tilleggssikkerhet':'tilleggssikker',
'tilleggstilråding':'tilleggstilråding',
'tilleggstrafikk':'tilleggstrafikk',
'tilleggsytelse':'tilleggsyt',
'tilleggsytelsene':'tilleggsyt',
'tilleggsytelser':'tilleggsyt',
'tillet':'till',
'tilligge':'tilligg',
'tilligger':'tilligg',
'tillit':'tillit',
'tillita':'tillit',
'tillitsmann':'tillitsmann',
'tillitsperson':'tillitsperson',
'tillitsvald':'tillitsvald',
'tillitsvalde':'tillitsvald',
'tillitsverv':'tillitsverv',
'tilnavn':'tilnavn',
'tilnytingsbanda':'tilnytingsband',
'tilnærma':'tilnærm',
'tilnærmet':'tilnærm',
'tilnærming':'tilnærming',
'tilnærminger':'tilnærming',
'tilordnede':'tilordn',
'tilordnes':'tilordn',
'tilordnet':'tilordn',
'tilordning':'tilordning',
'tilordningen':'tilordning',
'tilpasning':'tilpasning',
'tilpasningen':'tilpasning',
'tilpasninger':'tilpasning',
'tilpasningsdyktig':'tilpasningsdykt',
'tilpasningsmuligheter':'tilpasningsmu',
'tilpassa':'tilpass',
'tilpassast':'tilpass',
'tilpasse':'tilpass',
'tilpasseleg':'tilpass',
'tilpasser':'tilpass',
'tilpasses':'tilpass',
'tilpasset':'tilpass',
'tilpassete':'tilpasset',
'tilpassing':'tilpassing',
'tilpassingar':'tilpassing',
'tilpassingsvanskar':'tilpassingsvansk',
'tilrettelagde':'tilrettelagd',
'tilrettelegge':'tilrettelegg',
'tilretteleggelse':'tilrettelegg',
'tilretteleggende':'tilrettelegg',
'tilrettelegges':'tilrettelegg',
'tilrettelegging':'tilrettelegging',
'tilrå':'tilrå',
'tilrådeleg':'tilråd',
'tilrådelig':'tilråd',
'tilråding':'tilråding',
'tilrådinger':'tilråding',
'tilrår':'tilrår',
'tilseie':'tilsei',
'tilseier':'tilsei',
'tilseiing':'tilseiing',
'tilsendt':'tilsend',
'tilsetje':'tilsetj',
'tilsetjing':'tilsetjing',
'tilsetjinga':'tilsetjing',
'tilsetjingar':'tilsetjing',
'tilsetjingsavtale':'tilsetjingsavtal',
'tilsetjingsforhold':'tilsetjingsforhold',
'tilsett':'tilsett',
'tilsette':'tilsett',
'tilsi':'tilsi',
'tilsier':'tilsi',
'tilskot':'tilskot',
'tilskota':'tilskot',
'tilskotsordningar':'tilskotsordning',
'tilskrives':'tilskriv',
'tilskudd':'tilskudd',
'tilskuddene':'tilskudd',
'tilskuddet':'tilskudd',
'tilskuddsordning':'tilskuddsordning',
'tilskuddssatsene':'tilskuddssats',
'tilskuddssatser':'tilskuddssats',
'tilskuere':'tilskuer',
'tilslutning':'tilslutning',
'tilslutta':'tilslutt',
'tilsluttet':'tilslutt',
'tilslørte':'tilslørt',
'tilstanden':'tilstand',
'tilstede':'tilst',
'tilstrebe':'tilstreb',
'tilstreber':'tilstreb',
'tilstrebes':'tilstreb',
'tilstrekkeleg':'tilstrekk',
'tilstrekkelig':'tilstrekk',
'tilstrekkelige':'tilstrekk',
'tilstøtende':'tilstøt',
'tilstøytande':'tilstøyt',
'tilsvarande':'tilsvar',
'tilsvare':'tilsvar',
'tilsvarende':'tilsvar',
'tilsvarer':'tilsvar',
'tilsyn':'tilsyn',
'tilsynelatende':'tilsynelat',
'tilsynet':'tilsyn',
'tilsynsansvar':'tilsynsansv',
'tilsynsansvaret':'tilsynsansvar',
'tilsynsarbeidet':'tilsynsarbeid',
'tilsynsbehovet':'tilsynsbehov',
'tilsynsfunksjonane':'tilsynsfunksjon',
'tilsynsloven':'tilsyn',
'tilsynsmessige':'tilsynsmess',
'tilsynsmyndighet':'tilsynsmynd',
'tilsynsmyndigheten':'tilsynsmynd',
'tilsynsmyndighetene':'tilsynsmynd',
'tilsynsmyndigheter':'tilsynsmynd',
'tilsynsoppgaven':'tilsynsoppgav',
'tilsynsoppgåva':'tilsynsoppgåv',
'tilsynsorgan':'tilsynsorgan',
'tilsynsretten':'tilsynsrett',
'tiltagende':'tiltag',
'tiltak':'tiltak',
'tiltaka':'tiltak',
'tiltakende':'tiltak',
'tiltakene':'tiltak',
'tiltaket':'tiltak',
'tiltar':'tilt',
'tiltrekke':'tiltrekk',
'tiltrekkes':'tiltrekk',
'tiltrer':'tiltr',
'tiltro':'tiltro',
'tiltrådt':'tiltråd',
'tilvekst':'tilvekst',
'tilveksten':'tilvekst',
'tilvisast':'tilvis',
'tilvising':'tilvising',
'tilvisinga':'tilvising',
'tilvisingar':'tilvising',
'tilvisning':'tilvisning',
'tilværelse':'tilvær',
'tilårskomne':'tilårskomn',
'time':'tim',
'timer':'tim',
'timers':'tim',
'timersregelen':'timersregel',
'times':'tim',
'timing':'timing',
'ting':'ting',
'tingar':'ting',
'tinglysast':'tinglys',
'tinglyse':'tinglys',
'tinglysing':'tinglysing',
'tinglysinga':'tinglysing',
'tinglysingsdommarane':'tinglysingsdommar',
'tinglysingsfeilen':'tinglysingsfeil',
'tinglysingskontora':'tinglysingskontor',
'tinglysingslova':'tinglysing',
'tinglysingslovutkastet':'tinglysingslovutkast',
'tinglysingssperre':'tinglysingssperr',
'tinglysingsutkastet':'tinglysingsutkast',
'tinglyst':'tinglyst',
'tinglyste':'tinglyst',
'tings':'ting',
'tingsinnskot':'tingsinnskot',
'tinningen':'tinning',
'tinningene':'tinning',
'tirret':'tirr',
'tisker':'tisk',
'tisket':'tisk',
'tispe':'tisp',
'tissler':'tissl',
'tista':'tist',
'titt':'titt',
'tittelen':'tittel',
'tiår':'tiår',
'tiåret':'tiår',
'tiårperioden':'tiårperiod',
'tiårsperiode':'tiårsperiod',
'tja':'tja',
'tjalve':'tjalv',
'tjanster':'tjanst',
'tjatse':'tjats',
'tjatses':'tjats',
'tjene':'tjen',
'tjener':'tjen',
'tjenere':'tjener',
'tjeneste':'tjenest',
'tjenestefolk':'tjenestefolk',
'tjenestefolkene':'tjenestefolk',
'tjenesteforhold':'tjenesteforhold',
'tjenesteforholdet':'tjenesteforhold',
'tjenestejente':'tjenestejent',
'tjenestekvinner':'tjenestekvinn',
'tjenestemannsgruppen':'tjenestemannsgrupp',
'tjenestemenn':'tjenestemenn',
'tjenestemessige':'tjenestemess',
'tjenestene':'tjenest',
'tjenestepensjon':'tjenestepensjon',
'tjenestepensjoner':'tjenestepensjon',
'tjenestepensjonsdekningen':'tjenestepensjonsdekning',
'tjenestepensjonsforsikring':'tjenestepensjonsforsikring',
'tjenestepensjonsforsikringen':'tjenestepensjonsforsikring',
'tjenestepensjonsområdet':'tjenestepensjonsområd',
'tjenestepensjonsordning':'tjenestepensjonsordning',
'tjenestepensjonsordningen':'tjenestepensjonsordning',
'tjenestepensjonsordningene':'tjenestepensjonsordning',
'tjenestepensjonsordninger':'tjenestepensjonsordning',
'tjenestepensjonssystemet':'tjenestepensjonssystem',
'tjenestepike':'tjenestepik',
'tjenestepikene':'tjenestepik',
'tjenester':'tjenest',
'tjenestetid':'tjenestetid',
'tjenestetilbud':'tjenestetilbud',
'tjenesteytere':'tjenesteyter',
'tjenesteyteren':'tjenesteyter',
'tjenesteyting':'tjenesteyting',
'tjent':'tjent',
'tjern':'tjern',
'tjoret':'tjor',
'tjue':'tjue',
'tjuedagarsfristen':'tjuedagarsfrist',
'tjuedel':'tjuedel',
'tjuvholmen':'tjuvholm',
'tjærebredd':'tjærebredd',
'to':'to',
'todagers':'todag',
'tofte':'toft',
'tog':'tog',
'toger':'tog',
'tok':'tok',
'tokk':'tokk',
'tole':'tol',
'tolka':'tolk',
'tolkast':'tolk',
'tolke':'tolk',
'tolket':'tolk',
'tolking':'tolking',
'tolkinga':'tolking',
'tolkingstvil':'tolkingstvil',
'tolkning':'tolkning',
'tolkningen':'tolkning',
'tolkningspraksis':'tolkningspraksis',
'tollbehandlingen':'tollbehandling',
'tolldepartementet':'tolldepartement',
'tollekniven':'tollekniv',
'tollmaterialets':'tollmaterial',
'tollpost':'tollpost',
'tollsted':'tollsted',
'tollvesenet':'tollvesen',
'tollvesenets':'tollvesen',
'tolv':'tolv',
'tom':'tom',
'tomannsbustadene':'tomannsbustad',
'tomannsbustader':'tomannsbustad',
'tomcontainere':'tomcontainer',
'tomhendt':'tomhend',
'tomheten':'tom',
'tomkjøring':'tomkjøring',
'tomme':'tomm',
'tommelen':'tommel',
'tomrommet':'tomromm',
'tomsekker':'tomsekk',
'tomt':'tomt',
'tomta':'tomt',
'tomtefesteforhold':'tomtefesteforhold',
'tomtegrunn':'tomtegrunn',
'tomtekjøp':'tomtekjøp',
'tomtene':'tomt',
'tomteområde':'tomteområd',
'tomtepriser':'tomtepris',
'tomter':'tomt',
'tomteseljar':'tomteselj',
'tomteselskap':'tomteselskap',
'tomtetildeling':'tomtetildeling',
'tonn':'tonn',
'tonnasje':'tonnasj',
'tonnasjen':'tonnasj',
'tonnkilometer':'tonnkilomet',
'tonnkm':'tonnkm',
'tonnmengder':'tonnmengd',
'top':'top',
'toplanskrysset':'toplanskryss',
'topografiske':'topografisk',
'topp':'topp',
'toppen':'topp',
'topper':'topp',
'topphastighet':'topphast',
'toppskatt':'toppskatt',
'tor':'tor',
'torde':'tord',
'torden':'tord',
'tordengud':'tordengud',
'tordenguden':'tordengud',
'tordenskrall':'tordenskrall',
'tordner':'tordn',
'tornehekk':'tornehekk',
'tors':'tor',
'torskeøynene':'torskeøyn',
'torsønnene':'torsønn',
'toseksjonsregelen':'toseksjonsregel',
'tosk':'tosk',
'tosken':'tosk',
'tosporet':'tospor',
'total':'total',
'totale':'total',
'totalkostnaden':'totalkostnad',
'totalkostnader':'totalkostnad',
'totalløsninger':'totalløsning',
'totalmengde':'totalmengd',
'totalpremien':'totalpremi',
'totalskadd':'totalskadd',
'totalskadde':'totalskadd',
'totalskade':'totalskad',
'totalt':'totalt',
'totaltall':'totaltall',
'totalvolumene':'totalvolum',
'towards':'toward',
'toårsfristen':'toårsfrist',
'toårsperiodar':'toårsperiod',
'toårsregelen':'toårsregel',
'tpes':'tpes',
'trade':'trad',
'tradisjon':'tradisjon',
'tradisjonar':'tradisjon',
'tradisjonell':'tradisjonell',
'tradisjonelle':'tradisjonell',
'tradisjonelt':'tradisjonelt',
'tradisjonen':'tradisjon',
'traff':'traff',
'trafikk':'trafikk',
'trafikkavvikling':'trafikkavvikling',
'trafikkavviklingen':'trafikkavvikling',
'trafikkavviklingsproblemer':'trafikkavviklingsproblem',
'trafikkbegrensninger':'trafikkbegrensning',
'trafikkbelastede':'trafikkbelast',
'trafikkbelastning':'trafikkbelastning',
'trafikkbelastningen':'trafikkbelastning',
'trafikkbelastninger':'trafikkbelastning',
'trafikkbildet':'trafikkbild',
'trafikken':'trafikk',
'trafikkens':'trafikk',
'trafikkerer':'trafikker',
'trafikkert':'trafikker',
'trafikkerte':'trafikker',
'trafikkfordeling':'trafikkfordeling',
'trafikkgenererende':'trafikkgenerer',
'trafikkhavn':'trafikkhavn',
'trafikkhavneanlegg':'trafikkhavneanlegg',
'trafikkhavnene':'trafikkhavn',
'trafikkhavner':'trafikkhavn',
'trafikkhavnvirksomhet':'trafikkhavnvirksom',
'trafikkløsning':'trafikkløsning',
'trafikkmengde':'trafikkmengd',
'trafikkmengder':'trafikkmengd',
'trafikkmessig':'trafikkmess',
'trafikkmessige':'trafikkmess',
'trafikkmønster':'trafikkmønst',
'trafikkmønsteret':'trafikkmønster',
'trafikkoverskudd':'trafikkoverskudd',
'trafikkproblemer':'trafikkproblem',
'trafikkregulerende':'trafikkreguler',
'trafikkreguleringer':'trafikkregulering',
'trafikksentral':'trafikksentral',
'trafikksentralen':'trafikksentral',
'trafikksentralene':'trafikksentral',
'trafikksentraler':'trafikksentral',
'trafikkseparasjonssystem':'trafikkseparasjonssystem',
'trafikkseparasjonssytem':'trafikkseparasjonssytem',
'trafikksikkerhet':'trafikksikker',
'trafikksituasjon':'trafikksituasjon',
'trafikksituasjonen':'trafikksituasjon',
'trafikkstrømmer':'trafikkstrømm',
'trafikktellinger':'trafikktelling',
'trafikktetthet':'trafikktett',
'trafikktettheten':'trafikktett',
'trafikkutvikling':'trafikkutvikling',
'trafikkutviklingen':'trafikkutvikling',
'trafikkvekst':'trafikkvekst',
'trafikkveksten':'trafikkvekst',
'trafikkvekstrater':'trafikkvekstrat',
'trafikkvolumer':'trafikkvolum',
'trafikkvolumet':'trafikkvolum',
'trafikkvolums':'trafikkvolum',
'trafikkøkning':'trafikkøkning',
'trafikkøkninger':'trafikkøkning',
'trailer':'trail',
'trailere':'trailer',
'trailerutstyr':'trailerutstyr',
'traktaten':'traktat',
'traktatens':'traktat',
'traller':'trall',
'trampe':'tramp',
'trampende':'tramp',
'tramper':'tramp',
'trampet':'tramp',
'trang':'trang',
'trange':'trang',
'trangt':'trangt',
'tranportarbeidet':'tranportarbeid',
'trans':'tran',
'transaksjon':'transaksjon',
'transaksjonar':'transaksjon',
'transaksjonen':'transaksjon',
'transeuropeiske':'transeuropeisk',
'transformering':'transformering',
'transitt':'transitt',
'transittcontainerne':'transittcontainern',
'transittgods':'transittgod',
'transitthavn':'transitthavn',
'transittlast':'transittl',
'transittrafikk':'transittrafikk',
'transport':'transport',
'transportalternativ':'transportalternativ',
'transportarbeid':'transportarbeid',
'transportarbeidet':'transportarbeid',
'transportavstand':'transportavstand',
'transportavstander':'transportavstand',
'transportavvikling':'transportavvikling',
'transportbedriftene':'transportbedrift',
'transportbedrifter':'transportbedrift',
'transportbehov':'transportbehov',
'transportbildet':'transportbild',
'transportbransjen':'transportbransj',
'transportbrukere':'transportbruker',
'transportbrukeren':'transportbruker',
'transportbrukerne':'transportbrukern',
'transportbrukernes':'transportbrukern',
'transportbærere':'transportbærer',
'transportdistanse':'transportdistans',
'transporten':'transport',
'transportene':'transport',
'transporter':'transport',
'transporterer':'transporter',
'transporteres':'transporter',
'transportert':'transporter',
'transporterte':'transporter',
'transportetater':'transportetat',
'transportetterspørsel':'transportetterspørsel',
'transportfirmaer':'transportfirma',
'transportform':'transportform',
'transportformen':'transportform',
'transportformene':'transportform',
'transportformer':'transportform',
'transportinfrastruktur':'transportinfrastruktur',
'transportkjeden':'transportkjed',
'transportkjeder':'transportkjed',
'transportkjøpere':'transportkjøper',
'transportkjøperne':'transportkjøpern',
'transportkjøpernes':'transportkjøpern',
'transportknutepunkt':'transportknutepunkt',
'transportkorridor':'transportkorridor',
'transportkorridorene':'transportkorridor',
'transportkorridorer':'transportkorridor',
'transportkostnad':'transportkostnad',
'transportkostnadene':'transportkostnad',
'transportkostnader':'transportkostnad',
'transportkostnadsmessige':'transportkostnadsmess',
'transportkrevende':'transportkrev',
'transportkvalitet':'transportkvalit',
'transportkvaliteten':'transportkvalitet',
'transportleddene':'transportledd',
'transportleddet':'transportledd',
'transportlenker':'transportlenk',
'transportlinkene':'transportlink',
'transportløsning':'transportløsning',
'transportløsningen':'transportløsning',
'transportløsningene':'transportløsning',
'transportløsninger':'transportløsning',
'transportmarkedet':'transportmarked',
'transportmateriell':'transportmateriell',
'transportmessig':'transportmess',
'transportmessige':'transportmess',
'transportmiddel':'transportmiddel',
'transportmiddelfordelingen':'transportmiddelfordeling',
'transportmidlene':'transportmidl',
'transportmidler':'transportmidl',
'transportmåte':'transportmåt',
'transportmåter':'transportmåt',
'transportmønstre':'transportmønstr',
'transportmønstrene':'transportmønstr',
'transportnett':'transportnett',
'transportnettet':'transportnett',
'transportnettverket':'transportnettverk',
'transportomfang':'transportomfang',
'transportoppgaver':'transportoppgav',
'transportopplegg':'transportopplegg',
'transportorganisering':'transportorganisering',
'transportperspektiv':'transportperspektiv',
'transportplan':'transportplan',
'transportplanen':'transportplan',
'transportplanlegging':'transportplanlegging',
'transportpolitikken':'transportpolitikk',
'transportpris':'transportpris',
'transportprosjekter':'transportprosjekt',
'transportrelaterte':'transportrelater',
'transportruten':'transportrut',
'transportsammenheng':'transportsammenheng',
'transportsektoren':'transportsektor',
'transportsektorene':'transportsektor',
'transportselskapene':'transportselskap',
'transportselskaper':'transportselskap',
'transportstrømmene':'transportstrømm',
'transportstrømmer':'transportstrømm',
'transportsystem':'transportsystem',
'transportsystemer':'transportsystem',
'transportsystemers':'transportsystem',
'transportsystemet':'transportsystem',
'transportsystemets':'transportsystem',
'transporttilbud':'transporttilbud',
'transporttjeneste':'transporttjenest',
'transporttjenester':'transporttjenest',
'transportutviklingen':'transportutvikling',
'transportveg':'transportveg',
'transportvirksomhet':'transportvirksom',
'transportvirksomheter':'transportvirksom',
'transportvolumer':'transportvolum',
'transportårene':'transportår',
'transportøkonomi':'transportøkonomi',
'transportøkonomisk':'transportøkonomisk',
'transportørene':'transportør',
'transportørenes':'transportør',
'transportørens':'transportør',
'transportører':'transportør',
'trappes':'trapp',
'trase':'tras',
'trasket':'trask',
'trass':'trass',
'trassen':'trass',
'trassig':'trass',
'trau':'trau',
'trauet':'trauet',
'travemunde':'travemund',
'travle':'travl',
'tre':'tre',
'tredel':'tredel',
'tredelar':'tredel',
'tredeler':'tredel',
'tredeling':'tredeling',
'tredels':'tredel',
'tredje':'tredj',
'tredjedel':'tredjedel',
'tredjedelen':'tredjedel',
'tredjedeler':'tredjedel',
'tredjedels':'tredjedel',
'tredjemann':'tredjemann',
'tredjemenn':'tredjemenn',
'tredjeparten':'tredjepart',
'tredjeperson':'tredjeperson',
'tredjepersonen':'tredjeperson',
'tredjepersons':'tredjeperson',
'tredoblet':'tredobl',
'treet':'treet',
'treffande':'treff',
'treffe':'treff',
'treffende':'treff',
'treffer':'treff',
'treffes':'treff',
'trefigurer':'trefigur',
'trefork':'trefork',
'tregere':'treger',
'trekk':'trekk',
'trekke':'trekk',
'trekker':'trekk',
'trekkes':'trekk',
'trekkjast':'trekkj',
'trekkje':'trekkj',
'trekkjer':'trekkj',
'trekkvogn':'trekkvogn',
'trekte':'trekt',
'trelast':'trel',
'trelemmer':'trelemm',
'trell':'trell',
'trellene':'trell',
'treller':'trell',
'trellkvinnene':'trellkvinn',
'trellkvinner':'trellkvinn',
'tremasse':'tremass',
'trend':'trend',
'trenden':'trend',
'trender':'trend',
'trendmessig':'trendmess',
'trendmessige':'trendmess',
'trends':'trend',
'trener':'tren',
'trenere':'trener',
'trenet':'tren',
'treng':'treng',
'trenge':'treng',
'trenger':'treng',
'trengje':'trengj',
'trengs':'treng',
'trengst':'trengst',
'trengt':'trengt',
'trengte':'trengt',
'trening':'trening',
'trepunktsløsning':'trepunktsløsning',
'trer':'trer',
'trerot':'trerot',
'tresleiv':'tresleiv',
'trestammen':'trestamm',
'trestammer':'trestamm',
'trestokkene':'trestokk',
'trestokker':'trestokk',
'tretoppene':'tretopp',
'trett':'trett',
'trette':'trett',
'tretti':'tretti',
'trettiårsregelen':'trettiårsregel',
'triangel':'triangel',
'triller':'trill',
'trimrommet':'trimromm',
'trinn':'trinn',
'trinnene':'trinn',
'trinns':'trinn',
'trist':'trist',
'trives':'triv',
'trivielt':'trivielt',
'trivsel':'trivsel',
'trivst':'trivst',
'tro':'tro',
'trodd':'trodd',
'trodde':'trodd',
'trofast':'trof',
'trofaste':'trofast',
'trolig':'trol',
'troll':'troll',
'trollborg':'trollborg',
'trollborgen':'trollborg',
'trolldom':'trolldom',
'trolldomskunster':'trolldomskunst',
'trollene':'troll',
'trollet':'troll',
'trolljenter':'trolljent',
'trolljern':'trolljern',
'trollkjerring':'trollkjerring',
'trollkjerringer':'trollkjerring',
'trollkongen':'trollkong',
'trollkvinne':'trollkvinn',
'trollkvinnen':'trollkvinn',
'trollmann':'trollmann',
'trollskog':'trollskog',
'trollslekten':'trollslekt',
'tromsø':'tromsø',
'trond':'trond',
'trondheim':'trondheim',
'tronen':'tron',
'troner':'tron',
'trongare':'trongar',
'trongst':'trongst',
'tronstolen':'tronstol',
'tror':'tror',
'troskap':'troskap',
'troskaps':'troskap',
'troskyldig':'troskyld',
'troskyldighet':'troskyld',
'tross':'tross',
'trosser':'tross',
'troverdigheten':'troverd',
'tru':'tru',
'trucker':'truck',
'trud':'trud',
'trudvang':'trudvang',
'TRUE':'TRUE',
'truende':'truend',
'truer':'truer',
'truet':'truet',
'truffet':'truff',
'trukket':'trukk',
'truleg':'truleg',
'trur':'trur',
'trusler':'trusl',
'trutt':'trutt',
'truvedkjenning':'truvedkjenning',
'truverdet':'truverd',
'trygd':'trygd',
'trygda':'trygd',
'trygdeavgift':'trygdeavgift',
'trygdeavgiften':'trygdeavgift',
'trygdeetatens':'trygdeetat',
'trygdens':'trygd',
'trygdeordninger':'trygdeordning',
'trygdesystemet':'trygdesystem',
'trygdet':'trygd',
'trygdgjeving':'trygdgjeving',
'trygdgjevinga':'trygdgjeving',
'trygg':'trygg',
'tryggare':'tryggar',
'trygge':'trygg',
'tryggeste':'tryggest',
'tryggingsventilen':'tryggingsventil',
'tryggleik':'tryggleik',
'trygt':'trygt',
'trykker':'trykk',
'trykkjer':'trykkj',
'trykte':'trykt',
'trylleord':'trylleord',
'tryller':'tryll',
'tryllesangene':'tryllesang',
'tryllesanger':'tryllesang',
'tryllesøvn':'tryllesøvn',
'tryllevers':'tryllev',
'trym':'trym',
'trymheim':'trymheim',
'tryms':'trym',
'trymsheim':'trymsheim',
'trå':'trå',
'tråd':'tråd',
'tråder':'tråd',
'trådt':'trådt',
'trådte':'trådt',
'trær':'trær',
'trærne':'trærn',
'trøndelag':'trøndelag',
'trør':'trør',
'trøst':'trøst',
'trøste':'trøst',
'trøster':'trøst',
'tu':'tu',
'tumler':'tuml',
'tummelumsk':'tummelumsk',
'tunet':'tun',
'tung':'tung',
'tunge':'tung',
'tungen':'tung',
'tunger':'tung',
'tungt':'tungt',
'tungtrafikk':'tungtrafikk',
'tungtrafikken':'tungtrafikk',
'tungtrafikkens':'tungtrafikk',
'tungtransport':'tungtransport',
'tungtransportbil':'tungtransportbil',
'tungtransporten':'tungtransport',
'tungtvegande':'tungtveg',
'tungvint':'tungvint',
'tunnel':'tunnel',
'tunnelen':'tunnel',
'tunneler':'tunnel',
'tunnelgjennomføring':'tunnelgjennomføring',
'tunnelløsning':'tunnelløsning',
'tuntre':'tuntr',
'tuntreet':'tuntre',
'tur':'tur',
'ture':'tur',
'tusen':'tus',
'tusener':'tusen',
'tusenvis':'tusenvis',
'tusler':'tusl',
'tussefolkets':'tussefolk',
'tussekongen':'tussekong',
'tussekonger':'tussekong',
'tusser':'tuss',
'tussete':'tusset',
'tuter':'tut',
'tv':'tv',
'tvang':'tvang',
'tvangsakkord':'tvangsakkord',
'tvangsbruk':'tvangsbruk',
'tvangsdekning':'tvangsdekning',
'tvangsdekningen':'tvangsdekning',
'tvangsfråviking':'tvangsfråviking',
'tvangsfullbyrdelse':'tvangsfullbyrd',
'tvangsfullføring':'tvangsfullføring',
'tvangsfullføringslova':'tvangsfullføring',
'tvangsfullført':'tvangsfullført',
'tvangsgrunnlaga':'tvangsgrunnlag',
'tvangsinndrivelse':'tvangsinndriv',
'tvangsl':'tvangsl',
'tvangslova':'tvang',
'tvangsmiddel':'tvangsmiddel',
'tvangsmulkt':'tvangsmulkt',
'tvangssal':'tvangssal',
'tvangssalet':'tvangssal',
'tvangssalg':'tvangssalg',
'tvangsseld':'tvangsseld',
'tvangsseljast':'tvangsselj',
'tvekamp':'tvekamp',
'tvekampen':'tvekamp',
'tverrbjelken':'tverrbjelk',
'tverrgående':'tverrgå',
'tverrpolitisk':'tverrpolitisk',
'tverrsektoriell':'tverrsektoriell',
'tvers':'tver',
'tvert':'tvert',
'tvertimot':'tvertimot',
'tvi':'tvi',
'tvil':'tvil',
'tvilande':'tvil',
'tvile':'tvil',
'tvilen':'tvil',
'tvilende':'tvil',
'tviler':'tvil',
'tvilsamt':'tvilsamt',
'tvilsomt':'tvilsomt',
'tvilsspørsmål':'tvilsspørsmål',
'tvilsspørsmåla':'tvilsspørsmål',
'tvilstilfelle':'tvilstilfell',
'tvinge':'tving',
'tvinger':'tving',
'tvinges':'tving',
'tvist':'tvist',
'tvistane':'tvist',
'tvistar':'tvist',
'tvisteløysing':'tvisteløysing',
'tvisteløysingsorgan':'tvisteløysingsorgan',
'tvistemål':'tvistemål',
'tvistemålslov':'tvistemål',
'tvistemålslova':'tvistemål',
'tvisten':'tvist',
'tvungen':'tvung',
'tvunget':'tvung',
'twenty':'twenty',
'tyde':'tyd',
'tydeleg':'tyd',
'tydelig':'tyd',
'tydelige':'tyd',
'tydeligere':'tydeliger',
'tydeliggjort':'tydeliggjort',
'tydeliggjøre':'tydeliggjør',
'tyder':'tyd',
'tyding':'tyding',
'tydinga':'tyding',
'tydingar':'tyding',
'tygger':'tygg',
'tygget':'tygg',
'tykke':'tykk',
'tykkere':'tykker',
'tykt':'tykt',
'tyngdepunkt':'tyngdepunkt',
'tyngdepunktet':'tyngdepunkt',
'tyngende':'tyng',
'tyngjande':'tyngj',
'tyngst':'tyngst',
'tyngste':'tyngst',
'tynn':'tynn',
'tynt':'tynt',
'typane':'typ',
'typar':'typ',
'type':'typ',
'typeeksempel':'typeeksempel',
'typen':'typ',
'typene':'typ',
'typer':'typ',
'typisk':'typisk',
'typiske':'typisk',
'tyr':'tyr',
'tyrs':'tyr',
'tysk':'tysk',
'tyske':'tysk',
'tyskland':'tyskland',
'tysvær':'tysvær',
'tyter':'tyt',
'tyv':'tyv',
'tyve':'tyv',
'tyven':'tyv',
'tyveriet':'tyveri',
'tå':'tå',
'tåa':'tåa',
'tåke':'tåk',
'tåkeheimen':'tåkeheim',
'tåken':'tåk',
'tåle':'tål',
'tåler':'tål',
'tålmodig':'tålmod',
'tålmodighet':'tålmod',
'tålmodigheten':'tålmod',
'tålt':'tålt',
'tåre':'tår',
'tårene':'tår',
'tårer':'tår',
'tårn':'tårn',
'tårner':'tårn',
'tælen':'tæl',
'tær':'tær',
'tære':'tær',
'tærne':'tærn',
'tøi':'tøi',
'tømme':'tømm',
'tømmene':'tømm',
'tømmer':'tømm',
'tømmerimporten':'tømmerimport',
'tømmerstokkene':'tømmerstokk',
'tømmertransport':'tømmertransport',
'tømmertransporter':'tømmertransport',
'tømmervegger':'tømmervegg',
'tømmes':'tømm',
'tønnen':'tønn',
'tønner':'tønn',
'tønsberg':'tønsberg',
'tør':'tør',
'tørker':'tørk',
'tørner':'tørn',
'tørr':'tørr',
'tørrbulk':'tørrbulk',
'tørrbulkgodset':'tørrbulkgods',
'tørrbulkkai':'tørrbulkkai',
'tørrbulkomslag':'tørrbulkomslag',
'tørre':'tørr',
'tørst':'tørst',
'tørste':'tørst',
'tørstet':'tørst',
'tørt':'tørt',
'tøy':'tøy',
'tøyle':'tøyl',
'tøyler':'tøyl',
'tøyser':'tøys',
'uakseptable':'uakseptabl',
'uaktuelt':'uaktuelt',
'uanseelig':'uanse',
'uansett':'uansett',
'uavbrutt':'uavbrutt',
'uavgjort':'uavgjort',
'uavgrensa':'uavgrens',
'uavhengig':'uavheng',
'uavhengige':'uavheng',
'uavkortet':'uavkort',
'ubalanse':'ubalans',
'ubedte':'ubed',
'ubegrensede':'ubegrens',
'ubegrenset':'ubegrens',
'ubegripelig':'ubegrip',
'ubenyttet':'ubenytt',
'uberegnelig':'uberegn',
'uberegnelige':'uberegn',
'uberørte':'uberørt',
'ubestemt':'ubestemt',
'ubetinga':'ubeting',
'ubetinget':'ubeting',
'ubetydelig':'ubetyd',
'ubunden':'ubund',
'ubygde':'ubygd',
'ubøyelig':'ubøy',
'udekket':'udekk',
'udekte':'udekt',
'udgave':'udgav',
'udjerv':'udjerv',
'udyret':'udyr',
'udyrs':'udyr',
'ueigentleg':'ueigent',
'ueinsarta':'ueinsart',
'uendelig':'uend',
'uendelige':'uend',
'uendret':'uendr',
'ufattelig':'ufatt',
'uforanderlig':'uforander',
'uforandret':'uforandr',
'uforfalle':'uforfall',
'uforholdsmessig':'uforholdsmess',
'uforlikte':'uforlikt',
'uformelig':'uform',
'uformelle':'uformell',
'uforsiktig':'uforsikt',
'uforsvarleg':'uforsvar',
'uforsvarlege':'uforsvar',
'uforsvarlig':'uforsvar',
'uforutsett':'uforutsett',
'ufred':'ufred',
'ufri':'ufri',
'ufriviljuge':'ufriviljug',
'ufråvikeleg':'ufråvik',
'ufråvikelege':'ufråvik',
'ufullstendig':'ufullstend',
'ufullstendige':'ufullstend',
'ufyselig':'ufys',
'ufyselige':'ufys',
'ufør':'ufør',
'uføre':'ufør',
'uføreforsikring':'uføreforsikring',
'uføregraden':'uføregrad',
'uførepensjon':'uførepensjon',
'uførepensjonen':'uførepensjon',
'uførepensjonens':'uførepensjon',
'uførepensjoner':'uførepensjon',
'uførepensjonister':'uførepensjonist',
'uførepensjonsforsikring':'uførepensjonsforsikring',
'uføretidspunktet':'uføretidspunkt',
'uføreytelser':'uføreyt',
'uførhet':'ufør',
'uførheten':'ufør',
'ugagnskråka':'ugagnskråk',
'ugagnskråke':'ugagnskråk',
'uggla':'uggl',
'ugifte':'ugift',
'ugjennomsiktig':'ugjennomsikt',
'ugjennomsiktige':'ugjennomsikt',
'ugjerne':'ugjern',
'ugjerningsmenn':'ugjerningsmenn',
'ugyldig':'ugyld',
'ugyldige':'ugyld',
'uheldig':'uheld',
'uheldige':'uheld',
'uhell':'uhell',
'uhensiktsmessig':'uhensiktsmess',
'uhevnet':'uhevn',
'uhyggelig':'uhygg',
'uhyggelige':'uhygg',
'uhyrer':'uhyr',
'uhyret':'uhyr',
'uinnfridde':'uinnfridd',
'ujamn':'ujamn',
'uke':'uke',
'ukedager':'ukedag',
'uken':'uken',
'ukentlig':'ukent',
'uker':'uker',
'ukjent':'ukjent',
'ukjente':'ukjent',
'uklar':'ukl',
'uklare':'uklar',
'uklarhet':'uklar',
'uklarheter':'uklar',
'uklart':'uklart',
'uklok':'uklok',
'uklokt':'uklokt',
'uklår':'uklår',
'uklåre':'uklår',
'uklårt':'uklårt',
'ukrevet':'ukrev',
'ulage':'ulag',
'ulefoss':'ulefoss',
'ulempa':'ulemp',
'ulempe':'ulemp',
'ulempene':'ulemp',
'ulemper':'ulemp',
'uler':'uler',
'uleselige':'ules',
'ulik':'ulik',
'ulike':'ulik',
'ulikhet':'ulik',
'ulikhetene':'ulik',
'ulikheter':'ulik',
'ulikskap':'ulikskap',
'ulikskapen':'ulikskap',
'ulikt':'ulikt',
'ull':'ull',
'ullen':'ull',
'ulmer':'ulm',
'ulovfesta':'ulovfest',
'ulovleg':'ulov',
'ulv':'ulv',
'ulve':'ulv',
'ulvegapet':'ulvegap',
'ulveham':'ulveham',
'ulvekjeften':'ulvekjeft',
'ulven':'ulv',
'ulvene':'ulv',
'ulver':'ulv',
'ulveskikkelse':'ulveskikk',
'ulvesnute':'ulvesnut',
'ulvestemmer':'ulvestemm',
'ulvetid':'ulvetid',
'ulvungen':'ulvung',
'ulydighet':'ulyd',
'ulykke':'ulykk',
'ulykkelig':'ulykk',
'ulykkelige':'ulykk',
'ulykker':'ulykk',
'ulykkesfugl':'ulykkesfugl',
'ulykkeskostnader':'ulykkeskostnad',
'ulyst':'ulyst',
'ulønsamt':'ulønsamt',
'umettelige':'umett',
'umiddelbar':'umiddelb',
'umiddelbart':'umiddelbart',
'umogleg':'umog',
'umulig':'umu',
'umulige':'umu',
'umyndig':'umynd',
'umyndiggjorde':'umyndiggjord',
'und':'und',
'undamentet':'undament',
'under':'und',
'underdekning':'underdekning',
'underdirektør':'underdirektør',
'underforstått':'underforstått',
'underfulle':'underfull',
'undergang':'undergang',
'undergis':'undergis',
'undergitt':'undergitt',
'undergruppe':'undergrupp',
'underjordisk':'underjordisk',
'underjordiske':'underjordisk',
'underkant':'underkant',
'underkapitler':'underkapitl',
'underkjeven':'underkjev',
'underkurs':'underkur',
'underlag':'underlag',
'underlagde':'underlagd',
'underlagsmateriale':'underlagsmaterial',
'underlagsmaterialet':'underlagsmaterial',
'underlagt':'underlagt',
'underlegges':'underlegg',
'underleveranser':'underleverans',
'underlig':'under',
'underlige':'under',
'underliggende':'underligg',
'underminere':'underminer',
'underordna':'underordn',
'underordnet':'underordn',
'underordnete':'underordnet',
'underpris':'underpris',
'underprising':'underprising',
'underrapportering':'underrapportering',
'underrapporteringen':'underrapportering',
'underskift':'underskift',
'underskifter':'underskift',
'underskot':'underskot',
'underskrift':'underskrift',
'underskriftene':'underskrift',
'underskrifter':'underskrift',
'underskrivast':'underskriv',
'underskrive':'underskriv',
'underskriven':'underskriv',
'underskriving':'underskriving',
'underskudd':'underskudd',
'underskuddet':'underskudd',
'understreke':'understrek',
'understreker':'understrek',
'understrekes':'understrek',
'understøtter':'understøtt',
'undersøke':'undersøk',
'undersøkelse':'undersøk',
'undersøkelsen':'undersøk',
'undersøkelsene':'undersøk',
'undersøkelser':'undersøk',
'undersøkingane':'undersøking',
'undersøkingar':'undersøking',
'undersøkje':'undersøkj',
'undersøkjer':'undersøkj',
'undersøkt':'undersøkt',
'underteiknast':'underteikn',
'undervegs':'underveg',
'underveis':'underveis',
'undervurderer':'undervurder',
'undervurderte':'undervurder',
'undre':'undr',
'undrende':'undr',
'undrer':'undr',
'undres':'undr',
'undring':'undring',
'ung':'ung',
'ungdom':'ungdom',
'ungdommen':'ungdomm',
'ungdoms':'ungdom',
'ungdomsbustader':'ungdomsbustad',
'unge':'ung',
'ungen':'ung',
'unger':'ung',
'ungr':'ungr',
'union':'union',
'unionen':'union',
'uniship':'uniship',
'unit':'unit',
'united':'united',
'unitisering':'unitisering',
'unitiseringsgraden':'unitiseringsgrad',
'unitisert':'unitiser',
'unitiserte':'unitiser',
'universell':'universell',
'universitat':'universitat',
'university':'university',
'unna':'unn',
'unne':'unn',
'unner':'unn',
'unngå':'unngå',
'unngår':'unngår',
'unngås':'unngås',
'unngått':'unngått',
'unnlate':'unnlat',
'unnlater':'unnlat',
'unnlatt':'unnlatt',
'unnslippe':'unnslipp',
'unntak':'unntak',
'unntaka':'unntak',
'unntaket':'unntak',
'unntaksregel':'unntaksregel',
'unntaksregelen':'unntaksregel',
'unntakstilfeller':'unntakstilfell',
'unntaksvis':'unntaksvis',
'unntas':'unnt',
'unntatt':'unntatt',
'unormale':'unormal',
'unødig':'unød',
'unødvendig':'unødvend',
'unødvendige':'unødvend',
'uomtvista':'uomtvist',
'uoppseieleg':'uoppsei',
'uoppseielege':'uoppsei',
'uorden':'uord',
'uoverdrageleg':'uoverdrag',
'uoversiktleg':'uoversikt',
'uoversiktlig':'uoversikt',
'up':'up',
'uppfyller':'uppfyll',
'upplagan':'upplagan',
'uppsala':'uppsal',
'upraktisk':'upraktisk',
'upraktiske':'upraktisk',
'uprøvd':'uprøvd',
'ur':'ur',
'urasjonell':'urasjonell',
'urd':'urd',
'urds':'urd',
'urealisert':'urealiser',
'urealiserte':'urealiser',
'urealistisk':'urealistisk',
'uredd':'uredd',
'uregistrert':'uregistrer',
'uregistrerte':'uregistrer',
'uregulert':'ureguler',
'urette':'urett',
'urettferdig':'urettferd',
'urimeleg':'urim',
'urimelege':'urim',
'urimelig':'urim',
'uro':'uro',
'urolig':'uro',
'urolige':'uro',
'urter':'urt',
'urtiden':'urtid',
'uråd':'uråd',
'urørlig':'urør',
'usa':'usa',
'usakleg':'usak',
'usamde':'usamd',
'uselde':'useld',
'useljeleg':'uselj',
'usemje':'usemj',
'usikker':'usikk',
'usikkerhet':'usikker',
'usikkerheten':'usikker',
'usikkerhetsperiode':'usikkerhetsperiod',
'usikkert':'usikker',
'usikra':'usikr',
'usikre':'usikr',
'uskadd':'uskadd',
'uskadde':'uskadd',
'uskifte':'uskift',
'uskiftet':'uskift',
'uskikk':'uskikk',
'uskyidige':'uskyid',
'uskylden':'uskyld',
'uskyldig':'uskyld',
'uskyldsrene':'uskyldsr',
'uslingen':'usling',
'uslinger':'usling',
'uspesisfisert':'uspesisfiser',
'ustyrtelig':'ustyrt',
'ustøtt':'ustøtt',
'usynlige':'usyn',
'usårlig':'usår',
'ut':'ut',
'utad':'utad',
'utafor':'utafor',
'utaforståande':'utaforstå',
'utafrå':'utafrå',
'utan':'utan',
'utanfrå':'utanfrå',
'utanom':'utanom',
'utarbeidast':'utarbeid',
'utarbeidde':'utarbeidd',
'utarbeide':'utarbeid',
'utarbeidelse':'utarbeid',
'utarbeidelsen':'utarbeid',
'utarbeider':'utarbeid',
'utarbeides':'utarbeid',
'utarbeidet':'utarbeid',
'utarbeiding':'utarbeiding',
'utbedring':'utbedring',
'utbedringer':'utbedring',
'utbetale':'utbetal',
'utbetaler':'utbetal',
'utbetales':'utbetal',
'utbetaling':'utbetaling',
'utbetalingane':'utbetaling',
'utbetalingar':'utbetaling',
'utbetalingen':'utbetaling',
'utbetalingene':'utbetaling',
'utbetalingenes':'utbetaling',
'utbetalinger':'utbetaling',
'utbetalingsdagen':'utbetalingsdag',
'utbetalingsdatoen':'utbetalingsdato',
'utbetalingsfasen':'utbetalingsfas',
'utbetalingsperiode':'utbetalingsperiod',
'utbetalingsperioden':'utbetalingsperiod',
'utbetalingsperiodens':'utbetalingsperiod',
'utbetalingsprofil':'utbetalingsprofil',
'utbetalingsprofilen':'utbetalingsprofil',
'utbetalingsreglene':'utbetalingsregl',
'utbetalingstiden':'utbetalingstid',
'utbetalingstidspunktet':'utbetalingstidspunkt',
'utbetalingsår':'utbetalingsår',
'utbetalt':'utbetalt',
'utbetalte':'utbetalt',
'utbetrast':'utbetr',
'utbetre':'utbetr',
'utbetring':'utbetring',
'utbetringa':'utbetring',
'utbetringsplikt':'utbetringsplikt',
'utbredelse':'utbred',
'utbredelsen':'utbred',
'utbredt':'utbred',
'utbreidd':'utbreidd',
'utbreidde':'utbreidd',
'utbrent':'utbrent',
'utbudet':'utbud',
'utbygd':'utbygd',
'utbygde':'utbygd',
'utbygg':'utbygg',
'utbygger':'utbygg',
'utbygges':'utbygg',
'utbygget':'utbygg',
'utbygging':'utbygging',
'utbygginga':'utbygging',
'utbyggingen':'utbygging',
'utbyggingene':'utbygging',
'utbygginger':'utbygging',
'utbyggings':'utbygging',
'utbyggingsalternativet':'utbyggingsalternativ',
'utbyggingsbehov':'utbyggingsbehov',
'utbyggingsform':'utbyggingsform',
'utbyggingsgrad':'utbyggingsgrad',
'utbyggingskostnader':'utbyggingskostnad',
'utbyggingsmodell':'utbyggingsmodell',
'utbyggingsmodellen':'utbyggingsmodell',
'utbyggingsmuligheter':'utbyggingsmu',
'utbyggingsmønsteret':'utbyggingsmønster',
'utbyggingsområde':'utbyggingsområd',
'utbyggingsplan':'utbyggingsplan',
'utbyggingsplanene':'utbyggingsplan',
'utbyggingsplaner':'utbyggingsplan',
'utbyggingspotensiale':'utbyggingspotensial',
'utbyggingstakten':'utbyggingstakt',
'utbyggingstiltak':'utbyggingstiltak',
'utbyggjar':'utbyggj',
'utbyggjarane':'utbyggjar',
'utbyggjarar':'utbyggjar',
'utbyggjaren':'utbyggjar',
'utbyggjarrolle':'utbyggjarroll',
'utbyggjarsida':'utbyggjarsid',
'utbyggjartilfella':'utbyggjartilfell',
'utbygt':'utbygt',
'utbyte':'utbyt',
'utbytte':'utbytt',
'utdanning':'utdanning',
'utdeling':'utdeling',
'utdelinga':'utdeling',
'utdelingane':'utdeling',
'utdelingar':'utdeling',
'utdelt':'utdelt',
'utdelte':'utdelt',
'utdrag':'utdrag',
'utdypes':'utdyp',
'utdyping':'utdyping',
'utdypninger':'utdypning',
'ute':'ute',
'uteanlegg':'uteanlegg',
'uteareal':'uteareal',
'utearealene':'uteareal',
'utelukke':'utelukk',
'utelukkende':'utelukk',
'utelukker':'utelukk',
'utelukkes':'utelukk',
'utelukket':'utelukk',
'uten':'uten',
'utenfor':'utenfor',
'utenfra':'utenfr',
'utenlands':'utenland',
'utenlandsferjene':'utenlandsferj',
'utenlandsforbindelser':'utenlandsforbind',
'utenlandsk':'utenlandsk',
'utenlandske':'utenlandsk',
'utenlandstrafikk':'utenlandstrafikk',
'utenleg':'uten',
'utenom':'utenom',
'utenriks':'utenriks',
'utenriksfart':'utenriksfart',
'utenrikshandel':'utenrikshandel',
'utenrikshandelen':'utenrikshandel',
'utenrikshandelens':'utenrikshandel',
'utenrikshandelsstatistikk':'utenrikshandelsstatistikk',
'utenrikshandelsstatistikken':'utenrikshandelsstatistikk',
'utenrikshandelstransporter':'utenrikshandelstransport',
'utenrikshavn':'utenrikshavn',
'utenrikstrafikk':'utenrikstrafikk',
'utenrikstransporter':'utenrikstransport',
'uteområde':'uteområd',
'utestengd':'utestengd',
'utestenging':'utestenging',
'utestenginga':'utestenging',
'utette':'utett',
'utetter':'utett',
'utfalding':'utfalding',
'utfall':'utfall',
'utfallet':'utfall',
'utfart':'utfart',
'utferda':'utferd',
'utferdaren':'utferdar',
'utferdast':'utferd',
'utferde':'utferd',
'utferdige':'utferd',
'utferding':'utferding',
'utfisjonering':'utfisjonering',
'utflytting':'utflytting',
'utfor':'utfor',
'utfordre':'utfordr',
'utfordring':'utfordring',
'utfordringen':'utfordring',
'utfordringene':'utfordring',
'utfordringer':'utfordring',
'utforma':'utform',
'utformast':'utform',
'utforme':'utform',
'utformer':'utform',
'utformes':'utform',
'utformet':'utform',
'utforming':'utforming',
'utforminga':'utforming',
'utformingen':'utforming',
'utformingene':'utforming',
'utforminger':'utforming',
'utfyllande':'utfyll',
'utfyllende':'utfyll',
'utfyller':'utfyll',
'utfylles':'utfyll',
'utfylling':'utfylling',
'utfyllingene':'utfylling',
'utfyllinger':'utfylling',
'utfyllingsområde':'utfyllingsområd',
'utførande':'utfør',
'utførast':'utfør',
'utføre':'utfør',
'utførelse':'utfør',
'utfører':'utfør',
'utføres':'utfør',
'utføring':'utføring',
'utføringa':'utføring',
'utførleg':'utfør',
'utførlege':'utfør',
'utført':'utført',
'utgang':'utgang',
'utgangen':'utgang',
'utgangspunkt':'utgangspunkt',
'utgangspunktet':'utgangspunkt',
'utgard':'utgard',
'utgiften':'utgift',
'utgiftene':'utgift',
'utgifter':'utgift',
'utgiftsføres':'utgiftsfør',
'utgiftsgodtgjørelser':'utgiftsgodtgjør',
'utgiftsmønsteret':'utgiftsmønster',
'utgitt':'utgitt',
'utgivelse':'utgiv',
'utgjer':'utgj',
'utgjere':'utgjer',
'utgjeven':'utgjev',
'utgjorde':'utgjord',
'utgjort':'utgjort',
'utgjør':'utgjør',
'utgjøre':'utgjør',
'utgjøres':'utgjør',
'utgreiing':'utgreiing',
'utgreiinga':'utgreiing',
'utgreiingar':'utgreiing',
'utgå':'utgå',
'utgående':'utgå',
'utgår':'utgår',
'utgått':'utgått',
'utholing':'utholing',
'utilfredsstillende':'utilfredsstill',
'utilsiktede':'utilsikt',
'utilsiktet':'utilsikt',
'utjamningsmeldinga':'utjamningsmelding',
'utjevne':'utjevn',
'utjevner':'utjevn',
'utjevnes':'utjevn',
'utjevning':'utjevning',
'utjevningsbehovet':'utjevningsbehov',
'utkanten':'utkant',
'utkantområde':'utkantområd',
'utkast':'utk',
'utkasta':'utkast',
'utkastet':'utkast',
'utkastets':'utkast',
'utkasting':'utkasting',
'utkastingssaker':'utkastingssak',
'utkjøring':'utkjøring',
'utland':'utland',
'utlandet':'utland',
'utlegg':'utlegg',
'utlegget':'utlegg',
'utlegging':'utlegging',
'utleggspant':'utleggspant',
'utleggstakaren':'utleggstakar',
'utleid':'utleid',
'utleidd':'utleidd',
'utleidde':'utleidd',
'utleie':'utlei',
'utleieren':'utleier',
'utleiga':'utl',
'utleigar':'utl',
'utleigarar':'utleigar',
'utleigaren':'utleigar',
'utleigarselskap':'utleigarselskap',
'utleigd':'utleigd',
'utleige':'utl',
'utleigebustad':'utleigebustad',
'utleigebustadene':'utleigebustad',
'utleigebustader':'utleigebustad',
'utleigebygg':'utleigebygg',
'utleigeeigdom':'utleigeeigdom',
'utleigeeigedom':'utleigeeigedom',
'utleigeeigedommar':'utleigeeigedomm',
'utleigemodellar':'utleigemodell',
'utleigeobjekt':'utleigeobjekt',
'utleigeselskap':'utleigeselskap',
'utleigeverksemd':'utleigeverksemd',
'utleigeverksemda':'utleigeverksemd',
'utlikner':'utlikn',
'utlikninga':'utlikning',
'utlån':'utlån',
'utlåna':'utlån',
'utlånaren':'utlånar',
'utlånsordningane':'utlånsordning',
'utlånsverksemd':'utlånsverksemd',
'utløp':'utløp',
'utløpet':'utløp',
'utløse':'utløs',
'utløser':'utløs',
'utløses':'utløs',
'utløst':'utløst',
'utløyser':'utløys',
'utløysing':'utløysing',
'utløysinga':'utløysing',
'utløysingsgrunnen':'utløysingsgrunn',
'utløysingsregel':'utløysingsregel',
'utløysingssaka':'utløysingssak',
'utløysingssummen':'utløysingssumm',
'utløyst':'utløyst',
'utmarkande':'utmark',
'utmattet':'utmatt',
'utmeldelse':'utmeld',
'utmelding':'utmelding',
'utmeldinga':'utmelding',
'utmerket':'utmerk',
'utmåles':'utmål',
'utmålinga':'utmåling',
'utnytta':'utnytt',
'utnytte':'utnytt',
'utnyttelse':'utnytt',
'utnyttelsen':'utnytt',
'utnyttelsesgrad':'utnyttelsesgrad',
'utnyttes':'utnytt',
'utnytting':'utnytting',
'utnyttinga':'utnytting',
'utover':'utov',
'utpeiking':'utpeiking',
'utpeikte':'utpeikt',
'utpekt':'utpekt',
'utpenslinga':'utpensling',
'utporsjoneres':'utporsjoner',
'utprøving':'utprøving',
'utpå':'utpå',
'utrasing':'utrasing',
'utrede':'utr',
'utredes':'utr',
'utredet':'utred',
'utredning':'utredning',
'utredningen':'utredning',
'utredningens':'utredning',
'utredninger':'utredning',
'utredningsarbeid':'utredningsarbeid',
'utredningsarbeider':'utredningsarbeid',
'utredningsbehov':'utredningsbehov',
'utredningsinstruksen':'utredningsinstruks',
'utredningsplikten':'utredningsplikt',
'utredningsprosjekter':'utredningsprosjekt',
'utrenet':'utren',
'utro':'utro',
'utrolig':'utro',
'utrolige':'utro',
'utrustning':'utrustning',
'utrygg':'utrygg',
'utrygge':'utrygg',
'utrygt':'utrygt',
'utrådt':'utråd',
'utrøstelig':'utrøst',
'utsagn':'utsagn',
'utsatt':'utsatt',
'utseende':'utse',
'utseendet':'utseend',
'utsetjande':'utsetj',
'utsetje':'utsetj',
'utsett':'utsett',
'utsette':'utsett',
'utsettelse':'utsett',
'utsettes':'utsett',
'utsikt':'utsikt',
'utsiktene':'utsikt',
'utsikter':'utsikt',
'utskifting':'utskifting',
'utskiftingar':'utskifting',
'utskiljing':'utskiljing',
'utskillelse':'utskill',
'utskillelsen':'utskill',
'utskilles':'utskill',
'utskilling':'utskilling',
'utskilt':'utskilt',
'utskilte':'utskilt',
'utskiping':'utskiping',
'utskjemt':'utskjemt',
'utskrift':'utskrift',
'utskriving':'utskriving',
'utskårne':'utskårn',
'utslag':'utslag',
'utslaget':'utslag',
'utslagsgivende':'utslagsgiv',
'utslipp':'utslipp',
'utslippskostnader':'utslippskostnad',
'utslått':'utslått',
'utslåtte':'utslått',
'utspring':'utspring',
'utstede':'utst',
'utstedelse':'utsted',
'utsteder':'utsted',
'utstedes':'utst',
'utstedt':'utsted',
'utstengd':'utstengd',
'utstenging':'utstenging',
'utstenginga':'utstenging',
'utstoppete':'utstoppet',
'utstrakt':'utstrakt',
'utstrekning':'utstrekning',
'utstyr':'utstyr',
'utstyres':'utstyr',
'utstyret':'utstyr',
'utsyn':'utsyn',
'uttak':'uttak',
'uttala':'uttal',
'uttale':'uttal',
'uttalelse':'uttal',
'uttalelser':'uttal',
'uttaler':'uttal',
'uttalt':'uttalt',
'uttalte':'uttalt',
'uttrade':'uttrad',
'uttransport':'uttransport',
'uttreden':'uttred',
'uttrykk':'uttrykk',
'uttrykkelig':'uttrykk',
'uttrykkene':'uttrykk',
'uttrykker':'uttrykk',
'uttrykket':'uttrykk',
'uttrykt':'uttrykt',
'uttømmande':'uttømm',
'uttømmende':'uttømm',
'utval':'utval',
'utvala':'utval',
'utvalet':'utval',
'utvalets':'utval',
'utvalg':'utvalg',
'utvalget':'utvalg',
'utvalgets':'utvalg',
'utvalgsarbeid':'utvalgsarbeid',
'utvalgsarbeidet':'utvalgsarbeid',
'utvalgsundersøkelse':'utvalgsundersøk',
'utvalgte':'utvalgt',
'utvalsmedlemmene':'utvalsmedlemm',
'utvalsmedlemmer':'utvalsmedlemm',
'utvatna':'utvatn',
'utveg':'utveg',
'utvegen':'utveg',
'utvei':'utvei',
'utvendig':'utvend',
'utvida':'utvid',
'utvide':'utvid',
'utvidede':'utvid',
'utvidelse':'utvid',
'utvidelsen':'utvid',
'utvidelser':'utvid',
'utvidelsesmuligheter':'utvidelsesmu',
'utvidelsesplanene':'utvidelsesplan',
'utvidelsesplaner':'utvidelsesplan',
'utvidelsespotensiale':'utvidelsespotensial',
'utvidelsestrinnet':'utvidelsestrinn',
'utvider':'utvid',
'utvides':'utvid',
'utvidet':'utvid',
'utvidete':'utvidet',
'utvidingar':'utviding',
'utvikla':'utvikl',
'utvikle':'utvikl',
'utvikler':'utvikl',
'utvikles':'utvikl',
'utviklet':'utvikl',
'utvikling':'utvikling',
'utviklinga':'utvikling',
'utviklingen':'utvikling',
'utviklingpotensiale':'utviklingpotensial',
'utviklingsbaner':'utviklingsban',
'utviklingsforløp':'utviklingsforløp',
'utviklingsmuligheter':'utviklingsmu',
'utviklingsområder':'utviklingsområd',
'utviklingsplan':'utviklingsplan',
'utviklingsplanene':'utviklingsplan',
'utviklingsplaner':'utviklingsplan',
'utviklingsprosjekt':'utviklingsprosjekt',
'utviklingstendens':'utviklingstend',
'utviklingstendenser':'utviklingstendens',
'utviklingstrekk':'utviklingstrekk',
'utvise':'utvis',
'utviska':'utvisk',
'utydelig':'utyd',
'utyskene':'utysk',
'utysket':'utysk',
'utålmodig':'utålmod',
'utøvande':'utøv',
'utøvast':'utøv',
'utøve':'utøv',
'utøvelse':'utøv',
'utøvelsen':'utøv',
'utøver':'utøv',
'utøves':'utøv',
'utøvinga':'utøving',
'uvand':'uvand',
'uvanleg':'uvan',
'uvanlege':'uvan',
'uvanlig':'uvan',
'uvant':'uvant',
'uvedkomande':'uvedkom',
'uvenns':'uvenn',
'uventa':'uvent',
'uventede':'uvent',
'uventet':'uvent',
'uver':'uver',
'uvesentleg':'uvesent',
'uvisse':'uviss',
'uvisshet':'uviss',
'uværet':'uvær',
'v':'v',
'vader':'vad',
'vagger':'vagg',
'vagt':'vagt',
'vakker':'vakk',
'vakkert':'vakker',
'vakler':'vakl',
'vakre':'vakr',
'vakrere':'vakrer',
'vakreste':'vakrest',
'vakse':'vaks',
'vakt':'vakt',
'vakter':'vakt',
'vaktmann':'vaktmann',
'vaktmeistertenester':'vaktmeistertenest',
'vaktmenn':'vaktmenn',
'vaktmester':'vaktmest',
'vaktpost':'vaktpost',
'vaktsomme':'vaktsomm',
'vakttårnet':'vakttårn',
'val':'val',
'vald':'vald',
'valda':'vald',
'valdar':'vald',
'valde':'vald',
'vale':'val',
'valet':'val',
'valfridom':'valfridom',
'valfridommen':'valfridomm',
'valg':'valg',
'valgene':'valg',
'valget':'valg',
'valgfrihet':'valgfri',
'valgfritt':'valgfritt',
'valgmulighetene':'valgmu',
'valgmuligheter':'valgmu',
'valgrett':'valgrett',
'valgt':'valgt',
'valgte':'valgt',
'valhall':'valhall',
'valkomite':'valkomit',
'valkomiteen':'valkomite',
'valkyrjen':'valkyrj',
'valkyrjene':'valkyrj',
'valkyrjer':'valkyrj',
'valmåten':'valmåt',
'valperioden':'valperiod',
'valrett':'valrett',
'valretten':'valrett',
'valt':'valt',
'vanaheim':'vanaheim',
'vandalene':'vandal',
'vande':'vand',
'vandre':'vandr',
'vandrende':'vandr',
'vandrer':'vandr',
'vandret':'vandr',
'vandringene':'vandring',
'vandringsmann':'vandringsmann',
'vandringsmannen':'vandringsmann',
'vandringsstaven':'vandringsstav',
'vanene':'van',
'vanenes':'van',
'vaner':'van',
'vaneslekt':'vaneslekt',
'vanleg':'van',
'vanlege':'van',
'vanlegvis':'vanlegvis',
'vanlig':'van',
'vanlige':'van',
'vanligste':'vanligst',
'vanligvis':'vanligvis',
'vann':'vann',
'vannet':'vann',
'vannkanten':'vannkant',
'vannkvalitet':'vannkvalit',
'vanns':'vann',
'vanntroll':'vanntroll',
'vannveger':'vannveg',
'vanskane':'vansk',
'vanskar':'vansk',
'vanske':'vansk',
'vanskeleg':'vansk',
'vanskelegare':'vanskelegar',
'vanskelege':'vansk',
'vanskelegstilte':'vanskelegstilt',
'vanskelig':'vansk',
'vanskelige':'vansk',
'vanskeligere':'vanskeliger',
'vanskeliggjøre':'vanskeliggjør',
'vanskeligheter':'vansk',
'vanskeligstilte':'vanskeligstilt',
'vansker':'vansk',
'vant':'vant',
'vanta':'vant',
'vantar':'vant',
'vante':'vant',
'vantro':'vantro',
'vanvidd':'vanvidd',
'vanviddet':'vanvidd',
'var':'var',
'varafolk':'varafolk',
'varaleiaren':'varaleiar',
'varamedlem':'varamedlem',
'varamedlemmen':'varamedlemm',
'varamedlemmer':'varamedlemm',
'varandra':'varandr',
'varanger':'varang',
'varangerfjorden':'varangerfjord',
'vararevisor':'vararevisor',
'vararevisorar':'vararevisor',
'vare':'var',
'varedistribusjon':'varedistribusjon',
'vareeier':'vareei',
'vareeiere':'vareeier',
'vareeierene':'vareeier',
'vareeierens':'vareeier',
'vareeierne':'vareeiern',
'vareeiernes':'vareeiern',
'vareeiers':'vareei',
'vareflyt':'vareflyt',
'varegrupper':'varegrupp',
'varehandel':'varehandel',
'varehandelen':'varehandel',
'varehandelskjedene':'varehandelskjed',
'vareinnsats':'vareinnsat',
'varemedlemmer':'varemedlemm',
'varen':'var',
'varene':'var',
'varens':'var',
'vareomsetning':'vareomsetning',
'vareproduksjon':'vareproduksjon',
'vareproduksjonen':'vareproduksjon',
'vareprodusenter':'vareprodusent',
'vareprofil':'vareprofil',
'varer':'var',
'vareslag':'vareslag',
'varestrømmene':'varestrømm',
'varetransport':'varetransport',
'varetypene':'varetyp',
'varetyper':'varetyp',
'vareutval':'vareutval',
'vareverdi':'vareverdi',
'vareverdiene':'vareverdi',
'varg':'varg',
'variabel':'variabel',
'variablene':'variabl',
'varians':'varian',
'variant':'variant',
'variantar':'variant',
'varianten':'variant',
'variasjon':'variasjon',
'variasjonane':'variasjon',
'variasjonene':'variasjon',
'variasjoner':'variasjon',
'variasjonsbreidd':'variasjonsbreidd',
'varierande':'varier',
'variere':'varier',
'varierende':'varier',
'varierer':'varier',
'variert':'varier',
'varierte':'varier',
'varig':'var',
'varige':'var',
'varighet':'var',
'varm':'varm',
'varme':'varm',
'varmen':'varm',
'varmer':'varm',
'varmesentralar':'varmesentral',
'varmeste':'varmest',
'varmt':'varmt',
'varsam':'varsam',
'varsel':'varsel',
'varselet':'varsel',
'varsla':'varsl',
'varslast':'varsl',
'varsle':'varsl',
'varslet':'varsl',
'varsling':'varsling',
'vart':'vart',
'varte':'vart',
'vask':'vask',
'vasker':'vask',
'vaskeri':'vaskeri',
'vaskes':'vask',
'vasket':'vask',
'vasser':'vass',
'vasst':'vasst',
'vassum':'vassum',
'vatn':'vatn',
've':'ve',
'ved':'ved',
'veddemål':'veddemål',
'veddemålet':'veddemål',
'vedder':'vedd',
'vederlag':'vederlag',
'vederlaget':'vederlag',
'vederlags':'vederlag',
'vederlagssatsar':'vederlagssats',
'vedgår':'vedgår',
'vedkjem':'vedkjem',
'vedkomande':'vedkom',
'vedkommende':'vedkomm',
'vedkommendes':'vedkomm',
'vedlagte':'vedlagt',
'vedlegg':'vedlegg',
'vedlegga':'vedlegg',
'vedlegget':'vedlegg',
'vedleggets':'vedlegg',
'vedlikehald':'vedlikehald',
'vedlikehalden':'vedlikehald',
'vedlikehaldet':'vedlikehald',
'vedlikehalds':'vedlikehald',
'vedlikehaldsarbeid':'vedlikehaldsarbeid',
'vedlikehaldsmanglar':'vedlikehaldsmangl',
'vedlikehaldsplikt':'vedlikehaldsplikt',
'vedlikehaldsplikta':'vedlikehaldsplikt',
'vedlikehaldspliktene':'vedlikehaldsplikt',
'vedlikehaldsreglane':'vedlikehaldsregl',
'vedlikehold':'vedlikehold',
'vedrørende':'vedrør',
'vedrører':'vedrør',
'vedskive':'vedskiv',
'vedta':'vedt',
'vedtagelse':'vedtag',
'vedtagelsen':'vedtag',
'vedtak':'vedtak',
'vedtaka':'vedtak',
'vedtakast':'vedtak',
'vedtaket':'vedtak',
'vedtaking':'vedtaking',
'vedtakinga':'vedtaking',
'vedtaksført':'vedtaksført',
'vedtaksmessig':'vedtaksmess',
'vedtaksmyndighet':'vedtaksmynd',
'vedtaksprosedyre':'vedtaksprosedyr',
'vedtaksprosedyren':'vedtaksprosedyr',
'vedtaksprotokoll':'vedtaksprotokoll',
'vedtar':'vedt',
'vedtas':'vedt',
'vedtatt':'vedtatt',
'vedtatte':'vedtatt',
'vedtek':'vedtek',
'vedteke':'vedtek',
'vedteken':'vedtek',
'vedtekne':'vedtekn',
'vedtekstfesta':'vedtekstfest',
'vedtektene':'vedtekt',
'vedtektenes':'vedtekt',
'vedtekter':'vedtekt',
'vedtektfeste':'vedtektfest',
'vedtektsendring':'vedtektsendring',
'vedtektsendringane':'vedtektsendring',
'vedtektsendringar':'vedtektsendring',
'vedtektsfesta':'vedtektsfest',
'vedtektsfestar':'vedtektsfest',
'vedtektsfestast':'vedtektsfest',
'vedtektsfeste':'vedtektsfest',
'vedtektsfestede':'vedtektsfest',
'vedtektsfesting':'vedtektsfesting',
'vedtektsfleirtal':'vedtektsfleirtal',
'vedtektsføresegn':'vedtektsføresegn',
'vedtektsføresegna':'vedtektsføresegn',
'vedtektsføresegnene':'vedtektsføresegn',
'vedtektsføresegner':'vedtektsføresegn',
'vedtektsklausul':'vedtektsklausul',
'vedtektsklausular':'vedtektsklausul',
'vedtektsveto':'vedtektsveto',
'vedtektsvetoet':'vedtektsveto',
'vedtok':'vedtok',
'veene':'veen',
'veg':'veg',
'veganlegg':'veganlegg',
'vegar':'veg',
'vegast':'veg',
'vegdirektoratet':'vegdirektorat',
'vegen':'veg',
'vegene':'veg',
'veger':'veg',
'vegforbindelse':'vegforbind',
'vegforbindelsen':'vegforbind',
'vegforbindelsene':'vegforbind',
'vegformål':'vegformål',
'vegframføring':'vegframføring',
'vegføringer':'vegføring',
'vegg':'vegg',
'veggen':'vegg',
'veggene':'vegg',
'vegger':'vegg',
'veggkonstruksjonar':'veggkonstruksjon',
'vegkontoret':'vegkontor',
'vegkorridorer':'vegkorridor',
'vegkostnader':'vegkostnad',
'vegløsning':'vegløsning',
'vegløsningene':'vegløsning',
'vegløsninger':'vegløsning',
'vegne':'vegn',
'vegner':'vegn',
'vegnett':'vegnett',
'vegnettet':'vegnett',
'vegnettets':'vegnett',
'vegnormal':'vegnormal',
'vegpakke':'vegpakk',
'vegplaner':'vegplan',
'vegruten':'vegrut',
'vegsalt':'vegsalt',
'vegsiden':'vegsid',
'vegsituasjonen':'vegsituasjon',
'vegslitasje':'vegslitasj',
'vegstrekningen':'vegstrekning',
'vegstrekninger':'vegstrekning',
'vegsystem':'vegsystem',
'vegsystemene':'vegsystem',
'vegsystemer':'vegsystem',
'vegtam':'vegtam',
'vegtilknytning':'vegtilknytning',
'vegtilknytningen':'vegtilknytning',
'vegtiltak':'vegtiltak',
'vegtrafikk':'vegtrafikk',
'vegtrafikken':'vegtrafikk',
'vegtrafikkplan':'vegtrafikkplan',
'vegtransport':'vegtransport',
'vegtransporten':'vegtransport',
'vegtransportkostnadene':'vegtransportkostnad',
'vegtraseen':'vegtrase',
'vegvesen':'vegves',
'vegvesens':'vegves',
'vei':'vei',
'veie':'veie',
'veien':'veien',
'veier':'veier',
'veierud':'veierud',
'veies':'veies',
'veiet':'veiet',
'veik':'veik',
'veikare':'veikar',
'veike':'veik',
'veikro':'veikro',
'veiledende':'veiled',
'veiledning':'veiledning',
'veit':'veit',
'veiver':'veiv',
'veka':'vek',
'veke':'vek',
'veker':'vek',
'vekers':'vek',
'vekes':'vek',
'veket':'vek',
'vekk':'vekk',
'vekke':'vekk',
'vekker':'vekk',
'vekkes':'vekk',
'vekket':'vekk',
'vekse':'veks',
'vekselflak':'vekselflak',
'vekslande':'veksl',
'vekst':'vekst',
'vekstanslag':'vekstanslag',
'veksten':'vekst',
'vekstforutsetningene':'vekstforutsetning',
'vekstkraftig':'vekstkraft',
'vekstområde':'vekstområd',
'vekstpotensial':'vekstpotensial',
'vekstpotensiale':'vekstpotensial',
'vekstpotensialet':'vekstpotensial',
'vekstrate':'vekstrat',
'vekstratene':'vekstrat',
'vekstrater':'vekstrat',
'veksttakten':'veksttakt',
'veksttendenser':'veksttendens',
'veksttrend':'veksttrend',
'vekt':'vekt',
'vekta':'vekt',
'vektede':'vekt',
'vekten':'vekt',
'vektenhet':'vekten',
'vekter':'vekt',
'vektet':'vekt',
'vekting':'vekting',
'vektlagt':'vektlagt',
'vektlegge':'vektlegg',
'vektlegges':'vektlegg',
'vektlegging':'vektlegging',
'vektleggingen':'vektlegging',
'vel':'vel',
'velbegrunnede':'velbegrunn',
'velberget':'velberg',
'veldig':'veld',
'veldige':'veld',
'velegnet':'velegn',
'veletablerte':'veletabler',
'velferd':'velferd',
'velferdsgode':'velferdsgod',
'velferdsmeldingen':'velferdsmelding',
'velferdsmeldingens':'velferdsmelding',
'velferdsordninger':'velferdsordning',
'velferdstilbod':'velferdstilbod',
'velferdstiltak':'velferdstiltak',
'velforeining':'velforeining',
'velfunderte':'velfunder',
'velg':'velg',
'velge':'velg',
'velger':'velg',
'velges':'velg',
'velgjerd':'velgjerd',
'veljast':'velj',
'velje':'velj',
'velkjend':'velkjend',
'velkommen':'velkomm',
'velluktende':'vellukt',
'vellykkete':'vellykket',
'velsigner':'velsign',
'velskapt':'velskapt',
'velstand':'velstand',
'velta':'velt',
'veltalende':'veltal',
'veltalenhet':'veltalen',
'veltast':'velt',
'velte':'velt',
'velter':'velt',
'veltet':'velt',
'velutformede':'velutform',
'vende':'vend',
'vender':'vend',
'vendes':'vend',
'vendt':'vend',
'vendte':'vend',
'vene':'ven',
'venn':'venn',
'vennen':'venn',
'vennene':'venn',
'venner':'venn',
'vennet':'venn',
'vennlig':'venn',
'vennlige':'venn',
'vennskap':'vennskap',
'vennskapet':'vennskap',
'venstre':'venstr',
'venstreparti':'venstreparti',
'vent':'vent',
'vente':'vent',
'venteleg':'vent',
'ventelister':'ventelist',
'ventende':'vent',
'venter':'vent',
'ventes':'vent',
'ventet':'vent',
'ventetid':'ventetid',
'ventilar':'ventil',
'ver':'ver',
'vera':'ver',
'verande':'ver',
'verd':'verd',
'verda':'verd',
'verdande':'verd',
'verden':'verd',
'verdener':'verden',
'verdens':'verd',
'verdensbanken':'verdensbank',
'verdensbankens':'verdensbank',
'verdenshavet':'verdenshav',
'verdenstreet':'verdenstre',
'verdi':'verdi',
'verdiane':'verdi',
'verdiansettelse':'verdiansett',
'verdiar':'verdi',
'verdien':'verdi',
'verdiene':'verdi',
'verdier':'verdi',
'verdifall':'verdifall',
'verdifastsettelser':'verdifastsett',
'verdiforholda':'verdiforhold',
'verdiforholdet':'verdiforhold',
'verdifulle':'verdifull',
'verdifullt':'verdifullt',
'verdig':'verd',
'verdighet':'verd',
'verdikjeden':'verdikjed',
'verdikjedene':'verdikjed',
'verdikjeder':'verdikjed',
'verdilaus':'verdilaus',
'verdipapir':'verdipapir',
'verdipapirene':'verdipapir',
'verdipapirer':'verdipapir',
'verdipapirfond':'verdipapirfond',
'verdipapirfondenes':'verdipapirfond',
'verdipapirfondloven':'verdipapirfond',
'verdipapirfondsforvaltning':'verdipapirfondsforvaltning',
'verdipapirfondsforvaltningen':'verdipapirfondsforvaltning',
'verdipapirfondsloven':'verdipapirfond',
'verdipapirhandel':'verdipapirhandel',
'verdipapirsentralen':'verdipapirsentral',
'verdireduksjon':'verdireduksjon',
'verdireduksjonen':'verdireduksjon',
'verdiskaping':'verdiskaping',
'verdiskapingen':'verdiskaping',
'verdiskapning':'verdiskapning',
'verdistiging':'verdistiging',
'verdistigning':'verdistigning',
'verditilhøva':'verditilhøv',
'verdiutvikling':'verdiutvikling',
'verdiutviklingen':'verdiutvikling',
'verdiøkning':'verdiøkning',
'verdiøkninger':'verdiøkning',
'verdsetjing':'verdsetjing',
'verdskrigen':'verdskr',
'verdt':'verd',
'vere':'ver',
'verge':'verg',
'vergeløs':'vergeløs',
'veritas':'verit',
'verk':'verk',
'verka':'verk',
'verkar':'verk',
'verke':'verk',
'verkeleg':'verk',
'verkemiddel':'verkemiddel',
'verken':'verk',
'verkeområde':'verkeområd',
'verkeområdet':'verkeområd',
'verknad':'verknad',
'verknadane':'verknad',
'verknaden':'verknad',
'verknadene':'verknad',
'verknader':'verknad',
'verks':'verk',
'verksamhet':'verksam',
'verksemd':'verksemd',
'verksemda':'verksemd',
'verksemdsområde':'verksemdsområd',
'verksemdstypar':'verksemdstyp',
'verktøy':'verktøy',
'vern':'vern',
'verna':'vern',
'vernast':'vern',
'verne':'vern',
'vernende':'vern',
'verner':'vern',
'vernereglar':'verneregl',
'vernes':'vern',
'vernet':'vern',
'vernetiltak':'vernetiltak',
'verneverdier':'verneverdi',
'verre':'verr',
'versjonen':'versjon',
'verst':'verst',
'verste':'verst',
'vert':'vert',
'vertsfolket':'vertsfolk',
'vertskap':'vertskap',
'vertskommunen':'vertskommun',
'vertskommuner':'vertskommun',
'vertslandet':'vertsland',
'verv':'verv',
'vervet':'verv',
'vesen':'ves',
'vesenene':'vesen',
'vesener':'vesen',
'vesentleg':'vesent',
'vesentlege':'vesent',
'vesentlig':'vesent',
'vesentlige':'vesent',
'vesentligste':'vesentligst',
'vesle':'vesl',
'vest':'vest',
'vesterelven':'vesterelv',
'vestfjorden':'vestfjord',
'vestfold':'vestfold',
'vestfoldbanen':'vestfoldban',
'vesthavn':'vesthavn',
'vestlige':'vest',
'vestover':'vestov',
'vestre':'vestr',
'vestsiden':'vestsid',
'vet':'vet',
'veto':'veto',
'vetoet':'veto',
'vett':'vett',
'vettene':'vett',
'vetter':'vett',
'vettet':'vett',
'vev':'vev',
'vevsider':'vevsid',
'vi':'vi',
'via':'via',
'vibeke':'vibek',
'victorin':'victorin',
'vid':'vid',
'vidar':'vid',
'vidare':'vidar',
'vidareførast':'vidarefør',
'vidareføre':'vidarefør',
'vidareføring':'vidareføring',
'vidareført':'vidareført',
'vidaresal':'vidaresal',
'vidareutvikling':'vidareutvikling',
'vidder':'vidd',
'vide':'vid',
'videokamera':'videokamer',
'videre':'vider',
'viderebefordres':'viderebefordr',
'viderebefordring':'viderebefordring',
'videredistribusjon':'videredistribusjon',
'videredistribusjonen':'videredistribusjon',
'videreføre':'viderefør',
'viderefører':'viderefør',
'videreføres':'viderefør',
'videreføring':'videreføring',
'videreført':'videreført',
'videretransport':'videretransport',
'videreutvikling':'videreutvikling',
'videreutviklingen':'videreutvikling',
'vidgrunna':'vidgrunn',
'vidi':'vidi',
'vidt':'vidt',
'vidtgående':'vidtgå',
'vies':'vies',
'viet':'viet',
'vifte':'vift',
'vifter':'vift',
'vigrid':'vigrid',
'vii':'vii',
'viii':'viii',
'vik':'vik',
'vike':'vik',
'viker':'vik',
'vikinga':'viking',
'viktig':'vikt',
'viktigare':'viktigar',
'viktigast':'vikt',
'viktigaste':'viktigast',
'viktige':'vikt',
'viktigere':'viktiger',
'viktighet':'vikt',
'viktigheten':'vikt',
'viktigst':'viktigst',
'viktigste':'viktigst',
'vil':'vil',
'vilja':'vilj',
'vilje':'vilj',
'viljeløse':'viljeløs',
'viljug':'viljug',
'viljuge':'viljug',
'vilken':'vilk',
'vilkår':'vilkår',
'vilkåra':'vilkår',
'vilkårene':'vilkår',
'vilkåret':'vilkår',
'vilkårleg':'vilkår',
'vilkårlege':'vilkår',
'vill':'vill',
'ville':'vill',
'villere':'viller',
'villeste':'villest',
'villhester':'villhest',
'villig':'vil',
'villkor':'villkor',
'villmann':'villmann',
'villmannens':'villmann',
'villrede':'villr',
'vilt':'vilt',
'vimsete':'vimset',
'vimur':'vimur',
'vin':'vin',
'vind':'vind',
'vindauge':'vindaug',
'vinden':'vind',
'vinder':'vind',
'vindforholdene':'vindforhold',
'vindkast':'vindk',
'vindpust':'vindpust',
'vinduene':'vindu',
'vingene':'ving',
'vinger':'ving',
'vingeslag':'vingeslag',
'vingeslagene':'vingeslag',
'vinglete':'vinglet',
'vinilene':'vinil',
'vinilenes':'vinil',
'vinker':'vink',
'vinket':'vink',
'vinkling':'vinkling',
'vinn':'vinn',
'vinnast':'vinn',
'vinne':'vinn',
'vinner':'vinn',
'vinninger':'vinning',
'vinst':'vinst',
'vinstar':'vinst',
'vinsten':'vinst',
'vinter':'vint',
'vinterbro':'vinterbro',
'vinterdag':'vinterdag',
'vintere':'vinter',
'vinteren':'vinter',
'vintre':'vintr',
'vippetangen':'vippetang',
'vippetangens':'vippetang',
'virke':'virk',
'virkedag':'virkedag',
'virkedøgn':'virkedøgn',
'virkelig':'virk',
'virkelige':'virk',
'virkeligheten':'virk',
'virkemiddel':'virkemiddel',
'virkemiddelbruk':'virkemiddelbruk',
'virkemiddelet':'virkemiddel',
'virkemidlene':'virkemidl',
'virkemidler':'virkemidl',
'virkeområde':'virkeområd',
'virker':'virk',
'virket':'virk',
'virkning':'virkning',
'virkningen':'virkning',
'virkningene':'virkning',
'virkninger':'virkning',
'virksom':'virksom',
'virksomhet':'virksom',
'virksomheten':'virksom',
'virksomhetene':'virksom',
'virksomhetens':'virksom',
'virksomheter':'virksom',
'virksomhetsbegrep':'virksomhetsbegrep',
'virksomhetsbegrepet':'virksomhetsbegrep',
'virksomhetsoverdragelse':'virksomhetsoverdrag',
'virksomhetsoverdragelsen':'virksomhetsoverdrag',
'virksomhetsoverdragelser':'virksomhetsoverdrag',
'virksomhetsutøvelsen':'virksomhetsutøv',
'virvlende':'virvl',
'vis':'vis',
'visast':'vis',
'visdom':'visdom',
'visdommens':'visdomm',
'visdomskrefter':'visdomskreft',
'vise':'vis',
'viser':'vis',
'visere':'viser',
'vises':'vis',
'viset':'vis',
'visjoner':'visjon',
'vislie':'visli',
'vismenn':'vismenn',
'visne':'visn',
'viss':'viss',
'vissa':'viss',
'visse':'viss',
'visst':'visst',
'visste':'visst',
'vist':'vist',
'viste':'vist',
'visuelle':'visuell',
'visuelt':'visuelt',
'vit':'vit',
'vital':'vital',
'vitalisere':'vitaliser',
'vite':'vit',
'vitne':'vitn',
'vogn':'vogn',
'vognen':'vogn',
'vognene':'vogn',
'vognskifteområde':'vognskifteområd',
'vogntog':'vogntog',
'voks':'voks',
'vokse':'voks',
'voksen':'voks',
'vokser':'voks',
'voksne':'voksn',
'vokst':'vokst',
'vokste':'vokst',
'vokte':'vokt',
'vokter':'vokt',
'vold':'vold',
'voldgift':'voldgift',
'voldsfjorden':'voldsfjord',
'voldsom':'voldsom',
'voldsomme':'voldsomm',
'voldsomt':'voldsomt',
'voldtas':'vold',
'volen':'vol',
'volker':'volk',
'vollen':'voll',
'vollsfjorden':'vollsfjord',
'volsung':'volsung',
'volumer':'volum',
'volumet':'volum',
'volummessig':'volummess',
'volumutvikling':'volumutvikling',
'volumvekst':'volumvekst',
'volve':'volv',
'volven':'volv',
'von':'von',
'vond':'vond',
'vonde':'vond',
'vondt':'vond',
'voner':'von',
'vore':'vor',
'vorte':'vort',
'vorten':'vort',
'vorter':'vort',
'vortne':'vortn',
'voss':'voss',
'voteringsforslag':'voteringsforslag',
'vott':'vott',
'votten':'vott',
'votter':'vott',
'vrang':'vrang',
'vrangt':'vrangt',
'vred':'vred',
'vrenge':'vreng',
'vrenger':'vreng',
'vri':'vri',
'vridd':'vridd',
'vridde':'vridd',
'vridende':'vrid',
'vridning':'vridning',
'vrikke':'vrikk',
'vrikker':'vrikk',
'vrinsker':'vrinsk',
'vrir':'vrir',
'vris':'vris',
'vrister':'vrist',
'vuggen':'vugg',
'vugger':'vugg',
'vunnet':'vunn',
'vurderast':'vurder',
'vurdere':'vurder',
'vurderer':'vurder',
'vurderes':'vurder',
'vurdering':'vurdering',
'vurderinga':'vurdering',
'vurderingar':'vurdering',
'vurderingen':'vurdering',
'vurderingene':'vurdering',
'vurderinger':'vurdering',
'vurdert':'vurder',
'vurderte':'vurder',
'vvo':'vvo',
'våge':'våg',
'våger':'våg',
'vågestykket':'vågestykk',
'våget':'våg',
'våken':'våk',
'våkne':'våkn',
'våkner':'våkn',
'våknet':'våkn',
'våle':'vål',
'våpen':'våp',
'våpenlek':'våpenlek',
'våpenleken':'våpenlek',
'våpenløs':'våpenløs',
'våpnene':'våpn',
'vår':'vår',
'våre':'vår',
'våren':'vår',
'våres':'vår',
'vårgresset':'vårgress',
'vårt':'vårt',
'våt':'våt',
'våtbulk':'våtbulk',
'våte':'våt',
'vått':'vått',
'vægne':'vægn',
'vækerøkrysset':'vækerøkryss',
'væpnet':'væpn',
'vær':'vær',
'være':'vær',
'værende':'vær',
'været':'vær',
'værforhold':'værforhold',
'værforholdene':'værforhold',
'værhår':'værhår',
'værs':'vær',
'vært':'vært',
'væske':'væsk',
'væsker':'væsk',
'wealth':'wealth',
'welche':'welch',
'wenche':'wench',
'wessel':'wessel',
'wielockx':'wielockx',
'willy':'willy',
'wirtschaft':'wirtschaft',
'wirtschaftsgenossenschaften':'wirtschaftsgenossenschaft',
'wohnungsbaugenossenschaften':'wohnungsbaugenossenschaft',
'wohnungseigentum':'wohnungseigentum',
'wohnungsgenossenschaftsmitglieder':'wohnungsgenossenschaftsmitglied',
'wohnungsnutzergenossenschaften':'wohnungsnutzergenossenschaft',
'wohnungsunternehmen':'wohnungsunternehm',
'wolff':'wolff',
'wto':'wto',
'xviii':'xviii',
'xx':'xx',
'ydmyket':'ydmyk',
'ydt':'ydt',
'ygg':'ygg',
'yggdrasil':'yggdrasil',
'yggdrasils':'yggdrasil',
'yh':'yh',
'yin':'yin',
'ymer':'ymer',
'ymers':'ymer',
'ymse':'yms',
'yngre':'yngr',
'yngste':'yngst',
'york':'york',
'yr':'yr',
'yre':'yre',
'yrke':'yrk',
'yrkedagen':'yrkedag',
'yrkesaktiv':'yrkesaktiv',
'yrkesaktive':'yrkesaktiv',
'yrkesaktivitet':'yrkesaktivit',
'yrkesaktiviteten':'yrkesaktivitet',
'yrkesaktivt':'yrkesaktiv',
'yrkesbaserte':'yrkesbaser',
'yrkesdeltagelse':'yrkesdeltag',
'yrkesdeltakelse':'yrkesdeltak',
'yrkesdeltaking':'yrkesdeltaking',
'yrkesdøgn':'yrkesdøgn',
'yrkesdøgntrafikk':'yrkesdøgntrafikk',
'yrkesgrupper':'yrkesgrupp',
'yrkeskarrieren':'yrkeskarrier',
'yrkesmessige':'yrkesmess',
'yrkesorganisasjonenes':'yrkesorganisasjon',
'yrkesskadeforsikring':'yrkesskadeforsikring',
'yrkesutøvar':'yrkesutøv',
'yrkesutøvarar':'yrkesutøvar',
'yte':'yte',
'ytelse':'ytels',
'ytelsen':'ytels',
'ytelsene':'ytels',
'ytelsenes':'ytels',
'ytelsens':'ytels',
'ytelser':'ytels',
'ytelses':'ytels',
'ytelsesbasert':'ytelsesbaser',
'ytelsesbaserte':'ytelsesbaser',
'ytelsesberettigede':'ytelsesberett',
'ytelsesnivå':'ytelsesnivå',
'ytelsesnivåene':'ytelsesnivå',
'ytelsesnivået':'ytelsesnivå',
'ytelsesoppbygging':'ytelsesoppbygging',
'ytelsesordning':'ytelsesordning',
'ytelsesordninger':'ytelsesordning',
'ytelsesperioden':'ytelsesperiod',
'ytelsesperiodens':'ytelsesperiod',
'ytelsesplan':'ytelsesplan',
'ytelsesplanen':'ytelsesplan',
'ytelsessammensetningen':'ytelsessammensetning',
'yter':'yter',
'ytes':'ytes',
'yting':'yting',
'ytinga':'yting',
'ytingar':'yting',
'ytingsbaserte':'ytingsbaser',
'ytre':'ytr',
'ytste':'ytst',
'ytt':'ytt',
'ytte':'ytt',
'ytterdør':'ytterdør',
'ytterdører':'ytterdør',
'ytterkant':'ytterkant',
'ytterligere':'ytterliger',
'ytterliggere':'ytterligger',
'ytterst':'ytterst',
'ytterste':'ytterst',
'zapffe':'zapff',
'zealand':'zealand',
'zeebrugge':'zeebrugg',
'zu':'zu',
'å':'å',
'åe':'åe',
'åge':'åge',
'åker':'åker',
'åkeren':'åker',
'åkre':'åkr',
'ål':'ål',
'åle':'åle',
'åleine':'ålein',
'ålesund':'ålesund',
'åmsvartne':'åmsvartn',
'åndalsnes':'åndalsn',
'ånde':'ånd',
'åpen':'åpen',
'åpenbare':'åpenbar',
'åpenbart':'åpenbart',
'åpenlys':'åpenly',
'åpent':'åpent',
'åpne':'åpn',
'åpner':'åpn',
'åpnes':'åpn',
'åpnet':'åpn',
'åpning':'åpning',
'år':'år',
'åra':'åra',
'årene':'åren',
'årer':'årer',
'året':'året',
'åretak':'åretak',
'årets':'året',
'årevis':'årevis',
'århundre':'århundr',
'århundret':'århundr',
'årleg':'årleg',
'årlege':'årleg',
'årlig':'årl',
'årlige':'årl',
'årrekkje':'årrekkj',
'årringen':'årring',
'årringer':'årring',
'års':'års',
'årsak':'årsak',
'årsaka':'årsak',
'årsaken':'årsak',
'årsaker':'årsak',
'årsbalanse':'årsbalans',
'årsbasis':'årsbasis',
'årsberetning':'årsberetning',
'årsmelding':'årsmelding',
'årsmeldinga':'årsmelding',
'årsoppgjer':'årsoppgj',
'årsoppgjeret':'årsoppgjer',
'årsoppgjør':'årsoppgjør',
'årsoppgjøret':'årsoppgjør',
'årsoppgjørsforskriften':'årsoppgjørsforskrift',
'årsoppgjørskapittelet':'årsoppgjørskapittel',
'årspengane':'årspeng',
'årspengar':'årspeng',
'årsperiode':'årsperiod',
'årspremier':'årspremi',
'årsregnskapet':'årsregnskap',
'årsregnskapsreglene':'årsregnskapsregl',
'årsrekneskap':'årsrekneskap',
'årsrekneskapen':'årsrekneskap',
'årsresultatet':'årsresultat',
'årsskifte':'årsskift',
'årsskiftet':'årsskift',
'årum':'årum',
'årvold':'årvold',
'årvoldkrysset':'årvoldkryss',
'åsa':'åsa',
'åsene':'åsen',
'åser':'åser',
'åsgard':'åsgard',
'åsgards':'åsgard',
'åshild':'åshild',
'åsmund':'åsmund',
'åt':'åt',
'åtferd':'åtferd',
'åtgjerd':'åtgjerd',
'ått':'ått',
'åttande':'ått',
'åtte':'ått',
'åttefotede':'åttefot',
'åttefotete':'åttefotet',
'åttende':'ått',
'åtvaring':'åtvaring',
'ægir':'ægir',
'ægirs':'ægir',
'ærbødighet':'ærbød',
'ære':'ære',
'ærefullt':'ærefullt',
'æren':'æren',
'ærend':'ærend',
'ærendet':'ærend',
'ærlegdom':'ærlegdom',
'ærlig':'ærl',
'æsene':'æsen',
'æsenes':'æsen',
'æser':'æser',
'ætt':'ætt',
'øde':'øde',
'ødelagt':'ødelagt',
'ødelagte':'ødelagt',
'ødelegge':'ødelegg',
'ødemarken':'ødemark',
'ødemarker':'ødemark',
'øke':'øke',
'økende':'økend',
'øker':'øker',
'økes':'økes',
'økning':'økning',
'økningen':'økning',
'økningene':'økning',
'økninger':'økning',
'økonomi':'økonomi',
'økonomien':'økonomi',
'økonomiene':'økonomi',
'økonomiplan':'økonomiplan',
'økonomisk':'økonomisk',
'økonomiske':'økonomisk',
'øks':'øks',
'økser':'øks',
'økt':'økt',
'økte':'økt',
'øl':'øl',
'øldunkene':'øldunk',
'ølet':'ølet',
'ølkruset':'ølkrus',
'ønske':'ønsk',
'ønskede':'ønsk',
'ønskelig':'ønsk',
'ønskelige':'ønsk',
'ønskeligheten':'ønsk',
'ønsker':'ønsk',
'ønskes':'ønsk',
'ønsket':'ønsk',
'ønskje':'ønskj',
'ønskjeleg':'ønskj',
'ønskjer':'ønskj',
'ønskt':'ønskt',
'ønskte':'ønskt',
'ør':'ør',
'øra':'øra',
'øre':'øre',
'ørefik':'ørefik',
'øremerkede':'øremerk',
'øremerket':'øremerk',
'øremerking':'øremerking',
'ørene':'øren',
'ører':'ører',
'øresundforbindelsen':'øresundforbind',
'øresundsforbindelsen':'øresundsforbind',
'øret':'øret',
'ørje':'ørj',
'ørn':'ørn',
'ørneham':'ørneham',
'ørnehammen':'ørnehamm',
'ørnen':'ørn',
'ørner':'ørn',
'ørske':'ørsk',
'øse':'øse',
'øsekar':'øsek',
'øser':'øser',
'øst':'øst',
'østfold':'østfold',
'østfoldhavnene':'østfoldhavn',
'østfoldhavner':'østfoldhavn',
'østfra':'østfr',
'østhavn':'østhavn',
'østhavna':'østhavn',
'østland':'østland',
'østlandet':'østland',
'østlandsfylkene':'østlandsfylk',
'østlandshavnene':'østlandshavn',
'østlandsområder':'østlandsområd',
'østlandsområdet':'østlandsområd',
'østlandsregionen':'østlandsregion',
'østlandsregionens':'østlandsregion',
'østlandssamarbeidet':'østlandssamarbeid',
'østlig':'øst',
'østlige':'øst',
'østmoe':'østmo',
'østover':'østov',
'østre':'østr',
'østside':'østsid',
'østsiden':'østsid',
'østvendte':'østvend',
'øve':'øve',
'øver':'øver',
'øverste':'øverst',
'øvre':'øvr',
'øvrig':'øvr',
'øvrige':'øvr',
'øvste':'øvst',
'øy':'øy',
'øya':'øya',
'øydelagde':'øydelagd',
'øydelagt':'øydelagt',
'øydelegging':'øydelegging',
'øydelegginga':'øydelegging',
'øye':'øye',
'øyeblikk':'øyeblikk',
'øyeblikkelig':'øyeblikk',
'øyeblikket':'øyeblikk',
'øyelokkene':'øyelokk',
'øyelokket':'øyelokk',
'øyenvipper':'øyenvipp',
'øyet':'øyet',
'øyne':'øyn',
'øynene':'øyn',
'øystein':'øystein',
}
