# coding: utf8
from __future__ import unicode_literals

LOOKUP = {
    " furtherst": "further",
    " skilled": "skill",
    "'cause": "because",
    "'d": "would",
    "'em": "them",
    "'ll": "will",
    "'m": "be",
    "'n'": "and",
    "'s": "have",
    "'til": "until",
    "'ve": "have",
    "A-bombs": "A-bomb",
    "AGMs": "AGM",
    "Aborigines": "Aborigine",
    "Admiralties": "Admiralty",
    "Afghans": "Afghan",
    "Africans": "African",
    "Afrikaners": "Afrikaner",
    "Afros": "Afro",
    "Albanians": "Albanian",
    "Algerians": "Algerian",
    "Alsatians": "Alsatian",
    "Americanisms": "Americanism",
    "Americanized": "Americanize",
    "Americanizes": "Americanize",
    "Americanizing": "Americanize",
    "Americans": "American",
    "Anglo-Saxons": "Anglo-Saxon",
    "Angolans": "Angolan",
    "Aprils": "April",
    "Aquariuses": "Aquarius",
    "Arabs": "Arab",
    "Argentines": "Argentine",
    "Argentinians": "Argentinian",
    "Armenians": "Armenian",
    "Asians": "Asian",
    "Asiatics": "Asiatic",
    "Augusts": "August",
    "Australians": "Australian",
    "Austrians": "Austrian",
    "B's": "B",
    "BAs": "BA",
    "BCGs": "BCG",
    "BScs": "BSc",
    "Bahamians": "Bahamian",
    "Bahrainis": "Bahraini",
    "Band-Aids": "Band-Aid",
    "Bangladeshis": "Bangladeshi",
    "Baptists": "Baptist",
    "Bedouins": "Bedouin",
    "Beefeaters": "Beefeater",
    "Belgians": "Belgian",
    "Benedictines": "Benedictine",
    "Bengalis": "Bengali",
    "Biros": "Biro",
    "Boers": "Boer",
    "Bolivians": "Bolivian",
    "Bolsheviks": "Bolshevik",
    "Brahmins": "Brahmin",
    "Brazilians": "Brazilian",
    "Breathalyzers": "Breathalyzer",
    "Britishers": "Britisher",
    "Britons": "Briton",
    "Brits": "Brit",
    "Buddhas": "Buddha",
    "Buddhists": "Buddhist",
    "Bulgarians": "Bulgarian",
    "Burgomasters": "Burgomaster",
    "Burgundies": "Burgundy",
    "Burmeses": "Burmese",
    "Bushmen": "Bushman",
    "CBIs": "CBI",
    "CCTVs": "CCTV",
    "CDs": "CD",
    "CIDs": "CID",
    "CV's": "CV",
    "CVs": "CV",
    "Caesareans": "Caesarean",
    "Cambodians": "Cambodian",
    "Canadians": "Canadian",
    "Catholics": "Catholic",
    "Celts": "Celt",
    "Chileans": "Chilean",
    "Chineses": "Chinese",
    "Christians": "Christian",
    "Christmases": "Christmas",
    "Colombians": "Colombian",
    "Conservatrices": "Conservatrice",
    "Cubans": "Cuban",
    "Cypriots": "Cypriot",
    "Czechoslovakians": "Czechoslovakian",
    "Czechs": "Czech",
    "D-days": "D-day",
    "DJs": "DJ",
    "Dalmatians": "Dalmatian",
    "Danes": "Dane",
    "Decembers": "December",
    "Dictaphones": "Dictaphone",
    "Drs": "Dr",
    "Dutchmen": "Dutchman",
    "Dutchwomen": "Dutchwoman",
    "EECs": "EEC",
    "EPs": "EP",
    "Easters": "Easter",
    "Ecuadorians": "Ecuadorian",
    "Egyptians": "Egyptian",
    "Englishmen": "Englishman",
    "Englishwomen": "Englishwoman",
    "Eskimos": "Eskimo",
    "Ethiopians": "Ethiopian",
    "Eucharists": "Eucharist",
    "Eurasians": "Eurasian",
    "Europeans": "European",
    "Excellencies": "Excellency",
    "Februaries": "February",
    "Fedexed": "Fedex",
    "Fedexes": "Fedex",
    "Fedexing": "Fedex",
    "Filipinos": "Filipino",
    "Finns": "Finn",
    "Freemasons": "Freemason",
    "Frenchmen": "Frenchman",
    "Frenchwomen": "Frenchwoman",
    "Fridays": "Friday",
    "Frisbees": "Frisbee",
    "G-strings": "G-string",
    "GCEs": "GCE",
    "GIs": "GI",
    "GPs": "GP",
    "Germans": "German",
    "Ghanaians": "Ghanaian",
    "Governor-Generals": "Governor-General",
    "Governors-General": "Governor-General",
    "Govts": "Govt",
    "Greeks": "Greek",
    "Guatemalans": "Guatemalan",
    "H-bombs": "H-bomb",
    "HQs": "HQ",
    "Haitians": "Haitian",
    "Hebrews": "Hebrew",
    "Highnesses": "Highness",
    "Hindus": "Hindu",
    "Hondurans": "Honduran",
    "Hungarians": "Hungarian",
    "IOUs": "IOU",
    "IQs": "IQ",
    "IUDs": "IUD",
    "Icelanders": "Icelander",
    "Indians": "Indian",
    "Indonesians": "Indonesian",
    "Iranians": "Iranian",
    "Iraqis": "Iraqi",
    "Irishmen": "Irishman",
    "Irishwomen": "Irishwoman",
    "Israelis": "Israeli",
    "Israelites": "Israelite",
    "Italians": "Italian",
    "JPs": "JP",
    "Jacuzzis": "Jacuzzi",
    "Jamaicans": "Jamaican",
    "Januaries": "January",
    "Jesuits": "Jesuit",
    "Jewesses": "Jewess",
    "Jews": "Jew",
    "Jordanians": "Jordanian",
    "Judases": "Judas",
    "Jugoslavs": "Jugoslav",
    "Julies": "July",
    "Junes": "June",
    "KO'd": "KO",
    "KO's": "KO",
    "Kenyans": "Kenyan",
    "Koreans": "Korean",
    "Kuwaitis": "Kuwaiti",
    "L-plates": "L-plate",
    "LPs": "LP",
    "LV's": "LV",
    "Land-Rovers": "Land-Rover",
    "Laotians": "Laotian",
    "Lapps": "Lapp",
    "Latins": "Latin",
    "Laundromats": "Laundromat",
    "Leos": "Leo",
    "Liberians": "Liberian",
    "Libras": "Libra",
    "Libyans": "Libyan",
    "Lilos": "Lilo",
    "Lithuanians": "Lithuanian",
    "Londoners": "Londoner",
    "Lordships": "Lordship",
    "Luddites": "Luddite",
    "MBEs": "MBE",
    "MCPs": "MCP",
    "MCs": "MC",
    "MDs": "MD",
    "MEPs": "MEP",
    "MO's": "MO",
    "MOTs": "MOT",
    "MOs": "MO",
    "MPs": "MP",
    "MScs": "MSc",
    "Madonnas": "Madonna",
    "Malayans": "Malayan",
    "Malays": "Malay",
    "Malaysians": "Malaysian",
    "Martians": "Martian",
    "Marxists": "Marxist",
    "Mauritians": "Mauritian",
    "Messrs": "Mr",
    "Methodists": "Methodist",
    "Mexicans": "Mexican",
    "Mimeographed": "Mimeograph",
    "Mimeographing": "Mimeograph",
    "Mimeographs": "Mimeograph",
    "Mohammedans": "Mohammedan",
    "Mondays": "Monday",
    "Mongolians": "Mongolian",
    "Mongols": "Mongol",
    "Monsignors": "Monsignor",
    "Mormons": "Mormon",
    "Moroccans": "Moroccan",
    "Moslems": "Moslem",
    "Mse": "Ms",
    "Mts": "Mt",
    "Muhammadans": "Muhammadan",
    "Muslims": "Muslim",
    "NCO's": "NCO",
    "Negresses": "Negress",
    "Negroes": "Negro",
    "Nepalis": "Nepali",
    "Nicaraguans": "Nicaraguan",
    "Nigerians": "Nigerian",
    "Normans": "Norman",
    "Norwegians": "Norwegian",
    "Nos": "No",
    "Novembers": "November",
    "OAP's": "OAP",
    "Octobers": "October",
    "Olympics": "Olympic",
    "PA's": "PA",
    "PAs": "PA",
    "PC's": "PC",
    "PCs": "PC",
    "PSs": "PS",
    "PTA's": "PTA",
    "PTAs": "PTA",
    "Pakistanis": "Pakistani",
    "Palestinians": "Palestinian",
    "Panamanians": "Panamanian",
    "Paraguayans": "Paraguayan",
    "Parisians": "Parisian",
    "Passovers": "Passover",
    "Pentecosts": "Pentecost",
    "Persians": "Persian",
    "Peruvians": "Peruvian",
    "PhDs": "PhD",
    "Pharisees": "Pharisee",
    "Polaroids": "Polaroid",
    "Politburos": "Politburo",
    "Polynesians": "Polynesian",
    "Popes": "Pope",
    "Popsicles": "Popsicle",
    "Pre-Raphaelites": "Pre-Raphaelite",
    "Primuses": "Primus",
    "Protestants": "Protestant",
    "Provos": "Provo",
    "Pullmans": "Pullman",
    "Punjabis": "Punjabi",
    "QCs": "QC",
    "Quakers": "Quaker",
    "RC's": "RC",
    "RCs": "RC",
    "Rastafarians": "Rastafarian",
    "Rastas": "Rasta",
    "Republicans": "Republican",
    "Rhodesians": "Rhodesian",
    "Romanians": "Romanian",
    "Romanies": "Romany",
    "Romans": "Roman",
    "Rumanians": "Rumanian",
    "Russians": "Russian",
    "SRNs": "SRN",
    "STs": "ST",
    "Sagittariuses": "Sagittarius",
    "Samaritans": "Samaritan",
    "Samoans": "Samoan",
    "Saturdays": "Saturday",
    "Saudis": "Saudi",
    "Scandinavians": "Scandinavian",
    "Scorpios": "Scorpio",
    "Scots": "Scot",
    "Scotsmen": "Scotsman",
    "Scotswomen": "Scotswoman",
    "Scotticisms": "Scotticism",
    "Sellotaped": "Sellotape",
    "Sellotapes": "Sellotape",
    "Sellotaping": "Sellotape",
    "Septembers": "September",
    "Shahs": "Shah",
    "Siberians": "Siberian",
    "Sicilians": "Sicilian",
    "Sikhs": "Sikh",
    "Singaporeans": "Singaporean",
    "Slavs": "Slav",
    "Somalis": "Somali",
    "Soviets": "Soviet",
    "Spaniards": "Spaniard",
    "Sundays": "Sunday",
    "Syrians": "Syrian",
    "T-shirts": "T-shirt",
    "T-squares": "T-square",
    "TVs": "TV",
    "Tahitians": "Tahitian",
    "Tanzanians": "Tanzanian",
    "Taurusses": "Taurus",
    "Teasmaids": "Teasmaid",
    "Teds": "Ted",
    "Teletypes": "Teletype",
    "Thais": "Thai",
    "Thanksgivings": "Thanksgiving",
    "Thermoses": "Thermos",
    "Thurs": "Thur",
    "Thursdays": "Thursday",
    "Tibetans": "Tibetan",
    "Tories": "Tory",
    "Trojans": "Trojan",
    "Trotskyists": "Trotskyist",
    "Trotskyites": "Trotskyite",
    "Tues": "Tue",
    "Tuesdays": "Tuesday",
    "Tunisians": "Tunisian",
    "Turks": "Turk",
    "U's": "U",
    "U-bends": "U-bend",
    "U-turns": "U-turn",
    "UFOs": "UFO",
    "Ugandans": "Ugandan",
    "Ukrainians": "Ukrainian",
    "Uruguayans": "Uruguayan",
    "V-necks": "V-neck",
    "V-signs": "V-sign",
    "VCRs": "VCR",
    "VCs": "VC",
    "VDUs": "VDU",
    "VIPs": "VIP",
    "Venetians": "Venetian",
    "Venezuelans": "Venezuelan",
    "Victorians": "Victorian",
    "Vikings": "Viking",
    "Virgos": "Virgo",
    "WPCs": "WPC",
    "Walkmans": "Walkman",
    "Wednesdays": "Wednesday",
    "Welshes": "Welsh",
    "Welshmen": "Welshman",
    "Welshwomen": "Welshwoman",
    "Whigs": "Whig",
    "Whitsuns": "Whitsun",
    "X-rayed": "X-ray",
    "X-raying": "X-ray",
    "X-rays": "X-ray",
    "Xeroxed": "Xerox",
    "Xeroxes": "Xerox",
    "Xeroxing": "Xerox",
    "Xmasses": "Xmas",
    "YMCAs": "YMCA",
    "YWCAs": "YWCA",
    "Yankees": "Yankee",
    "Yemenis": "Yemeni",
    "Yugoslavs": "Yugoslav",
    "Yules": "Yule",
    "Zaireans": "Zairean",
    "Zambians": "Zambian",
    "Zimbabweans": "Zimbabwean",
    "Zionists": "Zionist",
    "Zulus": "Zulu",
    "aardvarks": "aardvark",
    "abaci": "abacus",
    "abacuses": "abacus",
    "abalones": "abalone",
    "abandoned": "abandon",
    "abandoning": "abandon",
    "abandons": "abandon",
    "abased": "abase",
    "abases": "abase",
    "abashed": "abash",
    "abashes": "abash",
    "abashing": "abash",
    "abasing": "abase",
    "abated": "abate",
    "abatements": "abatement",
    "abates": "abate",
    "abating": "abate",
    "abattoirs": "abattoir",
    "abbesses": "abbess",
    "abbeys": "abbey",
    "abbots": "abbot",
    "abbreviated": "abbreviate",
    "abbreviates": "abbreviate",
    "abbreviating": "abbreviate",
    "abbreviations": "abbreviation",
    "abcesses": "abcess",
    "abdicated": "abdicate",
    "abdicates": "abdicate",
    "abdicating": "abdicate",
    "abdications": "abdication",
    "abdomens": "abdomen",
    "abdomina": "abdomen",
    "abdominals": "abdominal",
    "abducted": "abduct",
    "abducting": "abduct",
    "abductions": "abduction",
    "abductors": "abductor",
    "abducts": "abduct",
    "aberrations": "aberration",
    "abets": "abet",
    "abetted": "abet",
    "abetting": "abet",
    "abhorred": "abhor",
    "abhorring": "abhor",
    "abhors": "abhor",
    "abided": "abide",
    "abides": "abide",
    "abiding": "abide",
    "abilities": "ability",
    "abjected": "abject",
    "abjecters": "abjecter",
    "abjecting": "abject",
    "abjects": "abject",
    "abjured": "abjure",
    "abjures": "abjure",
    "abjuring": "abjure",
    "ablated": "ablate",
    "ablates": "ablate",
    "ablating": "ablate",
    "ablations": "ablation",
    "abler": "able",
    "ablest": "able",
    "ablutions": "ablution",
    "abnormalities": "abnormality",
    "abode": "abide",
    "abodes": "abode",
    "abolished": "abolish",
    "abolishes": "abolish",
    "abolishing": "abolish",
    "abolitionists": "abolitionist",
    "abolitions": "abolition",
    "abominated": "abominate",
    "abominates": "abominate",
    "abominating": "abominate",
    "abominations": "abomination",
    "aboriginals": "aboriginal",
    "aborted": "abort",
    "abortifacients": "abortifacient",
    "aborting": "abort",
    "abortionists": "abortionist",
    "abortions": "abortion",
    "aborts": "abort",
    "abounded": "abound",
    "abounding": "abound",
    "abounds": "abound",
    "about-faces": "about-face",
    "about-turns": "about-turn",
    "abraded": "abrade",
    "abrades": "abrade",
    "abrading": "abrade",
    "abrasions": "abrasion",
    "abrasives": "abrasive",
    "abreactions": "abreaction",
    "abridged": "abridge",
    "abridgements": "abridgement",
    "abridges": "abridge",
    "abridging": "abridge",
    "abridgments": "abridgment",
    "abrogated": "abrogate",
    "abrogates": "abrogate",
    "abrogating": "abrogate",
    "abrogations": "abrogation",
    "abrupter": "abrupt",
    "abruptest": "abrupt",
    "abruptions": "abruption",
    "abs": "ab",
    "abscesses": "abscess",
    "abscissae": "abscissa",
    "abscissas": "abscissa",
    "absconded": "abscond",
    "absconding": "abscond",
    "absconds": "abscond",
    "abseiled": "abseil",
    "abseiling": "abseil",
    "abseils": "abseil",
    "absences": "absence",
    "absented": "absent",
    "absenteeisms": "absenteeism",
    "absentees": "absentee",
    "absenting": "absent",
    "absents": "absent",
    "absinthes": "absinthe",
    "absolutes": "absolute",
    "absolutists": "absolutist",
    "absolved": "absolve",
    "absolves": "absolve",
    "absolving": "absolve",
    "absorbances": "absorbance",
    "absorbed": "absorb",
    "absorbencies": "absorbency",
    "absorbents": "absorbent",
    "absorbers": "absorber",
    "absorbing": "absorb",
    "absorbs": "absorb",
    "absorptiometries": "absorptiometry",
    "absorptions": "absorption",
    "abstained": "abstain",
    "abstainers": "abstainer",
    "abstaining": "abstain",
    "abstains": "abstain",
    "abstentions": "abstention",
    "abstinences": "abstinence",
    "abstracted": "abstract",
    "abstracting": "abstract",
    "abstractions": "abstraction",
    "abstracts": "abstract",
    "absurdities": "absurdity",
    "abundances": "abundance",
    "abused": "abuse",
    "abusers": "abuser",
    "abuses": "abuse",
    "abusing": "abuse",
    "abutments": "abutment",
    "abuts": "abut",
    "abutted": "abut",
    "abutting": "abut",
    "abysses": "abyss",
    "acacias": "acacia",
    "academicians": "academician",
    "academics": "academic",
    "academies": "academy",
    "acanthi": "acanthus",
    "acanthuses": "acanthus",
    "acaricides": "acaricide",
    "acceded": "accede",
    "accedes": "accede",
    "acceding": "accede",
    "accelerants": "accelerant",
    "accelerated": "accelerate",
    "accelerates": "accelerate",
    "accelerating": "accelerate",
    "accelerations": "acceleration",
    "accelerators": "accelerator",
    "accelerometers": "accelerometer",
    "accented": "accent",
    "accenting": "accent",
    "accents": "accent",
    "accentuated": "accentuate",
    "accentuates": "accentuate",
    "accentuating": "accentuate",
    "accentuations": "accentuation",
    "acceptabilities": "acceptability",
    "acceptances": "acceptance",
    "acceptations": "acceptation",
    "accepted": "accept",
    "accepting": "accept",
    "acceptors": "acceptor",
    "accepts": "accept",
    "accessed": "access",
    "accesses": "access",
    "accessibilities": "accessibility",
    "accessing": "access",
    "accessioned": "accession",
    "accessioning": "accession",
    "accessions": "accession",
    "accessories": "accessory",
    "accidentals": "accidental",
    "accidents": "accident",
    "acclaimed": "acclaim",
    "acclaiming": "acclaim",
    "acclaims": "acclaim",
    "acclimated": "acclimate",
    "acclimates": "acclimate",
    "acclimating": "acclimate",
    "acclimations": "acclimation",
    "acclimatisations": "acclimatisation",
    "acclimatised": "acclimatise",
    "acclimatises": "acclimatise",
    "acclimatising": "acclimatise",
    "acclimatizations": "acclimatization",
    "acclimatized": "acclimatize",
    "acclimatizes": "acclimatize",
    "acclimatizing": "acclimatize",
    "acclivities": "acclivity",
    "accolades": "accolade",
    "accommodated": "accommodate",
    "accommodates": "accommodate",
    "accommodating": "accommodate",
    "accommodations": "accommodation",
    "accompanied": "accompany",
    "accompanies": "accompany",
    "accompaniments": "accompaniment",
    "accompanists": "accompanist",
    "accompanying": "accompany",
    "accomplices": "accomplice",
    "accomplished": "accomplish",
    "accomplishes": "accomplish",
    "accomplishing": "accomplish",
    "accomplishments": "accomplishment",
    "accorded": "accord",
    "according": "accord",
    "accordions": "accordion",
    "accords": "accord",
    "accosted": "accost",
    "accosting": "accost",
    "accosts": "accost",
    "accountabilities": "accountability",
    "accountants": "accountant",
    "accounted": "account",
    "accounting": "account",
    "accountings": "accounting",
    "accounts": "account",
    "accoutrements": "accoutrement",
    "accreditations": "accreditation",
    "accredited": "accredit",
    "accrediting": "accredit",
    "accredits": "accredit",
    "accreted": "accrete",
    "accretes": "accrete",
    "accreting": "accrete",
    "accretions": "accretion",
    "accruals": "accrual",
    "accrued": "accrue",
    "accrues": "accrue",
    "accruing": "accrue",
    "accs": "acc",
    "accumulated": "accumulate",
    "accumulates": "accumulate",
    "accumulating": "accumulate",
    "accumulations": "accumulation",
    "accumulators": "accumulator",
    "accuracies": "accuracy",
    "accusations": "accusation",
    "accusatives": "accusative",
    "accused": "accuse",
    "accusers": "accuser",
    "accuses": "accuse",
    "accusing": "accuse",
    "accustomed": "accustom",
    "accustoming": "accustom",
    "accustoms": "accustom",
    "aces": "ace",
    "acetabula": "acetabulum",
    "acetabulums": "acetabulum",
    "acetals": "acetal",
    "acetates": "acetate",
    "acetazolamides": "acetazolamide",
    "acetones": "acetone",
    "acetylations": "acetylation",
    "acetylcholines": "acetylcholine",
    "acetylcholinesterases": "acetylcholinesterase",
    "acetylenes": "acetylene",
    "acetyls": "acetyl",
    "ached": "ache",
    "aches": "ache",
    "achieved": "achieve",
    "achievements": "achievement",
    "achievers": "achiever",
    "achieves": "achieve",
    "achieving": "achieve",
    "aching": "ache",
    "achromats": "achromat",
    "acidifications": "acidification",
    "acidified": "acidify",
    "acidifies": "acidify",
    "acidifying": "acidify",
    "acidities": "acidity",
    "acidoses": "acidosis",
    "acids": "acid",
    "acknowledged": "acknowledge",
    "acknowledgements": "acknowledgement",
    "acknowledges": "acknowledge",
    "acknowledging": "acknowledge",
    "acknowledgments": "acknowledgment",
    "acks": "ack",
    "acnes": "acne",
    "acolytes": "acolyte",
    "aconites": "aconite",
    "acorns": "acorn",
    "acousticians": "acoustician",
    "acoustics": "acoustic",
    "acquaintances": "acquaintance",
    "acquaintanceships": "acquaintanceship",
    "acquainted": "acquaint",
    "acquainting": "acquaint",
    "acquaints": "acquaint",
    "acquiesced": "acquiesce",
    "acquiescences": "acquiescence",
    "acquiesces": "acquiesce",
    "acquiescing": "acquiesce",
    "acquired": "acquire",
    "acquirements": "acquirement",
    "acquires": "acquire",
    "acquiring": "acquire",
    "acquisitions": "acquisition",
    "acquits": "acquit",
    "acquittals": "acquittal",
    "acquitted": "acquit",
    "acquitting": "acquit",
    "acreages": "acreage",
    "acres": "acre",
    "acrobatics": "acrobatic",
    "acrobats": "acrobat",
    "acromegalies": "acromegaly",
    "acronyms": "acronym",
    "acrostics": "acrostic",
    "acrylamides": "acrylamide",
    "acrylics": "acrylic",
    "acted": "act",
    "acting": "act",
    "actings": "acting",
    "actins": "actin",
    "actioned": "action",
    "actioning": "action",
    "actions": "action",
    "activated": "activate",
    "activates": "activate",
    "activating": "activate",
    "activations": "activation",
    "activators": "activator",
    "actives": "active",
    "activists": "activist",
    "activities": "activity",
    "actors": "actor",
    "actresses": "actress",
    "acts": "act",
    "actualisations": "actualisation",
    "actualised": "actualise",
    "actualises": "actualise",
    "actualising": "actualise",
    "actualities": "actuality",
    "actualizations": "actualization",
    "actualized": "actualize",
    "actualizes": "actualize",
    "actualizing": "actualize",
    "actuaries": "actuary",
    "actuated": "actuate",
    "actuates": "actuate",
    "actuating": "actuate",
    "actuations": "actuation",
    "actuators": "actuator",
    "acuities": "acuity",
    "acumens": "acumen",
    "acupoints": "acupoint",
    "acupunctures": "acupuncture",
    "acupuncturists": "acupuncturist",
    "acutenesses": "acuteness",
    "acuter": "acute",
    "acutest": "acute",
    "acyls": "acyl",
    "ad-libbed": "ad-lib",
    "ad-libbing": "ad-lib",
    "ad-libs": "ad-lib",
    "adages": "adage",
    "adaptabilities": "adaptability",
    "adaptations": "adaptation",
    "adapted": "adapt",
    "adapters": "adapter",
    "adapting": "adapt",
    "adaptions": "adaption",
    "adaptogens": "adaptogen",
    "adaptors": "adaptor",
    "adapts": "adapt",
    "add-ons": "add-on",
    "added": "add",
    "addenda": "addendum",
    "addendums": "addendum",
    "adders": "adder",
    "addicted": "addict",
    "addicting": "addict",
    "addictions": "addiction",
    "addicts": "addict",
    "adding": "add",
    "adding-machines": "adding-machine",
    "additions": "addition",
    "additives": "additive",
    "addled": "addle",
    "addles": "addle",
    "addling": "addle",
    "addressed": "address",
    "addressees": "addressee",
    "addresses": "address",
    "addressing": "address",
    "adds": "add",
    "adduced": "adduce",
    "adduces": "adduce",
    "adducing": "adduce",
    "adducted": "adduct",
    "adducting": "adduct",
    "adductors": "adductor",
    "adducts": "adduct",
    "adenines": "adenine",
    "adenocarcinomas": "adenocarcinoma",
    "adenocarcinomata": "adenocarcinoma",
    "adenoids": "adenoid",
    "adenomas": "adenoma",
    "adenomata": "adenoma",
    "adenosines": "adenosine",
    "adenoviruses": "adenovirus",
    "adepts": "adept",
    "adequacies": "adequacy",
    "ades": "ade",
    "adhered": "adhere",
    "adherences": "adherence",
    "adherents": "adherent",
    "adheres": "adhere",
    "adhering": "adhere",
    "adhesins": "adhesin",
    "adhesions": "adhesion",
    "adhesives": "adhesive",
    "adieus": "adieu",
    "adieux": "adieu",
    "adipocytes": "adipocyte",
    "adits": "adit",
    "adjacencies": "adjacency",
    "adjectives": "adjective",
    "adjoined": "adjoin",
    "adjoining": "adjoin",
    "adjoins": "adjoin",
    "adjoints": "adjoint",
    "adjourned": "adjourn",
    "adjourning": "adjourn",
    "adjournments": "adjournment",
    "adjourns": "adjourn",
    "adjudged": "adjudge",
    "adjudges": "adjudge",
    "adjudging": "adjudge",
    "adjudicated": "adjudicate",
    "adjudicates": "adjudicate",
    "adjudicating": "adjudicate",
    "adjudications": "adjudication",
    "adjudicators": "adjudicator",
    "adjuncts": "adjunct",
    "adjured": "adjure",
    "adjures": "adjure",
    "adjuring": "adjure",
    "adjusted": "adjust",
    "adjusters": "adjuster",
    "adjusting": "adjust",
    "adjustments": "adjustment",
    "adjusts": "adjust",
    "adjutants": "adjutant",
    "adjuvants": "adjuvant",
    "administered": "administer",
    "administering": "administer",
    "administers": "administer",
    "administrated": "administrate",
    "administrates": "administrate",
    "administrating": "administrate",
    "administrations": "administration",
    "administrators": "administrator",
    "admirals": "admiral",
    "admired": "admire",
    "admirers": "admirer",
    "admires": "admire",
    "admiring": "admire",
    "admissions": "admission",
    "admits": "admit",
    "admittances": "admittance",
    "admitted": "admit",
    "admitting": "admit",
    "admixed": "admix",
    "admixes": "admix",
    "admixing": "admix",
    "admixtures": "admixture",
    "admonished": "admonish",
    "admonishes": "admonish",
    "admonishing": "admonish",
    "admonitions": "admonition",
    "adms": "adm",
    "adolescents": "adolescent",
    "adopted": "adopt",
    "adoptees": "adoptee",
    "adopters": "adopter",
    "adopting": "adopt",
    "adoptions": "adoption",
    "adopts": "adopt",
    "adored": "adore",
    "adorers": "adorer",
    "adores": "adore",
    "adoring": "adore",
    "adorned": "adorn",
    "adorning": "adorn",
    "adornments": "adornment",
    "adorns": "adorn",
    "adrenals": "adrenal",
    "adroiter": "adroit",
    "adroitest": "adroit",
    "ads": "ad",
    "adsorbates": "adsorbate",
    "adsorbed": "adsorb",
    "adsorbents": "adsorbent",
    "adsorbers": "adsorber",
    "adsorbing": "adsorb",
    "adsorbs": "adsorb",
    "adsorptions": "adsorption",
    "adulterants": "adulterant",
    "adulterated": "adulterate",
    "adulterates": "adulterate",
    "adulterating": "adulterate",
    "adulterations": "adulteration",
    "adulterers": "adulterer",
    "adulteresses": "adulteress",
    "adulteries": "adultery",
    "adults": "adult",
    "adumbrated": "adumbrate",
    "adumbrates": "adumbrate",
    "adumbrating": "adumbrate",
    "advanced": "advance",
    "advancements": "advancement",
    "advances": "advance",
    "advancing": "advance",
    "advantaged": "advantage",
    "advantages": "advantage",
    "advantaging": "advantage",
    "advents": "advent",
    "adventured": "adventure",
    "adventurers": "adventurer",
    "adventures": "adventure",
    "adventuring": "adventure",
    "adverbials": "adverbial",
    "adverbs": "adverb",
    "adversaries": "adversary",
    "adversities": "adversity",
    "adverted": "advert",
    "adverting": "advert",
    "advertised": "advertise",
    "advertisements": "advertisement",
    "advertisers": "advertiser",
    "advertises": "advertise",
    "advertising": "advertise",
    "advertized": "advertize",
    "advertizes": "advertize",
    "advertizing": "advertize",
    "advertorials": "advertorial",
    "adverts": "advert",
    "advices": "advice",
    "advisabilities": "advisability",
    "advised": "advise",
    "advisers": "adviser",
    "advises": "advise",
    "advising": "advise",
    "advisories": "advisory",
    "advisors": "advisor",
    "advocacies": "advocacy",
    "advocated": "advocate",
    "advocates": "advocate",
    "advocating": "advocate",
    "adzes": "adze",
    "aegises": "aegis",
    "aeons": "aeon",
    "aerated": "aerate",
    "aerates": "aerate",
    "aerating": "aerate",
    "aerations": "aeration",
    "aerators": "aerator",
    "aerials": "aerial",
    "aerodromes": "aerodrome",
    "aerodynamics": "aerodynamic",
    "aerogels": "aerogel",
    "aeronauts": "aeronaut",
    "aeroplanes": "aeroplane",
    "aerosols": "aerosol",
    "aerospaces": "aerospace",
    "aesthetes": "aesthete",
    "aesthetics": "aesthetic",
    "aethers": "aether",
    "aetiologies": "aetiology",
    "affairs": "affair",
    "affectations": "affectation",
    "affected": "affect",
    "affecting": "affect",
    "affections": "affection",
    "affectives": "affective",
    "affects": "affect",
    "affidavits": "affidavit",
    "affiliated": "affiliate",
    "affiliates": "affiliate",
    "affiliating": "affiliate",
    "affiliations": "affiliation",
    "affinities": "affinity",
    "affirmations": "affirmation",
    "affirmatives": "affirmative",
    "affirmed": "affirm",
    "affirming": "affirm",
    "affirms": "affirm",
    "affixed": "affix",
    "affixes": "affix",
    "affixing": "affix",
    "afflicted": "afflict",
    "afflicting": "afflict",
    "afflictions": "affliction",
    "afflicts": "afflict",
    "affluents": "affluent",
    "affordances": "affordance",
    "afforded": "afford",
    "affording": "afford",
    "affords": "afford",
    "afforested": "afforest",
    "afforesting": "afforest",
    "afforests": "afforest",
    "affrays": "affray",
    "affronted": "affront",
    "affronting": "affront",
    "affronts": "affront",
    "aflatoxins": "aflatoxin",
    "afs": "af",
    "after-effects": "after-effect",
    "after-shaves": "after-shave",
    "afterbirths": "afterbirth",
    "afterburners": "afterburner",
    "afterglows": "afterglow",
    "afterlives": "afterlife",
    "aftermaths": "aftermath",
    "afternoons": "afternoon",
    "afters": "after",
    "aftershaves": "aftershave",
    "aftertastes": "aftertaste",
    "afterthoughts": "afterthought",
    "agars": "agar",
    "agates": "agate",
    "agaves": "agave",
    "age-groups": "age-group",
    "aged": "age",
    "ageing": "age",
    "agencies": "agency",
    "agenda": "agendum",
    "agendas": "agenda",
    "agendums": "agendum",
    "agents": "agent",
    "ages": "age",
    "agglomerated": "agglomerate",
    "agglomerates": "agglomerate",
    "agglomerating": "agglomerate",
    "agglomerations": "agglomeration",
    "agglutinated": "agglutinate",
    "agglutinates": "agglutinate",
    "agglutinating": "agglutinate",
    "agglutinations": "agglutination",
    "aggrandized": "aggrandize",
    "aggrandizes": "aggrandize",
    "aggrandizing": "aggrandize",
    "aggravated": "aggravate",
    "aggravates": "aggravate",
    "aggravating": "aggravate",
    "aggravations": "aggravation",
    "aggregated": "aggregate",
    "aggregated.": "aggregate",
    "aggregates": "aggregate",
    "aggregating": "aggregate",
    "aggregations": "aggregation",
    "aggregators": "aggregator",
    "aggressions": "aggression",
    "aggressivenesses": "aggressiveness",
    "aggressors": "aggressor",
    "aggrieved": "aggrieve",
    "aggrieves": "aggrieve",
    "aggrieving": "aggrieve",
    "aging": "age",
    "agitated": "agitate",
    "agitates": "agitate",
    "agitating": "agitate",
    "agitations": "agitation",
    "agitators": "agitator",
    "agnostics": "agnostic",
    "agonies": "agony",
    "agonised": "agonise",
    "agonises": "agonise",
    "agonising": "agonise",
    "agonists": "agonist",
    "agonized": "agonize",
    "agonizes": "agonize",
    "agonizing": "agonize",
    "agorae": "agora",
    "agoraphobias": "agoraphobia",
    "agoraphobics": "agoraphobic",
    "agoutis": "agouti",
    "agranulocytoses": "agranulocytosis",
    "agreed": "agree",
    "agreeing": "agree",
    "agreements": "agreement",
    "agrees": "agree",
    "agribusinesses": "agribusiness",
    "agriculturalists": "agriculturalist",
    "agriculturists": "agriculturist",
    "agrochemicals": "agrochemical",
    "agroecosystems": "agroecosystem",
    "agronomists": "agronomist",
    "ags": "ag",
    "aguas": "agua",
    "agues": "ague",
    "ahs": "ah",
    "ai": "have",
    "aided": "aid",
    "aides": "aide",
    "aides-de-camp": "aide-de-camp",
    "aiding": "aid",
    "aids": "aid",
    "ailed": "ail",
    "ailerons": "aileron",
    "ailing": "ail",
    "ailments": "ailment",
    "ails": "ail",
    "aimed": "aim",
    "aiming": "aim",
    "aims": "aim",
    "air-bladders": "air-bladder",
    "air-conditioners": "air-conditioner",
    "air-dried": "air-dry",
    "air-dries": "air-dry",
    "air-drying": "air-dry",
    "air-flows": "air-flow",
    "air-pumps": "air-pump",
    "air-raids": "air-raid",
    "air-shafts": "air-shaft",
    "airbags": "airbag",
    "airbeds": "airbed",
    "airbricks": "airbrick",
    "airbrushed": "airbrush",
    "airbrushes": "airbrush",
    "airbrushing": "airbrush",
    "airbuses": "airbus",
    "airconditionings": "airconditioning",
    "aircrafts": "aircraft",
    "aircrews": "aircrew",
    "airdrops": "airdrop",
    "aired": "air",
    "airfields": "airfield",
    "airflows": "airflow",
    "airframes": "airframe",
    "airguns": "airgun",
    "airier": "airy",
    "airiest": "airy",
    "airig": "aire",
    "airing": "air",
    "airlanes": "airlane",
    "airletters": "airletter",
    "airlifted": "airlift",
    "airlifting": "airlift",
    "airlifts": "airlift",
    "airliners": "airliner",
    "airlines": "airline",
    "airlocks": "airlock",
    "airmailed": "airmail",
    "airmailing": "airmail",
    "airmails": "airmail",
    "airmen": "airman",
    "airplanes": "airplane",
    "airports": "airport",
    "airs": "air",
    "airships": "airship",
    "airspaces": "airspace",
    "airspeeds": "airspeed",
    "airstreams": "airstream",
    "airstrikes": "airstrike",
    "airstrips": "airstrip",
    "airwaves": "airwave",
    "airways": "airway",
    "airwomen": "airwoman",
    "aisles": "aisle",
    "aitches": "aitch",
    "alae": "ala",
    "alanines": "alanine",
    "alarmed": "alarm",
    "alarming": "alarm",
    "alarmists": "alarmist",
    "alarms": "alarm",
    "albatrosses": "albatross",
    "albedos": "albedo",
    "albinisms": "albinism",
    "albinos": "albino",
    "albumens": "albumen",
    "albumins": "albumin",
    "albums": "album",
    "alchemies": "alchemy",
    "alchemists": "alchemist",
    "alcoholics": "alcoholic",
    "alcoholisms": "alcoholism",
    "alcohols": "alcohol",
    "alcopops": "alcopop",
    "alcoves": "alcove",
    "aldehydes": "aldehyde",
    "aldermen": "alderman",
    "alders": "alder",
    "aldosterones": "aldosterone",
    "alerted": "alert",
    "alerting": "alert",
    "alerts": "alert",
    "ales": "ale",
    "alexandrines": "alexandrine",
    "alexithymias": "alexithymia",
    "alfalfas": "alfalfa",
    "alfas": "alfa",
    "algae": "alga",
    "algas": "alga",
    "algebras": "algebra",
    "alginates": "alginate",
    "algorithms": "algorithm",
    "aliased": "alias",
    "aliases": "alias",
    "aliasing": "alias",
    "alibis": "alibi",
    "alienated": "alienate",
    "alienates": "alienate",
    "alienating": "alienate",
    "alienations": "alienation",
    "aliens": "alien",
    "alighted": "alight",
    "alighting": "alight",
    "alights": "alight",
    "aligned": "align",
    "aligning": "align",
    "alignments": "alignment",
    "aligns": "align",
    "aliments": "aliment",
    "aliquots": "aliquot",
    "alit": "alight",
    "alkalies": "alkali",
    "alkalinities": "alkalinity",
    "alkalis": "alkali",
    "alkaloids": "alkaloid",
    "alkaloses": "alkalosis",
    "alkanes": "alkane",
    "alkenes": "alkene",
    "alkylations": "alkylation",
    "alkyls": "alkyl",
    "alkynes": "alkyne",
    "all-rounders": "all-rounder",
    "all-rounders.": "all-rounder",
    "allantoins": "allantoin",
    "allayed": "allay",
    "allaying": "allay",
    "allays": "allay",
    "allegations": "allegation",
    "alleged": "allege",
    "alleges": "allege",
    "allegiances": "allegiance",
    "alleging": "allege",
    "allegories": "allegory",
    "alleles": "allele",
    "allergenicities": "allergenicity",
    "allergens": "allergen",
    "allergies": "allergy",
    "alleviated": "alleviate",
    "alleviates": "alleviate",
    "alleviating": "alleviate",
    "alley-ways": "alley-way",
    "alleys": "alley",
    "alleyways": "alleyway",
    "alliances": "alliance",
    "allied": "ally",
    "allies": "ally",
    "alligators": "alligator",
    "alliterations": "alliteration",
    "alliums": "allium",
    "alloantibodies": "alloantibody",
    "allocated": "allocate",
    "allocates": "allocate",
    "allocating": "allocate",
    "allocations": "allocation",
    "allocators": "allocator",
    "allografts": "allograft",
    "allophones": "allophone",
    "allotments": "allotment",
    "allots": "allot",
    "allotted": "allot",
    "allotting": "allot",
    "allowances": "allowance",
    "allowed": "allow",
    "allowing": "allow",
    "allows": "allow",
    "alloyed": "alloy",
    "alloying": "alloy",
    "alloys": "alloy",
    "allspices": "allspice",
    "alluded": "allude",
    "alludes": "allude",
    "alluding": "allude",
    "allured": "allure",
    "allurements": "allurement",
    "allures": "allure",
    "alluring": "allure",
    "allusions": "allusion",
    "alluvia": "alluvium",
    "alluviums": "alluvium",
    "allying": "ally",
    "almanacs": "almanac",
    "almonds": "almond",
    "almshouses": "almshouse",
    "aloes": "aloe",
    "alohas": "aloha",
    "alopecias": "alopecia",
    "alpacas": "alpaca",
    "alpha-blockers": "alpha-blocker",
    "alpha-helices": "alpha-helix",
    "alpha-helixes": "alpha-helix",
    "alphabetized": "alphabetize",
    "alphabetizes": "alphabetize",
    "alphabetizing": "alphabetize",
    "alphabets": "alphabet",
    "alphas": "alpha",
    "alpines": "alpine",
    "also": "conjurer",
    "also-rans": "also-ran",
    "altars": "altar",
    "alterations": "alteration",
    "altercations": "altercation",
    "altered": "alter",
    "altering": "alter",
    "alternated": "alternate",
    "alternates": "alternate",
    "alternating": "alternate",
    "alternations": "alternation",
    "alternatives": "alternative",
    "alternators": "alternator",
    "alters": "alter",
    "altimeters": "altimeter",
    "altitudes": "altitude",
    "altos": "alto",
    "altruisms": "altruism",
    "aluminiums": "aluminium",
    "aluminums": "aluminum",
    "alumnae": "alumna",
    "alumni": "alumnus",
    "alveoli": "alveolus",
    "am": "be",
    "amahs": "amah",
    "amalgamated": "amalgamate",
    "amalgamates": "amalgamate",
    "amalgamating": "amalgamate",
    "amalgamations": "amalgamation",
    "amalgams": "amalgam",
    "amanuenses": "amanuensis",
    "amaranths": "amaranth",
    "amassed": "amass",
    "amasses": "amass",
    "amassing": "amass",
    "amateurs": "amateur",
    "amazed": "amaze",
    "amazes": "amaze",
    "amazing": "amaze",
    "amazons": "amazon",
    "ambassadors": "ambassador",
    "ambiances": "ambiance",
    "ambiences": "ambience",
    "ambiguities": "ambiguity",
    "ambitions": "ambition",
    "ambits": "ambit",
    "ambled": "amble",
    "ambles": "amble",
    "ambling": "amble",
    "amblyopias": "amblyopia",
    "ambulancemen": "ambulanceman",
    "ambulances": "ambulance",
    "ambulations": "ambulation",
    "ambushed": "ambush",
    "ambushes": "ambush",
    "ambushing": "ambush",
    "ameliorated": "ameliorate",
    "ameliorates": "ameliorate",
    "ameliorating": "ameliorate",
    "ameliorations": "amelioration",
    "amended": "amend",
    "amending": "amend",
    "amendments": "amendment",
    "amends": "amend",
    "amenities": "amenity",
    "amenorrheas": "amenorrhea",
    "amenorrhoeas": "amenorrhoea",
    "amens": "amen",
    "amerced": "amerce",
    "amerces": "amerce",
    "amercing": "amerce",
    "americanised": "americanise",
    "americanises": "americanise",
    "americanising": "americanise",
    "amethysts": "amethyst",
    "amides": "amide",
    "amines": "amine",
    "amino-acids": "amino-acid",
    "aminoacids": "aminoacid",
    "aminoglycosides": "aminoglycoside",
    "amirs": "amir",
    "ammeters": "ammeter",
    "ammonias": "ammonia",
    "ammonites": "ammonite",
    "ammoniums": "ammonium",
    "ammunitions": "ammunition",
    "amnesias": "amnesia",
    "amnesties": "amnesty",
    "amniocenteses": "amniocentesis",
    "amoebae": "amoeba",
    "amoebas": "amoeba",
    "amors": "amor",
    "amortisations": "amortisation",
    "amortised": "amortise",
    "amortises": "amortise",
    "amortising": "amortise",
    "amortizations": "amortization",
    "amortized": "amortize",
    "amortizes": "amortize",
    "amortizing": "amortize",
    "amounted": "amount",
    "amounting": "amount",
    "amounts": "amount",
    "amours": "amour",
    "amperages": "amperage",
    "amperes": "ampere",
    "ampersands": "ampersand",
    "amphetamines": "amphetamine",
    "amphibians": "amphibian",
    "amphiboles": "amphibole",
    "amphipods": "amphipod",
    "amphitheaters": "amphitheater",
    "amphitheatres": "amphitheatre",
    "amphorae": "amphora",
    "amphoras": "amphora",
    "amphotericins": "amphotericin",
    "ampler": "ample",
    "amplest": "ample",
    "amplicons": "amplicon",
    "amplifications": "amplification",
    "amplified": "amplify",
    "amplifiers": "amplifier",
    "amplifies": "amplify",
    "amplifying": "amplify",
    "amplitudes": "amplitude",
    "ampoules": "ampoule",
    "amps": "amp",
    "amputated": "amputate",
    "amputates": "amputate",
    "amputating": "amputate",
    "amputations": "amputation",
    "amputees": "amputee",
    "ampères": "ampère",
    "amulets": "amulet",
    "amused": "amuse",
    "amusements": "amusement",
    "amuses": "amuse",
    "amusing": "amuse",
    "amygdalae": "amygdala",
    "amygdalas": "amygdala",
    "amylases": "amylase",
    "amyloidoses": "amyloidosis",
    "amyloids": "amyloid",
    "amyls": "amyl",
    "an": "a",
    "anachronisms": "anachronism",
    "anacondas": "anaconda",
    "anaemias": "anaemia",
    "anaerobes": "anaerobe",
    "anaesthesias": "anaesthesia",
    "anaesthesiologists": "anaesthesiologist",
    "anaesthetics": "anaesthetic",
    "anaesthetised": "anaesthetise",
    "anaesthetises": "anaesthetise",
    "anaesthetising": "anaesthetise",
    "anaesthetists": "anaesthetist",
    "anaesthetized": "anaesthetize",
    "anaesthetizes": "anaesthetize",
    "anaesthetizing": "anaesthetize",
    "anagrams": "anagram",
    "analgesias": "analgesia",
    "analgesics": "analgesic",
    "analogies": "analogy",
    "analogs": "analog",
    "analogues": "analogue",
    "analysands": "analysand",
    "analysed": "analyse",
    "analysers": "analyser",
    "analyses": "analysis",
    "analysing": "analyse",
    "analysts": "analyst",
    "analytes": "analyte",
    "analytical": "analytic",
    "analyzed": "analyze",
    "analyzers": "analyzer",
    "analyzes": "analyze",
    "analyzing": "analyze",
    "anapaests": "anapaest",
    "anaphylaxes": "anaphylaxis",
    "anarchies": "anarchy",
    "anarchists": "anarchist",
    "anastomoses": "anastomosis",
    "anathemas": "anathema",
    "anathemata": "anathema",
    "anatomies": "anatomy",
    "anatomists": "anatomist",
    "ancestors": "ancestor",
    "ancestries": "ancestry",
    "anchorages": "anchorage",
    "anchored": "anchor",
    "anchoresses": "anchoress",
    "anchoring": "anchor",
    "anchorites": "anchorite",
    "anchormen": "anchorman",
    "anchors": "anchor",
    "anchovies": "anchovy",
    "ancients": "ancient",
    "ancillaries": "ancillary",
    "andantes": "andante",
    "androgens": "androgen",
    "androids": "android",
    "anecdotes": "anecdote",
    "anemias": "anemia",
    "anemometers": "anemometer",
    "anemones": "anemone",
    "anesthesias": "anesthesia",
    "anesthesiologists": "anesthesiologist",
    "anesthetics": "anesthetic",
    "anesthetists": "anesthetist",
    "anesthetized": "anesthetize",
    "anesthetizes": "anesthetize",
    "anesthetizing": "anesthetize",
    "aneuploidies": "aneuploidy",
    "aneurysms": "aneurysm",
    "angelfishes": "angelfish",
    "angelicas": "angelica",
    "angels": "angel",
    "angered": "anger",
    "angering": "anger",
    "angers": "anger",
    "anginas": "angina",
    "angioedemas": "angioedema",
    "angioedemata": "angioedema",
    "angiogeneses": "angiogenesis",
    "angiograms": "angiogram",
    "angiographies": "angiography",
    "angiosperms": "angiosperm",
    "angiotensins": "angiotensin",
    "angled": "angle",
    "anglers": "angler",
    "angles": "angle",
    "anglicans": "anglican",
    "anglicised": "anglicise",
    "anglicises": "anglicise",
    "anglicising": "anglicise",
    "anglicized": "anglicize",
    "anglicizes": "anglicize",
    "anglicizing": "anglicize",
    "angling": "angle",
    "anglings": "angling",
    "anglophones": "anglophone",
    "angoras": "angora",
    "angrier": "angry",
    "angriest": "angry",
    "angstroms": "angstrom",
    "angularities": "angularity",
    "angulations": "angulation",
    "anhydrases": "anhydrase",
    "anhydrides": "anhydride",
    "ani": "anus",
    "anilines": "aniline",
    "animals": "animal",
    "animated": "animate",
    "animates": "animate",
    "animating": "animate",
    "animations": "animation",
    "animators": "animator",
    "animists": "animist",
    "animosities": "animosity",
    "anions": "anion",
    "anisotropies": "anisotropy",
    "ankles": "ankle",
    "anklets": "anklet",
    "ankylosed": "ankylose",
    "ankyloses": "ankylose",
    "ankylosing": "ankylose",
    "annas": "anna",
    "annealed": "anneal",
    "annealing": "anneal",
    "anneals": "anneal",
    "annexations": "annexation",
    "annexed": "annexe",
    "annexes": "annexe",
    "annexing": "annexe",
    "annihilated": "annihilate",
    "annihilates": "annihilate",
    "annihilating": "annihilate",
    "annihilations": "annihilation",
    "anniversaries": "anniversary",
    "annotated": "annotate",
    "annotates": "annotate",
    "annotating": "annotate",
    "annotations": "annotation",
    "annotators": "annotator",
    "announced": "announce",
    "announcements": "announcement",
    "announcers": "announcer",
    "announces": "announce",
    "announcing": "announce",
    "annoyances": "annoyance",
    "annoyed": "annoy",
    "annoying": "annoy",
    "annoys": "annoy",
    "annualised": "annualise",
    "annualises": "annualise",
    "annualising": "annualise",
    "annuals": "annual",
    "annuitants": "annuitant",
    "annuities": "annuity",
    "annuli": "annulus",
    "annulled": "annul",
    "annulling": "annul",
    "annulments": "annulment",
    "annuls": "annul",
    "annuluses": "annulus",
    "anodes": "anode",
    "anodised": "anodise",
    "anodises": "anodise",
    "anodising": "anodise",
    "anodized": "anodize",
    "anodizes": "anodize",
    "anodizing": "anodize",
    "anodynes": "anodyne",
    "anointed": "anoint",
    "anointing": "anoint",
    "anoints": "anoint",
    "anomalies": "anomaly",
    "anomies": "anomie",
    "anonymised": "anonymise",
    "anonymises": "anonymise",
    "anonymising": "anonymise",
    "anophthalmias": "anophthalmia",
    "anoraks": "anorak",
    "anorexias": "anorexia",
    "anorexics": "anorexic",
    "anoxias": "anoxia",
    "answered": "answer",
    "answering": "answer",
    "answers": "answer",
    "ant-eaters": "ant-eater",
    "ant-hills": "ant-hill",
    "antacids": "antacid",
    "antagonised": "antagonise",
    "antagonises": "antagonise",
    "antagonising": "antagonise",
    "antagonisms": "antagonism",
    "antagonists": "antagonist",
    "antagonized": "antagonize",
    "antagonizes": "antagonize",
    "antagonizing": "antagonize",
    "anteaters": "anteater",
    "antecedents": "antecedent",
    "antechambers": "antechamber",
    "anted": "ante",
    "antedated": "antedate",
    "antedates": "antedate",
    "antedating": "antedate",
    "antelopes": "antelope",
    "antenatals": "antenatal",
    "antennae": "antenna",
    "antennas": "antenna",
    "anterooms": "anteroom",
    "antes": "ante",
    "anthelmintics": "anthelmintic",
    "anthems": "anthem",
    "anthers": "anther",
    "anthocyanins": "anthocyanin",
    "anthologies": "anthology",
    "anthraces": "anthrax",
    "anthracnoses": "anthracnose",
    "anthracyclines": "anthracycline",
    "anthropoids": "anthropoid",
    "anthropologies": "anthropology",
    "anthropologists": "anthropologist",
    "anti-Semites": "anti-Semite",
    "anti-abortionists": "anti-abortionist",
    "anti-biotics": "anti-biotic",
    "anti-coagulants": "anti-coagulant",
    "anti-depressants": "anti-depressant",
    "anti-ds": "anti-d",
    "anti-emetics": "anti-emetic",
    "anti-freezes": "anti-freeze",
    "anti-fungals": "anti-fungal",
    "anti-heroes": "anti-hero",
    "anti-histamines": "anti-histamine",
    "anti-inflammatories": "anti-inflammatory",
    "anti-malarials": "anti-malarial",
    "anti-oxidants": "anti-oxidant",
    "anti-retrovirals": "anti-retroviral",
    "anti-viruses": "anti-virus",
    "antiarrhythmics": "antiarrhythmic",
    "antibacterials": "antibacterial",
    "antibiograms": "antibiogram",
    "antibiotics": "antibiotic",
    "antibodies": "antibody",
    "anticardiolipins": "anticardiolipin",
    "anticholinergics": "anticholinergic",
    "anticholinesterases": "anticholinesterase",
    "anticipated": "anticipate",
    "anticipates": "anticipate",
    "anticipating": "anticipate",
    "anticipations": "anticipation",
    "anticlimaxes": "anticlimax",
    "anticoagulants": "anticoagulant",
    "anticonvulsants": "anticonvulsant",
    "antics": "antic",
    "anticses": "antics",
    "anticyclones": "anticyclone",
    "antidepressants": "antidepressant",
    "antidotes": "antidote",
    "antiemetics": "antiemetic",
    "antifreezes": "antifreeze",
    "antifungals": "antifungal",
    "antigens": "antigen",
    "antiheroes": "antihero",
    "antihistamines": "antihistamine",
    "antihypertensives": "antihypertensive",
    "antimacassars": "antimacassar",
    "antimicrobials": "antimicrobial",
    "anting": "ante",
    "antinomies": "antinomy",
    "antioxidants": "antioxidant",
    "antipathies": "antipathy",
    "antiplatelets": "antiplatelet",
    "antipodes": "antipode",
    "antiprotons": "antiproton",
    "antipsychotics": "antipsychotic",
    "antiquarians": "antiquarian",
    "antiquaries": "antiquary",
    "antiques": "antique",
    "antiquities": "antiquity",
    "antiretrovirals": "antiretroviral",
    "antis": "anti",
    "antisenses": "antisense",
    "antiseptics": "antiseptic",
    "antisera": "antiserum",
    "antiserums": "antiserum",
    "antitheses": "antithesis",
    "antitoxins": "antitoxin",
    "antitrypsins": "antitrypsin",
    "antivirals": "antiviral",
    "antiviruses": "antivirus",
    "antlers": "antler",
    "antonyms": "antonym",
    "ants": "ant",
    "anuses": "anus",
    "anvils": "anvil",
    "anxieties": "anxiety",
    "aortae": "aorta",
    "aortas": "aorta",
    "apartheids": "apartheid",
    "apartments": "apartment",
    "apatites": "apatite",
    "aped": "ape",
    "aperitifs": "aperitif",
    "apertures": "aperture",
    "apes": "ape",
    "apexes": "apex",
    "aphasias": "aphasia",
    "aphasics": "aphasic",
    "aphelia": "aphelion",
    "aphereses": "apheresis",
    "aphids": "aphid",
    "aphorisms": "aphorism",
    "aphrodisiacs": "aphrodisiac",
    "apiaries": "apiary",
    "apices": "apex",
    "aping": "ape",
    "apneas": "apnea",
    "apnoeas": "apnoea",
    "apocalypses": "apocalypse",
    "apogees": "apogee",
    "apolipoproteins": "apolipoprotein",
    "apologiae": "apologia",
    "apologias": "apologia",
    "apologies": "apology",
    "apologised": "apologise",
    "apologises": "apologise",
    "apologising": "apologise",
    "apologists": "apologist",
    "apologized": "apologize",
    "apologizes": "apologize",
    "apologizing": "apologize",
    "apoplexies": "apoplexy",
    "apoptoses": "apoptosis",
    "aporias": "aporia",
    "apostasies": "apostasy",
    "apostates": "apostate",
    "apostatized": "apostatize",
    "apostatizes": "apostatize",
    "apostatizing": "apostatize",
    "apostles": "apostle",
    "apostleships": "apostleship",
    "apostolates": "apostolate",
    "apostrophes": "apostrophe",
    "apothecaries": "apothecary",
    "apotheoses": "apotheosis",
    "appaled": "appal",
    "appaling": "appal",
    "appalled": "appall",
    "appalling": "appall",
    "appalls": "appall",
    "appals": "appal",
    "apparati": "apparatus",
    "apparatuses": "apparatus",
    "apparels": "apparel",
    "apparenter": "apparent",
    "apparentest": "apparent",
    "apparitions": "apparition",
    "appealed": "appeal",
    "appealing": "appeal",
    "appeals": "appeal",
    "appearances": "appearance",
    "appeared": "appear",
    "appearing": "appear",
    "appears": "appear",
    "appeased": "appease",
    "appeasements": "appeasement",
    "appeases": "appease",
    "appeasing": "appease",
    "appellants": "appellant",
    "appellations": "appellation",
    "appendages": "appendage",
    "appendectomies": "appendectomy",
    "appended": "append",
    "appendicectomies": "appendicectomy",
    "appendices": "appendix",
    "appendicitides": "appendicitis",
    "appending": "append",
    "appendixes": "appendix",
    "appends": "append",
    "appertained": "appertain",
    "appertaining": "appertain",
    "appertains": "appertain",
    "appetisers": "appetiser",
    "appetites": "appetite",
    "appetizers": "appetizer",
    "applauded": "applaud",
    "applauding": "applaud",
    "applauds": "applaud",
    "applauses": "applause",
    "apples": "apple",
    "applets": "applet",
    "appliances": "appliance",
    "applicabilities": "applicability",
    "applicants": "applicant",
    "applications": "application",
    "applicators": "applicator",
    "applied": "apply",
    "applies": "apply",
    "appliques": "applique",
    "applying": "apply",
    "appointed": "appoint",
    "appointees": "appointee",
    "appointing": "appoint",
    "appointments": "appointment",
    "appoints": "appoint",
    "apportioned": "apportion",
    "apportioning": "apportion",
    "apportionments": "apportionment",
    "apportions": "apportion",
    "apposed": "appose",
    "apposes": "appose",
    "apposing": "appose",
    "appositions": "apposition",
    "appraisals": "appraisal",
    "appraised": "appraise",
    "appraisees": "appraisee",
    "appraisers": "appraiser",
    "appraises": "appraise",
    "appraising": "appraise",
    "appreciated": "appreciate",
    "appreciates": "appreciate",
    "appreciating": "appreciate",
    "appreciations": "appreciation",
    "apprehended": "apprehend",
    "apprehending": "apprehend",
    "apprehends": "apprehend",
    "apprehensions": "apprehension",
    "apprenticed": "apprentice",
    "apprentices": "apprentice",
    "apprenticeships": "apprenticeship",
    "apprenticing": "apprentice",
    "apprised": "apprise",
    "apprises": "apprise",
    "apprising": "apprise",
    "approached": "approach",
    "approaches": "approach",
    "approaching": "approach",
    "appropriated": "appropriate",
    "appropriates": "appropriate",
    "appropriating": "appropriate",
    "appropriations": "appropriation",
    "approvals": "approval",
    "approved": "approve",
    "approves": "approve",
    "approving": "approve",
    "approximated": "approximate",
    "approximates": "approximate",
    "approximating": "approximate",
    "approximations": "approximation",
    "apps": "app",
    "appurtenances": "appurtenance",
    "apricots": "apricot",
    "aprons": "apron",
    "apter": "apt",
    "aptest": "apt",
    "aptitudes": "aptitude",
    "aquacultures": "aquaculture",
    "aquae": "aqua",
    "aqualungs": "aqualung",
    "aquamarines": "aquamarine",
    "aquaplaned": "aquaplane",
    "aquaplanes": "aquaplane",
    "aquaplaning": "aquaplane",
    "aquaria": "aquarium",
    "aquarists": "aquarist",
    "aquariums": "aquarium",
    "aquas": "aqua",
    "aquatics": "aquatic",
    "aquatints": "aquatint",
    "aqueducts": "aqueduct",
    "aquifers": "aquifer",
    "arabesques": "arabesque",
    "arachnids": "arachnid",
    "aragonites": "aragonite",
    "aramids": "aramid",
    "aras": "ara",
    "arbiters": "arbiter",
    "arbitrated": "arbitrate",
    "arbitrates": "arbitrate",
    "arbitrating": "arbitrate",
    "arbitrations": "arbitration",
    "arbitrators": "arbitrator",
    "arboreta": "arboretum",
    "arboretums": "arboretum",
    "arboricultures": "arboriculture",
    "arbors": "arbor",
    "arbours": "arbour",
    "arcades": "arcade",
    "arcana": "arcanum",
    "arced": "arc",
    "arch-enemies": "arch-enemy",
    "archaea": "archaeum",
    "archaeologies": "archaeology",
    "archaeologists": "archaeologist",
    "archaisms": "archaism",
    "archangels": "archangel",
    "archbishoprics": "archbishopric",
    "archbishops": "archbishop",
    "archdeacons": "archdeacon",
    "archdioceses": "archdiocese",
    "arched": "arch",
    "archeologies": "archeology",
    "archeologists": "archeologist",
    "archer": "arch",
    "archers": "archer",
    "arches": "arch",
    "archest": "arch",
    "archetypes": "archetype",
    "arching": "arch",
    "archipelagoes": "archipelago",
    "archipelagos": "archipelago",
    "architects": "architect",
    "architectures": "architecture",
    "architraves": "architrave",
    "archived": "archive",
    "archives": "archive",
    "archiving": "archive",
    "archivists": "archivist",
    "archs": "arch",
    "archways": "archway",
    "arcing": "arc",
    "arcked": "arc",
    "arcking": "arc",
    "arcs": "arc",
    "arctangents": "arctangent",
    "ardors": "ardor",
    "ardours": "ardour",
    "are": "be",
    "areae": "area",
    "areas": "area",
    "arenas": "arena",
    "arginines": "arginine",
    "argots": "argot",
    "argued": "argue",
    "argues": "argue",
    "arguing": "argue",
    "argumentations": "argumentation",
    "arguments": "argument",
    "arguses": "argus",
    "arias": "aria",
    "arisen": "arise",
    "arises": "arise",
    "arising": "arise",
    "aristocracies": "aristocracy",
    "aristocrats": "aristocrat",
    "arithmetics": "arithmetic",
    "arks": "ark",
    "arm-chairs": "arm-chair",
    "armadas": "armada",
    "armadilloes": "armadillo",
    "armadillos": "armadillo",
    "armaments": "armament",
    "armatures": "armature",
    "armbands": "armband",
    "armchairs": "armchair",
    "armed": "arm",
    "armfuls": "armful",
    "armholes": "armhole",
    "armies": "army",
    "arming": "arm",
    "armistices": "armistice",
    "armories": "armory",
    "armourers": "armourer",
    "armouries": "armoury",
    "armpits": "armpit",
    "armrests": "armrest",
    "arms": "arm",
    "armsful": "armful",
    "aromas": "aroma",
    "aromatases": "aromatase",
    "aromatherapies": "aromatherapy",
    "aromatherapists": "aromatherapist",
    "aromaticities": "aromaticity",
    "aromatics": "aromatic",
    "arose": "arise",
    "arousals": "arousal",
    "aroused": "arouse",
    "arouses": "arouse",
    "arousing": "arouse",
    "arpeggios": "arpeggio",
    "arraigned": "arraign",
    "arraigning": "arraign",
    "arraignments": "arraignment",
    "arraigns": "arraign",
    "arranged": "arrange",
    "arrangements": "arrangement",
    "arrangers": "arranger",
    "arranges": "arrange",
    "arranging": "arrange",
    "arrayed": "array",
    "arraying": "array",
    "arrays": "array",
    "arrested": "arrest",
    "arrestees": "arrestee",
    "arresting": "arrest",
    "arrestors": "arrestor",
    "arrests": "arrest",
    "arrhythmias": "arrhythmia",
    "arrivals": "arrival",
    "arrived": "arrive",
    "arrives": "arrive",
    "arriving": "arrive",
    "arrogated": "arrogate",
    "arrogates": "arrogate",
    "arrogating": "arrogate",
    "arrondissements": "arrondissement",
    "arrowheads": "arrowhead",
    "arrows": "arrow",
    "arsed": "arse",
    "arseholes": "arsehole",
    "arsenals": "arsenal",
    "arsenics": "arsenic",
    "arsenides": "arsenide",
    "arses": "arsis",
    "arsing": "arse",
    "arsonists": "arsonist",
    "arsons": "arson",
    "arst": "be",
    "artefacts": "artefact",
    "artemisinins": "artemisinin",
    "arteries": "artery",
    "arterioles": "arteriole",
    "arterioscleroses": "arteriosclerosis",
    "arteritides": "arteritis",
    "artforms": "artform",
    "arthralgias": "arthralgia",
    "arthritics": "arthritic",
    "arthritides": "arthritis",
    "arthritises": "arthritis",
    "arthrodeses": "arthrodesis",
    "arthropathies": "arthropathy",
    "arthroplasties": "arthroplasty",
    "arthropods": "arthropod",
    "arthroscopies": "arthroscopy",
    "artichokes": "artichoke",
    "articled": "article",
    "articles": "article",
    "articling": "article",
    "articulated": "articulate",
    "articulates": "articulate",
    "articulating": "articulate",
    "articulations": "articulation",
    "articulators": "articulator",
    "artifacts": "artifact",
    "artificers": "artificer",
    "artifices": "artifice",
    "artilleries": "artillery",
    "artillerymen": "artilleryman",
    "artisans": "artisan",
    "artistes": "artiste",
    "artists": "artist",
    "arts": "art",
    "artworks": "artwork",
    "arums": "arum",
    "aryls": "aryl",
    "as": "a",
    "asbestoses": "asbestosis",
    "ascendants": "ascendant",
    "ascended": "ascend",
    "ascending": "ascend",
    "ascends": "ascend",
    "ascents": "ascent",
    "ascertained": "ascertain",
    "ascertaining": "ascertain",
    "ascertainments": "ascertainment",
    "ascertains": "ascertain",
    "asceticisms": "asceticism",
    "ascetics": "ascetic",
    "asci": "ascus",
    "ascidians": "ascidian",
    "ascospores": "ascospore",
    "ascribed": "ascribe",
    "ascribes": "ascribe",
    "ascribing": "ascribe",
    "ascriptions": "ascription",
    "ashcans": "ashcan",
    "ashes": "ash",
    "ashtrays": "ashtray",
    "asides": "aside",
    "asked": "ask",
    "asking": "ask",
    "asks": "ask",
    "aspartames": "aspartame",
    "aspartates": "aspartate",
    "aspects": "aspect",
    "aspens": "aspen",
    "aspergers": "asperger",
    "asperities": "asperity",
    "aspersions": "aspersion",
    "asphalted": "asphalt",
    "asphalting": "asphalt",
    "asphalts": "asphalt",
    "asphodels": "asphodel",
    "asphyxias": "asphyxia",
    "asphyxiated": "asphyxiate",
    "asphyxiates": "asphyxiate",
    "asphyxiating": "asphyxiate",
    "asphyxiations": "asphyxiation",
    "aspirants": "aspirant",
    "aspirated": "aspirate",
    "aspirates": "aspirate",
    "aspirating": "aspirate",
    "aspirations": "aspiration",
    "aspired": "aspire",
    "aspires": "aspire",
    "aspiring": "aspire",
    "aspirins": "aspirin",
    "asps": "asp",
    "assailants": "assailant",
    "assailed": "assail",
    "assailing": "assail",
    "assails": "assail",
    "assassinated": "assassinate",
    "assassinates": "assassinate",
    "assassinating": "assassinate",
    "assassinations": "assassination",
    "assassins": "assassin",
    "assaulted": "assault",
    "assaulting": "assault",
    "assaults": "assault",
    "assayed": "assay",
    "assaying": "assay",
    "assays": "assay",
    "assemblages": "assemblage",
    "assembled": "assemble",
    "assemblers": "assembler",
    "assembles": "assemble",
    "assemblies": "assembly",
    "assembling": "assemble",
    "assented": "assent",
    "assenting": "assent",
    "assents": "assent",
    "asserted": "assert",
    "asserting": "assert",
    "assertions": "assertion",
    "asserts": "assert",
    "asses": "asse",
    "assessed": "assess",
    "assesses": "assess",
    "assessing": "assess",
    "assessments": "assessment",
    "assessors": "assessor",
    "assets": "asset",
    "assiduities": "assiduity",
    "assignations": "assignation",
    "assigned": "assign",
    "assignees": "assignee",
    "assigning": "assign",
    "assignments": "assignment",
    "assignors": "assignor",
    "assigns": "assign",
    "assimilated": "assimilate",
    "assimilates": "assimilate",
    "assimilating": "assimilate",
    "assimilations": "assimilation",
    "assistants": "assistant",
    "assisted": "assist",
    "assisting": "assist",
    "assists": "assist",
    "assizes": "assize",
    "associated": "associate",
    "associates": "associate",
    "associateships": "associateship",
    "associating": "associate",
    "associations": "association",
    "associativities": "associativity",
    "assonances": "assonance",
    "assortments": "assortment",
    "assuaged": "assuage",
    "assuages": "assuage",
    "assuaging": "assuage",
    "assumed": "assume",
    "assumes": "assume",
    "assuming": "assume",
    "assumptions": "assumption",
    "assurances": "assurance",
    "assured": "assure",
    "assures": "assure",
    "assuring": "assure",
    "asterisked": "asterisk",
    "asterisking": "asterisk",
    "asterisks": "asterisk",
    "asteroids": "asteroid",
    "asters": "aster",
    "asthenias": "asthenia",
    "asthmas": "asthma",
    "asthmatics": "asthmatic",
    "astigmatisms": "astigmatism",
    "astonished": "astonish",
    "astonishes": "astonish",
    "astonishing": "astonish",
    "astonishments": "astonishment",
    "astounded": "astound",
    "astounding": "astound",
    "astounds": "astound",
    "astringents": "astringent",
    "astrocytes": "astrocyte",
    "astrocytomas": "astrocytoma",
    "astrocytomata": "astrocytoma",
    "astrologers": "astrologer",
    "astronauts": "astronaut",
    "astronomers": "astronomer",
    "astrophysicists": "astrophysicist",
    "asylums": "asylum",
    "asymmetries": "asymmetry",
    "asymptotes": "asymptote",
    "asymptotics": "asymptotic",
    "ataxias": "ataxia",
    "ate": "eat",
    "atheists": "atheist",
    "atheromas": "atheroma",
    "atheromata": "atheroma",
    "atheroscleroses": "atherosclerosis",
    "athletes": "athlete",
    "athletics": "athletic",
    "atlas": "atla",
    "atlases": "atlas",
    "atmospheres": "atmosphere",
    "atolls": "atoll",
    "atomised": "atomise",
    "atomisers": "atomiser",
    "atomises": "atomise",
    "atomising": "atomise",
    "atomized": "atomize",
    "atomizers": "atomizer",
    "atomizes": "atomize",
    "atomizing": "atomize",
    "atoms": "atom",
    "atoned": "atone",
    "atones": "atone",
    "atoning": "atone",
    "atopies": "atopy",
    "atresias": "atresia",
    "atria": "atrium",
    "atriums": "atrium",
    "atrocities": "atrocity",
    "atrophied": "atrophy",
    "atrophies": "atrophy",
    "atrophying": "atrophy",
    "attached": "attach",
    "attaches": "attache",
    "attaching": "attach",
    "attachments": "attachment",
    "attachés": "attaché",
    "attacked": "attack",
    "attackers": "attacker",
    "attacking": "attack",
    "attacks": "attack",
    "attained": "attain",
    "attaining": "attain",
    "attainments": "attainment",
    "attains": "attain",
    "attainted": "attaint",
    "attainting": "attaint",
    "attaints": "attaint",
    "attempted": "attempt",
    "attempting": "attempt",
    "attempts": "attempt",
    "attendances": "attendance",
    "attendants": "attendant",
    "attended": "attend",
    "attendees": "attendee",
    "attenders": "attender",
    "attending": "attend",
    "attends": "attend",
    "attentions": "attention",
    "attenuated": "attenuate",
    "attenuates": "attenuate",
    "attenuating": "attenuate",
    "attenuations": "attenuation",
    "attenuators": "attenuator",
    "attestations": "attestation",
    "attested": "attest",
    "attesting": "attest",
    "attests": "attest",
    "attics": "attic",
    "attired": "attire",
    "attires": "attire",
    "attiring": "attire",
    "attitudes": "attitude",
    "attorneys": "attorney",
    "attractants": "attractant",
    "attracted": "attract",
    "attracting": "attract",
    "attractions": "attraction",
    "attractors": "attractor",
    "attracts": "attract",
    "attributed": "attribute",
    "attributes": "attribute",
    "attributing": "attribute",
    "attributions": "attribution",
    "atts": "att",
    "attuned": "attune",
    "attunes": "attune",
    "attuning": "attune",
    "aubergines": "aubergine",
    "auctioned": "auction",
    "auctioneers": "auctioneer",
    "auctioning": "auction",
    "auctions": "auction",
    "audacities": "audacity",
    "audiences": "audience",
    "audio-typists": "audio-typist",
    "audiocassettes": "audiocassette",
    "audiograms": "audiogram",
    "audiologies": "audiology",
    "audiologists": "audiologist",
    "audiometries": "audiometry",
    "audiotapes": "audiotape",
    "audited": "audit",
    "auditing": "audit",
    "auditioned": "audition",
    "auditioning": "audition",
    "auditions": "audition",
    "auditoria": "auditorium",
    "auditoriums": "auditorium",
    "auditors": "auditor",
    "audits": "audit",
    "augers": "auger",
    "augmentations": "augmentation",
    "augmented": "augment",
    "augmenting": "augment",
    "augments": "augment",
    "augured": "augur",
    "auguries": "augury",
    "auguring": "augur",
    "augurs": "augur",
    "auks": "auk",
    "aunties": "aunty",
    "aunts": "aunt",
    "aurae": "aura",
    "auras": "aura",
    "aurei": "aureus",
    "aureoles": "aureole",
    "aurochs": "auroch",
    "aurorae": "aurora",
    "auroras": "aurora",
    "auscultations": "auscultation",
    "auspices": "auspice",
    "austerer": "austere",
    "austerest": "austere",
    "austerities": "austerity",
    "autarchies": "autarchy",
    "authenticated": "authenticate",
    "authenticates": "authenticate",
    "authenticating": "authenticate",
    "authored": "author",
    "authoresses": "authoress",
    "authoring": "author",
    "authorisations": "authorisation",
    "authorised": "authorise",
    "authorises": "authorise",
    "authorising": "authorise",
    "authoritarianisms": "authoritarianism",
    "authoritarians": "authoritarian",
    "authorities": "authority",
    "authorizations": "authorization",
    "authorized": "authorize",
    "authorizes": "authorize",
    "authorizing": "authorize",
    "authors": "author",
    "authorships": "authorship",
    "autisms": "autism",
    "autoantibodies": "autoantibody",
    "autobahns": "autobahn",
    "autobiographies": "autobiography",
    "autoclaved": "autoclave",
    "autoclaves": "autoclave",
    "autoclaving": "autoclave",
    "autocorrelations": "autocorrelation",
    "autocracies": "autocracy",
    "autocrats": "autocrat",
    "autocues": "autocue",
    "autografts": "autograft",
    "autographed": "autograph",
    "autographing": "autograph",
    "autographs": "autograph",
    "autoimmunities": "autoimmunity",
    "automakers": "automaker",
    "automata": "automaton",
    "automated": "automate",
    "automates": "automate",
    "automaticities": "automaticity",
    "automatics": "automatic",
    "automating": "automate",
    "automations": "automation",
    "automatisms": "automatism",
    "automatons": "automaton",
    "automats": "automat",
    "automobiles": "automobile",
    "automorphisms": "automorphism",
    "autonomies": "autonomy",
    "autonomists": "autonomist",
    "autopoieses": "autopoiesis",
    "autopsies": "autopsy",
    "autos": "auto",
    "autumns": "autumn",
    "auxiliaries": "auxiliary",
    "auxins": "auxin",
    "availabilities": "availability",
    "availed": "avail",
    "availing": "avail",
    "avails": "avail",
    "avalanched": "avalanche",
    "avalanches": "avalanche",
    "avalanching": "avalanche",
    "avant-gardes": "avant-garde",
    "avatars": "avatar",
    "avenged": "avenge",
    "avengers": "avenger",
    "avenges": "avenge",
    "avenging": "avenge",
    "avens": "aven",
    "avenues": "avenue",
    "averaged": "average",
    "averages": "average",
    "averaging": "average",
    "averred": "aver",
    "averring": "aver",
    "avers": "aver",
    "aversions": "aversion",
    "averted": "avert",
    "averting": "avert",
    "averts": "avert",
    "aviaries": "aviary",
    "aviators": "aviator",
    "avidities": "avidity",
    "avifaunas": "avifauna",
    "avocadoes": "avocado",
    "avocados": "avocado",
    "avocations": "avocation",
    "avocets": "avocet",
    "avoidances": "avoidance",
    "avoided": "avoid",
    "avoiders": "avoider",
    "avoiding": "avoid",
    "avoids": "avoid",
    "avowals": "avowal",
    "avowed": "avow",
    "avowing": "avow",
    "avows": "avow",
    "avs": "av",
    "avulsions": "avulsion",
    "awaited": "await",
    "awaiting": "await",
    "awaits": "await",
    "awakened": "awaken",
    "awakening": "awaken",
    "awakenings": "awakening",
    "awakens": "awaken",
    "awakes": "awake",
    "awaking": "awake",
    "awarded": "award",
    "awardees": "awardee",
    "awarding": "award",
    "awards": "award",
    "awarenesses": "awareness",
    "awed": "awe",
    "aweing": "awe",
    "awes": "awe",
    "awfuller": "awful",
    "awfullest": "awful",
    "awing": "awe",
    "awkwarder": "awkward",
    "awkwardest": "awkward",
    "awkwardnesses": "awkwardness",
    "awls": "awl",
    "awnings": "awning",
    "awns": "awn",
    "awoke": "awake",
    "awoken": "awake",
    "aws": "aw",
    "axed": "axe",
    "axes": "axis",
    "axillae": "axilla",
    "axillas": "axilla",
    "axils": "axil",
    "axing": "axe",
    "axioms": "axiom",
    "axises": "axis",
    "axles": "axle",
    "axons": "axon",
    "ayahuascas": "ayahuasca",
    "ayes": "aye",
    "azaleas": "azalea",
    "azimuths": "azimuth",
    "azures": "azure",
    "baas": "baa",
    "babbled": "babble",
    "babblers": "babbler",
    "babbles": "babble",
    "babbling": "babble",
    "babes": "babe",
    "babies": "baby",
    "baboons": "baboon",
    "baby-boomers": "baby-boomer",
    "baby-minders": "baby-minder",
    "baby-sat": "baby-sit",
    "baby-sits": "baby-sit",
    "baby-sitted": "baby-sit",
    "baby-sitters": "baby-sitter",
    "baby-sitting": "baby-sit",
    "babyhoods": "babyhood",
    "babysitters": "babysitter",
    "baccalaureates": "baccalaureate",
    "bachelors": "bachelor",
    "bacilli": "bacillus",
    "back-cloths": "back-cloth",
    "back-combed": "back-comb",
    "back-combing": "back-comb",
    "back-combs": "back-comb",
    "back-drops": "back-drop",
    "back-packs": "back-pack",
    "back-pedalled": "back-pedal",
    "back-pedalling": "back-pedal",
    "back-pedals": "back-pedal",
    "back-ups": "back-up",
    "backaches": "backache",
    "backbenchers": "backbencher",
    "backbenches": "backbench",
    "backbit": "backbite",
    "backbites": "backbite",
    "backbiting": "backbite",
    "backbitten": "backbite",
    "backboards": "backboard",
    "backbones": "backbone",
    "backcloths": "backcloth",
    "backdated": "backdate",
    "backdates": "backdate",
    "backdating": "backdate",
    "backdrops": "backdrop",
    "backed": "back",
    "backers": "backer",
    "backfilled": "backfill",
    "backfilling": "backfill",
    "backfills": "backfill",
    "backfired": "backfire",
    "backfires": "backfire",
    "backfiring": "backfire",
    "backflows": "backflow",
    "backgammons": "backgammon",
    "backgrounders": "backgrounder",
    "backgrounds": "background",
    "backhanders": "backhander",
    "backhands": "backhand",
    "backhoes": "backhoe",
    "backing": "back",
    "backings": "backing",
    "backlashes": "backlash",
    "backlighted": "backlight",
    "backlighting": "backlight",
    "backlights": "backlight",
    "backlit": "backlight",
    "backlogs": "backlog",
    "backpackers": "backpacker",
    "backpacks": "backpack",
    "backrests": "backrest",
    "backrooms": "backroom",
    "backs": "back",
    "backseats": "backseat",
    "backsides": "backside",
    "backslid": "backslide",
    "backslidden": "backslide",
    "backsliders": "backslider",
    "backslides": "backslide",
    "backsliding": "backslide",
    "backspaces": "backspace",
    "backstabbings": "backstabbing",
    "backstrokes": "backstroke",
    "backtracked": "backtrack",
    "backtracking": "backtrack",
    "backtracks": "backtrack",
    "backups": "backup",
    "backwashes": "backwash",
    "backwaters": "backwater",
    "backyards": "backyard",
    "bacons": "bacon",
    "bacteraemias": "bacteraemia",
    "bacteria": "bacterium",
    "bacterias": "bacteria",
    "bacteriologies": "bacteriology",
    "bacteriologists": "bacteriologist",
    "bacteriophages": "bacteriophage",
    "bacteriurias": "bacteriuria",
    "baculoviruses": "baculovirus",
    "baddies": "baddy",
    "bade": "bid",
    "badged": "badge",
    "badgered": "badger",
    "badgering": "badger",
    "badgers": "badger",
    "badges": "badge",
    "badging": "badge",
    "badnesses": "badness",
    "baffled": "baffle",
    "baffles": "baffle",
    "baffling": "baffle",
    "bagatelles": "bagatelle",
    "bagels": "bagel",
    "bagfuls": "bagful",
    "bagged": "bag",
    "baggers": "bagger",
    "baggier": "baggy",
    "baggiest": "baggy",
    "bagging": "bag",
    "bagpipes": "bagpipe",
    "bags": "bag",
    "bagsful": "bagful",
    "baguettes": "baguette",
    "bailed": "bail",
    "bailees": "bailee",
    "baileys": "bailey",
    "bailiffs": "bailiff",
    "bailing": "bail",
    "bailiwicks": "bailiwick",
    "bailouts": "bailout",
    "bails": "bail",
    "bairns": "bairn",
    "baited": "bait",
    "baiting": "bait",
    "baits": "bait",
    "baked": "bake",
    "bakeries": "bakery",
    "bakers": "baker",
    "bakes": "bake",
    "baking": "bake",
    "balaclavas": "balaclava",
    "balalaikas": "balalaika",
    "balanced": "balance",
    "balancers": "balancer",
    "balances": "balance",
    "balancing": "balance",
    "balconies": "balcony",
    "balder": "bald",
    "baldest": "bald",
    "baled": "bale",
    "balers": "baler",
    "bales": "bale",
    "baling": "bale",
    "balked": "balk",
    "balking": "balk",
    "balks": "balk",
    "ball-bearings": "ball-bearing",
    "ball-points": "ball-point",
    "ballads": "ballad",
    "ballasted": "ballast",
    "ballasting": "ballast",
    "ballasts": "ballast",
    "ballcocks": "ballcock",
    "balled": "ball",
    "ballerinas": "ballerina",
    "ballets": "ballet",
    "ballgames": "ballgame",
    "balling": "ball",
    "ballistics": "ballistic",
    "ballooned": "balloon",
    "ballooning": "balloon",
    "balloonists": "balloonist",
    "balloons": "balloon",
    "balloted": "ballot",
    "balloting": "ballot",
    "ballots": "ballot",
    "ballplayers": "ballplayer",
    "ballpoints": "ballpoint",
    "ballrooms": "ballroom",
    "balls": "ball",
    "balmier": "balmy",
    "balmiest": "balmy",
    "balms": "balm",
    "balsams": "balsam",
    "balsas": "balsa",
    "balustrades": "balustrade",
    "bamboos": "bamboo",
    "bamboozled": "bamboozle",
    "bamboozles": "bamboozle",
    "bamboozling": "bamboozle",
    "bams": "bam",
    "banalities": "banality",
    "bananas": "banana",
    "bancos": "banco",
    "bandaged": "bandage",
    "bandages": "bandage",
    "bandaging": "bandage",
    "bandannas": "bandanna",
    "bandas": "banda",
    "banded": "band",
    "bandied": "bandy",
    "bandier": "bandy",
    "bandies": "bandy",
    "banding": "band",
    "bandings": "banding",
    "bandits": "bandit",
    "bandpasses": "bandpass",
    "bands": "band",
    "bandsmen": "bandsman",
    "bandstands": "bandstand",
    "bandwagons": "bandwagon",
    "bandwidths": "bandwidth",
    "bandying": "bandy",
    "banged": "bang",
    "bangers": "banger",
    "banging": "bang",
    "bangles": "bangle",
    "bangs": "bang",
    "banished": "banish",
    "banishes": "banish",
    "banishing": "banish",
    "banishments": "banishment",
    "banisters": "banister",
    "banjoes": "banjo",
    "banjos": "banjo",
    "banked": "bank",
    "bankers": "banker",
    "banking": "bank",
    "bankings": "banking",
    "banknotes": "banknote",
    "bankrolled": "bankroll",
    "bankrolling": "bankroll",
    "bankrolls": "bankroll",
    "bankruptcies": "bankruptcy",
    "bankrupted": "bankrupt",
    "bankrupting": "bankrupt",
    "bankrupts": "bankrupt",
    "banks": "bank",
    "banned": "ban",
    "banners": "banner",
    "banning": "ban",
    "bannisters": "bannister",
    "banqueted": "banquet",
    "banqueting": "banquet",
    "banquets": "banquet",
    "banquettes": "banquette",
    "bans": "ban",
    "banshees": "banshee",
    "bantam-weights": "bantam-weight",
    "bantams": "bantam",
    "bantered": "banter",
    "bantering": "banter",
    "banters": "banter",
    "banyans": "banyan",
    "baobabs": "baobab",
    "baptised": "baptise",
    "baptises": "baptise",
    "baptising": "baptise",
    "baptisms": "baptism",
    "baptized": "baptize",
    "baptizes": "baptize",
    "baptizing": "baptize",
    "bar-codes": "bar-code",
    "barbarians": "barbarian",
    "barbarisms": "barbarism",
    "barbarities": "barbarity",
    "barbecued": "barbecue",
    "barbecues": "barbecue",
    "barbecuing": "barbecue",
    "barbells": "barbell",
    "barbels": "barbel",
    "barbeques": "barbeque",
    "barbered": "barber",
    "barbering": "barber",
    "barbers": "barber",
    "barbershops": "barbershop",
    "barbets": "barbet",
    "barbicans": "barbican",
    "barbiturates": "barbiturate",
    "barbs": "barb",
    "barcodes": "barcode",
    "bards": "bard",
    "bared": "bare",
    "barer": "bare",
    "bares": "bare",
    "barest": "bare",
    "barflies": "barfly",
    "bargained": "bargain",
    "bargaining": "bargain",
    "bargains": "bargain",
    "barged": "barge",
    "bargees": "bargee",
    "barges": "barge",
    "barging": "barge",
    "baring": "bare",
    "barites": "barite",
    "baritones": "baritone",
    "barked": "bark",
    "barkers": "barker",
    "barking": "bark",
    "barks": "bark",
    "barleys": "barley",
    "barmaids": "barmaid",
    "barmen": "barman",
    "barmier": "barmy",
    "barmiest": "barmy",
    "barnacles": "barnacle",
    "barns": "barn",
    "barnstormed": "barnstorm",
    "barnstorming": "barnstorm",
    "barnstorms": "barnstorm",
    "barnyards": "barnyard",
    "barometers": "barometer",
    "baronesses": "baroness",
    "baronetcies": "baronetcy",
    "baronets": "baronet",
    "baronies": "barony",
    "barons": "baron",
    "barotraumas": "barotrauma",
    "barotraumata": "barotrauma",
    "barques": "barque",
    "barracked": "barrack",
    "barracking": "barrack",
    "barracks": "barrack",
    "barracudas": "barracuda",
    "barrages": "barrage",
    "barred": "bar",
    "barreled": "barrel",
    "barreling": "barrel",
    "barrelled": "barrel",
    "barrelling": "barrel",
    "barrels": "barrel",
    "barrener": "barren",
    "barrenest": "barren",
    "barres": "barre",
    "barricaded": "barricade",
    "barricades": "barricade",
    "barricading": "barricade",
    "barriers": "barrier",
    "barring": "bar",
    "barrios": "barrio",
    "barristers": "barrister",
    "barrows": "barrow",
    "bars": "bar",
    "bartenders": "bartender",
    "bartered": "barter",
    "bartering": "barter",
    "barters": "barter",
    "baryons": "baryon",
    "barytes": "baryte",
    "bas-reliefs": "bas-relief",
    "basalts": "basalt",
    "bascules": "bascule",
    "base-lines": "base-line",
    "base-pairs": "base-pair",
    "baseballs": "baseball",
    "baseboards": "baseboard",
    "based": "base",
    "baselines": "baseline",
    "basements": "basement",
    "baseplates": "baseplate",
    "baser": "base",
    "bases": "basis",
    "basest": "base",
    "bashed": "bash",
    "bashes": "bash",
    "bashing": "bash",
    "bashings": "bashing",
    "basics": "basic",
    "basilicas": "basilica",
    "basilisks": "basilisk",
    "basils": "basil",
    "basing": "base",
    "basins": "basin",
    "basked": "bask",
    "basketballs": "basketball",
    "baskets": "basket",
    "basking": "bask",
    "basks": "bask",
    "basophils": "basophil",
    "basses": "bass",
    "bassets": "basset",
    "bassi": "basso",
    "bassinets": "bassinet",
    "bassists": "bassist",
    "bassoons": "bassoon",
    "bassos": "basso",
    "bastards": "bastard",
    "basted": "baste",
    "bastes": "baste",
    "basting": "baste",
    "bastions": "bastion",
    "batched": "batch",
    "batches": "batch",
    "batching": "batch",
    "bated": "bate",
    "bates": "bate",
    "bath-houses": "bath-house",
    "bath-towels": "bath-towel",
    "bathchairs": "bathchair",
    "bathed": "bathe",
    "bathers": "bather",
    "bathes": "bathe",
    "bathhouses": "bathhouse",
    "bathing": "bathe",
    "bathings": "bathing",
    "bathmats": "bathmat",
    "bathrobes": "bathrobe",
    "bathrooms": "bathroom",
    "baths": "bath",
    "bathtubs": "bathtub",
    "bathwaters": "bathwater",
    "bathymetries": "bathymetry",
    "batiks": "batik",
    "bating": "bate",
    "batmen": "batman",
    "batons": "baton",
    "bats": "bat",
    "batsmen": "batsman",
    "battalions": "battalion",
    "batted": "bat",
    "battened": "batten",
    "battening": "batten",
    "battens": "batten",
    "battered": "batter",
    "batteries": "battery",
    "battering": "batter",
    "batterings": "battering",
    "batters": "batter",
    "battier": "batty",
    "battiest": "batty",
    "batting": "bat",
    "battle-axes": "battle-axe",
    "battle-fields": "battle-field",
    "battled": "battle",
    "battlefields": "battlefield",
    "battlegrounds": "battleground",
    "battles": "battle",
    "battleships": "battleship",
    "battling": "battle",
    "batts": "batt",
    "baubles": "bauble",
    "bauds": "baud",
    "baulked": "baulk",
    "baulking": "baulk",
    "baulks": "baulk",
    "bauxites": "bauxite",
    "bawdier": "bawdy",
    "bawdiest": "bawdy",
    "bawled": "bawl",
    "bawling": "bawl",
    "bawls": "bawl",
    "bayed": "bay",
    "baying": "bay",
    "bayoneted": "bayonet",
    "bayoneting": "bayonet",
    "bayonets": "bayonet",
    "bayonetted": "bayonet",
    "bayonetting": "bayonet",
    "bayous": "bayou",
    "bays": "bay",
    "bazaars": "bazaar",
    "bazookas": "bazooka",
    "bds": "bd",
    "beachcombers": "beachcomber",
    "beached": "beach",
    "beaches": "beach",
    "beachheads": "beachhead",
    "beaching": "beach",
    "beacons": "beacon",
    "beaded": "bead",
    "beadier": "beady",
    "beadiest": "beady",
    "beading": "bead",
    "beads": "bead",
    "beagles": "beagle",
    "beakers": "beaker",
    "beaks": "beak",
    "beamed": "beam",
    "beamers": "beamer",
    "beaming": "beam",
    "beamlines": "beamline",
    "beams": "beam",
    "beanbags": "beanbag",
    "beanpoles": "beanpole",
    "beans": "bean",
    "beansprouts": "beansprout",
    "bearded": "beard",
    "bearding": "beard",
    "beards": "beard",
    "bearers": "bearer",
    "bearing": "bear",
    "bearings": "bearing",
    "bears": "bear",
    "bearskins": "bearskin",
    "beasties": "beastie",
    "beastlier": "beastly",
    "beastliest": "beastly",
    "beasts": "beast",
    "beaten": "beat",
    "beaters": "beater",
    "beatifications": "beatification",
    "beatified": "beatify",
    "beatifies": "beatify",
    "beatifying": "beatify",
    "beating": "beat",
    "beatings": "beating",
    "beatitudes": "beatitude",
    "beatniks": "beatnik",
    "beats": "beat",
    "beaus": "beau",
    "beauticians": "beautician",
    "beauties": "beauty",
    "beautified": "beautify",
    "beautifies": "beautify",
    "beautifying": "beautify",
    "beauts": "beaut",
    "beaux": "beau",
    "beavered": "beaver",
    "beavering": "beaver",
    "beavers": "beaver",
    "became": "become",
    "beckoned": "beckon",
    "beckoning": "beckon",
    "beckons": "beckon",
    "becks": "beck",
    "becomes": "become",
    "becoming": "become",
    "becquerels": "becquerel",
    "bed-baths": "bed-bath",
    "bed-times": "bed-time",
    "bedazzled": "bedazzle",
    "bedazzles": "bedazzle",
    "bedazzling": "bedazzle",
    "bedbugs": "bedbug",
    "bedchambers": "bedchamber",
    "bedcovers": "bedcover",
    "bedded": "bed",
    "bedding": "bed",
    "beddings": "bedding",
    "bedecked": "bedeck",
    "bedecking": "bedeck",
    "bedecks": "bedeck",
    "bedeviled": "bedevil",
    "bedeviling": "bedevil",
    "bedevilled": "bedevil",
    "bedevilling": "bedevil",
    "bedevils": "bedevil",
    "bedfellows": "bedfellow",
    "bedforms": "bedform",
    "bedheads": "bedhead",
    "bedlams": "bedlam",
    "bedpans": "bedpan",
    "bedposts": "bedpost",
    "bedrolls": "bedroll",
    "bedrooms": "bedroom",
    "beds": "bed",
    "bedsheets": "bedsheet",
    "bedsides": "bedside",
    "bedsits": "bedsit",
    "bedsitters": "bedsitter",
    "bedspreads": "bedspread",
    "bedsteads": "bedstead",
    "bedstraws": "bedstraw",
    "bedtimes": "bedtime",
    "beeches": "beech",
    "beefburgers": "beefburger",
    "beefed": "beef",
    "beefier": "beefy",
    "beefiest": "beefy",
    "beefing": "beef",
    "beefs": "beef",
    "beefsteaks": "beefsteak",
    "beehives": "beehive",
    "beekeepers": "beekeeper",
    "beelines": "beeline",
    "been": "be",
    "beeped": "beep",
    "beepers": "beeper",
    "beeping": "beep",
    "beeps": "beep",
    "beerier": "beery",
    "beers": "beer",
    "bees": "bee",
    "beetled": "beetle",
    "beetles": "beetle",
    "beetling": "beetle",
    "beetroots": "beetroot",
    "beets": "beet",
    "beeves": "beef",
    "befalled": "befall",
    "befallen": "befall",
    "befalling": "befall",
    "befalls": "befall",
    "befell": "befall",
    "befits": "befit",
    "befitted": "befit",
    "befitting": "befit",
    "befriended": "befriend",
    "befriending": "befriend",
    "befriends": "befriend",
    "befuddled": "befuddle",
    "befuddles": "befuddle",
    "befuddling": "befuddle",
    "began": "begin",
    "begat": "beget",
    "begets": "beget",
    "begetting": "beget",
    "beggared": "beggar",
    "beggaring": "beggar",
    "beggars": "beggar",
    "begged": "beg",
    "begging": "beg",
    "beginners": "beginner",
    "beginning": "begin",
    "beginnings": "beginning",
    "begins": "begin",
    "begomoviruses": "begomovirus",
    "begonias": "begonia",
    "begot": "beget",
    "begotten": "beget",
    "begrudged": "begrudge",
    "begrudges": "begrudge",
    "begrudging": "begrudge",
    "begs": "beg",
    "beguiled": "beguile",
    "beguiles": "beguile",
    "beguiling": "beguile",
    "begun": "begin",
    "behalfs": "behalf",
    "behaved": "behave",
    "behaves": "behave",
    "behaving": "behave",
    "behaviorists": "behaviorist",
    "behaviors": "behavior",
    "behaviourists": "behaviourist",
    "behaviours": "behaviour",
    "beheaded": "behead",
    "beheading": "behead",
    "beheads": "behead",
    "beheld": "behold",
    "behemoths": "behemoth",
    "behests": "behest",
    "behinds": "behind",
    "behinds.": "behind",
    "beholders": "beholder",
    "beholders.": "beholder",
    "beholding": "behold",
    "beholds": "behold",
    "behooved": "behoove",
    "behooves": "behoove",
    "behooving": "behoove",
    "behoved": "behove",
    "behoves": "behove",
    "behoving": "behove",
    "beiger": "beige",
    "beiges": "beige",
    "beigest": "beige",
    "being": "be",
    "beings": "being",
    "belaboured": "belabour",
    "belabouring": "belabour",
    "belabours": "belabour",
    "belayed": "belay",
    "belaying": "belay",
    "belays": "belay",
    "belched": "belch",
    "belchers": "belcher",
    "belches": "belch",
    "belching": "belch",
    "beleaguered": "beleaguer",
    "beleaguering": "beleaguer",
    "beleaguers": "beleaguer",
    "belemnites": "belemnite",
    "belfries": "belfry",
    "belied": "bely",
    "beliefs": "belief",
    "belies": "bely",
    "believed": "believe",
    "believers": "believer",
    "believes": "believe",
    "believing": "believe",
    "belittled": "belittle",
    "belittles": "belittle",
    "belittling": "belittle",
    "bellboys": "bellboy",
    "belles": "belle",
    "bellhops": "bellhop",
    "bellied": "belly",
    "bellies": "belly",
    "belligerents": "belligerent",
    "bellowed": "bellow",
    "bellowing": "bellow",
    "bellows": "bellow",
    "bells": "bell",
    "bellyached": "bellyache",
    "bellyaches": "bellyache",
    "bellyaching": "bellyache",
    "bellybuttons": "bellybutton",
    "bellying": "belly",
    "belonged": "belong",
    "belonging": "belong",
    "belongings": "belonging",
    "belongs": "belong",
    "beloveds": "beloved",
    "belted": "belt",
    "belting": "belt",
    "belts": "belt",
    "belugas": "beluga",
    "belying": "bely",
    "bemoaned": "bemoan",
    "bemoaning": "bemoan",
    "bemoans": "bemoan",
    "bemused": "bemuse",
    "bemuses": "bemuse",
    "bemusing": "bemuse",
    "benches": "bench",
    "benchmarked": "benchmark",
    "benchmarking": "benchmark",
    "benchmarks": "benchmark",
    "bended": "bend",
    "benders": "bender",
    "bendier": "bendy",
    "bendiest": "bendy",
    "bending": "bend",
    "bends": "bend",
    "benedictions": "benediction",
    "benefactions": "benefaction",
    "benefactors": "benefactor",
    "benefactresses": "benefactress",
    "benefices": "benefice",
    "beneficiaries": "beneficiary",
    "benefited": "benefit",
    "benefiting": "benefit",
    "benefits": "benefit",
    "benefitted": "benefit",
    "benefitting": "benefit",
    "benigner": "benign",
    "benignest": "benign",
    "bent": "bend",
    "bentgrasses": "bentgrass",
    "bents": "bent",
    "benzenes": "benzene",
    "benzoates": "benzoate",
    "benzodiazepines": "benzodiazepine",
    "benzoyls": "benzoyl",
    "bequeathed": "bequeath",
    "bequeathing": "bequeath",
    "bequeaths": "bequeath",
    "bequests": "bequest",
    "berated": "berate",
    "berates": "berate",
    "berating": "berate",
    "bereaved": "bereave",
    "bereavements": "bereavement",
    "bereaves": "bereave",
    "bereaving": "bereave",
    "bereft": "bereave",
    "berets": "beret",
    "bergamots": "bergamot",
    "bergs": "berg",
    "berks": "berk",
    "berlins": "berlin",
    "berms": "berm",
    "berried": "berry",
    "berries": "berry",
    "berrying": "berry",
    "berthed": "berth",
    "berthing": "berth",
    "berths": "berth",
    "beryls": "beryl",
    "beseeched": "beseech",
    "beseeches": "beseech",
    "beseeching": "beseech",
    "besets": "beset",
    "besetting": "beset",
    "besieged": "besiege",
    "besieges": "besiege",
    "besieging": "besiege",
    "besmirched": "besmirch",
    "besmirches": "besmirch",
    "besmirching": "besmirch",
    "besoms": "besom",
    "besought": "beseech",
    "bespeaking": "bespeak",
    "bespeaks": "bespeak",
    "bespoke": "bespeak",
    "bespoken": "bespeak",
    "best": "well",
    "best-kept": "well-kept",
    "best-looking": "good-looking",
    "best-trained": "well-trained",
    "bested": "best",
    "bestiaries": "bestiary",
    "besting": "best",
    "bestirred": "bestir",
    "bestirring": "bestir",
    "bestirs": "bestir",
    "bestowed": "bestow",
    "bestowing": "bestow",
    "bestows": "bestow",
    "bestrewed": "bestrew",
    "bestrewing": "bestrew",
    "bestrewn": "bestrew",
    "bestrews": "bestrew",
    "bestrid": "bestride",
    "bestridden": "bestride",
    "bestrides": "bestride",
    "bestriding": "bestride",
    "bestrode": "bestride",
    "bests": "best",
    "bestsellers": "bestseller",
    "beta-blockers": "beta-blocker",
    "beta-carotenes": "beta-carotene",
    "beta-sheets": "beta-sheet",
    "betaines": "betaine",
    "betaken": "betake",
    "betakes": "betake",
    "betaking": "betake",
    "betas": "beta",
    "betels": "betel",
    "bethinking": "bethink",
    "bethinks": "bethink",
    "bethought": "bethink",
    "betokened": "betoken",
    "betokening": "betoken",
    "betokens": "betoken",
    "betook": "betake",
    "betrayals": "betrayal",
    "betrayed": "betray",
    "betraying": "betray",
    "betrays": "betray",
    "betrode": "bestride",
    "betrothals": "betrothal",
    "betrothed": "betroth",
    "betrothing": "betroth",
    "betroths": "betroth",
    "bets": "bet",
    "betted": "bet",
    "better": "well",
    "better-defined": "well-defined",
    "better-humoured": "good-humoured",
    "better-kept": "well-kept",
    "better-looking": "good-looking",
    "better-trained": "well-trained",
    "bettered": "better",
    "bettering": "better",
    "betters": "better",
    "betting": "bet",
    "bettings": "betting",
    "bettors": "bettor",
    "beurres": "beurre",
    "bevels": "bevel",
    "beverages": "beverage",
    "bevies": "bevy",
    "bewailed": "bewail",
    "bewailing": "bewail",
    "bewails": "bewail",
    "bewared": "beware",
    "bewares": "beware",
    "bewaring": "beware",
    "bewildered": "bewilder",
    "bewildering": "bewilder",
    "bewilderments": "bewilderment",
    "bewilders": "bewilder",
    "bewitched": "bewitch",
    "bewitches": "bewitch",
    "bewitching": "bewitch",
    "bfs": "bf",
    "bi-monthlies": "bi-monthly",
    "biased": "bias",
    "biases": "bias",
    "biasing": "bias",
    "biassed": "bias",
    "biasses": "bias",
    "biassing": "bias",
    "biathlons": "biathlon",
    "bibles": "bible",
    "bibliographers": "bibliographer",
    "bibliographies": "bibliography",
    "bibliophiles": "bibliophile",
    "bibs": "bib",
    "bicarbonates": "bicarbonate",
    "bicarbs": "bicarb",
    "bicentenaries": "bicentenary",
    "biceps": "bicep",
    "bicepses": "biceps",
    "bickered": "bicker",
    "bickering": "bicker",
    "bickers": "bicker",
    "bicycled": "bicycle",
    "bicycles": "bicycle",
    "bicycling": "bicycle",
    "bicyclists": "bicyclist",
    "bidden": "bid",
    "bidders": "bidder",
    "biddies": "biddy",
    "bidding": "bid",
    "biddings": "bidding",
    "bided": "bide",
    "bides": "bide",
    "bidets": "bidet",
    "biding": "bide",
    "bids": "bid",
    "biennials": "biennial",
    "biers": "bier",
    "biffed": "biff",
    "biffing": "biff",
    "biffs": "biff",
    "bifidobacteria": "bifidobacterium",
    "bifocals": "bifocal",
    "bifurcated": "bifurcate",
    "bifurcates": "bifurcate",
    "bifurcating": "bifurcate",
    "bifurcations": "bifurcation",
    "bigamies": "bigamy",
    "bigamists": "bigamist",
    "bigger": "big",
    "biggest": "big",
    "biggies": "biggy",
    "bights": "bight",
    "bigotries": "bigotry",
    "bigots": "bigot",
    "bigrams": "bigram",
    "bigwigs": "bigwig",
    "bijoux": "bijou",
    "bikers": "biker",
    "bikes": "bike",
    "bikinis": "bikini",
    "bilayers": "bilayer",
    "bilberries": "bilberry",
    "biles": "bile",
    "bilges": "bilge",
    "bilinguals": "bilingual",
    "bilirubins": "bilirubin",
    "bill-hooks": "bill-hook",
    "billboards": "billboard",
    "billed": "bill",
    "billers": "biller",
    "billeted": "billet",
    "billeting": "billet",
    "billets": "billet",
    "billets-doux": "billet-doux",
    "billfolds": "billfold",
    "billiards": "billiard",
    "billies": "billy",
    "billing": "bill",
    "billions": "billion",
    "billionth": "1000000000",
    "billowed": "billow",
    "billowing": "billow",
    "billows": "billow",
    "bills": "bill",
    "billy-goats": "billy-goat",
    "billys": "billy",
    "binaries": "binary",
    "binders": "binder",
    "binding": "bind",
    "bindings": "binding",
    "binds": "bind",
    "binges": "binge",
    "bingos": "bingo",
    "bings": "bing",
    "binned": "bin",
    "binning": "bin",
    "binomials": "binomial",
    "bins": "bin",
    "bio-diesels": "bio-diesel",
    "bio-fuels": "bio-fuel",
    "bio-technologies": "bio-technology",
    "bio-weapons": "bio-weapon",
    "bioaccumulations": "bioaccumulation",
    "bioassays": "bioassay",
    "bioavailabilities": "bioavailability",
    "biocatalysts": "biocatalyst",
    "biochemistries": "biochemistry",
    "biochemists": "biochemist",
    "biochips": "biochip",
    "biocides": "biocide",
    "biocontrols": "biocontrol",
    "biodegradabilities": "biodegradability",
    "biodegradations": "biodegradation",
    "biodegraded": "biodegrade",
    "biodegrades": "biodegrade",
    "biodegrading": "biodegrade",
    "biodiesels": "biodiesel",
    "bioethicists": "bioethicist",
    "bioethics": "bioethic",
    "biofilms": "biofilm",
    "bioflavonoids": "bioflavonoid",
    "biofuels": "biofuel",
    "biogases": "biogas",
    "biogeneses": "biogenesis",
    "biographers": "biographer",
    "biographies": "biography",
    "biohazards": "biohazard",
    "biologics": "biologic",
    "biologies": "biology",
    "biologists": "biologist",
    "biomarkers": "biomarker",
    "biomasses": "biomass",
    "biomaterials": "biomaterial",
    "biomedicines": "biomedicine",
    "biomes": "biome",
    "biomolecules": "biomolecule",
    "biopesticides": "biopesticide",
    "biopharmaceuticals": "biopharmaceutical",
    "biopolymers": "biopolymer",
    "bioprocesses": "bioprocess",
    "biopsies": "biopsy",
    "bioreactors": "bioreactor",
    "bioremediations": "bioremediation",
    "biorhythms": "biorhythm",
    "bios": "bio",
    "bioscientists": "bioscientist",
    "biosensors": "biosensor",
    "biosolids": "biosolid",
    "biospheres": "biosphere",
    "biosyntheses": "biosynthesis",
    "biotas": "biota",
    "biotechnologies": "biotechnology",
    "biotechnologists": "biotechnologist",
    "biotechs": "biotech",
    "bioterrorists": "bioterrorist",
    "biotins": "biotin",
    "biotites": "biotite",
    "biotopes": "biotope",
    "biotoxins": "biotoxin",
    "biotransformations": "biotransformation",
    "bioweapons": "bioweapon",
    "bipeds": "biped",
    "biphenyls": "biphenyl",
    "biplanes": "biplane",
    "bipods": "bipod",
    "birched": "birch",
    "birches": "birch",
    "birching": "birch",
    "birdcages": "birdcage",
    "birders": "birder",
    "birdies": "birdie",
    "birds": "bird",
    "birdsongs": "birdsong",
    "birth-rates": "birth-rate",
    "birthdates": "birthdate",
    "birthdays": "birthday",
    "birthmarks": "birthmark",
    "birthplaces": "birthplace",
    "birthrates": "birthrate",
    "birthrights": "birthright",
    "births": "birth",
    "birthweights": "birthweight",
    "biscuits": "biscuit",
    "bisected": "bisect",
    "bisecting": "bisect",
    "bisections": "bisection",
    "bisects": "bisect",
    "bisexuals": "bisexual",
    "bishoprics": "bishopric",
    "bishops": "bishop",
    "bismuths": "bismuth",
    "bisons": "bison",
    "bisphosphonates": "bisphosphonate",
    "bistros": "bistro",
    "bit": "bite",
    "bitched": "bitch",
    "bitches": "bitch",
    "bitchier": "bitchy",
    "bitchiest": "bitchy",
    "bitching": "bitch",
    "biters": "biter",
    "bites": "bite",
    "biting": "bite",
    "bits": "bit",
    "bitten": "bite",
    "bitterer": "bitter",
    "bitterest": "bitter",
    "bitterns": "bittern",
    "bitters": "bitter",
    "bittier": "bitty",
    "bittiest": "bitty",
    "bitumens": "bitumen",
    "bivalves": "bivalve",
    "bivouaced": "bivouac",
    "bivouacing": "bivouac",
    "bivouacked": "bivouac",
    "bivouacking": "bivouac",
    "bivouacs": "bivouac",
    "bizarrer": "bizarre",
    "bizarrest": "bizarre",
    "blabbed": "blab",
    "blabbered": "blabber",
    "blabbering": "blabber",
    "blabbermouths": "blabbermouth",
    "blabbers": "blabber",
    "blabbing": "blab",
    "blabs": "blab",
    "black-marketeers": "black-marketeer",
    "black-outs": "black-out",
    "blackballed": "blackball",
    "blackballing": "blackball",
    "blackballs": "blackball",
    "blackberries": "blackberry",
    "blackbirds": "blackbird",
    "blackboards": "blackboard",
    "blackcaps": "blackcap",
    "blackcurrants": "blackcurrant",
    "blacked": "black",
    "blackened": "blacken",
    "blackening": "blacken",
    "blackens": "blacken",
    "blacker": "black",
    "blackest": "black",
    "blackflies": "blackfly",
    "blackguards": "blackguard",
    "blackheads": "blackhead",
    "blacking": "black",
    "blackjacks": "blackjack",
    "blacklegs": "blackleg",
    "blacklisted": "blacklist",
    "blacklisting": "blacklist",
    "blacklists": "blacklist",
    "blackmailed": "blackmail",
    "blackmailers": "blackmailer",
    "blackmailing": "blackmail",
    "blackmails": "blackmail",
    "blackouts": "blackout",
    "blacks": "black",
    "blacksmiths": "blacksmith",
    "blackspots": "blackspot",
    "blackthorns": "blackthorn",
    "bladders": "bladder",
    "blades": "blade",
    "blamed": "blame",
    "blames": "blame",
    "blaming": "blame",
    "blanched": "blanch",
    "blanches": "blanch",
    "blanching": "blanch",
    "blancmanges": "blancmange",
    "blander": "bland",
    "blandest": "bland",
    "blandishments": "blandishment",
    "blanked": "blank",
    "blanker": "blank",
    "blankest": "blank",
    "blanketed": "blanket",
    "blanketing": "blanket",
    "blankets": "blanket",
    "blanking": "blank",
    "blanks": "blank",
    "blared": "blare",
    "blares": "blare",
    "blaring": "blare",
    "blasphemed": "blaspheme",
    "blasphemes": "blaspheme",
    "blasphemies": "blasphemy",
    "blaspheming": "blaspheme",
    "blast-offs": "blast-off",
    "blasted": "blast",
    "blasters": "blaster",
    "blasting": "blast",
    "blastocysts": "blastocyst",
    "blasts": "blast",
    "blathered": "blather",
    "blathering": "blather",
    "blathers": "blather",
    "blats": "blat",
    "blatted": "blat",
    "blatting": "blat",
    "blazed": "blaze",
    "blazers": "blazer",
    "blazes": "blaze",
    "blazing": "blaze",
    "blazoned": "blazon",
    "blazoning": "blazon",
    "blazons": "blazon",
    "bleached": "bleach",
    "bleachers": "bleacher",
    "bleaches": "bleach",
    "bleaching": "bleach",
    "bleaker": "bleak",
    "bleakest": "bleak",
    "blearier": "bleary",
    "bleariest": "bleary",
    "bleated": "bleat",
    "bleating": "bleat",
    "bleats": "bleat",
    "bled": "bleed",
    "bleeders": "bleeder",
    "bleeding": "bleed",
    "bleedings": "bleeding",
    "bleeds": "bleed",
    "bleeped": "bleep",
    "bleepers": "bleeper",
    "bleeping": "bleep",
    "bleeps": "bleep",
    "blemished": "blemish",
    "blemishes": "blemish",
    "blemishing": "blemish",
    "blenched": "blench",
    "blenches": "blench",
    "blenching": "blench",
    "blended": "blend",
    "blenders": "blender",
    "blending": "blend",
    "blends": "blend",
    "blennies": "blenny",
    "blent": "blend",
    "bleomycins": "bleomycin",
    "blepharitides": "blepharitis",
    "blessed": "bless",
    "blesses": "bless",
    "blessing": "bless",
    "blessings": "blessing",
    "blest": "bless",
    "blethered": "blether",
    "blethering": "blether",
    "blethers": "blether",
    "blew": "blow",
    "blighted": "blight",
    "blighters": "blighter",
    "blighting": "blight",
    "blights": "blight",
    "blimps": "blimp",
    "blinded": "blind",
    "blinder": "blind",
    "blindest": "blind",
    "blindfolded": "blindfold",
    "blindfolding": "blindfold",
    "blindfolds": "blindfold",
    "blinding": "blind",
    "blindnesses": "blindness",
    "blinds": "blind",
    "blinked": "blink",
    "blinkered": "blinker",
    "blinkering": "blinker",
    "blinkers": "blinker",
    "blinking": "blink",
    "blinks": "blink",
    "blips": "blip",
    "blisses": "bliss",
    "blistered": "blister",
    "blistering": "blister",
    "blisters": "blister",
    "blither": "blithe",
    "blithest": "blithe",
    "blitzed": "blitz",
    "blitzes": "blitz",
    "blitzing": "blitz",
    "blitzkriegs": "blitzkrieg",
    "blizzards": "blizzard",
    "bloated": "bloat",
    "bloaters": "bloater",
    "bloating": "bloat",
    "bloats": "bloat",
    "blobbed": "blob",
    "blobbing": "blob",
    "blobs": "blob",
    "blockaded": "blockade",
    "blockades": "blockade",
    "blockading": "blockade",
    "blockages": "blockage",
    "blockbusters": "blockbuster",
    "blocked": "block",
    "blockers": "blocker",
    "blockheads": "blockhead",
    "blockhouses": "blockhouse",
    "blocking": "block",
    "blocks": "block",
    "blocs": "bloc",
    "blokes": "bloke",
    "blonder": "blonde",
    "blondes": "blonde",
    "blondest": "blonde",
    "blonds": "blond",
    "blood-lettings": "blood-letting",
    "blood-pressures": "blood-pressure",
    "blood-ties": "blood-tie",
    "bloodbaths": "bloodbath",
    "blooded": "blood",
    "bloodhounds": "bloodhound",
    "bloodied": "bloody",
    "bloodier": "bloody",
    "bloodies": "bloody",
    "bloodiest": "bloody",
    "blooding": "blood",
    "bloodlettings": "bloodletting",
    "bloodlines": "bloodline",
    "bloods": "blood",
    "bloodstains": "bloodstain",
    "bloodstreams": "bloodstream",
    "bloodsuckers": "bloodsucker",
    "bloodthirstier": "bloodthirsty",
    "bloodthirstiest": "bloodthirsty",
    "bloodworms": "bloodworm",
    "bloodying": "bloody",
    "bloomed": "bloom",
    "bloomers": "bloomer",
    "blooming": "bloom",
    "blooms": "bloom",
    "blossomed": "blossom",
    "blossoming": "blossom",
    "blossoms": "blossom",
    "blotches": "blotch",
    "blotchier": "blotchy",
    "blotchiest": "blotchy",
    "blots": "blot",
    "blotted": "blot",
    "blotters": "blotter",
    "blotting": "blot",
    "blouses": "blouse",
    "blow-dried": "blow-dry",
    "blow-dries": "blow-dry",
    "blow-drying": "blow-dry",
    "blow-outs": "blow-out",
    "blow-ups": "blow-up",
    "blowed": "blow",
    "blowers": "blower",
    "blowflies": "blowfly",
    "blowholes": "blowhole",
    "blowing": "blow",
    "blowlamps": "blowlamp",
    "blown": "blow",
    "blowouts": "blowout",
    "blowpipes": "blowpipe",
    "blows": "blow",
    "blowsier": "blowsy",
    "blowsiest": "blowsy",
    "blowtorches": "blowtorch",
    "blubbered": "blubber",
    "blubbering": "blubber",
    "blubbers": "blubber",
    "bludgeoned": "bludgeon",
    "bludgeoning": "bludgeon",
    "bludgeons": "bludgeon",
    "blue-prints": "blue-print",
    "bluebells": "bluebell",
    "blueberries": "blueberry",
    "bluebirds": "bluebird",
    "bluebottles": "bluebottle",
    "blued": "blue",
    "bluefins": "bluefin",
    "blueing": "blue",
    "bluenoses": "bluenose",
    "blueprints": "blueprint",
    "bluer": "blue",
    "blues": "blue",
    "bluesmen": "bluesman",
    "bluest": "blue",
    "bluestockings": "bluestocking",
    "bluestones": "bluestone",
    "bluffed": "bluff",
    "bluffer": "bluff",
    "bluffest": "bluff",
    "bluffing": "bluff",
    "bluffs": "bluff",
    "bluing": "blue",
    "blunderbusses": "blunderbuss",
    "blundered": "blunder",
    "blundering": "blunder",
    "blunders": "blunder",
    "blunted": "blunt",
    "blunter": "blunt",
    "bluntest": "blunt",
    "blunting": "blunt",
    "blunts": "blunt",
    "blurbs": "blurb",
    "blurred": "blur",
    "blurrier": "blurry",
    "blurriest": "blurry",
    "blurring": "blur",
    "blurs": "blur",
    "blurted": "blurt",
    "blurting": "blurt",
    "blurts": "blurt",
    "blushed": "blush",
    "blushers": "blusher",
    "blushes": "blush",
    "blushing": "blush",
    "blustered": "bluster",
    "blustering": "bluster",
    "blusters": "bluster",
    "bo'suns": "bo'sun",
    "board-rooms": "board-room",
    "boarded": "board",
    "boarders": "boarder",
    "boarding": "board",
    "boarding-schools": "boarding-school",
    "boardings": "boarding",
    "boardrooms": "boardroom",
    "boards": "board",
    "boardwalks": "boardwalk",
    "boars": "boar",
    "boas": "boa",
    "boasted": "boast",
    "boasting": "boast",
    "boasts": "boast",
    "boat-hooks": "boat-hook",
    "boatbuilders": "boatbuilder",
    "boated": "boat",
    "boaters": "boater",
    "boathouses": "boathouse",
    "boating": "boat",
    "boatings": "boating",
    "boatmen": "boatman",
    "boats": "boat",
    "boatswains": "boatswain",
    "bobbed": "bob",
    "bobbies": "bobby",
    "bobbing": "bob",
    "bobbins": "bobbin",
    "bobbles": "bobble",
    "bobs": "bob",
    "bobsleds": "bobsled",
    "bobsleighs": "bobsleigh",
    "bode": "bide",
    "boded": "bode",
    "bodegas": "bodega",
    "bodes": "bode",
    "bodged": "bodge",
    "bodges": "bodge",
    "bodging": "bodge",
    "bodices": "bodice",
    "bodies": "body",
    "boding": "bode",
    "bodkins": "bodkin",
    "bods": "bod",
    "bodybuilders": "bodybuilder",
    "bodyguards": "bodyguard",
    "bodysuits": "bodysuit",
    "bodyweights": "bodyweight",
    "boffins": "boffin",
    "bogeymen": "bogeyman",
    "bogeys": "bogey",
    "bogged": "bog",
    "boggier": "boggy",
    "boggiest": "boggy",
    "bogging": "bog",
    "boggled": "boggle",
    "boggles": "boggle",
    "boggling": "boggle",
    "bogies": "bogy",
    "bogs": "bog",
    "bohemians": "bohemian",
    "boiled": "boil",
    "boilers": "boiler",
    "boiling": "boil",
    "boils": "boil",
    "bolder": "bold",
    "boldest": "bold",
    "boldfaces": "boldface",
    "boldnesses": "boldness",
    "bolds": "bold",
    "boleros": "bolero",
    "boles": "bole",
    "boli": "bolus",
    "bollards": "bollard",
    "bolls": "boll",
    "bollworms": "bollworm",
    "bolstered": "bolster",
    "bolstering": "bolster",
    "bolsters": "bolster",
    "bolt-holes": "bolt-hole",
    "bolted": "bolt",
    "bolting": "bolt",
    "bolts": "bolt",
    "boluses": "bolus",
    "bomas": "boma",
    "bombarded": "bombard",
    "bombarding": "bombard",
    "bombardments": "bombardment",
    "bombards": "bombard",
    "bombed": "bomb",
    "bombers": "bomber",
    "bombing": "bomb",
    "bombings": "bombing",
    "bomblets": "bomblet",
    "bombs": "bomb",
    "bombshells": "bombshell",
    "bombsites": "bombsite",
    "boms": "bom",
    "bonanzas": "bonanza",
    "bonbons": "bonbon",
    "bonded": "bond",
    "bondholders": "bondholder",
    "bonding": "bond",
    "bondmen": "bondman",
    "bonds": "bond",
    "bondsmen": "bondsman",
    "boned": "bone",
    "bonefishes": "bonefish",
    "bones": "bone",
    "boneshakers": "boneshaker",
    "bonfires": "bonfire",
    "bongoes": "bongo",
    "bongos": "bongo",
    "bongs": "bong",
    "bonier": "bony",
    "boniest": "bony",
    "boning": "bone",
    "bonnes": "bonne",
    "bonnets": "bonnet",
    "bonnier": "bonny",
    "bonniest": "bonny",
    "bonobos": "bonobo",
    "bonsais": "bonsai",
    "bonuses": "bonus",
    "boobed": "boob",
    "boobies": "booby",
    "boobing": "boob",
    "boobs": "boob",
    "booby-trapped": "booby-trap",
    "booby-trapping": "booby-trap",
    "booby-traps": "booby-trap",
    "booed": "boo",
    "boogied": "boogie",
    "boogieing": "boogie",
    "boogies": "boogie",
    "boohooed": "boohoo",
    "boohooing": "boohoo",
    "boohoos": "boohoo",
    "booing": "boo",
    "bookbinders": "bookbinder",
    "bookbindings": "bookbinding",
    "bookcases": "bookcase",
    "booked": "book",
    "bookends": "bookend",
    "bookies": "bookie",
    "booking": "book",
    "bookings": "booking",
    "bookkeepers": "bookkeeper",
    "booklets": "booklet",
    "booklists": "booklist",
    "bookmakers": "bookmaker",
    "bookmarked": "bookmark",
    "bookmarking": "bookmark",
    "bookmarks": "bookmark",
    "bookplates": "bookplate",
    "books": "book",
    "booksellers": "bookseller",
    "bookshelfs": "bookshelf",
    "bookshelves": "bookshelf",
    "bookshops": "bookshop",
    "bookstalls": "bookstall",
    "bookstores": "bookstore",
    "bookworms": "bookworm",
    "boomed": "boom",
    "boomeranged": "boomerang",
    "boomeranging": "boomerang",
    "boomerangs": "boomerang",
    "boomers": "boomer",
    "booming": "boom",
    "booms": "boom",
    "boons": "boon",
    "boors": "boor",
    "boos": "boo",
    "boosted": "boost",
    "boosters": "booster",
    "boosting": "boost",
    "boosts": "boost",
    "booted": "boot",
    "bootees": "bootee",
    "booths": "booth",
    "booties": "bootie",
    "booting": "boot",
    "bootlaces": "bootlace",
    "bootlegged": "bootleg",
    "bootleggers": "bootlegger",
    "bootlegging": "bootleg",
    "bootlegs": "bootleg",
    "boots": "boot",
    "bootstrapped": "bootstrap",
    "bootstrapping": "bootstrap",
    "bootstraps": "bootstrap",
    "booze-ups": "booze-up",
    "boozed": "booze",
    "boozers": "boozer",
    "boozes": "booze",
    "boozier": "boozy",
    "boozing": "booze",
    "bopped": "bop",
    "bopping": "bop",
    "bops": "bop",
    "borates": "borate",
    "bordellos": "bordello",
    "bordered": "border",
    "bordering": "border",
    "borderlands": "borderland",
    "borderlines": "borderline",
    "borders": "border",
    "bords": "bord",
    "bordures": "bordure",
    "bore": "bear",
    "bore-holes": "bore-hole",
    "bored": "bore",
    "boredoms": "boredom",
    "boreholes": "borehole",
    "borers": "borer",
    "bores": "bore",
    "boring": "bore",
    "born": "bear",
    "borne": "bear",
    "boroughs": "borough",
    "borrowed": "borrow",
    "borrowers": "borrower",
    "borrowing": "borrow",
    "borrowings": "borrowing",
    "borrows": "borrow",
    "borstals": "borstal",
    "bosoms": "bosom",
    "bosons": "boson",
    "bossed": "boss",
    "bosses": "boss",
    "bossier": "bossy",
    "bossiest": "bossy",
    "bossing": "boss",
    "bosuns": "bosun",
    "botanicals": "botanical",
    "botanies": "botany",
    "botanists": "botanist",
    "botched": "botch",
    "botches": "botch",
    "botching": "botch",
    "bothered": "bother",
    "bothering": "bother",
    "bothers": "bother",
    "bots": "bot",
    "bottle-fed": "bottle-feed",
    "bottle-feeding": "bottle-feed",
    "bottle-feeds": "bottle-feed",
    "bottle-openers": "bottle-opener",
    "bottled": "bottle",
    "bottlenecks": "bottleneck",
    "bottlenoses": "bottlenose",
    "bottlers": "bottler",
    "bottles": "bottle",
    "bottling": "bottle",
    "bottomed": "bottom",
    "bottoming": "bottom",
    "bottoms": "bottom",
    "botulisms": "botulism",
    "boudoirs": "boudoir",
    "bouffants": "bouffant",
    "bougainvillaeas": "bougainvillaea",
    "boughs": "bough",
    "bought": "buy",
    "bouillons": "bouillon",
    "boulders": "boulder",
    "boulevards": "boulevard",
    "bounced": "bounce",
    "bouncers": "bouncer",
    "bounces": "bounce",
    "bouncier": "bouncy",
    "bounciest": "bouncy",
    "bouncing": "bounce",
    "bound": "bind",
    "boundaries": "boundary",
    "bounded": "bound",
    "bounders": "bounder",
    "bounding": "bound",
    "bounds": "bound",
    "bounties": "bounty",
    "bouquets": "bouquet",
    "bourbons": "bourbon",
    "bourgeoisies": "bourgeoisie",
    "bourses": "bourse",
    "boutiques": "boutique",
    "bouts": "bout",
    "bovines": "bovine",
    "bow-wows": "bow-wow",
    "bowed": "bow",
    "bowels": "bowel",
    "bowers": "bower",
    "bowing": "bow",
    "bowled": "bowl",
    "bowlers": "bowler",
    "bowling": "bowl",
    "bowls": "bowl",
    "bowmen": "bowman",
    "bows": "bow",
    "bowstrings": "bowstring",
    "boxcars": "boxcar",
    "boxed": "box",
    "boxers": "boxer",
    "boxes": "box",
    "boxing": "box",
    "boxing-gloves": "boxing-glove",
    "boxing-rings": "boxing-ring",
    "boxrooms": "boxroom",
    "boycotted": "boycott",
    "boycotting": "boycott",
    "boycotts": "boycott",
    "boyfriends": "boyfriend",
    "boyhoods": "boyhood",
    "boys": "boy",
    "boysenberries": "boysenberry",
    "bps": "bp",
    "braced": "brace",
    "bracelets": "bracelet",
    "braces": "brace",
    "brachiopods": "brachiopod",
    "brachytherapies": "brachytherapy",
    "bracing": "brace",
    "brackens": "bracken",
    "bracketed": "bracket",
    "bracketing": "bracket",
    "brackets": "bracket",
    "bracts": "bract",
    "bradycardias": "bradycardia",
    "braggarts": "braggart",
    "bragged": "brag",
    "bragging": "brag",
    "brags": "brag",
    "brahmans": "brahman",
    "braided": "braid",
    "braiding": "braid",
    "braids": "braid",
    "brain-drains": "brain-drain",
    "brained": "brain",
    "brainier": "brainy",
    "brainiest": "brainy",
    "braining": "brain",
    "brains": "brain",
    "brainstems": "brainstem",
    "brainstormed": "brainstorm",
    "brainstorming": "brainstorm",
    "brainstorms": "brainstorm",
    "brainwashed": "brainwash",
    "brainwashes": "brainwash",
    "brainwashing": "brainwash",
    "brainwaves": "brainwave",
    "braised": "braise",
    "braises": "braise",
    "braising": "braise",
    "braked": "brake",
    "brakes": "brake",
    "braking": "brake",
    "brambles": "bramble",
    "branched": "branch",
    "branches": "branch",
    "branchier": "branchy",
    "branching": "branch",
    "branded": "brand",
    "brandies": "brandy",
    "branding": "brand",
    "brandished": "brandish",
    "brandishes": "brandish",
    "brandishing": "brandish",
    "brands": "brand",
    "brans": "bran",
    "bras": "bra",
    "brasher": "brash",
    "brashest": "brash",
    "brasseries": "brasserie",
    "brasses": "brass",
    "brassicas": "brassica",
    "brassier": "brassy",
    "brassieres": "brassiere",
    "brassiest": "brassy",
    "brassières": "brassière",
    "brats": "brat",
    "bravadoes": "bravado",
    "bravados": "bravado",
    "braved": "brave",
    "braver": "brave",
    "braves": "brave",
    "bravest": "brave",
    "braving": "brave",
    "bravoes": "bravo",
    "bravos": "bravo",
    "brawled": "brawl",
    "brawling": "brawl",
    "brawls": "brawl",
    "brawnier": "brawny",
    "brawniest": "brawny",
    "brayed": "bray",
    "braying": "bray",
    "brays": "bray",
    "brazed": "braze",
    "brazened": "brazen",
    "brazening": "brazen",
    "brazens": "brazen",
    "brazes": "braze",
    "braziers": "brazier",
    "brazing": "braze",
    "breached": "breach",
    "breaches": "breach",
    "breaching": "breach",
    "bread-bins": "bread-bin",
    "bread-boards": "bread-board",
    "breadboards": "breadboard",
    "breadcrumbs": "breadcrumb",
    "breadcrusts": "breadcrust",
    "breadfruits": "breadfruit",
    "breads": "bread",
    "breadths": "breadth",
    "breadwinners": "breadwinner",
    "break-downs": "break-down",
    "break-ins": "break-in",
    "break-throughs": "break-through",
    "break-ups": "break-up",
    "breakages": "breakage",
    "breakaways": "breakaway",
    "breakdowns": "breakdown",
    "breakers": "breaker",
    "breakfasted": "breakfast",
    "breakfasting": "breakfast",
    "breakfasts": "breakfast",
    "breaking": "break",
    "breakouts": "breakout",
    "breakpoints": "breakpoint",
    "breaks": "break",
    "breakthroughs": "breakthrough",
    "breakups": "breakup",
    "breakwaters": "breakwater",
    "breams": "bream",
    "breast-fed": "breast-feed",
    "breast-feeding": "breast-feed",
    "breast-feeds": "breast-feed",
    "breast-strokes": "breast-stroke",
    "breastbones": "breastbone",
    "breasted": "breast",
    "breastfed": "breastfeed",
    "breastfeeding": "breastfeed",
    "breastfeeds": "breastfeed",
    "breasting": "breast",
    "breastplates": "breastplate",
    "breasts": "breast",
    "breathalysers": "breathalyser",
    "breathalyzed": "breathalyze",
    "breathalyzes": "breathalyze",
    "breathalyzing": "breathalyze",
    "breathed": "breathe",
    "breathers": "breather",
    "breathes": "breathe",
    "breathing": "breathe",
    "breathings": "breathing",
    "breaths": "breath",
    "breccias": "breccia",
    "bred": "breed",
    "breeches": "breech",
    "breeders": "breeder",
    "breeding": "breed",
    "breedings": "breeding",
    "breeds": "breed",
    "breeze-blocks": "breeze-block",
    "breezed": "breeze",
    "breezes": "breeze",
    "breezier": "breezy",
    "breeziest": "breezy",
    "breezing": "breeze",
    "brethren": "brother",
    "breves": "breve",
    "brevets": "brevet",
    "brewed": "brew",
    "breweries": "brewery",
    "brewers": "brewer",
    "brewing": "brew",
    "brews": "brew",
    "briars": "briar",
    "bribed": "bribe",
    "bribes": "bribe",
    "bribing": "bribe",
    "bricked": "brick",
    "bricking": "brick",
    "bricklayers": "bricklayer",
    "brickmakers": "brickmaker",
    "bricks": "brick",
    "brickworks": "brickwork",
    "bridegrooms": "bridegroom",
    "brides": "bride",
    "brides-to-be": "bride-to-be",
    "bridesmaids": "bridesmaid",
    "bridged": "bridge",
    "bridgeheads": "bridgehead",
    "bridges": "bridge",
    "bridging": "bridge",
    "bridled": "bridle",
    "bridles": "bridle",
    "bridleways": "bridleway",
    "bridling": "bridle",
    "briefcases": "briefcase",
    "briefed": "brief",
    "briefer": "brief",
    "briefest": "brief",
    "briefing": "brief",
    "briefings": "briefing",
    "briefs": "brief",
    "briers": "brier",
    "brigades": "brigade",
    "brigadiers": "brigadier",
    "brigands": "brigand",
    "brightened": "brighten",
    "brightening": "brighten",
    "brightens": "brighten",
    "brighter": "bright",
    "brightest": "bright",
    "brightnesses": "brightness",
    "brigs": "brig",
    "brimmed": "brim",
    "brimming": "brim",
    "brims": "brim",
    "brimstones": "brimstone",
    "brines": "brine",
    "bringers": "bringer",
    "bringing": "bring",
    "brings": "bring",
    "brinier": "briny",
    "brinks": "brink",
    "briquettes": "briquette",
    "brisker": "brisk",
    "briskest": "brisk",
    "bristled": "bristle",
    "bristles": "bristle",
    "bristlier": "bristly",
    "bristliest": "bristly",
    "bristling": "bristle",
    "brittler": "brittle",
    "brittlest": "brittle",
    "broached": "broach",
    "broaches": "broach",
    "broaching": "broach",
    "broadcasted": "broadcast",
    "broadcasters": "broadcaster",
    "broadcasting": "broadcast",
    "broadcastings": "broadcasting",
    "broadcasts": "broadcast",
    "broadened": "broaden",
    "broadening": "broaden",
    "broadens": "broaden",
    "broader": "broad",
    "broadest": "broad",
    "broadleafs": "broadleaf",
    "broadleaves": "broadleaf",
    "broads": "broad",
    "broadsheets": "broadsheet",
    "broadsides": "broadside",
    "brocades": "brocade",
    "brochures": "brochure",
    "brogues": "brogue",
    "broiled": "broil",
    "broilers": "broiler",
    "broiling": "broil",
    "broils": "broil",
    "broke": "break",
    "broken": "break",
    "broker": "broke",
    "brokerages": "brokerage",
    "brokered": "broker",
    "brokering": "broker",
    "brokers": "broker",
    "brokest": "broke",
    "brollies": "brolly",
    "bromelains": "bromelain",
    "bromeliads": "bromeliad",
    "bromes": "brome",
    "bromides": "bromide",
    "brominated": "brominate",
    "brominates": "brominate",
    "brominating": "brominate",
    "bromines": "bromine",
    "bronchi": "bronchus",
    "bronchioles": "bronchiole",
    "bronchiolitides": "bronchiolitis",
    "bronchitides": "bronchitis",
    "bronchodilators": "bronchodilator",
    "bronchopneumoniae": "bronchopneumonia",
    "bronchopneumonias": "bronchopneumonia",
    "bronchoscopies": "bronchoscopy",
    "bronchospasms": "bronchospasm",
    "broncos": "bronco",
    "brontosauri": "brontosaurus",
    "brontosauruses": "brontosaurus",
    "bronzed": "bronze",
    "bronzer": "bronze",
    "bronzes": "bronze",
    "bronzest": "bronze",
    "bronzing": "bronze",
    "brooches": "brooch",
    "brooded": "brood",
    "brooders": "brooder",
    "broodier": "broody",
    "broodiest": "broody",
    "brooding": "brood",
    "broodings": "brooding",
    "broods": "brood",
    "brooked": "brook",
    "brooking": "brook",
    "brooks": "brook",
    "brooms": "broom",
    "broomsticks": "broomstick",
    "brothels": "brothel",
    "brotherhoods": "brotherhood",
    "brothers": "brother",
    "brothers-in-law": "brother-in-law",
    "broths": "broth",
    "broughams": "brougham",
    "brought": "bring",
    "brouhahas": "brouhaha",
    "browbeaten": "browbeat",
    "browbeating": "browbeat",
    "browbeats": "browbeat",
    "browned": "brown",
    "browner": "brown",
    "brownest": "brown",
    "brownies": "brownie",
    "browning": "brown",
    "browns": "brown",
    "brownstones": "brownstone",
    "brows": "brow",
    "browsed": "browse",
    "browsers": "browser",
    "browses": "browse",
    "browsing": "browse",
    "brucelloses": "brucellosis",
    "bruised": "bruise",
    "bruisers": "bruiser",
    "bruises": "bruise",
    "bruising": "bruise",
    "brunches": "brunch",
    "brunettes": "brunette",
    "brunts": "brunt",
    "brush-offs": "brush-off",
    "brushed": "brush",
    "brushes": "brush",
    "brushing": "brush",
    "brushwoods": "brushwood",
    "brutalised": "brutalise",
    "brutalises": "brutalise",
    "brutalising": "brutalise",
    "brutalities": "brutality",
    "brutalized": "brutalize",
    "brutalizes": "brutalize",
    "brutalizing": "brutalize",
    "brutes": "brute",
    "bryophytes": "bryophyte",
    "bryozoans": "bryozoan",
    "bs": "b",
    "bubbled": "bubble",
    "bubbles": "bubble",
    "bubblier": "bubbly",
    "bubbliest": "bubbly",
    "bubbling": "bubble",
    "buboes": "bubo",
    "buccaneers": "buccaneer",
    "bucked": "buck",
    "bucketed": "bucket",
    "bucketfuls": "bucketful",
    "bucketing": "bucket",
    "buckets": "bucket",
    "bucketsful": "bucketful",
    "bucking": "buck",
    "buckled": "buckle",
    "buckles": "buckle",
    "buckling": "buckle",
    "bucks": "buck",
    "buckskins": "buckskin",
    "buckthorns": "buckthorn",
    "budded": "bud",
    "buddied": "buddy",
    "buddies": "buddy",
    "budding": "bud",
    "buddleias": "buddleia",
    "buddying": "buddy",
    "budged": "budge",
    "budgerigars": "budgerigar",
    "budges": "budge",
    "budgeted": "budget",
    "budgeting": "budget",
    "budgets": "budget",
    "budgies": "budgie",
    "budging": "budge",
    "buds": "bud",
    "buffaloed": "buffalo",
    "buffaloes": "buffalo",
    "buffaloing": "buffalo",
    "buffalos": "buffalo",
    "buffed": "buff",
    "buffer": "buff",
    "buffered": "buffer",
    "buffering": "buffer",
    "buffers": "buffer",
    "buffest": "buff",
    "buffeted": "buffet",
    "buffeting": "buffet",
    "buffets": "buffet",
    "buffing": "buff",
    "buffoons": "buffoon",
    "buffs": "buff",
    "bugbears": "bugbear",
    "bugged": "bug",
    "buggered": "bugger",
    "buggering": "bugger",
    "buggers": "bugger",
    "buggies": "buggy",
    "bugging": "bug",
    "buglers": "bugler",
    "bugles": "bugle",
    "bugs": "bug",
    "build-ups": "build-up",
    "builders": "builder",
    "building": "build",
    "buildings": "building",
    "builds": "build",
    "buildups": "buildup",
    "built": "build",
    "built-ins": "built-in",
    "bulbouser": "bulbous",
    "bulbousest": "bulbous",
    "bulbs": "bulb",
    "bulbuls": "bulbul",
    "bulged": "bulge",
    "bulges": "bulge",
    "bulging": "bulge",
    "bulked": "bulk",
    "bulkheads": "bulkhead",
    "bulkier": "bulky",
    "bulkiest": "bulky",
    "bulking": "bulk",
    "bulks": "bulk",
    "bull's-eyes": "bull's-eye",
    "bullae": "bulla",
    "bulldogs": "bulldog",
    "bulldozed": "bulldoze",
    "bulldozers": "bulldozer",
    "bulldozes": "bulldoze",
    "bulldozing": "bulldoze",
    "bulled": "bull",
    "bulletins": "bulletin",
    "bullets": "bullet",
    "bullfighters": "bullfighter",
    "bullfights": "bullfight",
    "bullfinches": "bullfinch",
    "bullfrogs": "bullfrog",
    "bullheads": "bullhead",
    "bullied": "bully",
    "bullies": "bully",
    "bulling": "bull",
    "bullocks": "bullock",
    "bullrings": "bullring",
    "bulls": "bull",
    "bullseyes": "bullseye",
    "bullshits": "bullshit",
    "bullshitted": "bullshit",
    "bullshitting": "bullshit",
    "bullying": "bully",
    "bulrushes": "bulrush",
    "bulwarks": "bulwark",
    "bumblebees": "bumblebee",
    "bumbled": "bumble",
    "bumbles": "bumble",
    "bumbling": "bumble",
    "bummed": "bum",
    "bummers": "bummer",
    "bumming": "bum",
    "bumped": "bump",
    "bumpers": "bumper",
    "bumpier": "bumpy",
    "bumpiest": "bumpy",
    "bumping": "bump",
    "bumpkins": "bumpkin",
    "bumps": "bump",
    "bums": "bum",
    "bunched": "bunch",
    "bunches": "bunch",
    "bunching": "bunch",
    "bunde": "bund",
    "bundled": "bundle",
    "bundles": "bundle",
    "bundling": "bundle",
    "bungalows": "bungalow",
    "bunged": "bung",
    "bungees": "bungee",
    "bunging": "bung",
    "bungled": "bungle",
    "bunglers": "bungler",
    "bungles": "bungle",
    "bungling": "bungle",
    "bungs": "bung",
    "bunions": "bunion",
    "bunked": "bunk",
    "bunkered": "bunker",
    "bunkering": "bunker",
    "bunkers": "bunker",
    "bunking": "bunk",
    "bunks": "bunk",
    "bunnies": "bunny",
    "buns": "bun",
    "buntings": "bunting",
    "bunts": "bunt",
    "buoyancies": "buoyancy",
    "buoyed": "buoy",
    "buoying": "buoy",
    "buoys": "buoy",
    "burbled": "burble",
    "burbles": "burble",
    "burbling": "burble",
    "burdened": "burden",
    "burdening": "burden",
    "burdens": "burden",
    "burdocks": "burdock",
    "bureaucracies": "bureaucracy",
    "bureaucratised": "bureaucratise",
    "bureaucratises": "bureaucratise",
    "bureaucratising": "bureaucratise",
    "bureaucrats": "bureaucrat",
    "bureaus": "bureau",
    "bureaux": "bureau",
    "burettes": "burette",
    "burgeoned": "burgeon",
    "burgeoning": "burgeon",
    "burgeons": "burgeon",
    "burgesses": "burgess",
    "burghers": "burgher",
    "burglaries": "burglary",
    "burglarized": "burglarize",
    "burglarizes": "burglarize",
    "burglarizing": "burglarize",
    "burglars": "burglar",
    "burgled": "burgle",
    "burgles": "burgle",
    "burgling": "burgle",
    "burials": "burial",
    "buried": "bury",
    "buries": "bury",
    "burkas": "burka",
    "burlesques": "burlesque",
    "burlier": "burly",
    "burliest": "burly",
    "burned": "burn",
    "burned-out": "burnt-out",
    "burners": "burner",
    "burnets": "burnet",
    "burning": "burn",
    "burnings": "burning",
    "burnished": "burnish",
    "burnishes": "burnish",
    "burnishing": "burnish",
    "burnouts": "burnout",
    "burns": "burn",
    "burnt": "burn",
    "burped": "burp",
    "burping": "burp",
    "burps": "burp",
    "burred": "burr",
    "burring": "burr",
    "burritos": "burrito",
    "burros": "burro",
    "burrowed": "burrow",
    "burrowing": "burrow",
    "burrows": "burrow",
    "burrs": "burr",
    "burs": "bur",
    "bursae": "bursa",
    "bursaries": "bursary",
    "bursars": "bursar",
    "bursas": "bursa",
    "bursitides": "bursitis",
    "bursting": "burst",
    "bursts": "burst",
    "burying": "bury",
    "bus-shelters": "bus-shelter",
    "bus-stops": "bus-stop",
    "busbies": "busby",
    "bused": "bus",
    "buses": "bus",
    "bushbabies": "bushbaby",
    "bushels": "bushel",
    "bushes": "bush",
    "bushier": "bushy",
    "bushiest": "bushy",
    "bushings": "bushing",
    "busied": "busy",
    "busier": "busy",
    "busies": "busy",
    "busiest": "busy",
    "businesses": "business",
    "businessmen": "businessman",
    "businesswomen": "businesswoman",
    "busing": "bus",
    "busked": "busk",
    "buskers": "busker",
    "busking": "busk",
    "busks": "busk",
    "bussed": "bus",
    "busses": "bus",
    "bussing": "bus",
    "bust-ups": "bust-up",
    "bustards": "bustard",
    "busted": "bust",
    "busters": "buster",
    "bustier": "busty",
    "bustiest": "busty",
    "busting": "bust",
    "bustled": "bustle",
    "bustles": "bustle",
    "bustling": "bustle",
    "busts": "bust",
    "busybodies": "busybody",
    "busying": "busy",
    "butadienes": "butadiene",
    "butanes": "butane",
    "butchered": "butcher",
    "butcheries": "butchery",
    "butchering": "butcher",
    "butchers": "butcher",
    "butches": "butch",
    "butlers": "butler",
    "buts": "but",
    "butted": "butt",
    "butter-dishes": "butter-dish",
    "buttercups": "buttercup",
    "buttered": "butter",
    "butterfats": "butterfat",
    "butterflies": "butterfly",
    "butteries": "buttery",
    "buttering": "butter",
    "buttermilks": "buttermilk",
    "butternuts": "butternut",
    "butters": "butter",
    "butterscotches": "butterscotch",
    "butting": "butt",
    "buttocks": "buttock",
    "buttoned": "button",
    "buttonholed": "buttonhole",
    "buttonholes": "buttonhole",
    "buttonholing": "buttonhole",
    "buttoning": "button",
    "buttons": "button",
    "buttressed": "buttress",
    "buttresses": "buttress",
    "buttressing": "buttress",
    "butts": "butt",
    "butyls": "butyl",
    "butyrates": "butyrate",
    "buxomer": "buxom",
    "buxomest": "buxom",
    "buyers": "buyer",
    "buying": "buy",
    "buys": "buy",
    "buzz-words": "buzz-word",
    "buzzards": "buzzard",
    "buzzed": "buzz",
    "buzzers": "buzzer",
    "buzzes": "buzz",
    "buzzing": "buzz",
    "buzzwords": "buzzword",
    "bvs": "bv",
    "by-elections": "by-election",
    "by-laws": "by-law",
    "by-lines": "by-line",
    "by-passed": "by-pass",
    "by-passes": "by-pass",
    "by-passing": "by-pass",
    "by-products": "by-product",
    "bye-laws": "bye-law",
    "byelaws": "byelaw",
    "bygones": "bygone",
    "bylaws": "bylaw",
    "bylines": "byline",
    "bypassed": "bypass",
    "bypasses": "bypass",
    "bypassing": "bypass",
    "byproducts": "byproduct",
    "byres": "byre",
    "bystanders": "bystander",
    "bytes": "byte",
    "byways": "byway",
    "bywords": "byword",
    "c's": "c",
    "cabals": "cabal",
    "cabanas": "cabana",
    "cabarets": "cabaret",
    "cabbages": "cabbage",
    "cabbies": "cabbie",
    "cabernets": "cabernet",
    "cabinet-makers": "cabinet-maker",
    "cabinetmakers": "cabinetmaker",
    "cabinets": "cabinet",
    "cabins": "cabin",
    "cable-cars": "cable-car",
    "cabled": "cable",
    "cablegrams": "cablegram",
    "cables": "cable",
    "cabling": "cable",
    "cabmen": "cabman",
    "cabs": "cab",
    "cacaos": "cacao",
    "cached": "cache",
    "caches": "cache",
    "cachets": "cachet",
    "cachexias": "cachexia",
    "cachexies": "cachexia",
    "caching": "cache",
    "cackled": "cackle",
    "cackles": "cackle",
    "cackling": "cackle",
    "cacophonies": "cacophony",
    "cacti": "cactus",
    "cactuses": "cactus",
    "cadavera": "cadaver",
    "cadavers": "cadaver",
    "caddied": "caddie",
    "caddies": "caddy",
    "caddises": "caddis",
    "caddying": "caddie",
    "cadences": "cadence",
    "cadenzas": "cadenza",
    "cadets": "cadet",
    "cadged": "cadge",
    "cadgers": "cadger",
    "cadges": "cadge",
    "cadging": "cadge",
    "cadres": "cadre",
    "cads": "cad",
    "caeca": "caecum",
    "caecums": "caecum",
    "caerulei": "caeruleus",
    "caesars": "caesar",
    "caesurae": "caesura",
    "caesuras": "caesura",
    "cafes": "cafe",
    "cafeterias": "cafeteria",
    "caftans": "caftan",
    "cafés": "café",
    "caged": "cage",
    "cages": "cage",
    "cageyer": "cagey",
    "cageyest": "cagey",
    "caging": "cage",
    "cagoules": "cagoule",
    "caimans": "caiman",
    "cairns": "cairn",
    "caissons": "caisson",
    "cajoled": "cajole",
    "cajoles": "cajole",
    "cajoling": "cajole",
    "caked": "cake",
    "cakes": "cake",
    "caking": "cake",
    "calabashes": "calabash",
    "calabreses": "calabrese",
    "calamities": "calamity",
    "calcifications": "calcification",
    "calcified": "calcify",
    "calcifies": "calcify",
    "calcifying": "calcify",
    "calcined": "calcine",
    "calcines": "calcine",
    "calcining": "calcine",
    "calcites": "calcite",
    "calcitonins": "calcitonin",
    "calciums": "calcium",
    "calcs": "calc",
    "calculated": "calculate",
    "calculates": "calculate",
    "calculating": "calculate",
    "calculations": "calculation",
    "calculators": "calculator",
    "calculi": "calculus",
    "calculuses": "calculus",
    "calderas": "caldera",
    "calendared": "calendar",
    "calendaring": "calendar",
    "calendars": "calendar",
    "calenders": "calender",
    "calendulas": "calendula",
    "calfs": "calf",
    "calibers": "caliber",
    "calibrated": "calibrate",
    "calibrates": "calibrate",
    "calibrating": "calibrate",
    "calibrations": "calibration",
    "calibrators": "calibrator",
    "calibres": "calibre",
    "caliciviruses": "calicivirus",
    "calicoes": "calico",
    "calicos": "calico",
    "califs": "calif",
    "calipered": "caliper",
    "calipering": "caliper",
    "calipers": "caliper",
    "caliphates": "caliphate",
    "caliphs": "caliph",
    "call-girls": "call-girl",
    "call-ups": "call-up",
    "callboxes": "callbox",
    "called": "call",
    "callers": "caller",
    "calli": "callus",
    "calligraphers": "calligrapher",
    "calling": "call",
    "callings": "calling",
    "callipers": "calliper",
    "callosa": "callosum",
    "calls": "call",
    "calluses": "callus",
    "calmed": "calm",
    "calmer": "calm",
    "calmest": "calm",
    "calming": "calm",
    "calmodulins": "calmodulin",
    "calms": "calm",
    "calories": "calory",
    "calorifiers": "calorifier",
    "calorimeters": "calorimeter",
    "calorimetries": "calorimetry",
    "cals": "cal",
    "calumnies": "calumny",
    "calved": "calve",
    "calves": "calve",
    "calving": "calve",
    "calyces": "calyx",
    "calypsos": "calypso",
    "calyxes": "calyx",
    "cambered": "camber",
    "cambering": "camber",
    "cambers": "camber",
    "camcorders": "camcorder",
    "came": "come",
    "camelids": "camelid",
    "camellias": "camellia",
    "camels": "camel",
    "cameos": "cameo",
    "camerae": "camera",
    "cameramen": "cameraman",
    "cameras": "camera",
    "camisoles": "camisole",
    "camomiles": "camomile",
    "camouflaged": "camouflage",
    "camouflages": "camouflage",
    "camouflaging": "camouflage",
    "campaigned": "campaign",
    "campaigners": "campaigner",
    "campaigning": "campaign",
    "campaigns": "campaign",
    "campaniles": "campanile",
    "campbeds": "campbed",
    "camped": "camp",
    "campers": "camper",
    "campfires": "campfire",
    "campgrounds": "campground",
    "camping": "camp",
    "campions": "campion",
    "camps": "camp",
    "campsites": "campsite",
    "campuses": "campus",
    "campylobacters": "campylobacter",
    "cams": "cam",
    "camshafts": "camshaft",
    "can-cans": "can-can",
    "can-openers": "can-opener",
    "canalisations": "canalisation",
    "canalised": "canalise",
    "canalises": "canalise",
    "canalising": "canalise",
    "canalized": "canalize",
    "canalizes": "canalize",
    "canalizing": "canalize",
    "canals": "canal",
    "canapes": "canape",
    "canards": "canard",
    "canaries": "canary",
    "canceled": "cancel",
    "canceling": "cancel",
    "cancellations": "cancellation",
    "cancelled": "cancel",
    "cancelling": "cancel",
    "cancels": "cancel",
    "cancers": "cancer",
    "candelabra": "candelabrum",
    "candelabras": "candelabra",
    "candelabrum": "candelabra",
    "candelabrums": "candelabrum",
    "candidacies": "candidacy",
    "candidae": "candida",
    "candidates": "candidate",
    "candidatures": "candidature",
    "candidiases": "candidiasis",
    "candies": "candy",
    "candled": "candle",
    "candles": "candle",
    "candlesticks": "candlestick",
    "candling": "candle",
    "caned": "cane",
    "canes": "cane",
    "canines": "canine",
    "caning": "cane",
    "canisters": "canister",
    "cankers": "canker",
    "cannabinoids": "cannabinoid",
    "cannas": "canna",
    "canned": "can",
    "canneries": "cannery",
    "cannibalised": "cannibalise",
    "cannibalises": "cannibalise",
    "cannibalising": "cannibalise",
    "cannibalized": "cannibalize",
    "cannibalizes": "cannibalize",
    "cannibalizing": "cannibalize",
    "cannibals": "cannibal",
    "cannier": "canny",
    "canniest": "canny",
    "canning": "can",
    "cannisters": "cannister",
    "cannonades": "cannonade",
    "cannonballs": "cannonball",
    "cannoned": "cannon",
    "cannoning": "cannon",
    "cannons": "cannon",
    "cannulae": "cannula",
    "cannulas": "cannula",
    "cannulations": "cannulation",
    "canoed": "canoe",
    "canoeing": "canoe",
    "canoeists": "canoeist",
    "canoes": "canoe",
    "canonisations": "canonisation",
    "canonised": "canonise",
    "canonises": "canonise",
    "canonising": "canonise",
    "canonizations": "canonization",
    "canonized": "canonize",
    "canonizes": "canonize",
    "canonizing": "canonize",
    "canonries": "canonry",
    "canons": "canon",
    "canoodled": "canoodle",
    "canoodles": "canoodle",
    "canoodling": "canoodle",
    "canopied": "canopy",
    "canopies": "canopy",
    "canopying": "canopy",
    "cans": "can",
    "cantaloupes": "cantaloupe",
    "cantatas": "cantata",
    "canted": "cant",
    "canteens": "canteen",
    "cantered": "canter",
    "cantering": "canter",
    "canters": "canter",
    "canticles": "canticle",
    "cantilevered": "cantilever",
    "cantilevering": "cantilever",
    "cantilevers": "cantilever",
    "canting": "cant",
    "cantonments": "cantonment",
    "cantons": "canton",
    "cantors": "cantor",
    "cantos": "canto",
    "cants": "cant",
    "canvases": "canvas",
    "canvassed": "canvass",
    "canvassers": "canvasser",
    "canvasses": "canvass",
    "canvassing": "canvass",
    "canyons": "canyon",
    "capabilities": "capability",
    "capacitances": "capacitance",
    "capacities": "capacity",
    "capacitors": "capacitor",
    "capercaillies": "capercaillie",
    "capered": "caper",
    "capering": "caper",
    "capers": "caper",
    "capes": "cape",
    "capillaries": "capillary",
    "capita": "caput",
    "capitalisations": "capitalisation",
    "capitalised": "capitalise",
    "capitalises": "capitalise",
    "capitalising": "capitalise",
    "capitalists": "capitalist",
    "capitalizations": "capitalization",
    "capitalized": "capitalize",
    "capitalizes": "capitalize",
    "capitalizing": "capitalize",
    "capitals": "capital",
    "capitols": "capitol",
    "capitulated": "capitulate",
    "capitulates": "capitulate",
    "capitulating": "capitulate",
    "capitulations": "capitulation",
    "capons": "capon",
    "capped": "cap",
    "capping": "cap",
    "cappings": "capping",
    "cappuccinos": "cappuccino",
    "caprices": "caprice",
    "caps": "cap",
    "capsicums": "capsicum",
    "capsids": "capsid",
    "capsized": "capsize",
    "capsizes": "capsize",
    "capsizing": "capsize",
    "capstans": "capstan",
    "capsules": "capsule",
    "captaincies": "captaincy",
    "captained": "captain",
    "captaining": "captain",
    "captains": "captain",
    "captioned": "caption",
    "captioning": "caption",
    "captions": "caption",
    "captivated": "captivate",
    "captivates": "captivate",
    "captivating": "captivate",
    "captives": "captive",
    "captors": "captor",
    "captured": "capture",
    "captures": "capture",
    "capturing": "capture",
    "capuchins": "capuchin",
    "car-ferries": "car-ferry",
    "carafes": "carafe",
    "caramelised": "caramelise",
    "caramelises": "caramelise",
    "caramelising": "caramelise",
    "caramels": "caramel",
    "carapaces": "carapace",
    "carats": "carat",
    "caravaned": "caravan",
    "caravaning": "caravan",
    "caravanned": "caravan",
    "caravanning": "caravan",
    "caravans": "caravan",
    "caravanserais": "caravanserai",
    "caraways": "caraway",
    "carbamates": "carbamate",
    "carbides": "carbide",
    "carbines": "carbine",
    "carbocations": "carbocation",
    "carbohydrates": "carbohydrate",
    "carbonated": "carbonate",
    "carbonates": "carbonate",
    "carbonating": "carbonate",
    "carbonised": "carbonise",
    "carbonises": "carbonise",
    "carbonising": "carbonise",
    "carbons": "carbon",
    "carbonyls": "carbonyl",
    "carboxylates": "carboxylate",
    "carboxyls": "carboxyl",
    "carbuncles": "carbuncle",
    "carburetors": "carburetor",
    "carburettors": "carburettor",
    "carcases": "carcase",
    "carcasses": "carcass",
    "carcinogeneses": "carcinogenesis",
    "carcinogenicities": "carcinogenicity",
    "carcinogens": "carcinogen",
    "carcinoids": "carcinoid",
    "carcinomas": "carcinoma",
    "carcinomata": "carcinoma",
    "cardamoms": "cardamom",
    "cardboards": "cardboard",
    "carded": "card",
    "cardholders": "cardholder",
    "cardigans": "cardigan",
    "cardinals": "cardinal",
    "carding": "card",
    "cardiologies": "cardiology",
    "cardiologists": "cardiologist",
    "cardiomyopathies": "cardiomyopathy",
    "cardioversions": "cardioversion",
    "cards": "card",
    "care-givers": "care-giver",
    "cared": "care",
    "careened": "careen",
    "careening": "careen",
    "careens": "careen",
    "careered": "career",
    "careering": "career",
    "careerists": "careerist",
    "careers": "career",
    "caregivers": "caregiver",
    "carelessnesses": "carelessness",
    "carers": "carer",
    "cares": "care",
    "caressed": "caress",
    "caresses": "caress",
    "caressing": "caress",
    "caretakers": "caretaker",
    "carets": "caret",
    "cargoes": "cargo",
    "cargos": "cargo",
    "caribbeans": "caribbean",
    "caribous": "caribou",
    "caricatured": "caricature",
    "caricatures": "caricature",
    "caricaturing": "caricature",
    "caricaturists": "caricaturist",
    "carillons": "carillon",
    "caring": "care",
    "carnations": "carnation",
    "carnes": "caro",
    "carnitines": "carnitine",
    "carnivals": "carnival",
    "carnivores": "carnivore",
    "carobs": "carob",
    "caroled": "carol",
    "carolines": "caroline",
    "caroling": "carol",
    "carolled": "carol",
    "carolling": "carol",
    "carols": "carol",
    "carotenes": "carotene",
    "carotenoids": "carotenoid",
    "caroused": "carouse",
    "carousels": "carousel",
    "carouses": "carouse",
    "carousing": "carouse",
    "carpals": "carpal",
    "carped": "carp",
    "carpels": "carpel",
    "carpenters": "carpenter",
    "carpetbaggers": "carpetbagger",
    "carpeted": "carpet",
    "carpeting": "carpet",
    "carpets": "carpet",
    "carping": "carp",
    "carps": "carp",
    "carrels": "carrel",
    "carriages": "carriage",
    "carriageways": "carriageway",
    "carried": "carry",
    "carriers": "carrier",
    "carries": "carry",
    "carrots": "carrot",
    "carrs": "carr",
    "carry-overs": "carry-over",
    "carrycots": "carrycot",
    "carrying": "carry",
    "carryovers": "carryover",
    "cars": "car",
    "carted": "cart",
    "cartels": "cartel",
    "cartes": "carte",
    "carthorses": "carthorse",
    "cartilages": "cartilage",
    "carting": "cart",
    "cartograms": "cartogram",
    "cartographers": "cartographer",
    "cartographies": "cartography",
    "cartons": "carton",
    "cartoonists": "cartoonist",
    "cartoons": "cartoon",
    "cartridges": "cartridge",
    "carts": "cart",
    "cartwheeled": "cartwheel",
    "cartwheeling": "cartwheel",
    "cartwheels": "cartwheel",
    "carved": "carve",
    "carveries": "carvery",
    "carvers": "carver",
    "carves": "carve",
    "carving": "carve",
    "carvings": "carving",
    "cascaded": "cascade",
    "cascades": "cascade",
    "cascading": "cascade",
    "casebooks": "casebook",
    "cased": "case",
    "caseins": "casein",
    "caseloads": "caseload",
    "casements": "casement",
    "casenotes": "casenote",
    "cases": "case",
    "caseworkers": "caseworker",
    "cash-and-carries": "cash-and-carry",
    "cash-desks": "cash-desk",
    "cashbooks": "cashbook",
    "cashdesks": "cashdesk",
    "cashed": "cash",
    "cashes": "cash",
    "cashews": "cashew",
    "cashiered": "cashier",
    "cashiering": "cashier",
    "cashiers": "cashier",
    "cashing": "cash",
    "cashpoints": "cashpoint",
    "casing": "case",
    "casings": "casing",
    "casinos": "casino",
    "caskets": "casket",
    "casks": "cask",
    "caspases": "caspase",
    "cassavas": "cassava",
    "casseroled": "casserole",
    "casseroles": "casserole",
    "casseroling": "casserole",
    "cassettes": "cassette",
    "cassocks": "cassock",
    "cassoulets": "cassoulet",
    "cast-offs": "cast-off",
    "castanets": "castanet",
    "castaways": "castaway",
    "casters": "caster",
    "castes": "caste",
    "castigated": "castigate",
    "castigates": "castigate",
    "castigating": "castigate",
    "casting": "cast",
    "castings": "casting",
    "castled": "castle",
    "castles": "castle",
    "castling": "castle",
    "castors": "castor",
    "castrated": "castrate",
    "castrates": "castrate",
    "castrati": "castrato",
    "castrating": "castrate",
    "castrations": "castration",
    "casts": "cast",
    "casuals": "casual",
    "casualties": "casualty",
    "casuistries": "casuistry",
    "cat's-eyes": "cat's-eye",
    "catabolisms": "catabolism",
    "cataclysms": "cataclysm",
    "catacombs": "catacomb",
    "catalases": "catalase",
    "cataloged": "catalog",
    "cataloging": "catalog",
    "catalogs": "catalog",
    "catalogued": "catalogue",
    "cataloguers": "cataloguer",
    "catalogues": "catalogue",
    "cataloguing": "catalogue",
    "catalysed": "catalyse",
    "catalyses": "catalysis",
    "catalysing": "catalyse",
    "catalysts": "catalyst",
    "catalyzed": "catalyze",
    "catalyzes": "catalyze",
    "catalyzing": "catalyze",
    "catamarans": "catamaran",
    "catapulted": "catapult",
    "catapulting": "catapult",
    "catapults": "catapult",
    "cataracts": "cataract",
    "catarrhs": "catarrh",
    "catastrophes": "catastrophe",
    "catatonias": "catatonia",
    "catcalls": "catcall",
    "catch-phrases": "catch-phrase",
    "catchers": "catcher",
    "catches": "catch",
    "catchier": "catchy",
    "catchiest": "catchy",
    "catching": "catch",
    "catchings": "catching",
    "catchments": "catchment",
    "catchphrases": "catchphrase",
    "catechins": "catechin",
    "catechisms": "catechism",
    "catecholamines": "catecholamine",
    "categories": "category",
    "categorisations": "categorisation",
    "categorised": "categorise",
    "categorises": "categorise",
    "categorising": "categorise",
    "categorizations": "categorization",
    "categorized": "categorize",
    "categorizes": "categorize",
    "categorizing": "categorize",
    "catenaries": "catenary",
    "catered": "cater",
    "caterers": "caterer",
    "catering": "cater",
    "caterings": "catering",
    "caterpillars": "caterpillar",
    "caters": "cater",
    "caterwauled": "caterwaul",
    "caterwauling": "caterwaul",
    "caterwauls": "caterwaul",
    "catfishes": "catfish",
    "catguts": "catgut",
    "catharses": "catharsis",
    "cathedrals": "cathedral",
    "cathepsins": "cathepsin",
    "catheterisations": "catheterisation",
    "catheterised": "catheterise",
    "catheterises": "catheterise",
    "catheterising": "catheterise",
    "catheterizations": "catheterization",
    "catheters": "catheter",
    "cathodes": "cathode",
    "catholicisms": "catholicism",
    "caths": "cath",
    "cations": "cation",
    "catkins": "catkin",
    "catnapped": "catnap",
    "catnapping": "catnap",
    "catnaps": "catnap",
    "cats": "cat",
    "catteries": "cattery",
    "cattier": "catty",
    "cattiest": "catty",
    "cattle-grids": "cattle-grid",
    "cattlemen": "cattleman",
    "catwalks": "catwalk",
    "caucuses": "caucus",
    "caudae": "cauda",
    "caught": "catch",
    "cauldrons": "cauldron",
    "cauliflowers": "cauliflower",
    "caulked": "caulk",
    "caulking": "caulk",
    "caulks": "caulk",
    "cauls": "caul",
    "causalities": "causality",
    "causals": "causal",
    "causations": "causation",
    "caused": "cause",
    "causes": "cause",
    "causeways": "causeway",
    "causing": "cause",
    "cauteries": "cautery",
    "cauterized": "cauterize",
    "cauterizes": "cauterize",
    "cauterizing": "cauterize",
    "cautioned": "caution",
    "cautioning": "caution",
    "cautions": "caution",
    "cautiousnesses": "cautiousness",
    "cava": "cavum",
    "cavalcades": "cavalcade",
    "cavaliers": "cavalier",
    "cavalries": "cavalry",
    "cavalrymen": "cavalryman",
    "cavas": "cava",
    "cave-ins": "cave-in",
    "caveats": "caveat",
    "caved": "cave",
    "cavemen": "caveman",
    "caverns": "cavern",
    "cavers": "caver",
    "caves": "cave",
    "cavies": "cavy",
    "cavilled": "cavil",
    "cavilling": "cavil",
    "cavils": "cavil",
    "caving": "cave",
    "cavitations": "cavitation",
    "cavities": "cavity",
    "cavorted": "cavort",
    "cavorting": "cavort",
    "cavorts": "cavort",
    "cawed": "caw",
    "cawing": "caw",
    "caws": "caw",
    "caymans": "cayman",
    "cays": "cay",
    "ccs": "cc",
    "ceased": "cease",
    "ceasefires": "ceasefire",
    "ceases": "cease",
    "ceasing": "cease",
    "cedars": "cedar",
    "ceded": "cede",
    "cedes": "cede",
    "cedillas": "cedilla",
    "ceding": "cede",
    "ceilidhs": "ceilidh",
    "ceiling": "ceil",
    "ceilings": "ceiling",
    "ceils": "ceil",
    "celandines": "celandine",
    "celebrants": "celebrant",
    "celebrated": "celebrate",
    "celebrates": "celebrate",
    "celebrating": "celebrate",
    "celebrations": "celebration",
    "celebrities": "celebrity",
    "celeriacs": "celeriac",
    "celestes": "celeste",
    "celibates": "celibate",
    "cellars": "cellar",
    "cellists": "cellist",
    "cellophanes": "cellophane",
    "cellos": "cello",
    "cellphones": "cellphone",
    "cells": "cell",
    "cellulases": "cellulase",
    "cellulitides": "cellulitis",
    "celluloses": "cellulose",
    "cels": "cel",
    "cementations": "cementation",
    "cemented": "cement",
    "cementing": "cement",
    "cements": "cement",
    "cemeteries": "cemetery",
    "cenotaphs": "cenotaph",
    "censored": "censor",
    "censoring": "censor",
    "censors": "censor",
    "censorships": "censorship",
    "censured": "censure",
    "censures": "censure",
    "censuring": "censure",
    "censuses": "census",
    "centary": "century",
    "centaurs": "centaur",
    "centenarians": "centenarian",
    "centenaries": "centenary",
    "centennials": "centennial",
    "centered": "center",
    "centering": "center",
    "centerlines": "centerline",
    "centerpieces": "centerpiece",
    "centers": "center",
    "centigrams": "centigram",
    "centiles": "centile",
    "centiliters": "centiliter",
    "centilitres": "centilitre",
    "centimes": "centime",
    "centimeters": "centimeter",
    "centimetres": "centimetre",
    "centipedes": "centipede",
    "centra": "centrum",
    "centralisations": "centralisation",
    "centralised": "centralise",
    "centralises": "centralise",
    "centralising": "centralise",
    "centralities": "centrality",
    "centralizations": "centralization",
    "centralized": "centralize",
    "centralizers": "centralizer",
    "centralizes": "centralize",
    "centralizing": "centralize",
    "centrals": "central",
    "centre-forwards": "centre-forward",
    "centre-halfs": "centre-half",
    "centred": "centre",
    "centrepieces": "centrepiece",
    "centres": "centre",
    "centrifugations": "centrifugation",
    "centrifuged": "centrifuge",
    "centrifuges": "centrifuge",
    "centrifuging": "centrifuge",
    "centring": "centre",
    "centrisms": "centrism",
    "centrists": "centrist",
    "centroids": "centroid",
    "centromeres": "centromere",
    "centrosomes": "centrosome",
    "centrums": "centrum",
    "cents": "cent",
    "centuries": "century",
    "centurions": "centurion",
    "cephalopods": "cephalopod",
    "cephalosporins": "cephalosporin",
    "ceps": "cep",
    "ceramicists": "ceramicist",
    "ceramics": "ceramic",
    "ceramides": "ceramide",
    "ceramists": "ceramist",
    "cercariae": "cercaria",
    "cereals": "cereal",
    "cerebella": "cerebellum",
    "cerebellums": "cerebellum",
    "cerebra": "cerebrum",
    "cerebrums": "cerebrum",
    "ceremonials": "ceremonial",
    "ceremonies": "ceremony",
    "cerevisiae": "cerevisia",
    "certainties": "certainty",
    "certificated": "certificate",
    "certificates": "certificate",
    "certificating": "certificate",
    "certifications": "certification",
    "certified": "certify",
    "certifiers": "certifier",
    "certifies": "certify",
    "certifying": "certify",
    "certitudes": "certitude",
    "certs": "cert",
    "cervices": "cervix",
    "cervixes": "cervix",
    "cessations": "cessation",
    "cessions": "cession",
    "cesspits": "cesspit",
    "cesspools": "cesspool",
    "cetaceans": "cetacean",
    "cfcs": "cfc",
    "cha-chas": "cha-cha",
    "chads": "chad",
    "chafed": "chafe",
    "chafers": "chafer",
    "chafes": "chafe",
    "chaffed": "chaff",
    "chaffinches": "chaffinch",
    "chaffing": "chaff",
    "chaffs": "chaff",
    "chafing": "chafe",
    "chagrined": "chagrin",
    "chagrining": "chagrin",
    "chagrins": "chagrin",
    "chain-smoked": "chain-smoke",
    "chain-smokers": "chain-smoker",
    "chain-smokes": "chain-smoke",
    "chain-smoking": "chain-smoke",
    "chained": "chain",
    "chaining": "chain",
    "chains": "chain",
    "chainsaws": "chainsaw",
    "chaired": "chair",
    "chairing": "chair",
    "chairlifts": "chairlift",
    "chairmanships": "chairmanship",
    "chairmen": "chairman",
    "chairpersons": "chairperson",
    "chairs": "chair",
    "chairwomen": "chairwoman",
    "chaises": "chaise",
    "chalets": "chalet",
    "chalices": "chalice",
    "chalkboards": "chalkboard",
    "chalked": "chalk",
    "chalkier": "chalky",
    "chalkiest": "chalky",
    "chalking": "chalk",
    "chalks": "chalk",
    "challenged": "challenge",
    "challengers": "challenger",
    "challenges": "challenge",
    "challenging": "challenge",
    "chamberlains": "chamberlain",
    "chambermaids": "chambermaid",
    "chamberpots": "chamberpot",
    "chambers": "chamber",
    "chameleons": "chameleon",
    "chamfered": "chamfer",
    "chamfering": "chamfer",
    "chamfers": "chamfer",
    "chammies": "chammy",
    "chamomiles": "chamomile",
    "champagnes": "champagne",
    "champed": "champ",
    "champing": "champ",
    "championed": "champion",
    "championing": "champion",
    "champions": "champion",
    "championships": "championship",
    "champs": "champ",
    "chanced": "chance",
    "chancellors": "chancellor",
    "chancels": "chancel",
    "chances": "chance",
    "chancier": "chancy",
    "chanciest": "chancy",
    "chancing": "chance",
    "chandeliers": "chandelier",
    "chandlers": "chandler",
    "change-overs": "change-over",
    "changeabilities": "changeability",
    "changed": "change",
    "changelings": "changeling",
    "changeovers": "changeover",
    "changers": "changer",
    "changes": "change",
    "changing": "change",
    "changings": "changing",
    "channeled": "channel",
    "channeling": "channel",
    "channelled": "channel",
    "channelling": "channel",
    "channels": "channel",
    "chanted": "chant",
    "chanting": "chant",
    "chants": "chant",
    "chaoses": "chaos",
    "chapatis": "chapati",
    "chapattis": "chapatti",
    "chapels": "chapel",
    "chaperoned": "chaperone",
    "chaperones": "chaperone",
    "chaperoning": "chaperone",
    "chaperonins": "chaperonin",
    "chaperons": "chaperon",
    "chaplaincies": "chaplaincy",
    "chaplains": "chaplain",
    "chapmen": "chapman",
    "chapped": "chap",
    "chapping": "chap",
    "chaps": "chap",
    "chapters": "chapter",
    "charabancs": "charabanc",
    "characterisations": "characterisation",
    "characterised": "characterise",
    "characterises": "characterise",
    "characterising": "characterise",
    "characteristics": "characteristic",
    "characterizations": "characterization",
    "characterized": "characterize",
    "characterizes": "characterize",
    "characterizing": "characterize",
    "characters": "character",
    "charades": "charade",
    "charcoals": "charcoal",
    "charcuteries": "charcuterie",
    "charged": "charge",
    "chargers": "charger",
    "charges": "charge",
    "charging": "charge",
    "charioteers": "charioteer",
    "chariots": "chariot",
    "charismata": "charisma",
    "charisms": "charism",
    "charities": "charity",
    "charladies": "charlady",
    "charlatans": "charlatan",
    "charlestons": "charleston",
    "charmed": "charm",
    "charmers": "charmer",
    "charming": "charm",
    "charms": "charm",
    "charred": "char",
    "charring": "char",
    "charrs": "charr",
    "chars": "char",
    "charted": "chart",
    "chartered": "charter",
    "charterers": "charterer",
    "chartering": "charter",
    "charters": "charter",
    "charting": "chart",
    "charts": "chart",
    "charwomen": "charwoman",
    "chased": "chase",
    "chasers": "chaser",
    "chases": "chase",
    "chasing": "chase",
    "chasms": "chasm",
    "chastened": "chasten",
    "chastening": "chasten",
    "chastens": "chasten",
    "chastised": "chastise",
    "chastises": "chastise",
    "chastising": "chastise",
    "chateaus": "chateau",
    "chateaux": "chateau",
    "chats": "chat",
    "chatted": "chat",
    "chattels": "chattel",
    "chatterboxes": "chatterbox",
    "chattered": "chatter",
    "chatterers": "chatterer",
    "chattering": "chatter",
    "chatters": "chatter",
    "chattier": "chatty",
    "chattiest": "chatty",
    "chatting": "chat",
    "chauffeured": "chauffeur",
    "chauffeuring": "chauffeur",
    "chauffeurs": "chauffeur",
    "chauvinists": "chauvinist",
    "cheapened": "cheapen",
    "cheapening": "cheapen",
    "cheapens": "cheapen",
    "cheaper": "cheap",
    "cheapest": "cheap",
    "cheated": "cheat",
    "cheaters": "cheater",
    "cheating": "cheat",
    "cheats": "cheat",
    "check-boxes": "check-box",
    "check-ins": "check-in",
    "check-lists": "check-list",
    "check-outs": "check-out",
    "check-points": "check-point",
    "check-ups": "check-up",
    "checkbooks": "checkbook",
    "checkboxes": "checkbox",
    "checked": "check",
    "checkerboards": "checkerboard",
    "checkered": "checker",
    "checkering": "checker",
    "checkers": "checker",
    "checking": "check",
    "checklists": "checklist",
    "checkmarks": "checkmark",
    "checkmated": "checkmate",
    "checkmates": "checkmate",
    "checkmating": "checkmate",
    "checkouts": "checkout",
    "checkpoints": "checkpoint",
    "checks": "check",
    "checkups": "checkup",
    "cheddars": "cheddar",
    "cheekbones": "cheekbone",
    "cheeked": "cheek",
    "cheekier": "cheeky",
    "cheekiest": "cheeky",
    "cheeking": "cheek",
    "cheeks": "cheek",
    "cheered": "cheer",
    "cheerfulnesses": "cheerfulness",
    "cheerier": "cheery",
    "cheeriest": "cheery",
    "cheering": "cheer",
    "cheerios": "cheerio",
    "cheerleaders": "cheerleader",
    "cheers": "cheer",
    "cheeseboards": "cheeseboard",
    "cheeseburgers": "cheeseburger",
    "cheesecakes": "cheesecake",
    "cheesecloths": "cheesecloth",
    "cheesemakers": "cheesemaker",
    "cheeses": "cheese",
    "cheesier": "cheesy",
    "cheesiest": "cheesy",
    "cheetahs": "cheetah",
    "chefs": "chef",
    "chefs-d'oeuvre": "chef-d'oeuvre",
    "chelated": "chelate",
    "chelates": "chelate",
    "chelating": "chelate",
    "chemicals": "chemical",
    "chemises": "chemise",
    "chemistries": "chemistry",
    "chemists": "chemist",
    "chemokines": "chemokine",
    "chemoprophylaxes": "chemoprophylaxis",
    "chemoradiotherapies": "chemoradiotherapy",
    "chemotaxes": "chemotaxis",
    "chemotherapies": "chemotherapy",
    "chems": "chem",
    "chequebooks": "chequebook",
    "chequerboards": "chequerboard",
    "chequered": "chequer",
    "chequering": "chequer",
    "chequers": "chequer",
    "cheques": "cheque",
    "cherished": "cherish",
    "cherishes": "cherish",
    "cherishing": "cherish",
    "cheroots": "cheroot",
    "cherries": "cherry",
    "cherts": "chert",
    "cherubs": "cherub",
    "chessboards": "chessboard",
    "chessmen": "chessman",
    "chessplayers": "chessplayer",
    "chestnuts": "chestnut",
    "chests": "chest",
    "chevrons": "chevron",
    "chewed": "chew",
    "chewier": "chewy",
    "chewiest": "chewy",
    "chewing": "chew",
    "chewing-gums": "chewing-gum",
    "chews": "chew",
    "chi-squares": "chi-square",
    "chiaroscuros": "chiaroscuro",
    "chiasms": "chiasm",
    "chicaneries": "chicanery",
    "chicken-houses": "chicken-house",
    "chickened": "chicken",
    "chickening": "chicken",
    "chickens": "chicken",
    "chickpeas": "chickpea",
    "chicks": "chick",
    "chicories": "chicory",
    "chics": "chic",
    "chid": "chide",
    "chidded": "chide",
    "chidden": "chide",
    "chided": "chide",
    "chides": "chide",
    "chiding": "chide",
    "chiefdoms": "chiefdom",
    "chiefs": "chief",
    "chieftains": "chieftain",
    "chihuahuas": "chihuahua",
    "chilblains": "chilblain",
    "childbirths": "childbirth",
    "childhoods": "childhood",
    "childminders": "childminder",
    "children": "child",
    "childs": "child",
    "chiles": "chile",
    "chilies": "chili",
    "chillblains": "chillblain",
    "chilled": "chill",
    "chiller": "chill",
    "chillers": "chiller",
    "chillest": "chill",
    "chillier": "chilly",
    "chillies": "chilli",
    "chilliest": "chilly",
    "chilling": "chill",
    "chills": "chill",
    "chimaerae": "chimaera",
    "chimaeras": "chimaera",
    "chimed": "chime",
    "chimerae": "chimera",
    "chimeras": "chimera",
    "chimes": "chime",
    "chiming": "chime",
    "chimneys": "chimney",
    "chimpanzees": "chimpanzee",
    "chimps": "chimp",
    "chinas": "china",
    "chinchillas": "chinchilla",
    "chines": "chine",
    "chinked": "chink",
    "chinking": "chink",
    "chinks": "chink",
    "chins": "chin",
    "chipboards": "chipboard",
    "chipmunks": "chipmunk",
    "chipped": "chip",
    "chippers": "chipper",
    "chipping": "chip",
    "chippings": "chipping",
    "chips": "chip",
    "chiralities": "chirality",
    "chiropodies": "chiropody",
    "chiropodists": "chiropodist",
    "chiropractors": "chiropractor",
    "chirped": "chirp",
    "chirpier": "chirpy",
    "chirpiest": "chirpy",
    "chirping": "chirp",
    "chirps": "chirp",
    "chirruped": "chirrup",
    "chirruping": "chirrup",
    "chirrups": "chirrup",
    "chis": "chi",
    "chiseled": "chisel",
    "chiseling": "chisel",
    "chiselled": "chisel",
    "chiselling": "chisel",
    "chisels": "chisel",
    "chitchats": "chitchat",
    "chitins": "chitin",
    "chits": "chit",
    "chittered": "chitter",
    "chittering": "chitter",
    "chitters": "chitter",
    "chitties": "chitty",
    "chives": "chive",
    "chivvied": "chivvy",
    "chivvies": "chivvy",
    "chivvying": "chivvy",
    "chlamydiae": "chlamydia",
    "chlamydias": "chlamydia",
    "chloramines": "chloramine",
    "chloramphenicols": "chloramphenicol",
    "chlorhexidines": "chlorhexidine",
    "chlorides": "chloride",
    "chlorinated": "chlorinate",
    "chlorinates": "chlorinate",
    "chlorinating": "chlorinate",
    "chlorinations": "chlorination",
    "chlorines": "chlorine",
    "chlorobenzenes": "chlorobenzene",
    "chlorofluorocarbons": "chlorofluorocarbon",
    "chloroformed": "chloroform",
    "chloroforming": "chloroform",
    "chloroforms": "chloroform",
    "chlorophylls": "chlorophyll",
    "chloroplasts": "chloroplast",
    "chloroses": "chlorosis",
    "chlorpromazines": "chlorpromazine",
    "choc-ices": "choc-ice",
    "chocked": "chock",
    "chocking": "chock",
    "chocks": "chock",
    "chocolates": "chocolate",
    "choicer": "choice",
    "choices": "choice",
    "choicest": "choice",
    "choirboys": "choirboy",
    "choirmasters": "choirmaster",
    "choirs": "choir",
    "choked": "choke",
    "chokers": "choker",
    "chokes": "choke",
    "choking": "choke",
    "cholangiocarcinomas": "cholangiocarcinoma",
    "cholangiocarcinomata": "cholangiocarcinoma",
    "cholecystectomies": "cholecystectomy",
    "cholecystitides": "cholecystitis",
    "cholerae": "cholera",
    "choleras": "cholera",
    "cholestases": "cholestasis",
    "cholesterols": "cholesterol",
    "cholines": "choline",
    "cholinesterases": "cholinesterase",
    "chomped": "chomp",
    "chomping": "chomp",
    "chomps": "chomp",
    "chondrocytes": "chondrocyte",
    "chondroitins": "chondroitin",
    "choosers": "chooser",
    "chooses": "choose",
    "choosier": "choosy",
    "choosiest": "choosy",
    "choosing": "choose",
    "chopped": "chop",
    "choppers": "chopper",
    "choppier": "choppy",
    "choppiest": "choppy",
    "chopping": "chop",
    "chops": "chop",
    "chopsticks": "chopstick",
    "chorales": "chorale",
    "chorals": "choral",
    "chords": "chord",
    "choreas": "chorea",
    "choreographed": "choreograph",
    "choreographers": "choreographer",
    "choreographies": "choreography",
    "choreographing": "choreograph",
    "choreographs": "choreograph",
    "chores": "chore",
    "choristers": "chorister",
    "chortled": "chortle",
    "chortles": "chortle",
    "chortling": "chortle",
    "chorused": "chorus",
    "choruses": "chorus",
    "chorusing": "chorus",
    "chose": "choose",
    "chosen": "choose",
    "choughs": "chough",
    "choux": "chou",
    "chowders": "chowder",
    "chows": "chow",
    "christened": "christen",
    "christening": "christen",
    "christenings": "christening",
    "christens": "christen",
    "christianized": "christianize",
    "christianizes": "christianize",
    "christianizing": "christianize",
    "christs": "christ",
    "chromas": "chroma",
    "chromates": "chromate",
    "chromatids": "chromatid",
    "chromatins": "chromatin",
    "chromatograms": "chromatogram",
    "chromatographies": "chromatography",
    "chromatographs": "chromatograph",
    "chromes": "chrome",
    "chromophores": "chromophore",
    "chromosomes": "chromosome",
    "chronicities": "chronicity",
    "chronicled": "chronicle",
    "chroniclers": "chronicler",
    "chronicles": "chronicle",
    "chronicling": "chronicle",
    "chronics": "chronic",
    "chronographs": "chronograph",
    "chronologies": "chronology",
    "chronometers": "chronometer",
    "chrysalides": "chrysalis",
    "chrysalises": "chrysalis",
    "chrysanthemums": "chrysanthemum",
    "chs": "ch",
    "chubbier": "chubby",
    "chubbiest": "chubby",
    "chubs": "chub",
    "chucked": "chuck",
    "chucking": "chuck",
    "chuckled": "chuckle",
    "chuckles": "chuckle",
    "chuckling": "chuckle",
    "chucks": "chuck",
    "chugged": "chug",
    "chugging": "chug",
    "chugs": "chug",
    "chummed": "chum",
    "chummier": "chummy",
    "chummiest": "chummy",
    "chumming": "chum",
    "chumps": "chump",
    "chums": "chum",
    "chunked": "chunk",
    "chunkier": "chunky",
    "chunkiest": "chunky",
    "chunking": "chunk",
    "chunks": "chunk",
    "church-goers": "church-goer",
    "churches": "church",
    "churchgoers": "churchgoer",
    "churchmen": "churchman",
    "churchwardens": "churchwarden",
    "churchyards": "churchyard",
    "churls": "churl",
    "churned": "churn",
    "churning": "churn",
    "churns": "churn",
    "chutes": "chute",
    "chutneys": "chutney",
    "cicadae": "cicada",
    "cicadas": "cicada",
    "cichlids": "cichlid",
    "ciclosporins": "ciclosporin",
    "ciders": "cider",
    "cigarettes": "cigarette",
    "cigars": "cigar",
    "cilia": "cilium",
    "ciliates": "ciliate",
    "cinched": "cinch",
    "cinches": "cinch",
    "cinching": "cinch",
    "cinders": "cinder",
    "cine-films": "cine-film",
    "cine-projectors": "cine-projector",
    "cinema-goers": "cinema-goer",
    "cinemagoers": "cinemagoer",
    "cinemas": "cinema",
    "cinematographers": "cinematographer",
    "cinematographs": "cinematograph",
    "cines": "cine",
    "cinnamons": "cinnamon",
    "cinquefoils": "cinquefoil",
    "ciphered": "cipher",
    "ciphering": "cipher",
    "ciphers": "cipher",
    "circled": "circle",
    "circles": "circle",
    "circlets": "circlet",
    "circling": "circle",
    "circuitries": "circuitry",
    "circuits": "circuit",
    "circulars": "circular",
    "circulated": "circulate",
    "circulates": "circulate",
    "circulating": "circulate",
    "circulations": "circulation",
    "circulators": "circulator",
    "circumcised": "circumcise",
    "circumcises": "circumcise",
    "circumcising": "circumcise",
    "circumcisions": "circumcision",
    "circumferences": "circumference",
    "circumflexes": "circumflex",
    "circumlocutions": "circumlocution",
    "circumnavigated": "circumnavigate",
    "circumnavigates": "circumnavigate",
    "circumnavigating": "circumnavigate",
    "circumnavigations": "circumnavigation",
    "circumscribed": "circumscribe",
    "circumscribes": "circumscribe",
    "circumscribing": "circumscribe",
    "circumscriptions": "circumscription",
    "circumstances": "circumstance",
    "circumvented": "circumvent",
    "circumventing": "circumvent",
    "circumventions": "circumvention",
    "circumvents": "circumvent",
    "circuses": "circus",
    "cirrhoses": "cirrhosis",
    "cirri": "cirrus",
    "ciscoes": "cisco",
    "cissies": "cissy",
    "cisterns": "cistern",
    "citadels": "citadel",
    "citations": "citation",
    "cited": "cite",
    "cites": "cite",
    "cities": "city",
    "citing": "cite",
    "citizens": "citizen",
    "citizenships": "citizenship",
    "citrates": "citrate",
    "citrines": "citrine",
    "citrons": "citron",
    "citruses": "citrus",
    "cits": "cit",
    "city-states": "city-state",
    "civets": "civet",
    "civiler": "civil",
    "civilest": "civil",
    "civilians": "civilian",
    "civilisations": "civilisation",
    "civilised": "civilise",
    "civilises": "civilise",
    "civilising": "civilise",
    "civilities": "civility",
    "civilizations": "civilization",
    "civilized": "civilize",
    "civilizes": "civilize",
    "civilizing": "civilize",
    "civils": "civil",
    "clacked": "clack",
    "clacking": "clack",
    "clacks": "clack",
    "clad": "clothe",
    "cladded": "clad",
    "cladding": "clad",
    "clades": "clade",
    "clads": "clad",
    "claimants": "claimant",
    "claimed": "claim",
    "claiming": "claim",
    "claims": "claim",
    "clairvoyants": "clairvoyant",
    "clambered": "clamber",
    "clambering": "clamber",
    "clambers": "clamber",
    "clammed": "clam",
    "clammier": "clammy",
    "clammiest": "clammy",
    "clamming": "clam",
    "clamored": "clamor",
    "clamoring": "clamor",
    "clamors": "clamor",
    "clamoured": "clamour",
    "clamouring": "clamour",
    "clamours": "clamour",
    "clampdowns": "clampdown",
    "clamped": "clamp",
    "clamping": "clamp",
    "clamps": "clamp",
    "clams": "clam",
    "clamshells": "clamshell",
    "clanged": "clang",
    "clangers": "clanger",
    "clanging": "clang",
    "clangs": "clang",
    "clanked": "clank",
    "clanking": "clank",
    "clanks": "clank",
    "clans": "clan",
    "clansmen": "clansman",
    "clapboards": "clapboard",
    "clapped": "clap",
    "clapperboards": "clapperboard",
    "clappers": "clapper",
    "clapping": "clap",
    "clappings": "clapping",
    "claps": "clap",
    "claptraps": "claptrap",
    "clarets": "claret",
    "claries": "clary",
    "clarifications": "clarification",
    "clarified": "clarify",
    "clarifies": "clarify",
    "clarifying": "clarify",
    "clarinetists": "clarinetist",
    "clarinets": "clarinet",
    "clarinettists": "clarinettist",
    "clarities": "clarity",
    "clashed": "clash",
    "clashes": "clash",
    "clashing": "clash",
    "clasped": "clasp",
    "clasping": "clasp",
    "clasps": "clasp",
    "classed": "class",
    "classes": "class",
    "classicists": "classicist",
    "classics": "classic",
    "classier": "classy",
    "classiest": "classy",
    "classifications": "classification",
    "classified": "classify",
    "classifieds": "classified",
    "classifiers": "classifier",
    "classifies": "classify",
    "classifying": "classify",
    "classing": "class",
    "classmates": "classmate",
    "classrooms": "classroom",
    "clattered": "clatter",
    "clattering": "clatter",
    "clatters": "clatter",
    "claudications": "claudication",
    "clauses": "clause",
    "claustrophobias": "claustrophobia",
    "claves": "clave",
    "clavichords": "clavichord",
    "clavicles": "clavicle",
    "clawed": "claw",
    "clawing": "claw",
    "claws": "claw",
    "clays": "clay",
    "clean-ups": "clean-up",
    "cleaned": "clean",
    "cleaner": "clean",
    "cleaners": "cleaner",
    "cleanest": "clean",
    "cleaning": "clean",
    "cleanings": "cleaning",
    "cleanlier": "cleanly",
    "cleanrooms": "cleanroom",
    "cleans": "clean",
    "cleansed": "cleanse",
    "cleansers": "cleanser",
    "cleanses": "cleanse",
    "cleansing": "cleanse",
    "cleansings": "cleansing",
    "clear-cutting": "clear-cut",
    "clearances": "clearance",
    "cleared": "clear",
    "clearer": "clear",
    "clearer-headed": "clear-headed",
    "clearest": "clear",
    "clearest-headed": "clear-headed",
    "clearing": "clear",
    "clearinghouses": "clearinghouse",
    "clearings": "clearing",
    "clears": "clear",
    "clearways": "clearway",
    "cleats": "cleat",
    "cleavages": "cleavage",
    "cleaved": "cleave",
    "cleavers": "cleaver",
    "cleaves": "cleave",
    "cleaving": "cleave",
    "clefs": "clef",
    "cleft": "cleave",
    "clefts": "cleft",
    "clenched": "clench",
    "clenches": "clench",
    "clenching": "clench",
    "clergies": "clergy",
    "clergymen": "clergyman",
    "clerics": "cleric",
    "clerked": "clerk",
    "clerking": "clerk",
    "clerks": "clerk",
    "clerkships": "clerkship",
    "cleverer": "clever",
    "cleverest": "clever",
    "clews": "clew",
    "cliches": "cliche",
    "clichés": "cliché",
    "clicked": "click",
    "clickers": "clicker",
    "clicking": "click",
    "clicks": "click",
    "clienteles": "clientele",
    "clients": "client",
    "cliff-hangers": "cliff-hanger",
    "cliffs": "cliff",
    "climacterics": "climacteric",
    "climates": "climate",
    "climatologies": "climatology",
    "climatologists": "climatologist",
    "climaxed": "climax",
    "climaxes": "climax",
    "climaxing": "climax",
    "climb-downs": "climb-down",
    "climbed": "climb",
    "climbers": "climber",
    "climbing": "climb",
    "climbs": "climb",
    "climes": "clime",
    "clinched": "clinch",
    "clinchers": "clincher",
    "clinches": "clinch",
    "clinching": "clinch",
    "clinging": "cling",
    "clings": "cling",
    "clinicians": "clinician",
    "clinics": "clinic",
    "clinked": "clink",
    "clinkers": "clinker",
    "clinking": "clink",
    "clinks": "clink",
    "clinometers": "clinometer",
    "clints": "clint",
    "clipboards": "clipboard",
    "clipped": "clip",
    "clippers": "clipper",
    "clipping": "clip",
    "clippings": "clipping",
    "clips": "clip",
    "cliques": "clique",
    "clitics": "clitic",
    "clitorides": "clitoris",
    "clitorises": "clitoris",
    "cloaked": "cloak",
    "cloaking": "cloak",
    "cloakrooms": "cloakroom",
    "cloaks": "cloak",
    "clobbered": "clobber",
    "clobbering": "clobber",
    "clobbers": "clobber",
    "cloches": "cloche",
    "clocked": "clock",
    "clocking": "clock",
    "clocks": "clock",
    "clockworks": "clockwork",
    "clodhoppers": "clodhopper",
    "clods": "clod",
    "clogged": "clog",
    "cloggier": "cloggy",
    "clogging": "clog",
    "clogs": "clog",
    "cloistered": "cloister",
    "cloistering": "cloister",
    "cloisters": "cloister",
    "cloned": "clone",
    "clones": "clone",
    "cloning": "clone",
    "clonked": "clonk",
    "clonking": "clonk",
    "clonks": "clonk",
    "clopped": "clop",
    "clopping": "clop",
    "clops": "clop",
    "clos": "clo",
    "close-ups": "close-up",
    "closed": "close",
    "closed-loops": "closed-loop",
    "closeouts": "closeout",
    "closer": "close",
    "closers": "closer",
    "closes": "close",
    "closest": "close",
    "closeted": "closet",
    "closeting": "closet",
    "closets": "closet",
    "closeups": "closeup",
    "closing": "close",
    "closings": "closing",
    "clostridia": "clostridium",
    "closures": "closure",
    "clothed": "clothe",
    "clothes": "clothe",
    "clothiers": "clothier",
    "clothing": "clothe",
    "cloths": "cloth",
    "clots": "clot",
    "clotted": "clot",
    "clotting": "clot",
    "cloudbursts": "cloudburst",
    "clouded": "cloud",
    "cloudier": "cloudy",
    "cloudiest": "cloudy",
    "clouding": "cloud",
    "clouds": "cloud",
    "clouted": "clout",
    "clouting": "clout",
    "clouts": "clout",
    "clove": "cleave",
    "cloven": "cleave",
    "cloverleafs": "cloverleaf",
    "clovers": "clover",
    "cloves": "clove",
    "clowned": "clown",
    "clownfishes": "clownfish",
    "clowning": "clown",
    "clowns": "clown",
    "cloyed": "cloy",
    "cloying": "cloy",
    "cloys": "cloy",
    "cls": "cl",
    "clubbed": "club",
    "clubbier": "clubby",
    "clubbing": "club",
    "clubfeet": "clubfoot",
    "clubhouses": "clubhouse",
    "clubs": "club",
    "clucked": "cluck",
    "clucking": "cluck",
    "clucks": "cluck",
    "clued": "clue",
    "clues": "clue",
    "cluing": "clue",
    "clumped": "clump",
    "clumping": "clump",
    "clumps": "clump",
    "clumsier": "clumsy",
    "clumsiest": "clumsy",
    "clung": "cling",
    "clunked": "clunk",
    "clunking": "clunk",
    "clunks": "clunk",
    "clustered": "cluster",
    "clustering": "cluster",
    "clusters": "cluster",
    "clutched": "clutch",
    "clutches": "clutch",
    "clutching": "clutch",
    "cluttered": "clutter",
    "cluttering": "clutter",
    "clutters": "clutter",
    "cms": "cm",
    "co-administrations": "co-administration",
    "co-authored": "co-author",
    "co-authoring": "co-author",
    "co-authors": "co-author",
    "co-conspirators": "co-conspirator",
    "co-developed": "co-develop",
    "co-developing": "co-develop",
    "co-develops": "co-develop",
    "co-directors": "co-director",
    "co-editors": "co-editor",
    "co-efficients": "co-efficient",
    "co-enzymes": "co-enzyme",
    "co-evolved": "co-evolve",
    "co-evolves": "co-evolve",
    "co-evolving": "co-evolve",
    "co-existed": "co-exist",
    "co-existences": "co-existence",
    "co-existing": "co-exist",
    "co-exists": "co-exist",
    "co-factors": "co-factor",
    "co-founders": "co-founder",
    "co-habitations": "co-habitation",
    "co-habited": "co-habit",
    "co-habiting": "co-habit",
    "co-habits": "co-habit",
    "co-incided": "co-incide",
    "co-incidences": "co-incidence",
    "co-incides": "co-incide",
    "co-inciding": "co-incide",
    "co-infections": "co-infection",
    "co-investigators": "co-investigator",
    "co-leaders": "co-leader",
    "co-managed": "co-manage",
    "co-manages": "co-manage",
    "co-managing": "co-manage",
    "co-morbidities": "co-morbidity",
    "co-occurrences": "co-occurrence",
    "co-operated": "co-operate",
    "co-operates": "co-operate",
    "co-operating": "co-operate",
    "co-operations": "co-operation",
    "co-operatives": "co-operative",
    "co-operators": "co-operator",
    "co-ops": "co-op",
    "co-opted": "co-opt",
    "co-opting": "co-opt",
    "co-opts": "co-opt",
    "co-ordinated": "co-ordinate",
    "co-ordinates": "co-ordinate",
    "co-ordinating": "co-ordinate",
    "co-ordinations": "co-ordination",
    "co-ordinators": "co-ordinator",
    "co-payments": "co-payment",
    "co-pilots": "co-pilot",
    "co-processors": "co-processor",
    "co-produced": "co-produce",
    "co-producers": "co-producer",
    "co-produces": "co-produce",
    "co-producing": "co-produce",
    "co-productions": "co-production",
    "co-products": "co-product",
    "co-promoters": "co-promoter",
    "co-religionists": "co-religionist",
    "co-requisites": "co-requisite",
    "co-signatories": "co-signatory",
    "co-sponsored": "co-sponsor",
    "co-sponsoring": "co-sponsor",
    "co-sponsors": "co-sponsor",
    "co-starred": "co-star",
    "co-starring": "co-star",
    "co-stars": "co-star",
    "co-workers": "co-worker",
    "coach-and-fours": "coach-and-four",
    "coached": "coach",
    "coachees": "coachee",
    "coaches": "coach",
    "coaching": "coach",
    "coachloads": "coachload",
    "coachmen": "coachman",
    "coachmen.": "coachman",
    "coagulants": "coagulant",
    "coagulated": "coagulate",
    "coagulates": "coagulate",
    "coagulating": "coagulate",
    "coagulations": "coagulation",
    "coagulopathies": "coagulopathy",
    "coalesced": "coalesce",
    "coalesces": "coalesce",
    "coalescing": "coalesce",
    "coalfaces": "coalface",
    "coalfields": "coalfield",
    "coalfishes": "coalfish",
    "coalitions": "coalition",
    "coalmen": "coalman",
    "coalminers": "coalminer",
    "coalmines": "coalmine",
    "coals": "coal",
    "coarsely": "coarse",
    "coarsened": "coarsen",
    "coarsening": "coarsen",
    "coarsens": "coarsen",
    "coarser": "coarse",
    "coarsest": "coarse",
    "coasted": "coast",
    "coasters": "coaster",
    "coastguards": "coastguard",
    "coasting": "coast",
    "coastlines": "coastline",
    "coasts": "coast",
    "coated": "coat",
    "coating": "coat",
    "coatings": "coating",
    "coats": "coat",
    "coauthored": "coauthor",
    "coauthoring": "coauthor",
    "coauthors": "coauthor",
    "coaxed": "coax",
    "coaxes": "coax",
    "coaxing": "coax",
    "cob-nuts": "cob-nut",
    "cobalamins": "cobalamin",
    "cobbled": "cobble",
    "cobblers": "cobbler",
    "cobbles": "cobble",
    "cobblestones": "cobblestone",
    "cobbling": "cobble",
    "cobras": "cobra",
    "cobs": "cob",
    "cobwebs": "cobweb",
    "cocas": "coca",
    "cocci": "coccus",
    "coccidioses": "coccidiosis",
    "coccolithophores": "coccolithophore",
    "coccoliths": "coccolith",
    "coccyges": "coccyx",
    "coccyxes": "coccyx",
    "cochleae": "cochlea",
    "cochleas": "cochlea",
    "cock-crows": "cock-crow",
    "cock-ups": "cock-up",
    "cockatiels": "cockatiel",
    "cockatoos": "cockatoo",
    "cockatoos.": "cockatoo",
    "cocked": "cock",
    "cockerels": "cockerel",
    "cockers": "cocker",
    "cockfights": "cockfight",
    "cockier": "cocky",
    "cockiest": "cocky",
    "cocking": "cock",
    "cockles": "cockle",
    "cockleshells": "cockleshell",
    "cockneys": "cockney",
    "cockpits": "cockpit",
    "cockroaches": "cockroach",
    "cocks": "cock",
    "cockscombs": "cockscomb",
    "cocktails": "cocktail",
    "cocoas": "cocoa",
    "coconuts": "coconut",
    "cocooned": "cocoon",
    "cocooning": "cocoon",
    "cocoons": "cocoon",
    "cocos": "coco",
    "codas": "coda",
    "coddled": "coddle",
    "coddles": "coddle",
    "coddling": "coddle",
    "codebooks": "codebook",
    "coded": "code",
    "coders": "coder",
    "codes": "code",
    "codewords": "codeword",
    "codgers": "codger",
    "codices": "codex",
    "codicils": "codicil",
    "codifications": "codification",
    "codified": "codify",
    "codifies": "codify",
    "codifying": "codify",
    "coding": "code",
    "codings": "coding",
    "codlings": "codling",
    "codons": "codon",
    "codpieces": "codpiece",
    "cods": "cod",
    "coefficients": "coefficient",
    "coelacanths": "coelacanth",
    "coenzymes": "coenzyme",
    "coerced": "coerce",
    "coerces": "coerce",
    "coercing": "coerce",
    "coercions": "coercion",
    "coeurs": "coeur",
    "coexisted": "coexist",
    "coexistences": "coexistence",
    "coexisting": "coexist",
    "coexists": "coexist",
    "cofactors": "cofactor",
    "coffeepots": "coffeepot",
    "coffees": "coffee",
    "coffers": "coffer",
    "coffins": "coffin",
    "cofounders": "cofounder",
    "cogitated": "cogitate",
    "cogitates": "cogitate",
    "cogitating": "cogitate",
    "cogitations": "cogitation",
    "cognacs": "cognac",
    "cognates": "cognate",
    "cognitions": "cognition",
    "cognized": "cognize",
    "cognizes": "cognize",
    "cognizing": "cognize",
    "cognoscenti": "cognoscente",
    "cogs": "cog",
    "cogwheels": "cogwheel",
    "cohabitants": "cohabitant",
    "cohabitations": "cohabitation",
    "cohabited": "cohabit",
    "cohabitees": "cohabitee",
    "cohabiting": "cohabit",
    "cohabits": "cohabit",
    "cohabitted": "cohabit",
    "cohabitting": "cohabit",
    "cohered": "cohere",
    "coherences": "coherence",
    "coherencies": "coherency",
    "coherenter": "coherent",
    "coherentest": "coherent",
    "coheres": "cohere",
    "cohering": "cohere",
    "cohesions": "cohesion",
    "cohomologies": "cohomology",
    "cohorts": "cohort",
    "coiffures": "coiffure",
    "coiled": "coil",
    "coiling": "coil",
    "coils": "coil",
    "coin-boxes": "coin-box",
    "coinages": "coinage",
    "coincided": "coincide",
    "coincidences": "coincidence",
    "coincides": "coincide",
    "coinciding": "coincide",
    "coined": "coin",
    "coining": "coin",
    "coins": "coin",
    "cointegrations": "cointegration",
    "coituses": "coitus",
    "coked": "coke",
    "cokes": "coke",
    "coking": "coke",
    "cola": "colon",
    "colanders": "colander",
    "colas": "cola",
    "colchicines": "colchicine",
    "cold-shouldered": "cold-shoulder",
    "cold-shouldering": "cold-shoulder",
    "cold-shoulders": "cold-shoulder",
    "colder": "cold",
    "coldest": "cold",
    "colds": "cold",
    "colectomies": "colectomy",
    "colics": "colic",
    "coliforms": "coliform",
    "colins": "colin",
    "colitides": "colitis",
    "collaborated": "collaborate",
    "collaborates": "collaborate",
    "collaborating": "collaborate",
    "collaborations": "collaboration",
    "collaborators": "collaborator",
    "collagens": "collagen",
    "collages": "collage",
    "collapsed": "collapse",
    "collapses": "collapse",
    "collapsing": "collapse",
    "collarbones": "collarbone",
    "collared": "collar",
    "collaring": "collar",
    "collars": "collar",
    "collated": "collate",
    "collaterals": "collateral",
    "collates": "collate",
    "collating": "collate",
    "collations": "collation",
    "collators": "collator",
    "colleagues": "colleague",
    "collectables": "collectable",
    "collected": "collect",
    "collectibles": "collectible",
    "collecting": "collect",
    "collections": "collection",
    "collectives": "collective",
    "collectivised": "collectivise",
    "collectivises": "collectivise",
    "collectivising": "collectivise",
    "collectivists": "collectivist",
    "collectivities": "collectivity",
    "collectivizations": "collectivization",
    "collectivized": "collectivize",
    "collectivizes": "collectivize",
    "collectivizing": "collectivize",
    "collectors": "collector",
    "collects": "collect",
    "colleens": "colleen",
    "colleges": "college",
    "collided": "collide",
    "colliders": "collider",
    "collides": "collide",
    "colliding": "collide",
    "collieries": "colliery",
    "colliers": "collier",
    "collies": "collie",
    "collimated": "collimate",
    "collimates": "collimate",
    "collimating": "collimate",
    "collimations": "collimation",
    "collimators": "collimator",
    "collisions": "collision",
    "collocated": "collocate",
    "collocates": "collocate",
    "collocating": "collocate",
    "collocations": "collocation",
    "colloids": "colloid",
    "colloquia": "colloquium",
    "colloquialisms": "colloquialism",
    "colloquies": "colloquy",
    "colloquiums": "colloquium",
    "colls": "coll",
    "colluded": "collude",
    "colludes": "collude",
    "colluding": "collude",
    "collusions": "collusion",
    "colobomas": "coloboma",
    "colobomata": "coloboma",
    "colobuses": "colobus",
    "colognes": "cologne",
    "colonels": "colonel",
    "colonialists": "colonialist",
    "colonials": "colonial",
    "colonies": "colony",
    "colonisations": "colonisation",
    "colonised": "colonise",
    "colonisers": "coloniser",
    "colonises": "colonise",
    "colonising": "colonise",
    "colonists": "colonist",
    "colonizations": "colonization",
    "colonized": "colonize",
    "colonizers": "colonizer",
    "colonizes": "colonize",
    "colonizing": "colonize",
    "colonnades": "colonnade",
    "colonoscopies": "colonoscopy",
    "colons": "colon",
    "colophonies": "colophony",
    "colorants": "colorant",
    "colorations": "coloration",
    "colorcasted": "colorcast",
    "colorcasting": "colorcast",
    "colorcasts": "colorcast",
    "colored": "color",
    "colorimeters": "colorimeter",
    "coloring": "color",
    "colorized": "colorize",
    "colorizes": "colorize",
    "colorizing": "colorize",
    "colors": "color",
    "colossi": "colossus",
    "colossuses": "colossus",
    "colostomies": "colostomy",
    "colostra": "colostrum",
    "colostrums": "colostrum",
    "colourants": "colourant",
    "colourations": "colouration",
    "coloured": "colour",
    "coloureds": "coloured",
    "colouring": "colour",
    "colours": "colour",
    "colposcopies": "colposcopy",
    "cols": "col",
    "cols.": "col.",
    "colts": "colt",
    "columbines": "columbine",
    "columnists": "columnist",
    "columns": "column",
    "comae": "coma",
    "comas": "coma",
    "combatants": "combatant",
    "combated": "combat",
    "combating": "combat",
    "combats": "combat",
    "combatted": "combat",
    "combatting": "combat",
    "combed": "comb",
    "combes": "combe",
    "combinations": "combination",
    "combined": "combine",
    "combiners": "combiner",
    "combines": "combine",
    "combing": "comb",
    "combining": "combine",
    "combos": "combo",
    "combs": "comb",
    "combusted": "combust",
    "combustibles": "combustible",
    "combusting": "combust",
    "combustions": "combustion",
    "combustors": "combustor",
    "combusts": "combust",
    "come-backs": "come-back",
    "come-ons": "come-on",
    "comebacks": "comeback",
    "comedians": "comedian",
    "comediennes": "comedienne",
    "comedies": "comedy",
    "comedowns": "comedown",
    "comelier": "comely",
    "comeliest": "comely",
    "comers": "comer",
    "comes": "come",
    "comets": "comet",
    "comfier": "comfy",
    "comfiest": "comfy",
    "comforted": "comfort",
    "comforters": "comforter",
    "comforting": "comfort",
    "comforts": "comfort",
    "comics": "comic",
    "coming": "come",
    "comings": "coming",
    "comings-of-age": "coming-of-age",
    "comities": "comity",
    "commandants": "commandant",
    "commanded": "command",
    "commandeered": "commandeer",
    "commandeering": "commandeer",
    "commandeers": "commandeer",
    "commanders": "commander",
    "commanders-in-chief": "commander-in-chief",
    "commanding": "command",
    "commandments": "commandment",
    "commandoes": "commando",
    "commandos": "commando",
    "commands": "command",
    "commas": "comma",
    "commemorated": "commemorate",
    "commemorates": "commemorate",
    "commemorating": "commemorate",
    "commemorations": "commemoration",
    "commenced": "commence",
    "commencements": "commencement",
    "commences": "commence",
    "commencing": "commence",
    "commendations": "commendation",
    "commended": "commend",
    "commending": "commend",
    "commends": "commend",
    "commentaries": "commentary",
    "commentated": "commentate",
    "commentates": "commentate",
    "commentating": "commentate",
    "commentators": "commentator",
    "commented": "comment",
    "commenting": "comment",
    "comments": "comment",
    "commerces": "commerce",
    "commercialised": "commercialise",
    "commercialises": "commercialise",
    "commercialising": "commercialise",
    "commercialized": "commercialize",
    "commercializes": "commercialize",
    "commercializing": "commercialize",
    "commercials": "commercial",
    "commies": "commie",
    "commingled": "commingle",
    "commingles": "commingle",
    "commingling": "commingle",
    "commiserated": "commiserate",
    "commiserates": "commiserate",
    "commiserating": "commiserate",
    "commiserations": "commiseration",
    "commissariats": "commissariat",
    "commissaries": "commissary",
    "commissars": "commissar",
    "commissionaires": "commissionaire",
    "commissioned": "commission",
    "commissioners": "commissioner",
    "commissioning": "commission",
    "commissions": "commission",
    "commitments": "commitment",
    "commits": "commit",
    "committals": "committal",
    "committed": "commit",
    "committeemen": "committeeman",
    "committees": "committee",
    "committeewomen": "committeewoman",
    "committing": "commit",
    "commodes": "commode",
    "commodified": "commodify",
    "commodifies": "commodify",
    "commodifying": "commodify",
    "commodities": "commodity",
    "commoditisations": "commoditisation",
    "commodores": "commodore",
    "commonalities": "commonality",
    "commonalties": "commonalty",
    "commoner": "common",
    "commoners": "commoner",
    "commonest": "common",
    "commonner": "common",
    "commonnest": "common",
    "commonplaces": "commonplace",
    "commonrooms": "commonroom",
    "commons": "common",
    "commonwealths": "commonwealth",
    "commotions": "commotion",
    "communed": "commune",
    "communes": "commune",
    "communicants": "communicant",
    "communicated": "communicate",
    "communicates": "communicate",
    "communicating": "communicate",
    "communications": "communication",
    "communicators": "communicator",
    "communing": "commune",
    "communions": "communion",
    "communiques": "communique",
    "communiqués": "communiqué",
    "communists": "communist",
    "communitarians": "communitarian",
    "communities": "community",
    "commutations": "commutation",
    "commutators": "commutator",
    "commuted": "commute",
    "commuters": "commuter",
    "commutes": "commute",
    "commuting": "commute",
    "comorbidities": "comorbidity",
    "compacted": "compact",
    "compacter": "compact",
    "compactest": "compact",
    "compacting": "compact",
    "compactors": "compactor",
    "compacts": "compact",
    "companies": "company",
    "companions": "companion",
    "companionways": "companionway",
    "comparatives": "comparative",
    "comparators": "comparator",
    "compared": "compare",
    "compares": "compare",
    "comparing": "compare",
    "comparisons": "comparison",
    "compartmentalisations": "compartmentalisation",
    "compartmentalised": "compartmentalise",
    "compartmentalises": "compartmentalise",
    "compartmentalising": "compartmentalise",
    "compartmentalized": "compartmentalize",
    "compartmentalizes": "compartmentalize",
    "compartmentalizing": "compartmentalize",
    "compartments": "compartment",
    "compassed": "compass",
    "compasses": "compass",
    "compassing": "compass",
    "compatibilities": "compatibility",
    "compatibles": "compatible",
    "compatriots": "compatriot",
    "compelled": "compel",
    "compelling": "compel",
    "compels": "compel",
    "compendia": "compendium",
    "compendiums": "compendium",
    "compensated": "compensate",
    "compensates": "compensate",
    "compensating": "compensate",
    "compensations": "compensation",
    "compensators": "compensator",
    "compered": "compere",
    "comperes": "compere",
    "compering": "compere",
    "competed": "compete",
    "competences": "competence",
    "competencies": "competency",
    "competes": "compete",
    "competing": "compete",
    "competitions": "competition",
    "competitives": "competitive",
    "competitors": "competitor",
    "compilations": "compilation",
    "compiled": "compile",
    "compilers": "compiler",
    "compiles": "compile",
    "compiling": "compile",
    "complainants": "complainant",
    "complained": "complain",
    "complainers": "complainer",
    "complaining": "complain",
    "complains": "complain",
    "complaints": "complaint",
    "complementarities": "complementarity",
    "complementations": "complementation",
    "complemented": "complement",
    "complementing": "complement",
    "complements": "complement",
    "completed": "complete",
    "completer": "complete",
    "completers": "completer",
    "completes": "complete",
    "completest": "complete",
    "completing": "complete",
    "completions": "completion",
    "complexed": "complex",
    "complexes": "complex",
    "complexing": "complex",
    "complexions": "complexion",
    "complexities": "complexity",
    "compliances": "compliance",
    "complicated": "complicate",
    "complicates": "complicate",
    "complicating": "complicate",
    "complications": "complication",
    "complied": "comply",
    "complies": "comply",
    "complimented": "compliment",
    "complimenting": "compliment",
    "compliments": "compliment",
    "complying": "comply",
    "components": "component",
    "comported": "comport",
    "comporting": "comport",
    "comports": "comport",
    "compos": "compo",
    "composed": "compose",
    "composers": "composer",
    "composes": "compose",
    "composing": "compose",
    "composited": "composite",
    "composites": "composite",
    "compositing": "composite",
    "compositions": "composition",
    "compositors": "compositor",
    "composted": "compost",
    "composting": "compost",
    "composts": "compost",
    "composures": "composure",
    "compotes": "compote",
    "compounded": "compound",
    "compounders": "compounder",
    "compounding": "compound",
    "compounds": "compound",
    "comprehended": "comprehend",
    "comprehending": "comprehend",
    "comprehends": "comprehend",
    "comprehensions": "comprehension",
    "comprehensives": "comprehensive",
    "compressed": "compress",
    "compresses": "compress",
    "compressibilities": "compressibility",
    "compressing": "compress",
    "compressions": "compression",
    "compressors": "compressor",
    "comprised": "comprise",
    "comprises": "comprise",
    "comprising": "comprise",
    "compromised": "compromise",
    "compromises": "compromise",
    "compromising": "compromise",
    "comptrollers": "comptroller",
    "compulsions": "compulsion",
    "computations": "computation",
    "computed": "compute",
    "computerised": "computerise",
    "computerises": "computerise",
    "computerising": "computerise",
    "computerized": "computerize",
    "computerizes": "computerize",
    "computerizing": "computerize",
    "computers": "computer",
    "computes": "compute",
    "computing": "compute",
    "compères": "compère",
    "comrades": "comrade",
    "concatenated": "concatenate",
    "concatenates": "concatenate",
    "concatenating": "concatenate",
    "concatenations": "concatenation",
    "concavities": "concavity",
    "concealed": "conceal",
    "concealing": "conceal",
    "conceals": "conceal",
    "conceded": "concede",
    "concedes": "concede",
    "conceding": "concede",
    "conceits": "conceit",
    "conceived": "conceive",
    "conceives": "conceive",
    "conceiving": "conceive",
    "concentrated": "concentrate",
    "concentrates": "concentrate",
    "concentrating": "concentrate",
    "concentrations": "concentration",
    "concentrators": "concentrator",
    "conceptions": "conception",
    "concepts": "concept",
    "conceptualisations": "conceptualisation",
    "conceptualised": "conceptualise",
    "conceptualises": "conceptualise",
    "conceptualising": "conceptualise",
    "conceptualizations": "conceptualization",
    "conceptualized": "conceptualize",
    "conceptualizes": "conceptualize",
    "conceptualizing": "conceptualize",
    "concerned": "concern",
    "concerning": "concern",
    "concerns": "concern",
    "concert-goers": "concert-goer",
    "concerti": "concerto",
    "concertinaed": "concertina",
    "concertinaing": "concertina",
    "concertinas": "concertina",
    "concertmasters": "concertmaster",
    "concertos": "concerto",
    "concerts": "concert",
    "concessionaires": "concessionaire",
    "concessions": "concession",
    "conches": "conch",
    "conchs": "conch",
    "concierges": "concierge",
    "conciliated": "conciliate",
    "conciliates": "conciliate",
    "conciliating": "conciliate",
    "conciliators": "conciliator",
    "conclaves": "conclave",
    "concluded": "conclude",
    "concludes": "conclude",
    "concluding": "conclude",
    "conclusions": "conclusion",
    "concocted": "concoct",
    "concocting": "concoct",
    "concoctions": "concoction",
    "concocts": "concoct",
    "concomitants": "concomitant",
    "concordances": "concordance",
    "concordats": "concordat",
    "concords": "concord",
    "concourses": "concourse",
    "concreted": "concrete",
    "concretes": "concrete",
    "concreting": "concrete",
    "concretions": "concretion",
    "concs": "conc",
    "concubines": "concubine",
    "concurred": "concur",
    "concurrences": "concurrence",
    "concurrencies": "concurrency",
    "concurring": "concur",
    "concurs": "concur",
    "concussed": "concuss",
    "concusses": "concuss",
    "concussing": "concuss",
    "concussions": "concussion",
    "condemnations": "condemnation",
    "condemned": "condemn",
    "condemning": "condemn",
    "condemns": "condemn",
    "condensates": "condensate",
    "condensations": "condensation",
    "condensed": "condense",
    "condensers": "condenser",
    "condenses": "condense",
    "condensing": "condense",
    "condescended": "condescend",
    "condescending": "condescend",
    "condescends": "condescend",
    "condescensions": "condescension",
    "condiments": "condiment",
    "conditionalities": "conditionality",
    "conditionals": "conditional",
    "conditioned": "condition",
    "conditioners": "conditioner",
    "conditioning": "condition",
    "conditionings": "conditioning",
    "conditions": "condition",
    "condoled": "condole",
    "condolences": "condolence",
    "condoles": "condole",
    "condoling": "condole",
    "condominiums": "condominium",
    "condoms": "condom",
    "condoned": "condone",
    "condones": "condone",
    "condoning": "condone",
    "condors": "condor",
    "conduced": "conduce",
    "conduces": "conduce",
    "conducing": "conduce",
    "conductances": "conductance",
    "conducted": "conduct",
    "conducting": "conduct",
    "conductions": "conduction",
    "conductivities": "conductivity",
    "conductors": "conductor",
    "conductresses": "conductress",
    "conducts": "conduct",
    "conduits": "conduit",
    "condyles": "condyle",
    "coned": "cone",
    "cones": "cone",
    "confectionaries": "confectionary",
    "confectioneries": "confectionery",
    "confectioners": "confectioner",
    "confections": "confection",
    "confederacies": "confederacy",
    "confederates": "confederate",
    "confederations": "confederation",
    "conferences": "conference",
    "conferments": "conferment",
    "conferred": "confer",
    "conferring": "confer",
    "confers": "confer",
    "confessed": "confess",
    "confesses": "confess",
    "confessing": "confess",
    "confessionals": "confessional",
    "confessions": "confession",
    "confessors": "confessor",
    "confidantes": "confidante",
    "confidants": "confidant",
    "confided": "confide",
    "confidences": "confidence",
    "confidentialities": "confidentiality",
    "confides": "confide",
    "confiding": "confide",
    "configurations": "configuration",
    "configured": "configure",
    "configures": "configure",
    "configuring": "configure",
    "confined": "confine",
    "confinements": "confinement",
    "confines": "confine",
    "confining": "confine",
    "confirmations": "confirmation",
    "confirmed": "confirm",
    "confirming": "confirm",
    "confirms": "confirm",
    "confiscated": "confiscate",
    "confiscates": "confiscate",
    "confiscating": "confiscate",
    "confiscations": "confiscation",
    "conflagrations": "conflagration",
    "conflated": "conflate",
    "conflates": "conflate",
    "conflating": "conflate",
    "conflicted": "conflict",
    "conflicting": "conflict",
    "conflicts": "conflict",
    "confluences": "confluence",
    "conformations": "conformation",
    "conformed": "conform",
    "conformers": "conformer",
    "conforming": "conform",
    "conformists": "conformist",
    "conformities": "conformity",
    "conforms": "conform",
    "confounded": "confound",
    "confounders": "confounder",
    "confounding": "confound",
    "confounds": "confound",
    "confraternities": "confraternity",
    "confrontations": "confrontation",
    "confronted": "confront",
    "confronting": "confront",
    "confronts": "confront",
    "confused": "confuse",
    "confuses": "confuse",
    "confusing": "confuse",
    "confusions": "confusion",
    "confuted": "confute",
    "confutes": "confute",
    "confuting": "confute",
    "congealed": "congeal",
    "congealing": "congeal",
    "congeals": "congeal",
    "congeners": "congener",
    "congers": "conger",
    "congested": "congest",
    "congesting": "congest",
    "congestions": "congestion",
    "congests": "congest",
    "conglomerates": "conglomerate",
    "conglomerations": "conglomeration",
    "congratulated": "congratulate",
    "congratulates": "congratulate",
    "congratulating": "congratulate",
    "congratulations": "congratulation",
    "congregants": "congregant",
    "congregated": "congregate",
    "congregates": "congregate",
    "congregating": "congregate",
    "congregations": "congregation",
    "congresses": "congress",
    "congressmen": "congressman",
    "congresswomen": "congresswoman",
    "congruences": "congruence",
    "congruities": "congruity",
    "conidia": "conidium",
    "conifers": "conifer",
    "coning": "cone",
    "conjectured": "conjecture",
    "conjectures": "conjecture",
    "conjecturing": "conjecture",
    "conjoined": "conjoin",
    "conjoining": "conjoin",
    "conjoins": "conjoin",
    "conjugated": "conjugate",
    "conjugates": "conjugate",
    "conjugating": "conjugate",
    "conjugations": "conjugation",
    "conjunctions": "conjunction",
    "conjunctivae": "conjunctiva",
    "conjunctivas": "conjunctiva",
    "conjunctivitides": "conjunctivitis",
    "conjuncts": "conjunct",
    "conjured": "conjure",
    "conjurers": "conjurer",
    "conjures": "conjure",
    "conjuring": "conjure",
    "conjurors": "conjuror",
    "conked": "conk",
    "conkers": "conker",
    "conking": "conk",
    "conks": "conk",
    "conmen": "conman",
    "connected": "connect",
    "connecting": "connect",
    "connectionists": "connectionist",
    "connections": "connection",
    "connectives": "connective",
    "connectivities": "connectivity",
    "connectors": "connector",
    "connects": "connect",
    "conned": "con",
    "connexions": "connexion",
    "conning": "con",
    "connived": "connive",
    "connives": "connive",
    "conniving": "connive",
    "connoisseurs": "connoisseur",
    "connotations": "connotation",
    "connoted": "connote",
    "connotes": "connote",
    "connoting": "connote",
    "conns": "conn",
    "conquered": "conquer",
    "conquering": "conquer",
    "conquerors": "conqueror",
    "conquers": "conquer",
    "conquests": "conquest",
    "conquistadores": "conquistador",
    "conquistadors": "conquistador",
    "cons": "con",
    "consanguinities": "consanguinity",
    "consciences": "conscience",
    "consciousnesses": "consciousness",
    "conscripted": "conscript",
    "conscripting": "conscript",
    "conscripts": "conscript",
    "consecrated": "consecrate",
    "consecrates": "consecrate",
    "consecrating": "consecrate",
    "consensuses": "consensus",
    "consented": "consent",
    "consenting": "consent",
    "consents": "consent",
    "consequences": "consequence",
    "consequentialists": "consequentialist",
    "conservancies": "conservancy",
    "conservationists": "conservationist",
    "conservations": "conservation",
    "conservatives": "conservative",
    "conservatories": "conservatory",
    "conservators": "conservator",
    "conserved": "conserve",
    "conservers": "conserver",
    "conserves": "conserve",
    "conserving": "conserve",
    "considerations": "consideration",
    "considered": "consider",
    "considering": "consider",
    "considers": "consider",
    "consigned": "consign",
    "consignees": "consignee",
    "consigning": "consign",
    "consignments": "consignment",
    "consigns": "consign",
    "consisted": "consist",
    "consisteded": "consisted",
    "consisteding": "consisted",
    "consisteds": "consisted",
    "consistencies": "consistency",
    "consisting": "consist",
    "consists": "consist",
    "consolations": "consolation",
    "consoled": "console",
    "consoles": "console",
    "consolidated": "consolidate",
    "consolidates": "consolidate",
    "consolidating": "consolidate",
    "consolidations": "consolidation",
    "consolidators": "consolidator",
    "consoling": "console",
    "consommes": "consomme",
    "consonances": "consonance",
    "consonants": "consonant",
    "consorted": "consort",
    "consortia": "consortium",
    "consorting": "consort",
    "consortiums": "consortium",
    "consorts": "consort",
    "conspecifics": "conspecific",
    "conspiracies": "conspiracy",
    "conspirators": "conspirator",
    "conspired": "conspire",
    "conspires": "conspire",
    "conspiring": "conspire",
    "constables": "constable",
    "constabularies": "constabulary",
    "constancies": "constancy",
    "constants": "constant",
    "constellations": "constellation",
    "constipated": "constipate",
    "constipates": "constipate",
    "constipating": "constipate",
    "constituencies": "constituency",
    "constituents": "constituent",
    "constituted": "constitute",
    "constitutes": "constitute",
    "constituting": "constitute",
    "constitutionals": "constitutional",
    "constitutions": "constitution",
    "constrained": "constrain",
    "constraining": "constrain",
    "constrains": "constrain",
    "constraints": "constraint",
    "constricted": "constrict",
    "constricting": "constrict",
    "constrictions": "constriction",
    "constrictors": "constrictor",
    "constricts": "constrict",
    "construals": "construal",
    "constructed": "construct",
    "constructing": "construct",
    "constructionists": "constructionist",
    "constructions": "construction",
    "constructivists": "constructivist",
    "constructors": "constructor",
    "constructs": "construct",
    "construed": "construe",
    "construes": "construe",
    "construing": "construe",
    "consts": "const",
    "consulates": "consulate",
    "consuls": "consul",
    "consulships": "consulship",
    "consultancies": "consultancy",
    "consultants": "consultant",
    "consultations": "consultation",
    "consulted": "consult",
    "consultees": "consultee",
    "consulting": "consult",
    "consults": "consult",
    "consumables": "consumable",
    "consumed": "consume",
    "consumerisms": "consumerism",
    "consumerists": "consumerist",
    "consumers": "consumer",
    "consumes": "consume",
    "consuming": "consume",
    "consummated": "consummate",
    "consummates": "consummate",
    "consummating": "consummate",
    "consummations": "consummation",
    "consumptions": "consumption",
    "consumptives": "consumptive",
    "contacted": "contact",
    "contactees": "contactee",
    "contacting": "contact",
    "contactors": "contactor",
    "contacts": "contact",
    "contagions": "contagion",
    "contained": "contain",
    "containerised": "containerise",
    "containerises": "containerise",
    "containerising": "containerise",
    "containers": "container",
    "containing": "contain",
    "containments": "containment",
    "contains": "contain",
    "contaminants": "contaminant",
    "contaminated": "contaminate",
    "contaminates": "contaminate",
    "contaminating": "contaminate",
    "contaminations": "contamination",
    "contemned": "contemn",
    "contemning": "contemn",
    "contemns": "contemn",
    "contemplated": "contemplate",
    "contemplates": "contemplate",
    "contemplating": "contemplate",
    "contemplations": "contemplation",
    "contemporaries": "contemporary",
    "contempts": "contempt",
    "contended": "contend",
    "contenders": "contender",
    "contending": "contend",
    "contends": "contend",
    "contented": "content",
    "contenting": "content",
    "contentions": "contention",
    "contents": "content",
    "contestants": "contestant",
    "contested": "contest",
    "contesting": "contest",
    "contests": "contest",
    "contexts": "context",
    "contextualised": "contextualise",
    "contextualises": "contextualise",
    "contextualising": "contextualise",
    "contextualized": "contextualize",
    "contextualizes": "contextualize",
    "contextualizing": "contextualize",
    "contigs": "contig",
    "contiguities": "contiguity",
    "continentals": "continental",
    "continents": "continent",
    "contingencies": "contingency",
    "contingents": "contingent",
    "continua": "continuum",
    "continuances": "continuance",
    "continuations": "continuation",
    "continued": "continue",
    "continues": "continue",
    "continuing": "continue",
    "continuities": "continuity",
    "continuums": "continuum",
    "contorted": "contort",
    "contorting": "contort",
    "contortionists": "contortionist",
    "contortions": "contortion",
    "contorts": "contort",
    "contoured": "contour",
    "contouring": "contour",
    "contours": "contour",
    "contra-indicated": "contra-indicate",
    "contra-indicates": "contra-indicate",
    "contra-indicating": "contra-indicate",
    "contra-indications": "contra-indication",
    "contraceptions": "contraception",
    "contraceptives": "contraceptive",
    "contracted": "contract",
    "contractilities": "contractility",
    "contracting": "contract",
    "contractions": "contraction",
    "contractors": "contractor",
    "contracts": "contract",
    "contractures": "contracture",
    "contradicted": "contradict",
    "contradicting": "contradict",
    "contradictions": "contradiction",
    "contradictories": "contradictory",
    "contradicts": "contradict",
    "contraindicated": "contraindicate",
    "contraindicates": "contraindicate",
    "contraindicating": "contraindicate",
    "contraindications": "contraindication",
    "contralti": "contralto",
    "contraltos": "contralto",
    "contraptions": "contraption",
    "contrarians": "contrarian",
    "contraries": "contrary",
    "contras": "contra",
    "contrasted": "contrast",
    "contrasting": "contrast",
    "contrasts": "contrast",
    "contravened": "contravene",
    "contravenes": "contravene",
    "contravening": "contravene",
    "contraventions": "contravention",
    "contributed": "contribute",
    "contributes": "contribute",
    "contributing": "contribute",
    "contributions": "contribution",
    "contributors": "contributor",
    "contrivances": "contrivance",
    "contrived": "contrive",
    "contrives": "contrive",
    "contriving": "contrive",
    "controled": "control",
    "controling": "control",
    "controllabilities": "controllability",
    "controlled": "control",
    "controllers": "controller",
    "controlling": "control",
    "controls": "control",
    "controversialists": "controversialist",
    "controversies": "controversy",
    "controverted": "controvert",
    "controverting": "controvert",
    "controverts": "controvert",
    "conts": "cont",
    "contusions": "contusion",
    "conundrums": "conundrum",
    "conurbations": "conurbation",
    "conures": "conure",
    "convalesced": "convalesce",
    "convalescences": "convalescence",
    "convalescents": "convalescent",
    "convalesces": "convalesce",
    "convalescing": "convalesce",
    "convectors": "convector",
    "convened": "convene",
    "conveners": "convener",
    "convenes": "convene",
    "convenienced": "convenience",
    "conveniences": "convenience",
    "conveniencing": "convenience",
    "convening": "convene",
    "convenors": "convenor",
    "conventionalities": "conventionality",
    "conventions": "convention",
    "convents": "convent",
    "converged": "converge",
    "convergences": "convergence",
    "converges": "converge",
    "converging": "converge",
    "conversationalists": "conversationalist",
    "conversations": "conversation",
    "conversed": "converse",
    "converses": "converse",
    "conversing": "converse",
    "conversions": "conversion",
    "converted": "convert",
    "converters": "converter",
    "convertibles": "convertible",
    "converting": "convert",
    "convertors": "convertor",
    "converts": "convert",
    "convexities": "convexity",
    "conveyances": "conveyance",
    "conveyed": "convey",
    "conveyers": "conveyer",
    "conveying": "convey",
    "conveyors": "conveyor",
    "conveys": "convey",
    "convicted": "convict",
    "convicting": "convict",
    "convictions": "conviction",
    "convicts": "convict",
    "convinced": "convince",
    "convinces": "convince",
    "convincing": "convince",
    "convivialities": "conviviality",
    "convocations": "convocation",
    "convoked": "convoke",
    "convokes": "convoke",
    "convoking": "convoke",
    "convoluted": "convolute",
    "convolutes": "convolute",
    "convoluting": "convolute",
    "convolutions": "convolution",
    "convolved": "convolve",
    "convolves": "convolve",
    "convolving": "convolve",
    "convoys": "convoy",
    "convulsed": "convulse",
    "convulses": "convulse",
    "convulsing": "convulse",
    "convulsions": "convulsion",
    "cooed": "coo",
    "cooing": "coo",
    "cookbooks": "cookbook",
    "cooked": "cook",
    "cookeries": "cookery",
    "cookers": "cooker",
    "cookies": "cooky",
    "cooking": "cook",
    "cooking-pots": "cooking-pot",
    "cooks": "cook",
    "coolants": "coolant",
    "cooled": "cool",
    "cooler": "cool",
    "coolers": "cooler",
    "coolest": "cool",
    "coolies": "cooly",
    "cooling": "cool",
    "cools": "cool",
    "coombes": "coombe",
    "coombs": "coomb",
    "coons": "coon",
    "cooped": "coop",
    "cooperages": "cooperage",
    "cooperated": "cooperate",
    "cooperates": "cooperate",
    "cooperating": "cooperate",
    "cooperations": "cooperation",
    "cooperatives": "cooperative",
    "cooperators": "cooperator",
    "cooping": "coop",
    "coops": "coop",
    "coopted": "coopt",
    "coopting": "coopt",
    "coopts": "coopt",
    "coordinated": "coordinate",
    "coordinates": "coordinate",
    "coordinating": "coordinate",
    "coordinations": "coordination",
    "coordinators": "coordinator",
    "coos": "coo",
    "coots": "coot",
    "cop-outs": "cop-out",
    "copayments": "copayment",
    "coped": "cope",
    "copepods": "copepod",
    "copes": "cope",
    "copied": "copy",
    "copiers": "copier",
    "copies": "copy",
    "coping": "cope",
    "copings": "coping",
    "copolymers": "copolymer",
    "copped": "cop",
    "coppers": "copper",
    "coppiced": "coppice",
    "coppices": "coppice",
    "coppicing": "coppice",
    "copping": "cop",
    "coprocessors": "coprocessor",
    "cops": "cop",
    "copses": "copse",
    "copulae": "copula",
    "copulas": "copula",
    "copulated": "copulate",
    "copulates": "copulate",
    "copulating": "copulate",
    "copulations": "copulation",
    "copycats": "copycat",
    "copying": "copy",
    "copyists": "copyist",
    "copyrighted": "copyright",
    "copyrighting": "copyright",
    "copyrights": "copyright",
    "copywriters": "copywriter",
    "coquettes": "coquette",
    "coracles": "coracle",
    "corals": "coral",
    "corbeled": "corbel",
    "corbeling": "corbel",
    "corbelled": "corbel",
    "corbelling": "corbel",
    "corbels": "corbel",
    "corded": "cord",
    "cordialities": "cordiality",
    "cordials": "cordial",
    "cording": "cord",
    "cordoned": "cordon",
    "cordoning": "cordon",
    "cordons": "cordon",
    "cords": "cord",
    "corduroys": "corduroy",
    "cored": "core",
    "cores": "core",
    "corgis": "corgi",
    "coring": "core",
    "corkages": "corkage",
    "corked": "cork",
    "corkers": "corker",
    "corking": "cork",
    "corks": "cork",
    "corkscrews": "corkscrew",
    "cormorants": "cormorant",
    "corms": "corm",
    "cornea": "corneum",
    "corneae": "cornea",
    "corneas": "cornea",
    "corned": "corn",
    "corner-stones": "corner-stone",
    "cornered": "corner",
    "cornering": "corner",
    "corners": "corner",
    "cornerstones": "cornerstone",
    "cornets": "cornet",
    "cornfields": "cornfield",
    "cornflakes": "cornflake",
    "cornflowers": "cornflower",
    "cornices": "cornice",
    "cornier": "corny",
    "corniest": "corny",
    "corning": "corn",
    "corns": "corn",
    "cornstarches": "cornstarch",
    "cornucopias": "cornucopia",
    "corollaries": "corollary",
    "corollas": "corolla",
    "coronae": "corona",
    "coronaries": "coronary",
    "coronas": "corona",
    "coronations": "coronation",
    "coronaviruses": "coronavirus",
    "coroners": "coroner",
    "coronets": "coronet",
    "corpora": "corpus",
    "corporals": "corporal",
    "corporations": "corporation",
    "corps": "corp",
    "corpses": "corpse",
    "corpuscles": "corpuscle",
    "corpuses": "corpus",
    "corraled": "corral",
    "corraling": "corral",
    "corralled": "corral",
    "corralling": "corral",
    "corrals": "corral",
    "corrected": "correct",
    "correcting": "correct",
    "corrections": "correction",
    "correctives": "corrective",
    "correctors": "corrector",
    "corrects": "correct",
    "correlated": "correlate",
    "correlates": "correlate",
    "correlating": "correlate",
    "correlations": "correlation",
    "correlatives": "correlative",
    "correlators": "correlator",
    "corresponded": "correspond",
    "correspondences": "correspondence",
    "correspondents": "correspondent",
    "corresponding": "correspond",
    "corresponds": "correspond",
    "corridors": "corridor",
    "corries": "corrie",
    "corroborated": "corroborate",
    "corroborates": "corroborate",
    "corroborating": "corroborate",
    "corroborations": "corroboration",
    "corroded": "corrode",
    "corrodes": "corrode",
    "corroding": "corrode",
    "corrosions": "corrosion",
    "corrosives": "corrosive",
    "corrugated": "corrugate",
    "corrugateds": "corrugated",
    "corrugates": "corrugate",
    "corrugating": "corrugate",
    "corrugations": "corrugation",
    "corrupted": "corrupt",
    "corrupting": "corrupt",
    "corruptions": "corruption",
    "corrupts": "corrupt",
    "cors": "cor",
    "corsages": "corsage",
    "corseted": "corset",
    "corseting": "corset",
    "corsets": "corset",
    "corteges": "cortege",
    "cortexes": "cortex",
    "cortices": "cortex",
    "corticosteroids": "corticosteroid",
    "cortisols": "cortisol",
    "cortisones": "cortisone",
    "cortèges": "cortège",
    "coruscated": "coruscate",
    "coruscates": "coruscate",
    "coruscating": "coruscate",
    "corvids": "corvid",
    "cos": "co",
    "coshed": "cosh",
    "coshes": "cosh",
    "coshing": "cosh",
    "cosier": "cosy",
    "cosies": "cosy",
    "cosiest": "cosy",
    "cosignatories": "cosignatory",
    "cosines": "cosine",
    "cosmetics": "cosmetic",
    "cosmogonies": "cosmogony",
    "cosmologies": "cosmology",
    "cosmologists": "cosmologist",
    "cosmonauts": "cosmonaut",
    "cosmopolitans": "cosmopolitan",
    "cosmoses": "cosmos",
    "cosseted": "cosset",
    "cosseting": "cosset",
    "cossets": "cosset",
    "costae": "costa",
    "costed": "cost",
    "costing": "cost",
    "costings": "costing",
    "costlier": "costly",
    "costliest": "costly",
    "costs": "cost",
    "costumed": "costume",
    "costumes": "costume",
    "costumiers": "costumier",
    "costuming": "costume",
    "coteries": "coterie",
    "cotes": "cote",
    "cotinines": "cotinine",
    "cotoneasters": "cotoneaster",
    "cots": "cot",
    "cotswolds": "cotswold",
    "cottagers": "cottager",
    "cottages": "cottage",
    "cottas": "cotta",
    "cottoned": "cotton",
    "cottoning": "cotton",
    "cottons": "cotton",
    "cottonseeds": "cottonseed",
    "cotyledons": "cotyledon",
    "couched": "couch",
    "couches": "couch",
    "couchettes": "couchette",
    "couching": "couch",
    "cougars": "cougar",
    "coughed": "cough",
    "coughing": "cough",
    "coughs": "cough",
    "could": "can",
    "coulombs": "coulomb",
    "coumarins": "coumarin",
    "councillors": "councillor",
    "councilors": "councilor",
    "councils": "council",
    "councilwomen": "councilwoman",
    "counseled": "counsel",
    "counseling": "counsel",
    "counselled": "counsel",
    "counselling": "counsel",
    "counsellors": "counsellor",
    "counselors": "counselor",
    "counsels": "counsel",
    "countdowns": "countdown",
    "counted": "count",
    "countenanced": "countenance",
    "countenances": "countenance",
    "countenancing": "countenance",
    "counter-arguments": "counter-argument",
    "counter-attacked": "counter-attack",
    "counter-attacking": "counter-attack",
    "counter-attacks": "counter-attack",
    "counter-balanced": "counter-balance",
    "counter-balances": "counter-balance",
    "counter-balancing": "counter-balance",
    "counter-claims": "counter-claim",
    "counter-cultures": "counter-culture",
    "counter-espionages": "counter-espionage",
    "counter-examples": "counter-example",
    "counter-insurgencies": "counter-insurgency",
    "counter-ions": "counter-ion",
    "counter-measures": "counter-measure",
    "counter-proposals": "counter-proposal",
    "counter-revolutionaries": "counter-revolutionary",
    "counter-revolutions": "counter-revolution",
    "counteracted": "counteract",
    "counteracting": "counteract",
    "counteracts": "counteract",
    "counterattacked": "counterattack",
    "counterattacking": "counterattack",
    "counterattacks": "counterattack",
    "counterbalanced": "counterbalance",
    "counterbalances": "counterbalance",
    "counterbalancing": "counterbalance",
    "counterclaims": "counterclaim",
    "countercultures": "counterculture",
    "countered": "counter",
    "counterexamples": "counterexample",
    "counterfactuals": "counterfactual",
    "counterfeited": "counterfeit",
    "counterfeiters": "counterfeiter",
    "counterfeiting": "counterfeit",
    "counterfeits": "counterfeit",
    "counterfoils": "counterfoil",
    "countering": "counter",
    "counterinsurgencies": "counterinsurgency",
    "counterions": "counterion",
    "countermanded": "countermand",
    "countermanding": "countermand",
    "countermands": "countermand",
    "countermeasures": "countermeasure",
    "counterpanes": "counterpane",
    "counterparts": "counterpart",
    "counterpoints": "counterpoint",
    "counterposed": "counterpose",
    "counterposes": "counterpose",
    "counterposing": "counterpose",
    "counterrevolutions": "counterrevolution",
    "counters": "counter",
    "countersank": "countersink",
    "countersigned": "countersign",
    "countersigning": "countersign",
    "countersigns": "countersign",
    "countersinking": "countersink",
    "countersinks": "countersink",
    "countersunk": "countersink",
    "countertenors": "countertenor",
    "countervailed": "countervail",
    "countervailing": "countervail",
    "countervails": "countervail",
    "counterweights": "counterweight",
    "countesses": "countess",
    "counties": "county",
    "counting": "count",
    "countries": "country",
    "countrymen": "countryman",
    "countrywomen": "countrywoman",
    "counts": "count",
    "coupes": "coupe",
    "coupled": "couple",
    "couplers": "coupler",
    "couples": "couple",
    "couplets": "couplet",
    "coupling": "couple",
    "couplings": "coupling",
    "coupons": "coupon",
    "coups": "coup",
    "coupés": "coupé",
    "courages": "courage",
    "courgettes": "courgette",
    "couriers": "courier",
    "coursed": "course",
    "coursers": "courser",
    "courses": "course",
    "coursing": "course",
    "court-houses": "court-house",
    "court-martialed": "court-martial",
    "court-martialing": "court-martial",
    "court-martialled": "court-martial",
    "court-martialling": "court-martial",
    "court-martials": "court-martial",
    "courted": "court",
    "courtesans": "courtesan",
    "courtesies": "courtesy",
    "courthouses": "courthouse",
    "courtiers": "courtier",
    "courting": "court",
    "courtlier": "courtly",
    "courtliest": "courtly",
    "courtrooms": "courtroom",
    "courts": "court",
    "courts-martial": "court-martial",
    "courtships": "courtship",
    "courtyards": "courtyard",
    "cousins": "cousin",
    "couturiers": "couturier",
    "covariances": "covariance",
    "covariates": "covariate",
    "covenanted": "covenant",
    "covenanting": "covenant",
    "covenants": "covenant",
    "covens": "coven",
    "cover-ups": "cover-up",
    "coverages": "coverage",
    "covered": "cover",
    "covering": "cover",
    "coverings": "covering",
    "coverlets": "coverlet",
    "covers": "cover",
    "coverslips": "coverslip",
    "coverts": "covert",
    "coves": "cove",
    "coveted": "covet",
    "coveting": "covet",
    "covets": "covet",
    "cowards": "coward",
    "cowboys": "cowboy",
    "cowed": "cow",
    "cowered": "cower",
    "cowering": "cower",
    "cowers": "cower",
    "cowhands": "cowhand",
    "cowhides": "cowhide",
    "cowing": "cow",
    "cowls": "cowl",
    "cowmen": "cowman",
    "coworkers": "coworker",
    "cowpats": "cowpat",
    "cowpeas": "cowpea",
    "cowries": "cowrie",
    "cows": "cow",
    "cowsheds": "cowshed",
    "cowslips": "cowslip",
    "coxibs": "coxib",
    "coxs": "cox",
    "coxswains": "coxswain",
    "coyer": "coy",
    "coyest": "coy",
    "coyotes": "coyote",
    "cozier": "cozy",
    "cpds": "cpd",
    "cps": "cp",
    "cpus": "cpu",
    "crabbed": "crab",
    "crabbing": "crab",
    "crabs": "crab",
    "crack-ups": "crack-up",
    "crackdowns": "crackdown",
    "cracked": "crack",
    "crackers": "cracker",
    "cracking": "crack",
    "crackled": "crackle",
    "crackles": "crackle",
    "crackling": "crackle",
    "cracklings": "crackling",
    "crackpots": "crackpot",
    "cracks": "crack",
    "cradle-snatchers": "cradle-snatcher",
    "cradled": "cradle",
    "cradles": "cradle",
    "cradling": "cradle",
    "crafted": "craft",
    "craftier": "crafty",
    "craftiest": "crafty",
    "crafting": "craft",
    "crafts": "craft",
    "craftsmanships": "craftsmanship",
    "craftsmen": "craftsman",
    "craftspersons": "craftsperson",
    "craftswomen": "craftswoman",
    "craggier": "craggy",
    "craggiest": "craggy",
    "crags": "crag",
    "crakes": "crake",
    "crammed": "cram",
    "cramming": "cram",
    "crammings": "cramming",
    "cramped": "cramp",
    "cramping": "cramp",
    "crampons": "crampon",
    "cramps": "cramp",
    "crams": "cram",
    "cranberries": "cranberry",
    "craned": "crane",
    "craneflies": "cranefly",
    "cranes": "crane",
    "crania": "cranium",
    "craning": "crane",
    "craniotomies": "craniotomy",
    "craniums": "cranium",
    "crankcases": "crankcase",
    "cranked": "crank",
    "crankier": "cranky",
    "crankiest": "cranky",
    "cranking": "crank",
    "cranks": "crank",
    "crankshafts": "crankshaft",
    "crannies": "cranny",
    "crapped": "crap",
    "crappier": "crappy",
    "crappiest": "crappy",
    "crapping": "crap",
    "craps": "crap",
    "crash-landed": "crash-land",
    "crash-landing": "crash-land",
    "crash-landings": "crash-landing",
    "crash-lands": "crash-land",
    "crashed": "crash",
    "crashes": "crash",
    "crashing": "crash",
    "crasser": "crass",
    "crassest": "crass",
    "crated": "crate",
    "cratered": "crater",
    "cratering": "crater",
    "craters": "crater",
    "crates": "crate",
    "crating": "crate",
    "cravates": "cravate",
    "cravats": "cravat",
    "craved": "crave",
    "craves": "crave",
    "craving": "crave",
    "cravings": "craving",
    "crawfishes": "crawfish",
    "crawled": "crawl",
    "crawlers": "crawler",
    "crawling": "crawl",
    "crawls": "crawl",
    "craws": "craw",
    "crayfishes": "crayfish",
    "crayoned": "crayon",
    "crayoning": "crayon",
    "crayons": "crayon",
    "crazed": "craze",
    "crazes": "craze",
    "crazier": "crazy",
    "crazies": "crazy",
    "craziest": "crazy",
    "crazing": "craze",
    "creaked": "creak",
    "creakier": "creaky",
    "creakiest": "creaky",
    "creaking": "creak",
    "creaks": "creak",
    "creamed": "cream",
    "creamer": "cream",
    "creameries": "creamery",
    "creamest": "cream",
    "creamier": "creamy",
    "creamiest": "creamy",
    "creaming": "cream",
    "creams": "cream",
    "creased": "crease",
    "creases": "crease",
    "creasing": "crease",
    "created": "create",
    "creates": "create",
    "creatines": "creatine",
    "creating": "create",
    "creatinines": "creatinine",
    "creationists": "creationist",
    "creations": "creation",
    "creatives": "creative",
    "creators": "creator",
    "creatures": "creature",
    "creches": "creche",
    "credentials": "credential",
    "credibilities": "credibility",
    "credited": "credit",
    "crediting": "credit",
    "creditors": "creditor",
    "credits": "credit",
    "credos": "credo",
    "credulities": "credulity",
    "creeds": "creed",
    "creeks": "creek",
    "creels": "creel",
    "creepers": "creeper",
    "creepier": "creepy",
    "creepiest": "creepy",
    "creeping": "creep",
    "creeps": "creep",
    "creepy-crawlies": "creepy-crawly",
    "cremated": "cremate",
    "cremates": "cremate",
    "cremating": "cremate",
    "cremations": "cremation",
    "crematoria": "crematorium",
    "crematoriums": "crematorium",
    "creoles": "creole",
    "creosoted": "creosote",
    "creosotes": "creosote",
    "creosoting": "creosote",
    "crept": "creep",
    "crescendi": "crescendo",
    "crescendoes": "crescendo",
    "crescendos": "crescendo",
    "crescents": "crescent",
    "cresses": "cress",
    "crested": "crest",
    "cresting": "crest",
    "crests": "crest",
    "cretins": "cretin",
    "crevasses": "crevasse",
    "crevices": "crevice",
    "crew": "crow",
    "crew-necks": "crew-neck",
    "crewed": "crew",
    "crewing": "crew",
    "crewmembers": "crewmember",
    "crewmen": "crewman",
    "crews": "crew",
    "cribbed": "crib",
    "cribbing": "crib",
    "cribs": "crib",
    "cricked": "crick",
    "cricketers": "cricketer",
    "crickets": "cricket",
    "cricking": "crick",
    "cricks": "crick",
    "cried": "cry",
    "criers": "crier",
    "cries": "cry",
    "crimes": "crime",
    "criminalised": "criminalise",
    "criminalises": "criminalise",
    "criminalising": "criminalise",
    "criminalized": "criminalize",
    "criminalizes": "criminalize",
    "criminalizing": "criminalize",
    "criminals": "criminal",
    "criminologies": "criminology",
    "criminologists": "criminologist",
    "crimped": "crimp",
    "crimping": "crimp",
    "crimps": "crimp",
    "crimsoner": "crimson",
    "crimsonest": "crimson",
    "crimsons": "crimson",
    "cringed": "cringe",
    "cringes": "cringe",
    "cringing": "cringe",
    "crinkled": "crinkle",
    "crinkles": "crinkle",
    "crinklier": "crinkly",
    "crinkling": "crinkle",
    "crinoids": "crinoid",
    "crinolines": "crinoline",
    "crippled": "cripple",
    "cripples": "cripple",
    "crippling": "cripple",
    "crises": "crisis",
    "crispbreads": "crispbread",
    "crisped": "crisp",
    "crisper": "crisp",
    "crispest": "crisp",
    "crispier": "crispy",
    "crispiest": "crispy",
    "crisping": "crisp",
    "crisps": "crisp",
    "criss-crossed": "criss-cross",
    "criss-crosses": "criss-cross",
    "criss-crossing": "criss-cross",
    "crisscrossed": "crisscross",
    "crisscrosses": "crisscross",
    "crisscrossing": "crisscross",
    "criteria": "criterium",
    "criterions": "criterion",
    "criticised": "criticise",
    "criticises": "criticise",
    "criticising": "criticise",
    "criticisms": "criticism",
    "criticized": "criticize",
    "criticizes": "criticize",
    "criticizing": "criticize",
    "critics": "critic",
    "critiqued": "critique",
    "critiques": "critique",
    "critiquing": "critique",
    "critters": "critter",
    "croaked": "croak",
    "croaking": "croak",
    "croaks": "croak",
    "crocheted": "crochet",
    "crocheting": "crochet",
    "crochets": "crochet",
    "croci": "crocus",
    "crocidolites": "crocidolite",
    "crocks": "crock",
    "crocodiles": "crocodile",
    "crocuses": "crocus",
    "crofters": "crofter",
    "crofts": "croft",
    "croissants": "croissant",
    "crones": "crone",
    "cronies": "crony",
    "crooked": "crook",
    "crooking": "crook",
    "crooks": "crook",
    "crooned": "croon",
    "crooners": "crooner",
    "crooning": "croon",
    "croons": "croon",
    "croplands": "cropland",
    "cropped": "crop",
    "croppers": "cropper",
    "cropping": "crop",
    "crops": "crop",
    "croquets": "croquet",
    "croquettes": "croquette",
    "cross-bars": "cross-bar",
    "cross-bred": "cross-breed",
    "cross-breeding": "cross-breed",
    "cross-breeds": "cross-breed",
    "cross-checked": "cross-check",
    "cross-checking": "cross-check",
    "cross-checks": "cross-check",
    "cross-correlations": "cross-correlation",
    "cross-dressed": "cross-dress",
    "cross-dresses": "cross-dress",
    "cross-dressing": "cross-dress",
    "cross-examinations": "cross-examination",
    "cross-examined": "cross-examine",
    "cross-examines": "cross-examine",
    "cross-examining": "cross-examine",
    "cross-infections": "cross-infection",
    "cross-linked": "cross-link",
    "cross-linking": "cross-link",
    "cross-links": "cross-link",
    "cross-overs": "cross-over",
    "cross-pollinated": "cross-pollinate",
    "cross-pollinates": "cross-pollinate",
    "cross-pollinating": "cross-pollinate",
    "cross-pollinations": "cross-pollination",
    "cross-questioned": "cross-question",
    "cross-questioning": "cross-question",
    "cross-questions": "cross-question",
    "cross-referenced": "cross-reference",
    "cross-references": "cross-reference",
    "cross-referencing": "cross-reference",
    "cross-referred": "cross-refer",
    "cross-referring": "cross-refer",
    "cross-refers": "cross-refer",
    "cross-roads": "cross-road",
    "cross-sections": "cross-section",
    "cross-stitches": "cross-stitch",
    "cross-talks": "cross-talk",
    "cross-validations": "cross-validation",
    "crossbars": "crossbar",
    "crossbills": "crossbill",
    "crossbows": "crossbow",
    "crossbred": "crossbreed",
    "crossbreeding": "crossbreed",
    "crossbreeds": "crossbreed",
    "crosscountries": "crosscountry",
    "crossed": "cross",
    "crosser": "cross",
    "crossers": "crosser",
    "crosses": "crosse",
    "crossest": "cross",
    "crossfires": "crossfire",
    "crossing": "cross",
    "crossings": "crossing",
    "crossovers": "crossover",
    "crossroads": "crossroad",
    "crosstalks": "crosstalk",
    "crosswinds": "crosswind",
    "crosswords": "crossword",
    "crotches": "crotch",
    "crotchetier": "crotchety",
    "crotchetiest": "crotchety",
    "crotchets": "crotchet",
    "crouched": "crouch",
    "crouches": "crouch",
    "crouching": "crouch",
    "croupiers": "croupier",
    "croutons": "crouton",
    "crowbars": "crowbar",
    "crowded": "crowd",
    "crowding": "crowd",
    "crowds": "crowd",
    "crowed": "crow",
    "crowing": "crow",
    "crowned": "crown",
    "crowning": "crown",
    "crowns": "crown",
    "crows": "crow",
    "croziers": "crozier",
    "cruces": "crux",
    "crucians": "crucian",
    "crucibles": "crucible",
    "crucified": "crucify",
    "crucifies": "crucify",
    "crucifixes": "crucifix",
    "crucifixions": "crucifixion",
    "cruciforms": "cruciform",
    "crucifying": "crucify",
    "cruder": "crude",
    "crudes": "crude",
    "crudest": "crude",
    "crudities": "crudity",
    "crueler": "cruel",
    "cruelest": "cruel",
    "crueller": "cruel",
    "cruellest": "cruel",
    "cruelties": "cruelty",
    "cruets": "cruet",
    "cruised": "cruise",
    "cruisers": "cruiser",
    "cruises": "cruise",
    "cruising": "cruise",
    "crumbled": "crumble",
    "crumbles": "crumble",
    "crumblier": "crumbly",
    "crumbliest": "crumbly",
    "crumbling": "crumble",
    "crumbs": "crumb",
    "crummier": "crummy",
    "crummiest": "crummy",
    "crumpets": "crumpet",
    "crumpled": "crumple",
    "crumples": "crumple",
    "crumpling": "crumple",
    "crunched": "crunch",
    "crunchers": "cruncher",
    "crunches": "crunch",
    "crunchier": "crunchy",
    "crunchiest": "crunchy",
    "crunching": "crunch",
    "crura": "crus",
    "crusaded": "crusade",
    "crusaders": "crusader",
    "crusades": "crusade",
    "crusading": "crusade",
    "crushed": "crush",
    "crushers": "crusher",
    "crushes": "crush",
    "crushing": "crush",
    "crustaceans": "crustacean",
    "crusted": "crust",
    "crustier": "crusty",
    "crustiest": "crusty",
    "crusting": "crust",
    "crusts": "crust",
    "crutches": "crutch",
    "cruxes": "crux",
    "cry-babies": "cry-baby",
    "crying": "cry",
    "cryopreservations": "cryopreservation",
    "cryostats": "cryostat",
    "cryotherapies": "cryotherapy",
    "cryptograms": "cryptogram",
    "cryptographers": "cryptographer",
    "cryptos": "crypto",
    "cryptosporidia": "cryptosporidium",
    "cryptosporidioses": "cryptosporidiosis",
    "crypts": "crypt",
    "crystalised": "crystalise",
    "crystalises": "crystalise",
    "crystalising": "crystalise",
    "crystallinities": "crystallinity",
    "crystallisations": "crystallisation",
    "crystallised": "crystallise",
    "crystallises": "crystallise",
    "crystallising": "crystallise",
    "crystallites": "crystallite",
    "crystallizations": "crystallization",
    "crystallized": "crystallize",
    "crystallizes": "crystallize",
    "crystallizing": "crystallize",
    "crystallographers": "crystallographer",
    "crystallographies": "crystallography",
    "crystals": "crystal",
    "crèches": "crèche",
    "crêpes": "crêpe",
    "cs": "c",
    "cubbyholes": "cubbyhole",
    "cubed": "cube",
    "cubes": "cube",
    "cubicles": "cubicle",
    "cubing": "cube",
    "cubister": "cubist",
    "cubistest": "cubist",
    "cubists": "cubist",
    "cuboids": "cuboid",
    "cubs": "cub",
    "cuckolded": "cuckold",
    "cuckolding": "cuckold",
    "cuckolds": "cuckold",
    "cuckoos": "cuckoo",
    "cucumbers": "cucumber",
    "cucurbits": "cucurbit",
    "cuddled": "cuddle",
    "cuddles": "cuddle",
    "cuddlier": "cuddly",
    "cuddliest": "cuddly",
    "cuddling": "cuddle",
    "cudgelled": "cudgel",
    "cudgelling": "cudgel",
    "cudgels": "cudgel",
    "cuds": "cud",
    "cued": "cue",
    "cueing": "cue",
    "cues": "cue",
    "cuffed": "cuff",
    "cuffing": "cuff",
    "cufflinks": "cufflink",
    "cuffs": "cuff",
    "cuing": "cue",
    "cuirasses": "cuirass",
    "cuis": "cui",
    "cuisines": "cuisine",
    "cul-de-sacs": "cul-de-sac",
    "culled": "cull",
    "culling": "cull",
    "culls": "cull",
    "culminated": "culminate",
    "culminates": "culminate",
    "culminating": "culminate",
    "culminations": "culmination",
    "culms": "culm",
    "culpae": "culpa",
    "culprits": "culprit",
    "culs-de-sac": "cul-de-sac",
    "cultists": "cultist",
    "cultivars": "cultivar",
    "cultivated": "cultivate",
    "cultivates": "cultivate",
    "cultivating": "cultivate",
    "cultivations": "cultivation",
    "cultivators": "cultivator",
    "cults": "cult",
    "cultured": "culture",
    "cultures": "culture",
    "culturing": "culture",
    "culverts": "culvert",
    "cumins": "cumin",
    "cummerbunds": "cummerbund",
    "cummins": "cummin",
    "cumulated": "cumulate",
    "cumulates": "cumulate",
    "cumulating": "cumulate",
    "cumuli": "cumulus",
    "cumulonimbi": "cumulonimbus",
    "cumulonimbuses": "cumulonimbus",
    "cuneiforms": "cuneiform",
    "cuniculi": "cuniculus",
    "cunts": "cunt",
    "cup-ties": "cup-tie",
    "cupboards": "cupboard",
    "cupcakes": "cupcake",
    "cupfuls": "cupful",
    "cupids": "cupid",
    "cupolas": "cupola",
    "cuppas": "cuppa",
    "cupped": "cup",
    "cupping": "cup",
    "cuppings": "cupping",
    "cups": "cup",
    "cupsful": "cupful",
    "curacies": "curacy",
    "curated": "curate",
    "curates": "curate",
    "curating": "curate",
    "curatives": "curative",
    "curators": "curator",
    "curbed": "curb",
    "curbing": "curb",
    "curbs": "curb",
    "curdled": "curdle",
    "curdles": "curdle",
    "curdling": "curdle",
    "curds": "curd",
    "cure-alls": "cure-all",
    "cured": "cure",
    "cures": "cure",
    "curettages": "curettage",
    "curfews": "curfew",
    "curiae": "curia",
    "curies": "curie",
    "curing": "cure",
    "curios": "curio",
    "curiosities": "curiosity",
    "curled": "curl",
    "curlers": "curler",
    "curlews": "curlew",
    "curlier": "curly",
    "curliest": "curly",
    "curling": "curl",
    "curlings": "curling",
    "curls": "curl",
    "curmudgeons": "curmudgeon",
    "currants": "currant",
    "currencies": "currency",
    "currents": "current",
    "curricula": "curriculum",
    "curriculums": "curriculum",
    "curried": "curry",
    "curries": "curry",
    "currying": "curry",
    "curs": "cur",
    "cursed": "curse",
    "curses": "curse",
    "cursing": "curse",
    "cursors": "cursor",
    "curst": "curse",
    "curtailed": "curtail",
    "curtailing": "curtail",
    "curtailments": "curtailment",
    "curtails": "curtail",
    "curtain-raisers": "curtain-raiser",
    "curtained": "curtain",
    "curtaining": "curtain",
    "curtains": "curtain",
    "curter": "curt",
    "curtest": "curt",
    "curtseyed": "curtsey",
    "curtseying": "curtsey",
    "curtseys": "curtsey",
    "curtsied": "curtsy",
    "curtsies": "curtsy",
    "curtsying": "curtsy",
    "curvatures": "curvature",
    "curved": "curve",
    "curves": "curve",
    "curvier": "curvy",
    "curviest": "curvy",
    "curving": "curve",
    "cushier": "cushy",
    "cushiest": "cushy",
    "cushioned": "cushion",
    "cushioning": "cushion",
    "cushions": "cushion",
    "cusps": "cusp",
    "cussed": "cuss",
    "cusses": "cuss",
    "cussing": "cuss",
    "custards": "custard",
    "custodians": "custodian",
    "custodies": "custody",
    "customers": "customer",
    "customisations": "customisation",
    "customised": "customise",
    "customises": "customise",
    "customising": "customise",
    "customizations": "customization",
    "customized": "customize",
    "customizes": "customize",
    "customizing": "customize",
    "customs": "custom",
    "cut-backs": "cut-back",
    "cut-downs": "cut-down",
    "cut-offs": "cut-off",
    "cut-outs": "cut-out",
    "cut-throats": "cut-throat",
    "cutaways": "cutaway",
    "cutbacks": "cutback",
    "cuter": "cute",
    "cutest": "cute",
    "cuticles": "cuticle",
    "cutlasses": "cutlass",
    "cutlets": "cutlet",
    "cutoffs": "cutoff",
    "cutouts": "cutout",
    "cuts": "cut",
    "cutters": "cutter",
    "cutting": "cut",
    "cuttings": "cutting",
    "cuttlefishes": "cuttlefish",
    "cuvettes": "cuvette",
    "cyanides": "cyanide",
    "cyanobacteria": "cyanobacterium",
    "cyanoses": "cyanosis",
    "cycads": "cycad",
    "cyclamens": "cyclamen",
    "cyclases": "cyclase",
    "cycled": "cycle",
    "cycles": "cycle",
    "cycling": "cycle",
    "cyclings": "cycling",
    "cyclins": "cyclin",
    "cyclisations": "cyclisation",
    "cyclists": "cyclist",
    "cyclo-oxygenases": "cyclo-oxygenase",
    "cyclodextrins": "cyclodextrin",
    "cyclohexanes": "cyclohexane",
    "cyclones": "cyclone",
    "cyclophosphamides": "cyclophosphamide",
    "cyclosporines": "cyclosporine",
    "cyclosporins": "cyclosporin",
    "cyclotrons": "cyclotron",
    "cycs": "cyc",
    "cygnets": "cygnet",
    "cylinders": "cylinder",
    "cyls": "cyl",
    "cymbals": "cymbal",
    "cynicisms": "cynicism",
    "cynics": "cynic",
    "cyphers": "cypher",
    "cypresses": "cypress",
    "cystatins": "cystatin",
    "cystectomies": "cystectomy",
    "cysteines": "cysteine",
    "cystitides": "cystitis",
    "cystoscopies": "cystoscopy",
    "cysts": "cyst",
    "cytarabines": "cytarabine",
    "cytochromes": "cytochrome",
    "cytokines": "cytokine",
    "cytokineses": "cytokinesis",
    "cytologies": "cytology",
    "cytomegaloviruses": "cytomegalovirus",
    "cytometries": "cytometry",
    "cytoplasms": "cytoplasm",
    "cytosines": "cytosine",
    "cytoskeletons": "cytoskeleton",
    "cytosols": "cytosol",
    "cytotoxicities": "cytotoxicity",
    "cytotoxics": "cytotoxic",
    "czarinas": "czarina",
    "czars": "czar",
    "d": "have",
    "dabbed": "dab",
    "dabbing": "dab",
    "dabbled": "dabble",
    "dabblers": "dabbler",
    "dabbles": "dabble",
    "dabbling": "dabble",
    "dabs": "dab",
    "dachshunds": "dachshund",
    "dactyls": "dactyl",
    "daddies": "daddy",
    "dadoed": "dado",
    "dadoes": "dado",
    "dadoing": "dado",
    "dados": "dado",
    "dads": "dad",
    "daemons": "daemon",
    "daffodils": "daffodil",
    "dafter": "daft",
    "daftest": "daft",
    "daggers": "dagger",
    "dagos": "dago",
    "daguerreotypes": "daguerreotype",
    "dahlias": "dahlia",
    "dahs": "dah",
    "dailies": "daily",
    "daintier": "dainty",
    "daintiest": "dainty",
    "daiquiris": "daiquiri",
    "dairies": "dairy",
    "dairymaids": "dairymaid",
    "dairymen": "dairyman",
    "daises": "dais",
    "daisies": "daisy",
    "daisywheels": "daisywheel",
    "dales": "dale",
    "dalliances": "dalliance",
    "dallied": "dally",
    "dallies": "dally",
    "dallying": "dally",
    "dals": "dal",
    "daltons": "dalton",
    "damaged": "damage",
    "damages": "damage",
    "damaging": "damage",
    "damasks": "damask",
    "dames": "dame",
    "dammed": "dam",
    "damming": "dam",
    "damned": "damn",
    "damning": "damn",
    "damns": "damn",
    "damped": "damp",
    "dampened": "dampen",
    "dampeners": "dampener",
    "dampening": "dampen",
    "dampens": "dampen",
    "damper": "damp",
    "dampers": "damper",
    "dampest": "damp",
    "damping": "damp",
    "dampnesses": "dampness",
    "damps": "damp",
    "dams": "dam",
    "damselflies": "damselfly",
    "damsels": "damsel",
    "damsons": "damson",
    "danced": "dance",
    "dancers": "dancer",
    "dances": "dance",
    "dancing": "dance",
    "dandelions": "dandelion",
    "danders": "dander",
    "dandier": "dandy",
    "dandies": "dandy",
    "dandiest": "dandy",
    "dandruffs": "dandruff",
    "dangers": "danger",
    "dangled": "dangle",
    "dangles": "dangle",
    "dangling": "dangle",
    "danker": "dank",
    "dankest": "dank",
    "dappled": "dapple",
    "dapples": "dapple",
    "dappling": "dapple",
    "dared": "dare",
    "daredevils": "daredevil",
    "dares": "dare",
    "daring": "dare",
    "darkened": "darken",
    "darkening": "darken",
    "darkens": "darken",
    "darker": "dark",
    "darkest": "dark",
    "darkies": "darkie",
    "darkrooms": "darkroom",
    "darks": "dark",
    "darlings": "darling",
    "darned": "darn",
    "darning": "darn",
    "darns": "darn",
    "dartboards": "dartboard",
    "darted": "dart",
    "darters": "darter",
    "darting": "dart",
    "darts": "dart",
    "dashboards": "dashboard",
    "dashed": "dash",
    "dashes": "dash",
    "dashing": "dash",
    "data": "datum",
    "data-sets": "data-set",
    "databanks": "databank",
    "databases": "database",
    "datasets": "dataset",
    "dated": "date",
    "datelines": "dateline",
    "dates": "date",
    "dating": "date",
    "datums": "datum",
    "daubed": "daub",
    "daubing": "daub",
    "daubs": "daub",
    "daughters": "daughter",
    "daughters-in-law": "daughter-in-law",
    "daunted": "daunt",
    "daunting": "daunt",
    "daunts": "daunt",
    "davits": "davit",
    "dawdled": "dawdle",
    "dawdles": "dawdle",
    "dawdling": "dawdle",
    "dawned": "dawn",
    "dawning": "dawn",
    "dawns": "dawn",
    "day-dreamed": "day-dream",
    "day-dreaming": "day-dream",
    "day-dreams": "day-dream",
    "day-trippers": "day-tripper",
    "day-trips": "day-trip",
    "daydreamed": "daydream",
    "daydreaming": "daydream",
    "daydreams": "daydream",
    "daylights": "daylight",
    "days": "day",
    "dazed": "daze",
    "dazes": "daze",
    "dazing": "daze",
    "dazzled": "dazzle",
    "dazzles": "dazzle",
    "dazzling": "dazzle",
    "dbs": "db",
    "ddis": "ddi",
    "dds": "dd",
    "de-activated": "de-activate",
    "de-activates": "de-activate",
    "de-activating": "de-activate",
    "de-emphasised": "de-emphasise",
    "de-emphasises": "de-emphasise",
    "de-emphasising": "de-emphasise",
    "de-emphasized": "de-emphasize",
    "de-emphasizes": "de-emphasize",
    "de-emphasizing": "de-emphasize",
    "de-escalated": "de-escalate",
    "de-escalates": "de-escalate",
    "de-escalating": "de-escalate",
    "de-iced": "de-ice",
    "de-icers": "de-icer",
    "de-ices": "de-ice",
    "de-icing": "de-ice",
    "de-regulations": "de-regulation",
    "de-selected": "de-select",
    "de-selecting": "de-select",
    "de-selects": "de-select",
    "de-stabilised": "de-stabilise",
    "de-stabilises": "de-stabilise",
    "de-stabilising": "de-stabilise",
    "deaconesses": "deaconess",
    "deacons": "deacon",
    "deactivated": "deactivate",
    "deactivates": "deactivate",
    "deactivating": "deactivate",
    "deactivations": "deactivation",
    "deadbeats": "deadbeat",
    "deadened": "deaden",
    "deadening": "deaden",
    "deadens": "deaden",
    "deader": "dead",
    "deadest": "dead",
    "deadlier": "deadly",
    "deadliest": "deadly",
    "deadlines": "deadline",
    "deadlocked": "deadlock",
    "deadlocking": "deadlock",
    "deadlocks": "deadlock",
    "deads": "dead",
    "deadweights": "deadweight",
    "deaf-aids": "deaf-aid",
    "deaf-mutes": "deaf-mute",
    "deafened": "deafen",
    "deafening": "deafen",
    "deafens": "deafen",
    "deafer": "deaf",
    "deafest": "deaf",
    "deafnesses": "deafness",
    "dealers": "dealer",
    "dealing": "deal",
    "deals": "deal",
    "dealt": "deal",
    "deaneries": "deanery",
    "deans": "dean",
    "dearer": "dear",
    "dearest": "dear",
    "dearies": "deary",
    "dears": "dear",
    "dearths": "dearth",
    "death-beds": "death-bed",
    "death-rates": "death-rate",
    "deathbeds": "deathbed",
    "deathlier": "deathly",
    "deathliest": "deathly",
    "deaths": "death",
    "debacles": "debacle",
    "debarked": "debark",
    "debarking": "debark",
    "debarks": "debark",
    "debarred": "debar",
    "debarring": "debar",
    "debars": "debar",
    "debased": "debase",
    "debasements": "debasement",
    "debases": "debase",
    "debasing": "debase",
    "debated": "debate",
    "debaters": "debater",
    "debates": "debate",
    "debating": "debate",
    "debauched": "debauch",
    "debauches": "debauch",
    "debauching": "debauch",
    "debilitated": "debilitate",
    "debilitates": "debilitate",
    "debilitating": "debilitate",
    "debilities": "debility",
    "debited": "debit",
    "debiting": "debit",
    "debits": "debit",
    "debridements": "debridement",
    "debriefed": "debrief",
    "debriefing": "debrief",
    "debriefs": "debrief",
    "debs": "deb",
    "debtors": "debtor",
    "debts": "debt",
    "debugged": "debug",
    "debugging": "debug",
    "debugs": "debug",
    "debunked": "debunk",
    "debunking": "debunk",
    "debunks": "debunk",
    "debutantes": "debutante",
    "debuted": "debut",
    "debuting": "debut",
    "debuts": "debut",
    "decadences": "decadence",
    "decades": "decade",
    "decaffeinated": "decaffeinate",
    "decaffeinates": "decaffeinate",
    "decaffeinating": "decaffeinate",
    "decalogues": "decalogue",
    "decals": "decal",
    "decamped": "decamp",
    "decamping": "decamp",
    "decamps": "decamp",
    "decanted": "decant",
    "decanters": "decanter",
    "decanting": "decant",
    "decants": "decant",
    "decapitated": "decapitate",
    "decapitates": "decapitate",
    "decapitating": "decapitate",
    "decapitations": "decapitation",
    "decarbonized": "decarbonize",
    "decarbonizes": "decarbonize",
    "decarbonizing": "decarbonize",
    "decathlons": "decathlon",
    "decayed": "decay",
    "decaying": "decay",
    "decays": "decay",
    "deceased": "decease",
    "deceases": "decease",
    "deceasing": "decease",
    "deceits": "deceit",
    "deceived": "deceive",
    "deceivers": "deceiver",
    "deceives": "deceive",
    "deceiving": "deceive",
    "decelerated": "decelerate",
    "decelerates": "decelerate",
    "decelerating": "decelerate",
    "decelerations": "deceleration",
    "decencies": "decency",
    "decentralisations": "decentralisation",
    "decentralised": "decentralise",
    "decentralises": "decentralise",
    "decentralising": "decentralise",
    "decentralizations": "decentralization",
    "decentralized": "decentralize",
    "decentralizes": "decentralize",
    "decentralizing": "decentralize",
    "deceptions": "deception",
    "decibels": "decibel",
    "decided": "decide",
    "deciders": "decider",
    "decides": "decide",
    "deciding": "decide",
    "deciles": "decile",
    "decimalized": "decimalize",
    "decimalizes": "decimalize",
    "decimalizing": "decimalize",
    "decimals": "decimal",
    "decimated": "decimate",
    "decimates": "decimate",
    "decimating": "decimate",
    "deciphered": "decipher",
    "deciphering": "decipher",
    "deciphers": "decipher",
    "decision-makers": "decision-maker",
    "decisions": "decision",
    "deckchairs": "deckchair",
    "decked": "deck",
    "deckers": "decker",
    "deckhands": "deckhand",
    "decking": "deck",
    "decks": "deck",
    "declaimed": "declaim",
    "declaiming": "declaim",
    "declaims": "declaim",
    "declamations": "declamation",
    "declarations": "declaration",
    "declared": "declare",
    "declares": "declare",
    "declaring": "declare",
    "declassified": "declassify",
    "declassifies": "declassify",
    "declassifying": "declassify",
    "declensions": "declension",
    "declinations": "declination",
    "declined": "decline",
    "declines": "decline",
    "declining": "decline",
    "declivities": "declivity",
    "declutched": "declutch",
    "declutches": "declutch",
    "declutching": "declutch",
    "decoctions": "decoction",
    "decoded": "decode",
    "decoders": "decoder",
    "decodes": "decode",
    "decoding": "decode",
    "decolletes": "decollete",
    "decolonized": "decolonize",
    "decolonizes": "decolonize",
    "decolonizing": "decolonize",
    "decommissioned": "decommission",
    "decommissioning": "decommission",
    "decommissions": "decommission",
    "decomposed": "decompose",
    "decomposers": "decomposer",
    "decomposes": "decompose",
    "decomposing": "decompose",
    "decompositions": "decomposition",
    "decompressed": "decompress",
    "decompresses": "decompress",
    "decompressing": "decompress",
    "decompressions": "decompression",
    "decongestants": "decongestant",
    "deconstructed": "deconstruct",
    "deconstructing": "deconstruct",
    "deconstructionists": "deconstructionist",
    "deconstructions": "deconstruction",
    "deconstructs": "deconstruct",
    "decontaminated": "decontaminate",
    "decontaminates": "decontaminate",
    "decontaminating": "decontaminate",
    "decontaminations": "decontamination",
    "deconvolutions": "deconvolution",
    "decorated": "decorate",
    "decorates": "decorate",
    "decorating": "decorate",
    "decorations": "decoration",
    "decorators": "decorator",
    "decoupled": "decouple",
    "decouples": "decouple",
    "decoupling": "decouple",
    "decoyed": "decoy",
    "decoying": "decoy",
    "decoys": "decoy",
    "decreased": "decrease",
    "decreases": "decrease",
    "decreasing": "decrease",
    "decreed": "decree",
    "decreeing": "decree",
    "decrees": "decree",
    "decremented": "decrement",
    "decrementing": "decrement",
    "decrements": "decrement",
    "decried": "decry",
    "decries": "decry",
    "decriminalised": "decriminalise",
    "decriminalises": "decriminalise",
    "decriminalising": "decriminalise",
    "decrying": "decry",
    "decrypted": "decrypt",
    "decrypting": "decrypt",
    "decryptions": "decryption",
    "decrypts": "decrypt",
    "dedicated": "dedicate",
    "dedicates": "dedicate",
    "dedicating": "dedicate",
    "dedications": "dedication",
    "deduced": "deduce",
    "deduces": "deduce",
    "deducing": "deduce",
    "deducted": "deduct",
    "deductibles": "deductible",
    "deducting": "deduct",
    "deductions": "deduction",
    "deducts": "deduct",
    "deeded": "deed",
    "deeding": "deed",
    "deeds": "deed",
    "deemed": "deem",
    "deeming": "deem",
    "deems": "deem",
    "deep-freezed": "deep-freeze",
    "deep-freezes": "deep-freeze",
    "deep-freezing": "deep-freeze",
    "deep-fried": "deep-fry",
    "deep-fries": "deep-fry",
    "deep-froze": "deep-freeze",
    "deep-frozen": "deep-freeze",
    "deep-frying": "deep-fry",
    "deepened": "deepen",
    "deepening": "deepen",
    "deepens": "deepen",
    "deeper": "deep",
    "deepest": "deep",
    "deeps": "deep",
    "deers": "deer",
    "deerskins": "deerskin",
    "defaced": "deface",
    "defaces": "deface",
    "defacing": "deface",
    "defamed": "defame",
    "defames": "defame",
    "defaming": "defame",
    "defaulted": "default",
    "defaulters": "defaulter",
    "defaulting": "default",
    "defaults": "default",
    "defeated": "defeat",
    "defeating": "defeat",
    "defeatists": "defeatist",
    "defeats": "defeat",
    "defecated": "defecate",
    "defecates": "defecate",
    "defecating": "defecate",
    "defecations": "defecation",
    "defected": "defect",
    "defecting": "defect",
    "defections": "defection",
    "defectives": "defective",
    "defectors": "defector",
    "defects": "defect",
    "defences": "defence",
    "defendants": "defendant",
    "defended": "defend",
    "defenders": "defender",
    "defending": "defend",
    "defends": "defend",
    "defenses": "defense",
    "deferments": "deferment",
    "deferrals": "deferral",
    "deferred": "defer",
    "deferring": "defer",
    "defers": "defer",
    "defibrillations": "defibrillation",
    "defibrillators": "defibrillator",
    "deficiencies": "deficiency",
    "deficits": "deficit",
    "defied": "defy",
    "defies": "defy",
    "defiled": "defile",
    "defilers": "defiler",
    "defiles": "defile",
    "defiling": "defile",
    "defined": "define",
    "defines": "define",
    "defining": "define",
    "definitions": "definition",
    "deflated": "deflate",
    "deflates": "deflate",
    "deflating": "deflate",
    "deflations": "deflation",
    "deflators": "deflator",
    "deflected": "deflect",
    "deflecting": "deflect",
    "deflections": "deflection",
    "deflectors": "deflector",
    "deflects": "deflect",
    "deflexions": "deflexion",
    "deflowered": "deflower",
    "deflowering": "deflower",
    "deflowers": "deflower",
    "defoliants": "defoliant",
    "defoliated": "defoliate",
    "defoliates": "defoliate",
    "defoliating": "defoliate",
    "deforested": "deforest",
    "deforesting": "deforest",
    "deforests": "deforest",
    "deformations": "deformation",
    "deformed": "deform",
    "deforming": "deform",
    "deformities": "deformity",
    "deforms": "deform",
    "defragmented": "defragment",
    "defragmenting": "defragment",
    "defragments": "defragment",
    "defrauded": "defraud",
    "defrauding": "defraud",
    "defrauds": "defraud",
    "defrayed": "defray",
    "defraying": "defray",
    "defrays": "defray",
    "defrosted": "defrost",
    "defrosting": "defrost",
    "defrosts": "defrost",
    "defter": "deft",
    "deftest": "deft",
    "defused": "defuse",
    "defuses": "defuse",
    "defusing": "defuse",
    "defying": "defy",
    "degases": "degas",
    "degassed": "degas",
    "degassing": "degas",
    "degaussed": "degauss",
    "degausses": "degauss",
    "degaussing": "degauss",
    "degeneracies": "degeneracy",
    "degenerated": "degenerate",
    "degenerates": "degenerate",
    "degenerating": "degenerate",
    "degenerations": "degeneration",
    "degradations": "degradation",
    "degraded": "degrade",
    "degrades": "degrade",
    "degrading": "degrade",
    "degreased": "degrease",
    "degreases": "degrease",
    "degreasing": "degrease",
    "degrees": "degree",
    "degs": "deg",
    "degus": "degu",
    "dehumanised": "dehumanise",
    "dehumanises": "dehumanise",
    "dehumanising": "dehumanise",
    "dehumanized": "dehumanize",
    "dehumanizes": "dehumanize",
    "dehumanizing": "dehumanize",
    "dehumidified": "dehumidify",
    "dehumidifiers": "dehumidifier",
    "dehumidifies": "dehumidify",
    "dehumidifying": "dehumidify",
    "dehydrated": "dehydrate",
    "dehydrates": "dehydrate",
    "dehydrating": "dehydrate",
    "dehydrations": "dehydration",
    "dehydrogenases": "dehydrogenase",
    "deified": "deify",
    "deifies": "deify",
    "deifying": "deify",
    "deigned": "deign",
    "deigning": "deign",
    "deigns": "deign",
    "deists": "deist",
    "deities": "deity",
    "dejected": "deject",
    "dejecting": "deject",
    "dejections": "dejection",
    "dejects": "deject",
    "delaminated": "delaminate",
    "delaminates": "delaminate",
    "delaminating": "delaminate",
    "delaminations": "delamination",
    "delayed": "delay",
    "delaying": "delay",
    "delays": "delay",
    "delegated": "delegate",
    "delegates": "delegate",
    "delegating": "delegate",
    "delegations": "delegation",
    "deleted": "delete",
    "deletes": "delete",
    "deleting": "delete",
    "deletions": "deletion",
    "deliberated": "deliberate",
    "deliberates": "deliberate",
    "deliberating": "deliberate",
    "deliberations": "deliberation",
    "delicacies": "delicacy",
    "delicatessens": "delicatessen",
    "delicts": "delict",
    "delighted": "delight",
    "delighting": "delight",
    "delights": "delight",
    "delimitations": "delimitation",
    "delimited": "delimit",
    "delimiters": "delimiter",
    "delimiting": "delimit",
    "delimits": "delimit",
    "delineated": "delineate",
    "delineates": "delineate",
    "delineating": "delineate",
    "delineations": "delineation",
    "delinquencies": "delinquency",
    "delinquents": "delinquent",
    "deliria": "delirium",
    "deliriums": "delirium",
    "delis": "deli",
    "delisted": "delist",
    "delisting": "delist",
    "delists": "delist",
    "deliverables": "deliverable",
    "delivered": "deliver",
    "deliverers": "deliverer",
    "deliveries": "delivery",
    "delivering": "deliver",
    "delivers": "deliver",
    "dells": "dell",
    "delocalisations": "delocalisation",
    "delocalised": "delocalise",
    "delocalises": "delocalise",
    "delocalising": "delocalise",
    "deloused": "delouse",
    "delouses": "delouse",
    "delousing": "delouse",
    "delphinia": "delphinium",
    "delphiniums": "delphinium",
    "dels": "del",
    "deltas": "delta",
    "deltoids": "deltoid",
    "deluded": "delude",
    "deludes": "delude",
    "deluding": "delude",
    "deluged": "deluge",
    "deluges": "deluge",
    "deluging": "deluge",
    "delusions": "delusion",
    "delved": "delve",
    "delves": "delve",
    "delving": "delve",
    "demagogues": "demagogue",
    "demanded": "demand",
    "demanding": "demand",
    "demands": "demand",
    "demarcated": "demarcate",
    "demarcates": "demarcate",
    "demarcating": "demarcate",
    "demarcations": "demarcation",
    "dematerialised": "dematerialise",
    "dematerialises": "dematerialise",
    "dematerialising": "dematerialise",
    "demeaned": "demean",
    "demeaning": "demean",
    "demeans": "demean",
    "demented": "dement",
    "dementias": "dementia",
    "dementing": "dement",
    "dements": "dement",
    "demerits": "demerit",
    "demijohns": "demijohn",
    "demilitarised": "demilitarise",
    "demilitarises": "demilitarise",
    "demilitarising": "demilitarise",
    "demilitarized": "demilitarize",
    "demilitarizes": "demilitarize",
    "demilitarizing": "demilitarize",
    "demised": "demise",
    "demises": "demise",
    "demising": "demise",
    "demisted": "demist",
    "demisters": "demister",
    "demisting": "demist",
    "demists": "demist",
    "demobbed": "demob",
    "demobbing": "demob",
    "demobilisations": "demobilisation",
    "demobilised": "demobilise",
    "demobilises": "demobilise",
    "demobilising": "demobilise",
    "demobilizations": "demobilization",
    "demobilized": "demobilize",
    "demobilizes": "demobilize",
    "demobilizing": "demobilize",
    "demobs": "demob",
    "democracies": "democracy",
    "democratised": "democratise",
    "democratises": "democratise",
    "democratising": "democratise",
    "democratized": "democratize",
    "democratizes": "democratize",
    "democratizing": "democratize",
    "democrats": "democrat",
    "demodulated": "demodulate",
    "demodulates": "demodulate",
    "demodulating": "demodulate",
    "demodulations": "demodulation",
    "demodulators": "demodulator",
    "demographers": "demographer",
    "demographies": "demography",
    "demolished": "demolish",
    "demolishes": "demolish",
    "demolishing": "demolish",
    "demolitions": "demolition",
    "demonised": "demonise",
    "demonises": "demonise",
    "demonising": "demonise",
    "demonized": "demonize",
    "demonizes": "demonize",
    "demonizing": "demonize",
    "demonologies": "demonology",
    "demons": "demon",
    "demonstrated": "demonstrate",
    "demonstrates": "demonstrate",
    "demonstrating": "demonstrate",
    "demonstrations": "demonstration",
    "demonstratives": "demonstrative",
    "demonstrators": "demonstrator",
    "demoralisations": "demoralisation",
    "demoralised": "demoralise",
    "demoralises": "demoralise",
    "demoralising": "demoralise",
    "demoralizations": "demoralization",
    "demoralized": "demoralize",
    "demoralizes": "demoralize",
    "demoralizing": "demoralize",
    "demos": "demo",
    "demoted": "demote",
    "demotes": "demote",
    "demoting": "demote",
    "demotions": "demotion",
    "demurer": "demure",
    "demurest": "demure",
    "demurred": "demur",
    "demurring": "demur",
    "demurs": "demur",
    "demyelinations": "demyelination",
    "demystified": "demystify",
    "demystifies": "demystify",
    "demystifying": "demystify",
    "denarii": "denarius",
    "denationalized": "denationalize",
    "denationalizes": "denationalize",
    "denationalizing": "denationalize",
    "denaturations": "denaturation",
    "denatured": "denature",
    "denatures": "denature",
    "denaturing": "denature",
    "dendrites": "dendrite",
    "dendrograms": "dendrogram",
    "denervations": "denervation",
    "denials": "denial",
    "denied": "deny",
    "deniers": "denier",
    "denies": "deny",
    "denigrated": "denigrate",
    "denigrates": "denigrate",
    "denigrating": "denigrate",
    "denims": "denim",
    "denitrifications": "denitrification",
    "denizens": "denizen",
    "denned": "den",
    "denning": "den",
    "denominated": "denominate",
    "denominates": "denominate",
    "denominating": "denominate",
    "denominations": "denomination",
    "denominators": "denominator",
    "denotations": "denotation",
    "denoted": "denote",
    "denotes": "denote",
    "denoting": "denote",
    "denouements": "denouement",
    "denounced": "denounce",
    "denounces": "denounce",
    "denouncing": "denounce",
    "dens": "den",
    "denser": "dense",
    "densest": "dense",
    "densities": "density",
    "densitometries": "densitometry",
    "dented": "dent",
    "dentes": "dens",
    "dentifrices": "dentifrice",
    "dentines": "dentine",
    "denting": "dent",
    "dentists": "dentist",
    "dentitions": "dentition",
    "dents": "dent",
    "dentures": "denture",
    "denudations": "denudation",
    "denuded": "denude",
    "denudes": "denude",
    "denuding": "denude",
    "denunciations": "denunciation",
    "denying": "deny",
    "deodorants": "deodorant",
    "deodorised": "deodorise",
    "deodorises": "deodorise",
    "deodorising": "deodorise",
    "deodorized": "deodorize",
    "deodorizes": "deodorize",
    "deodorizing": "deodorize",
    "deoxygenated": "deoxygenate",
    "deoxygenates": "deoxygenate",
    "deoxygenating": "deoxygenate",
    "departed": "depart",
    "departing": "depart",
    "departments": "department",
    "departs": "depart",
    "departures": "departure",
    "dependances": "dependance",
    "dependancies": "dependancy",
    "dependants": "dependant",
    "depended": "depend",
    "dependences": "dependence",
    "dependencies": "dependency",
    "dependents": "dependent",
    "depending": "depend",
    "depends": "depend",
    "depersonalisations": "depersonalisation",
    "depersonalised": "depersonalise",
    "depersonalises": "depersonalise",
    "depersonalising": "depersonalise",
    "depicted": "depict",
    "depicting": "depict",
    "depictions": "depiction",
    "depicts": "depict",
    "depleted": "deplete",
    "depletes": "deplete",
    "depleting": "deplete",
    "depletions": "depletion",
    "deplored": "deplore",
    "deplores": "deplore",
    "deploring": "deplore",
    "deployed": "deploy",
    "deploying": "deploy",
    "deployments": "deployment",
    "deploys": "deploy",
    "depolarisations": "depolarisation",
    "depolarised": "depolarise",
    "depolarises": "depolarise",
    "depolarising": "depolarise",
    "depoliticised": "depoliticise",
    "depoliticises": "depoliticise",
    "depoliticising": "depoliticise",
    "depopulated": "depopulate",
    "depopulates": "depopulate",
    "depopulating": "depopulate",
    "depopulations": "depopulation",
    "deportations": "deportation",
    "deported": "deport",
    "deportees": "deportee",
    "deporting": "deport",
    "deportments": "deportment",
    "deports": "deport",
    "deposed": "depose",
    "deposes": "depose",
    "deposing": "depose",
    "depositaries": "depositary",
    "deposited": "deposit",
    "depositing": "deposit",
    "depositions": "deposition",
    "depositories": "depository",
    "depositors": "depositor",
    "deposits": "deposit",
    "depots": "depot",
    "depraved": "deprave",
    "depraves": "deprave",
    "depraving": "deprave",
    "depravities": "depravity",
    "deprecated": "deprecate",
    "deprecates": "deprecate",
    "deprecating": "deprecate",
    "depreciated": "depreciate",
    "depreciates": "depreciate",
    "depreciating": "depreciate",
    "depreciations": "depreciation",
    "depredations": "depredation",
    "depressants": "depressant",
    "depressed": "depress",
    "depresses": "depress",
    "depressing": "depress",
    "depressions": "depression",
    "depressives": "depressive",
    "deprivations": "deprivation",
    "deprived": "deprive",
    "deprives": "deprive",
    "depriving": "deprive",
    "depths": "depth",
    "depts": "dept",
    "deputations": "deputation",
    "deputed": "depute",
    "deputes": "depute",
    "deputies": "deputy",
    "deputing": "depute",
    "deputised": "deputise",
    "deputises": "deputise",
    "deputising": "deputise",
    "deputized": "deputize",
    "deputizes": "deputize",
    "deputizing": "deputize",
    "derailed": "derail",
    "derailing": "derail",
    "derailments": "derailment",
    "derails": "derail",
    "deranged": "derange",
    "derangements": "derangement",
    "deranges": "derange",
    "deranging": "derange",
    "derated": "derate",
    "derates": "derate",
    "derating": "derate",
    "derbies": "derby",
    "deregulated": "deregulate",
    "deregulates": "deregulate",
    "deregulating": "deregulate",
    "deregulations": "deregulation",
    "derelictions": "dereliction",
    "derelicts": "derelict",
    "derided": "deride",
    "derides": "deride",
    "deriding": "deride",
    "derivates": "derivate",
    "derivations": "derivation",
    "derivatives": "derivative",
    "derived": "derive",
    "derives": "derive",
    "deriving": "derive",
    "dermatitides": "dermatitis",
    "dermatitises": "dermatitis",
    "dermatologies": "dermatology",
    "dermatologists": "dermatologist",
    "dermatomyositides": "dermatomyositis",
    "dermatophytes": "dermatophyte",
    "dermatoses": "dermatosis",
    "dermises": "dermis",
    "derogated": "derogate",
    "derogates": "derogate",
    "derogating": "derogate",
    "derricks": "derrick",
    "ders": "der",
    "dervishes": "dervish",
    "descaled": "descale",
    "descales": "descale",
    "descaling": "descale",
    "descants": "descant",
    "descendants": "descendant",
    "descended": "descend",
    "descendents": "descendent",
    "descending": "descend",
    "descends": "descend",
    "descents": "descent",
    "described": "describe",
    "describers": "describer",
    "describes": "describe",
    "describing": "describe",
    "descried": "descry",
    "descries": "descry",
    "descriptions": "description",
    "descriptors": "descriptor",
    "descrying": "descry",
    "desecrated": "desecrate",
    "desecrates": "desecrate",
    "desecrating": "desecrate",
    "desecrations": "desecration",
    "deseeded": "deseed",
    "deseeding": "deseed",
    "deseeds": "deseed",
    "desegregated": "desegregate",
    "desegregates": "desegregate",
    "desegregating": "desegregate",
    "deselected": "deselect",
    "deselecting": "deselect",
    "deselects": "deselect",
    "desensitisations": "desensitisation",
    "desensitised": "desensitise",
    "desensitises": "desensitise",
    "desensitising": "desensitise",
    "desensitizations": "desensitization",
    "desensitized": "desensitize",
    "desensitizes": "desensitize",
    "desensitizing": "desensitize",
    "deserted": "desert",
    "deserters": "deserter",
    "desertifications": "desertification",
    "deserting": "desert",
    "desertions": "desertion",
    "deserts": "desert",
    "deserved": "deserve",
    "deserves": "deserve",
    "deserving": "deserve",
    "desiccants": "desiccant",
    "desiccated": "desiccate",
    "desiccates": "desiccate",
    "desiccating": "desiccate",
    "desiccations": "desiccation",
    "desiderata": "desideratum",
    "designated": "designate",
    "designates": "designate",
    "designating": "designate",
    "designations": "designation",
    "designators": "designator",
    "designed": "design",
    "designees": "designee",
    "designers": "designer",
    "designing": "design",
    "designs": "design",
    "desirabilities": "desirability",
    "desired": "desire",
    "desires": "desire",
    "desiring": "desire",
    "desisted": "desist",
    "desisting": "desist",
    "desists": "desist",
    "desks": "desk",
    "desmids": "desmid",
    "desolated": "desolate",
    "desolates": "desolate",
    "desolating": "desolate",
    "despaired": "despair",
    "despairing": "despair",
    "despairs": "despair",
    "despatched": "despatch",
    "despatches": "despatch",
    "despatching": "despatch",
    "desperadoes": "desperado",
    "desperados": "desperado",
    "despised": "despise",
    "despises": "despise",
    "despising": "despise",
    "despoiled": "despoil",
    "despoiling": "despoil",
    "despoils": "despoil",
    "despotisms": "despotism",
    "despots": "despot",
    "desserts": "dessert",
    "dessertspoons": "dessertspoon",
    "destabilisations": "destabilisation",
    "destabilised": "destabilise",
    "destabilises": "destabilise",
    "destabilising": "destabilise",
    "destabilizations": "destabilization",
    "destabilized": "destabilize",
    "destabilizes": "destabilize",
    "destabilizing": "destabilize",
    "destinations": "destination",
    "destined": "destine",
    "destines": "destine",
    "destinies": "destiny",
    "destining": "destine",
    "destroyed": "destroy",
    "destroyers": "destroyer",
    "destroying": "destroy",
    "destroys": "destroy",
    "destructed": "destruct",
    "destructing": "destruct",
    "destructions": "destruction",
    "destructors": "destructor",
    "destructs": "destruct",
    "detached": "detach",
    "detaches": "detach",
    "detaching": "detach",
    "detachments": "detachment",
    "detailed": "detail",
    "detailing": "detail",
    "details": "detail",
    "detained": "detain",
    "detainees": "detainee",
    "detaining": "detain",
    "detains": "detain",
    "detectabilities": "detectability",
    "detected": "detect",
    "detecting": "detect",
    "detections": "detection",
    "detectives": "detective",
    "detectors": "detector",
    "detects": "detect",
    "detentions": "detention",
    "detergents": "detergent",
    "deteriorated": "deteriorate",
    "deteriorates": "deteriorate",
    "deteriorating": "deteriorate",
    "deteriorations": "deterioration",
    "determinacies": "determinacy",
    "determinants": "determinant",
    "determinations": "determination",
    "determinatives": "determinative",
    "determined": "determine",
    "determiners": "determiner",
    "determines": "determine",
    "determining": "determine",
    "determinisms": "determinism",
    "determinists": "determinist",
    "deterred": "deter",
    "deterrents": "deterrent",
    "deterring": "deter",
    "deters": "deter",
    "detested": "detest",
    "detesting": "detest",
    "detests": "detest",
    "dethroned": "dethrone",
    "dethrones": "dethrone",
    "dethroning": "dethrone",
    "detonated": "detonate",
    "detonates": "detonate",
    "detonating": "detonate",
    "detonations": "detonation",
    "detonators": "detonator",
    "detoured": "detour",
    "detouring": "detour",
    "detours": "detour",
    "detoxed": "detox",
    "detoxes": "detox",
    "detoxifications": "detoxification",
    "detoxified": "detoxify",
    "detoxifies": "detoxify",
    "detoxifying": "detoxify",
    "detoxing": "detox",
    "detracted": "detract",
    "detracting": "detract",
    "detractions": "detraction",
    "detractors": "detractor",
    "detracts": "detract",
    "detrained": "detrain",
    "detraining": "detrain",
    "detrains": "detrain",
    "detriments": "detriment",
    "detrusors": "detrusor",
    "deuces": "deuce",
    "deuteria": "deuterium",
    "deuteriums": "deuterium",
    "devaluations": "devaluation",
    "devalued": "devalue",
    "devalues": "devalue",
    "devaluing": "devalue",
    "devastated": "devastate",
    "devastates": "devastate",
    "devastating": "devastate",
    "devastations": "devastation",
    "developed": "develope",
    "developements": "developement",
    "developers": "developer",
    "developes": "develope",
    "developing": "develope",
    "developments": "development",
    "develops": "develop",
    "deviances": "deviance",
    "deviancies": "deviancy",
    "deviants": "deviant",
    "deviated": "deviate",
    "deviates": "deviate",
    "deviating": "deviate",
    "deviations": "deviation",
    "devices": "device",
    "deviled": "devil",
    "deviling": "devil",
    "devilled": "devil",
    "devilling": "devil",
    "devils": "devil",
    "devised": "devise",
    "devises": "devise",
    "devising": "devise",
    "devitalised": "devitalise",
    "devitalises": "devitalise",
    "devitalising": "devitalise",
    "devolutions": "devolution",
    "devolved": "devolve",
    "devolves": "devolve",
    "devolving": "devolve",
    "devoted": "devote",
    "devotees": "devotee",
    "devotes": "devote",
    "devoting": "devote",
    "devotions": "devotion",
    "devoured": "devour",
    "devouring": "devour",
    "devours": "devour",
    "devouter": "devout",
    "devoutest": "devout",
    "devs": "dev",
    "dewier": "dewy",
    "dewiest": "dewy",
    "dewlaps": "dewlap",
    "dewpoints": "dewpoint",
    "dews": "dew",
    "dexterities": "dexterity",
    "dexters": "dexter",
    "dhals": "dhal",
    "dharmas": "dharma",
    "dhotis": "dhoti",
    "diabetics": "diabetic",
    "diabetologists": "diabetologist",
    "diablos": "diablo",
    "diaconates": "diaconate",
    "diacritics": "diacritic",
    "diadems": "diadem",
    "diageneses": "diagenesis",
    "diagnosed": "diagnose",
    "diagnoses": "diagnosis",
    "diagnosing": "diagnose",
    "diagnosticians": "diagnostician",
    "diagonals": "diagonal",
    "diagramed": "diagram",
    "diagraming": "diagram",
    "diagrammed": "diagram",
    "diagramming": "diagram",
    "diagrams": "diagram",
    "dialectics": "dialectic",
    "dialects": "dialect",
    "dialed": "dial",
    "dialing": "dial",
    "dialled": "dial",
    "dialling": "dial",
    "dialoged": "dialog",
    "dialoging": "dialog",
    "dialogs": "dialog",
    "dialogued": "dialogue",
    "dialogues": "dialogue",
    "dialoguing": "dialogue",
    "dials": "dial",
    "dialysed": "dialyse",
    "dialysers": "dialyser",
    "dialyses": "dialysis",
    "dialysing": "dialyse",
    "diameters": "diameter",
    "diamonds": "diamond",
    "diapasons": "diapason",
    "diapers": "diaper",
    "diaphragms": "diaphragm",
    "diaries": "diary",
    "diarists": "diarist",
    "diarrheas": "diarrhea",
    "diarrhoeas": "diarrhoea",
    "diasporas": "diaspora",
    "diastoles": "diastole",
    "diathermies": "diathermy",
    "diatoms": "diatom",
    "diatribes": "diatribe",
    "dice": "die",
    "dice-boxes": "dice-box",
    "diced": "dice",
    "dices": "dice",
    "dichotomies": "dichotomy",
    "dichroisms": "dichroism",
    "dicier": "dicey",
    "diciest": "dicey",
    "dicing": "dice",
    "dickier": "dicky",
    "dickies": "dicky",
    "dickiest": "dicky",
    "dicky-birds": "dicky-bird",
    "dicta": "dictum",
    "dictated": "dictate",
    "dictates": "dictate",
    "dictating": "dictate",
    "dictations": "dictation",
    "dictators": "dictator",
    "dictatorships": "dictatorship",
    "dictionaries": "dictionary",
    "dictions": "diction",
    "dictums": "dictum",
    "did": "do",
    "didactically": "didactic",
    "diddled": "diddle",
    "diddles": "diddle",
    "diddling": "diddle",
    "died": "die",
    "diehards": "diehard",
    "dielectrics": "dielectric",
    "dies": "die",
    "diesels": "diesel",
    "dieses": "diesis",
    "dietaries": "dietary",
    "dieted": "diet",
    "dieters": "dieter",
    "dieticians": "dietician",
    "dieting": "diet",
    "dietitians": "dietitian",
    "diets": "diet",
    "differed": "differ",
    "differences": "difference",
    "differentials": "differential",
    "differentiated": "differentiate",
    "differentiates": "differentiate",
    "differentiating": "differentiate",
    "differentiations": "differentiation",
    "differentiators": "differentiator",
    "differing": "differ",
    "differs": "differ",
    "difficulties": "difficulty",
    "diffracted": "diffract",
    "diffracting": "diffract",
    "diffractions": "diffraction",
    "diffractometers": "diffractometer",
    "diffracts": "diffract",
    "diffused": "diffuse",
    "diffusers": "diffuser",
    "diffuses": "diffuse",
    "diffusing": "diffuse",
    "diffusions": "diffusion",
    "diffusivities": "diffusivity",
    "digested": "digest",
    "digesters": "digester",
    "digestibilities": "digestibility",
    "digesting": "digest",
    "digestions": "digestion",
    "digestives": "digestive",
    "digests": "digest",
    "diggers": "digger",
    "digging": "dig",
    "digitalisations": "digitalisation",
    "digitalised": "digitalise",
    "digitalises": "digitalise",
    "digitalising": "digitalise",
    "digitisations": "digitisation",
    "digitised": "digitise",
    "digitisers": "digitiser",
    "digitises": "digitise",
    "digitising": "digitise",
    "digitizations": "digitization",
    "digitized": "digitize",
    "digitizers": "digitizer",
    "digitizes": "digitize",
    "digitizing": "digitize",
    "digits": "digit",
    "dignified": "dignify",
    "dignifies": "dignify",
    "dignifying": "dignify",
    "dignitaries": "dignitary",
    "dignities": "dignity",
    "digraphs": "digraph",
    "digressed": "digress",
    "digresses": "digress",
    "digressing": "digress",
    "digressions": "digression",
    "digs": "dig",
    "dikes": "dike",
    "dilatations": "dilatation",
    "dilated": "dilate",
    "dilates": "dilate",
    "dilating": "dilate",
    "dilations": "dilation",
    "dilators": "dilator",
    "dildoes": "dildo",
    "dildos": "dildo",
    "dilemmas": "dilemma",
    "dilemmata": "dilemma",
    "dilettantes": "dilettante",
    "dilettanti": "dilettante",
    "diligences": "diligence",
    "diluents": "diluent",
    "diluted": "dilute",
    "dilutes": "dilute",
    "diluting": "dilute",
    "dilutions": "dilution",
    "dimensionalities": "dimensionality",
    "dimensions": "dimension",
    "dimers": "dimer",
    "dimes": "dime",
    "diminished": "diminish",
    "diminishes": "diminish",
    "diminishing": "diminish",
    "diminutions": "diminution",
    "diminutiver": "diminutive",
    "diminutivest": "diminutive",
    "dimmed": "dim",
    "dimmer": "dim",
    "dimmers": "dimmer",
    "dimmest": "dim",
    "dimming": "dim",
    "dimorphisms": "dimorphism",
    "dimpled": "dimple",
    "dimples": "dimple",
    "dimpling": "dimple",
    "dims": "dim",
    "dimwits": "dimwit",
    "dinars": "dinar",
    "dined": "dine",
    "diners": "diner",
    "dines": "dine",
    "ding-dongs": "ding-dong",
    "dinghies": "dinghy",
    "dingier": "dingy",
    "dingiest": "dingy",
    "dingoes": "dingo",
    "dings": "ding",
    "dining": "dine",
    "dinkier": "dinky",
    "dinned": "din",
    "dinners": "dinner",
    "dinning": "din",
    "dinoflagellates": "dinoflagellate",
    "dinosaurs": "dinosaur",
    "dins": "din",
    "dints": "dint",
    "dinucleotides": "dinucleotide",
    "dioceses": "diocese",
    "diodes": "diode",
    "diopters": "diopter",
    "dioptres": "dioptre",
    "dioramas": "diorama",
    "dioxides": "dioxide",
    "dioxins": "dioxin",
    "diphosphates": "diphosphate",
    "diphtherias": "diphtheria",
    "diphthongs": "diphthong",
    "diplomas": "diploma",
    "diplomates": "diplomate",
    "diplomats": "diplomat",
    "diplopias": "diplopia",
    "dipoles": "dipole",
    "dipped": "dip",
    "dippers": "dipper",
    "dipping": "dip",
    "dips": "dip",
    "dipsticks": "dipstick",
    "diptherias": "diptheria",
    "diptychs": "diptych",
    "directed": "direct",
    "directing": "direct",
    "directionalities": "directionality",
    "directions": "direction",
    "directives": "directive",
    "directivities": "directivity",
    "director-generals": "director-general",
    "directorates": "directorate",
    "directories": "directory",
    "directors": "director",
    "directors-general": "director-general",
    "directorships": "directorship",
    "directs": "direct",
    "direr": "dire",
    "direst": "dire",
    "dirges": "dirge",
    "dirks": "dirk",
    "dirndls": "dirndl",
    "dirtied": "dirty",
    "dirtier": "dirty",
    "dirties": "dirty",
    "dirtiest": "dirty",
    "dirts": "dirt",
    "dirtying": "dirty",
    "disabilities": "disability",
    "disabled": "disable",
    "disablements": "disablement",
    "disables": "disable",
    "disabling": "disable",
    "disabused": "disabuse",
    "disabuses": "disabuse",
    "disabusing": "disabuse",
    "disaccharides": "disaccharide",
    "disadvantaged": "disadvantage",
    "disadvantages": "disadvantage",
    "disadvantaging": "disadvantage",
    "disaffiliated": "disaffiliate",
    "disaffiliates": "disaffiliate",
    "disaffiliating": "disaffiliate",
    "disaggregated": "disaggregate",
    "disaggregates": "disaggregate",
    "disaggregating": "disaggregate",
    "disaggregations": "disaggregation",
    "disagreed": "disagree",
    "disagreeing": "disagree",
    "disagreements": "disagreement",
    "disagrees": "disagree",
    "disallowances": "disallowance",
    "disallowed": "disallow",
    "disallowing": "disallow",
    "disallows": "disallow",
    "disambiguated": "disambiguate",
    "disambiguates": "disambiguate",
    "disambiguating": "disambiguate",
    "disambiguations": "disambiguation",
    "disappearances": "disappearance",
    "disappeared": "disappear",
    "disappearing": "disappear",
    "disappears": "disappear",
    "disappointed": "disappoint",
    "disappointing": "disappoint",
    "disappointments": "disappointment",
    "disappoints": "disappoint",
    "disapproved": "disapprove",
    "disapproves": "disapprove",
    "disapproving": "disapprove",
    "disarmed": "disarm",
    "disarmers": "disarmer",
    "disarming": "disarm",
    "disarms": "disarm",
    "disarranged": "disarrange",
    "disarranges": "disarrange",
    "disarranging": "disarrange",
    "disarticulated": "disarticulate",
    "disarticulates": "disarticulate",
    "disarticulating": "disarticulate",
    "disassembled": "disassemble",
    "disassembles": "disassemble",
    "disassemblies": "disassembly",
    "disassembling": "disassemble",
    "disassociated": "disassociate",
    "disassociates": "disassociate",
    "disassociating": "disassociate",
    "disasters": "disaster",
    "disavowals": "disavowal",
    "disavowed": "disavow",
    "disavowing": "disavow",
    "disavows": "disavow",
    "disbanded": "disband",
    "disbanding": "disband",
    "disbands": "disband",
    "disbarred": "disbar",
    "disbarring": "disbar",
    "disbars": "disbar",
    "disbelieved": "disbelieve",
    "disbelievers": "disbeliever",
    "disbelieves": "disbelieve",
    "disbelieving": "disbelieve",
    "disbenefits": "disbenefit",
    "disbursed": "disburse",
    "disbursements": "disbursement",
    "disburses": "disburse",
    "disbursing": "disburse",
    "discarded": "discard",
    "discarding": "discard",
    "discards": "discard",
    "discerned": "discern",
    "discerning": "discern",
    "discerns": "discern",
    "discharged": "discharge",
    "discharges": "discharge",
    "discharging": "discharge",
    "disci": "discus",
    "disciples": "disciple",
    "disciplinarians": "disciplinarian",
    "disciplined": "discipline",
    "disciplines": "discipline",
    "disciplining": "discipline",
    "disclaimed": "disclaim",
    "disclaimers": "disclaimer",
    "disclaiming": "disclaim",
    "disclaims": "disclaim",
    "disclinations": "disclination",
    "disclosed": "disclose",
    "discloses": "disclose",
    "disclosing": "disclose",
    "disclosures": "disclosure",
    "discographies": "discography",
    "discolorations": "discoloration",
    "discolored": "discolor",
    "discoloring": "discolor",
    "discolors": "discolor",
    "discolourations": "discolouration",
    "discoloured": "discolour",
    "discolouring": "discolour",
    "discolours": "discolour",
    "discomfited": "discomfit",
    "discomfiting": "discomfit",
    "discomfits": "discomfit",
    "discomforts": "discomfort",
    "disconcerted": "disconcert",
    "disconcerting": "disconcert",
    "disconcerts": "disconcert",
    "disconnected": "disconnect",
    "disconnecting": "disconnect",
    "disconnections": "disconnection",
    "disconnects": "disconnect",
    "discontented": "discontent",
    "discontenting": "discontent",
    "discontents": "discontent",
    "discontinuances": "discontinuance",
    "discontinuations": "discontinuation",
    "discontinued": "discontinue",
    "discontinues": "discontinue",
    "discontinuing": "discontinue",
    "discontinuities": "discontinuity",
    "discordances": "discordance",
    "discords": "discord",
    "discos": "disco",
    "discotheques": "discotheque",
    "discothèques": "discothèque",
    "discounted": "discount",
    "discountenanced": "discountenance",
    "discountenances": "discountenance",
    "discountenancing": "discountenance",
    "discounting": "discount",
    "discounts": "discount",
    "discouraged": "discourage",
    "discouragements": "discouragement",
    "discourages": "discourage",
    "discouraging": "discourage",
    "discoursed": "discourse",
    "discourses": "discourse",
    "discoursing": "discourse",
    "discourtesies": "discourtesy",
    "discovered": "discover",
    "discoverers": "discoverer",
    "discoveries": "discovery",
    "discovering": "discover",
    "discovers": "discover",
    "discredited": "discredit",
    "discrediting": "discredit",
    "discredits": "discredit",
    "discrepancies": "discrepancy",
    "discretisations": "discretisation",
    "discretizations": "discretization",
    "discriminants": "discriminant",
    "discriminated": "discriminate",
    "discriminates": "discriminate",
    "discriminating": "discriminate",
    "discriminations": "discrimination",
    "discriminators": "discriminator",
    "discs": "disc",
    "discuses": "discus",
    "discussants": "discussant",
    "discussed": "discuss",
    "discusses": "discuss",
    "discussing": "discuss",
    "discussions": "discussion",
    "disdained": "disdain",
    "disdainfully": "disdainful",
    "disdaining": "disdain",
    "disdains": "disdain",
    "diseases": "disease",
    "disembarkations": "disembarkation",
    "disembarked": "disembark",
    "disembarking": "disembark",
    "disembarks": "disembark",
    "disemboweled": "disembowel",
    "disemboweling": "disembowel",
    "disembowelled": "disembowel",
    "disembowelling": "disembowel",
    "disembowels": "disembowel",
    "disempowered": "disempower",
    "disempowering": "disempower",
    "disempowers": "disempower",
    "disenchanted": "disenchant",
    "disenchanting": "disenchant",
    "disenchantments": "disenchantment",
    "disenchants": "disenchant",
    "disenfranchised": "disenfranchise",
    "disenfranchises": "disenfranchise",
    "disenfranchising": "disenfranchise",
    "disengaged": "disengage",
    "disengagements": "disengagement",
    "disengages": "disengage",
    "disengaging": "disengage",
    "disentangled": "disentangle",
    "disentangles": "disentangle",
    "disentangling": "disentangle",
    "disequilibria": "disequilibrium",
    "disequilibriums": "disequilibrium",
    "dises": "dis",
    "disestablished": "disestablish",
    "disestablishes": "disestablish",
    "disestablishing": "disestablish",
    "disfavoured": "disfavour",
    "disfavouring": "disfavour",
    "disfavours": "disfavour",
    "disfigured": "disfigure",
    "disfigurements": "disfigurement",
    "disfigures": "disfigure",
    "disfiguring": "disfigure",
    "disfranchised": "disfranchise",
    "disfranchises": "disfranchise",
    "disfranchising": "disfranchise",
    "disgorged": "disgorge",
    "disgorges": "disgorge",
    "disgorging": "disgorge",
    "disgraced": "disgrace",
    "disgraces": "disgrace",
    "disgracing": "disgrace",
    "disguised": "disguise",
    "disguises": "disguise",
    "disguising": "disguise",
    "disgusted": "disgust",
    "disgustedly": "disgusted",
    "disgusting": "disgust",
    "disgusts": "disgust",
    "disharmonies": "disharmony",
    "dishcloths": "dishcloth",
    "disheartened": "dishearten",
    "disheartening": "dishearten",
    "disheartens": "dishearten",
    "dished": "dish",
    "dishes": "dish",
    "disheveled": "dishevel",
    "disheveling": "dishevel",
    "dishevelled": "dishevel",
    "dishevelling": "dishevel",
    "dishevels": "dishevel",
    "dishier": "dishy",
    "dishing": "dish",
    "dishonesties": "dishonesty",
    "dishonored": "dishonor",
    "dishonoring": "dishonor",
    "dishonors": "dishonor",
    "dishonoured": "dishonour",
    "dishonouring": "dishonour",
    "dishonours": "dishonour",
    "dishwashers": "dishwasher",
    "disillusioned": "disillusion",
    "disillusioning": "disillusion",
    "disillusionments": "disillusionment",
    "disillusions": "disillusion",
    "disincentives": "disincentive",
    "disinclined": "disincline",
    "disinclines": "disincline",
    "disinclining": "disincline",
    "disinfectants": "disinfectant",
    "disinfected": "disinfect",
    "disinfecting": "disinfect",
    "disinfections": "disinfection",
    "disinfects": "disinfect",
    "disinherited": "disinherit",
    "disinheriting": "disinherit",
    "disinherits": "disinherit",
    "disintegrated": "disintegrate",
    "disintegrates": "disintegrate",
    "disintegrating": "disintegrate",
    "disintegrations": "disintegration",
    "disinterred": "disinter",
    "disinterring": "disinter",
    "disinters": "disinter",
    "disinvestments": "disinvestment",
    "disjoined": "disjoin",
    "disjoining": "disjoin",
    "disjoins": "disjoin",
    "disjointed": "disjoint",
    "disjointing": "disjoint",
    "disjoints": "disjoint",
    "disjunctions": "disjunction",
    "diskettes": "diskette",
    "disks": "disk",
    "disliked": "dislike",
    "dislikes": "dislike",
    "disliking": "dislike",
    "dislocated": "dislocate",
    "dislocates": "dislocate",
    "dislocating": "dislocate",
    "dislocations": "dislocation",
    "dislodged": "dislodge",
    "dislodges": "dislodge",
    "dislodging": "dislodge",
    "disloyalties": "disloyalty",
    "dismaller": "dismal",
    "dismallest": "dismal",
    "dismantled": "dismantle",
    "dismantles": "dismantle",
    "dismantling": "dismantle",
    "dismasted": "dismast",
    "dismasting": "dismast",
    "dismasts": "dismast",
    "dismayed": "dismay",
    "dismaying": "dismay",
    "dismays": "dismay",
    "dismembered": "dismember",
    "dismembering": "dismember",
    "dismemberments": "dismemberment",
    "dismembers": "dismember",
    "dismissals": "dismissal",
    "dismissed": "dismiss",
    "dismisses": "dismiss",
    "dismissing": "dismiss",
    "dismounted": "dismount",
    "dismounting": "dismount",
    "dismounts": "dismount",
    "dismutases": "dismutase",
    "disobeyed": "disobey",
    "disobeying": "disobey",
    "disobeys": "disobey",
    "disopyramides": "disopyramide",
    "disordered": "disorder",
    "disordering": "disorder",
    "disorders": "disorder",
    "disorganisations": "disorganisation",
    "disorganised": "disorganise",
    "disorganises": "disorganise",
    "disorganising": "disorganise",
    "disorganizations": "disorganization",
    "disorganized": "disorganize",
    "disorganizes": "disorganize",
    "disorganizing": "disorganize",
    "disorientated": "disorientate",
    "disorientates": "disorientate",
    "disorientating": "disorientate",
    "disorientations": "disorientation",
    "disoriented": "disorient",
    "disorienting": "disorient",
    "disorients": "disorient",
    "disowned": "disown",
    "disowning": "disown",
    "disowns": "disown",
    "disparaged": "disparage",
    "disparagements": "disparagement",
    "disparages": "disparage",
    "disparaging": "disparage",
    "disparities": "disparity",
    "dispatched": "dispatch",
    "dispatchers": "dispatcher",
    "dispatches": "dispatch",
    "dispatching": "dispatch",
    "dispelled": "dispel",
    "dispelling": "dispel",
    "dispels": "dispel",
    "dispensaries": "dispensary",
    "dispensations": "dispensation",
    "dispensed": "dispense",
    "dispensers": "dispenser",
    "dispenses": "dispense",
    "dispensing": "dispense",
    "dispersals": "dispersal",
    "dispersants": "dispersant",
    "dispersed": "disperse",
    "disperses": "disperse",
    "dispersing": "disperse",
    "dispersions": "dispersion",
    "dispirited": "dispirit",
    "dispiriting": "dispirit",
    "dispirits": "dispirit",
    "displaced": "displace",
    "displacements": "displacement",
    "displaces": "displace",
    "displacing": "displace",
    "displayed": "display",
    "displaying": "display",
    "displays": "display",
    "displeased": "displease",
    "displeases": "displease",
    "displeasing": "displease",
    "disported": "disport",
    "disporting": "disport",
    "disports": "disport",
    "disposables": "disposable",
    "disposals": "disposal",
    "disposed": "dispose",
    "disposers": "disposer",
    "disposes": "dispose",
    "disposing": "dispose",
    "dispositions": "disposition",
    "dispossessed": "dispossess",
    "dispossesses": "dispossess",
    "dispossessing": "dispossess",
    "dispossessions": "dispossession",
    "disproofs": "disproof",
    "disproportionalities": "disproportionality",
    "disproportions": "disproportion",
    "disproved": "disprove",
    "disproven": "disprove",
    "disproves": "disprove",
    "disproving": "disprove",
    "disputants": "disputant",
    "disputations": "disputation",
    "disputed": "dispute",
    "disputes": "dispute",
    "disputing": "dispute",
    "disqualifications": "disqualification",
    "disqualified": "disqualify",
    "disqualifies": "disqualify",
    "disqualifying": "disqualify",
    "disquieted": "disquiet",
    "disquieting": "disquiet",
    "disquiets": "disquiet",
    "disquisitions": "disquisition",
    "disregarded": "disregard",
    "disregarding": "disregard",
    "disregards": "disregard",
    "disrespected": "disrespect",
    "disrespecting": "disrespect",
    "disrespects": "disrespect",
    "disrobed": "disrobe",
    "disrobes": "disrobe",
    "disrobing": "disrobe",
    "disrupted": "disrupt",
    "disrupters": "disrupter",
    "disrupting": "disrupt",
    "disruptions": "disruption",
    "disruptors": "disruptor",
    "disrupts": "disrupt",
    "dissatisfactions": "dissatisfaction",
    "dissatisfied": "dissatisfy",
    "dissatisfies": "dissatisfy",
    "dissatisfying": "dissatisfy",
    "dissected": "dissect",
    "dissecting": "dissect",
    "dissections": "dissection",
    "dissects": "dissect",
    "dissed": "dis",
    "dissembled": "dissemble",
    "dissembles": "dissemble",
    "dissembling": "dissemble",
    "disseminated": "disseminate",
    "disseminates": "disseminate",
    "disseminating": "disseminate",
    "disseminations": "dissemination",
    "disseminators": "disseminator",
    "dissensions": "dissension",
    "dissented": "dissent",
    "dissenters": "dissenter",
    "dissenting": "dissent",
    "dissents": "dissent",
    "dissertations": "dissertation",
    "disservices": "disservice",
    "dissidents": "dissident",
    "dissimilarities": "dissimilarity",
    "dissimulated": "dissimulate",
    "dissimulates": "dissimulate",
    "dissimulating": "dissimulate",
    "dissimulations": "dissimulation",
    "dissing": "dis",
    "dissipated": "dissipate",
    "dissipates": "dissipate",
    "dissipating": "dissipate",
    "dissipations": "dissipation",
    "dissociated": "dissociate",
    "dissociates": "dissociate",
    "dissociating": "dissociate",
    "dissociations": "dissociation",
    "dissolutions": "dissolution",
    "dissolved": "dissolve",
    "dissolves": "dissolve",
    "dissolving": "dissolve",
    "dissonances": "dissonance",
    "dissuaded": "dissuade",
    "dissuades": "dissuade",
    "dissuading": "dissuade",
    "distaffs": "distaff",
    "distanced": "distance",
    "distances": "distance",
    "distancing": "distance",
    "distastes": "distaste",
    "distended": "distend",
    "distending": "distend",
    "distends": "distend",
    "distensions": "distension",
    "distichs": "distich",
    "distillates": "distillate",
    "distillations": "distillation",
    "distilled": "distill",
    "distilleries": "distillery",
    "distillers": "distiller",
    "distilling": "distill",
    "distills": "distill",
    "distils": "distil",
    "distinctions": "distinction",
    "distinguished": "distinguish",
    "distinguishes": "distinguish",
    "distinguishing": "distinguish",
    "distorted": "distort",
    "distorting": "distort",
    "distortions": "distortion",
    "distorts": "distort",
    "distracted": "distract",
    "distracters": "distracter",
    "distracting": "distract",
    "distractions": "distraction",
    "distractors": "distractor",
    "distracts": "distract",
    "distrained": "distrain",
    "distraining": "distrain",
    "distrains": "distrain",
    "distressed": "distress",
    "distresses": "distress",
    "distressing": "distress",
    "distributed": "distribute",
    "distributers": "distributer",
    "distributes": "distribute",
    "distributing": "distribute",
    "distributions": "distribution",
    "distributors": "distributor",
    "distributorships": "distributorship",
    "districts": "district",
    "distrusted": "distrust",
    "distrusting": "distrust",
    "distrusts": "distrust",
    "disturbances": "disturbance",
    "disturbed": "disturb",
    "disturbers": "disturber",
    "disturbing": "disturb",
    "disturbs": "disturb",
    "disulfides": "disulfide",
    "disulphides": "disulphide",
    "disunited": "disunite",
    "disunites": "disunite",
    "disuniting": "disunite",
    "ditched": "ditch",
    "ditches": "ditch",
    "ditching": "ditch",
    "dithered": "dither",
    "dithering": "dither",
    "dithers": "dither",
    "dits": "dit",
    "ditties": "ditty",
    "dittoes": "ditto",
    "dittos": "ditto",
    "diureses": "diuresis",
    "diuretics": "diuretic",
    "divans": "divan",
    "divas": "diva",
    "dive": "diva",
    "dive-bombed": "dive-bomb",
    "dive-bombing": "dive-bomb",
    "dive-bombs": "dive-bomb",
    "dived": "dive",
    "diverged": "diverge",
    "divergences": "divergence",
    "diverges": "diverge",
    "diverging": "diverge",
    "divers": "diver",
    "diversifications": "diversification",
    "diversified": "diversify",
    "diversifies": "diversify",
    "diversifying": "diversify",
    "diversions": "diversion",
    "diversities": "diversity",
    "diverted": "divert",
    "diverters": "diverter",
    "diverticula": "diverticulum",
    "diverticulitides": "diverticulitis",
    "diverting": "divert",
    "divertissements": "divertissement",
    "diverts": "divert",
    "dives": "dive",
    "divested": "divest",
    "divesting": "divest",
    "divestitures": "divestiture",
    "divests": "divest",
    "divided": "divide",
    "dividends": "dividend",
    "dividers": "divider",
    "divides": "divide",
    "dividing": "divide",
    "divinations": "divination",
    "divined": "divine",
    "divinely": "divine",
    "diviner": "divine",
    "diviners": "diviner",
    "divines": "divine",
    "divinest": "divine",
    "diving": "dive",
    "divining": "divine",
    "divinities": "divinity",
    "divisions": "division",
    "divisors": "divisor",
    "divorced": "divorce",
    "divorcees": "divorcee",
    "divorces": "divorce",
    "divorcing": "divorce",
    "divots": "divot",
    "divs": "div",
    "divulged": "divulge",
    "divulges": "divulge",
    "divulging": "divulge",
    "divvied": "divvy",
    "divvies": "divvy",
    "divvying": "divvy",
    "dizzied": "dizzy",
    "dizzier": "dizzy",
    "dizzies": "dizzy",
    "dizziest": "dizzy",
    "dizzying": "dizzy",
    "dls": "dl",
    "dmfts": "dmft",
    "do-gooders": "do-gooder",
    "do-it-yourselfs": "do-it-yourself",
    "docked": "dock",
    "dockers": "docker",
    "docketed": "docket",
    "docketing": "docket",
    "dockets": "docket",
    "docking": "dock",
    "docklands": "dockland",
    "docks": "dock",
    "dockyards": "dockyard",
    "docs": "doc",
    "doctorates": "doctorate",
    "doctored": "doctor",
    "doctoring": "doctor",
    "doctors": "doctor",
    "doctrines": "doctrine",
    "documentaries": "documentary",
    "documentations": "documentation",
    "documented": "document",
    "documenting": "document",
    "documents": "document",
    "doddered": "dodder",
    "doddering": "dodder",
    "dodders": "dodder",
    "dodecahedra": "dodecahedron",
    "dodecahedrons": "dodecahedron",
    "dodged": "dodge",
    "dodgems": "dodgem",
    "dodgers": "dodger",
    "dodges": "dodge",
    "dodgier": "dodgy",
    "dodgiest": "dodgy",
    "dodging": "dodge",
    "dodoes": "dodo",
    "dodos": "dodo",
    "doers": "doer",
    "does": "doe",
    "doffed": "doff",
    "doffing": "doff",
    "doffs": "doff",
    "dog-collars": "dog-collar",
    "dog-legs": "dog-leg",
    "dogcarts": "dogcart",
    "dogfights": "dogfight",
    "dogfishes": "dogfish",
    "dogged": "dog",
    "doggies": "doggy",
    "dogging": "dog",
    "doghouses": "doghouse",
    "doglegs": "dogleg",
    "dogmas": "dogma",
    "dogmata": "dogma",
    "dogmatists": "dogmatist",
    "dogmatized": "dogmatize",
    "dogmatizes": "dogmatize",
    "dogmatizing": "dogmatize",
    "dogs": "dog",
    "dogsbodies": "dogsbody",
    "dogwoods": "dogwood",
    "doilies": "doily",
    "doing": "do",
    "doings": "doing",
    "doled": "dole",
    "dolerites": "dolerite",
    "doles": "dole",
    "doling": "dole",
    "dollars": "dollar",
    "dolled": "doll",
    "dollhouses": "dollhouse",
    "dollies": "dolly",
    "dolling": "doll",
    "dollops": "dollop",
    "dolls": "doll",
    "dolores": "dolor",
    "dolphins": "dolphin",
    "dolts": "dolt",
    "domains": "domain",
    "domed": "dome",
    "domes": "dome",
    "domesticated": "domesticate",
    "domesticates": "domesticate",
    "domesticating": "domesticate",
    "domestications": "domestication",
    "domesticities": "domesticity",
    "domestics": "domestic",
    "domiciled": "domicile",
    "domiciles": "domicile",
    "domiciling": "domicile",
    "dominated": "dominate",
    "dominates": "dominate",
    "dominating": "dominate",
    "domineered": "domineer",
    "domineering": "domineer",
    "domineers": "domineer",
    "doming": "dome",
    "dominions": "dominion",
    "dominoes": "domino",
    "dominos": "domino",
    "donated": "donate",
    "donates": "donate",
    "donating": "donate",
    "donations": "donation",
    "donators": "donator",
    "done": "do",
    "donees": "donee",
    "dongs": "dong",
    "donkeys": "donkey",
    "donned": "don",
    "donning": "don",
    "donors": "donor",
    "dons": "don",
    "dont": "do",
    "donuts": "donut",
    "doodahs": "doodah",
    "doodled": "doodle",
    "doodles": "doodle",
    "doodling": "doodle",
    "doomed": "doom",
    "dooming": "doom",
    "dooms": "doom",
    "doomsayers": "doomsayer",
    "doomsdays": "doomsday",
    "door-handles": "door-handle",
    "door-knobs": "door-knob",
    "door-steps": "door-step",
    "doorbells": "doorbell",
    "doorkeepers": "doorkeeper",
    "doorknobs": "doorknob",
    "doormats": "doormat",
    "doormen": "doorman",
    "doorposts": "doorpost",
    "doors": "door",
    "doorsteps": "doorstep",
    "doorstops": "doorstop",
    "doorways": "doorway",
    "dopaminergics": "dopaminergic",
    "dopants": "dopant",
    "doped": "dope",
    "dopes": "dope",
    "dopeyer": "dopey",
    "dopeyest": "dopey",
    "doping": "dope",
    "dories": "dory",
    "dormancies": "dormancy",
    "dormers": "dormer",
    "dormice": "dormouse",
    "dormitories": "dormitory",
    "dorms": "dorm",
    "dorsiflexions": "dorsiflexion",
    "dosages": "dosage",
    "dose-responses": "dose-response",
    "dosed": "dose",
    "doses": "dose",
    "dosimeters": "dosimeter",
    "dosimetries": "dosimetry",
    "dosing": "dose",
    "doss-houses": "doss-house",
    "dossed": "doss",
    "dossers": "dosser",
    "dosses": "doss",
    "dossiers": "dossier",
    "dossing": "doss",
    "doted": "dote",
    "dotes": "dote",
    "doting": "dote",
    "dots": "dot",
    "dotted": "dot",
    "dottier": "dotty",
    "dottiest": "dotty",
    "dotting": "dot",
    "double-checked": "double-check",
    "double-checking": "double-check",
    "double-checks": "double-check",
    "double-crossed": "double-cross",
    "double-crosses": "double-cross",
    "double-crossing": "double-cross",
    "double-deckers": "double-decker",
    "double-glazed": "double-glaze",
    "double-glazes": "double-glaze",
    "double-glazing": "double-glaze",
    "double-parked": "double-park",
    "double-parking": "double-park",
    "double-parks": "double-park",
    "double-takes": "double-take",
    "doublecrossed": "doublecross",
    "doublecrosses": "doublecross",
    "doublecrossing": "doublecross",
    "doubled": "double",
    "doubledeckers": "doubledecker",
    "doublers": "doubler",
    "doubles": "double",
    "doublets": "doublet",
    "doubling": "double",
    "doublings": "doubling",
    "doubted": "doubt",
    "doubters": "doubter",
    "doubting": "doubt",
    "doubts": "doubt",
    "douches": "douche",
    "doughier": "doughy",
    "doughiest": "doughy",
    "doughnuts": "doughnut",
    "doughs": "dough",
    "doughtier": "doughty",
    "doughtiest": "doughty",
    "doulas": "doula",
    "dourer": "dour",
    "dourest": "dour",
    "doused": "douse",
    "douses": "douse",
    "dousing": "douse",
    "dove": "dive",
    "dovecotes": "dovecote",
    "doves": "dove",
    "dovetailed": "dovetail",
    "dovetailing": "dovetail",
    "dovetails": "dovetail",
    "dowagers": "dowager",
    "dowdier": "dowdy",
    "dowdiest": "dowdy",
    "dowed": "dow",
    "dowels": "dowel",
    "dowing": "dow",
    "down-and-outs": "down-and-out",
    "down-regulations": "down-regulation",
    "down-sides": "down-side",
    "down-times": "down-time",
    "downbeats": "downbeat",
    "downcasts": "downcast",
    "downed": "down",
    "downers": "downer",
    "downfalls": "downfall",
    "downgraded": "downgrade",
    "downgrades": "downgrade",
    "downgrading": "downgrade",
    "downier": "downy",
    "downiest": "downy",
    "downing": "down",
    "downloaded": "download",
    "downloading": "download",
    "downloads": "download",
    "downplayed": "downplay",
    "downplaying": "downplay",
    "downplays": "downplay",
    "downpours": "downpour",
    "downs": "down",
    "downshifts": "downshift",
    "downsides": "downside",
    "downsized": "downsize",
    "downsizes": "downsize",
    "downsizing": "downsize",
    "downslopes": "downslope",
    "downswings": "downswing",
    "downtimes": "downtime",
    "downtowns": "downtown",
    "downturns": "downturn",
    "dowries": "dowry",
    "dows": "dow",
    "dowsed": "dowse",
    "dowsers": "dowser",
    "dowses": "dowse",
    "dowsing": "dowse",
    "doyennes": "doyenne",
    "doyens": "doyen",
    "dozed": "doze",
    "dozens": "dozen",
    "dozes": "doze",
    "dozier": "dozy",
    "doziest": "dozy",
    "dozing": "doze",
    "dpms": "dpm",
    "drabber": "drab",
    "drabbest": "drab",
    "drachmae": "drachma",
    "drachmai": "drachma",
    "drachmas": "drachma",
    "drachms": "drachm",
    "drafted": "draft",
    "draftees": "draftee",
    "drafters": "drafter",
    "drafting": "draft",
    "drafts": "draft",
    "draftsmen": "draftsman",
    "dragged": "drag",
    "dragging": "drag",
    "dragonflies": "dragonfly",
    "dragons": "dragon",
    "dragooned": "dragoon",
    "dragooning": "dragoon",
    "dragoons": "dragoon",
    "drags": "drag",
    "drainages": "drainage",
    "drained": "drain",
    "drainers": "drainer",
    "draining": "drain",
    "drainpipes": "drainpipe",
    "drains": "drain",
    "drakes": "drake",
    "dramas": "drama",
    "dramatics": "dramatic",
    "dramatisations": "dramatisation",
    "dramatised": "dramatise",
    "dramatises": "dramatise",
    "dramatising": "dramatise",
    "dramatists": "dramatist",
    "dramatizations": "dramatization",
    "dramatized": "dramatize",
    "dramatizes": "dramatize",
    "dramatizing": "dramatize",
    "drams": "dram",
    "drank": "drink",
    "draped": "drape",
    "draperies": "drapery",
    "drapers": "draper",
    "drapes": "drape",
    "draping": "drape",
    "draughtboards": "draughtboard",
    "draughtier": "draughty",
    "draughtiest": "draughty",
    "draughts": "draught",
    "draughtsmen": "draughtsman",
    "drawbacks": "drawback",
    "drawbridges": "drawbridge",
    "drawdowns": "drawdown",
    "drawers": "drawer",
    "drawing": "draw",
    "drawings": "drawing",
    "drawled": "drawl",
    "drawling": "drawl",
    "drawls": "drawl",
    "drawn": "draw",
    "drawn-in": "draw-in",
    "draws": "draw",
    "draws-in": "draw-in",
    "drawstrings": "drawstring",
    "drays": "dray",
    "dreaded": "dread",
    "dreading": "dread",
    "dreadlocks": "dreadlock",
    "dreads": "dread",
    "dreamed": "dream",
    "dreamers": "dreamer",
    "dreamier": "dreamy",
    "dreamiest": "dreamy",
    "dreaming": "dream",
    "dreams": "dream",
    "dreamt": "dream",
    "drearier": "dreary",
    "dreariest": "dreary",
    "dredged": "dredge",
    "dredgers": "dredger",
    "dredges": "dredge",
    "dredging": "dredge",
    "dregs": "dreg",
    "drenched": "drench",
    "drenches": "drench",
    "drenching": "drench",
    "dressed": "dress",
    "dressers": "dresser",
    "dresses": "dress",
    "dressier": "dressy",
    "dressiest": "dressy",
    "dressing": "dress",
    "dressing-gowns": "dressing-gown",
    "dressings": "dressing",
    "dressmakers": "dressmaker",
    "drew": "draw",
    "drew-in": "draw-in",
    "dribbled": "dribble",
    "dribbles": "dribble",
    "dribbling": "dribble",
    "dried": "dry",
    "drier": "dry",
    "driers": "drier",
    "dries": "dry",
    "driest": "dry",
    "drifted": "drift",
    "drifters": "drifter",
    "drifting": "drift",
    "drifts": "drift",
    "drilled": "drill",
    "drillers": "driller",
    "drilling": "drill",
    "drills": "drill",
    "drinkers": "drinker",
    "drinking": "drink",
    "drinks": "drink",
    "drip-feeded": "drip-feed",
    "drip-feeding": "drip-feed",
    "drip-feeds": "drip-feed",
    "dripped": "drip",
    "dripping": "drip",
    "drips": "drip",
    "dript": "drip",
    "drive-ins": "drive-in",
    "driveled": "drivel",
    "driveling": "drivel",
    "drivelled": "drivel",
    "drivelling": "drivel",
    "drivels": "drivel",
    "driven": "drive",
    "drivers": "driver",
    "drives": "drive",
    "driveways": "driveway",
    "driving": "drive",
    "drivings": "driving",
    "drizzled": "drizzle",
    "drizzles": "drizzle",
    "drizzling": "drizzle",
    "droits": "droit",
    "droller": "droll",
    "drollest": "droll",
    "dromedaries": "dromedary",
    "droned": "drone",
    "drones": "drone",
    "droning": "drone",
    "drooled": "drool",
    "drooling": "drool",
    "drools": "drool",
    "drooped": "droop",
    "droopier": "droopy",
    "droopiest": "droopy",
    "drooping": "droop",
    "droops": "droop",
    "drop-outs": "drop-out",
    "droplets": "droplet",
    "dropouts": "dropout",
    "dropped": "drop",
    "droppers": "dropper",
    "dropping": "drop",
    "drops": "drop",
    "dropsies": "dropsy",
    "droughts": "drought",
    "drove": "drive",
    "drovers": "drover",
    "droves": "drove",
    "drowned": "drown",
    "drowning": "drown",
    "drownings": "drowning",
    "drowns": "drown",
    "drowsed": "drowse",
    "drowses": "drowse",
    "drowsier": "drowsy",
    "drowsiest": "drowsy",
    "drowsing": "drowse",
    "drudges": "drudge",
    "drugged": "drug",
    "drugging": "drug",
    "druggists": "druggist",
    "drugs": "drug",
    "drugstores": "drugstore",
    "druids": "druid",
    "drumbeats": "drumbeat",
    "drummed": "drum",
    "drummers": "drummer",
    "drumming": "drum",
    "drums": "drum",
    "drumsticks": "drumstick",
    "drunk": "drink",
    "drunkards": "drunkard",
    "drunker": "drunk",
    "drunkest": "drunk",
    "drunks": "drunk",
    "dry": "spin-dry",
    "dry-cleaned": "dry-clean",
    "dry-cleaners": "dry-cleaner",
    "dry-cleaning": "dry-clean",
    "dry-cleanings": "dry-cleaning",
    "dry-cleans": "dry-clean",
    "dryads": "dryad",
    "dryer": "dry",
    "dryers": "dryer",
    "drying": "dry",
    "ds": "d",
    "dualisms": "dualism",
    "dualists": "dualist",
    "dualities": "duality",
    "dubbed": "dub",
    "dubbing": "dub",
    "dubs": "dub",
    "duchesses": "duchess",
    "duchies": "duchy",
    "ducked": "duck",
    "ducking": "duck",
    "duckings": "ducking",
    "ducklings": "duckling",
    "ducks": "duck",
    "ducts": "duct",
    "dudes": "dude",
    "duds": "dud",
    "dueled": "duel",
    "dueling": "duel",
    "duelled": "duel",
    "duelling": "duel",
    "duels": "duel",
    "dues": "due",
    "duets": "duet",
    "duffed": "duff",
    "duffels": "duffel",
    "duffers": "duffer",
    "duffing": "duff",
    "duffles": "duffle",
    "duffs": "duff",
    "dug": "dig",
    "dugongs": "dugong",
    "dugouts": "dugout",
    "duikers": "duiker",
    "dukedoms": "dukedom",
    "dukes": "duke",
    "dullards": "dullard",
    "dulled": "dull",
    "duller": "dull",
    "dullest": "dull",
    "dulling": "dull",
    "dulls": "dull",
    "dum-dums": "dum-dum",
    "dumbbells": "dumbbell",
    "dumber": "dumb",
    "dumbest": "dumb",
    "dumbos": "dumbo",
    "dummies": "dummy",
    "dumped": "dump",
    "dumpers": "dumper",
    "dumpier": "dumpy",
    "dumpiest": "dumpy",
    "dumping": "dump",
    "dumplings": "dumpling",
    "dumps": "dump",
    "dunces": "dunce",
    "dunes": "dune",
    "dungeons": "dungeon",
    "dunked": "dunk",
    "dunking": "dunk",
    "dunks": "dunk",
    "dunlins": "dunlin",
    "dunned": "dun",
    "dunning": "dun",
    "dunnocks": "dunnock",
    "duns": "dun",
    "duodena": "duodenum",
    "duodenums": "duodenum",
    "duopolies": "duopoly",
    "duos": "duo",
    "duped": "dupe",
    "dupes": "dupe",
    "duping": "dupe",
    "duplexes": "duplex",
    "duplicated": "duplicate",
    "duplicates": "duplicate",
    "duplicating": "duplicate",
    "duplications": "duplication",
    "duplicators": "duplicator",
    "duplices": "duplex",
    "duplicities": "duplicity",
    "durabilities": "durability",
    "durables": "durable",
    "duras": "dura",
    "durations": "duration",
    "duresses": "duress",
    "dus": "du",
    "duskier": "dusky",
    "duskiest": "dusky",
    "dusks": "dusk",
    "dust-jackets": "dust-jacket",
    "dust-ups": "dust-up",
    "dustbins": "dustbin",
    "dustcarts": "dustcart",
    "dusted": "dust",
    "dusters": "duster",
    "dustier": "dusty",
    "dustiest": "dusty",
    "dusting": "dust",
    "dustmen": "dustman",
    "dustpans": "dustpan",
    "dusts": "dust",
    "dustsheets": "dustsheet",
    "duties": "duty",
    "dutifully": "dutiful",
    "duvets": "duvet",
    "dvs": "dv",
    "dwarfed": "dwarf",
    "dwarfing": "dwarf",
    "dwarfisms": "dwarfism",
    "dwarfs": "dwarf",
    "dwarves": "dwarf",
    "dwelled": "dwell",
    "dwellers": "dweller",
    "dwelling": "dwell",
    "dwelling-houses": "dwelling-house",
    "dwellings": "dwelling",
    "dwells": "dwell",
    "dwelt": "dwell",
    "dwindled": "dwindle",
    "dwindles": "dwindle",
    "dwindling": "dwindle",
    "dws": "dw",
    "dxs": "dx",
    "dyads": "dyad",
    "dyed": "dye",
    "dyeing": "dye",
    "dyers": "dyer",
    "dyes": "dye",
    "dyestuffs": "dyestuff",
    "dying": "dye",
    "dyked": "dyke",
    "dykes": "dyke",
    "dyking": "dyke",
    "dynamics": "dynamic",
    "dynamisms": "dynamism",
    "dynamited": "dynamite",
    "dynamites": "dynamite",
    "dynamiting": "dynamite",
    "dynamometers": "dynamometer",
    "dynamos": "dynamo",
    "dynasties": "dynasty",
    "dyneins": "dynein",
    "dyns": "dyn",
    "dysarthrias": "dysarthria",
    "dyscalculias": "dyscalculia",
    "dyscrasias": "dyscrasia",
    "dysenteries": "dysentery",
    "dysfunctions": "dysfunction",
    "dyskinesiae": "dyskinesia",
    "dyskinesias": "dyskinesia",
    "dyslexias": "dyslexia",
    "dyslexics": "dyslexic",
    "dyslipidemias": "dyslipidemia",
    "dysmenorrhoeas": "dysmenorrhoea",
    "dyspepsiae": "dyspepsia",
    "dyspepsias": "dyspepsia",
    "dysphagias": "dysphagia",
    "dysphasias": "dysphasia",
    "dysphorias": "dysphoria",
    "dysplasiae": "dysplasia",
    "dysplasias": "dysplasia",
    "dyspnoeas": "dyspnoea",
    "dysregulations": "dysregulation",
    "dystocias": "dystocia",
    "dystonias": "dystonia",
    "dystopias": "dystopia",
    "dystrophies": "dystrophy",
    "dystrophins": "dystrophin",
    "débuts": "début",
    "décors": "décor",
    "e-mailed": "e-mail",
    "e-mailing": "e-mail",
    "e-mails": "e-mail",
    "eagles": "eagle",
    "earaches": "earache",
    "eardrums": "eardrum",
    "eared": "ear",
    "earing": "ear",
    "earldoms": "earldom",
    "earlier": "early",
    "earliest": "early",
    "earlobes": "earlobe",
    "earls": "earl",
    "earmarked": "earmark",
    "earmarking": "earmark",
    "earmarks": "earmark",
    "earmoulds": "earmould",
    "earned": "earn",
    "earners": "earner",
    "earning": "earn",
    "earns": "earn",
    "earphones": "earphone",
    "earpieces": "earpiece",
    "earplugs": "earplug",
    "earrings": "earring",
    "ears": "ear",
    "earthed": "earth",
    "earthier": "earthy",
    "earthing": "earth",
    "earthquakes": "earthquake",
    "earths": "earth",
    "earthworks": "earthwork",
    "earthworms": "earthworm",
    "earwigs": "earwig",
    "eased": "ease",
    "easels": "easel",
    "easements": "easement",
    "eases": "ease",
    "easier": "easy",
    "easiest": "easy",
    "easing": "ease",
    "easterlies": "easterly",
    "easterners": "easterner",
    "eastwards": "eastward",
    "eaten": "eat",
    "eateries": "eatery",
    "eaters": "eater",
    "eating": "eat",
    "eats": "eat",
    "eaves": "eave",
    "eavesdropped": "eavesdrop",
    "eavesdroppers": "eavesdropper",
    "eavesdropping": "eavesdrop",
    "eavesdrops": "eavesdrop",
    "ebbed": "ebb",
    "ebbing": "ebb",
    "ebbs": "ebb",
    "ebonies": "ebony",
    "eccentricities": "eccentricity",
    "eccentrics": "eccentric",
    "ecclesiae": "ecclesia",
    "ecclesiastics": "ecclesiastic",
    "echelons": "echelon",
    "echinoderms": "echinoderm",
    "echinoids": "echinoid",
    "echocardiograms": "echocardiogram",
    "echocardiographies": "echocardiography",
    "echoed": "echo",
    "echoes": "echo",
    "echoing": "echo",
    "echolocations": "echolocation",
    "echos": "echo",
    "eclairs": "eclair",
    "eclampsias": "eclampsia",
    "eclectics": "eclectic",
    "eclipsed": "eclipse",
    "eclipses": "eclipse",
    "eclipsing": "eclipse",
    "eco-systems": "eco-system",
    "ecologies": "ecology",
    "ecologists": "ecologist",
    "economies": "economy",
    "economised": "economise",
    "economises": "economise",
    "economising": "economise",
    "economists": "economist",
    "economized": "economize",
    "economizes": "economize",
    "economizing": "economize",
    "ecosystems": "ecosystem",
    "ecstasies": "ecstasy",
    "ectoparasites": "ectoparasite",
    "ecus": "ecu",
    "eczemas": "eczema",
    "eczemata": "eczema",
    "eddied": "eddy",
    "eddies": "eddy",
    "eddying": "eddy",
    "edemas": "edema",
    "edemata": "edema",
    "edged": "edge",
    "edges": "edge",
    "edgier": "edgy",
    "edgiest": "edgy",
    "edging": "edge",
    "edgings": "edging",
    "edicts": "edict",
    "edifices": "edifice",
    "edified": "edify",
    "edifies": "edify",
    "edifying": "edify",
    "edited": "edit",
    "editing": "edit",
    "editions": "edition",
    "editorialised": "editorialise",
    "editorialises": "editorialise",
    "editorialising": "editorialise",
    "editorialized": "editorialize",
    "editorializes": "editorialize",
    "editorializing": "editorialize",
    "editorials": "editorial",
    "editors": "editor",
    "editorships": "editorship",
    "edits": "edit",
    "eds": "ed",
    "educated": "educate",
    "educates": "educate",
    "educating": "educate",
    "educationalists": "educationalist",
    "educationists": "educationist",
    "educations": "education",
    "educators": "educator",
    "eels": "eel",
    "eerier": "eery",
    "eeriest": "eery",
    "effaced": "efface",
    "effaces": "efface",
    "effacing": "efface",
    "effected": "effect",
    "effecting": "effect",
    "effectivenesses": "effectiveness",
    "effectors": "effector",
    "effects": "effect",
    "effectuated": "effectuate",
    "effectuates": "effectuate",
    "effectuating": "effectuate",
    "effed": "eff",
    "efficacies": "efficacy",
    "efficiencies": "efficiency",
    "effigies": "effigy",
    "effing": "eff",
    "efflorescences": "efflorescence",
    "effluents": "effluent",
    "effluvia": "effluvium",
    "effluviums": "effluvium",
    "effluxes": "efflux",
    "efforts": "effort",
    "effronteries": "effrontery",
    "effs": "eff",
    "effusions": "effusion",
    "egalitarians": "egalitarian",
    "egg-timers": "egg-timer",
    "eggcups": "eggcup",
    "egged": "egg",
    "eggheads": "egghead",
    "egging": "egg",
    "eggplants": "eggplant",
    "eggs": "egg",
    "eggshells": "eggshell",
    "egoists": "egoist",
    "egomaniacs": "egomaniac",
    "egos": "ego",
    "egotists": "egotist",
    "egressed": "egress",
    "egresses": "egress",
    "egressing": "egress",
    "egrets": "egret",
    "egs": "eg",
    "eiderdowns": "eiderdown",
    "eiders": "eider",
    "eigenfunctions": "eigenfunction",
    "eigenstates": "eigenstate",
    "eigenvalues": "eigenvalue",
    "eigenvectors": "eigenvector",
    "eight-hundredth": "800",
    "eighteenth": "18",
    "eighteenths": "eighteenth",
    "eighth": "8",
    "eighths": "eighth",
    "eighties": "eighty",
    "eightieth": "80",
    "eightieths": "eightieth",
    "eights": "eight",
    "eighty-eighth": "88",
    "eighty-eighths": "eighty-eighth",
    "eighty-fifth": "85",
    "eighty-fifths": "eighty-fifth",
    "eighty-first": "81",
    "eighty-firsts": "eighty-first",
    "eighty-fourth": "84",
    "eighty-fourths": "eighty-fourth",
    "eighty-ninth": "89",
    "eighty-ninths": "eighty-ninth",
    "eighty-second": "82",
    "eighty-seconds": "eighty-second",
    "eighty-seventh": "87",
    "eighty-sevenths": "eighty-seventh",
    "eighty-sixth": "86",
    "eighty-sixths": "eighty-sixth",
    "eighty-third": "83",
    "eighty-thirds": "eighty-third",
    "eisteddfods": "eisteddfod",
    "ejaculated": "ejaculate",
    "ejaculates": "ejaculate",
    "ejaculating": "ejaculate",
    "ejaculations": "ejaculation",
    "ejected": "eject",
    "ejecting": "eject",
    "ejections": "ejection",
    "ejectors": "ejector",
    "ejects": "eject",
    "ejidos": "ejido",
    "eked": "eke",
    "ekes": "eke",
    "eking": "eke",
    "elaborated": "elaborate",
    "elaborates": "elaborate",
    "elaborating": "elaborate",
    "elaborations": "elaboration",
    "elands": "eland",
    "elapsed": "elapse",
    "elapses": "elapse",
    "elapsing": "elapse",
    "elasticities": "elasticity",
    "elastics": "elastic",
    "elastins": "elastin",
    "elastomers": "elastomer",
    "elated": "elate",
    "elates": "elate",
    "elating": "elate",
    "elbowed": "elbow",
    "elbowing": "elbow",
    "elbows": "elbow",
    "elder": "old",
    "elderberries": "elderberry",
    "elderlier": "elderly",
    "elderliest": "elderly",
    "elders": "elder",
    "elected": "elect",
    "electing": "elect",
    "electioneered": "electioneer",
    "electioneering": "electioneer",
    "electioneers": "electioneer",
    "elections": "election",
    "electives": "elective",
    "electorates": "electorate",
    "electors": "elector",
    "electricians": "electrician",
    "electrified": "electrify",
    "electrifies": "electrify",
    "electrifying": "electrify",
    "electrocardiograms": "electrocardiogram",
    "electrochemistries": "electrochemistry",
    "electrocuted": "electrocute",
    "electrocutes": "electrocute",
    "electrocuting": "electrocute",
    "electrocutions": "electrocution",
    "electrodes": "electrode",
    "electroencephalograms": "electroencephalogram",
    "electrolyses": "electrolysis",
    "electrolytes": "electrolyte",
    "electromagnets": "electromagnet",
    "electrometers": "electrometer",
    "electromyographies": "electromyography",
    "electronegativities": "electronegativity",
    "electronics": "electronic",
    "electrons": "electron",
    "electrophiles": "electrophile",
    "electrophoreses": "electrophoresis",
    "electrophysiologies": "electrophysiology",
    "electroplated": "electroplate",
    "electroplates": "electroplate",
    "electroplating": "electroplate",
    "electros": "electro",
    "electroscopes": "electroscope",
    "electrotherapies": "electrotherapy",
    "elects": "elect",
    "elegies": "elegy",
    "elements": "element",
    "elephants": "elephant",
    "elevated": "elevate",
    "elevates": "elevate",
    "elevating": "elevate",
    "elevations": "elevation",
    "elevators": "elevator",
    "elevens": "eleven",
    "eleventh": "11",
    "elevenths": "eleventh",
    "elicitations": "elicitation",
    "elicited": "elicit",
    "eliciting": "elicit",
    "elicitors": "elicitor",
    "elicits": "elicit",
    "elided": "elide",
    "elides": "elide",
    "eliding": "elide",
    "eligibilities": "eligibility",
    "eligibles": "eligible",
    "eliminated": "eliminate",
    "eliminates": "eliminate",
    "eliminating": "eliminate",
    "eliminations": "elimination",
    "eliminators": "eliminator",
    "elisions": "elision",
    "elites": "elite",
    "elitists": "elitist",
    "elixirs": "elixir",
    "elks": "elk",
    "ellipses": "ellipsis",
    "ellipsoids": "ellipsoid",
    "ellipticals": "elliptical",
    "ellipticities": "ellipticity",
    "elms": "elm",
    "elongated": "elongate",
    "elongates": "elongate",
    "elongating": "elongate",
    "elongations": "elongation",
    "eloped": "elope",
    "elopements": "elopement",
    "elopes": "elope",
    "eloping": "elope",
    "els": "el",
    "elucidated": "elucidate",
    "elucidates": "elucidate",
    "elucidating": "elucidate",
    "elucidations": "elucidation",
    "eluded": "elude",
    "eludes": "elude",
    "eluding": "elude",
    "eluted": "elute",
    "elutes": "elute",
    "eluting": "elute",
    "elutions": "elution",
    "elves": "elf",
    "elytra": "elytrum",
    "elytrons": "elytron",
    "emaciated": "emaciate",
    "emaciates": "emaciate",
    "emaciating": "emaciate",
    "emailed": "email",
    "emailing": "email",
    "emails": "email",
    "emanated": "emanate",
    "emanates": "emanate",
    "emanating": "emanate",
    "emanations": "emanation",
    "emancipated": "emancipate",
    "emancipates": "emancipate",
    "emancipating": "emancipate",
    "emasculated": "emasculate",
    "emasculates": "emasculate",
    "emasculating": "emasculate",
    "emasculations": "emasculation",
    "embalmed": "embalm",
    "embalming": "embalm",
    "embalms": "embalm",
    "embanked": "embank",
    "embanking": "embank",
    "embankments": "embankment",
    "embanks": "embank",
    "embargoed": "embargo",
    "embargoes": "embargo",
    "embargoing": "embargo",
    "embargos": "embargo",
    "embarkations": "embarkation",
    "embarked": "embark",
    "embarking": "embark",
    "embarks": "embark",
    "embarrassed": "embarrass",
    "embarrasses": "embarrass",
    "embarrassing": "embarrass",
    "embarrassments": "embarrassment",
    "embassies": "embassy",
    "embayments": "embayment",
    "embedded": "embed",
    "embedding": "embed",
    "embeds": "embed",
    "embellished": "embellish",
    "embellishes": "embellish",
    "embellishing": "embellish",
    "embellishments": "embellishment",
    "embers": "ember",
    "embezzled": "embezzle",
    "embezzlements": "embezzlement",
    "embezzles": "embezzle",
    "embezzling": "embezzle",
    "embittered": "embitter",
    "embittering": "embitter",
    "embitters": "embitter",
    "emblazoned": "emblazon",
    "emblazoning": "emblazon",
    "emblazons": "emblazon",
    "emblems": "emblem",
    "embodied": "embody",
    "embodies": "embody",
    "embodiments": "embodiment",
    "embodying": "embody",
    "emboldened": "embolden",
    "emboldening": "embolden",
    "emboldens": "embolden",
    "emboli": "embolus",
    "embolisations": "embolisation",
    "embolisms": "embolism",
    "embolizations": "embolization",
    "embossed": "emboss",
    "embossers": "embosser",
    "embosses": "emboss",
    "embossing": "emboss",
    "embouchures": "embouchure",
    "embraced": "embrace",
    "embraces": "embrace",
    "embracing": "embrace",
    "embrasures": "embrasure",
    "embrocations": "embrocation",
    "embroidered": "embroider",
    "embroiderers": "embroiderer",
    "embroideries": "embroidery",
    "embroidering": "embroider",
    "embroiders": "embroider",
    "embroiled": "embroil",
    "embroiling": "embroil",
    "embroils": "embroil",
    "embryoes": "embryo",
    "embryogeneses": "embryogenesis",
    "embryologies": "embryology",
    "embryos": "embryo",
    "emendations": "emendation",
    "emended": "emend",
    "emending": "emend",
    "emends": "emend",
    "emeralds": "emerald",
    "emerged": "emerge",
    "emergences": "emergence",
    "emergencies": "emergency",
    "emerges": "emerge",
    "emerging": "emerge",
    "emeries": "emery",
    "emeses": "emesis",
    "emetics": "emetic",
    "emigrants": "emigrant",
    "emigrated": "emigrate",
    "emigrates": "emigrate",
    "emigrating": "emigrate",
    "emigrations": "emigration",
    "emigres": "emigre",
    "eminences": "eminence",
    "emirates": "emirate",
    "emirs": "emir",
    "emissaries": "emissary",
    "emissions": "emission",
    "emissivities": "emissivity",
    "emits": "emit",
    "emitted": "emit",
    "emitters": "emitter",
    "emitting": "emit",
    "emollients": "emollient",
    "emoluments": "emolument",
    "emoted": "emote",
    "emotes": "emote",
    "emoting": "emote",
    "emotionalities": "emotionality",
    "emotions": "emotion",
    "empathised": "empathise",
    "empathises": "empathise",
    "empathising": "empathise",
    "empathized": "empathize",
    "empathizes": "empathize",
    "empathizing": "empathize",
    "emperors": "emperor",
    "emphases": "emphasis",
    "emphasised": "emphasise",
    "emphasises": "emphasise",
    "emphasising": "emphasise",
    "emphasized": "emphasize",
    "emphasizes": "emphasize",
    "emphasizing": "emphasize",
    "emphysemas": "emphysema",
    "emphysemata": "emphysema",
    "empires": "empire",
    "empiricists": "empiricist",
    "emplaced": "emplace",
    "emplacements": "emplacement",
    "emplaces": "emplace",
    "emplacing": "emplace",
    "employabilities": "employability",
    "employed": "employ",
    "employees": "employee",
    "employers": "employer",
    "employing": "employ",
    "employments": "employment",
    "employs": "employ",
    "emporia": "emporium",
    "emporiums": "emporium",
    "empowered": "empower",
    "empowering": "empower",
    "empowers": "empower",
    "empresses": "empress",
    "emptied": "empty",
    "emptier": "empty",
    "empties": "empty",
    "emptiest": "empty",
    "emptying": "empty",
    "emulated": "emulate",
    "emulates": "emulate",
    "emulating": "emulate",
    "emulations": "emulation",
    "emulators": "emulator",
    "emulsified": "emulsify",
    "emulsifiers": "emulsifier",
    "emulsifies": "emulsify",
    "emulsifying": "emulsify",
    "emulsioned": "emulsion",
    "emulsioning": "emulsion",
    "emulsions": "emulsion",
    "emus": "emu",
    "enabled": "enable",
    "enablers": "enabler",
    "enables": "enable",
    "enabling": "enable",
    "enacted": "enact",
    "enacting": "enact",
    "enactments": "enactment",
    "enacts": "enact",
    "enameled": "enamel",
    "enameling": "enamel",
    "enamelled": "enamel",
    "enamelling": "enamel",
    "enamels": "enamel",
    "enamored": "enamor",
    "enamoring": "enamor",
    "enamors": "enamor",
    "enamoured": "enamour",
    "enamouring": "enamour",
    "enamours": "enamour",
    "enantiomers": "enantiomer",
    "encamped": "encamp",
    "encamping": "encamp",
    "encampments": "encampment",
    "encamps": "encamp",
    "encapsulated": "encapsulate",
    "encapsulates": "encapsulate",
    "encapsulating": "encapsulate",
    "encapsulations": "encapsulation",
    "encased": "encase",
    "encases": "encase",
    "encashed": "encash",
    "encashes": "encash",
    "encashing": "encash",
    "encasing": "encase",
    "encephalitides": "encephalitis",
    "encephalomyelitides": "encephalomyelitis",
    "encephalopathies": "encephalopathy",
    "enchanted": "enchant",
    "enchanters": "enchanter",
    "enchanting": "enchant",
    "enchantments": "enchantment",
    "enchantresses": "enchantress",
    "enchants": "enchant",
    "enciphered": "encipher",
    "enciphering": "encipher",
    "enciphers": "encipher",
    "encircled": "encircle",
    "encirclements": "encirclement",
    "encircles": "encircle",
    "encircling": "encircle",
    "enclaves": "enclave",
    "enclosed": "enclose",
    "encloses": "enclose",
    "enclosing": "enclose",
    "enclosures": "enclosure",
    "encoded": "encode",
    "encoders": "encoder",
    "encodes": "encode",
    "encoding": "encode",
    "encodings": "encoding",
    "encomia": "encomium",
    "encomiums": "encomium",
    "encompassed": "encompass",
    "encompasses": "encompass",
    "encompassing": "encompass",
    "encored": "encore",
    "encores": "encore",
    "encoring": "encore",
    "encountered": "encounter",
    "encountering": "encounter",
    "encounters": "encounter",
    "encouraged": "encourage",
    "encouragements": "encouragement",
    "encouragers": "encourager",
    "encourages": "encourage",
    "encouraging": "encourage",
    "encroached": "encroach",
    "encroaches": "encroach",
    "encroaching": "encroach",
    "encroachments": "encroachment",
    "encrustations": "encrustation",
    "encrusted": "encrust",
    "encrusting": "encrust",
    "encrusts": "encrust",
    "encrypted": "encrypt",
    "encrypting": "encrypt",
    "encryptions": "encryption",
    "encrypts": "encrypt",
    "encumbered": "encumber",
    "encumbering": "encumber",
    "encumbers": "encumber",
    "encumbrances": "encumbrance",
    "encyclicals": "encyclical",
    "encyclopaedias": "encyclopaedia",
    "encyclopedias": "encyclopedia",
    "encysted": "encyst",
    "encysting": "encyst",
    "encysts": "encyst",
    "end-points": "end-point",
    "end-products": "end-product",
    "end-users": "end-user",
    "endangered": "endanger",
    "endangering": "endanger",
    "endangers": "endanger",
    "endarterectomies": "endarterectomy",
    "endeared": "endear",
    "endearing": "endear",
    "endearments": "endearment",
    "endears": "endear",
    "endeavored": "endeavor",
    "endeavoring": "endeavor",
    "endeavors": "endeavor",
    "endeavoured": "endeavour",
    "endeavouring": "endeavour",
    "endeavours": "endeavour",
    "ended": "end",
    "endemics": "endemic",
    "endemisms": "endemism",
    "endgames": "endgame",
    "ending": "end",
    "endings": "ending",
    "endives": "endive",
    "endocarditides": "endocarditis",
    "endocrines": "endocrine",
    "endocrinologies": "endocrinology",
    "endocrinologists": "endocrinologist",
    "endocytoses": "endocytosis",
    "endoderms": "endoderm",
    "endometria": "endometrium",
    "endometrioses": "endometriosis",
    "endometriums": "endometrium",
    "endonucleases": "endonuclease",
    "endorphins": "endorphin",
    "endorsed": "endorse",
    "endorsements": "endorsement",
    "endorsers": "endorser",
    "endorses": "endorse",
    "endorsing": "endorse",
    "endoscopes": "endoscope",
    "endoscopies": "endoscopy",
    "endoscopists": "endoscopist",
    "endosperms": "endosperm",
    "endosulfans": "endosulfan",
    "endothelia": "endothelium",
    "endotheliums": "endothelium",
    "endotoxins": "endotoxin",
    "endowed": "endow",
    "endowing": "endow",
    "endowments": "endowment",
    "endows": "endow",
    "endpoints": "endpoint",
    "ends": "end",
    "endued": "endue",
    "endues": "endue",
    "enduing": "endue",
    "endured": "endure",
    "endures": "endure",
    "enduring": "endure",
    "enemas": "enema",
    "enemata": "enema",
    "enemies": "enemy",
    "energies": "energy",
    "energised": "energise",
    "energisers": "energiser",
    "energises": "energise",
    "energising": "energise",
    "energized": "energize",
    "energizes": "energize",
    "energizing": "energize",
    "enervated": "enervate",
    "enervates": "enervate",
    "enervating": "enervate",
    "enfeebled": "enfeeble",
    "enfeebles": "enfeeble",
    "enfeebling": "enfeeble",
    "enfolded": "enfold",
    "enfolding": "enfold",
    "enfolds": "enfold",
    "enforced": "enforce",
    "enforcers": "enforcer",
    "enforces": "enforce",
    "enforcing": "enforce",
    "enfranchised": "enfranchise",
    "enfranchises": "enfranchise",
    "enfranchising": "enfranchise",
    "engaged": "engage",
    "engagements": "engagement",
    "engages": "engage",
    "engaging": "engage",
    "engendered": "engender",
    "engendering": "engender",
    "engenders": "engender",
    "engineered": "engineer",
    "engineering": "engineer",
    "engineers": "engineer",
    "enginemen": "engineman",
    "engines": "engine",
    "engorged": "engorge",
    "engorges": "engorge",
    "engorging": "engorge",
    "engrafted": "engraft",
    "engrafting": "engraft",
    "engraftments": "engraftment",
    "engrafts": "engraft",
    "engrailed": "engrail",
    "engrailing": "engrail",
    "engrails": "engrail",
    "engrained": "engrain",
    "engraining": "engrain",
    "engrains": "engrain",
    "engrams": "engram",
    "engraved": "engrave",
    "engravers": "engraver",
    "engraves": "engrave",
    "engraving": "engrave",
    "engravings": "engraving",
    "engrossed": "engross",
    "engrosses": "engross",
    "engrossing": "engross",
    "engulfed": "engulf",
    "engulfing": "engulf",
    "engulfs": "engulf",
    "enhanced": "enhance",
    "enhancements": "enhancement",
    "enhancers": "enhancer",
    "enhances": "enhance",
    "enhancing": "enhance",
    "enigmas": "enigma",
    "enigmata": "enigma",
    "enjoined": "enjoin",
    "enjoining": "enjoin",
    "enjoins": "enjoin",
    "enjoyed": "enjoy",
    "enjoying": "enjoy",
    "enjoyments": "enjoyment",
    "enjoys": "enjoy",
    "enlarged": "enlarge",
    "enlargements": "enlargement",
    "enlargers": "enlarger",
    "enlarges": "enlarge",
    "enlarging": "enlarge",
    "enlightened": "enlighten",
    "enlightening": "enlighten",
    "enlightens": "enlighten",
    "enlisted": "enlist",
    "enlisting": "enlist",
    "enlistments": "enlistment",
    "enlists": "enlist",
    "enlivened": "enliven",
    "enlivening": "enliven",
    "enlivens": "enliven",
    "enmeshed": "enmesh",
    "enmeshes": "enmesh",
    "enmeshing": "enmesh",
    "enmities": "enmity",
    "ennobled": "ennoble",
    "ennobles": "ennoble",
    "ennobling": "ennoble",
    "enols": "enol",
    "enormities": "enormity",
    "enoughs": "enough",
    "enquired": "enquire",
    "enquirers": "enquirer",
    "enquires": "enquire",
    "enquiries": "enquiry",
    "enquiring": "enquire",
    "enraged": "enrage",
    "enrages": "enrage",
    "enraging": "enrage",
    "enraptured": "enrapture",
    "enraptures": "enrapture",
    "enrapturing": "enrapture",
    "enriched": "enrich",
    "enriches": "enrich",
    "enriching": "enrich",
    "enrichments": "enrichment",
    "enroled": "enrol",
    "enroling": "enrol",
    "enrolled": "enroll",
    "enrollees": "enrollee",
    "enrolling": "enroll",
    "enrollments": "enrollment",
    "enrolls": "enroll",
    "enrolments": "enrolment",
    "enrols": "enrol",
    "ensconced": "ensconce",
    "ensconces": "ensconce",
    "ensconcing": "ensconce",
    "ensembles": "ensemble",
    "enshrined": "enshrine",
    "enshrines": "enshrine",
    "enshrining": "enshrine",
    "enshrouded": "enshroud",
    "enshrouding": "enshroud",
    "enshrouds": "enshroud",
    "ensigns": "ensign",
    "enslaved": "enslave",
    "enslavements": "enslavement",
    "enslaves": "enslave",
    "enslaving": "enslave",
    "ensnared": "ensnare",
    "ensnares": "ensnare",
    "ensnaring": "ensnare",
    "ensued": "ensue",
    "ensues": "ensue",
    "ensuing": "ensue",
    "ensured": "ensure",
    "ensures": "ensure",
    "ensuring": "ensure",
    "entailed": "entail",
    "entailing": "entail",
    "entails": "entail",
    "entangled": "entangle",
    "entanglements": "entanglement",
    "entangles": "entangle",
    "entangling": "entangle",
    "ententes": "entente",
    "entered": "enter",
    "entering": "enter",
    "enteritides": "enteritis",
    "enterococci": "enterococcus",
    "enteropathies": "enteropathy",
    "enteroviruses": "enterovirus",
    "enterprises": "enterprise",
    "enters": "enter",
    "entertained": "entertain",
    "entertainers": "entertainer",
    "entertaining": "entertain",
    "entertainments": "entertainment",
    "entertains": "entertain",
    "enthalpies": "enthalpy",
    "enthraled": "enthral",
    "enthraling": "enthral",
    "enthralled": "enthrall",
    "enthralling": "enthrall",
    "enthralls": "enthrall",
    "enthrals": "enthral",
    "enthroned": "enthrone",
    "enthronements": "enthronement",
    "enthrones": "enthrone",
    "enthroning": "enthrone",
    "enthused": "enthuse",
    "enthuses": "enthuse",
    "enthusiasms": "enthusiasm",
    "enthusiasts": "enthusiast",
    "enthusing": "enthuse",
    "enticed": "entice",
    "enticements": "enticement",
    "entices": "entice",
    "enticing": "entice",
    "entireties": "entirety",
    "entities": "entity",
    "entitled": "entitle",
    "entitlements": "entitlement",
    "entitles": "entitle",
    "entitling": "entitle",
    "entombed": "entomb",
    "entombing": "entomb",
    "entombs": "entomb",
    "entomologies": "entomology",
    "entomologists": "entomologist",
    "entourages": "entourage",
    "entrained": "entrain",
    "entraining": "entrain",
    "entrainments": "entrainment",
    "entrains": "entrain",
    "entranced": "entrance",
    "entrances": "entrance",
    "entranceways": "entranceway",
    "entrancing": "entrance",
    "entrants": "entrant",
    "entrapments": "entrapment",
    "entrapped": "entrap",
    "entrapping": "entrap",
    "entraps": "entrap",
    "entreated": "entreat",
    "entreaties": "entreaty",
    "entreating": "entreat",
    "entreats": "entreat",
    "entrees": "entree",
    "entrenched": "entrench",
    "entrenches": "entrench",
    "entrenching": "entrench",
    "entrenchments": "entrenchment",
    "entrepreneurs": "entrepreneur",
    "entries": "entry",
    "entropies": "entropy",
    "entrusted": "entrust",
    "entrusting": "entrust",
    "entrusts": "entrust",
    "entrées": "entrée",
    "entwined": "entwine",
    "entwines": "entwine",
    "entwining": "entwine",
    "enucleations": "enucleation",
    "enumerated": "enumerate",
    "enumerates": "enumerate",
    "enumerating": "enumerate",
    "enumerations": "enumeration",
    "enumerators": "enumerator",
    "enunciated": "enunciate",
    "enunciates": "enunciate",
    "enunciating": "enunciate",
    "enunciations": "enunciation",
    "enureses": "enuresis",
    "enveloped": "envelop",
    "envelopes": "envelope",
    "enveloping": "envelop",
    "envelopped": "envelop",
    "envelopping": "envelop",
    "envelops": "envelop",
    "envied": "envy",
    "envies": "envy",
    "environed": "environ",
    "environing": "environ",
    "environmentalists": "environmentalist",
    "environments": "environment",
    "environs": "environ",
    "envisaged": "envisage",
    "envisages": "envisage",
    "envisaging": "envisage",
    "envisioned": "envision",
    "envisioning": "envision",
    "envisions": "envision",
    "envoys": "envoy",
    "envs": "env",
    "envying": "envy",
    "enzymes": "enzyme",
    "enzymologies": "enzymology",
    "eons": "eon",
    "eosinophilias": "eosinophilia",
    "eosinophils": "eosinophil",
    "eosins": "eosin",
    "epaulets": "epaulet",
    "epaulettes": "epaulette",
    "ephedras": "ephedra",
    "ephedrines": "ephedrine",
    "ephemerae": "ephemera",
    "ephemerals": "ephemeral",
    "ephemeras": "ephemera",
    "ephemerides": "ephemeris",
    "epicenters": "epicenter",
    "epicentres": "epicentre",
    "epics": "epic",
    "epicures": "epicure",
    "epidemics": "epidemic",
    "epidemiologies": "epidemiology",
    "epidemiologists": "epidemiologist",
    "epidermides": "epidermis",
    "epidermises": "epidermis",
    "epidurals": "epidural",
    "epiglottides": "epiglottis",
    "epiglottises": "epiglottis",
    "epigrams": "epigram",
    "epigraphs": "epigraph",
    "epilepsies": "epilepsy",
    "epileptics": "epileptic",
    "epilogues": "epilogue",
    "epiphanies": "epiphany",
    "epiphyses": "epiphysis",
    "epiphytes": "epiphyte",
    "episcopalians": "episcopalian",
    "episiotomies": "episiotomy",
    "episodes": "episode",
    "epistemologies": "epistemology",
    "epistles": "epistle",
    "epitaphs": "epitaph",
    "epithelia": "epithelium",
    "epitheliums": "epithelium",
    "epithets": "epithet",
    "epitomes": "epitome",
    "epitomised": "epitomise",
    "epitomises": "epitomise",
    "epitomising": "epitomise",
    "epitomized": "epitomize",
    "epitomizes": "epitomize",
    "epitomizing": "epitomize",
    "epitopes": "epitope",
    "epochs": "epoch",
    "eponyms": "eponym",
    "epoxides": "epoxide",
    "epoxies": "epoxy",
    "eqs": "eq",
    "equaled": "equal",
    "equaling": "equal",
    "equalisations": "equalisation",
    "equalised": "equalise",
    "equalisers": "equaliser",
    "equalises": "equalise",
    "equalising": "equalise",
    "equalities": "equality",
    "equalizations": "equalization",
    "equalized": "equalize",
    "equalizers": "equalizer",
    "equalizes": "equalize",
    "equalizing": "equalize",
    "equalled": "equal",
    "equalling": "equal",
    "equals": "equal",
    "equated": "equate",
    "equates": "equate",
    "equating": "equate",
    "equations": "equation",
    "equators": "equator",
    "equerries": "equerry",
    "equestrians": "equestrian",
    "equilibrated": "equilibrate",
    "equilibrates": "equilibrate",
    "equilibrating": "equilibrate",
    "equilibrations": "equilibration",
    "equilibria": "equilibrium",
    "equilibriums": "equilibrium",
    "equines": "equine",
    "equinoxes": "equinox",
    "equipments": "equipment",
    "equipped": "equip",
    "equipping": "equip",
    "equips": "equip",
    "equities": "equity",
    "equivalences": "equivalence",
    "equivalencies": "equivalency",
    "equivalents": "equivalent",
    "equivocated": "equivocate",
    "equivocates": "equivocate",
    "equivocating": "equivocate",
    "equivocations": "equivocation",
    "equivs": "equiv",
    "eradicated": "eradicate",
    "eradicates": "eradicate",
    "eradicating": "eradicate",
    "eradications": "eradication",
    "eras": "era",
    "erased": "erase",
    "erasers": "eraser",
    "erases": "erase",
    "erasing": "erase",
    "erasure": "erasures",
    "erasures": "erasure",
    "erected": "erect",
    "erecting": "erect",
    "erections": "erection",
    "erectors": "erector",
    "erects": "erect",
    "ergometers": "ergometer",
    "ergonomists": "ergonomist",
    "ergotamines": "ergotamine",
    "ergots": "ergot",
    "ergs": "erg",
    "ermines": "ermine",
    "eroded": "erode",
    "erodes": "erode",
    "eroding": "erode",
    "erosions": "erosion",
    "errands": "errand",
    "errata": "erratum",
    "erred": "err",
    "erring": "err",
    "errors": "error",
    "errs": "err",
    "ers": "er",
    "erupted": "erupt",
    "erupting": "erupt",
    "eruptions": "eruption",
    "erupts": "erupt",
    "erythemas": "erythema",
    "erythemata": "erythema",
    "erythrocytes": "erythrocyte",
    "erythromycins": "erythromycin",
    "erythropoietins": "erythropoietin",
    "escalated": "escalate",
    "escalates": "escalate",
    "escalating": "escalate",
    "escalations": "escalation",
    "escalators": "escalator",
    "escalopes": "escalope",
    "escapades": "escapade",
    "escaped": "escape",
    "escapees": "escapee",
    "escapements": "escapement",
    "escapers": "escaper",
    "escapes": "escape",
    "escaping": "escape",
    "escapists": "escapist",
    "escapologists": "escapologist",
    "escarpments": "escarpment",
    "eschewed": "eschew",
    "eschewing": "eschew",
    "eschews": "eschew",
    "escorted": "escort",
    "escorting": "escort",
    "escorts": "escort",
    "escutcheons": "escutcheon",
    "eskers": "esker",
    "esophagi": "esophagus",
    "esophaguses": "esophagus",
    "espied": "espy",
    "espies": "espy",
    "esplanades": "esplanade",
    "espousals": "espousal",
    "espoused": "espouse",
    "espouses": "espouse",
    "espousing": "espouse",
    "espressos": "espresso",
    "esprits": "esprit",
    "espying": "espy",
    "esquires": "esquire",
    "essayed": "essay",
    "essaying": "essay",
    "essayists": "essayist",
    "essays": "essay",
    "essences": "essence",
    "essentialists": "essentialist",
    "essentials": "essential",
    "established": "establish",
    "establishes": "establish",
    "establishing": "establish",
    "establishments": "establishment",
    "estates": "estate",
    "esteemed": "esteem",
    "esteeming": "esteem",
    "esteems": "esteem",
    "esterases": "esterase",
    "esters": "ester",
    "estimated": "estimate",
    "estimates": "estimate",
    "estimating": "estimate",
    "estimations": "estimation",
    "estimators": "estimator",
    "estradiols": "estradiol",
    "estranged": "estrange",
    "estrangements": "estrangement",
    "estranges": "estrange",
    "estranging": "estrange",
    "estrogens": "estrogen",
    "estuaries": "estuary",
    "etched": "etch",
    "etchers": "etcher",
    "etches": "etch",
    "etching": "etch",
    "etchings": "etching",
    "etchs": "etch",
    "etcs": "etc",
    "eternities": "eternity",
    "ethanes": "ethane",
    "ethanols": "ethanol",
    "ethenes": "ethene",
    "ethers": "ether",
    "ethicists": "ethicist",
    "ethics": "ethic",
    "ethnicities": "ethnicity",
    "ethnics": "ethnic",
    "ethnographers": "ethnographer",
    "ethnographies": "ethnography",
    "ethnologies": "ethnology",
    "ethnologists": "ethnologist",
    "ethnomethodologies": "ethnomethodology",
    "ethologies": "ethology",
    "ethologists": "ethologist",
    "ethoxylates": "ethoxylate",
    "ethylenes": "ethylene",
    "etiolated": "etiolate",
    "etiolating": "etiolate",
    "etiologies": "etiology",
    "etudes": "etude",
    "etymologies": "etymology",
    "eucalypti": "eucalyptus",
    "eucalypts": "eucalypt",
    "eucalyptuses": "eucalyptus",
    "eukaryotes": "eukaryote",
    "eulogies": "eulogy",
    "eulogized": "eulogize",
    "eulogizes": "eulogize",
    "eulogizing": "eulogize",
    "eunuchs": "eunuch",
    "euphemisms": "euphemism",
    "euphonies": "euphony",
    "euphorias": "euphoria",
    "euros": "euro",
    "eutectics": "eutectic",
    "euthanasias": "euthanasia",
    "euthanised": "euthanise",
    "euthanises": "euthanise",
    "euthanising": "euthanise",
    "euthanized": "euthanize",
    "euthanizes": "euthanize",
    "euthanizing": "euthanize",
    "evacuated": "evacuate",
    "evacuates": "evacuate",
    "evacuating": "evacuate",
    "evacuations": "evacuation",
    "evacuees": "evacuee",
    "evaded": "evade",
    "evades": "evade",
    "evading": "evade",
    "evals": "eval",
    "evaluated": "evaluate",
    "evaluates": "evaluate",
    "evaluating": "evaluate",
    "evaluations": "evaluation",
    "evaluators": "evaluator",
    "evanesced": "evanesce",
    "evanesces": "evanesce",
    "evanescing": "evanesce",
    "evangelists": "evangelist",
    "evangelized": "evangelize",
    "evangelizes": "evangelize",
    "evangelizing": "evangelize",
    "evaporated": "evaporate",
    "evaporates": "evaporate",
    "evaporating": "evaporate",
    "evaporations": "evaporation",
    "evaporators": "evaporator",
    "evasions": "evasion",
    "evened": "even",
    "evening": "even",
    "evenings": "evening",
    "evenner": "even",
    "evennesses": "evenness",
    "evennest": "even",
    "evens": "even",
    "evensongs": "evensong",
    "events": "event",
    "eventualities": "eventuality",
    "evergreens": "evergreen",
    "everted": "evert",
    "everting": "evert",
    "everts": "evert",
    "eves": "eve",
    "evicted": "evict",
    "evicting": "evict",
    "evictions": "eviction",
    "evicts": "evict",
    "evidenced": "evidence",
    "evidences": "evidence",
    "evidencing": "evidence",
    "evil-doers": "evil-doer",
    "evildoers": "evildoer",
    "eviller": "evil",
    "evillest": "evil",
    "evils": "evil",
    "evinced": "evince",
    "evinces": "evince",
    "evincing": "evince",
    "eviscerated": "eviscerate",
    "eviscerates": "eviscerate",
    "eviscerating": "eviscerate",
    "eviscerations": "evisceration",
    "evocations": "evocation",
    "evoked": "evoke",
    "evokes": "evoke",
    "evoking": "evoke",
    "evolutionarier": "evolutionary",
    "evolutionariest": "evolutionary",
    "evolutionists": "evolutionist",
    "evolutions": "evolution",
    "evolvabilities": "evolvability",
    "evolved": "evolve",
    "evolves": "evolve",
    "evolving": "evolve",
    "evs": "ev",
    "ewers": "ewer",
    "ewes": "ewe",
    "ex-servicemen": "ex-serviceman",
    "ex-smokers": "ex-smoker",
    "ex-wives": "ex-wife",
    "exacerbated": "exacerbate",
    "exacerbates": "exacerbate",
    "exacerbating": "exacerbate",
    "exacerbations": "exacerbation",
    "exacted": "exact",
    "exacter": "exact",
    "exactest": "exact",
    "exacting": "exact",
    "exactions": "exaction",
    "exactitudes": "exactitude",
    "exacts": "exact",
    "exaggerated": "exaggerate",
    "exaggerates": "exaggerate",
    "exaggerating": "exaggerate",
    "exaggerations": "exaggeration",
    "exaltations": "exaltation",
    "exalted": "exalt",
    "exalting": "exalt",
    "exalts": "exalt",
    "examinations": "examination",
    "examined": "examine",
    "examinees": "examinee",
    "examiners": "examiner",
    "examines": "examine",
    "examining": "examine",
    "examples": "example",
    "exams": "exam",
    "exasperated": "exasperate",
    "exasperates": "exasperate",
    "exasperating": "exasperate",
    "excavated": "excavate",
    "excavates": "excavate",
    "excavating": "excavate",
    "excavations": "excavation",
    "excavators": "excavator",
    "exceedances": "exceedance",
    "exceeded": "exceed",
    "exceedences": "exceedence",
    "exceeding": "exceed",
    "exceeds": "exceed",
    "excelled": "excel",
    "excellences": "excellence",
    "excelling": "excel",
    "excels": "excel",
    "excepted": "except",
    "excepting": "except",
    "exceptions": "exception",
    "excepts": "except",
    "excerpted": "excerpt",
    "excerpting": "excerpt",
    "excerpts": "excerpt",
    "excesses": "excess",
    "exchange-rates": "exchange-rate",
    "exchanged": "exchange",
    "exchangers": "exchanger",
    "exchanges": "exchange",
    "exchanging": "exchange",
    "exchequers": "exchequer",
    "excipients": "excipient",
    "excised": "excise",
    "excises": "excise",
    "excising": "excise",
    "excisions": "excision",
    "excitabilities": "excitability",
    "excitations": "excitation",
    "excited": "excite",
    "excitedly": "excited",
    "excitements": "excitement",
    "exciters": "exciter",
    "excites": "excite",
    "exciting": "excite",
    "excitons": "exciton",
    "exclaimed": "exclaim",
    "exclaiming": "exclaim",
    "exclaims": "exclaim",
    "exclamations": "exclamation",
    "exclosures": "exclosure",
    "excluded": "exclude",
    "excluders": "excluder",
    "excludes": "exclude",
    "excluding": "exclude",
    "exclusions": "exclusion",
    "exclusives": "exclusive",
    "exclusivists": "exclusivist",
    "excommunicated": "excommunicate",
    "excommunicates": "excommunicate",
    "excommunicating": "excommunicate",
    "excommunications": "excommunication",
    "excoriated": "excoriate",
    "excoriates": "excoriate",
    "excoriating": "excoriate",
    "excrements": "excrement",
    "excrescences": "excrescence",
    "excreta": "excretion",
    "excreted": "excrete",
    "excretes": "excrete",
    "excreting": "excrete",
    "excretions": "excretion",
    "exculpated": "exculpate",
    "exculpates": "exculpate",
    "exculpating": "exculpate",
    "excursions": "excursion",
    "excused": "excuse",
    "excuses": "excuse",
    "excusing": "excuse",
    "execrated": "execrate",
    "execrates": "execrate",
    "execrating": "execrate",
    "executables": "executable",
    "executed": "execute",
    "executes": "execute",
    "executing": "execute",
    "executioners": "executioner",
    "executions": "execution",
    "executives": "executive",
    "executors": "executor",
    "executrices": "executrix",
    "exegeses": "exegesis",
    "exegetes": "exegete",
    "exempla": "exemplum",
    "exemplars": "exemplar",
    "exemplifications": "exemplification",
    "exemplified": "exemplify",
    "exemplifies": "exemplify",
    "exemplifying": "exemplify",
    "exempted": "exempt",
    "exempting": "exempt",
    "exemptions": "exemption",
    "exempts": "exempt",
    "exercise-books": "exercise-book",
    "exercised": "exercise",
    "exercisers": "exerciser",
    "exercises": "exercise",
    "exercising": "exercise",
    "exerted": "exert",
    "exerting": "exert",
    "exertions": "exertion",
    "exerts": "exert",
    "exes": "ex",
    "exfoliated": "exfoliate",
    "exfoliates": "exfoliate",
    "exfoliating": "exfoliate",
    "exfoliations": "exfoliation",
    "exhalations": "exhalation",
    "exhaled": "exhale",
    "exhales": "exhale",
    "exhaling": "exhale",
    "exhausted": "exhaust",
    "exhausting": "exhaust",
    "exhausts": "exhaust",
    "exhibited": "exhibit",
    "exhibiting": "exhibit",
    "exhibitionists": "exhibitionist",
    "exhibitions": "exhibition",
    "exhibitors": "exhibitor",
    "exhibits": "exhibit",
    "exhilarated": "exhilarate",
    "exhilarates": "exhilarate",
    "exhilarating": "exhilarate",
    "exhortations": "exhortation",
    "exhorted": "exhort",
    "exhorting": "exhort",
    "exhorts": "exhort",
    "exhumations": "exhumation",
    "exhumed": "exhume",
    "exhumes": "exhume",
    "exhuming": "exhume",
    "exigencies": "exigency",
    "exiled": "exile",
    "exiles": "exile",
    "exiling": "exile",
    "existed": "exist",
    "existences": "existence",
    "existentialists": "existentialist",
    "existing": "exist",
    "exists": "exist",
    "exited": "exit",
    "exiting": "exit",
    "exits": "exit",
    "exocytoses": "exocytosis",
    "exoduses": "exodus",
    "exonerated": "exonerate",
    "exonerates": "exonerate",
    "exonerating": "exonerate",
    "exonerations": "exoneration",
    "exons": "exon",
    "exorcised": "exorcise",
    "exorcises": "exorcise",
    "exorcising": "exorcise",
    "exorcisms": "exorcism",
    "exorcists": "exorcist",
    "exorcized": "exorcize",
    "exorcizes": "exorcize",
    "exorcizing": "exorcize",
    "exoskeletons": "exoskeleton",
    "exostoses": "exostosis",
    "exotics": "exotic",
    "expanded": "expand",
    "expanders": "expander",
    "expanding": "expand",
    "expands": "expand",
    "expanses": "expanse",
    "expansions": "expansion",
    "expatiated": "expatiate",
    "expatiates": "expatiate",
    "expatiating": "expatiate",
    "expatriated": "expatriate",
    "expatriates": "expatriate",
    "expatriating": "expatriate",
    "expectancies": "expectancy",
    "expectations": "expectation",
    "expected": "expect",
    "expecting": "expect",
    "expectorants": "expectorant",
    "expects": "expect",
    "expediencies": "expediency",
    "expedients": "expedient",
    "expedited": "expedite",
    "expedites": "expedite",
    "expediting": "expedite",
    "expeditioners": "expeditioner",
    "expeditions": "expedition",
    "expelled": "expel",
    "expelling": "expel",
    "expels": "expel",
    "expendables": "expendable",
    "expended": "expend",
    "expending": "expend",
    "expenditures": "expenditure",
    "expends": "expend",
    "expenses": "expense",
    "experienced": "experience",
    "experiencers": "experiencer",
    "experiences": "experience",
    "experiencing": "experience",
    "experimentalists": "experimentalist",
    "experimentations": "experimentation",
    "experimented": "experiment",
    "experimenters": "experimenter",
    "experimenting": "experiment",
    "experiments": "experiment",
    "expertises": "expertise",
    "experts": "expert",
    "expiated": "expiate",
    "expiates": "expiate",
    "expiating": "expiate",
    "expirations": "expiration",
    "expired": "expire",
    "expires": "expire",
    "expiries": "expiry",
    "expiring": "expire",
    "explained": "explain",
    "explaining": "explain",
    "explains": "explain",
    "explanations": "explanation",
    "explants": "explant",
    "expletives": "expletive",
    "explicated": "explicate",
    "explicates": "explicate",
    "explicating": "explicate",
    "explications": "explication",
    "exploded": "explode",
    "explodes": "explode",
    "exploding": "explode",
    "exploitations": "exploitation",
    "exploited": "exploit",
    "exploiters": "exploiter",
    "exploiting": "exploit",
    "exploits": "exploit",
    "explorations": "exploration",
    "explored": "explore",
    "explorers": "explorer",
    "explores": "explore",
    "exploring": "explore",
    "explosions": "explosion",
    "explosives": "explosive",
    "exponentials": "exponential",
    "exponents": "exponent",
    "exported": "export",
    "exporters": "exporter",
    "exporting": "export",
    "exports": "export",
    "exposed": "expose",
    "exposes": "expose",
    "exposing": "expose",
    "expositions": "exposition",
    "expositors": "expositor",
    "expostulated": "expostulate",
    "expostulates": "expostulate",
    "expostulating": "expostulate",
    "exposures": "exposure",
    "expounded": "expound",
    "expounding": "expound",
    "expounds": "expound",
    "expressed": "express",
    "expresses": "express",
    "expressing": "express",
    "expressionists": "expressionist",
    "expressions": "expression",
    "expressivities": "expressivity",
    "expressos": "expresso",
    "expressways": "expressway",
    "expropriated": "expropriate",
    "expropriates": "expropriate",
    "expropriating": "expropriate",
    "expropriations": "expropriation",
    "exps": "exp",
    "expulsions": "expulsion",
    "expunged": "expunge",
    "expunges": "expunge",
    "expunging": "expunge",
    "expurgated": "expurgate",
    "expurgates": "expurgate",
    "expurgating": "expurgate",
    "extemporized": "extemporize",
    "extemporizes": "extemporize",
    "extemporizing": "extemporize",
    "extended": "extend",
    "extenders": "extender",
    "extending": "extend",
    "extends": "extend",
    "extensibilities": "extensibility",
    "extensions": "extension",
    "extensors": "extensor",
    "extents": "extent",
    "extenuated": "extenuate",
    "extenuates": "extenuate",
    "extenuating": "extenuate",
    "exteriors": "exterior",
    "exterminated": "exterminate",
    "exterminates": "exterminate",
    "exterminating": "exterminate",
    "exterminations": "extermination",
    "exterminators": "exterminator",
    "externalisations": "externalisation",
    "externalised": "externalise",
    "externalises": "externalise",
    "externalising": "externalise",
    "externalists": "externalist",
    "externalities": "externality",
    "externalized": "externalize",
    "externalizes": "externalize",
    "externalizing": "externalize",
    "externals": "external",
    "externs": "extern",
    "extinctions": "extinction",
    "extinguished": "extinguish",
    "extinguishers": "extinguisher",
    "extinguishes": "extinguish",
    "extinguishing": "extinguish",
    "extirpated": "extirpate",
    "extirpates": "extirpate",
    "extirpating": "extirpate",
    "extirpations": "extirpation",
    "extolled": "extol",
    "extolling": "extol",
    "extols": "extol",
    "extorted": "extort",
    "extorting": "extort",
    "extortions": "extortion",
    "extorts": "extort",
    "extracted": "extract",
    "extracting": "extract",
    "extractions": "extraction",
    "extractives": "extractive",
    "extractors": "extractor",
    "extracts": "extract",
    "extradited": "extradite",
    "extradites": "extradite",
    "extraditing": "extradite",
    "extraditions": "extradition",
    "extrapolated": "extrapolate",
    "extrapolates": "extrapolate",
    "extrapolating": "extrapolate",
    "extrapolations": "extrapolation",
    "extras": "extra",
    "extraterrestrials": "extraterrestrial",
    "extravagances": "extravagance",
    "extravaganzas": "extravaganza",
    "extrema": "extremum",
    "extremer": "extreme",
    "extremes": "extreme",
    "extremest": "extreme",
    "extremists": "extremist",
    "extremities": "extremity",
    "extricated": "extricate",
    "extricates": "extricate",
    "extricating": "extricate",
    "extroverts": "extrovert",
    "extruded": "extrude",
    "extruders": "extruder",
    "extrudes": "extrude",
    "extruding": "extrude",
    "extrusions": "extrusion",
    "exudates": "exudate",
    "exuded": "exude",
    "exudes": "exude",
    "exuding": "exude",
    "exultations": "exultation",
    "exulted": "exult",
    "exulting": "exult",
    "exults": "exult",
    "eye-openers": "eye-opener",
    "eye-witnesses": "eye-witness",
    "eyeballs": "eyeball",
    "eyebaths": "eyebath",
    "eyebrows": "eyebrow",
    "eyecups": "eyecup",
    "eyed": "eye",
    "eyedrops": "eyedrop",
    "eyefuls": "eyeful",
    "eyeglasses": "eyeglass",
    "eyeing": "eye",
    "eyelashes": "eyelash",
    "eyelets": "eyelet",
    "eyelids": "eyelid",
    "eyeliners": "eyeliner",
    "eyepieces": "eyepiece",
    "eyes": "eye",
    "eyesful": "eyeful",
    "eyeshadows": "eyeshadow",
    "eyesores": "eyesore",
    "eyespots": "eyespot",
    "eyewashes": "eyewash",
    "eyewitnesses": "eyewitness",
    "eying": "eye",
    "eyries": "eyrie",
    "fables": "fable",
    "fabricated": "fabricate",
    "fabricates": "fabricate",
    "fabricating": "fabricate",
    "fabrications": "fabrication",
    "fabricators": "fabricator",
    "fabrics": "fabric",
    "facades": "facade",
    "facdes": "facde",
    "face-lifts": "face-lift",
    "face-offs": "face-off",
    "face-savers": "face-saver",
    "faced": "face",
    "facelifts": "facelift",
    "facemasks": "facemask",
    "facepieces": "facepiece",
    "faceplates": "faceplate",
    "faces": "face",
    "faceted": "facet",
    "faceting": "facet",
    "facets": "facet",
    "facetted": "facet",
    "facetting": "facet",
    "faciae": "facia",
    "facials": "facial",
    "facias": "facia",
    "facilitated": "facilitate",
    "facilitates": "facilitate",
    "facilitating": "facilitate",
    "facilitations": "facilitation",
    "facilitators": "facilitator",
    "facilities": "facility",
    "facing": "face",
    "facings": "facing",
    "facsimiles": "facsimile",
    "factions": "faction",
    "factoids": "factoid",
    "factored": "factor",
    "factorials": "factorial",
    "factories": "factory",
    "factoring": "factor",
    "factorisations": "factorisation",
    "factorised": "factorise",
    "factorises": "factorise",
    "factorising": "factorise",
    "factorizations": "factorization",
    "factors": "factor",
    "factotums": "factotum",
    "facts": "fact",
    "faculties": "faculty",
    "faddier": "faddy",
    "faddiest": "faddy",
    "fade-outs": "fade-out",
    "faded": "fade",
    "faders": "fader",
    "fades": "fade",
    "fading": "fade",
    "fads": "fad",
    "fagged": "fag",
    "fagging": "fag",
    "faggots": "faggot",
    "fags": "fag",
    "failed": "fail",
    "failing": "fail",
    "failings": "failing",
    "fails": "fail",
    "failures": "failure",
    "fainted": "faint",
    "fainter": "faint",
    "faintest": "faint",
    "fainting": "faint",
    "faints": "faint",
    "faired": "fair",
    "fairer": "fair",
    "fairest": "fair",
    "fairgrounds": "fairground",
    "fairies": "fairy",
    "fairing": "fair",
    "fairings": "fairing",
    "fairs": "fair",
    "fairways": "fairway",
    "fairylands": "fairyland",
    "faithfuls": "faithful",
    "faiths": "faith",
    "faked": "fake",
    "faker": "fake",
    "fakers": "faker",
    "fakes": "fake",
    "fakest": "fake",
    "faking": "fake",
    "falconers": "falconer",
    "falcons": "falcon",
    "fall-offs": "fall-off",
    "fall-outs": "fall-out",
    "fallacies": "fallacy",
    "fallbacks": "fallback",
    "fallen": "fall",
    "fallers": "faller",
    "fallibilities": "fallibility",
    "falling": "fall",
    "falloffs": "falloff",
    "fallouts": "fallout",
    "fallowed": "fallow",
    "fallowing": "fallow",
    "fallows": "fallow",
    "falls": "fall",
    "falsehoods": "falsehood",
    "falser": "FALSE",
    "falses": "false",
    "falsest": "FALSE",
    "falsettos": "falsetto",
    "falsifications": "falsification",
    "falsified": "falsify",
    "falsifies": "falsify",
    "falsifying": "falsify",
    "falsities": "falsity",
    "faltered": "falter",
    "faltering": "falter",
    "falters": "falter",
    "famed": "fame",
    "fames": "fame",
    "familiarised": "familiarise",
    "familiarises": "familiarise",
    "familiarising": "familiarise",
    "familiarities": "familiarity",
    "familiarized": "familiarize",
    "familiarizes": "familiarize",
    "familiarizing": "familiarize",
    "familiars": "familiar",
    "families": "family",
    "famines": "famine",
    "faming": "fame",
    "famished": "famish",
    "famishes": "famish",
    "famishing": "famish",
    "fanaticisms": "fanaticism",
    "fanatics": "fanatic",
    "fancied": "fancy",
    "fancier": "fancy",
    "fanciers": "fancier",
    "fancies": "fancy",
    "fanciest": "fancy",
    "fancying": "fancy",
    "fandangos": "fandango",
    "fanfares": "fanfare",
    "fangs": "fang",
    "fanlights": "fanlight",
    "fanned": "fan",
    "fannies": "fanny",
    "fanning": "fan",
    "fans": "fan",
    "fantails": "fantail",
    "fantasias": "fantasia",
    "fantasies": "fantasy",
    "fantasised": "fantasise",
    "fantasises": "fantasise",
    "fantasising": "fantasise",
    "fantasized": "fantasize",
    "fantasizes": "fantasize",
    "fantasizing": "fantasize",
    "farces": "farce",
    "fared": "fare",
    "fares": "fare",
    "farewells": "farewell",
    "faring": "fare",
    "farm-hands": "farm-hand",
    "farmed": "farm",
    "farmers": "farmer",
    "farmhands": "farmhand",
    "farmhouses": "farmhouse",
    "farming": "farm",
    "farmlands": "farmland",
    "farms": "farm",
    "farmsteads": "farmstead",
    "farmworkers": "farmworker",
    "farmyards": "farmyard",
    "farrieries": "farriery",
    "farriers": "farrier",
    "farrowed": "farrow",
    "farrowing": "farrow",
    "farrows": "farrow",
    "fars": "far",
    "farted": "fart",
    "farther": "far",
    "farthest": "far",
    "farthings": "farthing",
    "farting": "fart",
    "farts": "fart",
    "fasciae": "fascia",
    "fascias": "fascia",
    "fascicles": "fascicle",
    "fasciitides": "fasciitis",
    "fascinated": "fascinate",
    "fascinates": "fascinate",
    "fascinating": "fascinate",
    "fascinations": "fascination",
    "fascists": "fascist",
    "fashioned": "fashion",
    "fashioning": "fashion",
    "fashions": "fashion",
    "fasted": "fast",
    "fastened": "fasten",
    "fasteners": "fastener",
    "fastening": "fasten",
    "fastenings": "fastening",
    "fastens": "fasten",
    "faster": "fast",
    "faster-growing": "fast-growing",
    "faster-moving": "fast-moving",
    "fastest": "fast",
    "fastest-growing": "fast-growing",
    "fastest-moving": "fast-moving",
    "fasting": "fast",
    "fastnesses": "fastness",
    "fasts": "fast",
    "fatalisms": "fatalism",
    "fatalists": "fatalist",
    "fatalities": "fatality",
    "fated": "fate",
    "fates": "fate",
    "fatheads": "fathead",
    "father-in-laws": "father-in-law",
    "fathered": "father",
    "fathering": "father",
    "fatherlands": "fatherland",
    "fathers": "father",
    "fathers-in-law": "father-in-law",
    "fathomed": "fathom",
    "fathoming": "fathom",
    "fathoms": "fathom",
    "fatigued": "fatigue",
    "fatigues": "fatigue",
    "fatiguing": "fatigue",
    "fating": "fate",
    "fats": "fat",
    "fatted": "fat",
    "fattened": "fatten",
    "fattening": "fatten",
    "fattens": "fatten",
    "fatter": "fat",
    "fattest": "fat",
    "fattier": "fatty",
    "fatties": "fatty",
    "fattiest": "fatty",
    "fatting": "fat",
    "fatwas": "fatwa",
    "faucets": "faucet",
    "faulted": "fault",
    "faultier": "faulty",
    "faultiest": "faulty",
    "faulting": "fault",
    "faults": "fault",
    "faunae": "fauna",
    "faunas": "fauna",
    "fauns": "faun",
    "favored": "favor",
    "favoring": "favor",
    "favorites": "favorite",
    "favors": "favor",
    "favoured": "favour",
    "favouring": "favour",
    "favourites": "favourite",
    "favours": "favour",
    "fawned": "fawn",
    "fawner": "fawn",
    "fawnest": "fawn",
    "fawning": "fawn",
    "fawns": "fawn",
    "faxed": "fax",
    "faxes": "fax",
    "faxing": "fax",
    "fazed": "faze",
    "fazes": "faze",
    "fazing": "faze",
    "façades": "façade",
    "fealties": "fealty",
    "feared": "fear",
    "fearing": "fear",
    "fears": "fear",
    "feasibilities": "feasibility",
    "feasted": "feast",
    "feasting": "feast",
    "feasts": "feast",
    "feather-weights": "feather-weight",
    "feathered": "feather",
    "feathering": "feather",
    "feathers": "feather",
    "featherweights": "featherweight",
    "feats": "feat",
    "featured": "feature",
    "features": "feature",
    "featuring": "feature",
    "fecundities": "fecundity",
    "fed": "feed",
    "federalists": "federalist",
    "federated": "federate",
    "federates": "federate",
    "federating": "federate",
    "federations": "federation",
    "feebler": "feeble",
    "feeblest": "feeble",
    "feed": "fee",
    "feed-backs": "feed-back",
    "feedbacks": "feedback",
    "feeders": "feeder",
    "feedforwards": "feedforward",
    "feeding": "feed",
    "feeding-bottles": "feeding-bottle",
    "feedings": "feeding",
    "feeds": "feed",
    "feedstocks": "feedstock",
    "feedstuffs": "feedstuff",
    "feeing": "fee",
    "feelers": "feeler",
    "feeling": "feel",
    "feelings": "feeling",
    "feels": "feel",
    "fees": "fee",
    "feet": "foot",
    "feigned": "feign",
    "feigning": "feign",
    "feigns": "feign",
    "feinted": "feint",
    "feinting": "feint",
    "feints": "feint",
    "feistier": "feisty",
    "feldspars": "feldspar",
    "felicities": "felicity",
    "felines": "feline",
    "fell": "fall",
    "felled": "fell",
    "fellers": "feller",
    "fellies": "felly",
    "felling": "fell",
    "fellows": "fellow",
    "fellowships": "fellowship",
    "fells": "fell",
    "felonies": "felony",
    "felons": "felon",
    "felt": "feel",
    "felt-tips": "felt-tip",
    "felts": "felt",
    "females": "female",
    "femininities": "femininity",
    "feminised": "feminise",
    "feminises": "feminise",
    "feminising": "feminise",
    "feminisms": "feminism",
    "feminists": "feminist",
    "femmes": "femme",
    "femora": "femur",
    "femtoseconds": "femtosecond",
    "femurs": "femur",
    "fenced": "fence",
    "fencers": "fencer",
    "fences": "fence",
    "fencing": "fence",
    "fended": "fend",
    "fenders": "fender",
    "fending": "fend",
    "fends": "fend",
    "fenestrations": "fenestration",
    "fens": "fen",
    "fentanyls": "fentanyl",
    "feriae": "feria",
    "ferias": "feria",
    "fermentations": "fermentation",
    "fermented": "ferment",
    "fermenters": "fermenter",
    "fermenting": "ferment",
    "ferments": "ferment",
    "fermions": "fermion",
    "ferneries": "fernery",
    "fernier": "ferny",
    "ferns": "fern",
    "ferocities": "ferocity",
    "ferreted": "ferret",
    "ferreting": "ferret",
    "ferrets": "ferret",
    "ferried": "ferry",
    "ferries": "ferry",
    "ferrites": "ferrite",
    "ferritins": "ferritin",
    "ferrules": "ferrule",
    "ferryboats": "ferryboat",
    "ferrying": "ferry",
    "fertiles": "fertile",
    "fertilisations": "fertilisation",
    "fertilised": "fertilise",
    "fertilisers": "fertiliser",
    "fertilises": "fertilise",
    "fertilising": "fertilise",
    "fertilities": "fertility",
    "fertilizations": "fertilization",
    "fertilized": "fertilize",
    "fertilizers": "fertilizer",
    "fertilizes": "fertilize",
    "fertilizing": "fertilize",
    "fes": "fe",
    "fescues": "fescue",
    "festered": "fester",
    "festering": "fester",
    "festers": "fester",
    "festivals": "festival",
    "festivities": "festivity",
    "festooned": "festoon",
    "festooning": "festoon",
    "festoons": "festoon",
    "fests": "fest",
    "festschriften": "festschrift",
    "festschrifts": "festschrift",
    "fetched": "fetch",
    "fetches": "fetch",
    "fetching": "fetch",
    "feted": "fete",
    "fetes": "fete",
    "feti": "fetus",
    "feting": "fete",
    "fetishes": "fetish",
    "fetishised": "fetishise",
    "fetishises": "fetishise",
    "fetishising": "fetishise",
    "fetishisms": "fetishism",
    "fetishists": "fetishist",
    "fetlocks": "fetlock",
    "fettered": "fetter",
    "fettering": "fetter",
    "fetters": "fetter",
    "fettled": "fettle",
    "fettles": "fettle",
    "fettling": "fettle",
    "fettlings": "fettling",
    "fetuses": "fetus",
    "feudalisms": "feudalism",
    "feuded": "feud",
    "feuding": "feud",
    "feuds": "feud",
    "feued": "feu",
    "feuing": "feu",
    "feus": "feu",
    "fevers": "fever",
    "fewer": "few",
    "fewest": "few",
    "fezes": "fez",
    "fezzes": "fez",
    "fgs": "fg",
    "fiancees": "fiancee",
    "fiances": "fiance",
    "fiancées": "fiancée",
    "fiancés": "fiancé",
    "fiant": "fiat",
    "fiascoes": "fiasco",
    "fiascos": "fiasco",
    "fiats": "fiat",
    "fibbed": "fib",
    "fibbers": "fibber",
    "fibbing": "fib",
    "fiberglasses": "fiberglass",
    "fibers": "fiber",
    "fibrates": "fibrate",
    "fibreglasses": "fibreglass",
    "fibres": "fibre",
    "fibrillations": "fibrillation",
    "fibrils": "fibril",
    "fibrinogens": "fibrinogen",
    "fibrinolyses": "fibrinolysis",
    "fibrins": "fibrin",
    "fibroblasts": "fibroblast",
    "fibroids": "fibroid",
    "fibromyalgias": "fibromyalgia",
    "fibronectins": "fibronectin",
    "fibroses": "fibrosis",
    "fibs": "fib",
    "fibulae": "fibula",
    "fibulas": "fibula",
    "fickler": "fickle",
    "ficklest": "fickle",
    "fictionalised": "fictionalise",
    "fictionalises": "fictionalise",
    "fictionalising": "fictionalise",
    "fictionalizations": "fictionalization",
    "fictionalized": "fictionalize",
    "fictionalizes": "fictionalize",
    "fictionalizing": "fictionalize",
    "fictions": "fiction",
    "fiddled": "fiddle",
    "fiddlers": "fiddler",
    "fiddles": "fiddle",
    "fiddlier": "fiddly",
    "fiddliest": "fiddly",
    "fiddling": "fiddle",
    "fidelities": "fidelity",
    "fidgeted": "fidget",
    "fidgetier": "fidgety",
    "fidgetiest": "fidgety",
    "fidgeting": "fidget",
    "fidgets": "fidget",
    "fiefs": "fief",
    "field-tested": "field-test",
    "field-testing": "field-test",
    "field-tests": "field-test",
    "field-works": "field-work",
    "fielded": "field",
    "fielders": "fielder",
    "fieldfares": "fieldfare",
    "fielding": "field",
    "fieldmice": "fieldmouse",
    "fields": "field",
    "fieldworkers": "fieldworker",
    "fieldworks": "fieldwork",
    "fiends": "fiend",
    "fiercer": "fierce",
    "fiercest": "fierce",
    "fierier": "fiery",
    "fieriest": "fiery",
    "fiestas": "fiesta",
    "fifes": "fife",
    "fifteens": "fifteen",
    "fifteenth": "15",
    "fifteenths": "fifteenth",
    "fifth": "5",
    "fifths": "fifth",
    "fifties": "fifty",
    "fiftieth": "50",
    "fiftieths": "fiftieth",
    "fifty-eighth": "58",
    "fifty-eighths": "fifty-eighth",
    "fifty-fifth": "55",
    "fifty-fifths": "fifty-fifth",
    "fifty-first": "51",
    "fifty-firsts": "fifty-first",
    "fifty-fourth": "54",
    "fifty-fourths": "fifty-fourth",
    "fifty-ninth": "59",
    "fifty-ninths": "fifty-ninth",
    "fifty-second": "52",
    "fifty-seconds": "fifty-second",
    "fifty-seventh": "57",
    "fifty-sevenths": "fifty-seventh",
    "fifty-sixth": "56",
    "fifty-sixths": "fifty-sixth",
    "fifty-third": "53",
    "fifty-thirds": "fifty-third",
    "fighters": "fighter",
    "fighting": "fight",
    "fights": "fight",
    "figments": "figment",
    "figs": "fig",
    "figured": "figure",
    "figureheads": "figurehead",
    "figures": "figure",
    "figurines": "figurine",
    "figuring": "figure",
    "filaments": "filament",
    "filariases": "filariasis",
    "filched": "filch",
    "filches": "filch",
    "filching": "filch",
    "filed": "file",
    "filenames": "filename",
    "filers": "filer",
    "files": "file",
    "fileservers": "fileserver",
    "filets": "filet",
    "filibustered": "filibuster",
    "filibustering": "filibuster",
    "filibusters": "filibuster",
    "filing": "file",
    "filings": "filing",
    "fill-ins": "fill-in",
    "filled": "fill",
    "fillers": "filler",
    "filleted": "fillet",
    "filleting": "fillet",
    "fillets": "fillet",
    "fillies": "filly",
    "filling": "fill",
    "fillings": "filling",
    "fillips": "fillip",
    "fills": "fill",
    "film-strips": "film-strip",
    "filmed": "film",
    "filmier": "filmy",
    "filmiest": "filmy",
    "filming": "film",
    "films": "film",
    "filmstrips": "filmstrip",
    "filtered": "filter",
    "filtering": "filter",
    "filters": "filter",
    "filthier": "filthy",
    "filthiest": "filthy",
    "filtrates": "filtrate",
    "filtrations": "filtration",
    "finales": "finale",
    "finalisations": "finalisation",
    "finalised": "finalise",
    "finalises": "finalise",
    "finalising": "finalise",
    "finalists": "finalist",
    "finalities": "finality",
    "finalizations": "finalization",
    "finalized": "finalize",
    "finalizes": "finalize",
    "finalizing": "finalize",
    "finals": "final",
    "financed": "finance",
    "finances": "finance",
    "financiers": "financier",
    "financing": "finance",
    "financings": "financing",
    "finasterides": "finasteride",
    "finches": "finch",
    "finders": "finder",
    "finding": "find",
    "findings": "finding",
    "finds": "find",
    "fined": "fine",
    "finer": "fine",
    "fines": "fine",
    "finessed": "finesse",
    "finesses": "finesse",
    "finessing": "finesse",
    "finest": "fine",
    "finger-tips": "finger-tip",
    "fingered": "finger",
    "fingering": "finger",
    "fingerlings": "fingerling",
    "fingermarks": "fingermark",
    "fingernails": "fingernail",
    "fingerprinted": "fingerprint",
    "fingerprinting": "fingerprint",
    "fingerprints": "fingerprint",
    "fingers": "finger",
    "fingerstalls": "fingerstall",
    "fingertips": "fingertip",
    "finials": "finial",
    "finickier": "finicky",
    "finickiest": "finicky",
    "fining": "fine",
    "finished": "finish",
    "finishers": "finisher",
    "finishes": "finish",
    "finishing": "finish",
    "finked": "fink",
    "finking": "fink",
    "finks": "fink",
    "finned": "fin",
    "finning": "fin",
    "fins": "fin",
    "fiords": "fiord",
    "fire-arms": "fire-arm",
    "fire-eaters": "fire-eater",
    "fire-fighters": "fire-fighter",
    "fire-storms": "fire-storm",
    "firearms": "firearm",
    "fireballs": "fireball",
    "firebombs": "firebomb",
    "firebrands": "firebrand",
    "firebreaks": "firebreak",
    "firebricks": "firebrick",
    "firecrackers": "firecracker",
    "fired": "fire",
    "firefighters": "firefighter",
    "fireflies": "firefly",
    "fireguards": "fireguard",
    "firemen": "fireman",
    "fireplaces": "fireplace",
    "fireplugs": "fireplug",
    "fireproofed": "fireproof",
    "fireproofing": "fireproof",
    "fireproofs": "fireproof",
    "firers": "firer",
    "fires": "fire",
    "firesides": "fireside",
    "firestorms": "firestorm",
    "firewalls": "firewall",
    "fireworks": "firework",
    "firing": "fire",
    "firings": "firing",
    "firmed": "firm",
    "firmer": "firm",
    "firmest": "firm",
    "firming": "firm",
    "firmnesses": "firmness",
    "firms": "firm",
    "firs": "fir",
    "first": "﻿1",
    "firstborns": "firstborn",
    "firsts": "first",
    "firths": "firth",
    "fishcakes": "fishcake",
    "fished": "fish",
    "fisheries": "fishery",
    "fishermen": "fisherman",
    "fishes": "fish",
    "fisheyes": "fisheye",
    "fishier": "fishy",
    "fishiest": "fishy",
    "fishing": "fish",
    "fishmeals": "fishmeal",
    "fishmongers": "fishmonger",
    "fishnets": "fishnet",
    "fishponds": "fishpond",
    "fishwives": "fishwife",
    "fissions": "fission",
    "fissured": "fissure",
    "fissures": "fissure",
    "fissuring": "fissure",
    "fistfuls": "fistful",
    "fists": "fist",
    "fistulae": "fistula",
    "fistulas": "fistula",
    "fitfully": "fitful",
    "fitments": "fitment",
    "fitnesses": "fitness",
    "fits": "fit",
    "fitted": "fit",
    "fitter": "fit",
    "fitters": "fitter",
    "fittest": "fit",
    "fitting": "fit",
    "fittings": "fitting",
    "five-hundredth": "500",
    "fivers": "fiver",
    "fives": "five",
    "fixated": "fixate",
    "fixates": "fixate",
    "fixating": "fixate",
    "fixations": "fixation",
    "fixatives": "fixative",
    "fixators": "fixator",
    "fixed": "fix",
    "fixers": "fixer",
    "fixes": "fix",
    "fixing": "fix",
    "fixtures": "fixture",
    "fizzed": "fizz",
    "fizzes": "fizz",
    "fizzier": "fizzy",
    "fizziest": "fizzy",
    "fizzing": "fizz",
    "fizzled": "fizzle",
    "fizzles": "fizzle",
    "fizzling": "fizzle",
    "fjords": "fjord",
    "flabbergasted": "flabbergast",
    "flabbergasting": "flabbergast",
    "flabbergasts": "flabbergast",
    "flabbier": "flabby",
    "flabbiest": "flabby",
    "flagella": "flagellum",
    "flagellated": "flagellate",
    "flagellates": "flagellate",
    "flagellating": "flagellate",
    "flagellums": "flagellum",
    "flagged": "flag",
    "flagging": "flag",
    "flagons": "flagon",
    "flagpoles": "flagpole",
    "flags": "flag",
    "flagships": "flagship",
    "flagstaffs": "flagstaff",
    "flagstaves": "flagstaff",
    "flagstones": "flagstone",
    "flailed": "flail",
    "flailing": "flail",
    "flails": "flail",
    "flaked": "flake",
    "flakes": "flake",
    "flakier": "flaky",
    "flakiest": "flaky",
    "flaking": "flake",
    "flambeed": "flambe",
    "flambeing": "flambe",
    "flambes": "flambe",
    "flame-throwers": "flame-thrower",
    "flamed": "flame",
    "flamencos": "flamenco",
    "flames": "flame",
    "flaming": "flame",
    "flamingoes": "flamingo",
    "flamingos": "flamingo",
    "flammabilities": "flammability",
    "flanged": "flange",
    "flanges": "flange",
    "flanging": "flange",
    "flanked": "flank",
    "flankers": "flanker",
    "flanking": "flank",
    "flanks": "flank",
    "flannelled": "flannel",
    "flannelling": "flannel",
    "flannels": "flannel",
    "flans": "flan",
    "flapjacks": "flapjack",
    "flapped": "flap",
    "flappers": "flapper",
    "flapping": "flap",
    "flaps": "flap",
    "flare-ups": "flare-up",
    "flared": "flare",
    "flares": "flare",
    "flaring": "flare",
    "flas": "fla",
    "flashbacks": "flashback",
    "flashbulbs": "flashbulb",
    "flashcards": "flashcard",
    "flashcubes": "flashcube",
    "flashed": "flash",
    "flashers": "flasher",
    "flashes": "flash",
    "flashguns": "flashgun",
    "flashier": "flashy",
    "flashiest": "flashy",
    "flashing": "flash",
    "flashlights": "flashlight",
    "flashpoints": "flashpoint",
    "flasks": "flask",
    "flatfishes": "flatfish",
    "flatirons": "flatiron",
    "flatlets": "flatlet",
    "flatmates": "flatmate",
    "flatnesses": "flatness",
    "flats": "flat",
    "flattened": "flatten",
    "flattening": "flatten",
    "flattens": "flatten",
    "flatter": "flat",
    "flattered": "flatter",
    "flatterers": "flatterer",
    "flatteries": "flattery",
    "flattering": "flatter",
    "flatters": "flatter",
    "flattest": "flat",
    "flatulences": "flatulence",
    "flatworms": "flatworm",
    "flaunted": "flaunt",
    "flaunting": "flaunt",
    "flaunts": "flaunt",
    "flautists": "flautist",
    "flavonoids": "flavonoid",
    "flavored": "flavor",
    "flavoring": "flavor",
    "flavorings": "flavoring",
    "flavors": "flavor",
    "flavoured": "flavour",
    "flavouring": "flavour",
    "flavourings": "flavouring",
    "flavours": "flavour",
    "flawed": "flaw",
    "flawing": "flaw",
    "flaws": "flaw",
    "flaxes": "flax",
    "flaxseeds": "flaxseed",
    "flayed": "flay",
    "flaying": "flay",
    "flays": "flay",
    "fleapits": "fleapit",
    "fleas": "flea",
    "flecked": "fleck",
    "flecking": "fleck",
    "flecks": "fleck",
    "fled": "flee",
    "fledged": "fledge",
    "fledgelings": "fledgeling",
    "fledges": "fledge",
    "fledging": "fledge",
    "fledglings": "fledgling",
    "fleeced": "fleece",
    "fleeces": "fleece",
    "fleecier": "fleecy",
    "fleecing": "fleece",
    "fleeing": "flee",
    "flees": "flee",
    "fleeter": "fleet",
    "fleets": "fleet",
    "fleshed": "flesh",
    "fleshes": "flesh",
    "fleshier": "fleshy",
    "fleshiest": "fleshy",
    "fleshing": "flesh",
    "fleshpots": "fleshpot",
    "fletchers": "fletcher",
    "fleurs-de-lis": "fleur-de-lis",
    "fleurs-de-lys": "fleur-de-lys",
    "flew": "fly",
    "flexed": "flex",
    "flexes": "flex",
    "flexibilities": "flexibility",
    "flexing": "flex",
    "flexions": "flexion",
    "flexors": "flexor",
    "flexures": "flexure",
    "flibbertigibbets": "flibbertigibbet",
    "flick-knives": "flick-knife",
    "flicked": "flick",
    "flickered": "flicker",
    "flickering": "flicker",
    "flickers": "flicker",
    "flicking": "flick",
    "flicks": "flick",
    "flied": "fly",
    "fliers": "flier",
    "flies": "fly",
    "flightier": "flighty",
    "flightiest": "flighty",
    "flightpaths": "flightpath",
    "flights": "flight",
    "flimsier": "flimsy",
    "flimsies": "flimsy",
    "flimsiest": "flimsy",
    "flinched": "flinch",
    "flinches": "flinch",
    "flinching": "flinch",
    "flinging": "fling",
    "flings": "fling",
    "flintier": "flinty",
    "flintlocks": "flintlock",
    "flints": "flint",
    "flip-flops": "flip-flop",
    "flippancies": "flippancy",
    "flipped": "flip",
    "flippers": "flipper",
    "flipping": "flip",
    "flips": "flip",
    "flirtations": "flirtation",
    "flirted": "flirt",
    "flirting": "flirt",
    "flirts": "flirt",
    "flits": "flit",
    "flitted": "flit",
    "flitting": "flit",
    "floatations": "floatation",
    "floated": "float",
    "floaters": "floater",
    "floating": "float",
    "floats": "float",
    "flocculations": "flocculation",
    "flocked": "flock",
    "flocking": "flock",
    "flocks": "flock",
    "flocs": "floc",
    "flogged": "flog",
    "flogging": "flog",
    "flogs": "flog",
    "flooded": "flood",
    "floodgates": "floodgate",
    "flooding": "flood",
    "floodings": "flooding",
    "floodlighted": "floodlight",
    "floodlighting": "floodlight",
    "floodlights": "floodlight",
    "floodlit": "floodlight",
    "floodplains": "floodplain",
    "floods": "flood",
    "floodwaters": "floodwater",
    "floorboards": "floorboard",
    "floorcoverings": "floorcovering",
    "floored": "floor",
    "flooring": "floor",
    "floorings": "flooring",
    "floors": "floor",
    "floozies": "floozy",
    "flopped": "flop",
    "floppier": "floppy",
    "floppies": "floppy",
    "floppiest": "floppy",
    "flopping": "flop",
    "flops": "flop",
    "florae": "flora",
    "floras": "flora",
    "florets": "floret",
    "floribundas": "floribunda",
    "florins": "florin",
    "florists": "florist",
    "flossed": "floss",
    "flosses": "floss",
    "flossing": "floss",
    "flotations": "flotation",
    "flotillas": "flotilla",
    "flounced": "flounce",
    "flounces": "flounce",
    "flouncing": "flounce",
    "floundered": "flounder",
    "floundering": "flounder",
    "flounders": "flounder",
    "floured": "flour",
    "flourier": "floury",
    "flouring": "flour",
    "flourished": "flourish",
    "flourishes": "flourish",
    "flourishing": "flourish",
    "flours": "flour",
    "flouted": "flout",
    "flouting": "flout",
    "flouts": "flout",
    "flow-charts": "flow-chart",
    "flowcharts": "flowchart",
    "flowed": "flow",
    "flowerbeds": "flowerbed",
    "flowered": "flower",
    "flowerier": "flowery",
    "floweriest": "flowery",
    "flowering": "flower",
    "flowerings": "flowering",
    "flowerpots": "flowerpot",
    "flowers": "flower",
    "flowing": "flow",
    "flowmeters": "flowmeter",
    "flown": "fly",
    "flowrates": "flowrate",
    "flows": "flow",
    "fluconazoles": "fluconazole",
    "fluctuated": "fluctuate",
    "fluctuates": "fluctuate",
    "fluctuating": "fluctuate",
    "fluctuations": "fluctuation",
    "fluences": "fluence",
    "fluencies": "fluency",
    "flues": "flue",
    "fluffed": "fluff",
    "fluffier": "fluffy",
    "fluffiest": "fluffy",
    "fluffing": "fluff",
    "fluffs": "fluff",
    "fluidised": "fluidise",
    "fluidises": "fluidise",
    "fluidising": "fluidise",
    "fluidities": "fluidity",
    "fluids": "fluid",
    "flukes": "fluke",
    "flumes": "flume",
    "flummoxed": "flummox",
    "flummoxes": "flummox",
    "flummoxing": "flummox",
    "flung": "fling",
    "flunked": "flunk",
    "flunkeys": "flunkey",
    "flunkies": "flunky",
    "flunking": "flunk",
    "flunks": "flunk",
    "fluoresced": "fluoresce",
    "fluoresceins": "fluorescein",
    "fluorescences": "fluorescence",
    "fluoresces": "fluoresce",
    "fluorescing": "fluoresce",
    "fluoridated": "fluoridate",
    "fluoridates": "fluoridate",
    "fluoridating": "fluoridate",
    "fluorides": "fluoride",
    "fluorinated": "fluorinate",
    "fluorinates": "fluorinate",
    "fluorinating": "fluorinate",
    "fluorines": "fluorine",
    "fluorocarbons": "fluorocarbon",
    "fluorochromes": "fluorochrome",
    "fluorophores": "fluorophore",
    "fluoroquinolones": "fluoroquinolone",
    "fluoroscopies": "fluoroscopy",
    "fluoroses": "fluorosis",
    "flurries": "flurry",
    "flushed": "flush",
    "flusher": "flush",
    "flushes": "flush",
    "flushest": "flush",
    "flushing": "flush",
    "flustered": "fluster",
    "flustering": "fluster",
    "flusters": "fluster",
    "fluted": "flute",
    "flutes": "flute",
    "fluting": "flute",
    "flutists": "flutist",
    "fluttered": "flutter",
    "fluttering": "flutter",
    "flutters": "flutter",
    "fluxes": "flux",
    "flybys": "flyby",
    "flycatchers": "flycatcher",
    "flyers": "flyer",
    "flying": "fly",
    "flyleaves": "flyleaf",
    "flyovers": "flyover",
    "flypasts": "flypast",
    "flysheets": "flysheet",
    "flywheels": "flywheel",
    "foaled": "foal",
    "foaling": "foal",
    "foals": "foal",
    "foamed": "foam",
    "foamier": "foamy",
    "foamiest": "foamy",
    "foaming": "foam",
    "foams": "foam",
    "fobbed": "fob",
    "fobbing": "fob",
    "fobs": "fob",
    "foci": "focus",
    "focused": "focus",
    "focusers": "focuser",
    "focuses": "focus",
    "focusing": "focus",
    "focussed": "focus",
    "focusses": "focus",
    "focussing": "focus",
    "fodders": "fodder",
    "foes": "foe",
    "foeti": "foetus",
    "foetuses": "foetus",
    "fogeys": "fogey",
    "fogged": "fog",
    "foggier": "foggy",
    "foggiest": "foggy",
    "fogging": "fog",
    "foghorns": "foghorn",
    "fogs": "fog",
    "foibles": "foible",
    "foiled": "foil",
    "foiling": "foil",
    "foils": "foil",
    "foisted": "foist",
    "foisting": "foist",
    "foists": "foist",
    "folates": "folate",
    "folded": "fold",
    "folders": "folder",
    "folding": "fold",
    "foldings": "folding",
    "folds": "fold",
    "folios": "folio",
    "folklores": "folklore",
    "folklorists": "folklorist",
    "folks": "folk",
    "folksongs": "folksong",
    "folktales": "folktale",
    "follicles": "follicle",
    "follies": "folly",
    "follow-ons": "follow-on",
    "follow-throughs": "follow-through",
    "follow-ups": "follow-up",
    "followed": "follow",
    "followers": "follower",
    "following": "follow",
    "followings": "following",
    "follows": "follow",
    "followups": "followup",
    "fomented": "foment",
    "fomenting": "foment",
    "foments": "foment",
    "fondants": "fondant",
    "fonder": "fond",
    "fondest": "fond",
    "fondled": "fondle",
    "fondles": "fondle",
    "fondling": "fondle",
    "fondues": "fondue",
    "fons": "fon",
    "fonts": "font",
    "foodplants": "foodplant",
    "foods": "food",
    "foodservices": "foodservice",
    "foodstuffs": "foodstuff",
    "fooled": "fool",
    "fooling": "fool",
    "fools": "fool",
    "footballers": "footballer",
    "footballs": "football",
    "footbaths": "footbath",
    "footboards": "footboard",
    "footbrakes": "footbrake",
    "footbridges": "footbridge",
    "footed": "foot",
    "footers": "footer",
    "footfalls": "footfall",
    "foothills": "foothill",
    "footholds": "foothold",
    "footing": "foot",
    "footings": "footing",
    "footlights": "footlight",
    "footmarks": "footmark",
    "footmen": "footman",
    "footnoted": "footnote",
    "footnotes": "footnote",
    "footnoting": "footnote",
    "footpads": "footpad",
    "footpaths": "footpath",
    "footplates": "footplate",
    "footprintings": "footprinting",
    "footprints": "footprint",
    "footrests": "footrest",
    "foots": "foot",
    "footsteps": "footstep",
    "footstools": "footstool",
    "footswitches": "footswitch",
    "footways": "footway",
    "footwears": "footwear",
    "fops": "fop",
    "fora": "forum",
    "foraged": "forage",
    "foragers": "forager",
    "forages": "forage",
    "foraging": "forage",
    "foramens": "foramen",
    "foramina": "foramen",
    "foraminifera": "foraminifer",
    "foraminifers": "foraminifer",
    "forays": "foray",
    "forbad": "forbid",
    "forbade": "forbid",
    "forbearing": "forbear",
    "forbears": "forbear",
    "forbidden": "forbid",
    "forbidding": "forbid",
    "forbids": "forbid",
    "forbore": "forbear",
    "forborne": "forbear",
    "force-fed": "force-feed",
    "force-feeding": "force-feed",
    "force-feeds": "force-feed",
    "forced": "force",
    "forceps": "forcep",
    "forcepses": "forceps",
    "forces": "force",
    "forcing": "force",
    "forcipes": "forceps",
    "forded": "ford",
    "fording": "ford",
    "fords": "ford",
    "fore-runners": "fore-runner",
    "forearms": "forearm",
    "forebears": "forebear",
    "foreboded": "forebode",
    "forebodes": "forebode",
    "foreboding": "forebode",
    "forebodings": "foreboding",
    "forebrains": "forebrain",
    "forecasted": "forecast",
    "forecasters": "forecaster",
    "forecasting": "forecast",
    "forecastles": "forecastle",
    "forecasts": "forecast",
    "foreclosed": "foreclose",
    "forecloses": "foreclose",
    "foreclosing": "foreclose",
    "foreclosures": "foreclosure",
    "forecourts": "forecourt",
    "forefathers": "forefather",
    "forefeet": "forefoot",
    "forefingers": "forefinger",
    "forefronts": "forefront",
    "foregoes": "forego",
    "foregoing": "forego",
    "foregone": "forego",
    "foregrounded": "foreground",
    "foregrounding": "foreground",
    "foregrounds": "foreground",
    "forehands": "forehand",
    "foreheads": "forehead",
    "foreigners": "foreigner",
    "foreigns": "foreign",
    "foreknew": "foreknow",
    "foreknowing": "foreknow",
    "foreknown": "foreknow",
    "foreknows": "foreknow",
    "forelands": "foreland",
    "forelegs": "foreleg",
    "forelimbs": "forelimb",
    "forelocks": "forelock",
    "foremen": "foreman",
    "forenames": "forename",
    "foreordained": "foreordain",
    "foreordaining": "foreordain",
    "foreordains": "foreordain",
    "foreparts": "forepart",
    "forerunners": "forerunner",
    "fores": "fore",
    "foresaken": "foresake",
    "foresakes": "foresake",
    "foresaking": "foresake",
    "foresaw": "foresee",
    "foreseeing": "foresee",
    "foreseen": "foresee",
    "foresees": "foresee",
    "foreshadowed": "foreshadow",
    "foreshadowing": "foreshadow",
    "foreshadows": "foreshadow",
    "foreshores": "foreshore",
    "foreshortened": "foreshorten",
    "foreshortening": "foreshorten",
    "foreshortens": "foreshorten",
    "foreshowed": "foreshow",
    "foreshowing": "foreshow",
    "foreshown": "foreshow",
    "foreshows": "foreshow",
    "foresights": "foresight",
    "foreskins": "foreskin",
    "foresook": "foresake",
    "forestalled": "forestall",
    "forestalling": "forestall",
    "forestalls": "forestall",
    "forested": "forest",
    "foresters": "forester",
    "foresting": "forest",
    "forestries": "forestry",
    "forests": "forest",
    "foretastes": "foretaste",
    "foretelling": "foretell",
    "foretells": "foretell",
    "forethoughts": "forethought",
    "foretold": "foretell",
    "forewarned": "forewarn",
    "forewarning": "forewarn",
    "forewarns": "forewarn",
    "forewent": "forego",
    "forewings": "forewing",
    "forewords": "foreword",
    "forfeited": "forfeit",
    "forfeiting": "forfeit",
    "forfeits": "forfeit",
    "forgave": "forgive",
    "forged": "forge",
    "forgeries": "forgery",
    "forgers": "forger",
    "forges": "forge",
    "forget-me-nots": "forget-me-not",
    "forgets": "forget",
    "forgetting": "forget",
    "forging": "forge",
    "forgiven": "forgive",
    "forgives": "forgive",
    "forgiving": "forgive",
    "forgoes": "forgo",
    "forgoing": "forgo",
    "forgone": "forgo",
    "forgot": "forget",
    "forgotten": "forget",
    "fork-lifts": "fork-lift",
    "forked": "fork",
    "forking": "fork",
    "forklifts": "forklift",
    "forks": "fork",
    "formalisations": "formalisation",
    "formalised": "formalise",
    "formalises": "formalise",
    "formalising": "formalise",
    "formalisms": "formalism",
    "formalists": "formalist",
    "formalities": "formality",
    "formalizations": "formalization",
    "formalized": "formalize",
    "formalizes": "formalize",
    "formalizing": "formalize",
    "formants": "formant",
    "formations": "formation",
    "formatives": "formative",
    "formats": "format",
    "formatted": "format",
    "formatters": "formatter",
    "formatting": "format",
    "formed": "form",
    "formers": "former",
    "formes": "forme",
    "forming": "form",
    "forms": "form",
    "formulae": "formula",
    "formularies": "formulary",
    "formulas": "formula",
    "formulated": "formulate",
    "formulates": "formulate",
    "formulating": "formulate",
    "formulations": "formulation",
    "formulÃƒÂ¦": "formula",
    "fornicated": "fornicate",
    "fornicates": "fornicate",
    "fornicating": "fornicate",
    "forsaked": "forsake",
    "forsaken": "forsake",
    "forsakes": "forsake",
    "forsaking": "forsake",
    "forsook": "forsake",
    "forswearing": "forswear",
    "forswears": "forswear",
    "forswore": "forswear",
    "forsworn": "forswear",
    "fortes": "fortis",
    "forties": "fourty",
    "fortieth": "40",
    "fortieths": "fortieth",
    "fortifications": "fortification",
    "fortified": "fortify",
    "fortifies": "fortify",
    "fortifying": "fortify",
    "fortnights": "fortnight",
    "fortresses": "fortress",
    "forts": "fort",
    "fortune-tellers": "fortune-teller",
    "fortunes": "fortune",
    "forty-eighth": "48",
    "forty-eighths": "forty-eighth",
    "forty-fifth": "45",
    "forty-fifths": "forty-fifth",
    "forty-first": "41",
    "forty-firsts": "forty-first",
    "forty-fourth": "44",
    "forty-fourths": "forty-fourth",
    "forty-ninth": "49",
    "forty-ninths": "forty-ninth",
    "forty-second": "42",
    "forty-seconds": "forty-second",
    "forty-seventh": "47",
    "forty-sevenths": "forty-seventh",
    "forty-sixth": "46",
    "forty-sixths": "forty-sixth",
    "forty-third": "43",
    "forty-thirds": "forty-third",
    "forums": "forum",
    "forwarded": "forward",
    "forwarders": "forwarder",
    "forwarding": "forward",
    "forwards": "forward",
    "forwent": "forgo",
    "fossae": "fossa",
    "fossas": "fossa",
    "fosses": "fosse",
    "fossilised": "fossilise",
    "fossilises": "fossilise",
    "fossilising": "fossilise",
    "fossilized": "fossilize",
    "fossilizes": "fossilize",
    "fossilizing": "fossilize",
    "fossils": "fossil",
    "foster-parents": "foster-parent",
    "fostered": "foster",
    "fosterers": "fosterer",
    "fostering": "foster",
    "fosters": "foster",
    "fought": "fight",
    "foul-ups": "foul-up",
    "fouled": "foul",
    "fouler": "foul",
    "foulest": "foul",
    "fouling": "foul",
    "fouls": "foul",
    "found": "find",
    "foundations": "foundation",
    "founded": "found",
    "foundered": "founder",
    "foundering": "founder",
    "founders": "founder",
    "founding": "found",
    "foundlings": "foundling",
    "foundries": "foundry",
    "founds": "found",
    "fountains": "fountain",
    "founts": "fount",
    "four-hundredth": "400",
    "four-posters": "four-poster",
    "fourpences": "fourpence",
    "fours": "four",
    "foursomes": "foursome",
    "fourteenth": "14",
    "fourteenths": "fourteenth",
    "fourth": "4",
    "fourths": "fourth",
    "foveae": "fovea",
    "foveas": "fovea",
    "fowlers": "fowler",
    "fowls": "fowl",
    "foxed": "fox",
    "foxes": "fox",
    "foxgloves": "foxglove",
    "foxholes": "foxhole",
    "foxhounds": "foxhound",
    "foxhunted": "foxhunt",
    "foxhunting": "foxhunt",
    "foxhunts": "foxhunt",
    "foxier": "foxy",
    "foxiest": "foxy",
    "foxing": "fox",
    "foyers": "foyer",
    "fps": "fp",
    "fracases": "fracas",
    "fractals": "fractal",
    "fractionated": "fractionate",
    "fractionates": "fractionate",
    "fractionating": "fractionate",
    "fractionations": "fractionation",
    "fractions": "fraction",
    "fractured": "fracture",
    "fractures": "fracture",
    "fracturing": "fracture",
    "fragilities": "fragility",
    "fragmentations": "fragmentation",
    "fragmented": "fragment",
    "fragmenting": "fragment",
    "fragments": "fragment",
    "fragrances": "fragrance",
    "frailer": "frail",
    "frailest": "frail",
    "frailties": "frailty",
    "frame-ups": "frame-up",
    "framed": "frame",
    "framers": "framer",
    "frames": "frame",
    "frameshifts": "frameshift",
    "frameworks": "framework",
    "framing": "frame",
    "franchised": "franchise",
    "franchisees": "franchisee",
    "franchisers": "franchiser",
    "franchises": "franchise",
    "franchising": "franchise",
    "franchisors": "franchisor",
    "francophones": "francophone",
    "francs": "franc",
    "franked": "frank",
    "franker": "frank",
    "frankest": "frank",
    "frankfurters": "frankfurter",
    "franking": "frank",
    "franks": "frank",
    "fras": "fra",
    "fraternised": "fraternise",
    "fraternises": "fraternise",
    "fraternising": "fraternise",
    "fraternities": "fraternity",
    "fraternized": "fraternize",
    "fraternizes": "fraternize",
    "fraternizing": "fraternize",
    "fratricides": "fratricide",
    "frats": "frat",
    "frauds": "fraud",
    "fraudsters": "fraudster",
    "frayed": "fray",
    "fraying": "fray",
    "frays": "fray",
    "frazzled": "frazzle",
    "frazzles": "frazzle",
    "frazzling": "frazzle",
    "freaked": "freak",
    "freakier": "freaky",
    "freakiest": "freaky",
    "freaking": "freak",
    "freaks": "freak",
    "freckled": "freckle",
    "freckles": "freckle",
    "freckling": "freckle",
    "free-for-alls": "free-for-all",
    "freebies": "freeby",
    "freed": "free",
    "freedmen": "freedman",
    "freedoms": "freedom",
    "freeholds": "freehold",
    "freeing": "free",
    "freelanced": "freelance",
    "freelances": "freelance",
    "freelancing": "freelance",
    "freeloaders": "freeloader",
    "freemen": "freeman",
    "freer": "free",
    "frees": "free",
    "freesias": "freesia",
    "freest": "free",
    "freethinkers": "freethinker",
    "freeways": "freeway",
    "freewheeled": "freewheel",
    "freewheeling": "freewheel",
    "freewheels": "freewheel",
    "freeze-dried": "freeze-dry",
    "freeze-dries": "freeze-dry",
    "freeze-drying": "freeze-dry",
    "freeze-frames": "freeze-frame",
    "freeze-ups": "freeze-up",
    "freezers": "freezer",
    "freezes": "freeze",
    "freezing": "freeze",
    "freezings": "freezing",
    "freighted": "freight",
    "freighters": "freighter",
    "freighting": "freight",
    "freights": "freight",
    "frenzies": "frenzy",
    "frequencies": "frequency",
    "frequented": "frequent",
    "frequenters": "frequenter",
    "frequenting": "frequent",
    "frequents": "frequent",
    "frescoes": "fresco",
    "frescos": "fresco",
    "freshed": "fresh",
    "freshened": "freshen",
    "fresheners": "freshener",
    "freshening": "freshen",
    "freshens": "freshen",
    "fresher": "fresh",
    "freshers": "fresher",
    "freshes": "fresh",
    "freshest": "fresh",
    "freshing": "fresh",
    "freshmen": "freshman",
    "freshnesses": "freshness",
    "freshwaters": "freshwater",
    "fretfully": "fretful",
    "frets": "fret",
    "fretted": "fret",
    "fretting": "fret",
    "friars": "friar",
    "fricassees": "fricassee",
    "fricatives": "fricative",
    "frictions": "friction",
    "fridges": "fridge",
    "fried": "fry",
    "friendlier": "friendly",
    "friendlies": "friendly",
    "friendliest": "friendly",
    "friends": "friend",
    "friendships": "friendship",
    "fries": "fry",
    "friezes": "frieze",
    "frigates": "frigate",
    "frigged": "frig",
    "frigging": "frig",
    "frightened": "frighten",
    "frightening": "frighten",
    "frightens": "frighten",
    "frights": "fright",
    "frigider": "frigid",
    "frigidest": "frigid",
    "frigs": "frig",
    "frilled": "frill",
    "frillier": "frilly",
    "frilling": "frill",
    "frills": "frill",
    "fringed": "fringe",
    "fringes": "fringe",
    "fringing": "fringe",
    "fripperies": "frippery",
    "frisked": "frisk",
    "friskier": "frisky",
    "friskiest": "frisky",
    "frisking": "frisk",
    "frisks": "frisk",
    "frissons": "frisson",
    "fritillaries": "fritillary",
    "frittered": "fritter",
    "frittering": "fritter",
    "fritters": "fritter",
    "frivolities": "frivolity",
    "frizzed": "frizz",
    "frizzes": "frizz",
    "frizzier": "frizzy",
    "frizziest": "frizzy",
    "frizzing": "frizz",
    "frizzled": "frizzle",
    "frizzles": "frizzle",
    "frizzling": "frizzle",
    "frocks": "frock",
    "frog-marched": "frog-march",
    "frog-marches": "frog-march",
    "frog-marching": "frog-march",
    "frogged": "frog",
    "frogging": "frog",
    "froggings": "frogging",
    "froglets": "froglet",
    "frogmen": "frogman",
    "frogs": "frog",
    "frolicked": "frolic",
    "frolicking": "frolic",
    "frolics": "frolic",
    "fronds": "frond",
    "front-runners": "front-runner",
    "frontages": "frontage",
    "frontbenchers": "frontbencher",
    "fronted": "front",
    "frontiers": "frontier",
    "frontiersmen": "frontiersman",
    "fronting": "front",
    "frontispieces": "frontispiece",
    "fronts": "front",
    "frostbites": "frostbite",
    "frosted": "frost",
    "frostier": "frosty",
    "frostiest": "frosty",
    "frosting": "frost",
    "frosts": "frost",
    "frothed": "froth",
    "frothier": "frothy",
    "frothiest": "frothy",
    "frothing": "froth",
    "froths": "froth",
    "frowned": "frown",
    "frowning": "frown",
    "frowns": "frown",
    "froze": "freeze",
    "frozen": "freeze",
    "fructoses": "fructose",
    "frugalities": "frugality",
    "fruitcakes": "fruitcake",
    "fruited": "fruit",
    "fruiterers": "fruiterer",
    "fruitflies": "fruitfly",
    "fruitier": "fruity",
    "fruitiest": "fruity",
    "fruiting": "fruit",
    "fruits": "fruit",
    "frumpier": "frumpy",
    "frumps": "frump",
    "frustrated": "frustrate",
    "frustrates": "frustrate",
    "frustrating": "frustrate",
    "frustrations": "frustration",
    "fry-ups": "fry-up",
    "fryers": "fryer",
    "frying": "fry",
    "fuchsias": "fuchsia",
    "fucked": "fuck",
    "fucking": "fuck",
    "fucks": "fuck",
    "fuddy-duddies": "fuddy-duddy",
    "fudged": "fudge",
    "fudges": "fudge",
    "fudging": "fudge",
    "fueled": "fuel",
    "fueling": "fuel",
    "fuelled": "fuel",
    "fuelling": "fuel",
    "fuels": "fuel",
    "fuggier": "fuggy",
    "fugitives": "fugitive",
    "fugues": "fugue",
    "fulcra": "fulcrum",
    "fulcrums": "fulcrum",
    "fulfilled": "fulfill",
    "fulfilling": "fulfill",
    "fulfillments": "fulfillment",
    "fulfills": "fulfill",
    "fulfilments": "fulfilment",
    "fulfils": "fulfil",
    "full-backs": "full-back",
    "full-terms": "full-term",
    "full-times": "full-time",
    "fullbacks": "fullback",
    "fulled": "full",
    "fuller": "full",
    "fullerenes": "fullerene",
    "fullers": "fuller",
    "fullest": "full",
    "fulling": "full",
    "fulls": "full",
    "fulmars": "fulmar",
    "fulminated": "fulminate",
    "fulminates": "fulminate",
    "fulminating": "fulminate",
    "fumarates": "fumarate",
    "fumbled": "fumble",
    "fumbles": "fumble",
    "fumbling": "fumble",
    "fumed": "fume",
    "fumes": "fume",
    "fumigants": "fumigant",
    "fumigated": "fumigate",
    "fumigates": "fumigate",
    "fumigating": "fumigate",
    "fumigations": "fumigation",
    "fuming": "fume",
    "fumonisins": "fumonisin",
    "functionalists": "functionalist",
    "functionalities": "functionality",
    "functionals": "functional",
    "functionaries": "functionary",
    "functioned": "function",
    "functioning": "function",
    "functions": "function",
    "functors": "functor",
    "fund-raisers": "fund-raiser",
    "fundamentalists": "fundamentalist",
    "fundamentals": "fundamental",
    "fundaments": "fundament",
    "funded": "fund",
    "funders": "funder",
    "fundholders": "fundholder",
    "fundholdings": "fundholding",
    "fundi": "fundus",
    "funding": "fund",
    "fundings": "funding",
    "fundoplications": "fundoplication",
    "fundraisers": "fundraiser",
    "funds": "fund",
    "funerals": "funeral",
    "funfairs": "funfair",
    "fungated": "fungate",
    "fungates": "fungate",
    "fungating": "fungate",
    "fungi": "fungus",
    "fungicides": "fungicide",
    "funguses": "fungus",
    "funiculars": "funicular",
    "funked": "funk",
    "funkier": "funky",
    "funkiest": "funky",
    "funking": "funk",
    "funks": "funk",
    "funneled": "funnel",
    "funneling": "funnel",
    "funnelled": "funnel",
    "funnelling": "funnel",
    "funnels": "funnel",
    "funnier": "funny",
    "funnies": "funny",
    "funniest": "funny",
    "funnymen": "funnyman",
    "furans": "furan",
    "furbished": "furbish",
    "furbishes": "furbish",
    "furbishing": "furbish",
    "furies": "fury",
    "furled": "furl",
    "furling": "furl",
    "furlongs": "furlong",
    "furloughs": "furlough",
    "furls": "furl",
    "furnaces": "furnace",
    "furnished": "furnish",
    "furnishes": "furnish",
    "furnishing": "furnish",
    "furnishings": "furnishing",
    "furred": "fur",
    "furrier": "furry",
    "furriers": "furrier",
    "furriest": "furry",
    "furring": "fur",
    "furrowed": "furrow",
    "furrowing": "furrow",
    "furrows": "furrow",
    "furs": "fur",
    "further": "far",
    "furthered": "further",
    "furthering": "further",
    "furthers": "further",
    "furthest": "far",
    "furtiver": "furtive",
    "furtivest": "furtive",
    "fused": "fuse",
    "fuselages": "fuselage",
    "fuses": "fuse",
    "fusiliers": "fusilier",
    "fusing": "fuse",
    "fusions": "fusion",
    "fussed": "fuss",
    "fusses": "fuss",
    "fussier": "fussy",
    "fussiest": "fussy",
    "fussing": "fuss",
    "fusspots": "fusspot",
    "fustier": "fusty",
    "fustiest": "fusty",
    "futons": "futon",
    "futures": "future",
    "futurists": "futurist",
    "futurities": "futurity",
    "futurologists": "futurologist",
    "fuzed": "fuze",
    "fuzes": "fuze",
    "fuzing": "fuze",
    "fuzzed": "fuzz",
    "fuzzes": "fuzz",
    "fuzzier": "fuzzy",
    "fuzziest": "fuzzy",
    "fuzzinesses": "fuzziness",
    "fuzzing": "fuzz",
    "fêtes": "fête",
    "gabapentins": "gabapentin",
    "gabardines": "gabardine",
    "gabbed": "gab",
    "gabbing": "gab",
    "gabbled": "gabble",
    "gabbles": "gabble",
    "gabbling": "gabble",
    "gabbros": "gabbro",
    "gables": "gable",
    "gabs": "gab",
    "gadded": "gad",
    "gadding": "gad",
    "gadflies": "gadfly",
    "gadgets": "gadget",
    "gads": "gad",
    "gaffers": "gaffer",
    "gaffes": "gaffe",
    "gaged": "gage",
    "gages": "gage",
    "gagged": "gag",
    "gagging": "gag",
    "gaggles": "gaggle",
    "gaging": "gage",
    "gags": "gag",
    "gained": "gain",
    "gainers": "gainer",
    "gaines": "gaine",
    "gaining": "gain",
    "gains": "gain",
    "gainsaid": "gainsay",
    "gainsayed": "gainsay",
    "gainsaying": "gainsay",
    "gainsays": "gainsay",
    "gaiters": "gaiter",
    "gaits": "gait",
    "galactoses": "galactose",
    "galas": "gala",
    "galaxies": "galaxy",
    "gales": "gale",
    "gall-bladders": "gall-bladder",
    "gallantries": "gallantry",
    "gallbladders": "gallbladder",
    "galled": "gall",
    "galleons": "galleon",
    "galleries": "gallery",
    "galleys": "galley",
    "galling": "gall",
    "gallivanted": "gallivant",
    "gallivanting": "gallivant",
    "gallivants": "gallivant",
    "gallons": "gallon",
    "galloped": "gallop",
    "galloping": "gallop",
    "gallops": "gallop",
    "galls": "gall",
    "gallstones": "gallstone",
    "gals": "gal",
    "galvanised": "galvanise",
    "galvanises": "galvanise",
    "galvanising": "galvanise",
    "galvanized": "galvanize",
    "galvanizes": "galvanize",
    "galvanizing": "galvanize",
    "gambas": "gamba",
    "gambits": "gambit",
    "gambled": "gamble",
    "gamblers": "gambler",
    "gambles": "gamble",
    "gambling": "gamble",
    "gamboled": "gambol",
    "gamboling": "gambol",
    "gambolled": "gambol",
    "gambolling": "gambol",
    "gambols": "gambol",
    "gamekeepers": "gamekeeper",
    "gamer": "game",
    "games": "game",
    "gametes": "gamete",
    "gamier": "gamy",
    "gamma-rays": "gamma-ray",
    "gammas": "gamma",
    "gammons": "gammon",
    "gams": "gam",
    "gamuts": "gamut",
    "ganders": "gander",
    "ganged": "gang",
    "ganging": "gang",
    "ganglia": "ganglion",
    "ganglias": "ganglia",
    "ganglions": "ganglion",
    "gangplanks": "gangplank",
    "gangrenes": "gangrene",
    "gangs": "gang",
    "gangsters": "gangster",
    "gangways": "gangway",
    "gannets": "gannet",
    "gantries": "gantry",
    "gaoled": "gaol",
    "gaolers": "gaoler",
    "gaoling": "gaol",
    "gaols": "gaol",
    "gaped": "gape",
    "gapes": "gape",
    "gaping": "gape",
    "gaps": "gap",
    "garaged": "garage",
    "garages": "garage",
    "garaging": "garage",
    "garbages": "garbage",
    "garbed": "garb",
    "garbing": "garb",
    "garbled": "garble",
    "garbles": "garble",
    "garbling": "garble",
    "garbs": "garb",
    "gardened": "garden",
    "gardeners": "gardener",
    "gardenias": "gardenia",
    "gardening": "garden",
    "gardens": "garden",
    "gargled": "gargle",
    "gargles": "gargle",
    "gargling": "gargle",
    "gargoyles": "gargoyle",
    "garlanded": "garland",
    "garlanding": "garland",
    "garlands": "garland",
    "garments": "garment",
    "garnered": "garner",
    "garnering": "garner",
    "garners": "garner",
    "garnets": "garnet",
    "garnished": "garnish",
    "garnishes": "garnish",
    "garnishing": "garnish",
    "garrets": "garret",
    "garrisoned": "garrison",
    "garrisoning": "garrison",
    "garrisons": "garrison",
    "garrotted": "garrotte",
    "garrottes": "garrotte",
    "garrotting": "garrotte",
    "gars": "gar",
    "garters": "garter",
    "gas-masks": "gas-mask",
    "gasbags": "gasbag",
    "gases": "gas",
    "gashed": "gash",
    "gashes": "gash",
    "gashing": "gash",
    "gasholders": "gasholder",
    "gasifiers": "gasifier",
    "gaskets": "gasket",
    "gaslights": "gaslight",
    "gasmasks": "gasmask",
    "gasmen": "gasman",
    "gasolines": "gasoline",
    "gasometers": "gasometer",
    "gasped": "gasp",
    "gasping": "gasp",
    "gasps": "gasp",
    "gassed": "gas",
    "gasses": "gas",
    "gassier": "gassy",
    "gassiest": "gassy",
    "gassing": "gas",
    "gastrectomies": "gastrectomy",
    "gastrins": "gastrin",
    "gastritides": "gastritis",
    "gastro-enteritides": "gastro-enteritis",
    "gastroenteritides": "gastroenteritis",
    "gastroenterologists": "gastroenterologist",
    "gastropods": "gastropod",
    "gastroscopies": "gastroscopy",
    "gastrostomies": "gastrostomy",
    "gaswells": "gaswell",
    "gate-keepers": "gate-keeper",
    "gateaux": "gateau",
    "gatecrashed": "gatecrash",
    "gatecrashers": "gatecrasher",
    "gatecrashes": "gatecrash",
    "gatecrashing": "gatecrash",
    "gated": "gate",
    "gatehouses": "gatehouse",
    "gatekeepers": "gatekeeper",
    "gateposts": "gatepost",
    "gates": "gate",
    "gateways": "gateway",
    "gathered": "gather",
    "gatherers": "gatherer",
    "gathering": "gather",
    "gatherings": "gathering",
    "gathers": "gather",
    "gating": "gate",
    "gats": "gat",
    "gaucher": "gauche",
    "gauchest": "gauche",
    "gauchos": "gaucho",
    "gaudier": "gaudy",
    "gaudiest": "gaudy",
    "gauged": "gauge",
    "gauges": "gauge",
    "gauging": "gauge",
    "gaunter": "gaunt",
    "gauntest": "gaunt",
    "gauntlets": "gauntlet",
    "gausses": "gauss",
    "gauzes": "gauze",
    "gauzier": "gauzy",
    "gave": "give",
    "gavels": "gavel",
    "gawked": "gawk",
    "gawkier": "gawky",
    "gawkiest": "gawky",
    "gawking": "gawk",
    "gawks": "gawk",
    "gawped": "gawp",
    "gawping": "gawp",
    "gawps": "gawp",
    "gayer": "gay",
    "gayest": "gay",
    "gays": "gay",
    "gazed": "gaze",
    "gazelles": "gazelle",
    "gazers": "gazer",
    "gazes": "gaze",
    "gazetted": "gazette",
    "gazetteers": "gazetteer",
    "gazettes": "gazette",
    "gazetting": "gazette",
    "gazing": "gaze",
    "gazs": "gaz",
    "gazumped": "gazump",
    "gazumping": "gazump",
    "gazumps": "gazump",
    "gearboxes": "gearbox",
    "geared": "gear",
    "gearing": "gear",
    "gears": "gear",
    "gearshifts": "gearshift",
    "geckoes": "gecko",
    "geckos": "gecko",
    "geese": "goose",
    "geezers": "geezer",
    "geishas": "geisha",
    "gelatines": "gelatine",
    "gelatins": "gelatin",
    "gelded": "geld",
    "gelding": "geld",
    "geldings": "gelding",
    "gelds": "geld",
    "gelled": "gell",
    "gelling": "gell",
    "gells": "gell",
    "gels": "gel",
    "gemini": "geminus",
    "geminiviruses": "geminivirus",
    "gemmae": "gemma",
    "gems": "gem",
    "gemstones": "gemstone",
    "gendarmes": "gendarme",
    "gendered": "gender",
    "gendering": "gender",
    "genderings": "gendering",
    "genders": "gender",
    "genealogies": "genealogy",
    "genealogists": "genealogist",
    "genera": "genus",
    "generalisations": "generalisation",
    "generalised": "generalise",
    "generalises": "generalise",
    "generalising": "generalise",
    "generalissimos": "generalissimo",
    "generalists": "generalist",
    "generalities": "generality",
    "generalizations": "generalization",
    "generalized": "generalize",
    "generalizes": "generalize",
    "generalizing": "generalize",
    "generals": "general",
    "generated": "generate",
    "generates": "generate",
    "generating": "generate",
    "generations": "generation",
    "generators": "generator",
    "generics": "generic",
    "generosities": "generosity",
    "genes": "gene",
    "geneses": "genesis",
    "geneticists": "geneticist",
    "genetics": "genetic",
    "genies": "genie",
    "genisteins": "genistein",
    "genitals": "genital",
    "genitives": "genitive",
    "geniuses": "genius",
    "genned": "gen",
    "genning": "gen",
    "genocides": "genocide",
    "genomes": "genome",
    "genotoxicities": "genotoxicity",
    "genotyped": "genotype",
    "genotypes": "genotype",
    "genotyping": "genotype",
    "genotypings": "genotyping",
    "genres": "genre",
    "gens": "gen",
    "gentamicins": "gentamicin",
    "genteeler": "genteel",
    "genteelest": "genteel",
    "gentians": "gentian",
    "gentiles": "gentile",
    "gentlemen": "gentleman",
    "gentler": "gentle",
    "gentlest": "gentle",
    "gentlewomen": "gentlewoman",
    "gentoos": "gentoo",
    "gentrified": "gentrify",
    "gentrifies": "gentrify",
    "gentrifying": "gentrify",
    "gents": "gent",
    "genuflected": "genuflect",
    "genuflecting": "genuflect",
    "genuflects": "genuflect",
    "genuses": "genus",
    "geographers": "geographer",
    "geographies": "geography",
    "geologies": "geology",
    "geologists": "geologist",
    "geometers": "geometer",
    "geometries": "geometry",
    "geomorphologies": "geomorphology",
    "geomorphologists": "geomorphologist",
    "geophysicists": "geophysicist",
    "geosciences": "geoscience",
    "geraniums": "geranium",
    "gerbils": "gerbil",
    "geriatricians": "geriatrician",
    "geriatrics": "geriatric",
    "gerim": "ger",
    "germ-lines": "germ-line",
    "germinated": "germinate",
    "germinates": "germinate",
    "germinating": "germinate",
    "germinations": "germination",
    "germlines": "germline",
    "germs": "germ",
    "gerontologies": "gerontology",
    "gerunds": "gerund",
    "gestalten": "gestalt",
    "gestalts": "gestalt",
    "gestapos": "gestapo",
    "gestated": "gestate",
    "gestates": "gestate",
    "gestating": "gestate",
    "gestations": "gestation",
    "gesticulated": "gesticulate",
    "gesticulates": "gesticulate",
    "gesticulating": "gesticulate",
    "gesticulations": "gesticulation",
    "gestured": "gesture",
    "gestures": "gesture",
    "gesturing": "gesture",
    "get-togethers": "get-together",
    "get-ups": "get-up",
    "getaways": "getaway",
    "gets": "get",
    "getting": "get",
    "gewgaws": "gewgaw",
    "geysers": "geyser",
    "ghastlier": "ghastly",
    "ghastliest": "ghastly",
    "gherkins": "gherkin",
    "ghettoes": "ghetto",
    "ghettoisations": "ghettoisation",
    "ghettoised": "ghettoise",
    "ghettoises": "ghettoise",
    "ghettoising": "ghettoise",
    "ghettos": "ghetto",
    "ghost-writers": "ghost-writer",
    "ghost-writes": "ghost-write",
    "ghost-writing": "ghost-write",
    "ghost-written": "ghost-write",
    "ghost-wrote": "ghost-write",
    "ghosted": "ghost",
    "ghosting": "ghost",
    "ghosts": "ghost",
    "ghostwrites": "ghostwrite",
    "ghostwriting": "ghostwrite",
    "ghostwritten": "ghostwrite",
    "ghostwrote": "ghostwrite",
    "ghouls": "ghoul",
    "giantesses": "giantess",
    "giants": "giant",
    "gibbered": "gibber",
    "gibbering": "gibber",
    "gibbers": "gibber",
    "gibbets": "gibbet",
    "gibbons": "gibbon",
    "gibed": "gibe",
    "gibes": "gibe",
    "gibing": "gibe",
    "giblets": "giblet",
    "giddier": "giddy",
    "giddiest": "giddy",
    "gifted": "gift",
    "gifting": "gift",
    "gifts": "gift",
    "gigabits": "gigabit",
    "gigawatts": "gigawatt",
    "giggled": "giggle",
    "giggles": "giggle",
    "giggling": "giggle",
    "gigolos": "gigolo",
    "gigs": "gig",
    "gilded": "gild",
    "gilding": "gild",
    "gilds": "gild",
    "gills": "gill",
    "gilt": "gild",
    "gilts": "gilt",
    "gimlets": "gimlet",
    "gimmers": "gimmer",
    "gimmicks": "gimmick",
    "gimps": "gimp",
    "gingerbreads": "gingerbread",
    "gingivae": "gingiva",
    "gingivitides": "gingivitis",
    "gingkoes": "gingko",
    "gingkos": "gingko",
    "ginkgoes": "ginkgo",
    "ginkgos": "ginkgo",
    "gins": "gin",
    "ginsengs": "ginseng",
    "gipsies": "gipsy",
    "giraffes": "giraffe",
    "girded": "gird",
    "girders": "girder",
    "girding": "gird",
    "girdled": "girdle",
    "girdles": "girdle",
    "girdling": "girdle",
    "girds": "gird",
    "girlfriends": "girlfriend",
    "girliest": "girly",
    "girls": "girl",
    "giros": "giro",
    "girt": "gird",
    "girths": "girth",
    "gists": "gist",
    "gits": "git",
    "give-aways": "give-away",
    "giveaways": "giveaway",
    "given": "give",
    "givens": "given",
    "givers": "giver",
    "gives": "give",
    "giving": "give",
    "gizzards": "gizzard",
    "gks": "gk",
    "glaciated": "glaciate",
    "glaciates": "glaciate",
    "glaciating": "glaciate",
    "glaciations": "glaciation",
    "glaciers": "glacier",
    "gladdened": "gladden",
    "gladdening": "gladden",
    "gladdens": "gladden",
    "gladder": "glad",
    "gladdest": "glad",
    "glades": "glade",
    "gladiators": "gladiator",
    "gladioli": "gladiolus",
    "gladioluses": "gladiolus",
    "glamorised": "glamorise",
    "glamorises": "glamorise",
    "glamorising": "glamorise",
    "glamorized": "glamorize",
    "glamorizes": "glamorize",
    "glamorizing": "glamorize",
    "glanced": "glance",
    "glances": "glance",
    "glancing": "glance",
    "glandes": "glans",
    "glands": "gland",
    "glared": "glare",
    "glares": "glare",
    "glaring": "glare",
    "glassblowers": "glassblower",
    "glassed": "glass",
    "glasses": "glass",
    "glasshouses": "glasshouse",
    "glassier": "glassy",
    "glassiest": "glassy",
    "glassing": "glass",
    "glassmakers": "glassmaker",
    "glaucomas": "glaucoma",
    "glaucomata": "glaucoma",
    "glazed": "glaze",
    "glazes": "glaze",
    "glaziers": "glazier",
    "glazing": "glaze",
    "glazings": "glazing",
    "gleamed": "gleam",
    "gleaming": "gleam",
    "gleams": "gleam",
    "gleaned": "glean",
    "gleaning": "glean",
    "gleans": "glean",
    "glees": "glee",
    "glens": "glen",
    "glibber": "glib",
    "glibbest": "glib",
    "glided": "glide",
    "gliders": "glider",
    "glides": "glide",
    "gliding": "glide",
    "glimmered": "glimmer",
    "glimmering": "glimmer",
    "glimmerings": "glimmering",
    "glimmers": "glimmer",
    "glimpsed": "glimpse",
    "glimpses": "glimpse",
    "glimpsing": "glimpse",
    "glinted": "glint",
    "glinting": "glint",
    "glints": "glint",
    "glioblastomas": "glioblastoma",
    "glioblastomata": "glioblastoma",
    "gliomas": "glioma",
    "gliomata": "glioma",
    "glissandi": "glissando",
    "glissandos": "glissando",
    "glistened": "glisten",
    "glistening": "glisten",
    "glistens": "glisten",
    "glittered": "glitter",
    "glittering": "glitter",
    "glitters": "glitter",
    "glitzier": "glitzy",
    "glitziest": "glitzy",
    "gloated": "gloat",
    "gloating": "gloat",
    "gloats": "gloat",
    "globalised": "globalise",
    "globalises": "globalise",
    "globalising": "globalise",
    "globalized": "globalize",
    "globalizes": "globalize",
    "globalizing": "globalize",
    "globe-trots": "globe-trot",
    "globe-trotted": "globe-trot",
    "globe-trotting": "globe-trot",
    "globes": "globe",
    "globetrotters": "globetrotter",
    "globi": "globus",
    "globins": "globin",
    "globs": "glob",
    "globules": "globule",
    "globulins": "globulin",
    "glomeruli": "glomerulus",
    "glomerulonephritides": "glomerulonephritis",
    "gloomier": "gloomy",
    "gloomiest": "gloomy",
    "gloried": "glory",
    "glories": "glory",
    "glorified": "glorify",
    "glorifies": "glorify",
    "glorifying": "glorify",
    "glorying": "glory",
    "glossaries": "glossary",
    "glossed": "gloss",
    "glosses": "gloss",
    "glossier": "glossy",
    "glossies": "glossy",
    "glossiest": "glossy",
    "glossing": "gloss",
    "gloved": "glove",
    "gloves": "glove",
    "gloving": "glove",
    "glow-worms": "glow-worm",
    "glowed": "glow",
    "glowered": "glower",
    "glowering": "glower",
    "glowers": "glower",
    "glowing": "glow",
    "glows": "glow",
    "glucagons": "glucagon",
    "glucocerebrosidases": "glucocerebrosidase",
    "glucocorticoids": "glucocorticoid",
    "gluconates": "gluconate",
    "gluconeogeneses": "gluconeogenesis",
    "glucosamines": "glucosamine",
    "glucoses": "glucose",
    "glued": "glue",
    "glueing": "glue",
    "glues": "glue",
    "gluing": "glue",
    "glummer": "glum",
    "glummest": "glum",
    "gluons": "gluon",
    "glutamates": "glutamate",
    "glutamines": "glutamine",
    "glutathiones": "glutathione",
    "glutei": "gluteus",
    "glutens": "gluten",
    "gluts": "glut",
    "glutted": "glut",
    "glutting": "glut",
    "gluttonies": "gluttony",
    "gluttons": "glutton",
    "glycerols": "glycerol",
    "glycines": "glycine",
    "glycogens": "glycogen",
    "glycolipids": "glycolipid",
    "glycols": "glycol",
    "glycolyses": "glycolysis",
    "glycoproteins": "glycoprotein",
    "glycosaminoglycans": "glycosaminoglycan",
    "glycosides": "glycoside",
    "glycosylations": "glycosylation",
    "glyphs": "glyph",
    "gms": "gm",
    "gnarled": "gnarl",
    "gnarling": "gnarl",
    "gnarls": "gnarl",
    "gnashed": "gnash",
    "gnashes": "gnash",
    "gnashing": "gnash",
    "gnats": "gnat",
    "gnawed": "gnaw",
    "gnawing": "gnaw",
    "gnaws": "gnaw",
    "gneisses": "gneiss",
    "gnomes": "gnome",
    "gns": "gn",
    "gnus": "gnu",
    "go-aheads": "go-ahead",
    "go-betweens": "go-between",
    "go-carts": "go-cart",
    "go-getters": "go-getter",
    "go-karts": "go-kart",
    "go-slows": "go-slow",
    "goaded": "goad",
    "goading": "goad",
    "goads": "goad",
    "goal-posts": "goal-post",
    "goalies": "goalie",
    "goalkeepers": "goalkeeper",
    "goalposts": "goalpost",
    "goals": "goal",
    "goatees": "goatee",
    "goatherds": "goatherd",
    "goats": "goat",
    "goatskins": "goatskin",
    "gobbets": "gobbet",
    "gobbled": "gobble",
    "gobblers": "gobbler",
    "gobbles": "gobble",
    "gobbling": "gobble",
    "gobies": "goby",
    "goblets": "goblet",
    "goblins": "goblin",
    "gobos": "gobo",
    "gobs": "gob",
    "godchildren": "godchild",
    "goddaughters": "goddaughter",
    "goddesses": "goddess",
    "godfathers": "godfather",
    "godlier": "godly",
    "godmothers": "godmother",
    "godparents": "godparent",
    "gods": "god",
    "godsends": "godsend",
    "godsons": "godson",
    "godwits": "godwit",
    "goers": "goer",
    "goes": "go",
    "goggled": "goggle",
    "goggles": "goggle",
    "goggling": "goggle",
    "going": "go",
    "goings-over": "going-over",
    "goitres": "goitre",
    "gold-plated": "gold-plate",
    "gold-plates": "gold-plate",
    "gold-plating": "gold-plate",
    "goldcrests": "goldcrest",
    "goldeneyes": "goldeneye",
    "goldfields": "goldfield",
    "goldfinches": "goldfinch",
    "goldfishes": "goldfish",
    "goldmines": "goldmine",
    "golds": "gold",
    "goldsmiths": "goldsmith",
    "golfed": "golf",
    "golfers": "golfer",
    "golfing": "golf",
    "golfs": "golf",
    "gollies": "golly",
    "golliwogs": "golliwog",
    "gonadotrophins": "gonadotrophin",
    "gonadotropins": "gonadotropin",
    "gonads": "gonad",
    "gondolas": "gondola",
    "gondoliers": "gondolier",
    "gone": "go",
    "goners": "goner",
    "gongs": "gong",
    "goniometers": "goniometer",
    "gonorrheae": "gonorrhea",
    "gonorrheas": "gonorrhea",
    "gonorrhoeae": "gonorrhoea",
    "gonorrhoeas": "gonorrhoea",
    "good-byes": "good-bye",
    "good-for-nothings": "good-for-nothing",
    "goodbyes": "goodbye",
    "gooder": "good",
    "goodest": "good",
    "goodies": "goody",
    "goodlier": "goodly",
    "goodliest": "goodly",
    "goodnights": "goodnight",
    "goods": "good",
    "goody-goodies": "goody-goody",
    "goofed": "goof",
    "goofier": "goofy",
    "goofiest": "goofy",
    "goofing": "goof",
    "goofs": "goof",
    "gooier": "gooey",
    "gooiest": "gooey",
    "gooks": "gook",
    "goons": "goon",
    "goosanders": "goosander",
    "goose-stepped": "goose-step",
    "goose-stepping": "goose-step",
    "goose-steps": "goose-step",
    "gooseberries": "gooseberry",
    "goosed": "goose",
    "gooses": "goose",
    "goosing": "goose",
    "gophers": "gopher",
    "gored": "gore",
    "gores": "gore",
    "gorged": "gorge",
    "gorges": "gorge",
    "gorging": "gorge",
    "gorgonians": "gorgonian",
    "gorier": "gory",
    "goriest": "gory",
    "gorillas": "gorilla",
    "goring": "gore",
    "gorn": "go",
    "goshawks": "goshawk",
    "goslings": "gosling",
    "gospels": "gospel",
    "gossiped": "gossip",
    "gossiping": "gossip",
    "gossipped": "gossip",
    "gossipping": "gossip",
    "gossips": "gossip",
    "got": "get",
    "goths": "goth",
    "gotten": "get",
    "gouged": "gouge",
    "gouges": "gouge",
    "gouging": "gouge",
    "goulashes": "goulash",
    "gouramies": "gourami",
    "gouramis": "gourami",
    "gourds": "gourd",
    "gourmands": "gourmand",
    "gourmets": "gourmet",
    "goutier": "gouty",
    "gouts": "gout",
    "governed": "govern",
    "governesses": "governess",
    "governing": "govern",
    "governments": "government",
    "governorates": "governorate",
    "governors": "governor",
    "governorships": "governorship",
    "governs": "govern",
    "govs": "gov",
    "gowned": "gown",
    "gowning": "gown",
    "gowns": "gown",
    "grabbed": "grab",
    "grabbers": "grabber",
    "grabbing": "grab",
    "grabs": "grab",
    "graced": "grace",
    "graces": "grace",
    "gracilides": "gracilis",
    "gracing": "grace",
    "gradations": "gradation",
    "graded": "grade",
    "graders": "grader",
    "grades": "grade",
    "gradients": "gradient",
    "grading": "grade",
    "gradings": "grading",
    "gradiometers": "gradiometer",
    "grads": "grad",
    "graduands": "graduand",
    "graduated": "graduate",
    "graduates": "graduate",
    "graduating": "graduate",
    "graduations": "graduation",
    "graffiti": "graffito",
    "grafted": "graft",
    "grafting": "graft",
    "graftings": "grafting",
    "grafts": "graft",
    "grails": "grail",
    "grained": "grain",
    "grainier": "grainy",
    "graining": "grain",
    "grains": "grain",
    "grammarians": "grammarian",
    "grammars": "grammar",
    "grammes": "gramme",
    "gramophones": "gramophone",
    "grams": "gram",
    "granaries": "granary",
    "grand-daughters": "grand-daughter",
    "grandaddies": "grandaddy",
    "grandads": "grandad",
    "grandchildren": "grandchild",
    "granddads": "granddad",
    "granddaughters": "granddaughter",
    "grandees": "grandee",
    "grander": "grand",
    "grandest": "grand",
    "grandeurs": "grandeur",
    "grandfathered": "grandfather",
    "grandfathering": "grandfather",
    "grandfathers": "grandfather",
    "grandmas": "grandma",
    "grandmothers": "grandmother",
    "grandparents": "grandparent",
    "grandpas": "grandpa",
    "grands": "grand",
    "grandsons": "grandson",
    "grandstands": "grandstand",
    "granges": "grange",
    "grannies": "granny",
    "granolas": "granola",
    "grans": "gran",
    "granted": "grant",
    "grantees": "grantee",
    "granting": "grant",
    "grants": "grant",
    "grants-in-aid": "grant-in-aid",
    "granularities": "granularity",
    "granulated": "granulate",
    "granulates": "granulate",
    "granulating": "granulate",
    "granulations": "granulation",
    "granulators": "granulator",
    "granules": "granule",
    "granulocytes": "granulocyte",
    "granulomas": "granuloma",
    "granulomata": "granuloma",
    "grapefruits": "grapefruit",
    "grapes": "grape",
    "grapevines": "grapevine",
    "graphed": "graph",
    "graphemes": "grapheme",
    "graphics": "graphic",
    "graphing": "graph",
    "graphs": "graph",
    "grapnels": "grapnel",
    "grappled": "grapple",
    "grapples": "grapple",
    "grappling": "grapple",
    "grasped": "grasp",
    "grasping": "grasp",
    "grasps": "grasp",
    "grassed": "grass",
    "grasses": "grass",
    "grasshoppers": "grasshopper",
    "grassier": "grassy",
    "grassiest": "grassy",
    "grassing": "grass",
    "grasslands": "grassland",
    "grated": "grate",
    "graters": "grater",
    "grates": "grate",
    "graticules": "graticule",
    "gratifications": "gratification",
    "gratified": "gratify",
    "gratifies": "gratify",
    "gratifying": "gratify",
    "grating": "grate",
    "gratings": "grating",
    "gratuities": "gratuity",
    "gratuitouser": "gratuitous",
    "gratuitousest": "gratuitous",
    "graved": "grave",
    "gravediggers": "gravedigger",
    "graveled": "gravel",
    "graveling": "gravel",
    "gravelled": "gravel",
    "gravelling": "gravel",
    "gravels": "gravel",
    "graven": "grave",
    "graver": "grave",
    "graves": "grave",
    "gravest": "grave",
    "gravestones": "gravestone",
    "graveyards": "graveyard",
    "gravies": "gravy",
    "graving": "grave",
    "gravitated": "gravitate",
    "gravitates": "gravitate",
    "gravitating": "gravitate",
    "gravities": "gravity",
    "gravures": "gravure",
    "grayed": "gray",
    "grayer": "gray",
    "grayest": "gray",
    "graying": "gray",
    "graylings": "grayling",
    "grays": "gray",
    "grazed": "graze",
    "grazers": "grazer",
    "grazes": "graze",
    "graziers": "grazier",
    "grazing": "graze",
    "greased": "grease",
    "greasers": "greaser",
    "greases": "grease",
    "greasier": "greasy",
    "greasiest": "greasy",
    "greasing": "grease",
    "great-grandfathers": "great-grandfather",
    "great-grandmothers": "great-grandmother",
    "great-uncles": "great-uncle",
    "greatcoats": "greatcoat",
    "greater": "great",
    "greatest": "great",
    "greats": "great",
    "grebes": "grebe",
    "greedier": "greedy",
    "greediest": "greedy",
    "greenbacks": "greenback",
    "greener": "green",
    "greenest": "green",
    "greenfinches": "greenfinch",
    "greenflies": "greenfly",
    "greengages": "greengage",
    "greengrocers": "greengrocer",
    "greenhorns": "greenhorn",
    "greenhouses": "greenhouse",
    "greenings": "greening",
    "greenrooms": "greenroom",
    "greens": "green",
    "greenshanks": "greenshank",
    "greenstones": "greenstone",
    "greeted": "greet",
    "greeting": "greet",
    "greetings": "greeting",
    "greets": "greet",
    "gremlins": "gremlin",
    "grenades": "grenade",
    "grenadiers": "grenadier",
    "grew": "grow",
    "grey-greens": "grey-green",
    "greyed": "grey",
    "greyer": "grey",
    "greyest": "grey",
    "greyhounds": "greyhound",
    "greying": "grey",
    "greylags": "greylag",
    "greys": "grey",
    "greywackes": "greywacke",
    "griddles": "griddle",
    "gridirons": "gridiron",
    "grids": "grid",
    "griefs": "grief",
    "grievances": "grievance",
    "grieved": "grieve",
    "grieves": "grieve",
    "grieving": "grieve",
    "griffins": "griffin",
    "griffons": "griffon",
    "grilled": "grill",
    "grilles": "grille",
    "grilling": "grill",
    "grillrooms": "grillroom",
    "grills": "grill",
    "grilses": "grilse",
    "grimaced": "grimace",
    "grimaces": "grimace",
    "grimacing": "grimace",
    "grimed": "grime",
    "grimes": "grime",
    "grimier": "grimy",
    "grimiest": "grimy",
    "griming": "grime",
    "grimmer": "grim",
    "grimmest": "grim",
    "grinders": "grinder",
    "grinding": "grind",
    "grinds": "grind",
    "grindstones": "grindstone",
    "grinned": "grin",
    "grinning": "grin",
    "grins": "grin",
    "griped": "gripe",
    "gripes": "gripe",
    "griping": "gripe",
    "gripped": "grip",
    "grippers": "gripper",
    "gripping": "grip",
    "grips": "grip",
    "grislier": "grisly",
    "grisliest": "grisly",
    "grits": "grit",
    "gritted": "grit",
    "grittier": "gritty",
    "grittiest": "gritty",
    "gritting": "grit",
    "grizzled": "grizzle",
    "grizzles": "grizzle",
    "grizzlies": "grizzly",
    "grizzling": "grizzle",
    "groaned": "groan",
    "groaning": "groan",
    "groans": "groan",
    "groats": "groat",
    "groceries": "grocery",
    "grocers": "grocer",
    "groggier": "groggy",
    "groggiest": "groggy",
    "groins": "groin",
    "grommets": "grommet",
    "groomed": "groom",
    "groomers": "groomer",
    "grooming": "groom",
    "grooms": "groom",
    "grooved": "groove",
    "grooves": "groove",
    "groovier": "groovy",
    "grooviest": "groovy",
    "grooving": "groove",
    "groped": "grope",
    "gropes": "grope",
    "groping": "grope",
    "grossed": "gross",
    "grosser": "gross",
    "grosses": "gross",
    "grossest": "gross",
    "grossing": "gross",
    "grottier": "grotty",
    "grottiest": "grotty",
    "grottoes": "grotto",
    "grottos": "grotto",
    "grouched": "grouch",
    "grouches": "grouch",
    "grouchier": "grouchy",
    "grouching": "grouch",
    "ground": "grind",
    "groundcloths": "groundcloth",
    "grounded": "ground",
    "grounding": "ground",
    "groundnuts": "groundnut",
    "grounds": "ground",
    "groundsheets": "groundsheet",
    "groundsmen": "groundsman",
    "groundswells": "groundswell",
    "groundwaters": "groundwater",
    "groundworks": "groundwork",
    "grouped": "group",
    "groupers": "grouper",
    "groupies": "groupy",
    "grouping": "group",
    "groupings": "grouping",
    "groups": "group",
    "groused": "grouse",
    "grouses": "grouse",
    "grousing": "grouse",
    "groveled": "grovel",
    "groveling": "grovel",
    "grovelled": "grovel",
    "grovelling": "grovel",
    "grovels": "grovel",
    "groves": "grove",
    "growers": "grower",
    "growing": "grow",
    "growled": "growl",
    "growling": "growl",
    "growls": "growl",
    "grown": "grow",
    "grown-ups": "grown-up",
    "grownups": "grownup",
    "grows": "grow",
    "growths": "growth",
    "grs": "gr",
    "grubbed": "grub",
    "grubbier": "grubby",
    "grubbiest": "grubby",
    "grubbing": "grub",
    "grubs": "grub",
    "grudged": "grudge",
    "grudges": "grudge",
    "grudging": "grudge",
    "gruels": "gruel",
    "gruffer": "gruff",
    "gruffest": "gruff",
    "grumbled": "grumble",
    "grumbles": "grumble",
    "grumbling": "grumble",
    "grumpier": "grumpy",
    "grumpiest": "grumpy",
    "grunted": "grunt",
    "grunting": "grunt",
    "grunts": "grunt",
    "gs": "g",
    "guanacos": "guanaco",
    "guanines": "guanine",
    "guanos": "guano",
    "guaranteed": "guarantee",
    "guaranteeing": "guarantee",
    "guarantees": "guarantee",
    "guaranties": "guaranty",
    "guarantors": "guarantor",
    "guard-rails": "guard-rail",
    "guarded": "guard",
    "guardians": "guardian",
    "guardianships": "guardianship",
    "guarding": "guard",
    "guardrails": "guardrail",
    "guards": "guard",
    "guardsmen": "guardsman",
    "guavas": "guava",
    "gudgeons": "gudgeon",
    "guerillas": "guerilla",
    "guerrillas": "guerrilla",
    "guessed": "guess",
    "guesses": "guess",
    "guessing": "guess",
    "guesstimates": "guesstimate",
    "guest-houses": "guest-house",
    "guest-rooms": "guest-room",
    "guested": "guest",
    "guestimates": "guestimate",
    "guesting": "guest",
    "guests": "guest",
    "guffawed": "guffaw",
    "guffawing": "guffaw",
    "guffaws": "guffaw",
    "guidances": "guidance",
    "guide-books": "guide-book",
    "guide-lines": "guide-line",
    "guidebooks": "guidebook",
    "guided": "guide",
    "guidelines": "guideline",
    "guideposts": "guidepost",
    "guides": "guide",
    "guiding": "guide",
    "guilders": "guilder",
    "guildhalls": "guildhall",
    "guilds": "guild",
    "guillemots": "guillemot",
    "guillotined": "guillotine",
    "guillotines": "guillotine",
    "guillotining": "guillotine",
    "guiltier": "guilty",
    "guiltiest": "guilty",
    "guilts": "guilt",
    "guinea-pigs": "guinea-pig",
    "guineas": "guinea",
    "guises": "guise",
    "guitarists": "guitarist",
    "guitars": "guitar",
    "gulags": "gulag",
    "gulches": "gulch",
    "gulfs": "gulf",
    "gulled": "gull",
    "gullets": "gullet",
    "gullies": "gully",
    "gulling": "gull",
    "gulls": "gull",
    "gulped": "gulp",
    "gulping": "gulp",
    "gulps": "gulp",
    "gumboils": "gumboil",
    "gumboots": "gumboot",
    "gumdrops": "gumdrop",
    "gummed": "gum",
    "gummier": "gummy",
    "gummiest": "gummy",
    "gumming": "gum",
    "gums": "gum",
    "gun-runners": "gun-runner",
    "gunboats": "gunboat",
    "gundogs": "gundog",
    "gunfights": "gunfight",
    "gunfires": "gunfire",
    "gunmen": "gunman",
    "gunned": "gun",
    "gunnels": "gunnel",
    "gunners": "gunner",
    "gunning": "gun",
    "gunpowders": "gunpowder",
    "guns": "gun",
    "gunshots": "gunshot",
    "gunslingers": "gunslinger",
    "gunsmiths": "gunsmith",
    "guppies": "guppy",
    "gurgled": "gurgle",
    "gurgles": "gurgle",
    "gurgling": "gurgle",
    "gurneys": "gurney",
    "gurus": "guru",
    "gushed": "gush",
    "gushes": "gush",
    "gushing": "gush",
    "gussets": "gusset",
    "gusted": "gust",
    "gustier": "gusty",
    "gusting": "gust",
    "gusts": "gust",
    "guts": "gut",
    "gutsier": "gutsy",
    "guttae": "gutta",
    "gutted": "gut",
    "guttered": "gutter",
    "guttering": "gutter",
    "gutters": "gutter",
    "gutting": "gut",
    "guvnors": "guvnor",
    "guyed": "guy",
    "guying": "guy",
    "guys": "guy",
    "guzzled": "guzzle",
    "guzzlers": "guzzler",
    "guzzles": "guzzle",
    "guzzling": "guzzle",
    "gybed": "gybe",
    "gybes": "gybe",
    "gybing": "gybe",
    "gymkhanas": "gymkhana",
    "gymnasia": "gymnasium",
    "gymnasiums": "gymnasium",
    "gymnastics": "gymnastic",
    "gymnasts": "gymnast",
    "gyms": "gym",
    "gymslips": "gymslip",
    "gynaecologies": "gynaecology",
    "gynaecologists": "gynaecologist",
    "gynecologies": "gynecology",
    "gynecologists": "gynecologist",
    "gypsies": "gypsy",
    "gypsums": "gypsum",
    "gyrated": "gyrate",
    "gyrates": "gyrate",
    "gyrating": "gyrate",
    "gyrations": "gyration",
    "gyres": "gyre",
    "gyri": "gyrus",
    "gyroscopes": "gyroscope",
    "haberdasheries": "haberdashery",
    "haberdashers": "haberdasher",
    "habitants": "habitant",
    "habitations": "habitation",
    "habitats": "habitat",
    "habits": "habit",
    "habituated": "habituate",
    "habituates": "habituate",
    "habituating": "habituate",
    "habituations": "habituation",
    "habitues": "habitue",
    "haciendas": "hacienda",
    "hacked": "hack",
    "hackers": "hacker",
    "hacking": "hack",
    "hackles": "hackle",
    "hackneys": "hackney",
    "hacks": "hack",
    "hacksaws": "hacksaw",
    "had": "have",
    "haddocks": "haddock",
    "hadjis": "hadji",
    "hadrons": "hadron",
    "haemagglutinins": "haemagglutinin",
    "haematocrits": "haematocrit",
    "haematologies": "haematology",
    "haematologists": "haematologist",
    "haematomae": "haematoma",
    "haematomas": "haematoma",
    "haematomata": "haematoma",
    "haematoxylins": "haematoxylin",
    "haematurias": "haematuria",
    "haemochromatoses": "haemochromatosis",
    "haemodialyses": "haemodialysis",
    "haemoglobinopathies": "haemoglobinopathy",
    "haemoglobins": "haemoglobin",
    "haemolyses": "haemolysis",
    "haemophiliacs": "haemophiliac",
    "haemophilias": "haemophilia",
    "haemorrhaged": "haemorrhage",
    "haemorrhages": "haemorrhage",
    "haemorrhaging": "haemorrhage",
    "haemorrhoids": "haemorrhoid",
    "haemostases": "haemostasis",
    "haems": "haem",
    "hafts": "haft",
    "haggises": "haggis",
    "haggled": "haggle",
    "haggles": "haggle",
    "haggling": "haggle",
    "hagiographies": "hagiography",
    "hags": "hag",
    "hailed": "hail",
    "hailing": "hail",
    "hails": "hail",
    "hailstones": "hailstone",
    "hailstorms": "hailstorm",
    "hair-driers": "hair-drier",
    "hair-grips": "hair-grip",
    "hairballs": "hairball",
    "hairbrushes": "hairbrush",
    "haircuts": "haircut",
    "hairdos": "hairdo",
    "hairdressers": "hairdresser",
    "hairdryers": "hairdryer",
    "hairier": "hairy",
    "hairiest": "hairy",
    "hairlines": "hairline",
    "hairnets": "hairnet",
    "hairpieces": "hairpiece",
    "hairpins": "hairpin",
    "hairs": "hair",
    "hairsprays": "hairspray",
    "hairstyles": "hairstyle",
    "hajjes": "hajj",
    "half-backs": "half-back",
    "half-brothers": "half-brother",
    "half-castes": "half-caste",
    "half-days": "half-day",
    "half-dozens": "half-dozen",
    "half-hours": "half-hour",
    "half-lifes": "half-life",
    "half-litres": "half-litre",
    "half-lives": "half-life",
    "half-moons": "half-moon",
    "half-notes": "half-note",
    "half-pipes": "half-pipe",
    "half-sisters": "half-sister",
    "half-terms": "half-term",
    "half-times": "half-time",
    "half-wits": "half-wit",
    "halfpence": "halfpenny",
    "halfpennies": "halfpenny",
    "halfpipes": "halfpipe",
    "halfs": "half",
    "halftimes": "halftime",
    "halftones": "halftone",
    "halibuts": "halibut",
    "halides": "halide",
    "halitoses": "halitosis",
    "hallmarked": "hallmark",
    "hallmarking": "hallmark",
    "hallmarks": "hallmark",
    "hallowed": "hallow",
    "hallowing": "hallow",
    "hallows": "hallow",
    "halls": "hall",
    "hallstands": "hallstand",
    "halluces": "hallux",
    "hallucinated": "hallucinate",
    "hallucinates": "hallucinate",
    "hallucinating": "hallucinate",
    "hallucinations": "hallucination",
    "hallucinogens": "hallucinogen",
    "halluxes": "hallux",
    "hallways": "hallway",
    "haloes": "halo",
    "halogenated": "halogenate",
    "halogenates": "halogenate",
    "halogenating": "halogenate",
    "halogens": "halogen",
    "halons": "halon",
    "halos": "halo",
    "halted": "halt",
    "halters": "halter",
    "halting": "halt",
    "halts": "halt",
    "halved": "halve",
    "halves": "halve",
    "halving": "halve",
    "hamburgers": "hamburger",
    "hamlets": "hamlet",
    "hammed": "ham",
    "hammered": "hammer",
    "hammerheads": "hammerhead",
    "hammering": "hammer",
    "hammers": "hammer",
    "hamming": "ham",
    "hammocks": "hammock",
    "hampered": "hamper",
    "hampering": "hamper",
    "hampers": "hamper",
    "hams": "ham",
    "hamsters": "hamster",
    "hamstringing": "hamstring",
    "hamstrings": "hamstring",
    "hamstrung": "hamstring",
    "hand-luggaged": "hand-luggage",
    "hand-luggages": "hand-luggage",
    "hand-luggaging": "hand-luggage",
    "hand-me-downs": "hand-me-down",
    "hand-outs": "hand-out",
    "hand-picked": "hand-pick",
    "hand-picking": "hand-pick",
    "hand-picks": "hand-pick",
    "hand-washings": "hand-washing",
    "hand-writings": "hand-writing",
    "handbags": "handbag",
    "handballs": "handball",
    "handbasins": "handbasin",
    "handbills": "handbill",
    "handbooks": "handbook",
    "handbrakes": "handbrake",
    "handcarts": "handcart",
    "handcrafted": "handcraft",
    "handcrafting": "handcraft",
    "handcrafts": "handcraft",
    "handcuffed": "handcuff",
    "handcuffing": "handcuff",
    "handcuffs": "handcuff",
    "handed": "hand",
    "handednesses": "handedness",
    "handfuls": "handful",
    "handgrips": "handgrip",
    "handguns": "handgun",
    "handhelds": "handheld",
    "handicapped": "handicap",
    "handicappers": "handicapper",
    "handicapping": "handicap",
    "handicaps": "handicap",
    "handicrafts": "handicraft",
    "handier": "handy",
    "handiest": "handy",
    "handing": "hand",
    "handkerchiefs": "handkerchief",
    "handkerchieves": "handkerchief",
    "handlebars": "handlebar",
    "handled": "handle",
    "handlers": "handler",
    "handles": "handle",
    "handling": "handle",
    "handlings": "handling",
    "handmaidens": "handmaiden",
    "handouts": "handout",
    "handovers": "handover",
    "handpicked": "handpick",
    "handpicking": "handpick",
    "handpicks": "handpick",
    "handprints": "handprint",
    "handrails": "handrail",
    "hands": "hand",
    "handsets": "handset",
    "handsful": "handful",
    "handshakes": "handshake",
    "handsomer": "handsome",
    "handsomest": "handsome",
    "handstands": "handstand",
    "handwashings": "handwashing",
    "handwritings": "handwriting",
    "handymen": "handyman",
    "hang-gliders": "hang-glider",
    "hang-glidings": "hang-gliding",
    "hang-ups": "hang-up",
    "hangars": "hangar",
    "hanged": "hang",
    "hanger-ons": "hanger-on",
    "hangers": "hanger",
    "hangers-on": "hanger-on",
    "hanging": "hang",
    "hangings": "hanging",
    "hangmen": "hangman",
    "hangouts": "hangout",
    "hangovers": "hangover",
    "hangs": "hang",
    "hankered": "hanker",
    "hankering": "hanker",
    "hankerings": "hankering",
    "hankers": "hanker",
    "hankies": "hanky",
    "hanks": "hank",
    "hansoms": "hansom",
    "haplotypes": "haplotype",
    "happed": "hap",
    "happened": "happen",
    "happening": "happen",
    "happenings": "happening",
    "happens": "happen",
    "happenstances": "happenstance",
    "happier": "happy",
    "happiest": "happy",
    "happing": "hap",
    "haps": "hap",
    "harangued": "harangue",
    "harangues": "harangue",
    "haranguing": "harangue",
    "harassed": "harass",
    "harassers": "harasser",
    "harasses": "harass",
    "harassing": "harass",
    "harassments": "harassment",
    "harbingers": "harbinger",
    "harbored": "harbor",
    "harboring": "harbor",
    "harbors": "harbor",
    "harboured": "harbour",
    "harbouring": "harbour",
    "harbours": "harbour",
    "hard-copies": "hard-copy",
    "hardbacks": "hardback",
    "hardboards": "hardboard",
    "hardcopies": "hardcopy",
    "hardened": "harden",
    "hardeners": "hardener",
    "hardening": "harden",
    "hardens": "harden",
    "harder": "hard",
    "hardest": "hard",
    "hardier": "hardy",
    "hardiest": "hardy",
    "hardliners": "hardliner",
    "hardmetals": "hardmetal",
    "hardnesses": "hardness",
    "hardships": "hardship",
    "hardwoods": "hardwood",
    "harebells": "harebell",
    "hared": "hare",
    "harelips": "harelip",
    "harems": "harem",
    "hares": "hare",
    "haricots": "haricot",
    "haring": "hare",
    "harked": "hark",
    "harkened": "harken",
    "harkening": "harken",
    "harkens": "harken",
    "harking": "hark",
    "harks": "hark",
    "harlequins": "harlequin",
    "harlots": "harlot",
    "harmans": "harman",
    "harmed": "harm",
    "harming": "harm",
    "harmonicas": "harmonica",
    "harmonics": "harmonic",
    "harmonies": "harmony",
    "harmonised": "harmonise",
    "harmonises": "harmonise",
    "harmonising": "harmonise",
    "harmoniums": "harmonium",
    "harmonized": "harmonize",
    "harmonizes": "harmonize",
    "harmonizing": "harmonize",
    "harms": "harm",
    "harnessed": "harness",
    "harnesses": "harness",
    "harnessing": "harness",
    "harped": "harp",
    "harpies": "harpy",
    "harping": "harp",
    "harpists": "harpist",
    "harpooned": "harpoon",
    "harpooning": "harpoon",
    "harpoons": "harpoon",
    "harps": "harp",
    "harpsichords": "harpsichord",
    "harried": "harry",
    "harriers": "harrier",
    "harries": "harry",
    "harrowed": "harrow",
    "harrowing": "harrow",
    "harrows": "harrow",
    "harrying": "harry",
    "harsher": "harsh",
    "harshest": "harsh",
    "harts": "hart",
    "harvested": "harvest",
    "harvesters": "harvester",
    "harvesting": "harvest",
    "harvestings": "harvesting",
    "harvestmen": "harvestman",
    "harvests": "harvest",
    "has": "have",
    "has-beens": "has-been",
    "hashed": "hash",
    "hashes": "hash",
    "hashing": "hash",
    "hasps": "hasp",
    "hassled": "hassle",
    "hassles": "hassle",
    "hassling": "hassle",
    "hassocks": "hassock",
    "hastened": "hasten",
    "hastening": "hasten",
    "hastens": "hasten",
    "hastier": "hasty",
    "hastiest": "hasty",
    "hatbands": "hatband",
    "hatboxes": "hatbox",
    "hatchbacks": "hatchback",
    "hatched": "hatch",
    "hatcheries": "hatchery",
    "hatches": "hatch",
    "hatchets": "hatchet",
    "hatching": "hatch",
    "hatchlings": "hatchling",
    "hatchways": "hatchway",
    "hated": "hate",
    "hates": "hate",
    "hating": "hate",
    "hatpins": "hatpin",
    "hatreds": "hatred",
    "hats": "hat",
    "hatstands": "hatstand",
    "hatters": "hatter",
    "haughtier": "haughty",
    "haughtiest": "haughty",
    "hauled": "haul",
    "haulers": "hauler",
    "hauliers": "haulier",
    "hauling": "haul",
    "haulms": "haulm",
    "hauls": "haul",
    "haunches": "haunch",
    "haunted": "haunt",
    "haunting": "haunt",
    "haunts": "haunt",
    "havens": "haven",
    "haversacks": "haversack",
    "having": "have",
    "havocs": "havoc",
    "hawed": "haw",
    "hawing": "haw",
    "hawked": "hawk",
    "hawkers": "hawker",
    "hawking": "hawk",
    "hawkmoths": "hawkmoth",
    "hawks": "hawk",
    "haws": "haw",
    "hawsers": "hawser",
    "hawthorns": "hawthorn",
    "haycocks": "haycock",
    "haystacks": "haystack",
    "hazarded": "hazard",
    "hazarding": "hazard",
    "hazards": "hazard",
    "hazed": "haze",
    "hazelnuts": "hazelnut",
    "hazels": "hazel",
    "hazes": "haze",
    "hazier": "hazy",
    "haziest": "hazy",
    "hazing": "haze",
    "he-bears": "he-bear",
    "he-men": "he-man",
    "head-hunters": "head-hunter",
    "head-rested": "head-rest",
    "head-resting": "head-rest",
    "head-rests": "head-rest",
    "head-shrinkers": "head-shrinker",
    "headaches": "headache",
    "headbands": "headband",
    "headbangers": "headbanger",
    "headboards": "headboard",
    "headdresses": "headdress",
    "headed": "head",
    "headers": "header",
    "headgears": "headgear",
    "headhunted": "headhunt",
    "headhunters": "headhunter",
    "headhunting": "headhunt",
    "headhunts": "headhunt",
    "headier": "heady",
    "headiest": "heady",
    "heading": "head",
    "headings": "heading",
    "headlamps": "headlamp",
    "headlands": "headland",
    "headlice": "headlouse",
    "headlights": "headlight",
    "headlined": "headline",
    "headlines": "headline",
    "headlining": "headline",
    "headmasters": "headmaster",
    "headmen": "headman",
    "headmistresses": "headmistress",
    "headphones": "headphone",
    "headpieces": "headpiece",
    "headquartered": "headquarter",
    "headquartering": "headquarter",
    "headquarters": "headquarter",
    "headrests": "headrest",
    "headrooms": "headroom",
    "heads": "head",
    "headscarfs": "headscarf",
    "headscarves": "headscarf",
    "headsets": "headset",
    "headships": "headship",
    "headspaces": "headspace",
    "headstands": "headstand",
    "headstones": "headstone",
    "headwalls": "headwall",
    "headwaters": "headwater",
    "headwinds": "headwind",
    "headwords": "headword",
    "healed": "heal",
    "healers": "healer",
    "healing": "heal",
    "healings": "healing",
    "heals": "heal",
    "healthier": "healthy",
    "healthiest": "healthy",
    "heaped": "heap",
    "heaping": "heap",
    "heaps": "heap",
    "heard": "hear",
    "hearers": "hearer",
    "hearing": "hear",
    "hearing-aids": "hearing-aid",
    "hearings": "hearing",
    "hearkened": "hearken",
    "hearkening": "hearken",
    "hearkens": "hearken",
    "hears": "hear",
    "hearses": "hearse",
    "heart-rates": "heart-rate",
    "heart-to-hearts": "heart-to-heart",
    "heartaches": "heartache",
    "heartbeats": "heartbeat",
    "heartbreaks": "heartbreak",
    "heartened": "hearten",
    "heartening": "hearten",
    "heartens": "hearten",
    "hearthrugs": "hearthrug",
    "hearths": "hearth",
    "heartier": "hearty",
    "heartiest": "hearty",
    "heartlands": "heartland",
    "hearts": "heart",
    "heartthrobs": "heartthrob",
    "heat-treated": "heat-treat",
    "heat-treating": "heat-treat",
    "heat-treats": "heat-treat",
    "heated": "heat",
    "heaters": "heater",
    "heathens": "heathen",
    "heaths": "heath",
    "heating": "heat",
    "heatings": "heating",
    "heats": "heat",
    "heatstrokes": "heatstroke",
    "heatwaves": "heatwave",
    "heaved": "heave",
    "heavens": "heaven",
    "heaves": "heave",
    "heavier": "heavy",
    "heavies": "heavy",
    "heaviest": "heavy",
    "heaving": "heave",
    "heavyweights": "heavyweight",
    "heckled": "heckle",
    "hecklers": "heckler",
    "heckles": "heckle",
    "heckling": "heckle",
    "hectares": "hectare",
    "hectometres": "hectometre",
    "hectored": "hector",
    "hectoring": "hector",
    "hectors": "hector",
    "hedged": "hedge",
    "hedgehogs": "hedgehog",
    "hedgerows": "hedgerow",
    "hedges": "hedge",
    "hedging": "hedge",
    "hedonists": "hedonist",
    "hee-haws": "hee-haw",
    "heeded": "heed",
    "heeding": "heed",
    "heeds": "heed",
    "heeled": "heel",
    "heeling": "heel",
    "heels": "heel",
    "hefted": "heft",
    "heftier": "hefty",
    "heftiest": "hefty",
    "hefting": "heft",
    "hefts": "heft",
    "hegemonies": "hegemony",
    "heifers": "heifer",
    "heightened": "heighten",
    "heightening": "heighten",
    "heightens": "heighten",
    "heights": "height",
    "heiresses": "heiress",
    "heirlooms": "heirloom",
    "heirs": "heir",
    "held": "hold",
    "helicases": "helicase",
    "helices": "helix",
    "helicities": "helicity",
    "helicopters": "helicopter",
    "heliports": "heliport",
    "helixes": "helix",
    "hellenised": "hellenise",
    "hellenises": "hellenise",
    "hellenising": "hellenise",
    "hellenized": "hellenize",
    "hellenizes": "hellenize",
    "hellenizing": "hellenize",
    "hellos": "hello",
    "hells": "hell",
    "helmed": "helm",
    "helmets": "helmet",
    "helming": "helm",
    "helminths": "helminth",
    "helms": "helm",
    "helmsmen": "helmsman",
    "help-lines": "help-line",
    "helped": "help",
    "helpers": "helper",
    "helping": "help",
    "helpings": "helping",
    "helplines": "helpline",
    "helpmates": "helpmate",
    "helps": "help",
    "helter-skelters": "helter-skelter",
    "hemes": "heme",
    "hemiplegias": "hemiplegia",
    "hemispheres": "hemisphere",
    "hemlines": "hemline",
    "hemlocks": "hemlock",
    "hemmed": "hem",
    "hemming": "hem",
    "hemoglobins": "hemoglobin",
    "hemophilias": "hemophilia",
    "hemorrhages": "hemorrhage",
    "hemorrhoids": "hemorrhoid",
    "hems": "hem",
    "henchmen": "henchman",
    "hennaed": "henna",
    "hennaing": "henna",
    "hennas": "henna",
    "henries": "henry",
    "hens": "hen",
    "heparins": "heparin",
    "hepaticas": "hepatica",
    "hepatics": "hepatic",
    "hepatitides": "hepatitis",
    "hepatitises": "hepatitis",
    "hepatocytes": "hepatocyte",
    "hepatologies": "hepatology",
    "hepatotoxicities": "hepatotoxicity",
    "heptagons": "heptagon",
    "heptameters": "heptameter",
    "heralded": "herald",
    "heralding": "herald",
    "heralds": "herald",
    "herbalists": "herbalist",
    "herbals": "herbal",
    "herbaria": "herbarium",
    "herbicides": "herbicide",
    "herbivores": "herbivore",
    "herbs": "herb",
    "herded": "herd",
    "herders": "herder",
    "herding": "herd",
    "herds": "herd",
    "herdsmen": "herdsman",
    "hereafters": "hereafter",
    "heresies": "heresy",
    "heretics": "heretic",
    "heritabilities": "heritability",
    "heritages": "heritage",
    "hermaphrodites": "hermaphrodite",
    "hermits": "hermit",
    "herniae": "hernia",
    "hernias": "hernia",
    "herniations": "herniation",
    "heroes": "hero",
    "heroics": "heroic",
    "heroines": "heroine",
    "heroins": "heroin",
    "heroisms": "heroism",
    "heronries": "heronry",
    "herons": "heron",
    "herpesviruses": "herpesvirus",
    "herrings": "herring",
    "hesitancies": "hesitancy",
    "hesitated": "hesitate",
    "hesitates": "hesitate",
    "hesitating": "hesitate",
    "hesitations": "hesitation",
    "heteroatoms": "heteroatom",
    "heterocycles": "heterocycle",
    "heterodimers": "heterodimer",
    "heterogeneities": "heterogeneity",
    "heteros": "hetero",
    "heterosexuals": "heterosexual",
    "heterostructures": "heterostructure",
    "heterozygosities": "heterozygosity",
    "heterozygotes": "heterozygote",
    "heuristics": "heuristic",
    "hewed": "hew",
    "hewers": "hewer",
    "hewing": "hew",
    "hewn": "hew",
    "hews": "hew",
    "hexadecimals": "hexadecimal",
    "hexafluorides": "hexafluoride",
    "hexagons": "hexagon",
    "hexagrams": "hexagram",
    "hexamers": "hexamer",
    "hexameters": "hexameter",
    "hexanes": "hexane",
    "hexed": "hex",
    "hexes": "hex",
    "hexing": "hex",
    "heydays": "heyday",
    "hhs": "hh",
    "hi": "hello",
    "hi-fis": "hi-fi",
    "hiatuses": "hiatus",
    "hibernacula": "hibernaculum",
    "hibernated": "hibernate",
    "hibernates": "hibernate",
    "hibernating": "hibernate",
    "hibernations": "hibernation",
    "hibiscuses": "hibiscus",
    "hiccoughed": "hiccough",
    "hiccoughing": "hiccough",
    "hiccoughs": "hiccough",
    "hiccupped": "hiccup",
    "hiccupping": "hiccup",
    "hiccups": "hiccup",
    "hickories": "hickory",
    "hicks": "hick",
    "hid": "hide",
    "hidden": "hide",
    "hide-outs": "hide-out",
    "hideaways": "hideaway",
    "hideouts": "hideout",
    "hides": "hide",
    "hiding": "hide",
    "hiding-places": "hiding-place",
    "hidings": "hiding",
    "hied": "hie",
    "hieing": "hie",
    "hierarchies": "hierarchy",
    "hieroglyphics": "hieroglyphic",
    "hieroglyphs": "hieroglyph",
    "hies": "hie",
    "high-fliers": "high-flier",
    "high-flyers": "high-flyer",
    "high-frequencies": "high-frequency",
    "high-ups": "high-up",
    "highballs": "highball",
    "highbrows": "highbrow",
    "highchairs": "highchair",
    "higher": "high",
    "higher-quality": "high-quality",
    "highest": "high",
    "highest-quality": "high-quality",
    "highjacked": "highjack",
    "highjacking": "highjack",
    "highjacks": "highjack",
    "highlanders": "highlander",
    "highlands": "highland",
    "highlighted": "highlight",
    "highlighting": "highlight",
    "highlights": "highlight",
    "highroads": "highroad",
    "highs": "high",
    "highschools": "highschool",
    "highwaymen": "highwayman",
    "highways": "highway",
    "hijacked": "hijack",
    "hijackers": "hijacker",
    "hijacking": "hijack",
    "hijackings": "hijacking",
    "hijacks": "hijack",
    "hiked": "hike",
    "hikers": "hiker",
    "hikes": "hike",
    "hiking": "hike",
    "hillbillies": "hillbilly",
    "hillier": "hilly",
    "hilliest": "hilly",
    "hillocks": "hillock",
    "hills": "hill",
    "hillsides": "hillside",
    "hilltops": "hilltop",
    "hilts": "hilt",
    "hinderances": "hinderance",
    "hindered": "hinder",
    "hindering": "hinder",
    "hinders": "hinder",
    "hindfeet": "hindfoot",
    "hindquarters": "hindquarter",
    "hindrances": "hindrance",
    "hinds": "hind",
    "hindwings": "hindwing",
    "hinged": "hinge",
    "hinges": "hinge",
    "hinging": "hinge",
    "hinked": "hink",
    "hinking": "hink",
    "hinks": "hink",
    "hinted": "hint",
    "hinterlands": "hinterland",
    "hinting": "hint",
    "hints": "hint",
    "hip-baths": "hip-bath",
    "hippies": "hippy",
    "hippocampi": "hippocampus",
    "hippopotami": "hippopotamus",
    "hippopotamuses": "hippopotamus",
    "hippopotamusses": "hippopotamus",
    "hippos": "hippo",
    "hips": "hip",
    "hired": "hire",
    "hirelings": "hireling",
    "hirers": "hirer",
    "hires": "hire",
    "hiring": "hire",
    "hirsutisms": "hirsutism",
    "hirundines": "hirundine",
    "hissed": "hiss",
    "hisses": "hiss",
    "hissing": "hiss",
    "histamines": "histamine",
    "histidines": "histidine",
    "histocompatibilities": "histocompatibility",
    "histograms": "histogram",
    "histologies": "histology",
    "histones": "histone",
    "histopathologies": "histopathology",
    "histopathologists": "histopathologist",
    "historians": "historian",
    "histories": "history",
    "historiographies": "historiography",
    "histrionics": "histrionic",
    "hists": "hist",
    "hitch-hiked": "hitch-hike",
    "hitch-hikers": "hitch-hiker",
    "hitch-hikes": "hitch-hike",
    "hitch-hiking": "hitch-hike",
    "hitched": "hitch",
    "hitches": "hitch",
    "hitchhiked": "hitchhike",
    "hitchhikers": "hitchhiker",
    "hitchhikes": "hitchhike",
    "hitchhiking": "hitchhike",
    "hitching": "hitch",
    "hits": "hit",
    "hitters": "hitter",
    "hitting": "hit",
    "hived": "hive",
    "hives": "hive",
    "hiving": "hive",
    "hls": "hl",
    "hoarded": "hoard",
    "hoarders": "hoarder",
    "hoarding": "hoard",
    "hoardings": "hoarding",
    "hoards": "hoard",
    "hoarier": "hoary",
    "hoariest": "hoary",
    "hoarsenesses": "hoarseness",
    "hoarser": "hoarse",
    "hoarsest": "hoarse",
    "hoaxed": "hoax",
    "hoaxes": "hoax",
    "hoaxing": "hoax",
    "hobbies": "hobby",
    "hobbled": "hobble",
    "hobbles": "hobble",
    "hobbling": "hobble",
    "hobby-horses": "hobby-horse",
    "hobbyists": "hobbyist",
    "hobgoblins": "hobgoblin",
    "hobnobbed": "hobnob",
    "hobnobbing": "hobnob",
    "hobnobs": "hobnob",
    "hoboes": "hobo",
    "hobos": "hobo",
    "hobs": "hob",
    "hockey-fields": "hockey-field",
    "hockey-sticks": "hockey-stick",
    "hocks": "hock",
    "hodgepodges": "hodgepodge",
    "hods": "hod",
    "hoed": "hoe",
    "hoeing": "hoe",
    "hoes": "hoe",
    "hogged": "hog",
    "hogging": "hog",
    "hogs": "hog",
    "hoing": "hoe",
    "hoisted": "hoist",
    "hoisting": "hoist",
    "hoists": "hoist",
    "hold-ups": "hold-up",
    "holdalls": "holdall",
    "holders": "holder",
    "holdfasts": "holdfast",
    "holding": "hold",
    "holdings": "holding",
    "holdouts": "holdout",
    "holdovers": "holdover",
    "holds": "hold",
    "holdups": "holdup",
    "holed": "hole",
    "holes": "hole",
    "holiday-makers": "holiday-maker",
    "holidayed": "holiday",
    "holidaying": "holiday",
    "holidaymakers": "holidaymaker",
    "holidays": "holiday",
    "holier": "holy",
    "holiest": "holy",
    "holing": "hole",
    "hollered": "holler",
    "hollering": "holler",
    "hollers": "holler",
    "hollies": "holly",
    "hollowed": "hollow",
    "hollower": "hollow",
    "hollowest": "hollow",
    "hollowing": "hollow",
    "hollows": "hollow",
    "hollyhocks": "hollyhock",
    "holms": "holm",
    "holocausts": "holocaust",
    "holograms": "hologram",
    "holographies": "holography",
    "holographs": "holograph",
    "hols": "hol",
    "holsters": "holster",
    "homages": "homage",
    "homebuyers": "homebuyer",
    "homecomings": "homecoming",
    "homed": "home",
    "homelands": "homeland",
    "homelier": "homely",
    "homeliest": "homely",
    "homemakers": "homemaker",
    "homeopaths": "homeopath",
    "homeostases": "homeostasis",
    "homeowners": "homeowner",
    "homepages": "homepage",
    "homers": "homer",
    "homes": "home",
    "homesteads": "homestead",
    "hometowns": "hometown",
    "homeworks": "homework",
    "homicides": "homicide",
    "homilies": "homily",
    "homing": "home",
    "hominids": "hominid",
    "hommes": "homme",
    "homocysteines": "homocysteine",
    "homoeopaths": "homoeopath",
    "homogenates": "homogenate",
    "homogeneities": "homogeneity",
    "homogenisations": "homogenisation",
    "homogenised": "homogenise",
    "homogenises": "homogenise",
    "homogenising": "homogenise",
    "homogenizations": "homogenization",
    "homogenized": "homogenize",
    "homogenizes": "homogenize",
    "homogenizing": "homogenize",
    "homographs": "homograph",
    "homologated": "homologate",
    "homologates": "homologate",
    "homologating": "homologate",
    "homologies": "homology",
    "homologs": "homolog",
    "homologues": "homologue",
    "homomorphisms": "homomorphism",
    "homonyms": "homonym",
    "homophobes": "homophobe",
    "homophones": "homophone",
    "homos": "homo",
    "homosexualities": "homosexuality",
    "homosexuals": "homosexual",
    "homozygotes": "homozygote",
    "homunculi": "homunculus",
    "honed": "hone",
    "hones": "hone",
    "honeybees": "honeybee",
    "honeycombed": "honeycomb",
    "honeycombing": "honeycomb",
    "honeycombs": "honeycomb",
    "honeymooned": "honeymoon",
    "honeymooners": "honeymooner",
    "honeymooning": "honeymoon",
    "honeymoons": "honeymoon",
    "honeys": "honey",
    "honeysuckles": "honeysuckle",
    "hongs": "hong",
    "honing": "hone",
    "honked": "honk",
    "honking": "honk",
    "honks": "honk",
    "honoraria": "honorarium",
    "honorariums": "honorarium",
    "honored": "honor",
    "honorifics": "honorific",
    "honoring": "honor",
    "honors": "honor",
    "honourables": "honourable",
    "honoured": "honour",
    "honouring": "honour",
    "honours": "honour",
    "hoodlums": "hoodlum",
    "hoods": "hood",
    "hoodwinked": "hoodwink",
    "hoodwinking": "hoodwink",
    "hoodwinks": "hoodwink",
    "hoofs": "hoof",
    "hook-ups": "hook-up",
    "hookahs": "hookah",
    "hooked": "hook",
    "hookers": "hooker",
    "hooking": "hook",
    "hooks": "hook",
    "hookups": "hookup",
    "hookworms": "hookworm",
    "hooligans": "hooligan",
    "hoop-las": "hoop-la",
    "hooped": "hoop",
    "hooping": "hoop",
    "hoopoes": "hoopoe",
    "hoops": "hoop",
    "hooted": "hoot",
    "hooters": "hooter",
    "hooting": "hoot",
    "hoots": "hoot",
    "hoovered": "hoover",
    "hoovering": "hoover",
    "hoovers": "hoover",
    "hooves": "hoof",
    "hoped": "hope",
    "hopefuls": "hopeful",
    "hopes": "hope",
    "hoping": "hope",
    "hopped": "hop",
    "hoppers": "hopper",
    "hopping": "hop",
    "hops": "hop",
    "hordes": "horde",
    "horizons": "horizon",
    "horizontals": "horizontal",
    "hormones": "hormone",
    "hornbeams": "hornbeam",
    "hornbills": "hornbill",
    "horned": "horn",
    "hornets": "hornet",
    "hornier": "horny",
    "horniest": "horny",
    "horning": "horn",
    "horns": "horn",
    "horoscopes": "horoscope",
    "horrified": "horrify",
    "horrifies": "horrify",
    "horrifying": "horrify",
    "horrors": "horror",
    "horse-racings": "horse-racing",
    "horse-shoes": "horse-shoe",
    "horsed": "horse",
    "horseflies": "horsefly",
    "horsemen": "horseman",
    "horsepowers": "horsepower",
    "horses": "horse",
    "horseshoes": "horseshoe",
    "horsetails": "horsetail",
    "horsewhipped": "horsewhip",
    "horsewhipping": "horsewhip",
    "horsewhips": "horsewhip",
    "horsewomen": "horsewoman",
    "horsier": "horsy",
    "horsiest": "horsy",
    "horsing": "horse",
    "horticulturalists": "horticulturalist",
    "horticulturists": "horticulturist",
    "hosed": "hose",
    "hoses": "hose",
    "hosing": "hose",
    "hospices": "hospice",
    "hospitalisations": "hospitalisation",
    "hospitalised": "hospitalise",
    "hospitalises": "hospitalise",
    "hospitalising": "hospitalise",
    "hospitalities": "hospitality",
    "hospitalizations": "hospitalization",
    "hospitalized": "hospitalize",
    "hospitalizes": "hospitalize",
    "hospitalizing": "hospitalize",
    "hospitals": "hospital",
    "hostages": "hostage",
    "hostas": "hosta",
    "hosted": "host",
    "hostelries": "hostelry",
    "hostels": "hostel",
    "hostesses": "hostess",
    "hostilities": "hostility",
    "hosting": "host",
    "hosts": "host",
    "hot-houses": "hot-house",
    "hot-plates": "hot-plate",
    "hot-spots": "hot-spot",
    "hotbeds": "hotbed",
    "hotdogged": "hotdog",
    "hotdogging": "hotdog",
    "hotdogs": "hotdog",
    "hoteliers": "hotelier",
    "hotels": "hotel",
    "hotheads": "hothead",
    "hothouses": "hothouse",
    "hotlines": "hotline",
    "hotplates": "hotplate",
    "hotpots": "hotpot",
    "hots": "hot",
    "hotspots": "hotspot",
    "hotted": "hot",
    "hotter": "hot",
    "hottest": "hot",
    "hotting": "hot",
    "hounded": "hound",
    "hounding": "hound",
    "hounds": "hound",
    "hourglasses": "hourglass",
    "hours": "hour",
    "house-fronts": "house-front",
    "house-owners": "house-owner",
    "house-parties": "house-party",
    "house-warmings": "house-warming",
    "houseboats": "houseboat",
    "houseboys": "houseboy",
    "housebreakers": "housebreaker",
    "housebreakings": "housebreaking",
    "housebuilders": "housebuilder",
    "housecoats": "housecoat",
    "housed": "house",
    "housefathers": "housefather",
    "houseflies": "housefly",
    "housefuls": "houseful",
    "householders": "householder",
    "households": "household",
    "housekeepers": "housekeeper",
    "housekeepings": "housekeeping",
    "housemaids": "housemaid",
    "housemasters": "housemaster",
    "housemates": "housemate",
    "housemen": "houseman",
    "housemistresses": "housemistress",
    "housemothers": "housemother",
    "houseplants": "houseplant",
    "houses": "house",
    "housetops": "housetop",
    "housetrained": "housetrain",
    "housetraining": "housetrain",
    "housetrains": "housetrain",
    "housewives": "housewife",
    "houseworks": "housework",
    "housing": "house",
    "housings": "housing",
    "hove": "heave",
    "hovels": "hovel",
    "hovercrafts": "hovercraft",
    "hovered": "hover",
    "hoverflies": "hoverfly",
    "hovering": "hover",
    "hovers": "hover",
    "howdahs": "howdah",
    "howitzers": "howitzer",
    "howled": "howl",
    "howlers": "howler",
    "howling": "howl",
    "howls": "howl",
    "hows": "how",
    "hrs": "hr",
    "hs": "h",
    "hubbies": "hubby",
    "hubcaps": "hubcap",
    "hubs": "hub",
    "hucksters": "huckster",
    "huddled": "huddle",
    "huddles": "huddle",
    "huddling": "huddle",
    "huds": "hud",
    "hues": "hue",
    "huffed": "huff",
    "huffier": "huffy",
    "huffiest": "huffy",
    "huffing": "huff",
    "huffs": "huff",
    "huger": "huge",
    "hugest": "huge",
    "hugged": "hug",
    "huggers": "hugger",
    "hugging": "hug",
    "hugs": "hug",
    "hulas": "hula",
    "hulks": "hulk",
    "hullabaloos": "hullabaloo",
    "hulled": "hull",
    "hulling": "hull",
    "hulls": "hull",
    "humaner": "humane",
    "humanest": "humane",
    "humanised": "humanise",
    "humanises": "humanise",
    "humanising": "humanise",
    "humanists": "humanist",
    "humanitarians": "humanitarian",
    "humanities": "humanity",
    "humanized": "humanize",
    "humanizes": "humanize",
    "humanizing": "humanize",
    "humanoids": "humanoid",
    "humans": "human",
    "humbled": "humble",
    "humbler": "humble",
    "humbles": "humble",
    "humblest": "humble",
    "humbling": "humble",
    "humbugs": "humbug",
    "humdingers": "humdinger",
    "humeri": "humerus",
    "humeruses": "humerus",
    "humidified": "humidify",
    "humidifiers": "humidifier",
    "humidifies": "humidify",
    "humidifying": "humidify",
    "humidities": "humidity",
    "humiliated": "humiliate",
    "humiliates": "humiliate",
    "humiliating": "humiliate",
    "humiliations": "humiliation",
    "hummed": "hum",
    "hummers": "hummer",
    "humming": "hum",
    "hummingbirds": "hummingbird",
    "hummocks": "hummock",
    "humored": "humor",
    "humoring": "humor",
    "humorists": "humorist",
    "humors": "humor",
    "humoured": "humour",
    "humouring": "humour",
    "humourists": "humourist",
    "humours": "humour",
    "humpbacks": "humpback",
    "humped": "hump",
    "humping": "hump",
    "humps": "hump",
    "hums": "hum",
    "hunchbacks": "hunchback",
    "hunched": "hunch",
    "hunches": "hunch",
    "hunching": "hunch",
    "hundreds": "hundred",
    "hundredth": "100",
    "hundredths": "hundredth",
    "hundredweights": "hundredweight",
    "hung": "hang",
    "hungered": "hunger",
    "hungering": "hunger",
    "hungers": "hunger",
    "hungrier": "hungry",
    "hungriest": "hungry",
    "hunkered": "hunker",
    "hunkering": "hunker",
    "hunkers": "hunker",
    "hunks": "hunk",
    "huns": "hun",
    "hunted": "hunt",
    "hunter-gatherers": "hunter-gatherer",
    "hunters": "hunter",
    "hunting": "hunt",
    "hunts": "hunt",
    "huntsmen": "huntsman",
    "hurdled": "hurdle",
    "hurdlers": "hurdler",
    "hurdles": "hurdle",
    "hurdling": "hurdle",
    "hurled": "hurl",
    "hurling": "hurl",
    "hurls": "hurl",
    "hurly-burlies": "hurly-burly",
    "hurrahs": "hurrah",
    "hurrays": "hurray",
    "hurricanes": "hurricane",
    "hurried": "hurry",
    "hurriedly-whispered": "hurriedly-whisper",
    "hurries": "hurry",
    "hurrying": "hurry",
    "hursts": "hurst",
    "hurting": "hurt",
    "hurtled": "hurtle",
    "hurtles": "hurtle",
    "hurtling": "hurtle",
    "hurts": "hurt",
    "husbanded": "husband",
    "husbanding": "husband",
    "husbandmen": "husbandman",
    "husbandries": "husbandry",
    "husbands": "husband",
    "hushed": "hush",
    "hushes": "hush",
    "hushing": "hush",
    "husked": "husk",
    "huskier": "husky",
    "huskies": "husky",
    "huskiest": "husky",
    "husking": "husk",
    "husks": "husk",
    "hussars": "hussar",
    "hussies": "hussy",
    "hustled": "hustle",
    "hustlers": "hustler",
    "hustles": "hustle",
    "hustling": "hustle",
    "hutches": "hutch",
    "huts": "hut",
    "hyacinths": "hyacinth",
    "hyaenas": "hyaena",
    "hybridisations": "hybridisation",
    "hybridised": "hybridise",
    "hybridises": "hybridise",
    "hybridising": "hybridise",
    "hybridities": "hybridity",
    "hybridizations": "hybridization",
    "hybridized": "hybridize",
    "hybridizes": "hybridize",
    "hybridizing": "hybridize",
    "hybrids": "hybrid",
    "hydrae": "hydra",
    "hydrangeas": "hydrangea",
    "hydrants": "hydrant",
    "hydras": "hydra",
    "hydrated": "hydrate",
    "hydrates": "hydrate",
    "hydrating": "hydrate",
    "hydrations": "hydration",
    "hydraulics": "hydraulic",
    "hydrides": "hydride",
    "hydrocarbons": "hydrocarbon",
    "hydrocephali": "hydrocephalus",
    "hydrochlorides": "hydrochloride",
    "hydrocolloids": "hydrocolloid",
    "hydrocortisones": "hydrocortisone",
    "hydrofoils": "hydrofoil",
    "hydrogels": "hydrogel",
    "hydrogenated": "hydrogenate",
    "hydrogenates": "hydrogenate",
    "hydrogenating": "hydrogenate",
    "hydrogenations": "hydrogenation",
    "hydrogens": "hydrogen",
    "hydrographs": "hydrograph",
    "hydroids": "hydroid",
    "hydrolases": "hydrolase",
    "hydrologies": "hydrology",
    "hydrologists": "hydrologist",
    "hydrolysates": "hydrolysate",
    "hydrolysed": "hydrolyse",
    "hydrolyses": "hydrolysis",
    "hydrolysing": "hydrolyse",
    "hydrolyzed": "hydrolyze",
    "hydrolyzes": "hydrolyze",
    "hydrolyzing": "hydrolyze",
    "hydrometers": "hydrometer",
    "hydrophobicities": "hydrophobicity",
    "hydrophones": "hydrophone",
    "hydroplaned": "hydroplane",
    "hydroplanes": "hydroplane",
    "hydroplaning": "hydroplane",
    "hydroquinones": "hydroquinone",
    "hydros": "hydro",
    "hydrotherapies": "hydrotherapy",
    "hydroxides": "hydroxide",
    "hydroxyapatites": "hydroxyapatite",
    "hydroxylases": "hydroxylase",
    "hydroxyls": "hydroxyl",
    "hyenas": "hyena",
    "hygienists": "hygienist",
    "hygrometers": "hygrometer",
    "hying": "hie",
    "hymens": "hymen",
    "hymnals": "hymnal",
    "hymned": "hymn",
    "hymning": "hymn",
    "hymns": "hymn",
    "hyped": "hype",
    "hyper-links": "hyper-link",
    "hyperactivities": "hyperactivity",
    "hyperacuses": "hyperacusis",
    "hyperbolas": "hyperbola",
    "hyperboles": "hyperbole",
    "hypercalcaemias": "hypercalcaemia",
    "hypercalcemias": "hypercalcemia",
    "hypercholesterolaemias": "hypercholesterolaemia",
    "hypercubes": "hypercube",
    "hyperextensions": "hyperextension",
    "hyperglycaemias": "hyperglycaemia",
    "hyperhidroses": "hyperhidrosis",
    "hyperinflations": "hyperinflation",
    "hyperkalaemias": "hyperkalaemia",
    "hyperlinked": "hyperlink",
    "hyperlinking": "hyperlink",
    "hyperlinks": "hyperlink",
    "hyperlipidaemias": "hyperlipidaemia",
    "hypermarkets": "hypermarket",
    "hypermedia": "hypermedium",
    "hypermobilities": "hypermobility",
    "hyperparathyroidisms": "hyperparathyroidism",
    "hyperplasias": "hyperplasia",
    "hypersecretions": "hypersecretion",
    "hypersensitivities": "hypersensitivity",
    "hyperspaces": "hyperspace",
    "hypertensions": "hypertension",
    "hypertensives": "hypertensive",
    "hyperthermias": "hyperthermia",
    "hyperthyroidisms": "hyperthyroidism",
    "hypertrophies": "hypertrophy",
    "hyperventilated": "hyperventilate",
    "hyperventilates": "hyperventilate",
    "hyperventilating": "hyperventilate",
    "hyperventilations": "hyperventilation",
    "hypes": "hype",
    "hyphae": "hypha",
    "hyphenated": "hyphenate",
    "hyphenates": "hyphenate",
    "hyphenating": "hyphenate",
    "hyphenations": "hyphenation",
    "hyphens": "hyphen",
    "hyping": "hype",
    "hypnoses": "hypnosis",
    "hypnotherapies": "hypnotherapy",
    "hypnotherapists": "hypnotherapist",
    "hypnotics": "hypnotic",
    "hypnotised": "hypnotise",
    "hypnotises": "hypnotise",
    "hypnotising": "hypnotise",
    "hypnotists": "hypnotist",
    "hypnotized": "hypnotize",
    "hypnotizes": "hypnotize",
    "hypnotizing": "hypnotize",
    "hypocalcaemias": "hypocalcaemia",
    "hypochlorites": "hypochlorite",
    "hypochondria": "hypochondrium",
    "hypochondriacs": "hypochondriac",
    "hypochondrias": "hypochondria",
    "hypocotyls": "hypocotyl",
    "hypocrisies": "hypocrisy",
    "hypocrites": "hypocrite",
    "hypodermics": "hypodermic",
    "hypoglycaemias": "hypoglycaemia",
    "hypoglycemias": "hypoglycemia",
    "hypogonadisms": "hypogonadism",
    "hypokalaemias": "hypokalaemia",
    "hypomanias": "hypomania",
    "hyponatraemias": "hyponatraemia",
    "hypopituitarisms": "hypopituitarism",
    "hypoplasias": "hypoplasia",
    "hypos": "hypo",
    "hypostases": "hypostasis",
    "hypostasises": "hypostasis",
    "hypotensions": "hypotension",
    "hypotenuses": "hypotenuse",
    "hypothalami": "hypothalamus",
    "hypothalamuses": "hypothalamus",
    "hypothecated": "hypothecate",
    "hypothecates": "hypothecate",
    "hypothecating": "hypothecate",
    "hypothermias": "hypothermia",
    "hypotheses": "hypothesis",
    "hypothesised": "hypothesise",
    "hypothesises": "hypothesise",
    "hypothesising": "hypothesise",
    "hypothesized": "hypothesize",
    "hypothesizes": "hypothesize",
    "hypothesizing": "hypothesize",
    "hypothyroidisms": "hypothyroidism",
    "hypoventilations": "hypoventilation",
    "hypovolaemias": "hypovolaemia",
    "hypoxaemias": "hypoxaemia",
    "hypoxias": "hypoxia",
    "hys": "hy",
    "hysterectomies": "hysterectomy",
    "hystereses": "hysteresis",
    "hysterias": "hysteria",
    "hysteroscopies": "hysteroscopy",
    "i's": "i",
    "iambs": "iamb",
    "ibexes": "ibex",
    "ibises": "ibis",
    "ice-boxes": "ice-box",
    "ice-breakers": "ice-breaker",
    "ice-buckets": "ice-bucket",
    "ice-caps": "ice-cap",
    "ice-colder": "ice-cold",
    "ice-coldest": "ice-cold",
    "ice-creams": "ice-cream",
    "ice-skated": "ice-skate",
    "ice-skates": "ice-skate",
    "ice-skating": "ice-skate",
    "icebergs": "iceberg",
    "iceboxes": "icebox",
    "icebreakers": "icebreaker",
    "iced": "ice",
    "ices": "ice",
    "icicles": "icicle",
    "icier": "icy",
    "iciest": "icy",
    "icing": "ice",
    "iconoclasts": "iconoclast",
    "iconographies": "iconography",
    "icons": "icon",
    "icosahedra": "icosahedron",
    "icosahedrons": "icosahedron",
    "idealisations": "idealisation",
    "idealised": "idealise",
    "idealises": "idealise",
    "idealising": "idealise",
    "idealists": "idealist",
    "idealizations": "idealization",
    "idealized": "idealize",
    "idealizes": "idealize",
    "idealizing": "idealize",
    "ideals": "ideal",
    "ideas": "idea",
    "ideations": "ideation",
    "identifications": "identification",
    "identified": "identify",
    "identifiers": "identifier",
    "identifies": "identify",
    "identifying": "identify",
    "identikits": "identikit",
    "identities": "identity",
    "ideograms": "ideogram",
    "ideologies": "ideology",
    "ideologists": "ideologist",
    "idiocies": "idiocy",
    "idioms": "idiom",
    "idiosyncracies": "idiosyncracy",
    "idiosyncrasies": "idiosyncrasy",
    "idiots": "idiot",
    "idled": "idle",
    "idlenesses": "idleness",
    "idler": "idle",
    "idlers": "idler",
    "idles": "idle",
    "idlest": "idle",
    "idling": "idle",
    "idolatries": "idolatry",
    "idolised": "idolise",
    "idolises": "idolise",
    "idolising": "idolise",
    "idolized": "idolize",
    "idolizes": "idolize",
    "idolizing": "idolize",
    "idols": "idol",
    "ids": "id",
    "idylls": "idyll",
    "ifosfamides": "ifosfamide",
    "igloos": "igloo",
    "ignited": "ignite",
    "ignites": "ignite",
    "igniting": "ignite",
    "ignitions": "ignition",
    "ignominies": "ignominy",
    "ignoramuses": "ignoramus",
    "ignored": "ignore",
    "ignores": "ignore",
    "ignoring": "ignore",
    "iguanas": "iguana",
    "ikons": "ikon",
    "ilea": "ileum",
    "ileostomies": "ileostomy",
    "iles": "ile",
    "ileums": "ileum",
    "ill-treated": "ill-treat",
    "ill-treating": "ill-treat",
    "ill-treats": "ill-treat",
    "illegalities": "illegality",
    "illegals": "illegal",
    "illegitimacies": "illegitimacy",
    "iller": "ill",
    "illest": "ill",
    "illiterates": "illiterate",
    "illnesses": "illness",
    "illogicalities": "illogicality",
    "ills": "ill",
    "illuminances": "illuminance",
    "illuminants": "illuminant",
    "illuminated": "illuminate",
    "illuminates": "illuminate",
    "illuminating": "illuminate",
    "illuminations": "illumination",
    "illuminators": "illuminator",
    "illumined": "illumine",
    "illumines": "illumine",
    "illumining": "illumine",
    "illusionists": "illusionist",
    "illusions": "illusion",
    "illustrated": "illustrate",
    "illustrates": "illustrate",
    "illustrating": "illustrate",
    "illustrations": "illustration",
    "illustrators": "illustrator",
    "imaged": "image",
    "imageries": "imagery",
    "imagers": "imager",
    "images": "image",
    "imaginations": "imagination",
    "imagined": "imagine",
    "imagines": "imago",
    "imaging": "image",
    "imagings": "imaging",
    "imagining": "imagine",
    "imagoes": "imago",
    "imagos": "imago",
    "imams": "imam",
    "imbalances": "imbalance",
    "imbeciles": "imbecile",
    "imbecilities": "imbecility",
    "imbedded": "imbed",
    "imbedding": "imbed",
    "imbeds": "imbed",
    "imbibed": "imbibe",
    "imbibes": "imbibe",
    "imbibing": "imbibe",
    "imbroglios": "imbroglio",
    "imbued": "imbue",
    "imbues": "imbue",
    "imbuing": "imbue",
    "imidazoles": "imidazole",
    "imipramines": "imipramine",
    "imitated": "imitate",
    "imitates": "imitate",
    "imitating": "imitate",
    "imitations": "imitation",
    "imitators": "imitator",
    "immaturer": "immature",
    "immatures": "immature",
    "immaturest": "immature",
    "immaturities": "immaturity",
    "immediacies": "immediacy",
    "immenser": "immense",
    "immensest": "immense",
    "immensities": "immensity",
    "immersed": "immerse",
    "immerses": "immerse",
    "immersing": "immerse",
    "immersions": "immersion",
    "immigrants": "immigrant",
    "immigrated": "immigrate",
    "immigrates": "immigrate",
    "immigrating": "immigrate",
    "immigrations": "immigration",
    "immobilisations": "immobilisation",
    "immobilised": "immobilise",
    "immobilisers": "immobiliser",
    "immobilises": "immobilise",
    "immobilising": "immobilise",
    "immobilities": "immobility",
    "immobilizations": "immobilization",
    "immobilized": "immobilize",
    "immobilizes": "immobilize",
    "immobilizing": "immobilize",
    "immortalised": "immortalise",
    "immortalises": "immortalise",
    "immortalising": "immortalise",
    "immortalized": "immortalize",
    "immortalizes": "immortalize",
    "immortalizing": "immortalize",
    "immortals": "immortal",
    "immunisations": "immunisation",
    "immunised": "immunise",
    "immunises": "immunise",
    "immunising": "immunise",
    "immunities": "immunity",
    "immunizations": "immunization",
    "immunized": "immunize",
    "immunizes": "immunize",
    "immunizing": "immunize",
    "immunoassays": "immunoassay",
    "immunocompromised": "immunocompromise",
    "immunocompromises": "immunocompromise",
    "immunocompromising": "immunocompromise",
    "immunocytochemistries": "immunocytochemistry",
    "immunodeficiencies": "immunodeficiency",
    "immunofluorescences": "immunofluorescence",
    "immunogenicities": "immunogenicity",
    "immunoglobulins": "immunoglobulin",
    "immunohistochemistries": "immunohistochemistry",
    "immunologies": "immunology",
    "immunologists": "immunologist",
    "immunoreactivities": "immunoreactivity",
    "immunostainings": "immunostaining",
    "immunosuppressants": "immunosuppressant",
    "immunosuppressions": "immunosuppression",
    "immunotherapies": "immunotherapy",
    "immured": "immure",
    "immures": "immure",
    "immuring": "immure",
    "impacted": "impact",
    "impacting": "impact",
    "impactions": "impaction",
    "impactors": "impactor",
    "impacts": "impact",
    "impaired": "impair",
    "impairing": "impair",
    "impairments": "impairment",
    "impairs": "impair",
    "impalas": "impala",
    "impaled": "impale",
    "impales": "impale",
    "impaling": "impale",
    "imparted": "impart",
    "imparting": "impart",
    "imparts": "impart",
    "impasses": "impasse",
    "impassively": "impassive",
    "impeached": "impeach",
    "impeaches": "impeach",
    "impeaching": "impeach",
    "impedances": "impedance",
    "impeded": "impede",
    "impedes": "impede",
    "impediments": "impediment",
    "impeding": "impede",
    "impelled": "impel",
    "impellers": "impeller",
    "impelling": "impel",
    "impels": "impel",
    "impended": "impend",
    "impending": "impend",
    "impends": "impend",
    "imperatives": "imperative",
    "imperfections": "imperfection",
    "imperfects": "imperfect",
    "imperialists": "imperialist",
    "imperiled": "imperil",
    "imperiling": "imperil",
    "imperilled": "imperil",
    "imperilling": "imperil",
    "imperils": "imperil",
    "impersonated": "impersonate",
    "impersonates": "impersonate",
    "impersonating": "impersonate",
    "impersonations": "impersonation",
    "impersonators": "impersonator",
    "impertinences": "impertinence",
    "impetuses": "impetus",
    "impieties": "impiety",
    "impinged": "impinge",
    "impingements": "impingement",
    "impinges": "impinge",
    "impinging": "impinge",
    "implantations": "implantation",
    "implanted": "implant",
    "implanting": "implant",
    "implants": "implant",
    "implausibilities": "implausibility",
    "implementations": "implementation",
    "implemented": "implement",
    "implementers": "implementer",
    "implementing": "implement",
    "implementors": "implementor",
    "implements": "implement",
    "implicated": "implicate",
    "implicates": "implicate",
    "implicating": "implicate",
    "implications": "implication",
    "implicatures": "implicature",
    "implied": "imply",
    "implies": "imply",
    "imploded": "implode",
    "implodes": "implode",
    "imploding": "implode",
    "implored": "implore",
    "implores": "implore",
    "imploring": "implore",
    "implosions": "implosion",
    "implying": "imply",
    "imponderables": "imponderable",
    "importances": "importance",
    "importations": "importation",
    "imported": "import",
    "importers": "importer",
    "importing": "import",
    "imports": "import",
    "importuned": "importune",
    "importunes": "importune",
    "importuning": "importune",
    "importunities": "importunity",
    "imposed": "impose",
    "imposes": "impose",
    "imposing": "impose",
    "impositions": "imposition",
    "impossibilities": "impossibility",
    "imposters": "imposter",
    "impostors": "impostor",
    "impostures": "imposture",
    "impounded": "impound",
    "impounding": "impound",
    "impoundments": "impoundment",
    "impounds": "impound",
    "impoverished": "impoverish",
    "impoverishes": "impoverish",
    "impoverishing": "impoverish",
    "impracticalities": "impracticality",
    "imprecations": "imprecation",
    "imprecisions": "imprecision",
    "impregnated": "impregnate",
    "impregnates": "impregnate",
    "impregnating": "impregnate",
    "impregnations": "impregnation",
    "impresarios": "impresario",
    "impressed": "impress",
    "impresses": "impress",
    "impressing": "impress",
    "impressionists": "impressionist",
    "impressions": "impression",
    "imprinted": "imprint",
    "imprinting": "imprint",
    "imprints": "imprint",
    "imprisoned": "imprison",
    "imprisoning": "imprison",
    "imprisonments": "imprisonment",
    "imprisons": "imprison",
    "improbabilities": "improbability",
    "improprieties": "impropriety",
    "improved": "improve",
    "improvements": "improvement",
    "improvers": "improver",
    "improves": "improve",
    "improving": "improve",
    "improvisations": "improvisation",
    "improvised": "improvise",
    "improvises": "improvise",
    "improvising": "improvise",
    "imprudences": "imprudence",
    "imps": "imp",
    "impudences": "impudence",
    "impugned": "impugn",
    "impugning": "impugn",
    "impugns": "impugn",
    "impulses": "impulse",
    "impulsions": "impulsion",
    "impurer": "impure",
    "impurest": "impure",
    "impurities": "impurity",
    "imputations": "imputation",
    "imputed": "impute",
    "imputes": "impute",
    "imputing": "impute",
    "in-fightings": "in-fighting",
    "in-fills": "in-fill",
    "in-groups": "in-group",
    "in-laws": "in-law",
    "in-migrations": "in-migration",
    "in-patients": "in-patient",
    "in-roads": "in-road",
    "inabilities": "inability",
    "inaccuracies": "inaccuracy",
    "inactions": "inaction",
    "inactivated": "inactivate",
    "inactivates": "inactivate",
    "inactivating": "inactivate",
    "inactivations": "inactivation",
    "inactivities": "inactivity",
    "inadequacies": "inadequacy",
    "inaptitudes": "inaptitude",
    "inattentions": "inattention",
    "inaugurated": "inaugurate",
    "inaugurates": "inaugurate",
    "inaugurating": "inaugurate",
    "inaugurations": "inauguration",
    "inbred": "inbreed",
    "inbreeding": "inbreed",
    "inbreeds": "inbreed",
    "incantations": "incantation",
    "incapabilities": "incapability",
    "incapacitated": "incapacitate",
    "incapacitates": "incapacitate",
    "incapacitating": "incapacitate",
    "incapacitations": "incapacitation",
    "incapacities": "incapacity",
    "incarcerated": "incarcerate",
    "incarcerates": "incarcerate",
    "incarcerating": "incarcerate",
    "incarcerations": "incarceration",
    "incarnated": "incarnate",
    "incarnates": "incarnate",
    "incarnating": "incarnate",
    "incarnations": "incarnation",
    "incased": "incase",
    "incases": "incase",
    "incasing": "incase",
    "incautiously": "incautious",
    "incendiaries": "incendiary",
    "incensed": "incense",
    "incenses": "incense",
    "incensing": "incense",
    "incentives": "incentive",
    "incentivisations": "incentivisation",
    "incentivised": "incentivise",
    "incentivises": "incentivise",
    "incentivising": "incentivise",
    "inceptions": "inception",
    "inched": "inch",
    "inches": "inch",
    "inching": "inch",
    "incidences": "incidence",
    "incidentals": "incidental",
    "incidents": "incident",
    "incinerated": "incinerate",
    "incinerates": "incinerate",
    "incinerating": "incinerate",
    "incinerations": "incineration",
    "incinerators": "incinerator",
    "incised": "incise",
    "incises": "incise",
    "incising": "incise",
    "incisions": "incision",
    "incisors": "incisor",
    "incited": "incite",
    "incitements": "incitement",
    "incites": "incite",
    "inciting": "incite",
    "incivilities": "incivility",
    "inclinations": "inclination",
    "inclined": "incline",
    "inclines": "incline",
    "inclining": "incline",
    "inclosed": "inclose",
    "incloses": "inclose",
    "inclosing": "inclose",
    "inclosures": "inclosure",
    "included": "include",
    "includes": "include",
    "including": "include",
    "inclusions": "inclusion",
    "incognitos": "incognito",
    "income-taxes": "income-tax",
    "incomes": "income",
    "incommensurabilities": "incommensurability",
    "incompatibilities": "incompatibility",
    "incompetents": "incompetent",
    "incongruities": "incongruity",
    "inconsistencies": "inconsistency",
    "inconstancies": "inconstancy",
    "incontinences": "incontinence",
    "inconvenienced": "inconvenience",
    "inconveniences": "inconvenience",
    "inconveniencing": "inconvenience",
    "incorporated": "incorporate",
    "incorporates": "incorporate",
    "incorporating": "incorporate",
    "incorporations": "incorporation",
    "increased": "increase",
    "increases": "increase",
    "increasing": "increase",
    "incremented": "increment",
    "incrementing": "increment",
    "increments": "increment",
    "incriminated": "incriminate",
    "incriminates": "incriminate",
    "incriminating": "incriminate",
    "incubated": "incubate",
    "incubates": "incubate",
    "incubating": "incubate",
    "incubations": "incubation",
    "incubators": "incubator",
    "incubi": "incubus",
    "incubuses": "incubus",
    "inculcated": "inculcate",
    "inculcates": "inculcate",
    "inculcating": "inculcate",
    "incumbencies": "incumbency",
    "incumbents": "incumbent",
    "incunabula": "incunabulum",
    "incurables": "incurable",
    "incurred": "incur",
    "incurring": "incur",
    "incurs": "incur",
    "incursions": "incursion",
    "indecencies": "indecency",
    "indemnifications": "indemnification",
    "indemnified": "indemnify",
    "indemnifies": "indemnify",
    "indemnifying": "indemnify",
    "indemnities": "indemnity",
    "indentations": "indentation",
    "indented": "indent",
    "indenters": "indenter",
    "indenting": "indent",
    "indents": "indent",
    "indentured": "indenture",
    "indentures": "indenture",
    "indenturing": "indenture",
    "independences": "independence",
    "independencies": "independency",
    "independents": "independent",
    "indeterminacies": "indeterminacy",
    "indexed": "index",
    "indexers": "indexer",
    "indexes": "index",
    "indexing": "index",
    "indexings": "indexing",
    "indicated": "indicate",
    "indicates": "indicate",
    "indicating": "indicate",
    "indications": "indication",
    "indicators": "indicator",
    "indices": "index",
    "indicted": "indict",
    "indicting": "indict",
    "indictments": "indictment",
    "indictors": "indictor",
    "indicts": "indict",
    "indifferences": "indifference",
    "indignations": "indignation",
    "indignities": "indignity",
    "indigoes": "indigo",
    "indigos": "indigo",
    "indirecter": "indirect",
    "indirectest": "indirect",
    "indirections": "indirection",
    "indiscretions": "indiscretion",
    "indispositions": "indisposition",
    "individualised": "individualise",
    "individualises": "individualise",
    "individualising": "individualise",
    "individualists": "individualist",
    "individualities": "individuality",
    "individualized": "individualize",
    "individualizes": "individualize",
    "individualizing": "individualize",
    "individuals": "individual",
    "individuated": "individuate",
    "individuates": "individuate",
    "individuating": "individuate",
    "individuations": "individuation",
    "indivisibilities": "indivisibility",
    "indoctrinated": "indoctrinate",
    "indoctrinates": "indoctrinate",
    "indoctrinating": "indoctrinate",
    "indoctrinations": "indoctrination",
    "indoles": "indole",
    "indoors": "indoor",
    "induced": "induce",
    "inducements": "inducement",
    "inducers": "inducer",
    "induces": "induce",
    "inducing": "induce",
    "inductances": "inductance",
    "inducted": "induct",
    "inductees": "inductee",
    "inducting": "induct",
    "inductions": "induction",
    "inductors": "inductor",
    "inducts": "induct",
    "indulged": "indulge",
    "indulgences": "indulgence",
    "indulges": "indulge",
    "indulging": "indulge",
    "industrialised": "industrialise",
    "industrialises": "industrialise",
    "industrialising": "industrialise",
    "industrialists": "industrialist",
    "industrialized": "industrialize",
    "industrializes": "industrialize",
    "industrializing": "industrialize",
    "industrials": "industrial",
    "industries": "industry",
    "indwelled": "indwell",
    "indwelling": "indwell",
    "indwells": "indwell",
    "indwelt": "indwell",
    "inebriated": "inebriate",
    "inebriates": "inebriate",
    "inebriating": "inebriate",
    "inefficiencies": "inefficiency",
    "ineligibilities": "ineligibility",
    "ineligibles": "ineligible",
    "inepter": "inept",
    "ineptest": "inept",
    "inequalities": "inequality",
    "inequities": "inequity",
    "inertias": "inertia",
    "inessentials": "inessential",
    "inevitabilities": "inevitability",
    "inexperiences": "inexperience",
    "infamies": "infamy",
    "infancies": "infancy",
    "infanticides": "infanticide",
    "infantries": "infantry",
    "infantrymen": "infantryman",
    "infants": "infant",
    "infarctions": "infarction",
    "infarcts": "infarct",
    "infatuations": "infatuation",
    "infected": "infect",
    "infecting": "infect",
    "infections": "infection",
    "infectivities": "infectivity",
    "infects": "infect",
    "inferences": "inference",
    "inferiors": "inferior",
    "infernos": "inferno",
    "inferred": "infer",
    "inferring": "infer",
    "infers": "infer",
    "infertilities": "infertility",
    "infestations": "infestation",
    "infested": "infest",
    "infesting": "infest",
    "infests": "infest",
    "infidelities": "infidelity",
    "infidels": "infidel",
    "infields": "infield",
    "infilled": "infill",
    "infilling": "infill",
    "infills": "infill",
    "infiltrated": "infiltrate",
    "infiltrates": "infiltrate",
    "infiltrating": "infiltrate",
    "infiltrations": "infiltration",
    "infiltrators": "infiltrator",
    "infinities": "infinity",
    "infinitives": "infinitive",
    "infirmaries": "infirmary",
    "infirmities": "infirmity",
    "infixes": "infix",
    "inflamed": "inflame",
    "inflames": "inflame",
    "inflaming": "inflame",
    "inflammations": "inflammation",
    "inflatables": "inflatable",
    "inflated": "inflate",
    "inflates": "inflate",
    "inflating": "inflate",
    "inflations": "inflation",
    "inflators": "inflator",
    "inflected": "inflect",
    "inflecting": "inflect",
    "inflections": "inflection",
    "inflects": "inflect",
    "inflexions": "inflexion",
    "inflicted": "inflict",
    "inflicting": "inflict",
    "inflictions": "infliction",
    "inflicts": "inflict",
    "inflorescences": "inflorescence",
    "inflows": "inflow",
    "influenced": "influence",
    "influencers": "influencer",
    "influences": "influence",
    "influencing": "influence",
    "influenzas": "influenza",
    "influenzaviruses": "influenzavirus",
    "influxes": "influx",
    "infomercials": "infomercial",
    "informalities": "informality",
    "informants": "informant",
    "informations": "information",
    "informed": "inform",
    "informers": "informer",
    "informing": "inform",
    "informs": "inform",
    "infra-structures": "infra-structure",
    "infractions": "infraction",
    "infrasounds": "infrasound",
    "infrastructures": "infrastructure",
    "infrequencies": "infrequency",
    "infringed": "infringe",
    "infringements": "infringement",
    "infringes": "infringe",
    "infringing": "infringe",
    "infs": "inf",
    "infuriated": "infuriate",
    "infuriates": "infuriate",
    "infuriating": "infuriate",
    "infused": "infuse",
    "infuses": "infuse",
    "infusing": "infuse",
    "infusions": "infusion",
    "infusors": "infusor",
    "ingenues": "ingenue",
    "ingenuities": "ingenuity",
    "ingested": "ingest",
    "ingesting": "ingest",
    "ingestions": "ingestion",
    "ingests": "ingest",
    "ingles": "ingle",
    "ingots": "ingot",
    "ingratiated": "ingratiate",
    "ingratiates": "ingratiate",
    "ingratiating": "ingratiate",
    "ingredients": "ingredient",
    "inhabitants": "inhabitant",
    "inhabitations": "inhabitation",
    "inhabited": "inhabit",
    "inhabiting": "inhabit",
    "inhabits": "inhabit",
    "inhalants": "inhalant",
    "inhalations": "inhalation",
    "inhaled": "inhale",
    "inhalers": "inhaler",
    "inhales": "inhale",
    "inhaling": "inhale",
    "inhered": "inhere",
    "inheres": "inhere",
    "inhering": "inhere",
    "inheritances": "inheritance",
    "inherited": "inherit",
    "inheriting": "inherit",
    "inheritors": "inheritor",
    "inherits": "inherit",
    "inhibited": "inhibit",
    "inhibiting": "inhibit",
    "inhibitions": "inhibition",
    "inhibitors": "inhibitor",
    "inhibits": "inhibit",
    "inhomogeneities": "inhomogeneity",
    "inhumanities": "inhumanity",
    "inhumations": "inhumation",
    "iniquities": "iniquity",
    "initialed": "initial",
    "initialing": "initial",
    "initialisations": "initialisation",
    "initialised": "initialise",
    "initialises": "initialise",
    "initialising": "initialise",
    "initializations": "initialization",
    "initialized": "initialize",
    "initializes": "initialize",
    "initializing": "initialize",
    "initialled": "initial",
    "initialling": "initial",
    "initials": "initial",
    "initiated": "initiate",
    "initiates": "initiate",
    "initiating": "initiate",
    "initiations": "initiation",
    "initiatives": "initiative",
    "initiators": "initiator",
    "injected": "inject",
    "injecting": "inject",
    "injections": "injection",
    "injectors": "injector",
    "injects": "inject",
    "injunctions": "injunction",
    "injured": "injure",
    "injures": "injure",
    "injuries": "injury",
    "injuring": "injure",
    "injustices": "injustice",
    "ink-pencils": "ink-pencil",
    "inked": "ink",
    "inkier": "inky",
    "inkiest": "inky",
    "inking": "ink",
    "inklings": "inkling",
    "inks": "ink",
    "inkstands": "inkstand",
    "inkwells": "inkwell",
    "inlaid": "inlay",
    "inlayed": "inlay",
    "inlaying": "inlay",
    "inlayinging": "inlay",
    "inlays": "inlay",
    "inlets": "inlet",
    "inletting": "inlet",
    "inmates": "inmate",
    "inners": "inner",
    "innervated": "innervate",
    "innervates": "innervate",
    "innervating": "innervate",
    "innervations": "innervation",
    "innings": "inning",
    "innkeepers": "innkeeper",
    "innocents": "innocent",
    "innovated": "innovate",
    "innovates": "innovate",
    "innovating": "innovate",
    "innovations": "innovation",
    "innovators": "innovator",
    "inns": "inn",
    "innuendoes": "innuendo",
    "innuendos": "innuendo",
    "inocula": "inoculum",
    "inoculants": "inoculant",
    "inoculated": "inoculate",
    "inoculates": "inoculate",
    "inoculating": "inoculate",
    "inoculations": "inoculation",
    "inoculums": "inoculum",
    "inositols": "inositol",
    "inpatients": "inpatient",
    "input-outputs": "input-output",
    "inputs": "input",
    "inputted": "input",
    "inputting": "input",
    "inquests": "inquest",
    "inquired": "inquire",
    "inquirers": "inquirer",
    "inquires": "inquire",
    "inquiries": "inquiry",
    "inquiring": "inquire",
    "inquisitions": "inquisition",
    "inquisitors": "inquisitor",
    "inroads": "inroad",
    "insaner": "insane",
    "insanest": "insane",
    "insanities": "insanity",
    "inscribed": "inscribe",
    "inscribes": "inscribe",
    "inscribing": "inscribe",
    "inscriptions": "inscription",
    "insecticides": "insecticide",
    "insects": "insect",
    "insecurities": "insecurity",
    "inseminated": "inseminate",
    "inseminates": "inseminate",
    "inseminating": "inseminate",
    "inseminations": "insemination",
    "insensitivities": "insensitivity",
    "inserted": "insert",
    "inserters": "inserter",
    "inserting": "insert",
    "insertions": "insertion",
    "inserts": "insert",
    "insets": "inset",
    "insetted": "inset",
    "insetting": "inset",
    "insiders": "insider",
    "insides": "inside",
    "insights": "insight",
    "insignia": "insigne",
    "insignias": "insignia",
    "insincerities": "insincerity",
    "insinuated": "insinuate",
    "insinuates": "insinuate",
    "insinuating": "insinuate",
    "insinuations": "insinuation",
    "insisted": "insist",
    "insisting": "insist",
    "insists": "insist",
    "insoles": "insole",
    "insolvencies": "insolvency",
    "insomniacs": "insomniac",
    "insomnias": "insomnia",
    "inspected": "inspect",
    "inspecting": "inspect",
    "inspections": "inspection",
    "inspectorates": "inspectorate",
    "inspectors": "inspector",
    "inspects": "inspect",
    "inspirations": "inspiration",
    "inspired": "inspire",
    "inspires": "inspire",
    "inspiring": "inspire",
    "instabilities": "instability",
    "installations": "installation",
    "installed": "install",
    "installers": "installer",
    "installing": "install",
    "installments": "installment",
    "installs": "install",
    "instalments": "instalment",
    "instals": "instal",
    "instanced": "instance",
    "instances": "instance",
    "instancing": "instance",
    "instantiated": "instantiate",
    "instantiates": "instantiate",
    "instantiating": "instantiate",
    "instantiations": "instantiation",
    "instants": "instant",
    "instars": "instar",
    "instated": "instate",
    "instates": "instate",
    "instating": "instate",
    "insteps": "instep",
    "instigated": "instigate",
    "instigates": "instigate",
    "instigating": "instigate",
    "instigators": "instigator",
    "instillations": "instillation",
    "instilled": "instill",
    "instilling": "instill",
    "instills": "instill",
    "instils": "instil",
    "instincts": "instinct",
    "instituted": "institute",
    "institutes": "institute",
    "instituting": "institute",
    "institutionalisations": "institutionalisation",
    "institutionalised": "institutionalise",
    "institutionalises": "institutionalise",
    "institutionalising": "institutionalise",
    "institutionalizations": "institutionalization",
    "institutionalized": "institutionalize",
    "institutionalizes": "institutionalize",
    "institutionalizing": "institutionalize",
    "institutions": "institution",
    "instructed": "instruct",
    "instructing": "instruct",
    "instructions": "instruction",
    "instructors": "instructor",
    "instructs": "instruct",
    "instrumentalists": "instrumentalist",
    "instrumentalities": "instrumentality",
    "instrumentations": "instrumentation",
    "instrumented": "instrument",
    "instrumenting": "instrument",
    "instruments": "instrument",
    "insubordinations": "insubordination",
    "insufficiencies": "insufficiency",
    "insulae": "insula",
    "insulated": "insulate",
    "insulates": "insulate",
    "insulating": "insulate",
    "insulations": "insulation",
    "insulators": "insulator",
    "insulinomas": "insulinoma",
    "insulinomata": "insulinoma",
    "insulins": "insulin",
    "insulted": "insult",
    "insulting": "insult",
    "insults": "insult",
    "insurances": "insurance",
    "insured": "insure",
    "insureds": "insured",
    "insurers": "insurer",
    "insures": "insure",
    "insurgencies": "insurgency",
    "insurgents": "insurgent",
    "insuring": "insure",
    "insurrections": "insurrection",
    "intaglios": "intaglio",
    "intakes": "intake",
    "intangibles": "intangible",
    "integers": "integer",
    "integrals": "integral",
    "integrases": "integrase",
    "integrated": "integrate",
    "integrates": "integrate",
    "integrating": "integrate",
    "integrationists": "integrationist",
    "integrations": "integration",
    "integrators": "integrator",
    "integrins": "integrin",
    "integrities": "integrity",
    "intellects": "intellect",
    "intellectualised": "intellectualise",
    "intellectualises": "intellectualise",
    "intellectualising": "intellectualise",
    "intellectualisms": "intellectualism",
    "intellectuals": "intellectual",
    "intelligentsias": "intelligentsia",
    "intelligibilities": "intelligibility",
    "intended": "intend",
    "intending": "intend",
    "intends": "intend",
    "intenser": "intense",
    "intensest": "intense",
    "intensifications": "intensification",
    "intensified": "intensify",
    "intensifiers": "intensifier",
    "intensifies": "intensify",
    "intensifying": "intensify",
    "intensions": "intension",
    "intensities": "intensity",
    "intentions": "intention",
    "intents": "intent",
    "inter-actions": "inter-action",
    "inter-connections": "inter-connection",
    "inter-dependences": "inter-dependence",
    "inter-relations": "inter-relation",
    "inter-relationships": "inter-relationship",
    "inter-universities": "inter-university",
    "interacted": "interact",
    "interacting": "interact",
    "interactionists": "interactionist",
    "interactions": "interaction",
    "interactivities": "interactivity",
    "interactors": "interactor",
    "interacts": "interact",
    "interbred": "interbreed",
    "interbreeding": "interbreed",
    "interbreeds": "interbreed",
    "intercalated": "intercalate",
    "intercalates": "intercalate",
    "intercalating": "intercalate",
    "intercalations": "intercalation",
    "interceded": "intercede",
    "intercedes": "intercede",
    "interceding": "intercede",
    "intercepted": "intercept",
    "intercepting": "intercept",
    "interceptions": "interception",
    "interceptors": "interceptor",
    "intercepts": "intercept",
    "intercessions": "intercession",
    "intercessors": "intercessor",
    "interchangeabilities": "interchangeability",
    "interchanged": "interchange",
    "interchanges": "interchange",
    "interchanging": "interchange",
    "intercommunications": "intercommunication",
    "intercomparisons": "intercomparison",
    "intercoms": "intercom",
    "interconnected": "interconnect",
    "interconnecting": "interconnect",
    "interconnections": "interconnection",
    "interconnectivities": "interconnectivity",
    "interconnects": "interconnect",
    "intercoolers": "intercooler",
    "intercourses": "intercourse",
    "intercropped": "intercrop",
    "intercropping": "intercrop",
    "intercrops": "intercrop",
    "intercrossed": "intercross",
    "intercrosses": "intercross",
    "intercrossing": "intercross",
    "intercuts": "intercut",
    "intercutted": "intercut",
    "intercutting": "intercut",
    "interdependences": "interdependence",
    "interdependencies": "interdependency",
    "interdicted": "interdict",
    "interdicting": "interdict",
    "interdictions": "interdiction",
    "interdicts": "interdict",
    "interested": "interest",
    "interesting": "interest",
    "interests": "interest",
    "interfaced": "interface",
    "interfaces": "interface",
    "interfacing": "interface",
    "interfacings": "interfacing",
    "interfered": "interfere",
    "interferences": "interference",
    "interferers": "interferer",
    "interferes": "interfere",
    "interfering": "interfere",
    "interferograms": "interferogram",
    "interferometers": "interferometer",
    "interferometries": "interferometry",
    "interferons": "interferon",
    "interglacials": "interglacial",
    "interims": "interim",
    "interiors": "interior",
    "interjected": "interject",
    "interjecting": "interject",
    "interjections": "interjection",
    "interjects": "interject",
    "interlaced": "interlace",
    "interlaces": "interlace",
    "interlacing": "interlace",
    "interlanguages": "interlanguage",
    "interlayers": "interlayer",
    "interleaved": "interleave",
    "interleaves": "interleave",
    "interleaving": "interleave",
    "interleukins": "interleukin",
    "interlined": "interline",
    "interlines": "interline",
    "interlining": "interline",
    "interlinked": "interlink",
    "interlinking": "interlink",
    "interlinks": "interlink",
    "interlocked": "interlock",
    "interlocking": "interlock",
    "interlocks": "interlock",
    "interlocutors": "interlocutor",
    "interlopers": "interloper",
    "interludes": "interlude",
    "intermarriages": "intermarriage",
    "intermarried": "intermarry",
    "intermarries": "intermarry",
    "intermarrying": "intermarry",
    "intermedia": "intermedium",
    "intermediae": "intermedia",
    "intermediaries": "intermediary",
    "intermediates": "intermediate",
    "intermediums": "intermedium",
    "interments": "interment",
    "intermingled": "intermingle",
    "intermingles": "intermingle",
    "intermingling": "intermingle",
    "intermissions": "intermission",
    "intermittencies": "intermittency",
    "intermixed": "intermix",
    "intermixes": "intermix",
    "intermixing": "intermix",
    "intermodulations": "intermodulation",
    "internalisations": "internalisation",
    "internalised": "internalise",
    "internalises": "internalise",
    "internalising": "internalise",
    "internalizations": "internalization",
    "internalized": "internalize",
    "internalizes": "internalize",
    "internalizing": "internalize",
    "internals": "internal",
    "internationalised": "internationalise",
    "internationalises": "internationalise",
    "internationalising": "internationalise",
    "internationalists": "internationalist",
    "internationalized": "internationalize",
    "internationalizes": "internationalize",
    "internationalizing": "internationalize",
    "internationals": "international",
    "interned": "intern",
    "internees": "internee",
    "internets": "internet",
    "interneurons": "interneuron",
    "interning": "intern",
    "internists": "internist",
    "internments": "internment",
    "internodes": "internode",
    "interns": "intern",
    "internships": "internship",
    "interoperated": "interoperate",
    "interoperates": "interoperate",
    "interoperating": "interoperate",
    "interpenetrated": "interpenetrate",
    "interpenetrates": "interpenetrate",
    "interpenetrating": "interpenetrate",
    "interpenetrations": "interpenetration",
    "interphases": "interphase",
    "interplays": "interplay",
    "interpolated": "interpolate",
    "interpolates": "interpolate",
    "interpolating": "interpolate",
    "interpolations": "interpolation",
    "interposed": "interpose",
    "interposes": "interpose",
    "interposing": "interpose",
    "interpositions": "interposition",
    "interpretants": "interpretant",
    "interpretations": "interpretation",
    "interpreted": "interpret",
    "interpreters": "interpreter",
    "interpreting": "interpret",
    "interprets": "interpret",
    "interprocesses": "interprocess",
    "interred": "inter",
    "interregna": "interregnum",
    "interregnums": "interregnum",
    "interrelated": "interrelate",
    "interrelates": "interrelate",
    "interrelating": "interrelate",
    "interrelations": "interrelation",
    "interrelationships": "interrelationship",
    "interring": "inter",
    "interrogated": "interrogate",
    "interrogates": "interrogate",
    "interrogating": "interrogate",
    "interrogations": "interrogation",
    "interrogatives": "interrogative",
    "interrogators": "interrogator",
    "interrupted": "interrupt",
    "interrupters": "interrupter",
    "interrupting": "interrupt",
    "interruptions": "interruption",
    "interrupts": "interrupt",
    "inters": "inter",
    "intersected": "intersect",
    "intersecting": "intersect",
    "intersections": "intersection",
    "intersects": "intersect",
    "intersexes": "intersex",
    "interspersed": "intersperse",
    "intersperses": "intersperse",
    "interspersing": "intersperse",
    "interstates": "interstate",
    "interstices": "interstice",
    "interstitials": "interstitial",
    "intertwined": "intertwine",
    "intertwines": "intertwine",
    "intertwining": "intertwine",
    "intervals": "interval",
    "intervened": "intervene",
    "interveners": "intervener",
    "intervenes": "intervene",
    "intervening": "intervene",
    "interventionists": "interventionist",
    "interventions": "intervention",
    "interviewed": "interview",
    "interviewees": "interviewee",
    "interviewers": "interviewer",
    "interviewing": "interview",
    "interviews": "interview",
    "interweaved": "interweave",
    "interweaves": "interweave",
    "interweaving": "interweave",
    "interworkings": "interworking",
    "interwove": "interweave",
    "interwoven": "interweave",
    "intestates": "intestate",
    "intestines": "intestine",
    "intimacies": "intimacy",
    "intimated": "intimate",
    "intimates": "intimate",
    "intimating": "intimate",
    "intimations": "intimation",
    "intimidated": "intimidate",
    "intimidates": "intimidate",
    "intimidating": "intimidate",
    "intimidations": "intimidation",
    "intolerances": "intolerance",
    "intonations": "intonation",
    "intoned": "intone",
    "intones": "intone",
    "intoning": "intone",
    "intoxicants": "intoxicant",
    "intoxicated": "intoxicate",
    "intoxicates": "intoxicate",
    "intoxicating": "intoxicate",
    "intoxications": "intoxication",
    "intranets": "intranet",
    "intricacies": "intricacy",
    "intrigued": "intrigue",
    "intrigues": "intrigue",
    "intriguing": "intrigue",
    "intrinsics": "intrinsic",
    "introduced": "introduce",
    "introducers": "introducer",
    "introduces": "introduce",
    "introducing": "introduce",
    "introductions": "introduction",
    "introjected": "introject",
    "introjecting": "introject",
    "introjections": "introjection",
    "introjects": "introject",
    "introns": "intron",
    "intros": "intro",
    "introspections": "introspection",
    "introverted": "introvert",
    "introverting": "introvert",
    "introverts": "introvert",
    "intruded": "intrude",
    "intruders": "intruder",
    "intrudes": "intrude",
    "intruding": "intrude",
    "intrusions": "intrusion",
    "intrusted": "intrust",
    "intrusting": "intrust",
    "intrusts": "intrust",
    "intubated": "intubate",
    "intubates": "intubate",
    "intubating": "intubate",
    "intubations": "intubation",
    "intuited": "intuit",
    "intuiting": "intuit",
    "intuitions": "intuition",
    "intuits": "intuit",
    "inundated": "inundate",
    "inundates": "inundate",
    "inundating": "inundate",
    "inundations": "inundation",
    "inured": "inure",
    "inures": "inure",
    "inuring": "inure",
    "invaded": "invade",
    "invaders": "invader",
    "invades": "invade",
    "invading": "invade",
    "invalidated": "invalidate",
    "invalidates": "invalidate",
    "invalidating": "invalidate",
    "invalidations": "invalidation",
    "invalided": "invalid",
    "invaliding": "invalid",
    "invalidities": "invalidity",
    "invalids": "invalid",
    "invariances": "invariance",
    "invariants": "invariant",
    "invasions": "invasion",
    "invectives": "invective",
    "inveighed": "inveigh",
    "inveighing": "inveigh",
    "inveighs": "inveigh",
    "inveigled": "inveigle",
    "inveigles": "inveigle",
    "inveigling": "inveigle",
    "invented": "invent",
    "inventing": "invent",
    "inventions": "invention",
    "inventoried": "inventory",
    "inventories": "inventory",
    "inventors": "inventor",
    "inventorying": "inventory",
    "invents": "invent",
    "inverses": "inverse",
    "inversions": "inversion",
    "invertebrates": "invertebrate",
    "inverted": "invert",
    "inverters": "inverter",
    "inverting": "invert",
    "inverts": "invert",
    "invested": "invest",
    "investigated": "investigate",
    "investigates": "investigate",
    "investigating": "investigate",
    "investigations": "investigation",
    "investigators": "investigator",
    "investing": "invest",
    "investitures": "investiture",
    "investments": "investment",
    "investors": "investor",
    "invests": "invest",
    "invigilated": "invigilate",
    "invigilates": "invigilate",
    "invigilating": "invigilate",
    "invigilators": "invigilator",
    "invigorated": "invigorate",
    "invigorates": "invigorate",
    "invigorating": "invigorate",
    "invitations": "invitation",
    "invited": "invite",
    "invitees": "invitee",
    "invites": "invite",
    "inviting": "invite",
    "invitinger": "inviting",
    "invitingest": "inviting",
    "invocations": "invocation",
    "invoiced": "invoice",
    "invoices": "invoice",
    "invoicing": "invoice",
    "invoked": "invoke",
    "invokes": "invoke",
    "invoking": "invoke",
    "involutions": "involution",
    "involved": "involve",
    "involvements": "involvement",
    "involves": "involve",
    "involving": "involve",
    "invs": "inv",
    "iodides": "iodide",
    "iodines": "iodine",
    "ionisations": "ionisation",
    "ionised": "ionise",
    "ionisers": "ioniser",
    "ionises": "ionise",
    "ionising": "ionise",
    "ionizations": "ionization",
    "ionized": "ionize",
    "ionizes": "ionize",
    "ionizing": "ionize",
    "ions": "ion",
    "irides": "iris",
    "irises": "iris",
    "irked": "irk",
    "irking": "irk",
    "irks": "irk",
    "ironed": "iron",
    "ironies": "irony",
    "ironing": "iron",
    "ironmasters": "ironmaster",
    "ironmongers": "ironmonger",
    "irons": "iron",
    "irradiances": "irradiance",
    "irradiated": "irradiate",
    "irradiates": "irradiate",
    "irradiating": "irradiate",
    "irradiations": "irradiation",
    "irrationalities": "irrationality",
    "irregularities": "irregularity",
    "irregulars": "irregular",
    "irrelevances": "irrelevance",
    "irrelevancies": "irrelevancy",
    "irreversibilities": "irreversibility",
    "irrigated": "irrigate",
    "irrigates": "irrigate",
    "irrigating": "irrigate",
    "irrigations": "irrigation",
    "irritabilities": "irritability",
    "irritancies": "irritancy",
    "irritants": "irritant",
    "irritated": "irritate",
    "irritates": "irritate",
    "irritating": "irritate",
    "irritations": "irritation",
    "irruptions": "irruption",
    "is": "be",
    "ischaemias": "ischaemia",
    "ischemias": "ischemia",
    "islanders": "islander",
    "islands": "island",
    "isles": "isle",
    "islets": "islet",
    "isobars": "isobar",
    "isochrones": "isochrone",
    "isocyanates": "isocyanate",
    "isoenzymes": "isoenzyme",
    "isoflavones": "isoflavone",
    "isoforms": "isoform",
    "isolated": "isolate",
    "isolates": "isolate",
    "isolating": "isolate",
    "isolationists": "isolationist",
    "isolations": "isolation",
    "isolators": "isolator",
    "isomerisms": "isomerism",
    "isomers": "isomer",
    "isometrics": "isometric",
    "isometries": "isometry",
    "isomorphisms": "isomorphism",
    "isoprenes": "isoprene",
    "isoprenoids": "isoprenoid",
    "isopropyls": "isopropyl",
    "isotherms": "isotherm",
    "isothiocyanates": "isothiocyanate",
    "isotopes": "isotope",
    "isotypes": "isotype",
    "isozymes": "isozyme",
    "isself": "himself",
    "issuances": "issuance",
    "issued": "issue",
    "issuers": "issuer",
    "issues": "issue",
    "issuing": "issue",
    "isthmi": "isthmus",
    "isthmuses": "isthmus",
    "italicised": "italicise",
    "italicises": "italicise",
    "italicising": "italicise",
    "italicized": "italicize",
    "italicizes": "italicize",
    "italicizing": "italicize",
    "itched": "itch",
    "itches": "itch",
    "itchier": "itchy",
    "itchiest": "itchy",
    "itching": "itch",
    "itemised": "itemise",
    "itemises": "itemise",
    "itemising": "itemise",
    "itemized": "itemize",
    "itemizes": "itemize",
    "itemizing": "itemize",
    "items": "item",
    "iterated": "iterate",
    "iterates": "iterate",
    "iterating": "iterate",
    "iterations": "iteration",
    "iters": "iter",
    "itinera": "iter",
    "itinerants": "itinerant",
    "itineraries": "itinerary",
    "ivies": "ivy",
    "ivories": "ivory",
    "jabbed": "jab",
    "jabbered": "jabber",
    "jabbering": "jabber",
    "jabbers": "jabber",
    "jabbing": "jab",
    "jabs": "jab",
    "jack-in-the-boxes": "jack-in-the-box",
    "jack-knifed": "jack-knife",
    "jack-knifes": "jack-knife",
    "jack-knifing": "jack-knife",
    "jack-knives": "jack-knife",
    "jackals": "jackal",
    "jackasses": "jackass",
    "jackboots": "jackboot",
    "jackdaws": "jackdaw",
    "jacked": "jack",
    "jacketed": "jacket",
    "jacketing": "jacket",
    "jackets": "jacket",
    "jacking": "jack",
    "jackpots": "jackpot",
    "jacks": "jack",
    "jacks-of-all-trades": "jack-of-all-trades",
    "jacquards": "jacquard",
    "jaffas": "jaffa",
    "jagged": "jag",
    "jaggier": "jaggy",
    "jagging": "jag",
    "jags": "jag",
    "jaguars": "jaguar",
    "jailbirds": "jailbird",
    "jailbreaks": "jailbreak",
    "jailed": "jail",
    "jailers": "jailer",
    "jailing": "jail",
    "jails": "jail",
    "jalapenos": "jalapeno",
    "jalopies": "jalopy",
    "jam-jars": "jam-jar",
    "jamborees": "jamboree",
    "jambs": "jamb",
    "jammed": "jam",
    "jammers": "jammer",
    "jammier": "jammy",
    "jammiest": "jammy",
    "jamming": "jam",
    "jams": "jam",
    "jangled": "jangle",
    "jangles": "jangle",
    "jangling": "jangle",
    "janitors": "janitor",
    "japonicas": "japonica",
    "japs": "jap",
    "jargons": "jargon",
    "jarred": "jar",
    "jarring": "jar",
    "jars": "jar",
    "jasmines": "jasmine",
    "jasmins": "jasmin",
    "jaundices": "jaundice",
    "jaunted": "jaunt",
    "jauntier": "jaunty",
    "jauntiest": "jaunty",
    "jaunting": "jaunt",
    "jaunts": "jaunt",
    "javas": "java",
    "javelins": "javelin",
    "jawbones": "jawbone",
    "jawed": "jaw",
    "jawing": "jaw",
    "jaws": "jaw",
    "jays": "jay",
    "jaywalkers": "jaywalker",
    "jazzed": "jazz",
    "jazzes": "jazz",
    "jazzier": "jazzy",
    "jazziest": "jazzy",
    "jazzing": "jazz",
    "jazzmen": "jazzman",
    "jealousies": "jealousy",
    "jeeps": "jeep",
    "jeered": "jeer",
    "jeering": "jeer",
    "jeers": "jeer",
    "jejuna": "jejunum",
    "jejunums": "jejunum",
    "jellies": "jelly",
    "jellyfishes": "jellyfish",
    "jemmies": "jemmy",
    "jennies": "jenny",
    "jeopardies": "jeopardy",
    "jeopardised": "jeopardise",
    "jeopardises": "jeopardise",
    "jeopardising": "jeopardise",
    "jeopardized": "jeopardize",
    "jeopardizes": "jeopardize",
    "jeopardizing": "jeopardize",
    "jerked": "jerk",
    "jerkier": "jerky",
    "jerkiest": "jerky",
    "jerking": "jerk",
    "jerkins": "jerkin",
    "jerks": "jerk",
    "jerry-building": "jerry-build",
    "jerry-builds": "jerry-build",
    "jerry-built": "jerry-build",
    "jerseys": "jersey",
    "jested": "jest",
    "jesters": "jester",
    "jesting": "jest",
    "jests": "jest",
    "jet-lags": "jet-lag",
    "jetlags": "jetlag",
    "jets": "jet",
    "jetted": "jet",
    "jetties": "jetty",
    "jetting": "jet",
    "jettisoned": "jettison",
    "jettisoning": "jettison",
    "jettisons": "jettison",
    "jewelers": "jeweler",
    "jewellers": "jeweller",
    "jewels": "jewel",
    "jibbed": "jib",
    "jibbing": "jib",
    "jibed": "jibe",
    "jibes": "jibe",
    "jibing": "jibe",
    "jibs": "jib",
    "jiffies": "jiffy",
    "jig-saws": "jig-saw",
    "jigged": "jig",
    "jiggers": "jigger",
    "jigging": "jig",
    "jiggled": "jiggle",
    "jiggles": "jiggle",
    "jiggling": "jiggle",
    "jigs": "jig",
    "jigsawed": "jigsaw",
    "jigsawing": "jigsaw",
    "jigsawn": "jigsaw",
    "jigsaws": "jigsaw",
    "jilted": "jilt",
    "jilting": "jilt",
    "jilts": "jilt",
    "jimmies": "jimmy",
    "jingled": "jingle",
    "jingles": "jingle",
    "jingling": "jingle",
    "jinked": "jink",
    "jinking": "jink",
    "jinks": "jink",
    "jinxed": "jinx",
    "jinxes": "jinx",
    "jinxing": "jinx",
    "jittered": "jitter",
    "jittering": "jitter",
    "jitters": "jitter",
    "jived": "jive",
    "jives": "jive",
    "jiving": "jive",
    "jobbed": "job",
    "jobbing": "job",
    "jobholders": "jobholder",
    "jobs": "job",
    "jobseekers": "jobseeker",
    "jockeyed": "jockey",
    "jockeying": "jockey",
    "jockeys": "jockey",
    "jocks": "jock",
    "jockstraps": "jockstrap",
    "joes": "joe",
    "jogged": "jog",
    "joggers": "jogger",
    "jogging": "jog",
    "joggled": "joggle",
    "joggles": "joggle",
    "joggling": "joggle",
    "jogs": "jog",
    "joined": "join",
    "joineries": "joinery",
    "joiners": "joiner",
    "joining": "join",
    "joins": "join",
    "jointed": "joint",
    "jointing": "joint",
    "joints": "joint",
    "joists": "joist",
    "jojobas": "jojoba",
    "joked": "joke",
    "jokers": "joker",
    "jokes": "joke",
    "joking": "joke",
    "jollied": "jolly",
    "jollier": "jolly",
    "jollies": "jolly",
    "jolliest": "jolly",
    "jollying": "jolly",
    "jolted": "jolt",
    "joltier": "jolty",
    "jolting": "jolt",
    "jolts": "jolt",
    "josses": "joss",
    "jostled": "jostle",
    "jostles": "jostle",
    "jostling": "jostle",
    "jots": "jot",
    "jotted": "jot",
    "jotters": "jotter",
    "jotting": "jot",
    "jottings": "jotting",
    "joules": "joule",
    "journalists": "journalist",
    "journals": "journal",
    "journeyed": "journey",
    "journeying": "journey",
    "journeymen": "journeyman",
    "journeys": "journey",
    "jours": "jour",
    "jousted": "joust",
    "jousting": "joust",
    "jousts": "joust",
    "jovialities": "joviality",
    "jowlier": "jowly",
    "jowls": "jowl",
    "joyridden": "joyride",
    "joyrides": "joyride",
    "joyriding": "joyride",
    "joyrode": "joyride",
    "joys": "joy",
    "joysticks": "joystick",
    "jubilations": "jubilation",
    "jubilees": "jubilee",
    "juddered": "judder",
    "juddering": "judder",
    "judders": "judder",
    "judged": "judge",
    "judgements": "judgement",
    "judges": "judge",
    "judging": "judge",
    "judgments": "judgment",
    "judiciaries": "judiciary",
    "juggernauts": "juggernaut",
    "juggled": "juggle",
    "jugglers": "juggler",
    "juggles": "juggle",
    "juggling": "juggle",
    "jugs": "jug",
    "jugulars": "jugular",
    "juiced": "juice",
    "juices": "juice",
    "juicier": "juicy",
    "juiciest": "juicy",
    "juicing": "juice",
    "jukeboxes": "jukebox",
    "julias": "julia",
    "jumbled": "jumble",
    "jumbles": "jumble",
    "jumbling": "jumble",
    "jumbos": "jumbo",
    "jumped": "jump",
    "jumpers": "jumper",
    "jumpier": "jumpy",
    "jumpiest": "jumpy",
    "jumping": "jump",
    "jumps": "jump",
    "junctions": "junction",
    "junctures": "juncture",
    "jungles": "jungle",
    "junglier": "jungly",
    "juniors": "junior",
    "junipers": "juniper",
    "junk-shops": "junk-shop",
    "junked": "junk",
    "junkets": "junket",
    "junkies": "junky",
    "junking": "junk",
    "junks": "junk",
    "junkshops": "junkshop",
    "juns": "jun",
    "juntas": "junta",
    "juries": "jury",
    "jurisdictions": "jurisdiction",
    "jurists": "jurist",
    "jurors": "juror",
    "jurymen": "juryman",
    "jus": "ju",
    "juster": "just",
    "justest": "just",
    "justices": "justice",
    "justifications": "justification",
    "justified": "justify",
    "justifies": "justify",
    "justifying": "justify",
    "juts": "jut",
    "jutted": "jut",
    "jutting": "jut",
    "juveniles": "juvenile",
    "juxtaposed": "juxtapose",
    "juxtaposes": "juxtapose",
    "juxtaposing": "juxtapose",
    "juxtapositions": "juxtaposition",
    "kaffirs": "kaffir",
    "kafirs": "kafir",
    "kaisers": "kaiser",
    "kaleidoscopes": "kaleidoscope",
    "kales": "kale",
    "kamikazes": "kamikaze",
    "kanamycins": "kanamycin",
    "kangaroos": "kangaroo",
    "kans": "kan",
    "kappas": "kappa",
    "karsts": "karst",
    "karyotypes": "karyotype",
    "karyotypings": "karyotyping",
    "kas": "ka",
    "katas": "kata",
    "kats": "kat",
    "kavas": "kava",
    "kayakers": "kayaker",
    "kayaks": "kayak",
    "kbps": "kbp",
    "kbs": "kb",
    "kcals": "kcal",
    "kcs": "kc",
    "kds": "kd",
    "kebabs": "kebab",
    "keeled": "keel",
    "keeling": "keel",
    "keels": "keel",
    "keened": "keen",
    "keener": "keen",
    "keenest": "keen",
    "keening": "keen",
    "keens": "keen",
    "keepers": "keeper",
    "keeping": "keep",
    "keeps": "keep",
    "keepsakes": "keepsake",
    "kegs": "keg",
    "keloids": "keloid",
    "kelps": "kelp",
    "kenned": "ken",
    "kenneled": "kennel",
    "kenneling": "kennel",
    "kennelled": "kennel",
    "kennelling": "kennel",
    "kennels": "kennel",
    "kenning": "ken",
    "kens": "ken",
    "kented": "kent",
    "kenting": "kent",
    "kents": "kent",
    "kept": "keep",
    "keratinocytes": "keratinocyte",
    "keratins": "keratin",
    "keratitides": "keratitis",
    "keratoses": "keratosis",
    "kerbs": "kerb",
    "kerchiefs": "kerchief",
    "kerchieves": "kerchief",
    "kerfuffles": "kerfuffle",
    "kerneled": "kernel",
    "kerneling": "kernel",
    "kernelled": "kernel",
    "kernelling": "kernel",
    "kernels": "kernel",
    "kerosenes": "kerosene",
    "kestrels": "kestrel",
    "ketamines": "ketamine",
    "ketches": "ketch",
    "ketoacidoses": "ketoacidosis",
    "ketoconazoles": "ketoconazole",
    "ketones": "ketone",
    "ketoprofens": "ketoprofen",
    "ketoses": "ketosis",
    "kettledrums": "kettledrum",
    "kettles": "kettle",
    "key-words": "key-word",
    "keyboarded": "keyboard",
    "keyboarding": "keyboard",
    "keyboards": "keyboard",
    "keyed": "key",
    "keyholes": "keyhole",
    "keying": "key",
    "keynotes": "keynote",
    "keypads": "keypad",
    "keypresses": "keypress",
    "keyrings": "keyring",
    "keys": "key",
    "keystones": "keystone",
    "keystrokes": "keystroke",
    "keywords": "keyword",
    "kgs": "kg",
    "khakier": "khaki",
    "khakiest": "khaki",
    "khans": "khan",
    "khs": "kh",
    "khzs": "khz",
    "kibbles": "kibble",
    "kibbutzes": "kibbutz",
    "kibbutzim": "kibbutz",
    "kick-offed": "kick-off",
    "kick-offing": "kick-off",
    "kick-offs": "kick-off",
    "kick-starts": "kick-start",
    "kickbacks": "kickback",
    "kicked": "kick",
    "kickers": "kicker",
    "kicking": "kick",
    "kickoffs": "kickoff",
    "kicks": "kick",
    "kidded": "kid",
    "kiddies": "kiddie",
    "kidding": "kid",
    "kidnaped": "kidnap",
    "kidnaping": "kidnap",
    "kidnapped": "kidnap",
    "kidnappers": "kidnapper",
    "kidnapping": "kidnap",
    "kidnappings": "kidnapping",
    "kidnaps": "kidnap",
    "kidneys": "kidney",
    "kids": "kid",
    "killed": "kill",
    "killers": "killer",
    "killing": "kill",
    "killings": "killing",
    "killjoys": "killjoy",
    "kills": "kill",
    "kilned": "kiln",
    "kilning": "kiln",
    "kilns": "kiln",
    "kilobits": "kilobit",
    "kilocalories": "kilocalorie",
    "kilogrammes": "kilogramme",
    "kilograms": "kilogram",
    "kiloliters": "kiloliter",
    "kilometers": "kilometer",
    "kilometres": "kilometre",
    "kilos": "kilo",
    "kilowatt-hours": "kilowatt-hour",
    "kilowatts": "kilowatt",
    "kilts": "kilt",
    "kimonos": "kimono",
    "kinases": "kinase",
    "kinder": "kind",
    "kindergartens": "kindergarten",
    "kindest": "kind",
    "kindled": "kindle",
    "kindles": "kindle",
    "kindlier": "kindly",
    "kindliest": "kindly",
    "kindling": "kindle",
    "kindnesses": "kindness",
    "kindreds": "kindred",
    "kinds": "kind",
    "kinesins": "kinesin",
    "kinetics": "kinetic",
    "kingdoms": "kingdom",
    "kingfishers": "kingfisher",
    "kingpins": "kingpin",
    "kings": "king",
    "kinked": "kink",
    "kinkier": "kinky",
    "kinkiest": "kinky",
    "kinking": "kink",
    "kinks": "kink",
    "kinships": "kinship",
    "kinsmen": "kinsman",
    "kinswomen": "kinswoman",
    "kiosks": "kiosk",
    "kipped": "kip",
    "kippers": "kipper",
    "kipping": "kip",
    "kips": "kip",
    "kirks": "kirk",
    "kissed": "kiss",
    "kisses": "kiss",
    "kissing": "kiss",
    "kitbags": "kitbag",
    "kitchenettes": "kitchenette",
    "kitchens": "kitchen",
    "kited": "kite",
    "kites": "kite",
    "kiting": "kite",
    "kits": "kit",
    "kitted": "kit",
    "kittened": "kitten",
    "kittening": "kitten",
    "kittens": "kitten",
    "kitties": "kitty",
    "kitting": "kit",
    "kittiwakes": "kittiwake",
    "kiwis": "kiwi",
    "klaxons": "klaxon",
    "kleptomaniacs": "kleptomaniac",
    "kms": "km",
    "knackers": "knacker",
    "knacks": "knack",
    "knapped": "knap",
    "knapping": "knap",
    "knaps": "knap",
    "knapsacks": "knapsack",
    "knapweeds": "knapweed",
    "knaves": "knave",
    "kneaded": "knead",
    "kneading": "knead",
    "kneads": "knead",
    "kneecapped": "kneecap",
    "kneecapping": "kneecap",
    "kneecaps": "kneecap",
    "kneed": "knee",
    "kneeing": "knee",
    "kneeled": "kneel",
    "kneelers": "kneeler",
    "kneeling": "kneel",
    "kneels": "kneel",
    "knees": "knee",
    "knees-ups": "knees-up",
    "knells": "knell",
    "knelt": "kneel",
    "knew": "know",
    "knife-edges": "knife-edge",
    "knifed": "knife",
    "knifes": "knife",
    "knifing": "knife",
    "knighted": "knight",
    "knighthoods": "knighthood",
    "knighting": "knight",
    "knights": "knight",
    "knits": "knit",
    "knitted": "knit",
    "knitters": "knitter",
    "knitting": "knit",
    "knittings": "knitting",
    "knitwears": "knitwear",
    "knives": "knife",
    "knobbed": "knob",
    "knobbing": "knob",
    "knobblier": "knobbly",
    "knobs": "knob",
    "knock-ons": "knock-on",
    "knock-ups": "knock-up",
    "knocked": "knock",
    "knockers": "knocker",
    "knocking": "knock",
    "knockouts": "knockout",
    "knocks": "knock",
    "knolls": "knoll",
    "knots": "knot",
    "knotted": "knot",
    "knottier": "knotty",
    "knottiest": "knotty",
    "knotting": "knot",
    "knotweeds": "knotweed",
    "know-alls": "know-all",
    "know-it-alls": "know-it-all",
    "knowers": "knower",
    "knowing": "know",
    "knowledge-bases": "knowledge-base",
    "knowledgebases": "knowledgebase",
    "knowledges": "knowledge",
    "known": "know",
    "knowns": "known",
    "knows": "know",
    "knuckle-dusters": "knuckle-duster",
    "knuckled": "knuckle",
    "knuckles": "knuckle",
    "knuckling": "knuckle",
    "koalas": "koala",
    "koans": "koan",
    "kois": "koi",
    "kolas": "kola",
    "koras": "kora",
    "kow-towed": "kow-tow",
    "kow-towing": "kow-tow",
    "kow-tows": "kow-tow",
    "kowtowed": "kowtow",
    "kowtowing": "kowtow",
    "kowtows": "kowtow",
    "kraals": "kraal",
    "krills": "krill",
    "kroner": "krone",
    "kronor": "krona",
    "krugerrands": "krugerrand",
    "ks": "k",
    "kudos": "kudo",
    "kudus": "kudu",
    "kumquats": "kumquat",
    "kurtoses": "kurtosis",
    "labeled": "label",
    "labeling": "label",
    "labelled": "label",
    "labelling": "label",
    "labels": "label",
    "labia": "labium",
    "labilities": "lability",
    "laboratories": "laboratory",
    "labored": "labor",
    "laborers": "laborer",
    "laboring": "labor",
    "labors": "labor",
    "laboured": "labour",
    "labourers": "labourer",
    "labouring": "labour",
    "labours": "labour",
    "labs": "lab",
    "laburnums": "laburnum",
    "labyrinths": "labyrinth",
    "lace-ups": "lace-up",
    "laced": "lace",
    "lacerated": "lacerate",
    "lacerates": "lacerate",
    "lacerating": "lacerate",
    "lacerations": "laceration",
    "laces": "lace",
    "lacewings": "lacewing",
    "lacier": "lacy",
    "laciest": "lacy",
    "lacing": "lace",
    "lacked": "lack",
    "lackeys": "lackey",
    "lacking": "lack",
    "lacks": "lack",
    "lacquered": "lacquer",
    "lacquering": "lacquer",
    "lacquers": "lacquer",
    "lacta": "lac",
    "lactases": "lactase",
    "lactated": "lactate",
    "lactates": "lactate",
    "lactating": "lactate",
    "lactations": "lactation",
    "lactobacilli": "lactobacillus",
    "lactoferrins": "lactoferrin",
    "lactones": "lactone",
    "lacunae": "lacuna",
    "lacunas": "lacuna",
    "laddered": "ladder",
    "laddering": "ladder",
    "ladders": "ladder",
    "laddies": "laddie",
    "laded": "lade",
    "laden": "lade",
    "lades": "lade",
    "ladies": "lady",
    "ladies-in-waiting": "lady-in-waiting",
    "lading": "lade",
    "ladings": "lading",
    "ladled": "ladle",
    "ladles": "ladle",
    "ladling": "ladle",
    "lads": "lad",
    "lady-in-waitings": "lady-in-waiting",
    "lady-killers": "lady-killer",
    "ladybirds": "ladybird",
    "ladyships": "ladyship",
    "lagers": "lager",
    "laggards": "laggard",
    "lagged": "lag",
    "lagging": "lag",
    "lagoons": "lagoon",
    "lags": "lag",
    "lagunas": "laguna",
    "laid": "lay",
    "lain": "lie",
    "lairds": "laird",
    "lairs": "lair",
    "laities": "laity",
    "lakes": "lake",
    "lakhs": "lakh",
    "lamas": "lama",
    "lambasted": "lambaste",
    "lambastes": "lambaste",
    "lambasting": "lambaste",
    "lambasts": "lambast",
    "lambdas": "lambda",
    "lambed": "lamb",
    "lamberts": "lambert",
    "lambing": "lamb",
    "lambs": "lamb",
    "lambskins": "lambskin",
    "lamellae": "lamella",
    "lamellas": "lamella",
    "lamenesses": "lameness",
    "lamentations": "lamentation",
    "lamented": "lament",
    "lamenting": "lament",
    "laments": "lament",
    "lamer": "lame",
    "lamest": "lame",
    "laminae": "lamina",
    "laminas": "lamina",
    "laminated": "laminate",
    "laminates": "laminate",
    "laminating": "laminate",
    "laminations": "lamination",
    "laminators": "laminator",
    "laminins": "laminin",
    "laminitides": "laminitis",
    "lamins": "lamin",
    "lammed": "lam",
    "lamming": "lam",
    "lamp-posts": "lamp-post",
    "lampooned": "lampoon",
    "lampooning": "lampoon",
    "lampoons": "lampoon",
    "lampposts": "lamppost",
    "lampreys": "lamprey",
    "lamps": "lamp",
    "lampshades": "lampshade",
    "lams": "lam",
    "lanais": "lanai",
    "lanced": "lance",
    "lancers": "lancer",
    "lances": "lance",
    "lancets": "lancet",
    "lancing": "lance",
    "landaus": "landau",
    "landed": "land",
    "landers": "lander",
    "landfalls": "landfall",
    "landfills": "landfill",
    "landforms": "landform",
    "landholders": "landholder",
    "landholdings": "landholding",
    "landing": "land",
    "landings": "landing",
    "landladies": "landlady",
    "landlords": "landlord",
    "landlubbers": "landlubber",
    "landmarks": "landmark",
    "landmasses": "landmass",
    "landmines": "landmine",
    "landowners": "landowner",
    "landraces": "landrace",
    "lands": "land",
    "landscaped": "landscape",
    "landscapers": "landscaper",
    "landscapes": "landscape",
    "landscaping": "landscape",
    "landslides": "landslide",
    "landslips": "landslip",
    "lanes": "lane",
    "languages": "language",
    "languished": "languish",
    "languishes": "languish",
    "languishing": "languish",
    "languors": "languor",
    "langurs": "langur",
    "lanker": "lank",
    "lankest": "lank",
    "lankier": "lanky",
    "lankiest": "lanky",
    "lanterns": "lantern",
    "lanthanides": "lanthanide",
    "lanyards": "lanyard",
    "laparoscopes": "laparoscope",
    "laparoscopies": "laparoscopy",
    "laparotomies": "laparotomy",
    "lapdogs": "lapdog",
    "lapels": "lapel",
    "lapped": "lap",
    "lapping": "lap",
    "laps": "lap",
    "lapsed": "lapse",
    "lapses": "lapse",
    "lapsing": "lapse",
    "laptops": "laptop",
    "lapwings": "lapwing",
    "larcenies": "larceny",
    "larches": "larch",
    "larded": "lard",
    "larders": "larder",
    "larding": "lard",
    "lards": "lard",
    "larger": "large",
    "largest": "large",
    "larked": "lark",
    "larking": "lark",
    "larks": "lark",
    "lars": "lar",
    "larvae": "larva",
    "larvas": "larva",
    "laryngectomies": "laryngectomy",
    "larynges": "larynx",
    "laryngitides": "laryngitis",
    "larynxes": "larynx",
    "lased": "lase",
    "lasers": "laser",
    "lases": "lase",
    "lashed": "lash",
    "lashes": "lash",
    "lashing": "lash",
    "lashings": "lashing",
    "lasing": "lase",
    "lasses": "lass",
    "lassies": "lassie",
    "lassoed": "lasso",
    "lassoes": "lasso",
    "lassoing": "lasso",
    "lassos": "lasso",
    "lasted": "last",
    "lasting": "last",
    "lasts": "last",
    "latched": "latch",
    "latches": "latch",
    "latching": "latch",
    "latchkeys": "latchkey",
    "latecomers": "latecomer",
    "latencies": "latency",
    "later": "late",
    "lateralisations": "lateralisation",
    "lateralities": "laterality",
    "laterals": "lateral",
    "laterites": "laterite",
    "latest": "late",
    "latexes": "latex",
    "lathed": "lathe",
    "lathered": "lather",
    "lathering": "lather",
    "lathers": "lather",
    "lathes": "lathe",
    "lathing": "lathe",
    "laths": "lath",
    "latices": "latex",
    "latitudes": "latitude",
    "latrines": "latrine",
    "lats": "lat",
    "latters": "latter",
    "lattes": "latte",
    "lattices": "lattice",
    "latticeworks": "latticework",
    "lauded": "laud",
    "lauding": "laud",
    "lauds": "laud",
    "laughed": "laugh",
    "laughing": "laugh",
    "laughs": "laugh",
    "laughters": "laughter",
    "launched": "launch",
    "launchers": "launcher",
    "launches": "launch",
    "launching": "launch",
    "launchings": "launching",
    "laundered": "launder",
    "launderers": "launderer",
    "launderettes": "launderette",
    "laundering": "launder",
    "launderings": "laundering",
    "launders": "launder",
    "laundresses": "laundress",
    "laundrettes": "laundrette",
    "laundries": "laundry",
    "lauras": "laura",
    "laureates": "laureate",
    "laurels": "laurel",
    "lauryls": "lauryl",
    "lavas": "lava",
    "lavatories": "lavatory",
    "lavenders": "lavender",
    "lavers": "laver",
    "lavished": "lavish",
    "lavishes": "lavish",
    "lavishing": "lavish",
    "law-breakers": "law-breaker",
    "law-makers": "law-maker",
    "lawbreakers": "lawbreaker",
    "lawmakers": "lawmaker",
    "lawmen": "lawman",
    "lawnmowers": "lawnmower",
    "lawns": "lawn",
    "laws": "law",
    "lawsuits": "lawsuit",
    "lawyers": "lawyer",
    "laxatives": "laxative",
    "laxer": "lax",
    "laxest": "lax",
    "laxities": "laxity",
    "lay": "lie",
    "lay-bied": "lay-by",
    "lay-bies": "lay-by",
    "lay-bying": "lay-by",
    "lay-bys": "lay-by",
    "lay-offs": "lay-off",
    "lay-outs": "lay-out",
    "layabouts": "layabout",
    "layered": "layer",
    "layering": "layer",
    "layers": "layer",
    "layettes": "layette",
    "laying": "lay",
    "laymen": "layman",
    "layoffs": "layoff",
    "layouts": "layout",
    "layovers": "layover",
    "laypeople": "layperson",
    "laypersons": "layperson",
    "lays": "lay",
    "lazed": "laze",
    "lazes": "laze",
    "lazier": "lazy",
    "laziest": "lazy",
    "lazing": "laze",
    "lbs": "lb",
    "lds": "ld",
    "leachates": "leachate",
    "leached": "leach",
    "leaches": "leach",
    "leaching": "leach",
    "lead-ins": "lead-in",
    "leaders": "leader",
    "leaderships": "leadership",
    "leading": "lead",
    "leads": "lead",
    "leafed": "leaf",
    "leafhoppers": "leafhopper",
    "leafier": "leafy",
    "leafiest": "leafy",
    "leafing": "leaf",
    "leafleted": "leaflet",
    "leafleting": "leaflet",
    "leaflets": "leaflet",
    "leafletted": "leaflet",
    "leafletting": "leaflet",
    "leafs": "leaf",
    "leagues": "league",
    "leakages": "leakage",
    "leaked": "leak",
    "leakier": "leaky",
    "leakiest": "leaky",
    "leaking": "leak",
    "leaks": "leak",
    "lean-toed": "lean-to",
    "lean-toing": "lean-to",
    "lean-tos": "lean-to",
    "leaned": "lean",
    "leaner": "lean",
    "leanest": "lean",
    "leaning": "lean",
    "leanings": "leaning",
    "leans": "lean",
    "leant": "lean",
    "leap-frogged": "leap-frog",
    "leap-frogging": "leap-frog",
    "leap-frogs": "leap-frog",
    "leaped": "leap",
    "leapfrogged": "leapfrog",
    "leapfrogging": "leapfrog",
    "leapfrogs": "leapfrog",
    "leaping": "leap",
    "leaps": "leap",
    "leapt": "leap",
    "learned": "learn",
    "learners": "learner",
    "learning": "learn",
    "learnings": "learning",
    "learns": "learn",
    "learnt": "learn",
    "leas": "lea",
    "leased": "lease",
    "leaseholders": "leaseholder",
    "leases": "lease",
    "leashes": "leash",
    "leasing": "lease",
    "least": "little",
    "leatherbacks": "leatherback",
    "leathered": "leather",
    "leathering": "leather",
    "leatherjackets": "leatherjacket",
    "leathers": "leather",
    "leavened": "leaven",
    "leavening": "leaven",
    "leavens": "leaven",
    "leavers": "leaver",
    "leaves": "leave",
    "leaving": "leave",
    "lebaneses": "lebanese",
    "lechers": "lecher",
    "lecithins": "lecithin",
    "lecterns": "lectern",
    "lectins": "lectin",
    "lectionaries": "lectionary",
    "lectured": "lecture",
    "lecturers": "lecturer",
    "lectures": "lecture",
    "lectureships": "lectureship",
    "lecturing": "lecture",
    "led": "lead",
    "ledgers": "ledger",
    "ledges": "ledge",
    "leeched": "leech",
    "leeches": "leech",
    "leeching": "leech",
    "leeks": "leek",
    "leered": "leer",
    "leering": "leer",
    "leers": "leer",
    "lees": "lee",
    "leeways": "leeway",
    "left": "leave",
    "left-handeds": "left-handed",
    "left-handers": "left-hander",
    "left-overs": "left-over",
    "left-wingers": "left-winger",
    "lefties": "lefty",
    "leftists": "leftist",
    "leftovers": "leftover",
    "lefts": "left",
    "legacies": "legacy",
    "legalisations": "legalisation",
    "legalised": "legalise",
    "legalises": "legalise",
    "legalising": "legalise",
    "legalisms": "legalism",
    "legalities": "legality",
    "legalizations": "legalization",
    "legalized": "legalize",
    "legalizes": "legalize",
    "legalizing": "legalize",
    "legatees": "legatee",
    "legates": "legate",
    "legations": "legation",
    "legends": "legend",
    "leges": "lex",
    "legged": "leg",
    "leggier": "leggy",
    "leggiest": "leggy",
    "legging": "leg",
    "leggings": "legging",
    "legibilities": "legibility",
    "legionellae": "legionella",
    "legionellas": "legionella",
    "legionelloses": "legionellosis",
    "legionnaires": "legionnaire",
    "legions": "legion",
    "legislated": "legislate",
    "legislates": "legislate",
    "legislating": "legislate",
    "legislations": "legislation",
    "legislators": "legislator",
    "legislatures": "legislature",
    "legitimated": "legitimate",
    "legitimates": "legitimate",
    "legitimating": "legitimate",
    "legitimised": "legitimise",
    "legitimises": "legitimise",
    "legitimising": "legitimise",
    "legitimized": "legitimize",
    "legitimizes": "legitimize",
    "legitimizing": "legitimize",
    "legs": "leg",
    "legumes": "legume",
    "lei": "leu",
    "leiomyomas": "leiomyoma",
    "leiomyomata": "leiomyoma",
    "leishmaniases": "leishmaniasis",
    "leisures": "leisure",
    "leks": "lek",
    "lemmata": "lemma",
    "lemmings": "lemming",
    "lemonades": "lemonade",
    "lemons": "lemon",
    "lemurs": "lemur",
    "lenders": "lender",
    "lending": "lend",
    "lends": "lend",
    "lengthened": "lengthen",
    "lengthening": "lengthen",
    "lengthens": "lengthen",
    "lengthier": "lengthy",
    "lengthiest": "lengthy",
    "lengths": "length",
    "lenses": "lense",
    "lent": "lend",
    "lenticels": "lenticel",
    "lentils": "lentil",
    "lentiviruses": "lentivirus",
    "leopards": "leopard",
    "leotards": "leotard",
    "lepers": "leper",
    "leprechauns": "leprechaun",
    "leprosies": "leprosy",
    "lepta": "lepton",
    "leptins": "leptin",
    "leptons": "lepton",
    "leptospiroses": "leptospirosis",
    "lesbians": "lesbian",
    "lesions": "lesion",
    "less": "little",
    "lessees": "lessee",
    "lessened": "lessen",
    "lessening": "lessen",
    "lessens": "lessen",
    "lessons": "lesson",
    "lessors": "lessor",
    "let-ups": "let-up",
    "letdowns": "letdown",
    "lethalities": "lethality",
    "lethargies": "lethargy",
    "lets": "let",
    "letter-bombs": "letter-bomb",
    "letterboxes": "letterbox",
    "lettered": "letter",
    "letterforms": "letterform",
    "letterheads": "letterhead",
    "lettering": "letter",
    "letterpresses": "letterpress",
    "letters": "letter",
    "letting": "let",
    "lettuces": "lettuce",
    "leucines": "leucine",
    "leucocytes": "leucocyte",
    "leucopenias": "leucopenia",
    "leukaemias": "leukaemia",
    "leukemias": "leukemia",
    "leukocytes": "leukocyte",
    "leukotrienes": "leukotriene",
    "levees": "levee",
    "leveled": "level",
    "leveling": "level",
    "levelled": "level",
    "leveller": "level",
    "levellest": "level",
    "levelling": "level",
    "levels": "level",
    "leveraged": "leverage",
    "leverages": "leverage",
    "leveraging": "leverage",
    "levered": "lever",
    "leverets": "leveret",
    "levering": "lever",
    "levers": "lever",
    "leviathans": "leviathan",
    "levied": "levy",
    "levies": "levy",
    "levitated": "levitate",
    "levitates": "levitate",
    "levitating": "levitate",
    "levitations": "levitation",
    "levities": "levity",
    "levonorgestrels": "levonorgestrel",
    "levying": "levy",
    "lewder": "lewd",
    "lewdest": "lewd",
    "lexemes": "lexeme",
    "lexica": "lexicon",
    "lexicographers": "lexicographer",
    "lexicons": "lexicon",
    "lexises": "lexis",
    "leys": "ley",
    "liabilities": "liability",
    "liaised": "liaise",
    "liaises": "liaise",
    "liaising": "liaise",
    "liaisons": "liaison",
    "lianas": "liana",
    "liars": "liar",
    "libations": "libation",
    "libeled": "libel",
    "libeling": "libel",
    "libelled": "libel",
    "libelling": "libel",
    "libels": "libel",
    "liberalisations": "liberalisation",
    "liberalised": "liberalise",
    "liberalises": "liberalise",
    "liberalising": "liberalise",
    "liberalities": "liberality",
    "liberalizations": "liberalization",
    "liberalized": "liberalize",
    "liberalizes": "liberalize",
    "liberalizing": "liberalize",
    "liberals": "liberal",
    "liberated": "liberate",
    "liberates": "liberate",
    "liberating": "liberate",
    "liberationists": "liberationist",
    "liberations": "liberation",
    "liberators": "liberator",
    "libers": "liber",
    "libertarians": "libertarian",
    "liberties": "liberty",
    "libertines": "libertine",
    "libidines": "libido",
    "libidos": "libido",
    "librarians": "librarian",
    "libraries": "library",
    "libretti": "libretto",
    "librettists": "librettist",
    "librettos": "libretto",
    "lice": "louse",
    "licenced": "licence",
    "licences": "licence",
    "licencing": "licence",
    "licensed": "license",
    "licensees": "licensee",
    "licenses": "license",
    "licensing": "license",
    "licensors": "licensor",
    "licensures": "licensure",
    "licentiates": "licentiate",
    "lichens": "lichen",
    "licked": "lick",
    "licking": "lick",
    "lickings": "licking",
    "licks": "lick",
    "licorices": "licorice",
    "lidos": "lido",
    "lids": "lid",
    "lied": "lie",
    "lieges": "liege",
    "liens": "lien",
    "lies": "lie",
    "lieutenants": "lieutenant",
    "life-cycles": "life-cycle",
    "life-expectancies": "life-expectancy",
    "life-forms": "life-form",
    "life-jackets": "life-jacket",
    "life-lines": "life-line",
    "life-rafts": "life-raft",
    "life-savers": "life-saver",
    "life-spans": "life-span",
    "life-styles": "life-style",
    "life-times": "life-time",
    "life-worlds": "life-world",
    "lifebelts": "lifebelt",
    "lifeboats": "lifeboat",
    "lifebuoys": "lifebuoy",
    "lifecycles": "lifecycle",
    "lifeguards": "lifeguard",
    "lifejackets": "lifejacket",
    "lifelines": "lifeline",
    "lifers": "lifer",
    "lifes": "life",
    "lifesavers": "lifesaver",
    "lifespans": "lifespan",
    "lifestyles": "lifestyle",
    "lifetimes": "lifetime",
    "lifeworlds": "lifeworld",
    "lift-offs": "lift-off",
    "lift-shafts": "lift-shaft",
    "lifted": "lift",
    "lifters": "lifter",
    "lifting": "lift",
    "lifts": "lift",
    "ligaments": "ligament",
    "ligands": "ligand",
    "ligases": "ligase",
    "ligated": "ligate",
    "ligates": "ligate",
    "ligating": "ligate",
    "ligations": "ligation",
    "ligatures": "ligature",
    "light-weights": "light-weight",
    "light-years": "light-year",
    "lightboxes": "lightbox",
    "lightbulbs": "lightbulb",
    "lighted": "light",
    "lightened": "lighten",
    "lightening": "lighten",
    "lightens": "lighten",
    "lighter": "light",
    "lighters": "lighter",
    "lightest": "light",
    "lighthouses": "lighthouse",
    "lighting": "light",
    "lightings": "lighting",
    "lightnesses": "lightness",
    "lightnings": "lightning",
    "lights": "light",
    "lightships": "lightship",
    "lightweights": "lightweight",
    "lignans": "lignan",
    "lignes": "ligne",
    "lignins": "lignin",
    "liked": "like",
    "likelier": "likely",
    "likeliest": "likely",
    "likelihoods": "likelihood",
    "likened": "liken",
    "likenesses": "likeness",
    "likening": "liken",
    "likens": "liken",
    "likes": "like",
    "liking": "like",
    "likings": "liking",
    "lilacs": "lilac",
    "lilies": "lily",
    "lilted": "lilt",
    "lilting": "lilt",
    "lilts": "lilt",
    "limbered": "limber",
    "limbering": "limber",
    "limbers": "limber",
    "limbos": "limbo",
    "limbs": "limb",
    "limed": "lime",
    "limericks": "limerick",
    "limes": "lime",
    "limestones": "limestone",
    "limeys": "limey",
    "liming": "lime",
    "limitations": "limitation",
    "limited": "limit",
    "limiters": "limiter",
    "limiting": "limit",
    "limits": "limit",
    "limousines": "limousine",
    "limped": "limp",
    "limper": "limp",
    "limpest": "limp",
    "limpets": "limpet",
    "limping": "limp",
    "limps": "limp",
    "lims": "lim",
    "linchpins": "linchpin",
    "linctuses": "linctus",
    "lindens": "linden",
    "line-ups": "line-up",
    "lineae": "linea",
    "lineages": "lineage",
    "lineaments": "lineament",
    "linearisations": "linearisation",
    "linearised": "linearise",
    "linearises": "linearise",
    "linearising": "linearise",
    "linearities": "linearity",
    "linearizations": "linearization",
    "linearized": "linearize",
    "linearizes": "linearize",
    "linearizing": "linearize",
    "lined": "line",
    "linemen": "lineman",
    "linens": "linen",
    "liners": "liner",
    "lines": "line",
    "linesmen": "linesman",
    "lineups": "lineup",
    "linewidths": "linewidth",
    "lingered": "linger",
    "lingering": "linger",
    "lingers": "linger",
    "lingoes": "lingo",
    "lings": "ling",
    "linguae": "lingua",
    "linguistics": "linguistic",
    "linguists": "linguist",
    "liniments": "liniment",
    "lining": "line",
    "linings": "lining",
    "link-ups": "link-up",
    "linkages": "linkage",
    "linked": "link",
    "linkers": "linker",
    "linking": "link",
    "linkmen": "linkman",
    "links": "link",
    "linnets": "linnet",
    "linoleums": "linoleum",
    "linos": "lino",
    "linseeds": "linseed",
    "lintels": "lintel",
    "lionesses": "lioness",
    "lions": "lion",
    "lip-reading": "lip-read",
    "lip-readings": "lip-reading",
    "lip-reads": "lip-read",
    "lipases": "lipase",
    "lipids": "lipid",
    "lipoatrophies": "lipoatrophy",
    "lipodystrophies": "lipodystrophy",
    "lipomas": "lipoma",
    "lipomata": "lipoma",
    "lipopolysaccharides": "lipopolysaccharide",
    "lipoproteins": "lipoprotein",
    "liposomes": "liposome",
    "liposuctions": "liposuction",
    "lipped": "lip",
    "lipping": "lip",
    "lipreaded": "lipread",
    "lipreading": "lipread",
    "lipreadings": "lipreading",
    "lipreads": "lipread",
    "lips": "lip",
    "lipsticks": "lipstick",
    "liquefied": "liquefy",
    "liquefies": "liquefy",
    "liquefying": "liquefy",
    "liqueurs": "liqueur",
    "liquidated": "liquidate",
    "liquidates": "liquidate",
    "liquidating": "liquidate",
    "liquidations": "liquidation",
    "liquidators": "liquidator",
    "liquidised": "liquidise",
    "liquidises": "liquidise",
    "liquidising": "liquidise",
    "liquidized": "liquidize",
    "liquidizers": "liquidizer",
    "liquidizes": "liquidize",
    "liquidizing": "liquidize",
    "liquids": "liquid",
    "liquified": "liquify",
    "liquifies": "liquify",
    "liquifying": "liquify",
    "liquores": "liquor",
    "liquorices": "liquorice",
    "liquors": "liquor",
    "liras": "lira",
    "lire": "lira",
    "lisas": "lisa",
    "lisped": "lisp",
    "lisping": "lisp",
    "lisps": "lisp",
    "listed": "list",
    "listened": "listen",
    "listeners": "listener",
    "listening": "listen",
    "listens": "listen",
    "listerias": "listeria",
    "listers": "lister",
    "listing": "list",
    "listings": "listing",
    "lists": "list",
    "listservers": "listserver",
    "listserves": "listserve",
    "listservs": "listserv",
    "lit": "light",
    "litanies": "litany",
    "literacies": "literacy",
    "literalisms": "literalism",
    "literalists": "literalist",
    "literals": "literal",
    "literates": "literate",
    "literatures": "literature",
    "liters": "liter",
    "lites": "lite",
    "lither": "lithe",
    "lithest": "lithe",
    "lithographed": "lithograph",
    "lithographing": "lithograph",
    "lithographs": "lithograph",
    "lithologies": "lithology",
    "lithospheres": "lithosphere",
    "lithotripsies": "lithotripsy",
    "litigants": "litigant",
    "litigated": "litigate",
    "litigates": "litigate",
    "litigating": "litigate",
    "litigations": "litigation",
    "litigators": "litigator",
    "litres": "litre",
    "litterbugs": "litterbug",
    "littered": "litter",
    "littering": "litter",
    "littermates": "littermate",
    "litters": "litter",
    "littler": "little",
    "littlest": "little",
    "littorals": "littoral",
    "liturgies": "liturgy",
    "lived": "live",
    "livelier": "lively",
    "liveliest": "lively",
    "livelihoods": "livelihood",
    "livened": "liven",
    "livening": "liven",
    "livens": "liven",
    "liver": "live",
    "liveries": "livery",
    "livers": "liver",
    "liverworts": "liverwort",
    "lives": "live",
    "livest": "live",
    "liveweights": "liveweight",
    "lividder": "livid",
    "lividdest": "livid",
    "living": "live",
    "living-rooms": "living-room",
    "livings": "living",
    "livres": "livre",
    "lizards": "lizard",
    "ll": "will",
    "llamas": "llama",
    "loaches": "loach",
    "loaded": "load",
    "loaders": "loader",
    "loading": "load",
    "loadings": "loading",
    "loads": "load",
    "loafed": "loaf",
    "loafers": "loafer",
    "loafing": "loaf",
    "loafs": "loaf",
    "loamier": "loamy",
    "loams": "loam",
    "loaned": "loan",
    "loaning": "loan",
    "loans": "loan",
    "loathed": "loathe",
    "loathes": "loathe",
    "loathing": "loathe",
    "loathlier": "loathly",
    "loaves": "loaf",
    "lobbed": "lob",
    "lobbied": "lobby",
    "lobbies": "lobby",
    "lobbing": "lob",
    "lobbying": "lobby",
    "lobbyings": "lobbying",
    "lobbyists": "lobbyist",
    "lobectomies": "lobectomy",
    "lobes": "lobe",
    "lobotomies": "lobotomy",
    "lobs": "lob",
    "lobsters": "lobster",
    "lobules": "lobule",
    "loca": "locus",
    "locales": "locale",
    "localisations": "localisation",
    "localised": "localise",
    "localises": "localise",
    "localising": "localise",
    "localisms": "localism",
    "localists": "localist",
    "localities": "locality",
    "localizations": "localization",
    "localized": "localize",
    "localizes": "localize",
    "localizing": "localize",
    "locals": "local",
    "located": "locate",
    "locates": "locate",
    "locating": "locate",
    "locations": "location",
    "locators": "locator",
    "lochs": "loch",
    "loci": "locus",
    "lock-outs": "lock-out",
    "locked": "lock",
    "lockers": "locker",
    "lockets": "locket",
    "locking": "lock",
    "lockouts": "lockout",
    "locks": "lock",
    "locksmiths": "locksmith",
    "lockups": "lockup",
    "locoes": "loco",
    "locomotions": "locomotion",
    "locomotives": "locomotive",
    "locos": "loco",
    "locums": "locum",
    "locuses": "locus",
    "locusts": "locust",
    "locutions": "locution",
    "lodes": "lode",
    "lodged": "lodge",
    "lodgements": "lodgement",
    "lodgers": "lodger",
    "lodges": "lodge",
    "lodging": "lodge",
    "lodgings": "lodging",
    "lodgments": "lodgment",
    "lofted": "loft",
    "loftier": "lofty",
    "loftiest": "lofty",
    "lofting": "loft",
    "lofts": "loft",
    "log-ins": "log-in",
    "loganberries": "loganberry",
    "logarithms": "logarithm",
    "logbooks": "logbook",
    "logged": "log",
    "loggerheads": "loggerhead",
    "loggers": "logger",
    "loggias": "loggia",
    "logging": "log",
    "logicians": "logician",
    "logics": "logic",
    "logins": "login",
    "logisticians": "logistician",
    "logistics": "logistic",
    "logos": "logo",
    "logs": "log",
    "loincloths": "loincloth",
    "loins": "loin",
    "loitered": "loiter",
    "loitering": "loiter",
    "loiters": "loiter",
    "lolled": "loll",
    "lollies": "lolly",
    "lolling": "loll",
    "lollipops": "lollipop",
    "lolloped": "lollop",
    "lolloping": "lollop",
    "lollops": "lollop",
    "lolls": "loll",
    "lonelier": "lonely",
    "loneliest": "lonely",
    "loner": "lone",
    "loners": "loner",
    "lonest": "lone",
    "longbows": "longbow",
    "longed": "long",
    "longer": "long",
    "longer-lasting": "long-lasting",
    "longer-lived": "long-lived",
    "longer-term": "long-term",
    "longest": "long",
    "longest-lasting": "long-lasting",
    "longest-lived": "long-lived",
    "longest-running": "long-running",
    "longevities": "longevity",
    "longhorns": "longhorn",
    "longhouses": "longhouse",
    "longing": "long",
    "longings": "longing",
    "longitudes": "longitude",
    "longs": "long",
    "longshoremen": "longshoreman",
    "longues": "longue",
    "loofahs": "loofah",
    "look-alikes": "look-alike",
    "look-outs": "look-out",
    "look-ups": "look-up",
    "lookalikes": "lookalike",
    "looked": "look",
    "lookers": "looker",
    "lookers-on": "looker-on",
    "looking": "look",
    "looking-glasses": "looking-glass",
    "lookouts": "lookout",
    "looks": "look",
    "lookups": "lookup",
    "loomed": "loom",
    "looming": "loom",
    "looms": "loom",
    "loonier": "loony",
    "loonies": "loony",
    "looniest": "loony",
    "loons": "loon",
    "looped": "loop",
    "loopholes": "loophole",
    "loopier": "loopy",
    "looping": "loop",
    "loops": "loop",
    "loos": "loo",
    "loosed": "loose",
    "loosened": "loosen",
    "loosening": "loosen",
    "loosenings": "loosening",
    "loosens": "loosen",
    "looser": "loose",
    "looses": "loose",
    "loosest": "loose",
    "loosing": "loose",
    "looted": "loot",
    "looters": "looter",
    "looting": "loot",
    "loots": "loot",
    "loped": "lope",
    "lopes": "lope",
    "loping": "lope",
    "lopped": "lop",
    "lopping": "lop",
    "lops": "lop",
    "lorded": "lord",
    "lording": "lord",
    "lordlier": "lordly",
    "lordliest": "lordly",
    "lords": "lord",
    "lorgnettes": "lorgnette",
    "loricae": "lorica",
    "lorries": "lorry",
    "lors": "lor",
    "losers": "loser",
    "loses": "lose",
    "losing": "lose",
    "losses": "loss",
    "lost": "lose",
    "lotions": "lotion",
    "lots": "lot",
    "lotteries": "lottery",
    "lottos": "lotto",
    "lotus-eaters": "lotus-eater",
    "lotuses": "lotus",
    "louder": "loud",
    "loudest": "loud",
    "loudhailers": "loudhailer",
    "loudmouths": "loudmouth",
    "loudnesses": "loudness",
    "loudspeakers": "loudspeaker",
    "loughs": "lough",
    "lounged": "lounge",
    "lounges": "lounge",
    "lounging": "lounge",
    "loupes": "loupe",
    "loused": "louse",
    "louses": "louse",
    "lousier": "lousy",
    "lousiest": "lousy",
    "lousing": "louse",
    "louts": "lout",
    "louvers": "louver",
    "louvres": "louvre",
    "lovastatins": "lovastatin",
    "love-affairs": "love-affair",
    "lovebirds": "lovebird",
    "loved": "love",
    "lovelier": "lovely",
    "lovelies": "lovely",
    "loveliest": "lovely",
    "lovers": "lover",
    "loves": "love",
    "lovesongs": "lovesong",
    "loving": "love",
    "low-downs": "low-down",
    "lowdowns": "lowdown",
    "lowed": "low",
    "lower": "low",
    "lower-fat": "low-fat",
    "lower-priced": "low-priced",
    "lower-risk": "low-risk",
    "lowered": "lower",
    "lowering": "lower",
    "lowers": "lower",
    "lowest": "low",
    "lowest-priced": "low-priced",
    "lowing": "low",
    "lowlands": "lowland",
    "lowlier": "lowly",
    "lowliest": "lowly",
    "lows": "low",
    "loyalists": "loyalist",
    "loyaller": "loyal",
    "loyalties": "loyalty",
    "lozenges": "lozenge",
    "ls": "l",
    "lubes": "lube",
    "lubricants": "lubricant",
    "lubricated": "lubricate",
    "lubricates": "lubricate",
    "lubricating": "lubricate",
    "lubrications": "lubrication",
    "lubricators": "lubricator",
    "lubricities": "lubricity",
    "luciferases": "luciferase",
    "lucked": "luck",
    "luckier": "lucky",
    "luckiest": "lucky",
    "lucking": "luck",
    "lucks": "luck",
    "luffs": "luff",
    "lugged": "lug",
    "lugging": "lug",
    "lugholes": "lughole",
    "lugs": "lug",
    "lugworms": "lugworm",
    "lullabies": "lullaby",
    "lulled": "lull",
    "lulling": "lull",
    "lulls": "lull",
    "lumbered": "lumber",
    "lumbering": "lumber",
    "lumberjacks": "lumberjack",
    "lumbers": "lumber",
    "lumberyards": "lumberyard",
    "lumens": "lumen",
    "lumina": "lumen",
    "luminances": "luminance",
    "luminaries": "luminary",
    "luminosities": "luminosity",
    "lumped": "lump",
    "lumpier": "lumpy",
    "lumpiest": "lumpy",
    "lumping": "lump",
    "lumps": "lump",
    "lunacies": "lunacy",
    "lunas": "luna",
    "lunatics": "lunatic",
    "lunations": "lunation",
    "lunched": "lunch",
    "luncheons": "luncheon",
    "lunches": "lunch",
    "lunching": "lunch",
    "lunchtimes": "lunchtime",
    "lunettes": "lunette",
    "lunged": "lunge",
    "lunges": "lunge",
    "lunging": "lunge",
    "lungs": "lung",
    "lungworms": "lungworm",
    "lupins": "lupin",
    "lurched": "lurch",
    "lurchers": "lurcher",
    "lurches": "lurch",
    "lurching": "lurch",
    "lured": "lure",
    "lures": "lure",
    "luring": "lure",
    "lurked": "lurk",
    "lurkers": "lurker",
    "lurking": "lurk",
    "lurks": "lurk",
    "lusher": "lush",
    "lushest": "lush",
    "lusted": "lust",
    "lusters": "luster",
    "lustier": "lusty",
    "lustiest": "lusty",
    "lusting": "lust",
    "lustres": "lustre",
    "lusts": "lust",
    "luteinized": "luteinize",
    "luteinizes": "luteinize",
    "luteinizing": "luteinize",
    "lutes": "lute",
    "lutherans": "lutheran",
    "luxes": "luxe",
    "luxuriated": "luxuriate",
    "luxuriates": "luxuriate",
    "luxuriating": "luxuriate",
    "luxuries": "luxury",
    "lychees": "lychee",
    "lychgates": "lychgate",
    "lyes": "lye",
    "lying": "lie",
    "lymphadenectomies": "lymphadenectomy",
    "lymphadenopathies": "lymphadenopathy",
    "lymphatics": "lymphatic",
    "lymphocytes": "lymphocyte",
    "lymphoedemas": "lymphoedema",
    "lymphomas": "lymphoma",
    "lymphomata": "lymphoma",
    "lymphs": "lymph",
    "lynched": "lynch",
    "lynches": "lynch",
    "lynching": "lynch",
    "lynchpins": "lynchpin",
    "lynchs": "lynch",
    "lynxes": "lynx",
    "lyrae": "lyra",
    "lyras": "lyra",
    "lyres": "lyre",
    "lyricists": "lyricist",
    "lyrics": "lyric",
    "lys": "ly",
    "lysed": "lyse",
    "lyses": "lysis",
    "lysimeters": "lysimeter",
    "lysines": "lysine",
    "lysing": "lyse",
    "lysosomes": "lysosome",
    "lysozymes": "lysozyme",
    "m": "be",
    "macadamias": "macadamia",
    "macaques": "macaque",
    "macaroons": "macaroon",
    "macaws": "macaw",
    "macerated": "macerate",
    "macerates": "macerate",
    "macerating": "macerate",
    "macerations": "maceration",
    "maces": "mace",
    "machetes": "machete",
    "machinations": "machination",
    "machine-guns": "machine-gun",
    "machined": "machine",
    "machineries": "machinery",
    "machines": "machine",
    "machining": "machine",
    "machinists": "machinist",
    "machos": "macho",
    "machs": "mach",
    "macintoshes": "macintosh",
    "mackerels": "mackerel",
    "mackintoshes": "mackintosh",
    "macro-levels": "macro-level",
    "macrocosms": "macrocosm",
    "macrocycles": "macrocycle",
    "macrofossils": "macrofossil",
    "macroinvertebrates": "macroinvertebrate",
    "macrolides": "macrolide",
    "macromolecules": "macromolecule",
    "macronutrients": "macronutrient",
    "macrophages": "macrophage",
    "macrophytes": "macrophyte",
    "macros": "macro",
    "macs": "mac",
    "maculae": "macula",
    "maculas": "macula",
    "madams": "madam",
    "maddened": "madden",
    "maddening": "madden",
    "maddens": "madden",
    "madder": "mad",
    "madders": "madder",
    "maddest": "mad",
    "made": "make",
    "madeiras": "madeira",
    "madhouses": "madhouse",
    "madmen": "madman",
    "madnesses": "madness",
    "madrigals": "madrigal",
    "madwomen": "madwoman",
    "maelstroms": "maelstrom",
    "maestri": "maestro",
    "maestros": "maestro",
    "mafias": "mafia",
    "magazines": "magazine",
    "magentas": "magenta",
    "mages": "mage",
    "maggots": "maggot",
    "magi": "magus",
    "magicians": "magician",
    "magicked": "magic",
    "magicking": "magic",
    "magics": "magic",
    "magisteria": "magisterium",
    "magisteriums": "magisterium",
    "magistracies": "magistracy",
    "magistrates": "magistrate",
    "magmas": "magma",
    "magnates": "magnate",
    "magnesiums": "magnesium",
    "magnetisations": "magnetisation",
    "magnetised": "magnetise",
    "magnetises": "magnetise",
    "magnetising": "magnetise",
    "magnetites": "magnetite",
    "magnetizations": "magnetization",
    "magnetized": "magnetize",
    "magnetizes": "magnetize",
    "magnetizing": "magnetize",
    "magnetometers": "magnetometer",
    "magnetos": "magneto",
    "magnetospheres": "magnetosphere",
    "magnetrons": "magnetron",
    "magnets": "magnet",
    "magnifications": "magnification",
    "magnified": "magnify",
    "magnifiers": "magnifier",
    "magnifies": "magnify",
    "magnifying": "magnify",
    "magnitudes": "magnitude",
    "magnolias": "magnolia",
    "magnums": "magnum",
    "magpies": "magpie",
    "mags": "mag",
    "mah-jongg": "mah-jong",
    "maharajas": "maharaja",
    "mahoganies": "mahogany",
    "maidens": "maiden",
    "maids": "maid",
    "maidservants": "maidservant",
    "mail-orders": "mail-order",
    "mailbags": "mailbag",
    "mailboxes": "mailbox",
    "mailed": "mail",
    "mailers": "mailer",
    "mailing": "mail",
    "mailings": "mailing",
    "mailmen": "mailman",
    "mails": "mail",
    "maimed": "maim",
    "maiming": "maim",
    "maims": "maim",
    "mainframes": "mainframe",
    "mainlined": "mainline",
    "mainlines": "mainline",
    "mainlining": "mainline",
    "mains": "main",
    "mainsprings": "mainspring",
    "mainstays": "mainstay",
    "mainstreamed": "mainstream",
    "mainstreaming": "mainstream",
    "mainstreamings": "mainstreaming",
    "mainstreams": "mainstream",
    "maintained": "maintain",
    "maintainers": "maintainer",
    "maintaining": "maintain",
    "maintains": "maintain",
    "maisonettes": "maisonette",
    "maizes": "maize",
    "majesties": "majesty",
    "major-domos": "major-domo",
    "major-generals": "major-general",
    "majored": "major",
    "majorettes": "majorette",
    "majoring": "major",
    "majoritarians": "majoritarian",
    "majorities": "majority",
    "majors": "major",
    "make-ups": "make-up",
    "make-weights": "make-weight",
    "makeovers": "makeover",
    "makers": "maker",
    "makes": "make",
    "makeups": "makeup",
    "making": "make",
    "makings": "making",
    "malabsorptions": "malabsorption",
    "maladies": "malady",
    "maladjustments": "maladjustment",
    "maladministrations": "maladministration",
    "malaises": "malaise",
    "malarias": "malaria",
    "malarkeys": "malarkey",
    "malarkies": "malarky",
    "malates": "malate",
    "malcontents": "malcontent",
    "malefactors": "malefactor",
    "malenesses": "maleness",
    "males": "male",
    "malfeasances": "malfeasance",
    "malformations": "malformation",
    "malfunctioned": "malfunction",
    "malfunctioning": "malfunction",
    "malfunctions": "malfunction",
    "malignancies": "malignancy",
    "maligned": "malign",
    "maligning": "malign",
    "malignities": "malignity",
    "maligns": "malign",
    "malingered": "malinger",
    "malingerers": "malingerer",
    "malingering": "malinger",
    "malingers": "malinger",
    "malkins": "malkin",
    "mallards": "mallard",
    "malleoli": "malleolus",
    "mallets": "mallet",
    "mallows": "mallow",
    "malls": "mall",
    "malocclusions": "malocclusion",
    "malodours": "malodour",
    "malpractices": "malpractice",
    "mals": "mal",
    "malted": "malt",
    "malting": "malt",
    "maltodextrins": "maltodextrin",
    "maltoses": "maltose",
    "maltreated": "maltreat",
    "maltreating": "maltreat",
    "maltreats": "maltreat",
    "malts": "malt",
    "maltsters": "maltster",
    "mamas": "mama",
    "mambas": "mamba",
    "mammae": "mamma",
    "mammals": "mammal",
    "mammas": "mamma",
    "mammies": "mammy",
    "mammograms": "mammogram",
    "mammographies": "mammography",
    "mammoths": "mammoth",
    "mams": "mam",
    "man-eaters": "man-eater",
    "man-hours": "man-hour",
    "manacled": "manacle",
    "manacles": "manacle",
    "manacling": "manacle",
    "managed": "manage",
    "managements": "management",
    "manageresses": "manageress",
    "managerialists": "managerialist",
    "managers": "manager",
    "manages": "manage",
    "managing": "manage",
    "manas": "mana",
    "manatees": "manatee",
    "mandalas": "mandala",
    "mandarins": "mandarin",
    "mandated": "mandate",
    "mandates": "mandate",
    "mandating": "mandate",
    "mandibles": "mandible",
    "mandolines": "mandoline",
    "mandolins": "mandolin",
    "mandrakes": "mandrake",
    "mandrels": "mandrel",
    "mandrills": "mandrill",
    "mands": "mand",
    "manes": "mane",
    "maneuvered": "maneuver",
    "maneuvering": "maneuver",
    "maneuverings": "maneuvering",
    "maneuvers": "maneuver",
    "manganeses": "manganese",
    "manganites": "manganite",
    "mangers": "manger",
    "mangier": "mangy",
    "mangiest": "mangy",
    "mangled": "mangle",
    "mangles": "mangle",
    "mangling": "mangle",
    "mangoes": "mango",
    "mangos": "mango",
    "mangroves": "mangrove",
    "manhandled": "manhandle",
    "manhandles": "manhandle",
    "manhandling": "manhandle",
    "manholes": "manhole",
    "manhunts": "manhunt",
    "maniacs": "maniac",
    "manias": "mania",
    "manic-depressives": "manic-depressive",
    "manicured": "manicure",
    "manicures": "manicure",
    "manicuring": "manicure",
    "manicurists": "manicurist",
    "manifestations": "manifestation",
    "manifested": "manifest",
    "manifesting": "manifest",
    "manifestoes": "manifesto",
    "manifestos": "manifesto",
    "manifests": "manifest",
    "manifolds": "manifold",
    "manikins": "manikin",
    "manilas": "manila",
    "manipulated": "manipulate",
    "manipulates": "manipulate",
    "manipulating": "manipulate",
    "manipulations": "manipulation",
    "manipulatives": "manipulative",
    "manipulators": "manipulator",
    "manlier": "manly",
    "manliest": "manly",
    "manned": "man",
    "mannequins": "mannequin",
    "mannerisms": "mannerism",
    "manners": "manner",
    "manning": "man",
    "mannoses": "mannose",
    "manoeuvers": "manoeuver",
    "manoeuvrabilities": "manoeuvrability",
    "manoeuvred": "manoeuvre",
    "manoeuvres": "manoeuvre",
    "manoeuvring": "manoeuvre",
    "manoeuvrings": "manoeuvring",
    "manometers": "manometer",
    "manors": "manor",
    "manos": "mano",
    "mans": "man",
    "manservants": "manservant",
    "manses": "manse",
    "mansions": "mansion",
    "manslaughters": "manslaughter",
    "mantelpieces": "mantelpiece",
    "mantels": "mantel",
    "mantelshelves": "mantelshelf",
    "mantes": "mantis",
    "mantids": "mantid",
    "mantises": "mantis",
    "mantissas": "mantissa",
    "mantled": "mantle",
    "mantles": "mantle",
    "mantling": "mantle",
    "mantras": "mantra",
    "manuals": "manual",
    "manufactories": "manufactory",
    "manufactured": "manufacture",
    "manufacturers": "manufacturer",
    "manufactures": "manufacture",
    "manufacturing": "manufacture",
    "manured": "manure",
    "manures": "manure",
    "manuring": "manure",
    "manuscripts": "manuscript",
    "maples": "maple",
    "mapped": "map",
    "mappers": "mapper",
    "mapping": "map",
    "mappings": "mapping",
    "maps": "map",
    "marathoners": "marathoner",
    "marathons": "marathon",
    "marauded": "maraud",
    "marauders": "marauder",
    "marauding": "maraud",
    "marauds": "maraud",
    "marbled": "marble",
    "marbles": "marble",
    "marbling": "marble",
    "marblings": "marbling",
    "march-pasts": "march-past",
    "marched": "march",
    "marchers": "marcher",
    "marches": "march",
    "marching": "march",
    "marchionesses": "marchioness",
    "marcs": "marc",
    "mares": "mare",
    "margarines": "margarine",
    "margaritas": "margarita",
    "marginalised": "marginalise",
    "marginalises": "marginalise",
    "marginalising": "marginalise",
    "marginalized": "marginalize",
    "marginalizes": "marginalize",
    "marginalizing": "marginalize",
    "marginals": "marginal",
    "margins": "margin",
    "maria": "mare",
    "marigolds": "marigold",
    "marimbas": "marimba",
    "marinaded": "marinade",
    "marinades": "marinade",
    "marinading": "marinade",
    "marinas": "marina",
    "marinated": "marinate",
    "marinates": "marinate",
    "marinating": "marinate",
    "mariners": "mariner",
    "marines": "marine",
    "marionettes": "marionette",
    "marjorams": "marjoram",
    "mark-downs": "mark-down",
    "mark-ups": "mark-up",
    "marked": "mark",
    "markers": "marker",
    "market-places": "market-place",
    "marketed": "market",
    "marketeers": "marketeer",
    "marketers": "marketer",
    "marketing": "market",
    "marketings": "marketing",
    "marketplaces": "marketplace",
    "markets": "market",
    "marking": "mark",
    "markings": "marking",
    "marks": "mark",
    "marksmen": "marksman",
    "markups": "markup",
    "marlins": "marlin",
    "marmites": "marmite",
    "marmosets": "marmoset",
    "marmots": "marmot",
    "marooned": "maroon",
    "marooning": "maroon",
    "maroons": "maroon",
    "marquees": "marquee",
    "marquesses": "marquess",
    "marquises": "marquis",
    "marrams": "marram",
    "marred": "mar",
    "marriages": "marriage",
    "married": "marry",
    "marries": "marry",
    "marring": "mar",
    "marrows": "marrow",
    "marrying": "marry",
    "mars": "mar",
    "marshaled": "marshal",
    "marshaling": "marshal",
    "marshalled": "marshal",
    "marshalling": "marshal",
    "marshals": "marshal",
    "marshes": "marsh",
    "marshier": "marshy",
    "marshiest": "marshy",
    "marshlands": "marshland",
    "marshmallows": "marshmallow",
    "marsupials": "marsupial",
    "martens": "marten",
    "martinets": "martinet",
    "martingales": "martingale",
    "martinis": "martini",
    "martins": "martin",
    "marts": "mart",
    "martyrdoms": "martyrdom",
    "martyred": "martyr",
    "martyring": "martyr",
    "martyrs": "martyr",
    "marveled": "marvel",
    "marveling": "marvel",
    "marvelled": "marvel",
    "marvelling": "marvel",
    "marvels": "marvel",
    "marzipans": "marzipan",
    "mas": "ma",
    "masalas": "masala",
    "mascaras": "mascara",
    "mascarpones": "mascarpone",
    "mascots": "mascot",
    "masculines": "masculine",
    "masculinities": "masculinity",
    "masers": "maser",
    "mashed": "mash",
    "mashes": "mash",
    "mashing": "mash",
    "masked": "mask",
    "maskers": "masker",
    "masking": "mask",
    "masks": "mask",
    "masochists": "masochist",
    "masons": "mason",
    "masqueraded": "masquerade",
    "masquerades": "masquerade",
    "masquerading": "masquerade",
    "masques": "masque",
    "mass-produced": "mass-produce",
    "mass-produces": "mass-produce",
    "mass-producing": "mass-produce",
    "massacred": "massacre",
    "massacres": "massacre",
    "massacring": "massacre",
    "massaged": "massage",
    "massagers": "massager",
    "massages": "massage",
    "massaging": "massage",
    "massed": "mass",
    "masses": "masse",
    "masseurs": "masseur",
    "masseuses": "masseuse",
    "massier": "massy",
    "massifs": "massif",
    "massing": "mass",
    "mastectomies": "mastectomy",
    "masted": "mast",
    "masterclasses": "masterclass",
    "mastered": "master",
    "mastering": "master",
    "masterminded": "mastermind",
    "masterminding": "mastermind",
    "masterminds": "mastermind",
    "masterpieces": "masterpiece",
    "masterplans": "masterplan",
    "masters": "master",
    "masterstrokes": "masterstroke",
    "masterworks": "masterwork",
    "mastheads": "masthead",
    "masticated": "masticate",
    "masticates": "masticate",
    "masticating": "masticate",
    "mastiffs": "mastiff",
    "masting": "mast",
    "mastitides": "mastitis",
    "mastodons": "mastodon",
    "masts": "mast",
    "masturbated": "masturbate",
    "masturbates": "masturbate",
    "masturbating": "masturbate",
    "masturbations": "masturbation",
    "matadors": "matador",
    "matchboxes": "matchbox",
    "matched": "match",
    "matchers": "matcher",
    "matches": "match",
    "matching": "match",
    "matchings": "matching",
    "matchmakers": "matchmaker",
    "matchsticks": "matchstick",
    "mated": "mate",
    "materiae": "materia",
    "materialised": "materialise",
    "materialises": "materialise",
    "materialising": "materialise",
    "materialists": "materialist",
    "materialized": "materialize",
    "materializes": "materialize",
    "materializing": "materialize",
    "materials": "material",
    "maternities": "maternity",
    "maters": "mater",
    "mates": "mate",
    "mateyer": "matey",
    "mateyest": "matey",
    "mathematicians": "mathematician",
    "maths": "math",
    "matier": "maty",
    "matiest": "maty",
    "matinees": "matinee",
    "mating": "mate",
    "matings": "mating",
    "matinées": "matinée",
    "matriarchies": "matriarchy",
    "matriarchs": "matriarch",
    "matrices": "matrix",
    "matriculated": "matriculate",
    "matriculates": "matriculate",
    "matriculating": "matriculate",
    "matriculations": "matriculation",
    "matrixes": "matrix",
    "matrons": "matron",
    "mats": "mat",
    "matted": "mat",
    "matter": "matt",
    "mattered": "matter",
    "mattering": "matter",
    "matters": "matter",
    "mattest": "matt",
    "matting": "mat",
    "mattings": "matting",
    "mattocks": "mattock",
    "mattresses": "mattress",
    "maturations": "maturation",
    "matured": "mature",
    "maturer": "mature",
    "matures": "mature",
    "maturest": "mature",
    "maturing": "mature",
    "maturities": "maturity",
    "mauled": "maul",
    "mauling": "maul",
    "mauls": "maul",
    "mausolea": "mausoleum",
    "mausoleums": "mausoleum",
    "mauver": "mauve",
    "mauves": "mauve",
    "mauvest": "mauve",
    "mavericks": "maverick",
    "maws": "maw",
    "maxed": "max",
    "maxes": "max",
    "maxillae": "maxilla",
    "maxillas": "maxilla",
    "maxima": "maximum",
    "maximisations": "maximisation",
    "maximised": "maximise",
    "maximises": "maximise",
    "maximising": "maximise",
    "maximizations": "maximization",
    "maximized": "maximize",
    "maximizes": "maximize",
    "maximizing": "maximize",
    "maxims": "maxim",
    "maximums": "maximum",
    "maxing": "max",
    "maxs": "max",
    "maydays": "mayday",
    "mayed": "may",
    "mayflies": "mayfly",
    "mayhems": "mayhem",
    "maying": "may",
    "mayonnaises": "mayonnaise",
    "mayoralties": "mayoralty",
    "mayoresses": "mayoress",
    "mayors": "mayor",
    "maypoles": "maypole",
    "mays": "may",
    "mazes": "maze",
    "mcgs": "mcg",
    "meadows": "meadow",
    "mealier": "mealy",
    "mealiest": "mealy",
    "meals": "meal",
    "mealtimes": "mealtime",
    "mealworms": "mealworm",
    "mealybugs": "mealybug",
    "meandered": "meander",
    "meandering": "meander",
    "meanders": "meander",
    "meaner": "mean",
    "meanest": "mean",
    "meanies": "meanie",
    "meaning": "mean",
    "meanings": "meaning",
    "means": "mean",
    "meant": "mean",
    "measles": "measle",
    "measlier": "measly",
    "measliest": "measly",
    "measured": "measure",
    "measurements": "measurement",
    "measurers": "measurer",
    "measures": "measure",
    "measuring": "measure",
    "meatballs": "meatball",
    "meatier": "meaty",
    "meatiest": "meaty",
    "meats": "meat",
    "meccas": "mecca",
    "mechanics": "mechanic",
    "mechanised": "mechanise",
    "mechanises": "mechanise",
    "mechanising": "mechanise",
    "mechanisms": "mechanism",
    "mechanized": "mechanize",
    "mechanizes": "mechanize",
    "mechanizing": "mechanize",
    "meconiums": "meconium",
    "medalists": "medalist",
    "medallions": "medallion",
    "medallists": "medallist",
    "medals": "medal",
    "meddled": "meddle",
    "meddlers": "meddler",
    "meddles": "meddle",
    "meddling": "meddle",
    "media": "medium",
    "medials": "medial",
    "medians": "median",
    "mediated": "mediate",
    "mediates": "mediate",
    "mediating": "mediate",
    "mediations": "mediation",
    "mediators": "mediator",
    "medicalised": "medicalise",
    "medicalises": "medicalise",
    "medicalising": "medicalise",
    "medicals": "medical",
    "medicaments": "medicament",
    "medicated": "medicate",
    "medicates": "medicate",
    "medicating": "medicate",
    "medications": "medication",
    "medicinals": "medicinal",
    "medicines": "medicine",
    "medicos": "medico",
    "medics": "medic",
    "medievalists": "medievalist",
    "mediocrities": "mediocrity",
    "medios": "medio",
    "meditated": "meditate",
    "meditates": "meditate",
    "meditating": "meditate",
    "meditations": "meditation",
    "meditatively": "meditative",
    "meditators": "meditator",
    "mediums": "medium",
    "medleys": "medley",
    "medullae": "medulla",
    "medullas": "medulla",
    "medulloblastomas": "medulloblastoma",
    "medulloblastomata": "medulloblastoma",
    "medusae": "medusa",
    "meeker": "meek",
    "meekest": "meek",
    "meerkats": "meerkat",
    "meeting": "meet",
    "meetings": "meeting",
    "meets": "meet",
    "megabytes": "megabyte",
    "megaliths": "megalith",
    "megalomaniacs": "megalomaniac",
    "megaphones": "megaphone",
    "megapixels": "megapixel",
    "megatons": "megaton",
    "megawatts": "megawatt",
    "megs": "meg",
    "meioses": "meiosis",
    "melancholias": "melancholia",
    "melancholics": "melancholic",
    "melanges": "melange",
    "melanins": "melanin",
    "melanocytes": "melanocyte",
    "melanomas": "melanoma",
    "melanomata": "melanoma",
    "melded": "meld",
    "melding": "meld",
    "melds": "meld",
    "melees": "melee",
    "mellowed": "mellow",
    "mellower": "mellow",
    "mellowest": "mellow",
    "mellowing": "mellow",
    "mellows": "mellow",
    "melodies": "melody",
    "melodramas": "melodrama",
    "melons": "melon",
    "meltdowns": "meltdown",
    "melted": "melt",
    "melting": "melt",
    "melts": "melt",
    "members": "member",
    "memberships": "membership",
    "membranes": "membrane",
    "mementoes": "memento",
    "mementos": "memento",
    "memes": "meme",
    "memoirs": "memoir",
    "memorabilia": "memorabile",
    "memoranda": "memorandum",
    "memorandums": "memorandum",
    "memorialized": "memorialize",
    "memorializes": "memorialize",
    "memorializing": "memorialize",
    "memorials": "memorial",
    "memories": "memory",
    "memorised": "memorise",
    "memorises": "memorise",
    "memorising": "memorise",
    "memorized": "memorize",
    "memorizes": "memorize",
    "memorizing": "memorize",
    "memos": "memo",
    "memsahibs": "memsahib",
    "men": "man",
    "menaced": "menace",
    "menaces": "menace",
    "menacing": "menace",
    "menageries": "menagerie",
    "menages": "menage",
    "mendacities": "mendacity",
    "mended": "mend",
    "mendicants": "mendicant",
    "mending": "mend",
    "mendings": "mending",
    "mends": "mend",
    "menials": "menial",
    "meninges": "meninx",
    "meningiomas": "meningioma",
    "meningiomata": "meningioma",
    "meningitides": "meningitis",
    "meningococci": "meningococcus",
    "menisci": "meniscus",
    "meniscuses": "meniscus",
    "menopauses": "menopause",
    "menorrhagias": "menorrhagia",
    "menservants": "manservant",
    "menstruated": "menstruate",
    "menstruates": "menstruate",
    "menstruating": "menstruate",
    "menstruations": "menstruation",
    "mentalists": "mentalist",
    "mentalities": "mentality",
    "mentees": "mentee",
    "mentioned": "mention",
    "mentioning": "mention",
    "mentions": "mention",
    "mentored": "mentor",
    "mentoring": "mentor",
    "mentors": "mentor",
    "mentorships": "mentorship",
    "menus": "menu",
    "mercenaries": "mercenary",
    "merchandised": "merchandise",
    "merchandisers": "merchandiser",
    "merchandises": "merchandise",
    "merchandising": "merchandise",
    "merchandisings": "merchandising",
    "merchants": "merchant",
    "mercies": "mercy",
    "mercuries": "mercury",
    "merer": "mere",
    "meres": "mere",
    "merest": "mere",
    "mergansers": "merganser",
    "merged": "merge",
    "mergers": "merger",
    "merges": "merge",
    "merging": "merge",
    "meridians": "meridian",
    "meringues": "meringue",
    "meristems": "meristem",
    "merited": "merit",
    "meriting": "merit",
    "meritocracies": "meritocracy",
    "merits": "merit",
    "merlins": "merlin",
    "mermaids": "mermaid",
    "merrier": "merry",
    "merriest": "merry",
    "merry-go-rounds": "merry-go-round",
    "mesas": "mesa",
    "meshed": "mesh",
    "meshes": "mesh",
    "meshing": "mesh",
    "mesmerised": "mesmerise",
    "mesmerises": "mesmerise",
    "mesmerising": "mesmerise",
    "mesmerized": "mesmerize",
    "mesmerizes": "mesmerize",
    "mesmerizing": "mesmerize",
    "mesoderms": "mesoderm",
    "mesons": "meson",
    "mesospheres": "mesosphere",
    "mesotheliomas": "mesothelioma",
    "mesotheliomata": "mesothelioma",
    "mess-ups": "mess-up",
    "messaged": "message",
    "messages": "message",
    "messaging": "message",
    "messed": "mess",
    "messengers": "messenger",
    "messes": "mess",
    "messiahs": "messiah",
    "messier": "messy",
    "messiest": "messy",
    "messieurs": "monsieur",
    "messing": "mess",
    "mestizoes": "mestizo",
    "mestizos": "mestizo",
    "met": "meet",
    "meta-analyses": "meta-analysis",
    "meta-languages": "meta-language",
    "meta-models": "meta-model",
    "meta-searches": "meta-search",
    "metabolised": "metabolise",
    "metabolises": "metabolise",
    "metabolising": "metabolise",
    "metabolisms": "metabolism",
    "metabolites": "metabolite",
    "metabolized": "metabolize",
    "metabolizes": "metabolize",
    "metabolizing": "metabolize",
    "metacognitions": "metacognition",
    "metalanguages": "metalanguage",
    "metalled": "metal",
    "metalling": "metal",
    "metallisations": "metallisation",
    "metallised": "metallise",
    "metallises": "metallise",
    "metallising": "metallise",
    "metallocenes": "metallocene",
    "metalloproteinases": "metalloproteinase",
    "metallurgies": "metallurgy",
    "metallurgists": "metallurgist",
    "metals": "metal",
    "metalworkers": "metalworker",
    "metamorphisms": "metamorphism",
    "metamorphosed": "metamorphose",
    "metamorphoses": "metamorphosis",
    "metamorphosing": "metamorphose",
    "metaphases": "metaphase",
    "metaphors": "metaphor",
    "metaphysicians": "metaphysician",
    "metaplasias": "metaplasia",
    "metapopulations": "metapopulation",
    "metarules": "metarule",
    "metasearches": "metasearch",
    "metastases": "metastasis",
    "metastasized": "metastasize",
    "metastasizes": "metastasize",
    "metastasizing": "metastasize",
    "metatarsalgias": "metatarsalgia",
    "metatarsals": "metatarsal",
    "meted": "mete",
    "meteorites": "meteorite",
    "meteoroids": "meteoroid",
    "meteorologies": "meteorology",
    "meteorologists": "meteorologist",
    "meteors": "meteor",
    "metered": "meter",
    "metering": "meter",
    "meters": "meter",
    "metes": "mete",
    "methacrylates": "methacrylate",
    "methadones": "methadone",
    "methamphetamines": "methamphetamine",
    "methanes": "methane",
    "methanols": "methanol",
    "methinks": "methink",
    "methionines": "methionine",
    "methodologies": "methodology",
    "methodologists": "methodologist",
    "methods": "method",
    "methought": "methink",
    "meths": "meth",
    "methylated": "methylate",
    "methylates": "methylate",
    "methylating": "methylate",
    "methylations": "methylation",
    "methylenes": "methylene",
    "methyls": "methyl",
    "methyltransferases": "methyltransferase",
    "metiers": "metier",
    "meting": "mete",
    "metonymies": "metonymy",
    "metred": "metre",
    "metres": "metre",
    "metrications": "metrication",
    "metrics": "metric",
    "metring": "metre",
    "metrologies": "metrology",
    "metronomes": "metronome",
    "metropolides": "metropolis",
    "metropolises": "metropolis",
    "metropolitans": "metropolitan",
    "metros": "metro",
    "mewed": "mew",
    "mewing": "mew",
    "mews": "mew",
    "mezzanines": "mezzanine",
    "mezzo-sopranos": "mezzo-soprano",
    "mezzos": "mezzo",
    "mezzotints": "mezzotint",
    "mgs": "mg",
    "miaowed": "miaow",
    "miaowing": "miaow",
    "miaows": "miaow",
    "miasmas": "miasma",
    "miasmata": "miasma",
    "micas": "mica",
    "mice": "mouse",
    "micelles": "micelle",
    "mickeys": "mickey",
    "micra": "micron",
    "micro-climates": "micro-climate",
    "micro-computers": "micro-computer",
    "micro-controllers": "micro-controller",
    "micro-managed": "micro-manage",
    "micro-manages": "micro-manage",
    "micro-managing": "micro-manage",
    "micro-organisms": "micro-organism",
    "micro-waves": "micro-wave",
    "microalbuminurias": "microalbuminuria",
    "microanalyses": "microanalysis",
    "microarchitectures": "microarchitecture",
    "microarrays": "microarray",
    "microbes": "microbe",
    "microbicides": "microbicide",
    "microbiologies": "microbiology",
    "microbiologists": "microbiologist",
    "microbreweries": "microbrewery",
    "microcavities": "microcavity",
    "microchannels": "microchannel",
    "microchips": "microchip",
    "microcirculations": "microcirculation",
    "microclimates": "microclimate",
    "microcomputers": "microcomputer",
    "microcontrollers": "microcontroller",
    "microcosms": "microcosm",
    "microdrives": "microdrive",
    "microelectrodes": "microelectrode",
    "microelectronics": "microelectronic",
    "microenvironments": "microenvironment",
    "microfiches": "microfiche",
    "microfilmed": "microfilm",
    "microfilming": "microfilm",
    "microfilms": "microfilm",
    "microfilters": "microfilter",
    "microflorae": "microflora",
    "microfloras": "microflora",
    "microforms": "microform",
    "microfossils": "microfossil",
    "micrograms": "microgram",
    "micrographs": "micrograph",
    "microgravities": "microgravity",
    "microhabitats": "microhabitat",
    "microlenses": "microlens",
    "microliths": "microlith",
    "micrometers": "micrometer",
    "micrometres": "micrometre",
    "micromorphologies": "micromorphology",
    "microms": "microm",
    "microns": "micron",
    "micronuclei": "micronucleus",
    "micronucleuses": "micronucleus",
    "micronutrients": "micronutrient",
    "microorganisms": "microorganism",
    "microphones": "microphone",
    "microprobes": "microprobe",
    "microprocessors": "microprocessor",
    "micros": "micro",
    "microsatellites": "microsatellite",
    "microscopes": "microscope",
    "microscopies": "microscopy",
    "microscopists": "microscopist",
    "microseconds": "microsecond",
    "microspheres": "microsphere",
    "microstates": "microstate",
    "microstructures": "microstructure",
    "microsurgeries": "microsurgery",
    "microswitches": "microswitch",
    "microsystems": "microsystem",
    "microtomes": "microtome",
    "microtubules": "microtubule",
    "microwaved": "microwave",
    "microwaves": "microwave",
    "microwaving": "microwave",
    "microworlds": "microworld",
    "micturitions": "micturition",
    "mid-courses": "mid-course",
    "mid-fields": "mid-field",
    "mid-lines": "mid-line",
    "mid-points": "mid-point",
    "mid-sections": "mid-section",
    "mid-terms": "mid-term",
    "mid-waters": "mid-water",
    "mid-winters": "mid-winter",
    "midbrains": "midbrain",
    "middays": "midday",
    "middens": "midden",
    "middlebrows": "middlebrow",
    "middlemen": "middleman",
    "middles": "middle",
    "middleweights": "middleweight",
    "midfielders": "midfielder",
    "midfields": "midfield",
    "midges": "midge",
    "midgets": "midget",
    "midlines": "midline",
    "midnights": "midnight",
    "midpoints": "midpoint",
    "midranges": "midrange",
    "midribs": "midrib",
    "midriffs": "midriff",
    "mids": "mid",
    "midsections": "midsection",
    "midshipmen": "midshipman",
    "midsts": "midst",
    "midsummers": "midsummer",
    "midterms": "midterm",
    "midweeks": "midweek",
    "midwinters": "midwinter",
    "midwives": "midwife",
    "midyears": "midyear",
    "mifepristones": "mifepristone",
    "might": "may",
    "mighted": "might",
    "mightier": "mighty",
    "mightiest": "mighty",
    "mighting": "might",
    "mights": "might",
    "migraines": "migraine",
    "migrants": "migrant",
    "migrated": "migrate",
    "migrates": "migrate",
    "migrating": "migrate",
    "migrations": "migration",
    "migs": "mig",
    "miked": "mike",
    "mikes": "mike",
    "miking": "mike",
    "milder": "mild",
    "mildest": "mild",
    "mileages": "mileage",
    "mileometers": "mileometer",
    "mileposts": "milepost",
    "miles": "mile",
    "milestones": "milestone",
    "milieus": "milieu",
    "milieux": "milieu",
    "militants": "militant",
    "militaries": "military",
    "militarised": "militarise",
    "militarises": "militarise",
    "militarising": "militarise",
    "militarists": "militarist",
    "militarized": "militarize",
    "militarizes": "militarize",
    "militarizing": "militarize",
    "militated": "militate",
    "militates": "militate",
    "militating": "militate",
    "militiamen": "militiaman",
    "militias": "militia",
    "milk-shakes": "milk-shake",
    "milked": "milk",
    "milkers": "milker",
    "milkier": "milky",
    "milkiest": "milky",
    "milking": "milk",
    "milkmaids": "milkmaid",
    "milkmen": "milkman",
    "milks": "milk",
    "milkshakes": "milkshake",
    "milled": "mill",
    "millenia": "millenium",
    "milleniums": "millenium",
    "millennia": "millennium",
    "millenniums": "millennium",
    "millers": "miller",
    "milles": "mille",
    "millets": "millet",
    "milliards": "milliard",
    "millibars": "millibar",
    "milligrams": "milligram",
    "milliliters": "milliliter",
    "millilitres": "millilitre",
    "millimeters": "millimeter",
    "millimetres": "millimetre",
    "milliners": "milliner",
    "milling": "mill",
    "millionaires": "millionaire",
    "millions": "million",
    "millionth": "1000000",
    "millionths": "millionth",
    "millipedes": "millipede",
    "milliseconds": "millisecond",
    "milliwatts": "milliwatt",
    "mills": "mill",
    "millstones": "millstone",
    "millwheels": "millwheel",
    "millwrights": "millwright",
    "milometers": "milometer",
    "mils": "mil",
    "mimed": "mime",
    "mimes": "mime",
    "mimiced": "mimic",
    "mimicing": "mimic",
    "mimicked": "mimick",
    "mimicking": "mimick",
    "mimicks": "mimick",
    "mimicries": "mimicry",
    "mimics": "mimic",
    "miming": "mime",
    "mimosas": "mimosa",
    "minae": "mina",
    "minarets": "minaret",
    "minced": "mince",
    "mincers": "mincer",
    "minces": "mince",
    "mincing": "mince",
    "mind-sets": "mind-set",
    "minded": "mind",
    "minders": "minder",
    "minding": "mind",
    "minds": "mind",
    "mindsets": "mindset",
    "mined": "mine",
    "minefields": "minefield",
    "mineralisations": "mineralisation",
    "mineralised": "mineralise",
    "mineralises": "mineralise",
    "mineralising": "mineralise",
    "mineralizations": "mineralization",
    "mineralogies": "mineralogy",
    "mineralogists": "mineralogist",
    "minerals": "mineral",
    "miners": "miner",
    "mines": "mine",
    "minesweepers": "minesweeper",
    "mingier": "mingy",
    "mingiest": "mingy",
    "mingled": "mingle",
    "mingles": "mingle",
    "mingling": "mingle",
    "mings": "ming",
    "mini-buses": "mini-bus",
    "mini-busses": "mini-bus",
    "mini-skirts": "mini-skirt",
    "miniatures": "miniature",
    "miniaturisations": "miniaturisation",
    "miniaturised": "miniaturise",
    "miniaturises": "miniaturise",
    "miniaturising": "miniaturise",
    "miniaturists": "miniaturist",
    "miniaturizations": "miniaturization",
    "miniaturized": "miniaturize",
    "miniaturizes": "miniaturize",
    "miniaturizing": "miniaturize",
    "minibuses": "minibus",
    "minibusses": "minibus",
    "minicabs": "minicab",
    "minicomputers": "minicomputer",
    "minima": "minimum",
    "minimalists": "minimalist",
    "minimisations": "minimisation",
    "minimised": "minimise",
    "minimises": "minimise",
    "minimising": "minimise",
    "minimizations": "minimization",
    "minimized": "minimize",
    "minimizes": "minimize",
    "minimizing": "minimize",
    "minims": "minim",
    "minimums": "minimum",
    "mining": "mine",
    "minions": "minion",
    "minis": "mini",
    "minisatellites": "minisatellite",
    "miniskirts": "miniskirt",
    "ministered": "minister",
    "ministering": "minister",
    "ministers": "minister",
    "ministrations": "ministration",
    "ministries": "ministry",
    "minivans": "minivan",
    "minkes": "minke",
    "minks": "mink",
    "minnows": "minnow",
    "minorities": "minority",
    "minors": "minor",
    "mins": "min",
    "minsters": "minster",
    "minstrels": "minstrel",
    "minted": "mint",
    "minting": "mint",
    "mints": "mint",
    "minuets": "minuet",
    "minuses": "minus",
    "minusses": "minus",
    "minuted": "minute",
    "minuter": "minute",
    "minutes": "minute",
    "minutest": "minute",
    "minutiae": "minutia",
    "minuting": "minute",
    "minxes": "minx",
    "miracles": "miracle",
    "mirages": "mirage",
    "mired": "mire",
    "mires": "mire",
    "mirier": "miry",
    "miring": "mire",
    "mirror-images": "mirror-image",
    "mirrored": "mirror",
    "mirroring": "mirror",
    "mirrors": "mirror",
    "mis-matches": "mis-match",
    "mis-uses": "mis-use",
    "misadventures": "misadventure",
    "misalignments": "misalignment",
    "misanthropes": "misanthrope",
    "misanthropists": "misanthropist",
    "misapplications": "misapplication",
    "misapplied": "misapply",
    "misapplies": "misapply",
    "misapplying": "misapply",
    "misapprehended": "misapprehend",
    "misapprehending": "misapprehend",
    "misapprehends": "misapprehend",
    "misapprehensions": "misapprehension",
    "misappropriated": "misappropriate",
    "misappropriates": "misappropriate",
    "misappropriating": "misappropriate",
    "misappropriations": "misappropriation",
    "misattributed": "misattribute",
    "misattributes": "misattribute",
    "misattributing": "misattribute",
    "misbecame": "misbecome",
    "misbecomes": "misbecome",
    "misbecoming": "misbecome",
    "misbehaved": "misbehave",
    "misbehaves": "misbehave",
    "misbehaving": "misbehave",
    "misbehaviours": "misbehaviour",
    "miscalculated": "miscalculate",
    "miscalculates": "miscalculate",
    "miscalculating": "miscalculate",
    "miscalculations": "miscalculation",
    "miscarriages": "miscarriage",
    "miscarried": "miscarry",
    "miscarries": "miscarry",
    "miscarrying": "miscarry",
    "miscasting": "miscast",
    "miscasts": "miscast",
    "miscellanies": "miscellany",
    "mischances": "mischance",
    "mischief-makers": "mischief-maker",
    "misclassifications": "misclassification",
    "misclassified": "misclassify",
    "misclassifies": "misclassify",
    "misclassifying": "misclassify",
    "miscommunications": "miscommunication",
    "misconceived": "misconceive",
    "misconceives": "misconceive",
    "misconceiving": "misconceive",
    "misconceptions": "misconception",
    "misconducted": "misconduct",
    "misconducting": "misconduct",
    "misconducts": "misconduct",
    "misconstructions": "misconstruction",
    "misconstrued": "misconstrue",
    "misconstrues": "misconstrue",
    "misconstruing": "misconstrue",
    "miscounted": "miscount",
    "miscounting": "miscount",
    "miscounts": "miscount",
    "miscreants": "miscreant",
    "miscued": "miscue",
    "miscues": "miscue",
    "miscuing": "miscue",
    "misdealing": "misdeal",
    "misdeals": "misdeal",
    "misdealt": "misdeal",
    "misdeeds": "misdeed",
    "misdemeanors": "misdemeanor",
    "misdemeanours": "misdemeanour",
    "misdiagnosed": "misdiagnose",
    "misdiagnoses": "misdiagnosis",
    "misdiagnosing": "misdiagnose",
    "misdid": "misdo",
    "misdirected": "misdirect",
    "misdirecting": "misdirect",
    "misdirections": "misdirection",
    "misdirects": "misdirect",
    "misdoes": "misdo",
    "misdoing": "misdo",
    "misdone": "misdo",
    "miseries": "misery",
    "misers": "miser",
    "mises": "mise",
    "misfired": "misfire",
    "misfires": "misfire",
    "misfiring": "misfire",
    "misfits": "misfit",
    "misfortunes": "misfortune",
    "misgivings": "misgiving",
    "misguided": "misguide",
    "misguides": "misguide",
    "misguiding": "misguide",
    "mishandled": "mishandle",
    "mishandles": "mishandle",
    "mishandling": "mishandle",
    "mishaps": "mishap",
    "misheard": "mishear",
    "mishearing": "mishear",
    "mishears": "mishear",
    "mishits": "mishit",
    "mishitting": "mishit",
    "mishmashes": "mishmash",
    "misidentifications": "misidentification",
    "misidentified": "misidentify",
    "misidentifies": "misidentify",
    "misidentifying": "misidentify",
    "misinformed": "misinform",
    "misinforming": "misinform",
    "misinforms": "misinform",
    "misinterpretations": "misinterpretation",
    "misinterpreted": "misinterpret",
    "misinterpreting": "misinterpret",
    "misinterprets": "misinterpret",
    "misjudged": "misjudge",
    "misjudgements": "misjudgement",
    "misjudges": "misjudge",
    "misjudging": "misjudge",
    "misjudgments": "misjudgment",
    "mislabeled": "mislabel",
    "mislabeling": "mislabel",
    "mislabelled": "mislabel",
    "mislabelling": "mislabel",
    "mislabels": "mislabel",
    "mislaid": "mislay",
    "mislaying": "mislay",
    "mislays": "mislay",
    "misleaded": "mislead",
    "misleading": "mislead",
    "misleads": "mislead",
    "misled": "mislead",
    "mismanaged": "mismanage",
    "mismanages": "mismanage",
    "mismanaging": "mismanage",
    "mismatched": "mismatch",
    "mismatches": "mismatch",
    "mismatching": "mismatch",
    "misnamed": "misname",
    "misnames": "misname",
    "misnaming": "misname",
    "misnomers": "misnomer",
    "misogynists": "misogynist",
    "misperceptions": "misperception",
    "misplaced": "misplace",
    "misplaces": "misplace",
    "misplacing": "misplace",
    "misprinted": "misprint",
    "misprinting": "misprint",
    "misprints": "misprint",
    "mispronounced": "mispronounce",
    "mispronounces": "mispronounce",
    "mispronouncing": "mispronounce",
    "mispronunciations": "mispronunciation",
    "misquotations": "misquotation",
    "misquoted": "misquote",
    "misquotes": "misquote",
    "misquoting": "misquote",
    "misreaded": "misread",
    "misreading": "misread",
    "misreads": "misread",
    "misreported": "misreport",
    "misreporting": "misreport",
    "misreports": "misreport",
    "misrepresentations": "misrepresentation",
    "misrepresented": "misrepresent",
    "misrepresenting": "misrepresent",
    "misrepresents": "misrepresent",
    "misruled": "misrule",
    "misrules": "misrule",
    "misruling": "misrule",
    "missals": "missal",
    "missed": "miss",
    "misses": "miss",
    "missiles": "missile",
    "missing": "miss",
    "missionaries": "missionary",
    "missions": "mission",
    "missives": "missive",
    "misspelled": "misspell",
    "misspelling": "misspell",
    "misspellings": "misspelling",
    "misspells": "misspell",
    "misspelt": "misspell",
    "misspending": "misspend",
    "misspends": "misspend",
    "misspent": "misspend",
    "misstated": "misstate",
    "misstatements": "misstatement",
    "misstates": "misstate",
    "misstating": "misstate",
    "missteps": "misstep",
    "mistaken": "mistake",
    "mistakes": "mistake",
    "mistaking": "mistake",
    "misted": "mist",
    "misters": "mister",
    "mistier": "misty",
    "mistiest": "misty",
    "mistimed": "mistime",
    "mistimes": "mistime",
    "mistiming": "mistime",
    "misting": "mist",
    "mistletoes": "mistletoe",
    "mistook": "mistake",
    "mistranslated": "mistranslate",
    "mistranslates": "mistranslate",
    "mistranslating": "mistranslate",
    "mistranslations": "mistranslation",
    "mistreated": "mistreat",
    "mistreating": "mistreat",
    "mistreats": "mistreat",
    "mistresses": "mistress",
    "mistrusted": "mistrust",
    "mistrusting": "mistrust",
    "mistrusts": "mistrust",
    "mists": "mist",
    "mistyped": "mistype",
    "mistypes": "mistype",
    "mistyping": "mistype",
    "misunderstanding": "misunderstand",
    "misunderstandings": "misunderstanding",
    "misunderstands": "misunderstand",
    "misunderstood": "misunderstand",
    "misused": "misuse",
    "misusers": "misuser",
    "misuses": "misuse",
    "misusing": "misuse",
    "mites": "mite",
    "mitigated": "mitigate",
    "mitigates": "mitigate",
    "mitigating": "mitigate",
    "mitigations": "mitigation",
    "mitochondria": "mitochondrion",
    "mitochondrions": "mitochondrion",
    "mitomycins": "mitomycin",
    "mitoses": "mitosis",
    "mitred": "mitre",
    "mitres": "mitre",
    "mitring": "mitre",
    "mittens": "mitten",
    "mitts": "mitt",
    "mix-ups": "mix-up",
    "mixed": "mix",
    "mixers": "mixer",
    "mixes": "mix",
    "mixing": "mix",
    "mixtures": "mixture",
    "mjs": "mj",
    "mls": "ml",
    "mmols": "mmol",
    "mms": "mm",
    "mnemonics": "mnemonic",
    "moaned": "moan",
    "moaners": "moaner",
    "moaning": "moan",
    "moanings": "moaning",
    "moans": "moan",
    "moats": "moat",
    "mobbed": "mob",
    "mobbing": "mob",
    "mobiles": "mobile",
    "mobilisations": "mobilisation",
    "mobilised": "mobilise",
    "mobilises": "mobilise",
    "mobilising": "mobilise",
    "mobilities": "mobility",
    "mobilizations": "mobilization",
    "mobilized": "mobilize",
    "mobilizes": "mobilize",
    "mobilizing": "mobilize",
    "mobs": "mob",
    "mobsters": "mobster",
    "moccasins": "moccasin",
    "mock-ups": "mock-up",
    "mocked": "mock",
    "mockeries": "mockery",
    "mocking": "mock",
    "mockingbirds": "mockingbird",
    "mocks": "mock",
    "mockups": "mockup",
    "modalities": "modality",
    "modals": "modal",
    "modeled": "model",
    "modelers": "modeler",
    "modeling": "model",
    "modelings": "modeling",
    "modelled": "model",
    "modellers": "modeller",
    "modelling": "model",
    "models": "model",
    "modems": "modem",
    "moderated": "moderate",
    "moderates": "moderate",
    "moderating": "moderate",
    "moderators": "moderator",
    "moderner": "modern",
    "modernest": "modern",
    "modernisations": "modernisation",
    "modernised": "modernise",
    "modernisers": "moderniser",
    "modernises": "modernise",
    "modernising": "modernise",
    "modernisms": "modernism",
    "modernists": "modernist",
    "modernizations": "modernization",
    "modernized": "modernize",
    "modernizes": "modernize",
    "modernizing": "modernize",
    "moderns": "modern",
    "modes": "mode",
    "modicums": "modicum",
    "modifications": "modification",
    "modified": "modify",
    "modifiers": "modifier",
    "modifies": "modify",
    "modifying": "modify",
    "modioli": "modiolus",
    "mods": "mod",
    "modularised": "modularise",
    "modularises": "modularise",
    "modularising": "modularise",
    "modulated": "modulate",
    "modulates": "modulate",
    "modulating": "modulate",
    "modulations": "modulation",
    "modulators": "modulator",
    "modules": "module",
    "moduli": "modulus",
    "moggies": "moggy",
    "moguls": "mogul",
    "moieties": "moiety",
    "mois": "moi",
    "moistened": "moisten",
    "moistening": "moisten",
    "moistens": "moisten",
    "moister": "moist",
    "moistest": "moist",
    "moistures": "moisture",
    "moisturised": "moisturise",
    "moisturisers": "moisturiser",
    "moisturises": "moisturise",
    "moisturising": "moisturise",
    "moisturized": "moisturize",
    "moisturizers": "moisturizer",
    "moisturizes": "moisturize",
    "moisturizing": "moisturize",
    "molars": "molar",
    "molas": "mola",
    "molded": "mold",
    "moldier": "moldy",
    "molding": "mold",
    "moldings": "molding",
    "molds": "mold",
    "molecules": "molecule",
    "molehills": "molehill",
    "moles": "mole",
    "molestations": "molestation",
    "molested": "molest",
    "molesters": "molester",
    "molesting": "molest",
    "molests": "molest",
    "mollies": "molly",
    "mollified": "mollify",
    "mollifies": "mollify",
    "mollifying": "mollify",
    "molls": "moll",
    "molluscs": "mollusc",
    "mollusks": "mollusk",
    "mollycoddled": "mollycoddle",
    "mollycoddles": "mollycoddle",
    "mollycoddling": "mollycoddle",
    "mols": "mol",
    "molted": "molt",
    "molten": "melt",
    "molting": "molt",
    "molts": "molt",
    "momenta": "momentum",
    "momentoes": "momento",
    "momentos": "momento",
    "moments": "moment",
    "momentums": "momentum",
    "mommas": "momma",
    "moms": "mom",
    "monads": "monad",
    "monarches": "monarch",
    "monarchies": "monarchy",
    "monarchists": "monarchist",
    "monarchs": "monarch",
    "monasteries": "monastery",
    "monetarier": "monetary",
    "monetariest": "monetary",
    "monetarists": "monetarist",
    "monetised": "monetise",
    "monetises": "monetise",
    "monetising": "monetise",
    "monetized": "monetize",
    "monetizes": "monetize",
    "monetizing": "monetize",
    "money-boxes": "money-box",
    "money-lenders": "money-lender",
    "money-makers": "money-maker",
    "moneylenders": "moneylender",
    "moneys": "money",
    "mongers": "monger",
    "mongooses": "mongoose",
    "mongrels": "mongrel",
    "monies": "money",
    "monikers": "moniker",
    "monitored": "monitor",
    "monitoring": "monitor",
    "monitorings": "monitoring",
    "monitors": "monitor",
    "monkeyed": "monkey",
    "monkeying": "monkey",
    "monkeys": "monkey",
    "monkfishes": "monkfish",
    "monks": "monk",
    "monoblocs": "monobloc",
    "monochromators": "monochromator",
    "monocles": "monocle",
    "monoclonals": "monoclonal",
    "monocultures": "monoculture",
    "monocytes": "monocyte",
    "monocytogenes": "monocytogene",
    "monofilaments": "monofilament",
    "monograms": "monogram",
    "monographs": "monograph",
    "monohydrates": "monohydrate",
    "monolayers": "monolayer",
    "monoliths": "monolith",
    "monologs": "monolog",
    "monologues": "monologue",
    "monomers": "monomer",
    "monoplanes": "monoplane",
    "monopoles": "monopole",
    "monopolies": "monopoly",
    "monopolised": "monopolise",
    "monopolises": "monopolise",
    "monopolising": "monopolise",
    "monopolists": "monopolist",
    "monopolized": "monopolize",
    "monopolizes": "monopolize",
    "monopolizing": "monopolize",
    "monorails": "monorail",
    "monos": "mono",
    "monosaccharides": "monosaccharide",
    "monosyllables": "monosyllable",
    "monotheists": "monotheist",
    "monotherapies": "monotherapy",
    "monotones": "monotone",
    "monoxides": "monoxide",
    "mons": "mon",
    "monsoons": "monsoon",
    "monsters": "monster",
    "monstrosities": "monstrosity",
    "montages": "montage",
    "montes": "monte",
    "monthlies": "monthly",
    "months": "month",
    "monuments": "monument",
    "mooched": "mooch",
    "mooches": "mooch",
    "mooching": "mooch",
    "moodier": "moody",
    "moodiest": "moody",
    "moods": "mood",
    "mooed": "moo",
    "mooing": "moo",
    "moonbeams": "moonbeam",
    "mooned": "moon",
    "moonier": "moony",
    "mooning": "moon",
    "moonlighted": "moonlight",
    "moonlighting": "moonlight",
    "moonlights": "moonlight",
    "moons": "moon",
    "moonscapes": "moonscape",
    "moonstones": "moonstone",
    "moored": "moor",
    "moorhens": "moorhen",
    "mooring": "moor",
    "moorings": "mooring",
    "moorlands": "moorland",
    "moors": "moor",
    "moos": "moo",
    "mooted": "moot",
    "mooting": "moot",
    "moots": "moot",
    "moped": "mope",
    "mopeds": "moped",
    "mopes": "mope",
    "moping": "mope",
    "mopped": "mop",
    "mopping": "mop",
    "mops": "mop",
    "moquettes": "moquette",
    "moraines": "moraine",
    "moralised": "moralise",
    "moralises": "moralise",
    "moralising": "moralise",
    "moralists": "moralist",
    "moralities": "morality",
    "moralized": "moralize",
    "moralizes": "moralize",
    "moralizing": "moralize",
    "morals": "moral",
    "morasses": "morass",
    "moratoria": "moratorium",
    "moratoriums": "moratorium",
    "morays": "moray",
    "morbidities": "morbidity",
    "mordants": "mordant",
    "more": "much",
    "morels": "morel",
    "morgans": "morgan",
    "morgues": "morgue",
    "mornings": "morning",
    "morns": "morn",
    "morons": "moron",
    "morphed": "morph",
    "morphemes": "morpheme",
    "morphines": "morphine",
    "morphing": "morph",
    "morphogeneses": "morphogenesis",
    "morphologies": "morphology",
    "morphometries": "morphometry",
    "morphs": "morph",
    "morsels": "morsel",
    "mortalities": "mortality",
    "mortals": "mortal",
    "mortarboards": "mortarboard",
    "mortared": "mortar",
    "mortaring": "mortar",
    "mortars": "mortar",
    "mortgaged": "mortgage",
    "mortgagees": "mortgagee",
    "mortgages": "mortgage",
    "mortgaging": "mortgage",
    "mortices": "mortice",
    "morticians": "mortician",
    "mortifications": "mortification",
    "mortified": "mortify",
    "mortifies": "mortify",
    "mortifying": "mortify",
    "mortised": "mortise",
    "mortises": "mortise",
    "mortising": "mortise",
    "mortuaries": "mortuary",
    "mosaicities": "mosaicity",
    "mosaics": "mosaic",
    "moseyed": "mosey",
    "moseying": "mosey",
    "moseys": "mosey",
    "mosques": "mosque",
    "mosquitoes": "mosquito",
    "mosquitos": "mosquito",
    "mosses": "moss",
    "mossier": "mossy",
    "mossiest": "mossy",
    "most": "much",
    "motels": "motel",
    "mothballs": "mothball",
    "mother-in-laws": "mother-in-law",
    "mother-tongues": "mother-tongue",
    "mothered": "mother",
    "mothering": "mother",
    "motherlands": "motherland",
    "mothers": "mother",
    "mothers-in-law": "mother-in-law",
    "mothers-to-be": "mother-to-be",
    "mothproofed": "mothproof",
    "mothproofing": "mothproof",
    "mothproofs": "mothproof",
    "moths": "moth",
    "motifs": "motif",
    "motilities": "motility",
    "motioned": "motion",
    "motioning": "motion",
    "motions": "motion",
    "motivated": "motivate",
    "motivates": "motivate",
    "motivating": "motivate",
    "motivations": "motivation",
    "motivators": "motivator",
    "motives": "motive",
    "motor-cars": "motor-car",
    "motor-cycles": "motor-cycle",
    "motorbikes": "motorbike",
    "motorboats": "motorboat",
    "motorcades": "motorcade",
    "motorcars": "motorcar",
    "motorcycled": "motorcycle",
    "motorcycles": "motorcycle",
    "motorcycling": "motorcycle",
    "motorcyclists": "motorcyclist",
    "motored": "motor",
    "motoring": "motor",
    "motorised": "motorise",
    "motorises": "motorise",
    "motorising": "motorise",
    "motorists": "motorist",
    "motorized": "motorize",
    "motorizes": "motorize",
    "motorizing": "motorize",
    "motors": "motor",
    "motorways": "motorway",
    "mottes": "motte",
    "mottled": "mottle",
    "mottles": "mottle",
    "mottling": "mottle",
    "mottoes": "motto",
    "mottos": "motto",
    "moulded": "mould",
    "mouldered": "moulder",
    "mouldering": "moulder",
    "moulders": "moulder",
    "mouldier": "mouldy",
    "mouldiest": "mouldy",
    "moulding": "mould",
    "mouldings": "moulding",
    "moulds": "mould",
    "moulted": "moult",
    "moulting": "moult",
    "moults": "moult",
    "mounds": "mound",
    "mountaineers": "mountaineer",
    "mountains": "mountain",
    "mountainsides": "mountainside",
    "mountants": "mountant",
    "mountebanks": "mountebank",
    "mounted": "mount",
    "mounting": "mount",
    "mountings": "mounting",
    "mounts": "mount",
    "mourned": "mourn",
    "mourners": "mourner",
    "mourning": "mourn",
    "mournings": "mourning",
    "mourns": "mourn",
    "mouse-clicks": "mouse-click",
    "mousetraps": "mousetrap",
    "mousier": "mousy",
    "mousiest": "mousy",
    "moustaches": "moustache",
    "mouthed": "mouth",
    "mouthfuls": "mouthful",
    "mouthguards": "mouthguard",
    "mouthing": "mouth",
    "mouthparts": "mouthpart",
    "mouthpieces": "mouthpiece",
    "mouths": "mouth",
    "mouthsful": "mouthful",
    "mouthwashes": "mouthwash",
    "movables": "movable",
    "moveables": "moveable",
    "moved": "move",
    "movements": "movement",
    "movers": "mover",
    "moves": "move",
    "moviegoers": "moviegoer",
    "movies": "movie",
    "moving": "move",
    "mowed": "mow",
    "mowers": "mower",
    "mowing": "mow",
    "mown": "mow",
    "mows": "mow",
    "moxibustions": "moxibustion",
    "moyens": "moyen",
    "mpgs": "mpg",
    "msecs": "msec",
    "mss": "ms",
    "mucilages": "mucilage",
    "mucins": "mucin",
    "mucked": "muck",
    "muckier": "mucky",
    "muckiest": "mucky",
    "mucking": "muck",
    "muckrakers": "muckraker",
    "mucks": "muck",
    "mucosae": "mucosa",
    "mucosas": "mucosa",
    "mucositides": "mucositis",
    "muddied": "muddy",
    "muddier": "muddy",
    "muddies": "muddy",
    "muddiest": "muddy",
    "muddled": "muddle",
    "muddles": "muddle",
    "muddling": "muddle",
    "muddying": "muddy",
    "mudflats": "mudflat",
    "mudguards": "mudguard",
    "mudpacks": "mudpack",
    "mudras": "mudra",
    "muezzins": "muezzin",
    "muffed": "muff",
    "muffing": "muff",
    "muffins": "muffin",
    "muffled": "muffle",
    "mufflers": "muffler",
    "muffles": "muffle",
    "muffling": "muffle",
    "muffs": "muff",
    "muftis": "mufti",
    "mugged": "mug",
    "muggers": "mugger",
    "muggier": "muggy",
    "muggiest": "muggy",
    "mugging": "mug",
    "muggings": "mugging",
    "mugs": "mug",
    "mulattoes": "mulatto",
    "mulattos": "mulatto",
    "mulberries": "mulberry",
    "mulched": "mulch",
    "mulches": "mulch",
    "mulching": "mulch",
    "mules": "mule",
    "mullahs": "mullah",
    "mulled": "mull",
    "mullets": "mullet",
    "mulling": "mull",
    "mulls": "mull",
    "multi-authors": "multi-author",
    "multi-billions": "multi-billion",
    "multi-columns": "multi-column",
    "multi-criteria": "multi-criterion",
    "multi-disciplines": "multi-discipline",
    "multi-employers": "multi-employer",
    "multi-functions": "multi-function",
    "multi-layers": "multi-layer",
    "multi-members": "multi-member",
    "multi-millionaires": "multi-millionaire",
    "multi-millions": "multi-million",
    "multi-nationals": "multi-national",
    "multi-processors": "multi-processor",
    "multi-sectors": "multi-sector",
    "multi-stakeholders": "multi-stakeholder",
    "multi-tasked": "multi-task",
    "multi-tasking": "multi-task",
    "multi-tasks": "multi-task",
    "multi-users": "multi-user",
    "multi-vendors": "multi-vendor",
    "multi-vitamins": "multi-vitamin",
    "multi-words": "multi-word",
    "multibillions": "multibillion",
    "multicasts": "multicast",
    "multiculturalisms": "multiculturalism",
    "multiculturalists": "multiculturalist",
    "multifunctions": "multifunction",
    "multilayers": "multilayer",
    "multimaps": "multimap",
    "multimeters": "multimeter",
    "multimodalities": "multimodality",
    "multinationals": "multinational",
    "multinomials": "multinomial",
    "multipaths": "multipath",
    "multiples": "multiple",
    "multiplexers": "multiplexer",
    "multiplexes": "multiplex",
    "multiplexors": "multiplexor",
    "multiplications": "multiplication",
    "multiplicities": "multiplicity",
    "multiplied": "multiply",
    "multipliers": "multiplier",
    "multiplies": "multiply",
    "multiplying": "multiply",
    "multipoles": "multipole",
    "multiprocessors": "multiprocessor",
    "multiresolutions": "multiresolution",
    "multis": "multi",
    "multisets": "multiset",
    "multitasked": "multitask",
    "multitasking": "multitask",
    "multitasks": "multitask",
    "multitudes": "multitude",
    "multiusers": "multiuser",
    "multivitamins": "multivitamin",
    "multiwords": "multiword",
    "mumbled": "mumble",
    "mumbles": "mumble",
    "mumbling": "mumble",
    "mummies": "mummy",
    "mummifications": "mummification",
    "mummified": "mummify",
    "mummifies": "mummify",
    "mummifying": "mummify",
    "mumps": "mump",
    "mums": "mum",
    "munched": "munch",
    "munches": "munch",
    "munching": "munch",
    "municipalities": "municipality",
    "munitions": "munition",
    "muntjacs": "muntjac",
    "muons": "muon",
    "murals": "mural",
    "murdered": "murder",
    "murderers": "murderer",
    "murderesses": "murderess",
    "murdering": "murder",
    "murders": "murder",
    "murkier": "murky",
    "murkiest": "murky",
    "murmured": "murmur",
    "murmuring": "murmur",
    "murmurs": "murmur",
    "mus": "mu",
    "muscled": "muscle",
    "muscles": "muscle",
    "muscling": "muscle",
    "musculatures": "musculature",
    "mused": "muse",
    "muses": "muse",
    "museums": "museum",
    "mushers": "musher",
    "mushes": "mush",
    "mushier": "mushy",
    "mushiest": "mushy",
    "mushroomed": "mushroom",
    "mushrooming": "mushroom",
    "mushrooms": "mushroom",
    "musicals": "musical",
    "musicians": "musician",
    "musicologists": "musicologist",
    "musics": "music",
    "musing": "muse",
    "musings": "musing",
    "musketeers": "musketeer",
    "muskets": "musket",
    "muskier": "musky",
    "musks": "musk",
    "mussels": "mussel",
    "mustaches": "mustache",
    "mustangs": "mustang",
    "mustards": "mustard",
    "mustered": "muster",
    "mustering": "muster",
    "musters": "muster",
    "mustier": "musty",
    "mustiest": "musty",
    "musts": "must",
    "mutabilities": "mutability",
    "mutageneses": "mutagenesis",
    "mutagenicities": "mutagenicity",
    "mutagens": "mutagen",
    "mutants": "mutant",
    "mutated": "mutate",
    "mutates": "mutate",
    "mutating": "mutate",
    "mutations": "mutation",
    "mutators": "mutator",
    "muted": "mute",
    "muter": "mute",
    "mutes": "mute",
    "mutest": "mute",
    "mutilated": "mutilate",
    "mutilates": "mutilate",
    "mutilating": "mutilate",
    "mutilations": "mutilation",
    "mutineers": "mutineer",
    "muting": "mute",
    "mutinied": "mutiny",
    "mutinies": "mutiny",
    "mutinying": "mutiny",
    "muttered": "mutter",
    "muttering": "mutter",
    "mutters": "mutter",
    "mutts": "mutt",
    "mutualisms": "mutualism",
    "mutualities": "mutuality",
    "muzzier": "muzzy",
    "muzzled": "muzzle",
    "muzzles": "muzzle",
    "muzzling": "muzzle",
    "mvs": "mv",
    "myalgias": "myalgia",
    "myasthenias": "myasthenia",
    "mycelia": "mycelium",
    "mycobacteria": "mycobacterium",
    "mycologies": "mycology",
    "mycologists": "mycologist",
    "mycoplamae": "mycoplasma",
    "mycoplasmas": "mycoplasma",
    "mycoplasmata": "mycoplasma",
    "mycorrhizae": "mycorrhiza",
    "mycorrhizas": "mycorrhiza",
    "mycoses": "mycosis",
    "mycotoxins": "mycotoxin",
    "myelins": "myelin",
    "myelofibroses": "myelofibrosis",
    "myelomas": "myeloma",
    "myelomata": "myeloma",
    "myocardia": "myocardium",
    "myocloni": "myoclonus",
    "myocytes": "myocyte",
    "myoglobins": "myoglobin",
    "myopathies": "myopathy",
    "myopias": "myopia",
    "myosins": "myosin",
    "myositides": "myositis",
    "myriads": "myriad",
    "mysteries": "mystery",
    "mysticisms": "mysticism",
    "mystics": "mystic",
    "mystified": "mystify",
    "mystifies": "mystify",
    "mystifying": "mystify",
    "mythologies": "mythology",
    "mythologised": "mythologise",
    "mythologises": "mythologise",
    "mythologising": "mythologise",
    "mythologized": "mythologize",
    "mythologizes": "mythologize",
    "mythologizing": "mythologize",
    "myths": "myth",
    "myxomatoses": "myxomatosis",
    "n't": "not",
    "nabbed": "nab",
    "nabbing": "nab",
    "nabs": "nab",
    "nacelles": "nacelle",
    "nadirs": "nadir",
    "naevi": "naevus",
    "naevuses": "naevus",
    "nagged": "nag",
    "nagging": "nag",
    "nags": "nag",
    "nailbrushes": "nailbrush",
    "nailed": "nail",
    "nailfiles": "nailfile",
    "nailing": "nail",
    "nails": "nail",
    "naiver": "naive",
    "naivest": "naive",
    "named": "name",
    "nameplates": "nameplate",
    "names": "name",
    "namesakes": "namesake",
    "naming": "name",
    "nanas": "nana",
    "nannies": "nanny",
    "nano-particles": "nano-particle",
    "nano-technologies": "nano-technology",
    "nanocrystals": "nanocrystal",
    "nanometers": "nanometer",
    "nanometres": "nanometre",
    "nanoparticles": "nanoparticle",
    "nanoseconds": "nanosecond",
    "nanostructures": "nanostructure",
    "nanotechnologies": "nanotechnology",
    "nanotubes": "nanotube",
    "nanowires": "nanowire",
    "napalmed": "napalm",
    "napalming": "napalm",
    "napalms": "napalm",
    "napes": "nape",
    "naphthalenes": "naphthalene",
    "naphthas": "naphtha",
    "napkins": "napkin",
    "napped": "nap",
    "nappies": "nappy",
    "napping": "nap",
    "naproxens": "naproxen",
    "naps": "nap",
    "narcissi": "narcissus",
    "narcissisms": "narcissism",
    "narcissists": "narcissist",
    "narcissuses": "narcissus",
    "narcolepsies": "narcolepsy",
    "narcoses": "narcosis",
    "narcotics": "narcotic",
    "narked": "nark",
    "narking": "nark",
    "narks": "nark",
    "narrated": "narrate",
    "narrates": "narrate",
    "narrating": "narrate",
    "narrations": "narration",
    "narratives": "narrative",
    "narrators": "narrator",
    "narrowed": "narrow",
    "narrower": "narrow",
    "narrowest": "narrow",
    "narrowing": "narrow",
    "narrowings": "narrowing",
    "narrownesses": "narrowness",
    "narrows": "narrow",
    "nasals": "nasal",
    "nasopharynges": "nasopharynx",
    "nasopharynxes": "nasopharynx",
    "nastier": "nasty",
    "nastiest": "nasty",
    "nasturtiums": "nasturtium",
    "nation-states": "nation-state",
    "nationalisations": "nationalisation",
    "nationalised": "nationalise",
    "nationalises": "nationalise",
    "nationalising": "nationalise",
    "nationalists": "nationalist",
    "nationalities": "nationality",
    "nationalizations": "nationalization",
    "nationalized": "nationalize",
    "nationalizes": "nationalize",
    "nationalizing": "nationalize",
    "nationals": "national",
    "nations": "nation",
    "natives": "native",
    "natriuretics": "natriuretic",
    "nattered": "natter",
    "nattering": "natter",
    "natterjacks": "natterjack",
    "natters": "natter",
    "nattier": "natty",
    "nattiest": "natty",
    "naturalisations": "naturalisation",
    "naturalised": "naturalise",
    "naturalises": "naturalise",
    "naturalising": "naturalise",
    "naturalists": "naturalist",
    "naturalizations": "naturalization",
    "naturalized": "naturalize",
    "naturalizes": "naturalize",
    "naturalizing": "naturalize",
    "naturals": "natural",
    "natures": "nature",
    "naturists": "naturist",
    "naturopaths": "naturopath",
    "naughtier": "naughty",
    "naughtiest": "naughty",
    "naughts": "naught",
    "nauseated": "nauseate",
    "nauseates": "nauseate",
    "nauseating": "nauseate",
    "nautili": "nautilus",
    "nautiluses": "nautilus",
    "navels": "navel",
    "naves": "nave",
    "navies": "navy",
    "navigated": "navigate",
    "navigates": "navigate",
    "navigating": "navigate",
    "navigators": "navigator",
    "navvies": "navvy",
    "nays": "nay",
    "naysayers": "naysayer",
    "naïver": "naïve",
    "naïvest": "naïve",
    "neared": "near",
    "nearer": "near",
    "nearest": "near",
    "nearing": "near",
    "nears": "near",
    "nearsides": "nearside",
    "neater": "neat",
    "neatest": "neat",
    "nebs": "neb",
    "nebulae": "nebula",
    "nebulas": "nebula",
    "nebulisers": "nebuliser",
    "necessaries": "necessary",
    "necessitated": "necessitate",
    "necessitates": "necessitate",
    "necessitating": "necessitate",
    "necessities": "necessity",
    "necked": "neck",
    "neckerchiefs": "neckerchief",
    "necking": "neck",
    "necklaces": "necklace",
    "necklets": "necklet",
    "necklines": "neckline",
    "necks": "neck",
    "neckties": "necktie",
    "necromancers": "necromancer",
    "necropoleis": "necropolis",
    "necropoles": "necropolis",
    "necropoli": "necropolis",
    "necropolises": "necropolis",
    "necropsies": "necropsy",
    "necroses": "necrosis",
    "necrotised": "necrotise",
    "necrotises": "necrotise",
    "necrotising": "necrotise",
    "nectarines": "nectarine",
    "needed": "need",
    "needier": "needy",
    "neediest": "needy",
    "needing": "need",
    "needled": "needle",
    "needlepoints": "needlepoint",
    "needles": "needle",
    "needlesticks": "needlestick",
    "needlewomen": "needlewoman",
    "needling": "needle",
    "needs": "need",
    "neems": "neem",
    "nefs": "nef",
    "negated": "negate",
    "negates": "negate",
    "negating": "negate",
    "negations": "negation",
    "negatived": "negative",
    "negatively": "negative",
    "negatives": "negative",
    "negativing": "negative",
    "negativities": "negativity",
    "neglected": "neglect",
    "neglecting": "neglect",
    "neglects": "neglect",
    "negligees": "negligee",
    "negotiated": "negotiate",
    "negotiates": "negotiate",
    "negotiating": "negotiate",
    "negotiations": "negotiation",
    "negotiators": "negotiator",
    "negroids": "negroid",
    "neighbored": "neighbor",
    "neighborhoods": "neighborhood",
    "neighboring": "neighbor",
    "neighbors": "neighbor",
    "neighboured": "neighbour",
    "neighbourhoods": "neighbourhood",
    "neighbouring": "neighbour",
    "neighbours": "neighbour",
    "neighed": "neigh",
    "neighing": "neigh",
    "neighs": "neigh",
    "nematicides": "nematicide",
    "nematodes": "nematode",
    "nemeses": "nemesis",
    "neo-Nazis": "neo-Nazi",
    "neocortices": "neocortex",
    "neologisms": "neologism",
    "neomycins": "neomycin",
    "neonates": "neonate",
    "neons": "neon",
    "neophytes": "neophyte",
    "neoplasiae": "neoplasia",
    "neoplasias": "neoplasia",
    "neoplasms": "neoplasm",
    "neoprenes": "neoprene",
    "nephews": "nephew",
    "nephrectomies": "nephrectomy",
    "nephritides": "nephritis",
    "nephrologies": "nephrology",
    "nephrologists": "nephrologist",
    "nephrons": "nephron",
    "nephropathies": "nephropathy",
    "nerdier": "nerdy",
    "nerds": "nerd",
    "nerved": "nerve",
    "nerves": "nerve",
    "nerving": "nerve",
    "nested": "nest",
    "nesters": "nester",
    "nesting": "nest",
    "nestled": "nestle",
    "nestles": "nestle",
    "nestling": "nestle",
    "nestlings": "nestling",
    "nests": "nest",
    "netizens": "netizen",
    "nets": "net",
    "netted": "net",
    "netting": "net",
    "nettled": "nettle",
    "nettles": "nettle",
    "nettling": "nettle",
    "networked": "network",
    "networkers": "networker",
    "networking": "network",
    "networks": "network",
    "neuralgias": "neuralgia",
    "neuraminidases": "neuraminidase",
    "neurites": "neurite",
    "neuritides": "neuritis",
    "neuroanatomies": "neuroanatomy",
    "neurobiologies": "neurobiology",
    "neuroblastomas": "neuroblastoma",
    "neuroblastomata": "neuroblastoma",
    "neuroendocrines": "neuroendocrine",
    "neurofibromatoses": "neurofibromatosis",
    "neurogeneses": "neurogenesis",
    "neuroimagings": "neuroimaging",
    "neuroleptics": "neuroleptic",
    "neurologies": "neurology",
    "neurologists": "neurologist",
    "neuromas": "neuroma",
    "neuromata": "neuroma",
    "neurones": "neurone",
    "neurons": "neuron",
    "neuropathies": "neuropathy",
    "neuropathologies": "neuropathology",
    "neuropathologists": "neuropathologist",
    "neuropeptides": "neuropeptide",
    "neurophysiologies": "neurophysiology",
    "neurophysiologists": "neurophysiologist",
    "neuropsychologists": "neuropsychologist",
    "neurosciences": "neuroscience",
    "neuroscientists": "neuroscientist",
    "neuroses": "neurosis",
    "neurosurgeons": "neurosurgeon",
    "neurosurgeries": "neurosurgery",
    "neurotics": "neurotic",
    "neurotoxicities": "neurotoxicity",
    "neurotoxins": "neurotoxin",
    "neurotransmissions": "neurotransmission",
    "neurotransmitters": "neurotransmitter",
    "neus": "neu",
    "neutered": "neuter",
    "neutering": "neuter",
    "neuters": "neuter",
    "neutralisations": "neutralisation",
    "neutralised": "neutralise",
    "neutralises": "neutralise",
    "neutralising": "neutralise",
    "neutralizations": "neutralization",
    "neutralized": "neutralize",
    "neutralizes": "neutralize",
    "neutralizing": "neutralize",
    "neutrals": "neutral",
    "neutrinos": "neutrino",
    "neutrons": "neutron",
    "neutropenias": "neutropenia",
    "neutrophils": "neutrophil",
    "nevi": "nevus",
    "nevuses": "nevus",
    "newborns": "newborn",
    "newcomers": "newcomer",
    "newels": "newel",
    "newer": "new",
    "newest": "new",
    "newlyweds": "newlywed",
    "news-stands": "news-stand",
    "newsagents": "newsagent",
    "newscasters": "newscaster",
    "newsflashes": "newsflash",
    "newsgroups": "newsgroup",
    "newsier": "newsy",
    "newsletters": "newsletter",
    "newsmen": "newsman",
    "newspapermen": "newspaperman",
    "newspapers": "newspaper",
    "newsreaders": "newsreader",
    "newsreels": "newsreel",
    "newsrooms": "newsroom",
    "newsstands": "newsstand",
    "newtons": "newton",
    "newts": "newt",
    "nexuses": "nexus",
    "ngs": "ng",
    "nibbled": "nibble",
    "nibbles": "nibble",
    "nibbling": "nibble",
    "nibs": "nib",
    "nicer": "nice",
    "nicest": "nice",
    "niceties": "nicety",
    "niches": "niche",
    "nicked": "nick",
    "nickels": "nickel",
    "nicking": "nick",
    "nicknamed": "nickname",
    "nicknames": "nickname",
    "nicknaming": "nickname",
    "nicks": "nick",
    "nicotines": "nicotine",
    "nieces": "niece",
    "niffier": "niffy",
    "niftier": "nifty",
    "niftiest": "nifty",
    "niggers": "nigger",
    "niggled": "niggle",
    "niggles": "niggle",
    "niggling": "niggle",
    "night-clubs": "night-club",
    "night-shifts": "night-shift",
    "night-times": "night-time",
    "night-watchmen": "night-watchman",
    "nightcaps": "nightcap",
    "nightclubs": "nightclub",
    "nightdresses": "nightdress",
    "nightfalls": "nightfall",
    "nightgowns": "nightgown",
    "nighties": "nighty",
    "nightingales": "nightingale",
    "nightjars": "nightjar",
    "nightlights": "nightlight",
    "nightmares": "nightmare",
    "nights": "night",
    "nightshades": "nightshade",
    "nightshifts": "nightshift",
    "nightshirts": "nightshirt",
    "nightsticks": "nightstick",
    "nigras": "nigra",
    "nihilists": "nihilist",
    "nils": "nil",
    "nimbi": "nimbus",
    "nimbler": "nimble",
    "nimblest": "nimble",
    "nimbuses": "nimbus",
    "nincompoops": "nincompoop",
    "nine-hundredth": "900",
    "nines": "nine",
    "nineteenth": "19",
    "nineteenths": "nineteenth",
    "nineties": "ninety",
    "ninetieth": "90",
    "ninetieths": "ninetieth",
    "ninety-eighth": "98",
    "ninety-eighths": "ninety-eighth",
    "ninety-fifth": "95",
    "ninety-fifths": "ninety-fifth",
    "ninety-first": "91",
    "ninety-firsts": "ninety-first",
    "ninety-fourth": "94",
    "ninety-fourths": "ninety-fourth",
    "ninety-ninth": "99",
    "ninety-ninths": "ninety-ninth",
    "ninety-second": "92",
    "ninety-seconds": "ninety-second",
    "ninety-seventh": "97",
    "ninety-sevenths": "ninety-seventh",
    "ninety-sixth": "96",
    "ninety-sixths": "ninety-sixth",
    "ninety-third": "93",
    "ninety-thirds": "ninety-third",
    "ninjas": "ninja",
    "ninnies": "ninny",
    "ninth": "9",
    "ninths": "ninth",
    "nipped": "nip",
    "nippers": "nipper",
    "nippier": "nippy",
    "nippiest": "nippy",
    "nipping": "nip",
    "nipples": "nipple",
    "nips": "nip",
    "nirvanas": "nirvana",
    "nit-pickings": "nit-picking",
    "nitpickings": "nitpicking",
    "nitrates": "nitrate",
    "nitrides": "nitride",
    "nitriles": "nitrile",
    "nitrites": "nitrite",
    "nitrogens": "nitrogen",
    "nitrosamines": "nitrosamine",
    "nits": "nit",
    "nitwits": "nitwit",
    "nixed": "nix",
    "nixes": "nix",
    "nixing": "nix",
    "nms": "nm",
    "nns": "nn",
    "no's": "no",
    "no-brainers": "no-brainer",
    "nobbled": "nobble",
    "nobbles": "nobble",
    "nobbling": "nobble",
    "nobilities": "nobility",
    "noblemen": "nobleman",
    "nobler": "noble",
    "nobles": "noble",
    "noblest": "noble",
    "noblewomen": "noblewoman",
    "nobodies": "nobody",
    "nobs": "nob",
    "nociceptors": "nociceptor",
    "nocturnes": "nocturne",
    "nodded": "nod",
    "noddies": "noddy",
    "nodding": "nod",
    "noddles": "noddle",
    "nodes": "node",
    "nodi": "nodus",
    "nods": "nod",
    "nodules": "nodule",
    "noes": "no",
    "noggins": "noggin",
    "noirs": "noir",
    "noised": "noise",
    "noises": "noise",
    "noisier": "noisy",
    "noisiest": "noisy",
    "noising": "noise",
    "nomads": "nomad",
    "nomenclatures": "nomenclature",
    "nomina": "nomen",
    "nominalists": "nominalist",
    "nominals": "nominal",
    "nominated": "nominate",
    "nominates": "nominate",
    "nominating": "nominate",
    "nominations": "nomination",
    "nominatives": "nominative",
    "nominators": "nominator",
    "nominees": "nominee",
    "non-actions": "non-action",
    "non-attendances": "non-attendance",
    "non-attenders": "non-attender",
    "non-believers": "non-believer",
    "non-citizens": "non-citizen",
    "non-combatants": "non-combatant",
    "non-compliances": "non-compliance",
    "non-conformities": "non-conformity",
    "non-contacts": "non-contact",
    "non-deliveries": "non-delivery",
    "non-drinkers": "non-drinker",
    "non-drugs": "non-drug",
    "non-emergencies": "non-emergency",
    "non-employees": "non-employee",
    "non-entities": "non-entity",
    "non-events": "non-event",
    "non-executives": "non-executive",
    "non-existences": "non-existence",
    "non-experts": "non-expert",
    "non-graduates": "non-graduate",
    "non-groups": "non-group",
    "non-humans": "non-human",
    "non-interventions": "non-intervention",
    "non-issues": "non-issue",
    "non-linearities": "non-linearity",
    "non-melanomas": "non-melanoma",
    "non-members": "non-member",
    "non-metals": "non-metal",
    "non-participants": "non-participant",
    "non-persons": "non-person",
    "non-players": "non-player",
    "non-prices": "non-price",
    "non-principals": "non-principal",
    "non-professionals": "non-professional",
    "non-profits": "non-profit",
    "non-readers": "non-reader",
    "non-residents": "non-resident",
    "non-respondents": "non-respondent",
    "non-responders": "non-responder",
    "non-responses": "non-response",
    "non-returns": "non-return",
    "non-runners": "non-runner",
    "non-scientists": "non-scientist",
    "non-smokers": "non-smoker",
    "non-specialists": "non-specialist",
    "non-starters": "non-starter",
    "non-steroidals": "non-steroidal",
    "non-students": "non-student",
    "non-targets": "non-target",
    "non-users": "non-user",
    "non-whites": "non-white",
    "non-words": "non-word",
    "noncitizens": "noncitizen",
    "noncombatants": "noncombatant",
    "noncompliances": "noncompliance",
    "nonconformists": "nonconformist",
    "nonconformities": "nonconformity",
    "nonentities": "nonentity",
    "nonets": "nonet",
    "nonexistences": "nonexistence",
    "nongroups": "nongroup",
    "nonhumans": "nonhuman",
    "noninterventions": "nonintervention",
    "nonlinearities": "nonlinearity",
    "nonpluses": "nonplus",
    "nonplussed": "nonplus",
    "nonplussing": "nonplus",
    "nonprofits": "nonprofit",
    "nonreplacements": "nonreplacement",
    "nonsmokers": "nonsmoker",
    "nonwords": "nonword",
    "nonylphenols": "nonylphenol",
    "noodles": "noodle",
    "nooks": "nook",
    "noons": "noon",
    "nooses": "noose",
    "nope": "no",
    "normalcies": "normalcy",
    "normalisations": "normalisation",
    "normalised": "normalise",
    "normalises": "normalise",
    "normalising": "normalise",
    "normalities": "normality",
    "normalizations": "normalization",
    "normalized": "normalize",
    "normalizers": "normalizer",
    "normalizes": "normalize",
    "normalizing": "normalize",
    "normals": "normal",
    "norms": "norm",
    "northeasts": "northeast",
    "northerners": "northerner",
    "northerns": "northern",
    "norths": "north",
    "northwards": "northward",
    "northwests": "northwest",
    "nose-dives": "nose-dive",
    "nosebags": "nosebag",
    "nosebleeds": "nosebleed",
    "nosed": "nose",
    "nosedived": "nosedive",
    "nosedives": "nosedive",
    "nosediving": "nosedive",
    "nosegays": "nosegay",
    "nosepieces": "nosepiece",
    "nosers-out": "noser-out",
    "noses": "nose",
    "nosey-parkers": "nosey-parker",
    "noseyer": "nosey",
    "noseyest": "nosey",
    "noshes": "nosh",
    "nosier": "nosy",
    "nosiest": "nosy",
    "nosing": "nose",
    "nostra": "nostrum",
    "nostrils": "nostril",
    "nostrums": "nostrum",
    "nota": "notum",
    "notabilities": "notability",
    "notables": "notable",
    "notaries": "notary",
    "notarized": "notarize",
    "notarizes": "notarize",
    "notarizing": "notarize",
    "notated": "notate",
    "notates": "notate",
    "notating": "notate",
    "notations": "notation",
    "notched": "notch",
    "notches": "notch",
    "notching": "notch",
    "note-cases": "note-case",
    "notebooks": "notebook",
    "noted": "note",
    "notepads": "notepad",
    "notes": "note",
    "nothings": "nothing",
    "notice-boards": "notice-board",
    "noticeboards": "noticeboard",
    "noticed": "notice",
    "notices": "notice",
    "noticing": "notice",
    "notifications": "notification",
    "notified": "notify",
    "notifiers": "notifier",
    "notifies": "notify",
    "notifying": "notify",
    "noting": "note",
    "notions": "notion",
    "noughts": "nought",
    "nouns": "noun",
    "nourished": "nourish",
    "nourishes": "nourish",
    "nourishing": "nourish",
    "nouveaux-riches": "nouveau-riche",
    "novae": "nova",
    "novas": "nova",
    "novelettes": "novelette",
    "novelisations": "novelisation",
    "novelists": "novelist",
    "novelizations": "novelization",
    "novellas": "novella",
    "novelle": "novella",
    "novels": "novel",
    "novelties": "novelty",
    "novices": "novice",
    "nozzles": "nozzle",
    "nt": "not",
    "nts": "nt",
    "nuances": "nuance",
    "nubs": "nub",
    "nucleated": "nucleate",
    "nucleates": "nucleate",
    "nucleating": "nucleate",
    "nucleations": "nucleation",
    "nuclei": "nucleus",
    "nucleoli": "nucleolus",
    "nucleons": "nucleon",
    "nucleophiles": "nucleophile",
    "nucleosides": "nucleoside",
    "nucleosomes": "nucleosome",
    "nucleotides": "nucleotide",
    "nucleuses": "nucleus",
    "nuclides": "nuclide",
    "nuder": "nude",
    "nudes": "nude",
    "nudest": "nude",
    "nudged": "nudge",
    "nudges": "nudge",
    "nudging": "nudge",
    "nudibranchs": "nudibranch",
    "nudists": "nudist",
    "nuggets": "nugget",
    "nuisances": "nuisance",
    "nuked": "nuke",
    "nukes": "nuke",
    "nuking": "nuke",
    "nullified": "nullify",
    "nullifies": "nullify",
    "nullifying": "nullify",
    "nullities": "nullity",
    "numbed": "numb",
    "number": "numb",
    "numbered": "number",
    "numbering": "number",
    "numbers": "number",
    "numbest": "numb",
    "numbing": "numb",
    "numbs": "numb",
    "numerals": "numeral",
    "numerations": "numeration",
    "numerators": "numerator",
    "numerics": "numeric",
    "numskulls": "numskull",
    "nunneries": "nunnery",
    "nuns": "nun",
    "nuptials": "nuptial",
    "nursed": "nurse",
    "nursemaids": "nursemaid",
    "nurseries": "nursery",
    "nurserymen": "nurseryman",
    "nurses": "nurse",
    "nursing": "nurse",
    "nursings": "nursing",
    "nurtured": "nurture",
    "nurtures": "nurture",
    "nurturing": "nurture",
    "nutcases": "nutcase",
    "nutcrackers": "nutcracker",
    "nuthatches": "nuthatch",
    "nuthouses": "nuthouse",
    "nutmegs": "nutmeg",
    "nutraceuticals": "nutraceutical",
    "nutrients": "nutrient",
    "nutriments": "nutriment",
    "nutritionists": "nutritionist",
    "nuts": "nut",
    "nutshells": "nutshell",
    "nutted": "nut",
    "nutters": "nutter",
    "nuttier": "nutty",
    "nuttiest": "nutty",
    "nutting": "nut",
    "nuzzled": "nuzzle",
    "nuzzles": "nuzzle",
    "nuzzling": "nuzzle",
    "nylons": "nylon",
    "nymphomaniacs": "nymphomaniac",
    "nymphos": "nympho",
    "nymphs": "nymph",
    "nystagmuses": "nystagmus",
    "o-rings": "o-ring",
    "oafs": "oaf",
    "oaks": "oak",
    "oars": "oar",
    "oarsmen": "oarsman",
    "oases": "oasis",
    "oasts": "oast",
    "oatcakes": "oatcake",
    "oaths": "oath",
    "oats": "oat",
    "obediences": "obedience",
    "obeisances": "obeisance",
    "obelisks": "obelisk",
    "obesities": "obesity",
    "obeyed": "obey",
    "obeying": "obey",
    "obeys": "obey",
    "obfuscated": "obfuscate",
    "obfuscates": "obfuscate",
    "obfuscating": "obfuscate",
    "obits": "obit",
    "obituaries": "obituary",
    "objected": "object",
    "objectifications": "objectification",
    "objectified": "objectify",
    "objectifies": "objectify",
    "objectifying": "objectify",
    "objecting": "object",
    "objections": "objection",
    "objectives": "objective",
    "objectivists": "objectivist",
    "objectivities": "objectivity",
    "objectors": "objector",
    "objects": "object",
    "oblasts": "oblast",
    "oblations": "oblation",
    "obligated": "obligate",
    "obligates": "obligate",
    "obligating": "obligate",
    "obligations": "obligation",
    "obliged": "oblige",
    "obliges": "oblige",
    "obliging": "oblige",
    "obliques": "oblique",
    "obliquities": "obliquity",
    "obliterated": "obliterate",
    "obliterates": "obliterate",
    "obliterating": "obliterate",
    "obliterations": "obliteration",
    "oblongs": "oblong",
    "oboes": "oboe",
    "oboists": "oboist",
    "obscenities": "obscenity",
    "obscurantists": "obscurantist",
    "obscurations": "obscuration",
    "obscured": "obscure",
    "obscurer": "obscure",
    "obscures": "obscure",
    "obscurest": "obscure",
    "obscuring": "obscure",
    "obscurities": "obscurity",
    "observables": "observable",
    "observances": "observance",
    "observations": "observation",
    "observatories": "observatory",
    "observed": "observe",
    "observers": "observer",
    "observes": "observe",
    "observing": "observe",
    "obsessed": "obsess",
    "obsesses": "obsess",
    "obsessing": "obsess",
    "obsessions": "obsession",
    "obsessives": "obsessive",
    "obstacles": "obstacle",
    "obstetricians": "obstetrician",
    "obstetrics": "obstetric",
    "obstructed": "obstruct",
    "obstructing": "obstruct",
    "obstructions": "obstruction",
    "obstructs": "obstruct",
    "obtained": "obtain",
    "obtaining": "obtain",
    "obtains": "obtain",
    "obtruded": "obtrude",
    "obtrudes": "obtrude",
    "obtruding": "obtrude",
    "obverses": "obverse",
    "obviated": "obviate",
    "obviates": "obviate",
    "obviating": "obviate",
    "ocarinas": "ocarina",
    "occasioned": "occasion",
    "occasioning": "occasion",
    "occasions": "occasion",
    "occipita": "occiput",
    "occiputs": "occiput",
    "occluded": "occlude",
    "occludes": "occlude",
    "occluding": "occlude",
    "occlusions": "occlusion",
    "occultations": "occultation",
    "occulted": "occult",
    "occulting": "occult",
    "occultists": "occultist",
    "occults": "occult",
    "occupancies": "occupancy",
    "occupants": "occupant",
    "occupations": "occupation",
    "occupied": "occupy",
    "occupiers": "occupier",
    "occupies": "occupy",
    "occupying": "occupy",
    "occured": "occur",
    "occurences": "occurence",
    "occuring": "occur",
    "occurred": "occur",
    "occurrences": "occurrence",
    "occurring": "occur",
    "occurs": "occur",
    "oceanographers": "oceanographer",
    "oceans": "ocean",
    "ocelots": "ocelot",
    "ochratoxins": "ochratoxin",
    "octagons": "octagon",
    "octahedra": "octahedron",
    "octahedrons": "octahedron",
    "octanes": "octane",
    "octaves": "octave",
    "octavos": "octavo",
    "octets": "octet",
    "octogenarians": "octogenarian",
    "octopi": "octopus",
    "octopuses": "octopus",
    "oculars": "ocular",
    "oculi": "oculus",
    "oculists": "oculist",
    "oddballs": "oddball",
    "odder": "odd",
    "oddest": "odd",
    "oddities": "oddity",
    "oddments": "oddment",
    "odds": "odd",
    "odes": "ode",
    "odometers": "odometer",
    "odors": "odor",
    "odours": "odour",
    "ods": "od",
    "odysseys": "odyssey",
    "oedemas": "oedema",
    "oedemata": "oedema",
    "oesophagi": "oesophagus",
    "oesophagitides": "oesophagitis",
    "oesophaguses": "oesophagus",
    "oestradiols": "oestradiol",
    "oestrogens": "oestrogen",
    "oestruses": "oestrus",
    "oeuvres": "oeuvre",
    "off-days": "off-day",
    "off-licences": "off-licence",
    "off-loaded": "off-load",
    "off-loading": "off-load",
    "off-loads": "off-load",
    "off-sets": "off-set",
    "off-shoots": "off-shoot",
    "off-springs": "off-spring",
    "off-whites": "off-white",
    "offals": "offal",
    "offences": "offence",
    "offended": "offend",
    "offenders": "offender",
    "offending": "offend",
    "offends": "offend",
    "offenses": "offense",
    "offensives": "offensive",
    "offered": "offer",
    "offering": "offer",
    "offerings": "offering",
    "offers": "offer",
    "office-holders": "office-holder",
    "officers": "officer",
    "offices": "office",
    "officials": "official",
    "officiants": "officiant",
    "officiated": "officiate",
    "officiates": "officiate",
    "officiating": "officiate",
    "offloaded": "offload",
    "offloading": "offload",
    "offloads": "offload",
    "offsets": "offset",
    "offsetted": "offset",
    "offsetting": "offset",
    "offshoots": "offshoot",
    "offsides": "offside",
    "offsprings": "offspring",
    "offtakes": "offtake",
    "oftener": "often",
    "oftenest": "often",
    "ogees": "ogee",
    "ogled": "ogle",
    "ogles": "ogle",
    "ogling": "ogle",
    "ogres": "ogre",
    "ohms": "ohm",
    "oilcans": "oilcan",
    "oiled": "oil",
    "oilers": "oiler",
    "oilfields": "oilfield",
    "oilier": "oily",
    "oiliest": "oily",
    "oiling": "oil",
    "oilmen": "oilman",
    "oilrigs": "oilrig",
    "oils": "oil",
    "oilseeds": "oilseed",
    "oilskins": "oilskin",
    "ointments": "ointment",
    "ok": "okay",
    "okayed": "okay",
    "okaying": "okay",
    "okays": "okay",
    "old-timers": "old-timer",
    "older": "old",
    "oldest": "old",
    "oldies": "oldie",
    "olds": "old",
    "oldthinkers": "oldthinker",
    "oleanders": "oleander",
    "olefins": "olefin",
    "oleoresins": "oleoresin",
    "oligarchies": "oligarchy",
    "oligarchs": "oligarch",
    "oligodendrocytes": "oligodendrocyte",
    "oligomers": "oligomer",
    "oligonucleotides": "oligonucleotide",
    "oligopolies": "oligopoly",
    "oligos": "oligo",
    "oligosaccharides": "oligosaccharide",
    "olives": "olive",
    "olivines": "olivine",
    "ologies": "ology",
    "ombudsmen": "ombudsman",
    "omegas": "omega",
    "omelets": "omelet",
    "omelettes": "omelette",
    "omens": "omen",
    "omenta": "omentum",
    "omentums": "omentum",
    "omissions": "omission",
    "omits": "omit",
    "omitted": "omit",
    "omitting": "omit",
    "omnibuses": "omnibus",
    "omnibusses": "omnibus",
    "omnivores": "omnivore",
    "on-sets": "on-set",
    "onchocerciases": "onchocerciasis",
    "oncogenes": "oncogene",
    "oncologies": "oncology",
    "oncologists": "oncologist",
    "one-billionths": "one-billionth",
    "one-hundred-millionths": "one-hundred-millionth",
    "one-hundred-thousandths": "one-hundred-thousandth",
    "one-liners": "one-liner",
    "one-millionths": "one-millionth",
    "one-offs": "one-off",
    "one-steps": "one-step",
    "ones": "one",
    "onions": "onion",
    "onlookers": "onlooker",
    "onsets": "onset",
    "onslaughts": "onslaught",
    "ontogenies": "ontogeny",
    "ontologies": "ontology",
    "onuses": "onus",
    "onwards": "onward",
    "oocysts": "oocyst",
    "oocytes": "oocyte",
    "oophorectomies": "oophorectomy",
    "oos": "oo",
    "oozed": "ooze",
    "oozes": "ooze",
    "oozier": "oozy",
    "oozing": "ooze",
    "opacities": "opacity",
    "opals": "opal",
    "opened": "open",
    "openers": "opener",
    "opening": "open",
    "openings": "opening",
    "opens": "open",
    "opera": "opus",
    "operands": "operand",
    "operas": "opera",
    "operated": "operate",
    "operates": "operate",
    "operating": "operate",
    "operationalisations": "operationalisation",
    "operationalised": "operationalise",
    "operationalises": "operationalise",
    "operationalising": "operationalise",
    "operationalized": "operationalize",
    "operationalizes": "operationalize",
    "operationalizing": "operationalize",
    "operations": "operation",
    "operatives": "operative",
    "operators": "operator",
    "opercula": "operculum",
    "operculums": "operculum",
    "operettas": "operetta",
    "operons": "operon",
    "ophthalmologies": "ophthalmology",
    "ophthalmologists": "ophthalmologist",
    "ophthalmoscopes": "ophthalmoscope",
    "ophthalmoscopies": "ophthalmoscopy",
    "opiates": "opiate",
    "opined": "opine",
    "opines": "opine",
    "opining": "opine",
    "opinions": "opinion",
    "opioids": "opioid",
    "opossums": "opossum",
    "opponents": "opponent",
    "opportunists": "opportunist",
    "opportunities": "opportunity",
    "opposed": "oppose",
    "opposers": "opposer",
    "opposes": "oppose",
    "opposing": "oppose",
    "opposites": "opposite",
    "oppositions": "opposition",
    "oppressed": "oppress",
    "oppresses": "oppress",
    "oppressing": "oppress",
    "oppressions": "oppression",
    "oppressors": "oppressor",
    "ops": "op",
    "opsins": "opsin",
    "opted": "opt",
    "opticians": "optician",
    "optics": "optic",
    "optima": "optimum",
    "optimisations": "optimisation",
    "optimised": "optimise",
    "optimises": "optimise",
    "optimising": "optimise",
    "optimists": "optimist",
    "optimizations": "optimization",
    "optimized": "optimize",
    "optimizes": "optimize",
    "optimizing": "optimize",
    "optimums": "optimum",
    "opting": "opt",
    "optioned": "option",
    "optioning": "option",
    "options": "option",
    "optometrists": "optometrist",
    "opts": "opt",
    "opuses": "opus",
    "ora": "os",
    "oracles": "oracle",
    "orae": "ora",
    "orals": "oral",
    "orang-outangs": "orang-outang",
    "orang-utans": "orang-utan",
    "oranger": "orange",
    "orangeries": "orangery",
    "oranges": "orange",
    "orangest": "orange",
    "orangs": "orang",
    "orangutans": "orangutan",
    "orations": "oration",
    "oratories": "oratory",
    "oratorios": "oratorio",
    "orators": "orator",
    "orbitals": "orbital",
    "orbited": "orbit",
    "orbiters": "orbiter",
    "orbiting": "orbit",
    "orbits": "orbit",
    "orbs": "orb",
    "orchards": "orchard",
    "orchestras": "orchestra",
    "orchestrated": "orchestrate",
    "orchestrates": "orchestrate",
    "orchestrating": "orchestrate",
    "orchestrations": "orchestration",
    "orchestrators": "orchestrator",
    "orchidectomies": "orchidectomy",
    "orchids": "orchid",
    "orcs": "orc",
    "ordained": "ordain",
    "ordaining": "ordain",
    "ordains": "ordain",
    "ordeals": "ordeal",
    "ordered": "order",
    "ordering": "order",
    "orderings": "ordering",
    "orderlies": "orderly",
    "orders": "order",
    "ordinals": "ordinal",
    "ordinances": "ordinance",
    "ordinands": "ordinand",
    "ordinated": "ordinate",
    "ordinates": "ordinate",
    "ordinating": "ordinate",
    "ordinations": "ordination",
    "ordinators": "ordinator",
    "oreganos": "oregano",
    "ores": "ore",
    "orfes": "orfe",
    "organ-grinders": "organ-grinder",
    "organelles": "organelle",
    "organics": "organic",
    "organisations": "organisation",
    "organised": "organise",
    "organisers": "organiser",
    "organises": "organise",
    "organising": "organise",
    "organisms": "organism",
    "organists": "organist",
    "organizations": "organization",
    "organized": "organize",
    "organizers": "organizer",
    "organizes": "organize",
    "organizing": "organize",
    "organochlorines": "organochlorine",
    "organophosphates": "organophosphate",
    "organs": "organ",
    "orgasms": "orgasm",
    "orgies": "orgy",
    "orientalisms": "orientalism",
    "orientalists": "orientalist",
    "orientals": "oriental",
    "orientated": "orientate",
    "orientates": "orientate",
    "orientating": "orientate",
    "orientations": "orientation",
    "oriented": "orient",
    "orienteers": "orienteer",
    "orienting": "orient",
    "orients": "orient",
    "orifices": "orifice",
    "originals": "original",
    "originated": "originate",
    "originates": "originate",
    "originating": "originate",
    "originations": "origination",
    "originators": "originator",
    "origins": "origin",
    "orioles": "oriole",
    "ornamentals": "ornamental",
    "ornamentations": "ornamentation",
    "ornamented": "ornament",
    "ornamenting": "ornament",
    "ornaments": "ornament",
    "ornithines": "ornithine",
    "ornithologists": "ornithologist",
    "oropharynges": "oropharynx",
    "oropharynxes": "oropharynx",
    "orphanages": "orphanage",
    "orphaned": "orphan",
    "orphaning": "orphan",
    "orphans": "orphan",
    "orthodontists": "orthodontist",
    "orthodoxies": "orthodoxy",
    "orthographies": "orthography",
    "orthoptists": "orthoptist",
    "orthoses": "orthosis",
    "orthotics": "orthotic",
    "orthotists": "orthotist",
    "oryxes": "oryx",
    "oscillated": "oscillate",
    "oscillates": "oscillate",
    "oscillating": "oscillate",
    "oscillations": "oscillation",
    "oscillators": "oscillator",
    "oscilloscopes": "oscilloscope",
    "osiers": "osier",
    "osmolalities": "osmolality",
    "osmoses": "osmosis",
    "ospreys": "osprey",
    "ossa": "os",
    "ossicles": "ossicle",
    "ossifications": "ossification",
    "ossified": "ossify",
    "ossifies": "ossify",
    "ossifying": "ossify",
    "ossuaries": "ossuary",
    "osteoarthritides": "osteoarthritis",
    "osteoblasts": "osteoblast",
    "osteoclasts": "osteoclast",
    "osteogeneses": "osteogenesis",
    "osteomalacias": "osteomalacia",
    "osteomyelitides": "osteomyelitis",
    "osteopathies": "osteopathy",
    "osteopaths": "osteopath",
    "osteopenias": "osteopenia",
    "osteophytes": "osteophyte",
    "osteoporoses": "osteoporosis",
    "osteosarcomas": "osteosarcoma",
    "osteosarcomata": "osteosarcoma",
    "osteotomies": "osteotomy",
    "ostracised": "ostracise",
    "ostracises": "ostracise",
    "ostracising": "ostracise",
    "ostracized": "ostracize",
    "ostracizes": "ostracize",
    "ostracizing": "ostracize",
    "ostracods": "ostracod",
    "ostriches": "ostrich",
    "others": "other",
    "otitides": "otitis",
    "otoliths": "otolith",
    "otters": "otter",
    "ouches": "ouch",
    "oughter": "have",
    "ounces": "ounce",
    "ousted": "oust",
    "ousting": "oust",
    "ousts": "oust",
    "out-competed": "out-compete",
    "out-competes": "out-compete",
    "out-competing": "out-compete",
    "out-groups": "out-group",
    "out-migrations": "out-migration",
    "out-patients": "out-patient",
    "out-performed": "out-perform",
    "out-performing": "out-perform",
    "out-performs": "out-perform",
    "out-reaches": "out-reach",
    "out-sourced": "out-source",
    "out-sources": "out-source",
    "out-sourcing": "out-source",
    "outages": "outage",
    "outbacks": "outback",
    "outbade": "outbid",
    "outbidded": "outbid",
    "outbidden": "outbid",
    "outbidding": "outbid",
    "outbids": "outbid",
    "outboards": "outboard",
    "outbreaks": "outbreak",
    "outbuildings": "outbuilding",
    "outbursts": "outburst",
    "outcasts": "outcast",
    "outclassed": "outclass",
    "outclasses": "outclass",
    "outclassing": "outclass",
    "outcomes": "outcome",
    "outcries": "outcry",
    "outcroped": "outcrop",
    "outcroping": "outcrop",
    "outcrops": "outcrop",
    "outcrossed": "outcross",
    "outcrosses": "outcross",
    "outcrossing": "outcross",
    "outdated": "outdate",
    "outdates": "outdate",
    "outdating": "outdate",
    "outdid": "outdo",
    "outdistanced": "outdistance",
    "outdistances": "outdistance",
    "outdistancing": "outdistance",
    "outdoed": "outdo",
    "outdoes": "outdo",
    "outdoing": "outdo",
    "outdone": "outdo",
    "outdoors": "outdoor",
    "outdos": "outdo",
    "outers": "outer",
    "outfalls": "outfall",
    "outfighting": "outfight",
    "outfights": "outfight",
    "outfits": "outfit",
    "outfitted": "outfit",
    "outfitters": "outfitter",
    "outfitting": "outfit",
    "outflanked": "outflank",
    "outflanking": "outflank",
    "outflanks": "outflank",
    "outflows": "outflow",
    "outfought": "outfight",
    "outfoxed": "outfox",
    "outfoxes": "outfox",
    "outfoxing": "outfox",
    "outgases": "outgas",
    "outgassed": "outgas",
    "outgassing": "outgas",
    "outgoings": "outgoing",
    "outgrew": "outgrow",
    "outgroups": "outgroup",
    "outgrowing": "outgrow",
    "outgrown": "outgrow",
    "outgrows": "outgrow",
    "outgrowths": "outgrowth",
    "outhouses": "outhouse",
    "outings": "outing",
    "outlaid": "outlay",
    "outlasted": "outlast",
    "outlasting": "outlast",
    "outlasts": "outlast",
    "outlawed": "outlaw",
    "outlawing": "outlaw",
    "outlaws": "outlaw",
    "outlaying": "outlay",
    "outlays": "outlay",
    "outlets": "outlet",
    "outliers": "outlier",
    "outlined": "outline",
    "outlines": "outline",
    "outlining": "outline",
    "outlived": "outlive",
    "outlives": "outlive",
    "outliving": "outlive",
    "outlooks": "outlook",
    "outmanoeuvred": "outmanoeuvre",
    "outmanoeuvres": "outmanoeuvre",
    "outmanoeuvring": "outmanoeuvre",
    "outmatched": "outmatch",
    "outmatches": "outmatch",
    "outmatching": "outmatch",
    "outnumbered": "outnumber",
    "outnumbering": "outnumber",
    "outnumbers": "outnumber",
    "outpaced": "outpace",
    "outpaces": "outpace",
    "outpacing": "outpace",
    "outpatients": "outpatient",
    "outperformed": "outperform",
    "outperforming": "outperform",
    "outperforms": "outperform",
    "outplayed": "outplay",
    "outplaying": "outplay",
    "outplays": "outplay",
    "outposts": "outpost",
    "outpourings": "outpouring",
    "outputs": "output",
    "outputted": "output",
    "outputting": "output",
    "outraged": "outrage",
    "outrages": "outrage",
    "outraging": "outrage",
    "outran": "outrun",
    "outranked": "outrank",
    "outranking": "outrank",
    "outranks": "outrank",
    "outreached": "outreach",
    "outreaches": "outreach",
    "outreaching": "outreach",
    "outridden": "outride",
    "outriders": "outrider",
    "outrides": "outride",
    "outriding": "outride",
    "outriggers": "outrigger",
    "outrode": "outride",
    "outrunning": "outrun",
    "outruns": "outrun",
    "outs": "out",
    "outselling": "outsell",
    "outsells": "outsell",
    "outsets": "outset",
    "outshines": "outshine",
    "outshining": "outshine",
    "outshone": "outshine",
    "outsiders": "outsider",
    "outsides": "outside",
    "outsmarted": "outsmart",
    "outsmarting": "outsmart",
    "outsmarts": "outsmart",
    "outsold": "outsell",
    "outsourced": "outsource",
    "outsources": "outsource",
    "outsourcing": "outsource",
    "outspreading": "outspread",
    "outspreads": "outspread",
    "outstations": "outstation",
    "outstayed": "outstay",
    "outstaying": "outstay",
    "outstays": "outstay",
    "outstretched": "outstretch",
    "outstretches": "outstretch",
    "outstretching": "outstretch",
    "outstripped": "outstrip",
    "outstripping": "outstrip",
    "outstrips": "outstrip",
    "outvoted": "outvote",
    "outvotes": "outvote",
    "outvoting": "outvote",
    "outwards": "outward",
    "outwearing": "outwear",
    "outwears": "outwear",
    "outweighed": "outweigh",
    "outweighing": "outweigh",
    "outweighs": "outweigh",
    "outwits": "outwit",
    "outwitted": "outwit",
    "outwitting": "outwit",
    "outwore": "outwear",
    "outworked": "outwork",
    "outworking": "outwork",
    "outworks": "outwork",
    "outworn": "outwear",
    "ova": "ovum",
    "ovals": "oval",
    "ovaries": "ovary",
    "ovations": "ovation",
    "ovens": "oven",
    "over-ate": "over-eat",
    "over-burdened": "over-burden",
    "over-burdening": "over-burden",
    "over-burdens": "over-burden",
    "over-capacities": "over-capacity",
    "over-charged": "over-charge",
    "over-charges": "over-charge",
    "over-charging": "over-charge",
    "over-eaten": "over-eat",
    "over-eating": "over-eat",
    "over-eats": "over-eat",
    "over-emphases": "over-emphasis",
    "over-emphasised": "over-emphasise",
    "over-emphasises": "over-emphasise",
    "over-emphasising": "over-emphasise",
    "over-estimated": "over-estimate",
    "over-estimates": "over-estimate",
    "over-estimating": "over-estimate",
    "over-exposed": "over-expose",
    "over-exposes": "over-expose",
    "over-exposing": "over-expose",
    "over-exposures": "over-exposure",
    "over-expressed": "over-express",
    "over-expresses": "over-express",
    "over-expressing": "over-express",
    "over-expressions": "over-expression",
    "over-heated": "over-heat",
    "over-heating": "over-heat",
    "over-heats": "over-heat",
    "over-indulged": "over-indulge",
    "over-indulgences": "over-indulgence",
    "over-indulges": "over-indulge",
    "over-indulging": "over-indulge",
    "over-looked": "over-look",
    "over-looking": "over-look",
    "over-looks": "over-look",
    "over-payments": "over-payment",
    "over-produced": "over-produce",
    "over-produces": "over-produce",
    "over-producing": "over-produce",
    "over-reacted": "over-react",
    "over-reacting": "over-react",
    "over-reactions": "over-reaction",
    "over-reacts": "over-react",
    "over-representations": "over-representation",
    "over-represented": "over-represent",
    "over-representing": "over-represent",
    "over-represents": "over-represent",
    "over-ridden": "over-ride",
    "over-rides": "over-ride",
    "over-riding": "over-ride",
    "over-rode": "over-ride",
    "over-simplifications": "over-simplification",
    "over-simplified": "over-simplify",
    "over-simplifies": "over-simplify",
    "over-simplifying": "over-simplify",
    "over-stretched": "over-stretch",
    "over-stretches": "over-stretch",
    "over-stretching": "over-stretch",
    "over-supplies": "over-supply",
    "over-used": "over-use",
    "over-uses": "over-use",
    "over-using": "over-use",
    "over-valued": "over-value",
    "over-values": "over-value",
    "over-valuing": "over-value",
    "over-writes": "over-write",
    "over-writing": "over-write",
    "over-written": "over-write",
    "over-wrote": "over-write",
    "overacted": "overact",
    "overacting": "overact",
    "overacts": "overact",
    "overalls": "overall",
    "overarched": "overarch",
    "overarches": "overarch",
    "overarching": "overarch",
    "overate": "overeat",
    "overawed": "overawe",
    "overawes": "overawe",
    "overawing": "overawe",
    "overbalanced": "overbalance",
    "overbalances": "overbalance",
    "overbalancing": "overbalance",
    "overbearing": "overbear",
    "overbears": "overbear",
    "overbidding": "overbid",
    "overbids": "overbid",
    "overbooked": "overbook",
    "overbooking": "overbook",
    "overbookings": "overbooking",
    "overbooks": "overbook",
    "overbore": "overbear",
    "overborne": "overbear",
    "overbridges": "overbridge",
    "overburdened": "overburden",
    "overburdening": "overburden",
    "overburdens": "overburden",
    "overcame": "overcome",
    "overcapacities": "overcapacity",
    "overcharged": "overcharge",
    "overcharges": "overcharge",
    "overcharging": "overcharge",
    "overcoats": "overcoat",
    "overcomes": "overcome",
    "overcoming": "overcome",
    "overcompensated": "overcompensate",
    "overcompensates": "overcompensate",
    "overcompensating": "overcompensate",
    "overcooked": "overcook",
    "overcooking": "overcook",
    "overcooks": "overcook",
    "overcrowded": "overcrowd",
    "overcrowding": "overcrowd",
    "overcrowds": "overcrowd",
    "overdeveloped": "overdevelop",
    "overdeveloping": "overdevelop",
    "overdevelops": "overdevelop",
    "overdid": "overdo",
    "overdoed": "overdo",
    "overdoes": "overdo",
    "overdoing": "overdo",
    "overdone": "overdo",
    "overdos": "overdo",
    "overdosages": "overdosage",
    "overdosed": "overdose",
    "overdoses": "overdose",
    "overdosing": "overdose",
    "overdrafts": "overdraft",
    "overdrawing": "overdraw",
    "overdrawn": "overdraw",
    "overdraws": "overdraw",
    "overdressed": "overdress",
    "overdresses": "overdress",
    "overdressing": "overdress",
    "overdrew": "overdraw",
    "overdriven": "overdrive",
    "overdrives": "overdrive",
    "overdriving": "overdrive",
    "overdrove": "overdrive",
    "overeaten": "overeat",
    "overeating": "overeat",
    "overeats": "overeat",
    "overemphases": "overemphasis",
    "overemphasised": "overemphasise",
    "overemphasises": "overemphasise",
    "overemphasising": "overemphasise",
    "overemphasized": "overemphasize",
    "overemphasizes": "overemphasize",
    "overemphasizing": "overemphasize",
    "overestimated": "overestimate",
    "overestimates": "overestimate",
    "overestimating": "overestimate",
    "overestimations": "overestimation",
    "overexertions": "overexertion",
    "overexposed": "overexpose",
    "overexposes": "overexpose",
    "overexposing": "overexpose",
    "overexposures": "overexposure",
    "overexpressed": "overexpress",
    "overexpresses": "overexpress",
    "overexpressing": "overexpress",
    "overexpressions": "overexpression",
    "overextended": "overextend",
    "overextending": "overextend",
    "overextends": "overextend",
    "overextensions": "overextension",
    "overfed": "overfeed",
    "overfeeding": "overfeed",
    "overfeeds": "overfeed",
    "overfilled": "overfill",
    "overfilling": "overfill",
    "overfills": "overfill",
    "overflew": "overfly",
    "overflies": "overfly",
    "overflights": "overflight",
    "overflowed": "overflow",
    "overflowing": "overflow",
    "overflown": "overfly",
    "overflows": "overflow",
    "overflying": "overfly",
    "overgrazings": "overgrazing",
    "overgrew": "overgrow",
    "overgrowing": "overgrow",
    "overgrown": "overgrow",
    "overgrows": "overgrow",
    "overgrowths": "overgrowth",
    "overhanged": "overhang",
    "overhanging": "overhang",
    "overhangs": "overhang",
    "overhauled": "overhaul",
    "overhauling": "overhaul",
    "overhauls": "overhaul",
    "overheads": "overhead",
    "overheard": "overhear",
    "overhearing": "overhear",
    "overhears": "overhear",
    "overheated": "overheat",
    "overheating": "overheat",
    "overheats": "overheat",
    "overhung": "overhang",
    "overindulged": "overindulge",
    "overindulgences": "overindulgence",
    "overindulges": "overindulge",
    "overindulging": "overindulge",
    "overlaid": "overlay",
    "overlain": "overlie",
    "overlapped": "overlap",
    "overlapping": "overlap",
    "overlaps": "overlap",
    "overlay": "overlie",
    "overlayed": "overlay",
    "overlayers": "overlayer",
    "overlaying": "overlay",
    "overlays": "overlay",
    "overleaped": "overleap",
    "overleaping": "overleap",
    "overleaps": "overleap",
    "overleapt": "overleap",
    "overlies": "overlie",
    "overloaded": "overload",
    "overloading": "overload",
    "overloads": "overload",
    "overlooked": "overlook",
    "overlooking": "overlook",
    "overlooks": "overlook",
    "overlords": "overlord",
    "overlying": "overlie",
    "overpaid": "overpay",
    "overpasses": "overpass",
    "overpaying": "overpay",
    "overpayments": "overpayment",
    "overpays": "overpay",
    "overplayed": "overplay",
    "overplaying": "overplay",
    "overplays": "overplay",
    "overpowered": "overpower",
    "overpowering": "overpower",
    "overpowers": "overpower",
    "overpressures": "overpressure",
    "overprinted": "overprint",
    "overprinting": "overprint",
    "overprints": "overprint",
    "overproduced": "overproduce",
    "overproduces": "overproduce",
    "overproducing": "overproduce",
    "overran": "overrun",
    "overrated": "overrate",
    "overrates": "overrate",
    "overrating": "overrate",
    "overreached": "overreach",
    "overreaches": "overreach",
    "overreaching": "overreach",
    "overreacted": "overreact",
    "overreacting": "overreact",
    "overreactions": "overreaction",
    "overreacts": "overreact",
    "overridden": "override",
    "overrides": "override",
    "overriding": "override",
    "overrode": "override",
    "overruled": "overrule",
    "overrules": "overrule",
    "overruling": "overrule",
    "overrunning": "overrun",
    "overruns": "overrun",
    "oversaw": "oversee",
    "overseeing": "oversee",
    "overseen": "oversee",
    "overseers": "overseer",
    "oversees": "oversee",
    "overselling": "oversell",
    "oversells": "oversell",
    "oversets": "overset",
    "oversetting": "overset",
    "overshadowed": "overshadow",
    "overshadowing": "overshadow",
    "overshadows": "overshadow",
    "overshoes": "overshoe",
    "overshooting": "overshoot",
    "overshoots": "overshoot",
    "overshot": "overshoot",
    "oversights": "oversight",
    "oversimplifications": "oversimplification",
    "oversimplified": "oversimplify",
    "oversimplifies": "oversimplify",
    "oversimplifying": "oversimplify",
    "oversleeping": "oversleep",
    "oversleeps": "oversleep",
    "overslept": "oversleep",
    "oversold": "oversell",
    "overspended": "overspend",
    "overspending": "overspend",
    "overspends": "overspend",
    "overspent": "overspend",
    "overspills": "overspill",
    "overspreading": "overspread",
    "overspreads": "overspread",
    "overstated": "overstate",
    "overstatements": "overstatement",
    "overstates": "overstate",
    "overstating": "overstate",
    "overstayed": "overstay",
    "overstaying": "overstay",
    "overstays": "overstay",
    "overstepped": "overstep",
    "overstepping": "overstep",
    "oversteps": "overstep",
    "overstocked": "overstock",
    "overstocking": "overstock",
    "overstocks": "overstock",
    "overstressed": "overstress",
    "overstresses": "overstress",
    "overstressing": "overstress",
    "overstretched": "overstretch",
    "overstretches": "overstretch",
    "overstretching": "overstretch",
    "overstuffed": "overstuff",
    "overstuffing": "overstuff",
    "overstuffs": "overstuff",
    "oversupplies": "oversupply",
    "overtaken": "overtake",
    "overtakes": "overtake",
    "overtaking": "overtake",
    "overtaxed": "overtax",
    "overtaxes": "overtax",
    "overtaxing": "overtax",
    "overthrew": "overthrow",
    "overthrowing": "overthrow",
    "overthrown": "overthrow",
    "overthrows": "overthrow",
    "overtones": "overtone",
    "overtook": "overtake",
    "overtopped": "overtop",
    "overtopping": "overtop",
    "overtops": "overtop",
    "overtrained": "overtrain",
    "overtraining": "overtrain",
    "overtrains": "overtrain",
    "overtures": "overture",
    "overturned": "overturn",
    "overturning": "overturn",
    "overturns": "overturn",
    "overused": "overuse",
    "overuses": "overuse",
    "overusing": "overuse",
    "overvalued": "overvalue",
    "overvalues": "overvalue",
    "overvaluing": "overvalue",
    "overviews": "overview",
    "overwhelmed": "overwhelm",
    "overwhelming": "overwhelm",
    "overwhelms": "overwhelm",
    "overwintered": "overwinter",
    "overwintering": "overwinter",
    "overwinters": "overwinter",
    "overworked": "overwork",
    "overworking": "overwork",
    "overworks": "overwork",
    "overwrites": "overwrite",
    "overwriting": "overwrite",
    "overwritten": "overwrite",
    "overwrote": "overwrite",
    "ovipositors": "ovipositor",
    "ovulated": "ovulate",
    "ovulates": "ovulate",
    "ovulating": "ovulate",
    "ovulations": "ovulation",
    "ovules": "ovule",
    "owed": "owe",
    "owes": "owe",
    "owing": "owe",
    "owls": "owl",
    "owned": "own",
    "owner-occupiers": "owner-occupier",
    "owners": "owner",
    "owning": "own",
    "owns": "own",
    "oxalates": "oxalate",
    "oxbows": "oxbow",
    "oxcarts": "oxcart",
    "oxen": "ox",
    "oxfords": "oxford",
    "oxidants": "oxidant",
    "oxidases": "oxidase",
    "oxidations": "oxidation",
    "oxides": "oxide",
    "oxidisations": "oxidisation",
    "oxidised": "oxidise",
    "oxidises": "oxidise",
    "oxidising": "oxidise",
    "oxidized": "oxidize",
    "oxidizes": "oxidize",
    "oxidizing": "oxidize",
    "oximeters": "oximeter",
    "oximetries": "oximetry",
    "oxtails": "oxtail",
    "oxycodones": "oxycodone",
    "oxygenated": "oxygenate",
    "oxygenates": "oxygenate",
    "oxygenating": "oxygenate",
    "oxygenations": "oxygenation",
    "oxygens": "oxygen",
    "oxymora": "oxymoron",
    "oxymorons": "oxymoron",
    "oxytetracyclines": "oxytetracycline",
    "oystercatchers": "oystercatcher",
    "oysters": "oyster",
    "ozones": "ozone",
    "p-values": "p-value",
    "paced": "pace",
    "pacemakers": "pacemaker",
    "pacers": "pacer",
    "paces": "pace",
    "pacesetters": "pacesetter",
    "pachyderms": "pachyderm",
    "pacified": "pacify",
    "pacifiers": "pacifier",
    "pacifies": "pacify",
    "pacifists": "pacifist",
    "pacifying": "pacify",
    "pacing": "pace",
    "packaged": "package",
    "packages": "package",
    "packaging": "package",
    "packagings": "packaging",
    "packed": "pack",
    "packers": "packer",
    "packets": "packet",
    "packhorses": "packhorse",
    "packing": "pack",
    "packings": "packing",
    "packs": "pack",
    "pacs": "pac",
    "pacts": "pact",
    "padded": "pad",
    "paddies": "paddy",
    "padding": "pad",
    "paddled": "paddle",
    "paddlers": "paddler",
    "paddles": "paddle",
    "paddling": "paddle",
    "paddocks": "paddock",
    "padlocked": "padlock",
    "padlocking": "padlock",
    "padlocks": "padlock",
    "padres": "padre",
    "pads": "pad",
    "paeans": "paean",
    "paediatricians": "paediatrician",
    "paedophilias": "paedophilia",
    "pagans": "pagan",
    "page-ends": "page-end",
    "pageants": "pageant",
    "pageboys": "pageboy",
    "paged": "page",
    "pagers": "pager",
    "pages": "page",
    "paginated": "paginate",
    "paginates": "paginate",
    "paginating": "paginate",
    "paging": "page",
    "pagodas": "pagoda",
    "paid": "pay",
    "pails": "pail",
    "pain-killers": "pain-killer",
    "pained": "pain",
    "paining": "pain",
    "painkillers": "painkiller",
    "pains": "pain",
    "paint-strippers": "paint-stripper",
    "paintballs": "paintball",
    "paintboxes": "paintbox",
    "paintbrushes": "paintbrush",
    "painted": "paint",
    "painters": "painter",
    "painting": "paint",
    "paintings": "painting",
    "paints": "paint",
    "paired": "pair",
    "pairing": "pair",
    "pairings": "pairing",
    "pairs": "pair",
    "pajamas": "pajama",
    "palaces": "palace",
    "palaeontologies": "palaeontology",
    "palaeontologists": "palaeontologist",
    "palatabilities": "palatability",
    "palates": "palate",
    "palavers": "palaver",
    "paled": "pale",
    "paleontologies": "paleontology",
    "paleontologists": "paleontologist",
    "paler": "pale",
    "pales": "pale",
    "palest": "pale",
    "palettes": "palette",
    "palimpsests": "palimpsest",
    "palindromes": "palindrome",
    "paling": "pale",
    "palisaded": "palisade",
    "palisades": "palisade",
    "palisading": "palisade",
    "pallbearers": "pallbearer",
    "palled": "pall",
    "palletised": "palletise",
    "palletises": "palletise",
    "palletising": "palletise",
    "pallets": "pallet",
    "palliated": "palliate",
    "palliates": "palliate",
    "palliating": "palliate",
    "palliations": "palliation",
    "palliatives": "palliative",
    "pallier": "pally",
    "palliest": "pally",
    "palling": "pall",
    "pallors": "pallor",
    "palls": "pall",
    "palmed": "palm",
    "palmettoes": "palmetto",
    "palmettos": "palmetto",
    "palmier": "palmy",
    "palming": "palm",
    "palmists": "palmist",
    "palmitates": "palmitate",
    "palms": "palm",
    "palmtops": "palmtop",
    "palominos": "palomino",
    "palpated": "palpate",
    "palpates": "palpate",
    "palpating": "palpate",
    "palpations": "palpation",
    "palpitated": "palpitate",
    "palpitates": "palpitate",
    "palpitating": "palpitate",
    "palpitations": "palpitation",
    "pals": "pal",
    "palsies": "palsy",
    "paltrier": "paltry",
    "paltriest": "paltry",
    "pampas": "pampa",
    "pampered": "pamper",
    "pampering": "pamper",
    "pampers": "pamper",
    "pamphleteers": "pamphleteer",
    "pamphlets": "pamphlet",
    "panaceas": "panacea",
    "panatellas": "panatella",
    "pancakes": "pancake",
    "pancreases": "pancreas",
    "pancreata": "pancreas",
    "pancreatitides": "pancreatitis",
    "pancreatitises": "pancreatitis",
    "pandas": "panda",
    "pandemics": "pandemic",
    "pandemoniums": "pandemonium",
    "pandered": "pander",
    "pandering": "pander",
    "panders": "pander",
    "pandits": "pandit",
    "panegyrics": "panegyric",
    "panelists": "panelist",
    "panelled": "panel",
    "panelling": "panel",
    "panellists": "panellist",
    "panels": "panel",
    "panes": "pane",
    "pangas": "panga",
    "pangs": "pang",
    "panicked": "panic",
    "panicking": "panic",
    "panicles": "panicle",
    "panics": "panic",
    "panned": "pan",
    "panniers": "pannier",
    "pannikins": "pannikin",
    "panning": "pan",
    "panoplies": "panoply",
    "panoramas": "panorama",
    "pans": "pan",
    "pansies": "pansy",
    "pantaloons": "pantaloon",
    "pantechnicons": "pantechnicon",
    "panted": "pant",
    "pantheists": "pantheist",
    "pantheons": "pantheon",
    "panthers": "panther",
    "panties": "panty",
    "panting": "pant",
    "pantographs": "pantograph",
    "pantomimed": "pantomime",
    "pantomimes": "pantomime",
    "pantomiming": "pantomime",
    "pantos": "panto",
    "pantries": "pantry",
    "pants": "pant",
    "panzers": "panzer",
    "papacies": "papacy",
    "papas": "papa",
    "papayas": "papaya",
    "paper-clips": "paper-clip",
    "paper-knives": "paper-knife",
    "paper-makings": "paper-making",
    "paperbacks": "paperback",
    "paperboys": "paperboy",
    "paperclips": "paperclip",
    "papered": "paper",
    "papering": "paper",
    "papermakers": "papermaker",
    "papermakings": "papermaking",
    "papers": "paper",
    "paperweights": "paperweight",
    "papier-mâchés": "papier-mâché",
    "papillae": "papilla",
    "papillas": "papilla",
    "papillomas": "papilloma",
    "papillomata": "papilloma",
    "papillomaviruses": "papillomavirus",
    "paprikas": "paprika",
    "paps": "pap",
    "papules": "papule",
    "papyri": "papyrus",
    "papyruses": "papyrus",
    "parabens": "paraben",
    "parables": "parable",
    "parabolas": "parabola",
    "parachuted": "parachute",
    "parachutes": "parachute",
    "parachuting": "parachute",
    "parachutists": "parachutist",
    "paraded": "parade",
    "parades": "parade",
    "paradigmata": "paradigm",
    "paradigms": "paradigm",
    "parading": "parade",
    "paradises": "paradise",
    "paradoxes": "paradox",
    "parae": "para",
    "paraesthesiae": "paraesthesia",
    "paraesthesias": "paraesthesia",
    "paraffins": "paraffin",
    "paraglided": "paraglide",
    "paraglides": "paraglide",
    "paragliding": "paraglide",
    "paragons": "paragon",
    "paragraphed": "paragraph",
    "paragraphing": "paragraph",
    "paragraphs": "paragraph",
    "parakeets": "parakeet",
    "paralegals": "paralegal",
    "paralleled": "parallel",
    "paralleling": "parallel",
    "parallelisations": "parallelisation",
    "parallelised": "parallelise",
    "parallelises": "parallelise",
    "parallelising": "parallelise",
    "parallelisms": "parallelism",
    "parallelizations": "parallelization",
    "parallelized": "parallelize",
    "parallelizes": "parallelize",
    "parallelizing": "parallelize",
    "parallelled": "parallel",
    "parallelling": "parallel",
    "parallelograms": "parallelogram",
    "parallels": "parallel",
    "paralysed": "paralyse",
    "paralyses": "paralysis",
    "paralysing": "paralyse",
    "paralytics": "paralytic",
    "paralyzed": "paralyze",
    "paralyzes": "paralyze",
    "paralyzing": "paralyze",
    "paramedics": "paramedic",
    "parameterisations": "parameterisation",
    "parameterised": "parameterise",
    "parameterises": "parameterise",
    "parameterising": "parameterise",
    "parameterizations": "parameterization",
    "parameterized": "parameterize",
    "parameterizes": "parameterize",
    "parameterizing": "parameterize",
    "parameters": "parameter",
    "parametrizations": "parametrization",
    "paramours": "paramour",
    "paranoiacs": "paranoiac",
    "paranoias": "paranoia",
    "paranoids": "paranoid",
    "parapets": "parapet",
    "paraphrased": "paraphrase",
    "paraphrases": "paraphrasis",
    "paraphrasing": "paraphrase",
    "paraplegias": "paraplegia",
    "paraplegics": "paraplegic",
    "paraprofessionals": "paraprofessional",
    "paraproteins": "paraprotein",
    "parapsychologies": "parapsychology",
    "parapsychologists": "parapsychologist",
    "paras": "para",
    "parasites": "parasite",
    "parasitised": "parasitise",
    "parasitises": "parasitise",
    "parasitising": "parasitise",
    "parasitisms": "parasitism",
    "parasitoids": "parasitoid",
    "parasitologies": "parasitology",
    "parasols": "parasol",
    "paratroopers": "paratrooper",
    "paratuberculoses": "paratuberculosis",
    "parboiled": "parboil",
    "parboiling": "parboil",
    "parboils": "parboil",
    "parceled": "parcel",
    "parceling": "parcel",
    "parcelled": "parcel",
    "parcelling": "parcel",
    "parcels": "parcel",
    "parched": "parch",
    "parches": "parch",
    "parching": "parch",
    "parchments": "parchment",
    "pardoned": "pardon",
    "pardoning": "pardon",
    "pardons": "pardon",
    "pared": "pare",
    "parenchymae": "parenchyma",
    "parenchymas": "parenchyma",
    "parentheses": "parenthesis",
    "parenthesized": "parenthesize",
    "parenthesizes": "parenthesize",
    "parenthesizing": "parenthesize",
    "parenthetically": "parenthetical",
    "parents": "parent",
    "parents-in-law": "parent-in-law",
    "pares": "pare",
    "pareses": "paresis",
    "pariahs": "pariah",
    "paring": "pare",
    "parings": "paring",
    "parishes": "parish",
    "parishioners": "parishioner",
    "parities": "parity",
    "parkas": "parka",
    "parked": "park",
    "parking": "park",
    "parkins": "parkin",
    "parkinsonisms": "parkinsonism",
    "parklands": "parkland",
    "parks": "park",
    "parkways": "parkway",
    "parleyed": "parley",
    "parleying": "parley",
    "parleys": "parley",
    "parliamentarians": "parliamentarian",
    "parliaments": "parliament",
    "parlors": "parlor",
    "parlourmaids": "parlourmaid",
    "parlours": "parlour",
    "parodied": "parody",
    "parodies": "parody",
    "parodists": "parodist",
    "parodying": "parody",
    "paroled": "parole",
    "paroles": "parole",
    "paroling": "parole",
    "paroxysms": "paroxysm",
    "parquets": "parquet",
    "parricides": "parricide",
    "parried": "parry",
    "parries": "parry",
    "parroted": "parrot",
    "parrotfishes": "parrotfish",
    "parroting": "parrot",
    "parrots": "parrot",
    "parrs": "parr",
    "parrying": "parry",
    "pars": "par",
    "parsed": "parse",
    "parsers": "parser",
    "parses": "parse",
    "parsing": "parse",
    "parsnips": "parsnip",
    "parsonages": "parsonage",
    "parsons": "parson",
    "part-timers": "part-timer",
    "parta": "partum",
    "partaken": "partake",
    "partakers": "partaker",
    "partakes": "partake",
    "partaking": "partake",
    "parted": "part",
    "partes": "pars",
    "partials": "partial",
    "participants": "participant",
    "participated": "participate",
    "participates": "participate",
    "participating": "participate",
    "participations": "participation",
    "participators": "participator",
    "participles": "participle",
    "particleboards": "particleboard",
    "particles": "particle",
    "particularised": "particularise",
    "particularises": "particularise",
    "particularising": "particularise",
    "particularisms": "particularism",
    "particularities": "particularity",
    "particularized": "particularize",
    "particularizes": "particularize",
    "particularizing": "particularize",
    "particulars": "particular",
    "particulates": "particulate",
    "partied": "party",
    "parties": "party",
    "parting": "part",
    "partings": "parting",
    "partisans": "partisan",
    "partitioned": "partition",
    "partitioning": "partition",
    "partitions": "partition",
    "partitives": "partitive",
    "partnered": "partner",
    "partnering": "partner",
    "partners": "partner",
    "partnerships": "partnership",
    "partook": "partake",
    "partridges": "partridge",
    "parts": "part",
    "parturitions": "parturition",
    "partygoers": "partygoer",
    "partying": "party",
    "parvenus": "parvenu",
    "parvoviruses": "parvovirus",
    "pascals": "pascal",
    "pashminas": "pashmina",
    "passages": "passage",
    "passageways": "passageway",
    "passbands": "passband",
    "passed": "pass",
    "passengers": "passenger",
    "passerines": "passerine",
    "passers": "passer",
    "passers-by": "passer-by",
    "passersby": "passerby",
    "passes": "pass",
    "passing": "pass",
    "passions": "passion",
    "passives": "passive",
    "passivized": "passivize",
    "passivizes": "passivize",
    "passivizing": "passivize",
    "passports": "passport",
    "passwords": "password",
    "pastae": "pasta",
    "pastas": "pasta",
    "pasteboards": "pasteboard",
    "pasted": "paste",
    "pastels": "pastel",
    "pasterns": "pastern",
    "pastes": "paste",
    "pasteurisations": "pasteurisation",
    "pasteurised": "pasteurise",
    "pasteurises": "pasteurise",
    "pasteurising": "pasteurise",
    "pasteurized": "pasteurize",
    "pasteurizes": "pasteurize",
    "pasteurizing": "pasteurize",
    "pastiches": "pastiche",
    "pastier": "pasty",
    "pasties": "pasty",
    "pastiest": "pasty",
    "pastilles": "pastille",
    "pastimes": "pastime",
    "pasting": "paste",
    "pastoralists": "pastoralist",
    "pastorates": "pastorate",
    "pastored": "pastor",
    "pastoring": "pastor",
    "pastors": "pastor",
    "pastries": "pastry",
    "pasts": "past",
    "pastured": "pasture",
    "pastures": "pasture",
    "pasturing": "pasture",
    "patched": "patch",
    "patches": "patch",
    "patchier": "patchy",
    "patchiest": "patchy",
    "patching": "patch",
    "patchoulies": "patchouli",
    "patchoulis": "patchouli",
    "patchworks": "patchwork",
    "patellae": "patella",
    "patellas": "patella",
    "patencies": "patency",
    "patens": "paten",
    "patented": "patent",
    "patentees": "patentee",
    "patenting": "patent",
    "patents": "patent",
    "paterfamiliases": "paterfamilias",
    "paternities": "paternity",
    "paternosters": "paternoster",
    "paters": "pater",
    "pates": "pate",
    "pathfinders": "pathfinder",
    "pathogeneses": "pathogenesis",
    "pathogenicities": "pathogenicity",
    "pathogens": "pathogen",
    "pathologies": "pathology",
    "pathologists": "pathologist",
    "pathophysiologies": "pathophysiology",
    "paths": "path",
    "pathways": "pathway",
    "patients": "patient",
    "patinae": "patina",
    "patinas": "patina",
    "patios": "patio",
    "patisseries": "patisserie",
    "patriarchies": "patriarchy",
    "patriarchs": "patriarch",
    "patricians": "patrician",
    "patricides": "patricide",
    "patrimonies": "patrimony",
    "patriots": "patriot",
    "patrolled": "patrol",
    "patrollers": "patroller",
    "patrolling": "patrol",
    "patrolmen": "patrolman",
    "patrols": "patrol",
    "patronesses": "patroness",
    "patronised": "patronise",
    "patronises": "patronise",
    "patronising": "patronise",
    "patronized": "patronize",
    "patronizes": "patronize",
    "patronizing": "patronize",
    "patrons": "patron",
    "patronymics": "patronymic",
    "pats": "pat",
    "patsies": "patsy",
    "patted": "pat",
    "pattens": "patten",
    "pattered": "patter",
    "pattering": "patter",
    "patterned": "pattern",
    "patterning": "pattern",
    "patterns": "pattern",
    "patters": "patter",
    "patties": "patty",
    "patting": "pat",
    "paucities": "paucity",
    "paunches": "paunch",
    "paunchier": "paunchy",
    "paunchiest": "paunchy",
    "pauperizes": "pauperize",
    "pauperizing": "pauperize",
    "paupers": "pauper",
    "paused": "pause",
    "pauses": "pause",
    "pausing": "pause",
    "paved": "pave",
    "pavements": "pavement",
    "pavers": "paver",
    "paves": "pave",
    "pavilions": "pavilion",
    "paving": "pave",
    "pawed": "paw",
    "pawing": "paw",
    "pawls": "pawl",
    "pawnbrokers": "pawnbroker",
    "pawned": "pawn",
    "pawning": "pawn",
    "pawns": "pawn",
    "pawnshops": "pawnshop",
    "pawpaws": "pawpaw",
    "paws": "paw",
    "paxes": "pax",
    "pay-beds": "pay-bed",
    "pay-offs": "pay-off",
    "paybacks": "payback",
    "paychecks": "paycheck",
    "paydays": "payday",
    "payed": "pay",
    "payees": "payee",
    "payers": "payer",
    "paying": "pay",
    "payloads": "payload",
    "paymasters": "paymaster",
    "payments": "payment",
    "payoffs": "payoff",
    "payouts": "payout",
    "payrolls": "payroll",
    "pays": "pay",
    "payslips": "payslip",
    "pdfs": "pdf",
    "pea-soupers": "pea-souper",
    "peacekeepers": "peacekeeper",
    "peacemakers": "peacemaker",
    "peaches": "peach",
    "peacocks": "peacock",
    "peahens": "peahen",
    "peaked": "peak",
    "peakier": "peaky",
    "peakiest": "peaky",
    "peaking": "peak",
    "peaks": "peak",
    "pealed": "peal",
    "pealing": "peal",
    "peals": "peal",
    "peanuts": "peanut",
    "pearled": "pearl",
    "pearlier": "pearly",
    "pearling": "pearl",
    "pearls": "pearl",
    "pears": "pear",
    "peas": "pea",
    "peasantries": "peasantry",
    "peasants": "peasant",
    "peashooters": "peashooter",
    "peatier": "peaty",
    "peatiest": "peaty",
    "peats": "peat",
    "pebbled": "pebble",
    "pebbles": "pebble",
    "pebblier": "pebbly",
    "pebbliest": "pebbly",
    "pebbling": "pebble",
    "pecans": "pecan",
    "peccadilloes": "peccadillo",
    "peccadillos": "peccadillo",
    "pecked": "peck",
    "peckers": "pecker",
    "pecking": "peck",
    "pecks": "peck",
    "pectins": "pectin",
    "pectoralides": "pectoralis",
    "pectorals": "pectoral",
    "peculiarities": "peculiarity",
    "pedagogies": "pedagogy",
    "pedagogues": "pedagogue",
    "pedaled": "pedal",
    "pedaling": "pedal",
    "pedalled": "pedal",
    "pedalling": "pedal",
    "pedals": "pedal",
    "pedants": "pedant",
    "peddled": "peddle",
    "peddlers": "peddler",
    "peddles": "peddle",
    "peddling": "peddle",
    "pedes": "pes",
    "pedestals": "pedestal",
    "pedestrianised": "pedestrianise",
    "pedestrianises": "pedestrianise",
    "pedestrianising": "pedestrianise",
    "pedestrianized": "pedestrianize",
    "pedestrianizes": "pedestrianize",
    "pedestrianizing": "pedestrianize",
    "pedestrians": "pedestrian",
    "pediatricians": "pediatrician",
    "pedicures": "pedicure",
    "pedigrees": "pedigree",
    "pediments": "pediment",
    "pedlars": "pedlar",
    "pedometers": "pedometer",
    "pedophiles": "pedophile",
    "peds": "ped",
    "peed": "pee",
    "peeing": "pee",
    "peeked": "peek",
    "peeking": "peek",
    "peeks": "peek",
    "peeled": "peel",
    "peelers": "peeler",
    "peeling": "peel",
    "peels": "peel",
    "peeped": "peep",
    "peepholes": "peephole",
    "peeping": "peep",
    "peeps": "peep",
    "peepshows": "peepshow",
    "peerages": "peerage",
    "peered": "peer",
    "peeresses": "peeress",
    "peering": "peer",
    "peers": "peer",
    "pees": "pee",
    "peeved": "peeve",
    "peeves": "peeve",
    "peeving": "peeve",
    "peewits": "peewit",
    "peg-legs": "peg-leg",
    "pegboards": "pegboard",
    "pegged": "peg",
    "pegging": "peg",
    "pegmatites": "pegmatite",
    "pegs": "peg",
    "pekineses": "pekinese",
    "pekingeses": "pekingese",
    "pelicans": "pelican",
    "pelleted": "pellet",
    "pelleting": "pellet",
    "pellets": "pellet",
    "pelmets": "pelmet",
    "pelotons": "peloton",
    "pelted": "pelt",
    "pelting": "pelt",
    "pelts": "pelt",
    "pelves": "pelvis",
    "pelvises": "pelvis",
    "pen-friends": "pen-friend",
    "penalised": "penalise",
    "penalises": "penalise",
    "penalising": "penalise",
    "penalized": "penalize",
    "penalizes": "penalize",
    "penalizing": "penalize",
    "penals": "penal",
    "penalties": "penalty",
    "penances": "penance",
    "pence": "penny",
    "penciled": "pencil",
    "penciling": "pencil",
    "pencilled": "pencil",
    "pencilling": "pencil",
    "pencils": "pencil",
    "pendants": "pendant",
    "pended": "pend",
    "pendents": "pendent",
    "pending": "pend",
    "pends": "pend",
    "pendula": "pendulum",
    "pendulums": "pendulum",
    "penes": "penis",
    "penetrances": "penetrance",
    "penetrated": "penetrate",
    "penetrates": "penetrate",
    "penetrating": "penetrate",
    "penetrations": "penetration",
    "penetrators": "penetrator",
    "penfriends": "penfriend",
    "penguins": "penguin",
    "penholders": "penholder",
    "penicillins": "penicillin",
    "peninsulas": "peninsula",
    "penises": "penis",
    "penitences": "penitence",
    "penitentiaries": "penitentiary",
    "penitents": "penitent",
    "penknives": "penknife",
    "pennants": "pennant",
    "penned": "pen",
    "pennies": "penny",
    "penning": "pen",
    "penny-farthings": "penny-farthing",
    "penpushers": "penpusher",
    "pens": "pen",
    "pensioned": "pension",
    "pensioners": "pensioner",
    "pensioning": "pension",
    "pensions": "pension",
    "pent": "pen",
    "pentagons": "pentagon",
    "pentagrams": "pentagram",
    "pentameters": "pentameter",
    "pentathlons": "pentathlon",
    "penthouses": "penthouse",
    "penumbrae": "penumbra",
    "penumbras": "penumbra",
    "peones": "peon",
    "peonies": "peony",
    "peons": "peon",
    "peopled": "people",
    "peoples": "people",
    "peopling": "people",
    "pepped": "pep",
    "peppercorns": "peppercorn",
    "peppered": "pepper",
    "peppering": "pepper",
    "peppermints": "peppermint",
    "pepperpots": "pepperpot",
    "peppers": "pepper",
    "pepping": "pep",
    "peps": "pep",
    "pepsins": "pepsin",
    "peptalks": "peptalk",
    "peptidases": "peptidase",
    "peptides": "peptide",
    "perambulated": "perambulate",
    "perambulates": "perambulate",
    "perambulating": "perambulate",
    "perambulators": "perambulator",
    "perceived": "perceive",
    "perceivers": "perceiver",
    "perceives": "perceive",
    "perceiving": "perceive",
    "percentages": "percentage",
    "percentiles": "percentile",
    "percents": "percent",
    "perceptions": "perception",
    "perceptrons": "perceptron",
    "percepts": "percept",
    "perched": "perch",
    "perches": "perch",
    "perching": "perch",
    "percolated": "percolate",
    "percolates": "percolate",
    "percolating": "percolate",
    "percolations": "percolation",
    "percolators": "percolator",
    "percussionists": "percussionist",
    "percussions": "percussion",
    "peregrinas": "peregrina",
    "peregrinations": "peregrination",
    "peregrines": "peregrine",
    "perennials": "perennial",
    "perfected": "perfect",
    "perfecting": "perfect",
    "perfectionists": "perfectionist",
    "perfects": "perfect",
    "perfidies": "perfidy",
    "perforated": "perforate",
    "perforates": "perforate",
    "perforating": "perforate",
    "perforations": "perforation",
    "performances": "performance",
    "performatives": "performative",
    "performed": "perform",
    "performers": "performer",
    "performing": "perform",
    "performs": "perform",
    "perfumed": "perfume",
    "perfumers": "perfumer",
    "perfumes": "perfume",
    "perfuming": "perfume",
    "perfused": "perfuse",
    "perfuses": "perfuse",
    "perfusing": "perfuse",
    "perfusions": "perfusion",
    "pergolas": "pergola",
    "pericardia": "pericardium",
    "pericarditides": "pericarditis",
    "perigees": "perigee",
    "perihelia": "perihelion",
    "perils": "peril",
    "perimeters": "perimeter",
    "perinea": "perineum",
    "perineums": "perineum",
    "periodicals": "periodical",
    "periodicities": "periodicity",
    "periodontitides": "periodontitis",
    "periods": "period",
    "peripheraller": "peripheral",
    "peripherallest": "peripheral",
    "peripherals": "peripheral",
    "peripheries": "periphery",
    "periscopes": "periscope",
    "perishables": "perishable",
    "perished": "perish",
    "perishers": "perisher",
    "perishes": "perish",
    "perishing": "perish",
    "peristalses": "peristalsis",
    "peritonea": "peritoneum",
    "peritoneums": "peritoneum",
    "peritonitides": "peritonitis",
    "periwinkles": "periwinkle",
    "perjured": "perjure",
    "perjures": "perjure",
    "perjuries": "perjury",
    "perjuring": "perjure",
    "perked": "perk",
    "perkier": "perky",
    "perkiest": "perky",
    "perking": "perk",
    "perks": "perk",
    "permanencies": "permanency",
    "permanganates": "permanganate",
    "permeabilities": "permeability",
    "permeated": "permeate",
    "permeates": "permeate",
    "permeating": "permeate",
    "permeations": "permeation",
    "permed": "perm",
    "perming": "perm",
    "permissions": "permission",
    "permissivenesses": "permissiveness",
    "permits": "permit",
    "permitted": "permit",
    "permitting": "permit",
    "permittivities": "permittivity",
    "perms": "perm",
    "permutations": "permutation",
    "permuted": "permute",
    "permutes": "permute",
    "permuting": "permute",
    "perorations": "peroration",
    "perovskites": "perovskite",
    "peroxidases": "peroxidase",
    "peroxidations": "peroxidation",
    "peroxides": "peroxide",
    "perpendiculars": "perpendicular",
    "perpetrated": "perpetrate",
    "perpetrates": "perpetrate",
    "perpetrating": "perpetrate",
    "perpetrators": "perpetrator",
    "perpetuated": "perpetuate",
    "perpetuates": "perpetuate",
    "perpetuating": "perpetuate",
    "perplexed": "perplex",
    "perplexes": "perplex",
    "perplexing": "perplex",
    "perplexities": "perplexity",
    "perquisites": "perquisite",
    "persecuted": "persecute",
    "persecutes": "persecute",
    "persecuting": "persecute",
    "persecutions": "persecution",
    "persecutors": "persecutor",
    "persevered": "persevere",
    "perseveres": "persevere",
    "persevering": "persevere",
    "persimmons": "persimmon",
    "persisted": "persist",
    "persistencies": "persistency",
    "persisting": "persist",
    "persists": "persist",
    "person-years": "person-year",
    "personae": "persona",
    "personages": "personage",
    "personalised": "personalise",
    "personalises": "personalise",
    "personalising": "personalise",
    "personalities": "personality",
    "personalized": "personalize",
    "personalizes": "personalize",
    "personalizing": "personalize",
    "personals": "personal",
    "personas": "persona",
    "personifications": "personification",
    "personified": "personify",
    "personifies": "personify",
    "personifying": "personify",
    "persons": "person",
    "perspectives": "perspective",
    "perspexes": "perspex",
    "perspired": "perspire",
    "perspires": "perspire",
    "perspiring": "perspire",
    "persuaded": "persuade",
    "persuaders": "persuader",
    "persuades": "persuade",
    "persuading": "persuade",
    "persuasions": "persuasion",
    "pertained": "pertain",
    "pertaining": "pertain",
    "pertains": "pertain",
    "perter": "pert",
    "pertest": "pert",
    "perturbations": "perturbation",
    "perturbed": "perturb",
    "perturbing": "perturb",
    "perturbs": "perturb",
    "pertusses": "pertussis",
    "perusals": "perusal",
    "perused": "peruse",
    "peruses": "peruse",
    "perusing": "peruse",
    "pervaded": "pervade",
    "pervades": "pervade",
    "pervading": "pervade",
    "perversions": "perversion",
    "perversities": "perversity",
    "perverted": "pervert",
    "perverting": "pervert",
    "perverts": "pervert",
    "pesetas": "peseta",
    "peskier": "pesky",
    "peskiest": "pesky",
    "pesos": "peso",
    "pessaries": "pessary",
    "pessimists": "pessimist",
    "pestered": "pester",
    "pestering": "pester",
    "pesters": "pester",
    "pesticides": "pesticide",
    "pestilences": "pestilence",
    "pestles": "pestle",
    "pests": "pest",
    "petals": "petal",
    "petards": "petard",
    "petechiae": "petechia",
    "petered": "peter",
    "petering": "peter",
    "peters": "peter",
    "petioles": "petiole",
    "petit-fours": "petit-four",
    "petitioned": "petition",
    "petitioners": "petitioner",
    "petitioning": "petition",
    "petitions": "petition",
    "petits-fours": "petit-four",
    "petrels": "petrel",
    "petrified": "petrify",
    "petrifies": "petrify",
    "petrifying": "petrify",
    "petrochemicals": "petrochemical",
    "petrolatums": "petrolatum",
    "petroleums": "petroleum",
    "pets": "pet",
    "petted": "pet",
    "petticoats": "petticoat",
    "pettier": "petty",
    "pettiest": "petty",
    "petting": "pet",
    "petunias": "petunia",
    "pews": "pew",
    "pfennigs": "pfennig",
    "pfs": "pf",
    "phaeochromocytomas": "phaeochromocytoma",
    "phaeochromocytomata": "phaeochromocytoma",
    "phages": "phage",
    "phagocytes": "phagocyte",
    "phagocytoses": "phagocytosis",
    "phalanges": "phalanx",
    "phalanxes": "phalanx",
    "phalli": "phallus",
    "phalluses": "phallus",
    "phantasies": "phantasy",
    "phantasmagorias": "phantasmagoria",
    "phantasms": "phantasm",
    "phantoms": "phantom",
    "pharaohs": "pharaoh",
    "pharmaceuticals": "pharmaceutical",
    "pharmacies": "pharmacy",
    "pharmacists": "pharmacist",
    "pharmacologies": "pharmacology",
    "pharmacologists": "pharmacologist",
    "pharmacopoeiae": "pharmacopoeia",
    "pharmacopoeias": "pharmacopoeia",
    "pharmacotherapies": "pharmacotherapy",
    "pharynges": "pharynx",
    "pharynxes": "pharynx",
    "phased": "phase",
    "phasers": "phaser",
    "phases": "phasis",
    "phasing": "phase",
    "pheasants": "pheasant",
    "phenobarbitals": "phenobarbital",
    "phenologies": "phenology",
    "phenols": "phenol",
    "phenomena": "phenomenon",
    "phenomenologies": "phenomenology",
    "phenomenons": "phenomenon",
    "phenothiazines": "phenothiazine",
    "phenotypes": "phenotype",
    "phenylalanines": "phenylalanine",
    "phenylamines": "phenylamine",
    "phenylketonurias": "phenylketonuria",
    "pheochromocytomae": "pheochromocytoma",
    "pheochromocytomas": "pheochromocytoma",
    "pheochromocytomata": "pheochromocytoma",
    "pheromones": "pheromone",
    "phials": "phial",
    "philandered": "philander",
    "philanderers": "philanderer",
    "philandering": "philander",
    "philanders": "philander",
    "philanthropies": "philanthropy",
    "philanthropists": "philanthropist",
    "philatelies": "philately",
    "philatelists": "philatelist",
    "philistines": "philistine",
    "philologies": "philology",
    "philologists": "philologist",
    "philosophers": "philosopher",
    "philosophies": "philosophy",
    "philosophised": "philosophise",
    "philosophises": "philosophise",
    "philosophising": "philosophise",
    "philosophized": "philosophize",
    "philosophizes": "philosophize",
    "philosophizing": "philosophize",
    "phis": "phi",
    "phlebotomies": "phlebotomy",
    "phlebotomists": "phlebotomist",
    "phlogistons": "phlogiston",
    "phobias": "phobia",
    "phobics": "phobic",
    "phoebes": "phoebe",
    "phoenixes": "phoenix",
    "phone-ins": "phone-in",
    "phoned": "phone",
    "phonemes": "phoneme",
    "phones": "phone",
    "phonetics": "phonetic",
    "phoneys": "phoney",
    "phonier": "phony",
    "phonies": "phony",
    "phoniest": "phony",
    "phoning": "phone",
    "phonograms": "phonogram",
    "phonographs": "phonograph",
    "phonologies": "phonology",
    "phonons": "phonon",
    "phonos": "phono",
    "phos": "pho",
    "phosphatases": "phosphatase",
    "phosphates": "phosphate",
    "phosphatidylcholines": "phosphatidylcholine",
    "phosphatidylinositols": "phosphatidylinositol",
    "phosphides": "phosphide",
    "phosphines": "phosphine",
    "phosphodiesterases": "phosphodiesterase",
    "phospholipases": "phospholipase",
    "phospholipids": "phospholipid",
    "phosphors": "phosphor",
    "phosphorylases": "phosphorylase",
    "phosphorylated": "phosphorylate",
    "phosphorylates": "phosphorylate",
    "phosphorylating": "phosphorylate",
    "phosphorylations": "phosphorylation",
    "photocells": "photocell",
    "photochemistries": "photochemistry",
    "photocoagulations": "photocoagulation",
    "photocopied": "photocopy",
    "photocopiers": "photocopier",
    "photocopies": "photocopy",
    "photocopying": "photocopy",
    "photodiodes": "photodiode",
    "photoelectrons": "photoelectron",
    "photoemissions": "photoemission",
    "photogrammetries": "photogrammetry",
    "photographed": "photograph",
    "photographers": "photographer",
    "photographies": "photography",
    "photographing": "photograph",
    "photographs": "photograph",
    "photojournalists": "photojournalist",
    "photolyses": "photolysis",
    "photometers": "photometer",
    "photometries": "photometry",
    "photomicrographies": "photomicrography",
    "photomicrographs": "photomicrograph",
    "photomultipliers": "photomultiplier",
    "photons": "photon",
    "photoperiods": "photoperiod",
    "photophobias": "photophobia",
    "photoreceptors": "photoreceptor",
    "photoresists": "photoresist",
    "photos": "photo",
    "photosensitisations": "photosensitisation",
    "photosensitivities": "photosensitivity",
    "photospheres": "photosphere",
    "photostated": "photostat",
    "photostating": "photostat",
    "photostats": "photostat",
    "photosyntheses": "photosynthesis",
    "photosynthesised": "photosynthesise",
    "photosynthesises": "photosynthesise",
    "photosynthesising": "photosynthesise",
    "photosystems": "photosystem",
    "phototherapies": "phototherapy",
    "phots": "phot",
    "phrasebooks": "phrasebook",
    "phrased": "phrase",
    "phrases": "phrase",
    "phrasing": "phrase",
    "phrenologies": "phrenology",
    "phthalates": "phthalate",
    "phyla": "phylum",
    "phylogenies": "phylogeny",
    "physicals": "physical",
    "physicians": "physician",
    "physicists": "physicist",
    "physics": "physic",
    "physiognomies": "physiognomy",
    "physiologies": "physiology",
    "physiologists": "physiologist",
    "physios": "physio",
    "physiotherapies": "physiotherapy",
    "physiotherapists": "physiotherapist",
    "physiques": "physique",
    "phytochemicals": "phytochemical",
    "phytoestrogens": "phytoestrogen",
    "phytonutrients": "phytonutrient",
    "phytoplanktons": "phytoplankton",
    "phytoplasmas": "phytoplasma",
    "phytoplasmata": "phytoplasma",
    "phytosterols": "phytosterol",
    "pianists": "pianist",
    "pianofortes": "pianoforte",
    "pianos": "piano",
    "piazzas": "piazza",
    "picas": "pica",
    "piccadillies": "piccadilly",
    "piccolos": "piccolo",
    "pick-me-ups": "pick-me-up",
    "pick-ups": "pick-up",
    "pickaxes": "pickaxe",
    "picked": "pick",
    "pickers": "picker",
    "picketed": "picket",
    "picketing": "picket",
    "pickets": "picket",
    "pickier": "picky",
    "pickiest": "picky",
    "picking": "pick",
    "pickled": "pickle",
    "pickles": "pickle",
    "pickling": "pickle",
    "pickpockets": "pickpocket",
    "picks": "pick",
    "pickups": "pickup",
    "picnicked": "picnic",
    "picnickers": "picnicker",
    "picnicking": "picnic",
    "picnics": "picnic",
    "picoseconds": "picosecond",
    "pics": "pic",
    "pictograms": "pictogram",
    "pictographs": "pictograph",
    "pictorials": "pictorial",
    "picture-frames": "picture-frame",
    "pictured": "picture",
    "pictures": "picture",
    "picturing": "picture",
    "piddled": "piddle",
    "piddles": "piddle",
    "piddling": "piddle",
    "pidgins": "pidgin",
    "pieced": "piece",
    "pieces": "piece",
    "piecing": "piece",
    "pieds-à-terre": "pied-à-terre",
    "pierced": "pierce",
    "piercers": "piercer",
    "pierces": "pierce",
    "piercing": "pierce",
    "piercings": "piercing",
    "pierrots": "pierrot",
    "piers": "pier",
    "pies": "pie",
    "pieties": "piety",
    "pigeon-holed": "pigeon-hole",
    "pigeon-holes": "pigeon-hole",
    "pigeon-holing": "pigeon-hole",
    "pigeonholed": "pigeonhole",
    "pigeonholes": "pigeonhole",
    "pigeonholing": "pigeonhole",
    "pigeons": "pigeon",
    "pigged": "pig",
    "piggeries": "piggery",
    "piggier": "piggy",
    "piggies": "piggy",
    "pigging": "pig",
    "piggybacked": "piggyback",
    "piggybacking": "piggyback",
    "piggybacks": "piggyback",
    "piggybanks": "piggybank",
    "piglets": "piglet",
    "pigmentations": "pigmentation",
    "pigments": "pigment",
    "pigmies": "pigmy",
    "pigpens": "pigpen",
    "pigs": "pig",
    "pigsties": "pigsty",
    "pigtails": "pigtail",
    "pikes": "pike",
    "pilasters": "pilaster",
    "pilaus": "pilau",
    "pilchards": "pilchard",
    "pile-ups": "pile-up",
    "piled": "pile",
    "piles": "pile",
    "pileups": "pileup",
    "pilfered": "pilfer",
    "pilfering": "pilfer",
    "pilfers": "pilfer",
    "pilgrimages": "pilgrimage",
    "pilgrims": "pilgrim",
    "pili": "pilus",
    "piling": "pile",
    "pillaged": "pillage",
    "pillages": "pillage",
    "pillaging": "pillage",
    "pillared": "pillar",
    "pillaring": "pillar",
    "pillars": "pillar",
    "pillboxes": "pillbox",
    "pillions": "pillion",
    "pilloried": "pillory",
    "pillories": "pillory",
    "pillorying": "pillory",
    "pillowcases": "pillowcase",
    "pillowed": "pillow",
    "pillowing": "pillow",
    "pillows": "pillow",
    "pillowslips": "pillowslip",
    "pills": "pill",
    "piloted": "pilot",
    "piloting": "pilot",
    "pilots": "pilot",
    "pimentos": "pimento",
    "pimped": "pimp",
    "pimping": "pimp",
    "pimples": "pimple",
    "pimplier": "pimply",
    "pimpliest": "pimply",
    "pimps": "pimp",
    "pin-pointed": "pin-point",
    "pin-pointing": "pin-point",
    "pin-points": "pin-point",
    "pin-tables": "pin-table",
    "pin-ups": "pin-up",
    "pinafores": "pinafore",
    "pincers": "pincer",
    "pinch-hits": "pinch-hit",
    "pinch-hitting": "pinch-hit",
    "pinched": "pinch",
    "pinches": "pinch",
    "pinching": "pinch",
    "pincushions": "pincushion",
    "pine-needles": "pine-needle",
    "pineapples": "pineapple",
    "pinecones": "pinecone",
    "pined": "pine",
    "pines": "pine",
    "pinewoods": "pinewood",
    "pinged": "ping",
    "pinging": "ping",
    "pings": "ping",
    "pinheads": "pinhead",
    "pinholes": "pinhole",
    "pining": "pine",
    "pinioned": "pinion",
    "pinioning": "pinion",
    "pinions": "pinion",
    "pinked": "pink",
    "pinker": "pink",
    "pinkest": "pink",
    "pinkies": "pinky",
    "pinking": "pink",
    "pinks": "pink",
    "pinnacles": "pinnacle",
    "pinnae": "pinna",
    "pinned": "pin",
    "pinnies": "pinny",
    "pinning": "pin",
    "pinpointed": "pinpoint",
    "pinpointing": "pinpoint",
    "pinpoints": "pinpoint",
    "pinpricks": "pinprick",
    "pins": "pin",
    "pinstripes": "pinstripe",
    "pintails": "pintail",
    "pints": "pint",
    "pinups": "pinup",
    "pioneered": "pioneer",
    "pioneering": "pioneer",
    "pioneers": "pioneer",
    "pions": "pion",
    "pipe-lines": "pipe-line",
    "piped": "pipe",
    "pipefishes": "pipefish",
    "pipelines": "pipeline",
    "pipers": "piper",
    "pipes": "pipe",
    "pipetted": "pipette",
    "pipettes": "pipette",
    "pipetting": "pipette",
    "piping": "pipe",
    "pipistrelles": "pipistrelle",
    "pipits": "pipit",
    "pipped": "pip",
    "pipping": "pip",
    "pips": "pip",
    "pipsqueaks": "pipsqueak",
    "piqued": "pique",
    "piques": "pique",
    "piquing": "pique",
    "piracies": "piracy",
    "piranhas": "piranha",
    "pirated": "pirate",
    "pirates": "pirate",
    "pirating": "pirate",
    "pirouetted": "pirouette",
    "pirouettes": "pirouette",
    "pirouetting": "pirouette",
    "piroxicams": "piroxicam",
    "pissed": "piss",
    "pisses": "piss",
    "pissing": "piss",
    "pistachios": "pistachio",
    "pistes": "piste",
    "pistils": "pistil",
    "pistols": "pistol",
    "pistons": "piston",
    "pitched": "pitch",
    "pitchers": "pitcher",
    "pitches": "pitch",
    "pitchforks": "pitchfork",
    "pitching": "pitch",
    "pitfalls": "pitfall",
    "pitheads": "pithead",
    "pithed": "pith",
    "pithier": "pithy",
    "pithiest": "pithy",
    "pithing": "pith",
    "piths": "pith",
    "pitied": "pity",
    "pities": "pity",
    "pitmans": "pitman",
    "pitmen": "pitman",
    "pitons": "piton",
    "pits": "pit",
    "pittances": "pittance",
    "pittas": "pitta",
    "pitted": "pit",
    "pitting": "pit",
    "pituitaries": "pituitary",
    "pitying": "pity",
    "pivoted": "pivot",
    "pivoting": "pivot",
    "pivots": "pivot",
    "pixels": "pixel",
    "pixies": "pixy",
    "pizzas": "pizza",
    "pks": "pk",
    "placards": "placard",
    "placated": "placate",
    "placates": "placate",
    "placating": "placate",
    "placeboes": "placebo",
    "placebos": "placebo",
    "placed": "place",
    "placeholders": "placeholder",
    "placements": "placement",
    "placentae": "placenta",
    "placentas": "placenta",
    "placers": "placer",
    "places": "place",
    "placing": "place",
    "plages": "plage",
    "plagiarised": "plagiarise",
    "plagiarises": "plagiarise",
    "plagiarising": "plagiarise",
    "plagiarisms": "plagiarism",
    "plagiarists": "plagiarist",
    "plagiarized": "plagiarize",
    "plagiarizes": "plagiarize",
    "plagiarizing": "plagiarize",
    "plagioclases": "plagioclase",
    "plagued": "plague",
    "plagues": "plague",
    "plaguing": "plague",
    "plaids": "plaid",
    "plainer": "plain",
    "plainest": "plain",
    "plains": "plain",
    "plaintiffs": "plaintiff",
    "plaints": "plaint",
    "plaited": "plait",
    "plaiting": "plait",
    "plaits": "plait",
    "planed": "plane",
    "planer": "plane",
    "planers": "planer",
    "planes": "plane",
    "planest": "plane",
    "planetaria": "planetarium",
    "planetariums": "planetarium",
    "planetoids": "planetoid",
    "planets": "planet",
    "planforms": "planform",
    "planing": "plane",
    "planked": "plank",
    "planking": "plank",
    "planks": "plank",
    "planned": "plan",
    "planners": "planner",
    "planning": "plan",
    "plannings": "planning",
    "plans": "plan",
    "plantains": "plantain",
    "plantations": "plantation",
    "planted": "plant",
    "planters": "planter",
    "planting": "plant",
    "plantings": "planting",
    "plantlets": "plantlet",
    "plants": "plant",
    "plaques": "plaque",
    "plasmas": "plasma",
    "plasmids": "plasmid",
    "plasminogens": "plasminogen",
    "plasmons": "plasmon",
    "plastered": "plaster",
    "plasterers": "plasterer",
    "plastering": "plaster",
    "plasters": "plaster",
    "plasticised": "plasticise",
    "plasticisers": "plasticiser",
    "plasticises": "plasticise",
    "plasticising": "plasticise",
    "plasticities": "plasticity",
    "plastics": "plastic",
    "plastids": "plastid",
    "plateaued": "plateau",
    "plateauing": "plateau",
    "plateaus": "plateau",
    "plateaux": "plateau",
    "plated": "plate",
    "platefuls": "plateful",
    "platelayers": "platelayer",
    "platelets": "platelet",
    "platens": "platen",
    "platers": "plater",
    "plates": "plate",
    "platesful": "plateful",
    "platforms": "platform",
    "plating": "plate",
    "platinums": "platinum",
    "platitudes": "platitude",
    "platoons": "platoon",
    "plats": "plat",
    "platters": "platter",
    "platypuses": "platypus",
    "plausibilities": "plausibility",
    "play-acted": "play-act",
    "play-acting": "play-act",
    "play-acts": "play-act",
    "play-offed": "play-off",
    "play-offing": "play-off",
    "play-offs": "play-off",
    "playas": "playa",
    "playbacks": "playback",
    "playbills": "playbill",
    "playboys": "playboy",
    "played": "play",
    "players": "player",
    "playgoers": "playgoer",
    "playgrounds": "playground",
    "playgroups": "playgroup",
    "playhouses": "playhouse",
    "playing": "play",
    "playlets": "playlet",
    "playmates": "playmate",
    "playoffs": "playoff",
    "playpens": "playpen",
    "playrooms": "playroom",
    "plays": "play",
    "playthings": "plaything",
    "playwrights": "playwright",
    "plazas": "plaza",
    "pleaded": "plead",
    "pleading": "plead",
    "pleadings": "pleading",
    "pleads": "plead",
    "pleas": "plea",
    "pleasanter": "pleasant",
    "pleasantest": "pleasant",
    "pleasantries": "pleasantry",
    "pleased": "please",
    "pleasers": "pleaser",
    "pleases": "please",
    "pleasing": "please",
    "pleasured": "pleasure",
    "pleasures": "pleasure",
    "pleasuring": "pleasure",
    "pleated": "pleat",
    "pleating": "pleat",
    "pleats": "pleat",
    "plebeians": "plebeian",
    "plebiscites": "plebiscite",
    "plebs": "pleb",
    "plectra": "plectrum",
    "plectrums": "plectrum",
    "pled": "plead",
    "pledged": "pledge",
    "pledges": "pledge",
    "pledging": "pledge",
    "plenaries": "plenary",
    "plenipotentiaries": "plenipotentiary",
    "plenums": "plenum",
    "pleurae": "pleura",
    "pleuras": "pleura",
    "pleurisies": "pleurisy",
    "pleuropneumoniae": "pleuropneumonia",
    "pleuropneumonias": "pleuropneumonia",
    "plexi": "plexus",
    "plexuses": "plexus",
    "plicae": "plica",
    "plied": "ply",
    "pliers": "plier",
    "plies": "ply",
    "plighted": "plight",
    "plighting": "plight",
    "plights": "plight",
    "plimsolls": "plimsoll",
    "plinths": "plinth",
    "plodded": "plod",
    "plodders": "plodder",
    "plodding": "plod",
    "plods": "plod",
    "plonked": "plonk",
    "plonking": "plonk",
    "plonks": "plonk",
    "plopped": "plop",
    "plopping": "plop",
    "plops": "plop",
    "plosives": "plosive",
    "plots": "plot",
    "plotted": "plot",
    "plotters": "plotter",
    "plotting": "plot",
    "plottings": "plotting",
    "ploughed": "plough",
    "ploughing": "plough",
    "ploughmen": "ploughman",
    "ploughs": "plough",
    "ploughshares": "ploughshare",
    "plovers": "plover",
    "plowed": "plow",
    "plowing": "plow",
    "plows": "plow",
    "ploys": "ploy",
    "pls": "pl",
    "plucked": "pluck",
    "pluckier": "plucky",
    "pluckiest": "plucky",
    "plucking": "pluck",
    "plucks": "pluck",
    "plugged": "plug",
    "pluggers": "plugger",
    "plugging": "plug",
    "plugholes": "plughole",
    "plugs": "plug",
    "plumbed": "plumb",
    "plumbers": "plumber",
    "plumbing": "plumb",
    "plumblines": "plumbline",
    "plumbs": "plumb",
    "plumed": "plume",
    "plumes": "plume",
    "pluming": "plume",
    "plummeted": "plummet",
    "plummeting": "plummet",
    "plummets": "plummet",
    "plummier": "plummy",
    "plummiest": "plummy",
    "plumped": "plump",
    "plumper": "plump",
    "plumpest": "plump",
    "plumping": "plump",
    "plumps": "plump",
    "plums": "plum",
    "plundered": "plunder",
    "plunderers": "plunderer",
    "plundering": "plunder",
    "plunders": "plunder",
    "plunged": "plunge",
    "plungers": "plunger",
    "plunges": "plunge",
    "plunging": "plunge",
    "plunked": "plunk",
    "plunking": "plunk",
    "plunks": "plunk",
    "pluperfects": "pluperfect",
    "pluralisms": "pluralism",
    "pluralists": "pluralist",
    "pluralities": "plurality",
    "plurals": "plural",
    "pluripotencies": "pluripotency",
    "pluses": "plus",
    "plusher": "plush",
    "plushest": "plush",
    "plushier": "plushy",
    "plushiest": "plushy",
    "plusses": "plus",
    "plutocracies": "plutocracy",
    "plutocrats": "plutocrat",
    "plying": "ply",
    "plywoods": "plywood",
    "pneumococci": "pneumococcus",
    "pneumoconioses": "pneumoconiosis",
    "pneumoniae": "pneumonia",
    "pneumonias": "pneumonia",
    "pneumonitides": "pneumonitis",
    "pneumothoraces": "pneumothorax",
    "pneumothoraxes": "pneumothorax",
    "poached": "poach",
    "poachers": "poacher",
    "poaches": "poach",
    "poaching": "poach",
    "pochards": "pochard",
    "pocketbooks": "pocketbook",
    "pocketed": "pocket",
    "pocketfuls": "pocketful",
    "pocketing": "pocket",
    "pockets": "pocket",
    "pockmarks": "pockmark",
    "pocks": "pock",
    "podded": "pod",
    "podding": "pod",
    "podgier": "podgy",
    "podgiest": "podgy",
    "podia": "podium",
    "podiatrists": "podiatrist",
    "podiums": "podium",
    "pods": "pod",
    "poems": "poem",
    "poetesses": "poetess",
    "poets": "poet",
    "pogroms": "pogrom",
    "poinsettias": "poinsettia",
    "pointed": "point",
    "pointers": "pointer",
    "pointier": "pointy",
    "pointing": "point",
    "points": "point",
    "pois": "poi",
    "poised": "poise",
    "poises": "poise",
    "poising": "poise",
    "poisoned": "poison",
    "poisoners": "poisoner",
    "poisoning": "poison",
    "poisonings": "poisoning",
    "poisons": "poison",
    "poked": "poke",
    "pokers": "poker",
    "pokes": "poke",
    "pokier": "poky",
    "pokiest": "poky",
    "poking": "poke",
    "polarimetries": "polarimetry",
    "polarisations": "polarisation",
    "polarised": "polarise",
    "polarisers": "polariser",
    "polarises": "polarise",
    "polarising": "polarise",
    "polarities": "polarity",
    "polarizabilities": "polarizability",
    "polarizations": "polarization",
    "polarized": "polarize",
    "polarizers": "polarizer",
    "polarizes": "polarize",
    "polarizing": "polarize",
    "polars": "polar",
    "polders": "polder",
    "poleaxed": "poleaxe",
    "poleaxes": "poleaxe",
    "poleaxing": "poleaxe",
    "polecats": "polecat",
    "poled": "pole",
    "polemicists": "polemicist",
    "polemics": "polemic",
    "poles": "pole",
    "policed": "police",
    "policemen": "policeman",
    "polices": "police",
    "policewomen": "policewoman",
    "policies": "policy",
    "policing": "police",
    "policy-holders": "policy-holder",
    "policy-makers": "policy-maker",
    "policyholders": "policyholder",
    "policymakers": "policymaker",
    "poling": "pole",
    "poliomyelitides": "poliomyelitis",
    "polios": "polio",
    "polioviruses": "poliovirus",
    "polished": "polish",
    "polishers": "polisher",
    "polishes": "polish",
    "polishing": "polish",
    "polishings": "polishing",
    "politer": "polite",
    "politest": "polite",
    "politicals": "political",
    "politicians": "politician",
    "politicised": "politicise",
    "politicises": "politicise",
    "politicising": "politicise",
    "politicized": "politicize",
    "politicizes": "politicize",
    "politicizing": "politicize",
    "politicoes": "politico",
    "politicos": "politico",
    "politics": "politic",
    "polities": "polity",
    "politiques": "politique",
    "polkas": "polka",
    "pollacks": "pollack",
    "pollarded": "pollard",
    "pollarding": "pollard",
    "pollards": "pollard",
    "polled": "poll",
    "pollens": "pollen",
    "pollinated": "pollinate",
    "pollinates": "pollinate",
    "pollinating": "pollinate",
    "pollinations": "pollination",
    "pollinators": "pollinator",
    "polling": "poll",
    "pollocks": "pollock",
    "polls": "poll",
    "pollsters": "pollster",
    "pollutants": "pollutant",
    "polluted": "pollute",
    "polluters": "polluter",
    "pollutes": "pollute",
    "polluting": "pollute",
    "pollutions": "pollution",
    "pols": "pol",
    "poltergeists": "poltergeist",
    "polyacrylamides": "polyacrylamide",
    "polyamides": "polyamide",
    "polycarbonates": "polycarbonate",
    "polyesters": "polyester",
    "polyethylenes": "polyethylene",
    "polyglots": "polyglot",
    "polygons": "polygon",
    "polygraphs": "polygraph",
    "polyhedra": "polyhedron",
    "polyhedrons": "polyhedron",
    "polyimides": "polyimide",
    "polymaths": "polymath",
    "polymerases": "polymerase",
    "polymerisations": "polymerisation",
    "polymerised": "polymerise",
    "polymerises": "polymerise",
    "polymerising": "polymerise",
    "polymerizations": "polymerization",
    "polymerized": "polymerize",
    "polymerizes": "polymerize",
    "polymerizing": "polymerize",
    "polymers": "polymer",
    "polymorphisms": "polymorphism",
    "polymorphs": "polymorph",
    "polymyositides": "polymyositis",
    "polynomials": "polynomial",
    "polynucleotides": "polynucleotide",
    "polyolefins": "polyolefin",
    "polyols": "polyol",
    "polyominoes": "polyomino",
    "polypeptides": "polypeptide",
    "polyphenols": "polyphenol",
    "polyposes": "polyposis",
    "polypropylenes": "polypropylene",
    "polyproteins": "polyprotein",
    "polyps": "polyp",
    "polys": "poly",
    "polysaccharides": "polysaccharide",
    "polystyrenes": "polystyrene",
    "polysyllables": "polysyllable",
    "polytechnics": "polytechnic",
    "polytheists": "polytheist",
    "polytypes": "polytype",
    "polyunsaturates": "polyunsaturate",
    "polyurethanes": "polyurethane",
    "polyurias": "polyuria",
    "polyvinyls": "polyvinyl",
    "pom-poms": "pom-pom",
    "pomegranates": "pomegranate",
    "pommeled": "pommel",
    "pommeling": "pommel",
    "pommels": "pommel",
    "pommies": "pommy",
    "pomposities": "pomposity",
    "pomps": "pomp",
    "poms": "pom",
    "ponced": "ponce",
    "ponces": "ponce",
    "ponchos": "poncho",
    "poncing": "ponce",
    "pondered": "ponder",
    "pondering": "ponder",
    "ponders": "ponder",
    "ponds": "pond",
    "pondweeds": "pondweed",
    "ponged": "pong",
    "ponging": "pong",
    "pongs": "pong",
    "ponies": "pony",
    "pontiffs": "pontiff",
    "pontificated": "pontificate",
    "pontificates": "pontificate",
    "pontificating": "pontificate",
    "pontoons": "pontoon",
    "ponts": "pont",
    "ponytails": "ponytail",
    "pooches": "pooch",
    "poodles": "poodle",
    "poofs": "poof",
    "pooh-poohed": "pooh-pooh",
    "pooh-poohing": "pooh-pooh",
    "pooh-poohs": "pooh-pooh",
    "pooled": "pool",
    "pooling": "pool",
    "pools": "pool",
    "poops": "poop",
    "poorer": "poor",
    "poorest": "poor",
    "poorhouses": "poorhouse",
    "pop-ups": "pop-up",
    "poplars": "poplar",
    "poppadums": "poppadum",
    "popped": "pop",
    "poppers": "popper",
    "poppets": "poppet",
    "poppies": "poppy",
    "popping": "pop",
    "pops": "pop",
    "populaces": "populace",
    "popularisations": "popularisation",
    "popularised": "popularise",
    "popularises": "popularise",
    "popularising": "popularise",
    "popularities": "popularity",
    "popularizations": "popularization",
    "popularized": "popularize",
    "popularizes": "popularize",
    "popularizing": "popularize",
    "populated": "populate",
    "populates": "populate",
    "populating": "populate",
    "populations": "population",
    "populists": "populist",
    "porcelains": "porcelain",
    "porches": "porch",
    "porcupines": "porcupine",
    "pored": "pore",
    "pores": "pore",
    "poring": "pore",
    "porkers": "porker",
    "pornographers": "pornographer",
    "porosities": "porosity",
    "porphyrias": "porphyria",
    "porphyries": "porphyry",
    "porphyrins": "porphyrin",
    "porpoises": "porpoise",
    "porridges": "porridge",
    "portables": "portable",
    "portae": "porta",
    "portages": "portage",
    "portals": "portal",
    "portcullises": "portcullis",
    "ported": "port",
    "portended": "portend",
    "portending": "portend",
    "portends": "portend",
    "portents": "portent",
    "porters": "porter",
    "portfolios": "portfolio",
    "portholes": "porthole",
    "porticoes": "portico",
    "porticos": "portico",
    "porting": "port",
    "portioned": "portion",
    "portioning": "portion",
    "portions": "portion",
    "portlier": "portly",
    "portliest": "portly",
    "portmanteaus": "portmanteau",
    "portmanteaux": "portmanteau",
    "portraitists": "portraitist",
    "portraits": "portrait",
    "portrayals": "portrayal",
    "portrayed": "portray",
    "portraying": "portray",
    "portrays": "portray",
    "ports": "port",
    "posed": "pose",
    "posers": "poser",
    "poses": "pose",
    "poseurs": "poseur",
    "posher": "posh",
    "poshest": "posh",
    "posies": "posy",
    "posing": "pose",
    "posited": "posit",
    "positing": "posit",
    "positioned": "position",
    "positioners": "positioner",
    "positioning": "position",
    "positions": "position",
    "positives": "positive",
    "positivists": "positivist",
    "positivities": "positivity",
    "positons": "positon",
    "positrons": "positron",
    "posits": "posit",
    "posses": "posse",
    "possessed": "possess",
    "possesses": "possess",
    "possessing": "possess",
    "possessions": "possession",
    "possessives": "possessive",
    "possessors": "possessor",
    "possibilities": "possibility",
    "possibles": "possible",
    "possums": "possum",
    "post-boxes": "post-box",
    "post-codes": "post-code",
    "post-dated": "post-date",
    "post-dates": "post-date",
    "post-dating": "post-date",
    "post-docs": "post-doc",
    "post-graduates": "post-graduate",
    "post-holders": "post-holder",
    "post-holes": "post-hole",
    "post-independences": "post-independence",
    "post-modernists": "post-modernist",
    "post-mortems": "post-mortem",
    "post-processors": "post-processor",
    "post-transfusions": "post-transfusion",
    "post-treatments": "post-treatment",
    "postbags": "postbag",
    "postboxes": "postbox",
    "postcards": "postcard",
    "postcodes": "postcode",
    "postdated": "postdate",
    "postdates": "postdate",
    "postdating": "postdate",
    "postdocs": "postdoc",
    "posted": "post",
    "posteriors": "posterior",
    "posterities": "posterity",
    "posterns": "postern",
    "posters": "poster",
    "postgrads": "postgrad",
    "postgraduates": "postgraduate",
    "postholders": "postholder",
    "postholes": "posthole",
    "posting": "post",
    "postings": "posting",
    "postmarked": "postmark",
    "postmarking": "postmark",
    "postmarks": "postmark",
    "postmasters": "postmaster",
    "postmen": "postman",
    "postmistresses": "postmistress",
    "postmodernists": "postmodernist",
    "postmortems": "postmortem",
    "postponed": "postpone",
    "postponements": "postponement",
    "postpones": "postpone",
    "postponing": "postpone",
    "posts": "post",
    "postscripts": "postscript",
    "poststructuralists": "poststructuralist",
    "postulated": "postulate",
    "postulates": "postulate",
    "postulating": "postulate",
    "postulations": "postulation",
    "postured": "posture",
    "postures": "posture",
    "posturing": "posture",
    "posturings": "posturing",
    "pot-boilers": "pot-boiler",
    "pot-holes": "pot-hole",
    "pot-shots": "pot-shot",
    "potatoes": "potato",
    "potencies": "potency",
    "potentates": "potentate",
    "potentialities": "potentiality",
    "potentials": "potential",
    "potentiated": "potentiate",
    "potentiates": "potentiate",
    "potentiating": "potentiate",
    "potentiations": "potentiation",
    "potentiometers": "potentiometer",
    "potholers": "potholer",
    "potholes": "pothole",
    "potions": "potion",
    "potpourris": "potpourri",
    "pots": "pot",
    "potshots": "potshot",
    "potted": "pot",
    "pottered": "potter",
    "potteries": "pottery",
    "pottering": "potter",
    "potters": "potter",
    "pottier": "potty",
    "potties": "potty",
    "pottiest": "potty",
    "potting": "pot",
    "potyviruses": "potyvirus",
    "pouched": "pouch",
    "pouches": "pouch",
    "pouching": "pouch",
    "pouffes": "pouffe",
    "poultices": "poultice",
    "poultries": "poultry",
    "poults": "poult",
    "pounced": "pounce",
    "pounces": "pounce",
    "pouncing": "pounce",
    "pounded": "pound",
    "pounding": "pound",
    "pounds": "pound",
    "poured": "pour",
    "pourers": "pourer",
    "pouring": "pour",
    "pours": "pour",
    "pouted": "pout",
    "pouting": "pout",
    "pouts": "pout",
    "powdered": "powder",
    "powdering": "powder",
    "powders": "powder",
    "powerboats": "powerboat",
    "powered": "power",
    "powerhouses": "powerhouse",
    "powering": "power",
    "powers": "power",
    "powwows": "powwow",
    "poxes": "pox",
    "poxviruses": "poxvirus",
    "ppbs": "ppb",
    "pps": "pp",
    "ppts": "ppt",
    "practic": "practice",
    "practica": "practicum",
    "practicabilities": "practicability",
    "practicalities": "practicality",
    "practicals": "practical",
    "practiced": "practice",
    "practices": "practice",
    "practicing": "practice",
    "practicums": "practicum",
    "practised": "practise",
    "practises": "practise",
    "practising": "practise",
    "practitioners": "practitioner",
    "praetors": "praetor",
    "pragmatics": "pragmatic",
    "pragmatists": "pragmatist",
    "prairies": "prairie",
    "praised": "praise",
    "praises": "praise",
    "praiseworthier": "praiseworthy",
    "praiseworthiest": "praiseworthy",
    "praising": "praise",
    "pralines": "praline",
    "prams": "pram",
    "pranced": "prance",
    "prances": "prance",
    "prancing": "prance",
    "pranks": "prank",
    "pranksters": "prankster",
    "prated": "prate",
    "prates": "prate",
    "prating": "prate",
    "pratiques": "pratique",
    "prats": "prat",
    "prattled": "prattle",
    "prattles": "prattle",
    "prattling": "prattle",
    "prawns": "prawn",
    "praxes": "praxis",
    "prayed": "pray",
    "prayers": "prayer",
    "praying": "pray",
    "prays": "pray",
    "pre-amplifiers": "pre-amplifier",
    "pre-approvals": "pre-approval",
    "pre-assessments": "pre-assessment",
    "pre-conceptions": "pre-conception",
    "pre-conferences": "pre-conference",
    "pre-constructions": "pre-construction",
    "pre-cursors": "pre-cursor",
    "pre-dated": "pre-date",
    "pre-dates": "pre-date",
    "pre-dating": "pre-date",
    "pre-defined": "pre-define",
    "pre-defines": "pre-define",
    "pre-defining": "pre-define",
    "pre-determined": "pre-determine",
    "pre-determines": "pre-determine",
    "pre-determining": "pre-determine",
    "pre-empted": "pre-empt",
    "pre-empting": "pre-empt",
    "pre-empts": "pre-empt",
    "pre-existed": "pre-exist",
    "pre-existing": "pre-exist",
    "pre-exists": "pre-exist",
    "pre-fabricated": "pre-fabricate",
    "pre-fabricates": "pre-fabricate",
    "pre-fabricating": "pre-fabricate",
    "pre-filled": "pre-fill",
    "pre-filling": "pre-fill",
    "pre-fills": "pre-fill",
    "pre-formed": "pre-form",
    "pre-forming": "pre-form",
    "pre-forms": "pre-form",
    "pre-heated": "pre-heat",
    "pre-heating": "pre-heat",
    "pre-heats": "pre-heat",
    "pre-loaded": "pre-load",
    "pre-loading": "pre-load",
    "pre-loads": "pre-load",
    "pre-movements": "pre-movement",
    "pre-occupations": "pre-occupation",
    "pre-ordered": "pre-order",
    "pre-ordering": "pre-order",
    "pre-orders": "pre-order",
    "pre-paid": "pre-pay",
    "pre-paying": "pre-pay",
    "pre-payments": "pre-payment",
    "pre-pays": "pre-pay",
    "pre-printed": "pre-print",
    "pre-printing": "pre-print",
    "pre-prints": "pre-print",
    "pre-processed": "pre-process",
    "pre-processes": "pre-process",
    "pre-processing": "pre-process",
    "pre-processors": "pre-processor",
    "pre-programmed": "pre-programme",
    "pre-programmes": "pre-programme",
    "pre-programming": "pre-programme",
    "pre-publications": "pre-publication",
    "pre-qualifications": "pre-qualification",
    "pre-qualified": "pre-qualify",
    "pre-qualifies": "pre-qualify",
    "pre-qualifying": "pre-qualify",
    "pre-releases": "pre-release",
    "pre-requisites": "pre-requisite",
    "pre-schoolers": "pre-schooler",
    "pre-schools": "pre-school",
    "pre-selected": "pre-select",
    "pre-selecting": "pre-select",
    "pre-selections": "pre-selection",
    "pre-selects": "pre-select",
    "pre-set": "preset",
    "pre-sets": "pre-set",
    "pre-setting": "preset",
    "pre-teens": "pre-teen",
    "pre-tests": "pre-test",
    "pre-treatments": "pre-treatment",
    "preached": "preach",
    "preachers": "preacher",
    "preaches": "preach",
    "preaching": "preach",
    "preachings": "preaching",
    "preambles": "preamble",
    "preamplifiers": "preamplifier",
    "prearranged": "prearrange",
    "prearranges": "prearrange",
    "prearranging": "prearrange",
    "prebiotics": "prebiotic",
    "precautions": "precaution",
    "preceded": "precede",
    "precedences": "precedence",
    "precedents": "precedent",
    "precedes": "precede",
    "preceding": "precede",
    "preceptors": "preceptor",
    "preceptorships": "preceptorship",
    "precepts": "precept",
    "precessed": "precess",
    "precesses": "precess",
    "precessing": "precess",
    "precessions": "precession",
    "precincts": "precinct",
    "precipices": "precipice",
    "precipitants": "precipitant",
    "precipitated": "precipitate",
    "precipitates": "precipitate",
    "precipitating": "precipitate",
    "precipitations": "precipitation",
    "precisions": "precision",
    "precluded": "preclude",
    "precludes": "preclude",
    "precluding": "preclude",
    "precognitions": "precognition",
    "preconceived": "preconceive",
    "preconceives": "preconceive",
    "preconceiving": "preconceive",
    "preconceptions": "preconception",
    "preconditioned": "precondition",
    "preconditioning": "precondition",
    "preconditions": "precondition",
    "precursors": "precursor",
    "predated": "predate",
    "predates": "predate",
    "predating": "predate",
    "predators": "predator",
    "predeceased": "predecease",
    "predeceases": "predecease",
    "predeceasing": "predecease",
    "predecessors": "predecessor",
    "predefined": "predefine",
    "predefines": "predefine",
    "predefining": "predefine",
    "predestined": "predestine",
    "predestines": "predestine",
    "predestining": "predestine",
    "predetermined": "predetermine",
    "predeterminers": "predeterminer",
    "predetermines": "predetermine",
    "predetermining": "predetermine",
    "predicaments": "predicament",
    "predicated": "predicate",
    "predicates": "predicate",
    "predicating": "predicate",
    "predications": "predication",
    "predictabilities": "predictability",
    "predicted": "predict",
    "predicting": "predict",
    "predictions": "prediction",
    "predictors": "predictor",
    "predicts": "predict",
    "predilections": "predilection",
    "predisposed": "predispose",
    "predisposes": "predispose",
    "predisposing": "predispose",
    "predispositions": "predisposition",
    "prednisolones": "prednisolone",
    "predominances": "predominance",
    "predominated": "predominate",
    "predominates": "predominate",
    "predominating": "predominate",
    "preempted": "preempt",
    "preempting": "preempt",
    "preempts": "preempt",
    "preened": "preen",
    "preening": "preen",
    "preens": "preen",
    "preexisted": "preexist",
    "preexisting": "preexist",
    "preexists": "preexist",
    "prefabricated": "prefabricate",
    "prefabricates": "prefabricate",
    "prefabricating": "prefabricate",
    "prefabs": "prefab",
    "prefaced": "preface",
    "prefaces": "preface",
    "prefacing": "preface",
    "prefects": "prefect",
    "prefectures": "prefecture",
    "preferences": "preference",
    "preferred": "prefer",
    "preferring": "prefer",
    "prefers": "prefer",
    "prefigured": "prefigure",
    "prefigures": "prefigure",
    "prefiguring": "prefigure",
    "prefixed": "prefix",
    "prefixes": "prefix",
    "prefixing": "prefix",
    "prefolds": "prefold",
    "preformed": "preform",
    "preforming": "preform",
    "preforms": "preform",
    "pregnancies": "pregnancy",
    "preheated": "preheat",
    "preheating": "preheat",
    "preheats": "preheat",
    "prehistorians": "prehistorian",
    "preimages": "preimage",
    "prejudged": "prejudge",
    "prejudges": "prejudge",
    "prejudging": "prejudge",
    "prejudiced": "prejudice",
    "prejudices": "prejudice",
    "prejudicing": "prejudice",
    "prelates": "prelate",
    "preliminaries": "preliminary",
    "preloaded": "preload",
    "preloading": "preload",
    "preloads": "preload",
    "preludes": "prelude",
    "prematurities": "prematurity",
    "premeditated": "premeditate",
    "premeditates": "premeditate",
    "premeditating": "premeditate",
    "premiered": "premiere",
    "premieres": "premiere",
    "premiering": "premiere",
    "premiers": "premier",
    "premised": "premise",
    "premises": "premise",
    "premising": "premise",
    "premisses": "premiss",
    "premiums": "premium",
    "premières": "première",
    "premolars": "premolar",
    "premonitions": "premonition",
    "prenatals": "prenatal",
    "preoccupations": "preoccupation",
    "preoccupied": "preoccupy",
    "preoccupies": "preoccupy",
    "preoccupying": "preoccupy",
    "preordained": "preordain",
    "preordaining": "preordain",
    "preordains": "preordain",
    "preordered": "preorder",
    "preordering": "preorder",
    "preorders": "preorder",
    "prepaid": "prepay",
    "preparations": "preparation",
    "prepared": "prepare",
    "preparers": "preparer",
    "prepares": "prepare",
    "preparing": "prepare",
    "prepaying": "prepay",
    "prepayments": "prepayment",
    "prepays": "prepay",
    "prepended": "prepend",
    "prepending": "prepend",
    "prepends": "prepend",
    "preponderated": "preponderate",
    "preponderates": "preponderate",
    "preponderating": "preponderate",
    "prepositions": "preposition",
    "prepped": "prep",
    "preppies": "preppy",
    "prepping": "prep",
    "preprocessed": "preprocess",
    "preprocesses": "preprocess",
    "preprocessing": "preprocess",
    "preprocessors": "preprocessor",
    "preps": "prep",
    "prerecorded": "prerecord",
    "prerecording": "prerecord",
    "prerecords": "prerecord",
    "prereleases": "prerelease",
    "prerequisites": "prerequisite",
    "prerogatives": "prerogative",
    "presaged": "presage",
    "presages": "presage",
    "presaging": "presage",
    "presbyterians": "presbyterian",
    "presbyteries": "presbytery",
    "preschoolers": "preschooler",
    "preschools": "preschool",
    "prescribed": "prescribe",
    "prescribers": "prescriber",
    "prescribes": "prescribe",
    "prescribing": "prescribe",
    "prescriptions": "prescription",
    "presences": "presence",
    "presenilins": "presenilin",
    "presentations": "presentation",
    "presented": "present",
    "presenters": "presenter",
    "presentiments": "presentiment",
    "presenting": "present",
    "presentments": "presentment",
    "presents": "present",
    "preservationists": "preservationist",
    "preservations": "preservation",
    "preservatives": "preservative",
    "preserved": "preserve",
    "preservers": "preserver",
    "preserves": "preserve",
    "preserving": "preserve",
    "presets": "preset",
    "presetting": "preset",
    "presided": "preside",
    "presidencies": "presidency",
    "president-elects": "president-elect",
    "presidents": "president",
    "presidents-elect": "president-elect",
    "presides": "preside",
    "presiding": "preside",
    "presidiums": "presidium",
    "press-ganged": "press-gang",
    "press-ganging": "press-gang",
    "press-gangs": "press-gang",
    "press-ups": "press-up",
    "pressed": "press",
    "presses": "press",
    "pressganged": "pressgang",
    "pressganging": "pressgang",
    "pressgangs": "pressgang",
    "pressing": "press",
    "pressmen": "pressman",
    "pressured": "pressure",
    "pressures": "pressure",
    "pressuring": "pressure",
    "pressurisations": "pressurisation",
    "pressurised": "pressurise",
    "pressurises": "pressurise",
    "pressurising": "pressurise",
    "pressurized": "pressurize",
    "pressurizes": "pressurize",
    "pressurizing": "pressurize",
    "prestressed": "prestress",
    "prestresses": "prestress",
    "prestressing": "prestress",
    "presumed": "presume",
    "presumes": "presume",
    "presuming": "presume",
    "presumptions": "presumption",
    "presupposed": "presuppose",
    "presupposes": "presuppose",
    "presupposing": "presuppose",
    "presuppositions": "presupposition",
    "pretences": "pretence",
    "pretended": "pretend",
    "pretenders": "pretender",
    "pretending": "pretend",
    "pretends": "pretend",
    "pretenses": "pretense",
    "pretensions": "pretension",
    "pretentions": "pretention",
    "preterites": "preterite",
    "preterms": "preterm",
    "pretexts": "pretext",
    "pretreatments": "pretreatment",
    "prettier": "pretty",
    "prettiest": "pretty",
    "prettified": "prettify",
    "prettifies": "prettify",
    "prettifying": "prettify",
    "pretzels": "pretzel",
    "prevailed": "prevail",
    "prevailing": "prevail",
    "prevails": "prevail",
    "prevalences": "prevalence",
    "prevaricated": "prevaricate",
    "prevaricates": "prevaricate",
    "prevaricating": "prevaricate",
    "prevarications": "prevarication",
    "prevented": "prevent",
    "preventers": "preventer",
    "preventing": "prevent",
    "preventions": "prevention",
    "preventives": "preventive",
    "prevents": "prevent",
    "previewed": "preview",
    "previewing": "preview",
    "previews": "preview",
    "preyed": "prey",
    "preying": "prey",
    "preys": "prey",
    "priced": "price",
    "prices": "price",
    "pricier": "pricey",
    "priciest": "pricey",
    "pricing": "price",
    "pricked": "prick",
    "pricking": "prick",
    "prickled": "prickle",
    "prickles": "prickle",
    "pricklier": "prickly",
    "prickliest": "prickly",
    "prickling": "prickle",
    "pricks": "prick",
    "prided": "pride",
    "prides": "pride",
    "priding": "pride",
    "pried": "pry",
    "pries": "pry",
    "priestesses": "priestess",
    "priestlier": "priestly",
    "priests": "priest",
    "prigs": "prig",
    "primacies": "primacy",
    "primaries": "primary",
    "primates": "primate",
    "primed": "prime",
    "primers": "primer",
    "primes": "prime",
    "priming": "prime",
    "primitives": "primitive",
    "primmer": "prim",
    "primmest": "prim",
    "primos": "primo",
    "primroses": "primrose",
    "primulae": "primula",
    "primulas": "primula",
    "princelier": "princely",
    "princes": "prince",
    "princesses": "princess",
    "principalities": "principality",
    "principals": "principal",
    "principes": "princeps",
    "principles": "principle",
    "print-outs": "print-out",
    "printed": "print",
    "printers": "printer",
    "printing": "print",
    "printings": "printing",
    "printingshops": "printingshop",
    "printouts": "printout",
    "prints": "print",
    "printshops": "printshop",
    "prions": "prion",
    "prioresses": "prioress",
    "priories": "priory",
    "priorities": "priority",
    "prioritisations": "prioritisation",
    "prioritised": "prioritise",
    "prioritises": "prioritise",
    "prioritising": "prioritise",
    "prioritizations": "prioritization",
    "prioritized": "prioritize",
    "prioritizes": "prioritize",
    "prioritizing": "prioritize",
    "priors": "prior",
    "prised": "prise",
    "prises": "prise",
    "prising": "prise",
    "prisms": "prism",
    "prisoners": "prisoner",
    "prisons": "prison",
    "prissier": "prissy",
    "prissiest": "prissy",
    "privateers": "privateer",
    "privates": "private",
    "privations": "privation",
    "privatisations": "privatisation",
    "privatised": "privatise",
    "privatises": "privatise",
    "privatising": "privatise",
    "privatizations": "privatization",
    "privatized": "privatize",
    "privatizes": "privatize",
    "privatizing": "privatize",
    "privets": "privet",
    "privies": "privy",
    "privileged": "privilege",
    "privileges": "privilege",
    "privileging": "privilege",
    "privities": "privity",
    "prize-fighters": "prize-fighter",
    "prize-givings": "prize-giving",
    "prized": "prize",
    "prizefighters": "prizefighter",
    "prizes": "prize",
    "prizewinners": "prizewinner",
    "prizing": "prize",
    "pro-lifers": "pro-lifer",
    "proactivities": "proactivity",
    "probabilities": "probability",
    "probands": "proband",
    "probates": "probate",
    "probationers": "probationer",
    "probations": "probation",
    "probed": "probe",
    "probes": "probe",
    "probing": "probe",
    "probiotics": "probiotic",
    "probits": "probit",
    "problematics": "problematic",
    "problematised": "problematise",
    "problematises": "problematise",
    "problematising": "problematise",
    "problematized": "problematize",
    "problematizes": "problematize",
    "problematizing": "problematize",
    "problems": "problem",
    "proboscides": "proboscis",
    "proboscises": "proboscis",
    "probs": "prob",
    "procedures": "procedure",
    "proceeded": "proceed",
    "proceeding": "proceed",
    "proceedings": "proceeding",
    "proceeds": "proceed",
    "processed": "process",
    "processes": "process",
    "processing": "process",
    "processings": "processing",
    "processionals": "processional",
    "processions": "procession",
    "processors": "processor",
    "proclaimed": "proclaim",
    "proclaiming": "proclaim",
    "proclaims": "proclaim",
    "proclamations": "proclamation",
    "proclivities": "proclivity",
    "procrastinated": "procrastinate",
    "procrastinates": "procrastinate",
    "procrastinating": "procrastinate",
    "procreated": "procreate",
    "procreates": "procreate",
    "procreating": "procreate",
    "procs": "proc",
    "proctors": "proctor",
    "procurators": "procurator",
    "procured": "procure",
    "procurements": "procurement",
    "procurers": "procurer",
    "procures": "procure",
    "procuresses": "procuress",
    "procuring": "procure",
    "prodded": "prod",
    "prodding": "prod",
    "prodigals": "prodigal",
    "prodigies": "prodigy",
    "prodrugs": "prodrug",
    "prods": "prod",
    "produced": "produce",
    "producers": "producer",
    "produces": "produce",
    "producing": "produce",
    "productions": "production",
    "productivities": "productivity",
    "products": "product",
    "profaned": "profane",
    "profanes": "profane",
    "profaning": "profane",
    "profanities": "profanity",
    "professed": "profess",
    "professes": "profess",
    "professing": "profess",
    "professionalised": "professionalise",
    "professionalises": "professionalise",
    "professionalising": "professionalise",
    "professionals": "professional",
    "professions": "profession",
    "professors": "professor",
    "professorships": "professorship",
    "proffered": "proffer",
    "proffering": "proffer",
    "proffers": "proffer",
    "proficiencies": "proficiency",
    "profiled": "profile",
    "profilers": "profiler",
    "profiles": "profile",
    "profiling": "profile",
    "profitabilities": "profitability",
    "profited": "profit",
    "profiteered": "profiteer",
    "profiteering": "profiteer",
    "profiteers": "profiteer",
    "profiting": "profit",
    "profits": "profit",
    "proformae": "proforma",
    "proformas": "proforma",
    "proformata": "proforma",
    "profounder": "profound",
    "profoundest": "profound",
    "profs": "prof",
    "profundities": "profundity",
    "profusions": "profusion",
    "progenies": "progeny",
    "progenitors": "progenitor",
    "progesterones": "progesterone",
    "progestins": "progestin",
    "progestogens": "progestogen",
    "prognoses": "prognosis",
    "prognostications": "prognostication",
    "prognostics": "prognostic",
    "programed": "program",
    "programing": "program",
    "programmed": "programme",
    "programmers": "programmer",
    "programmes": "programme",
    "programming": "programme",
    "programs": "program",
    "progressed": "progress",
    "progresses": "progress",
    "progressing": "progress",
    "progressions": "progression",
    "progressives": "progressive",
    "prohibited": "prohibit",
    "prohibiting": "prohibit",
    "prohibitionists": "prohibitionist",
    "prohibitions": "prohibition",
    "prohibits": "prohibit",
    "projected": "project",
    "projectiles": "projectile",
    "projecting": "project",
    "projectionists": "projectionist",
    "projections": "projection",
    "projectors": "projector",
    "projects": "project",
    "prokaryotes": "prokaryote",
    "prolactinomas": "prolactinoma",
    "prolactinomata": "prolactinoma",
    "prolactins": "prolactin",
    "prolapsed": "prolapse",
    "prolapses": "prolapse",
    "prolapsing": "prolapse",
    "proles": "prole",
    "proletarians": "proletarian",
    "proletariats": "proletariat",
    "proliferated": "proliferate",
    "proliferates": "proliferate",
    "proliferating": "proliferate",
    "proliferations": "proliferation",
    "proliferators": "proliferator",
    "prolines": "proline",
    "prologues": "prologue",
    "prolongations": "prolongation",
    "prolonged": "prolong",
    "prolonging": "prolong",
    "prolongs": "prolong",
    "promenaded": "promenade",
    "promenades": "promenade",
    "promenading": "promenade",
    "prominences": "prominence",
    "promised": "promise",
    "promises": "promise",
    "promising": "promise",
    "promontories": "promontory",
    "promoted": "promote",
    "promoters": "promoter",
    "promotes": "promote",
    "promoting": "promote",
    "promotions": "promotion",
    "promotors": "promotor",
    "prompted": "prompt",
    "prompter": "prompt",
    "prompters": "prompter",
    "promptest": "prompt",
    "prompting": "prompt",
    "promptings": "prompting",
    "prompts": "prompt",
    "proms": "prom",
    "promulgated": "promulgate",
    "promulgates": "promulgate",
    "promulgating": "promulgate",
    "pronated": "pronate",
    "pronates": "pronate",
    "pronating": "pronate",
    "pronations": "pronation",
    "pronenesses": "proneness",
    "prongs": "prong",
    "pronounced": "pronounce",
    "pronouncements": "pronouncement",
    "pronounces": "pronounce",
    "pronouncing": "pronounce",
    "pronouns": "pronoun",
    "pronunciations": "pronunciation",
    "proof-read": "proofread",
    "proof-reader": "proofreader",
    "proof-readers": "proofreader",
    "proof-reading": "proof-read",
    "proof-reads": "proof-read",
    "proofed": "proof",
    "proofing": "proof",
    "proofreaded": "proofread",
    "proofreaders": "proofreader",
    "proofreading": "proofread",
    "proofreads": "proofread",
    "proofs": "proof",
    "propagandised": "propagandise",
    "propagandises": "propagandise",
    "propagandising": "propagandise",
    "propagandists": "propagandist",
    "propagandized": "propagandize",
    "propagandizes": "propagandize",
    "propagandizing": "propagandize",
    "propagated": "propagate",
    "propagates": "propagate",
    "propagating": "propagate",
    "propagations": "propagation",
    "propagators": "propagator",
    "propanes": "propane",
    "propellants": "propellant",
    "propelled": "propel",
    "propellers": "propeller",
    "propelling": "propel",
    "propellors": "propellor",
    "propels": "propel",
    "propensities": "propensity",
    "properties": "property",
    "prophecies": "prophecy",
    "prophesied": "prophesy",
    "prophesies": "prophesy",
    "prophesying": "prophesy",
    "prophetesses": "prophetess",
    "prophetesss": "prophetess",
    "prophets": "prophet",
    "prophylactics": "prophylactic",
    "prophylaxes": "prophylaxis",
    "propionates": "propionate",
    "propitiated": "propitiate",
    "propitiates": "propitiate",
    "propitiating": "propitiate",
    "proponents": "proponent",
    "proportionalities": "proportionality",
    "proportionated": "proportionate",
    "proportionates": "proportionate",
    "proportionating": "proportionate",
    "proportioned": "proportion",
    "proportioning": "proportion",
    "proportions": "proportion",
    "proposals": "proposal",
    "proposed": "propose",
    "proposers": "proposer",
    "proposes": "propose",
    "proposing": "propose",
    "propositioned": "proposition",
    "propositioning": "proposition",
    "propositions": "proposition",
    "propounded": "propound",
    "propounding": "propound",
    "propounds": "propound",
    "propped": "prop",
    "proppers": "propper",
    "propping": "prop",
    "proprietaries": "proprietary",
    "proprieties": "propriety",
    "proprietors": "proprietor",
    "proprietresses": "proprietress",
    "proprioceptions": "proprioception",
    "props": "prop",
    "prorogued": "prorogue",
    "prorogues": "prorogue",
    "proroguing": "prorogue",
    "pros": "pro",
    "prosceniums": "proscenium",
    "proscribed": "proscribe",
    "proscribes": "proscribe",
    "proscribing": "proscribe",
    "proscriptions": "proscription",
    "prosecuted": "prosecute",
    "prosecutes": "prosecute",
    "prosecuting": "prosecute",
    "prosecutions": "prosecution",
    "prosecutors": "prosecutor",
    "proselytes": "proselyte",
    "proselytised": "proselytise",
    "proselytises": "proselytise",
    "proselytising": "proselytise",
    "proselytized": "proselytize",
    "proselytizes": "proselytize",
    "proselytizing": "proselytize",
    "proses": "prose",
    "prosier": "prosy",
    "prosiest": "prosy",
    "prosodies": "prosody",
    "prosopographies": "prosopography",
    "prospected": "prospect",
    "prospecting": "prospect",
    "prospectors": "prospector",
    "prospects": "prospect",
    "prospectuses": "prospectus",
    "prospered": "prosper",
    "prospering": "prosper",
    "prospers": "prosper",
    "prostaglandins": "prostaglandin",
    "prostatectomies": "prostatectomy",
    "prostates": "prostate",
    "prostatitides": "prostatitis",
    "prostheses": "prosthesis",
    "prosthetics": "prosthetic",
    "prosthetists": "prosthetist",
    "prostituted": "prostitute",
    "prostitutes": "prostitute",
    "prostituting": "prostitute",
    "prostrated": "prostrate",
    "prostrates": "prostrate",
    "prostrating": "prostrate",
    "prostrations": "prostration",
    "protagonists": "protagonist",
    "protamines": "protamine",
    "proteases": "protease",
    "proteasomes": "proteasome",
    "protectants": "protectant",
    "protected": "protect",
    "protecting": "protect",
    "protectionists": "protectionist",
    "protections": "protection",
    "protectorates": "protectorate",
    "protectors": "protector",
    "protects": "protect",
    "proteges": "protege",
    "proteinases": "proteinase",
    "proteins": "protein",
    "proteinurias": "proteinuria",
    "proteoglycans": "proteoglycan",
    "proteolyses": "proteolysis",
    "proteomes": "proteome",
    "protestations": "protestation",
    "protested": "protest",
    "protesters": "protester",
    "protesting": "protest",
    "protestors": "protestor",
    "protests": "protest",
    "prothrombins": "prothrombin",
    "protists": "protist",
    "protocols": "protocol",
    "protons": "proton",
    "protoplasms": "protoplasm",
    "protoplasts": "protoplast",
    "protostars": "protostar",
    "prototyped": "prototype",
    "prototypes": "prototype",
    "prototyping": "prototype",
    "protozoa": "protozoon",
    "protozoans": "protozoan",
    "protozoons": "protozoon",
    "protractors": "protractor",
    "protruded": "protrude",
    "protrudes": "protrude",
    "protruding": "protrude",
    "protrusions": "protrusion",
    "protuberances": "protuberance",
    "protégées": "protégée",
    "protégés": "protégé",
    "prouder": "proud",
    "proudest": "proud",
    "proved": "prove",
    "proven": "prove",
    "provenances": "provenance",
    "proverbs": "proverb",
    "proves": "prove",
    "provided": "provide",
    "providers": "provider",
    "provides": "provide",
    "providing": "provide",
    "provinces": "province",
    "provincialisms": "provincialism",
    "provincials": "provincial",
    "proving": "prove",
    "provings": "proving",
    "provisioned": "provision",
    "provisioning": "provision",
    "provisions": "provision",
    "provisoes": "proviso",
    "provisos": "proviso",
    "provocateurs": "provocateur",
    "provocations": "provocation",
    "provoked": "provoke",
    "provokes": "provoke",
    "provoking": "provoke",
    "provosts": "provost",
    "prowled": "prowl",
    "prowlers": "prowler",
    "prowling": "prowl",
    "prowls": "prowl",
    "prows": "prow",
    "proxies": "proxy",
    "proximities": "proximity",
    "prudences": "prudence",
    "prudes": "prude",
    "pruned": "prune",
    "pruners": "pruner",
    "prunes": "prune",
    "pruning": "prune",
    "prurituses": "pruritus",
    "prying": "pry",
    "psalmists": "psalmist",
    "psalms": "psalm",
    "pseudo-codes": "pseudo-code",
    "pseudo-sciences": "pseudo-science",
    "pseudogenes": "pseudogene",
    "pseudonyms": "pseudonym",
    "pseudosciences": "pseudoscience",
    "pseuds": "pseud",
    "psoralens": "psoralen",
    "psoriases": "psoriasis",
    "psps": "psp",
    "psyched": "psyche",
    "psychedelics": "psychedelic",
    "psyches": "psyche",
    "psychiatries": "psychiatry",
    "psychiatrists": "psychiatrist",
    "psychics": "psychic",
    "psyching": "psyche",
    "psycho-analyses": "psycho-analysis",
    "psychoanalysed": "psychoanalyse",
    "psychoanalyses": "psychoanalysis",
    "psychoanalysing": "psychoanalyse",
    "psychoanalysts": "psychoanalyst",
    "psychodramas": "psychodrama",
    "psychologies": "psychology",
    "psychologists": "psychologist",
    "psychopathies": "psychopathy",
    "psychopathologies": "psychopathology",
    "psychopaths": "psychopath",
    "psychopharmacologies": "psychopharmacology",
    "psychos": "psycho",
    "psychoses": "psychosis",
    "psychotherapies": "psychotherapy",
    "psychotherapists": "psychotherapist",
    "psychotics": "psychotic",
    "psychs": "psych",
    "ptarmigans": "ptarmigan",
    "pterodactyls": "pterodactyl",
    "pterosaurs": "pterosaur",
    "pts": "pt",
    "puberties": "puberty",
    "pubes": "pubis",
    "publicans": "publican",
    "publications": "publication",
    "publicised": "publicise",
    "publicises": "publicise",
    "publicising": "publicise",
    "publicists": "publicist",
    "publicities": "publicity",
    "publicized": "publicize",
    "publicizes": "publicize",
    "publicizing": "publicize",
    "publics": "public",
    "published": "publish",
    "publishers": "publisher",
    "publishes": "publish",
    "publishing": "publish",
    "publishings": "publishing",
    "pubs": "pub",
    "puckered": "pucker",
    "puckering": "pucker",
    "puckers": "pucker",
    "pucks": "puck",
    "puddings": "pudding",
    "puddled": "puddle",
    "puddles": "puddle",
    "puddling": "puddle",
    "pudgier": "pudgy",
    "puds": "pud",
    "pueblos": "pueblo",
    "puffballs": "puffball",
    "puffed": "puff",
    "puffers": "puffer",
    "puffier": "puffy",
    "puffiest": "puffy",
    "puffing": "puff",
    "puffins": "puffin",
    "puffs": "puff",
    "pugilists": "pugilist",
    "pugs": "pug",
    "puked": "puke",
    "pukes": "puke",
    "puking": "puke",
    "pull-backs": "pull-back",
    "pull-ins": "pull-in",
    "pull-offs": "pull-off",
    "pull-outs": "pull-out",
    "pull-ups": "pull-up",
    "pullbacks": "pullback",
    "pulldowns": "pulldown",
    "pulled": "pull",
    "pullers": "puller",
    "pullets": "pullet",
    "pulleys": "pulley",
    "pulling": "pull",
    "pullovers": "pullover",
    "pulls": "pull",
    "pulped": "pulp",
    "pulpier": "pulpy",
    "pulping": "pulp",
    "pulpits": "pulpit",
    "pulps": "pulp",
    "pulsars": "pulsar",
    "pulsated": "pulsate",
    "pulsates": "pulsate",
    "pulsating": "pulsate",
    "pulsations": "pulsation",
    "pulsed": "pulse",
    "pulses": "pulse",
    "pulsing": "pulse",
    "pulverised": "pulverise",
    "pulverises": "pulverise",
    "pulverising": "pulverise",
    "pulverized": "pulverize",
    "pulverizes": "pulverize",
    "pulverizing": "pulverize",
    "pumas": "puma",
    "pummelled": "pummel",
    "pummelling": "pummel",
    "pummels": "pummel",
    "pumped": "pump",
    "pumping": "pump",
    "pumpkins": "pumpkin",
    "pumps": "pump",
    "punch-lines": "punch-line",
    "punch-ups": "punch-up",
    "punchbags": "punchbag",
    "punchballs": "punchball",
    "punchbowls": "punchbowl",
    "punched": "punch",
    "punchers": "puncher",
    "punches": "punch",
    "punchier": "punchy",
    "punchiest": "punchy",
    "punching": "punch",
    "puncta": "punctum",
    "punctuated": "punctuate",
    "punctuates": "punctuate",
    "punctuating": "punctuate",
    "punctured": "puncture",
    "punctures": "puncture",
    "puncturing": "puncture",
    "pundits": "pundit",
    "pungencies": "pungency",
    "punier": "puny",
    "puniest": "puny",
    "punished": "punish",
    "punishes": "punish",
    "punishing": "punish",
    "punishments": "punishment",
    "punks": "punk",
    "punned": "pun",
    "punnets": "punnet",
    "punning": "pun",
    "puns": "pun",
    "punted": "punt",
    "punters": "punter",
    "punting": "punt",
    "punts": "punt",
    "pupae": "pupa",
    "pupas": "pupa",
    "pupated": "pupate",
    "pupates": "pupate",
    "pupating": "pupate",
    "pupations": "pupation",
    "pupils": "pupil",
    "pupped": "pup",
    "puppeteers": "puppeteer",
    "puppets": "puppet",
    "puppies": "puppy",
    "pupping": "pup",
    "pups": "pup",
    "pura": "pus",
    "purchased": "purchase",
    "purchasers": "purchaser",
    "purchases": "purchase",
    "purchasing": "purchase",
    "pureed": "puree",
    "pureeing": "puree",
    "purees": "puree",
    "purer": "pure",
    "purest": "pure",
    "purgatives": "purgative",
    "purgatories": "purgatory",
    "purged": "purge",
    "purges": "purge",
    "purging": "purge",
    "purifications": "purification",
    "purified": "purify",
    "purifiers": "purifier",
    "purifies": "purify",
    "purifying": "purify",
    "purines": "purine",
    "purists": "purist",
    "puritans": "puritan",
    "purities": "purity",
    "purled": "purl",
    "purling": "purl",
    "purloined": "purloin",
    "purloining": "purloin",
    "purloins": "purloin",
    "purls": "purl",
    "purpler": "purple",
    "purples": "purple",
    "purplest": "purple",
    "purported": "purport",
    "purporting": "purport",
    "purports": "purport",
    "purposed": "purpose",
    "purposes": "purpose",
    "purposing": "purpose",
    "purpurae": "purpura",
    "purpuras": "purpura",
    "purred": "purr",
    "purring": "purr",
    "purrs": "purr",
    "pursed": "purse",
    "pursers": "purser",
    "purses": "purse",
    "pursing": "purse",
    "pursued": "pursue",
    "pursuers": "pursuer",
    "pursues": "pursue",
    "pursuing": "pursue",
    "pursuits": "pursuit",
    "purveyed": "purvey",
    "purveying": "purvey",
    "purveyors": "purveyor",
    "purveys": "purvey",
    "purées": "purée",
    "push-pulls": "push-pull",
    "push-ups": "push-up",
    "pushbuttons": "pushbutton",
    "pushcarts": "pushcart",
    "pushchairs": "pushchair",
    "pushed": "push",
    "pushers": "pusher",
    "pushes": "push",
    "pushier": "pushy",
    "pushiest": "pushy",
    "pushing": "push",
    "pushovers": "pushover",
    "pusses": "puss",
    "pussies": "pussy",
    "pussy-cats": "pussy-cat",
    "pussy-willows": "pussy-willow",
    "pussyfooted": "pussyfoot",
    "pussyfooting": "pussyfoot",
    "pussyfoots": "pussyfoot",
    "pustules": "pustule",
    "put-downs": "put-down",
    "putamens": "putamen",
    "putamina": "putamen",
    "putrefied": "putrefy",
    "putrefies": "putrefy",
    "putrefying": "putrefy",
    "puts": "put",
    "putsches": "putsch",
    "putted": "putt",
    "puttered": "putter",
    "puttering": "putter",
    "putters": "putter",
    "putties": "putty",
    "putting": "putt",
    "putts": "putt",
    "puzzled": "puzzle",
    "puzzlements": "puzzlement",
    "puzzlers": "puzzler",
    "puzzles": "puzzle",
    "puzzling": "puzzle",
    "pvs": "pv",
    "pyelonephritides": "pyelonephritis",
    "pygmies": "pygmy",
    "pyjamas": "pyjama",
    "pylons": "pylon",
    "pylori": "pylorus",
    "pyramids": "pyramid",
    "pyres": "pyre",
    "pyrethroids": "pyrethroid",
    "pyrexiae": "pyrexia",
    "pyrexias": "pyrexia",
    "pyridines": "pyridine",
    "pyrimidines": "pyrimidine",
    "pyrolyses": "pyrolysis",
    "pyrophosphates": "pyrophosphate",
    "pyroxenes": "pyroxene",
    "pyrroles": "pyrrole",
    "pyruvates": "pyruvate",
    "pythons": "python",
    "pâtés": "pâté",
    "qqv": "qv",
    "quacked": "quack",
    "quackeries": "quackery",
    "quacking": "quack",
    "quacks": "quack",
    "quadrangles": "quadrangle",
    "quadrants": "quadrant",
    "quadratics": "quadratic",
    "quadrats": "quadrat",
    "quadratures": "quadrature",
    "quadrilaterals": "quadrilateral",
    "quadrilles": "quadrille",
    "quadrupeds": "quadruped",
    "quadrupled": "quadruple",
    "quadruples": "quadruple",
    "quadruplets": "quadruplet",
    "quadrupling": "quadruple",
    "quadrupoles": "quadrupole",
    "quads": "quad",
    "quaffed": "quaff",
    "quaffing": "quaff",
    "quaffs": "quaff",
    "quagmires": "quagmire",
    "quailed": "quail",
    "quailing": "quail",
    "quails": "quail",
    "quainter": "quaint",
    "quaintest": "quaint",
    "quaked": "quake",
    "quakes": "quake",
    "quaking": "quake",
    "qualia": "quale",
    "qualifications": "qualification",
    "qualified": "qualify",
    "qualifiers": "qualifier",
    "qualifies": "qualify",
    "qualifying": "qualify",
    "qualities": "quality",
    "qualms": "qualm",
    "quandaries": "quandary",
    "quanta": "quantum",
    "quantifications": "quantification",
    "quantified": "quantify",
    "quantifiers": "quantifier",
    "quantifies": "quantify",
    "quantifying": "quantify",
    "quantisations": "quantisation",
    "quantised": "quantise",
    "quantises": "quantise",
    "quantising": "quantise",
    "quantitations": "quantitation",
    "quantities": "quantity",
    "quantizations": "quantization",
    "quantized": "quantize",
    "quantizes": "quantize",
    "quantizing": "quantize",
    "quants": "quant",
    "quarantined": "quarantine",
    "quarantines": "quarantine",
    "quarantining": "quarantine",
    "quarks": "quark",
    "quarreled": "quarrel",
    "quarreling": "quarrel",
    "quarrelled": "quarrel",
    "quarrelling": "quarrel",
    "quarrels": "quarrel",
    "quarried": "quarry",
    "quarries": "quarry",
    "quarrying": "quarry",
    "quarrymen": "quarryman",
    "quarter-decks": "quarter-deck",
    "quarterbacks": "quarterback",
    "quarterdecks": "quarterdeck",
    "quartered": "quarter",
    "quarterfinals": "quarterfinal",
    "quartering": "quarter",
    "quarterlies": "quarterly",
    "quartermasters": "quartermaster",
    "quarters": "quarter",
    "quartets": "quartet",
    "quartettes": "quartette",
    "quartiles": "quartile",
    "quartos": "quarto",
    "quarts": "quart",
    "quartzes": "quartz",
    "quartzites": "quartzite",
    "quasars": "quasar",
    "quashed": "quash",
    "quashes": "quash",
    "quashing": "quash",
    "quasi-markets": "quasi-market",
    "quaternions": "quaternion",
    "quatrains": "quatrain",
    "quatrefoils": "quatrefoil",
    "quavered": "quaver",
    "quavering": "quaver",
    "quavers": "quaver",
    "quays": "quay",
    "qubits": "qubit",
    "queasier": "queasy",
    "queasiest": "queasy",
    "queened": "queen",
    "queening": "queen",
    "queenlier": "queenly",
    "queens": "queen",
    "queered": "queer",
    "queerer": "queer",
    "queeres": "queer",
    "queerest": "queer",
    "queering": "queer",
    "queers": "queer",
    "quelled": "quell",
    "quelling": "quell",
    "quells": "quell",
    "quenched": "quench",
    "quenches": "quench",
    "quenching": "quench",
    "queried": "query",
    "queries": "query",
    "querying": "query",
    "quested": "quest",
    "questing": "quest",
    "question-masters": "question-master",
    "questionaires": "questionaire",
    "questioned": "question",
    "questioners": "questioner",
    "questioning": "question",
    "questionnaires": "questionnaire",
    "questions": "question",
    "quests": "quest",
    "queued": "queue",
    "queueing": "queue",
    "queues": "queue",
    "queuing": "queue",
    "quibbled": "quibble",
    "quibbles": "quibble",
    "quibbling": "quibble",
    "quiches": "quiche",
    "quick-freezes": "quick-freeze",
    "quick-freezing": "quick-freeze",
    "quick-froze": "quick-freeze",
    "quick-frozen": "quick-freeze",
    "quickened": "quicken",
    "quickening": "quicken",
    "quickens": "quicken",
    "quicker": "quick",
    "quickest": "quick",
    "quickies": "quickie",
    "quicksands": "quicksand",
    "quicksteps": "quickstep",
    "quids": "quid",
    "quieted": "quiet",
    "quietened": "quieten",
    "quietening": "quieten",
    "quietens": "quieten",
    "quieter": "quiet",
    "quietest": "quiet",
    "quieting": "quiet",
    "quiets": "quiet",
    "quills": "quill",
    "quilted": "quilt",
    "quilting": "quilt",
    "quilts": "quilt",
    "quincentenaries": "quincentenary",
    "quinces": "quince",
    "quincunxes": "quincunx",
    "quinolones": "quinolone",
    "quinones": "quinone",
    "quins": "quin",
    "quintessences": "quintessence",
    "quintets": "quintet",
    "quintettes": "quintette",
    "quintiles": "quintile",
    "quints": "quint",
    "quintuplets": "quintuplet",
    "quipped": "quip",
    "quipping": "quip",
    "quips": "quip",
    "quirkier": "quirky",
    "quirkiest": "quirky",
    "quirks": "quirk",
    "quirts": "quirt",
    "quises": "quis",
    "quislings": "quisling",
    "quits": "quit",
    "quitted": "quit",
    "quitters": "quitter",
    "quitting": "quit",
    "quivered": "quiver",
    "quivering": "quiver",
    "quivers": "quiver",
    "quizmasters": "quizmaster",
    "quizzed": "quiz",
    "quizzes": "quiz",
    "quizzing": "quiz",
    "quoits": "quoit",
    "quorums": "quorum",
    "quotas": "quota",
    "quotations": "quotation",
    "quoted": "quote",
    "quotes": "quote",
    "quotients": "quotient",
    "quoting": "quote",
    "rabbis": "rabbi",
    "rabbited": "rabbit",
    "rabbiting": "rabbit",
    "rabbits": "rabbit",
    "rabbitted": "rabbit",
    "rabbitting": "rabbit",
    "rabble-rousers": "rabble-rouser",
    "rabbles": "rabble",
    "raccoons": "raccoon",
    "racecourses": "racecourse",
    "raced": "race",
    "racehorses": "racehorse",
    "racemes": "raceme",
    "racers": "racer",
    "races": "race",
    "racetracks": "racetrack",
    "racialists": "racialist",
    "racier": "racy",
    "raciest": "racy",
    "racing": "race",
    "racings": "racing",
    "racisms": "racism",
    "racists": "racist",
    "racked": "rack",
    "racketed": "racket",
    "racketeers": "racketeer",
    "racketing": "racket",
    "rackets": "racket",
    "racking": "rack",
    "racks": "rack",
    "raconteurs": "raconteur",
    "racoons": "racoon",
    "racquets": "racquet",
    "radars": "radar",
    "radials": "radial",
    "radiances": "radiance",
    "radians": "radian",
    "radiated": "radiate",
    "radiates": "radiate",
    "radiating": "radiate",
    "radiations": "radiation",
    "radiators": "radiator",
    "radicalisations": "radicalisation",
    "radicalised": "radicalise",
    "radicalises": "radicalise",
    "radicalising": "radicalise",
    "radicalizations": "radicalization",
    "radicalized": "radicalize",
    "radicalizes": "radicalize",
    "radicalizing": "radicalize",
    "radicals": "radical",
    "radices": "radix",
    "radicles": "radicle",
    "radii": "radius",
    "radio-frequencies": "radio-frequency",
    "radio-telephones": "radio-telephone",
    "radioactivities": "radioactivity",
    "radiocarbons": "radiocarbon",
    "radioed": "radio",
    "radiofrequencies": "radiofrequency",
    "radiograms": "radiogram",
    "radiographers": "radiographer",
    "radiographies": "radiography",
    "radiographs": "radiograph",
    "radioing": "radio",
    "radioisotopes": "radioisotope",
    "radiologies": "radiology",
    "radiologists": "radiologist",
    "radiometers": "radiometer",
    "radionuclides": "radionuclide",
    "radiopharmaceuticals": "radiopharmaceutical",
    "radios": "radio",
    "radiosondes": "radiosonde",
    "radiosurgeries": "radiosurgery",
    "radiotelephones": "radiotelephone",
    "radiotherapies": "radiotherapy",
    "radiotherapists": "radiotherapist",
    "radiowaves": "radiowave",
    "radishes": "radish",
    "radiuses": "radius",
    "radixes": "radix",
    "rads": "rad",
    "raffled": "raffle",
    "raffles": "raffle",
    "raffling": "raffle",
    "rafted": "raft",
    "rafters": "rafter",
    "rafting": "raft",
    "rafts": "raft",
    "ragamuffins": "ragamuffin",
    "ragas": "raga",
    "ragbags": "ragbag",
    "raged": "rage",
    "rages": "rage",
    "ragged": "rag",
    "ragging": "rag",
    "raging": "rage",
    "ragouts": "ragout",
    "rags": "rag",
    "ragworts": "ragwort",
    "raided": "raid",
    "raiders": "raider",
    "raiding": "raid",
    "raids": "raid",
    "railcards": "railcard",
    "railcars": "railcar",
    "railed": "rail",
    "railing": "rail",
    "railings": "railing",
    "railroaded": "railroad",
    "railroading": "railroad",
    "railroads": "railroad",
    "rails": "rail",
    "railwaymen": "railwayman",
    "railways": "railway",
    "rainbows": "rainbow",
    "raincoats": "raincoat",
    "raindrops": "raindrop",
    "rained": "rain",
    "rainfalls": "rainfall",
    "rainforests": "rainforest",
    "rainier": "rainy",
    "rainiest": "rainy",
    "raining": "rain",
    "rainmakers": "rainmaker",
    "rains": "rain",
    "rainstorms": "rainstorm",
    "raised": "raise",
    "raisers": "raiser",
    "raises": "raise",
    "raising": "raise",
    "raisins": "raisin",
    "rajahs": "rajah",
    "rajas": "raja",
    "rajes": "raj",
    "raked": "rake",
    "rakes": "rake",
    "raking": "rake",
    "rallied": "rally",
    "rallies": "rally",
    "rallying": "rally",
    "rambled": "ramble",
    "ramblers": "rambler",
    "rambles": "ramble",
    "rambling": "ramble",
    "ramifications": "ramification",
    "ramified": "ramify",
    "ramifies": "ramify",
    "ramifying": "ramify",
    "rammed": "ram",
    "ramming": "ram",
    "rampaged": "rampage",
    "rampages": "rampage",
    "rampaging": "rampage",
    "ramparts": "rampart",
    "ramped": "ramp",
    "ramping": "ramp",
    "ramps": "ramp",
    "ramrods": "ramrod",
    "rams": "ram",
    "ran": "run",
    "ranched": "ranch",
    "ranchers": "rancher",
    "ranches": "ranch",
    "ranching": "ranch",
    "randier": "randy",
    "randiest": "randy",
    "randomisations": "randomisation",
    "randomised": "randomise",
    "randomises": "randomise",
    "randomising": "randomise",
    "randomizations": "randomization",
    "randomized": "randomize",
    "randomizes": "randomize",
    "randomizing": "randomize",
    "rands": "rand",
    "rang": "ring",
    "ranged": "range",
    "rangefinders": "rangefinder",
    "rangelands": "rangeland",
    "rangers": "ranger",
    "ranges": "range",
    "ranging": "range",
    "ranked": "rank",
    "ranker": "rank",
    "rankest": "rank",
    "ranking": "rank",
    "rankings": "ranking",
    "rankled": "rankle",
    "rankles": "rankle",
    "rankling": "rankle",
    "ranks": "rank",
    "ransacked": "ransack",
    "ransacking": "ransack",
    "ransacks": "ransack",
    "ransomed": "ransom",
    "ransoming": "ransom",
    "ransoms": "ransom",
    "ranted": "rant",
    "ranting": "rant",
    "rantings": "ranting",
    "rants": "rant",
    "raped": "rape",
    "rapes": "rape",
    "rapeseeds": "rapeseed",
    "rapider": "rapid",
    "rapidest": "rapid",
    "rapidities": "rapidity",
    "rapids": "rapid",
    "rapiers": "rapier",
    "raping": "rape",
    "rapings": "raping",
    "rapists": "rapist",
    "rapped": "rap",
    "rapping": "rap",
    "rapports": "rapport",
    "rapprochements": "rapprochement",
    "raps": "rap",
    "raptors": "raptor",
    "raptures": "rapture",
    "rarebits": "rarebit",
    "rarefied": "rarefy",
    "rarefies": "rarefy",
    "rarefying": "rarefy",
    "rarer": "rare",
    "rarest": "rare",
    "rarified": "rarify",
    "rarifies": "rarify",
    "rarifying": "rarify",
    "rarities": "rarity",
    "rascals": "rascal",
    "rased": "rase",
    "rases": "rase",
    "rasher": "rash",
    "rashers": "rasher",
    "rashes": "rash",
    "rashest": "rash",
    "rasing": "rase",
    "raspberries": "raspberry",
    "rasped": "rasp",
    "rasping": "rasp",
    "raspingly": "rasping",
    "rasps": "rasp",
    "rasters": "raster",
    "ratas": "rata",
    "ratcheted": "ratchet",
    "ratcheting": "ratchet",
    "ratchets": "ratchet",
    "ratchetted": "ratchet",
    "ratchetting": "ratchet",
    "rated": "rate",
    "ratepayers": "ratepayer",
    "raters": "rater",
    "rates": "rate",
    "ratholes": "rathole",
    "ratifications": "ratification",
    "ratified": "ratify",
    "ratifies": "ratify",
    "ratifying": "ratify",
    "rating": "rate",
    "ratings": "rating",
    "rationales": "rationale",
    "rationalisations": "rationalisation",
    "rationalised": "rationalise",
    "rationalises": "rationalise",
    "rationalising": "rationalise",
    "rationalists": "rationalist",
    "rationalities": "rationality",
    "rationalizations": "rationalization",
    "rationalized": "rationalize",
    "rationalizes": "rationalize",
    "rationalizing": "rationalize",
    "rationals": "rational",
    "rationed": "ration",
    "rationing": "ration",
    "rations": "ration",
    "ratios": "ratio",
    "rats": "rat",
    "rattans": "rattan",
    "ratted": "rat",
    "ratters": "ratter",
    "rattier": "ratty",
    "rattiest": "ratty",
    "ratting": "rat",
    "rattled": "rattle",
    "rattlers": "rattler",
    "rattles": "rattle",
    "rattlesnakes": "rattlesnake",
    "rattling": "rattle",
    "raunchier": "raunchy",
    "raunchiest": "raunchy",
    "ravaged": "ravage",
    "ravages": "ravage",
    "ravaging": "ravage",
    "rave-ups": "rave-up",
    "raved": "rave",
    "raveled": "ravel",
    "raveling": "ravel",
    "ravelled": "ravel",
    "ravelling": "ravel",
    "ravels": "ravel",
    "ravens": "raven",
    "ravers": "raver",
    "raves": "rave",
    "ravines": "ravine",
    "raving": "rave",
    "ravings": "raving",
    "ravished": "ravish",
    "ravishes": "ravish",
    "ravishing": "ravish",
    "rawer": "raw",
    "rawest": "raw",
    "rawhides": "rawhide",
    "rayons": "rayon",
    "rays": "ray",
    "razed": "raze",
    "razes": "raze",
    "razing": "raze",
    "razorbills": "razorbill",
    "razors": "razor",
    "rbs": "rb",
    "re-accreditations": "re-accreditation",
    "re-acquired": "re-acquire",
    "re-acquires": "re-acquire",
    "re-acquiring": "re-acquire",
    "re-activated": "re-activate",
    "re-activates": "re-activate",
    "re-activating": "re-activate",
    "re-activations": "re-activation",
    "re-addressed": "re-address",
    "re-addresses": "re-address",
    "re-addressing": "re-address",
    "re-adjusted": "re-adjust",
    "re-adjusting": "re-adjust",
    "re-adjusts": "re-adjust",
    "re-admissions": "re-admission",
    "re-admits": "re-admit",
    "re-admitted": "re-admit",
    "re-admitting": "re-admit",
    "re-affirmed": "re-affirm",
    "re-affirming": "re-affirm",
    "re-affirms": "re-affirm",
    "re-aligned": "re-align",
    "re-aligning": "re-align",
    "re-alignments": "re-alignment",
    "re-aligns": "re-align",
    "re-allocated": "re-allocate",
    "re-allocates": "re-allocate",
    "re-allocating": "re-allocate",
    "re-analysed": "re-analyse",
    "re-analyses": "re-analysis",
    "re-analysing": "re-analyse",
    "re-analysises": "re-analysis",
    "re-appearances": "re-appearance",
    "re-appeared": "re-appear",
    "re-appearing": "re-appear",
    "re-appears": "re-appear",
    "re-applications": "re-application",
    "re-applied": "re-apply",
    "re-applies": "re-apply",
    "re-applying": "re-apply",
    "re-appraisals": "re-appraisal",
    "re-appraised": "re-appraise",
    "re-appraises": "re-appraise",
    "re-appraising": "re-appraise",
    "re-arranged": "re-arrange",
    "re-arrangements": "re-arrangement",
    "re-arranges": "re-arrange",
    "re-arranging": "re-arrange",
    "re-arrested": "re-arrest",
    "re-arresting": "re-arrest",
    "re-arrests": "re-arrest",
    "re-assembled": "re-assemble",
    "re-assembles": "re-assemble",
    "re-assemblies": "re-assembly",
    "re-assembling": "re-assemble",
    "re-assessed": "re-assess",
    "re-assesses": "re-assess",
    "re-assessing": "re-assess",
    "re-assessments": "re-assessment",
    "re-assurances": "re-assurance",
    "re-attached": "re-attach",
    "re-attaches": "re-attach",
    "re-attaching": "re-attach",
    "re-awakened": "re-awaken",
    "re-awakening": "re-awaken",
    "re-awakens": "re-awaken",
    "re-balanced": "re-balance",
    "re-balances": "re-balance",
    "re-balancing": "re-balance",
    "re-births": "re-birth",
    "re-building": "re-build",
    "re-builds": "re-build",
    "re-built": "re-build",
    "re-calculated": "re-calculate",
    "re-calculates": "re-calculate",
    "re-calculating": "re-calculate",
    "re-captured": "re-capture",
    "re-captures": "re-capture",
    "re-capturing": "re-capture",
    "re-certifications": "re-certification",
    "re-checked": "re-check",
    "re-checking": "re-check",
    "re-checks": "re-check",
    "re-circulated": "re-circulate",
    "re-circulates": "re-circulate",
    "re-circulating": "re-circulate",
    "re-classifications": "re-classification",
    "re-classified": "re-classify",
    "re-classifies": "re-classify",
    "re-classifying": "re-classify",
    "re-colonised": "re-colonise",
    "re-colonises": "re-colonise",
    "re-colonising": "re-colonise",
    "re-configurations": "re-configuration",
    "re-confirmed": "re-confirm",
    "re-confirming": "re-confirm",
    "re-confirms": "re-confirm",
    "re-considered": "re-consider",
    "re-considering": "re-consider",
    "re-considers": "re-consider",
    "re-constructed": "re-construct",
    "re-constructing": "re-construct",
    "re-constructions": "re-construction",
    "re-constructs": "re-construct",
    "re-counted": "re-count",
    "re-counting": "re-count",
    "re-counts": "re-count",
    "re-covered": "re-cover",
    "re-covering": "re-cover",
    "re-covers": "re-cover",
    "re-created": "re-create",
    "re-creates": "re-create",
    "re-creating": "re-create",
    "re-cycled": "re-cycle",
    "re-cycles": "re-cycle",
    "re-cycling": "re-cycle",
    "re-cyclings": "re-cycling",
    "re-decorated": "re-decorate",
    "re-decorates": "re-decorate",
    "re-decorating": "re-decorate",
    "re-defined": "re-define",
    "re-defines": "re-define",
    "re-defining": "re-define",
    "re-definitions": "re-definition",
    "re-deployments": "re-deployment",
    "re-designed": "re-design",
    "re-designing": "re-design",
    "re-designs": "re-design",
    "re-did": "re-do",
    "re-directed": "re-direct",
    "re-directing": "re-direct",
    "re-directions": "re-direction",
    "re-directs": "re-direct",
    "re-discoveries": "re-discovery",
    "re-distributions": "re-distribution",
    "re-does": "re-do",
    "re-doing": "re-do",
    "re-done": "re-do",
    "re-echoed": "re-echo",
    "re-echoes": "re-echo",
    "re-echoing": "re-echo",
    "re-edited": "re-edit",
    "re-editing": "re-edit",
    "re-edits": "re-edit",
    "re-educated": "re-educate",
    "re-educates": "re-educate",
    "re-educating": "re-educate",
    "re-elected": "re-elect",
    "re-electing": "re-elect",
    "re-elections": "re-election",
    "re-elects": "re-elect",
    "re-emerged": "re-emerge",
    "re-emerges": "re-emerge",
    "re-emerging": "re-emerge",
    "re-emphasised": "re-emphasise",
    "re-emphasises": "re-emphasise",
    "re-emphasising": "re-emphasise",
    "re-employments": "re-employment",
    "re-enacted": "re-enact",
    "re-enacting": "re-enact",
    "re-enactments": "re-enactment",
    "re-enacts": "re-enact",
    "re-energised": "re-energise",
    "re-energises": "re-energise",
    "re-energising": "re-energise",
    "re-enforced": "re-enforce",
    "re-enforcements": "re-enforcement",
    "re-enforces": "re-enforce",
    "re-enforcing": "re-enforce",
    "re-engaged": "re-engage",
    "re-engagements": "re-engagement",
    "re-engages": "re-engage",
    "re-engaging": "re-engage",
    "re-engineered": "re-engineer",
    "re-engineering": "re-engineer",
    "re-engineers": "re-engineer",
    "re-entered": "re-enter",
    "re-entering": "re-enter",
    "re-enters": "re-enter",
    "re-entries": "re-entry",
    "re-equipped": "re-equip",
    "re-equipping": "re-equip",
    "re-equips": "re-equip",
    "re-erected": "re-erect",
    "re-erecting": "re-erect",
    "re-erections": "re-erection",
    "re-erects": "re-erect",
    "re-established": "re-establish",
    "re-establishes": "re-establish",
    "re-establishing": "re-establish",
    "re-evaluated": "re-evaluate",
    "re-evaluates": "re-evaluate",
    "re-evaluating": "re-evaluate",
    "re-evaluations": "re-evaluation",
    "re-examinations": "re-examination",
    "re-examined": "re-examine",
    "re-examines": "re-examine",
    "re-examining": "re-examine",
    "re-exported": "re-export",
    "re-exporting": "re-export",
    "re-exports": "re-export",
    "re-filled": "re-fill",
    "re-filling": "re-fill",
    "re-fills": "re-fill",
    "re-focused": "re-focus",
    "re-focuses": "re-focus",
    "re-focusing": "re-focus",
    "re-focussed": "re-focus",
    "re-focussing": "re-focus",
    "re-formed": "re-form",
    "re-forming": "re-form",
    "re-forms": "re-form",
    "re-gained": "re-gain",
    "re-gaining": "re-gain",
    "re-gains": "re-gain",
    "re-generations": "re-generation",
    "re-grouped": "re-group",
    "re-grouping": "re-group",
    "re-groups": "re-group",
    "re-growed": "re-grow",
    "re-growing": "re-grow",
    "re-grows": "re-grow",
    "re-growths": "re-growth",
    "re-infections": "re-infection",
    "re-inforced": "re-inforce",
    "re-inforces": "re-inforce",
    "re-inforcing": "re-inforce",
    "re-inspections": "re-inspection",
    "re-installed": "re-install",
    "re-installing": "re-install",
    "re-installs": "re-install",
    "re-instated": "re-instate",
    "re-instatements": "re-instatement",
    "re-instates": "re-instate",
    "re-instating": "re-instate",
    "re-integrated": "re-integrate",
    "re-integrates": "re-integrate",
    "re-integrating": "re-integrate",
    "re-integrations": "re-integration",
    "re-interpretations": "re-interpretation",
    "re-interpreted": "re-interpret",
    "re-interpreting": "re-interpret",
    "re-interprets": "re-interpret",
    "re-interviewed": "re-interview",
    "re-interviewing": "re-interview",
    "re-interviews": "re-interview",
    "re-introduced": "re-introduce",
    "re-introduces": "re-introduce",
    "re-introducing": "re-introduce",
    "re-introductions": "re-introduction",
    "re-invented": "re-invent",
    "re-inventing": "re-invent",
    "re-inventions": "re-invention",
    "re-invents": "re-invent",
    "re-joined": "re-join",
    "re-joining": "re-join",
    "re-joins": "re-join",
    "re-launched": "re-launch",
    "re-launches": "re-launch",
    "re-launching": "re-launch",
    "re-learned": "re-learn",
    "re-learning": "re-learn",
    "re-learns": "re-learn",
    "re-learnt": "re-learn",
    "re-located": "re-locate",
    "re-locates": "re-locate",
    "re-locating": "re-locate",
    "re-locations": "re-location",
    "re-measurements": "re-measurement",
    "re-modeled": "re-model",
    "re-modeling": "re-model",
    "re-modelled": "re-model",
    "re-modelling": "re-model",
    "re-modellings": "re-modelling",
    "re-models": "re-model",
    "re-occurrences": "re-occurrence",
    "re-offended": "re-offend",
    "re-offending": "re-offend",
    "re-offends": "re-offend",
    "re-opened": "re-open",
    "re-opening": "re-open",
    "re-openings": "re-opening",
    "re-opens": "re-open",
    "re-ordered": "re-order",
    "re-ordering": "re-order",
    "re-orderings": "re-ordering",
    "re-orders": "re-order",
    "re-organisations": "re-organisation",
    "re-organised": "re-organise",
    "re-organises": "re-organise",
    "re-organising": "re-organise",
    "re-organized": "re-organize",
    "re-organizes": "re-organize",
    "re-organizing": "re-organize",
    "re-orientated": "re-orientate",
    "re-orientates": "re-orientate",
    "re-orientating": "re-orientate",
    "re-orientations": "re-orientation",
    "re-oriented": "re-orient",
    "re-orienting": "re-orient",
    "re-orients": "re-orient",
    "re-positioned": "re-position",
    "re-positioning": "re-position",
    "re-positions": "re-position",
    "re-presentations": "re-presentation",
    "re-presented": "re-present",
    "re-presenting": "re-present",
    "re-presents": "re-present",
    "re-programmed": "re-programme",
    "re-programmes": "re-programme",
    "re-programming": "re-programme",
    "re-reading": "re-read",
    "re-readings": "re-reading",
    "re-reads": "re-read",
    "re-recorded": "re-record",
    "re-recording": "re-record",
    "re-records": "re-record",
    "re-red": "re-read",
    "re-registered": "re-register",
    "re-registering": "re-register",
    "re-registers": "re-register",
    "re-registrations": "re-registration",
    "re-released": "re-release",
    "re-releases": "re-release",
    "re-releasing": "re-release",
    "re-roofed": "re-roof",
    "re-roofing": "re-roof",
    "re-roofs": "re-roof",
    "re-routed": "re-route",
    "re-routes": "re-route",
    "re-routing": "re-route",
    "re-runs": "re-run",
    "re-scheduled": "re-schedule",
    "re-schedules": "re-schedule",
    "re-scheduling": "re-schedule",
    "re-selected": "re-select",
    "re-selecting": "re-select",
    "re-selects": "re-select",
    "re-sending": "re-send",
    "re-sends": "re-send",
    "re-sent": "re-send",
    "re-shaped": "re-shape",
    "re-shapes": "re-shape",
    "re-shaping": "re-shape",
    "re-sited": "re-site",
    "re-sites": "re-site",
    "re-siting": "re-site",
    "re-started": "re-start",
    "re-starting": "re-start",
    "re-starts": "re-start",
    "re-stated": "re-state",
    "re-states": "re-state",
    "re-stating": "re-state",
    "re-structured": "re-structure",
    "re-structures": "re-structure",
    "re-structuring": "re-structure",
    "re-structurings": "re-structuring",
    "re-submissions": "re-submission",
    "re-supplies": "re-supply",
    "re-surveyed": "re-survey",
    "re-surveying": "re-survey",
    "re-surveys": "re-survey",
    "re-tested": "re-test",
    "re-testing": "re-test",
    "re-tests": "re-test",
    "re-thinking": "re-think",
    "re-thinks": "re-think",
    "re-thought": "re-think",
    "re-trained": "re-train",
    "re-training": "re-train",
    "re-trains": "re-train",
    "re-typed": "re-type",
    "re-types": "re-type",
    "re-typing": "re-type",
    "re-used": "re-use",
    "re-uses": "re-use",
    "re-using": "re-use",
    "re-validations": "re-validation",
    "re-visited": "re-visit",
    "re-visiting": "re-visit",
    "re-visits": "re-visit",
    "re-worked": "re-work",
    "re-working": "re-work",
    "re-works": "re-work",
    "re-writes": "re-write",
    "re-writing": "re-write",
    "re-written": "re-write",
    "re-wrote": "re-write",
    "reabsorbed": "reabsorb",
    "reabsorbing": "reabsorb",
    "reabsorbs": "reabsorb",
    "reabsorptions": "reabsorption",
    "reached": "reach",
    "reaches": "reach",
    "reaching": "reach",
    "reacquainted": "reacquaint",
    "reacquainting": "reacquaint",
    "reacquaints": "reacquaint",
    "reacquired": "reacquire",
    "reacquires": "reacquire",
    "reacquiring": "reacquire",
    "reactants": "reactant",
    "reacted": "react",
    "reacting": "react",
    "reactionaries": "reactionary",
    "reactions": "reaction",
    "reactivated": "reactivate",
    "reactivates": "reactivate",
    "reactivating": "reactivate",
    "reactivations": "reactivation",
    "reactivities": "reactivity",
    "reactors": "reactor",
    "reacts": "react",
    "readabilities": "readability",
    "readdressed": "readdress",
    "readdresses": "readdress",
    "readdressing": "readdress",
    "readers": "reader",
    "readerships": "readership",
    "readied": "ready",
    "readier": "ready",
    "readies": "ready",
    "readiest": "ready",
    "readinesses": "readiness",
    "reading": "read",
    "readings": "reading",
    "readjusted": "readjust",
    "readjusting": "readjust",
    "readjustments": "readjustment",
    "readjusts": "readjust",
    "readmissions": "readmission",
    "readmits": "readmit",
    "readmitted": "readmit",
    "readmitting": "readmit",
    "readouts": "readout",
    "reads": "read",
    "readying": "ready",
    "reae": "rea",
    "reaffirmations": "reaffirmation",
    "reaffirmed": "reaffirm",
    "reaffirming": "reaffirm",
    "reaffirms": "reaffirm",
    "reagents": "reagent",
    "realer": "real",
    "reales": "real",
    "realest": "real",
    "realigned": "realign",
    "realigning": "realign",
    "realignments": "realignment",
    "realigns": "realign",
    "realisations": "realisation",
    "realised": "realise",
    "realises": "realise",
    "realising": "realise",
    "realisms": "realism",
    "realists": "realist",
    "realities": "reality",
    "realizations": "realization",
    "realized": "realize",
    "realizes": "realize",
    "realizing": "realize",
    "reallocated": "reallocate",
    "reallocates": "reallocate",
    "reallocating": "reallocate",
    "reallocations": "reallocation",
    "realms": "realm",
    "reals": "real",
    "realtors": "realtor",
    "reamed": "ream",
    "reamers": "reamer",
    "reaming": "ream",
    "reams": "ream",
    "reanalyses": "reanalysis",
    "reanalysises": "reanalysis",
    "reanimated": "reanimate",
    "reanimates": "reanimate",
    "reanimating": "reanimate",
    "reaped": "reap",
    "reapers": "reaper",
    "reaping": "reap",
    "reappearances": "reappearance",
    "reappeared": "reappear",
    "reappearing": "reappear",
    "reappears": "reappear",
    "reapplications": "reapplication",
    "reapplied": "reapply",
    "reapplies": "reapply",
    "reapplying": "reapply",
    "reappointed": "reappoint",
    "reappointing": "reappoint",
    "reappointments": "reappointment",
    "reappoints": "reappoint",
    "reappraisals": "reappraisal",
    "reappraised": "reappraise",
    "reappraises": "reappraise",
    "reappraising": "reappraise",
    "reaps": "reap",
    "reared": "rear",
    "rearguards": "rearguard",
    "rearing": "rear",
    "rearings": "rearing",
    "rearmaments": "rearmament",
    "rearmed": "rearm",
    "rearming": "rearm",
    "rearms": "rearm",
    "rearranged": "rearrange",
    "rearrangements": "rearrangement",
    "rearranges": "rearrange",
    "rearranging": "rearrange",
    "rears": "rear",
    "reasoned": "reason",
    "reasoners": "reasoner",
    "reasoning": "reason",
    "reasons": "reason",
    "reassembled": "reassemble",
    "reassembles": "reassemble",
    "reassemblies": "reassembly",
    "reassembling": "reassemble",
    "reasserted": "reassert",
    "reasserting": "reassert",
    "reasserts": "reassert",
    "reassessed": "reassess",
    "reassesses": "reassess",
    "reassessing": "reassess",
    "reassessments": "reassessment",
    "reassigned": "reassign",
    "reassigning": "reassign",
    "reassignments": "reassignment",
    "reassigns": "reassign",
    "reassurances": "reassurance",
    "reassured": "reassure",
    "reassures": "reassure",
    "reassuring": "reassure",
    "reattached": "reattach",
    "reattaches": "reattach",
    "reattaching": "reattach",
    "reawakened": "reawaken",
    "reawakening": "reawaken",
    "reawakenings": "reawakening",
    "reawakens": "reawaken",
    "rebalanced": "rebalance",
    "rebalances": "rebalance",
    "rebalancing": "rebalance",
    "rebars": "rebar",
    "rebated": "rebate",
    "rebates": "rebate",
    "rebating": "rebate",
    "rebelled": "rebel",
    "rebelling": "rebel",
    "rebellions": "rebellion",
    "rebels": "rebel",
    "rebids": "rebid",
    "rebinding": "rebind",
    "rebinds": "rebind",
    "rebirths": "rebirth",
    "rebooked": "rebook",
    "rebooking": "rebook",
    "rebooks": "rebook",
    "rebooted": "reboot",
    "rebooting": "reboot",
    "reboots": "reboot",
    "rebound": "rebind",
    "rebounded": "rebound",
    "rebounding": "rebound",
    "rebounds": "rebound",
    "rebreathers": "rebreather",
    "rebroadcasted": "rebroadcast",
    "rebroadcasting": "rebroadcast",
    "rebroadcasts": "rebroadcast",
    "rebuffed": "rebuff",
    "rebuffing": "rebuff",
    "rebuffs": "rebuff",
    "rebuilded": "rebuild",
    "rebuilding": "rebuild",
    "rebuilds": "rebuild",
    "rebuilt": "rebuild",
    "rebuked": "rebuke",
    "rebukes": "rebuke",
    "rebuking": "rebuke",
    "reburials": "reburial",
    "reburied": "rebury",
    "reburies": "rebury",
    "reburying": "rebury",
    "rebuses": "rebus",
    "rebuts": "rebut",
    "rebuttals": "rebuttal",
    "rebutted": "rebut",
    "rebutting": "rebut",
    "recalculated": "recalculate",
    "recalculates": "recalculate",
    "recalculating": "recalculate",
    "recalculations": "recalculation",
    "recalibrated": "recalibrate",
    "recalibrates": "recalibrate",
    "recalibrating": "recalibrate",
    "recalibrations": "recalibration",
    "recalled": "recall",
    "recalling": "recall",
    "recalls": "recall",
    "recantations": "recantation",
    "recanted": "recant",
    "recanting": "recant",
    "recants": "recant",
    "recapitulated": "recapitulate",
    "recapitulates": "recapitulate",
    "recapitulating": "recapitulate",
    "recapitulations": "recapitulation",
    "recapped": "recap",
    "recapping": "recap",
    "recaps": "recap",
    "recaptured": "recapture",
    "recaptures": "recapture",
    "recapturing": "recapture",
    "recasting": "recast",
    "recasts": "recast",
    "receded": "recede",
    "recedes": "recede",
    "receding": "recede",
    "receipted": "receipt",
    "receipting": "receipt",
    "receipts": "receipt",
    "receivables": "receivable",
    "received": "receive",
    "receivers": "receiver",
    "receives": "receive",
    "receiving": "receive",
    "recencies": "recency",
    "recensions": "recension",
    "receptacles": "receptacle",
    "receptionists": "receptionist",
    "receptions": "reception",
    "receptivities": "receptivity",
    "receptors": "receptor",
    "recertifications": "recertification",
    "recessed": "recess",
    "recesses": "recess",
    "recessing": "recess",
    "recessions": "recession",
    "recessives": "recessive",
    "recharged": "recharge",
    "recharges": "recharge",
    "recharging": "recharge",
    "rechecked": "recheck",
    "rechecking": "recheck",
    "rechecks": "recheck",
    "rechristened": "rechristen",
    "rechristening": "rechristen",
    "rechristens": "rechristen",
    "recidivists": "recidivist",
    "recipes": "recipe",
    "recipients": "recipient",
    "reciprocals": "reciprocal",
    "reciprocated": "reciprocate",
    "reciprocates": "reciprocate",
    "reciprocating": "reciprocate",
    "reciprocities": "reciprocity",
    "recirculated": "recirculate",
    "recirculates": "recirculate",
    "recirculating": "recirculate",
    "recirculations": "recirculation",
    "recitals": "recital",
    "recitations": "recitation",
    "recitatives": "recitative",
    "recited": "recite",
    "recites": "recite",
    "reciting": "recite",
    "reckoned": "reckon",
    "reckoners": "reckoner",
    "reckoning": "reckon",
    "reckonings": "reckoning",
    "reckons": "reckon",
    "reclaimed": "reclaim",
    "reclaiming": "reclaim",
    "reclaims": "reclaim",
    "reclassifications": "reclassification",
    "reclassified": "reclassify",
    "reclassifies": "reclassify",
    "reclassifying": "reclassify",
    "reclined": "recline",
    "recliners": "recliner",
    "reclines": "recline",
    "reclining": "recline",
    "recluses": "recluse",
    "recoded": "recode",
    "recodes": "recode",
    "recoding": "recode",
    "recognised": "recognise",
    "recognisers": "recogniser",
    "recognises": "recognise",
    "recognising": "recognise",
    "recognitions": "recognition",
    "recognized": "recognize",
    "recognizers": "recognizer",
    "recognizes": "recognize",
    "recognizing": "recognize",
    "recoiled": "recoil",
    "recoiling": "recoil",
    "recoils": "recoil",
    "recollected": "recollect",
    "recollecting": "recollect",
    "recollections": "recollection",
    "recollects": "recollect",
    "recolonisations": "recolonisation",
    "recolonised": "recolonise",
    "recolonises": "recolonise",
    "recolonising": "recolonise",
    "recombinases": "recombinase",
    "recombinations": "recombination",
    "recombined": "recombine",
    "recombines": "recombine",
    "recombining": "recombine",
    "recommenced": "recommence",
    "recommences": "recommence",
    "recommencing": "recommence",
    "recommendations": "recommendation",
    "recommended": "recommend",
    "recommending": "recommend",
    "recommends": "recommend",
    "recommissioned": "recommission",
    "recommissioning": "recommission",
    "recommissions": "recommission",
    "recommits": "recommit",
    "recommitted": "recommit",
    "recommitting": "recommit",
    "recompensed": "recompense",
    "recompenses": "recompense",
    "recompensing": "recompense",
    "recompilations": "recompilation",
    "recompiled": "recompile",
    "recompiles": "recompile",
    "recompiling": "recompile",
    "recomposed": "recompose",
    "recomposes": "recompose",
    "recomposing": "recompose",
    "recompressed": "recompress",
    "recompresses": "recompress",
    "recompressing": "recompress",
    "recompressions": "recompression",
    "recomputed": "recompute",
    "recomputes": "recompute",
    "recomputing": "recompute",
    "reconceptualisations": "reconceptualisation",
    "reconceptualised": "reconceptualise",
    "reconceptualises": "reconceptualise",
    "reconceptualising": "reconceptualise",
    "reconcilabler": "reconcilable",
    "reconcilablest": "reconcilable",
    "reconciled": "reconcile",
    "reconciles": "reconcile",
    "reconciliations": "reconciliation",
    "reconciling": "reconcile",
    "reconditioned": "recondition",
    "reconditioning": "recondition",
    "reconditions": "recondition",
    "reconfigurations": "reconfiguration",
    "reconfigured": "reconfigure",
    "reconfigures": "reconfigure",
    "reconfiguring": "reconfigure",
    "reconfirmed": "reconfirm",
    "reconfirming": "reconfirm",
    "reconfirms": "reconfirm",
    "reconnaissances": "reconnaissance",
    "reconnected": "reconnect",
    "reconnecting": "reconnect",
    "reconnections": "reconnection",
    "reconnects": "reconnect",
    "reconnoitered": "reconnoiter",
    "reconnoitering": "reconnoiter",
    "reconnoiters": "reconnoiter",
    "reconnoitred": "reconnoitre",
    "reconnoitres": "reconnoitre",
    "reconnoitring": "reconnoitre",
    "reconquered": "reconquer",
    "reconquering": "reconquer",
    "reconquers": "reconquer",
    "reconquests": "reconquest",
    "recons": "recon",
    "reconsiderations": "reconsideration",
    "reconsidered": "reconsider",
    "reconsidering": "reconsider",
    "reconsiders": "reconsider",
    "reconstituted": "reconstitute",
    "reconstitutes": "reconstitute",
    "reconstituting": "reconstitute",
    "reconstitutions": "reconstitution",
    "reconstructed": "reconstruct",
    "reconstructing": "reconstruct",
    "reconstructions": "reconstruction",
    "reconstructs": "reconstruct",
    "reconvened": "reconvene",
    "reconvenes": "reconvene",
    "reconvening": "reconvene",
    "reconversions": "reconversion",
    "reconverted": "reconvert",
    "reconverting": "reconvert",
    "reconverts": "reconvert",
    "reconvicted": "reconvict",
    "reconvicting": "reconvict",
    "reconvictions": "reconviction",
    "reconvicts": "reconvict",
    "record-breakers": "record-breaker",
    "recorded": "record",
    "recorders": "recorder",
    "recording": "record",
    "recordings": "recording",
    "records": "record",
    "recounted": "recount",
    "recounting": "recount",
    "recounts": "recount",
    "recouped": "recoup",
    "recouping": "recoup",
    "recoups": "recoup",
    "recourses": "recourse",
    "recovered": "recover",
    "recoveries": "recovery",
    "recovering": "recover",
    "recovers": "recover",
    "recreated": "recreate",
    "recreates": "recreate",
    "recreating": "recreate",
    "recreations": "recreation",
    "recriminations": "recrimination",
    "recrossed": "recross",
    "recrosses": "recross",
    "recrossing": "recross",
    "recruited": "recruit",
    "recruiters": "recruiter",
    "recruiting": "recruit",
    "recruitments": "recruitment",
    "recruits": "recruit",
    "recrystallisations": "recrystallisation",
    "recs": "rec",
    "recta": "rectum",
    "rectangles": "rectangle",
    "recti": "rectus",
    "rectifications": "rectification",
    "rectified": "rectify",
    "rectifiers": "rectifier",
    "rectifies": "rectify",
    "rectifying": "rectify",
    "rectories": "rectory",
    "rectors": "rector",
    "rectos": "recto",
    "rects": "rect",
    "rectums": "rectum",
    "recuperated": "recuperate",
    "recuperates": "recuperate",
    "recuperating": "recuperate",
    "recuperations": "recuperation",
    "recurred": "recur",
    "recurrences": "recurrence",
    "recurring": "recur",
    "recurs": "recur",
    "recursions": "recursion",
    "recurved": "recurve",
    "recurves": "recurve",
    "recurving": "recurve",
    "recusants": "recusant",
    "recuts": "recut",
    "recutting": "recut",
    "recyclables": "recyclable",
    "recycled": "recycle",
    "recyclers": "recycler",
    "recycles": "recycle",
    "recycling": "recycle",
    "recyclings": "recycling",
    "red-eyes": "red-eye",
    "redacted": "redact",
    "redacting": "redact",
    "redactors": "redactor",
    "redacts": "redact",
    "redcoats": "redcoat",
    "redcurrants": "redcurrant",
    "reddened": "redden",
    "reddening": "redden",
    "reddens": "redden",
    "redder": "red",
    "reddest": "red",
    "redds": "redd",
    "redecorated": "redecorate",
    "redecorates": "redecorate",
    "redecorating": "redecorate",
    "redecorations": "redecoration",
    "rededicated": "rededicate",
    "rededicates": "rededicate",
    "rededicating": "rededicate",
    "redeemed": "redeem",
    "redeemers": "redeemer",
    "redeeming": "redeem",
    "redeems": "redeem",
    "redefined": "redefine",
    "redefines": "redefine",
    "redefining": "redefine",
    "redefinitions": "redefinition",
    "redeliveries": "redelivery",
    "redeployed": "redeploy",
    "redeploying": "redeploy",
    "redeployments": "redeployment",
    "redeploys": "redeploy",
    "redeposited": "redeposit",
    "redepositing": "redeposit",
    "redeposits": "redeposit",
    "redesignated": "redesignate",
    "redesignates": "redesignate",
    "redesignating": "redesignate",
    "redesigned": "redesign",
    "redesigning": "redesign",
    "redesigns": "redesign",
    "redeveloped": "redevelop",
    "redeveloping": "redevelop",
    "redevelopments": "redevelopment",
    "redevelopped": "redevelop",
    "redevelopping": "redevelop",
    "redevelops": "redevelop",
    "redeyes": "redeye",
    "redheads": "redhead",
    "redid": "redo",
    "redirected": "redirect",
    "redirecting": "redirect",
    "redirections": "redirection",
    "redirects": "redirect",
    "rediscovered": "rediscover",
    "rediscoveries": "rediscovery",
    "rediscovering": "rediscover",
    "rediscovers": "rediscover",
    "redisplayed": "redisplay",
    "redisplaying": "redisplay",
    "redisplays": "redisplay",
    "redistributed": "redistribute",
    "redistributes": "redistribute",
    "redistributing": "redistribute",
    "redistributions": "redistribution",
    "redlines": "redline",
    "redoed": "redo",
    "redoes": "redo",
    "redoing": "redo",
    "redone": "redo",
    "redos": "redo",
    "redoubled": "redouble",
    "redoubles": "redouble",
    "redoubling": "redouble",
    "redoubts": "redoubt",
    "redounded": "redound",
    "redounding": "redound",
    "redounds": "redound",
    "redrafted": "redraft",
    "redrafting": "redraft",
    "redrafts": "redraft",
    "redrawing": "redraw",
    "redrawn": "redraw",
    "redraws": "redraw",
    "redressed": "redress",
    "redresses": "redress",
    "redressing": "redress",
    "redrew": "redraw",
    "reds": "red",
    "redshanks": "redshank",
    "redshifts": "redshift",
    "redskins": "redskin",
    "redstarts": "redstart",
    "reduced": "reduce",
    "reducers": "reducer",
    "reduces": "reduce",
    "reducing": "reduce",
    "reductases": "reductase",
    "reductionisms": "reductionism",
    "reductionists": "reductionist",
    "reductions": "reduction",
    "redundancies": "redundancy",
    "reduplications": "reduplication",
    "redwings": "redwing",
    "redwoods": "redwood",
    "reedier": "reedy",
    "reediest": "reedy",
    "reeds": "reed",
    "reefed": "reef",
    "reefers": "reefer",
    "reefing": "reef",
    "reefs": "reef",
    "reeked": "reek",
    "reeking": "reek",
    "reeks": "reek",
    "reelected": "reelect",
    "reelecting": "reelect",
    "reelects": "reelect",
    "reeled": "reel",
    "reeling": "reel",
    "reels": "reel",
    "reemerged": "reemerge",
    "reemerges": "reemerge",
    "reemerging": "reemerge",
    "reenacted": "reenact",
    "reenacting": "reenact",
    "reenactments": "reenactment",
    "reenacts": "reenact",
    "reengineered": "reengineer",
    "reengineering": "reengineer",
    "reengineers": "reengineer",
    "reentered": "reenter",
    "reentering": "reenter",
    "reenters": "reenter",
    "reentries": "reentry",
    "reestablished": "reestablish",
    "reestablishes": "reestablish",
    "reestablishing": "reestablish",
    "reevaluated": "reevaluate",
    "reevaluates": "reevaluate",
    "reevaluating": "reevaluate",
    "reevaluations": "reevaluation",
    "reeves": "reeve",
    "reexaminations": "reexamination",
    "reexamined": "reexamine",
    "reexamines": "reexamine",
    "reexamining": "reexamine",
    "refaced": "reface",
    "refaces": "reface",
    "refacing": "reface",
    "refashioned": "refashion",
    "refashioning": "refashion",
    "refashions": "refashion",
    "refectories": "refectory",
    "refered": "refer",
    "refereed": "referee",
    "refereeing": "referee",
    "referees": "referee",
    "referenced": "reference",
    "references": "reference",
    "referencing": "reference",
    "referenda": "referendum",
    "referendums": "referendum",
    "referents": "referent",
    "refering": "refer",
    "referrals": "referral",
    "referred": "refer",
    "referrers": "referrer",
    "referring": "refer",
    "refers": "refer",
    "refilled": "refill",
    "refilling": "refill",
    "refills": "refill",
    "refinanced": "refinance",
    "refinances": "refinance",
    "refinancing": "refinance",
    "refined": "refine",
    "refinements": "refinement",
    "refineries": "refinery",
    "refiners": "refiner",
    "refines": "refine",
    "refining": "refine",
    "refinished": "refinish",
    "refinishes": "refinish",
    "refinishing": "refinish",
    "refits": "refit",
    "refitted": "refit",
    "refitting": "refit",
    "reflated": "reflate",
    "reflates": "reflate",
    "reflating": "reflate",
    "reflectances": "reflectance",
    "reflected": "reflect",
    "reflecting": "reflect",
    "reflections": "reflection",
    "reflectivities": "reflectivity",
    "reflectors": "reflector",
    "reflects": "reflect",
    "reflexes": "reflex",
    "reflexions": "reflexion",
    "reflexives": "reflexive",
    "reflexologists": "reflexologist",
    "refloated": "refloat",
    "refloating": "refloat",
    "refloats": "refloat",
    "refluxed": "reflux",
    "refluxes": "reflux",
    "refluxing": "reflux",
    "refocused": "refocus",
    "refocuses": "refocus",
    "refocusing": "refocus",
    "refocussed": "refocus",
    "refocussing": "refocus",
    "reforested": "reforest",
    "reforesting": "reforest",
    "reforests": "reforest",
    "reformations": "reformation",
    "reformatories": "reformatory",
    "reformats": "reformat",
    "reformatted": "reformat",
    "reformatting": "reformat",
    "reformed": "reform",
    "reformers": "reformer",
    "reforming": "reform",
    "reformists": "reformist",
    "reforms": "reform",
    "reformulated": "reformulate",
    "reformulates": "reformulate",
    "reformulating": "reformulate",
    "reformulations": "reformulation",
    "refounded": "refound",
    "refounding": "refound",
    "refounds": "refound",
    "refracted": "refract",
    "refracting": "refract",
    "refractions": "refraction",
    "refractories": "refractory",
    "refractors": "refractor",
    "refracts": "refract",
    "refrained": "refrain",
    "refraining": "refrain",
    "refrains": "refrain",
    "reframed": "reframe",
    "reframes": "reframe",
    "reframing": "reframe",
    "refreezes": "refreeze",
    "refreezing": "refreeze",
    "refreshed": "refresh",
    "refreshers": "refresher",
    "refreshes": "refresh",
    "refreshing": "refresh",
    "refreshments": "refreshment",
    "refrigerants": "refrigerant",
    "refrigerated": "refrigerate",
    "refrigerates": "refrigerate",
    "refrigerating": "refrigerate",
    "refrigerators": "refrigerator",
    "refroze": "refreeze",
    "refrozen": "refreeze",
    "refs": "ref",
    "refueled": "refuel",
    "refueling": "refuel",
    "refuelled": "refuel",
    "refuelling": "refuel",
    "refuels": "refuel",
    "refugees": "refugee",
    "refuges": "refuge",
    "refugia": "refugium",
    "refunded": "refund",
    "refunding": "refund",
    "refunds": "refund",
    "refurbished": "refurbish",
    "refurbishes": "refurbish",
    "refurbishing": "refurbish",
    "refurnished": "refurnish",
    "refurnishes": "refurnish",
    "refurnishing": "refurnish",
    "refusals": "refusal",
    "refused": "refuse",
    "refusers": "refuser",
    "refuses": "refuse",
    "refusing": "refuse",
    "refutations": "refutation",
    "refuted": "refute",
    "refutes": "refute",
    "refuting": "refute",
    "regained": "regain",
    "regaining": "regain",
    "regains": "regain",
    "regaled": "regale",
    "regales": "regale",
    "regalia": "regal",
    "regalias": "regalia",
    "regaling": "regale",
    "regals": "regal",
    "regarded": "regard",
    "regarding": "regard",
    "regards": "regard",
    "regattas": "regatta",
    "regencies": "regency",
    "regenerated": "regenerate",
    "regenerates": "regenerate",
    "regenerating": "regenerate",
    "regenerations": "regeneration",
    "regenerators": "regenerator",
    "regents": "regent",
    "reges": "rex",
    "regicides": "regicide",
    "regimens": "regimen",
    "regimented": "regiment",
    "regimenting": "regiment",
    "regiments": "regiment",
    "regimes": "regime",
    "regionalisations": "regionalisation",
    "regionalised": "regionalise",
    "regionalises": "regionalise",
    "regionalising": "regionalise",
    "regionalizations": "regionalization",
    "regionals": "regional",
    "regions": "region",
    "registered": "register",
    "registering": "register",
    "registers": "register",
    "registrants": "registrant",
    "registrars": "registrar",
    "registrations": "registration",
    "registries": "registry",
    "regraded": "regrade",
    "regrades": "regrade",
    "regrading": "regrade",
    "regradings": "regrading",
    "regressed": "regress",
    "regresses": "regress",
    "regressing": "regress",
    "regressions": "regression",
    "regressors": "regressor",
    "regrets": "regret",
    "regretted": "regret",
    "regretting": "regret",
    "regrouped": "regroup",
    "regrouping": "regroup",
    "regroups": "regroup",
    "regrowed": "regrow",
    "regrowing": "regrow",
    "regrows": "regrow",
    "regrowths": "regrowth",
    "regs": "reg",
    "regularisations": "regularisation",
    "regularised": "regularise",
    "regularises": "regularise",
    "regularising": "regularise",
    "regularities": "regularity",
    "regularizations": "regularization",
    "regularized": "regularize",
    "regularizes": "regularize",
    "regularizing": "regularize",
    "regulars": "regular",
    "regulated": "regulate",
    "regulates": "regulate",
    "regulating": "regulate",
    "regulations": "regulation",
    "regulators": "regulator",
    "regurgitated": "regurgitate",
    "regurgitates": "regurgitate",
    "regurgitating": "regurgitate",
    "regurgitations": "regurgitation",
    "rehabilitated": "rehabilitate",
    "rehabilitates": "rehabilitate",
    "rehabilitating": "rehabilitate",
    "rehabilitations": "rehabilitation",
    "rehanged": "rehang",
    "rehanging": "rehang",
    "rehangs": "rehang",
    "rehashed": "rehash",
    "rehashes": "rehash",
    "rehashing": "rehash",
    "reheard": "rehear",
    "rehearing": "rehear",
    "rehears": "rehear",
    "rehearsals": "rehearsal",
    "rehearsed": "rehearse",
    "rehearses": "rehearse",
    "rehearsing": "rehearse",
    "reheated": "reheat",
    "reheating": "reheat",
    "reheats": "reheat",
    "rehoused": "rehouse",
    "rehouses": "rehouse",
    "rehousing": "rehouse",
    "rehung": "rehang",
    "rehydrated": "rehydrate",
    "rehydrates": "rehydrate",
    "rehydrating": "rehydrate",
    "rehydrations": "rehydration",
    "rei": "reus",
    "reifications": "reification",
    "reified": "reify",
    "reifies": "reify",
    "reifying": "reify",
    "reigned": "reign",
    "reigning": "reign",
    "reignited": "reignite",
    "reignites": "reignite",
    "reigniting": "reignite",
    "reigns": "reign",
    "reimbursed": "reimburse",
    "reimbursements": "reimbursement",
    "reimburses": "reimburse",
    "reimbursing": "reimburse",
    "reimposed": "reimpose",
    "reimposes": "reimpose",
    "reimposing": "reimpose",
    "reincarnated": "reincarnate",
    "reincarnates": "reincarnate",
    "reincarnating": "reincarnate",
    "reincarnations": "reincarnation",
    "reindeers": "reindeer",
    "reined": "rein",
    "reinfections": "reinfection",
    "reinforced": "reinforce",
    "reinforcements": "reinforcement",
    "reinforcers": "reinforcer",
    "reinforces": "reinforce",
    "reinforcing": "reinforce",
    "reining": "rein",
    "reins": "rein",
    "reinscribed": "reinscribe",
    "reinscribes": "reinscribe",
    "reinscribing": "reinscribe",
    "reinserted": "reinsert",
    "reinserting": "reinsert",
    "reinserts": "reinsert",
    "reinstallations": "reinstallation",
    "reinstalled": "reinstall",
    "reinstalling": "reinstall",
    "reinstalls": "reinstall",
    "reinstated": "reinstate",
    "reinstatements": "reinstatement",
    "reinstates": "reinstate",
    "reinstating": "reinstate",
    "reinsured": "reinsure",
    "reinsurers": "reinsurer",
    "reinsures": "reinsure",
    "reinsuring": "reinsure",
    "reintegrated": "reintegrate",
    "reintegrates": "reintegrate",
    "reintegrating": "reintegrate",
    "reintegrations": "reintegration",
    "reinterpretations": "reinterpretation",
    "reinterpreted": "reinterpret",
    "reinterpreting": "reinterpret",
    "reinterprets": "reinterpret",
    "reintroduced": "reintroduce",
    "reintroduces": "reintroduce",
    "reintroducing": "reintroduce",
    "reintroductions": "reintroduction",
    "reinvented": "reinvent",
    "reinventing": "reinvent",
    "reinventions": "reinvention",
    "reinvents": "reinvent",
    "reinvested": "reinvest",
    "reinvesting": "reinvest",
    "reinvests": "reinvest",
    "reinvigorated": "reinvigorate",
    "reinvigorates": "reinvigorate",
    "reinvigorating": "reinvigorate",
    "reinvigorations": "reinvigoration",
    "reis": "real",
    "reissued": "reissue",
    "reissues": "reissue",
    "reissuing": "reissue",
    "reiterated": "reiterate",
    "reiterates": "reiterate",
    "reiterating": "reiterate",
    "reiterations": "reiteration",
    "rejected": "reject",
    "rejecting": "reject",
    "rejections": "rejection",
    "rejects": "reject",
    "rejigged": "rejig",
    "rejigging": "rejig",
    "rejigs": "rejig",
    "rejoiced": "rejoice",
    "rejoices": "rejoice",
    "rejoicing": "rejoice",
    "rejoicings": "rejoicing",
    "rejoinders": "rejoinder",
    "rejoined": "rejoin",
    "rejoining": "rejoin",
    "rejoins": "rejoin",
    "rejuvenated": "rejuvenate",
    "rejuvenates": "rejuvenate",
    "rejuvenating": "rejuvenate",
    "rejuvenations": "rejuvenation",
    "rekindled": "rekindle",
    "rekindles": "rekindle",
    "rekindling": "rekindle",
    "relabeled": "relabel",
    "relabeling": "relabel",
    "relabelled": "relabel",
    "relabelling": "relabel",
    "relabels": "relabel",
    "relaid": "relay",
    "relapsed": "relapse",
    "relapses": "relapse",
    "relapsing": "relapse",
    "related": "relate",
    "relatednesses": "relatedness",
    "relates": "relate",
    "relating": "relate",
    "relations": "relation",
    "relationships": "relationship",
    "relatives": "relative",
    "relativised": "relativise",
    "relativises": "relativise",
    "relativising": "relativise",
    "relativists": "relativist",
    "relativities": "relativity",
    "relativized": "relativize",
    "relativizes": "relativize",
    "relativizing": "relativize",
    "relators": "relator",
    "relaunched": "relaunch",
    "relaunches": "relaunch",
    "relaunching": "relaunch",
    "relaxants": "relaxant",
    "relaxations": "relaxation",
    "relaxed": "relax",
    "relaxes": "relax",
    "relaxing": "relax",
    "relayed": "relay",
    "relaying": "relay",
    "relays": "relay",
    "relearned": "relearn",
    "relearning": "relearn",
    "relearns": "relearn",
    "released": "release",
    "releases": "release",
    "releasing": "release",
    "relegated": "relegate",
    "relegates": "relegate",
    "relegating": "relegate",
    "relending": "relend",
    "relends": "relend",
    "relent": "relend",
    "relented": "relent",
    "relenting": "relent",
    "relents": "relent",
    "relevancies": "relevancy",
    "reliabilities": "reliability",
    "relics": "relic",
    "relicts": "relict",
    "relied": "rely",
    "reliefs": "relief",
    "relies": "rely",
    "relieved": "relieve",
    "relievers": "reliever",
    "relieves": "relieve",
    "relieving": "relieve",
    "relights": "relight",
    "religionists": "religionist",
    "religions": "religion",
    "relined": "reline",
    "relines": "reline",
    "relining": "reline",
    "relinquished": "relinquish",
    "relinquishes": "relinquish",
    "relinquishing": "relinquish",
    "reliquaries": "reliquary",
    "relished": "relish",
    "relishes": "relish",
    "relishing": "relish",
    "relived": "relive",
    "relives": "relive",
    "reliving": "relive",
    "reloaded": "reload",
    "reloading": "reload",
    "reloads": "reload",
    "relocated": "relocate",
    "relocates": "relocate",
    "relocating": "relocate",
    "relocations": "relocation",
    "rels": "rel",
    "relying": "rely",
    "remade": "remake",
    "remaindered": "remainder",
    "remaindering": "remainder",
    "remainders": "remainder",
    "remained": "remain",
    "remaining": "remain",
    "remains": "remain",
    "remakes": "remake",
    "remaking": "remake",
    "remanded": "remand",
    "remanding": "remand",
    "remands": "remand",
    "remanufactured": "remanufacture",
    "remanufactures": "remanufacture",
    "remanufacturing": "remanufacture",
    "remapped": "remap",
    "remapping": "remap",
    "remappings": "remapping",
    "remaps": "remap",
    "remarked": "remark",
    "remarketed": "remarket",
    "remarketing": "remarket",
    "remarkets": "remarket",
    "remarking": "remark",
    "remarks": "remark",
    "remarriages": "remarriage",
    "remarried": "remarry",
    "remarries": "remarry",
    "remarrying": "remarry",
    "remastered": "remaster",
    "remastering": "remaster",
    "remasters": "remaster",
    "rematches": "rematch",
    "remediated": "remediate",
    "remediates": "remediate",
    "remediating": "remediate",
    "remediations": "remediation",
    "remedied": "remedy",
    "remedies": "remedy",
    "remedying": "remedy",
    "remembered": "remember",
    "remembering": "remember",
    "remembers": "remember",
    "remembrances": "remembrance",
    "reminded": "remind",
    "reminders": "reminder",
    "reminding": "remind",
    "reminds": "remind",
    "reminisced": "reminisce",
    "reminiscences": "reminiscence",
    "reminiscently": "reminiscent",
    "reminisces": "reminisce",
    "reminiscing": "reminisce",
    "remissions": "remission",
    "remits": "remit",
    "remittances": "remittance",
    "remitted": "remit",
    "remitting": "remit",
    "remixed": "remix",
    "remixes": "remix",
    "remixing": "remix",
    "remnants": "remnant",
    "remodeled": "remodel",
    "remodeling": "remodel",
    "remodelings": "remodeling",
    "remodelled": "remodel",
    "remodelling": "remodel",
    "remodellings": "remodelling",
    "remodels": "remodel",
    "remonstrances": "remonstrance",
    "remonstrated": "remonstrate",
    "remonstrates": "remonstrate",
    "remonstrating": "remonstrate",
    "remoter": "remote",
    "remotes": "remote",
    "remotest": "remote",
    "remoulded": "remould",
    "remoulding": "remould",
    "remoulds": "remould",
    "remounted": "remount",
    "remounting": "remount",
    "remounts": "remount",
    "removals": "removal",
    "removed": "remove",
    "removers": "remover",
    "removes": "remove",
    "removing": "remove",
    "rems": "rem",
    "remunerated": "remunerate",
    "remunerates": "remunerate",
    "remunerating": "remunerate",
    "remunerations": "remuneration",
    "renaissances": "renaissance",
    "renamed": "rename",
    "renames": "rename",
    "renaming": "rename",
    "renationalisations": "renationalisation",
    "renationalised": "renationalise",
    "renationalises": "renationalise",
    "renationalising": "renationalise",
    "rendered": "render",
    "renderers": "renderer",
    "rendering": "render",
    "renderings": "rendering",
    "renders": "render",
    "rendez-voused": "rendez-vous",
    "rendez-vouses": "rendez-vous",
    "rendez-vousing": "rendez-vous",
    "rendezvoused": "rendezvous",
    "rendezvouses": "rendezvous",
    "rendezvousing": "rendezvous",
    "rending": "rend",
    "renditions": "rendition",
    "rends": "rend",
    "renegades": "renegade",
    "reneged": "renege",
    "reneges": "renege",
    "reneging": "renege",
    "renegotiated": "renegotiate",
    "renegotiates": "renegotiate",
    "renegotiating": "renegotiate",
    "renegotiations": "renegotiation",
    "renes": "ren",
    "renewables": "renewable",
    "renewals": "renewal",
    "renewed": "renew",
    "renewing": "renew",
    "renews": "renew",
    "renins": "renin",
    "renounced": "renounce",
    "renounces": "renounce",
    "renouncing": "renounce",
    "renovated": "renovate",
    "renovates": "renovate",
    "renovating": "renovate",
    "renovations": "renovation",
    "renovators": "renovator",
    "rent": "rend",
    "rentals": "rental",
    "rented": "rent",
    "renters": "renter",
    "renting": "rent",
    "rents": "rent",
    "renumbered": "renumber",
    "renumbering": "renumber",
    "renumbers": "renumber",
    "renunciations": "renunciation",
    "reoccupations": "reoccupation",
    "reoccupied": "reoccupy",
    "reoccupies": "reoccupy",
    "reoccupying": "reoccupy",
    "reoccurred": "reoccur",
    "reoccurrences": "reoccurrence",
    "reoccurring": "reoccur",
    "reoccurs": "reoccur",
    "reoffended": "reoffend",
    "reoffending": "reoffend",
    "reoffends": "reoffend",
    "reopened": "reopen",
    "reopening": "reopen",
    "reopenings": "reopening",
    "reopens": "reopen",
    "reordered": "reorder",
    "reordering": "reorder",
    "reorders": "reorder",
    "reorganisations": "reorganisation",
    "reorganised": "reorganise",
    "reorganises": "reorganise",
    "reorganising": "reorganise",
    "reorganizations": "reorganization",
    "reorganized": "reorganize",
    "reorganizes": "reorganize",
    "reorganizing": "reorganize",
    "reorientations": "reorientation",
    "reoriented": "reorient",
    "reorienting": "reorient",
    "reorients": "reorient",
    "repackaged": "repackage",
    "repackages": "repackage",
    "repackaging": "repackage",
    "repacked": "repack",
    "repacking": "repack",
    "repacks": "repack",
    "repaid": "repay",
    "repainted": "repaint",
    "repainting": "repaint",
    "repaints": "repaint",
    "repaired": "repair",
    "repairers": "repairer",
    "repairing": "repair",
    "repairmen": "repairman",
    "repairs": "repair",
    "reparations": "reparation",
    "repartees": "repartee",
    "repartitioned": "repartition",
    "repartitioning": "repartition",
    "repartitions": "repartition",
    "repasts": "repast",
    "repatriated": "repatriate",
    "repatriates": "repatriate",
    "repatriating": "repatriate",
    "repatriations": "repatriation",
    "repaying": "repay",
    "repayments": "repayment",
    "repays": "repay",
    "repealed": "repeal",
    "repealing": "repeal",
    "repeals": "repeal",
    "repeatabilities": "repeatability",
    "repeated": "repeat",
    "repeaters": "repeater",
    "repeating": "repeat",
    "repeats": "repeat",
    "repellants": "repellant",
    "repelled": "repel",
    "repellents": "repellent",
    "repelling": "repel",
    "repels": "repel",
    "repented": "repent",
    "repenting": "repent",
    "repents": "repent",
    "repercussions": "repercussion",
    "reperfusions": "reperfusion",
    "repertoires": "repertoire",
    "repertories": "repertory",
    "repetitions": "repetition",
    "rephrased": "rephrase",
    "rephrases": "rephrase",
    "rephrasing": "rephrase",
    "repined": "repine",
    "repines": "repine",
    "repining": "repine",
    "replaced": "replace",
    "replacements": "replacement",
    "replacers": "replacer",
    "replaces": "replace",
    "replacing": "replace",
    "replanted": "replant",
    "replanting": "replant",
    "replants": "replant",
    "replayed": "replay",
    "replaying": "replay",
    "replays": "replay",
    "replenished": "replenish",
    "replenishes": "replenish",
    "replenishing": "replenish",
    "replenishments": "replenishment",
    "replicas": "replica",
    "replicated": "replicate",
    "replicates": "replicate",
    "replicating": "replicate",
    "replications": "replication",
    "replicators": "replicator",
    "replied": "reply",
    "replies": "reply",
    "replying": "reply",
    "repointed": "repoint",
    "repointing": "repoint",
    "repoints": "repoint",
    "repopulated": "repopulate",
    "repopulates": "repopulate",
    "repopulating": "repopulate",
    "reported": "report",
    "reporters": "reporter",
    "reporting": "report",
    "reportings": "reporting",
    "reports": "report",
    "reposed": "repose",
    "reposes": "repose",
    "reposing": "repose",
    "repositioned": "reposition",
    "repositioning": "reposition",
    "repositions": "reposition",
    "repositories": "repository",
    "repossessed": "repossess",
    "repossesses": "repossess",
    "repossessing": "repossess",
    "repots": "repot",
    "repotted": "repot",
    "repotting": "repot",
    "representations": "representation",
    "representatives": "representative",
    "represented": "represent",
    "representing": "represent",
    "represents": "represent",
    "repressed": "repress",
    "represses": "repress",
    "repressing": "repress",
    "repressions": "repression",
    "repressors": "repressor",
    "reprieved": "reprieve",
    "reprieves": "reprieve",
    "reprieving": "reprieve",
    "reprimanded": "reprimand",
    "reprimanding": "reprimand",
    "reprimands": "reprimand",
    "reprinted": "reprint",
    "reprinting": "reprint",
    "reprints": "reprint",
    "reprisals": "reprisal",
    "reprised": "reprise",
    "reprises": "reprise",
    "reprising": "reprise",
    "reproached": "reproach",
    "reproaches": "reproach",
    "reproaching": "reproach",
    "reprobates": "reprobate",
    "reprocessed": "reprocess",
    "reprocesses": "reprocess",
    "reprocessing": "reprocess",
    "reprocessors": "reprocessor",
    "reproduced": "reproduce",
    "reproduces": "reproduce",
    "reproducibilities": "reproducibility",
    "reproducing": "reproduce",
    "reproductions": "reproduction",
    "reprogrammed": "reprogramme",
    "reprogrammes": "reprogramme",
    "reprogramming": "reprogramme",
    "reprograms": "reprogram",
    "reproofs": "reproof",
    "reproved": "reprove",
    "reproves": "reprove",
    "reproving": "reprove",
    "reps": "rep",
    "reptiles": "reptile",
    "republics": "republic",
    "republished": "republish",
    "republishes": "republish",
    "republishing": "republish",
    "repudiated": "repudiate",
    "repudiates": "repudiate",
    "repudiating": "repudiate",
    "repulsed": "repulse",
    "repulses": "repulse",
    "repulsing": "repulse",
    "repulsions": "repulsion",
    "repurchased": "repurchase",
    "repurchases": "repurchase",
    "repurchasing": "repurchase",
    "reputations": "reputation",
    "reputed": "repute",
    "reputes": "repute",
    "reputing": "repute",
    "requested": "request",
    "requesters": "requester",
    "requesting": "request",
    "requestors": "requestor",
    "requests": "request",
    "requiems": "requiem",
    "required": "require",
    "requirements": "requirement",
    "requires": "require",
    "requiring": "require",
    "requisites": "requisite",
    "requisitioned": "requisition",
    "requisitioning": "requisition",
    "requisitions": "requisition",
    "requited": "requite",
    "requites": "requite",
    "requiting": "requite",
    "reran": "rerun",
    "rereading": "reread",
    "rereads": "reread",
    "reregistered": "reregister",
    "reregistering": "reregister",
    "reregisters": "reregister",
    "rerouted": "reroute",
    "reroutes": "reroute",
    "rerouting": "reroute",
    "reruning": "rerun",
    "rerunning": "rerun",
    "reruns": "rerun",
    "resales": "resale",
    "resampled": "resample",
    "resamples": "resample",
    "resampling": "resample",
    "rescaled": "rescale",
    "rescales": "rescale",
    "rescaling": "rescale",
    "rescheduled": "reschedule",
    "reschedules": "reschedule",
    "rescheduling": "reschedule",
    "rescinded": "rescind",
    "rescinding": "rescind",
    "rescinds": "rescind",
    "rescissions": "rescission",
    "rescued": "rescue",
    "rescuers": "rescuer",
    "rescues": "rescue",
    "rescuing": "rescue",
    "resealed": "reseal",
    "resealing": "reseal",
    "reseals": "reseal",
    "researched": "research",
    "researchers": "researcher",
    "researches": "research",
    "researching": "research",
    "reseated": "reseat",
    "reseating": "reseat",
    "reseats": "reseat",
    "resected": "resect",
    "resecting": "resect",
    "resections": "resection",
    "resects": "resect",
    "reselected": "reselect",
    "reselecting": "reselect",
    "reselects": "reselect",
    "resellers": "reseller",
    "reselling": "resell",
    "resells": "resell",
    "resemblances": "resemblance",
    "resembled": "resemble",
    "resembles": "resemble",
    "resembling": "resemble",
    "resending": "resend",
    "resends": "resend",
    "resent": "resend",
    "resented": "resent",
    "resenting": "resent",
    "resentments": "resentment",
    "resents": "resent",
    "reservations": "reservation",
    "reserved": "reserve",
    "reserves": "reserve",
    "reserving": "reserve",
    "reservists": "reservist",
    "reservoirs": "reservoir",
    "resets": "reset",
    "resetting": "reset",
    "resettled": "resettle",
    "resettlements": "resettlement",
    "resettles": "resettle",
    "resettling": "resettle",
    "reshaped": "reshape",
    "reshapes": "reshape",
    "reshaping": "reshape",
    "reshuffled": "reshuffle",
    "reshuffles": "reshuffle",
    "reshuffling": "reshuffle",
    "resided": "reside",
    "residences": "residence",
    "residencies": "residency",
    "residents": "resident",
    "resides": "reside",
    "residing": "reside",
    "residuals": "residual",
    "residues": "residue",
    "resignations": "resignation",
    "resigned": "resign",
    "resigning": "resign",
    "resigns": "resign",
    "resiliencies": "resiliency",
    "resins": "resin",
    "resistances": "resistance",
    "resistants": "resistant",
    "resisted": "resist",
    "resisters": "resister",
    "resisting": "resist",
    "resistivities": "resistivity",
    "resistors": "resistor",
    "resists": "resist",
    "resited": "resite",
    "resites": "resite",
    "resiting": "resite",
    "resized": "resize",
    "resizes": "resize",
    "resizing": "resize",
    "resold": "resell",
    "resolutions": "resolution",
    "resolved": "resolve",
    "resolvers": "resolver",
    "resolves": "resolve",
    "resolving": "resolve",
    "resonances": "resonance",
    "resonated": "resonate",
    "resonates": "resonate",
    "resonating": "resonate",
    "resonators": "resonator",
    "resorptions": "resorption",
    "resorted": "resort",
    "resorting": "resort",
    "resorts": "resort",
    "resounded": "resound",
    "resounding": "resound",
    "resounds": "resound",
    "resourced": "resource",
    "resources": "resource",
    "resourcing": "resource",
    "respected": "respect",
    "respecters": "respecter",
    "respecting": "respect",
    "respects": "respect",
    "respirations": "respiration",
    "respirators": "respirator",
    "respired": "respire",
    "respires": "respire",
    "respiring": "respire",
    "respites": "respite",
    "responded": "respond",
    "respondees": "respondee",
    "respondents": "respondent",
    "responders": "responder",
    "responding": "respond",
    "responds": "respond",
    "responses": "response",
    "responsibilities": "responsibility",
    "responsivenesses": "responsiveness",
    "resprayed": "respray",
    "respraying": "respray",
    "resprays": "respray",
    "resps": "resp",
    "rest-homes": "rest-home",
    "restaged": "restage",
    "restages": "restage",
    "restaging": "restage",
    "restarted": "restart",
    "restarting": "restart",
    "restarts": "restart",
    "restated": "restate",
    "restatements": "restatement",
    "restates": "restate",
    "restating": "restate",
    "restaurants": "restaurant",
    "restaurateurs": "restaurateur",
    "rested": "rest",
    "restenoses": "restenosis",
    "resting": "rest",
    "restitutions": "restitution",
    "restocked": "restock",
    "restocking": "restock",
    "restocks": "restock",
    "restorationed": "restoration",
    "restorationing": "restoration",
    "restorations": "restoration",
    "restoratives": "restorative",
    "restored": "restore",
    "restorers": "restorer",
    "restores": "restore",
    "restoring": "restore",
    "restrained": "restrain",
    "restraining": "restrain",
    "restrains": "restrain",
    "restraints": "restraint",
    "restricted": "restrict",
    "restricting": "restrict",
    "restrictions": "restriction",
    "restrictors": "restrictor",
    "restricts": "restrict",
    "restrooms": "restroom",
    "restructured": "restructure",
    "restructures": "restructure",
    "restructuring": "restructure",
    "restructurings": "restructuring",
    "rests": "rest",
    "restyled": "restyle",
    "restyles": "restyle",
    "restyling": "restyle",
    "resubmissions": "resubmission",
    "resubmits": "resubmit",
    "resubmitted": "resubmit",
    "resubmitting": "resubmit",
    "resultants": "resultant",
    "resulted": "result",
    "resulting": "result",
    "results": "result",
    "resumed": "resume",
    "resumes": "resume",
    "resuming": "resume",
    "resumptions": "resumption",
    "resumés": "resumé",
    "resurfaced": "resurface",
    "resurfaces": "resurface",
    "resurfacing": "resurface",
    "resurged": "resurge",
    "resurgences": "resurgence",
    "resurges": "resurge",
    "resurging": "resurge",
    "resurrected": "resurrect",
    "resurrecting": "resurrect",
    "resurrections": "resurrection",
    "resurrects": "resurrect",
    "resurveyed": "resurvey",
    "resurveying": "resurvey",
    "resurveys": "resurvey",
    "resuscitated": "resuscitate",
    "resuscitates": "resuscitate",
    "resuscitating": "resuscitate",
    "resuscitations": "resuscitation",
    "resuspended": "resuspend",
    "resuspending": "resuspend",
    "resuspends": "resuspend",
    "resuspensions": "resuspension",
    "retailed": "retail",
    "retailers": "retailer",
    "retailing": "retail",
    "retails": "retail",
    "retained": "retain",
    "retainers": "retainer",
    "retaining": "retain",
    "retains": "retain",
    "retaken": "retake",
    "retakes": "retake",
    "retaking": "retake",
    "retaliated": "retaliate",
    "retaliates": "retaliate",
    "retaliating": "retaliate",
    "retaliations": "retaliation",
    "retardants": "retardant",
    "retardations": "retardation",
    "retarded": "retard",
    "retarders": "retarder",
    "retarding": "retard",
    "retards": "retard",
    "retched": "retch",
    "retches": "retch",
    "retching": "retch",
    "retelling": "retell",
    "retells": "retell",
    "retentions": "retention",
    "retested": "retest",
    "retesting": "retest",
    "retests": "retest",
    "rethinking": "rethink",
    "rethinks": "rethink",
    "rethought": "rethink",
    "reticences": "reticence",
    "reticles": "reticle",
    "reticula": "reticulum",
    "reticulated": "reticulate",
    "reticulates": "reticulate",
    "reticulating": "reticulate",
    "reticules": "reticule",
    "reticulums": "reticulum",
    "retimed": "retime",
    "retimes": "retime",
    "retiming": "retime",
    "retinae": "retina",
    "retinas": "retina",
    "retinitides": "retinitis",
    "retinoblastomas": "retinoblastoma",
    "retinoblastomata": "retinoblastoma",
    "retinoids": "retinoid",
    "retinols": "retinol",
    "retinopathies": "retinopathy",
    "retinues": "retinue",
    "retired": "retire",
    "retirees": "retiree",
    "retirements": "retirement",
    "retires": "retire",
    "retiring": "retire",
    "retold": "retell",
    "retook": "retake",
    "retooled": "retool",
    "retooling": "retool",
    "retools": "retool",
    "retorted": "retort",
    "retorting": "retort",
    "retorts": "retort",
    "retouched": "retouch",
    "retouches": "retouch",
    "retouching": "retouch",
    "retraced": "retrace",
    "retraces": "retrace",
    "retracing": "retrace",
    "retracted": "retract",
    "retracting": "retract",
    "retractions": "retraction",
    "retractors": "retractor",
    "retracts": "retract",
    "retrained": "retrain",
    "retraining": "retrain",
    "retrains": "retrain",
    "retransmissions": "retransmission",
    "retransmits": "retransmit",
    "retransmitted": "retransmit",
    "retransmitting": "retransmit",
    "retreaded": "retread",
    "retreading": "retread",
    "retreads": "retread",
    "retreated": "retreat",
    "retreating": "retreat",
    "retreats": "retreat",
    "retrenched": "retrench",
    "retrenches": "retrench",
    "retrenching": "retrench",
    "retrenchments": "retrenchment",
    "retrials": "retrial",
    "retributions": "retribution",
    "retried": "retry",
    "retries": "retry",
    "retrievals": "retrieval",
    "retrieved": "retrieve",
    "retrievers": "retriever",
    "retrieves": "retrieve",
    "retrieving": "retrieve",
    "retrofits": "retrofit",
    "retrofitted": "retrofit",
    "retrofitting": "retrofit",
    "retrograded": "retrograde",
    "retrogrades": "retrograde",
    "retrograding": "retrograde",
    "retrogressed": "retrogress",
    "retrogresses": "retrogress",
    "retrogressing": "retrogress",
    "retrospections": "retrospection",
    "retrospectives": "retrospective",
    "retrotransposons": "retrotransposon",
    "retroviruses": "retrovirus",
    "retrying": "retry",
    "rets": "ret",
    "retted": "ret",
    "retting": "ret",
    "retuned": "retune",
    "retunes": "retune",
    "retuning": "retune",
    "returned": "return",
    "returnees": "returnee",
    "returners": "returner",
    "returning": "return",
    "returns": "return",
    "retyped": "retype",
    "retypes": "retype",
    "retyping": "retype",
    "reunified": "reunify",
    "reunifies": "reunify",
    "reunifying": "reunify",
    "reunions": "reunion",
    "reunited": "reunite",
    "reunites": "reunite",
    "reuniting": "reunite",
    "reused": "reuse",
    "reuses": "reuse",
    "reusing": "reuse",
    "revalidated": "revalidate",
    "revalidates": "revalidate",
    "revalidating": "revalidate",
    "revalidations": "revalidation",
    "revaluations": "revaluation",
    "revalued": "revalue",
    "revalues": "revalue",
    "revaluing": "revalue",
    "revamped": "revamp",
    "revamping": "revamp",
    "revamps": "revamp",
    "revascularisations": "revascularisation",
    "revealed": "reveal",
    "revealing": "reveal",
    "reveals": "reveal",
    "reveilles": "reveille",
    "revelations": "revelation",
    "reveled": "revel",
    "reveling": "revel",
    "revelled": "revel",
    "revellers": "reveller",
    "revelling": "revel",
    "revelries": "revelry",
    "revels": "revel",
    "revenged": "revenge",
    "revenges": "revenge",
    "revenging": "revenge",
    "revenues": "revenue",
    "reverberated": "reverberate",
    "reverberates": "reverberate",
    "reverberating": "reverberate",
    "reverberations": "reverberation",
    "revered": "revere",
    "reverences": "reverence",
    "reverends": "reverend",
    "reveres": "revere",
    "reveries": "reverie",
    "revering": "revere",
    "reversals": "reversal",
    "reversed": "reverse",
    "reversers": "reverser",
    "reverses": "reverse",
    "reversibilities": "reversibility",
    "reversing": "reverse",
    "reversions": "reversion",
    "reverted": "revert",
    "reverting": "revert",
    "reverts": "revert",
    "revetments": "revetment",
    "reviewed": "review",
    "reviewers": "reviewer",
    "reviewing": "review",
    "reviews": "review",
    "reviled": "revile",
    "reviles": "revile",
    "reviling": "revile",
    "revised": "revise",
    "revisers": "reviser",
    "revises": "revise",
    "revising": "revise",
    "revisionists": "revisionist",
    "revisions": "revision",
    "revisited": "revisit",
    "revisiting": "revisit",
    "revisits": "revisit",
    "revitalised": "revitalise",
    "revitalises": "revitalise",
    "revitalising": "revitalise",
    "revitalized": "revitalize",
    "revitalizes": "revitalize",
    "revitalizing": "revitalize",
    "revivalists": "revivalist",
    "revivals": "revival",
    "revived": "revive",
    "revives": "revive",
    "revivified": "revivify",
    "revivifies": "revivify",
    "revivifying": "revivify",
    "reviving": "revive",
    "revocations": "revocation",
    "revoked": "revoke",
    "revokes": "revoke",
    "revoking": "revoke",
    "revolted": "revolt",
    "revolting": "revolt",
    "revolts": "revolt",
    "revolutionaries": "revolutionary",
    "revolutionised": "revolutionise",
    "revolutionises": "revolutionise",
    "revolutionising": "revolutionise",
    "revolutionists": "revolutionist",
    "revolutionized": "revolutionize",
    "revolutionizes": "revolutionize",
    "revolutionizing": "revolutionize",
    "revolutions": "revolution",
    "revolved": "revolve",
    "revolvers": "revolver",
    "revolves": "revolve",
    "revolving": "revolve",
    "revs": "rev",
    "revues": "revue",
    "revved": "rev",
    "revving": "rev",
    "rewarded": "reward",
    "rewarding": "reward",
    "rewards": "reward",
    "rewinded": "rewind",
    "rewinding": "rewind",
    "rewinds": "rewind",
    "rewired": "rewire",
    "rewires": "rewire",
    "rewiring": "rewire",
    "reworded": "reword",
    "rewording": "reword",
    "rewords": "reword",
    "reworked": "rework",
    "reworking": "rework",
    "reworks": "rework",
    "rewound": "rewind",
    "rewounded": "rewound",
    "rewounding": "rewound",
    "rewounds": "rewound",
    "rewrites": "rewrite",
    "rewriting": "rewrite",
    "rewritten": "rewrite",
    "rewrote": "rewrite",
    "rexes": "rex",
    "rhabdomyolyses": "rhabdomyolysis",
    "rhabdomyosarcomas": "rhabdomyosarcoma",
    "rhabdomyosarcomata": "rhabdomyosarcoma",
    "rhapsodies": "rhapsody",
    "rhapsodized": "rhapsodize",
    "rhapsodizes": "rhapsodize",
    "rhapsodizing": "rhapsodize",
    "rheologies": "rheology",
    "rheometers": "rheometer",
    "rheostats": "rheostat",
    "rhesuses": "rhesus",
    "rhetoricians": "rhetorician",
    "rheumatics": "rheumatic",
    "rheumatisms": "rheumatism",
    "rheumatologies": "rheumatology",
    "rheumatologists": "rheumatologist",
    "rhinestones": "rhinestone",
    "rhinitides": "rhinitis",
    "rhinitises": "rhinitis",
    "rhinoceroses": "rhinoceros",
    "rhinos": "rhino",
    "rhinotracheitides": "rhinotracheitis",
    "rhinoviruses": "rhinovirus",
    "rhizomes": "rhizome",
    "rhizospheres": "rhizosphere",
    "rhododendrons": "rhododendron",
    "rhodopsins": "rhodopsin",
    "rhombi": "rhombus",
    "rhomboids": "rhomboid",
    "rhombuses": "rhombus",
    "rhos": "rho",
    "rhymed": "rhyme",
    "rhymes": "rhyme",
    "rhyming": "rhyme",
    "rhyolites": "rhyolite",
    "rhythms": "rhythm",
    "rib-cages": "rib-cage",
    "rib-ticklers": "rib-tickler",
    "ribands": "riband",
    "ribbed": "rib",
    "ribbing": "rib",
    "ribbons": "ribbon",
    "ribcages": "ribcage",
    "riboflavins": "riboflavin",
    "riboses": "ribose",
    "ribosomes": "ribosome",
    "ribozymes": "ribozyme",
    "ribs": "rib",
    "ricefields": "ricefield",
    "rices": "rice",
    "richer": "rich",
    "riches": "rich",
    "richest": "rich",
    "richnesses": "richness",
    "ricins": "ricin",
    "ricked": "rick",
    "ricking": "rick",
    "ricks": "rick",
    "rickshaws": "rickshaw",
    "ricocheted": "ricochet",
    "ricocheting": "ricochet",
    "ricochets": "ricochet",
    "ricochetted": "ricochet",
    "ricochetting": "ricochet",
    "ricottas": "ricotta",
    "ridded": "rid",
    "ridden": "ride",
    "ridding": "rid",
    "riddled": "riddle",
    "riddles": "riddle",
    "riddling": "riddle",
    "riders": "rider",
    "rides": "ride",
    "ridged": "ridge",
    "ridges": "ridge",
    "ridging": "ridge",
    "ridiculed": "ridicule",
    "ridicules": "ridicule",
    "ridiculing": "ridicule",
    "riding": "ride",
    "ridings": "riding",
    "rids": "rid",
    "riffed": "riff",
    "riffing": "riff",
    "riffled": "riffle",
    "riffles": "riffle",
    "riffling": "riffle",
    "riffs": "riff",
    "rifled": "rifle",
    "riflemen": "rifleman",
    "rifles": "rifle",
    "rifling": "rifle",
    "rifts": "rift",
    "rig-outed": "rig-out",
    "rig-outing": "rig-out",
    "rig-outs": "rig-out",
    "rigged": "rig",
    "riggers": "rigger",
    "rigging": "rig",
    "riggings": "rigging",
    "right-angles": "right-angle",
    "right-handers": "right-hander",
    "right-of-ways": "right-of-way",
    "right-wingers": "right-winger",
    "righted": "right",
    "righter": "right",
    "rightest": "right",
    "righting": "right",
    "rightists": "rightist",
    "rights": "right",
    "rights-of-way": "right-of-way",
    "rigidities": "rigidity",
    "rigmaroles": "rigmarole",
    "rigors": "rigor",
    "rigours": "rigour",
    "rigs": "rig",
    "riled": "rile",
    "riles": "rile",
    "riling": "rile",
    "rills": "rill",
    "rimae": "rima",
    "rimes": "rime",
    "rimmed": "rim",
    "rimming": "rim",
    "rims": "rim",
    "rinds": "rind",
    "ringed": "ring",
    "ringers": "ringer",
    "ringing": "ring",
    "ringleaders": "ringleader",
    "ringlets": "ringlet",
    "ringmasters": "ringmaster",
    "rings": "ring",
    "ringsides": "ringside",
    "ringtails": "ringtail",
    "ringways": "ringway",
    "ringworms": "ringworm",
    "rinks": "rink",
    "rinsed": "rinse",
    "rinses": "rinse",
    "rinsing": "rinse",
    "rioted": "riot",
    "rioters": "rioter",
    "rioting": "riot",
    "riots": "riot",
    "rip-offs": "rip-off",
    "ripcords": "ripcord",
    "ripened": "ripen",
    "ripening": "ripen",
    "ripens": "ripen",
    "riper": "ripe",
    "ripest": "ripe",
    "riposted": "riposte",
    "ripostes": "riposte",
    "riposting": "riposte",
    "ripped": "rip",
    "rippers": "ripper",
    "ripping": "rip",
    "rippled": "ripple",
    "ripples": "ripple",
    "rippling": "ripple",
    "rips": "rip",
    "ris": "ri",
    "risen": "rise",
    "risers": "riser",
    "rises": "rise",
    "rising": "rise",
    "risings": "rising",
    "risk-takers": "risk-taker",
    "risked": "risk",
    "riskier": "risky",
    "riskiest": "risky",
    "risking": "risk",
    "risks": "risk",
    "risottos": "risotto",
    "risquéer": "risqué",
    "risquéest": "risqué",
    "rissoles": "rissole",
    "rites": "rite",
    "ritualised": "ritualise",
    "ritualises": "ritualise",
    "ritualising": "ritualise",
    "ritualized": "ritualize",
    "ritualizes": "ritualize",
    "ritualizing": "ritualize",
    "rituals": "ritual",
    "rivaled": "rival",
    "rivaling": "rival",
    "rivalled": "rival",
    "rivalling": "rival",
    "rivalries": "rivalry",
    "rivals": "rival",
    "rived": "rive",
    "riven": "rive",
    "riverbanks": "riverbank",
    "riverbeds": "riverbed",
    "rivers": "river",
    "riversides": "riverside",
    "rives": "rive",
    "riveted": "rivet",
    "riveting": "rivet",
    "rivets": "rivet",
    "riving": "rive",
    "rivulets": "rivulet",
    "rms": "rm",
    "rmss": "rms",
    "roaches": "roach",
    "road-sides": "road-side",
    "roadblocks": "roadblock",
    "roadhogs": "roadhog",
    "roadhouses": "roadhouse",
    "roadmaps": "roadmap",
    "roadrunners": "roadrunner",
    "roads": "road",
    "roadsides": "roadside",
    "roadsigns": "roadsign",
    "roadsters": "roadster",
    "roadways": "roadway",
    "roadworthier": "roadworthy",
    "roadworthiest": "roadworthy",
    "roamed": "roam",
    "roaming": "roam",
    "roams": "roam",
    "roans": "roan",
    "roared": "roar",
    "roaring": "roar",
    "roars": "roar",
    "roasted": "roast",
    "roasters": "roaster",
    "roasting": "roast",
    "roastings": "roasting",
    "roasts": "roast",
    "robbed": "rob",
    "robberies": "robbery",
    "robbers": "robber",
    "robbing": "rob",
    "robed": "robe",
    "robes": "robe",
    "robing": "robe",
    "robins": "robin",
    "robotics": "robotic",
    "robots": "robot",
    "robs": "rob",
    "robustas": "robusta",
    "robustnesses": "robustness",
    "rock-bottoms": "rock-bottom",
    "rock-climbers": "rock-climber",
    "rocked": "rock",
    "rockeries": "rockery",
    "rockers": "rocker",
    "rocketed": "rocket",
    "rocketing": "rocket",
    "rockets": "rocket",
    "rockier": "rocky",
    "rockiest": "rocky",
    "rocking": "rock",
    "rocks": "rock",
    "rockwools": "rockwool",
    "rococos": "rococo",
    "roddings": "rodding",
    "rode": "ride",
    "rodenticides": "rodenticide",
    "rodents": "rodent",
    "rodeos": "rodeo",
    "rods": "rod",
    "roes": "roe",
    "rogues": "rogue",
    "roiled": "roil",
    "roiling": "roil",
    "roils": "roil",
    "roles": "role",
    "roll-calls": "roll-call",
    "roll-offed": "roll-off",
    "roll-offing": "roll-off",
    "roll-offs": "roll-off",
    "roll-overs": "roll-over",
    "rollbacks": "rollback",
    "rolled": "roll",
    "roller-coasters": "roller-coaster",
    "roller-skated": "roller-skate",
    "roller-skates": "roller-skate",
    "roller-skating": "roller-skate",
    "rollerblades": "rollerblade",
    "rollers": "roller",
    "rolling": "roll",
    "rollings": "rolling",
    "rollovers": "rollover",
    "rolls": "roll",
    "roly-polies": "roly-poly",
    "romanced": "romance",
    "romances": "romance",
    "romancing": "romance",
    "romanised": "romanise",
    "romanises": "romanise",
    "romanising": "romanise",
    "romanized": "romanize",
    "romanizes": "romanize",
    "romanizing": "romanize",
    "romanticised": "romanticise",
    "romanticises": "romanticise",
    "romanticising": "romanticise",
    "romanticized": "romanticize",
    "romanticizes": "romanticize",
    "romanticizing": "romanticize",
    "romantics": "romantic",
    "romped": "romp",
    "romping": "romp",
    "romps": "romp",
    "roms": "rom",
    "rondos": "rondo",
    "rontgens": "rontgen",
    "roods": "rood",
    "roof-racks": "roof-rack",
    "roofed": "roof",
    "roofers": "roofer",
    "roofing": "roof",
    "roofs": "roof",
    "rooftops": "rooftop",
    "rooked": "rook",
    "rookeries": "rookery",
    "rookies": "rookie",
    "rooking": "rook",
    "rooks": "rook",
    "roomed": "room",
    "roomfuls": "roomful",
    "roomier": "roomy",
    "roomiest": "roomy",
    "rooming": "room",
    "roommates": "roommate",
    "rooms": "room",
    "roomsful": "roomful",
    "roosted": "roost",
    "roosters": "rooster",
    "roosting": "roost",
    "roosts": "roost",
    "rooted": "root",
    "rooting": "root",
    "rootlets": "rootlet",
    "roots": "root",
    "rootstocks": "rootstock",
    "rooves": "roof",
    "roped": "rope",
    "ropes": "rope",
    "ropier": "ropey",
    "ropiest": "ropey",
    "roping": "rope",
    "rosaries": "rosary",
    "rose": "rise",
    "rosebays": "rosebay",
    "rosebeds": "rosebed",
    "rosebuds": "rosebud",
    "rosebushes": "rosebush",
    "roser": "rose",
    "roses": "rose",
    "rosest": "rose",
    "rosettes": "rosette",
    "rosier": "rosy",
    "rosiest": "rosy",
    "rostered": "roster",
    "rostering": "roster",
    "rosters": "roster",
    "rostra": "rostrum",
    "rostrums": "rostrum",
    "rosés": "rosé",
    "rotas": "rota",
    "rotated": "rotate",
    "rotates": "rotate",
    "rotating": "rotate",
    "rotations": "rotation",
    "rotatores": "rotator",
    "rotators": "rotator",
    "rotaviruses": "rotavirus",
    "rotifers": "rotifer",
    "rotisseries": "rotisserie",
    "rotors": "rotor",
    "rots": "rot",
    "rotted": "rot",
    "rotters": "rotter",
    "rotting": "rot",
    "rottweilers": "rottweiler",
    "rotundas": "rotunda",
    "rotunder": "rotund",
    "rotundest": "rotund",
    "roubles": "rouble",
    "roues": "roue",
    "rouged": "rouge",
    "rougeing": "rouge",
    "rouges": "rouge",
    "roughages": "roughage",
    "roughcasting": "roughcast",
    "roughcasts": "roughcast",
    "roughed": "rough",
    "roughened": "roughen",
    "roughening": "roughen",
    "roughens": "roughen",
    "rougher": "rough",
    "roughest": "rough",
    "roughing": "rough",
    "roughnesses": "roughness",
    "roughs": "rough",
    "rouging": "rouge",
    "roulades": "roulade",
    "round-tables": "round-table",
    "round-ups": "round-up",
    "roundabouts": "roundabout",
    "rounded": "round",
    "roundels": "roundel",
    "rounder": "round",
    "roundest": "round",
    "rounding": "round",
    "rounds": "round",
    "roundtables": "roundtable",
    "roundups": "roundup",
    "roundworms": "roundworm",
    "roused": "rouse",
    "rouses": "rouse",
    "rousing": "rouse",
    "routed": "route",
    "routers": "router",
    "routes": "route",
    "routines": "routine",
    "routing": "route",
    "routs": "rout",
    "roved": "rove",
    "rovers": "rover",
    "roves": "rove",
    "roving": "rove",
    "rowans": "rowan",
    "rowboats": "rowboat",
    "rowdier": "rowdy",
    "rowdies": "rowdy",
    "rowdiest": "rowdy",
    "rowed": "row",
    "rowers": "rower",
    "rowing": "row",
    "rowlocks": "rowlock",
    "rows": "row",
    "royaler": "royal",
    "royalest": "royal",
    "royalister": "royalist",
    "royalistest": "royalist",
    "royalists": "royalist",
    "royals": "royal",
    "royalties": "royalty",
    "rs": "r",
    "rubbed": "rub",
    "rubber-stamped": "rubber-stamp",
    "rubber-stamping": "rubber-stamp",
    "rubber-stamps": "rubber-stamp",
    "rubberier": "rubbery",
    "rubberiest": "rubbery",
    "rubberised": "rubberise",
    "rubberises": "rubberise",
    "rubberising": "rubberise",
    "rubbers": "rubber",
    "rubbing": "rub",
    "rubbings": "rubbing",
    "rubbished": "rubbish",
    "rubbishes": "rubbish",
    "rubbishing": "rubbish",
    "rubbles": "rubble",
    "rubies": "ruby",
    "rubles": "ruble",
    "rubrics": "rubric",
    "rubs": "rub",
    "rucked": "ruck",
    "rucking": "ruck",
    "rucks": "ruck",
    "rucksacks": "rucksack",
    "ruckuses": "ruckus",
    "ructions": "ruction",
    "rudders": "rudder",
    "ruddier": "ruddy",
    "ruddiest": "ruddy",
    "rudenesses": "rudeness",
    "ruder": "rude",
    "rudest": "rude",
    "rudiments": "rudiment",
    "rued": "rue",
    "rues": "rue",
    "ruffed": "ruff",
    "ruffians": "ruffian",
    "ruffing": "ruff",
    "ruffled": "ruffle",
    "ruffles": "ruffle",
    "ruffling": "ruffle",
    "ruffs": "ruff",
    "ruggeder": "rugged",
    "ruggedest": "rugged",
    "ruggedised": "ruggedise",
    "ruggedises": "ruggedise",
    "ruggedising": "ruggedise",
    "rugs": "rug",
    "ruined": "ruin",
    "ruing": "rue",
    "ruining": "ruin",
    "ruins": "ruin",
    "ruled": "rule",
    "rulers": "ruler",
    "rules": "rule",
    "ruling": "rule",
    "rulings": "ruling",
    "rumbled": "rumble",
    "rumbles": "rumble",
    "rumbling": "rumble",
    "rumblings": "rumbling",
    "rumens": "rumen",
    "rumina": "rumen",
    "ruminants": "ruminant",
    "ruminated": "ruminate",
    "ruminates": "ruminate",
    "ruminating": "ruminate",
    "ruminations": "rumination",
    "rummaged": "rummage",
    "rummages": "rummage",
    "rummaging": "rummage",
    "rummer": "rum",
    "rummest": "rum",
    "rumored": "rumor",
    "rumoring": "rumor",
    "rumors": "rumor",
    "rumoured": "rumour",
    "rumouring": "rumour",
    "rumours": "rumour",
    "rumpled": "rumple",
    "rumples": "rumple",
    "rumpling": "rumple",
    "rumps": "rump",
    "rumpsteaks": "rumpsteak",
    "rumpuses": "rumpus",
    "rums": "rum",
    "run-ins": "run-in",
    "run-offs": "run-off",
    "run-throughed": "run-through",
    "run-throughing": "run-through",
    "run-throughs": "run-through",
    "run-times": "run-time",
    "run-ups": "run-up",
    "runabouts": "runabout",
    "runaways": "runaway",
    "rundowns": "rundown",
    "runes": "rune",
    "rung": "ring",
    "rungs": "rung",
    "runners": "runner",
    "runners-up": "runner-up",
    "runnier": "runny",
    "runniest": "runny",
    "running": "run",
    "runnings": "running",
    "runoffs": "runoff",
    "runs": "run",
    "runtimes": "runtime",
    "runts": "runt",
    "runways": "runway",
    "rupees": "rupee",
    "ruptured": "rupture",
    "ruptures": "rupture",
    "rupturing": "rupture",
    "ruses": "ruse",
    "rush-hours": "rush-hour",
    "rushed": "rush",
    "rushes": "rush",
    "rushier": "rushy",
    "rushing": "rush",
    "rusks": "rusk",
    "russets": "russet",
    "rusted": "rust",
    "rusticated": "rusticate",
    "rusticates": "rusticate",
    "rusticating": "rusticate",
    "rustics": "rustic",
    "rustier": "rusty",
    "rustiest": "rusty",
    "rusting": "rust",
    "rustled": "rustle",
    "rustles": "rustle",
    "rustling": "rustle",
    "rusts": "rust",
    "ruts": "rut",
    "rutted": "rut",
    "rutting": "rut",
    "ryegrasses": "ryegrass",
    "ryes": "rye",
    "régimes": "régime",
    "s'": "'s",
    "sabbaths": "sabbath",
    "sabbaticals": "sabbatical",
    "sabbats": "sabbat",
    "sabers": "saber",
    "sables": "sable",
    "sabotaged": "sabotage",
    "sabotages": "sabotage",
    "sabotaging": "sabotage",
    "saboteurs": "saboteur",
    "sabres": "sabre",
    "saccades": "saccade",
    "sachets": "sachet",
    "sacked": "sack",
    "sackfuls": "sackful",
    "sacking": "sack",
    "sackings": "sacking",
    "sackloads": "sackload",
    "sacks": "sack",
    "sacksful": "sackful",
    "sacra": "sacrum",
    "sacraments": "sacrament",
    "sacrificed": "sacrifice",
    "sacrifices": "sacrifice",
    "sacrificing": "sacrifice",
    "sacrileges": "sacrilege",
    "sacristies": "sacristy",
    "sacrums": "sacrum",
    "sacs": "sac",
    "saddened": "sadden",
    "saddening": "sadden",
    "saddens": "sadden",
    "sadder": "sad",
    "saddest": "sad",
    "saddlebags": "saddlebag",
    "saddled": "saddle",
    "saddlers": "saddler",
    "saddles": "saddle",
    "saddling": "saddle",
    "sadists": "sadist",
    "safaris": "safari",
    "safe-conducts": "safe-conduct",
    "safeguarded": "safeguard",
    "safeguarding": "safeguard",
    "safeguards": "safeguard",
    "safer": "safe",
    "safes": "safe",
    "safest": "safe",
    "safeties": "safety",
    "safety-valves": "safety-valve",
    "safflowers": "safflower",
    "saffrons": "saffron",
    "sagas": "saga",
    "sages": "sage",
    "sagged": "sag",
    "sagging": "sag",
    "sags": "sag",
    "sahibs": "sahib",
    "said": "say",
    "saigas": "saiga",
    "sailboats": "sailboat",
    "sailed": "sail",
    "sailing": "sail",
    "sailings": "sailing",
    "sailmakers": "sailmaker",
    "sailors": "sailor",
    "sails": "sail",
    "saintlier": "saintly",
    "saintliest": "saintly",
    "saints": "saint",
    "sakes": "sake",
    "salaamed": "salaam",
    "salaaming": "salaam",
    "salaams": "salaam",
    "salads": "salad",
    "salamanders": "salamander",
    "salamis": "salami",
    "salaries": "salary",
    "salerooms": "saleroom",
    "sales": "sale",
    "salesgirls": "salesgirl",
    "salesmen": "salesman",
    "salespeople": "salesperson",
    "salespersons": "salesperson",
    "saleswomen": "saleswoman",
    "salicylates": "salicylate",
    "salients": "salient",
    "salinas": "salina",
    "salines": "saline",
    "salinities": "salinity",
    "salivas": "saliva",
    "salivated": "salivate",
    "salivates": "salivate",
    "salivating": "salivate",
    "sallied": "sally",
    "sallies": "sally",
    "sallower": "sallow",
    "sallowest": "sallow",
    "sallows": "sallow",
    "sallying": "sally",
    "salmonellae": "salmonella",
    "salmonellas": "salmonella",
    "salmonelloses": "salmonellosis",
    "salmonids": "salmonid",
    "salmons": "salmon",
    "salons": "salon",
    "saloons": "saloon",
    "salt-marshes": "salt-marsh",
    "salted": "salt",
    "saltier": "salty",
    "saltiest": "salty",
    "salting": "salt",
    "saltmarshes": "saltmarsh",
    "salts": "salt",
    "salutations": "salutation",
    "saluted": "salute",
    "salutes": "salute",
    "saluting": "salute",
    "salvaged": "salvage",
    "salvages": "salvage",
    "salvaging": "salvage",
    "salved": "salve",
    "salvers": "salver",
    "salves": "salve",
    "salving": "salve",
    "salvoes": "salvo",
    "salvos": "salvo",
    "sambaed": "samba",
    "sambaing": "samba",
    "sambas": "samba",
    "samenesses": "sameness",
    "samovars": "samovar",
    "sampans": "sampan",
    "samphires": "samphire",
    "sampled": "sample",
    "samplers": "sampler",
    "samples": "sample",
    "sampling": "sample",
    "samplings": "sampling",
    "sanatoria": "sanatorium",
    "sanatoriums": "sanatorium",
    "sancta": "sanctum",
    "sanctifications": "sanctification",
    "sanctified": "sanctify",
    "sanctifies": "sanctify",
    "sanctifying": "sanctify",
    "sanctioned": "sanction",
    "sanctioning": "sanction",
    "sanctions": "sanction",
    "sanctuaries": "sanctuary",
    "sanctums": "sanctum",
    "sandals": "sandal",
    "sandalwoods": "sandalwood",
    "sandbagged": "sandbag",
    "sandbagging": "sandbag",
    "sandbags": "sandbag",
    "sandbanks": "sandbank",
    "sandblasted": "sandblast",
    "sandblasting": "sandblast",
    "sandblasts": "sandblast",
    "sandboxes": "sandbox",
    "sandcastles": "sandcastle",
    "sanded": "sand",
    "sanders": "sander",
    "sandflies": "sandfly",
    "sandhills": "sandhill",
    "sandier": "sandy",
    "sandiest": "sandy",
    "sanding": "sand",
    "sandpapered": "sandpaper",
    "sandpapering": "sandpaper",
    "sandpapers": "sandpaper",
    "sandpipers": "sandpiper",
    "sandpits": "sandpit",
    "sands": "sand",
    "sandstones": "sandstone",
    "sandstorms": "sandstorm",
    "sandwiched": "sandwich",
    "sandwiches": "sandwich",
    "sandwiching": "sandwich",
    "saner": "sane",
    "sanest": "sane",
    "sang": "sing",
    "sanitariums": "sanitarium",
    "sanitised": "sanitise",
    "sanitisers": "sanitiser",
    "sanitises": "sanitise",
    "sanitising": "sanitise",
    "sanitized": "sanitize",
    "sanitizers": "sanitizer",
    "sanitizes": "sanitize",
    "sanitizing": "sanitize",
    "sank": "sink",
    "sans": "san",
    "saplings": "sapling",
    "saponins": "saponin",
    "sapped": "sap",
    "sapphires": "sapphire",
    "sappier": "sappy",
    "sapping": "sap",
    "saprophytes": "saprophyte",
    "saps": "sap",
    "sarcasms": "sarcasm",
    "sarcoidoses": "sarcoidosis",
    "sarcoids": "sarcoid",
    "sarcomas": "sarcoma",
    "sarcomata": "sarcoma",
    "sarcomeres": "sarcomere",
    "sarcophagi": "sarcophagus",
    "sarcophaguses": "sarcophagus",
    "sardines": "sardine",
    "saris": "sari",
    "sarongs": "sarong",
    "sashes": "sash",
    "sassier": "sassy",
    "sassiest": "sassy",
    "sat": "sit",
    "satchels": "satchel",
    "sated": "sate",
    "satellites": "satellite",
    "sates": "sate",
    "satiated": "satiate",
    "satiates": "satiate",
    "satiating": "satiate",
    "satieties": "satiety",
    "sating": "sate",
    "satired": "satire",
    "satires": "satire",
    "satiring": "satire",
    "satirised": "satirise",
    "satirises": "satirise",
    "satirising": "satirise",
    "satirists": "satirist",
    "satirized": "satirize",
    "satirizes": "satirize",
    "satirizing": "satirize",
    "satisfactions": "satisfaction",
    "satisfied": "satisfy",
    "satisfies": "satisfy",
    "satisfying": "satisfy",
    "satsumas": "satsuma",
    "saturated": "saturate",
    "saturates": "saturate",
    "saturating": "saturate",
    "saturations": "saturation",
    "satyrs": "satyr",
    "sauce-boats": "sauce-boat",
    "saucepans": "saucepan",
    "saucers": "saucer",
    "sauces": "sauce",
    "saucier": "saucy",
    "sauciest": "saucy",
    "saunas": "sauna",
    "sauntered": "saunter",
    "sauntering": "saunter",
    "saunters": "saunter",
    "sauropods": "sauropod",
    "sausages": "sausage",
    "sauteed": "saute",
    "sauteing": "saute",
    "sautes": "saute",
    "savaged": "savage",
    "savageries": "savagery",
    "savages": "savage",
    "savaging": "savage",
    "savannahs": "savannah",
    "savannas": "savanna",
    "savants": "savant",
    "saved": "save",
    "savers": "saver",
    "saves": "save",
    "saving": "save",
    "savings": "saving",
    "saviors": "savior",
    "saviours": "saviour",
    "savored": "savor",
    "savoring": "savor",
    "savors": "savor",
    "savoured": "savour",
    "savouries": "savoury",
    "savouring": "savour",
    "savours": "savour",
    "savvier": "savvy",
    "savviest": "savvy",
    "saw": "see",
    "sawdusts": "sawdust",
    "sawed": "saw",
    "sawflies": "sawfly",
    "sawhorses": "sawhorse",
    "sawing": "saw",
    "sawmills": "sawmill",
    "sawn": "saw",
    "saws": "saw",
    "sawyers": "sawyer",
    "saxes": "sax",
    "saxophones": "saxophone",
    "saxophonists": "saxophonist",
    "sayas": "saya",
    "saying": "say",
    "sayings": "saying",
    "says": "say",
    "scabbards": "scabbard",
    "scabbed": "scab",
    "scabbier": "scabby",
    "scabbiest": "scabby",
    "scabbing": "scab",
    "scabs": "scab",
    "scaffolded": "scaffold",
    "scaffolders": "scaffolder",
    "scaffolding": "scaffold",
    "scaffolds": "scaffold",
    "scalae": "scala",
    "scalars": "scalar",
    "scalded": "scald",
    "scalding": "scald",
    "scalds": "scald",
    "scaled": "scale",
    "scalers": "scaler",
    "scales": "scale",
    "scalier": "scaly",
    "scaliest": "scaly",
    "scaling": "scale",
    "scalings": "scaling",
    "scallions": "scallion",
    "scalloped": "scallop",
    "scalloping": "scallop",
    "scallops": "scallop",
    "scallywags": "scallywag",
    "scalped": "scalp",
    "scalpels": "scalpel",
    "scalping": "scalp",
    "scalps": "scalp",
    "scammed": "scam",
    "scammers": "scammer",
    "scamming": "scam",
    "scamped": "scamp",
    "scampered": "scamper",
    "scampering": "scamper",
    "scampers": "scamper",
    "scamping": "scamp",
    "scamps": "scamp",
    "scams": "scam",
    "scandalized": "scandalize",
    "scandalizes": "scandalize",
    "scandalizing": "scandalize",
    "scandalmongers": "scandalmonger",
    "scandals": "scandal",
    "scanned": "scan",
    "scanners": "scanner",
    "scanning": "scan",
    "scans": "scan",
    "scansions": "scansion",
    "scanter": "scant",
    "scantest": "scant",
    "scantier": "scanty",
    "scantiest": "scanty",
    "scaped": "scape",
    "scapegoated": "scapegoat",
    "scapegoating": "scapegoat",
    "scapegoats": "scapegoat",
    "scapes": "scape",
    "scaping": "scape",
    "scapulae": "scapula",
    "scapulas": "scapula",
    "scarabs": "scarab",
    "scarcer": "scarce",
    "scarcest": "scarce",
    "scarcities": "scarcity",
    "scarecrows": "scarecrow",
    "scared": "scare",
    "scaremongers": "scaremonger",
    "scares": "scare",
    "scarfs": "scarf",
    "scarier": "scary",
    "scariest": "scary",
    "scarifications": "scarification",
    "scarified": "scarify",
    "scarifies": "scarify",
    "scarifying": "scarify",
    "scaring": "scare",
    "scarlets": "scarlet",
    "scarpered": "scarper",
    "scarpering": "scarper",
    "scarpers": "scarper",
    "scarred": "scar",
    "scarring": "scar",
    "scars": "scar",
    "scarves": "scarf",
    "scats": "scat",
    "scatterbrains": "scatterbrain",
    "scattered": "scatter",
    "scatterers": "scatterer",
    "scattering": "scatter",
    "scatterings": "scattering",
    "scatterplots": "scatterplot",
    "scatters": "scatter",
    "scattier": "scatty",
    "scattiest": "scatty",
    "scavenged": "scavenge",
    "scavengers": "scavenger",
    "scavenges": "scavenge",
    "scavenging": "scavenge",
    "scenarios": "scenario",
    "scene-shifters": "scene-shifter",
    "scenes": "scene",
    "scented": "scent",
    "scenting": "scent",
    "scents": "scent",
    "scepters": "scepter",
    "sceptics": "sceptic",
    "sceptres": "sceptre",
    "scheduled": "schedule",
    "schedulers": "scheduler",
    "schedules": "schedule",
    "scheduling": "schedule",
    "schemas": "schema",
    "schemata": "schema",
    "schematics": "schematic",
    "schemed": "scheme",
    "schemers": "schemer",
    "schemes": "scheme",
    "scheming": "scheme",
    "scherzi": "scherzo",
    "scherzos": "scherzo",
    "schismatics": "schismatic",
    "schisms": "schism",
    "schistosomes": "schistosome",
    "schistosomiases": "schistosomiasis",
    "schists": "schist",
    "schizophrenias": "schizophrenia",
    "schizophrenics": "schizophrenic",
    "schmaltzes": "schmaltz",
    "schmaltzier": "schmaltzy",
    "schmalzier": "schmalzy",
    "schmoozed": "schmooze",
    "schmoozes": "schmooze",
    "schmoozing": "schmooze",
    "scholars": "scholar",
    "scholarships": "scholarship",
    "school-children": "school-child",
    "school-leavers": "school-leaver",
    "schoolbooks": "schoolbook",
    "schoolboys": "schoolboy",
    "schoolchildren": "schoolchild",
    "schooldays": "schoolday",
    "schooled": "school",
    "schoolgirls": "schoolgirl",
    "schoolhouses": "schoolhouse",
    "schooling": "school",
    "schoolings": "schooling",
    "schoolmasters": "schoolmaster",
    "schoolmates": "schoolmate",
    "schoolmistresses": "schoolmistress",
    "schoolrooms": "schoolroom",
    "schools": "school",
    "schoolteachers": "schoolteacher",
    "schooners": "schooner",
    "sciaticas": "sciatica",
    "sciences": "science",
    "scientisms": "scientism",
    "scientists": "scientist",
    "scimitars": "scimitar",
    "scintigraphies": "scintigraphy",
    "scintillated": "scintillate",
    "scintillates": "scintillate",
    "scintillating": "scintillate",
    "scintillations": "scintillation",
    "scintillators": "scintillator",
    "scions": "scion",
    "scissored": "scissor",
    "scissoring": "scissor",
    "scissors": "scissor",
    "sclerae": "sclera",
    "scleras": "sclera",
    "sclerodermas": "scleroderma",
    "sclerodermata": "scleroderma",
    "scleroses": "sclerosis",
    "sclerotherapies": "sclerotherapy",
    "scoffed": "scoff",
    "scoffing": "scoff",
    "scoffs": "scoff",
    "scolded": "scold",
    "scolding": "scold",
    "scoldings": "scolding",
    "scolds": "scold",
    "scolioses": "scoliosis",
    "scones": "scone",
    "scooped": "scoop",
    "scoopfuls": "scoopful",
    "scooping": "scoop",
    "scoops": "scoop",
    "scooted": "scoot",
    "scooters": "scooter",
    "scooting": "scoot",
    "scoots": "scoot",
    "scoped": "scope",
    "scopes": "scope",
    "scoping": "scope",
    "scorched": "scorch",
    "scorchers": "scorcher",
    "scorches": "scorch",
    "scorching": "scorch",
    "scoreboards": "scoreboard",
    "scorecards": "scorecard",
    "scored": "score",
    "scorers": "scorer",
    "scores": "score",
    "scoring": "score",
    "scorned": "scorn",
    "scorning": "scorn",
    "scorns": "scorn",
    "scorpions": "scorpion",
    "scotched": "scotch",
    "scotches": "scotch",
    "scotching": "scotch",
    "scoters": "scoter",
    "scotomas": "scotoma",
    "scotomata": "scotoma",
    "scoundrels": "scoundrel",
    "scoured": "scour",
    "scourers": "scourer",
    "scourged": "scourge",
    "scourges": "scourge",
    "scourging": "scourge",
    "scouring": "scour",
    "scours": "scour",
    "scouted": "scout",
    "scouting": "scout",
    "scoutmasters": "scoutmaster",
    "scouts": "scout",
    "scowled": "scowl",
    "scowling": "scowl",
    "scowls": "scowl",
    "scrabbled": "scrabble",
    "scrabbles": "scrabble",
    "scrabbling": "scrabble",
    "scraggier": "scraggy",
    "scraggiest": "scraggy",
    "scrambled": "scramble",
    "scramblers": "scrambler",
    "scrambles": "scramble",
    "scrambling": "scramble",
    "scrammed": "scram",
    "scramming": "scram",
    "scrams": "scram",
    "scrap-heaps": "scrap-heap",
    "scrapbooks": "scrapbook",
    "scraped": "scrape",
    "scrapers": "scraper",
    "scrapes": "scrape",
    "scrapies": "scrapie",
    "scraping": "scrape",
    "scrapped": "scrap",
    "scrappers": "scrapper",
    "scrappier": "scrappy",
    "scrappiest": "scrappy",
    "scrapping": "scrap",
    "scraps": "scrap",
    "scratchcards": "scratchcard",
    "scratched": "scratch",
    "scratches": "scratch",
    "scratchier": "scratchy",
    "scratching": "scratch",
    "scrawled": "scrawl",
    "scrawling": "scrawl",
    "scrawls": "scrawl",
    "scrawnier": "scrawny",
    "scrawniest": "scrawny",
    "screamed": "scream",
    "screamers": "screamer",
    "screaming": "scream",
    "screams": "scream",
    "screeched": "screech",
    "screeches": "screech",
    "screeching": "screech",
    "screeds": "screed",
    "screened": "screen",
    "screeners": "screener",
    "screening": "screen",
    "screenings": "screening",
    "screenplays": "screenplay",
    "screens": "screen",
    "screenwriters": "screenwriter",
    "screes": "scree",
    "screw-tops": "screw-top",
    "screwdrivers": "screwdriver",
    "screwed": "screw",
    "screwier": "screwy",
    "screwiest": "screwy",
    "screwing": "screw",
    "screws": "screw",
    "scribbled": "scribble",
    "scribblers": "scribbler",
    "scribbles": "scribble",
    "scribbling": "scribble",
    "scribed": "scribe",
    "scribes": "scribe",
    "scribing": "scribe",
    "scrimmages": "scrimmage",
    "scrimped": "scrimp",
    "scrimping": "scrimp",
    "scrimps": "scrimp",
    "scripted": "script",
    "scripting": "script",
    "scriptoria": "scriptorium",
    "scripts": "script",
    "scriptures": "scripture",
    "scriptwriters": "scriptwriter",
    "scriveners": "scrivener",
    "scrolled": "scroll",
    "scrolling": "scroll",
    "scrolls": "scroll",
    "scrota": "scrotum",
    "scrotums": "scrotum",
    "scrounged": "scrounge",
    "scroungers": "scrounger",
    "scrounges": "scrounge",
    "scrounging": "scrounge",
    "scrs": "scr",
    "scrubbed": "scrub",
    "scrubbers": "scrubber",
    "scrubbier": "scrubby",
    "scrubbiest": "scrubby",
    "scrubbing": "scrub",
    "scrubs": "scrub",
    "scruffier": "scruffy",
    "scruffiest": "scruffy",
    "scruffs": "scruff",
    "scrummaged": "scrummage",
    "scrummages": "scrummage",
    "scrummaging": "scrummage",
    "scrums": "scrum",
    "scrunched": "scrunch",
    "scrunches": "scrunch",
    "scrunching": "scrunch",
    "scrupled": "scruple",
    "scruples": "scruple",
    "scrupling": "scruple",
    "scrutineers": "scrutineer",
    "scrutinies": "scrutiny",
    "scrutinised": "scrutinise",
    "scrutinises": "scrutinise",
    "scrutinising": "scrutinise",
    "scrutinized": "scrutinize",
    "scrutinizes": "scrutinize",
    "scrutinizing": "scrutinize",
    "scudded": "scud",
    "scudding": "scud",
    "scuds": "scud",
    "scuffed": "scuff",
    "scuffing": "scuff",
    "scuffled": "scuffle",
    "scuffles": "scuffle",
    "scuffling": "scuffle",
    "scuffs": "scuff",
    "sculled": "scull",
    "sculleries": "scullery",
    "scullers": "sculler",
    "sculling": "scull",
    "sculls": "scull",
    "sculpted": "sculpt",
    "sculpting": "sculpt",
    "sculptors": "sculptor",
    "sculpts": "sculpt",
    "sculptured": "sculpture",
    "sculptures": "sculpture",
    "sculpturing": "sculpture",
    "scummier": "scummy",
    "scums": "scum",
    "scuppered": "scupper",
    "scuppering": "scupper",
    "scuppers": "scupper",
    "scurfier": "scurfy",
    "scurried": "scurry",
    "scurries": "scurry",
    "scurrying": "scurry",
    "scuttled": "scuttle",
    "scuttles": "scuttle",
    "scuttling": "scuttle",
    "scythed": "scythe",
    "scythes": "scythe",
    "scything": "scythe",
    "sea-beds": "sea-bed",
    "sea-birds": "sea-bird",
    "sea-levels": "sea-level",
    "sea-waters": "sea-water",
    "seabeds": "seabed",
    "seabirds": "seabird",
    "seaboards": "seaboard",
    "seafarers": "seafarer",
    "seafoods": "seafood",
    "seafronts": "seafront",
    "seagrasses": "seagrass",
    "seagulls": "seagull",
    "seahorses": "seahorse",
    "sealants": "sealant",
    "sealed": "seal",
    "sealers": "sealer",
    "sealing": "seal",
    "sealions": "sealion",
    "seals": "seal",
    "seamed": "seam",
    "seamen": "seaman",
    "seamier": "seamy",
    "seamiest": "seamy",
    "seaming": "seam",
    "seamlesser": "seamless",
    "seamlessest": "seamless",
    "seams": "seam",
    "seamstresses": "seamstress",
    "seances": "seance",
    "seaplanes": "seaplane",
    "seaports": "seaport",
    "searched": "search",
    "searchers": "searcher",
    "searches": "search",
    "searching": "search",
    "searchlights": "searchlight",
    "seared": "sear",
    "searing": "sear",
    "sears": "sear",
    "seas": "sea",
    "seascapes": "seascape",
    "seashells": "seashell",
    "seashores": "seashore",
    "seasides": "seaside",
    "seasonalities": "seasonality",
    "seasoned": "season",
    "seasoning": "season",
    "seasonings": "seasoning",
    "seasons": "season",
    "seat-belts": "seat-belt",
    "seatbelts": "seatbelt",
    "seated": "seat",
    "seating": "seat",
    "seats": "seat",
    "seawards": "seaward",
    "seawaters": "seawater",
    "seaweeds": "seaweed",
    "seaworthier": "seaworthy",
    "seaworthiest": "seaworthy",
    "secants": "secant",
    "seceded": "secede",
    "secedes": "secede",
    "seceding": "secede",
    "secessionists": "secessionist",
    "secessions": "secession",
    "secluded": "seclude",
    "secludes": "seclude",
    "secluding": "seclude",
    "seclusions": "seclusion",
    "second": "2",
    "second-guessed": "second-guess",
    "second-guesses": "second-guess",
    "second-guessing": "second-guess",
    "secondaries": "secondary",
    "seconded": "second",
    "seconders": "seconder",
    "secondes": "seconde",
    "seconding": "second",
    "secondments": "secondment",
    "seconds": "second",
    "seconds-in-command": "second-in-command",
    "secretariats": "secretariat",
    "secretaries": "secretary",
    "secretaries-general": "secretary-general",
    "secretary-generals": "secretary-general",
    "secreted": "secrete",
    "secretes": "secrete",
    "secreting": "secrete",
    "secretions": "secretion",
    "secrets": "secret",
    "secs": "sec",
    "sectarians": "sectarian",
    "sectioned": "section",
    "sectioning": "section",
    "sections": "section",
    "sectors": "sector",
    "sects": "sect",
    "secularised": "secularise",
    "secularises": "secularise",
    "secularising": "secularise",
    "secularists": "secularist",
    "secularized": "secularize",
    "secularizes": "secularize",
    "secularizing": "secularize",
    "secured": "secure",
    "securer": "secure",
    "secures": "secure",
    "securest": "secure",
    "securing": "secure",
    "securities": "security",
    "sedans": "sedan",
    "sedated": "sedate",
    "sedates": "sedate",
    "sedating": "sedate",
    "sedations": "sedation",
    "sedatives": "sedative",
    "sedges": "sedge",
    "sedgier": "sedgy",
    "sedimentations": "sedimentation",
    "sediments": "sediment",
    "seds": "sed",
    "seduced": "seduce",
    "seducers": "seducer",
    "seduces": "seduce",
    "seducing": "seduce",
    "seductions": "seduction",
    "seductresses": "seductress",
    "see-saws": "see-saw",
    "seedbeds": "seedbed",
    "seedcakes": "seedcake",
    "seeded": "seed",
    "seeders": "seeder",
    "seedier": "seedy",
    "seediest": "seedy",
    "seeding": "seed",
    "seedings": "seeding",
    "seedlings": "seedling",
    "seeds": "seed",
    "seeing": "see",
    "seekers": "seeker",
    "seeking": "seek",
    "seeks": "seek",
    "seemed": "seem",
    "seeming": "seem",
    "seemlier": "seemly",
    "seems": "seem",
    "seen": "see",
    "seeped": "seep",
    "seeping": "seep",
    "seeps": "seep",
    "seers": "seer",
    "sees": "see",
    "seesaws": "seesaw",
    "seethed": "seethe",
    "seethes": "seethe",
    "seething": "seethe",
    "segmentations": "segmentation",
    "segmented": "segment",
    "segmenting": "segment",
    "segments": "segment",
    "segregated": "segregate",
    "segregates": "segregate",
    "segregating": "segregate",
    "segregations": "segregation",
    "segs": "seg",
    "segued": "segue",
    "segues": "segue",
    "seguing": "segue",
    "seiners": "seiner",
    "seines": "seine",
    "seismograms": "seismogram",
    "seismographs": "seismograph",
    "seismologists": "seismologist",
    "seismometers": "seismometer",
    "seized": "seize",
    "seizes": "seize",
    "seizing": "seize",
    "seizures": "seizure",
    "selected": "select",
    "selecting": "select",
    "selections": "selection",
    "selectivities": "selectivity",
    "selectors": "selector",
    "selects": "select",
    "self-administered": "self-administer",
    "self-administering": "self-administer",
    "self-administers": "self-administer",
    "self-analyses": "self-analysis",
    "self-assembled": "self-assemble",
    "self-assembles": "self-assemble",
    "self-assemblies": "self-assembly",
    "self-assembling": "self-assemble",
    "self-beliefs": "self-belief",
    "self-completions": "self-completion",
    "self-concepts": "self-concept",
    "self-contradictions": "self-contradiction",
    "self-corrected": "self-correct",
    "self-correcting": "self-correct",
    "self-corrects": "self-correct",
    "self-criticisms": "self-criticism",
    "self-deceptions": "self-deception",
    "self-definitions": "self-definition",
    "self-described": "self-describe",
    "self-describes": "self-describe",
    "self-describing": "self-describe",
    "self-descriptions": "self-description",
    "self-destructed": "self-destruct",
    "self-destructing": "self-destruct",
    "self-destructs": "self-destruct",
    "self-doubts": "self-doubt",
    "self-efficacies": "self-efficacy",
    "self-examinations": "self-examination",
    "self-exposures": "self-exposure",
    "self-fertilised": "self-fertilise",
    "self-fertilises": "self-fertilise",
    "self-fertilising": "self-fertilise",
    "self-harmed": "self-harm",
    "self-harming": "self-harm",
    "self-harms": "self-harm",
    "self-identified": "self-identify",
    "self-identifies": "self-identify",
    "self-identifying": "self-identify",
    "self-identities": "self-identity",
    "self-images": "self-image",
    "self-injuries": "self-injury",
    "self-interests": "self-interest",
    "self-justifications": "self-justification",
    "self-limited": "self-limit",
    "self-limiting": "self-limit",
    "self-limits": "self-limit",
    "self-medications": "self-medication",
    "self-monitored": "self-monitor",
    "self-monitoring": "self-monitor",
    "self-monitors": "self-monitor",
    "self-organised": "self-organise",
    "self-organises": "self-organise",
    "self-organising": "self-organise",
    "self-organized": "self-organize",
    "self-organizes": "self-organize",
    "self-organizing": "self-organize",
    "self-perceptions": "self-perception",
    "self-portraits": "self-portrait",
    "self-presentations": "self-presentation",
    "self-reflections": "self-reflection",
    "self-regulated": "self-regulate",
    "self-regulates": "self-regulate",
    "self-regulating": "self-regulate",
    "self-replicated": "self-replicate",
    "self-replicates": "self-replicate",
    "self-replicating": "self-replicate",
    "self-reports": "self-report",
    "self-representations": "self-representation",
    "self-revelations": "self-revelation",
    "self-sacrifices": "self-sacrifice",
    "self-selected": "self-select",
    "self-selecting": "self-select",
    "self-selects": "self-select",
    "self-starters": "self-starter",
    "self-studies": "self-study",
    "self-sustained": "self-sustain",
    "self-sustaining": "self-sustain",
    "self-sustains": "self-sustain",
    "self-testings": "self-testing",
    "self-treatments": "self-treatment",
    "sell-outs": "sell-out",
    "sellers": "seller",
    "selling": "sell",
    "sellouts": "sellout",
    "sells": "sell",
    "selvedges": "selvedge",
    "selves": "self",
    "semantics": "semantic",
    "semaphores": "semaphore",
    "semblances": "semblance",
    "semesters": "semester",
    "semi-circles": "semi-circle",
    "semi-colons": "semi-colon",
    "semi-conductors": "semi-conductor",
    "semi-deserts": "semi-desert",
    "semi-detacheds": "semi-detached",
    "semi-finalists": "semi-finalist",
    "semi-finals": "semi-final",
    "semi-professionals": "semi-professional",
    "semibreves": "semibreve",
    "semicircles": "semicircle",
    "semicolons": "semicolon",
    "semiconductors": "semiconductor",
    "semifinals": "semifinal",
    "semigroups": "semigroup",
    "seminarians": "seminarian",
    "seminaries": "seminary",
    "seminars": "seminar",
    "seminomas": "seminoma",
    "seminomata": "seminoma",
    "semiologies": "semiology",
    "semioticians": "semiotician",
    "semiotics": "semiotic",
    "semiquavers": "semiquaver",
    "semis": "semi",
    "semitones": "semitone",
    "senates": "senate",
    "senators": "senator",
    "send-offs": "send-off",
    "send-ups": "send-up",
    "senders": "sender",
    "sending": "send",
    "sends": "send",
    "seniorities": "seniority",
    "seniors": "senior",
    "sensationalised": "sensationalise",
    "sensationalises": "sensationalise",
    "sensationalising": "sensationalise",
    "sensationalists": "sensationalist",
    "sensations": "sensation",
    "sensed": "sense",
    "senseis": "sensei",
    "senses": "sense",
    "sensibilities": "sensibility",
    "sensing": "sense",
    "sensitisations": "sensitisation",
    "sensitised": "sensitise",
    "sensitisers": "sensitiser",
    "sensitises": "sensitise",
    "sensitising": "sensitise",
    "sensitivities": "sensitivity",
    "sensitizations": "sensitization",
    "sensitized": "sensitize",
    "sensitizes": "sensitize",
    "sensitizing": "sensitize",
    "sensors": "sensor",
    "sensuouser": "sensuous",
    "sensuousest": "sensuous",
    "sent": "send",
    "sentenced": "sentence",
    "sentences": "sentence",
    "sentencing": "sentence",
    "sententiously": "sententious",
    "sentimentalists": "sentimentalist",
    "sentimentalized": "sentimentalize",
    "sentimentalizes": "sentimentalize",
    "sentimentalizing": "sentimentalize",
    "sentiments": "sentiment",
    "sentinels": "sentinel",
    "sentries": "sentry",
    "sepals": "sepal",
    "separated": "separate",
    "separates": "separate",
    "separating": "separate",
    "separations": "separation",
    "separatists": "separatist",
    "separators": "separator",
    "sepia": "sepium",
    "sepses": "sepsis",
    "septa": "septum",
    "septets": "septet",
    "septicaemias": "septicaemia",
    "septuagenarians": "septuagenarian",
    "septums": "septum",
    "sepulchres": "sepulchre",
    "seqs": "seq",
    "sequelae": "sequela",
    "sequelas": "sequela",
    "sequels": "sequel",
    "sequenced": "sequence",
    "sequencers": "sequencer",
    "sequences": "sequence",
    "sequencing": "sequence",
    "sequencings": "sequencing",
    "sequestered": "sequester",
    "sequestering": "sequester",
    "sequesters": "sequester",
    "sequestrated": "sequestrate",
    "sequestrates": "sequestrate",
    "sequestrating": "sequestrate",
    "sequestrations": "sequestration",
    "sequins": "sequin",
    "sera": "serum",
    "seraphim": "seraph",
    "seraphs": "seraph",
    "serenaded": "serenade",
    "serenades": "serenade",
    "serenading": "serenade",
    "serendipities": "serendipity",
    "serener": "serene",
    "serenest": "serene",
    "serfs": "serf",
    "sergeants": "sergeant",
    "serialisations": "serialisation",
    "serialised": "serialise",
    "serialises": "serialise",
    "serialising": "serialise",
    "serializations": "serialization",
    "serialized": "serialize",
    "serializes": "serialize",
    "serializing": "serialize",
    "serials": "serial",
    "serifs": "serif",
    "serines": "serine",
    "sermonized": "sermonize",
    "sermonizes": "sermonize",
    "sermonizing": "sermonize",
    "sermons": "sermon",
    "seroconversions": "seroconversion",
    "serogroups": "serogroup",
    "serologies": "serology",
    "seroprevalences": "seroprevalence",
    "serotypes": "serotype",
    "serpents": "serpent",
    "serrated": "serrate",
    "serrates": "serrate",
    "serrating": "serrate",
    "serrations": "serration",
    "serums": "serum",
    "servants": "servant",
    "served": "serve",
    "servers": "server",
    "serves": "serve",
    "serviced": "service",
    "servicemen": "serviceman",
    "services": "service",
    "servicing": "service",
    "serviettes": "serviette",
    "serving": "serve",
    "servings": "serving",
    "servs": "serv",
    "sesquiterpenes": "sesquiterpene",
    "sessions": "session",
    "sestets": "sestet",
    "set-asides": "set-aside",
    "set-backs": "set-back",
    "set-tos": "set-to",
    "set-ups": "set-up",
    "setae": "seta",
    "setbacks": "setback",
    "setpoints": "setpoint",
    "sets": "set",
    "settees": "settee",
    "setters": "setter",
    "setting": "set",
    "settings": "setting",
    "settled": "settle",
    "settlements": "settlement",
    "settlers": "settler",
    "settles": "settle",
    "settling": "settle",
    "setts": "sett",
    "setups": "setup",
    "seven-hundredth": "700",
    "seventeenth": "17",
    "seventeenths": "seventeenth",
    "seventh": "7",
    "sevenths": "seventh",
    "seventies": "seventy",
    "seventieth": "70",
    "seventieths": "seventieth",
    "seventy-eighth": "78",
    "seventy-eighths": "seventy-eighth",
    "seventy-fifth": "75",
    "seventy-fifths": "seventy-fifth",
    "seventy-first": "71",
    "seventy-firsts": "seventy-first",
    "seventy-fourth": "74",
    "seventy-fourths": "seventy-fourth",
    "seventy-ninth": "79",
    "seventy-ninths": "seventy-ninth",
    "seventy-second": "72",
    "seventy-seconds": "seventy-second",
    "seventy-seventh": "77",
    "seventy-sevenths": "seventy-seventh",
    "seventy-sixth": "76",
    "seventy-sixths": "seventy-sixth",
    "seventy-third": "73",
    "seventy-thirds": "seventy-third",
    "severances": "severance",
    "severed": "sever",
    "severer": "severe",
    "severest": "severe",
    "severing": "sever",
    "severities": "severity",
    "severs": "sever",
    "sewed": "sew",
    "sewers": "sewer",
    "sewing": "sew",
    "sewn": "sew",
    "sews": "sew",
    "sexed": "sex",
    "sexes": "sex",
    "sexier": "sexy",
    "sexiest": "sexy",
    "sexing": "sex",
    "sexists": "sexist",
    "sextants": "sextant",
    "sextets": "sextet",
    "sextiles": "sextile",
    "sextons": "sexton",
    "sexualities": "sexuality",
    "shabbier": "shabby",
    "shabbiest": "shabby",
    "shacked": "shack",
    "shacking": "shack",
    "shackled": "shackle",
    "shackles": "shackle",
    "shackling": "shackle",
    "shacks": "shack",
    "shaded": "shade",
    "shaders": "shader",
    "shades": "shade",
    "shadier": "shady",
    "shadiest": "shady",
    "shading": "shade",
    "shadings": "shading",
    "shadowed": "shadow",
    "shadowier": "shadowy",
    "shadowiest": "shadowy",
    "shadowing": "shadow",
    "shadows": "shadow",
    "shads": "shad",
    "shafted": "shaft",
    "shafting": "shaft",
    "shafts": "shaft",
    "shagged": "shag",
    "shaggier": "shaggy",
    "shaggiest": "shaggy",
    "shagging": "shag",
    "shags": "shag",
    "shake-ups": "shake-up",
    "shakedowns": "shakedown",
    "shaken": "shake",
    "shakers": "shaker",
    "shakes": "shake",
    "shakeups": "shakeup",
    "shakier": "shaky",
    "shakiest": "shaky",
    "shaking": "shake",
    "shales": "shale",
    "shallots": "shallot",
    "shallowed": "shallow",
    "shallower": "shallow",
    "shallowest": "shallow",
    "shallowing": "shallow",
    "shallows": "shallow",
    "shalt": "shall",
    "shamans": "shaman",
    "shambled": "shamble",
    "shambles": "shamble",
    "shambling": "shamble",
    "shamed": "shame",
    "shames": "shame",
    "shaming": "shame",
    "shammed": "sham",
    "shamming": "sham",
    "shampooed": "shampoo",
    "shampooing": "shampoo",
    "shampoos": "shampoo",
    "shamrocks": "shamrock",
    "shams": "sham",
    "shandies": "shandy",
    "shanghaied": "shanghai",
    "shanghaiing": "shanghai",
    "shanghais": "shanghai",
    "shanks": "shank",
    "shanties": "shanty",
    "shantytowns": "shantytown",
    "shaped": "shape",
    "shapelier": "shapely",
    "shapeliest": "shapely",
    "shapers": "shaper",
    "shapes": "shape",
    "shaping": "shape",
    "shards": "shard",
    "share-outs": "share-out",
    "sharecroppers": "sharecropper",
    "shared": "share",
    "shareholders": "shareholder",
    "shareholdings": "shareholding",
    "sharers": "sharer",
    "shares": "share",
    "sharing": "share",
    "sharks": "shark",
    "sharpened": "sharpen",
    "sharpeners": "sharpener",
    "sharpening": "sharpen",
    "sharpens": "sharpen",
    "sharper": "sharp",
    "sharpest": "sharp",
    "sharps": "sharp",
    "sharpshooters": "sharpshooter",
    "shat": "shit",
    "shattered": "shatter",
    "shattering": "shatter",
    "shatters": "shatter",
    "shaved": "shave",
    "shaven": "shave",
    "shavers": "shaver",
    "shaves": "shave",
    "shaving": "shave",
    "shavings": "shaving",
    "shawls": "shawl",
    "shaykhs": "shaykh",
    "she-cats": "she-cat",
    "she-elephants": "she-elephant",
    "sheared": "shear",
    "shearers": "shearer",
    "shearing": "shear",
    "shearlings": "shearling",
    "shears": "shear",
    "shearwaters": "shearwater",
    "sheathed": "sheathe",
    "sheathes": "sheathe",
    "sheathing": "sheathe",
    "sheaths": "sheath",
    "sheaves": "sheaf",
    "shedding": "shed",
    "sheds": "shed",
    "sheens": "sheen",
    "sheepdips": "sheepdip",
    "sheepdogs": "sheepdog",
    "sheepfolds": "sheepfold",
    "sheepskins": "sheepskin",
    "sheered": "sheer",
    "sheerer": "sheer",
    "sheerest": "sheer",
    "sheering": "sheer",
    "sheers": "sheer",
    "sheeted": "sheet",
    "sheeting": "sheet",
    "sheets": "sheet",
    "sheikhdoms": "sheikhdom",
    "sheikhs": "sheikh",
    "sheiks": "sheik",
    "shelducks": "shelduck",
    "shelf-lives": "shelf-life",
    "shelled": "shell",
    "shellfishes": "shellfish",
    "shelling": "shell",
    "shells": "shell",
    "sheltered": "shelter",
    "sheltering": "shelter",
    "shelters": "shelter",
    "shelved": "shelve",
    "shelves": "shelve",
    "shelving": "shelve",
    "shenanigans": "shenanigan",
    "shepherded": "shepherd",
    "shepherdesses": "shepherdess",
    "shepherding": "shepherd",
    "shepherds": "shepherd",
    "sherbets": "sherbet",
    "sheriffs": "sheriff",
    "sherries": "sherry",
    "shibboleths": "shibboleth",
    "shied": "shy",
    "shielded": "shield",
    "shielding": "shield",
    "shields": "shield",
    "shier": "shy",
    "shies": "shy",
    "shiest": "shy",
    "shifted": "shift",
    "shifters": "shifter",
    "shiftier": "shifty",
    "shiftiest": "shifty",
    "shifting": "shift",
    "shifts": "shift",
    "shiitakes": "shiitake",
    "shillings": "shilling",
    "shilly-shallied": "shilly-shally",
    "shilly-shallies": "shilly-shally",
    "shilly-shallying": "shilly-shally",
    "shimmed": "shim",
    "shimmered": "shimmer",
    "shimmering": "shimmer",
    "shimmers": "shimmer",
    "shimmied": "shimmy",
    "shimmies": "shimmy",
    "shimming": "shim",
    "shimmying": "shimmy",
    "shims": "shim",
    "shindigs": "shindig",
    "shined": "shine",
    "shiners": "shiner",
    "shines": "shine",
    "shingled": "shingle",
    "shingles": "shingle",
    "shinglier": "shingly",
    "shingling": "shingle",
    "shinier": "shiny",
    "shiniest": "shiny",
    "shining": "shine",
    "shinned": "shin",
    "shinning": "shin",
    "shins": "shin",
    "shipbuilders": "shipbuilder",
    "shipmates": "shipmate",
    "shipments": "shipment",
    "shipowners": "shipowner",
    "shipped": "ship",
    "shippers": "shipper",
    "shipping": "ship",
    "ships": "ship",
    "shipwrecked": "shipwreck",
    "shipwrecking": "shipwreck",
    "shipwrecks": "shipwreck",
    "shipwrights": "shipwright",
    "shipyards": "shipyard",
    "shires": "shire",
    "shirked": "shirk",
    "shirking": "shirk",
    "shirks": "shirk",
    "shirtfronts": "shirtfront",
    "shirtier": "shirty",
    "shirtiest": "shirty",
    "shirts": "shirt",
    "shirttails": "shirttail",
    "shis": "shi",
    "shits": "shit",
    "shitting": "shit",
    "shivered": "shiver",
    "shivering": "shiver",
    "shivers": "shiver",
    "shoals": "shoal",
    "shocked": "shock",
    "shockers": "shocker",
    "shocking": "shock",
    "shocks": "shock",
    "shockwaves": "shockwave",
    "shod": "shoe",
    "shodden": "shoe",
    "shoddier": "shoddy",
    "shoddiest": "shoddy",
    "shoebrushes": "shoebrush",
    "shoed": "shoe",
    "shoehorned": "shoehorn",
    "shoehorning": "shoehorn",
    "shoehorns": "shoehorn",
    "shoeing": "shoe",
    "shoelaces": "shoelace",
    "shoemakers": "shoemaker",
    "shoes": "shoe",
    "shoeshops": "shoeshop",
    "shoestrings": "shoestring",
    "shoetrees": "shoetree",
    "shoing": "shoe",
    "shone": "shine",
    "shooed": "shoo",
    "shooing": "shoo",
    "shook": "shake",
    "shoos": "shoo",
    "shoot-outs": "shoot-out",
    "shooters": "shooter",
    "shooting": "shoot",
    "shootings": "shooting",
    "shootouts": "shootout",
    "shoots": "shoot",
    "shopkeepers": "shopkeeper",
    "shoplifted": "shoplift",
    "shoplifters": "shoplifter",
    "shoplifting": "shoplift",
    "shoplifts": "shoplift",
    "shopped": "shop",
    "shoppers": "shopper",
    "shopping": "shop",
    "shops": "shop",
    "shorebirds": "shorebird",
    "shored": "shore",
    "shorelines": "shoreline",
    "shores": "shore",
    "shoring": "shore",
    "shorn": "shear",
    "short-changed": "short-change",
    "short-changes": "short-change",
    "short-changing": "short-change",
    "short-circuited": "short-circuit",
    "short-circuiting": "short-circuit",
    "short-circuits": "short-circuit",
    "short-comings": "short-coming",
    "short-courses": "short-course",
    "short-cuts": "short-cut",
    "short-listed": "short-list",
    "short-listing": "short-list",
    "short-lists": "short-list",
    "short-waves": "short-wave",
    "short-weighted": "short-weight",
    "short-weighting": "short-weight",
    "short-weights": "short-weight",
    "shortages": "shortage",
    "shortchanged": "shortchange",
    "shortchanges": "shortchange",
    "shortchanging": "shortchange",
    "shortcomings": "shortcoming",
    "shortcuts": "shortcut",
    "shorted": "short",
    "shortened": "shorten",
    "shortening": "shorten",
    "shortenings": "shortening",
    "shortens": "shorten",
    "shorter": "short",
    "shortest": "short",
    "shortfalls": "shortfall",
    "shorties": "shorty",
    "shorting": "short",
    "shortlisted": "shortlist",
    "shortlisting": "shortlist",
    "shortlists": "shortlist",
    "shorts": "short",
    "shortwaves": "shortwave",
    "shot": "shoot",
    "shotguns": "shotgun",
    "shots": "shot",
    "shoulder-bags": "shoulder-bag",
    "shoulder-straps": "shoulder-strap",
    "shouldered": "shoulder",
    "shouldering": "shoulder",
    "shoulders": "shoulder",
    "shouted": "shout",
    "shouters": "shouter",
    "shouting": "shout",
    "shouts": "shout",
    "shoved": "shove",
    "shoveled": "shovel",
    "shovelers": "shoveler",
    "shovelfuls": "shovelful",
    "shoveling": "shovel",
    "shovelled": "shovel",
    "shovelling": "shovel",
    "shovels": "shovel",
    "shoves": "shove",
    "shoving": "shove",
    "show-offs": "show-off",
    "showcased": "showcase",
    "showcases": "showcase",
    "showcasing": "showcase",
    "showdowns": "showdown",
    "showed": "show",
    "showered": "shower",
    "showerier": "showery",
    "showering": "shower",
    "showers": "shower",
    "showgrounds": "showground",
    "showier": "showy",
    "showiest": "showy",
    "showing": "show",
    "showings": "showing",
    "showmen": "showman",
    "shown": "show",
    "showpieces": "showpiece",
    "showplaces": "showplace",
    "showrooms": "showroom",
    "shows": "show",
    "showstoppers": "showstopper",
    "shrank": "shrink",
    "shredded": "shred",
    "shredders": "shredder",
    "shredding": "shred",
    "shreds": "shred",
    "shrewder": "shrewd",
    "shrewdest": "shrewd",
    "shrewdnesses": "shrewdness",
    "shrews": "shrew",
    "shrieked": "shriek",
    "shrieking": "shriek",
    "shrieks": "shriek",
    "shrikes": "shrike",
    "shrilled": "shrill",
    "shriller": "shrill",
    "shrillest": "shrill",
    "shrilling": "shrill",
    "shrills": "shrill",
    "shrimps": "shrimp",
    "shrines": "shrine",
    "shrinkages": "shrinkage",
    "shrinking": "shrink",
    "shrinks": "shrink",
    "shriveled": "shrivel",
    "shriveling": "shrivel",
    "shrivelled": "shrivel",
    "shrivelling": "shrivel",
    "shrivels": "shrivel",
    "shrouded": "shroud",
    "shrouding": "shroud",
    "shrouds": "shroud",
    "shrubberies": "shrubbery",
    "shrubs": "shrub",
    "shrugged": "shrug",
    "shrugging": "shrug",
    "shrugs": "shrug",
    "shrunk": "shrink",
    "shrunken": "shrink",
    "shucked": "shuck",
    "shucking": "shuck",
    "shucks": "shuck",
    "shuddered": "shudder",
    "shuddering": "shudder",
    "shudders": "shudder",
    "shuffled": "shuffle",
    "shuffles": "shuffle",
    "shuffling": "shuffle",
    "shunned": "shun",
    "shunning": "shun",
    "shuns": "shun",
    "shunted": "shunt",
    "shunters": "shunter",
    "shunting": "shunt",
    "shunts": "shunt",
    "shushed": "shush",
    "shushes": "shush",
    "shushing": "shush",
    "shut-downs": "shut-down",
    "shut-offs": "shut-off",
    "shutdowns": "shutdown",
    "shuts": "shut",
    "shuttered": "shutter",
    "shuttering": "shutter",
    "shutters": "shutter",
    "shutting": "shut",
    "shuttlecocks": "shuttlecock",
    "shuttled": "shuttle",
    "shuttles": "shuttle",
    "shuttling": "shuttle",
    "shyed": "shy",
    "shyer": "shy",
    "shyest": "shy",
    "shying": "shy",
    "shys": "shy",
    "siblings": "sibling",
    "sibs": "sib",
    "sicced": "sic",
    "siccing": "sic",
    "sickbeds": "sickbed",
    "sickened": "sicken",
    "sickening": "sicken",
    "sickeninger": "sickening",
    "sickeningest": "sickening",
    "sickens": "sicken",
    "sicker": "sick",
    "sickest": "sick",
    "sickle-cells": "sickle-cell",
    "sickles": "sickle",
    "sicklier": "sickly",
    "sickliest": "sickly",
    "sicknesses": "sickness",
    "sickrooms": "sickroom",
    "sics": "sic",
    "side-chains": "side-chain",
    "side-effects": "side-effect",
    "side-stepped": "side-step",
    "side-stepping": "side-step",
    "side-steps": "side-step",
    "side-tracked": "side-track",
    "side-tracking": "side-track",
    "side-tracks": "side-track",
    "sidearms": "sidearm",
    "sidebands": "sideband",
    "sidebars": "sidebar",
    "sideboards": "sideboard",
    "sideburns": "sideburn",
    "sidecars": "sidecar",
    "sidechains": "sidechain",
    "sided": "side",
    "sidekicks": "sidekick",
    "sidelights": "sidelight",
    "sidelined": "sideline",
    "sidelines": "sideline",
    "sidelining": "sideline",
    "sides": "side",
    "sideshows": "sideshow",
    "sidestepped": "sidestep",
    "sidestepping": "sidestep",
    "sidesteps": "sidestep",
    "sideswipes": "sideswipe",
    "sidetracked": "sidetrack",
    "sidetracking": "sidetrack",
    "sidetracks": "sidetrack",
    "sidewalks": "sidewalk",
    "sidewalls": "sidewall",
    "siding": "side",
    "sidings": "siding",
    "sidled": "sidle",
    "sidles": "sidle",
    "sidling": "sidle",
    "sieges": "siege",
    "sierras": "sierra",
    "siestas": "siesta",
    "sieved": "sieve",
    "sieves": "sieve",
    "sieving": "sieve",
    "sifted": "sift",
    "sifting": "sift",
    "sifts": "sift",
    "sighed": "sigh",
    "sighing": "sigh",
    "sighs": "sigh",
    "sight-reading": "sight-read",
    "sight-reads": "sight-read",
    "sight-translated": "sight-translate",
    "sight-translates": "sight-translate",
    "sight-translations": "sight-translation",
    "sighted": "sight",
    "sighting": "sight",
    "sightings": "sighting",
    "sights": "sight",
    "sightseeing": "sightsee",
    "sightseers": "sightseer",
    "sigmas": "sigma",
    "sigmoidoscopies": "sigmoidoscopy",
    "signa": "signum",
    "signaled": "signal",
    "signaling": "signal",
    "signalised": "signalise",
    "signalises": "signalise",
    "signalising": "signalise",
    "signalled": "signal",
    "signallers": "signaller",
    "signalling": "signal",
    "signalmen": "signalman",
    "signals": "signal",
    "signatories": "signatory",
    "signatures": "signature",
    "signboards": "signboard",
    "signed": "sign",
    "signers": "signer",
    "signets": "signet",
    "significances": "significance",
    "significations": "signification",
    "signified": "signify",
    "signifiers": "signifier",
    "signifies": "signify",
    "signifying": "signify",
    "signing": "sign",
    "signposted": "signpost",
    "signposting": "signpost",
    "signposts": "signpost",
    "signs": "sign",
    "sigs": "sig",
    "silages": "silage",
    "silenced": "silence",
    "silencers": "silencer",
    "silences": "silence",
    "silencing": "silence",
    "silhouetted": "silhouette",
    "silhouettes": "silhouette",
    "silhouetting": "silhouette",
    "silicas": "silica",
    "silicates": "silicate",
    "silicones": "silicone",
    "silicoses": "silicosis",
    "silkier": "silky",
    "silkiest": "silky",
    "silks": "silk",
    "silkscreens": "silkscreen",
    "silkworms": "silkworm",
    "sillier": "silly",
    "sillies": "silly",
    "silliest": "silly",
    "sills": "sill",
    "silos": "silo",
    "silted": "silt",
    "silting": "silt",
    "silts": "silt",
    "silvered": "silver",
    "silverier": "silvery",
    "silvering": "silver",
    "silvers": "silver",
    "silversmiths": "silversmith",
    "simians": "simian",
    "similarities": "similarity",
    "similes": "simile",
    "similitudes": "similitude",
    "simmered": "simmer",
    "simmering": "simmer",
    "simmers": "simmer",
    "simpered": "simper",
    "simpering": "simper",
    "simpers": "simper",
    "simpled": "simple",
    "simpler": "simple",
    "simples": "simple",
    "simplest": "simple",
    "simpletons": "simpleton",
    "simplexes": "simplex",
    "simplices": "simplex",
    "simplicia": "simplex",
    "simplicities": "simplicity",
    "simplifications": "simplification",
    "simplified": "simplify",
    "simplifies": "simplify",
    "simplifying": "simplify",
    "simpling": "simple",
    "simulacra": "simulacrum",
    "simulacrums": "simulacrum",
    "simulated": "simulate",
    "simulates": "simulate",
    "simulating": "simulate",
    "simulations": "simulation",
    "simulators": "simulator",
    "simulcasted": "simulcast",
    "simulcasting": "simulcast",
    "simulcasts": "simulcast",
    "simultaneities": "simultaneity",
    "sincerer": "sincere",
    "sincerest": "sincere",
    "sinecures": "sinecure",
    "sinenses": "sinensis",
    "sines": "sine",
    "sinewaves": "sinewave",
    "sinews": "sinew",
    "sing-songs": "sing-song",
    "singed": "singe",
    "singeing": "singe",
    "singers": "singer",
    "singes": "singe",
    "singing": "singe",
    "singings": "singing",
    "single-deckers": "single-decker",
    "singled": "single",
    "singles": "single",
    "singletons": "singleton",
    "singlets": "singlet",
    "singling": "single",
    "sings": "sing",
    "singularities": "singularity",
    "singulars": "singular",
    "sinkers": "sinker",
    "sinkholes": "sinkhole",
    "sinking": "sink",
    "sinks": "sink",
    "sinned": "sin",
    "sinners": "sinner",
    "sinning": "sin",
    "sins": "sin",
    "sintered": "sinter",
    "sintering": "sinter",
    "sinters": "sinter",
    "sinuses": "sinus",
    "sinusitides": "sinusitis",
    "sinusitises": "sinusitis",
    "sinusoids": "sinusoid",
    "siphoned": "siphon",
    "siphoning": "siphon",
    "siphons": "siphon",
    "sipped": "sip",
    "sipping": "sip",
    "sips": "sip",
    "sired": "sire",
    "sirens": "siren",
    "sires": "sire",
    "siring": "sire",
    "sirloins": "sirloin",
    "siroccos": "sirocco",
    "sirs": "sir",
    "siskins": "siskin",
    "sissies": "sissy",
    "sister-in-laws": "sister-in-law",
    "sisterhoods": "sisterhood",
    "sisters": "sister",
    "sisters-in-law": "sister-in-law",
    "sit-downs": "sit-down",
    "sit-ins": "sit-in",
    "sitars": "sitar",
    "sitcoms": "sitcom",
    "sited": "site",
    "sitemaps": "sitemap",
    "sites": "site",
    "siting": "site",
    "sits": "sit",
    "sitters": "sitter",
    "sitting": "sit",
    "sitting-rooms": "sitting-room",
    "sittings": "sitting",
    "situated": "situate",
    "situates": "situate",
    "situating": "situate",
    "situationists": "situationist",
    "situations": "situation",
    "situses": "situs",
    "six-hundredth": "600",
    "sixes": "six",
    "sixpences": "sixpence",
    "sixteenth": "16",
    "sixteenths": "sixteenth",
    "sixth": "6",
    "sixths": "sixth",
    "sixties": "sixty",
    "sixtieth": "60",
    "sixtieths": "sixtieth",
    "sixty-eighth": "68",
    "sixty-eighths": "sixty-eighth",
    "sixty-fifth": "65",
    "sixty-fifths": "sixty-fifth",
    "sixty-first": "61",
    "sixty-firsts": "sixty-first",
    "sixty-fourth": "64",
    "sixty-fourths": "sixty-fourth",
    "sixty-ninth": "69",
    "sixty-ninths": "sixty-ninth",
    "sixty-second": "62",
    "sixty-seconds": "sixty-second",
    "sixty-seventh": "67",
    "sixty-sevenths": "sixty-seventh",
    "sixty-sixth": "66",
    "sixty-sixths": "sixty-sixth",
    "sixty-third": "63",
    "sixty-thirds": "sixty-third",
    "sized": "size",
    "sizes": "size",
    "sizing": "size",
    "sizzled": "sizzle",
    "sizzles": "sizzle",
    "sizzling": "sizzle",
    "skateboards": "skateboard",
    "skated": "skate",
    "skaters": "skater",
    "skates": "skate",
    "skating": "skate",
    "skatings": "skating",
    "skeins": "skein",
    "skeletons": "skeleton",
    "skepticisms": "skepticism",
    "skeptics": "skeptic",
    "sketchbooks": "sketchbook",
    "sketched": "sketch",
    "sketches": "sketch",
    "sketchier": "sketchy",
    "sketchiest": "sketchy",
    "sketching": "sketch",
    "sketchpads": "sketchpad",
    "skewed": "skew",
    "skewered": "skewer",
    "skewering": "skewer",
    "skewers": "skewer",
    "skewing": "skew",
    "skewnesses": "skewness",
    "skews": "skew",
    "skidded": "skid",
    "skidding": "skid",
    "skidmarks": "skidmark",
    "skids": "skid",
    "skied": "sky",
    "skiers": "skier",
    "skies": "sky",
    "skiffs": "skiff",
    "skiing": "ski",
    "skiis": "ski",
    "skillets": "skillet",
    "skills": "skill",
    "skimmed": "skim",
    "skimmers": "skimmer",
    "skimming": "skim",
    "skimped": "skimp",
    "skimpier": "skimpy",
    "skimpiest": "skimpy",
    "skimping": "skimp",
    "skimps": "skimp",
    "skims": "skim",
    "skin-divers": "skin-diver",
    "skinflints": "skinflint",
    "skinheads": "skinhead",
    "skinks": "skink",
    "skinned": "skin",
    "skinners": "skinner",
    "skinnier": "skinny",
    "skinniest": "skinny",
    "skinning": "skin",
    "skins": "skin",
    "skipped": "skip",
    "skippered": "skipper",
    "skippering": "skipper",
    "skippers": "skipper",
    "skipping": "skip",
    "skips": "skip",
    "skirmished": "skirmish",
    "skirmishes": "skirmish",
    "skirmishing": "skirmish",
    "skirted": "skirt",
    "skirting": "skirt",
    "skirtings": "skirting",
    "skirts": "skirt",
    "skis": "ski",
    "skits": "skit",
    "skittered": "skitter",
    "skittering": "skitter",
    "skitters": "skitter",
    "skittled": "skittle",
    "skittles": "skittle",
    "skittling": "skittle",
    "skived": "skive",
    "skives": "skive",
    "skiving": "skive",
    "skuas": "skua",
    "skulked": "skulk",
    "skulking": "skulk",
    "skulks": "skulk",
    "skullcaps": "skullcap",
    "skulls": "skull",
    "skunks": "skunk",
    "skydivers": "skydiver",
    "skying": "sky",
    "skylarks": "skylark",
    "skylights": "skylight",
    "skylines": "skyline",
    "skyrocketed": "skyrocket",
    "skyrocketing": "skyrocket",
    "skyrockets": "skyrocket",
    "skyscrapers": "skyscraper",
    "slabs": "slab",
    "slacked": "slack",
    "slackened": "slacken",
    "slackening": "slacken",
    "slackens": "slacken",
    "slacker": "slack",
    "slackers": "slacker",
    "slackest": "slack",
    "slacking": "slack",
    "slacknesses": "slackness",
    "slacks": "slack",
    "slagged": "slag",
    "slagging": "slag",
    "slags": "slag",
    "slain": "slay",
    "slaked": "slake",
    "slakes": "slake",
    "slaking": "slake",
    "slaloms": "slalom",
    "slammed": "slam",
    "slamming": "slam",
    "slams": "slam",
    "slandered": "slander",
    "slandering": "slander",
    "slanders": "slander",
    "slanged": "slang",
    "slangier": "slangy",
    "slangiest": "slangy",
    "slanging": "slang",
    "slangs": "slang",
    "slanted": "slant",
    "slanting": "slant",
    "slants": "slant",
    "slapped": "slap",
    "slapping": "slap",
    "slaps": "slap",
    "slashed": "slash",
    "slashers": "slasher",
    "slashes": "slash",
    "slashing": "slash",
    "slated": "slate",
    "slaters": "slater",
    "slates": "slate",
    "slatier": "slaty",
    "slating": "slate",
    "slats": "slat",
    "slatted": "slat",
    "slatterns": "slattern",
    "slatting": "slat",
    "slaughter-houses": "slaughter-house",
    "slaughtered": "slaughter",
    "slaughterers": "slaughterer",
    "slaughterhouses": "slaughterhouse",
    "slaughtering": "slaughter",
    "slaughtermen": "slaughterman",
    "slaughters": "slaughter",
    "slaved": "slave",
    "slavered": "slaver",
    "slavering": "slaver",
    "slavers": "slaver",
    "slaves": "slave",
    "slaving": "slave",
    "slayed": "slay",
    "slayers": "slayer",
    "slaying": "slay",
    "slays": "slay",
    "sleazier": "sleazy",
    "sleaziest": "sleazy",
    "sledded": "sled",
    "sledding": "sled",
    "sledge-hammers": "sledge-hammer",
    "sledged": "sledge",
    "sledgehammers": "sledgehammer",
    "sledges": "sledge",
    "sledging": "sledge",
    "sleds": "sled",
    "sleeker": "sleek",
    "sleekest": "sleek",
    "sleepers": "sleeper",
    "sleepier": "sleepy",
    "sleepiest": "sleepy",
    "sleeping": "sleep",
    "sleeps": "sleep",
    "sleepwalked": "sleepwalk",
    "sleepwalkers": "sleepwalker",
    "sleepwalking": "sleepwalk",
    "sleepwalks": "sleepwalk",
    "sleepyheads": "sleepyhead",
    "sleeted": "sleet",
    "sleetier": "sleety",
    "sleeting": "sleet",
    "sleets": "sleet",
    "sleeves": "sleeve",
    "sleighs": "sleigh",
    "sleights": "sleight",
    "slenderer": "slender",
    "slenderest": "slender",
    "slept": "sleep",
    "sleuthed": "sleuth",
    "sleuthing": "sleuth",
    "sleuths": "sleuth",
    "slew": "slay",
    "slewed": "slew",
    "slewing": "slew",
    "slews": "slew",
    "sliced": "slice",
    "slicers": "slicer",
    "slices": "slice",
    "slicing": "slice",
    "slicked": "slick",
    "slicker": "slick",
    "slickers": "slicker",
    "slickest": "slick",
    "slicking": "slick",
    "slicks": "slick",
    "slid": "slide",
    "sliders": "slider",
    "slides": "slide",
    "sliding": "slide",
    "slier": "sly",
    "sliest": "sly",
    "slighted": "slight",
    "slighter": "slight",
    "slightest": "slight",
    "slighting": "slight",
    "slights": "slight",
    "slimes": "slime",
    "slimier": "slimy",
    "slimiest": "slimy",
    "slimmed": "slim",
    "slimmer": "slim",
    "slimmers": "slimmer",
    "slimmest": "slim",
    "slimming": "slim",
    "slims": "slim",
    "slinging": "sling",
    "slings": "sling",
    "slingshots": "slingshot",
    "slinkier": "slinky",
    "slinkiest": "slinky",
    "slinking": "slink",
    "slinks": "slink",
    "slip-ups": "slip-up",
    "slipcovers": "slipcover",
    "slipknots": "slipknot",
    "slipovers": "slipover",
    "slippages": "slippage",
    "slipped": "slip",
    "slipperier": "slippery",
    "slipperiest": "slippery",
    "slippers": "slipper",
    "slipping": "slip",
    "slips": "slip",
    "slipstreams": "slipstream",
    "slipways": "slipway",
    "slithered": "slither",
    "slithering": "slither",
    "slithers": "slither",
    "slits": "slit",
    "slitted": "slit",
    "slitting": "slit",
    "slivers": "sliver",
    "slobbered": "slobber",
    "slobbering": "slobber",
    "slobbers": "slobber",
    "slobs": "slob",
    "sloes": "sloe",
    "sloganeered": "sloganeer",
    "sloganeering": "sloganeer",
    "sloganeers": "sloganeer",
    "slogans": "slogan",
    "slogged": "slog",
    "slogging": "slog",
    "slogs": "slog",
    "sloops": "sloop",
    "sloped": "slope",
    "slopes": "slope",
    "sloping": "slope",
    "slopped": "slop",
    "sloppier": "sloppy",
    "sloppiest": "sloppy",
    "slopping": "slop",
    "slops": "slop",
    "sloshed": "slosh",
    "sloshes": "slosh",
    "sloshing": "slosh",
    "sloths": "sloth",
    "slots": "slot",
    "slotted": "slot",
    "slotting": "slot",
    "slouched": "slouch",
    "slouches": "slouch",
    "slouching": "slouch",
    "sloughed": "slough",
    "sloughing": "slough",
    "sloughs": "slough",
    "slovenlier": "slovenly",
    "slovenliest": "slovenly",
    "slow-downs": "slow-down",
    "slowcoaches": "slowcoach",
    "slowdowns": "slowdown",
    "slowed": "slow",
    "slower": "slow",
    "slowest": "slow",
    "slowing": "slow",
    "slowpokes": "slowpoke",
    "slows": "slow",
    "slowworms": "slowworm",
    "sls": "sl",
    "sludges": "sludge",
    "sluggards": "sluggard",
    "slugged": "slug",
    "slugging": "slug",
    "slugs": "slug",
    "sluiced": "sluice",
    "sluices": "sluice",
    "sluicing": "sluice",
    "slumbered": "slumber",
    "slumbering": "slumber",
    "slumbers": "slumber",
    "slummed": "slum",
    "slummier": "slummy",
    "slummiest": "slummy",
    "slumming": "slum",
    "slumped": "slump",
    "slumping": "slump",
    "slumps": "slump",
    "slums": "slum",
    "slung": "sling",
    "slunk": "slink",
    "slurped": "slurp",
    "slurping": "slurp",
    "slurps": "slurp",
    "slurred": "slur",
    "slurries": "slurry",
    "slurring": "slur",
    "slurs": "slur",
    "slushier": "slushy",
    "slushiest": "slushy",
    "sluts": "slut",
    "slyer": "sly",
    "slyest": "sly",
    "smacked": "smack",
    "smacking": "smack",
    "smacks": "smack",
    "small-holders": "small-holder",
    "smaller": "small",
    "smallest": "small",
    "smallholders": "smallholder",
    "smallholdings": "smallholding",
    "smallisher": "smallish",
    "smallishest": "smallish",
    "smalls": "small",
    "smarmier": "smarmy",
    "smarmiest": "smarmy",
    "smarted": "smart",
    "smartened": "smarten",
    "smartening": "smarten",
    "smartens": "smarten",
    "smarter": "smart",
    "smartest": "smart",
    "smarties": "smarty",
    "smarting": "smart",
    "smarts": "smart",
    "smash-ups": "smash-up",
    "smashed": "smash",
    "smashers": "smasher",
    "smashes": "smash",
    "smashing": "smash",
    "smatterings": "smattering",
    "smeared": "smear",
    "smearier": "smeary",
    "smeariest": "smeary",
    "smearing": "smear",
    "smears": "smear",
    "smelled": "smell",
    "smellier": "smelly",
    "smelliest": "smelly",
    "smelling": "smell",
    "smells": "smell",
    "smelt": "smell",
    "smelted": "smelt",
    "smelters": "smelter",
    "smelting": "smelt",
    "smelts": "smelt",
    "smidgens": "smidgen",
    "smiled": "smile",
    "smiles": "smile",
    "smileys": "smiley",
    "smiling": "smile",
    "smirked": "smirk",
    "smirking": "smirk",
    "smirks": "smirk",
    "smites": "smite",
    "smithies": "smithy",
    "smiths": "smith",
    "smiting": "smite",
    "smitten": "smite",
    "smocks": "smock",
    "smogs": "smog",
    "smoked": "smoke",
    "smokehouses": "smokehouse",
    "smokers": "smoker",
    "smokes": "smoke",
    "smokescreens": "smokescreen",
    "smokestacks": "smokestack",
    "smokier": "smoky",
    "smokiest": "smoky",
    "smoking": "smoke",
    "smoldered": "smolder",
    "smoldering": "smolder",
    "smolders": "smolder",
    "smolts": "smolt",
    "smooched": "smooch",
    "smooches": "smooch",
    "smooching": "smooch",
    "smoothed": "smooth",
    "smoother": "smooth",
    "smoothes": "smooth",
    "smoothest": "smooth",
    "smoothies": "smoothie",
    "smoothing": "smooth",
    "smooths": "smooth",
    "smorgasbords": "smorgasbord",
    "smote": "smite",
    "smothered": "smother",
    "smothering": "smother",
    "smothers": "smother",
    "smouldered": "smoulder",
    "smouldering": "smoulder",
    "smoulders": "smoulder",
    "smudged": "smudge",
    "smudges": "smudge",
    "smudgier": "smudgy",
    "smudgiest": "smudgy",
    "smudging": "smudge",
    "smugger": "smug",
    "smuggest": "smug",
    "smuggled": "smuggle",
    "smugglers": "smuggler",
    "smuggles": "smuggle",
    "smuggling": "smuggle",
    "smugglings": "smuggling",
    "smuts": "smut",
    "smuttier": "smutty",
    "smuttiest": "smutty",
    "snacked": "snack",
    "snacking": "snack",
    "snacks": "snack",
    "snaffled": "snaffle",
    "snaffles": "snaffle",
    "snaffling": "snaffle",
    "snagged": "snag",
    "snagging": "snag",
    "snags": "snag",
    "snails": "snail",
    "snakebites": "snakebite",
    "snaked": "snake",
    "snakeheads": "snakehead",
    "snakes": "snake",
    "snakier": "snaky",
    "snakiest": "snaky",
    "snaking": "snake",
    "snap-shots": "snap-shot",
    "snapdragons": "snapdragon",
    "snapped": "snap",
    "snappers": "snapper",
    "snappier": "snappy",
    "snappiest": "snappy",
    "snapping": "snap",
    "snaps": "snap",
    "snapshots": "snapshot",
    "snared": "snare",
    "snares": "snare",
    "snaring": "snare",
    "snarl-ups": "snarl-up",
    "snarled": "snarl",
    "snarling": "snarl",
    "snarls": "snarl",
    "snatched": "snatch",
    "snatchers": "snatcher",
    "snatches": "snatch",
    "snatching": "snatch",
    "snazzier": "snazzy",
    "snazziest": "snazzy",
    "sneaked": "sneak",
    "sneakers": "sneaker",
    "sneakier": "sneaky",
    "sneakiest": "sneaky",
    "sneaking": "sneak",
    "sneaks": "sneak",
    "sneered": "sneer",
    "sneering": "sneer",
    "sneers": "sneer",
    "sneezed": "sneeze",
    "sneezes": "sneeze",
    "sneezing": "sneeze",
    "snicked": "snick",
    "snickered": "snicker",
    "snickering": "snicker",
    "snickers": "snicker",
    "snicking": "snick",
    "snicks": "snick",
    "snider": "snide",
    "snidest": "snide",
    "sniffed": "sniff",
    "sniffers": "sniffer",
    "sniffier": "sniffy",
    "sniffiest": "sniffy",
    "sniffing": "sniff",
    "sniffled": "sniffle",
    "sniffles": "sniffle",
    "sniffling": "sniffle",
    "sniffs": "sniff",
    "snifters": "snifter",
    "sniggered": "snigger",
    "sniggering": "snigger",
    "sniggers": "snigger",
    "sniped": "snipe",
    "snipers": "sniper",
    "snipes": "snipe",
    "sniping": "snipe",
    "snipped": "snip",
    "snippets": "snippet",
    "snipping": "snip",
    "snips": "snip",
    "snitched": "snitch",
    "snitches": "snitch",
    "snitching": "snitch",
    "sniveled": "snivel",
    "sniveling": "snivel",
    "snivelled": "snivel",
    "snivelling": "snivel",
    "snivels": "snivel",
    "snobberies": "snobbery",
    "snobbier": "snobby",
    "snobbiest": "snobby",
    "snobs": "snob",
    "snogged": "snog",
    "snogging": "snog",
    "snogs": "snog",
    "snookered": "snooker",
    "snookering": "snooker",
    "snookers": "snooker",
    "snooks": "snook",
    "snooped": "snoop",
    "snoopers": "snooper",
    "snooping": "snoop",
    "snoops": "snoop",
    "snootier": "snooty",
    "snootiest": "snooty",
    "snoozed": "snooze",
    "snoozes": "snooze",
    "snoozing": "snooze",
    "snored": "snore",
    "snorers": "snorer",
    "snores": "snore",
    "snoring": "snore",
    "snorkeled": "snorkel",
    "snorkeling": "snorkel",
    "snorkelled": "snorkel",
    "snorkelling": "snorkel",
    "snorkels": "snorkel",
    "snorted": "snort",
    "snortier": "snorty",
    "snorting": "snort",
    "snorts": "snort",
    "snottier": "snotty",
    "snottiest": "snotty",
    "snouts": "snout",
    "snowballed": "snowball",
    "snowballing": "snowball",
    "snowballs": "snowball",
    "snowboarders": "snowboarder",
    "snowboards": "snowboard",
    "snowdrifts": "snowdrift",
    "snowdrops": "snowdrop",
    "snowed": "snow",
    "snowfalls": "snowfall",
    "snowfields": "snowfield",
    "snowflakes": "snowflake",
    "snowier": "snowy",
    "snowiest": "snowy",
    "snowing": "snow",
    "snowmen": "snowman",
    "snowmobiles": "snowmobile",
    "snowploughs": "snowplough",
    "snows": "snow",
    "snowshoed": "snowshoe",
    "snowshoeing": "snowshoe",
    "snowshoes": "snowshoe",
    "snowstorms": "snowstorm",
    "snubbed": "snub",
    "snubbing": "snub",
    "snubs": "snub",
    "snuffboxes": "snuffbox",
    "snuffed": "snuff",
    "snuffers": "snuffer",
    "snuffing": "snuff",
    "snuffled": "snuffle",
    "snuffles": "snuffle",
    "snuffling": "snuffle",
    "snuffs": "snuff",
    "snugged": "snug",
    "snugger": "snug",
    "snuggest": "snug",
    "snugging": "snug",
    "snuggled": "snuggle",
    "snuggles": "snuggle",
    "snuggling": "snuggle",
    "snugs": "snug",
    "so-and-sos": "so-and-so",
    "soaked": "soak",
    "soakers": "soaker",
    "soaking": "soak",
    "soaks": "soak",
    "soap-operas": "soap-opera",
    "soapboxes": "soapbox",
    "soaped": "soap",
    "soapflakes": "soapflake",
    "soapier": "soapy",
    "soapiest": "soapy",
    "soaping": "soap",
    "soaps": "soap",
    "soared": "soar",
    "soaring": "soar",
    "soars": "soar",
    "sobbed": "sob",
    "sobbing": "sob",
    "sobered": "sober",
    "soberer": "sober",
    "soberest": "sober",
    "sobering": "sober",
    "sobers": "sober",
    "sobriquets": "sobriquet",
    "sobs": "sob",
    "sociabilities": "sociability",
    "socialisations": "socialisation",
    "socialised": "socialise",
    "socialisers": "socialiser",
    "socialises": "socialise",
    "socialising": "socialise",
    "socialists": "socialist",
    "socialites": "socialite",
    "socializations": "socialization",
    "socialized": "socialize",
    "socializes": "socialize",
    "socializing": "socialize",
    "socials": "social",
    "societies": "society",
    "sociologies": "sociology",
    "sociologists": "sociologist",
    "sociopaths": "sociopath",
    "socked": "sock",
    "sockets": "socket",
    "socking": "sock",
    "socks": "sock",
    "sodas": "soda",
    "sods": "sod",
    "sofa-beds": "sofa-bed",
    "sofas": "sofa",
    "soft-pedalled": "soft-pedal",
    "soft-pedalling": "soft-pedal",
    "soft-pedals": "soft-pedal",
    "soft-soaped": "soft-soap",
    "soft-soaping": "soft-soap",
    "soft-soaps": "soft-soap",
    "soft-tissues": "soft-tissue",
    "softballs": "softball",
    "softened": "soften",
    "softeners": "softener",
    "softening": "soften",
    "softenings": "softening",
    "softens": "soften",
    "softer": "soft",
    "softest": "soft",
    "softies": "softy",
    "softnesses": "softness",
    "softs": "soft",
    "softwoods": "softwood",
    "soggier": "soggy",
    "soggiest": "soggy",
    "soiled": "soil",
    "soiling": "soil",
    "soils": "soil",
    "soirees": "soiree",
    "sojourned": "sojourn",
    "sojourners": "sojourner",
    "sojourning": "sojourn",
    "sojourns": "sojourn",
    "solaced": "solace",
    "solaces": "solace",
    "solacing": "solace",
    "solaria": "solarium",
    "solariums": "solarium",
    "sold": "sell",
    "soldered": "solder",
    "soldering": "solder",
    "solders": "solder",
    "soldiered": "soldier",
    "soldiering": "soldier",
    "soldiers": "soldier",
    "solecisms": "solecism",
    "soled": "sole",
    "solemner": "solemn",
    "solemnest": "solemn",
    "solemnised": "solemnise",
    "solemnises": "solemnise",
    "solemnising": "solemnise",
    "solemnities": "solemnity",
    "solemnized": "solemnize",
    "solemnizes": "solemnize",
    "solemnizing": "solemnize",
    "solenoids": "solenoid",
    "soles": "sole",
    "soli": "solo",
    "solicitations": "solicitation",
    "solicited": "solicit",
    "soliciting": "solicit",
    "solicitors": "solicitor",
    "solicits": "solicit",
    "solidarities": "solidarity",
    "solider": "solid",
    "solidest": "solid",
    "solidi": "solidus",
    "solidifications": "solidification",
    "solidified": "solidify",
    "solidifies": "solidify",
    "solidifying": "solidify",
    "solidities": "solidity",
    "solids": "solid",
    "soliloquies": "soliloquy",
    "soliloquys": "soliloquy",
    "soling": "sole",
    "solitaires": "solitaire",
    "solitaries": "solitary",
    "solitons": "soliton",
    "solitudes": "solitude",
    "soloed": "solo",
    "soloing": "solo",
    "soloists": "soloist",
    "solomons": "solomon",
    "solos": "solo",
    "sols": "sol",
    "solstices": "solstice",
    "solubilities": "solubility",
    "solutes": "solute",
    "solutions": "solution",
    "solvated": "solvate",
    "solvates": "solvate",
    "solvating": "solvate",
    "solved": "solve",
    "solvents": "solvent",
    "solvers": "solver",
    "solves": "solve",
    "solving": "solve",
    "somas": "soma",
    "somata": "soma",
    "somatostatins": "somatostatin",
    "sombreros": "sombrero",
    "somersaulted": "somersault",
    "somersaulting": "somersault",
    "somersaults": "somersault",
    "somnambulists": "somnambulist",
    "sonars": "sonar",
    "sonatas": "sonata",
    "songbirds": "songbird",
    "songbooks": "songbook",
    "songs": "song",
    "songsters": "songster",
    "songstresses": "songstress",
    "songwriters": "songwriter",
    "sonnets": "sonnet",
    "sonograms": "sonogram",
    "sonographers": "sonographer",
    "sonographies": "sonography",
    "sonorities": "sonority",
    "sons": "son",
    "sons-in-law": "son-in-law",
    "sooner": "soon",
    "soonest": "soon",
    "soothed": "soothe",
    "soothers": "soother",
    "soothes": "soothe",
    "soothing": "soothe",
    "soothsayers": "soothsayer",
    "sootier": "sooty",
    "sootiest": "sooty",
    "sophisticated": "sophisticate",
    "sophisticates": "sophisticate",
    "sophisticating": "sophisticate",
    "sophistications": "sophistication",
    "sophistries": "sophistry",
    "sophists": "sophist",
    "sophomores": "sophomore",
    "soporifics": "soporific",
    "sopped": "sop",
    "soppier": "soppy",
    "soppiest": "soppy",
    "sopping": "sop",
    "sopranos": "soprano",
    "sops": "sop",
    "sorbets": "sorbet",
    "sorcerers": "sorcerer",
    "sorceresses": "sorceress",
    "sorceries": "sorcery",
    "sorer": "sore",
    "sores": "sore",
    "sorest": "sore",
    "sorghums": "sorghum",
    "sororities": "sorority",
    "sorrier": "sorry",
    "sorriest": "sorry",
    "sorrowed": "sorrow",
    "sorrowing": "sorrow",
    "sorrows": "sorrow",
    "sort-outs": "sort-out",
    "sorted": "sort",
    "sorters": "sorter",
    "sorties": "sortie",
    "sorting": "sort",
    "sorts": "sort",
    "sots": "sot",
    "sou'westers": "sou'wester",
    "souffles": "souffle",
    "soufflés": "soufflé",
    "soughed": "sough",
    "soughing": "sough",
    "soughs": "sough",
    "sought": "seek",
    "souls": "soul",
    "sound-tracks": "sound-track",
    "sounded": "sound",
    "sounder": "sound",
    "soundest": "sound",
    "sounding": "sound",
    "soundings": "sounding",
    "soundlessly": "soundless",
    "soundproofed": "soundproof",
    "soundproofing": "soundproof",
    "soundproofs": "soundproof",
    "sounds": "sound",
    "soundtracks": "soundtrack",
    "souped": "soup",
    "souping": "soup",
    "soups": "soup",
    "soupspoons": "soupspoon",
    "sourcebooks": "sourcebook",
    "sourced": "source",
    "sources": "source",
    "sourcing": "source",
    "sourdoughs": "sourdough",
    "soured": "sour",
    "sourer": "sour",
    "sourest": "sour",
    "souring": "sour",
    "sours": "sour",
    "sous": "sou",
    "southerners": "southerner",
    "southpaws": "southpaw",
    "souths": "south",
    "southwards": "southward",
    "souvenirs": "souvenir",
    "sovereigns": "sovereign",
    "sowed": "sow",
    "sowers": "sower",
    "sowing": "sow",
    "sown": "sow",
    "sows": "sow",
    "soxs": "sox",
    "soyabeans": "soyabean",
    "soyas": "soya",
    "soybeans": "soybean",
    "soys": "soy",
    "spacecrafts": "spacecraft",
    "spaced": "space",
    "spaceflights": "spaceflight",
    "spacemen": "spaceman",
    "spacers": "spacer",
    "spaces": "space",
    "spaceships": "spaceship",
    "spacesuits": "spacesuit",
    "spacewomen": "spacewoman",
    "spacing": "space",
    "spacings": "spacing",
    "spaded": "spade",
    "spades": "spade",
    "spadeworks": "spadework",
    "spading": "spade",
    "spake": "speak",
    "span": "spin",
    "spandrels": "spandrel",
    "spangled": "spangle",
    "spangles": "spangle",
    "spangling": "spangle",
    "spaniels": "spaniel",
    "spanked": "spank",
    "spanking": "spank",
    "spankings": "spanking",
    "spanks": "spank",
    "spanned": "span",
    "spanners": "spanner",
    "spanning": "span",
    "spans": "span",
    "spared": "spare",
    "sparer": "spare",
    "spares": "spare",
    "sparest": "spare",
    "sparing": "spare",
    "sparked": "spark",
    "sparking": "spark",
    "sparkled": "sparkle",
    "sparklers": "sparkler",
    "sparkles": "sparkle",
    "sparkling": "sparkle",
    "sparks": "spark",
    "sparred": "spar",
    "sparring": "spar",
    "sparrowhawks": "sparrowhawk",
    "sparrows": "sparrow",
    "spars": "spar",
    "sparsenesses": "sparseness",
    "sparser": "sparse",
    "sparsest": "sparse",
    "spas": "spa",
    "spasms": "spasm",
    "spasticities": "spasticity",
    "spastics": "spastic",
    "spat": "spit",
    "spates": "spate",
    "spathes": "spathe",
    "spatialities": "spatiality",
    "spats": "spat",
    "spattered": "spatter",
    "spattering": "spatter",
    "spatters": "spatter",
    "spatulae": "spatula",
    "spatulas": "spatula",
    "spawned": "spawn",
    "spawning": "spawn",
    "spawns": "spawn",
    "spayed": "spay",
    "spaying": "spay",
    "spays": "spay",
    "speakers": "speaker",
    "speaking": "speak",
    "speaks": "speak",
    "speakwrites": "speakwrite",
    "speared": "spear",
    "spearheaded": "spearhead",
    "spearheading": "spearhead",
    "spearheads": "spearhead",
    "spearing": "spear",
    "spearmen": "spearman",
    "spearmints": "spearmint",
    "spears": "spear",
    "specialisations": "specialisation",
    "specialised": "specialise",
    "specialises": "specialise",
    "specialising": "specialise",
    "specialisms": "specialism",
    "specialists": "specialist",
    "specialities": "speciality",
    "specializations": "specialization",
    "specialized": "specialize",
    "specializes": "specialize",
    "specializing": "specialize",
    "specials": "special",
    "specialties": "specialty",
    "speciations": "speciation",
    "specifications": "specification",
    "specificities": "specificity",
    "specifics": "specific",
    "specified": "specify",
    "specifiers": "specifier",
    "specifies": "specify",
    "specifying": "specify",
    "specimens": "specimen",
    "speckled": "speckle",
    "speckles": "speckle",
    "speckling": "speckle",
    "specks": "speck",
    "specs": "spec",
    "spectacles": "spectacle",
    "spectaculars": "spectacular",
    "spectators": "spectator",
    "specters": "specter",
    "spectra": "spectrum",
    "spectrals": "spectral",
    "spectres": "spectre",
    "spectrograms": "spectrogram",
    "spectrographs": "spectrograph",
    "spectrometers": "spectrometer",
    "spectrometries": "spectrometry",
    "spectrophotometers": "spectrophotometer",
    "spectrophotometries": "spectrophotometry",
    "spectroscopes": "spectroscope",
    "spectroscopies": "spectroscopy",
    "spectrums": "spectrum",
    "specula": "speculum",
    "speculated": "speculate",
    "speculates": "speculate",
    "speculating": "speculate",
    "speculations": "speculation",
    "speculators": "speculator",
    "speculums": "speculum",
    "sped": "speed",
    "speeches": "speech",
    "speed-ups": "speed-up",
    "speedboats": "speedboat",
    "speeded": "speed",
    "speeders": "speeder",
    "speedier": "speedy",
    "speediest": "speedy",
    "speeding": "speed",
    "speedometers": "speedometer",
    "speeds": "speed",
    "speedups": "speedup",
    "speedways": "speedway",
    "speedwells": "speedwell",
    "speleologists": "speleologist",
    "speleothems": "speleothem",
    "spellbinding": "spellbind",
    "spellbinds": "spellbind",
    "spellbound": "spellbind",
    "spelled": "spell",
    "spellers": "speller",
    "spelling": "spell",
    "spellings": "spelling",
    "spells": "spell",
    "spelt": "spell",
    "spelts": "spelt",
    "spencers": "spencer",
    "spences": "spence",
    "spenders": "spender",
    "spending": "spend",
    "spends": "spend",
    "spendthrifts": "spendthrift",
    "spent": "spend",
    "spermatogeneses": "spermatogenesis",
    "spermatozoa": "spermatozoon",
    "sperms": "sperm",
    "spewed": "spew",
    "spewing": "spew",
    "spews": "spew",
    "sphagnums": "sphagnum",
    "sphalerites": "sphalerite",
    "spheres": "sphere",
    "spheroids": "spheroid",
    "sphincters": "sphincter",
    "sphinxes": "sphinx",
    "sphygmomanometers": "sphygmomanometer",
    "spiced": "spice",
    "spices": "spice",
    "spicier": "spicy",
    "spiciest": "spicy",
    "spicing": "spice",
    "spicules": "spicule",
    "spiders": "spider",
    "spied": "spy",
    "spiels": "spiel",
    "spies": "spy",
    "spigots": "spigot",
    "spiked": "spike",
    "spikelets": "spikelet",
    "spikes": "spike",
    "spikier": "spiky",
    "spikiest": "spiky",
    "spiking": "spike",
    "spill-overs": "spill-over",
    "spillages": "spillage",
    "spilled": "spill",
    "spilling": "spill",
    "spillovers": "spillover",
    "spills": "spill",
    "spillways": "spillway",
    "spilt": "spill",
    "spin-dried": "spin-dry",
    "spin-driers": "spin-drier",
    "spin-dries": "spin-dry",
    "spin-drying": "spin-dry",
    "spin-offs": "spin-off",
    "spinae": "spina",
    "spinals": "spinal",
    "spindled": "spindle",
    "spindles": "spindle",
    "spindlier": "spindly",
    "spindliest": "spindly",
    "spindling": "spindle",
    "spinels": "spinel",
    "spines": "spine",
    "spinets": "spinet",
    "spinier": "spiny",
    "spinnakers": "spinnaker",
    "spinners": "spinner",
    "spinneys": "spinney",
    "spinning": "spin",
    "spins": "spin",
    "spinsters": "spinster",
    "spiraled": "spiral",
    "spiraling": "spiral",
    "spiralled": "spiral",
    "spiralling": "spiral",
    "spirals": "spiral",
    "spires": "spire",
    "spirited": "spirit",
    "spiriting": "spirit",
    "spirits": "spirit",
    "spiritualised": "spiritualise",
    "spiritualises": "spiritualise",
    "spiritualising": "spiritualise",
    "spiritualists": "spiritualist",
    "spiritualized": "spiritualize",
    "spiritualizes": "spiritualize",
    "spiritualizing": "spiritualize",
    "spirituals": "spiritual",
    "spirometries": "spirometry",
    "spironolactones": "spironolactone",
    "spited": "spite",
    "spites": "spite",
    "spitfires": "spitfire",
    "spiting": "spite",
    "spitroasted": "spitroast",
    "spitroasting": "spitroast",
    "spitroasts": "spitroast",
    "spits": "spit",
    "spitted": "spit",
    "spitting": "spit",
    "spittoons": "spittoon",
    "spivs": "spiv",
    "splashdowns": "splashdown",
    "splashed": "splash",
    "splashes": "splash",
    "splashing": "splash",
    "splattered": "splatter",
    "splattering": "splatter",
    "splatters": "splatter",
    "splayed": "splay",
    "splaying": "splay",
    "splays": "splay",
    "spleens": "spleen",
    "splendors": "splendor",
    "splendours": "splendour",
    "splenectomies": "splenectomy",
    "spliced": "splice",
    "splices": "splice",
    "splicing": "splice",
    "splines": "spline",
    "splinted": "splint",
    "splintered": "splinter",
    "splintering": "splinter",
    "splinters": "splinter",
    "splinting": "splint",
    "splints": "splint",
    "splits": "split",
    "splitters": "splitter",
    "splitting": "split",
    "splittings": "splitting",
    "splodges": "splodge",
    "splotches": "splotch",
    "splurged": "splurge",
    "splurges": "splurge",
    "splurging": "splurge",
    "spluttered": "splutter",
    "spluttering": "splutter",
    "splutters": "splutter",
    "spoiled": "spoil",
    "spoilers": "spoiler",
    "spoiling": "spoil",
    "spoils": "spoil",
    "spoilsports": "spoilsport",
    "spoilt": "spoil",
    "spoke": "speak",
    "spoken": "speak",
    "spokes": "spoke",
    "spokesmen": "spokesman",
    "spokespersons": "spokesperson",
    "spokeswomen": "spokeswoman",
    "spondees": "spondee",
    "spondylitides": "spondylitis",
    "spondyloarthropathies": "spondyloarthropathy",
    "spondyloses": "spondylosis",
    "spongebags": "spongebag",
    "sponged": "sponge",
    "spongers": "sponger",
    "sponges": "sponge",
    "spongier": "spongy",
    "spongiest": "spongy",
    "sponging": "sponge",
    "sponsored": "sponsor",
    "sponsoring": "sponsor",
    "sponsors": "sponsor",
    "spoofed": "spoof",
    "spoofing": "spoof",
    "spoofs": "spoof",
    "spooked": "spook",
    "spookier": "spooky",
    "spookiest": "spooky",
    "spooking": "spook",
    "spooks": "spook",
    "spooled": "spool",
    "spooling": "spool",
    "spools": "spool",
    "spoon-fed": "spoon-feed",
    "spoon-feeded": "spoon-feed",
    "spoon-feeding": "spoon-feed",
    "spoon-feeds": "spoon-feed",
    "spoonbills": "spoonbill",
    "spooned": "spoon",
    "spoonerisms": "spoonerism",
    "spoonfuls": "spoonful",
    "spooning": "spoon",
    "spoons": "spoon",
    "spoonsful": "spoonful",
    "sporangia": "sporangium",
    "spores": "spore",
    "sporrans": "sporran",
    "sported": "sport",
    "sportier": "sporty",
    "sportiest": "sporty",
    "sporting": "sport",
    "sports": "sport",
    "sportsmen": "sportsman",
    "sportswomen": "sportswoman",
    "sportswriters": "sportswriter",
    "sporulations": "sporulation",
    "spot-checked": "spot-check",
    "spot-checking": "spot-check",
    "spot-checks": "spot-check",
    "spotlighted": "spotlight",
    "spotlighting": "spotlight",
    "spotlights": "spotlight",
    "spots": "spot",
    "spotted": "spot",
    "spotters": "spotter",
    "spottier": "spotty",
    "spottiest": "spotty",
    "spotting": "spot",
    "spouses": "spouse",
    "spouted": "spout",
    "spouting": "spout",
    "spouts": "spout",
    "sprained": "sprain",
    "spraining": "sprain",
    "sprains": "sprain",
    "sprang": "spring",
    "sprats": "sprat",
    "sprawled": "sprawl",
    "sprawling": "sprawl",
    "sprawls": "sprawl",
    "sprayed": "spray",
    "sprayers": "sprayer",
    "spraying": "spray",
    "sprays": "spray",
    "spreaders": "spreader",
    "spreading": "spread",
    "spreads": "spread",
    "spreadsheets": "spreadsheet",
    "sprees": "spree",
    "sprier": "spry",
    "spriest": "spry",
    "sprightlier": "sprightly",
    "sprightliest": "sprightly",
    "sprigs": "sprig",
    "spring-cleaned": "spring-clean",
    "spring-cleaning": "spring-clean",
    "spring-cleans": "spring-clean",
    "springboards": "springboard",
    "springboks": "springbok",
    "springers": "springer",
    "springier": "springy",
    "springiest": "springy",
    "springing": "spring",
    "springs": "spring",
    "springtimes": "springtime",
    "sprinkled": "sprinkle",
    "sprinklers": "sprinkler",
    "sprinkles": "sprinkle",
    "sprinkling": "sprinkle",
    "sprinklings": "sprinkling",
    "sprinted": "sprint",
    "sprinters": "sprinter",
    "sprinting": "sprint",
    "sprints": "sprint",
    "sprites": "sprite",
    "sprockets": "sprocket",
    "sprouted": "sprout",
    "sprouting": "sprout",
    "sprouts": "sprout",
    "spruced": "spruce",
    "spruces": "spruce",
    "sprucing": "spruce",
    "sprues": "sprue",
    "sprung": "spring",
    "spryer": "spry",
    "spryest": "spry",
    "spuds": "spud",
    "spun": "spin-dry",
    "spunkier": "spunky",
    "spunkiest": "spunky",
    "spunks": "spunk",
    "spurges": "spurge",
    "spurned": "spurn",
    "spurning": "spurn",
    "spurns": "spurn",
    "spurred": "spur",
    "spurring": "spur",
    "spurs": "spur",
    "spurted": "spurt",
    "spurting": "spurt",
    "spurts": "spurt",
    "sputa": "sputum",
    "sputniks": "sputnik",
    "sputtered": "sputter",
    "sputtering": "sputter",
    "sputters": "sputter",
    "sputums": "sputum",
    "spying": "spy",
    "spyings": "spying",
    "squabbled": "squabble",
    "squabbles": "squabble",
    "squabbling": "squabble",
    "squabs": "squab",
    "squadrons": "squadron",
    "squads": "squad",
    "squalider": "squalid",
    "squalidest": "squalid",
    "squalled": "squall",
    "squalling": "squall",
    "squalls": "squall",
    "squalors": "squalor",
    "squandered": "squander",
    "squandering": "squander",
    "squanders": "squander",
    "squared": "square",
    "squarer": "square",
    "squares": "square",
    "squarest": "square",
    "squaring": "square",
    "squashed": "squash",
    "squashes": "squash",
    "squashier": "squashy",
    "squashiest": "squashy",
    "squashing": "squash",
    "squats": "squat",
    "squatted": "squat",
    "squatter": "squat",
    "squatters": "squatter",
    "squattest": "squat",
    "squatting": "squat",
    "squawked": "squawk",
    "squawking": "squawk",
    "squawks": "squawk",
    "squaws": "squaw",
    "squeaked": "squeak",
    "squeakers": "squeaker",
    "squeakier": "squeaky",
    "squeakiest": "squeaky",
    "squeaking": "squeak",
    "squeaks": "squeak",
    "squealed": "squeal",
    "squealing": "squeal",
    "squeals": "squeal",
    "squeegees": "squeegee",
    "squeezed": "squeeze",
    "squeezes": "squeeze",
    "squeezing": "squeeze",
    "squelched": "squelch",
    "squelches": "squelch",
    "squelching": "squelch",
    "squibs": "squib",
    "squids": "squid",
    "squiggles": "squiggle",
    "squigglier": "squiggly",
    "squinted": "squint",
    "squinting": "squint",
    "squints": "squint",
    "squires": "squire",
    "squirmed": "squirm",
    "squirming": "squirm",
    "squirms": "squirm",
    "squirrels": "squirrel",
    "squirted": "squirt",
    "squirting": "squirt",
    "squirts": "squirt",
    "squishiest": "squishy",
    "srcs": "src",
    "srs": "sr",
    "ss": "s",
    "stabbed": "stab",
    "stabbing": "stab",
    "stabbings": "stabbing",
    "stabilisations": "stabilisation",
    "stabilised": "stabilise",
    "stabilisers": "stabiliser",
    "stabilises": "stabilise",
    "stabilising": "stabilise",
    "stabilities": "stability",
    "stabilizations": "stabilization",
    "stabilized": "stabilize",
    "stabilizers": "stabilizer",
    "stabilizes": "stabilize",
    "stabilizing": "stabilize",
    "stable-boys": "stable-boy",
    "stabled": "stable",
    "stablemates": "stablemate",
    "stabler": "stable",
    "stables": "stable",
    "stablest": "stable",
    "stabling": "stable",
    "stablished": "stablish",
    "stablishes": "stablish",
    "stablishing": "stablish",
    "stabs": "stab",
    "stacked": "stack",
    "stackers": "stacker",
    "stacking": "stack",
    "stacks": "stack",
    "stadia": "stadium",
    "stadiums": "stadium",
    "staffed": "staff",
    "staffers": "staffer",
    "staffing": "staff",
    "staffs": "staff",
    "stage-managed": "stage-manage",
    "stage-managers": "stage-manager",
    "stage-manages": "stage-manage",
    "stage-managing": "stage-manage",
    "stagecoaches": "stagecoach",
    "staged": "stage",
    "stagehands": "stagehand",
    "stagers": "stager",
    "stages": "stage",
    "staggered": "stagger",
    "staggering": "stagger",
    "staggers": "stagger",
    "stagier": "stagy",
    "stagiest": "stagy",
    "staging": "stage",
    "stagnated": "stagnate",
    "stagnates": "stagnate",
    "stagnating": "stagnate",
    "stags": "stag",
    "staid": "stay",
    "staider": "staid",
    "staidest": "staid",
    "stained": "stain",
    "staining": "stain",
    "stains": "stain",
    "staircases": "staircase",
    "stairlifts": "stairlift",
    "stairs": "stair",
    "stairways": "stairway",
    "stairwells": "stairwell",
    "stake-holders": "stake-holder",
    "staked": "stake",
    "stakeholders": "stakeholder",
    "stakes": "stake",
    "staking": "stake",
    "stalactites": "stalactite",
    "stalagmites": "stalagmite",
    "stalemates": "stalemate",
    "staler": "stale",
    "stalest": "stale",
    "stalked": "stalk",
    "stalkers": "stalker",
    "stalking": "stalk",
    "stalks": "stalk",
    "stalled": "stall",
    "stallholders": "stallholder",
    "stalling": "stall",
    "stallions": "stallion",
    "stallkeepers": "stallkeeper",
    "stalls": "stall",
    "stalwarts": "stalwart",
    "stamens": "stamen",
    "stamina": "stamen",
    "stammered": "stammer",
    "stammerers": "stammerer",
    "stammering": "stammer",
    "stammers": "stammer",
    "stamped": "stamp",
    "stampeded": "stampede",
    "stampedes": "stampede",
    "stampeding": "stampede",
    "stampers": "stamper",
    "stamping": "stamp",
    "stamps": "stamp",
    "stances": "stance",
    "stanchions": "stanchion",
    "stand-bies": "stand-by",
    "stand-bys": "stand-by",
    "stand-ins": "stand-in",
    "stand-offs": "stand-off",
    "stand-ups": "stand-up",
    "standardisations": "standardisation",
    "standardised": "standardise",
    "standardises": "standardise",
    "standardising": "standardise",
    "standardizations": "standardization",
    "standardized": "standardize",
    "standardizes": "standardize",
    "standardizing": "standardize",
    "standards": "standard",
    "standbys": "standby",
    "standers": "stander",
    "standing": "stand",
    "standings": "standing",
    "standoffs": "standoff",
    "standpipes": "standpipe",
    "standpoints": "standpoint",
    "stands": "stand",
    "standstills": "standstill",
    "stank": "stink",
    "stanzas": "stanza",
    "staphylococci": "staphylococcus",
    "stapled": "staple",
    "staplers": "stapler",
    "staples": "staple",
    "stapling": "staple",
    "staplings": "stapling",
    "starboards": "starboard",
    "starbursts": "starburst",
    "starched": "starch",
    "starches": "starch",
    "starchier": "starchy",
    "starchiest": "starchy",
    "starching": "starch",
    "stardoms": "stardom",
    "stared": "stare",
    "stares": "stare",
    "starfishes": "starfish",
    "stargazers": "stargazer",
    "staring": "stare",
    "starker": "stark",
    "starkest": "stark",
    "starlets": "starlet",
    "starlings": "starling",
    "starred": "star",
    "starrier": "starry",
    "starriest": "starry",
    "starring": "star",
    "stars": "star",
    "started": "start",
    "starters": "starter",
    "starting": "start",
    "startled": "startle",
    "startles": "startle",
    "startling": "startle",
    "starts": "start",
    "starvations": "starvation",
    "starved": "starve",
    "starves": "starve",
    "starving": "starve",
    "stases": "stasis",
    "stashed": "stash",
    "stashes": "stash",
    "stashing": "stash",
    "stated": "state",
    "statelier": "stately",
    "stateliest": "stately",
    "statements": "statement",
    "staterooms": "stateroom",
    "states": "state",
    "statesmen": "statesman",
    "statics": "static",
    "stating": "state",
    "statins": "statin",
    "stationed": "station",
    "stationers": "stationer",
    "stationing": "station",
    "stationmasters": "stationmaster",
    "stations": "station",
    "statisticians": "statistician",
    "statistics": "statistic",
    "stators": "stator",
    "statues": "statue",
    "statuettes": "statuette",
    "statures": "stature",
    "statuses": "status",
    "statutes": "statute",
    "staunched": "staunch",
    "stauncher": "staunch",
    "staunches": "staunch",
    "staunchest": "staunch",
    "staunching": "staunch",
    "staved": "stave",
    "staves": "stay",
    "staving": "stave",
    "stay-at-homes": "stay-at-home",
    "stayed": "stay",
    "stayers": "stayer",
    "staying": "stay",
    "stays": "stay",
    "steadied": "steady",
    "steadier": "steady",
    "steadies": "steady",
    "steadiest": "steady",
    "steady-states": "steady-state",
    "steadying": "steady",
    "steakhouses": "steakhouse",
    "steaks": "steak",
    "stealing": "steal",
    "steals": "steal",
    "stealthier": "stealthy",
    "stealthiest": "stealthy",
    "steamboats": "steamboat",
    "steamed": "steam",
    "steamers": "steamer",
    "steamier": "steamy",
    "steamiest": "steamy",
    "steaming": "steam",
    "steamrollered": "steamroller",
    "steamrollering": "steamroller",
    "steamrollers": "steamroller",
    "steams": "steam",
    "stearates": "stearate",
    "steeds": "steed",
    "steeled": "steel",
    "steelier": "steely",
    "steeling": "steel",
    "steelmakers": "steelmaker",
    "steelmills": "steelmill",
    "steels": "steel",
    "steelworkers": "steelworker",
    "steeped": "steep",
    "steepened": "steepen",
    "steepening": "steepen",
    "steepens": "steepen",
    "steeper": "steep",
    "steepest": "steep",
    "steeping": "steep",
    "steeplechases": "steeplechase",
    "steeplejacks": "steeplejack",
    "steeples": "steeple",
    "steepnesses": "steepness",
    "steeps": "steep",
    "steered": "steer",
    "steerers": "steerer",
    "steering": "steer",
    "steers": "steer",
    "steersmen": "steersman",
    "steins": "stein",
    "stelae": "stela",
    "steles": "stele",
    "stellae": "stella",
    "stem-cells": "stem-cell",
    "stemmed": "stem",
    "stemming": "stem",
    "stems": "stem",
    "stenches": "stench",
    "stenciled": "stencil",
    "stenciling": "stencil",
    "stencilled": "stencil",
    "stencilling": "stencil",
    "stencils": "stencil",
    "stenographers": "stenographer",
    "stenoses": "stenosis",
    "stentings": "stenting",
    "stents": "stent",
    "step-daughters": "step-daughter",
    "step-mothers": "step-mother",
    "step-parents": "step-parent",
    "stepbrothers": "stepbrother",
    "stepchildren": "stepchild",
    "stepchilds": "stepchild",
    "stepdaughters": "stepdaughter",
    "stepfamilies": "stepfamily",
    "stepfathers": "stepfather",
    "stepladders": "stepladder",
    "stepmothers": "stepmother",
    "stepparents": "stepparent",
    "stepped": "step",
    "steppers": "stepper",
    "steppes": "steppe",
    "stepping": "step",
    "steppings": "stepping",
    "steps": "step",
    "stepsisters": "stepsister",
    "stepsons": "stepson",
    "stereochemistries": "stereochemistry",
    "stereograms": "stereogram",
    "stereos": "stereo",
    "stereoscopes": "stereoscope",
    "stereotyped": "stereotype",
    "stereotypes": "stereotype",
    "stereotypies": "stereotypy",
    "stereotyping": "stereotype",
    "sterilisations": "sterilisation",
    "sterilised": "sterilise",
    "sterilisers": "steriliser",
    "sterilises": "sterilise",
    "sterilising": "sterilise",
    "sterilities": "sterility",
    "sterilizations": "sterilization",
    "sterilized": "sterilize",
    "sterilizers": "sterilizer",
    "sterilizes": "sterilize",
    "sterilizing": "sterilize",
    "sterna": "sternum",
    "sterner": "stern",
    "sternest": "stern",
    "sterns": "stern",
    "sternums": "sternum",
    "steroids": "steroid",
    "sterols": "sterol",
    "stethoscopes": "stethoscope",
    "stetsons": "stetson",
    "stevedores": "stevedore",
    "stewarded": "steward",
    "stewardesses": "stewardess",
    "stewarding": "steward",
    "stewards": "steward",
    "stewed": "stew",
    "stewing": "stew",
    "stews": "stew",
    "stick-in-the-muds": "stick-in-the-mud",
    "stick-ups": "stick-up",
    "stickers": "sticker",
    "stickier": "sticky",
    "stickiest": "sticky",
    "sticking": "stick",
    "sticklebacks": "stickleback",
    "sticklers": "stickler",
    "stickpins": "stickpin",
    "sticks": "stick",
    "sties": "sty",
    "stiffened": "stiffen",
    "stiffeners": "stiffener",
    "stiffening": "stiffen",
    "stiffens": "stiffen",
    "stiffer": "stiff",
    "stiffest": "stiff",
    "stiffnesses": "stiffness",
    "stiffs": "stiff",
    "stifled": "stifle",
    "stifles": "stifle",
    "stifling": "stifle",
    "stigmas": "stigma",
    "stigmata": "stigma",
    "stigmatisations": "stigmatisation",
    "stigmatised": "stigmatise",
    "stigmatises": "stigmatise",
    "stigmatising": "stigmatise",
    "stigmatized": "stigmatize",
    "stigmatizes": "stigmatize",
    "stigmatizing": "stigmatize",
    "stiles": "stile",
    "stilettoes": "stiletto",
    "stilettos": "stiletto",
    "still-births": "still-birth",
    "still-lifes": "still-life",
    "stillbirths": "stillbirth",
    "stillborns": "stillborn",
    "stilled": "still",
    "stiller": "still",
    "stillest": "still",
    "stilling": "still",
    "stills": "still",
    "stilted": "stilt",
    "stilting": "stilt",
    "stilts": "stilt",
    "stims": "stim",
    "stimulants": "stimulant",
    "stimulated": "stimulate",
    "stimulates": "stimulate",
    "stimulating": "stimulate",
    "stimulations": "stimulation",
    "stimulators": "stimulator",
    "stimuli": "stimulus",
    "stimuluses": "stimulus",
    "stingers": "stinger",
    "stingier": "stingy",
    "stingiest": "stingy",
    "stinging": "sting",
    "stingrays": "stingray",
    "stings": "sting",
    "stinkers": "stinker",
    "stinking": "stink",
    "stinks": "stink",
    "stinted": "stint",
    "stinting": "stint",
    "stints": "stint",
    "stipendiaries": "stipendiary",
    "stipends": "stipend",
    "stipes": "stipe",
    "stippled": "stipple",
    "stipples": "stipple",
    "stippling": "stipple",
    "stipulated": "stipulate",
    "stipulates": "stipulate",
    "stipulating": "stipulate",
    "stipulations": "stipulation",
    "stir-fried": "stir-fry",
    "stir-fries": "stir-fry",
    "stir-frying": "stir-fry",
    "stirred": "stir",
    "stirrers": "stirrer",
    "stirring": "stir",
    "stirrings": "stirring",
    "stirrups": "stirrup",
    "stirs": "stir",
    "stitched": "stitch",
    "stitchers": "stitcher",
    "stitches": "stitch",
    "stitching": "stitch",
    "stoats": "stoat",
    "stock-cars": "stock-car",
    "stock-in-trades": "stock-in-trade",
    "stock-markets": "stock-market",
    "stock-takings": "stock-taking",
    "stockades": "stockade",
    "stockbreeders": "stockbreeder",
    "stockbrokers": "stockbroker",
    "stocked": "stock",
    "stockholders": "stockholder",
    "stockier": "stocky",
    "stockiest": "stocky",
    "stocking": "stock",
    "stockings": "stocking",
    "stockists": "stockist",
    "stockmen": "stockman",
    "stockpiled": "stockpile",
    "stockpiles": "stockpile",
    "stockpiling": "stockpile",
    "stockrooms": "stockroom",
    "stocks": "stock",
    "stocktakings": "stocktaking",
    "stodgier": "stodgy",
    "stodgiest": "stodgy",
    "stoical": "stoic",
    "stoichiometries": "stoichiometry",
    "stoics": "stoic",
    "stoked": "stoke",
    "stokers": "stoker",
    "stokes": "stoke",
    "stoking": "stoke",
    "stole": "steal",
    "stolen": "steal",
    "stoles": "stole",
    "stolider": "stolid",
    "stolidest": "stolid",
    "stolons": "stolon",
    "stomach-aches": "stomach-ache",
    "stomached": "stomach",
    "stomaches": "stomach",
    "stomaching": "stomach",
    "stomachs": "stomach",
    "stomas": "stoma",
    "stomata": "stoma",
    "stomatitides": "stomatitis",
    "stomped": "stomp",
    "stomping": "stomp",
    "stomps": "stomp",
    "stone-walled": "stone-wall",
    "stone-walling": "stone-wall",
    "stone-walls": "stone-wall",
    "stonebreakers": "stonebreaker",
    "stonechats": "stonechat",
    "stoned": "stone",
    "stonemasons": "stonemason",
    "stoners": "stoner",
    "stones": "stone",
    "stonewalled": "stonewall",
    "stonewalling": "stonewall",
    "stonewalls": "stonewall",
    "stoneworts": "stonewort",
    "stonier": "stony",
    "stoniest": "stony",
    "stoning": "stone",
    "stood": "stand",
    "stooges": "stooge",
    "stoolpigeons": "stoolpigeon",
    "stools": "stool",
    "stooped": "stoop",
    "stooping": "stoop",
    "stoops": "stoop",
    "stop-overs": "stop-over",
    "stop-pressed": "stop-press",
    "stop-presses": "stop-press",
    "stop-pressing": "stop-press",
    "stopcocks": "stopcock",
    "stoped": "stope",
    "stopes": "stope",
    "stopgaps": "stopgap",
    "stoping": "stope",
    "stoplights": "stoplight",
    "stopovers": "stopover",
    "stoppages": "stoppage",
    "stopped": "stop",
    "stoppered": "stopper",
    "stoppering": "stopper",
    "stoppers": "stopper",
    "stopping": "stop",
    "stops": "stop",
    "stopwatches": "stopwatch",
    "storages": "storage",
    "stored": "store",
    "storefronts": "storefront",
    "storehouses": "storehouse",
    "storekeepers": "storekeeper",
    "storerooms": "storeroom",
    "stores": "store",
    "storeys": "storey",
    "stories": "story",
    "storing": "store",
    "storks": "stork",
    "stormed": "storm",
    "stormier": "stormy",
    "stormiest": "stormy",
    "storming": "storm",
    "storms": "storm",
    "story-tellers": "story-teller",
    "storybooks": "storybook",
    "storytellers": "storyteller",
    "stouter": "stout",
    "stoutest": "stout",
    "stouts": "stout",
    "stove": "stave",
    "stovepipes": "stovepipe",
    "stoves": "stove",
    "stowages": "stowage",
    "stowaways": "stowaway",
    "stowed": "stow",
    "stowing": "stow",
    "stows": "stow",
    "straddled": "straddle",
    "straddles": "straddle",
    "straddling": "straddle",
    "strafed": "strafe",
    "strafes": "strafe",
    "strafing": "strafe",
    "straggled": "straggle",
    "stragglers": "straggler",
    "straggles": "straggle",
    "stragglier": "straggly",
    "straggliest": "straggly",
    "straggling": "straggle",
    "straightaways": "straightaway",
    "straightened": "straighten",
    "straighteners": "straightener",
    "straightening": "straighten",
    "straightens": "straighten",
    "straighter": "straight",
    "straightest": "straight",
    "straights": "straight",
    "strained": "strain",
    "strainers": "strainer",
    "straining": "strain",
    "strains": "strain",
    "straitjackets": "straitjacket",
    "straits": "strait",
    "strakes": "strake",
    "stranded": "strand",
    "stranding": "strand",
    "strands": "strand",
    "stranger": "strange",
    "strangers": "stranger",
    "strangest": "strange",
    "strangled": "strangle",
    "strangleholds": "stranglehold",
    "stranglers": "strangler",
    "strangles": "strangle",
    "strangling": "strangle",
    "strangulated": "strangulate",
    "strangulates": "strangulate",
    "strangulating": "strangulate",
    "strangulations": "strangulation",
    "strapped": "strap",
    "strapping": "strap",
    "straps": "strap",
    "strata": "stratum",
    "stratagems": "stratagem",
    "strategies": "strategy",
    "strategised": "strategise",
    "strategises": "strategise",
    "strategising": "strategise",
    "strategists": "strategist",
    "strategized": "strategize",
    "strategizes": "strategize",
    "strategizing": "strategize",
    "strati": "stratus",
    "stratifications": "stratification",
    "stratified": "stratify",
    "stratifies": "stratify",
    "stratifying": "stratify",
    "stratospheres": "stratosphere",
    "stratums": "stratum",
    "strawberries": "strawberry",
    "straws": "straw",
    "strayed": "stray",
    "straying": "stray",
    "strays": "stray",
    "streaked": "streak",
    "streakers": "streaker",
    "streakier": "streaky",
    "streakies": "streaky",
    "streakiest": "streaky",
    "streaking": "streak",
    "streaks": "streak",
    "streamed": "stream",
    "streamers": "streamer",
    "streaming": "stream",
    "streamlined": "streamline",
    "streamlines": "streamline",
    "streamlining": "streamline",
    "streams": "stream",
    "streetcars": "streetcar",
    "streetlamps": "streetlamp",
    "streetlevels": "streetlevel",
    "streetlights": "streetlight",
    "streets": "street",
    "streetwalkers": "streetwalker",
    "strengthened": "strengthen",
    "strengthening": "strengthen",
    "strengthens": "strengthen",
    "strengths": "strength",
    "streptococci": "streptococcus",
    "streptokinases": "streptokinase",
    "stressed": "stress",
    "stresses": "stress",
    "stressing": "stress",
    "stressors": "stressor",
    "stretched": "stretch",
    "stretcher-bearers": "stretcher-bearer",
    "stretchers": "stretcher",
    "stretches": "stretch",
    "stretchier": "stretchy",
    "stretchiest": "stretchy",
    "stretching": "stretch",
    "strewed": "strew",
    "strewing": "strew",
    "strewn": "strew",
    "strews": "strew",
    "striae": "stria",
    "strias": "stria",
    "striata": "striatum",
    "striated": "striate",
    "striates": "striate",
    "striating": "striate",
    "striations": "striation",
    "striatums": "striatum",
    "stricken": "strike",
    "stricter": "strict",
    "strictest": "strict",
    "strictures": "stricture",
    "strid": "stride",
    "stridden": "stride",
    "stridenter": "strident",
    "stridentest": "strident",
    "striders": "strider",
    "strides": "stride",
    "striding": "stride",
    "strike-breakers": "strike-breaker",
    "strikebreakers": "strikebreaker",
    "strikers": "striker",
    "strikes": "strike",
    "striking": "strike",
    "stringencies": "stringency",
    "stringier": "stringy",
    "stringiest": "stringy",
    "stringing": "string",
    "strings": "string",
    "striped": "stripe",
    "stripes": "stripe",
    "stripier": "stripy",
    "stripiest": "stripy",
    "striping": "stripe",
    "striplings": "stripling",
    "stripped": "strip",
    "strippers": "stripper",
    "stripping": "strip",
    "strips": "strip",
    "stripteases": "striptease",
    "strived": "strive",
    "striven": "strive",
    "strives": "strive",
    "striving": "strive",
    "strivings": "striving",
    "strobes": "strobe",
    "strode": "stride",
    "stroked": "stroke",
    "strokes": "stroke",
    "stroking": "stroke",
    "strolled": "stroll",
    "strollers": "stroller",
    "strolling": "stroll",
    "strolls": "stroll",
    "stromas": "stroma",
    "stromata": "stroma",
    "stronger": "strong",
    "strongest": "strong",
    "strongholds": "stronghold",
    "strongmen": "strongman",
    "strongrooms": "strongroom",
    "strophes": "strophe",
    "stropped": "strop",
    "stroppier": "stroppy",
    "stroppiest": "stroppy",
    "stropping": "strop",
    "strops": "strop",
    "strove": "strive",
    "strs": "str",
    "struck": "strike",
    "structs": "struct",
    "structuralists": "structuralist",
    "structured": "structure",
    "structures": "structure",
    "structuring": "structure",
    "strudels": "strudel",
    "struggled": "struggle",
    "strugglers": "struggler",
    "struggles": "struggle",
    "struggling": "struggle",
    "strummed": "strum",
    "strumming": "strum",
    "strumpets": "strumpet",
    "strums": "strum",
    "strung": "string",
    "struts": "strut",
    "strutted": "strut",
    "strutting": "strut",
    "stubbed": "stub",
    "stubbier": "stubby",
    "stubbiest": "stubby",
    "stubbing": "stub",
    "stubbles": "stubble",
    "stubblier": "stubbly",
    "stubborner": "stubborn",
    "stubbornest": "stubborn",
    "stubs": "stub",
    "stuccoes": "stucco",
    "stuccos": "stucco",
    "stuck": "stick",
    "studded": "stud",
    "studding": "stud",
    "students": "student",
    "studentships": "studentship",
    "studied": "study",
    "studies": "study",
    "studios": "studio",
    "studs": "stud",
    "studying": "study",
    "stuffed": "stuff",
    "stuffier": "stuffy",
    "stuffiest": "stuffy",
    "stuffing": "stuff",
    "stuffs": "stuff",
    "stultified": "stultify",
    "stultifies": "stultify",
    "stultifying": "stultify",
    "stumbled": "stumble",
    "stumbles": "stumble",
    "stumbling": "stumble",
    "stumped": "stump",
    "stumpier": "stumpy",
    "stumpiest": "stumpy",
    "stumping": "stump",
    "stumps": "stump",
    "stung": "sting",
    "stunk": "stink",
    "stunned": "stun",
    "stunners": "stunner",
    "stunning": "stun",
    "stuns": "stun",
    "stunted": "stunt",
    "stunting": "stunt",
    "stunts": "stunt",
    "stupefied": "stupefy",
    "stupefies": "stupefy",
    "stupefying": "stupefy",
    "stupider": "stupid",
    "stupidest": "stupid",
    "stupidities": "stupidity",
    "stupors": "stupor",
    "stupours": "stupour",
    "sturdier": "sturdy",
    "sturdiest": "sturdy",
    "sturgeons": "sturgeon",
    "stuttered": "stutter",
    "stuttering": "stutter",
    "stutters": "stutter",
    "styes": "stye",
    "styled": "style",
    "styles": "style",
    "styli": "stylus",
    "styling": "style",
    "stylised": "stylise",
    "stylises": "stylise",
    "stylising": "stylise",
    "stylistics": "stylistic",
    "stylists": "stylist",
    "stylized": "stylize",
    "stylizes": "stylize",
    "stylizing": "stylize",
    "styluses": "stylus",
    "stymied": "stymie",
    "stymieing": "stymie",
    "stymies": "stymie",
    "styrenes": "styrene",
    "suaver": "suave",
    "suavest": "suave",
    "sub-adults": "sub-adult",
    "sub-areas": "sub-area",
    "sub-assemblies": "sub-assembly",
    "sub-bases": "sub-base",
    "sub-bottoms": "sub-bottom",
    "sub-categories": "sub-category",
    "sub-classes": "sub-class",
    "sub-committees": "sub-committee",
    "sub-communities": "sub-community",
    "sub-components": "sub-component",
    "sub-continents": "sub-continent",
    "sub-contractors": "sub-contractor",
    "sub-cultures": "sub-culture",
    "sub-degrees": "sub-degree",
    "sub-departments": "sub-department",
    "sub-directories": "sub-directory",
    "sub-disciplines": "sub-discipline",
    "sub-districts": "sub-district",
    "sub-divided": "sub-divide",
    "sub-divides": "sub-divide",
    "sub-dividing": "sub-divide",
    "sub-divisions": "sub-division",
    "sub-domains": "sub-domain",
    "sub-elements": "sub-element",
    "sub-fields": "sub-field",
    "sub-floors": "sub-floor",
    "sub-groups": "sub-group",
    "sub-headings": "sub-heading",
    "sub-lieutenants": "sub-lieutenant",
    "sub-menus": "sub-menu",
    "sub-micra": "sub-micron",
    "sub-microns": "sub-micron",
    "sub-models": "sub-model",
    "sub-modules": "sub-module",
    "sub-panels": "sub-panel",
    "sub-pixels": "sub-pixel",
    "sub-plots": "sub-plot",
    "sub-populations": "sub-population",
    "sub-processes": "sub-process",
    "sub-projects": "sub-project",
    "sub-regions": "sub-region",
    "sub-samples": "sub-sample",
    "sub-scales": "sub-scale",
    "sub-sections": "sub-section",
    "sub-sectors": "sub-sector",
    "sub-sets": "sub-set",
    "sub-sites": "sub-site",
    "sub-soils": "sub-soil",
    "sub-specialities": "sub-speciality",
    "sub-specialties": "sub-specialty",
    "sub-structures": "sub-structure",
    "sub-surfaces": "sub-surface",
    "sub-systems": "sub-system",
    "sub-tasks": "sub-task",
    "sub-themes": "sub-theme",
    "sub-trees": "sub-tree",
    "sub-types": "sub-type",
    "sub-units": "sub-unit",
    "sub-zones": "sub-zone",
    "subalterns": "subaltern",
    "subareas": "subarea",
    "subarrays": "subarray",
    "subassemblies": "subassembly",
    "subbed": "sub",
    "subbing": "sub",
    "subcarriers": "subcarrier",
    "subcategories": "subcategory",
    "subcategorized": "subcategorize",
    "subcategorizes": "subcategorize",
    "subcategorizing": "subcategorize",
    "subclasses": "subclass",
    "subcommittees": "subcommittee",
    "subcomponents": "subcomponent",
    "subconsciouses": "subconscious",
    "subcontinents": "subcontinent",
    "subcontracted": "subcontract",
    "subcontracting": "subcontract",
    "subcontractors": "subcontractor",
    "subcontracts": "subcontract",
    "subcultures": "subculture",
    "subdirectories": "subdirectory",
    "subdivided": "subdivide",
    "subdivides": "subdivide",
    "subdividing": "subdivide",
    "subdivisions": "subdivision",
    "subdomains": "subdomain",
    "subdominants": "subdominant",
    "subducted": "subduct",
    "subducting": "subduct",
    "subducts": "subduct",
    "subdued": "subdue",
    "subdues": "subdue",
    "subduing": "subdue",
    "subed": "sub",
    "subeditors": "subeditor",
    "subfamilies": "subfamily",
    "subfields": "subfield",
    "subfloors": "subfloor",
    "subforms": "subform",
    "subgenera": "subgenus",
    "subgoals": "subgoal",
    "subgraphs": "subgraph",
    "subgrids": "subgrid",
    "subgroups": "subgroup",
    "subheadings": "subheading",
    "subimages": "subimage",
    "subing": "sub",
    "subjected": "subject",
    "subjecting": "subject",
    "subjectivists": "subjectivist",
    "subjectivities": "subjectivity",
    "subjects": "subject",
    "subjoined": "subjoin",
    "subjoining": "subjoin",
    "subjoins": "subjoin",
    "subjs": "subj",
    "subjugated": "subjugate",
    "subjugates": "subjugate",
    "subjugating": "subjugate",
    "subjunctives": "subjunctive",
    "subleases": "sublease",
    "sublets": "sublet",
    "subletted": "sublet",
    "subletting": "sublet",
    "sublicenses": "sublicense",
    "sublimated": "sublimate",
    "sublimates": "sublimate",
    "sublimating": "sublimate",
    "sublimations": "sublimation",
    "sublimer": "sublime",
    "sublimest": "sublime",
    "subluxations": "subluxation",
    "submariners": "submariner",
    "submarines": "submarine",
    "submediants": "submediant",
    "submenus": "submenu",
    "submerged": "submerge",
    "submerges": "submerge",
    "submerging": "submerge",
    "submersed": "submerse",
    "submerses": "submerse",
    "submersibles": "submersible",
    "submersing": "submerse",
    "submersions": "submersion",
    "submissions": "submission",
    "submits": "submit",
    "submitted": "submit",
    "submitters": "submitter",
    "submitting": "submit",
    "submodels": "submodel",
    "subnets": "subnet",
    "subnetworks": "subnetwork",
    "subordinated": "subordinate",
    "subordinates": "subordinate",
    "subordinating": "subordinate",
    "subordinations": "subordination",
    "suborned": "suborn",
    "suborning": "suborn",
    "suborns": "suborn",
    "subparagraphs": "subparagraph",
    "subparts": "subpart",
    "subpatterns": "subpattern",
    "subplots": "subplot",
    "subpoenaed": "subpoena",
    "subpoenaing": "subpoena",
    "subpoenas": "subpoena",
    "subpopulations": "subpopulation",
    "subprocesses": "subprocess",
    "subprograms": "subprogram",
    "subregions": "subregion",
    "subrogated": "subrogate",
    "subrogates": "subrogate",
    "subrogating": "subrogate",
    "subroutines": "subroutine",
    "subs": "sub",
    "subsamples": "subsample",
    "subscales": "subscale",
    "subscribed": "subscribe",
    "subscribers": "subscriber",
    "subscribes": "subscribe",
    "subscribing": "subscribe",
    "subscriptions": "subscription",
    "subscripts": "subscript",
    "subsections": "subsection",
    "subsectors": "subsector",
    "subsequences": "subsequence",
    "subserved": "subserve",
    "subserves": "subserve",
    "subserving": "subserve",
    "subsets": "subset",
    "subshells": "subshell",
    "subsided": "subside",
    "subsidences": "subsidence",
    "subsides": "subside",
    "subsidiaries": "subsidiary",
    "subsidies": "subsidy",
    "subsiding": "subside",
    "subsidised": "subsidise",
    "subsidises": "subsidise",
    "subsidising": "subsidise",
    "subsidized": "subsidize",
    "subsidizes": "subsidize",
    "subsidizing": "subsidize",
    "subsisted": "subsist",
    "subsisting": "subsist",
    "subsists": "subsist",
    "subsites": "subsite",
    "subsoils": "subsoil",
    "subspaces": "subspace",
    "subspecialties": "subspecialty",
    "substages": "substage",
    "substances": "substance",
    "substantiae": "substantia",
    "substantiated": "substantiate",
    "substantiates": "substantiate",
    "substantiating": "substantiate",
    "substantiations": "substantiation",
    "substations": "substation",
    "substituents": "substituent",
    "substituted": "substitute",
    "substitutes": "substitute",
    "substituting": "substitute",
    "substitutions": "substitution",
    "substorms": "substorm",
    "substrata": "substratum",
    "substrates": "substrate",
    "substratums": "substratum",
    "substrings": "substring",
    "substructures": "substructure",
    "subsumed": "subsume",
    "subsumes": "subsume",
    "subsuming": "subsume",
    "subsumptions": "subsumption",
    "subsurfaces": "subsurface",
    "subsystems": "subsystem",
    "subtasks": "subtask",
    "subtended": "subtend",
    "subtending": "subtend",
    "subtends": "subtend",
    "subterfuges": "subterfuge",
    "subtests": "subtest",
    "subtexts": "subtext",
    "subtitled": "subtitle",
    "subtitles": "subtitle",
    "subtitling": "subtitle",
    "subtler": "subtle",
    "subtlest": "subtle",
    "subtleties": "subtlety",
    "subtopics": "subtopic",
    "subtotals": "subtotal",
    "subtracted": "subtract",
    "subtracting": "subtract",
    "subtractions": "subtraction",
    "subtracts": "subtract",
    "subtrees": "subtree",
    "subtypes": "subtype",
    "subunits": "subunit",
    "suburbs": "suburb",
    "subventions": "subvention",
    "subversions": "subversion",
    "subversives": "subversive",
    "subverted": "subvert",
    "subverting": "subvert",
    "subverts": "subvert",
    "subways": "subway",
    "subwindows": "subwindow",
    "succeeded": "succeed",
    "succeeding": "succeed",
    "succeeds": "succeed",
    "successes": "success",
    "successions": "succession",
    "successors": "successor",
    "succinates": "succinate",
    "succoured": "succour",
    "succouring": "succour",
    "succours": "succour",
    "succulents": "succulent",
    "succumbed": "succumb",
    "succumbing": "succumb",
    "succumbs": "succumb",
    "sucked": "suck",
    "suckered": "sucker",
    "suckering": "sucker",
    "suckers": "sucker",
    "sucking": "suck",
    "suckled": "suckle",
    "sucklers": "suckler",
    "suckles": "suckle",
    "suckling": "suckle",
    "sucklings": "suckling",
    "sucks": "suck",
    "suctions": "suction",
    "suds": "sud",
    "sued": "sue",
    "sues": "sue",
    "suffered": "suffer",
    "sufferers": "sufferer",
    "suffering": "suffer",
    "sufferings": "suffering",
    "suffers": "suffer",
    "sufficed": "suffice",
    "suffices": "suffice",
    "sufficiencies": "sufficiency",
    "sufficing": "suffice",
    "suffixed": "suffix",
    "suffixes": "suffix",
    "suffixing": "suffix",
    "suffocated": "suffocate",
    "suffocates": "suffocate",
    "suffocating": "suffocate",
    "suffocations": "suffocation",
    "suffrages": "suffrage",
    "suffragettes": "suffragette",
    "suffragists": "suffragist",
    "suffused": "suffuse",
    "suffuses": "suffuse",
    "suffusing": "suffuse",
    "sugar-coated": "sugar-coat",
    "sugar-coating": "sugar-coat",
    "sugar-coats": "sugar-coat",
    "sugared": "sugar",
    "sugarier": "sugary",
    "sugaring": "sugar",
    "sugars": "sugar",
    "suggested": "suggest",
    "suggesting": "suggest",
    "suggestions": "suggestion",
    "suggests": "suggest",
    "suicides": "suicide",
    "suing": "sue",
    "suitabilities": "suitability",
    "suitcases": "suitcase",
    "suited": "suit",
    "suites": "suite",
    "suiting": "suit",
    "suitors": "suitor",
    "suits": "suit",
    "sulci": "sulcus",
    "sulfates": "sulfate",
    "sulfides": "sulfide",
    "sulfurs": "sulfur",
    "sulked": "sulk",
    "sulkier": "sulky",
    "sulkiest": "sulky",
    "sulking": "sulk",
    "sulks": "sulk",
    "sullas": "sulla",
    "sullied": "sully",
    "sullies": "sully",
    "sullying": "sully",
    "sulphasalazines": "sulphasalazine",
    "sulphated": "sulphate",
    "sulphates": "sulphate",
    "sulphating": "sulphate",
    "sulphides": "sulphide",
    "sulphites": "sulphite",
    "sulphonamides": "sulphonamide",
    "sulphonylureas": "sulphonylurea",
    "sulphurs": "sulphur",
    "sultanas": "sultana",
    "sultanates": "sultanate",
    "sultans": "sultan",
    "sultrier": "sultry",
    "sultriest": "sultry",
    "summae": "summa",
    "summaries": "summary",
    "summarisations": "summarisation",
    "summarised": "summarise",
    "summarises": "summarise",
    "summarising": "summarise",
    "summarizations": "summarization",
    "summarized": "summarize",
    "summarizes": "summarize",
    "summarizing": "summarize",
    "summations": "summation",
    "summed": "sum",
    "summerhouses": "summerhouse",
    "summerier": "summery",
    "summeriest": "summery",
    "summers": "summer",
    "summertimes": "summertime",
    "summing": "sum",
    "summings-up": "summing-up",
    "summits": "summit",
    "summoned": "summon",
    "summoning": "summon",
    "summons": "summon",
    "summonsed": "summons",
    "summonses": "summons",
    "summonsing": "summons",
    "sumps": "sump",
    "sums": "sum",
    "sunbathed": "sunbathe",
    "sunbathers": "sunbather",
    "sunbathes": "sunbathe",
    "sunbathing": "sunbathe",
    "sunbeams": "sunbeam",
    "sunbeds": "sunbed",
    "sunbirds": "sunbird",
    "sunblocks": "sunblock",
    "sunbonnets": "sunbonnet",
    "sunburned": "sunburn",
    "sunburning": "sunburn",
    "sunburns": "sunburn",
    "sunburnt": "sunburn",
    "sunbursts": "sunburst",
    "sundaes": "sundae",
    "sundecks": "sundeck",
    "sundered": "sunder",
    "sundering": "sunder",
    "sunders": "sunder",
    "sundews": "sundew",
    "sundials": "sundial",
    "sundowners": "sundowner",
    "sundowns": "sundown",
    "sundries": "sundry",
    "sunfishes": "sunfish",
    "sunflowers": "sunflower",
    "sung": "sing",
    "sunglasses": "sunglass",
    "sunhats": "sunhat",
    "sunk": "sink",
    "sunken": "sink",
    "sunlights": "sunlight",
    "sunned": "sun",
    "sunnier": "sunny",
    "sunniest": "sunny",
    "sunning": "sun",
    "sunrises": "sunrise",
    "sunroofs": "sunroof",
    "suns": "sun",
    "sunscreens": "sunscreen",
    "sunsets": "sunset",
    "sunshades": "sunshade",
    "sunspots": "sunspot",
    "sunstrokes": "sunstroke",
    "suntans": "suntan",
    "suntraps": "suntrap",
    "super-computers": "super-computer",
    "super-egos": "super-ego",
    "super-heroes": "super-hero",
    "super-powers": "super-power",
    "super-states": "super-state",
    "superber": "superb",
    "superbest": "superb",
    "superbugs": "superbug",
    "superceded": "supercede",
    "supercedes": "supercede",
    "superceding": "supercede",
    "supercells": "supercell",
    "supercharged": "supercharge",
    "superchargers": "supercharger",
    "supercharges": "supercharge",
    "supercharging": "supercharge",
    "superclasses": "superclass",
    "supercomplexes": "supercomplex",
    "supercomputers": "supercomputer",
    "superconductors": "superconductor",
    "supercooled": "supercool",
    "supercooling": "supercool",
    "supercools": "supercool",
    "superegos": "superego",
    "superfamilies": "superfamily",
    "superficialities": "superficiality",
    "superfluids": "superfluid",
    "superfluities": "superfluity",
    "supergiants": "supergiant",
    "supergravities": "supergravity",
    "supergroups": "supergroup",
    "superheated": "superheat",
    "superheating": "superheat",
    "superheats": "superheat",
    "superheroes": "superhero",
    "superhighways": "superhighway",
    "superimposed": "superimpose",
    "superimposes": "superimpose",
    "superimposing": "superimpose",
    "superimpositions": "superimposition",
    "superintended": "superintend",
    "superintendents": "superintendent",
    "superintending": "superintend",
    "superintends": "superintend",
    "superiorities": "superiority",
    "superiors": "superior",
    "superlatives": "superlative",
    "superlattices": "superlattice",
    "supermarkets": "supermarket",
    "supermen": "superman",
    "supermodels": "supermodel",
    "supernaturalisms": "supernaturalism",
    "supernovae": "supernova",
    "superoxides": "superoxide",
    "superposed": "superpose",
    "superposes": "superpose",
    "superposing": "superpose",
    "superpositions": "superposition",
    "superpowers": "superpower",
    "superscripts": "superscript",
    "superseded": "supersede",
    "supersedes": "supersede",
    "superseding": "supersede",
    "supersets": "superset",
    "superstars": "superstar",
    "superstitions": "superstition",
    "superstores": "superstore",
    "superstrings": "superstring",
    "superstructures": "superstructure",
    "supersymmetries": "supersymmetry",
    "supertankers": "supertanker",
    "supervened": "supervene",
    "supervenes": "supervene",
    "supervening": "supervene",
    "supervised": "supervise",
    "supervisees": "supervisee",
    "supervises": "supervise",
    "supervising": "supervise",
    "supervisors": "supervisor",
    "supped": "sup",
    "suppers": "supper",
    "supping": "sup",
    "supplanted": "supplant",
    "supplanting": "supplant",
    "supplants": "supplant",
    "supplementations": "supplementation",
    "supplemented": "supplement",
    "supplementing": "supplement",
    "supplements": "supplement",
    "suppler": "supple",
    "supplest": "supple",
    "supplicants": "supplicant",
    "supplicated": "supplicate",
    "supplicates": "supplicate",
    "supplicating": "supplicate",
    "supplications": "supplication",
    "supplied": "supply",
    "suppliers": "supplier",
    "supplies": "supply",
    "suppls": "suppl",
    "supplying": "supply",
    "supported": "support",
    "supporters": "supporter",
    "supporting": "support",
    "supports": "support",
    "supposed": "suppose",
    "supposes": "suppose",
    "supposing": "suppose",
    "suppositions": "supposition",
    "suppositories": "suppository",
    "suppressants": "suppressant",
    "suppressed": "suppress",
    "suppresses": "suppress",
    "suppressing": "suppress",
    "suppressions": "suppression",
    "suppressors": "suppressor",
    "suppurated": "suppurate",
    "suppurates": "suppurate",
    "suppurating": "suppurate",
    "supremacists": "supremacist",
    "sups": "sup",
    "surae": "sura",
    "surcharged": "surcharge",
    "surcharges": "surcharge",
    "surcharging": "surcharge",
    "surer": "sure",
    "surest": "sure",
    "sureties": "surety",
    "surfaced": "surface",
    "surfaces": "surface",
    "surfacing": "surface",
    "surfactants": "surfactant",
    "surfboards": "surfboard",
    "surfed": "surf",
    "surfeits": "surfeit",
    "surfers": "surfer",
    "surfing": "surf",
    "surfs": "surf",
    "surged": "surge",
    "surgeons": "surgeon",
    "surgeries": "surgery",
    "surges": "surge",
    "surging": "surge",
    "surlier": "surly",
    "surliest": "surly",
    "surmised": "surmise",
    "surmises": "surmise",
    "surmising": "surmise",
    "surmounted": "surmount",
    "surmounting": "surmount",
    "surmounts": "surmount",
    "surnames": "surname",
    "surpassed": "surpass",
    "surpasses": "surpass",
    "surpassing": "surpass",
    "surplices": "surplice",
    "surpluses": "surplus",
    "surprised": "surprise",
    "surprises": "surprise",
    "surprising": "surprise",
    "surprized": "surprize",
    "surprizes": "surprize",
    "surprizing": "surprize",
    "surrealists": "surrealist",
    "surrendered": "surrender",
    "surrendering": "surrender",
    "surrenders": "surrender",
    "surreys": "surrey",
    "surrogates": "surrogate",
    "surrounded": "surround",
    "surrounding": "surround",
    "surroundings": "surrounding",
    "surrounds": "surround",
    "surveillances": "surveillance",
    "surveyed": "survey",
    "surveying": "survey",
    "surveyors": "surveyor",
    "surveys": "survey",
    "survivabilities": "survivability",
    "survivalists": "survivalist",
    "survivals": "survival",
    "survived": "survive",
    "survives": "survive",
    "surviving": "survive",
    "survivors": "survivor",
    "susceptibilities": "susceptibility",
    "suspected": "suspect",
    "suspecting": "suspect",
    "suspects": "suspect",
    "suspended": "suspend",
    "suspenders": "suspender",
    "suspending": "suspend",
    "suspends": "suspend",
    "suspensions": "suspension",
    "suspicions": "suspicion",
    "sussed": "suss",
    "susses": "suss",
    "sussing": "suss",
    "sustained": "sustain",
    "sustainers": "sustainer",
    "sustaining": "sustain",
    "sustains": "sustain",
    "sutured": "suture",
    "sutures": "suture",
    "suturing": "suture",
    "svs": "sv",
    "swabbed": "swab",
    "swabbing": "swab",
    "swabs": "swab",
    "swaddled": "swaddle",
    "swaddles": "swaddle",
    "swaddling": "swaddle",
    "swaggered": "swagger",
    "swaggering": "swagger",
    "swaggers": "swagger",
    "swains": "swain",
    "swales": "swale",
    "swallowed": "swallow",
    "swallowers": "swallower",
    "swallowing": "swallow",
    "swallows": "swallow",
    "swallowtails": "swallowtail",
    "swam": "swim",
    "swamis": "swami",
    "swamped": "swamp",
    "swampier": "swampy",
    "swampiest": "swampy",
    "swamping": "swamp",
    "swamps": "swamp",
    "swang": "swing",
    "swanked": "swank",
    "swankier": "swanky",
    "swanking": "swank",
    "swanks": "swank",
    "swanned": "swan",
    "swanning": "swan",
    "swans": "swan",
    "swapped": "swap",
    "swapping": "swap",
    "swaps": "swap",
    "swards": "sward",
    "swarmed": "swarm",
    "swarming": "swarm",
    "swarms": "swarm",
    "swarthier": "swarthy",
    "swarthiest": "swarthy",
    "swashbucklers": "swashbuckler",
    "swashes": "swash",
    "swastikas": "swastika",
    "swathed": "swathe",
    "swathes": "swathe",
    "swathing": "swathe",
    "swaths": "swath",
    "swats": "swat",
    "swatted": "swat",
    "swatting": "swat",
    "swayed": "sway",
    "swaying": "sway",
    "sways": "sway",
    "swear-words": "swear-word",
    "swearing": "swear",
    "swears": "swear",
    "swearwords": "swearword",
    "sweatbands": "sweatband",
    "sweated": "sweat",
    "sweaters": "sweater",
    "sweatier": "sweaty",
    "sweatiest": "sweaty",
    "sweating": "sweat",
    "sweats": "sweat",
    "sweatshirts": "sweatshirt",
    "sweatshops": "sweatshop",
    "swedes": "swede",
    "sweepers": "sweeper",
    "sweeping": "sweep",
    "sweepings": "sweeping",
    "sweeps": "sweep",
    "sweepstakes": "sweepstake",
    "sweetbreads": "sweetbread",
    "sweetened": "sweeten",
    "sweeteners": "sweetener",
    "sweetening": "sweeten",
    "sweetens": "sweeten",
    "sweeter": "sweet",
    "sweetest": "sweet",
    "sweethearts": "sweetheart",
    "sweetmeats": "sweetmeat",
    "sweetnesses": "sweetness",
    "sweets": "sweet",
    "swelled": "swell",
    "sweller": "swell",
    "swellest": "swell",
    "swelling": "swell",
    "swellings": "swelling",
    "swells": "swell",
    "sweltered": "swelter",
    "sweltering": "swelter",
    "swelters": "swelter",
    "swept": "sweep",
    "swerved": "swerve",
    "swerves": "swerve",
    "swerving": "swerve",
    "swifter": "swift",
    "swiftest": "swift",
    "swiftlets": "swiftlet",
    "swifts": "swift",
    "swigged": "swig",
    "swigging": "swig",
    "swigs": "swig",
    "swilled": "swill",
    "swilling": "swill",
    "swills": "swill",
    "swimmers": "swimmer",
    "swimming": "swim",
    "swimmings": "swimming",
    "swims": "swim",
    "swimsuits": "swimsuit",
    "swindled": "swindle",
    "swindlers": "swindler",
    "swindles": "swindle",
    "swindling": "swindle",
    "swineherds": "swineherd",
    "swines": "swine",
    "swinged": "swinge",
    "swingers": "swinger",
    "swinges": "swinge",
    "swinging": "swinge",
    "swings": "swing",
    "swiped": "swipe",
    "swipes": "swipe",
    "swiping": "swipe",
    "swirled": "swirl",
    "swirling": "swirl",
    "swirls": "swirl",
    "swished": "swish",
    "swishes": "swish",
    "swishing": "swish",
    "switch-overs": "switch-over",
    "switchbacks": "switchback",
    "switchboards": "switchboard",
    "switched": "switch",
    "switchers": "switcher",
    "switches": "switch",
    "switching": "switch",
    "switchings": "switching",
    "switchovers": "switchover",
    "swiveled": "swivel",
    "swiveling": "swivel",
    "swivelled": "swivel",
    "swivelling": "swivel",
    "swivels": "swivel",
    "swollen": "swell",
    "swooned": "swoon",
    "swooning": "swoon",
    "swoons": "swoon",
    "swooped": "swoop",
    "swooping": "swoop",
    "swoops": "swoop",
    "swopped": "swop",
    "swopping": "swop",
    "swops": "swop",
    "swordfishes": "swordfish",
    "swords": "sword",
    "swordsmen": "swordsman",
    "swore": "swear",
    "sworn": "swear",
    "swots": "swot",
    "swotted": "swot",
    "swotting": "swot",
    "swum": "swim",
    "swung": "swing",
    "sycamores": "sycamore",
    "sycophants": "sycophant",
    "syllabi": "syllabus",
    "syllables": "syllable",
    "syllabuses": "syllabus",
    "syllogisms": "syllogism",
    "symbionts": "symbiont",
    "symbioses": "symbiosis",
    "symbolised": "symbolise",
    "symbolises": "symbolise",
    "symbolising": "symbolise",
    "symbolisms": "symbolism",
    "symbolized": "symbolize",
    "symbolizes": "symbolize",
    "symbolizing": "symbolize",
    "symbologies": "symbology",
    "symbols": "symbol",
    "symmetries": "symmetry",
    "sympathies": "sympathy",
    "sympathised": "sympathise",
    "sympathisers": "sympathiser",
    "sympathises": "sympathise",
    "sympathising": "sympathise",
    "sympathized": "sympathize",
    "sympathizers": "sympathizer",
    "sympathizes": "sympathize",
    "sympathizing": "sympathize",
    "symphonies": "symphony",
    "symphyses": "symphysis",
    "symposia": "symposium",
    "symposiums": "symposium",
    "symptomatologies": "symptomatology",
    "symptoms": "symptom",
    "synagogues": "synagogue",
    "synapses": "synapsis",
    "synced": "sync",
    "synched": "synch",
    "synches": "synch",
    "synching": "synch",
    "synchromeshes": "synchromesh",
    "synchronicities": "synchronicity",
    "synchronies": "synchrony",
    "synchronisations": "synchronisation",
    "synchronised": "synchronise",
    "synchronises": "synchronise",
    "synchronising": "synchronise",
    "synchronizations": "synchronization",
    "synchronized": "synchronize",
    "synchronizes": "synchronize",
    "synchronizing": "synchronize",
    "synchrotrons": "synchrotron",
    "syncing": "sync",
    "syncopated": "syncopate",
    "syncopates": "syncopate",
    "syncopating": "syncopate",
    "syncopes": "syncope",
    "syncretisms": "syncretism",
    "syncs": "sync",
    "syndicalists": "syndicalist",
    "syndicated": "syndicate",
    "syndicates": "syndicate",
    "syndicating": "syndicate",
    "syndromes": "syndrome",
    "synecdoches": "synecdoche",
    "synergies": "synergy",
    "syngases": "syngas",
    "synods": "synod",
    "synonymies": "synonymy",
    "synonyms": "synonym",
    "synopses": "synopsis",
    "synovitides": "synovitis",
    "syns": "syn",
    "syntaxes": "syntax",
    "synthases": "synthase",
    "syntheses": "synthesis",
    "synthesised": "synthesise",
    "synthesisers": "synthesiser",
    "synthesises": "synthesise",
    "synthesising": "synthesise",
    "synthesized": "synthesize",
    "synthesizers": "synthesizer",
    "synthesizes": "synthesize",
    "synthesizing": "synthesize",
    "synthetases": "synthetase",
    "synthetics": "synthetic",
    "syphilises": "syphilis",
    "syphoned": "syphon",
    "syphoning": "syphon",
    "syphons": "syphon",
    "syringed": "syringe",
    "syringes": "syrinx",
    "syringing": "syringe",
    "syrinxes": "syrinx",
    "syrupier": "syrupy",
    "syrupiest": "syrupy",
    "syrups": "syrup",
    "systematised": "systematise",
    "systematises": "systematise",
    "systematising": "systematise",
    "systematized": "systematize",
    "systematizes": "systematize",
    "systematizing": "systematize",
    "systemised": "systemise",
    "systemises": "systemise",
    "systemising": "systemise",
    "systems": "system",
    "systoles": "systole",
    "syzygies": "syzygy",
    "t-tests": "t-test",
    "tabbies": "tabby",
    "tabernacles": "tabernacle",
    "tableaus": "tableau",
    "tableaux": "tableau",
    "tablecloths": "tablecloth",
    "tabled": "table",
    "tablelands": "tableland",
    "tablemats": "tablemat",
    "tables": "table",
    "tablespoonfuls": "tablespoonful",
    "tablespoons": "tablespoon",
    "tablespoonsful": "tablespoonful",
    "tabletops": "tabletop",
    "tablets": "tablet",
    "tabling": "table",
    "tabloids": "tabloid",
    "tabooed": "taboo",
    "tabooing": "taboo",
    "taboos": "taboo",
    "tabs": "tab",
    "tabulae": "tabula",
    "tabulated": "tabulate",
    "tabulates": "tabulate",
    "tabulating": "tabulate",
    "tabulations": "tabulation",
    "tabulators": "tabulator",
    "taches": "tache",
    "tachographs": "tachograph",
    "tachometers": "tachometer",
    "tachycardias": "tachycardia",
    "tacitly": "tacit",
    "tacked": "tack",
    "tackier": "tacky",
    "tackiest": "tacky",
    "tacking": "tack",
    "tackled": "tackle",
    "tacklers": "tackler",
    "tackles": "tackle",
    "tackling": "tackle",
    "tacks": "tack",
    "tacticians": "tactician",
    "tactics": "tactic",
    "tadpoles": "tadpole",
    "taffies": "taffy",
    "tagged": "tag",
    "taggers": "tagger",
    "tagging": "tag",
    "tags": "tag",
    "tailbacks": "tailback",
    "tailed": "tail",
    "tailgated": "tailgate",
    "tailgates": "tailgate",
    "tailgating": "tailgate",
    "tailing": "tail",
    "taillights": "taillight",
    "tailored": "tailor",
    "tailoring": "tailor",
    "tailors": "tailor",
    "tailpieces": "tailpiece",
    "tailpipes": "tailpipe",
    "tails": "tail",
    "tailwinds": "tailwind",
    "tainted": "taint",
    "tainting": "taint",
    "taints": "taint",
    "take-offs": "take-off",
    "take-overs": "take-over",
    "takeaways": "takeaway",
    "taken": "take",
    "takeoffs": "takeoff",
    "takeovers": "takeover",
    "takers": "taker",
    "takes": "take",
    "taking": "take",
    "takings": "taking",
    "takins": "takin",
    "talcums": "talcum",
    "talents": "talent",
    "tales": "tale",
    "tali": "talus",
    "talismans": "talisman",
    "talismen": "talisman",
    "talked": "talk",
    "talkers": "talker",
    "talkies": "talkie",
    "talking": "talk",
    "talks": "talk",
    "tallboys": "tallboy",
    "taller": "tall",
    "tallest": "tall",
    "tallied": "tally",
    "tallies": "tally",
    "tallyhos": "tallyho",
    "tallying": "tally",
    "talons": "talon",
    "tals": "tal",
    "taluses": "talus",
    "tamarinds": "tamarind",
    "tamarins": "tamarin",
    "tamarisks": "tamarisk",
    "tambourines": "tambourine",
    "tamed": "tame",
    "tamer": "tame",
    "tames": "tame",
    "tamest": "tame",
    "taming": "tame",
    "tammies": "tammy",
    "tamped": "tamp",
    "tampered": "tamper",
    "tampering": "tamper",
    "tampers": "tamper",
    "tamping": "tamp",
    "tampons": "tampon",
    "tamps": "tamp",
    "tams": "tam",
    "tanagers": "tanager",
    "tandems": "tandem",
    "tangents": "tangent",
    "tangerines": "tangerine",
    "tangier": "tangy",
    "tangiest": "tangy",
    "tangled": "tangle",
    "tangles": "tangle",
    "tangling": "tangle",
    "tangoed": "tango",
    "tangoing": "tango",
    "tangos": "tango",
    "tangrams": "tangram",
    "tangs": "tang",
    "tankards": "tankard",
    "tanked": "tank",
    "tankers": "tanker",
    "tanking": "tank",
    "tanks": "tank",
    "tanned": "tan",
    "tanner": "tan",
    "tanneries": "tannery",
    "tanners": "tanner",
    "tanning": "tan",
    "tannins": "tannin",
    "tans": "tan",
    "tantalised": "tantalise",
    "tantalises": "tantalise",
    "tantalising": "tantalise",
    "tantalized": "tantalize",
    "tantalizes": "tantalize",
    "tantalizing": "tantalize",
    "tantrums": "tantrum",
    "tanzanites": "tanzanite",
    "tap-dancings": "tap-dancing",
    "tapas": "tapa",
    "tape-recorded": "tape-record",
    "tape-recording": "tape-record",
    "tape-recordings": "tape-recording",
    "tape-records": "tape-record",
    "taped": "tape",
    "tapered": "taper",
    "tapering": "taper",
    "tapers": "taper",
    "tapes": "tape",
    "tapestries": "tapestry",
    "tapeworms": "tapeworm",
    "taping": "tape",
    "tapirs": "tapir",
    "tapped": "tap",
    "tappers": "tapper",
    "tappets": "tappet",
    "tapping": "tap",
    "taproots": "taproot",
    "taps": "tap",
    "tarantulae": "tarantula",
    "tarantulas": "tarantula",
    "taras": "tara",
    "tardier": "tardy",
    "tardiest": "tardy",
    "tared": "tar",
    "tares": "tare",
    "targeted": "target",
    "targeting": "target",
    "targets": "target",
    "targetted": "target",
    "targetting": "target",
    "tariffs": "tariff",
    "taring": "tar",
    "tarmacked": "tarmac",
    "tarmacking": "tarmac",
    "tarmacs": "tarmac",
    "tarnished": "tarnish",
    "tarnishes": "tarnish",
    "tarnishing": "tarnish",
    "tarns": "tarn",
    "taros": "taro",
    "tarpaulins": "tarpaulin",
    "tarpons": "tarpon",
    "tarps": "tarp",
    "tarred": "tar",
    "tarried": "tarry",
    "tarries": "tarry",
    "tarring": "tar",
    "tarrying": "tarry",
    "tars": "tar",
    "tarsi": "tarsus",
    "tartans": "tartan",
    "tartars": "tartar",
    "tarted": "tart",
    "tarter": "tart",
    "tartest": "tart",
    "tarting": "tart",
    "tartlets": "tartlet",
    "tartrates": "tartrate",
    "tarts": "tart",
    "tasked": "task",
    "tasking": "task",
    "taskmasters": "taskmaster",
    "tasks": "task",
    "tasseled": "tassel",
    "tasseling": "tassel",
    "tasselled": "tassel",
    "tasselling": "tassel",
    "tassels": "tassel",
    "tasted": "taste",
    "tasters": "taster",
    "tastes": "taste",
    "tastier": "tasty",
    "tastiest": "tasty",
    "tasting": "taste",
    "tatamis": "tatami",
    "tatoos": "tatoo",
    "tattered": "tatter",
    "tattering": "tatter",
    "tatters": "tatter",
    "tattier": "tatty",
    "tattiest": "tatty",
    "tattooed": "tattoo",
    "tattooing": "tattoo",
    "tattooists": "tattooist",
    "tattoos": "tattoo",
    "taught": "teach",
    "taunted": "taunt",
    "taunting": "taunt",
    "taunts": "taunt",
    "taupes": "taupe",
    "taus": "tau",
    "tautened": "tauten",
    "tautening": "tauten",
    "tautens": "tauten",
    "tauter": "taut",
    "tautest": "taut",
    "tautologies": "tautology",
    "taverns": "tavern",
    "tawdrier": "tawdry",
    "tawdriest": "tawdry",
    "tawnier": "tawny",
    "tawniest": "tawny",
    "taxa": "taxon",
    "taxed": "tax",
    "taxes": "taxis",
    "taxicabs": "taxicab",
    "taxidermists": "taxidermist",
    "taxied": "taxi",
    "taxies": "taxi",
    "taxiing": "taxi",
    "taxing": "tax",
    "taxis": "taxi",
    "taxonomies": "taxonomy",
    "taxonomists": "taxonomist",
    "taxons": "taxon",
    "taxpayers": "taxpayer",
    "taxying": "taxi",
    "tbsps": "tbsp",
    "tea-rooms": "tea-room",
    "tea-towels": "tea-towel",
    "teacakes": "teacake",
    "teach-ins": "teach-in",
    "teachers": "teacher",
    "teaches": "teach",
    "teaching": "teach",
    "teachings": "teaching",
    "teacups": "teacup",
    "teacupsful": "teacupful",
    "teaks": "teak",
    "teals": "teal",
    "team-mates": "team-mate",
    "teamed": "team",
    "teaming": "team",
    "teammates": "teammate",
    "teams": "team",
    "teamsters": "teamster",
    "teapots": "teapot",
    "tear-jerkers": "tear-jerker",
    "tear-offs": "tear-off",
    "tearaways": "tearaway",
    "teardrops": "teardrop",
    "teargases": "teargas",
    "tearing": "tear",
    "tearooms": "tearoom",
    "tears": "tear",
    "teas": "tea",
    "teased": "tease",
    "teasels": "teasel",
    "teasers": "teaser",
    "teases": "tease",
    "teashops": "teashop",
    "teasing": "tease",
    "teaspoonfuls": "teaspoonful",
    "teaspoons": "teaspoon",
    "teaspoonsful": "teaspoonful",
    "teatimes": "teatime",
    "teats": "teat",
    "technicalities": "technicality",
    "technicians": "technician",
    "technics": "technic",
    "techniques": "technique",
    "technocracies": "technocracy",
    "technocrats": "technocrat",
    "technologies": "technology",
    "technologists": "technologist",
    "technophobes": "technophobe",
    "techs": "tech",
    "teddies": "teddy",
    "tee-shirts": "tee-shirt",
    "teed": "tee",
    "teeing": "tee",
    "teemed": "teem",
    "teeming": "teem",
    "teems": "teem",
    "teenagers": "teenager",
    "teenier": "teeny",
    "teeniest": "teeny",
    "teens": "teen",
    "teeny-boppers": "teeny-bopper",
    "teepees": "teepee",
    "tees": "tee",
    "teetered": "teeter",
    "teetering": "teeter",
    "teeters": "teeter",
    "teeth": "tooth",
    "teethed": "teethe",
    "teethers": "teether",
    "teethes": "teethe",
    "teething": "teethe",
    "teetotallers": "teetotaller",
    "telangiectasiae": "telangiectasia",
    "telangiectasias": "telangiectasia",
    "telecasted": "telecast",
    "telecasting": "telecast",
    "telecasts": "telecast",
    "telecommunications": "telecommunication",
    "telecommuters": "telecommuter",
    "telecoms": "telecom",
    "teleconferenced": "teleconference",
    "teleconferences": "teleconference",
    "teleconferencing": "teleconference",
    "telegrams": "telegram",
    "telegraphed": "telegraph",
    "telegraphing": "telegraph",
    "telegraphs": "telegraph",
    "telemarketers": "telemarketer",
    "telemetries": "telemetry",
    "telephoned": "telephone",
    "telephones": "telephone",
    "telephoning": "telephone",
    "telephonists": "telephonist",
    "telephotos": "telephoto",
    "teleportations": "teleportation",
    "teleported": "teleport",
    "teleporting": "teleport",
    "teleports": "teleport",
    "telepresences": "telepresence",
    "teleprinters": "teleprinter",
    "teleprocessings": "teleprocessing",
    "teleradiologies": "teleradiology",
    "telescoped": "telescope",
    "telescopes": "telescope",
    "telescoping": "telescope",
    "telescreens": "telescreen",
    "televiewers": "televiewer",
    "televised": "televise",
    "televises": "televise",
    "televising": "televise",
    "televisions": "television",
    "telexed": "telex",
    "telexes": "telex",
    "telexing": "telex",
    "tellers": "teller",
    "tellies": "telly",
    "telling": "tell",
    "tellings": "telling",
    "tellings-off": "telling-off",
    "tells": "tell",
    "telltales": "telltale",
    "telneted": "telnet",
    "telneting": "telnet",
    "telnets": "telnet",
    "telomerases": "telomerase",
    "telomeres": "telomere",
    "tels": "tel",
    "temperaments": "temperament",
    "temperatures": "temperature",
    "tempered": "temper",
    "tempering": "temper",
    "tempers": "temper",
    "tempests": "tempest",
    "tempi": "tempus",
    "temping": "temp",
    "templated": "template",
    "templates": "template",
    "templating": "template",
    "temples": "temple",
    "temporalities": "temporality",
    "temporaries": "temporary",
    "temporized": "temporize",
    "temporizes": "temporize",
    "temporizing": "temporize",
    "tempos": "tempo",
    "temps": "temp",
    "temptations": "temptation",
    "tempted": "tempt",
    "tempters": "tempter",
    "tempting": "tempt",
    "temptresses": "temptress",
    "tempts": "tempt",
    "ten-millionths": "ten-millionth",
    "ten-thousandths": "ten-thousandth",
    "tenancies": "tenancy",
    "tenanted": "tenant",
    "tenanting": "tenant",
    "tenants": "tenant",
    "tenches": "tench",
    "tended": "tend",
    "tendencies": "tendency",
    "tendered": "tender",
    "tenderer": "tender",
    "tenderers": "tenderer",
    "tenderest": "tender",
    "tendering": "tender",
    "tenderized": "tenderize",
    "tenderizes": "tenderize",
    "tenderizing": "tenderize",
    "tenderloins": "tenderloin",
    "tenders": "tender",
    "tending": "tend",
    "tendinitides": "tendinitis",
    "tendonitides": "tendonitis",
    "tendons": "tendon",
    "tendrils": "tendril",
    "tends": "tend",
    "tenements": "tenement",
    "tenets": "tenet",
    "tenners": "tenner",
    "tenons": "tenon",
    "tenors": "tenor",
    "tenosynovitides": "tenosynovitis",
    "tenpins": "tenpin",
    "tens": "ten",
    "tensed": "tense",
    "tenser": "tense",
    "tenses": "tense",
    "tensest": "tense",
    "tensing": "tense",
    "tensioned": "tension",
    "tensioners": "tensioner",
    "tensioning": "tension",
    "tensions": "tension",
    "tensors": "tensor",
    "tentacles": "tentacle",
    "tented": "tent",
    "tenth": "10",
    "tenths": "tenth",
    "tenting": "tent",
    "tents": "tent",
    "tenured": "tenure",
    "tenures": "tenure",
    "tenuring": "tenure",
    "tepees": "tepee",
    "tequilas": "tequila",
    "teratogenicities": "teratogenicity",
    "teratomas": "teratoma",
    "teratomata": "teratoma",
    "tercentenaries": "tercentenary",
    "tercets": "tercet",
    "terephthalates": "terephthalate",
    "termed": "term",
    "terminals": "terminal",
    "terminated": "terminate",
    "terminates": "terminate",
    "terminating": "terminate",
    "terminations": "termination",
    "terminators": "terminator",
    "terming": "term",
    "termini": "terminus",
    "terminologies": "terminology",
    "terminologists": "terminologist",
    "terminuses": "terminus",
    "termites": "termite",
    "terms": "term",
    "terns": "tern",
    "terpenes": "terpene",
    "terraced": "terrace",
    "terraces": "terrace",
    "terracing": "terrace",
    "terrae": "terra",
    "terrains": "terrain",
    "terrapins": "terrapin",
    "terraria": "terrarium",
    "terrariums": "terrarium",
    "terrestrials": "terrestrial",
    "terriers": "terrier",
    "terrified": "terrify",
    "terrifies": "terrify",
    "terrifying": "terrify",
    "territorials": "territorial",
    "territories": "territory",
    "terrorised": "terrorise",
    "terrorises": "terrorise",
    "terrorising": "terrorise",
    "terrorists": "terrorist",
    "terrorized": "terrorize",
    "terrorizes": "terrorize",
    "terrorizing": "terrorize",
    "terrors": "terror",
    "terser": "terse",
    "tersest": "terse",
    "teslas": "tesla",
    "tessellated": "tessellate",
    "tessellates": "tessellate",
    "tessellating": "tessellate",
    "tessellations": "tessellation",
    "tesserae": "tessera",
    "test-driven": "test-drive",
    "test-drives": "test-drive",
    "test-driving": "test-drive",
    "test-drove": "test-drive",
    "test-tubes": "test-tube",
    "testaments": "testament",
    "testators": "testator",
    "testbeds": "testbed",
    "tested": "test",
    "testers": "tester",
    "testes": "testis",
    "testicles": "testicle",
    "testier": "testy",
    "testiest": "testy",
    "testified": "testify",
    "testifies": "testify",
    "testifying": "testify",
    "testimonials": "testimonial",
    "testimonies": "testimony",
    "testing": "test",
    "testings": "testing",
    "testosterones": "testosterone",
    "tests": "test",
    "tetanuses": "tetanus",
    "tetchier": "tetchy",
    "tethered": "tether",
    "tethering": "tether",
    "tethers": "tether",
    "tetrachlorides": "tetrachloride",
    "tetracyclines": "tetracycline",
    "tetrads": "tetrad",
    "tetrahedra": "tetrahedron",
    "tetrahedrons": "tetrahedron",
    "tetramers": "tetramer",
    "tetrameters": "tetrameter",
    "tetrapods": "tetrapod",
    "tetras": "tetra",
    "tetraspanins": "tetraspanin",
    "text-books": "text-book",
    "textbooks": "textbook",
    "textiles": "textile",
    "texts": "text",
    "textured": "texture",
    "textures": "texture",
    "texturing": "texture",
    "texturings": "texturing",
    "thalami": "thalamus",
    "thalamuses": "thalamus",
    "thalassaemias": "thalassaemia",
    "thams": "tham",
    "thanked": "thank",
    "thanking": "thank",
    "thanks": "thank",
    "thankyous": "thankyou",
    "thars": "thar",
    "thatched": "thatch",
    "thatchers": "thatcher",
    "thatches": "thatch",
    "thatching": "thatch",
    "thawed": "thaw",
    "thawing": "thaw",
    "thaws": "thaw",
    "theaters": "theater",
    "theatre-goers": "theatre-goer",
    "theatregoers": "theatregoer",
    "theatres": "theatre",
    "theatricals": "theatrical",
    "thee": "you",
    "thefts": "theft",
    "theists": "theist",
    "themes": "theme",
    "theocracies": "theocracy",
    "theodolites": "theodolite",
    "theologians": "theologian",
    "theologies": "theology",
    "theophyllines": "theophylline",
    "theorems": "theorem",
    "theoreticians": "theoretician",
    "theories": "theory",
    "theorisations": "theorisation",
    "theorised": "theorise",
    "theorises": "theorise",
    "theorising": "theorise",
    "theorists": "theorist",
    "theorizations": "theorization",
    "theorized": "theorize",
    "theorizes": "theorize",
    "theorizing": "theorize",
    "therapies": "therapy",
    "therapists": "therapist",
    "thermals": "thermal",
    "thermistors": "thermistor",
    "thermoclines": "thermocline",
    "thermocouples": "thermocouple",
    "thermographies": "thermography",
    "thermometers": "thermometer",
    "thermoplastics": "thermoplastic",
    "thermoregulations": "thermoregulation",
    "thermosets": "thermoset",
    "thermostats": "thermostat",
    "therms": "therm",
    "theropods": "theropod",
    "thers": "ther",
    "thesauri": "thesaurus",
    "thesauruses": "thesaurus",
    "these": "this",
    "theses": "thesis",
    "thespians": "thespian",
    "thetas": "theta",
    "thiazides": "thiazide",
    "thickened": "thicken",
    "thickeners": "thickener",
    "thickening": "thicken",
    "thickenings": "thickening",
    "thickens": "thicken",
    "thicker": "thick",
    "thickest": "thick",
    "thickets": "thicket",
    "thicknesses": "thickness",
    "thieved": "thieve",
    "thieves": "thieve",
    "thieving": "thieve",
    "thievings": "thieving",
    "thigh-bones": "thigh-bone",
    "thighbones": "thighbone",
    "thighs": "thigh",
    "thimblefuls": "thimbleful",
    "thimbles": "thimble",
    "thin-sections": "thin-section",
    "thingamabobs": "thingamabob",
    "thingies": "thingy",
    "things": "thing",
    "thingummies": "thingummy",
    "think-tanks": "think-tank",
    "thinked": "think",
    "thinkers": "thinker",
    "thinking": "think",
    "thinks": "think",
    "thinned": "thin",
    "thinner": "thin",
    "thinners": "thinner",
    "thinnesses": "thinness",
    "thinnest": "thin",
    "thinning": "thin",
    "thins": "thin",
    "thiols": "thiol",
    "thiosulphates": "thiosulphate",
    "third": "3",
    "thirds": "third",
    "thirsted": "thirst",
    "thirstier": "thirsty",
    "thirstiest": "thirsty",
    "thirsting": "thirst",
    "thirsts": "thirst",
    "thirteenth": "13",
    "thirteenths": "thirteenth",
    "thirties": "thirty",
    "thirtieth": "30",
    "thirtieths": "thirtieth",
    "thirty-eighth": "38",
    "thirty-eighths": "thirty-eighth",
    "thirty-fifth": "35",
    "thirty-fifths": "thirty-fifth",
    "thirty-first": "31",
    "thirty-firsts": "thirty-first",
    "thirty-fourth": "34",
    "thirty-fourths": "thirty-fourth",
    "thirty-ninth": "39",
    "thirty-ninths": "thirty-ninth",
    "thirty-second": "32",
    "thirty-seconds": "thirty-second",
    "thirty-seventh": "37",
    "thirty-sevenths": "thirty-seventh",
    "thirty-sixth": "36",
    "thirty-sixths": "thirty-sixth",
    "thirty-somethings": "thirty-something",
    "thirty-third": "33",
    "thirty-thirds": "thirty-third",
    "thirtysomethings": "thirtysomething",
    "thistles": "thistle",
    "thongs": "thong",
    "thoraces": "thorax",
    "thoraxes": "thorax",
    "thornier": "thorny",
    "thorniest": "thorny",
    "thorns": "thorn",
    "thoroughbreds": "thoroughbred",
    "thoroughfares": "thoroughfare",
    "thors": "thor",
    "those": "that",
    "thought": "think",
    "thought-criminals": "thought-criminal",
    "thoughts": "thought",
    "thousands": "thousand",
    "thousandth": "1000",
    "thousandths": "thousandth",
    "thralls": "thrall",
    "thrashed": "thrash",
    "thrashes": "thrash",
    "thrashing": "thrash",
    "threaded": "thread",
    "threading": "thread",
    "threads": "thread",
    "threadworms": "threadworm",
    "threatened": "threaten",
    "threatening": "threaten",
    "threatens": "threaten",
    "threats": "threat",
    "three-hundredth": "300",
    "three-quarters": "three-quarter",
    "three-wheelers": "three-wheeler",
    "threes": "three",
    "threesomes": "threesome",
    "threonines": "threonine",
    "threshed": "thresh",
    "threshers": "thresher",
    "threshes": "thresh",
    "threshing": "thresh",
    "thresholds": "threshold",
    "threw": "throw",
    "thriftier": "thrifty",
    "thriftiest": "thrifty",
    "thrilled": "thrill",
    "thrillers": "thriller",
    "thrilling": "thrill",
    "thrills": "thrill",
    "thrived": "thrive",
    "thriven": "thrive",
    "thrives": "thrive",
    "thriving": "thrive",
    "throatier": "throaty",
    "throatiest": "throaty",
    "throats": "throat",
    "throbbed": "throb",
    "throbbing": "throb",
    "throbs": "throb",
    "throes": "throe",
    "thrombi": "thrombus",
    "thrombins": "thrombin",
    "thrombocytopenias": "thrombocytopenia",
    "thrombocytoses": "thrombocytosis",
    "thromboembolisms": "thromboembolism",
    "thrombolyses": "thrombolysis",
    "thrombophilias": "thrombophilia",
    "thrombosed": "thrombose",
    "thromboses": "thrombosis",
    "thrombosing": "thrombose",
    "thrombuses": "thrombus",
    "thrones": "throne",
    "thronged": "throng",
    "thronging": "throng",
    "throngs": "throng",
    "throttled": "throttle",
    "throttles": "throttle",
    "throttling": "throttle",
    "throughputs": "throughput",
    "throve": "thrive",
    "throw-ins": "throw-in",
    "throwbacks": "throwback",
    "throwers": "thrower",
    "throwing": "throw",
    "thrown": "throw",
    "throws": "throw",
    "thrummed": "thrum",
    "thrumming": "thrum",
    "thrums": "thrum",
    "thrushes": "thrush",
    "thrusters": "thruster",
    "thrusting": "thrust",
    "thrusts": "thrust",
    "thudded": "thud",
    "thudding": "thud",
    "thuds": "thud",
    "thugs": "thug",
    "thumbed": "thumb",
    "thumbing": "thumb",
    "thumbnails": "thumbnail",
    "thumbs": "thumb",
    "thumbscrews": "thumbscrew",
    "thumbtacks": "thumbtack",
    "thumped": "thump",
    "thumping": "thump",
    "thumps": "thump",
    "thunderbolts": "thunderbolt",
    "thunderclaps": "thunderclap",
    "thunderclouds": "thundercloud",
    "thundered": "thunder",
    "thundering": "thunder",
    "thunders": "thunder",
    "thunderstorms": "thunderstorm",
    "thwacked": "thwack",
    "thwacking": "thwack",
    "thwacks": "thwack",
    "thwarted": "thwart",
    "thwarting": "thwart",
    "thwarts": "thwart",
    "thymi": "thymus",
    "thymidines": "thymidine",
    "thymines": "thymine",
    "thymomas": "thymoma",
    "thymomata": "thymoma",
    "thymuses": "thymus",
    "thyroidectomies": "thyroidectomy",
    "thyroiditides": "thyroiditis",
    "thyroids": "thyroid",
    "thyroxines": "thyroxine",
    "tiaras": "tiara",
    "tibiae": "tibia",
    "tibialides": "tibialis",
    "tibias": "tibia",
    "ticked": "tick",
    "tickers": "ticker",
    "ticketed": "ticket",
    "ticketing": "ticket",
    "tickets": "ticket",
    "ticking": "tick",
    "tickled": "tickle",
    "tickles": "tickle",
    "tickling": "tickle",
    "ticks": "tick",
    "tics": "tic",
    "tidbits": "tidbit",
    "tiddlers": "tiddler",
    "tiddlier": "tiddly",
    "tiddliest": "tiddly",
    "tiddlywinks": "tiddlywink",
    "tided": "tide",
    "tidelines": "tideline",
    "tidemarks": "tidemark",
    "tides": "tide",
    "tidied": "tidy",
    "tidier": "tidy",
    "tidies": "tidy",
    "tidiest": "tidy",
    "tiding": "tide",
    "tidying": "tidy",
    "tie-breakers": "tie-breaker",
    "tie-breaks": "tie-break",
    "tie-dyed": "tie-dye",
    "tie-dyeing": "tie-dye",
    "tie-dyes": "tie-dye",
    "tie-ins": "tie-in",
    "tie-pins": "tie-pin",
    "tie-ups": "tie-up",
    "tied": "tie",
    "tieing": "tie",
    "tierces": "tierce",
    "tiered": "tier",
    "tiering": "tier",
    "tiers": "tier",
    "ties": "tie",
    "tiffs": "tiff",
    "tigers": "tiger",
    "tight-ropes": "tight-rope",
    "tightened": "tighten",
    "tightening": "tighten",
    "tightens": "tighten",
    "tighter": "tight",
    "tightest": "tight",
    "tightnesses": "tightness",
    "tightropes": "tightrope",
    "tights": "tight",
    "tigresses": "tigress",
    "tilapias": "tilapia",
    "tildes": "tilde",
    "tiled": "tile",
    "tilers": "tiler",
    "tiles": "tile",
    "tiling": "tile",
    "tilled": "till",
    "tillers": "tiller",
    "tilling": "till",
    "tills": "till",
    "tilted": "tilt",
    "tilting": "tilt",
    "tilts": "tilt",
    "timbered": "timber",
    "timbering": "timber",
    "timbers": "timber",
    "timbres": "timbre",
    "time-courses": "time-course",
    "time-frames": "time-frame",
    "time-lines": "time-line",
    "time-outs": "time-out",
    "time-scales": "time-scale",
    "time-shares": "time-share",
    "time-spans": "time-span",
    "time-tables": "time-table",
    "timed": "time",
    "timeframes": "timeframe",
    "timekeepers": "timekeeper",
    "timelier": "timely",
    "timeliest": "timely",
    "timelines": "timeline",
    "timeouts": "timeout",
    "timepieces": "timepiece",
    "timepoints": "timepoint",
    "timers": "timer",
    "times": "time",
    "timescales": "timescale",
    "timeservers": "timeserver",
    "timespans": "timespan",
    "timetabled": "timetable",
    "timetables": "timetable",
    "timetabling": "timetable",
    "timider": "timid",
    "timidest": "timid",
    "timidities": "timidity",
    "timing": "time",
    "timings": "timing",
    "timorously": "timorous",
    "timpanists": "timpanist",
    "tinctures": "tincture",
    "tinderboxes": "tinderbox",
    "tineas": "tinea",
    "tines": "tine",
    "tinged": "tinge",
    "tingeing": "tinge",
    "tinges": "tinge",
    "tinging": "tinge",
    "tingled": "tingle",
    "tingles": "tingle",
    "tingling": "tingle",
    "tings": "ting",
    "tinier": "tiny",
    "tiniest": "tiny",
    "tinkered": "tinker",
    "tinkering": "tinker",
    "tinkers": "tinker",
    "tinkled": "tinkle",
    "tinkles": "tinkle",
    "tinkling": "tinkle",
    "tinned": "tin",
    "tinnier": "tinny",
    "tinniest": "tinny",
    "tinning": "tin",
    "tins": "tin",
    "tinted": "tint",
    "tinting": "tint",
    "tints": "tint",
    "tip-offs": "tip-off",
    "tip-toes": "tip-toe",
    "tipped": "tip",
    "tippers": "tipper",
    "tippets": "tippet",
    "tipping": "tip",
    "tippled": "tipple",
    "tipplers": "tippler",
    "tipples": "tipple",
    "tippling": "tipple",
    "tips": "tip",
    "tipsier": "tipsy",
    "tipsiest": "tipsy",
    "tipsters": "tipster",
    "tiptoed": "tiptoe",
    "tiptoeing": "tiptoe",
    "tiptoes": "tiptoe",
    "tirades": "tirade",
    "tired": "tire",
    "tires": "tire",
    "tiring": "tire",
    "tiros": "tiro",
    "tis": "this",
    "tissues": "tissue",
    "titanias": "titania",
    "titans": "titan",
    "titbits": "titbit",
    "titchier": "titchy",
    "titchiest": "titchy",
    "titers": "titer",
    "titfers": "titfer",
    "tithed": "tithe",
    "tithes": "tithe",
    "tithing": "tithe",
    "titillated": "titillate",
    "titillates": "titillate",
    "titillating": "titillate",
    "title-holders": "title-holder",
    "titled": "title",
    "titles": "title",
    "titling": "title",
    "titrated": "titrate",
    "titrates": "titrate",
    "titrating": "titrate",
    "titrations": "titration",
    "titres": "titre",
    "tits": "tit",
    "tittered": "titter",
    "tittering": "titter",
    "titters": "titter",
    "tizzies": "tizzy",
    "to-dos": "to-do",
    "toadied": "toady",
    "toadies": "toady",
    "toads": "toad",
    "toadstools": "toadstool",
    "toadying": "toady",
    "toasted": "toast",
    "toasters": "toaster",
    "toasting": "toast",
    "toastmasters": "toastmaster",
    "toastracks": "toastrack",
    "toasts": "toast",
    "tobaccoes": "tobacco",
    "tobacconists": "tobacconist",
    "tobaccos": "tobacco",
    "tobogganed": "toboggan",
    "tobogganing": "toboggan",
    "toboggans": "toboggan",
    "toccatas": "toccata",
    "tocopherols": "tocopherol",
    "tocs": "toc",
    "todays": "today",
    "toddies": "toddy",
    "toddled": "toddle",
    "toddlers": "toddler",
    "toddles": "toddle",
    "toddling": "toddle",
    "toecaps": "toecap",
    "toed": "toe",
    "toeholds": "toehold",
    "toeing": "toe",
    "toenails": "toenail",
    "toes": "toe",
    "toffees": "toffee",
    "toffs": "toff",
    "tofus": "tofu",
    "togas": "toga",
    "toggled": "toggle",
    "toggles": "toggle",
    "toggling": "toggle",
    "togs": "tog",
    "toiled": "toil",
    "toilers": "toiler",
    "toilet-trained": "toilet-train",
    "toilet-training": "toilet-train",
    "toilet-trains": "toilet-train",
    "toiletries": "toiletry",
    "toilets": "toilet",
    "toilettes": "toilette",
    "toiling": "toil",
    "toils": "toil",
    "tokamaks": "tokamak",
    "tokens": "token",
    "told": "tell",
    "tolerabilities": "tolerability",
    "tolerances": "tolerance",
    "tolerated": "tolerate",
    "tolerates": "tolerate",
    "tolerating": "tolerate",
    "tollbridges": "tollbridge",
    "tolled": "toll",
    "tollhouses": "tollhouse",
    "tolling": "toll",
    "tolls": "toll",
    "toluenes": "toluene",
    "tom-toms": "tom-tom",
    "tomahawks": "tomahawk",
    "tomatoes": "tomato",
    "tombolas": "tombola",
    "tomboys": "tomboy",
    "tombs": "tomb",
    "tombstones": "tombstone",
    "tomcats": "tomcat",
    "tomes": "tome",
    "tomfooleries": "tomfoolery",
    "tomographies": "tomography",
    "tomorrows": "tomorrow",
    "toms": "tom",
    "tomtits": "tomtit",
    "tonalities": "tonality",
    "toned": "tone",
    "toners": "toner",
    "tones": "tone",
    "tongs": "tong",
    "tongue-twisters": "tongue-twister",
    "tongued": "tongue",
    "tongues": "tongue",
    "tonguing": "tongue",
    "tonics": "tonic",
    "tonified": "tonify",
    "tonifies": "tonify",
    "tonifying": "tonify",
    "toning": "tone",
    "tonnages": "tonnage",
    "tonnes": "tonne",
    "tons": "ton",
    "tonsillectomies": "tonsillectomy",
    "tonsillitides": "tonsillitis",
    "tonsils": "tonsil",
    "tonsures": "tonsure",
    "took": "take",
    "toolboxes": "toolbox",
    "tooled": "tool",
    "tooling": "tool",
    "toolkits": "toolkit",
    "toolmakers": "toolmaker",
    "tools": "tool",
    "toolsets": "toolset",
    "toons": "toon",
    "tooted": "toot",
    "toothaches": "toothache",
    "toothbrushes": "toothbrush",
    "toothbrushings": "toothbrushing",
    "toothier": "toothy",
    "toothiest": "toothy",
    "toothpastes": "toothpaste",
    "toothpicks": "toothpick",
    "toothpowders": "toothpowder",
    "tooting": "toot",
    "tootled": "tootle",
    "tootles": "tootle",
    "tootling": "tootle",
    "toots": "toot",
    "tootsies": "tootsie",
    "top-knots": "top-knot",
    "top-ups": "top-up",
    "topazes": "topaz",
    "topcoats": "topcoat",
    "topes": "tope",
    "topicalities": "topicality",
    "topics": "topic",
    "topographers": "topographer",
    "topographies": "topography",
    "topoisomerases": "topoisomerase",
    "topologies": "topology",
    "topped": "top",
    "toppers": "topper",
    "topping": "top",
    "toppings": "topping",
    "toppled": "topple",
    "topples": "topple",
    "toppling": "topple",
    "tops": "top",
    "torchbearers": "torchbearer",
    "torches": "torch",
    "tore": "tear",
    "tori": "torus",
    "tormented": "torment",
    "tormentils": "tormentil",
    "tormenting": "torment",
    "tormentors": "tormentor",
    "torments": "torment",
    "torn": "tear",
    "tornadoes": "tornado",
    "tornados": "tornado",
    "toroids": "toroid",
    "torpedoed": "torpedo",
    "torpedoes": "torpedo",
    "torpedoing": "torpedo",
    "torpedos": "torpedo",
    "torpors": "torpor",
    "torques": "torque",
    "torrents": "torrent",
    "tors": "tor",
    "torsi": "torso",
    "torsions": "torsion",
    "torsoes": "torso",
    "torsos": "torso",
    "tortillas": "tortilla",
    "tortoises": "tortoise",
    "tortoiseshells": "tortoiseshell",
    "torts": "tort",
    "tortured": "torture",
    "torturers": "torturer",
    "tortures": "torture",
    "torturing": "torture",
    "torus": "toru",
    "toruses": "torus",
    "tossed": "toss",
    "tosses": "toss",
    "tossing": "toss",
    "tost": "toss",
    "totaled": "total",
    "totaling": "total",
    "totalised": "totalise",
    "totalises": "totalise",
    "totalising": "totalise",
    "totalities": "totality",
    "totalized": "totalize",
    "totalizes": "totalize",
    "totalizing": "totalize",
    "totalled": "total",
    "totalling": "total",
    "totals": "total",
    "toted": "tote",
    "totems": "totem",
    "totes": "tote",
    "toting": "tote",
    "totos": "toto",
    "tots": "tot",
    "totted": "tot",
    "tottered": "totter",
    "tottering": "totter",
    "totters": "totter",
    "totting": "tot",
    "toucans": "toucan",
    "touch-screens": "touch-screen",
    "touch-typed": "touch-type",
    "touch-types": "touch-type",
    "touch-typing": "touch-type",
    "touch-ups": "touch-up",
    "touchdowns": "touchdown",
    "touched": "touch",
    "touches": "touch",
    "touchier": "touchy",
    "touchiest": "touchy",
    "touching": "touch",
    "touchlines": "touchline",
    "touchscreens": "touchscreen",
    "touchstones": "touchstone",
    "toughened": "toughen",
    "toughening": "toughen",
    "toughens": "toughen",
    "tougher": "tough",
    "toughest": "tough",
    "toughnesses": "toughness",
    "toughs": "tough",
    "toupees": "toupee",
    "toured": "tour",
    "touring": "tour",
    "tourists": "tourist",
    "tourmalines": "tourmaline",
    "tournaments": "tournament",
    "tourniquets": "tourniquet",
    "tours": "tour",
    "tousled": "tousle",
    "tousles": "tousle",
    "tousling": "tousle",
    "touted": "tout",
    "touting": "tout",
    "touts": "tout",
    "towed": "tow",
    "toweled": "towel",
    "toweling": "towel",
    "towelled": "towel",
    "towelling": "towel",
    "towels": "towel",
    "towered": "tower",
    "towering": "tower",
    "towers": "tower",
    "towing": "tow",
    "towlines": "towline",
    "townhouses": "townhouse",
    "townies": "townie",
    "towns": "town",
    "townships": "township",
    "townsmen": "townsman",
    "towpaths": "towpath",
    "towropes": "towrope",
    "tows": "tow",
    "toxicants": "toxicant",
    "toxicities": "toxicity",
    "toxicologies": "toxicology",
    "toxicologists": "toxicologist",
    "toxics": "toxic",
    "toxins": "toxin",
    "toxoids": "toxoid",
    "toxoplasmoses": "toxoplasmosis",
    "toyed": "toy",
    "toying": "toy",
    "toys": "toy",
    "toyshops": "toyshop",
    "traced": "trace",
    "traceries": "tracery",
    "tracers": "tracer",
    "traces": "trace",
    "tracheae": "trachea",
    "tracheas": "trachea",
    "tracheostomies": "tracheostomy",
    "tracheotomies": "tracheotomy",
    "trachomata": "trachoma",
    "tracing": "trace",
    "tracings": "tracing",
    "tracked": "track",
    "trackers": "tracker",
    "tracking": "track",
    "tracks": "track",
    "tracksuits": "tracksuit",
    "tractabilities": "tractability",
    "tractions": "traction",
    "tractors": "tractor",
    "tracts": "tract",
    "trade-ins": "trade-in",
    "trade-offs": "trade-off",
    "traded": "trade",
    "trademarked": "trademark",
    "trademarking": "trademark",
    "trademarks": "trademark",
    "tradenames": "tradename",
    "tradeoffs": "tradeoff",
    "traders": "trader",
    "trades": "trade",
    "tradesmen": "tradesman",
    "trading": "trade",
    "traditionalists": "traditionalist",
    "traditions": "tradition",
    "trads": "trad",
    "traduced": "traduce",
    "traduces": "traduce",
    "traducing": "traduce",
    "trafficked": "traffic",
    "traffickers": "trafficker",
    "trafficking": "traffic",
    "traffics": "traffic",
    "tragedies": "tragedy",
    "tragicomedies": "tragicomedy",
    "trailblazers": "trailblazer",
    "trailed": "trail",
    "trailers": "trailer",
    "trailing": "trail",
    "trails": "trail",
    "trained": "train",
    "trainees": "trainee",
    "traineeships": "traineeship",
    "trainers": "trainer",
    "training": "train",
    "trainings": "training",
    "trains": "train",
    "traipsed": "traipse",
    "traipses": "traipse",
    "traipsing": "traipse",
    "traitors": "traitor",
    "traits": "trait",
    "trajectories": "trajectory",
    "tramcars": "tramcar",
    "tramlines": "tramline",
    "trammelled": "trammel",
    "trammelling": "trammel",
    "trammels": "trammel",
    "tramped": "tramp",
    "tramping": "tramp",
    "trampled": "trample",
    "tramples": "trample",
    "trampling": "trample",
    "trampolines": "trampoline",
    "tramps": "tramp",
    "trams": "tram",
    "tramways": "tramway",
    "trances": "trance",
    "tranquilisers": "tranquiliser",
    "tranquilized": "tranquilize",
    "tranquilizers": "tranquilizer",
    "tranquilizes": "tranquilize",
    "tranquilizing": "tranquilize",
    "tranquillised": "tranquillise",
    "tranquillisers": "tranquilliser",
    "tranquillises": "tranquillise",
    "tranquillising": "tranquillise",
    "tranquillized": "tranquillize",
    "tranquillizers": "tranquillizer",
    "tranquillizes": "tranquillize",
    "tranquillizing": "tranquillize",
    "transacted": "transact",
    "transacting": "transact",
    "transactions": "transaction",
    "transacts": "transact",
    "transaminases": "transaminase",
    "transceivers": "transceiver",
    "transcended": "transcend",
    "transcending": "transcend",
    "transcends": "transcend",
    "transcribed": "transcribe",
    "transcribers": "transcriber",
    "transcribes": "transcribe",
    "transcribing": "transcribe",
    "transcriptases": "transcriptase",
    "transcriptions": "transcription",
    "transcriptomes": "transcriptome",
    "transcripts": "transcript",
    "transduced": "transduce",
    "transducers": "transducer",
    "transduces": "transduce",
    "transducing": "transduce",
    "transductions": "transduction",
    "transected": "transect",
    "transecting": "transect",
    "transects": "transect",
    "transepts": "transept",
    "transfected": "transfect",
    "transfecting": "transfect",
    "transfections": "transfection",
    "transfects": "transfect",
    "transferases": "transferase",
    "transferees": "transferee",
    "transferences": "transference",
    "transferrals": "transferral",
    "transferred": "transfer",
    "transferring": "transfer",
    "transferrins": "transferrin",
    "transfers": "transfer",
    "transfigurations": "transfiguration",
    "transfigured": "transfigure",
    "transfigures": "transfigure",
    "transfiguring": "transfigure",
    "transfixed": "transfix",
    "transfixes": "transfix",
    "transfixing": "transfix",
    "transformations": "transformation",
    "transformed": "transform",
    "transformers": "transformer",
    "transforming": "transform",
    "transforms": "transform",
    "transfused": "transfuse",
    "transfuses": "transfuse",
    "transfusing": "transfuse",
    "transfusions": "transfusion",
    "transgenders": "transgender",
    "transgenes": "transgene",
    "transgenics": "transgenic",
    "transgressed": "transgress",
    "transgresses": "transgress",
    "transgressing": "transgress",
    "transgressions": "transgression",
    "transgressors": "transgressor",
    "transients": "transient",
    "transistors": "transistor",
    "transited": "transit",
    "transiting": "transit",
    "transitioned": "transition",
    "transitioning": "transition",
    "transitions": "transition",
    "transits": "transit",
    "translated": "translate",
    "translates": "translate",
    "translating": "translate",
    "translations": "translation",
    "translators": "translator",
    "transliterated": "transliterate",
    "transliterates": "transliterate",
    "transliterating": "transliterate",
    "transliterations": "transliteration",
    "translocated": "translocate",
    "translocates": "translocate",
    "translocating": "translocate",
    "translocations": "translocation",
    "transls": "transl",
    "translucencies": "translucency",
    "transmissibilities": "transmissibility",
    "transmissions": "transmission",
    "transmits": "transmit",
    "transmittals": "transmittal",
    "transmittances": "transmittance",
    "transmitted": "transmit",
    "transmitters": "transmitter",
    "transmitting": "transmit",
    "transmogrified": "transmogrify",
    "transmogrifies": "transmogrify",
    "transmogrifying": "transmogrify",
    "transmutations": "transmutation",
    "transmuted": "transmute",
    "transmutes": "transmute",
    "transmuting": "transmute",
    "transnationals": "transnational",
    "transoms": "transom",
    "transparencies": "transparency",
    "transpired": "transpire",
    "transpires": "transpire",
    "transpiring": "transpire",
    "transplantations": "transplantation",
    "transplanted": "transplant",
    "transplanting": "transplant",
    "transplants": "transplant",
    "transponders": "transponder",
    "transportations": "transportation",
    "transported": "transport",
    "transporters": "transporter",
    "transporting": "transport",
    "transports": "transport",
    "transposed": "transpose",
    "transposes": "transpose",
    "transposing": "transpose",
    "transpositions": "transposition",
    "transposons": "transposon",
    "transsexuals": "transsexual",
    "transshipments": "transshipment",
    "transshipped": "transship",
    "transshipping": "transship",
    "transships": "transship",
    "transversals": "transversal",
    "transverters": "transverter",
    "transvestites": "transvestite",
    "trap-doors": "trap-door",
    "trapdoors": "trapdoor",
    "trapezes": "trapeze",
    "trapezia": "trapezium",
    "trapeziums": "trapezium",
    "trapeziuses": "trapezius",
    "trapezoids": "trapezoid",
    "trapped": "trap",
    "trappers": "trapper",
    "trapping": "trap",
    "trappings": "trapping",
    "traps": "trap",
    "trashcans": "trashcan",
    "trashes": "trash",
    "trashier": "trashy",
    "trashiest": "trashy",
    "traumas": "trauma",
    "traumata": "trauma",
    "traumatised": "traumatise",
    "traumatises": "traumatise",
    "traumatising": "traumatise",
    "traumatized": "traumatize",
    "traumatizes": "traumatize",
    "traumatizing": "traumatize",
    "travails": "travail",
    "traveled": "travel",
    "travelers": "traveler",
    "traveling": "travel",
    "travelled": "travel",
    "travellers": "traveller",
    "travelling": "travel",
    "travelogues": "travelogue",
    "travels": "travel",
    "traversed": "traverse",
    "traverses": "traverse",
    "traversing": "traverse",
    "travertines": "travertine",
    "travesties": "travesty",
    "trawled": "trawl",
    "trawlermen": "trawlerman",
    "trawlers": "trawler",
    "trawling": "trawl",
    "trawls": "trawl",
    "trays": "tray",
    "treacheries": "treachery",
    "treading": "tread",
    "treadles": "treadle",
    "treadmills": "treadmill",
    "treads": "tread",
    "treasons": "treason",
    "treasure-troves": "treasure-trove",
    "treasured": "treasure",
    "treasurers": "treasurer",
    "treasures": "treasure",
    "treasuries": "treasury",
    "treasuring": "treasure",
    "treated": "treat",
    "treaters": "treater",
    "treaties": "treaty",
    "treating": "treat",
    "treatises": "treatise",
    "treatments": "treatment",
    "treats": "treat",
    "trebled": "treble",
    "trebles": "treble",
    "trebling": "treble",
    "tree-tops": "tree-top",
    "tree-trunks": "tree-trunk",
    "treecreepers": "treecreeper",
    "treed": "tree",
    "treeing": "tree",
    "trees": "tree",
    "treetops": "treetop",
    "trefoils": "trefoil",
    "trekked": "trek",
    "trekkers": "trekker",
    "trekking": "trek",
    "treks": "trek",
    "trellised": "trellis",
    "trellises": "trellis",
    "trellising": "trellis",
    "trematodes": "trematode",
    "trembled": "tremble",
    "trembles": "tremble",
    "trembling": "tremble",
    "tremolos": "tremolo",
    "tremors": "tremor",
    "trenched": "trench",
    "trenches": "trench",
    "trenching": "trench",
    "trend-setters": "trend-setter",
    "trended": "trend",
    "trendier": "trendy",
    "trendies": "trendy",
    "trendiest": "trendy",
    "trending": "trend",
    "trends": "trend",
    "trendsetters": "trendsetter",
    "trepidations": "trepidation",
    "trespassed": "trespass",
    "trespassers": "trespasser",
    "trespasses": "trespass",
    "trespassing": "trespass",
    "tresses": "tress",
    "trestles": "trestle",
    "triads": "triad",
    "triaged": "triage",
    "triages": "triage",
    "triaging": "triage",
    "trialed": "trial",
    "trialing": "trial",
    "trialists": "trialist",
    "triallists": "triallist",
    "trials": "trial",
    "triangles": "triangle",
    "triangulated": "triangulate",
    "triangulates": "triangulate",
    "triangulating": "triangulate",
    "triangulations": "triangulation",
    "triathletes": "triathlete",
    "triathlons": "triathlon",
    "triazoles": "triazole",
    "tribes": "tribe",
    "tribesmen": "tribesman",
    "tribespeople": "tribesperson",
    "tribulations": "tribulation",
    "tribunals": "tribunal",
    "tribunes": "tribune",
    "tributaries": "tributary",
    "tributes": "tribute",
    "tricepses": "triceps",
    "trices": "trice",
    "trichiases": "trichiasis",
    "trichomes": "trichome",
    "trichotillomanias": "trichotillomania",
    "tricked": "trick",
    "trickier": "tricky",
    "trickiest": "tricky",
    "tricking": "trick",
    "trickled": "trickle",
    "trickles": "trickle",
    "trickling": "trickle",
    "tricks": "trick",
    "tricksters": "trickster",
    "tricolors": "tricolor",
    "tricolours": "tricolour",
    "tricycles": "tricycle",
    "tricyclics": "tricyclic",
    "tridents": "trident",
    "tried": "try",
    "triennia": "triennium",
    "trienniums": "triennium",
    "triers": "trier",
    "tries": "try",
    "trifled": "trifle",
    "trifles": "trifle",
    "trifling": "trifle",
    "triforia": "triforium",
    "triggered": "trigger",
    "triggerfishes": "triggerfish",
    "triggering": "trigger",
    "triggers": "trigger",
    "triglycerides": "triglyceride",
    "trigons": "trigon",
    "trigrams": "trigram",
    "trikes": "trike",
    "trilbies": "trilby",
    "trilled": "trill",
    "trilling": "trill",
    "trillions": "trillion",
    "trills": "trill",
    "trilobites": "trilobite",
    "trilogies": "trilogy",
    "trimarans": "trimaran",
    "trimers": "trimer",
    "trimesters": "trimester",
    "trimmed": "trim",
    "trimmer": "trim",
    "trimmers": "trimmer",
    "trimmest": "trim",
    "trimming": "trim",
    "trimmings": "trimming",
    "trims": "trim",
    "trines": "trine",
    "trinitarians": "trinitarian",
    "trinities": "trinity",
    "trinitrates": "trinitrate",
    "trinkets": "trinket",
    "trios": "trio",
    "trioxides": "trioxide",
    "tripes": "tripe",
    "triphosphates": "triphosphate",
    "tripled": "triple",
    "triples": "triple",
    "triplets": "triplet",
    "triplicates": "triplicate",
    "tripling": "triple",
    "tripods": "tripod",
    "tripped": "trip",
    "trippers": "tripper",
    "tripping": "trip",
    "trips": "trip",
    "triptans": "triptan",
    "triptychs": "triptych",
    "tripwires": "tripwire",
    "trisomies": "trisomy",
    "triter": "trite",
    "tritest": "trite",
    "triumphalists": "triumphalist",
    "triumphed": "triumph",
    "triumphing": "triumph",
    "triumphs": "triumph",
    "triumvirates": "triumvirate",
    "trivia": "trivium",
    "trivialised": "trivialise",
    "trivialises": "trivialise",
    "trivialising": "trivialise",
    "trivialities": "triviality",
    "trivialized": "trivialize",
    "trivializes": "trivialize",
    "trivializing": "trivialize",
    "trochanters": "trochanter",
    "trod": "tread",
    "trodden": "tread",
    "troglodytes": "troglodyte",
    "trogons": "trogon",
    "trolled": "troll",
    "trolleys": "trolley",
    "trollies": "trolly",
    "trolling": "troll",
    "trollops": "trollop",
    "trolls": "troll",
    "trombones": "trombone",
    "trombonists": "trombonist",
    "trompes": "trompe",
    "trooped": "troop",
    "troopers": "trooper",
    "trooping": "troop",
    "troops": "troop",
    "troopships": "troopship",
    "tropes": "trope",
    "trophies": "trophy",
    "trophoblasts": "trophoblast",
    "tropics": "tropic",
    "tropisms": "tropism",
    "troponins": "troponin",
    "tropopauses": "tropopause",
    "tropospheres": "troposphere",
    "trots": "trot",
    "trotted": "trot",
    "trotters": "trotter",
    "trotting": "trot",
    "troubadours": "troubadour",
    "trouble-makers": "trouble-maker",
    "troubled": "trouble",
    "troublemakers": "troublemaker",
    "troubles": "trouble",
    "troubleshooted": "troubleshoot",
    "troubleshooters": "troubleshooter",
    "troubleshooting": "troubleshoot",
    "troubleshoots": "troubleshoot",
    "troubling": "trouble",
    "troughs": "trough",
    "trounced": "trounce",
    "trounces": "trounce",
    "trouncing": "trounce",
    "troupers": "trouper",
    "troupes": "troupe",
    "trousers": "trouser",
    "trousseaus": "trousseau",
    "trousseaux": "trousseau",
    "trouts": "trout",
    "troves": "trove",
    "trowels": "trowel",
    "trs": "tr",
    "truancies": "truancy",
    "truants": "truant",
    "truces": "truce",
    "trucked": "truck",
    "truckers": "trucker",
    "trucking": "truck",
    "truckled": "truckle",
    "truckles": "truckle",
    "truckling": "truckle",
    "truckloads": "truckload",
    "trucks": "truck",
    "trudged": "trudge",
    "trudges": "trudge",
    "trudging": "trudge",
    "truer": "TRUE",
    "truest": "TRUE",
    "truffles": "truffle",
    "trugs": "trug",
    "truisms": "truism",
    "trumped": "trump",
    "trumpeted": "trumpet",
    "trumpeters": "trumpeter",
    "trumpeting": "trumpet",
    "trumpets": "trumpet",
    "trumping": "trump",
    "trumps": "trump",
    "truncated": "truncate",
    "truncates": "truncate",
    "truncating": "truncate",
    "truncations": "truncation",
    "truncheons": "truncheon",
    "trundled": "trundle",
    "trundles": "trundle",
    "trundling": "trundle",
    "trunks": "trunk",
    "trunnions": "trunnion",
    "trussed": "truss",
    "trusses": "truss",
    "trussing": "truss",
    "trusted": "trust",
    "trustees": "trustee",
    "trusteeships": "trusteeship",
    "trustier": "trusty",
    "trustiest": "trusty",
    "trusting": "trust",
    "trusts": "trust",
    "truths": "truth",
    "try-outs": "try-out",
    "trying": "try",
    "trypanosomes": "trypanosome",
    "trypanosomiases": "trypanosomiasis",
    "trypsins": "trypsin",
    "tryptophans": "tryptophan",
    "trysts": "tryst",
    "ts": "t",
    "tsarinas": "tsarina",
    "tsarists": "tsarist",
    "tsars": "tsar",
    "tsetses": "tsetse",
    "tsps": "tsp",
    "tsunamis": "tsunami",
    "tubae": "tuba",
    "tubas": "tuba",
    "tubbier": "tubby",
    "tubbiest": "tubby",
    "tubera": "tuber",
    "tubercles": "tubercle",
    "tuberculins": "tuberculin",
    "tuberculoses": "tuberculosis",
    "tuberosities": "tuberosity",
    "tubers": "tuber",
    "tubes": "tube",
    "tubings": "tubing",
    "tubs": "tub",
    "tubules": "tubule",
    "tubulins": "tubulin",
    "tucked": "tuck",
    "tucking": "tuck",
    "tucks": "tuck",
    "tufas": "tufa",
    "tuffs": "tuff",
    "tufts": "tuft",
    "tug-of-wars": "tug-of-war",
    "tugged": "tug",
    "tugging": "tug",
    "tugs": "tug",
    "tugs-of-war": "tug-of-war",
    "tuis": "tui",
    "tuitions": "tuition",
    "tulips": "tulip",
    "tumbled": "tumble",
    "tumblers": "tumbler",
    "tumbles": "tumble",
    "tumbleweeds": "tumbleweed",
    "tumbling": "tumble",
    "tumbrels": "tumbrel",
    "tummies": "tummy",
    "tumorigeneses": "tumorigenesis",
    "tumors": "tumor",
    "tumourigeneses": "tumourigenesis",
    "tumours": "tumour",
    "tums": "tum",
    "tumuli": "tumulus",
    "tumults": "tumult",
    "tumuluses": "tumulus",
    "tunas": "tuna",
    "tundras": "tundra",
    "tuned": "tune",
    "tuners": "tuner",
    "tunes": "tune",
    "tunicae": "tunica",
    "tunicates": "tunicate",
    "tunics": "tunic",
    "tuning": "tune",
    "tunings": "tuning",
    "tunneled": "tunnel",
    "tunneling": "tunnel",
    "tunnelled": "tunnel",
    "tunnelling": "tunnel",
    "tunnels": "tunnel",
    "tunnies": "tunny",
    "tuns": "tun",
    "tuples": "tuple",
    "turbans": "turban",
    "turbidities": "turbidity",
    "turbines": "turbine",
    "turbojets": "turbojet",
    "turbos": "turbo",
    "turbots": "turbot",
    "turbulences": "turbulence",
    "turds": "turd",
    "tureens": "tureen",
    "turfed": "turf",
    "turfgrasses": "turfgrass",
    "turfing": "turf",
    "turfs": "turf",
    "turkeys": "turkey",
    "turn-arounds": "turn-around",
    "turn-keys": "turn-key",
    "turn-offs": "turn-off",
    "turn-ons": "turn-on",
    "turn-outs": "turn-out",
    "turn-overs": "turn-over",
    "turn-ups": "turn-up",
    "turnabouts": "turnabout",
    "turnarounds": "turnaround",
    "turncoats": "turncoat",
    "turned": "turn",
    "turners": "turner",
    "turning": "turn",
    "turning-points": "turning-point",
    "turnings": "turning",
    "turnips": "turnip",
    "turnkeys": "turnkey",
    "turnouts": "turnout",
    "turnovers": "turnover",
    "turnpikes": "turnpike",
    "turnrounds": "turnround",
    "turns": "turn",
    "turnstiles": "turnstile",
    "turnstones": "turnstone",
    "turntables": "turntable",
    "turquoises": "turquoise",
    "turrets": "turret",
    "turtledoves": "turtledove",
    "turtlenecks": "turtleneck",
    "turtles": "turtle",
    "turves": "turf",
    "tusks": "tusk",
    "tussled": "tussle",
    "tussles": "tussle",
    "tussling": "tussle",
    "tussocks": "tussock",
    "tut-tuts": "tut-tut",
    "tut-tutted": "tut-tut",
    "tut-tutting": "tut-tut",
    "tutees": "tutee",
    "tutelages": "tutelage",
    "tutored": "tutor",
    "tutorials": "tutorial",
    "tutoring": "tutor",
    "tutors": "tutor",
    "tutus": "tutu",
    "tuxedoes": "tuxedo",
    "tuxedos": "tuxedo",
    "twains": "twain",
    "twanged": "twang",
    "twanging": "twang",
    "twangs": "twang",
    "tweaked": "tweak",
    "tweaking": "tweak",
    "tweaks": "tweak",
    "tweeds": "tweed",
    "tweeted": "tweet",
    "tweeting": "tweet",
    "tweets": "tweet",
    "tweezers": "tweezer",
    "twelfth": "12",
    "twelfths": "twelfth",
    "twelve-months": "twelve-month",
    "twelves": "twelve",
    "twenties": "twenty",
    "twentieth": "20",
    "twentieths": "twentieth",
    "twenty-eighth": "28",
    "twenty-eighths": "twenty-eighth",
    "twenty-fifth": "25",
    "twenty-fifths": "twenty-fifth",
    "twenty-first": "21",
    "twenty-firsts": "twenty-first",
    "twenty-fourth": "24",
    "twenty-fourths": "twenty-fourth",
    "twenty-ninth": "29",
    "twenty-ninths": "twenty-ninth",
    "twenty-second": "22",
    "twenty-seconds": "twenty-second",
    "twenty-seventh": "27",
    "twenty-sevenths": "twenty-seventh",
    "twenty-sixth": "26",
    "twenty-sixths": "twenty-sixth",
    "twenty-third": "23",
    "twenty-thirds": "twenty-third",
    "twerps": "twerp",
    "twiddled": "twiddle",
    "twiddles": "twiddle",
    "twiddling": "twiddle",
    "twigged": "twig",
    "twiggier": "twiggy",
    "twigging": "twig",
    "twigs": "twig",
    "twilights": "twilight",
    "twin-sets": "twin-set",
    "twined": "twine",
    "twines": "twine",
    "twinges": "twinge",
    "twining": "twine",
    "twinkled": "twinkle",
    "twinkles": "twinkle",
    "twinkling": "twinkle",
    "twinned": "twin",
    "twinning": "twin",
    "twins": "twin",
    "twirled": "twirl",
    "twirling": "twirl",
    "twirls": "twirl",
    "twisted": "twist",
    "twisters": "twister",
    "twistier": "twisty",
    "twisting": "twist",
    "twists": "twist",
    "twitched": "twitch",
    "twitchers": "twitcher",
    "twitches": "twitch",
    "twitching": "twitch",
    "twits": "twit",
    "twittered": "twitter",
    "twittering": "twitter",
    "twitters": "twitter",
    "two-faceds": "two-faced",
    "two-hundredth": "200",
    "two-pieces": "two-piece",
    "two-seaters": "two-seater",
    "two-timed": "two-time",
    "two-times": "two-time",
    "two-timing": "two-time",
    "twos": "two",
    "twosomes": "twosome",
    "tycoons": "tycoon",
    "tying": "tie",
    "tykes": "tyke",
    "tympana": "tympanum",
    "tympanums": "tympanum",
    "typecasting": "typecast",
    "typecasts": "typecast",
    "typed": "type",
    "typefaces": "typeface",
    "types": "type",
    "typescripts": "typescript",
    "typesets": "typeset",
    "typesetters": "typesetter",
    "typewriters": "typewriter",
    "typewrites": "typewrite",
    "typewriting": "typewrite",
    "typewritten": "typewrite",
    "typewrote": "typewrite",
    "typhoons": "typhoon",
    "typicalities": "typicality",
    "typifications": "typification",
    "typified": "typify",
    "typifies": "typify",
    "typifying": "typify",
    "typing": "type",
    "typings": "typing",
    "typists": "typist",
    "typographers": "typographer",
    "typographies": "typography",
    "typologies": "typology",
    "typos": "typo",
    "tyramines": "tyramine",
    "tyrannies": "tyranny",
    "tyrannized": "tyrannize",
    "tyrannizes": "tyrannize",
    "tyrannizing": "tyrannize",
    "tyrants": "tyrant",
    "tyred": "tyre",
    "tyres": "tyre",
    "tyring": "tyre",
    "tyros": "tyro",
    "tyrosines": "tyrosine",
    "tzars": "tzar",
    "ubiquitins": "ubiquitin",
    "udders": "udder",
    "uglier": "ugly",
    "ugliest": "ugly",
    "ukeleles": "ukelele",
    "ulcerated": "ulcerate",
    "ulcerates": "ulcerate",
    "ulcerating": "ulcerate",
    "ulcerations": "ulceration",
    "ulcers": "ulcer",
    "ulnae": "ulna",
    "ulnas": "ulna",
    "ultimata": "ultimatum",
    "ultimatums": "ultimatum",
    "ultra-lights": "ultra-light",
    "ultralights": "ultralight",
    "ultras": "ultra",
    "ultrasonographies": "ultrasonography",
    "ultrasounds": "ultrasound",
    "ultrastructures": "ultrastructure",
    "ultraviolets": "ultraviolet",
    "ululated": "ululate",
    "ululates": "ululate",
    "ululating": "ululate",
    "umbels": "umbel",
    "umbers": "umber",
    "umbilicals": "umbilical",
    "umbilici": "umbilicus",
    "umbilicuses": "umbilicus",
    "umbrae": "umbra",
    "umbrages": "umbrage",
    "umbrellas": "umbrella",
    "umlauts": "umlaut",
    "umpired": "umpire",
    "umpires": "umpire",
    "umpiring": "umpire",
    "ums": "um",
    "unabled": "unable",
    "unables": "unable",
    "unabling": "unable",
    "unanimities": "unanimity",
    "unbalanced": "unbalance",
    "unbalances": "unbalance",
    "unbalancing": "unbalance",
    "unbelievers": "unbeliever",
    "unbended": "unbend",
    "unbending": "unbend",
    "unbends": "unbend",
    "unbent": "unbend",
    "unbinding": "unbind",
    "unbinds": "unbind",
    "unbound": "unbind",
    "unbuckled": "unbuckle",
    "unbuckles": "unbuckle",
    "unbuckling": "unbuckle",
    "unbundled": "unbundle",
    "unbundles": "unbundle",
    "unbundling": "unbundle",
    "unburdened": "unburden",
    "unburdening": "unburden",
    "unburdens": "unburden",
    "unbuttoned": "unbutton",
    "unbuttoning": "unbutton",
    "unbuttons": "unbutton",
    "uncalled": "uncall",
    "uncannier": "uncanny",
    "uncanniest": "uncanny",
    "uncapped": "uncap",
    "uncapping": "uncap",
    "uncaps": "uncap",
    "uncertainties": "uncertainty",
    "unchained": "unchain",
    "unchaining": "unchain",
    "unchains": "unchain",
    "unclasped": "unclasp",
    "unclasping": "unclasp",
    "unclasps": "unclasp",
    "uncles": "uncle",
    "unclipped": "unclip",
    "unclipping": "unclip",
    "unclips": "unclip",
    "uncoiled": "uncoil",
    "uncoiling": "uncoil",
    "uncoils": "uncoil",
    "uncommanded": "uncommand",
    "uncommonner": "uncommon",
    "uncommonnest": "uncommon",
    "uncomprehending": "uncomprehend",
    "unconformities": "unconformity",
    "unconnected": "unconnect",
    "uncorked": "uncork",
    "uncorking": "uncork",
    "uncorks": "uncork",
    "uncoupled": "uncouple",
    "uncouples": "uncouple",
    "uncoupling": "uncouple",
    "uncovered": "uncover",
    "uncovering": "uncover",
    "uncovers": "uncover",
    "unctions": "unction",
    "uncured": "uncure",
    "uncurled": "uncurl",
    "uncurling": "uncurl",
    "uncurls": "uncurl",
    "undeceived": "undeceive",
    "undeceives": "undeceive",
    "undeceiving": "undeceive",
    "under-achieved": "under-achieve",
    "under-achievements": "under-achievement",
    "under-achievers": "under-achiever",
    "under-achieves": "under-achieve",
    "under-achieving": "under-achieve",
    "under-estimated": "under-estimate",
    "under-estimates": "under-estimate",
    "under-estimating": "under-estimate",
    "under-graduates": "under-graduate",
    "under-performed": "under-perform",
    "under-performing": "under-perform",
    "under-performs": "under-perform",
    "under-reported": "under-report",
    "under-reporting": "under-report",
    "under-reports": "under-report",
    "under-representations": "under-representation",
    "under-represented": "under-represent",
    "under-representing": "under-represent",
    "under-represents": "under-represent",
    "under-secretaries": "under-secretary",
    "under-utilised": "under-utilise",
    "under-utilises": "under-utilise",
    "under-utilising": "under-utilise",
    "underachieved": "underachieve",
    "underachievements": "underachievement",
    "underachievers": "underachiever",
    "underachieves": "underachieve",
    "underachieving": "underachieve",
    "underbellies": "underbelly",
    "underbidden": "underbid",
    "underbidding": "underbid",
    "underbids": "underbid",
    "underbodies": "underbody",
    "undercarriages": "undercarriage",
    "undercoated": "undercoat",
    "undercoating": "undercoat",
    "undercoats": "undercoat",
    "undercurrents": "undercurrent",
    "undercuts": "undercut",
    "undercutting": "undercut",
    "underdogs": "underdog",
    "underestimated": "underestimate",
    "underestimates": "underestimate",
    "underestimating": "underestimate",
    "underestimations": "underestimation",
    "underexposed": "underexpose",
    "underexposes": "underexpose",
    "underexposing": "underexpose",
    "underfed": "underfeed",
    "underfeeding": "underfeed",
    "underfeeds": "underfeed",
    "underfunded": "underfund",
    "underfunding": "underfund",
    "underfunds": "underfund",
    "undergarments": "undergarment",
    "undergirded": "undergird",
    "undergirding": "undergird",
    "undergirds": "undergird",
    "undergoes": "undergo",
    "undergoing": "undergo",
    "undergone": "undergo",
    "undergraduates": "undergraduate",
    "undergrounds": "underground",
    "underlaid": "underlay",
    "underlain": "underlie",
    "underlay": "underlie",
    "underlayed": "underlay",
    "underlaying": "underlay",
    "underlays": "underlay",
    "underlied": "underlie",
    "underlies": "underlie",
    "underlined": "underline",
    "underlines": "underline",
    "underlings": "underling",
    "underlining": "underline",
    "underlips": "underlip",
    "underlying": "underlie",
    "undermined": "undermine",
    "undermines": "undermine",
    "undermining": "undermine",
    "undernourished": "undernourish",
    "undernourishes": "undernourish",
    "undernourishing": "undernourish",
    "underpaid": "underpay",
    "underparts": "underpart",
    "underpasses": "underpass",
    "underpaying": "underpay",
    "underpayments": "underpayment",
    "underpays": "underpay",
    "underperformed": "underperform",
    "underperforming": "underperform",
    "underperforms": "underperform",
    "underpinned": "underpin",
    "underpinning": "underpin",
    "underpinnings": "underpinning",
    "underpins": "underpin",
    "underplayed": "underplay",
    "underplaying": "underplay",
    "underplays": "underplay",
    "underpriced": "underprice",
    "underprices": "underprice",
    "underpricing": "underprice",
    "underrated": "underrate",
    "underrates": "underrate",
    "underrating": "underrate",
    "underreported": "underreport",
    "underreporting": "underreport",
    "underreports": "underreport",
    "underrepresented": "underrepresent",
    "underrepresenting": "underrepresent",
    "underrepresents": "underrepresent",
    "underscored": "underscore",
    "underscores": "underscore",
    "underscoring": "underscore",
    "undersecretaries": "undersecretary",
    "underselling": "undersell",
    "undersells": "undersell",
    "undershirts": "undershirt",
    "undershooting": "undershoot",
    "undershoots": "undershoot",
    "undershot": "undershoot",
    "undersides": "underside",
    "underskirts": "underskirt",
    "undersold": "undersell",
    "understanding": "understand",
    "understandings": "understanding",
    "understands": "understand",
    "understated": "understate",
    "understatements": "understatement",
    "understates": "understate",
    "understating": "understate",
    "understood": "understand",
    "understudied": "understudy",
    "understudies": "understudy",
    "understudying": "understudy",
    "undertaken": "undertake",
    "undertakers": "undertaker",
    "undertakes": "undertake",
    "undertaking": "undertake",
    "undertakings": "undertaking",
    "undertones": "undertone",
    "undertook": "undertake",
    "undertows": "undertow",
    "underused": "underuse",
    "underuses": "underuse",
    "underusing": "underuse",
    "underutilized": "underutilize",
    "underutilizes": "underutilize",
    "underutilizing": "underutilize",
    "undervalued": "undervalue",
    "undervalues": "undervalue",
    "undervaluing": "undervalue",
    "underwent": "undergo",
    "underwhelmed": "underwhelm",
    "underwhelming": "underwhelm",
    "underwhelms": "underwhelm",
    "underwings": "underwing",
    "underworlds": "underworld",
    "underwriters": "underwriter",
    "underwrites": "underwrite",
    "underwriting": "underwrite",
    "underwritten": "underwrite",
    "underwrote": "underwrite",
    "undesirables": "undesirable",
    "undid": "undo",
    "undigested": "undigest",
    "undocked": "undock",
    "undocking": "undock",
    "undocks": "undock",
    "undoes": "undo",
    "undoing": "undo",
    "undoings": "undoing",
    "undone": "undo",
    "undos": "undo",
    "undrawing": "undraw",
    "undrawn": "undraw",
    "undraws": "undraw",
    "undressed": "undress",
    "undresses": "undress",
    "undressing": "undress",
    "undrew": "undraw",
    "undulated": "undulate",
    "undulates": "undulate",
    "undulating": "undulate",
    "undulations": "undulation",
    "unearthed": "unearth",
    "unearthing": "unearth",
    "unearthlier": "unearthly",
    "unearthliest": "unearthly",
    "unearths": "unearth",
    "uneasier": "uneasy",
    "uneasiest": "uneasy",
    "uneasinesses": "uneasiness",
    "unevennesses": "unevenness",
    "unexplored": "unexplore",
    "unfairer": "unfair",
    "unfairest": "unfair",
    "unfamiliarities": "unfamiliarity",
    "unfastened": "unfasten",
    "unfastening": "unfasten",
    "unfastens": "unfasten",
    "unfits": "unfit",
    "unfitted": "unfit",
    "unfitter": "unfit",
    "unfittest": "unfit",
    "unfitting": "unfit",
    "unfixed": "unfix",
    "unfixes": "unfix",
    "unfixing": "unfix",
    "unfolded": "unfold",
    "unfolding": "unfold",
    "unfolds": "unfold",
    "unfortunates": "unfortunate",
    "unfreezes": "unfreeze",
    "unfreezing": "unfreeze",
    "unfriendlier": "unfriendly",
    "unfriendliest": "unfriendly",
    "unfroze": "unfreeze",
    "unfrozen": "unfreeze",
    "unfurled": "unfurl",
    "unfurling": "unfurl",
    "unfurls": "unfurl",
    "ungainlier": "ungainly",
    "ungainliest": "ungainly",
    "ungirded": "ungird",
    "ungirding": "ungird",
    "ungirds": "ungird",
    "ungirt": "ungird",
    "ungulates": "ungulate",
    "unhappier": "unhappy",
    "unhappiest": "unhappy",
    "unhealthier": "unhealthy",
    "unhealthiest": "unhealthy",
    "unhinged": "unhinge",
    "unhinges": "unhinge",
    "unhinging": "unhinge",
    "unhooked": "unhook",
    "unhooking": "unhook",
    "unhooks": "unhook",
    "unicorns": "unicorn",
    "unicycles": "unicycle",
    "unifications": "unification",
    "unified": "unify",
    "unifies": "unify",
    "uniformed": "uniform",
    "uniforming": "uniform",
    "uniformities": "uniformity",
    "uniforms": "uniform",
    "unifying": "unify",
    "unionised": "unionise",
    "unionises": "unionise",
    "unionising": "unionise",
    "unionists": "unionist",
    "unionized": "unionize",
    "unionizes": "unionize",
    "unionizing": "unionize",
    "unions": "union",
    "uniquenesses": "uniqueness",
    "unisons": "unison",
    "united": "unite",
    "unites": "unite",
    "unities": "unity",
    "uniting": "unite",
    "units": "unit",
    "universalised": "universalise",
    "universalises": "universalise",
    "universalising": "universalise",
    "universalities": "universality",
    "universalized": "universalize",
    "universalizes": "universalize",
    "universalizing": "universalize",
    "universals": "universal",
    "universes": "universe",
    "universities": "university",
    "unkinder": "unkind",
    "unkindest": "unkind",
    "unkindnesses": "unkindness",
    "unknowns": "unknown",
    "unlearned": "unlearn",
    "unlearning": "unlearn",
    "unlearns": "unlearn",
    "unlearnt": "unlearn",
    "unleashed": "unleash",
    "unleashes": "unleash",
    "unleashing": "unleash",
    "unlikelier": "unlikely",
    "unlikeliest": "unlikely",
    "unlinked": "unlink",
    "unlinking": "unlink",
    "unlinks": "unlink",
    "unloaded": "unload",
    "unloading": "unload",
    "unloads": "unload",
    "unlocked": "unlock",
    "unlocking": "unlock",
    "unlocks": "unlock",
    "unluckier": "unlucky",
    "unluckiest": "unlucky",
    "unmade": "unmake",
    "unmaked": "unmake",
    "unmakes": "unmake",
    "unmaking": "unmake",
    "unmanned": "unman",
    "unmanning": "unman",
    "unmans": "unman",
    "unmasked": "unmask",
    "unmasking": "unmask",
    "unmasks": "unmask",
    "unmentionables": "unmentionable",
    "unnerved": "unnerve",
    "unnerves": "unnerve",
    "unnerving": "unnerve",
    "unpacked": "unpack",
    "unpacking": "unpack",
    "unpacks": "unpack",
    "unpersons": "unperson",
    "unpicked": "unpick",
    "unpicking": "unpick",
    "unpicks": "unpick",
    "unpleasantnesses": "unpleasantness",
    "unplugged": "unplug",
    "unplugging": "unplug",
    "unplugs": "unplug",
    "unpredictabilities": "unpredictability",
    "unraveled": "unravel",
    "unraveling": "unravel",
    "unravelled": "unravel",
    "unravelling": "unravel",
    "unravels": "unravel",
    "unrealised": "unrealise",
    "unrealises": "unrealise",
    "unrealising": "unrealise",
    "unreliabilities": "unreliability",
    "unreservedly": "unreserved",
    "unrolled": "unroll",
    "unrolling": "unroll",
    "unrolls": "unroll",
    "unrulier": "unruly",
    "unsaddled": "unsaddle",
    "unsaddles": "unsaddle",
    "unsaddling": "unsaddle",
    "unsaid": "unsay",
    "unsaying": "unsay",
    "unsays": "unsay",
    "unscrambled": "unscramble",
    "unscrambles": "unscramble",
    "unscrambling": "unscramble",
    "unscrewed": "unscrew",
    "unscrewing": "unscrew",
    "unscrews": "unscrew",
    "unsealed": "unseal",
    "unsealing": "unseal",
    "unseals": "unseal",
    "unseated": "unseat",
    "unseating": "unseat",
    "unseats": "unseat",
    "unsettled": "unsettle",
    "unsettles": "unsettle",
    "unsettling": "unsettle",
    "unsheathed": "unsheathe",
    "unsheathes": "unsheathe",
    "unsheathing": "unsheathe",
    "unskicks": "unstick",
    "unsticking": "unstick",
    "unsticks": "unstick",
    "unstrapped": "unstrap",
    "unstrapping": "unstrap",
    "unstraps": "unstrap",
    "unstringing": "unstring",
    "unstrings": "unstring",
    "unstrung": "unstring",
    "unstuck": "unstick",
    "untangled": "untangle",
    "untangles": "untangle",
    "untangling": "untangle",
    "untidier": "untidy",
    "untidiest": "untidy",
    "untied": "untie",
    "unties": "untie",
    "untouchables": "untouchable",
    "untruths": "untruth",
    "untying": "untie",
    "unveiled": "unveil",
    "unveiling": "unveil",
    "unveils": "unveil",
    "unwinded": "unwind",
    "unwinding": "unwind",
    "unwinds": "unwind",
    "unwound": "unwind",
    "unwrapped": "unwrap",
    "unwrapping": "unwrap",
    "unwraps": "unwrap",
    "unzipped": "unzip",
    "unzipping": "unzip",
    "unzips": "unzip",
    "up-dated": "up-date",
    "up-dates": "up-date",
    "up-dating": "up-date",
    "up-regulations": "up-regulation",
    "up-takes": "up-take",
    "upbeats": "upbeat",
    "upbraided": "upbraid",
    "upbraiding": "upbraid",
    "upbraids": "upbraid",
    "upbringings": "upbringing",
    "updated": "update",
    "updates": "update",
    "updating": "update",
    "upended": "upend",
    "upending": "upend",
    "upends": "upend",
    "upgraded": "upgrade",
    "upgrades": "upgrade",
    "upgrading": "upgrade",
    "upheavals": "upheaval",
    "upheld": "uphold",
    "upholded": "uphold",
    "upholders": "upholder",
    "upholding": "uphold",
    "upholds": "uphold",
    "upholstered": "upholster",
    "upholsterers": "upholsterer",
    "upholstering": "upholster",
    "upholsters": "upholster",
    "upkeeps": "upkeep",
    "uplands": "upland",
    "uplifted": "uplift",
    "uplifting": "uplift",
    "uplifts": "uplift",
    "uploaded": "upload",
    "uploading": "upload",
    "uploads": "upload",
    "upped": "up",
    "uppers": "upper",
    "upping": "up",
    "upraised": "upraise",
    "upraises": "upraise",
    "upraising": "upraise",
    "uprated": "uprate",
    "uprates": "uprate",
    "uprating": "uprate",
    "upregulations": "upregulation",
    "uprights": "upright",
    "uprisen": "uprise",
    "uprises": "uprise",
    "uprisings": "uprising",
    "uproars": "uproar",
    "uprooted": "uproot",
    "uprooting": "uproot",
    "uproots": "uproot",
    "uprose": "uprise",
    "ups": "up",
    "upsets": "upset",
    "upsetting": "upset",
    "upshots": "upshot",
    "upsides": "upside",
    "upslopes": "upslope",
    "upstaged": "upstage",
    "upstages": "upstage",
    "upstaging": "upstage",
    "upstarts": "upstart",
    "upsurges": "upsurge",
    "upsweeping": "upsweep",
    "upsweeps": "upsweep",
    "upswept": "upsweep",
    "upswings": "upswing",
    "uptakes": "uptake",
    "uptrends": "uptrend",
    "upturned": "upturn",
    "upturning": "upturn",
    "upturns": "upturn",
    "uraniums": "uranium",
    "uranyls": "uranyl",
    "urates": "urate",
    "urbanisations": "urbanisation",
    "urbanised": "urbanise",
    "urbanises": "urbanise",
    "urbanising": "urbanise",
    "urbanizations": "urbanization",
    "urbanized": "urbanize",
    "urbanizes": "urbanize",
    "urbanizing": "urbanize",
    "urchins": "urchin",
    "ureas": "urea",
    "ureters": "ureter",
    "urethanes": "urethane",
    "urethrae": "urethra",
    "urethras": "urethra",
    "urethritides": "urethritis",
    "urged": "urge",
    "urgencies": "urgency",
    "urges": "urge",
    "urging": "urge",
    "urinals": "urinal",
    "urinalyses": "urinalysis",
    "urinated": "urinate",
    "urinates": "urinate",
    "urinating": "urinate",
    "urinations": "urination",
    "urines": "urine",
    "urns": "urn",
    "urologies": "urology",
    "urologists": "urologist",
    "urticarias": "urticaria",
    "usages": "usage",
    "used": "use",
    "usefulnesses": "usefulness",
    "users": "user",
    "uses": "use",
    "ushered": "usher",
    "usherettes": "usherette",
    "ushering": "usher",
    "ushers": "usher",
    "using": "use",
    "usurers": "usurer",
    "usuries": "usury",
    "usurped": "usurp",
    "usurpers": "usurper",
    "usurping": "usurp",
    "usurps": "usurp",
    "utensils": "utensil",
    "uteri": "uterus",
    "uteruses": "uterus",
    "utilisations": "utilisation",
    "utilised": "utilise",
    "utilises": "utilise",
    "utilising": "utilise",
    "utilities": "utility",
    "utilizations": "utilization",
    "utilized": "utilize",
    "utilizes": "utilize",
    "utilizing": "utilize",
    "utmosts": "utmost",
    "utopians": "utopian",
    "utopias": "utopia",
    "utterances": "utterance",
    "uttered": "utter",
    "uttering": "utter",
    "utters": "utter",
    "uveitides": "uveitis",
    "uveitises": "uveitis",
    "vacancies": "vacancy",
    "vacated": "vacate",
    "vacates": "vacate",
    "vacating": "vacate",
    "vacationed": "vacation",
    "vacationers": "vacationer",
    "vacationing": "vacation",
    "vacations": "vacation",
    "vaccinated": "vaccinate",
    "vaccinates": "vaccinate",
    "vaccinating": "vaccinate",
    "vaccinations": "vaccination",
    "vaccines": "vaccine",
    "vaccinias": "vaccinia",
    "vacillated": "vacillate",
    "vacillates": "vacillate",
    "vacillating": "vacillate",
    "vacillations": "vacillation",
    "vacs": "vac",
    "vacua": "vacuum",
    "vacuities": "vacuity",
    "vacuoles": "vacuole",
    "vacuumed": "vacuum",
    "vacuuming": "vacuum",
    "vacuums": "vacuum",
    "vagabonds": "vagabond",
    "vagaries": "vagary",
    "vagi": "vagus",
    "vaginae": "vagina",
    "vaginas": "vagina",
    "vaginoses": "vaginosis",
    "vagrants": "vagrant",
    "vaguely": "vague",
    "vaguer": "vague",
    "vaguest": "vague",
    "vails": "vail",
    "vainer": "vain",
    "vainest": "vain",
    "valances": "valance",
    "valedictions": "valediction",
    "valences": "valence",
    "valencies": "valency",
    "valentines": "valentine",
    "valerians": "valerian",
    "vales": "vale",
    "valeted": "valet",
    "valeting": "valet",
    "valetings": "valeting",
    "valets": "valet",
    "validated": "validate",
    "validates": "validate",
    "validating": "validate",
    "validations": "validation",
    "validators": "validator",
    "validities": "validity",
    "valines": "valine",
    "valises": "valise",
    "valla": "vallum",
    "valleys": "valley",
    "valuables": "valuable",
    "valuations": "valuation",
    "valued": "value",
    "valuers": "valuer",
    "values": "value",
    "valuing": "value",
    "valves": "valve",
    "vamped": "vamp",
    "vamping": "vamp",
    "vampires": "vampire",
    "vamps": "vamp",
    "vandalised": "vandalise",
    "vandalises": "vandalise",
    "vandalising": "vandalise",
    "vandalized": "vandalize",
    "vandalizes": "vandalize",
    "vandalizing": "vandalize",
    "vandals": "vandal",
    "vanes": "vane",
    "vanguards": "vanguard",
    "vanillas": "vanilla",
    "vanished": "vanish",
    "vanishes": "vanish",
    "vanishing": "vanish",
    "vanities": "vanity",
    "vanquished": "vanquish",
    "vanquishes": "vanquish",
    "vanquishing": "vanquish",
    "vanquishs": "vanquish",
    "vans": "van",
    "vantages": "vantage",
    "vaporisations": "vaporisation",
    "vaporised": "vaporise",
    "vaporisers": "vaporiser",
    "vaporises": "vaporise",
    "vaporising": "vaporise",
    "vaporizations": "vaporization",
    "vaporized": "vaporize",
    "vaporizes": "vaporize",
    "vaporizing": "vaporize",
    "vapors": "vapor",
    "vapourised": "vapourise",
    "vapourises": "vapourise",
    "vapourising": "vapourise",
    "vapours": "vapour",
    "variabilities": "variability",
    "variables": "variable",
    "variances": "variance",
    "variants": "variant",
    "variates": "variate",
    "variations": "variation",
    "varices": "varix",
    "varied": "vary",
    "variegated": "variegate",
    "variegates": "variegate",
    "variegating": "variegate",
    "variegations": "variegation",
    "varies": "vary",
    "varieties": "variety",
    "varixes": "varix",
    "varnished": "varnish",
    "varnishes": "varnish",
    "varnishing": "varnish",
    "vars": "var",
    "varsities": "varsity",
    "varying": "vary",
    "vas": "va",
    "vasa": "vas",
    "vascularities": "vascularity",
    "vasculatures": "vasculature",
    "vasculitides": "vasculitis",
    "vasculitises": "vasculitis",
    "vasectomies": "vasectomy",
    "vaselines": "vaseline",
    "vases": "vase",
    "vasoconstrictions": "vasoconstriction",
    "vasodilatations": "vasodilatation",
    "vasodilations": "vasodilation",
    "vasodilators": "vasodilator",
    "vasopressins": "vasopressin",
    "vassals": "vassal",
    "vaster": "vast",
    "vastest": "vast",
    "vasti": "vastus",
    "vats": "vat",
    "vaudevilles": "vaudeville",
    "vaulted": "vault",
    "vaulters": "vaulter",
    "vaulting": "vault",
    "vaults": "vault",
    "vaunted": "vaunt",
    "vaunting": "vaunt",
    "vaunts": "vaunt",
    "ve": "have",
    "vectored": "vector",
    "vectoring": "vector",
    "vectors": "vector",
    "veered": "veer",
    "veering": "veer",
    "veers": "veer",
    "vegans": "vegan",
    "vegas": "vega",
    "vegetables": "vegetable",
    "vegetarians": "vegetarian",
    "vegetated": "vegetate",
    "vegetates": "vegetate",
    "vegetating": "vegetate",
    "vegetations": "vegetation",
    "vehicles": "vehicle",
    "veiled": "veil",
    "veiling": "veil",
    "veils": "veil",
    "veins": "vein",
    "vela": "velum",
    "veldts": "veldt",
    "vellums": "vellum",
    "velocities": "velocity",
    "velodromes": "velodrome",
    "velums": "velum",
    "venae": "vena",
    "vended": "vend",
    "venders": "vender",
    "vendettas": "vendetta",
    "vending": "vend",
    "vendors": "vendor",
    "vends": "vend",
    "veneered": "veneer",
    "veneering": "veneer",
    "veneers": "veneer",
    "venepunctures": "venepuncture",
    "venerated": "venerate",
    "venerates": "venerate",
    "venerating": "venerate",
    "venoms": "venom",
    "vented": "vent",
    "ventilated": "ventilate",
    "ventilates": "ventilate",
    "ventilating": "ventilate",
    "ventilations": "ventilation",
    "ventilators": "ventilator",
    "venting": "vent",
    "ventouses": "ventouse",
    "ventricles": "ventricle",
    "ventriloquists": "ventriloquist",
    "vents": "vent",
    "ventured": "venture",
    "venturers": "venturer",
    "ventures": "venture",
    "venturing": "venture",
    "venturis": "venturi",
    "venues": "venue",
    "verandahs": "verandah",
    "verandas": "veranda",
    "veras": "vera",
    "verbalisations": "verbalisation",
    "verbalised": "verbalise",
    "verbalises": "verbalise",
    "verbalising": "verbalise",
    "verbalized": "verbalize",
    "verbalizes": "verbalize",
    "verbalizing": "verbalize",
    "verboser": "verbose",
    "verbosest": "verbose",
    "verbs": "verb",
    "verdicts": "verdict",
    "verged": "verge",
    "vergences": "vergence",
    "vergers": "verger",
    "verges": "verge",
    "verging": "verge",
    "verifications": "verification",
    "verified": "verify",
    "verifiers": "verifier",
    "verifies": "verify",
    "verifying": "verify",
    "verisimilitudes": "verisimilitude",
    "verities": "verity",
    "vermiculites": "vermiculite",
    "vermouths": "vermouth",
    "vernaculars": "vernacular",
    "verniers": "vernier",
    "verrucae": "verruca",
    "verrucas": "verruca",
    "versatilities": "versatility",
    "verses": "verse",
    "versioned": "version",
    "versioning": "version",
    "versionings": "versioning",
    "versions": "version",
    "vertebrae": "vertebra",
    "vertebras": "vertebra",
    "vertebrates": "vertebrate",
    "vertexes": "vertex",
    "verticals": "vertical",
    "vertices": "vertex",
    "vertigines": "vertigo",
    "vertigoes": "vertigo",
    "vertigos": "vertigo",
    "verts": "vert",
    "verums": "verum",
    "vesicles": "vesicle",
    "vespers": "vesper",
    "vessels": "vessel",
    "vested": "vest",
    "vestibules": "vestibule",
    "vestiges": "vestige",
    "vesting": "vest",
    "vestments": "vestment",
    "vestries": "vestry",
    "vests": "vest",
    "vetches": "vetch",
    "veterans": "veteran",
    "veterinarians": "veterinarian",
    "vetoed": "veto",
    "vetoes": "veto",
    "vetoing": "veto",
    "vetos": "veto",
    "vets": "vet",
    "vetted": "vet",
    "vetting": "vet",
    "vexations": "vexation",
    "vexed": "vex",
    "vexes": "vex",
    "vexing": "vex",
    "viabilities": "viability",
    "viaducts": "viaduct",
    "vials": "vial",
    "vibraphones": "vibraphone",
    "vibrated": "vibrate",
    "vibrates": "vibrate",
    "vibrating": "vibrate",
    "vibrations": "vibration",
    "vibrators": "vibrator",
    "vibratos": "vibrato",
    "viburnums": "viburnum",
    "vicarages": "vicarage",
    "vicars": "vicar",
    "vice-chairs": "vice-chair",
    "vice-chancellors": "vice-chancellor",
    "vice-presidents": "vice-president",
    "viceroys": "viceroy",
    "vices": "vice",
    "vicinities": "vicinity",
    "vicissitudes": "vicissitude",
    "victimisations": "victimisation",
    "victimised": "victimise",
    "victimises": "victimise",
    "victimising": "victimise",
    "victimizations": "victimization",
    "victimized": "victimize",
    "victimizes": "victimize",
    "victimizing": "victimize",
    "victims": "victim",
    "victories": "victory",
    "victors": "victor",
    "victualed": "victual",
    "victualing": "victual",
    "victualled": "victual",
    "victualling": "victual",
    "victuals": "victual",
    "video-conferences": "video-conference",
    "video-discs": "video-disc",
    "video-taped": "video-tape",
    "video-tapes": "video-tape",
    "video-taping": "video-tape",
    "videocassettes": "videocassette",
    "videoconferences": "videoconference",
    "videodiscs": "videodisc",
    "videoed": "video",
    "videogames": "videogame",
    "videographies": "videography",
    "videoing": "video",
    "videos": "video",
    "videotaped": "videotape",
    "videotapes": "videotape",
    "videotaping": "videotape",
    "vied": "vie",
    "vies": "vie",
    "view-points": "view-point",
    "viewed": "view",
    "viewers": "viewer",
    "viewfinders": "viewfinder",
    "viewing": "view",
    "viewpoints": "viewpoint",
    "views": "view",
    "vigilantes": "vigilante",
    "vigils": "vigil",
    "vignetted": "vignette",
    "vignettes": "vignette",
    "vignetting": "vignette",
    "vignettings": "vignetting",
    "viler": "vile",
    "vilest": "vile",
    "vilifications": "vilification",
    "vilified": "vilify",
    "vilifies": "vilify",
    "vilifying": "vilify",
    "villagers": "villager",
    "villages": "village",
    "villainies": "villainy",
    "villains": "villain",
    "villas": "villa",
    "villi": "villus",
    "vinaigrettes": "vinaigrette",
    "vindicated": "vindicate",
    "vindicates": "vindicate",
    "vindicating": "vindicate",
    "vinegars": "vinegar",
    "vines": "vine",
    "vineyards": "vineyard",
    "vinifications": "vinification",
    "vinos": "vino",
    "vintages": "vintage",
    "vintners": "vintner",
    "vinyls": "vinyl",
    "violas": "viola",
    "violated": "violate",
    "violates": "violate",
    "violating": "violate",
    "violations": "violation",
    "violators": "violator",
    "violets": "violet",
    "violinists": "violinist",
    "violins": "violin",
    "violists": "violist",
    "viols": "viol",
    "vipers": "viper",
    "viragos": "virago",
    "virements": "virement",
    "virginals": "virginal",
    "virgins": "virgin",
    "virilities": "virility",
    "virions": "virion",
    "viroids": "viroid",
    "virologies": "virology",
    "virologists": "virologist",
    "virtualities": "virtuality",
    "virtues": "virtue",
    "virtuosi": "virtuoso",
    "virtuosos": "virtuoso",
    "virulences": "virulence",
    "viruses": "virus",
    "visaed": "visa",
    "visages": "visage",
    "visaing": "visa",
    "visas": "visa",
    "viscera": "viscus",
    "viscoses": "viscose",
    "viscosities": "viscosity",
    "viscountesses": "viscountess",
    "viscounts": "viscount",
    "vises": "vise",
    "visibilities": "visibility",
    "visibler": "visible",
    "visiblest": "visible",
    "visionaries": "visionary",
    "visioned": "vision",
    "visioning": "vision",
    "visions": "vision",
    "visitations": "visitation",
    "visited": "visit",
    "visiting": "visit",
    "visitors": "visitor",
    "visits": "visit",
    "visors": "visor",
    "vistas": "vista",
    "visualisations": "visualisation",
    "visualised": "visualise",
    "visualises": "visualise",
    "visualising": "visualise",
    "visualizations": "visualization",
    "visualized": "visualize",
    "visualizes": "visualize",
    "visualizing": "visualize",
    "visuals": "visual",
    "vitae": "vita",
    "vitalised": "vitalise",
    "vitalises": "vitalise",
    "vitalising": "vitalise",
    "vitalities": "vitality",
    "vitamins": "vitamin",
    "vitas": "vita",
    "vitiated": "vitiate",
    "vitiates": "vitiate",
    "vitiating": "vitiate",
    "vitrifications": "vitrification",
    "vitrified": "vitrify",
    "vitrifies": "vitrify",
    "vitrifying": "vitrify",
    "vivaria": "vivarium",
    "vivariums": "vivarium",
    "vivas": "viva",
    "vivider": "vivid",
    "vividest": "vivid",
    "vivified": "vivify",
    "vivifies": "vivify",
    "vivifying": "vivify",
    "vivisectionists": "vivisectionist",
    "vivisections": "vivisection",
    "vixens": "vixen",
    "viziers": "vizier",
    "vocabularies": "vocabulary",
    "vocalisations": "vocalisation",
    "vocalised": "vocalise",
    "vocalises": "vocalise",
    "vocalising": "vocalise",
    "vocalists": "vocalist",
    "vocalizations": "vocalization",
    "vocalized": "vocalize",
    "vocalizes": "vocalize",
    "vocalizing": "vocalize",
    "vocals": "vocal",
    "vocations": "vocation",
    "vocatives": "vocative",
    "voces": "vox",
    "vocoders": "vocoder",
    "vodkas": "vodka",
    "vogues": "vogue",
    "voice-overs": "voice-over",
    "voiced": "voice",
    "voices": "voice",
    "voicing": "voice",
    "voided": "void",
    "voiding": "void",
    "voids": "void",
    "vol-au-vents": "vol-au-vent",
    "volatiles": "volatile",
    "volatilities": "volatility",
    "volcanisms": "volcanism",
    "volcanoes": "volcano",
    "volcanos": "volcano",
    "voles": "vole",
    "volleyballs": "volleyball",
    "volleyed": "volley",
    "volleying": "volley",
    "volleys": "volley",
    "vols": "vol",
    "voltages": "voltage",
    "voltmeters": "voltmeter",
    "volts": "volt",
    "volumes": "volume",
    "voluntaries": "voluntary",
    "volunteered": "volunteer",
    "volunteering": "volunteer",
    "volunteers": "volunteer",
    "volvuli": "volvulus",
    "volvuluses": "volvulus",
    "vomited": "vomit",
    "vomiting": "vomit",
    "vomitings": "vomiting",
    "vomits": "vomit",
    "vortexes": "vortex",
    "vortices": "vortex",
    "vorticities": "vorticity",
    "votaries": "votary",
    "voted": "vote",
    "voters": "voter",
    "votes": "vote",
    "voting": "vote",
    "vouched": "vouch",
    "vouchers": "voucher",
    "vouches": "vouch",
    "vouching": "vouch",
    "vouchsafed": "vouchsafe",
    "vouchsafes": "vouchsafe",
    "vouchsafing": "vouchsafe",
    "vowed": "vow",
    "vowels": "vowel",
    "vowing": "vow",
    "vows": "vow",
    "voxels": "voxel",
    "voyaged": "voyage",
    "voyagers": "voyager",
    "voyages": "voyage",
    "voyaging": "voyage",
    "voyeurs": "voyeur",
    "vulgarities": "vulgarity",
    "vulnerabilities": "vulnerability",
    "vulnerabler": "vulnerable",
    "vulnerablest": "vulnerable",
    "vultures": "vulture",
    "vulvae": "vulva",
    "vulvas": "vulva",
    "vying": "vie",
    "wackier": "wacky",
    "wackiest": "wacky",
    "wacks": "wack",
    "wadded": "wad",
    "wadding": "wad",
    "waddled": "waddle",
    "waddles": "waddle",
    "waddling": "waddle",
    "waded": "wade",
    "waders": "wader",
    "wades": "wade",
    "wading": "wade",
    "wadis": "wadi",
    "wads": "wad",
    "wafers": "wafer",
    "waffled": "waffle",
    "waffles": "waffle",
    "waffling": "waffle",
    "wafted": "waft",
    "wafting": "waft",
    "wafts": "waft",
    "wage-packets": "wage-packet",
    "waged": "wage",
    "wagered": "wager",
    "wagering": "wager",
    "wagers": "wager",
    "wages": "wage",
    "wagged": "wag",
    "wagging": "wag",
    "waggled": "waggle",
    "waggles": "waggle",
    "waggling": "waggle",
    "waggons": "waggon",
    "waging": "wage",
    "wagons": "wagon",
    "wags": "wag",
    "wagtails": "wagtail",
    "waifs": "waif",
    "wailed": "wail",
    "wailing": "wail",
    "wails": "wail",
    "wainscots": "wainscot",
    "waistbands": "waistband",
    "waistcoats": "waistcoat",
    "waistlines": "waistline",
    "waists": "waist",
    "waited": "wait",
    "waiters": "waiter",
    "waiting": "wait",
    "waiting-rooms": "waiting-room",
    "waitresses": "waitress",
    "waits": "wait",
    "waived": "waive",
    "waivers": "waiver",
    "waives": "waive",
    "waiving": "waive",
    "wakeboards": "wakeboard",
    "waked": "wake",
    "wakefields": "wakefield",
    "wakened": "waken",
    "wakening": "waken",
    "wakens": "waken",
    "wakes": "wake",
    "wakeups": "wakeup",
    "waking": "wake",
    "waled": "wale",
    "wales": "wale",
    "waling": "wale",
    "walk-outs": "walk-out",
    "walk-ups": "walk-up",
    "walkabouts": "walkabout",
    "walked": "walk",
    "walkers": "walker",
    "walkie-talkies": "walkie-talkie",
    "walking": "walk",
    "walkmen": "walkman",
    "walkouts": "walkout",
    "walkovers": "walkover",
    "walks": "walk",
    "walkways": "walkway",
    "wallabies": "wallaby",
    "wallboards": "wallboard",
    "walled": "wall",
    "wallets": "wallet",
    "wallflowers": "wallflower",
    "wallies": "wally",
    "walling": "wall",
    "walloped": "wallop",
    "walloping": "wallop",
    "wallopings": "walloping",
    "wallops": "wallop",
    "wallowed": "wallow",
    "wallowing": "wallow",
    "wallows": "wallow",
    "wallpapered": "wallpaper",
    "wallpapering": "wallpaper",
    "wallpapers": "wallpaper",
    "walls": "wall",
    "walnuts": "walnut",
    "walruses": "walrus",
    "waltzed": "waltz",
    "waltzes": "waltz",
    "waltzing": "waltz",
    "wandered": "wander",
    "wanderers": "wanderer",
    "wandering": "wander",
    "wanderings": "wandering",
    "wanders": "wander",
    "wands": "wand",
    "waned": "wane",
    "wanes": "wane",
    "wangled": "wangle",
    "wangles": "wangle",
    "wangling": "wangle",
    "waning": "wane",
    "wanked": "wank",
    "wankers": "wanker",
    "wanking": "wank",
    "wanks": "wank",
    "wanned": "wan",
    "wanner": "wan",
    "wannest": "wan",
    "wanning": "wan",
    "wans": "wan",
    "wanted": "want",
    "wanting": "want",
    "wants": "want",
    "war-criminals": "war-criminal",
    "warbled": "warble",
    "warblers": "warbler",
    "warbles": "warble",
    "warbling": "warble",
    "warded": "ward",
    "wardens": "warden",
    "warders": "warder",
    "warding": "ward",
    "wardresses": "wardress",
    "wardrobes": "wardrobe",
    "wards": "ward",
    "wardships": "wardship",
    "wared": "ware",
    "warehoused": "warehouse",
    "warehousekeepers": "warehousekeeper",
    "warehousemen": "warehouseman",
    "warehouses": "warehouse",
    "warehousing": "warehouse",
    "wares": "ware",
    "warfares": "warfare",
    "warfarins": "warfarin",
    "warheads": "warhead",
    "warier": "wary",
    "wariest": "wary",
    "waring": "ware",
    "warlocks": "warlock",
    "warm-ups": "warm-up",
    "warmed": "warm",
    "warmer": "warm",
    "warmers": "warmer",
    "warmest": "warm",
    "warming": "warm",
    "warmongers": "warmonger",
    "warms": "warm",
    "warmths": "warmth",
    "warned": "warn",
    "warning": "warn",
    "warnings": "warning",
    "warns": "warn",
    "warped": "warp",
    "warping": "warp",
    "warps": "warp",
    "warranted": "warrant",
    "warrantees": "warrantee",
    "warranties": "warranty",
    "warranting": "warrant",
    "warrants": "warrant",
    "warred": "war",
    "warrens": "warren",
    "warring": "war",
    "warriors": "warrior",
    "wars": "war",
    "warships": "warship",
    "warthogs": "warthog",
    "warts": "wart",
    "was": "be",
    "wasabis": "wasabi",
    "wash-outs": "wash-out",
    "washbasins": "washbasin",
    "washbowls": "washbowl",
    "washcloths": "washcloth",
    "washdays": "washday",
    "washed": "wash",
    "washers": "washer",
    "washerwomen": "washerwoman",
    "washes": "wash",
    "washing": "wash",
    "washing-ups": "washing-up",
    "washouts": "washout",
    "washrooms": "washroom",
    "washstands": "washstand",
    "wasps": "wasp",
    "wass": "be",
    "wastages": "wastage",
    "wastebaskets": "wastebasket",
    "wastebins": "wastebin",
    "wasted": "waste",
    "wastelands": "wasteland",
    "wasters": "waster",
    "wastes": "waste",
    "wastewaters": "wastewater",
    "wasting": "waste",
    "watch-chains": "watch-chain",
    "watchbands": "watchband",
    "watchdogs": "watchdog",
    "watched": "watch",
    "watchers": "watcher",
    "watches": "watch",
    "watching": "watch",
    "watchmakers": "watchmaker",
    "watchmen": "watchman",
    "watchstraps": "watchstrap",
    "watchtowers": "watchtower",
    "watchwords": "watchword",
    "water-closets": "water-closet",
    "water-colours": "water-colour",
    "water-courses": "water-course",
    "water-ices": "water-ice",
    "water-lilies": "water-lily",
    "water-lines": "water-line",
    "water-meadows": "water-meadow",
    "water-skied": "water-ski",
    "water-skiing": "water-ski",
    "water-skis": "water-ski",
    "waterbaths": "waterbath",
    "waterbeds": "waterbed",
    "waterbirths": "waterbirth",
    "waterbodies": "waterbody",
    "watercolors": "watercolor",
    "watercolours": "watercolour",
    "watercourses": "watercourse",
    "watered": "water",
    "waterfalls": "waterfall",
    "waterfowls": "waterfowl",
    "waterfronts": "waterfront",
    "waterholes": "waterhole",
    "waterier": "watery",
    "watering": "water",
    "waterjets": "waterjet",
    "waterlilies": "waterlily",
    "waterlines": "waterline",
    "watermarked": "watermark",
    "watermarking": "watermark",
    "watermarks": "watermark",
    "watermelons": "watermelon",
    "watermen": "waterman",
    "watermills": "watermill",
    "waterproofed": "waterproof",
    "waterproofing": "waterproof",
    "waterproofs": "waterproof",
    "waters": "water",
    "watersheds": "watershed",
    "waterspouts": "waterspout",
    "waterways": "waterway",
    "waterwheels": "waterwheel",
    "wats": "wat",
    "wattages": "wattage",
    "wattles": "wattle",
    "watts": "watt",
    "wavebands": "waveband",
    "waved": "wave",
    "waveforms": "waveform",
    "wavefronts": "wavefront",
    "wavefunctions": "wavefunction",
    "waveguides": "waveguide",
    "wavelengths": "wavelength",
    "wavelets": "wavelet",
    "wavenumbers": "wavenumber",
    "wavered": "waver",
    "wavering": "waver",
    "wavers": "waver",
    "waves": "wave",
    "wavier": "wavy",
    "waviest": "wavy",
    "waving": "wave",
    "waxed": "wax",
    "waxes": "wax",
    "waxier": "waxy",
    "waxiest": "waxy",
    "waxing": "wax",
    "waxworks": "waxwork",
    "waybills": "waybill",
    "wayfarers": "wayfarer",
    "waylaid": "waylay",
    "waylayed": "waylay",
    "waylaying": "waylay",
    "waylays": "waylay",
    "ways": "way",
    "waysides": "wayside",
    "wc's": "wc",
    "weakened": "weaken",
    "weakening": "weaken",
    "weakens": "weaken",
    "weaker": "weak",
    "weakest": "weak",
    "weaklier": "weakly",
    "weaklings": "weakling",
    "weaknesses": "weakness",
    "weals": "weal",
    "wealthier": "wealthy",
    "wealthiest": "wealthy",
    "weaned": "wean",
    "weaners": "weaner",
    "weaning": "wean",
    "weans": "wean",
    "weapons": "weapon",
    "wearers": "wearer",
    "wearied": "weary",
    "wearier": "weary",
    "wearies": "weary",
    "weariest": "weary",
    "wearing": "wear",
    "wears": "wear",
    "wearying": "weary",
    "weasels": "weasel",
    "weather-vanes": "weather-vane",
    "weathercocks": "weathercock",
    "weathered": "weather",
    "weathering": "weather",
    "weathermen": "weatherman",
    "weathers": "weather",
    "weaved": "weave",
    "weavers": "weaver",
    "weaves": "weave",
    "weaving": "weave",
    "weavings": "weaving",
    "web-sites": "web-site",
    "webbed": "web",
    "webbing": "web",
    "webmasters": "webmaster",
    "webpages": "webpage",
    "webs": "web",
    "websites": "website",
    "wedded": "wed",
    "wedding": "wed",
    "weddings": "wedding",
    "wedged": "wedge",
    "wedges": "wedge",
    "wedging": "wedge",
    "weds": "wed",
    "weed": "wee",
    "weed-killers": "weed-killer",
    "weeded": "weed",
    "weeders": "weeder",
    "weedier": "weedy",
    "weediest": "weedy",
    "weeding": "weed",
    "weedkillers": "weedkiller",
    "weeds": "weed",
    "weeing": "wee",
    "week-days": "week-day",
    "week-ends": "week-end",
    "weekdays": "weekday",
    "weekenders": "weekender",
    "weekends": "weekend",
    "weeklies": "weekly",
    "weeknights": "weeknight",
    "weeks": "week",
    "weenier": "weeny",
    "weeniest": "weeny",
    "weepies": "weepy",
    "weeping": "weep",
    "weeps": "weep",
    "weer": "wee",
    "wees": "wee",
    "weevils": "weevil",
    "wefts": "weft",
    "weigh-ins": "weigh-in",
    "weighbridges": "weighbridge",
    "weighed": "weigh",
    "weighing": "weigh",
    "weighings": "weighing",
    "weighs": "weigh",
    "weighted": "weight",
    "weightier": "weighty",
    "weightiest": "weighty",
    "weighting": "weight",
    "weightings": "weighting",
    "weightlifters": "weightlifter",
    "weights": "weight",
    "weirder": "weird",
    "weirdest": "weird",
    "weirdos": "weirdo",
    "weirs": "weir",
    "welched": "welch",
    "welches": "welch",
    "welching": "welch",
    "welcomed": "welcome",
    "welcomes": "welcome",
    "welcoming": "welcome",
    "welded": "weld",
    "welders": "welder",
    "welding": "weld",
    "welds": "weld",
    "well": "good",
    "well-beings": "well-being",
    "well-wishers": "well-wisher",
    "welled": "well",
    "wellies": "welly",
    "welling": "well",
    "wellingtons": "wellington",
    "wells": "well",
    "wellsprings": "wellspring",
    "wellwishers": "wellwisher",
    "welshed": "welsh",
    "welshing": "welsh",
    "welterweights": "welterweight",
    "welts": "welt",
    "wenches": "wench",
    "wended": "wend",
    "wending": "wend",
    "wends": "wend",
    "wens": "wen",
    "went": "go",
    "wept": "weep",
    "were": "be",
    "werewolves": "werewolf",
    "westerners": "westerner",
    "westernised": "westernise",
    "westernises": "westernise",
    "westernising": "westernise",
    "westernized": "westernize",
    "westernizes": "westernize",
    "westernizing": "westernize",
    "westerns": "western",
    "westwards": "westward",
    "wet-nurses": "wet-nurse",
    "wethers": "wether",
    "wetlands": "wetland",
    "wets": "wet",
    "wetsuits": "wetsuit",
    "wetted": "wet",
    "wetter": "wet",
    "wetters": "wetter",
    "wettest": "wet",
    "wetting": "wet",
    "whacked": "whack",
    "whacking": "whack",
    "whackings": "whacking",
    "whacks": "whack",
    "whalers": "whaler",
    "whales": "whale",
    "wharfs": "wharf",
    "wharves": "wharf",
    "wheatears": "wheatear",
    "wheatgrasses": "wheatgrass",
    "wheats": "wheat",
    "wheedled": "wheedle",
    "wheedles": "wheedle",
    "wheedling": "wheedle",
    "wheel-chairs": "wheel-chair",
    "wheelbarrows": "wheelbarrow",
    "wheelbases": "wheelbase",
    "wheelchairs": "wheelchair",
    "wheeled": "wheel",
    "wheeler-dealers": "wheeler-dealer",
    "wheelhouses": "wheelhouse",
    "wheelies": "wheelie",
    "wheeling": "wheel",
    "wheels": "wheel",
    "wheelwrights": "wheelwright",
    "wheezed": "wheeze",
    "wheezes": "wheeze",
    "wheezier": "wheezy",
    "wheeziest": "wheezy",
    "wheezing": "wheeze",
    "whelks": "whelk",
    "whelmed": "whelm",
    "whelming": "whelm",
    "whelms": "whelm",
    "whelped": "whelp",
    "whelping": "whelp",
    "whelps": "whelp",
    "wherefores": "wherefore",
    "wheres": "where",
    "whets": "whet",
    "whetstones": "whetstone",
    "whetted": "whet",
    "whetting": "whet",
    "whies": "why",
    "whiffs": "whiff",
    "whiled": "while",
    "whiles": "while",
    "whiling": "while",
    "whimpered": "whimper",
    "whimpering": "whimper",
    "whimpers": "whimper",
    "whims": "whim",
    "whimsies": "whimsy",
    "whined": "whine",
    "whines": "whine",
    "whinged": "whinge",
    "whingeing": "whinge",
    "whinges": "whinge",
    "whinging": "whinge",
    "whining": "whine",
    "whinnied": "whinny",
    "whinnies": "whinny",
    "whinnying": "whinny",
    "whins": "whin",
    "whip-rounds": "whip-round",
    "whiplashes": "whiplash",
    "whipped": "whip",
    "whippersnappers": "whippersnapper",
    "whippets": "whippet",
    "whippier": "whippy",
    "whipping": "whip",
    "whips": "whip",
    "whirled": "whirl",
    "whirling": "whirl",
    "whirlpools": "whirlpool",
    "whirls": "whirl",
    "whirlwinds": "whirlwind",
    "whirred": "whirr",
    "whirring": "whirr",
    "whirrs": "whirr",
    "whirs": "whir",
    "whisked": "whisk",
    "whiskers": "whisker",
    "whiskeys": "whiskey",
    "whiskies": "whisky",
    "whisking": "whisk",
    "whisks": "whisk",
    "whispered": "whisper",
    "whispering": "whisper",
    "whispers": "whisper",
    "whistle-blowers": "whistle-blower",
    "whistleblowers": "whistleblower",
    "whistled": "whistle",
    "whistlers": "whistler",
    "whistles": "whistle",
    "whistling": "whistle",
    "whitebeams": "whitebeam",
    "whitefishes": "whitefish",
    "whiteflies": "whitefly",
    "whiteheads": "whitehead",
    "whitened": "whiten",
    "whiteners": "whitener",
    "whitening": "whiten",
    "whitens": "whiten",
    "whiteouts": "whiteout",
    "whiter": "white",
    "whites": "white",
    "whitest": "white",
    "whitethroats": "whitethroat",
    "whitewashed": "whitewash",
    "whitewashes": "whitewash",
    "whitewashing": "whitewash",
    "whitings": "whiting",
    "whittled": "whittle",
    "whittles": "whittle",
    "whittling": "whittle",
    "whized": "whiz",
    "whizes": "whiz",
    "whizing": "whiz",
    "whizz-kids": "whizz-kid",
    "whizzed": "whizz",
    "whizzes": "whizz",
    "whizzing": "whizz",
    "whodunits": "whodunit",
    "wholefoods": "wholefood",
    "wholes": "whole",
    "wholesaled": "wholesale",
    "wholesalers": "wholesaler",
    "wholesales": "wholesale",
    "wholesaling": "wholesale",
    "whooped": "whoop",
    "whooping": "whoop",
    "whoops": "whoop",
    "whooshed": "whoosh",
    "whooshes": "whoosh",
    "whooshing": "whoosh",
    "whoppers": "whopper",
    "whorehouses": "whorehouse",
    "whores": "whore",
    "whorls": "whorl",
    "whys": "why",
    "wickeder": "wicked",
    "wickedest": "wicked",
    "wicket-keepers": "wicket-keeper",
    "wicketkeepers": "wicketkeeper",
    "wickets": "wicket",
    "wicks": "wick",
    "widebands": "wideband",
    "widened": "widen",
    "widening": "widen",
    "widens": "widen",
    "wider": "wide",
    "wides": "wide",
    "widest": "wide",
    "widgets": "widget",
    "widowers": "widower",
    "widows": "widow",
    "widths": "width",
    "wielded": "wield",
    "wielding": "wield",
    "wields": "wield",
    "wifelier": "wifely",
    "wigeons": "wigeon",
    "wiggled": "wiggle",
    "wigglers": "wiggler",
    "wiggles": "wiggle",
    "wiggling": "wiggle",
    "wigs": "wig",
    "wigwams": "wigwam",
    "wild-types": "wild-type",
    "wildcats": "wildcat",
    "wildebeests": "wildebeest",
    "wilder": "wild",
    "wildernesses": "wilderness",
    "wildest": "wild",
    "wildfires": "wildfire",
    "wilds": "wild",
    "wiled": "wile",
    "wiles": "wile",
    "wilier": "wily",
    "wiliest": "wily",
    "wiling": "wile",
    "will-o-the-wisps": "will-o-the-wisp",
    "willed": "will",
    "willies": "willy",
    "willing": "will",
    "willow-herbs": "willow-herb",
    "willows": "willow",
    "wills": "will",
    "wilted": "wilt",
    "wilting": "wilt",
    "wilts": "wilt",
    "wimples": "wimple",
    "wimps": "wimp",
    "winced": "wince",
    "winces": "wince",
    "winched": "winch",
    "winches": "winch",
    "winching": "winch",
    "wincing": "wince",
    "windbags": "windbag",
    "windbreaks": "windbreak",
    "winded": "wind",
    "winders": "winder",
    "windfalls": "windfall",
    "windier": "windy",
    "windiest": "windy",
    "winding": "wind",
    "windings": "winding",
    "windlasses": "windlass",
    "windmills": "windmill",
    "window-boxes": "window-box",
    "window-dressers": "window-dresser",
    "window-frames": "window-frame",
    "window-panes": "window-pane",
    "window-shopped": "window-shop",
    "window-shopping": "window-shop",
    "window-shops": "window-shop",
    "window-sills": "window-sill",
    "windowed": "window",
    "windowing": "window",
    "windowpanes": "windowpane",
    "windows": "window",
    "windowsills": "windowsill",
    "windpipes": "windpipe",
    "winds": "wind",
    "windscreens": "windscreen",
    "windshields": "windshield",
    "windstorms": "windstorm",
    "windsurfers": "windsurfer",
    "wined": "wine",
    "wineglasses": "wineglass",
    "wineries": "winery",
    "wines": "wine",
    "winged": "wing",
    "wingers": "winger",
    "winging": "wing",
    "wingmen": "wingman",
    "wings": "wing",
    "wingspans": "wingspan",
    "wining": "wine",
    "winked": "wink",
    "winking": "wink",
    "winkled": "winkle",
    "winkles": "winkle",
    "winkling": "winkle",
    "winks": "wink",
    "winners": "winner",
    "winning": "win",
    "winnings": "winning",
    "winnowed": "winnow",
    "winnowing": "winnow",
    "winnows": "winnow",
    "wins": "win",
    "wintered": "winter",
    "winterier": "wintery",
    "wintering": "winter",
    "winters": "winter",
    "wintrier": "wintry",
    "wintriest": "wintry",
    "wiped": "wipe",
    "wipers": "wiper",
    "wipes": "wipe",
    "wiping": "wipe",
    "wire-tapped": "wire-tap",
    "wire-tapping": "wire-tap",
    "wire-taps": "wire-tap",
    "wired": "wire",
    "wiredrawing": "wiredraw",
    "wiredrawn": "wiredraw",
    "wiredraws": "wiredraw",
    "wiredrew": "wiredraw",
    "wirelesses": "wireless",
    "wires": "wire",
    "wiretaps": "wiretap",
    "wireworms": "wireworm",
    "wirier": "wiry",
    "wiriest": "wiry",
    "wiring": "wire",
    "wirings": "wiring",
    "wisdoms": "wisdom",
    "wisecracked": "wisecrack",
    "wisecracking": "wisecrack",
    "wisecracks": "wisecrack",
    "wised": "wise",
    "wiser": "wise",
    "wises": "wise",
    "wisest": "wise",
    "wishbones": "wishbone",
    "wished": "wish",
    "wishes": "wish",
    "wishing": "wish",
    "wising": "wise",
    "wispier": "wispy",
    "wisps": "wisp",
    "wist": "wit",
    "wisterias": "wisteria",
    "witch-hunts": "witch-hunt",
    "witches": "witch",
    "withdrawals": "withdrawal",
    "withdrawed": "withdraw",
    "withdrawing": "withdraw",
    "withdrawn": "withdraw",
    "withdraws": "withdraw",
    "withdrew": "withdraw",
    "withered": "wither",
    "withering": "wither",
    "withers": "wither",
    "withheld": "withhold",
    "withholded": "withhold",
    "withholding": "withhold",
    "withholds": "withhold",
    "withstanding": "withstand",
    "withstands": "withstand",
    "withstood": "withstand",
    "witness-boxes": "witness-box",
    "witnessed": "witness",
    "witnesses": "witness",
    "witnessing": "witness",
    "wits": "wit",
    "witted": "wit",
    "wittered": "witter",
    "wittering": "witter",
    "witters": "witter",
    "witticisms": "witticism",
    "wittier": "witty",
    "wittiest": "witty",
    "witting": "wit",
    "wives": "wife",
    "wizards": "wizard",
    "wks": "wk",
    "wobbled": "wobble",
    "wobblers": "wobbler",
    "wobbles": "wobble",
    "wobblier": "wobbly",
    "wobbling": "wobble",
    "wodges": "wodge",
    "woes": "woe",
    "wogs": "wog",
    "woke": "wake",
    "woken": "wake",
    "woks": "wok",
    "wolds": "wold",
    "wolf-whistled": "wolf-whistle",
    "wolf-whistles": "wolf-whistle",
    "wolf-whistling": "wolf-whistle",
    "wolfed": "wolf",
    "wolfhounds": "wolfhound",
    "wolfing": "wolf",
    "wolfs": "wolf",
    "wolverines": "wolverine",
    "wolves": "wolf",
    "womanised": "womanise",
    "womanises": "womanise",
    "womanising": "womanise",
    "womanizers": "womanizer",
    "womanlier": "womanly",
    "wombats": "wombat",
    "wombs": "womb",
    "women": "woman",
    "won": "win",
    "wondered": "wonder",
    "wondering": "wonder",
    "wonderlands": "wonderland",
    "wonders": "wonder",
    "wonkier": "wonky",
    "wonkiest": "wonky",
    "wood-carvings": "wood-carving",
    "woodchips": "woodchip",
    "woodcocks": "woodcock",
    "woodcrafts": "woodcraft",
    "woodcuts": "woodcut",
    "woodcutters": "woodcutter",
    "wooded": "wood",
    "woodier": "woody",
    "woodiest": "woody",
    "wooding": "wood",
    "woodlands": "woodland",
    "woodlice": "woodlouse",
    "woodmen": "woodman",
    "woodpeckers": "woodpecker",
    "woodpiles": "woodpile",
    "woods": "wood",
    "woodsheds": "woodshed",
    "woodsmen": "woodsman",
    "woodwinds": "woodwind",
    "woodworkers": "woodworker",
    "woodworms": "woodworm",
    "wooed": "woo",
    "wooers": "wooer",
    "woofed": "woof",
    "woofers": "woofer",
    "woofing": "woof",
    "woofs": "woof",
    "wooing": "woo",
    "woolier": "wooly",
    "woollens": "woollen",
    "woollier": "woolly",
    "woollies": "woolly",
    "woolliest": "woolly",
    "wools": "wool",
    "woos": "woo",
    "woozier": "woozy",
    "wooziest": "woozy",
    "wops": "wop",
    "worded": "word",
    "wordier": "wordy",
    "wordiest": "wordy",
    "wording": "word",
    "wordings": "wording",
    "wordlists": "wordlist",
    "words": "word",
    "wore": "wear",
    "work-forces": "work-force",
    "work-ins": "work-in",
    "work-lives": "work-life",
    "work-loads": "work-load",
    "work-outs": "work-out",
    "work-places": "work-place",
    "work-to-rules": "work-to-rule",
    "work-ups": "work-up",
    "workaholics": "workaholic",
    "workbenches": "workbench",
    "workbooks": "workbook",
    "workdays": "workday",
    "worked": "work",
    "workers": "worker",
    "workflows": "workflow",
    "workforces": "workforce",
    "workgroups": "workgroup",
    "workhorses": "workhorse",
    "workhouses": "workhouse",
    "working": "work",
    "workingmen": "workingman",
    "workings": "working",
    "worklists": "worklist",
    "workloads": "workload",
    "workmanships": "workmanship",
    "workmates": "workmate",
    "workmen": "workman",
    "workouts": "workout",
    "workplaces": "workplace",
    "workrooms": "workroom",
    "works": "work",
    "works-in-progress": "work-in-progress",
    "worksheets": "worksheet",
    "workshops": "workshop",
    "worksites": "worksite",
    "workspaces": "workspace",
    "workstations": "workstation",
    "worktops": "worktop",
    "world-views": "world-view",
    "worldlier": "worldly",
    "worldliest": "worldly",
    "worlds": "world",
    "worldviews": "worldview",
    "wormed": "worm",
    "wormers": "wormer",
    "wormier": "wormy",
    "wormiest": "wormy",
    "worming": "worm",
    "worms": "worm",
    "worn": "wear",
    "worried": "worry",
    "worriers": "worrier",
    "worries": "worry",
    "worrying": "worry",
    "worse": "wrong",
    "worsened": "worsen",
    "worsening": "worsen",
    "worsens": "worsen",
    "worshiped": "worship",
    "worshipers": "worshiper",
    "worshiping": "worship",
    "worshipped": "worship",
    "worshippers": "worshipper",
    "worshipping": "worship",
    "worships": "worship",
    "worst": "wrong",
    "worsted": "worst",
    "worsting": "worst",
    "worsts": "worst",
    "worthier": "worthy",
    "worthies": "worthy",
    "worthiest": "worthy",
    "worths": "worth",
    "worts": "wort",
    "wot": "wit",
    "wound": "wind",
    "wounded": "wound",
    "wounding": "wound",
    "wounds": "wound",
    "wove": "weave",
    "woven": "weave",
    "wowed": "wow",
    "wowing": "wow",
    "wows": "wow",
    "wracked": "wrack",
    "wracking": "wrack",
    "wracks": "wrack",
    "wraiths": "wraith",
    "wrangled": "wrangle",
    "wranglers": "wrangler",
    "wrangles": "wrangle",
    "wrangling": "wrangle",
    "wrapped": "wrap",
    "wrappers": "wrapper",
    "wrapping": "wrap",
    "wrappings": "wrapping",
    "wraps": "wrap",
    "wrapt": "wrap",
    "wrasses": "wrasse",
    "wreaked": "wreak",
    "wreaking": "wreak",
    "wreaks": "wreak",
    "wreathed": "wreathe",
    "wreathes": "wreathe",
    "wreathing": "wreathe",
    "wreaths": "wreath",
    "wrecked": "wreck",
    "wreckers": "wrecker",
    "wrecking": "wreck",
    "wrecks": "wreck",
    "wrenched": "wrench",
    "wrenches": "wrench",
    "wrenching": "wrench",
    "wrens": "wren",
    "wrested": "wrest",
    "wresting": "wrest",
    "wrestled": "wrestle",
    "wrestlers": "wrestler",
    "wrestles": "wrestle",
    "wrestling": "wrestle",
    "wrests": "wrest",
    "wretches": "wretch",
    "wrier": "wry",
    "wriest": "wry",
    "wriggled": "wriggle",
    "wriggles": "wriggle",
    "wriggling": "wriggle",
    "wrights": "wright",
    "wringers": "wringer",
    "wringing": "wring",
    "wrings": "wring",
    "wrinkled": "wrinkle",
    "wrinkles": "wrinkle",
    "wrinklier": "wrinkly",
    "wrinkling": "wrinkle",
    "wristbands": "wristband",
    "wrists": "wrist",
    "wristwatches": "wristwatch",
    "write-offs": "write-off",
    "write-ups": "write-up",
    "writers": "writer",
    "writes": "write",
    "writhed": "writhe",
    "writhes": "writhe",
    "writhing": "writhe",
    "writing": "write",
    "writings": "writing",
    "writs": "writ",
    "written": "write",
    "wrong-footed": "wrong-foot",
    "wrong-footing": "wrong-foot",
    "wrong-foots": "wrong-foot",
    "wrongdoers": "wrongdoer",
    "wrongdoings": "wrongdoing",
    "wronged": "wrong",
    "wronging": "wrong",
    "wrongs": "wrong",
    "wrote": "write",
    "wrought": "work",
    "wrung": "wring",
    "ws": "w",
    "wyes": "wye",
    "xanthines": "xanthine",
    "xenografts": "xenograft",
    "xenophobes": "xenophobe",
    "xenotransplantations": "xenotransplantation",
    "xis": "xi",
    "xrays": "xray",
    "xylems": "xylem",
    "xylenes": "xylene",
    "xylophones": "xylophone",
    "y-axides": "y-axis",
    "ya": "you",
    "yachted": "yacht",
    "yachting": "yacht",
    "yachtings": "yachting",
    "yachts": "yacht",
    "yachtsmen": "yachtsman",
    "yachtswomen": "yachtswoman",
    "yaks": "yak",
    "yammered": "yammer",
    "yammering": "yammer",
    "yammers": "yammer",
    "yams": "yam",
    "yangs": "yang",
    "yanked": "yank",
    "yanking": "yank",
    "yanks": "yank",
    "yaped": "yap",
    "yaping": "yap",
    "yapped": "yap",
    "yapping": "yap",
    "yaps": "yap",
    "yards": "yard",
    "yardsticks": "yardstick",
    "yarned": "yarn",
    "yarning": "yarn",
    "yarns": "yarn",
    "yashmaks": "yashmak",
    "yawed": "yaw",
    "yawing": "yaw",
    "yawned": "yawn",
    "yawning": "yawn",
    "yawns": "yawn",
    "yaws": "yaw",
    "yds": "yd",
    "yeah": "yes",
    "yearbooks": "yearbook",
    "yearlings": "yearling",
    "yearned": "yearn",
    "yearning": "yearn",
    "yearnings": "yearning",
    "yearns": "yearn",
    "years": "year",
    "yeastier": "yeasty",
    "yeasts": "yeast",
    "yelled": "yell",
    "yelling": "yell",
    "yellowed": "yellow",
    "yellower": "yellow",
    "yellowest": "yellow",
    "yellowhammers": "yellowhammer",
    "yellowing": "yellow",
    "yellows": "yellow",
    "yells": "yell",
    "yelped": "yelp",
    "yelping": "yelp",
    "yelps": "yelp",
    "yens": "yen",
    "yeomen": "yeoman",
    "yes-men": "yes-man",
    "yeses": "yes",
    "yesses": "yes",
    "yesterdays": "yesterday",
    "yesteryears": "yesteryear",
    "yetis": "yeti",
    "yews": "yew",
    "yielded": "yield",
    "yielding": "yield",
    "yields": "yield",
    "yips": "yip",
    "yms": "ym",
    "yo-yoes": "yo-yo",
    "yo-yos": "yo-yo",
    "yobbos": "yobbo",
    "yobs": "yob",
    "yodelled": "yodel",
    "yodelling": "yodel",
    "yodels": "yodel",
    "yoghourts": "yoghourt",
    "yoghurts": "yoghurt",
    "yogis": "yogi",
    "yogurts": "yogurt",
    "yoked": "yoke",
    "yokels": "yokel",
    "yokes": "yoke",
    "yoking": "yoke",
    "yolks": "yolk",
    "yorkers": "yorker",
    "younger": "young",
    "youngest": "young",
    "youngsters": "youngster",
    "yourselves": "yourself",
    "youths": "youth",
    "yowled": "yowl",
    "yowling": "yowl",
    "yowls": "yowl",
    "yrs": "yr",
    "ys": "y",
    "yuccas": "yucca",
    "yuletides": "yuletide",
    "yuppies": "yuppy",
    "zanders": "zander",
    "zanier": "zany",
    "zaniest": "zany",
    "zapped": "zap",
    "zapping": "zap",
    "zaps": "zap",
    "zealots": "zealot",
    "zebrafishes": "zebrafish",
    "zebras": "zebra",
    "zees": "zee",
    "zeitgeists": "zeitgeist",
    "zeniths": "zenith",
    "zeolites": "zeolite",
    "zephyrs": "zephyr",
    "zeroed": "zero",
    "zeroes": "zero",
    "zeroing": "zero",
    "zeros": "zero",
    "zidovudines": "zidovudine",
    "zig-zagged": "zig-zag",
    "zig-zagging": "zig-zag",
    "zig-zags": "zig-zag",
    "zigzagged": "zigzag",
    "zigzagging": "zigzag",
    "zigzags": "zigzag",
    "zincs": "zinc",
    "zings": "zing",
    "zipped": "zip",
    "zippers": "zipper",
    "zipping": "zip",
    "zips": "zip",
    "zirconia": "zirconium",
    "zircons": "zircon",
    "zithers": "zither",
    "zodiacs": "zodiac",
    "zombies": "zombie",
    "zonae": "zona",
    "zonations": "zonation",
    "zoned": "zone",
    "zones": "zone",
    "zoning": "zone",
    "zoologies": "zoology",
    "zoologists": "zoologist",
    "zoomed": "zoom",
    "zooming": "zoom",
    "zooms": "zoom",
    "zoonoses": "zoonosis",
    "zooplanktons": "zooplankton",
    "zoos": "zoo",
    "zoospores": "zoospore",
    "zucchinis": "zucchini",
    "zygotes": "zygote"
}
